/*
 * Organizationally Unique Identifier list downloaded on 2025-08-14
 * Automatically generated from https://standards-oui.ieee.org
 * For Netdiscover by Jaime Penalba
 *
 */

struct oui {
   char *prefix;   /* 24 bit global prefix */
   char *vendor;   /* Vendor id string     */
};

struct oui oui_table[] = {
    { "000000", "XEROX CORPORATION" },
    { "000001", "XEROX CORPORATION" },
    { "000002", "XEROX CORPORATION" },
    { "000003", "XEROX CORPORATION" },
    { "000004", "XEROX CORPORATION" },
    { "000005", "XEROX CORPORATION" },
    { "000006", "XEROX CORPORATION" },
    { "000007", "XEROX CORPORATION" },
    { "000008", "XEROX CORPORATION" },
    { "000009", "XEROX CORPORATION" },
    { "00000A", "OMRON TATEISI ELECTRONICS CO." },
    { "00000B", "MATRIX CORPORATION" },
    { "00000C", "Cisco Systems, Inc" },
    { "00000D", "FIBRONICS LTD." },
    { "00000E", "FUJITSU LIMITED" },
    { "00000F", "NEXT, INC." },
    { "000010", "SYTEK INC." },
    { "000011", "NORMEREL SYSTEMES" },
    { "000012", "INFORMATION TECHNOLOGY LIMITED" },
    { "000013", "CAMEX" },
    { "000014", "NETRONIX" },
    { "000015", "DATAPOINT CORPORATION" },
    { "000016", "DU PONT PIXEL SYSTEMS ." },
    { "000017", "Oracle" },
    { "000018", "WEBSTER COMPUTER CORPORATION" },
    { "000019", "APPLIED DYNAMICS INTERNATIONAL" },
    { "00001A", "ADVANCED MICRO DEVICES" },
    { "00001B", "Novell, Inc." },
    { "00001C", "BELL TECHNOLOGIES" },
    { "00001D", "Cabletron Systems, Inc." },
    { "00001E", "TELSIST INDUSTRIA ELECTRONICA" },
    { "00001F", "Telco Systems, Inc. " },
    { "000020", "DATAINDUSTRIER DIAB AB" },
    { "000021", "SUREMAN COMP. & COMMUN. CORP." },
    { "000022", "VISUAL TECHNOLOGY INC." },
    { "000023", "ABB INDUSTRIAL SYSTEMS AB" },
    { "000024", "CONNECT AS" },
    { "000025", "RAMTEK CORP." },
    { "000026", "SHA-KEN CO., LTD." },
    { "000027", "JAPAN RADIO COMPANY" },
    { "000028", "PRODIGY SYSTEMS CORPORATION" },
    { "000029", "IMC NETWORKS CORP." },
    { "00002A", "TRW - SEDD/INP" },
    { "00002B", "CRISP AUTOMATION, INC" },
    { "00002C", "AUTOTOTE LIMITED" },
    { "00002D", "CHROMATICS INC" },
    { "00002E", "SOCIETE EVIRA" },
    { "00002F", "TIMEPLEX INC." },
    { "000030", "VG LABORATORY SYSTEMS LTD" },
    { "000031", "QPSX COMMUNICATIONS, LTD." },
    { "000032", "Marconi plc" },
    { "000033", "EGAN MACHINERY COMPANY" },
    { "000034", "NETWORK RESOURCES CORPORATION" },
    { "000035", "SPECTRAGRAPHICS CORPORATION" },
    { "000036", "ATARI CORPORATION" },
    { "000037", "OXFORD METRICS LIMITED" },
    { "000038", "CSS LABS" },
    { "000039", "TOSHIBA CORPORATION" },
    { "00003A", "CHYRON CORPORATION" },
    { "00003B", "i Controls, Inc." },
    { "00003C", "AUSPEX SYSTEMS INC." },
    { "00003D", "UNISYS" },
    { "00003E", "SIMPACT" },
    { "00003F", "SYNTREX, INC." },
    { "000040", "APPLICON, INC." },
    { "000041", "ICE CORPORATION" },
    { "000042", "METIER MANAGEMENT SYSTEMS LTD." },
    { "000043", "MICRO TECHNOLOGY" },
    { "000044", "CASTELLE CORPORATION" },
    { "000045", "FORD AEROSPACE & COMM. CORP." },
    { "000046", "OLIVETTI NORTH AMERICA" },
    { "000047", "NICOLET INSTRUMENTS CORP." },
    { "000048", "Seiko Epson Corporation" },
    { "000049", "APRICOT COMPUTERS, LTD" },
    { "00004A", "ADC CODENOLL TECHNOLOGY CORP." },
    { "00004B", "ICL DATA OY" },
    { "00004C", "NEC Corporation" },
    { "00004D", "DCI CORPORATION" },
    { "00004E", "AMPEX CORPORATION" },
    { "00004F", "LOGICRAFT, INC." },
    { "000050", "RADISYS CORPORATION" },
    { "000051", "HOB ELECTRONIC GMBH & CO. KG" },
    { "000052", "Intrusion.com, Inc." },
    { "000053", "COMPUCORP" },
    { "000054", "Schneider Electric" },
    { "000055", "COMMISSARIAT A L`ENERGIE ATOM." },
    { "000056", "DR. B. STRUCK" },
    { "000057", "SCITEX CORPORATION LTD." },
    { "000058", "RACORE COMPUTER PRODUCTS INC." },
    { "000059", "Hellige GMBH" },
    { "00005A", "SysKonnect GmbH" },
    { "00005B", "ELTEC ELEKTRONIK AG" },
    { "00005C", "TELEMATICS INTERNATIONAL INC." },
    { "00005D", "CS TELECOM" },
    { "00005E", "ICANN, IANA Department" },
    { "00005F", "Sumitomo Electric Industries, Ltd" },
    { "000060", "Kontron Europe GmbH" },
    { "000061", "GATEWAY COMMUNICATIONS" },
    { "000062", "BULL HN INFORMATION SYSTEMS" },
    { "000063", "BARCO CONTROL ROOMS GMBH" },
    { "000064", "Yokogawa Digital Computer Corporation" },
    { "000065", "Network General Corporation" },
    { "000066", "TALARIS SYSTEMS, INC." },
    { "000067", "SOFT * RITE, INC." },
    { "000068", "ROSEMOUNT CONTROLS" },
    { "000069", "CONCORD COMMUNICATIONS INC" },
    { "00006A", "COMPUTER CONSOLES INC." },
    { "00006B", "Silicon Graphics" },
    { "00006C", "Schneider Electric" },
    { "00006D", "CRAY COMMUNICATIONS, LTD." },
    { "00006E", "Artisoft Inc." },
    { "00006F", "Madge Ltd." },
    { "000070", "HCL LIMITED" },
    { "000071", "ADRA SYSTEMS INC." },
    { "000072", "MINIWARE TECHNOLOGY" },
    { "000073", "SIECOR CORPORATION" },
    { "000074", "RICOH COMPANY LTD." },
    { "000075", "Nortel Networks" },
    { "000076", "ABEKAS VIDEO SYSTEM" },
    { "000077", "INTERPHASE CORPORATION" },
    { "000078", "LABTAM LIMITED" },
    { "000079", "NETWORTH INCORPORATED" },
    { "00007A", "DANA COMPUTER INC." },
    { "00007B", "RESEARCH MACHINES" },
    { "00007C", "AMPERE INCORPORATED" },
    { "00007D", "Oracle Corporation" },
    { "00007E", "CLUSTRIX CORPORATION" },
    { "00007F", "LINOTYPE-HELL AG" },
    { "000080", "CRAY COMMUNICATIONS A/S" },
    { "000081", "Bay Networks" },
    { "000082", "LECTRA SYSTEMES SA" },
    { "000083", "TADPOLE TECHNOLOGY PLC" },
    { "000084", "SUPERNET" },
    { "000085", "CANON INC." },
    { "000086", "MEGAHERTZ CORPORATION" },
    { "000087", "HITACHI, LTD." },
    { "000088", "Brocade Communications Systems LLC" },
    { "000089", "CAYMAN SYSTEMS INC." },
    { "00008A", "DATAHOUSE INFORMATION SYSTEMS" },
    { "00008B", "INFOTRON" },
    { "00008C", "Alloy Computer Products (Australia) Pty Ltd" },
    { "00008D", "Cryptek Inc." },
    { "00008E", "SOLBOURNE COMPUTER, INC." },
    { "00008F", "Raytheon" },
    { "000090", "MICROCOM" },
    { "000091", "ANRITSU CORPORATION" },
    { "000092", "COGENT DATA TECHNOLOGIES" },
    { "000093", "PROTEON INC." },
    { "000094", "ASANTE TECHNOLOGIES" },
    { "000095", "SONY TEKTRONIX CORP." },
    { "000096", "MARCONI ELECTRONICS LTD." },
    { "000097", "Dell EMC" },
    { "000098", "CROSSCOMM CORPORATION" },
    { "000099", "MTX, INC." },
    { "00009A", "RC COMPUTER A/S" },
    { "00009B", "INFORMATION INTERNATIONAL, INC" },
    { "00009C", "ROLM MIL-SPEC COMPUTERS" },
    { "00009D", "LOCUS COMPUTING CORPORATION" },
    { "00009E", "MARLI S.A." },
    { "00009F", "AMERISTAR TECHNOLOGIES INC." },
    { "0000A0", "SANYO Electric Co., Ltd." },
    { "0000A1", "MARQUETTE ELECTRIC CO." },
    { "0000A2", "Bay Networks" },
    { "0000A3", "NETWORK APPLICATION TECHNOLOGY" },
    { "0000A4", "ACORN COMPUTERS LIMITED" },
    { "0000A5", "Tattile SRL " },
    { "0000A6", "NETWORK GENERAL CORPORATION" },
    { "0000A7", "NETWORK COMPUTING DEVICES INC." },
    { "0000A8", "Stratus Technologies" },
    { "0000A9", "NETWORK SYSTEMS CORP." },
    { "0000AA", "XEROX CORPORATION" },
    { "0000AB", "LOGIC MODELING CORPORATION" },
    { "0000AC", "CONWARE COMPUTER CONSULTING" },
    { "0000AD", "BRUKER INSTRUMENTS INC." },
    { "0000AE", "DASSAULT ELECTRONIQUE" },
    { "0000AF", "Canberra Industries, Inc." },
    { "0000B0", "RND-RAD NETWORK DEVICES" },
    { "0000B1", "Alpha Micro" },
    { "0000B2", "TELEVIDEO SYSTEMS, INC." },
    { "0000B3", "CIMLINC INCORPORATED" },
    { "0000B4", "Edimax Technology Co. Ltd." },
    { "0000B5", "DATABILITY SOFTWARE SYS. INC." },
    { "0000B6", "MICRO-MATIC RESEARCH" },
    { "0000B7", "DOVE COMPUTER CORPORATION" },
    { "0000B8", "SEIKOSHA CO., LTD." },
    { "0000B9", "MCDONNELL DOUGLAS COMPUTER SYS" },
    { "0000BA", "SIIG, INC." },
    { "0000BB", "TRI-DATA" },
    { "0000BC", "Rockwell Automation" },
    { "0000BD", "RYOSEI, Ltd." },
    { "0000BE", "THE NTI GROUP" },
    { "0000BF", "SYMMETRIC COMPUTER SYSTEMS" },
    { "0000C0", "WESTERN DIGITAL CORPORATION" },
    { "0000C1", "Madge Ltd." },
    { "0000C2", "INFORMATION PRESENTATION TECH." },
    { "0000C3", "Harris Corporation" },
    { "0000C4", "WATERS DIV. OF MILLIPORE" },
    { "0000C5", "Commscope" },
    { "0000C6", "EON SYSTEMS" },
    { "0000C7", "ARIX CORPORATION" },
    { "0000C8", "ALTOS COMPUTER SYSTEMS" },
    { "0000C9", "Emulex Corporation" },
    { "0000CA", "Commscope" },
    { "0000CB", "COMPU-SHACK ELECTRONIC GMBH" },
    { "0000CC", "DENSAN CO., LTD." },
    { "0000CD", "Allied Telesis Labs Ltd" },
    { "0000CE", "MEGADATA CORP." },
    { "0000CF", "HAYES MICROCOMPUTER PRODUCTS" },
    { "0000D0", "DEVELCON ELECTRONICS LTD." },
    { "0000D1", "ADAPTEC INCORPORATED" },
    { "0000D2", "SBE, INC." },
    { "0000D3", "WANG LABORATORIES INC." },
    { "0000D4", "PURE DATA LTD." },
    { "0000D5", "MICROGNOSIS INTERNATIONAL" },
    { "0000D6", "PUNCH LINE HOLDING" },
    { "0000D7", "DARTMOUTH COLLEGE" },
    { "0000D8", "Novell, Inc." },
    { "0000D9", "NIPPON TELEGRAPH & TELEPHONE" },
    { "0000DA", "ATEX" },
    { "0000DB", "British Telecommunications plc" },
    { "0000DC", "HAYES MICROCOMPUTER PRODUCTS" },
    { "0000DD", "TCL INCORPORATED" },
    { "0000DE", "CETIA" },
    { "0000DF", "BELL & HOWELL PUB SYS DIV" },
    { "0000E0", "QUADRAM CORP." },
    { "0000E1", "GRID SYSTEMS" },
    { "0000E2", "ACER TECHNOLOGIES CORP." },
    { "0000E3", "INTEGRATED MICRO PRODUCTS LTD" },
    { "0000E4", "IN2 GROUPE INTERTECHNIQUE" },
    { "0000E5", "SIGMEX LTD." },
    { "0000E6", "APTOR PRODUITS DE COMM INDUST" },
    { "0000E7", "Star Gate Technologies" },
    { "0000E8", "ACCTON TECHNOLOGY CORP." },
    { "0000E9", "ISICAD, INC." },
    { "0000EA", "UPNOD AB" },
    { "0000EB", "MATSUSHITA COMM. IND. CO. LTD." },
    { "0000EC", "MICROPROCESS" },
    { "0000ED", "APRIL" },
    { "0000EE", "NETWORK DESIGNERS, LTD." },
    { "0000EF", "KTI" },
    { "0000F0", "Samsung Electronics Co.,Ltd" },
    { "0000F1", "MAGNA COMPUTER CORPORATION" },
    { "0000F2", "SPIDER COMMUNICATIONS" },
    { "0000F3", "GANDALF DATA LIMITED" },
    { "0000F4", "Allied Telesis, Inc." },
    { "0000F5", "DIAMOND SALES LIMITED" },
    { "0000F6", "APPLIED MICROSYSTEMS CORP." },
    { "0000F7", "YOUTH KEEP ENTERPRISE CO LTD" },
    { "0000F8", "DIGITAL EQUIPMENT CORPORATION" },
    { "0000F9", "QUOTRON SYSTEMS INC." },
    { "0000FA", "MICROSAGE COMPUTER SYSTEMS INC" },
    { "0000FB", "RECHNER ZUR KOMMUNIKATION" },
    { "0000FC", "MEIKO" },
    { "0000FD", "HIGH LEVEL HARDWARE" },
    { "0000FE", "Annapolis Micro Systems, Inc." },
    { "0000FF", "CAMTEC ELECTRONICS LTD." },
    { "000100", "EQUIP'TRANS" },
    { "000101", "Private" },
    { "000102", "3COM" },
    { "000103", "3COM" },
    { "000104", "DVICO Co., Ltd." },
    { "000105", "Beckhoff Automation GmbH" },
    { "000106", "Tews Datentechnik GmbH" },
    { "000107", "Leiser GmbH" },
    { "000108", "AVLAB Technology, Inc." },
    { "000109", "Nagano Japan Radio Co., Ltd." },
    { "00010A", "CIS TECHNOLOGY INC." },
    { "00010B", "Space CyberLink, Inc." },
    { "00010C", "System Talks Inc." },
    { "00010D", "Teledyne DALSA Inc." },
    { "00010E", "Bri-Link Technologies Co., Ltd" },
    { "00010F", "Brocade Communications Systems LLC" },
    { "000110", "Gotham Networks" },
    { "000111", "iDigm Inc." },
    { "000112", "Shark Multimedia Inc." },
    { "000113", "OLYMPUS CORPORATION" },
    { "000114", "KANDA TSUSHIN KOGYO CO., LTD." },
    { "000115", "EXTRATECH CORPORATION" },
    { "000116", "Netspect Technologies, Inc." },
    { "000117", "Canal +" },
    { "000118", "EZ Digital Co., Ltd." },
    { "000119", "RTUnet (Australia)" },
    { "00011A", "Hoffmann und Burmeister GbR" },
    { "00011B", "Unizone Technologies, Inc." },
    { "00011C", "Universal Talkware Corporation" },
    { "00011D", "Centillium Communications" },
    { "00011E", "Precidia Technologies, Inc." },
    { "00011F", "RC Networks, Inc." },
    { "000120", "OSCILLOQUARTZ S.A." },
    { "000121", "WatchGuard Technologies, Inc." },
    { "000122", "Trend Communications, Ltd." },
    { "000123", "Schneider Electric Japan Holdings Ltd." },
    { "000124", "Acer Incorporated" },
    { "000125", "YAESU MUSEN CO., LTD." },
    { "000126", "PAC Labs" },
    { "000127", "OPEN Networks Pty Ltd" },
    { "000128", "EnjoyWeb, Inc." },
    { "000129", "DFI Inc." },
    { "00012A", "Telematica Sistems Inteligente" },
    { "00012B", "TELENET Co., Ltd." },
    { "00012C", "Aravox Technologies, Inc." },
    { "00012D", "Komodo Technology" },
    { "00012E", "PC Partner Ltd." },
    { "00012F", "Twinhead International Corp" },
    { "000130", "Extreme Networks Headquarters" },
    { "000131", "Bosch Security Systems, Inc." },
    { "000132", "Dranetz - BMI" },
    { "000133", "KYOWA Electronic Instruments C" },
    { "000134", "Selectron Systems AG" },
    { "000135", "KDC Corp." },
    { "000136", "CyberTAN Technology Inc." },
    { "000137", "IT Farm Corporation" },
    { "000138", "XAVi Technologies Corp." },
    { "000139", "Point Multimedia Systems" },
    { "00013A", "SHELCAD COMMUNICATIONS, LTD." },
    { "00013B", "BNA SYSTEMS" },
    { "00013C", "TIW SYSTEMS" },
    { "00013D", "RiscStation Ltd." },
    { "00013E", "Ascom Tateco AB" },
    { "00013F", "Neighbor World Co., Ltd." },
    { "000140", "Sendtek Corporation" },
    { "000141", "CABLE PRINT" },
    { "000142", "Cisco Systems, Inc" },
    { "000143", "Cisco Systems, Inc" },
    { "000144", "Dell EMC" },
    { "000145", "WINSYSTEMS, INC." },
    { "000146", "Tesco Controls, Inc." },
    { "000147", "Zhone Technologies, Inc." },
    { "000148", "X-traWeb Inc." },
    { "000149", "TDT AG" },
    { "00014A", "Sony Corporation" },
    { "00014B", "Ennovate Networks, Inc." },
    { "00014C", "Berkeley Process Control" },
    { "00014D", "Shin Kin Enterprises Co., Ltd" },
    { "00014E", "WIN Enterprises, Inc." },
    { "00014F", "Adtran Inc" },
    { "000150", "GILAT COMMUNICATIONS, LTD." },
    { "000151", "Ensemble Communications" },
    { "000152", "CHROMATEK INC." },
    { "000153", "ARCHTEK TELECOM CORPORATION" },
    { "000154", "G3M Corporation" },
    { "000155", "Promise Technology, Inc." },
    { "000156", "FIREWIREDIRECT.COM, INC." },
    { "000157", "SYSWAVE CO., LTD" },
    { "000158", "Electro Industries/Gauge Tech" },
    { "000159", "S1 Corporation" },
    { "00015A", "Digital Video Broadcasting" },
    { "00015B", "ITALTEL S.p.A/RF-UP-I" },
    { "00015C", "CADANT INC." },
    { "00015D", "Oracle Corporation " },
    { "00015E", "BEST TECHNOLOGY CO., LTD." },
    { "00015F", "DIGITAL DESIGN GmbH" },
    { "000160", "ELMEX Co., LTD." },
    { "000161", "Meta Machine Technology" },
    { "000162", "Cygnet Technologies, Inc." },
    { "000163", "Cisco Systems, Inc" },
    { "000164", "Cisco Systems, Inc" },
    { "000165", "AirSwitch Corporation" },
    { "000166", "TC GROUP A/S" },
    { "000167", "HIOKI E.E. CORPORATION" },
    { "000168", "VITANA CORPORATION" },
    { "000169", "Celestix Networks Pte Ltd." },
    { "00016A", "ALITEC" },
    { "00016B", "LightChip, Inc." },
    { "00016C", "FOXCONN" },
    { "00016D", "CarrierComm Inc." },
    { "00016E", "Conklin Corporation" },
    { "00016F", "Inkel Corp." },
    { "000170", "ESE Embedded System Engineer'g" },
    { "000171", "Allied Data Technologies" },
    { "000172", "TechnoLand Co., LTD." },
    { "000173", "AMCC" },
    { "000174", "CyberOptics Corporation" },
    { "000175", "Radiant Communications Corp." },
    { "000176", "Orient Silver Enterprises" },
    { "000177", "EDSL" },
    { "000178", "MARGI Systems, Inc." },
    { "000179", "WIRELESS TECHNOLOGY, INC." },
    { "00017A", "Chengdu Maipu Electric Industrial Co., Ltd." },
    { "00017B", "Heidelberger Druckmaschinen AG" },
    { "00017C", "AG-E GmbH" },
    { "00017D", "ThermoQuest" },
    { "00017E", "ADTEK System Science Co., Ltd." },
    { "00017F", "Experience Music Project" },
    { "000180", "AOpen, Inc." },
    { "000181", "Nortel Networks" },
    { "000182", "DICA TECHNOLOGIES AG" },
    { "000183", "ANITE TELECOMS" },
    { "000184", "SIEB & MEYER AG" },
    { "000185", "Hitachi Aloka Medical, Ltd." },
    { "000186", "Uwe Disch" },
    { "000187", "I2SE GmbH" },
    { "000188", "LXCO Technologies ag" },
    { "000189", "Refraction Technology, Inc." },
    { "00018A", "ROI COMPUTER AG" },
    { "00018B", "NetLinks Co., Ltd." },
    { "00018C", "Mega Vision" },
    { "00018D", "AudeSi Technologies" },
    { "00018E", "Logitec Corporation" },
    { "00018F", "Kenetec, Inc." },
    { "000190", "SMK-M" },
    { "000191", "SYRED Data Systems" },
    { "000192", "Texas Digital Systems" },
    { "000193", "Hanbyul Telecom Co., Ltd." },
    { "000194", "Capital Equipment Corporation" },
    { "000195", "Sena Technologies, Inc." },
    { "000196", "Cisco Systems, Inc" },
    { "000197", "Cisco Systems, Inc" },
    { "000198", "Darim Vision" },
    { "000199", "HeiSei Electronics" },
    { "00019A", "LEUNIG GmbH" },
    { "00019B", "Kyoto Microcomputer Co., Ltd." },
    { "00019C", "Lumentum" },
    { "00019D", "E-Control Systems, Inc." },
    { "00019E", "ESS Technology, Inc." },
    { "00019F", "ReadyNet" },
    { "0001A0", "Infinilink Corporation" },
    { "0001A1", "Mag-Tek, Inc." },
    { "0001A2", "Logical Co., Ltd." },
    { "0001A3", "GENESYS LOGIC, INC." },
    { "0001A4", "Microlink Corporation" },
    { "0001A5", "Nextcomm, Inc." },
    { "0001A6", "Scientific-Atlanta Arcodan A/S" },
    { "0001A7", "UNEX TECHNOLOGY CORPORATION" },
    { "0001A8", "Welltech Computer Co., Ltd." },
    { "0001A9", "BMW AG" },
    { "0001AA", "Airspan Communications, Ltd." },
    { "0001AB", "Main Street Networks" },
    { "0001AC", "Sitara Networks, Inc." },
    { "0001AD", "Coach Master International d.b.a. CMI Worldwide, Inc." },
    { "0001AE", "Trex Enterprises" },
    { "0001AF", "Artesyn Embedded Technologies" },
    { "0001B0", "Fulltek Technology Co., Ltd." },
    { "0001B1", "General Bandwidth" },
    { "0001B2", "Digital Processing Systems, Inc." },
    { "0001B3", "Precision Electronic Manufacturing" },
    { "0001B4", "Wayport, Inc." },
    { "0001B5", "Turin Networks, Inc." },
    { "0001B6", "SAEJIN T&M Co., Ltd." },
    { "0001B7", "Centos, Inc." },
    { "0001B8", "Netsensity, Inc." },
    { "0001B9", "SKF (U.K.) Limited" },
    { "0001BA", "IC-Net, Inc." },
    { "0001BB", "Frequentis" },
    { "0001BC", "Brains Corporation" },
    { "0001BD", "Peterson Electro-Musical Products, Inc." },
    { "0001BE", "Gigalink Co., Ltd." },
    { "0001BF", "Teleforce Co., Ltd." },
    { "0001C0", "CompuLab, Ltd." },
    { "0001C1", "Vitesse Semiconductor Corporation" },
    { "0001C2", "ARK Research Corp." },
    { "0001C3", "Acromag, Inc." },
    { "0001C4", "NeoWave, Inc." },
    { "0001C5", "Simpler Networks" },
    { "0001C6", "Quarry Technologies" },
    { "0001C7", "Cisco Systems, Inc" },
    { "0001C8", "CONRAD CORP." },
    { "0001C8", "THOMAS CONRAD CORP." },
    { "0001C9", "Cisco Systems, Inc" },
    { "0001CA", "Geocast Network Systems, Inc." },
    { "0001CB", "EVR" },
    { "0001CC", "Brand Maker Enabler Inc. " },
    { "0001CD", "ARtem" },
    { "0001CE", "Custom Micro Products, Ltd." },
    { "0001CF", "Alpha Data Parallel Systems, Ltd." },
    { "0001D0", "VitalPoint, Inc." },
    { "0001D1", "CoNet Communications, Inc." },
    { "0001D2", "inXtron, Inc. " },
    { "0001D3", "PAXCOMM, Inc." },
    { "0001D4", "Leisure Time, Inc." },
    { "0001D5", "HAEDONG INFO & COMM CO., LTD" },
    { "0001D6", "manroland AG" },
    { "0001D7", "F5 Inc." },
    { "0001D8", "Teltronics, Inc." },
    { "0001D9", "Sigma, Inc." },
    { "0001DA", "WINCOMM Corporation" },
    { "0001DB", "Freecom Technologies GmbH" },
    { "0001DC", "Activetelco" },
    { "0001DD", "Avail Networks" },
    { "0001DE", "Trango Systems, Inc." },
    { "0001DF", "ISDN Communications, Ltd." },
    { "0001E0", "Fast Systems, Inc." },
    { "0001E1", "Kinpo Electronics, Inc." },
    { "0001E2", "Ando Electric Corporation" },
    { "0001E3", "Siemens AG" },
    { "0001E4", "Sitera, Inc." },
    { "0001E5", "Supernet, Inc." },
    { "0001E6", "Hewlett Packard" },
    { "0001E7", "Hewlett Packard" },
    { "0001E8", "Force10 Networks, Inc." },
    { "0001E9", "Litton Marine Systems B.V." },
    { "0001EA", "Cirilium Corp." },
    { "0001EB", "C-COM Corporation" },
    { "0001EC", "Ericsson Group" },
    { "0001ED", "SETA Corp." },
    { "0001EE", "Comtrol Europe, Ltd." },
    { "0001EF", "Camtel Technology Corp." },
    { "0001F0", "Tridium, Inc." },
    { "0001F1", "Innovative Concepts, Inc." },
    { "0001F2", "Mark of the Unicorn, Inc." },
    { "0001F3", "QPS, Inc." },
    { "0001F4", "Enterasys" },
    { "0001F5", "ERIM S.A." },
    { "0001F6", "Association of Musical Electronics Industry" },
    { "0001F7", "Image Display Systems, Inc." },
    { "0001F8", "TEXIO TECHNOLOGY CORPORATION" },
    { "0001F9", "TeraGlobal Communications Corp." },
    { "0001FA", "HOROSCAS" },
    { "0001FB", "DoTop Technology, Inc." },
    { "0001FC", "Keyence Corporation" },
    { "0001FD", "Digital Voice Systems, Inc." },
    { "0001FE", "DIGITAL EQUIPMENT CORPORATION" },
    { "0001FF", "Data Direct Networks, Inc." },
    { "000200", "Net & Sys Co., Ltd." },
    { "000201", "IFM Electronic gmbh" },
    { "000202", "Amino Communications, Ltd." },
    { "000203", "Woonsang Telecom, Inc." },
    { "000204", "Bodmann Industries Elektronik GmbH" },
    { "000205", "Hitachi Denshi, Ltd." },
    { "000206", "Telital R&D Denmark A/S" },
    { "000207", "VisionGlobal Network Corp." },
    { "000208", "Unify Networks, Inc." },
    { "000209", "Shenzhen SED Information Technology Co., Ltd." },
    { "00020A", "Gefran Spa" },
    { "00020B", "Native Networks, Inc." },
    { "00020C", "Metro-Optix" },
    { "00020D", "Micronpc.com" },
    { "00020E", "ECI Telecom Ltd." },
    { "00020F", "AATR" },
    { "000210", "Fenecom" },
    { "000211", "Nature Worldwide Technology Corp." },
    { "000212", "SierraCom" },
    { "000213", "S.D.E.L." },
    { "000214", "DTVRO" },
    { "000215", "Cotas Computer Technology A/B" },
    { "000216", "Cisco Systems, Inc" },
    { "000217", "Cisco Systems, Inc" },
    { "000218", "Advanced Scientific Corp" },
    { "000219", "Paralon Technologies" },
    { "00021A", "Zuma Networks" },
    { "00021B", "Kollmorgen-Servotronix" },
    { "00021C", "Network Elements, Inc." },
    { "00021D", "Data General Communication Ltd." },
    { "00021E", "SIMTEL S.R.L." },
    { "00021F", "Aculab PLC" },
    { "000220", "CANON FINETECH INC." },
    { "000221", "DSP Application, Ltd." },
    { "000222", "Chromisys, Inc." },
    { "000223", "ClickTV" },
    { "000224", "C-COR" },
    { "000225", "One Stop Systems" },
    { "000226", "XESystems, Inc." },
    { "000227", "ESD Electronic System Design GmbH" },
    { "000228", "Necsom, Ltd." },
    { "000229", "Adtec Corporation" },
    { "00022A", "Asound Electronic" },
    { "00022B", "SAXA, Inc." },
    { "00022C", "ABB Bomem, Inc." },
    { "00022D", "Agere Systems" },
    { "00022E", "TEAC Corp. R& D" },
    { "00022F", "P-Cube, Ltd." },
    { "000230", "Intersoft Electronics" },
    { "000231", "Ingersoll-Rand" },
    { "000232", "Avision, Inc." },
    { "000233", "Mantra Communications, Inc." },
    { "000234", "Imperial Technology, Inc." },
    { "000235", "Paragon Networks International" },
    { "000236", "INIT GmbH" },
    { "000237", "Cosmo Research Corp." },
    { "000238", "Serome Technology, Inc." },
    { "000239", "Visicom" },
    { "00023A", "ZSK Stickmaschinen GmbH" },
    { "00023B", "Ericsson" },
    { "00023C", "Creative Technology, Ltd." },
    { "00023D", "Cisco Systems, Inc" },
    { "00023E", "Selta Telematica S.p.a" },
    { "00023F", "Compal Electronics INC." },
    { "000240", "Seedek Co., Ltd." },
    { "000241", "Amer.com" },
    { "000242", "Videoframe Systems" },
    { "000243", "Raysis Co., Ltd." },
    { "000244", "SURECOM Technology Co." },
    { "000245", "Lampus Co, Ltd." },
    { "000246", "All-Win Tech Co., Ltd." },
    { "000247", "Great Dragon Information Technology (Group) Co., Ltd." },
    { "000248", "Pilz GmbH & Co." },
    { "000249", "Aviv Infocom Co, Ltd." },
    { "00024A", "Cisco Systems, Inc" },
    { "00024B", "Cisco Systems, Inc" },
    { "00024C", "SiByte, Inc." },
    { "00024D", "Mannesman Dematic Colby Pty. Ltd." },
    { "00024E", "Datacard Group" },
    { "00024F", "IPM Datacom S.R.L." },
    { "000250", "Geyser Networks, Inc." },
    { "000251", "Soma Networks, Inc." },
    { "000252", "Carrier Corporation" },
    { "000253", "Televideo, Inc." },
    { "000254", "WorldGate" },
    { "000255", "IBM Corp" },
    { "000256", "Alpha Processor, Inc." },
    { "000257", "Microcom Corp." },
    { "000258", "Flying Packets Communications" },
    { "000259", "Tsann Kuen China (Shanghai)Enterprise Co., Ltd. IT Group" },
    { "00025A", "Catena Networks" },
    { "00025B", "Cambridge Silicon Radio" },
    { "00025C", "SCI Systems (Kunshan) Co., Ltd." },
    { "00025D", "Calix Networks" },
    { "00025E", "High Technology Ltd" },
    { "00025F", "Nortel Networks" },
    { "000260", "Accordion Networks, Inc." },
    { "000261", "Tilgin AB" },
    { "000262", "Soyo Group Soyo Com Tech Co., Ltd" },
    { "000263", "RPS S.p.A." },
    { "000264", "AudioRamp.com" },
    { "000265", "Virditech Co. Ltd." },
    { "000266", "Thermalogic Corporation" },
    { "000267", "NODE RUNNER, INC." },
    { "000268", "Harris Government Communications" },
    { "000269", "Nadatel Co., Ltd" },
    { "00026A", "Cocess Telecom Co., Ltd." },
    { "00026B", "BCM Computers Co., Ltd." },
    { "00026C", "Philips CFT" },
    { "00026D", "Adept Telecom" },
    { "00026E", "NeGeN Access, Inc." },
    { "00026F", "Senao International Co., Ltd." },
    { "000270", "Crewave Co., Ltd." },
    { "000271", "Zhone Technologies, Inc." },
    { "000272", "CC&C Technologies, Inc." },
    { "000273", "Coriolis Networks" },
    { "000274", "Tommy Technologies Corp." },
    { "000275", "SMART Technologies, Inc." },
    { "000276", "Primax Electronics Ltd." },
    { "000277", "Cash Systemes Industrie" },
    { "000278", "SAMSUNG ELECTRO MECHANICS CO., LTD." },
    { "000279", "Control Applications, Ltd." },
    { "00027A", "IOI Technology Corporation" },
    { "00027B", "Amplify Net, Inc." },
    { "00027C", "Trilithic, Inc." },
    { "00027D", "Cisco Systems, Inc" },
    { "00027E", "Cisco Systems, Inc" },
    { "00027F", "ask-technologies.com" },
    { "000280", "Mu Net, Inc." },
    { "000281", "Madge Ltd." },
    { "000282", "ViaClix, Inc." },
    { "000283", "Spectrum Controls, Inc." },
    { "000284", "UK Grid Solutions Limited" },
    { "000285", "Riverstone Networks" },
    { "000286", "Occam Networks" },
    { "000287", "Adapcom" },
    { "000288", "GLOBAL VILLAGE COMMUNICATION" },
    { "000289", "DNE Technologies" },
    { "00028A", "Ambit Microsystems Corporation" },
    { "00028B", "VDSL Systems OY" },
    { "00028C", "Micrel-Synergy Semiconductor" },
    { "00028D", "Movita Technologies, Inc." },
    { "00028E", "Rapid 5 Networks, Inc." },
    { "00028F", "Globetek, Inc." },
    { "000290", "Woorigisool, Inc." },
    { "000291", "Open Network Co., Ltd." },
    { "000292", "Logic Innovations, Inc." },
    { "000293", "Solid Data Systems" },
    { "000294", "Tokyo Sokushin Co., Ltd." },
    { "000295", "MAVENIR IPA UK LTD" },
    { "000296", "Lectron Co,. Ltd." },
    { "000297", "C-COR.net" },
    { "000298", "Broadframe Corporation" },
    { "000299", "Apex, Inc." },
    { "00029A", "Storage Apps" },
    { "00029B", "Kreatel Communications AB" },
    { "00029C", "3COM" },
    { "00029D", "Merix Corp." },
    { "00029E", "Information Equipment Co., Ltd." },
    { "00029F", "L-3 Communication Aviation Recorders" },
    { "0002A0", "Flatstack Ltd." },
    { "0002A1", "World Wide Packets" },
    { "0002A2", "Hilscher GmbH" },
    { "0002A3", "Hitachi Energy Switzerland Ltd" },
    { "0002A4", "AddPac Technology Co., Ltd." },
    { "0002A5", "Hewlett Packard" },
    { "0002A6", "Effinet Systems Co., Ltd." },
    { "0002A7", "Vivace Networks" },
    { "0002A8", "Air Link Technology" },
    { "0002A9", "RACOM, s.r.o." },
    { "0002AA", "PLcom Co., Ltd." },
    { "0002AB", "CTC Union Technologies Co., Ltd." },
    { "0002AC", "3PAR data" },
    { "0002AD", "HOYA Corporation" },
    { "0002AE", "Scannex Electronics Ltd." },
    { "0002AF", "TeleCruz Technology, Inc." },
    { "0002B0", "Hokubu Communication & Industrial Co., Ltd." },
    { "0002B1", "Anritsu, Ltd." },
    { "0002B2", "Cablevision" },
    { "0002B3", "Intel Corporation" },
    { "0002B4", "DAPHNE" },
    { "0002B5", "Avnet, Inc." },
    { "0002B6", "Acrosser Technology Co., Ltd." },
    { "0002B7", "Watanabe Electric Industry Co., Ltd." },
    { "0002B8", "WHI KONSULT AB" },
    { "0002B9", "Cisco Systems, Inc" },
    { "0002BA", "Cisco Systems, Inc" },
    { "0002BB", "Continuous Computing Corp" },
    { "0002BC", "LVL 7 Systems, Inc." },
    { "0002BD", "Bionet Co., Ltd." },
    { "0002BE", "Totsu Engineering, Inc." },
    { "0002BF", "dotRocket, Inc." },
    { "0002C0", "Bencent Tzeng Industry Co., Ltd." },
    { "0002C1", "Innovative Electronic Designs, Inc." },
    { "0002C2", "Net Vision Telecom" },
    { "0002C3", "Arelnet Ltd." },
    { "0002C4", "OPT Machine Vision Tech Co., Ltd" },
    { "0002C5", "Evertz Microsystems Ltd." },
    { "0002C6", "Data Track Technology PLC" },
    { "0002C7", "ALPSALPINE CO,.LTD" },
    { "0002C8", "Technocom Communications Technology (pte) Ltd" },
    { "0002C9", "Mellanox Technologies, Inc." },
    { "0002CA", "EndPoints, Inc." },
    { "0002CB", "TriState Ltd." },
    { "0002CC", "M.C.C.I" },
    { "0002CD", "TeleDream, Inc." },
    { "0002CE", "FoxJet, Inc." },
    { "0002CF", "ZyGate Communications, Inc." },
    { "0002D0", "Comdial Corporation" },
    { "0002D1", "Vivotek, Inc." },
    { "0002D2", "Workstation AG" },
    { "0002D3", "NetBotz, Inc." },
    { "0002D4", "PDA Peripherals, Inc." },
    { "0002D5", "ACR" },
    { "0002D6", "NICE Systems" },
    { "0002D7", "EMPEG Ltd" },
    { "0002D8", "BRECIS Communications Corporation" },
    { "0002D9", "Reliable Controls" },
    { "0002DA", "ExiO Communications, Inc." },
    { "0002DB", "NETSEC" },
    { "0002DC", "Fujitsu General Limited" },
    { "0002DD", "Bromax Communications, Ltd." },
    { "0002DE", "Astrodesign, Inc." },
    { "0002DF", "Net Com Systems, Inc." },
    { "0002E0", "ETAS GmbH" },
    { "0002E1", "Integrated Network Corporation" },
    { "0002E2", "NDC Infared Engineering" },
    { "0002E3", "LITE-ON Communications, Inc." },
    { "0002E4", "JC HYUN Systems, Inc." },
    { "0002E5", "Timeware Ltd." },
    { "0002E6", "Gould Instrument Systems, Inc." },
    { "0002E7", "CAB GmbH & Co KG" },
    { "0002E8", "E.D.&A." },
    { "0002E9", "CS Systemes De Securite - C3S" },
    { "0002EA", "Focus Enhancements" },
    { "0002EB", "Pico Communications" },
    { "0002EC", "Maschoff Design Engineering" },
    { "0002ED", "DXO Telecom Co., Ltd." },
    { "0002EE", "Nokia Danmark A/S" },
    { "0002EF", "CCC Network Systems Group Ltd." },
    { "0002F0", "AME Optimedia Technology Co., Ltd." },
    { "0002F1", "Pinetron Co., Ltd." },
    { "0002F2", "eDevice, Inc." },
    { "0002F3", "Media Serve Co., Ltd." },
    { "0002F4", "PCTEL, Inc." },
    { "0002F5", "VIVE Synergies, Inc." },
    { "0002F6", "Equipe Communications" },
    { "0002F7", "ARM" },
    { "0002F8", "SEAKR Engineering, Inc." },
    { "0002F9", "MIMOS Berhad" },
    { "0002FA", "DX Antenna Co., Ltd." },
    { "0002FB", "Baumuller Aulugen-Systemtechnik GmbH" },
    { "0002FC", "Cisco Systems, Inc" },
    { "0002FD", "Cisco Systems, Inc" },
    { "0002FE", "Viditec, Inc." },
    { "0002FF", "Handan BroadInfoCom" },
    { "000300", "Barracuda Networks, Inc." },
    { "000301", "EXFO" },
    { "000302", "Charles Industries" },
    { "000303", "JAMA Electronics Co., Ltd." },
    { "000304", "Pacific Broadband Communications" },
    { "000305", "MSC Vertriebs GmbH" },
    { "000306", "Fusion In Tech Co., Ltd." },
    { "000307", "Secure Works, Inc." },
    { "000308", "AM Communications, Inc." },
    { "000309", "Texcel Technology PLC" },
    { "00030A", "Argus Technologies" },
    { "00030B", "Hunter Technology, Inc." },
    { "00030C", "Telesoft Technologies Ltd." },
    { "00030D", "Uniwill Computer Corp." },
    { "00030E", "Core Communications Co., Ltd." },
    { "00030F", "Digital China (Shanghai) Networks Ltd." },
    { "000310", "E-Globaledge Corporation" },
    { "000311", "Micro Technology Co., Ltd." },
    { "000312", "TRsystems GmbH" },
    { "000313", "Access Media SPA" },
    { "000314", "Teleware Network Systems" },
    { "000315", "Cidco Incorporated" },
    { "000316", "Nobell Communications, Inc." },
    { "000317", "Merlin Systems, Inc." },
    { "000318", "Cyras Systems, Inc." },
    { "000319", "Infineon AG" },
    { "00031A", "Beijing Broad Telecom Ltd., China" },
    { "00031B", "Cellvision Systems, Inc." },
    { "00031C", "Svenska Hardvarufabriken AB" },
    { "00031D", "Taiwan Commate Computer, Inc." },
    { "00031E", "Optranet, Inc." },
    { "00031F", "Condev Ltd." },
    { "000320", "Xpeed, Inc." },
    { "000321", "Reco Research Co., Ltd." },
    { "000322", "IDIS Co., Ltd." },
    { "000323", "Cornet Technology, Inc." },
    { "000324", "LIMNO Co., Ltd." },
    { "000325", "Arima Computer Corp." },
    { "000326", "Iwasaki Information Systems Co., Ltd." },
    { "000327", "HMS Industrial Networks" },
    { "000328", "Mace Group, Inc." },
    { "000329", "F3, Inc." },
    { "00032A", "UniData Communication Systems, Inc." },
    { "00032B", "GAI Datenfunksysteme GmbH" },
    { "00032C", "ABB Switzerland Ltd" },
    { "00032D", "IBASE Technology, Inc." },
    { "00032E", "Scope Information Management, Ltd." },
    { "00032F", "Global Sun Technology, Inc." },
    { "000330", "Imagenics, Co., Ltd." },
    { "000331", "Cisco Systems, Inc" },
    { "000332", "Cisco Systems, Inc" },
    { "000333", "Digitel Co., Ltd." },
    { "000334", "Omega Engineering Inc." },
    { "000335", "Mirae Technology" },
    { "000336", "Zetes Technologies" },
    { "000337", "Vaone, Inc." },
    { "000338", "Oak Technology" },
    { "000339", "Eurologic Systems, Ltd." },
    { "00033A", "Silicon Wave, Inc." },
    { "00033B", "TAMI Tech Co., Ltd." },
    { "00033C", "Daiden Co., Ltd." },
    { "00033D", "ILSHin Lab" },
    { "00033E", "Tateyama System Laboratory Co., Ltd." },
    { "00033F", "BigBand Networks, Ltd." },
    { "000340", "Floware Wireless Systems, Ltd." },
    { "000341", "EVS Broadcast Equipment" },
    { "000342", "Nortel Networks" },
    { "000343", "Martin Professional A/S" },
    { "000344", "Tietech.Co., Ltd." },
    { "000345", "Routrek Networks Corporation" },
    { "000346", "KOKUSAI DENKI Electric Inc." },
    { "000347", "Intel Corporation" },
    { "000348", "Norscan Instruments, Ltd." },
    { "000349", "Vidicode Datacommunicatie B.V." },
    { "00034A", "RIAS Corporation" },
    { "00034B", "Nortel Networks" },
    { "00034C", "Shanghai DigiVision Technology Co., Ltd." },
    { "00034D", "Chiaro Networks, Ltd." },
    { "00034E", "Pos Data Company, Ltd." },
    { "00034F", "Sur-Gard Security" },
    { "000350", "BTICINO SPA" },
    { "000351", "Diebold Nixdorf" },
    { "000352", "Colubris Networks" },
    { "000353", "Mitac, Inc." },
    { "000354", "Fiber Logic Communications" },
    { "000355", "TeraBeam Internet Systems" },
    { "000356", "Diebold Nixdorf" },
    { "000357", "Intervoice-Brite, Inc." },
    { "000358", "Hanyang Digitech Co.Ltd" },
    { "000359", "DigitalSis" },
    { "00035A", "Photron Limited" },
    { "00035B", "BridgeWave Communications" },
    { "00035C", "Saint Song Corp." },
    { "00035D", "Bosung Hi-Net Co., Ltd." },
    { "00035E", "Metropolitan Area Networks, Inc." },
    { "00035F", "Prüftechnik Condition Monitoring GmbH & Co. KG" },
    { "000360", "PAC Interactive Technology, Inc." },
    { "000361", "Widcomm, Inc." },
    { "000362", "Vodtel Communications, Inc." },
    { "000363", "Miraesys Co., Ltd." },
    { "000364", "Scenix Semiconductor, Inc." },
    { "000365", "Kira Information & Communications, Ltd." },
    { "000366", "ASM Pacific Technology" },
    { "000367", "Jasmine Networks, Inc." },
    { "000368", "Embedone Co., Ltd." },
    { "000369", "Nippon Antenna Co., Ltd." },
    { "00036A", "Mainnet, Ltd." },
    { "00036B", "Cisco Systems, Inc" },
    { "00036C", "Cisco Systems, Inc" },
    { "00036D", "Runtop, Inc." },
    { "00036E", "Nicon Systems (Pty) Limited" },
    { "00036F", "Telsey SPA" },
    { "000370", "NXTV, Inc." },
    { "000371", "Acomz Networks Corp." },
    { "000372", "ULAN" },
    { "000373", "Aselsan A.S" },
    { "000374", "Control Microsystems" },
    { "000375", "NetMedia, Inc." },
    { "000376", "Graphtec Technology, Inc." },
    { "000377", "Gigabit Wireless" },
    { "000378", "HUMAX Co., Ltd." },
    { "000379", "Proscend Communications, Inc." },
    { "00037A", "Taiyo Yuden Co., Ltd." },
    { "00037B", "IDEC IZUMI Corporation" },
    { "00037C", "Coax Media" },
    { "00037D", "Stellcom" },
    { "00037E", "PORTech Communications, Inc." },
    { "00037F", "Atheros Communications, Inc." },
    { "000380", "SSH Communications Security Corp." },
    { "000381", "Ingenico International" },
    { "000382", "A-One Co., Ltd." },
    { "000383", "Metera Networks, Inc." },
    { "000384", "AETA" },
    { "000385", "Actelis Networks, Inc." },
    { "000386", "Ho Net, Inc." },
    { "000387", "Blaze Network Products" },
    { "000388", "Fastfame Technology Co., Ltd." },
    { "000389", "PLANTRONICS, INC." },
    { "00038A", "America Online, Inc." },
    { "00038B", "PLUS-ONE I&T, Inc." },
    { "00038C", "Total Impact" },
    { "00038D", "PCS Revenue Control Systems, Inc." },
    { "00038E", "Atoga Systems, Inc." },
    { "00038F", "Weinschel Corporation" },
    { "000390", "Digital Video Communications, Inc." },
    { "000391", "Advanced Digital Broadcast, Ltd." },
    { "000392", "Hyundai Teletek Co., Ltd." },
    { "000393", "Apple, Inc." },
    { "000394", "Connect One" },
    { "000395", "California Amplifier" },
    { "000396", "EZ Cast Co., Ltd." },
    { "000397", "FireBrick Limited" },
    { "000398", "WISI" },
    { "000399", "Dongju Informations & Communications Co., Ltd." },
    { "00039A", "SiConnect" },
    { "00039B", "NetChip Technology, Inc." },
    { "00039C", "OptiMight Communications, Inc." },
    { "00039D", "Qisda Corporation" },
    { "00039E", "Tera System Co., Ltd." },
    { "00039F", "Cisco Systems, Inc" },
    { "0003A0", "Cisco Systems, Inc" },
    { "0003A1", "HIPER Information & Communication, Inc." },
    { "0003A2", "Catapult Communications" },
    { "0003A3", "MAVIX, Ltd." },
    { "0003A4", "Imation Corp." },
    { "0003A5", "Medea Corporation" },
    { "0003A6", "Traxit Technology, Inc." },
    { "0003A7", "Unixtar Technology, Inc." },
    { "0003A8", "IDOT Computers, Inc." },
    { "0003A9", "AXCENT Media AG" },
    { "0003AA", "Watlow" },
    { "0003AB", "Bridge Information Systems" },
    { "0003AC", "Fronius Schweissmaschinen" },
    { "0003AD", "Emerson Energy Systems AB" },
    { "0003AE", "Allied Advanced Manufacturing Pte, Ltd." },
    { "0003AF", "Paragea Communications" },
    { "0003B0", "Xsense Technology Corp." },
    { "0003B1", "ICU Medical, Inc." },
    { "0003B2", "Radware" },
    { "0003B3", "IA Link Systems Co., Ltd." },
    { "0003B4", "Macrotek International Corp." },
    { "0003B5", "Entra Technology Co." },
    { "0003B6", "QSI Corporation" },
    { "0003B7", "ZACCESS Systems" },
    { "0003B8", "NetKit Solutions, LLC" },
    { "0003B9", "Hualong Telecom Co., Ltd." },
    { "0003BA", "Oracle Corporation" },
    { "0003BB", "Signal Communications Limited" },
    { "0003BC", "COT GmbH" },
    { "0003BD", "OmniCluster Technologies, Inc." },
    { "0003BE", "Netility" },
    { "0003BF", "Centerpoint Broadband Technologies, Inc." },
    { "0003C0", "RFTNC Co., Ltd." },
    { "0003C1", "Packet Dynamics Ltd" },
    { "0003C2", "Solphone K.K." },
    { "0003C3", "Micronik Multimedia" },
    { "0003C4", "Tomra Systems ASA" },
    { "0003C5", "Mobotix AG" },
    { "0003C6", "ICUE Systems, Inc." },
    { "0003C7", "hopf Elektronik GmbH" },
    { "0003C8", "CML Emergency Services" },
    { "0003C9", "TECOM Co., Ltd." },
    { "0003CA", "MTS Systems Corp." },
    { "0003CB", "SystemGear Co., Ltd." },
    { "0003CC", "Momentum Computer, Inc." },
    { "0003CD", "Clovertech, Inc." },
    { "0003CE", "ETEN Technologies, Inc." },
    { "0003CF", "Muxcom, Inc." },
    { "0003D0", "KOANKEISO Co., Ltd." },
    { "0003D1", "Takaya Corporation" },
    { "0003D2", "Crossbeam Systems, Inc." },
    { "0003D3", "Internet Energy Systems, Inc." },
    { "0003D4", "Alloptic, Inc." },
    { "0003D5", "Advanced Communications Co., Ltd." },
    { "0003D6", "RADVision, Ltd." },
    { "0003D7", "NextNet Wireless, Inc." },
    { "0003D8", "iMPath Networks, Inc." },
    { "0003D9", "Secheron SA" },
    { "0003DA", "Takamisawa Cybernetics Co., Ltd." },
    { "0003DB", "Apogee Electronics Corp." },
    { "0003DC", "Lexar Media, Inc." },
    { "0003DD", "Comark Interactive Solutions" },
    { "0003DE", "OTC Wireless" },
    { "0003DF", "Desana Systems" },
    { "0003E0", "Commscope" },
    { "0003E1", "Winmate Communication, Inc." },
    { "0003E2", "Comspace Corporation" },
    { "0003E3", "Cisco Systems, Inc" },
    { "0003E4", "Cisco Systems, Inc" },
    { "0003E5", "Hermstedt SG" },
    { "0003E6", "Entone, Inc." },
    { "0003E7", "Logostek Co. Ltd." },
    { "0003E8", "Wavesight Limited" },
    { "0003E9", "Akara Canada, Inc." },
    { "0003EA", "Mega System Technologies, Inc." },
    { "0003EB", "Atrica" },
    { "0003EC", "ICG Research, Inc." },
    { "0003ED", "Shinkawa Electric Co., Ltd." },
    { "0003EE", "MKNet Corporation" },
    { "0003EF", "Oneline AG" },
    { "0003F0", "Redfern Broadband Networks" },
    { "0003F1", "Cicada Semiconductor, Inc." },
    { "0003F2", "Seneca Networks" },
    { "0003F3", "Dazzle Multimedia, Inc." },
    { "0003F4", "NetBurner" },
    { "0003F5", "Chip2Chip" },
    { "0003F6", "Allegro Networks, Inc." },
    { "0003F7", "Plast-Control GmbH" },
    { "0003F8", "SanCastle Technologies, Inc." },
    { "0003F9", "Pleiades Communications, Inc." },
    { "0003FA", "TiMetra Networks" },
    { "0003FB", "ENEGATE Co.,Ltd." },
    { "0003FC", "Intertex Data AB" },
    { "0003FD", "Cisco Systems, Inc" },
    { "0003FE", "Cisco Systems, Inc" },
    { "0003FF", "Microsoft Corporation" },
    { "000400", "LEXMARK INTERNATIONAL, INC." },
    { "000401", "Osaki Electric Co., Ltd." },
    { "000402", "Nexsan Technologies, Ltd." },
    { "000403", "Nexsi Corporation" },
    { "000404", "Makino Milling Machine Co., Ltd." },
    { "000405", "ACN Technologies" },
    { "000406", "Fa. Metabox AG" },
    { "000407", "Topcon Positioning Systems, Inc." },
    { "000408", "Sanko Electronics Co., Ltd." },
    { "000409", "Cratos Networks" },
    { "00040A", "Sage Systems" },
    { "00040B", "3COM EUROPE LTD" },
    { "00040C", "Kanno Works, Ltd." },
    { "00040D", "Avaya Inc" },
    { "00040E", "AVM GmbH" },
    { "00040F", "Asus Network Technologies, Inc." },
    { "000410", "Spinnaker Networks, Inc." },
    { "000411", "Inkra Networks, Inc." },
    { "000412", "WaveSmith Networks, Inc." },
    { "000413", "snom technology GmbH" },
    { "000414", "Umezawa Musen Denki Co., Ltd." },
    { "000415", "Rasteme Systems Co., Ltd." },
    { "000416", "Parks S/A Comunicacoes Digitais" },
    { "000417", "Schneider Electric" },
    { "000418", "Teltronic S.A.U." },
    { "000419", "Fibercycle Networks, Inc." },
    { "00041A", "Ines Test and Measurement GmbH & CoKG" },
    { "00041B", "Bridgeworks Ltd." },
    { "00041C", "ipDialog, Inc." },
    { "00041D", "Corega of America" },
    { "00041E", "Shikoku Instrumentation Co., Ltd." },
    { "00041F", "Sony Interactive Entertainment Inc." },
    { "000420", "Slim Devices, Inc." },
    { "000421", "Ocular Networks" },
    { "000422", "Studio Technologies, Inc" },
    { "000423", "Intel Corporation" },
    { "000424", "TMC s.r.l." },
    { "000425", "Atmel Corporation" },
    { "000426", "Autosys" },
    { "000427", "Cisco Systems, Inc" },
    { "000428", "Cisco Systems, Inc" },
    { "000429", "Pixord Corporation" },
    { "00042A", "Wireless Networks, Inc." },
    { "00042B", "IT Access Co., Ltd." },
    { "00042C", "Minet, Inc." },
    { "00042D", "Sarian Systems, Ltd." },
    { "00042E", "Netous Technologies, Ltd." },
    { "00042F", "International Communications Products, Inc." },
    { "000430", "Netgem" },
    { "000431", "GlobalStreams, Inc." },
    { "000432", "Voyetra Turtle Beach, Inc." },
    { "000433", "Cyberboard A/S" },
    { "000434", "Accelent Systems, Inc." },
    { "000435", "InfiNet LLC" },
    { "000436", "ELANsat Technologies, Inc." },
    { "000437", "Powin Information Technology, Inc." },
    { "000438", "Nortel Networks" },
    { "000439", "Rosco Entertainment Technology, Inc." },
    { "00043A", "Intelligent Telecommunications, Inc." },
    { "00043B", "Lava Computer Mfg., Inc." },
    { "00043C", "SONOS Co., Ltd." },
    { "00043D", "INDEL AG" },
    { "00043E", "Telencomm" },
    { "00043F", "ESTeem Wireless Modems, Inc" },
    { "000440", "cyberPIXIE, Inc." },
    { "000441", "Half Dome Systems, Inc." },
    { "000442", "NACT" },
    { "000443", "Agilent Technologies, Inc." },
    { "000444", "Western Multiplex Corporation" },
    { "000445", "LMS Skalar Instruments GmbH" },
    { "000446", "CYZENTECH Co., Ltd." },
    { "000447", "Acrowave Systems Co., Ltd." },
    { "000448", "Polaroid Corporation" },
    { "000449", "Mapletree Networks" },
    { "00044A", "iPolicy Networks, Inc." },
    { "00044B", "NVIDIA" },
    { "00044C", "JENOPTIK" },
    { "00044D", "Cisco Systems, Inc" },
    { "00044E", "Cisco Systems, Inc" },
    { "00044F", "Schubert System Elektronik Gmbh" },
    { "000450", "DMD Computers SRL" },
    { "000451", "Medrad, Inc." },
    { "000452", "RocketLogix, Inc." },
    { "000453", "YottaYotta, Inc." },
    { "000454", "Quadriga UK" },
    { "000455", "ANTARA.net" },
    { "000456", "Cambium Networks Limited" },
    { "000457", "Universal Access Technology, Inc." },
    { "000458", "Fusion X Co., Ltd." },
    { "000459", "Veristar Corporation" },
    { "00045A", "The Linksys Group, Inc." },
    { "00045B", "Techsan Electronics Co., Ltd." },
    { "00045C", "Mobiwave Pte Ltd" },
    { "00045D", "BEKA Elektronik" },
    { "00045E", "PolyTrax Information Technology AG" },
    { "00045F", "Avalue Technology, Inc." },
    { "000460", "Knilink Technology, Inc." },
    { "000461", "EPOX Computer Co., Ltd." },
    { "000462", "DAKOS Data & Communication Co., Ltd." },
    { "000463", "Bosch Security Systems" },
    { "000464", "Pulse-Link Inc" },
    { "000465", "i.s.t isdn-support technik GmbH" },
    { "000466", "ARMITEL Co." },
    { "000467", "Wuhan Research Institute of MII" },
    { "000468", "Vivity, Inc." },
    { "000469", "Innocom, Inc." },
    { "00046A", "Navini Networks" },
    { "00046B", "Palm Wireless, Inc." },
    { "00046C", "Cyber Technology Co., Ltd." },
    { "00046D", "Cisco Systems, Inc" },
    { "00046E", "Cisco Systems, Inc" },
    { "00046F", "Digitel S/A Industria Eletronica" },
    { "000470", "ipUnplugged AB" },
    { "000471", "IPrad" },
    { "000472", "Telelynx, Inc." },
    { "000473", "Photonex Corporation" },
    { "000474", "LEGRAND" },
    { "000475", "3COM" },
    { "000476", "3COM" },
    { "000477", "Scalant Systems, Inc." },
    { "000478", "G. Star Technology Corporation" },
    { "000479", "Radius Co., Ltd." },
    { "00047A", "AXXESSIT ASA" },
    { "00047B", "Schlumberger" },
    { "00047C", "Skidata AG" },
    { "00047D", "Motorola Solutions Inc." },
    { "00047E", "TKH Security B.V." },
    { "00047F", "Chr. Mayr GmbH & Co. KG" },
    { "000480", "Brocade Communications Systems LLC" },
    { "000481", "Econolite Control Products, Inc." },
    { "000482", "Medialogic Corp." },
    { "000483", "Deltron Technology, Inc." },
    { "000484", "Amann GmbH" },
    { "000485", "PicoLight" },
    { "000486", "ITTC, University of Kansas" },
    { "000487", "Cogency Semiconductor, Inc." },
    { "000488", "Eurotherm Controls" },
    { "000489", "YAFO Networks, Inc." },
    { "00048A", "Temia Vertriebs GmbH" },
    { "00048B", "Poscon Corporation" },
    { "00048C", "Nayna Networks, Inc." },
    { "00048D", "Teo Technologies, Inc" },
    { "00048E", "Ohm Tech Labs, Inc." },
    { "00048F", "TD Systems Corporation" },
    { "000490", "Optical Access" },
    { "000491", "Technovision, Inc." },
    { "000492", "Hive Internet, Ltd." },
    { "000493", "Tsinghua Unisplendour Co., Ltd." },
    { "000494", "Breezecom, Ltd." },
    { "000495", "Tejas Networks India Limited" },
    { "000496", "Extreme Networks Headquarters" },
    { "000497", "MacroSystem Digital Video AG" },
    { "000498", "Mahi Networks" },
    { "000499", "Chino Corporation" },
    { "00049A", "Cisco Systems, Inc" },
    { "00049B", "Cisco Systems, Inc" },
    { "00049C", "Surgient Networks, Inc." },
    { "00049D", "Ipanema Technologies" },
    { "00049E", "Wirelink Co., Ltd." },
    { "00049F", "Freescale Semiconductor" },
    { "0004A0", "Verity Instruments, Inc." },
    { "0004A1", "Pathway Connectivity" },
    { "0004A2", "L.S.I. Japan Co., Ltd." },
    { "0004A3", "Microchip Technology Inc." },
    { "0004A4", "NetEnabled, Inc." },
    { "0004A5", "Barco NV" },
    { "0004A6", "SAF Tehnika Ltd." },
    { "0004A7", "FabiaTech Corporation" },
    { "0004A8", "Broadmax Technologies, Inc." },
    { "0004A9", "SandStream Technologies, Inc." },
    { "0004AA", "Jetstream Communications" },
    { "0004AB", "Mavenir Inc." },
    { "0004AC", "IBM Corp" },
    { "0004AD", "Malibu Networks" },
    { "0004AE", "Sullair Corporation" },
    { "0004AF", "Digital Fountain, Inc." },
    { "0004B0", "ELESIGN Co., Ltd." },
    { "0004B1", "Signal Technology, Inc." },
    { "0004B2", "ESSEGI SRL" },
    { "0004B3", "Videotek, Inc." },
    { "0004B4", "CIAC" },
    { "0004B5", "Equitrac Corporation" },
    { "0004B6", "Stratex Networks, Inc." },
    { "0004B7", "AMB i.t. Holding" },
    { "0004B8", "Kumahira Co., Ltd." },
    { "0004B9", "S.I. Soubou, Inc." },
    { "0004BA", "KDD Media Will Corporation" },
    { "0004BB", "Bardac Corporation" },
    { "0004BC", "Giantec, Inc." },
    { "0004BD", "Commscope" },
    { "0004BE", "OptXCon, Inc." },
    { "0004BF", "VersaLogic Corp." },
    { "0004C0", "Cisco Systems, Inc" },
    { "0004C1", "Cisco Systems, Inc" },
    { "0004C2", "Magnipix, Inc." },
    { "0004C3", "CASTOR Informatique" },
    { "0004C4", "Audiotonix Group Limited" },
    { "0004C5", "ASE Technologies, USA" },
    { "0004C6", "YAMAHA MOTOR CO.,LTD" },
    { "0004C7", "NetMount" },
    { "0004C8", "LIBA Maschinenfabrik GmbH" },
    { "0004C9", "Micro Electron Co., Ltd." },
    { "0004CA", "FreeMs Corp." },
    { "0004CB", "Tdsoft Communication, Ltd." },
    { "0004CC", "Peek Traffic B.V." },
    { "0004CD", "Extenway Solutions Inc" },
    { "0004CE", "Patria Ailon" },
    { "0004CF", "Seagate Technology" },
    { "0004D0", "Softlink s.r.o." },
    { "0004D1", "Drew Technologies, Inc." },
    { "0004D2", "Adcon Telemetry GmbH" },
    { "0004D3", "Toyokeiki Co., Ltd." },
    { "0004D4", "Proview Electronics Co., Ltd." },
    { "0004D5", "Hitachi Information & Communication Engineering, Ltd." },
    { "0004D6", "Takagi Industrial Co., Ltd." },
    { "0004D7", "Omitec Instrumentation Ltd." },
    { "0004D8", "IPWireless, Inc." },
    { "0004D9", "Titan Electronics, Inc." },
    { "0004DA", "Relax Technology, Inc." },
    { "0004DB", "Tellus Group Corp." },
    { "0004DC", "Nortel Networks" },
    { "0004DD", "Cisco Systems, Inc" },
    { "0004DE", "Cisco Systems, Inc" },
    { "0004DF", "TERACOM TELEMATICA S.A" },
    { "0004E0", "Procket Networks" },
    { "0004E1", "Infinior Microsystems" },
    { "0004E2", "SMC Networks, Inc." },
    { "0004E3", "Accton Technology Corp" },
    { "0004E4", "Daeryung Ind., Inc." },
    { "0004E5", "Glonet Systems, Inc." },
    { "0004E6", "Banyan Network Private Limited" },
    { "0004E7", "Lightpointe Communications, Inc" },
    { "0004E8", "IER, Inc." },
    { "0004E9", "Infiniswitch Corporation" },
    { "0004EA", "Hewlett Packard" },
    { "0004EB", "Paxonet Communications, Inc." },
    { "0004EC", "Memobox SA" },
    { "0004ED", "Billion Electric Co., Ltd." },
    { "0004EE", "Lincoln Electric Company" },
    { "0004EF", "Polestar Corp." },
    { "0004F0", "International Computers, Ltd" },
    { "0004F1", "WhereNet" },
    { "0004F2", "Polycom" },
    { "0004F3", "FS FORTH-SYSTEME GmbH" },
    { "0004F4", "Infinite Electronics Inc." },
    { "0004F5", "SnowShore Networks, Inc." },
    { "0004F6", "Amphus" },
    { "0004F7", "Omega Band, Inc." },
    { "0004F8", "QUALICABLE TV Industria E Com., Ltda" },
    { "0004F9", "Xtera Communications, Inc." },
    { "0004FA", "NBS Technologies Inc." },
    { "0004FB", "Commtech, Inc." },
    { "0004FC", "Stratus Technologies" },
    { "0004FD", "Japan Control Engineering Co., Ltd." },
    { "0004FE", "Pelago Networks" },
    { "0004FF", "Acronet Co., Ltd." },
    { "000500", "Cisco Systems, Inc" },
    { "000501", "Cisco Systems, Inc" },
    { "000502", "Apple, Inc." },
    { "000503", "ICONAG" },
    { "000504", "Naray Information & Communication Enterprise" },
    { "000505", "Systems Integration Solutions, Inc." },
    { "000506", "Reddo Networks AB" },
    { "000507", "Fine Appliance Corp." },
    { "000508", "Inetcam, Inc." },
    { "000509", "AVOC Nishimura Ltd." },
    { "00050A", "ICS Spa" },
    { "00050B", "SICOM Systems, Inc." },
    { "00050C", "Network Photonics, Inc." },
    { "00050D", "Midstream Technologies, Inc." },
    { "00050E", "3ware, Inc." },
    { "00050F", "Tanaka S/S Ltd." },
    { "000510", "Infinite Shanghai Communication Terminals Ltd." },
    { "000511", "Complementary Technologies Ltd" },
    { "000512", "Zebra Technologies Inc" },
    { "000513", "VTLinx Multimedia Systems, Inc." },
    { "000514", "KDT Systems Co., Ltd." },
    { "000515", "Nuark Co., Ltd." },
    { "000516", "SMART Modular Technologies" },
    { "000517", "Shellcomm, Inc." },
    { "000518", "Jupiters Technology" },
    { "000519", "Siemens Building Technologies AG," },
    { "00051A", "3COM EUROPE LTD" },
    { "00051B", "Magic Control Technology Corporation" },
    { "00051C", "Xnet Technology Corp." },
    { "00051D", "Airocon, Inc." },
    { "00051E", "Brocade Communications Systems LLC" },
    { "00051F", "Taijin Media Co., Ltd." },
    { "000520", "Smartronix, Inc." },
    { "000521", "Control Microsystems" },
    { "000522", "LEA*D Corporation, Inc." },
    { "000523", "AVL List GmbH" },
    { "000524", "BTL System (HK) Limited" },
    { "000525", "Puretek Industrial Co., Ltd." },
    { "000526", "IPAS GmbH" },
    { "000527", "SJ Tek Co. Ltd" },
    { "000528", "New Focus, Inc." },
    { "000529", "Shanghai Broadan Communication Technology Co., Ltd" },
    { "00052A", "Ikegami Tsushinki Co., Ltd." },
    { "00052B", "HORIBA, Ltd." },
    { "00052C", "Supreme Magic Corporation" },
    { "00052D", "Zoltrix International Limited" },
    { "00052E", "Cinta Networks" },
    { "00052F", "Leviton Network Solutions" },
    { "000530", "Andiamo Systems, Inc." },
    { "000531", "Cisco Systems, Inc" },
    { "000532", "Cisco Systems, Inc" },
    { "000533", "Brocade Communications Systems LLC" },
    { "000534", "Northstar Engineering Ltd." },
    { "000535", "Chip PC Ltd." },
    { "000536", "Danam Communications, Inc." },
    { "000537", "Nets Technology Co., Ltd." },
    { "000538", "Merilus, Inc." },
    { "000539", "A Brand New World in Sweden AB" },
    { "00053A", "Willowglen Services Pte Ltd" },
    { "00053B", "Harbour Networks Ltd., Co. Beijing" },
    { "00053C", "XIRCOM" },
    { "00053D", "Agere Systems" },
    { "00053E", "KID Systeme GmbH" },
    { "00053F", "VisionTek, Inc." },
    { "000540", "Tokyo Electron Device Limited" },
    { "000541", "Advanced Systems Co., Ltd." },
    { "000542", "Otari, Inc." },
    { "000543", "IQ Wireless GmbH" },
    { "000544", "Valley Technologies, Inc." },
    { "000545", "Internet Photonics" },
    { "000546", "KDDI Network & Solultions Inc." },
    { "000547", "Starent Networks" },
    { "000548", "Disco Corporation" },
    { "000549", "Salira Optical Network Systems" },
    { "00054A", "Ario Data Networks, Inc." },
    { "00054B", "Eaton Automation AG" },
    { "00054C", "RF Innovations Pty Ltd" },
    { "00054D", "Brans Technologies, Inc." },
    { "00054E", "Philips" },
    { "00054F", "Garmin International" },
    { "000550", "Vcomms Connect Limited" },
    { "000551", "F & S Elektronik Systeme GmbH" },
    { "000552", "Xycotec Computer GmbH" },
    { "000553", "DVC Company, Inc." },
    { "000554", "Rangestar Wireless" },
    { "000555", "Japan Cash Machine Co., Ltd." },
    { "000556", "360 Systems" },
    { "000557", "Agile TV Corporation" },
    { "000558", "Synchronous, Inc." },
    { "000559", "Intracom S.A." },
    { "00055A", "Power Dsine Ltd." },
    { "00055B", "Charles Industries" },
    { "00055C", "Kowa Company, Ltd." },
    { "00055D", "D-LINK SYSTEMS, INC." },
    { "00055E", "Cisco Systems, Inc" },
    { "00055F", "Cisco Systems, Inc" },
    { "000560", "LEADER COMM.CO., LTD" },
    { "000561", "nac Image Technology, Inc." },
    { "000562", "Digital View Limited" },
    { "000563", "J-Works, Inc." },
    { "000564", "Tsinghua Bitway Co., Ltd." },
    { "000565", "Tailyn Communication Company Ltd." },
    { "000566", "Secui.com Corporation" },
    { "000567", "Etymonic Design, Inc." },
    { "000568", "Piltofish Networks AB" },
    { "000569", "VMware, Inc." },
    { "00056A", "Heuft Systemtechnik GmbH" },
    { "00056B", "C.P. Technology Co., Ltd." },
    { "00056C", "Hung Chang Co., Ltd." },
    { "00056D", "Pacific Corporation" },
    { "00056E", "National Enhance Technology, Inc." },
    { "00056F", "Innomedia Technologies Pvt. Ltd." },
    { "000570", "Baydel Ltd." },
    { "000571", "Seiwa Electronics Co." },
    { "000572", "Deonet Co., Ltd." },
    { "000573", "Cisco Systems, Inc" },
    { "000574", "Cisco Systems, Inc" },
    { "000575", "CDS-Electronics BV" },
    { "000576", "NSM Technology Ltd." },
    { "000577", "SM Information & Communication" },
    { "000578", "Private" },
    { "000579", "Universal Control Solution Corp." },
    { "00057A", "Overture Networks" },
    { "00057B", "Chung Nam Electronic Co., Ltd." },
    { "00057C", "RCO Security AB" },
    { "00057D", "Sun Communications, Inc." },
    { "00057E", "Eckelmann AG" },
    { "00057F", "Acqis Technology" },
    { "000580", "FibroLAN Ltd." },
    { "000581", "Snell" },
    { "000582", "ClearCube Technology" },
    { "000583", "ImageCom Limited" },
    { "000584", "AbsoluteValue Systems, Inc." },
    { "000585", "Juniper Networks" },
    { "000586", "Lucent Technologies" },
    { "000587", "Locus, Incorporated" },
    { "000588", "Sensoria Corp." },
    { "000589", "National Datacomputer" },
    { "00058A", "Netcom Co., Ltd." },
    { "00058B", "IPmental, Inc." },
    { "00058C", "Opentech Inc." },
    { "00058D", "Lynx Photonic Networks, Inc." },
    { "00058E", "Flextronics International GmbH & Co. Nfg. KG" },
    { "00058F", "CLCsoft co." },
    { "000590", "Swissvoice Ltd." },
    { "000591", "Active Silicon Ltd" },
    { "000592", "Pultek Corp." },
    { "000593", "Grammar Engine Inc." },
    { "000594", "HMS Industrial Networks" },
    { "000595", "Alesis Corporation" },
    { "000596", "Genotech Co., Ltd." },
    { "000597", "Eagle Traffic Control Systems" },
    { "000598", "CRONOS S.r.l." },
    { "000599", "DRS Test and Energy Management or DRS-TEM" },
    { "00059A", "Cisco Systems, Inc" },
    { "00059B", "Cisco Systems, Inc" },
    { "00059C", "Kleinknecht GmbH, Ing. Büro" },
    { "00059D", "Daniel Computing Systems, Inc." },
    { "00059E", "Zinwell Corporation" },
    { "00059F", "Yotta Networks, Inc." },
    { "0005A0", "MOBILINE Kft." },
    { "0005A1", "Zenocom" },
    { "0005A2", "CELOX Networks" },
    { "0005A3", "QEI, Inc." },
    { "0005A4", "Lucid Voice Ltd." },
    { "0005A5", "KOTT" },
    { "0005A6", "Extron Electronics" },
    { "0005A7", "HYPERCHIP Inc." },
    { "0005A8", "WYLE ELECTRONICS" },
    { "0005A9", "Princeton Networks, Inc." },
    { "0005AA", "Moore Industries International Inc." },
    { "0005AB", "Cyber Fone, Inc." },
    { "0005AC", "Northern Digital, Inc." },
    { "0005AD", "Topspin Communications, Inc." },
    { "0005AE", "Mediaport USA" },
    { "0005AF", "InnoScan Computing A/S" },
    { "0005B0", "Korea Computer Technology Co., Ltd." },
    { "0005B1", "ASB Technology BV" },
    { "0005B2", "Medison Co., Ltd." },
    { "0005B3", "Asahi-Engineering Co., Ltd." },
    { "0005B4", "Aceex Corporation" },
    { "0005B5", "Broadcom Technologies" },
    { "0005B6", "INSYS Microelectronics GmbH" },
    { "0005B7", "Arbor Technology Corp." },
    { "0005B8", "Electronic Design Associates, Inc." },
    { "0005B9", "Airvana, Inc." },
    { "0005BA", "Area Netwoeks, Inc." },
    { "0005BB", "Myspace AB" },
    { "0005BC", "Resource Data Management Ltd" },
    { "0005BD", "ROAX BV" },
    { "0005BE", "Kongsberg Seatex AS" },
    { "0005BF", "JustEzy Technology, Inc." },
    { "0005C0", "Digital Network Alacarte Co., Ltd." },
    { "0005C1", "A-Kyung Motion, Inc." },
    { "0005C2", "Soronti, Inc." },
    { "0005C3", "Pacific Instruments, Inc." },
    { "0005C4", "Telect, Inc." },
    { "0005C5", "Flaga HF" },
    { "0005C6", "Triz Communications" },
    { "0005C7", "I/F-COM A/S" },
    { "0005C8", "VERYTECH" },
    { "0005C9", "LG Innotek" },
    { "0005CA", "Hitron Technology, Inc." },
    { "0005CB", "ROIS Technologies, Inc." },
    { "0005CC", "Sumtel Communications, Inc." },
    { "0005CD", "D&M Holdings Inc." },
    { "0005CE", "Prolink Microsystems Corporation" },
    { "0005CF", "Thunder River Technologies, Inc." },
    { "0005D0", "Solinet Systems" },
    { "0005D1", "Metavector Technologies" },
    { "0005D2", "DAP Technologies" },
    { "0005D3", "eProduction Solutions, Inc." },
    { "0005D4", "FutureSmart Networks, Inc." },
    { "0005D5", "Speedcom Wireless" },
    { "0005D6", "L-3 Linkabit" },
    { "0005D7", "Vista Imaging, Inc." },
    { "0005D8", "Arescom, Inc." },
    { "0005D9", "Techno Valley, Inc." },
    { "0005DA", "Apex Automationstechnik" },
    { "0005DB", "PSI Nentec GmbH" },
    { "0005DC", "Cisco Systems, Inc" },
    { "0005DD", "Cisco Systems, Inc" },
    { "0005DE", "Gi Fone Korea, Inc." },
    { "0005DF", "Electronic Innovation, Inc." },
    { "0005E0", "Empirix Corp." },
    { "0005E1", "Trellis Photonics, Ltd." },
    { "0005E2", "Creativ Network Technologies" },
    { "0005E3", "LightSand Communications, Inc." },
    { "0005E4", "Red Lion Controls Inc." },
    { "0005E5", "Renishaw PLC" },
    { "0005E6", "Egenera, Inc." },
    { "0005E7", "Netrake an AudioCodes Company" },
    { "0005E8", "TurboWave, Inc." },
    { "0005E9", "Unicess Network, Inc." },
    { "0005EA", "Rednix" },
    { "0005EB", "Blue Ridge Networks, Inc." },
    { "0005EC", "Mosaic Systems Inc." },
    { "0005ED", "Technikum Joanneum GmbH" },
    { "0005EE", "Vanderbilt International (SWE) AB " },
    { "0005EF", "ADOIR Digital Technology" },
    { "0005F0", "SATEC" },
    { "0005F1", "Vrcom, Inc." },
    { "0005F2", "Power R, Inc." },
    { "0005F3", "Webyn" },
    { "0005F4", "System Base Co., Ltd." },
    { "0005F5", "Geospace Technologies" },
    { "0005F6", "Young Chang Co. Ltd." },
    { "0005F7", "Analog Devices, Inc." },
    { "0005F8", "Real Time Access, Inc." },
    { "0005F9", "TOA Corporation" },
    { "0005FA", "IPOptical, Inc." },
    { "0005FB", "ShareGate, Inc." },
    { "0005FC", "Schenck Pegasus Corp." },
    { "0005FD", "PacketLight Networks Ltd." },
    { "0005FE", "Traficon N.V." },
    { "0005FF", "SNS Solutions, Inc." },
    { "000600", "Toshiba Teli Corporation" },
    { "000601", "Otanikeiki Co., Ltd." },
    { "000602", "Cirkitech Electronics Co." },
    { "000603", "Baker Hughes Inc." },
    { "000604", "@Track Communications, Inc." },
    { "000605", "Inncom International, Inc." },
    { "000606", "RapidWAN, Inc." },
    { "000607", "Omni Directional Control Technology Inc." },
    { "000608", "At-Sky SAS" },
    { "000609", "Crossport Systems" },
    { "00060A", "Blue2space" },
    { "00060B", "Artesyn Embedded Technologies" },
    { "00060C", "Melco Industries, Inc." },
    { "00060D", "Wave7 Optics" },
    { "00060E", "IGYS Systems, Inc." },
    { "00060F", "Narad Networks Inc" },
    { "000610", "Abeona Networks Inc" },
    { "000611", "Zeus Wireless, Inc." },
    { "000612", "Accusys, Inc." },
    { "000613", "Kawasaki Microelectronics Incorporated" },
    { "000614", "Prism Holdings" },
    { "000615", "Kimoto Electric Co., Ltd." },
    { "000616", "Tel Net Co., Ltd." },
    { "000617", "Redswitch Inc." },
    { "000618", "DigiPower Manufacturing Inc." },
    { "000619", "Connection Technology Systems" },
    { "00061A", "Zetari Inc." },
    { "00061B", "Notebook Development Lab. Lenovo Japan Ltd." },
    { "00061C", "Hoshino Metal Industries, Ltd." },
    { "00061D", "MIP Telecom, Inc." },
    { "00061E", "Maxan Systems" },
    { "00061F", "Vision Components GmbH" },
    { "000620", "Serial System Ltd." },
    { "000621", "Hinox, Co., Ltd." },
    { "000622", "Chung Fu Chen Yeh Enterprise Corp." },
    { "000623", "MGE UPS Systems France" },
    { "000624", "Gentner Communications Corp." },
    { "000625", "The Linksys Group, Inc." },
    { "000626", "MWE GmbH" },
    { "000627", "Uniwide Technologies, Inc." },
    { "000628", "Cisco Systems, Inc" },
    { "000629", "IBM Corp" },
    { "00062A", "Cisco Systems, Inc" },
    { "00062B", "INTRASERVER TECHNOLOGY" },
    { "00062C", "Bivio Networks" },
    { "00062D", "TouchStar Technologies, L.L.C." },
    { "00062E", "Aristos Logic Corp." },
    { "00062F", "Pivotech Systems Inc." },
    { "000630", "Adtranz Sweden" },
    { "000631", "Calix Inc." },
    { "000632", "Mesco Engineering GmbH" },
    { "000633", "Crossmatch Technologies/HID Global" },
    { "000634", "GTE Airfone Inc." },
    { "000635", "PacketAir Networks, Inc." },
    { "000636", "Jedai Broadband Networks" },
    { "000637", "Toptrend-Meta Information (ShenZhen) Inc." },
    { "000638", "Sungjin C&C Co., Ltd." },
    { "000639", "Newtec" },
    { "00063A", "Dura Micro, Inc." },
    { "00063B", "Arcturus Networks Inc." },
    { "00063C", "Intrinsyc Software International Inc." },
    { "00063D", "Microwave Data Systems Inc." },
    { "00063E", "Opthos Inc." },
    { "00063F", "Everex Communications Inc." },
    { "000640", "White Rock Networks" },
    { "000641", "ITCN" },
    { "000642", "Genetel Systems Inc." },
    { "000643", "SONO Computer Co., Ltd." },
    { "000644", "NextGen Business Solutions, Inc" },
    { "000645", "Meisei Electric Co. Ltd." },
    { "000646", "ShenZhen XunBao Network Technology Co Ltd" },
    { "000647", "Etrali S.A." },
    { "000648", "Seedsware, Inc." },
    { "000649", "3M Deutschland GmbH" },
    { "00064A", "Honeywell Co., Ltd. (KOREA)" },
    { "00064B", "Alexon Co., Ltd." },
    { "00064C", "Invicta Networks, Inc." },
    { "00064D", "Sencore" },
    { "00064E", "Broad Net Technology Inc." },
    { "00064F", "PRO-NETS Technology Corporation" },
    { "000650", "Tiburon Networks, Inc." },
    { "000651", "Aspen Networks Inc." },
    { "000652", "Cisco Systems, Inc" },
    { "000653", "Cisco Systems, Inc" },
    { "000654", "Winpresa Building Automation Technologies GmbH" },
    { "000655", "Yipee, Inc." },
    { "000656", "Tactel AB" },
    { "000657", "Market Central, Inc." },
    { "000658", "Helmut Fischer GmbH Institut für Elektronik und Messtechnik" },
    { "000659", "EAL (Apeldoorn) B.V." },
    { "00065A", "Strix Systems" },
    { "00065B", "Dell Inc." },
    { "00065C", "Malachite Technologies, Inc." },
    { "00065D", "Heidelberg Web Systems" },
    { "00065E", "Photuris, Inc." },
    { "00065F", "ECI Telecom Ltd." },
    { "000660", "NADEX Co., Ltd." },
    { "000661", "NIA Home Technologies Corp." },
    { "000662", "MBM Technology Ltd." },
    { "000663", "Human Technology Co., Ltd." },
    { "000664", "Fostex Corporation" },
    { "000665", "Sunny Giken, Inc." },
    { "000666", "Roving Networks" },
    { "000667", "Tripp Lite" },
    { "000668", "Vicon Industries Inc." },
    { "000669", "Datasound Laboratories Ltd" },
    { "00066A", "InfiniCon Systems, Inc." },
    { "00066B", "Sysmex Corporation" },
    { "00066C", "Robinson Corporation" },
    { "00066D", "Compuprint S.P.A." },
    { "00066E", "Delta Electronics, Inc." },
    { "00066F", "Korea Data Systems" },
    { "000670", "Upponetti Oy" },
    { "000671", "Softing AG" },
    { "000672", "Netezza" },
    { "000673", "TKH Security Solutions USA" },
    { "000674", "Spectrum Control, Inc." },
    { "000675", "Banderacom, Inc." },
    { "000676", "Novra Technologies Inc." },
    { "000677", "SICK AG" },
    { "000678", "D&M Holdings Inc." },
    { "000679", "Konami Corporation" },
    { "00067A", "JMP Systems" },
    { "00067B", "Toplink C&C Corporation" },
    { "00067C", "Cisco Systems, Inc" },
    { "00067D", "Takasago Ltd." },
    { "00067E", "WinCom Systems, Inc." },
    { "00067F", "Digeo, Inc." },
    { "000680", "Card Access, Inc." },
    { "000681", "Goepel Electronic GmbH" },
    { "000682", "Convedia" },
    { "000683", "Bravara Communications, Inc." },
    { "000684", "Biacore AB" },
    { "000685", "NetNearU Corporation" },
    { "000686", "ZARDCOM Co., Ltd." },
    { "000687", "Omnitron Systems Technology, Inc." },
    { "000688", "Telways Communication Co., Ltd." },
    { "000689", "yLez Technologies Pte Ltd" },
    { "00068A", "NeuronNet Co. Ltd. R&D Center" },
    { "00068B", "AirRunner Technologies, Inc." },
    { "00068C", "3COM" },
    { "00068D", "SEPATON, Inc." },
    { "00068E", "HID Corporation" },
    { "00068F", "Telemonitor, Inc." },
    { "000690", "Euracom Communication GmbH" },
    { "000691", "PT Inovacao" },
    { "000692", "Intruvert Networks, Inc." },
    { "000693", "Flexus Computer Technology, Inc." },
    { "000694", "Mobillian Corporation" },
    { "000695", "Ensure Technologies, Inc." },
    { "000696", "Advent Networks" },
    { "000697", "R & D Center" },
    { "000698", "egnite GmbH" },
    { "000699", "Vida Design Co." },
    { "00069A", "e & Tel" },
    { "00069B", "AVT Audio Video Technologies GmbH" },
    { "00069C", "Transmode Systems AB" },
    { "00069D", "Petards Ltd" },
    { "00069E", "UNIQA, Inc." },
    { "00069F", "Kuokoa Networks" },
    { "0006A0", "Mx Imaging" },
    { "0006A1", "Celsian Technologies, Inc." },
    { "0006A2", "Microtune, Inc." },
    { "0006A3", "Bitran Corporation" },
    { "0006A4", "INNOWELL Corp." },
    { "0006A5", "PINON Corp." },
    { "0006A6", "Artistic Licence Engineering Ltd" },
    { "0006A7", "Primarion" },
    { "0006A8", "KC Technology, Inc." },
    { "0006A9", "Universal Instruments Corp." },
    { "0006AA", "VT Miltope" },
    { "0006AB", "W-Link Systems, Inc." },
    { "0006AC", "Intersoft Co." },
    { "0006AD", "KB Electronics Ltd." },
    { "0006AE", "Himachal Futuristic Communications Ltd" },
    { "0006AF", "Xalted Networks" },
    { "0006B0", "Comtech EF Data Corp." },
    { "0006B1", "Sonicwall" },
    { "0006B2", "Linxtek Co." },
    { "0006B3", "Diagraph Corporation" },
    { "0006B4", "Vorne Industries, Inc." },
    { "0006B5", "Source Photonics, Inc." },
    { "0006B6", "Nir-Or Israel Ltd." },
    { "0006B7", "TELEM GmbH" },
    { "0006B8", "Bandspeed Pty Ltd" },
    { "0006B9", "A5TEK Corp." },
    { "0006BA", "Westwave Communications" },
    { "0006BB", "ATI Technologies Inc." },
    { "0006BC", "Macrolink, Inc." },
    { "0006BD", "BNTECHNOLOGY Co., Ltd." },
    { "0006BE", "Baumer Optronic GmbH" },
    { "0006BF", "Accella Technologies Co., Ltd." },
    { "0006C0", "United Internetworks, Inc." },
    { "0006C1", "Cisco Systems, Inc" },
    { "0006C2", "Smartmatic Corporation" },
    { "0006C3", "Schindler Elevator Ltd." },
    { "0006C4", "Piolink Inc." },
    { "0006C5", "INNOVI Technologies Limited" },
    { "0006C6", "lesswire AG" },
    { "0006C7", "RFNET Technologies Pte Ltd (S)" },
    { "0006C8", "Sumitomo Metal Micro Devices, Inc." },
    { "0006C9", "Technical Marketing Research, Inc." },
    { "0006CA", "American Computer & Digital Components, Inc. (ACDC)" },
    { "0006CB", "Jotron Electronics A/S" },
    { "0006CC", "JMI Electronics Co., Ltd." },
    { "0006CD", "Leaf Imaging Ltd." },
    { "0006CE", "DATENO" },
    { "0006CF", "Thales Avionics In-Flight Systems, LLC" },
    { "0006D0", "Elgar Electronics Corp." },
    { "0006D1", "Tahoe Networks, Inc." },
    { "0006D2", "Tundra Semiconductor Corp." },
    { "0006D3", "Alpha Telecom, Inc. U.S.A." },
    { "0006D4", "Interactive Objects, Inc." },
    { "0006D5", "Diamond Systems Corp." },
    { "0006D6", "Cisco Systems, Inc" },
    { "0006D7", "Cisco Systems, Inc" },
    { "0006D8", "Maple Optical Systems" },
    { "0006D9", "IPM-Net S.p.A." },
    { "0006DA", "ITRAN Communications Ltd." },
    { "0006DB", "ICHIPS Co., Ltd." },
    { "0006DC", "Syabas Technology (Amquest)" },
    { "0006DD", "AT & T Laboratories - Cambridge Ltd" },
    { "0006DE", "Flash Technology" },
    { "0006DF", "AIDONIC Corporation" },
    { "0006E0", "MAT Co., Ltd." },
    { "0006E1", "Techno Trade s.a" },
    { "0006E2", "Ceemax Technology Co., Ltd." },
    { "0006E3", "Quantitative Imaging Corporation" },
    { "0006E4", "Citel Technologies Ltd." },
    { "0006E5", "Fujian Newland Computer Ltd. Co." },
    { "0006E6", "DongYang Telecom Co., Ltd." },
    { "0006E7", "Bit Blitz Communications Inc." },
    { "0006E8", "Optical Network Testing, Inc." },
    { "0006E9", "Intime Corp." },
    { "0006EA", "ELZET80 Mikrocomputer GmbH&Co. KG" },
    { "0006EB", "Global Data" },
    { "0006EC", "Harris Corporation" },
    { "0006ED", "Inara Networks" },
    { "0006EE", "Shenyang Neu-era Information & Technology Stock Co., Ltd" },
    { "0006EF", "Maxxan Systems, Inc." },
    { "0006F0", "Digeo, Inc." },
    { "0006F1", "Optillion" },
    { "0006F2", "Platys Communications" },
    { "0006F3", "AcceLight Networks" },
    { "0006F4", "Prime Electronics & Satellitics Inc." },
    { "0006F5", "ALPSALPINE CO,.LTD" },
    { "0006F6", "Cisco Systems, Inc" },
    { "0006F7", "ALPSALPINE CO,.LTD" },
    { "0006F8", "The Boeing Company" },
    { "0006F9", "Mitsui Zosen Systems Research Inc." },
    { "0006FA", "IP SQUARE Co, Ltd." },
    { "0006FB", "Hitachi Printing Solutions, Ltd." },
    { "0006FC", "Fnet Co., Ltd." },
    { "0006FD", "Comjet Information Systems Corp." },
    { "0006FE", "Ambrado, Inc" },
    { "0006FF", "Sheba Systems Co., Ltd." },
    { "000700", "Zettamedia Korea" },
    { "000701", "RACAL-DATACOM" },
    { "000702", "Varex Imaging" },
    { "000703", "CSEE Transport" },
    { "000704", "ALPSALPINE CO,.LTD" },
    { "000705", "Endress & Hauser GmbH & Co" },
    { "000706", "Sanritz Corporation" },
    { "000707", "Interalia Inc." },
    { "000708", "Bitrage Inc." },
    { "000709", "Westerstrand Urfabrik AB" },
    { "00070A", "Unicom Automation Co., Ltd." },
    { "00070B", "Novabase SGPS, SA" },
    { "00070C", "SVA-Intrusion.com Co. Ltd." },
    { "00070D", "Cisco Systems, Inc" },
    { "00070E", "Cisco Systems, Inc" },
    { "00070F", "Fujant, Inc." },
    { "000710", "Adax, Inc." },
    { "000711", "Acterna" },
    { "000712", "JAL Information Technology" },
    { "000713", "IP One, Inc." },
    { "000714", "Brightcom" },
    { "000715", "General Research of Electronics, Inc." },
    { "000716", "J & S Marine Ltd." },
    { "000717", "Wieland Electric GmbH" },
    { "000718", "iCanTek Co., Ltd." },
    { "000719", "Mobiis Co., Ltd." },
    { "00071A", "Finedigital Inc." },
    { "00071B", "CDVI Americas Ltd" },
    { "00071C", "AT&T" },
    { "00071D", "Satelsa Sistemas Y Aplicaciones De Telecomunicaciones, S.A." },
    { "00071E", "Tri-M Engineering / Nupak Dev. Corp." },
    { "00071F", "European Systems Integration" },
    { "000720", "Trutzschler GmbH & Co. KG" },
    { "000721", "Formac Elektronik GmbH" },
    { "000722", "The Nielsen Company" },
    { "000723", "ELCON Systemtechnik GmbH" },
    { "000724", "Telemax Co., Ltd." },
    { "000725", "Bematech International Corp." },
    { "000726", "SHENZHEN GONGJIN ELECTRONICS CO.,LT" },
    { "000727", "Zi Corporation (HK) Ltd." },
    { "000728", "Neo Telecom" },
    { "000729", "Kistler Instrumente AG" },
    { "00072A", "Innovance Networks" },
    { "00072B", "Jung Myung Telecom Co., Ltd." },
    { "00072C", "Fabricom" },
    { "00072D", "CNSystems" },
    { "00072E", "North Node AB" },
    { "00072F", "Intransa, Inc." },
    { "000730", "Hutchison OPTEL Telecom Technology Co., Ltd." },
    { "000731", "Ophir-Spiricon LLC" },
    { "000732", "AAEON Technology Inc." },
    { "000733", "DANCONTROL Engineering" },
    { "000734", "ONStor, Inc." },
    { "000735", "Flarion Technologies, Inc." },
    { "000736", "Data Video Technologies Co., Ltd." },
    { "000737", "Soriya Co. Ltd." },
    { "000738", "Young Technology Co., Ltd." },
    { "000739", "Scotty Group Austria Gmbh" },
    { "00073A", "INVENTEL" },
    { "00073B", "Tenovis GmbH & Co KG" },
    { "00073C", "Telecom Design" },
    { "00073D", "Nanjing Postel Telecommunications Co., Ltd." },
    { "00073E", "China Great-Wall Computer Shenzhen Co., Ltd." },
    { "00073F", "Woojyun Systec Co., Ltd." },
    { "000740", "BUFFALO.INC" },
    { "000741", "Sierra Automated Systems" },
    { "000742", "Ormazabal" },
    { "000743", "Chelsio Communications" },
    { "000744", "Unico, Inc." },
    { "000745", "Radlan Computer Communications Ltd." },
    { "000746", "TURCK, Inc." },
    { "000747", "Mecalc" },
    { "000748", "The Imaging Source Europe" },
    { "000749", "CENiX Inc." },
    { "00074A", "Carl Valentin GmbH" },
    { "00074B", "Daihen Corporation" },
    { "00074C", "Beicom Inc." },
    { "00074D", "Zebra Technologies Corp." },
    { "00074E", "IPFRONT Inc" },
    { "00074F", "Cisco Systems, Inc" },
    { "000750", "Cisco Systems, Inc" },
    { "000751", "m-u-t AG" },
    { "000752", "Rhythm Watch Co., Ltd." },
    { "000753", "Beijing Qxcomm Technology Co., Ltd." },
    { "000754", "Xyterra Computing, Inc." },
    { "000755", "Lafon" },
    { "000756", "Juyoung Telecom" },
    { "000757", "Topcall International AG" },
    { "000758", "DragonWave Inc." },
    { "000759", "Boris Manufacturing Corp." },
    { "00075A", "Air Products and Chemicals, Inc." },
    { "00075B", "Gibson Guitars" },
    { "00075C", "Eastman Kodak Company" },
    { "00075D", "Celleritas Inc." },
    { "00075E", "Ametek Power Instruments" },
    { "00075F", "VCS Video Communication Systems AG" },
    { "000760", "TOMIS Information & Telecom Corp." },
    { "000761", "29530" },
    { "000762", "Group Sense Limited" },
    { "000763", "Sunniwell Cyber Tech. Co., Ltd." },
    { "000764", "YoungWoo Telecom Co. Ltd." },
    { "000765", "Jade Quantum Technologies, Inc." },
    { "000766", "Chou Chin Industrial Co., Ltd." },
    { "000767", "Yuxing Electronics Company Limited" },
    { "000768", "Danfoss A/S" },
    { "000769", "Italiana Macchi SpA" },
    { "00076A", "NEXTEYE Co., Ltd." },
    { "00076B", "Stralfors AB" },
    { "00076C", "Daehanet, Inc." },
    { "00076D", "Flexlight Networks" },
    { "00076E", "Sinetica Corporation Limited" },
    { "00076F", "Synoptics Limited" },
    { "000770", "Ubiquoss Inc" },
    { "000771", "Embedded System Corporation" },
    { "000772", "Alcatel-Lucent Shanghai Bell Co., Ltd" },
    { "000773", "Ascom Powerline Communications Ltd." },
    { "000774", "GuangZhou Thinker Technology Co. Ltd." },
    { "000775", "Valence Semiconductor, Inc." },
    { "000776", "Federal APD" },
    { "000777", "Motah Ltd." },
    { "000778", "GERSTEL GmbH & Co. KG" },
    { "000779", "Sungil Telecom Co., Ltd." },
    { "00077A", "Infoware System Co., Ltd." },
    { "00077B", "Millimetrix Broadband Networks" },
    { "00077C", "Westermo Network Technologies AB" },
    { "00077D", "Cisco Systems, Inc" },
    { "00077E", "Elrest GmbH" },
    { "00077F", "J Communications Co., Ltd." },
    { "000780", "Bluegiga Technologies OY" },
    { "000781", "Itron Inc." },
    { "000782", "Oracle Corporation " },
    { "000783", "SynCom Network, Inc." },
    { "000784", "Cisco Systems, Inc" },
    { "000785", "Cisco Systems, Inc" },
    { "000786", "Wireless Networks Inc." },
    { "000787", "Idea System Co., Ltd." },
    { "000788", "Clipcomm, Inc." },
    { "000789", "Allradio Co., Ltd" },
    { "00078A", "Mentor Data System Inc." },
    { "00078B", "Wegener Communications, Inc." },
    { "00078C", "Elektronikspecialisten i Borlange AB" },
    { "00078D", "NetEngines Ltd." },
    { "00078E", "Garz & Fricke GmbH" },
    { "00078F", "Emkay Innovative Products" },
    { "000790", "Tri-M Technologies (s) Limited" },
    { "000791", "International Data Communications, Inc." },
    { "000792", "Sütron Electronic GmbH" },
    { "000793", "Shin Satellite Public Company Limited" },
    { "000794", "Simple Devices, Inc." },
    { "000795", "Elitegroup Computer Systems Co.,Ltd." },
    { "000796", "LSI Systems, Inc." },
    { "000797", "Netpower Co., Ltd." },
    { "000798", "Selea SRL" },
    { "000799", "Tipping Point Technologies, Inc." },
    { "00079A", "Verint Systems Inc" },
    { "00079B", "Aurora Networks" },
    { "00079C", "Golden Electronics Technology Co., Ltd." },
    { "00079D", "Musashi Co., Ltd." },
    { "00079E", "Ilinx Co., Ltd." },
    { "00079F", "Action Digital Inc." },
    { "0007A0", "e-Watch Inc." },
    { "0007A1", "VIASYS Healthcare GmbH" },
    { "0007A2", "Opteon Corporation" },
    { "0007A3", "Ositis Software, Inc." },
    { "0007A4", "GN Netcom Ltd." },
    { "0007A5", "Y.D.K Co. Ltd." },
    { "0007A6", "Leviton Manufacturing Co., Inc." },
    { "0007A7", "A-Z Inc." },
    { "0007A8", "Haier Group Technologies Ltd" },
    { "0007A9", "Novasonics" },
    { "0007AA", "Quantum Data Inc." },
    { "0007AB", "Samsung Electronics Co.,Ltd" },
    { "0007AC", "Eolring" },
    { "0007AD", "Pentacon GmbH Foto-und Feinwerktechnik" },
    { "0007AE", "Britestream Networks, Inc." },
    { "0007AF", "Red Lion Controls, LP" },
    { "0007B0", "Office Details, Inc." },
    { "0007B1", "Equator Technologies" },
    { "0007B2", "Transaccess S.A." },
    { "0007B3", "Cisco Systems, Inc" },
    { "0007B4", "Cisco Systems, Inc" },
    { "0007B5", "Any One Wireless Ltd." },
    { "0007B6", "Telecom Technology Ltd." },
    { "0007B7", "Samurai Ind. Prods Eletronicos Ltda" },
    { "0007B8", "Corvalent Corporation" },
    { "0007B9", "Ginganet Corporation" },
    { "0007BA", "UTStarcom Inc" },
    { "0007BB", "Candera Inc." },
    { "0007BC", "Identix Inc." },
    { "0007BD", "Radionet Ltd." },
    { "0007BE", "DataLogic SpA" },
    { "0007BF", "Armillaire Technologies, Inc." },
    { "0007C0", "NetZerver Inc." },
    { "0007C1", "Overture Networks, Inc." },
    { "0007C2", "Netsys Telecom" },
    { "0007C3", "Thomson" },
    { "0007C4", "JEAN Co. Ltd." },
    { "0007C5", "Gcom, Inc." },
    { "0007C6", "VDS Vosskuhler GmbH" },
    { "0007C7", "Synectics Systems Limited" },
    { "0007C8", "Brain21, Inc." },
    { "0007C9", "Technol Seven Co., Ltd." },
    { "0007CA", "Creatix Polymedia Ges Fur Kommunikaitonssysteme" },
    { "0007CB", "FREEBOX SAS" },
    { "0007CC", "Kaba Benzing GmbH" },
    { "0007CD", "Kumoh Electronic Co, Ltd" },
    { "0007CE", "Cabletime Limited" },
    { "0007CF", "Anoto AB" },
    { "0007D0", "Automat Engenharia de Automação Ltda." },
    { "0007D1", "Spectrum Signal Processing Inc." },
    { "0007D2", "Logopak Systeme GmbH & Co. KG" },
    { "0007D3", "SPGPrints B.V." },
    { "0007D4", "Zhejiang Yutong Network Communication Co Ltd." },
    { "0007D5", "3e Technologies Int;., Inc." },
    { "0007D6", "Commil Ltd." },
    { "0007D7", "Caporis Networks AG" },
    { "0007D8", "Hitron Technologies. Inc" },
    { "0007D9", "Splicecom" },
    { "0007DA", "Neuro Telecom Co., Ltd." },
    { "0007DB", "Kirana Networks, Inc." },
    { "0007DC", "Atek Co, Ltd." },
    { "0007DD", "Cradle Technologies" },
    { "0007DE", "eCopilt AB" },
    { "0007DF", "Vbrick Systems Inc." },
    { "0007E0", "Palm Inc." },
    { "0007E1", "WIS Communications Co. Ltd." },
    { "0007E2", "Bitworks, Inc." },
    { "0007E3", "Navcom Technology, Inc." },
    { "0007E4", "SoftRadio Co., Ltd." },
    { "0007E5", "Coup Corporation" },
    { "0007E6", "edgeflow Canada Inc." },
    { "0007E7", "FreeWave Technologies" },
    { "0007E8", "EdgeWave" },
    { "0007E9", "Intel Corporation" },
    { "0007EA", "Massana, Inc." },
    { "0007EB", "Cisco Systems, Inc" },
    { "0007EC", "Cisco Systems, Inc" },
    { "0007ED", "Altera Corporation" },
    { "0007EE", "telco Informationssysteme GmbH" },
    { "0007EF", "Lockheed Martin Tactical Systems" },
    { "0007F0", "LogiSync LLC" },
    { "0007F1", "TeraBurst Networks Inc." },
    { "0007F2", "IOA Corporation" },
    { "0007F3", "Thinkengine Networks" },
    { "0007F4", "Eletex Co., Ltd." },
    { "0007F5", "Bridgeco Co AG" },
    { "0007F6", "Qqest Software Systems" },
    { "0007F7", "Galtronics" },
    { "0007F8", "ITDevices, Inc." },
    { "0007F9", "Sensaphone" },
    { "0007FA", "ITT Co., Ltd." },
    { "0007FB", "Giga Stream UMTS Technologies GmbH" },
    { "0007FC", "Adept Systems Inc." },
    { "0007FD", "LANergy Ltd." },
    { "0007FE", "Rigaku Corporation" },
    { "0007FF", "Gluon Networks" },
    { "000800", "MULTITECH SYSTEMS, INC." },
    { "000801", "HighSpeed Surfing Inc." },
    { "000802", "Hewlett Packard" },
    { "000803", "Cos Tron" },
    { "000804", "ICA Inc." },
    { "000805", "Techno-Holon Corporation" },
    { "000806", "Raonet Systems, Inc." },
    { "000807", "Access Devices Limited" },
    { "000808", "PPT Vision, Inc." },
    { "000809", "Systemonic AG" },
    { "00080A", "Espera-Werke GmbH" },
    { "00080B", "Birka BPA Informationssystem AB" },
    { "00080C", "VDA Group S.p.a." },
    { "00080D", "Toshiba" },
    { "00080E", "Commscope" },
    { "00080F", "Proximion Fiber Optics AB" },
    { "000810", "Key Technology, Inc." },
    { "000811", "VOIX Corporation" },
    { "000812", "GM-2 Corporation" },
    { "000813", "Diskbank, Inc." },
    { "000814", "TIL Technologies" },
    { "000815", "CATS Co., Ltd." },
    { "000816", "Bluelon ApS" },
    { "000817", "EmergeCore Networks LLC" },
    { "000818", "Pixelworks, Inc." },
    { "000819", "Banksys" },
    { "00081A", "Sanrad Intelligence Storage Communications (2000) Ltd." },
    { "00081B", "Windigo Systems" },
    { "00081C", "@pos.com" },
    { "00081D", "Ipsil, Incorporated" },
    { "00081E", "Repeatit AB" },
    { "00081F", "Pou Yuen Tech Corp. Ltd." },
    { "000820", "Cisco Systems, Inc" },
    { "000821", "Cisco Systems, Inc" },
    { "000822", "InPro Comm" },
    { "000823", "Texa Corp." },
    { "000824", "Nuance Document Imaging" },
    { "000825", "Acme Packet" },
    { "000826", "Colorado Med Tech" },
    { "000827", "ADB Broadband Italia" },
    { "000828", "Koei Engineering Ltd." },
    { "000829", "TOKYO ELECTRON DEVICE NAGASAKI LIMITED" },
    { "00082A", "Powerwallz Network Security" },
    { "00082B", "Wooksung Electronics, Inc." },
    { "00082C", "Homag AG" },
    { "00082D", "Indus Teqsite Private Limited" },
    { "00082E", "Multitone Electronics PLC" },
    { "00082F", "Cisco Systems, Inc" },
    { "000830", "Cisco Systems, Inc" },
    { "000831", "Cisco Systems, Inc" },
    { "000832", "Cisco Systems, Inc" },
    { "00084E", "DivergeNet, Inc." },
    { "00084F", "Qualstar Corporation" },
    { "000850", "Arizona Instrument Corp." },
    { "000851", "Canadian Bank Note Company, Ltd." },
    { "000852", "Davolink Co. Inc." },
    { "000853", "Schleicher GmbH & Co. Relaiswerke KG" },
    { "000854", "Netronix, Inc." },
    { "000855", "NASA-Goddard Space Flight Center" },
    { "000856", "Gamatronic Electronic Industries Ltd." },
    { "000857", "Polaris Networks, Inc." },
    { "000858", "Novatechnology Inc." },
    { "000859", "ShenZhen Unitone Electronics Co., Ltd." },
    { "00085A", "IntiGate Inc." },
    { "00085B", "Hanbit Electronics Co., Ltd." },
    { "00085C", "Shanghai Dare Technologies Co. Ltd." },
    { "00085D", "Mitel Corporation" },
    { "00085E", "PCO AG" },
    { "00085F", "Picanol N.V." },
    { "000860", "LodgeNet Entertainment Corp." },
    { "000861", "SoftEnergy Co., Ltd." },
    { "000862", "NEC Eluminant Technologies, Inc." },
    { "000863", "Entrisphere Inc." },
    { "000864", "Fasy S.p.A." },
    { "000865", "JASCOM CO., LTD" },
    { "000866", "DSX Access Systems, Inc." },
    { "000867", "Uptime Devices" },
    { "000868", "PurOptix" },
    { "000869", "Command-e Technology Co.,Ltd." },
    { "00086A", "Securiton Gmbh" },
    { "00086B", "MIPSYS" },
    { "00086C", "Plasmon LMS" },
    { "00086D", "Missouri FreeNet" },
    { "00086E", "Hyglo AB" },
    { "00086F", "Resources Computer Network Ltd." },
    { "000870", "Rasvia Systems, Inc." },
    { "000871", "NORTHDATA Co., Ltd." },
    { "000872", "Sorenson Communications" },
    { "000873", "DapTechnology B.V." },
    { "000874", "Dell Inc." },
    { "000875", "Acorp Electronics Corp." },
    { "000876", "SDSystem" },
    { "000877", "Liebert-Hiross Spa" },
    { "000878", "Benchmark Storage Innovations" },
    { "000879", "CEM Corporation" },
    { "00087A", "Wipotec GmbH" },
    { "00087B", "RTX Telecom A/S" },
    { "00087C", "Cisco Systems, Inc" },
    { "00087D", "Cisco Systems, Inc" },
    { "00087E", "Bon Electro-Telecom Inc." },
    { "00087F", "SPAUN electronic GmbH & Co. KG" },
    { "000880", "BroadTel Canada Communications inc." },
    { "000881", "DIGITAL HANDS CO.,LTD." },
    { "000882", "SIGMA CORPORATION" },
    { "000883", "Hewlett Packard" },
    { "000884", "Index Braille AB" },
    { "000885", "EMS Dr. Thomas Wünsche" },
    { "000886", "Hansung Teliann, Inc." },
    { "000887", "Maschinenfabrik Reinhausen GmbH" },
    { "000888", "OULLIM Information Technology Inc,." },
    { "000889", "Dish Technologies Corp" },
    { "00088A", "Minds@Work" },
    { "00088B", "Tropic Networks Inc." },
    { "00088C", "Quanta Network Systems Inc." },
    { "00088D", "Sigma-Links Inc." },
    { "00088E", "Nihon Computer Co., Ltd." },
    { "00088F", "ADVANCED DIGITAL TECHNOLOGY" },
    { "000890", "AVILINKS SA" },
    { "000891", "Lyan Inc." },
    { "000892", "EM Solutions" },
    { "000893", "LE INFORMATION COMMUNICATION INC." },
    { "000894", "InnoVISION Multimedia Ltd." },
    { "000895", "DIRC Technologie GmbH & Co.KG" },
    { "000896", "Printronix, Inc." },
    { "000897", "Quake Technologies" },
    { "000898", "Gigabit Optics Corporation" },
    { "000899", "Netbind, Inc." },
    { "00089A", "Alcatel Microelectronics" },
    { "00089B", "ICP Electronics Inc." },
    { "00089C", "Elecs Industry Co., Ltd." },
    { "00089D", "UHD-Elektronik" },
    { "00089E", "Beijing Enter-Net co.LTD" },
    { "00089F", "EFM Networks" },
    { "0008A0", "Stotz Feinmesstechnik GmbH" },
    { "0008A1", "CNet Technology Inc." },
    { "0008A2", "ADI Engineering, Inc." },
    { "0008A3", "Cisco Systems, Inc" },
    { "0008A4", "Cisco Systems, Inc" },
    { "0008A5", "Peninsula Systems Inc." },
    { "0008A6", "Multiware & Image Co., Ltd." },
    { "0008A7", "iLogic Inc." },
    { "0008A8", "Systec Co., Ltd." },
    { "0008A9", "SangSang Technology, Inc." },
    { "0008AA", "KARAM" },
    { "0008AB", "EnerLinx.com, Inc." },
    { "0008AC", "BST GmbH" },
    { "0008AD", "Toyo-Linx Co., Ltd." },
    { "0008AE", "PacketFront Network Products AB" },
    { "0008AF", "Novatec Corporation" },
    { "0008B0", "HUBER+SUHNER BKtel GmbH" },
    { "0008B1", "ProQuent Systems" },
    { "0008B2", "SHENZHEN COMPASS TECHNOLOGY DEVELOPMENT CO.,LTD" },
    { "0008B3", "Fastwel" },
    { "0008B4", "SYSPOL" },
    { "0008B5", "TAI GUEN ENTERPRISE CO., LTD" },
    { "0008B6", "RouteFree, Inc." },
    { "0008B7", "HIT Incorporated" },
    { "0008B8", "E.F. Johnson" },
    { "0008B9", "Kaon Group Co., Ltd." },
    { "0008BA", "Erskine Systems Ltd" },
    { "0008BB", "NetExcell" },
    { "0008BC", "Ilevo AB" },
    { "0008BD", "TEPG-US" },
    { "0008BE", "XENPAK MSA Group" },
    { "0008BF", "Aptus Elektronik AB" },
    { "0008C0", "ASA SYSTEMS" },
    { "0008C1", "Avistar Communications Corporation" },
    { "0008C2", "Cisco Systems, Inc" },
    { "0008C3", "Contex A/S" },
    { "0008C4", "Hikari Co.,Ltd." },
    { "0008C5", "Liontech Co., Ltd." },
    { "0008C6", "Philips Consumer Communications" },
    { "0008C7", "Hewlett Packard" },
    { "0008C8", "Soneticom, Inc." },
    { "0008C9", "TechniSat Digital GmbH Daun" },
    { "0008CA", "TwinHan Technology Co.,Ltd" },
    { "0008CB", "Zeta Broadband Inc." },
    { "0008CC", "Remotec, Inc." },
    { "0008CD", "With-Net Inc" },
    { "0008CE", "IPMobileNet Inc." },
    { "0008CF", "Nippon Koei Power Systems Co., Ltd." },
    { "0008D0", "Musashi Engineering Co., LTD." },
    { "0008D1", "KAREL INC." },
    { "0008D2", "ZOOM Networks Inc." },
    { "0008D3", "Hercules Technologies S.A.S." },
    { "0008D4", "IneoQuest Technologies, Inc" },
    { "0008D5", "Vanguard Networks Solutions, LLC" },
    { "0008D6", "HASSNET Inc." },
    { "0008D7", "HOW CORPORATION" },
    { "0008D8", "Dowkey Microwave" },
    { "0008D9", "Mitadenshi Co.,LTD" },
    { "0008DA", "SofaWare Technologies Ltd." },
    { "0008DB", "Corrigent Systems" },
    { "0008DC", "Wiznet" },
    { "0008DD", "Telena Communications, Inc." },
    { "0008DE", "3UP Systems" },
    { "0008DF", "Alistel Inc." },
    { "0008E0", "ATO Technology Ltd." },
    { "0008E1", "Barix AG" },
    { "0008E2", "Cisco Systems, Inc" },
    { "0008E3", "Cisco Systems, Inc" },
    { "0008E4", "Envenergy Inc" },
    { "0008E5", "IDK Corporation" },
    { "0008E6", "Littlefeet" },
    { "0008E7", "SHI ControlSystems,Ltd." },
    { "0008E8", "Excel Master Ltd." },
    { "0008E9", "NextGig" },
    { "0008EA", "Motion Control Engineering, Inc" },
    { "0008EB", "ROMWin Co.,Ltd." },
    { "0008EC", "Optical Zonu Corporation" },
    { "0008ED", "ST&T Instrument Corp." },
    { "0008EE", "Logic Product Development" },
    { "0008EF", "DIBAL,S.A." },
    { "0008F0", "Next Generation Systems, Inc." },
    { "0008F1", "Voltaire" },
    { "0008F2", "C&S Technology" },
    { "0008F3", "WANY" },
    { "0008F4", "Bluetake Technology Co., Ltd." },
    { "0008F5", "YESTECHNOLOGY Co.,Ltd." },
    { "0008F6", "Sumitomo Electric Industries, Ltd" },
    { "0008F7", "Hitachi Ltd, Semiconductor & Integrated Circuits Gr" },
    { "0008F8", "UTC CCS" },
    { "0008F9", "Artesyn Embedded Technologies" },
    { "0008FA", "KEB Automation KG" },
    { "0008FB", "SonoSite, Inc." },
    { "0008FC", "Gigaphoton Inc." },
    { "0008FD", "BlueKorea Co., Ltd." },
    { "0008FE", "UNIK C&C Co.,Ltd." },
    { "0008FF", "Trilogy Communications Ltd" },
    { "000900", "TMT" },
    { "000901", "Shenzhen Shixuntong Information & Technoligy Co" },
    { "000902", "Redline Communications Inc." },
    { "000903", "Panasas, Inc" },
    { "000904", "MONDIAL electronic" },
    { "000905", "iTEC Technologies Ltd." },
    { "000906", "Esteem Networks" },
    { "000907", "Chrysalis Development" },
    { "000908", "VTech Technology Corp." },
    { "000909", "Telenor Connect A/S" },
    { "00090A", "SnedFar Technology Co., Ltd." },
    { "00090B", "MTL Instruments PLC" },
    { "00090C", "Mayekawa Mfg. Co. Ltd." },
    { "00090D", "LEADER ELECTRONICS CORP." },
    { "00090E", "Helix Technology Inc." },
    { "00090F", "Fortinet, Inc." },
    { "000910", "Simple Access Inc." },
    { "000911", "Cisco Systems, Inc" },
    { "000912", "Cisco Systems, Inc" },
    { "000913", "SystemK Corporation" },
    { "000914", "COMPUTROLS INC." },
    { "000915", "CAS Corp." },
    { "000916", "Listman Home Technologies, Inc." },
    { "000917", "WEM Technology Inc" },
    { "000918", "SAMSUNG TECHWIN CO.,LTD" },
    { "000919", "MDS Gateways" },
    { "00091A", "Macat Optics & Electronics Co., Ltd." },
    { "00091B", "Digital Generation Inc." },
    { "00091C", "CacheVision, Inc" },
    { "00091D", "Proteam Computer Corporation" },
    { "00091E", "Firstech Technology Corp." },
    { "00091F", "A&D Co., Ltd." },
    { "000920", "EpoX COMPUTER CO.,LTD." },
    { "000921", "Planmeca Oy" },
    { "000922", "TST Biometrics GmbH" },
    { "000923", "Heaman System Co., Ltd" },
    { "000924", "Telebau GmbH" },
    { "000925", "VSN Systemen BV" },
    { "000926", "YODA COMMUNICATIONS, INC." },
    { "000927", "TOYOKEIKI CO.,LTD." },
    { "000928", "Telecore" },
    { "000929", "Sanyo Industries (UK) Limited" },
    { "00092A", "MYTECS Co.,Ltd." },
    { "00092B", "iQstor Networks, Inc." },
    { "00092C", "Hitpoint Inc." },
    { "00092D", "HTC Corporation" },
    { "00092E", "B&Tech System Inc." },
    { "00092F", "Akom Technology Corporation" },
    { "000930", "AeroConcierge Inc." },
    { "000931", "Future Internet, Inc." },
    { "000932", "Omnilux" },
    { "000933", "Ophit Co.Ltd." },
    { "000934", "Dream-Multimedia-Tv GmbH" },
    { "000935", "Sandvine Incorporated" },
    { "000936", "Ipetronik GmbH & Co. KG" },
    { "000937", "Inventec Appliance Corp" },
    { "000938", "Allot Communications" },
    { "000939", "ShibaSoku Co.,Ltd." },
    { "00093A", "Molex CMS" },
    { "00093B", "HYUNDAI NETWORKS INC." },
    { "00093C", "Jacques Technologies P/L" },
    { "00093D", "Newisys,Inc." },
    { "00093E", "C&I Technologies" },
    { "00093F", "Double-Win Enterpirse CO., LTD" },
    { "000940", "AGFEO GmbH & Co. KG" },
    { "000941", "Allied Telesis K.K." },
    { "000942", "Wireless Technologies, Inc" },
    { "000943", "Cisco Systems, Inc" },
    { "000944", "Cisco Systems, Inc" },
    { "000945", "Palmmicro Communications Inc" },
    { "000946", "Cluster Labs GmbH" },
    { "000947", "Aztek, Inc." },
    { "000948", "Vista Control Systems, Corp." },
    { "000949", "Glyph Technologies Inc." },
    { "00094A", "Homenet Communications" },
    { "00094B", "FillFactory NV" },
    { "00094C", "Communication Weaver Co.,Ltd." },
    { "00094D", "Braintree Communications Pty Ltd" },
    { "00094E", "BARTECH SYSTEMS INTERNATIONAL, INC" },
    { "00094F", "elmegt GmbH & Co. KG" },
    { "000950", "Independent Storage Corporation" },
    { "000951", "Apogee Imaging Systems" },
    { "000952", "Auerswald GmbH & Co. KG" },
    { "000953", "Linkage System Integration Co.Ltd." },
    { "000954", "AMiT spol. s. r. o." },
    { "000955", "Young Generation International Corp." },
    { "000956", "Network Systems Group, Ltd. (NSG)" },
    { "000957", "Supercaller, Inc." },
    { "000958", "INTELNET S.A." },
    { "000959", "Sitecsoft" },
    { "00095A", "RACEWOOD TECHNOLOGY" },
    { "00095B", "NETGEAR" },
    { "00095C", "Philips Medical Systems - Cardiac and Monitoring Systems (CM" },
    { "00095D", "Dialogue Technology Corp." },
    { "00095E", "Masstech Group Inc." },
    { "00095F", "Telebyte, Inc." },
    { "000960", "YOZAN Inc." },
    { "000961", "Switchgear and Instrumentation Ltd" },
    { "000962", "Sonitor Technologies AS" },
    { "000963", "Dominion Lasercom Inc." },
    { "000964", "Hi-Techniques, Inc." },
    { "000965", "HyunJu Computer Co., Ltd." },
    { "000966", "TRIMBLE EUROPE BV" },
    { "000967", "Tachyon, Inc" },
    { "000968", "TECHNOVENTURE, INC." },
    { "000969", "Meret Optical Communications" },
    { "00096A", "Cloverleaf Communications Inc." },
    { "00096B", "IBM Corp" },
    { "00096C", "Imedia Semiconductor Corp." },
    { "00096D", "Powernet Technologies Corp." },
    { "00096E", "GIANT ELECTRONICS LTD." },
    { "00096F", "Beijing Zhongqing Elegant Tech. Corp.,Limited" },
    { "000970", "Vibration Research Corporation" },
    { "000971", "Time Management, Inc." },
    { "000972", "Securebase,Inc" },
    { "000973", "Lenten Technology Co., Ltd." },
    { "000974", "Innopia Technologies, Inc." },
    { "000975", "fSONA Communications Corporation" },
    { "000976", "Datasoft ISDN Systems GmbH" },
    { "000977", "Brunner Elektronik AG" },
    { "000978", "AIJI System Co., Ltd." },
    { "000979", "Advanced Television Systems Committee, Inc." },
    { "00097A", "Louis Design Labs." },
    { "00097B", "Cisco Systems, Inc" },
    { "00097C", "Cisco Systems, Inc" },
    { "00097D", "SecWell Networks Oy" },
    { "00097E", "IMI TECHNOLOGY CO., LTD" },
    { "00097F", "Vsecure 2000 LTD." },
    { "000980", "Power Zenith Inc." },
    { "000981", "Newport Networks" },
    { "000982", "Loewe Opta GmbH" },
    { "000983", "GlobalTop Technology, Inc." },
    { "000984", "MyCasa Network Inc." },
    { "000985", "Auto Telecom Company" },
    { "000986", "Metalink LTD." },
    { "000987", "NISHI NIPPON ELECTRIC WIRE & CABLE CO.,LTD." },
    { "000988", "Nudian Electron Co., Ltd." },
    { "000989", "VividLogic Inc." },
    { "00098A", "EqualLogic Inc" },
    { "00098B", "Entropic Communications, Inc." },
    { "00098C", "Option Wireless Sweden" },
    { "00098D", "Velocity Semiconductor" },
    { "00098E", "ipcas GmbH" },
    { "00098F", "Cetacean Networks" },
    { "000990", "ACKSYS Communications & systems" },
    { "000991", "Intelligent Platforms, LLC." },
    { "000992", "InterEpoch Technology,INC." },
    { "000993", "Visteon Corporation" },
    { "000994", "Cronyx Engineering" },
    { "000995", "Castle Technology Ltd" },
    { "000996", "RDI" },
    { "000997", "Nortel Networks" },
    { "000998", "Capinfo Company Limited" },
    { "000999", "CP GEORGES RENAULT" },
    { "00099A", "ELMO COMPANY, LIMITED" },
    { "00099B", "Western Telematic Inc." },
    { "00099C", "Naval Research Laboratory" },
    { "00099D", "Haliplex Communications" },
    { "00099E", "Testech, Inc." },
    { "00099F", "VIDEX INC." },
    { "0009A0", "Microtechno Corporation" },
    { "0009A1", "Telewise Communications, Inc." },
    { "0009A2", "Interface Co., Ltd." },
    { "0009A3", "Leadfly Techologies Corp. Ltd." },
    { "0009A4", "HARTEC Corporation" },
    { "0009A5", "HANSUNG ELETRONIC INDUSTRIES DEVELOPMENT CO., LTD" },
    { "0009A6", "Ignis Optics, Inc." },
    { "0009A7", "Bang & Olufsen A/S" },
    { "0009A8", "Eastmode Pte Ltd" },
    { "0009A9", "Ikanos Communications" },
    { "0009AA", "Data Comm for Business, Inc." },
    { "0009AB", "Netcontrol Oy" },
    { "0009AC", "LANVOICE" },
    { "0009AD", "HYUNDAI SYSCOMM, INC." },
    { "0009AE", "OKANO ELECTRIC CO.,LTD" },
    { "0009AF", "e-generis" },
    { "0009B0", "Onkyo Technology K.K." },
    { "0009B1", "Kanematsu Electronics, Ltd." },
    { "0009B2", "L&F Inc." },
    { "0009B3", "MCM Systems Ltd" },
    { "0009B4", "KISAN TELECOM CO., LTD." },
    { "0009B5", "3J Tech. Co., Ltd." },
    { "0009B6", "Cisco Systems, Inc" },
    { "0009B7", "Cisco Systems, Inc" },
    { "0009B8", "Entise Systems" },
    { "0009B9", "Action Imaging Solutions" },
    { "0009BA", "MAKU Informationstechik GmbH" },
    { "0009BB", "MathStar, Inc." },
    { "0009BC", "Utility, Inc" },
    { "0009BD", "Epygi Technologies, Ltd." },
    { "0009BE", "Mamiya-OP Co.,Ltd." },
    { "0009BF", "Nintendo Co., Ltd." },
    { "0009C0", "6WIND" },
    { "0009C1", "PROCES-DATA A/S" },
    { "0009C2", "Onity, Inc." },
    { "0009C3", "NETAS" },
    { "0009C4", "Medicore Co., Ltd" },
    { "0009C5", "KINGENE Technology Corporation" },
    { "0009C6", "Visionics Corporation" },
    { "0009C7", "Movistec" },
    { "0009C8", "SINAGAWA TSUSHIN KEISOU SERVICE" },
    { "0009C9", "BlueWINC Co., Ltd." },
    { "0009CA", "iMaxNetworks(Shenzhen)Limited." },
    { "0009CB", "HBrain" },
    { "0009CC", "Moog GmbH" },
    { "0009CD", "HUDSON SOFT CO.,LTD." },
    { "0009CE", "SpaceBridge Semiconductor Corp." },
    { "0009CF", "iAd GmbH" },
    { "0009D0", "Solacom Technologies Inc." },
    { "0009D1", "SERANOA NETWORKS INC" },
    { "0009D2", "Mai Logic Inc." },
    { "0009D3", "Western DataCom Co., Inc." },
    { "0009D4", "Transtech Networks" },
    { "0009D5", "Signal Communication, Inc." },
    { "0009D6", "KNC One GmbH" },
    { "0009D7", "DC Security Products" },
    { "0009D8", "Fält Communications AB" },
    { "0009D9", "Neoscale Systems, Inc" },
    { "0009DA", "Control Module Inc." },
    { "0009DB", "eSpace" },
    { "0009DC", "Galaxis Technology AG" },
    { "0009DD", "Mavin Technology Inc." },
    { "0009DE", "Samjin Information & Communications Co., Ltd." },
    { "0009DF", "Vestel Elektronik San ve Tic. A.S." },
    { "0009E0", "XEMICS S.A." },
    { "0009E1", "Gemtek Technology Co., Ltd." },
    { "0009E2", "Sinbon Electronics Co., Ltd." },
    { "0009E3", "Angel Iglesias S.A." },
    { "0009E4", "K Tech Infosystem Inc." },
    { "0009E5", "Hottinger Brüel & Kjaer GmbH" },
    { "0009E6", "Cyber Switching Inc." },
    { "0009E7", "ADC Techonology" },
    { "0009E8", "Cisco Systems, Inc" },
    { "0009E9", "Cisco Systems, Inc" },
    { "0009EA", "YEM Inc." },
    { "0009EB", "HuMANDATA LTD." },
    { "0009EC", "Daktronics, Inc." },
    { "0009ED", "CipherOptics" },
    { "0009EE", "MEIKYO ELECTRIC CO.,LTD" },
    { "0009EF", "Vocera Communications" },
    { "0009F0", "Shimizu Technology Inc." },
    { "0009F1", "Yamaki Electric Corporation" },
    { "0009F2", "Cohu, Inc., Electronics Division" },
    { "0009F3", "WELL Communication Corp." },
    { "0009F4", "Alcon Laboratories, Inc." },
    { "0009F5", "Emerson Network Power Co.,Ltd" },
    { "0009F6", "Shenzhen Eastern Digital Tech Ltd." },
    { "0009F7", "Calian Advanced Technologies" },
    { "0009F8", "UNIMO TECHNOLOGY CO., LTD." },
    { "0009F9", "ART JAPAN CO., LTD." },
    { "0009FB", "Philips Patient Monitoring" },
    { "0009FC", "IPFLEX Inc." },
    { "0009FD", "Ubinetics Limited" },
    { "0009FE", "Daisy Technologies, Inc." },
    { "0009FF", "X.net 2000 GmbH" },
    { "000A00", "MediaTek Inc" },
    { "000A01", "SOHOware, Inc." },
    { "000A02", "ANNSO CO., LTD." },
    { "000A03", "ENDESA SERVICIOS, S.L." },
    { "000A04", "3Com Ltd" },
    { "000A05", "Widax Corp." },
    { "000A06", "Teledex LLC" },
    { "000A07", "WebWayOne Ltd" },
    { "000A08", "Alps Alpine" },
    { "000A09", "TaraCom Integrated Products, Inc." },
    { "000A0A", "SUNIX Co., Ltd." },
    { "000A0B", "Sealevel Systems, Inc." },
    { "000A0C", "Scientific Research Corporation" },
    { "000A0D", "Amphenol" },
    { "000A0E", "Invivo Research Inc." },
    { "000A0F", "Ilryung Telesys, Inc" },
    { "000A10", "FAST media integrations AG" },
    { "000A11", "ExPet Technologies, Inc" },
    { "000A12", "Azylex Technology, Inc" },
    { "000A13", "Honeywell Video Systems" },
    { "000A14", "TECO a.s." },
    { "000A15", "Silicon Data, Inc" },
    { "000A16", "Lassen Research" },
    { "000A17", "NESTAR COMMUNICATIONS, INC" },
    { "000A18", "Vichel Inc." },
    { "000A19", "Valere Power, Inc." },
    { "000A1A", "Imerge Ltd" },
    { "000A1B", "Stream Labs" },
    { "000A1C", "Bridge Information Co., Ltd." },
    { "000A1D", "Optical Communications Products Inc." },
    { "000A1E", "Red-M Products Limited" },
    { "000A1F", "ART WARE Telecommunication Co., Ltd." },
    { "000A20", "SVA Networks, Inc." },
    { "000A21", "Integra Telecom Co. Ltd" },
    { "000A22", "Amperion Inc" },
    { "000A23", "Parama Networks Inc" },
    { "000A24", "Octave Communications" },
    { "000A25", "CERAGON NETWORKS" },
    { "000A26", "CEIA S.p.A." },
    { "000A27", "Apple, Inc." },
    { "000A28", "Motorola" },
    { "000A29", "Pan Dacom Networking AG" },
    { "000A2A", "QSI Systems Inc." },
    { "000A2B", "Etherstuff" },
    { "000A2C", "Active Tchnology Corporation" },
    { "000A2D", "Cabot Communications Limited" },
    { "000A2E", "MAPLE NETWORKS CO., LTD" },
    { "000A2F", "Artnix Inc." },
    { "000A30", "Visteon Corporation" },
    { "000A31", "HCV Consulting" },
    { "000A32", "Xsido Corporation" },
    { "000A33", "Emulex Corporation" },
    { "000A34", "Identicard Systems Incorporated" },
    { "000A35", "Xilinx" },
    { "000A36", "Synelec Telecom Multimedia" },
    { "000A37", "Procera Networks, Inc." },
    { "000A38", "Apani Networks" },
    { "000A39", "LoPA Information Technology" },
    { "000A3A", "J-THREE INTERNATIONAL Holding Co., Ltd." },
    { "000A3B", "GCT Semiconductor, Inc" },
    { "000A3C", "Enerpoint Ltd." },
    { "000A3D", "Elo Sistemas Eletronicos S.A." },
    { "000A3E", "EADS Telecom" },
    { "000A3F", "Data East Corporation" },
    { "000A40", "Crown Audio -- Harmanm International" },
    { "000A41", "Cisco Systems, Inc" },
    { "000A42", "Cisco Systems, Inc" },
    { "000A43", "Chunghwa Telecom Co., Ltd." },
    { "000A44", "Avery Dennison Deutschland GmbH" },
    { "000A45", "Audio-Technica Corp." },
    { "000A46", "ARO WELDING TECHNOLOGIES SAS" },
    { "000A47", "Allied Vision Technologies" },
    { "000A48", "Albatron Technology" },
    { "000A49", "F5 Inc." },
    { "000A4A", "Targa Systems Ltd." },
    { "000A4B", "DataPower Technology, Inc." },
    { "000A4C", "Molecular Devices Corporation" },
    { "000A4D", "Noritz Corporation" },
    { "000A4E", "UNITEK Electronics INC." },
    { "000A4F", "Brain Boxes Limited" },
    { "000A50", "REMOTEK CORPORATION" },
    { "000A51", "GyroSignal Technology Co., Ltd." },
    { "000A52", "AsiaRF Ltd." },
    { "000A53", "Intronics, Incorporated" },
    { "000A54", "Laguna Hills, Inc." },
    { "000A55", "MARKEM Corporation" },
    { "000A56", "HITACHI Maxell Ltd." },
    { "000A57", "Hewlett Packard" },
    { "000A58", "Freyer & Siegel Elektronik GmbH & Co. KG" },
    { "000A59", "HW server" },
    { "000A5A", "GreenNET Technologies Co.,Ltd." },
    { "000A5B", "Power-One as" },
    { "000A5C", "Carel s.p.a." },
    { "000A5D", "FingerTec Worldwide Sdn Bhd" },
    { "000A5E", "3COM" },
    { "000A5F", "almedio inc." },
    { "000A60", "Autostar Technology Pte Ltd" },
    { "000A61", "Cellinx Systems Inc." },
    { "000A62", "Crinis Networks, Inc." },
    { "000A63", "DHD GmbH" },
    { "000A64", "Eracom Technologies" },
    { "000A65", "GentechMedia.co.,ltd." },
    { "000A66", "MITSUBISHI ELECTRIC SYSTEM & SERVICE CO.,LTD." },
    { "000A67", "OngCorp" },
    { "000A68", "Solarflare Communications Inc." },
    { "000A69", "SUNNY bell Technology Co., Ltd." },
    { "000A6A", "SVM Microwaves s.r.o." },
    { "000A6B", "Tadiran Telecom Business Systems LTD" },
    { "000A6C", "Walchem Corporation" },
    { "000A6D", "EKS Elektronikservice GmbH" },
    { "000A6E", "Harmonic, Inc" },
    { "000A6F", "ZyFLEX Technologies Inc" },
    { "000A70", "MPLS Forum" },
    { "000A71", "Avrio Technologies, Inc" },
    { "000A72", "STEC, INC." },
    { "000A73", "Scientific Atlanta" },
    { "000A74", "Manticom Networks Inc." },
    { "000A75", "Caterpillar, Inc" },
    { "000A76", "Beida Jade Bird Huaguang Technology Co.,Ltd" },
    { "000A77", "Bluewire Technologies LLC" },
    { "000A78", "OLITEC" },
    { "000A79", "corega K.K" },
    { "000A7A", "Kyoritsu Electric Co., Ltd." },
    { "000A7B", "Cornelius Consult" },
    { "000A7C", "Tecton Ltd" },
    { "000A7D", "Valo, Inc." },
    { "000A7E", "The Advantage Group" },
    { "000A7F", "Teradon Industries, Inc" },
    { "000A80", "Telkonet Inc." },
    { "000A81", "TEIMA Audiotex S.L." },
    { "000A82", "TATSUTA SYSTEM ELECTRONICS CO.,LTD." },
    { "000A83", "SALTO SYSTEMS S.L." },
    { "000A84", "Rainsun Enterprise Co., Ltd." },
    { "000A85", "PLAT'C2,Inc" },
    { "000A86", "Lenze" },
    { "000A87", "Integrated Micromachines Inc." },
    { "000A88", "InCypher S.A." },
    { "000A89", "Creval Systems, Inc." },
    { "000A8A", "Cisco Systems, Inc" },
    { "000A8B", "Cisco Systems, Inc" },
    { "000A8C", "Guardware Systems Ltd." },
    { "000A8D", "EUROTHERM LIMITED" },
    { "000A8E", "Invacom Ltd" },
    { "000A8F", "Aska International Inc." },
    { "000A90", "Bayside Interactive, Inc." },
    { "000A91", "HemoCue AB" },
    { "000A92", "Presonus Corporation" },
    { "000A93", "W2 Networks, Inc." },
    { "000A94", "ShangHai cellink CO., LTD" },
    { "000A95", "Apple, Inc." },
    { "000A96", "MEWTEL TECHNOLOGY INC." },
    { "000A97", "SONICblue, Inc." },
    { "000A98", "M+F Gwinner GmbH & Co" },
    { "000A99", "Calamp Wireless Networks Inc" },
    { "000A9A", "Aiptek International Inc" },
    { "000A9B", "TB Group Inc" },
    { "000A9C", "Server Technology, Inc." },
    { "000A9D", "King Young Technology Co. Ltd." },
    { "000A9E", "BroadWeb Corportation" },
    { "000A9F", "Pannaway Technologies, Inc." },
    { "000AA0", "Cedar Point Communications" },
    { "000AA1", "V V S Limited" },
    { "000AA2", "SYSTEK INC." },
    { "000AA3", "SHIMAFUJI ELECTRIC CO.,LTD." },
    { "000AA4", "SHANGHAI SURVEILLANCE TECHNOLOGY CO,LTD" },
    { "000AA5", "MAXLINK INDUSTRIES LIMITED" },
    { "000AA6", "Hochiki Corporation" },
    { "000AA7", "FEI Electron Optics" },
    { "000AA8", "ePipe Pty. Ltd." },
    { "000AA9", "Brooks Automation GmbH" },
    { "000AAA", "AltiGen Communications Inc." },
    { "000AAB", "Toyota Technical Development Corporation" },
    { "000AAC", "TerraTec Electronic GmbH" },
    { "000AAD", "Stargames Corporation" },
    { "000AAE", "Rosemount Process Analytical" },
    { "000AAF", "Pipal Systems" },
    { "000AB0", "LOYTEC electronics GmbH" },
    { "000AB1", "GENETEC Corporation" },
    { "000AB2", "Fresnel Wireless Systems" },
    { "000AB3", "Fa. GIRA" },
    { "000AB4", "ETIC Telecommunications" },
    { "000AB5", "Digital Electronic Network" },
    { "000AB6", "COMPUNETIX, INC" },
    { "000AB7", "Cisco Systems, Inc" },
    { "000AB8", "Cisco Systems, Inc" },
    { "000AB9", "Astera Technologies Corp." },
    { "000ABA", "Arcon Technology Limited" },
    { "000ABB", "Taiwan Secom Co,. Ltd" },
    { "000ABC", "Seabridge Ltd." },
    { "000ABD", "Rupprecht & Patashnick Co." },
    { "000ABE", "OPNET Technologies CO., LTD." },
    { "000ABF", "HIROTA SS" },
    { "000AC0", "Fuyoh Video Industry CO., LTD." },
    { "000AC1", "Futuretel" },
    { "000AC2", "Wuhan FiberHome Digital Technology Co.,Ltd." },
    { "000AC3", "eM Technics Co., Ltd." },
    { "000AC4", "Daewoo Teletech Co., Ltd" },
    { "000AC5", "Color Kinetics" },
    { "000AC6", "Overture Networks." },
    { "000AC7", "Unication Group" },
    { "000AC8", "ZPSYS CO.,LTD. (Planning&Management)" },
    { "000AC9", "Zambeel Inc" },
    { "000ACA", "YOKOYAMA SHOKAI CO.,Ltd." },
    { "000ACB", "XPAK MSA Group" },
    { "000ACC", "Winnow Networks, Inc." },
    { "000ACD", "Sunrich Technology Limited" },
    { "000ACE", "RADIANTECH, INC." },
    { "000ACF", "PROVIDEO Multimedia Co. Ltd." },
    { "000AD0", "Niigata Develoment Center, F.I.T. Co., Ltd." },
    { "000AD1", "MWS" },
    { "000AD2", "JEPICO Corporation" },
    { "000AD3", "INITECH Co., Ltd" },
    { "000AD4", "CoreBell Systems Inc." },
    { "000AD5", "Brainchild Electronic Co., Ltd." },
    { "000AD6", "BeamReach Networks" },
    { "000AD7", "Origin Co., Ltd." },
    { "000AD8", "IPCserv Technology Corp." },
    { "000AD9", "Sony Corporation" },
    { "000ADA", "Vindicator Technologies" },
    { "000ADB", "Trilliant" },
    { "000ADC", "RuggedCom Inc." },
    { "000ADD", "Allworx Corp." },
    { "000ADE", "Happy Communication Co., Ltd." },
    { "000ADF", "Gennum Corporation" },
    { "000AE0", "Fujitsu Softek" },
    { "000AE1", "EG Technology" },
    { "000AE2", "Binatone Electronics International, Ltd" },
    { "000AE3", "YANG MEI TECHNOLOGY CO., LTD" },
    { "000AE4", "Wistron Corporation" },
    { "000AE5", "ScottCare Corporation" },
    { "000AE6", "Elitegroup Computer Systems Co.,Ltd." },
    { "000AE7", "ELIOP S.A." },
    { "000AE8", "Cathay Roxus Information Technology Co. LTD" },
    { "000AE9", "AirVast Technology Inc." },
    { "000AEA", "ADAM ELEKTRONIK LTD. ŞTI" },
    { "000AEB", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "000AEC", "Koatsu Gas Kogyo Co., Ltd." },
    { "000AED", "HARTING Electronics GmbH" },
    { "000AEE", "GCD Hard- & Software GmbH" },
    { "000AEF", "OTRUM ASA" },
    { "000AF0", "SHIN-OH ELECTRONICS CO., LTD. R&D" },
    { "000AF1", "Clarity Design, Inc." },
    { "000AF2", "NeoAxiom Corp." },
    { "000AF3", "Cisco Systems, Inc" },
    { "000AF4", "Cisco Systems, Inc" },
    { "000AF5", "Airgo Networks, Inc." },
    { "000AF6", "Copeland LP" },
    { "000AF7", "Broadcom" },
    { "000AF8", "American Telecare Inc." },
    { "000AF9", "HiConnect, Inc." },
    { "000AFA", "Traverse Technologies Australia" },
    { "000AFB", "Ambri Limited" },
    { "000AFC", "Core Tec Communications, LLC" },
    { "000AFD", "Kentec Electronics" },
    { "000AFE", "NovaPal Ltd" },
    { "000AFF", "Kilchherr Elektronik AG" },
    { "000B00", "FUJIAN START COMPUTER EQUIPMENT CO.,LTD" },
    { "000B01", "DAIICHI ELECTRONICS CO., LTD." },
    { "000B02", "Dallmeier electronic" },
    { "000B03", "Taekwang Industrial Co., Ltd" },
    { "000B04", "Volktek Corporation" },
    { "000B05", "Pacific Broadband Networks" },
    { "000B06", "Commscope" },
    { "000B07", "Voxpath Networks" },
    { "000B08", "Pillar Data Systems" },
    { "000B09", "Ifoundry Systems Singapore" },
    { "000B0A", "dBm Optics" },
    { "000B0B", "Corrent Corporation" },
    { "000B0C", "Agile Systems Inc." },
    { "000B0D", "Air2U, Inc." },
    { "000B0E", "Trapeze Networks" },
    { "000B0F", "Bosch Rexroth" },
    { "000B10", "11wave Technonlogy Co.,Ltd" },
    { "000B11", "HIMEJI ABC TRADING CO.,LTD." },
    { "000B12", "NURI Telecom Co., Ltd." },
    { "000B13", "ZETRON INC" },
    { "000B14", "ViewSonic Corporation" },
    { "000B15", "Platypus Technology" },
    { "000B16", "Communication Machinery Corporation" },
    { "000B17", "MKS Instruments" },
    { "000B18", "Private" },
    { "000B19", "Vernier Networks, Inc." },
    { "000B1A", "Industrial Defender, Inc." },
    { "000B1B", "Systronix, Inc." },
    { "000B1C", "SIBCO bv" },
    { "000B1D", "LayerZero Power Systems, Inc." },
    { "000B1E", "KAPPA opto-electronics GmbH" },
    { "000B1F", "I CON Computer Co." },
    { "000B20", "Hirata corporation" },
    { "000B21", "G-Star Communications Inc." },
    { "000B22", "Environmental Systems and Services" },
    { "000B23", "Siemens Home & Office Comm. Devices" },
    { "000B24", "AirLogic" },
    { "000B25", "Aeluros" },
    { "000B26", "Wetek Corporation" },
    { "000B27", "Scion Corporation" },
    { "000B28", "Quatech Inc." },
    { "000B29", "LS(LG) Industrial Systems co.,Ltd" },
    { "000B2A", "HOWTEL Co., Ltd." },
    { "000B2B", "HOSTNET CORPORATION" },
    { "000B2C", "Eiki Industrial Co. Ltd." },
    { "000B2D", "Danfoss Inc." },
    { "000B2E", "Cal-Comp Electronics & Communications Company Ltd." },
    { "000B2F", "bplan GmbH" },
    { "000B30", "Beijing Gongye Science & Technology Co.,Ltd" },
    { "000B31", "Yantai ZhiYang Scientific and technology industry CO., LTD" },
    { "000B32", "VORMETRIC, INC." },
    { "000B33", "Vivato Technologies" },
    { "000B34", "ShangHai Broadband Technologies CO.LTD" },
    { "000B35", "Quad Bit System co., Ltd." },
    { "000B36", "Productivity Systems, Inc." },
    { "000B37", "MANUFACTURE DES MONTRES ROLEX SA" },
    { "000B38", "Knürr GmbH" },
    { "000B39", "Keisoku Giken Co.,Ltd." },
    { "000B3A", "PESA Inc." },
    { "000B3B", "devolo AG" },
    { "000B3C", "Cygnal Integrated Products, Inc." },
    { "000B3D", "CONTAL OK Ltd." },
    { "000B3E", "BittWare, Inc" },
    { "000B3F", "Anthology Solutions Inc." },
    { "000B40", "Cambridge Industries Group (CIG)" },
    { "000B41", "Ing. Büro Dr. Beutlhauser" },
    { "000B42", "commax Co., Ltd." },
    { "000B43", "Microscan Systems, Inc." },
    { "000B44", "Concord Idea Corp." },
    { "000B45", "Cisco Systems, Inc" },
    { "000B46", "Cisco Systems, Inc" },
    { "000B47", "Advanced Energy" },
    { "000B48", "sofrel" },
    { "000B49", "RF-Link System Inc." },
    { "000B4A", "Visimetrics (UK) Ltd" },
    { "000B4B", "VISIOWAVE SA" },
    { "000B4C", "Clarion (M) Sdn Bhd" },
    { "000B4D", "Emuzed" },
    { "000B4E", "Communications & Power Industries" },
    { "000B4F", "Verifone, Inc." },
    { "000B50", "Oxygnet" },
    { "000B51", "Micetek International Inc." },
    { "000B52", "JOYMAX ELECTRONICS CO. LTD." },
    { "000B53", "INITIUM Co., Ltd." },
    { "000B54", "BiTMICRO Networks, Inc." },
    { "000B55", "ADInstruments" },
    { "000B56", "Cybernetics" },
    { "000B57", "Silicon Laboratories" },
    { "000B58", "Astronautics C.A LTD" },
    { "000B59", "ScriptPro, LLC" },
    { "000B5A", "HyperEdge" },
    { "000B5B", "Rincon Research Corporation" },
    { "000B5C", "Newtech Co.,Ltd" },
    { "000B5D", "FUJITSU LIMITED" },
    { "000B5E", "Audio Engineering Society Inc." },
    { "000B5F", "Cisco Systems, Inc" },
    { "000B60", "Cisco Systems, Inc" },
    { "000B61", "Friedrich Lütze GmbH & Co. KG" },
    { "000B62", "ib-mohnen KG" },
    { "000B63", "Kaleidescape" },
    { "000B64", "Kieback & Peter GmbH & Co KG" },
    { "000B65", "Sy.A.C. srl" },
    { "000B66", "Teralink Communications" },
    { "000B67", "Topview Technology Corporation" },
    { "000B68", "Addvalue Communications Pte Ltd" },
    { "000B69", "Franke Finland Oy" },
    { "000B6A", "Asiarock Technology Limited" },
    { "000B6B", "Wistron Neweb Corporation" },
    { "000B6C", "Sychip Inc." },
    { "000B6D", "SOLECTRON JAPAN NAKANIIDA" },
    { "000B6E", "Neff Instrument Corp." },
    { "000B6F", "Media Streaming Networks Inc" },
    { "000B70", "Load Technology, Inc." },
    { "000B71", "Litchfield Communications Inc." },
    { "000B72", "Lawo AG" },
    { "000B73", "Kodeos Communications" },
    { "000B74", "Kingwave Technology Co., Ltd." },
    { "000B75", "Iosoft Ltd." },
    { "000B76", "ET&T Technology Co. Ltd." },
    { "000B77", "Cogent Systems, Inc." },
    { "000B78", "TAIFATECH INC." },
    { "000B79", "X-COM, Inc." },
    { "000B7A", "L-3 Linkabit" },
    { "000B7B", "Test-Um Inc." },
    { "000B7C", "Telex Communications" },
    { "000B7D", "SOLOMON EXTREME INTERNATIONAL LTD." },
    { "000B7E", "SAGINOMIYA Seisakusho Inc." },
    { "000B7F", "Align Engineering LLC" },
    { "000B80", "Lycium Networks" },
    { "000B81", "Kaparel Corporation" },
    { "000B82", "Grandstream Networks, Inc." },
    { "000B83", "DATAWATT B.V." },
    { "000B84", "BODET" },
    { "000B85", "Cisco Systems, Inc" },
    { "000B86", "Hewlett Packard Enterprise" },
    { "000B87", "American Reliance Inc." },
    { "000B88", "Vidisco ltd." },
    { "000B89", "Top Global Technology, Ltd." },
    { "000B8A", "MITEQ Inc." },
    { "000B8B", "KERAJET, S.A." },
    { "000B8C", "Flextronics" },
    { "000B8D", "Avvio Networks" },
    { "000B8E", "Ascent Corporation" },
    { "000B8F", "AKITA ELECTRONICS SYSTEMS CO.,LTD." },
    { "000B90", "ADVA Optical Networking Ltd." },
    { "000B91", "Xovis Germany GmbH" },
    { "000B92", "Ascom Danmark A/S" },
    { "000B93", "Ritter Elektronik" },
    { "000B94", "Digital Monitoring Products, Inc." },
    { "000B95", "eBet Gaming Systems Pty Ltd" },
    { "000B96", "Innotrac Diagnostics Oy" },
    { "000B97", "Matsushita Electric Industrial Co.,Ltd." },
    { "000B98", "NiceTechVision" },
    { "000B99", "SensAble Technologies, Inc." },
    { "000B9A", "Shanghai Ulink Telecom Equipment Co. Ltd." },
    { "000B9B", "Sirius System Co, Ltd." },
    { "000B9C", "TriBeam Technologies, Inc." },
    { "000B9D", "TwinMOS Technologies Inc." },
    { "000B9E", "Yasing Technology Corp." },
    { "000B9F", "Neue ELSA GmbH" },
    { "000BA0", "T&L Information Inc." },
    { "000BA1", "Fujikura Solutions Ltd." },
    { "000BA2", "Sumitomo Electric Industries, Ltd" },
    { "000BA3", "Siemens AG" },
    { "000BA4", "Shiron Satellite Communications Ltd. (1996)" },
    { "000BA5", "Quasar Cipta Mandiri, PT" },
    { "000BA6", "Miyakawa Electric Works Ltd." },
    { "000BA7", "Maranti Networks" },
    { "000BA8", "HANBACK ELECTRONICS CO., LTD." },
    { "000BA9", "CloudShield Technologies, Inc." },
    { "000BAA", "Aiphone co.,Ltd" },
    { "000BAB", "Advantech Technology (CHINA) Co., Ltd." },
    { "000BAC", "3Com Ltd" },
    { "000BAD", "PC-PoS Inc." },
    { "000BAE", "Vitals System Inc." },
    { "000BAF", "WOOJU COMMUNICATIONS Co,.Ltd" },
    { "000BB0", "Sysnet Telematica srl" },
    { "000BB1", "Super Star Technology Co., Ltd." },
    { "000BB2", "SMALLBIG TECHNOLOGY" },
    { "000BB3", "RiT technologies Ltd." },
    { "000BB4", "RDC Semiconductor Inc.," },
    { "000BB5", "nStor Technologies, Inc." },
    { "000BB6", "Metalligence Technology Corp." },
    { "000BB7", "Micro Systems Co.,Ltd." },
    { "000BB8", "Kihoku Electronic Co." },
    { "000BB9", "Imsys AB" },
    { "000BBA", "Harmonic, Inc" },
    { "000BBB", "Etin Systems Co., Ltd" },
    { "000BBC", "En Garde Systems, Inc." },
    { "000BBD", "Connexionz Limited" },
    { "000BBE", "Cisco Systems, Inc" },
    { "000BBF", "Cisco Systems, Inc" },
    { "000BC0", "China IWNComm Co., Ltd." },
    { "000BC1", "Bay Microsystems, Inc." },
    { "000BC2", "Corinex Communication Corp." },
    { "000BC3", "Multiplex, Inc." },
    { "000BC4", "BIOTRONIK GmbH & Co" },
    { "000BC5", "SMC Networks, Inc." },
    { "000BC6", "ISAC, Inc." },
    { "000BC7", "ICET S.p.A." },
    { "000BC8", "AirFlow Networks" },
    { "000BC9", "Electroline Equipment" },
    { "000BCA", "DATAVAN TC" },
    { "000BCB", "Fagor Automation , S. Coop" },
    { "000BCC", "JUSAN, S.A." },
    { "000BCD", "Hewlett Packard" },
    { "000BCE", "Free2move AB" },
    { "000BCF", "AGFA NDT INC." },
    { "000BD0", "XiMeta Technology Americas Inc." },
    { "000BD1", "Aeronix, Inc." },
    { "000BD2", "Remopro Technology Inc." },
    { "000BD3", "cd3o" },
    { "000BD4", "Beijing Wise Technology & Science Development Co.Ltd" },
    { "000BD5", "Nvergence, Inc." },
    { "000BD6", "Paxton Access Ltd" },
    { "000BD7", "DORMA Time + Access GmbH" },
    { "000BD8", "Industrial Scientific Corp." },
    { "000BD9", "General Hydrogen" },
    { "000BDA", "EyeCross Co.,Inc." },
    { "000BDB", "Dell Inc." },
    { "000BDC", "AKCP" },
    { "000BDD", "TOHOKU RICOH Co., LTD." },
    { "000BDE", "TELDIX GmbH" },
    { "000BDF", "Shenzhen RouterD Networks Limited" },
    { "000BE0", "SercoNet Ltd." },
    { "000BE1", "Nokia NET Product Operations" },
    { "000BE2", "Lumenera Corporation" },
    { "000BE3", "Key Stream Co., Ltd." },
    { "000BE4", "Hosiden Corporation" },
    { "000BE5", "HIMS International Corporation" },
    { "000BE6", "Datel Electronics" },
    { "000BE7", "COMFLUX TECHNOLOGY INC." },
    { "000BE8", "AOIP" },
    { "000BE9", "Actel Corporation" },
    { "000BEA", "Zultys Technologies" },
    { "000BEB", "Systegra AG" },
    { "000BEC", "NIPPON ELECTRIC INSTRUMENT, INC." },
    { "000BED", "ELM Inc." },
    { "000BEE", "inc.jet, Incorporated" },
    { "000BEF", "Code Corporation" },
    { "000BF0", "MoTEX Products Co., Ltd." },
    { "000BF1", "LAP Laser Applikations" },
    { "000BF2", "Chih-Kan Technology Co., Ltd." },
    { "000BF3", "BAE SYSTEMS" },
    { "000BF4", "Private" },
    { "000BF5", "Shanghai Sibo Telecom Technology Co.,Ltd" },
    { "000BF6", "Nitgen Co., Ltd" },
    { "000BF7", "NIDEK CO.,LTD" },
    { "000BF8", "Infinera, Inc." },
    { "000BF9", "Gemstone Communications, Inc." },
    { "000BFA", "EXEMYS SRL" },
    { "000BFB", "D-NET International Corporation" },
    { "000BFC", "Cisco Systems, Inc" },
    { "000BFD", "Cisco Systems, Inc" },
    { "000BFE", "CASTEL Broadband Limited" },
    { "000BFF", "Berkeley Camera Engineering" },
    { "000C00", "BEB Industrie-Elektronik AG" },
    { "000C01", "Abatron AG" },
    { "000C02", "ABB Oy" },
    { "000C03", "HDMI Licensing, LLC" },
    { "000C04", "Tecnova" },
    { "000C05", "RPA Reserch Co., Ltd." },
    { "000C06", "Nixvue Systems Pte Ltd" },
    { "000C07", "Iftest AG" },
    { "000C08", "HUMEX Technologies Corp." },
    { "000C09", "Hitachi IE Systems Co., Ltd" },
    { "000C0A", "Guangdong Province Electronic Technology Research Institute" },
    { "000C0B", "Broadbus Technologies" },
    { "000C0C", "APPRO TECHNOLOGY INC." },
    { "000C0D", "Communications & Power Industries / Satcom Division" },
    { "000C0E", "XtremeSpectrum, Inc." },
    { "000C0F", "Techno-One Co., Ltd" },
    { "000C10", "PNI Corporation" },
    { "000C11", "NIPPON DEMPA CO.,LTD." },
    { "000C12", "Micro-Optronic-Messtechnik GmbH" },
    { "000C13", "MediaQ" },
    { "000C14", "Diagnostic Instruments, Inc." },
    { "000C15", "CyberPower Systems, Inc." },
    { "000C16", "Concorde Microsystems Inc." },
    { "000C17", "AJA Video Systems Inc" },
    { "000C18", "Zenisu Keisoku Inc." },
    { "000C19", "Telio Communications GmbH" },
    { "000C1A", "Quest Technical Solutions Inc." },
    { "000C1B", "ORACOM Co, Ltd." },
    { "000C1C", "MicroWeb Co., Ltd." },
    { "000C1D", "Mettler & Fuchs AG" },
    { "000C1E", "Global Cache" },
    { "000C1F", "Glimmerglass Networks" },
    { "000C20", "Fi WIn, Inc." },
    { "000C21", "Faculty of Science and Technology, Keio University" },
    { "000C22", "Double D Electronics Ltd" },
    { "000C23", "Beijing Lanchuan Tech. Co., Ltd." },
    { "000C24", "ANATOR" },
    { "000C25", "Allied Telesis Labs, Inc. " },
    { "000C26", "Weintek Labs. Inc." },
    { "000C27", "Sammy Corporation" },
    { "000C28", "RIFATRON" },
    { "000C29", "VMware, Inc." },
    { "000C2A", "OCTTEL Communication Co., Ltd." },
    { "000C2B", "ELIAS Technology, Inc." },
    { "000C2C", "Enwiser Inc." },
    { "000C2D", "FullWave Technology Co., Ltd." },
    { "000C2E", "Openet information technology(shenzhen) Co., Ltd." },
    { "000C2F", "SeorimTechnology Co.,Ltd." },
    { "000C30", "Cisco Systems, Inc" },
    { "000C31", "Cisco Systems, Inc" },
    { "000C32", "Avionic Design GmbH" },
    { "000C33", "Compucase Enterprise Co. Ltd." },
    { "000C34", "Vixen Co., Ltd." },
    { "000C35", "KaVo Dental GmbH & Co. KG" },
    { "000C36", "S-Takaya Electronics Industry Co.,Ltd." },
    { "000C37", "Geomation, Inc." },
    { "000C38", "TelcoBridges Inc." },
    { "000C39", "Sentinel Wireless Inc." },
    { "000C3A", "Oxance" },
    { "000C3B", "Orion Electric Co., Ltd." },
    { "000C3C", "MediaChorus, Inc." },
    { "000C3D", "Glsystech Co., Ltd." },
    { "000C3E", "Crest Audio" },
    { "000C3F", "Cogent Defence & Security Networks," },
    { "000C40", "Altech Controls" },
    { "000C41", "Cisco-Linksys, LLC" },
    { "000C42", "Routerboard.com" },
    { "000C43", "MediaTek Inc" },
    { "000C44", "Automated Interfaces, Inc." },
    { "000C45", "Animation Technologies Inc." },
    { "000C46", "Allied Telesyn Inc." },
    { "000C47", "SK Teletech(R&D Planning Team)" },
    { "000C48", "QoStek Corporation" },
    { "000C49", "Dangaard Telecom Denmark A/S" },
    { "000C4A", "Cygnus Microsystems (P) Limited" },
    { "000C4B", "Cheops Elektronik" },
    { "000C4C", "Arcor AG&Co." },
    { "000C4D", "Curtiss-Wright Controls Avionics & Electronics" },
    { "000C4E", "Winbest Technology CO,LT" },
    { "000C4F", "UDTech Japan Corporation" },
    { "000C50", "Seagate Technology" },
    { "000C51", "Scientific Technologies Inc." },
    { "000C52", "Roll Systems Inc." },
    { "000C53", "Private" },
    { "000C54", "Pedestal Networks, Inc" },
    { "000C55", "Microlink Communications Inc." },
    { "000C56", "Megatel Computer (1986) Corp." },
    { "000C57", "MACKIE Engineering Services Belgium BVBA" },
    { "000C58", "M&S Systems" },
    { "000C59", "Indyme Electronics, Inc." },
    { "000C5A", "IBSmm Embedded Electronics Consulting" },
    { "000C5B", "HANWANG TECHNOLOGY CO.,LTD" },
    { "000C5C", "GTN Systems B.V." },
    { "000C5D", "CHIC TECHNOLOGY (CHINA) CORP." },
    { "000C5E", "Calypso Medical" },
    { "000C5F", "Avtec, Inc." },
    { "000C60", "ACM Systems" },
    { "000C61", "AC Tech corporation DBA Advanced Digital" },
    { "000C62", "ABB AB, Cewe-Control " },
    { "000C63", "Zenith Electronics Corporation" },
    { "000C64", "X2 MSA Group" },
    { "000C65", "Sunin Telecom" },
    { "000C66", "Pronto Networks Inc" },
    { "000C67", "OYO ELECTRIC CO.,LTD" },
    { "000C68", "SigmaTel, Inc." },
    { "000C69", "National Radio Astronomy Observatory" },
    { "000C6A", "MBARI" },
    { "000C6B", "Kurz Industrie-Elektronik GmbH" },
    { "000C6C", "Eve Systems GmbH" },
    { "000C6D", "Edwards Ltd." },
    { "000C6E", "ASUSTek COMPUTER INC." },
    { "000C6F", "Amtek system co.,LTD." },
    { "000C70", "ACC GmbH" },
    { "000C71", "Wybron, Inc" },
    { "000C72", "Tempearl Industrial Co., Ltd." },
    { "000C73", "TELSON ELECTRONICS CO., LTD" },
    { "000C74", "RIVERTEC CORPORATION" },
    { "000C75", "Oriental integrated electronics. LTD" },
    { "000C76", "MICRO-STAR INTERNATIONAL CO., LTD." },
    { "000C77", "Life Racing Ltd" },
    { "000C78", "In-Tech Electronics Limited" },
    { "000C79", "Extel Communications P/L" },
    { "000C7A", "DaTARIUS Technologies GmbH" },
    { "000C7B", "ALPHA PROJECT Co.,Ltd." },
    { "000C7C", "Internet Information Image Inc." },
    { "000C7D", "TEIKOKU ELECTRIC MFG. CO., LTD" },
    { "000C7E", "Tellium Incorporated" },
    { "000C7F", "synertronixx GmbH" },
    { "000C80", "Opelcomm Inc." },
    { "000C81", "Schneider Electric (Australia) " },
    { "000C82", "NETWORK TECHNOLOGIES INC" },
    { "000C83", "Logical Solutions" },
    { "000C84", "Eazix, Inc." },
    { "000C85", "Cisco Systems, Inc" },
    { "000C86", "Cisco Systems, Inc" },
    { "000C87", "AMD" },
    { "000C88", "Apache Micro Peripherals, Inc." },
    { "000C89", "AC Electric Vehicles, Ltd." },
    { "000C8A", "Bose Corporation" },
    { "000C8B", "Connect Tech Inc" },
    { "000C8C", "KODICOM CO.,LTD." },
    { "000C8D", "Balluff MV GmbH" },
    { "000C8E", "Mentor Engineering Inc" },
    { "000C8F", "Nergal s.r.l." },
    { "000C90", "Octasic Inc." },
    { "000C91", "Riverhead Networks Inc." },
    { "000C92", "WolfVision Gmbh" },
    { "000C93", "Xeline Co., Ltd." },
    { "000C94", "United Electronic Industries, Inc. (EUI)" },
    { "000C95", "PrimeNet" },
    { "000C96", "OQO, Inc." },
    { "000C97", "NV ADB TTV Technologies SA" },
    { "000C98", "LETEK Communications Inc." },
    { "000C99", "HITEL LINK Co.,Ltd" },
    { "000C9A", "Hitech Electronics Corp." },
    { "000C9B", "EE Solutions, Inc" },
    { "000C9C", "Chongho information & communications" },
    { "000C9D", "UbeeAirWalk, Inc." },
    { "000C9E", "MemoryLink Corp." },
    { "000C9F", "NKE Corporation" },
    { "000CA0", "StorCase Technology, Inc." },
    { "000CA1", "SIGMACOM Co., LTD." },
    { "000CA2", "Harmonic Video Network" },
    { "000CA3", "Rancho Technology, Inc." },
    { "000CA4", "Prompttec Product Management GmbH" },
    { "000CA5", "Naman NZ LTd" },
    { "000CA6", "Mintera Corporation" },
    { "000CA7", "Metro (Suzhou) Technologies Co., Ltd." },
    { "000CA8", "Garuda Networks Corporation" },
    { "000CA9", "Ebtron Inc." },
    { "000CAA", "Cubic Transportation Systems Inc" },
    { "000CAB", "Commend International GmbH" },
    { "000CAC", "Citizen Watch Co., Ltd." },
    { "000CAD", "BTU International" },
    { "000CAE", "Ailocom Oy" },
    { "000CAF", "TRI TERM CO.,LTD." },
    { "000CB0", "Star Semiconductor Corporation" },
    { "000CB1", "Salland Engineering (Europe) BV" },
    { "000CB2", "UNION co., ltd." },
    { "000CB3", "ROUND Co.,Ltd." },
    { "000CB4", "AutoCell Laboratories, Inc." },
    { "000CB5", "Premier Technolgies, Inc" },
    { "000CB6", "NANJING SEU MOBILE & INTERNET TECHNOLOGY CO.,LTD" },
    { "000CB7", "Nanjing Huazhuo Electronics Co., Ltd." },
    { "000CB8", "MEDION AG" },
    { "000CB9", "LEA" },
    { "000CBA", "Jamex, Inc." },
    { "000CBB", "ISKRAEMECO" },
    { "000CBC", "Iscutum" },
    { "000CBD", "Interface Masters, Inc" },
    { "000CBE", "Innominate Security Technologies AG" },
    { "000CBF", "Holy Stone Ent. Co., Ltd." },
    { "000CC0", "Genera Oy" },
    { "000CC1", "Eaton Corporation" },
    { "000CC2", "ControlNet (India) Private Limited" },
    { "000CC3", "BeWAN systems" },
    { "000CC4", "Tiptel AG" },
    { "000CC5", "Nextlink Co., Ltd." },
    { "000CC6", "Ka-Ro electronics GmbH" },
    { "000CC7", "Intelligent Computer Solutions Inc." },
    { "000CC8", "Xytronix Research & Design, Inc." },
    { "000CC9", "ILWOO DATA & TECHNOLOGY CO.,LTD" },
    { "000CCA", "HGST a Western Digital Company" },
    { "000CCB", "Design Combus Ltd" },
    { "000CCC", "Aeroscout Ltd." },
    { "000CCD", "IEC - TC57" },
    { "000CCE", "Cisco Systems, Inc" },
    { "000CCF", "Cisco Systems, Inc" },
    { "000CD0", "Symetrix" },
    { "000CD1", "SFOM Technology Corp." },
    { "000CD2", "Schaffner EMV AG" },
    { "000CD3", "Prettl Elektronik Radeberg GmbH" },
    { "000CD4", "Positron Public Safety Systems inc." },
    { "000CD5", "Passave Inc." },
    { "000CD6", "PARTNER TECH" },
    { "000CD7", "Nallatech Ltd" },
    { "000CD8", "M. K. Juchheim GmbH & Co" },
    { "000CD9", "Itcare Co., Ltd" },
    { "000CDA", "FreeHand Systems, Inc." },
    { "000CDB", "Brocade Communications Systems LLC" },
    { "000CDC", "BECS Technology, Inc" },
    { "000CDD", "AOS technologies AG" },
    { "000CDE", "ABB STOTZ-KONTAKT GmbH" },
    { "000CDF", "JAI Manufacturing" },
    { "000CE0", "Trek Diagnostics Inc." },
    { "000CE1", "The Open Group" },
    { "000CE2", "Rolls-Royce" },
    { "000CE3", "Option International N.V." },
    { "000CE4", "NeuroCom International, Inc." },
    { "000CE5", "Commscope" },
    { "000CE6", "Fortinet, Inc." },
    { "000CE7", "MediaTek Inc" },
    { "000CE8", "GuangZhou AnJuBao Co., Ltd" },
    { "000CE9", "BLOOMBERG L.P." },
    { "000CEA", "aphona Kommunikationssysteme" },
    { "000CEB", "CNMP Networks, Inc." },
    { "000CEC", "Safran Trusted 4D Inc." },
    { "000CED", "Real Digital Media" },
    { "000CEE", "jp-embedded" },
    { "000CEF", "ONE Investment Group Limited" },
    { "000CF0", "M & N GmbH" },
    { "000CF1", "Intel Corporation" },
    { "000CF2", "GAMESA Eólica" },
    { "000CF3", "CALL IMAGE SA" },
    { "000CF4", "AKATSUKI ELECTRIC MFG.CO.,LTD." },
    { "000CF5", "InfoExpress" },
    { "000CF6", "Sitecom Europe BV" },
    { "000CF7", "Nortel Networks" },
    { "000CF8", "Nortel Networks" },
    { "000CF9", "Xylem Water Solutions" },
    { "000CFA", "Digital Systems Corp" },
    { "000CFB", "Korea Network Systems" },
    { "000CFC", "S2io Technologies Corp" },
    { "000CFD", "Hyundai ImageQuest Co.,Ltd." },
    { "000CFE", "Grand Electronic Co., Ltd" },
    { "000CFF", "MRO-TEK Realty Limited" },
    { "000D00", "Seaway Networks Inc." },
    { "000D01", "P&E Microcomputer Systems, Inc." },
    { "000D02", "NEC Platforms, Ltd." },
    { "000D03", "Matrics, Inc." },
    { "000D04", "Foxboro Eckardt Development GmbH" },
    { "000D05", "cybernet manufacturing inc." },
    { "000D06", "Compulogic Limited" },
    { "000D07", "Calrec Audio Ltd" },
    { "000D08", "AboveCable, Inc." },
    { "000D09", "Yuehua(Zhuhai) Electronic CO. LTD" },
    { "000D0A", "Barco Projection Systems NV" },
    { "000D0B", "BUFFALO.INC" },
    { "000D0C", "MDI Security Systems" },
    { "000D0D", "ITSupported, LLC" },
    { "000D0E", "Inqnet Systems, Inc." },
    { "000D0F", "Finlux Ltd" },
    { "000D10", "Embedtronics Oy" },
    { "000D11", "DENTSPLY - Gendex" },
    { "000D12", "AXELL Corporation" },
    { "000D13", "Wilhelm Rutenbeck GmbH&Co.KG" },
    { "000D14", "Vtech Innovation LP dba Advanced American Telephones" },
    { "000D15", "Voipac s.r.o." },
    { "000D16", "UHS Systems Pty Ltd" },
    { "000D17", "Turbo Networks Co.Ltd" },
    { "000D18", "Mega-Trend Electronics CO., LTD." },
    { "000D19", "ROBE Show lighting" },
    { "000D1A", "Mustek System Inc." },
    { "000D1B", "Kyoto Electronics Manufacturing Co., Ltd." },
    { "000D1C", "Amesys Defense" },
    { "000D1D", "HIGH-TEK HARNESS ENT. CO., LTD." },
    { "000D1E", "Control Techniques" },
    { "000D1F", "AV Digital" },
    { "000D20", "ASAHIKASEI TECHNOSYSTEM CO.,LTD." },
    { "000D21", "WISCORE Inc." },
    { "000D22", "Unitronics LTD" },
    { "000D23", "Smart Solution, Inc" },
    { "000D24", "SENTEC E&E CO., LTD." },
    { "000D25", "SANDEN CORPORATION" },
    { "000D26", "Primagraphics Limited" },
    { "000D27", "MICROPLEX Printware AG" },
    { "000D28", "Cisco Systems, Inc" },
    { "000D29", "Cisco Systems, Inc" },
    { "000D2A", "Scanmatic AS" },
    { "000D2B", "Racal Instruments" },
    { "000D2C", "Lantronix" },
    { "000D2D", "NCT Deutschland GmbH" },
    { "000D2E", "Matsushita Avionics Systems Corporation" },
    { "000D2F", "AIN Comm.Tech.Co., LTD" },
    { "000D30", "IceFyre Semiconductor" },
    { "000D31", "Compellent Technologies, Inc." },
    { "000D32", "DispenseSource, Inc." },
    { "000D33", "Prediwave Corp." },
    { "000D34", "Shell International Exploration and Production, Inc." },
    { "000D35", "PAC International Ltd" },
    { "000D36", "Wu Han Routon Electronic Co., Ltd" },
    { "000D37", "WIPLUG" },
    { "000D38", "NISSIN INC." },
    { "000D39", "Nevion" },
    { "000D3A", "Microsoft Corp." },
    { "000D3B", "Microelectronics Technology Inc." },
    { "000D3C", "i.Tech Dynamic Ltd" },
    { "000D3D", "Hammerhead Systems, Inc." },
    { "000D3E", "APLUX Communications Ltd." },
    { "000D3F", "VTI Instruments Corporation" },
    { "000D40", "Verint Loronix Video Solutions" },
    { "000D41", "Siemens AG ICM MP UC RD IT KLF1" },
    { "000D42", "Newbest Development Limited" },
    { "000D43", "DRS Tactical Systems Inc." },
    { "000D44", "Audio BU - Logitech" },
    { "000D45", "Tottori SANYO Electric Co., Ltd." },
    { "000D46", "Parker SSD Drives" },
    { "000D47", "Collex" },
    { "000D48", "AEWIN Technologies Co., Ltd." },
    { "000D49", "Triton Systems of Delaware, Inc." },
    { "000D4A", "Steag ETA-Optik" },
    { "000D4B", "Roku, Inc." },
    { "000D4C", "Outline Electronics Ltd." },
    { "000D4D", "Ninelanes" },
    { "000D4E", "NDR Co.,LTD." },
    { "000D4F", "Kenwood Corporation" },
    { "000D50", "Galazar Networks" },
    { "000D51", "DIVR Systems, Inc." },
    { "000D52", "Comart system" },
    { "000D53", "Beijing 5w Communication Corp." },
    { "000D54", "3Com Ltd" },
    { "000D55", "SANYCOM Technology Co.,Ltd" },
    { "000D56", "Dell Inc." },
    { "000D57", "Fujitsu I-Network Systems Limited." },
    { "000D58", "Private" },
    { "000D59", "Amity Systems, Inc." },
    { "000D5A", "Tiesse SpA" },
    { "000D5B", "Smart Empire Investments Limited" },
    { "000D5C", "Robert Bosch GmbH, VT-ATMO" },
    { "000D5D", "Raritan Computer, Inc" },
    { "000D5E", "NEC Personal Products" },
    { "000D5F", "Minds Inc" },
    { "000D60", "IBM Corp" },
    { "000D61", "Giga-Byte Technology Co., Ltd." },
    { "000D62", "Funkwerk Dabendorf GmbH" },
    { "000D63", "DENT Instruments, Inc." },
    { "000D64", "COMAG Handels AG" },
    { "000D65", "Cisco Systems, Inc" },
    { "000D66", "Cisco Systems, Inc" },
    { "000D67", "Ericsson" },
    { "000D68", "Vinci Systems, Inc." },
    { "000D69", "TMT&D Corporation" },
    { "000D6A", "Redwood Technologies LTD" },
    { "000D6B", "Mita-Teknik A/S" },
    { "000D6C", "M-Audio" },
    { "000D6D", "K-Tech Devices Corp." },
    { "000D6E", "K-Patents Oy" },
    { "000D6F", "Ember Corporation" },
    { "000D70", "Datamax Corporation" },
    { "000D71", "boca systems" },
    { "000D72", "2Wire Inc" },
    { "000D73", "Technical Support, Inc." },
    { "000D74", "Sand Network Systems, Inc." },
    { "000D75", "Kobian Pte Ltd - Taiwan Branch" },
    { "000D76", "Hokuto Denshi Co,. Ltd." },
    { "000D77", "FalconStor Software" },
    { "000D78", "Engineering & Security" },
    { "000D79", "Dynamic Solutions Co,.Ltd." },
    { "000D7A", "DiGATTO Asia Pacific Pte Ltd" },
    { "000D7B", "Consensys Computers Inc." },
    { "000D7C", "Codian Ltd" },
    { "000D7D", "Afco Systems" },
    { "000D7E", "Axiowave Networks, Inc." },
    { "000D7F", "MIDAS COMMUNICATION TECHNOLOGIES PTE LTD ( Foreign Branch)" },
    { "000D80", "Online Development Inc" },
    { "000D81", "Pepperl+Fuchs GmbH" },
    { "000D82", "PHSNET" },
    { "000D83", "Sanmina-SCI Hungary Ltd." },
    { "000D84", "Makus Inc." },
    { "000D85", "Tapwave, Inc." },
    { "000D86", "Huber + Suhner AG" },
    { "000D87", "Elitegroup Computer Systems Co.,Ltd." },
    { "000D88", "D-Link Corporation" },
    { "000D89", "Bils Technology Inc" },
    { "000D8A", "Winners Electronics Co., Ltd." },
    { "000D8B", "T&D Corporation" },
    { "000D8C", "Shanghai Wedone Digital Ltd. CO." },
    { "000D8D", "Prosoft Technology, Inc" },
    { "000D8E", "Koden Electronics Co., Ltd." },
    { "000D8F", "King Tsushin Kogyo Co., LTD." },
    { "000D90", "Factum Electronics AB" },
    { "000D91", "Eclipse (HQ Espana) S.L." },
    { "000D92", "ARIMA Communications Corp." },
    { "000D93", "Apple, Inc." },
    { "000D94", "AFAR Communications,Inc" },
    { "000D95", "Opti-cell, Inc." },
    { "000D96", "Vtera Technology Inc." },
    { "000D97", "Hitachi Energy USA Inc." },
    { "000D98", "S.W.A.C. Schmitt-Walter Automation Consult GmbH" },
    { "000D99", "Orbital Sciences Corp.; Launch Systems Group" },
    { "000D9A", "INFOTEC LTD" },
    { "000D9B", "Heraeus Electro-Nite International N.V." },
    { "000D9C", "K.A. Schmersal GmbH & Co. KG" },
    { "000D9D", "Hewlett Packard" },
    { "000D9E", "TOKUDEN OHIZUMI SEISAKUSYO Co.,Ltd." },
    { "000D9F", "RF Micro Devices" },
    { "000DA0", "NEDAP N.V." },
    { "000DA1", "MIRAE ITS Co.,LTD." },
    { "000DA2", "Infrant Technologies, Inc." },
    { "000DA3", "Emerging Technologies Limited" },
    { "000DA4", "DOSCH & AMAND SYSTEMS AG" },
    { "000DA5", "Fabric7 Systems, Inc" },
    { "000DA6", "Universal Switching Corporation" },
    { "000DA7", "Private" },
    { "000DA8", "Teletronics Technology Corporation" },
    { "000DA9", "INGETEAM" },
    { "000DAA", "S.A.Tehnology co.,Ltd." },
    { "000DAB", "Parker Hannifin GmbH Electromechanical Division Europe" },
    { "000DAC", "Japan CBM Corporation" },
    { "000DAD", "Dataprobe, Inc." },
    { "000DAE", "SAMSUNG HEAVY INDUSTRIES CO., LTD." },
    { "000DAF", "Plexus Corp (UK) Ltd" },
    { "000DB0", "Olym-tech Co.,Ltd." },
    { "000DB1", "Japan Network Service Co., Ltd." },
    { "000DB2", "Ammasso, Inc." },
    { "000DB3", "SDO Communication Corperation" },
    { "000DB4", "Stormshield" },
    { "000DB5", "GLOBALSAT TECHNOLOGY CORPORATION" },
    { "000DB6", "Broadcom" },
    { "000DB7", "SANKO ELECTRIC CO,.LTD" },
    { "000DB8", "SCHILLER AG" },
    { "000DB9", "PC Engines GmbH" },
    { "000DBA", "Océ Document Technologies GmbH" },
    { "000DBB", "Nippon Dentsu Co.,Ltd." },
    { "000DBC", "Cisco Systems, Inc" },
    { "000DBD", "Cisco Systems, Inc" },
    { "000DBE", "Bel Fuse Europe Ltd.,UK" },
    { "000DBF", "TekTone Sound & Signal Mfg., Inc." },
    { "000DC0", "Spagat AS" },
    { "000DC1", "SafeWeb Inc" },
    { "000DC2", "Private" },
    { "000DC3", "First Communication, Inc." },
    { "000DC4", "Emcore Corporation" },
    { "000DC5", "EchoStar Global B.V. " },
    { "000DC6", "DigiRose Technology Co., Ltd." },
    { "000DC7", "COSMIC ENGINEERING INC." },
    { "000DC8", "AirMagnet, Inc" },
    { "000DC9", "THALES Elektronik Systeme GmbH" },
    { "000DCA", "Tait Electronics" },
    { "000DCB", "Petcomkorea Co., Ltd." },
    { "000DCC", "NEOSMART Corp." },
    { "000DCD", "GROUPE TXCOM" },
    { "000DCE", "Dynavac Technology Pte Ltd" },
    { "000DCF", "Cidra Corp." },
    { "000DD0", "TetraTec Instruments GmbH" },
    { "000DD1", "Stryker Corporation" },
    { "000DD2", "Simrad Optronics ASA" },
    { "000DD3", "SAMWOO Telecommunication Co.,Ltd." },
    { "000DD4", "Symantec Corporation" },
    { "000DD5", "O'RITE TECHNOLOGY CO.,LTD" },
    { "000DD6", "ITI LTD" },
    { "000DD7", "Bright" },
    { "000DD8", "BBN" },
    { "000DD9", "Anton Paar GmbH" },
    { "000DDA", "ALLIED TELESIS K.K." },
    { "000DDB", "AIRWAVE TECHNOLOGIES INC." },
    { "000DDC", "VAC" },
    { "000DDD", "Profilo Telra Elektronik Sanayi ve Ticaret. A.Ş" },
    { "000DDE", "Joyteck Co., Ltd." },
    { "000DDF", "Japan Image & Network Inc." },
    { "000DE0", "ICPDAS Co.,LTD" },
    { "000DE1", "Control Products, Inc." },
    { "000DE2", "CMZ Sistemi Elettronici" },
    { "000DE3", "AT Sweden AB" },
    { "000DE4", "DIGINICS, Inc." },
    { "000DE5", "Samsung Thales" },
    { "000DE6", "YOUNGBO ENGINEERING CO.,LTD" },
    { "000DE7", "Snap-on OEM Group" },
    { "000DE8", "Nasaco Electronics Pte. Ltd" },
    { "000DE9", "Napatech Aps" },
    { "000DEA", "Kingtel Telecommunication Corp." },
    { "000DEB", "CompXs Limited" },
    { "000DEC", "Cisco Systems, Inc" },
    { "000DED", "Cisco Systems, Inc" },
    { "000DEE", "Andrew RF Power Amplifier Group" },
    { "000DEF", "Soc. Coop. Bilanciai" },
    { "000DF0", "QCOM TECHNOLOGY INC." },
    { "000DF1", "IONIX INC." },
    { "000DF2", "Private" },
    { "000DF3", "Asmax Solutions" },
    { "000DF4", "Watertek Co." },
    { "000DF5", "Teletronics International Inc." },
    { "000DF6", "Technology Thesaurus Corp." },
    { "000DF7", "Space Dynamics Lab" },
    { "000DF8", "ORGA Kartensysteme GmbH" },
    { "000DF9", "NDS Limited" },
    { "000DFA", "Micro Control Systems Ltd." },
    { "000DFB", "Komax AG" },
    { "000DFC", "ITFOR Inc." },
    { "000DFD", "Huges Hi-Tech Inc.," },
    { "000DFE", "Hauppauge Computer Works, Inc." },
    { "000DFF", "CHENMING MOLD INDUSTRY CORP." },
    { "000E00", "Atrie" },
    { "000E01", "ASIP Technologies Inc." },
    { "000E02", "Advantech AMT Inc." },
    { "000E03", "Emulex Corporation" },
    { "000E04", "CMA/Microdialysis AB" },
    { "000E05", "WIRELESS MATRIX CORP." },
    { "000E06", "Team Simoco Ltd" },
    { "000E07", "Sony Corporation" },
    { "000E08", "Cisco-Linksys, LLC" },
    { "000E09", "Shenzhen Coship Software Co.,LTD." },
    { "000E0A", "SAKUMA DESIGN OFFICE" },
    { "000E0B", "Netac Technology Co., Ltd." },
    { "000E0C", "Intel Corporation" },
    { "000E0D", "Hesch Schröder GmbH" },
    { "000E0E", "ESA elettronica S.P.A." },
    { "000E0F", "ERMME" },
    { "000E10", "C-guys, Inc." },
    { "000E11", "BDT Büro und Datentechnik GmbH & Co.KG " },
    { "000E12", "Adaptive Micro Systems Inc." },
    { "000E13", "Accu-Sort Systems inc." },
    { "000E14", "Visionary Solutions, Inc." },
    { "000E15", "Tadlys LTD" },
    { "000E16", "SouthWing S.L." },
    { "000E17", "Private" },
    { "000E18", "MyA Technology" },
    { "000E19", "LogicaCMG Pty Ltd" },
    { "000E1A", "JPS Communications" },
    { "000E1B", "IAV GmbH" },
    { "000E1C", "Hach Company" },
    { "000E1D", "ARION Technology Inc." },
    { "000E1E", "QLogic Corporation" },
    { "000E1F", "TCL Networks Equipment Co., Ltd." },
    { "000E20", "ACCESS Systems Americas, Inc." },
    { "000E21", "MTU Friedrichshafen GmbH" },
    { "000E22", "Private" },
    { "000E23", "Incipient, Inc." },
    { "000E24", "Huwell Technology Inc." },
    { "000E25", "Hannae Technology Co., Ltd" },
    { "000E26", "Gincom Technology Corp." },
    { "000E27", "Crere Networks, Inc." },
    { "000E28", "Dynamic Ratings P/L" },
    { "000E29", "Shester Communications Inc" },
    { "000E2A", "dormakaba USA Inc." },
    { "000E2B", "Safari Technologies" },
    { "000E2C", "Netcodec co." },
    { "000E2D", "Hyundai Digital Technology Co.,Ltd." },
    { "000E2E", "Edimax Technology Co. Ltd." },
    { "000E2F", "Roche Diagnostics GmbH" },
    { "000E30", "AERAS Networks, Inc." },
    { "000E31", "Olympus Soft Imaging Solutions GmbH" },
    { "000E32", "Kontron Medical" },
    { "000E33", "Shuko Electronics Co.,Ltd" },
    { "000E34", "NexGen City, LP" },
    { "000E35", "Intel Corporation" },
    { "000E36", "HEINESYS, Inc." },
    { "000E37", "Harms & Wende GmbH & Co.KG" },
    { "000E38", "Cisco Systems, Inc" },
    { "000E39", "Cisco Systems, Inc" },
    { "000E3A", "Cirrus Logic" },
    { "000E3B", "Hawking Technologies, Inc." },
    { "000E3C", "Transact Technologies Inc" },
    { "000E3D", "Televic N.V." },
    { "000E3E", "Sun Optronics Inc" },
    { "000E3F", "Soronti, Inc." },
    { "000E40", "Nortel Networks" },
    { "000E41", "NIHON MECHATRONICS CO.,LTD." },
    { "000E42", "Motic Incoporation Ltd." },
    { "000E43", "G-Tek Electronics Sdn. Bhd." },
    { "000E44", "Digital 5, Inc." },
    { "000E45", "Beijing Newtry Electronic Technology Ltd" },
    { "000E46", "Niigata Seimitsu Co.,Ltd." },
    { "000E47", "NCI System Co.,Ltd." },
    { "000E48", "Lipman TransAction Solutions" },
    { "000E49", "Forsway Scandinavia AB" },
    { "000E4A", "Changchun Huayu WEBPAD Co.,LTD" },
    { "000E4B", "atrium c and i" },
    { "000E4C", "Bermai Inc." },
    { "000E4D", "Numesa Inc." },
    { "000E4E", "Waveplus Technology Co., Ltd." },
    { "000E4F", "Trajet GmbH" },
    { "000E50", "Thomson Telecom Belgium" },
    { "000E51", "TECNA SpA" },
    { "000E52", "Optium Corporation" },
    { "000E53", "AV TECH CORPORATION" },
    { "000E54", "AlphaCell Wireless Ltd." },
    { "000E55", "AUVITRAN" },
    { "000E56", "4G Systems GmbH & Co. KG" },
    { "000E57", "Iworld Networking, Inc." },
    { "000E58", "Sonos, Inc." },
    { "000E59", "Sagemcom Broadband SAS" },
    { "000E5A", "TELEFIELD inc." },
    { "000E5B", "ParkerVision - Direct2Data" },
    { "000E5C", "Commscope" },
    { "000E5D", "Triple Play Technologies A/S" },
    { "000E5E", "Raisecom Technology CO., LTD" },
    { "000E5F", "activ-net GmbH & Co. KG" },
    { "000E60", "360SUN Digital Broadband Corporation" },
    { "000E61", "MICROTROL LIMITED" },
    { "000E62", "Nortel Networks" },
    { "000E63", "Lemke Diagnostics GmbH" },
    { "000E64", "Elphel, Inc" },
    { "000E65", "TransCore" },
    { "000E66", "Hitachi Industry & Control Solutions, Ltd." },
    { "000E67", "Eltis Microelectronics Ltd." },
    { "000E68", "E-TOP Network Technology Inc." },
    { "000E69", "China Electric Power Research Institute" },
    { "000E6A", "3Com Ltd" },
    { "000E6B", "Janitza electronics GmbH" },
    { "000E6C", "Device Drivers Limited" },
    { "000E6D", "Murata Manufacturing Co., Ltd." },
    { "000E6E", "MAT S.A. (Mircrelec Advanced Technology)" },
    { "000E6F", "IRIS Corporation Berhad" },
    { "000E70", "in2 Networks" },
    { "000E71", "Gemstar Technology Development Ltd." },
    { "000E72", "Arca Technologies S.r.l." },
    { "000E73", "Tpack A/S" },
    { "000E74", "Solar Telecom. Tech" },
    { "000E75", "New York Air Brake Corp." },
    { "000E76", "GEMSOC INNOVISION INC." },
    { "000E77", "Decru, Inc." },
    { "000E78", "Amtelco" },
    { "000E79", "Ample Communications Inc." },
    { "000E7A", "GemWon Communications Co., Ltd." },
    { "000E7B", "Toshiba" },
    { "000E7C", "Televes S.A." },
    { "000E7D", "Electronics Line 3000 Ltd." },
    { "000E7E", "ionSign Oy" },
    { "000E7F", "Hewlett Packard" },
    { "000E80", "Thomson Technology Inc" },
    { "000E81", "Devicescape Software, Inc." },
    { "000E82", "Infinity Tech" },
    { "000E83", "Cisco Systems, Inc" },
    { "000E84", "Cisco Systems, Inc" },
    { "000E85", "Catalyst Enterprises, Inc." },
    { "000E86", "Alcatel North America" },
    { "000E87", "adp Gauselmann GmbH" },
    { "000E88", "VIDEOTRON CORP." },
    { "000E89", "CLEMATIC" },
    { "000E8A", "Avara Technologies Pty. Ltd." },
    { "000E8B", "Astarte Technology Co, Ltd." },
    { "000E8C", "Siemens AG" },
    { "000E8D", "Systems in Progress Holding GmbH" },
    { "000E8E", "SparkLAN Communications, Inc." },
    { "000E8F", "Sercomm Corporation." },
    { "000E90", "PONICO CORP." },
    { "000E91", "Navico Auckland Ltd" },
    { "000E92", "Open Telecom" },
    { "000E93", "Milénio 3 Sistemas Electrónicos, Lda." },
    { "000E94", "Maas International BV" },
    { "000E95", "Fujiya Denki Seisakusho Co.,Ltd." },
    { "000E96", "Cubic Defense Applications, Inc." },
    { "000E97", "Ultracker Technology CO., Inc" },
    { "000E98", "HME Clear-Com LTD." },
    { "000E99", "Spectrum Digital, Inc" },
    { "000E9A", "BOE TECHNOLOGY GROUP CO.,LTD" },
    { "000E9B", "Ambit Microsystems Corporation" },
    { "000E9C", "Benchmark Electronics " },
    { "000E9D", "Tiscali UK Ltd" },
    { "000E9E", "Topfield Co., Ltd" },
    { "000E9F", "TEMIC SDS GmbH" },
    { "000EA0", "NetKlass Technology Inc." },
    { "000EA1", "Formosa Teletek Corporation" },
    { "000EA2", "McAfee, Inc" },
    { "000EA3", "CNCR-IT CO.,LTD,HangZhou P.R.CHINA" },
    { "000EA4", "Quantum Corp." },
    { "000EA5", "BLIP Systems" },
    { "000EA6", "ASUSTek COMPUTER INC." },
    { "000EA7", "Endace Technology" },
    { "000EA8", "United Technologists Europe Limited" },
    { "000EA9", "Shanghai Xun Shi Communications Equipment Ltd. Co." },
    { "000EAA", "Scalent Systems, Inc." },
    { "000EAB", "Cray Inc" },
    { "000EAC", "MINTRON ENTERPRISE CO., LTD." },
    { "000EAD", "Metanoia Technologies, Inc." },
    { "000EAE", "GAWELL TECHNOLOGIES CORP." },
    { "000EAF", "CASTEL" },
    { "000EB0", "Solutions Radio BV" },
    { "000EB1", "Newcotech,Ltd" },
    { "000EB2", "Micro-Research Finland Oy" },
    { "000EB3", "Hewlett Packard" },
    { "000EB4", "GUANGZHOU GAOKE COMMUNICATIONS TECHNOLOGY CO.LTD." },
    { "000EB5", "Ecastle Electronics Co., Ltd." },
    { "000EB6", "Riverbed Technology, Inc." },
    { "000EB7", "Knovative, Inc." },
    { "000EB8", "Iiga co.,Ltd" },
    { "000EB9", "HASHIMOTO Electronics Industry Co.,Ltd." },
    { "000EBA", "HANMI SEMICONDUCTOR CO., LTD." },
    { "000EBB", "Everbee Networks" },
    { "000EBC", "Paragon Fidelity GmbH" },
    { "000EBD", "Burdick, a Quinton Compny" },
    { "000EBE", "B&B Electronics Manufacturing Co." },
    { "000EBF", "Remsdaq Limited" },
    { "000EC0", "Nortel Networks" },
    { "000EC1", "MYNAH Technologies" },
    { "000EC2", "Lowrance Electronics, Inc." },
    { "000EC3", "Logic Controls, Inc." },
    { "000EC4", "Iskra Transmission d.d." },
    { "000EC5", "Digital Multitools Inc" },
    { "000EC6", "ASIX ELECTRONICS CORP." },
    { "000EC7", "Motorola Korea" },
    { "000EC8", "Zoran Corporation" },
    { "000EC9", "YOKO Technology Corp." },
    { "000ECA", "WTSS Inc" },
    { "000ECB", "VineSys Technology" },
    { "000ECC", "Tableau, LLC" },
    { "000ECD", "SKOV A/S" },
    { "000ECE", "S.I.T.T.I. S.p.A." },
    { "000ECF", "PROFIBUS Nutzerorganisation e.V." },
    { "000ED0", "Privaris, Inc." },
    { "000ED1", "Osaka Micro Computer." },
    { "000ED2", "Filtronic plc" },
    { "000ED3", "Epicenter, Inc." },
    { "000ED4", "CRESITT INDUSTRIE" },
    { "000ED5", "COPAN Systems Inc." },
    { "000ED6", "Cisco Systems, Inc" },
    { "000ED7", "Cisco Systems, Inc" },
    { "000ED8", "Positron Access Solutions Corp" },
    { "000ED9", "Aksys, Ltd." },
    { "000EDA", "C-TECH UNITED CORP." },
    { "000EDB", "XiNCOM Corp." },
    { "000EDC", "Tellion INC." },
    { "000EDD", "SHURE INCORPORATED" },
    { "000EDE", "REMEC, Inc." },
    { "000EDF", "PLX Technology" },
    { "000EE0", "Mcharge" },
    { "000EE1", "ExtremeSpeed Inc." },
    { "000EE2", "Custom Engineering" },
    { "000EE3", "Chiyu Technology Co.,Ltd" },
    { "000EE4", "BOE TECHNOLOGY GROUP CO.,LTD" },
    { "000EE5", "bitWallet, Inc." },
    { "000EE6", "Adimos Systems LTD" },
    { "000EE7", "AAC ELECTRONICS CORP." },
    { "000EE8", "Zioncom Electronics (Shenzhen) Ltd." },
    { "000EE9", "WayTech Development, Inc." },
    { "000EEA", "Shadong Luneng Jicheng Electronics,Co.,Ltd" },
    { "000EEB", "Sandmartin(zhong shan)Electronics Co.,Ltd" },
    { "000EEC", "Orban" },
    { "000EED", "Nokia Danmark A/S" },
    { "000EEE", "Muco Industrie BV" },
    { "000EEF", "Private" },
    { "000EF0", "Festo AG & Co. KG" },
    { "000EF1", "EZQUEST INC." },
    { "000EF2", "Infinico Corporation" },
    { "000EF3", "Smartlabs, Inc. " },
    { "000EF4", "Kasda Networks Inc" },
    { "000EF5", "iPAC Technology Co., Ltd." },
    { "000EF6", "E-TEN Information Systems Co., Ltd." },
    { "000EF7", "Vulcan Portals Inc" },
    { "000EF8", "SBC ASI" },
    { "000EF9", "REA Elektronik GmbH" },
    { "000EFA", "Optoway Technology Incorporation" },
    { "000EFB", "Macey Enterprises" },
    { "000EFC", "JTAG Technologies B.V." },
    { "000EFD", "FUJINON CORPORATION" },
    { "000EFE", "EndRun Technologies LLC" },
    { "000EFF", "Megasolution,Inc." },
    { "000F00", "Legra Systems, Inc." },
    { "000F01", "DIGITALKS INC" },
    { "000F02", "Digicube Technology Co., Ltd" },
    { "000F03", "COM&C CO., LTD" },
    { "000F04", "cim-usa inc" },
    { "000F05", "3B SYSTEM INC." },
    { "000F06", "Nortel Networks" },
    { "000F07", "Mangrove Systems, Inc." },
    { "000F08", "Indagon Oy" },
    { "000F09", "Private" },
    { "000F0A", "Clear Edge Networks" },
    { "000F0B", "Kentima Technologies AB" },
    { "000F0C", "SYNCHRONIC ENGINEERING" },
    { "000F0D", "Hunt Electronic Co., Ltd." },
    { "000F0E", "WaveSplitter Technologies, Inc." },
    { "000F0F", "Real ID Technology Co., Ltd." },
    { "000F10", "RDM Corporation" },
    { "000F11", "Prodrive B.V." },
    { "000F12", "Panasonic Europe Ltd." },
    { "000F13", "Nisca corporation" },
    { "000F14", "Mindray Co., Ltd." },
    { "000F15", "Icotera A/S" },
    { "000F16", "JAY HOW TECHNOLOGY CO.," },
    { "000F17", "Insta Elektro GmbH" },
    { "000F18", "Industrial Control Systems" },
    { "000F19", "Boston Scientific" },
    { "000F1A", "Gaming Support B.V." },
    { "000F1B", "Ego Systems Inc." },
    { "000F1C", "DigitAll World Co., Ltd" },
    { "000F1D", "Cosmo Techs Co., Ltd." },
    { "000F1E", "Chengdu KT Electric Co.of High & New Technology" },
    { "000F1F", "Dell Inc." },
    { "000F20", "Hewlett Packard" },
    { "000F21", "Scientific Atlanta, Inc" },
    { "000F22", "Helius, Inc." },
    { "000F23", "Cisco Systems, Inc" },
    { "000F24", "Cisco Systems, Inc" },
    { "000F25", "AimValley B.V." },
    { "000F26", "WorldAccxx LLC" },
    { "000F27", "TEAL Electronics, Inc." },
    { "000F28", "Itronix Corporation" },
    { "000F29", "Augmentix Corporation" },
    { "000F2A", "Cableware Electronics" },
    { "000F2B", "GREENBELL SYSTEMS" },
    { "000F2C", "Lantronix" },
    { "000F2D", "CHUNG-HSIN ELECTRIC & MACHINERY MFG.CORP." },
    { "000F2E", "Megapower International Corp." },
    { "000F2F", "W-LINX TECHNOLOGY CO., LTD." },
    { "000F30", "Raza Microelectronics Inc" },
    { "000F31", "Allied Vision Technologies Canada Inc" },
    { "000F32", "Lootom Telcovideo Network (Wuxi) Co Ltd" },
    { "000F33", "DUALi Inc." },
    { "000F34", "Cisco Systems, Inc" },
    { "000F35", "Cisco Systems, Inc" },
    { "000F36", "Accurate Techhnologies, Inc." },
    { "000F37", "Xambala Incorporated" },
    { "000F38", "Netstar" },
    { "000F39", "IRIS SENSORS" },
    { "000F3A", "HISHARP" },
    { "000F3B", "Fuji System Machines Co., Ltd." },
    { "000F3C", "Endeleo Limited" },
    { "000F3D", "D-Link Corporation" },
    { "000F3E", "CardioNet, Inc" },
    { "000F3F", "Big Bear Networks" },
    { "000F40", "Optical Internetworking Forum" },
    { "000F41", "Zipher Ltd" },
    { "000F42", "Xalyo Systems" },
    { "000F43", "Wasabi Systems Inc." },
    { "000F44", "Tivella Inc." },
    { "000F45", "Stretch, Inc." },
    { "000F46", "SINAR AG" },
    { "000F47", "ROBOX SPA" },
    { "000F48", "Polypix Inc." },
    { "000F49", "Northover Solutions Limited" },
    { "000F4A", "Kyushu-kyohan co.,ltd" },
    { "000F4B", "Oracle Corporation" },
    { "000F4C", "Elextech INC" },
    { "000F4D", "TalkSwitch" },
    { "000F4E", "Cellink" },
    { "000F4F", "PCS Systemtechnik GmbH" },
    { "000F50", "StreamScale Limited" },
    { "000F51", "Azul Systems, Inc." },
    { "000F52", "YORK Refrigeration, Marine & Controls" },
    { "000F53", "Solarflare Communications Inc." },
    { "000F54", "Entrelogic Corporation" },
    { "000F55", "Datawire Communication Networks Inc." },
    { "000F56", "Continuum Photonics Inc" },
    { "000F57", "CABLELOGIC Co., Ltd." },
    { "000F58", "Adder Technology Limited" },
    { "000F59", "Phonak AG" },
    { "000F5A", "Peribit Networks" },
    { "000F5B", "Delta Information Systems, Inc." },
    { "000F5C", "Day One Digital Media Limited" },
    { "000F5D", "Genexis BV" },
    { "000F5E", "Veo" },
    { "000F5F", "Nicety Technologies Inc. (NTS)" },
    { "000F60", "Lifetron Co.,Ltd" },
    { "000F61", "Hewlett Packard" },
    { "000F62", "Alcatel Bell Space N.V." },
    { "000F63", "Obzerv Technologies" },
    { "000F64", "D&R Electronica Weesp BV" },
    { "000F65", "icube Corp." },
    { "000F66", "Cisco-Linksys, LLC" },
    { "000F67", "West Instruments" },
    { "000F68", "Vavic Network Technology, Inc." },
    { "000F69", "SEW Eurodrive GmbH & Co. KG" },
    { "000F6A", "Nortel Networks" },
    { "000F6B", "GateWare Communications GmbH" },
    { "000F6C", "ADDI-DATA GmbH" },
    { "000F6D", "Midas Engineering" },
    { "000F6E", "BBox" },
    { "000F6F", "FTA Communication Technologies" },
    { "000F70", "Wintec Industries, inc." },
    { "000F71", "Sanmei Electronics Co.,Ltd" },
    { "000F72", "Sandburst" },
    { "000F73", "RS Automation Co., Ltd" },
    { "000F74", "Qamcom Technology AB" },
    { "000F75", "First Silicon Solutions" },
    { "000F76", "Digital Keystone, Inc." },
    { "000F77", "DENTUM CO.,LTD" },
    { "000F78", "Datacap Systems Inc" },
    { "000F79", "Bluetooth Interest Group Inc." },
    { "000F7A", "BeiJing NuQX Technology CO.,LTD" },
    { "000F7B", "Arce Sistemas, S.A." },
    { "000F7C", "ACTi Corporation" },
    { "000F7D", "Xirrus" },
    { "000F7E", "Ablerex Electronics Co., LTD" },
    { "000F7F", "UBSTORAGE Co.,Ltd." },
    { "000F80", "Trinity Security Systems,Inc." },
    { "000F81", "PAL Pacific Inc." },
    { "000F82", "Mortara Instrument, Inc." },
    { "000F83", "Brainium Technologies Inc." },
    { "000F84", "Astute Networks, Inc." },
    { "000F85", "ADDO-Japan Corporation" },
    { "000F86", "BlackBerry RTS" },
    { "000F87", "Maxcess International" },
    { "000F88", "AMETEK, Inc." },
    { "000F89", "Winnertec System Co., Ltd." },
    { "000F8A", "WideView" },
    { "000F8B", "Orion MultiSystems Inc" },
    { "000F8C", "Gigawavetech Pte Ltd" },
    { "000F8D", "FAST TV-Server AG" },
    { "000F8E", "DONGYANG TELECOM CO.,LTD." },
    { "000F8F", "Cisco Systems, Inc" },
    { "000F90", "Cisco Systems, Inc" },
    { "000F91", "Aerotelecom Co.,Ltd." },
    { "000F92", "Microhard Systems Inc." },
    { "000F93", "Landis+Gyr Ltd." },
    { "000F94", "Genexis BV" },
    { "000F95", "ELECOM Co.,LTD Laneed Division" },
    { "000F96", "Telco Systems, Inc. " },
    { "000F97", "Avanex Corporation" },
    { "000F98", "Avamax Co. Ltd." },
    { "000F99", "APAC opto Electronics Inc." },
    { "000F9A", "Synchrony, Inc." },
    { "000F9B", "Ross Video Limited" },
    { "000F9C", "Panduit Corp" },
    { "000F9D", "DisplayLink (UK) Ltd" },
    { "000F9E", "Murrelektronik GmbH" },
    { "000F9F", "Commscope" },
    { "000FA0", "Canon Korea Inc." },
    { "000FA1", "Gigabit Systems Inc." },
    { "000FA2", "2xWireless" },
    { "000FA3", "Alpha Networks Inc." },
    { "000FA4", "Sprecher Automation GmbH" },
    { "000FA5", "BWA Technology GmbH" },
    { "000FA6", "S2 Security Corporation" },
    { "000FA7", "Raptor Networks Technology" },
    { "000FA8", "Photometrics, Inc." },
    { "000FA9", "PC Fabrik" },
    { "000FAA", "Nexus Technologies" },
    { "000FAB", "Kyushu Electronics Systems Inc." },
    { "000FAC", "IEEE 802.11" },
    { "000FAD", "FMN communications GmbH" },
    { "000FAE", "E2O Communications" },
    { "000FAF", "Dialog Inc." },
    { "000FB0", "Compal Electronics INC." },
    { "000FB1", "Cognio Inc." },
    { "000FB2", "Broadband Pacenet (India) Pvt. Ltd." },
    { "000FB3", "Actiontec Electronics, Inc" },
    { "000FB4", "Timespace Technology" },
    { "000FB5", "NETGEAR" },
    { "000FB6", "Europlex Technologies" },
    { "000FB7", "Cavium" },
    { "000FB8", "CallURL Inc." },
    { "000FB9", "Adaptive Instruments" },
    { "000FBA", "Tevebox AB" },
    { "000FBB", "Nokia Siemens Networks GmbH & Co. KG." },
    { "000FBC", "Onkey Technologies, Inc." },
    { "000FBD", "MRV Communications (Networks) LTD" },
    { "000FBE", "e-w/you Inc." },
    { "000FBF", "DGT Sp. z o.o." },
    { "000FC0", "DELCOMp" },
    { "000FC1", "WAVE Corporation" },
    { "000FC2", "Uniwell Corporation" },
    { "000FC3", "PalmPalm Technology, Inc." },
    { "000FC4", "NST co.,LTD." },
    { "000FC5", "KeyMed Ltd" },
    { "000FC6", "Eurocom Industries A/S" },
    { "000FC7", "Dionica R&D Ltd." },
    { "000FC8", "Chantry Networks" },
    { "000FC9", "Allnet GmbH" },
    { "000FCA", "A-JIN TECHLINE CO, LTD" },
    { "000FCB", "3Com Ltd" },
    { "000FCC", "Commscope" },
    { "000FCD", "Nortel Networks" },
    { "000FCE", "Kikusui Electronics Corp." },
    { "000FCF", "DataWind Research" },
    { "000FD0", "ASTRI" },
    { "000FD1", "Applied Wireless Identifications Group, Inc." },
    { "000FD2", "EWA Technologies, Inc." },
    { "000FD3", "Digium" },
    { "000FD4", "Soundcraft" },
    { "000FD5", "Schwechat - RISE" },
    { "000FD6", "Sarotech Co., Ltd" },
    { "000FD7", "Harman Music Group" },
    { "000FD8", "Force, Inc." },
    { "000FD9", "FlexDSL Telecommunications AG" },
    { "000FDA", "YAZAKI CORPORATION" },
    { "000FDB", "Westell Technologies Inc." },
    { "000FDC", "Ueda Japan Radio Co., Ltd." },
    { "000FDD", "SORDIN AB" },
    { "000FDE", "Sony Corporation" },
    { "000FDF", "SOLOMON Technology Corp." },
    { "000FE0", "NComputing Co.,Ltd." },
    { "000FE1", "ID DIGITAL CORPORATION" },
    { "000FE2", "Hangzhou H3C Technologies Co., Limited" },
    { "000FE3", "Damm Cellular Systems A/S" },
    { "000FE4", "Pantech Co.,Ltd" },
    { "000FE5", "MERCURY SECURITY CORPORATION" },
    { "000FE6", "MBTech Systems, Inc." },
    { "000FE7", "Lutron Electronics Co., Inc." },
    { "000FE8", "Lobos, Inc." },
    { "000FE9", "GW TECHNOLOGIES CO.,LTD." },
    { "000FEA", "GIGA-BYTE TECHNOLOGY CO.,LTD." },
    { "000FEB", "Cylon Controls" },
    { "000FEC", "GopherTec Inc." },
    { "000FED", "Anam Electronics Co., Ltd" },
    { "000FEE", "XTec, Incorporated" },
    { "000FEF", "Thales e-Transactions GmbH" },
    { "000FF0", "Sunray Co. Ltd." },
    { "000FF1", "nex-G Systems Pte.Ltd" },
    { "000FF2", "Loud Technologies Inc." },
    { "000FF3", "Jung Myoung Communications&Technology" },
    { "000FF4", "Guntermann & Drunck GmbH" },
    { "000FF5", "GN&S company" },
    { "000FF6", "DARFON LIGHTING CORP" },
    { "000FF7", "Cisco Systems, Inc" },
    { "000FF8", "Cisco Systems, Inc" },
    { "000FF9", "Valcretec, Inc." },
    { "000FFA", "Optinel Systems, Inc." },
    { "000FFB", "Nippon Denso Industry Co., Ltd." },
    { "000FFC", "Merit Li-Lin Ent." },
    { "000FFD", "Glorytek Network Inc." },
    { "000FFE", "G-PRO COMPUTER" },
    { "000FFF", "Control4" },
    { "001000", "CABLE TELEVISION LABORATORIES, INC." },
    { "001001", "Citel" },
    { "001002", "ACTIA" },
    { "001003", "IMATRON, INC." },
    { "001004", "THE BRANTLEY COILE COMPANY,INC" },
    { "001005", "UEC COMMERCIAL" },
    { "001006", "Thales Contact Solutions Ltd." },
    { "001007", "Cisco Systems, Inc" },
    { "001008", "VIENNA SYSTEMS CORPORATION" },
    { "001009", "HORANET" },
    { "00100A", "WILLIAMS COMMUNICATIONS GROUP" },
    { "00100B", "Cisco Systems, Inc" },
    { "00100C", "ITO CO., LTD." },
    { "00100D", "Cisco Systems, Inc" },
    { "00100E", "MICRO LINEAR COPORATION" },
    { "00100F", "INDUSTRIAL CPU SYSTEMS" },
    { "001010", "INITIO CORPORATION" },
    { "001011", "Cisco Systems, Inc" },
    { "001012", "PROCESSOR SYSTEMS (I) PVT LTD" },
    { "001013", "Kontron America, Inc." },
    { "001014", "Cisco Systems, Inc" },
    { "001015", "OOmon Inc." },
    { "001016", "T.SQWARE" },
    { "001017", "Bosch Access Systems GmbH" },
    { "001018", "Broadcom" },
    { "001019", "SIRONA DENTAL SYSTEMS GmbH & Co. KG" },
    { "00101A", "PictureTel Corp." },
    { "00101B", "CORNET TECHNOLOGY, INC." },
    { "00101C", "OHM TECHNOLOGIES INTL, LLC" },
    { "00101D", "WINBOND ELECTRONICS CORP." },
    { "00101E", "MATSUSHITA ELECTRONIC INSTRUMENTS CORP." },
    { "00101F", "Cisco Systems, Inc" },
    { "001020", "Hand Held Products Inc" },
    { "001021", "ENCANTO NETWORKS, INC." },
    { "001022", "SatCom Media Corporation" },
    { "001023", "Network Equipment Technologies" },
    { "001024", "NAGOYA ELECTRIC WORKS CO., LTD" },
    { "001025", "Grayhill, Inc" },
    { "001026", "ACCELERATED NETWORKS, INC." },
    { "001027", "L-3 COMMUNICATIONS EAST" },
    { "001028", "COMPUTER TECHNICA, INC." },
    { "001029", "Cisco Systems, Inc" },
    { "00102A", "ZF MICROSYSTEMS, INC." },
    { "00102B", "UMAX DATA SYSTEMS, INC." },
    { "00102C", "Lasat Networks A/S" },
    { "00102D", "HITACHI SOFTWARE ENGINEERING" },
    { "00102E", "NETWORK SYSTEMS & TECHNOLOGIES PVT. LTD." },
    { "00102F", "Cisco Systems, Inc" },
    { "001030", "EION Inc." },
    { "001031", "OBJECTIVE COMMUNICATIONS, INC." },
    { "001032", "ALTA TECHNOLOGY" },
    { "001033", "ACCESSLAN COMMUNICATIONS, INC." },
    { "001034", "GNP Computers" },
    { "001035", "Elitegroup Computer Systems Co.,Ltd." },
    { "001036", "INTER-TEL INTEGRATED SYSTEMS" },
    { "001037", "CYQ've Technology Co., Ltd." },
    { "001038", "Micro Research Ltd." },
    { "001039", "Vectron Systems AG" },
    { "00103A", "DIAMOND NETWORK TECH" },
    { "00103B", "HIPPI NETWORKING FORUM" },
    { "00103C", "IC ENSEMBLE, INC." },
    { "00103D", "PHASECOM, LTD." },
    { "00103E", "NETSCHOOLS CORPORATION" },
    { "00103F", "TOLLGRADE COMMUNICATIONS, INC." },
    { "001040", "INTERMEC CORPORATION" },
    { "001041", "BRISTOL BABCOCK, INC." },
    { "001042", "Alacritech, Inc." },
    { "001043", "A2 CORPORATION" },
    { "001044", "InnoLabs Corporation" },
    { "001045", "Nortel Networks" },
    { "001046", "ALCORN MCBRIDE INC." },
    { "001047", "ECHO ELETRIC CO. LTD." },
    { "001048", "HTRC AUTOMATION, INC." },
    { "001049", "ShoreTel, Inc" },
    { "00104A", "The Parvus Corporation" },
    { "00104B", "3COM" },
    { "00104C", "Teledyne LeCroy, Inc" },
    { "00104D", "SURTEC INDUSTRIES, INC." },
    { "00104E", "CEOLOGIC" },
    { "00104F", "Oracle Corporation " },
    { "001050", "RION CO., LTD." },
    { "001051", "CMICRO CORPORATION" },
    { "001052", "METTLER-TOLEDO (ALBSTADT) GMBH" },
    { "001053", "COMPUTER TECHNOLOGY CORP." },
    { "001054", "Cisco Systems, Inc" },
    { "001055", "FUJITSU MICROELECTRONICS, INC." },
    { "001056", "SODICK CO., LTD." },
    { "001057", "Rebel.com, Inc." },
    { "001058", "ArrowPoint Communications" },
    { "001059", "DIABLO RESEARCH CO. LLC" },
    { "00105A", "3COM" },
    { "00105B", "NET INSIGHT AB" },
    { "00105C", "QUANTUM DESIGNS (H.K.) LTD." },
    { "00105D", "Draeger Medical" },
    { "00105E", "Spirent plc, Service Assurance Broadband" },
    { "00105F", "ZODIAC DATA SYSTEMS" },
    { "001060", "BILLIONTON SYSTEMS, INC." },
    { "001061", "HOSTLINK CORP." },
    { "001062", "NX SERVER, ILNC." },
    { "001063", "STARGUIDE DIGITAL NETWORKS" },
    { "001064", "DNPG, LLC" },
    { "001065", "RADYNE CORPORATION" },
    { "001066", "ADVANCED CONTROL SYSTEMS, INC." },
    { "001067", "Ericsson" },
    { "001068", "COMOS TELECOM" },
    { "001069", "HELIOSS COMMUNICATIONS, INC." },
    { "00106A", "DIGITAL MICROWAVE CORPORATION" },
    { "00106B", "SONUS NETWORKS, INC." },
    { "00106C", "EDNT GmbH" },
    { "00106D", "Axxcelera Broadband Wireless" },
    { "00106E", "TADIRAN COM. LTD." },
    { "00106F", "TRENTON TECHNOLOGY INC." },
    { "001070", "CARADON TREND LTD." },
    { "001071", "ADVANET INC." },
    { "001072", "GVN TECHNOLOGIES, INC." },
    { "001073", "TECHNOBOX, INC." },
    { "001074", "ATEN INTERNATIONAL CO., LTD." },
    { "001075", "Segate Technology LLC" },
    { "001076", "EUREM GmbH" },
    { "001077", "SAF DRIVE SYSTEMS, LTD." },
    { "001078", "NUERA COMMUNICATIONS, INC." },
    { "001079", "Cisco Systems, Inc" },
    { "00107A", "AmbiCom, Inc." },
    { "00107B", "Cisco Systems, Inc" },
    { "00107C", "P-COM, INC." },
    { "00107D", "AURORA COMMUNICATIONS, LTD." },
    { "00107E", "BACHMANN ELECTRONIC GmbH" },
    { "00107F", "CRESTRON ELECTRONICS, INC." },
    { "001080", "METAWAVE COMMUNICATIONS" },
    { "001081", "DPS, INC." },
    { "001082", "JNA TELECOMMUNICATIONS LIMITED" },
    { "001083", "Hewlett Packard" },
    { "001084", "K-BOT COMMUNICATIONS" },
    { "001085", "POLARIS COMMUNICATIONS, INC." },
    { "001086", "ATTO Technology, Inc." },
    { "001087", "XSTREAMIS PLC" },
    { "001088", "AMERICAN NETWORKS INC." },
    { "001089", "WebSonic" },
    { "00108A", "TeraLogic, Inc." },
    { "00108B", "LASERANIMATION SOLLINGER GMBH" },
    { "00108C", "Fujitsu Services Ltd" },
    { "00108D", "Johnson Controls, Inc." },
    { "00108E", "HUGH SYMONS CONCEPT Technologies Ltd." },
    { "00108F", "RAPTOR SYSTEMS" },
    { "001090", "CIMETRICS, INC." },
    { "001091", "NO WIRES NEEDED BV" },
    { "001092", "NETCORE INC." },
    { "001093", "CMS COMPUTERS, LTD." },
    { "001094", "Performance Analysis Broadband, Spirent plc" },
    { "001095", "Thomson Inc." },
    { "001096", "TRACEWELL SYSTEMS, INC." },
    { "001097", "WinNet Metropolitan Communications Systems, Inc." },
    { "001098", "STARNET TECHNOLOGIES, INC." },
    { "001099", "InnoMedia, Inc." },
    { "00109A", "NETLINE" },
    { "00109B", "Emulex Corporation" },
    { "00109C", "MG Co., Ltd." },
    { "00109D", "CLARINET SYSTEMS, INC." },
    { "00109E", "AWARE, INC." },
    { "00109F", "PAVO, INC." },
    { "0010A0", "INNOVEX TECHNOLOGIES, INC." },
    { "0010A1", "KENDIN SEMICONDUCTOR, INC." },
    { "0010A2", "TNS" },
    { "0010A3", "OMNITRONIX, INC." },
    { "0010A4", "XIRCOM" },
    { "0010A5", "OXFORD INSTRUMENTS" },
    { "0010A6", "Cisco Systems, Inc" },
    { "0010A7", "UNEX TECHNOLOGY CORPORATION" },
    { "0010A8", "RELIANCE COMPUTER CORP." },
    { "0010A9", "ADHOC TECHNOLOGIES" },
    { "0010AA", "MEDIA4, INC." },
    { "0010AB", "KOITO ELECTRIC INDUSTRIES, LTD." },
    { "0010AC", "IMCI TECHNOLOGIES" },
    { "0010AD", "SOFTRONICS USB, INC." },
    { "0010AE", "SHINKO ELECTRIC INDUSTRIES CO." },
    { "0010AF", "TAC SYSTEMS, INC." },
    { "0010B0", "MERIDIAN TECHNOLOGY CORP." },
    { "0010B1", "FOR-A CO., LTD." },
    { "0010B2", "COACTIVE AESTHETICS" },
    { "0010B3", "NOKIA MULTIMEDIA TERMINALS" },
    { "0010B4", "ATMOSPHERE NETWORKS" },
    { "0010B5", "Accton Technology Corp" },
    { "0010B6", "ENTRATA COMMUNICATIONS CORP." },
    { "0010B7", "COYOTE TECHNOLOGIES, LLC" },
    { "0010B8", "ISHIGAKI COMPUTER SYSTEM CO." },
    { "0010B9", "MAXTOR CORP." },
    { "0010BA", "MARTINHO-DAVIS SYSTEMS, INC." },
    { "0010BB", "DATA & INFORMATION TECHNOLOGY" },
    { "0010BC", "Aastra Telecom" },
    { "0010BD", "THE TELECOMMUNICATION TECHNOLOGY COMMITTEE (TTC)" },
    { "0010BE", "MARCH NETWORKS CORPORATION" },
    { "0010BF", "InterAir Wireless" },
    { "0010C0", "ARMA, Inc." },
    { "0010C1", "OI ELECTRIC CO.,LTD" },
    { "0010C2", "WILLNET, INC." },
    { "0010C3", "CSI-CONTROL SYSTEMS" },
    { "0010C4", "MEDIA GLOBAL LINKS CO., LTD." },
    { "0010C5", "PROTOCOL TECHNOLOGIES, INC." },
    { "0010C6", "Universal Global Scientific Industrial Co., Ltd." },
    { "0010C7", "DATA TRANSMISSION NETWORK" },
    { "0010C8", "COMMUNICATIONS ELECTRONICS SECURITY GROUP" },
    { "0010C9", "MITSUBISHI ELECTRONICS LOGISTIC SUPPORT CO." },
    { "0010CA", "Telco Systems, Inc. " },
    { "0010CB", "FACIT K.K." },
    { "0010CC", "CLP COMPUTER LOGISTIK PLANUNG GmbH" },
    { "0010CD", "INTERFACE CONCEPT" },
    { "0010CE", "VOLAMP, LTD." },
    { "0010CF", "FIBERLANE COMMUNICATIONS" },
    { "0010D0", "WITCOM, LTD." },
    { "0010D1", "Top Layer Networks, Inc." },
    { "0010D2", "NITTO TSUSHINKI CO., LTD" },
    { "0010D3", "GRIPS ELECTRONIC GMBH" },
    { "0010D4", "STORAGE COMPUTER CORPORATION" },
    { "0010D5", "IMASDE CANARIAS, S.A." },
    { "0010D6", "Exelis" },
    { "0010D7", "ARGOSY RESEARCH INC." },
    { "0010D8", "CALISTA" },
    { "0010D9", "IBM JAPAN, FUJISAWA MT+D" },
    { "0010DA", "Kollmorgen Corp" },
    { "0010DB", "Juniper Networks" },
    { "0010DC", "MICRO-STAR INTERNATIONAL CO., LTD." },
    { "0010DD", "ENABLE SEMICONDUCTOR, INC." },
    { "0010DE", "INTERNATIONAL DATACASTING CORPORATION" },
    { "0010DF", "RISE COMPUTER INC." },
    { "0010E0", "Oracle Corporation " },
    { "0010E1", "S.I. TECH, INC." },
    { "0010E2", "ArrayComm, Inc." },
    { "0010E3", "Hewlett Packard" },
    { "0010E4", "NSI CORPORATION" },
    { "0010E5", "SOLECTRON TEXAS" },
    { "0010E6", "APPLIED INTELLIGENT SYSTEMS, INC." },
    { "0010E7", "Breezecom, Ltd." },
    { "0010E8", "TELOCITY, INCORPORATED" },
    { "0010E9", "RAIDTEC LTD." },
    { "0010EA", "ADEPT TECHNOLOGY" },
    { "0010EB", "SELSIUS SYSTEMS, INC." },
    { "0010EC", "Embedded Planet" },
    { "0010ED", "SUNDANCE TECHNOLOGY, INC." },
    { "0010EE", "CTI PRODUCTS, INC." },
    { "0010EF", "DBTEL INCORPORATED" },
    { "0010F0", "RITTAL-WERK RUDOLF LOH GmbH & Co." },
    { "0010F1", "I-O CORPORATION" },
    { "0010F2", "ANTEC" },
    { "0010F3", "Nexcom International Co., Ltd." },
    { "0010F4", "Vertical Communications" },
    { "0010F5", "AMHERST SYSTEMS, INC." },
    { "0010F6", "Cisco Systems, Inc" },
    { "0010F7", "IRIICHI TECHNOLOGIES Inc." },
    { "0010F8", "TEXIO TECHNOLOGY CORPORATION" },
    { "0010F9", "UNIQUE SYSTEMS, INC." },
    { "0010FA", "Apple, Inc." },
    { "0010FB", "ZIDA TECHNOLOGIES LIMITED" },
    { "0010FC", "BROADBAND NETWORKS, INC." },
    { "0010FD", "COCOM A/S" },
    { "0010FE", "DIGITAL EQUIPMENT CORPORATION" },
    { "0010FF", "Cisco Systems, Inc" },
    { "001100", "Schneider Electric" },
    { "001101", "CET Technologies Pte Ltd" },
    { "001102", "Aurora Multimedia Corp." },
    { "001103", "kawamura electric inc." },
    { "001104", "TELEXY" },
    { "001105", "Sunplus Technology Co., Ltd." },
    { "001106", "Siemens NV (Belgium)" },
    { "001107", "RGB Networks Inc." },
    { "001108", "Orbital Data Corporation" },
    { "001109", "Micro-Star International" },
    { "00110A", "Hewlett Packard" },
    { "00110B", "Franklin Technology Systems" },
    { "00110C", "Atmark Techno, Inc." },
    { "00110D", "SANBlaze Technology, Inc." },
    { "00110E", "Tsurusaki Sealand Transportation Co. Ltd." },
    { "00110F", "netplat,Inc." },
    { "001110", "Maxanna Technology Co., Ltd." },
    { "001111", "Intel Corporation" },
    { "001112", "Honeywell CMSS" },
    { "001113", "Fraunhofer FOKUS" },
    { "001114", "EverFocus Electronics Corp." },
    { "001115", "EPIN Technologies, Inc." },
    { "001116", "COTEAU VERT CO., LTD." },
    { "001117", "CESNET" },
    { "001118", "BLX IC Design Corp., Ltd." },
    { "001119", "Solteras, Inc." },
    { "00111A", "Commscope" },
    { "00111B", "Targa Systems Div L-3 Communications" },
    { "00111C", "Pleora Technologies Inc." },
    { "00111D", "Hectrix Limited" },
    { "00111E", "ETHERNET Powerlink Standarization Group (EPSG)" },
    { "00111F", "Doremi Labs, Inc." },
    { "001120", "Cisco Systems, Inc" },
    { "001121", "Cisco Systems, Inc" },
    { "001122", "CIMSYS Inc" },
    { "001123", "Appointech, Inc." },
    { "001124", "Apple, Inc." },
    { "001125", "IBM Corp" },
    { "001126", "Venstar Inc." },
    { "001127", "TASI, Inc" },
    { "001128", "Streamit" },
    { "001129", "Paradise Datacom Ltd." },
    { "00112A", "Niko NV" },
    { "00112B", "NetModule AG" },
    { "00112C", "IZT GmbH" },
    { "00112D", "iPulse Systems" },
    { "00112E", "CEICOM" },
    { "00112F", "ASUSTek COMPUTER INC." },
    { "001130", "Allied Telesis (Hong Kong) Ltd." },
    { "001131", "UNATECH. CO.,LTD" },
    { "001132", "Synology Incorporated" },
    { "001133", "Siemens AG Austria" },
    { "001134", "MediaCell, Inc." },
    { "001135", "Grandeye Ltd" },
    { "001136", "Goodrich Sensor Systems" },
    { "001137", "AICHI ELECTRIC CO., LTD." },
    { "001138", "TAISHIN CO., LTD." },
    { "001139", "STOEBER ANTRIEBSTECHNIK GmbH + Co. KG." },
    { "00113A", "SHINBORAM" },
    { "00113B", "Micronet Communications Inc." },
    { "00113C", "Micronas GmbH" },
    { "00113D", "KN SOLTEC CO.,LTD." },
    { "00113E", "JL Corporation" },
    { "00113F", "Alcatel DI" },
    { "001140", "Nanometrics Inc." },
    { "001141", "GoodMan Corporation" },
    { "001142", "e-SMARTCOM INC." },
    { "001143", "Dell Inc." },
    { "001144", "Assurance Technology Corp" },
    { "001145", "ValuePoint Networks" },
    { "001146", "Telecard-Pribor Ltd" },
    { "001147", "Secom-Industry co.LTD." },
    { "001148", "Prolon Control Systems" },
    { "001149", "Proliphix Inc." },
    { "00114A", "KAYABA INDUSTRY Co,.Ltd." },
    { "00114B", "Francotyp-Postalia GmbH" },
    { "00114C", "caffeina applied research ltd." },
    { "00114D", "Atsumi Electric Co.,LTD." },
    { "00114E", "690885 Ontario Inc." },
    { "00114F", "US Digital Television, Inc" },
    { "001150", "Belkin Corporation" },
    { "001151", "Mykotronx" },
    { "001152", "Eidsvoll Electronics AS" },
    { "001153", "Trident Tek, Inc." },
    { "001154", "Webpro Technologies Inc." },
    { "001155", "Sevis Systems" },
    { "001156", "Pharos Systems NZ" },
    { "001157", "Oki Electric Industry Co., Ltd." },
    { "001158", "Nortel Networks" },
    { "001159", "MATISSE NETWORKS INC" },
    { "00115A", "Ivoclar Vivadent AG" },
    { "00115B", "Elitegroup Computer Systems Co.,Ltd." },
    { "00115C", "Cisco Systems, Inc" },
    { "00115D", "Cisco Systems, Inc" },
    { "00115E", "ProMinent Dosiertechnik GmbH" },
    { "00115F", "ITX Security Co., Ltd." },
    { "001160", "ARTDIO Company Co., LTD" },
    { "001161", "NetStreams, LLC" },
    { "001162", "STAR MICRONICS CO.,LTD." },
    { "001163", "SYSTEM SPA DEPT. ELECTRONICS" },
    { "001164", "ACARD Technology Corp." },
    { "001165", "ZNYX Networks, Inc." },
    { "001166", "Taelim Electronics Co., Ltd." },
    { "001167", "Integrated System Solution Corp." },
    { "001168", "HomeLogic LLC" },
    { "001169", "EMS Satcom" },
    { "00116A", "Domo Ltd" },
    { "00116B", "Digital Data Communications Asia Co.,Ltd" },
    { "00116C", "Nanwang Multimedia Inc.,Ltd" },
    { "00116D", "American Time and Signal" },
    { "00116E", "Peplink International Ltd." },
    { "00116F", "Netforyou Co., LTD." },
    { "001170", "GSC SRL" },
    { "001171", "DEXTER Communications, Inc." },
    { "001172", "COTRON CORPORATION" },
    { "001173", "SMART Storage Systems" },
    { "001174", "Mojo Networks, Inc." },
    { "001175", "Intel Corporation" },
    { "001176", "Intellambda Systems, Inc." },
    { "001177", "Coaxial Networks, Inc." },
    { "001178", "Chiron Technology Ltd" },
    { "001179", "Singular Technology Co. Ltd." },
    { "00117A", "Singim International Corp." },
    { "00117B", "Büchi Labortechnik AG" },
    { "00117C", "e-zy.net" },
    { "00117D", "ZMD America, Inc." },
    { "00117E", "Midmark Corp" },
    { "00117F", "Neotune Information Technology Corporation,.LTD" },
    { "001180", "Commscope" },
    { "001181", "InterEnergy Co.Ltd," },
    { "001182", "IMI Norgren Ltd" },
    { "001183", "Datalogic ADC, Inc." },
    { "001184", "Humo Laboratory,Ltd." },
    { "001185", "Hewlett Packard" },
    { "001186", "Prime Systems, Inc." },
    { "001187", "Category Solutions, Inc" },
    { "001188", "Enterasys" },
    { "001189", "Aerotech Inc" },
    { "00118A", "Viewtran Technology Limited" },
    { "00118B", "Alcatel-Lucent Enterprise" },
    { "00118C", "Missouri Department of Transportation" },
    { "00118D", "Hanchang System Corp." },
    { "00118E", "Halytech Mace" },
    { "00118F", "EUTECH INSTRUMENTS PTE. LTD." },
    { "001190", "Digital Design Corporation" },
    { "001191", "CTS-Clima Temperatur Systeme GmbH" },
    { "001192", "Cisco Systems, Inc" },
    { "001193", "Cisco Systems, Inc" },
    { "001194", "Chi Mei Communication Systems, Inc." },
    { "001195", "D-Link Corporation" },
    { "001196", "Actuality Systems, Inc." },
    { "001197", "Monitoring Technologies Limited" },
    { "001198", "Prism Media Products Limited" },
    { "001199", "2wcom Systems GmbH" },
    { "00119A", "Alkeria srl" },
    { "00119B", "Telesynergy Research Inc." },
    { "00119C", "EP&T Energy" },
    { "00119D", "Diginfo Technology Corporation" },
    { "00119E", "Solectron Brazil" },
    { "00119F", "Nokia Danmark A/S" },
    { "0011A0", "Vtech Engineering Canada Ltd" },
    { "0011A1", "VISION NETWARE CO.,LTD" },
    { "0011A2", "Manufacturing Technology Inc" },
    { "0011A3", "LanReady Technologies Inc." },
    { "0011A4", "JStream Technologies Inc." },
    { "0011A5", "Fortuna Electronic Corp." },
    { "0011A6", "Sypixx Networks" },
    { "0011A7", "Infilco Degremont Inc." },
    { "0011A8", "Quest Technologies" },
    { "0011A9", "Nurivoice Co., Ltd" },
    { "0011AA", "Uniclass Technology, Co., LTD" },
    { "0011AB", "TRUSTABLE TECHNOLOGY CO.,LTD." },
    { "0011AC", "Simtec Electronics" },
    { "0011AD", "Shanghai Ruijie Technology" },
    { "0011AE", "Commscope" },
    { "0011AF", "Medialink-i,Inc" },
    { "0011B0", "Fortelink Inc." },
    { "0011B1", "BlueExpert Technology Corp." },
    { "0011B2", "2001 Technology Inc." },
    { "0011B3", "YOSHIMIYA CO.,LTD." },
    { "0011B4", "Westermo Network Technologies AB" },
    { "0011B5", "Shenzhen Powercom Co.,Ltd" },
    { "0011B6", "Open Systems International" },
    { "0011B7", "Octalix B.V." },
    { "0011B8", "Liebherr - Elektronik GmbH" },
    { "0011B9", "Inner Range Pty. Ltd." },
    { "0011BA", "Elexol Pty Ltd" },
    { "0011BB", "Cisco Systems, Inc" },
    { "0011BC", "Cisco Systems, Inc" },
    { "0011BD", "Bombardier Transportation" },
    { "0011BE", "AGP Telecom Co. Ltd" },
    { "0011BF", "AESYS S.p.A." },
    { "0011C0", "Aday Technology Inc" },
    { "0011C1", "4P MOBILE DATA PROCESSING" },
    { "0011C2", "United Fiber Optic Communication" },
    { "0011C3", "Transceiving System Technology Corporation" },
    { "0011C4", "Terminales de Telecomunicacion Terrestre, S.L." },
    { "0011C5", "TEN Technology" },
    { "0011C6", "Seagate Technology" },
    { "0011C7", "Raymarine UK Ltd" },
    { "0011C8", "Powercom Co., Ltd." },
    { "0011C9", "MTT Corporation" },
    { "0011CA", "Long Range Systems, Inc." },
    { "0011CB", "Jacobsons AB" },
    { "0011CC", "Guangzhou Jinpeng Group Co.,Ltd." },
    { "0011CD", "Axsun Technologies" },
    { "0011CE", "Ubisense Limited" },
    { "0011CF", "Thrane & Thrane A/S" },
    { "0011D0", "Tandberg Data ASA" },
    { "0011D1", "Soft Imaging System GmbH" },
    { "0011D2", "Perception Digital Ltd" },
    { "0011D3", "NextGenTel Holding ASA" },
    { "0011D4", "NetEnrich, Inc" },
    { "0011D5", "Hangzhou Sunyard System Engineering Co.,Ltd." },
    { "0011D6", "HandEra, Inc." },
    { "0011D7", "eWerks Inc" },
    { "0011D8", "ASUSTek COMPUTER INC." },
    { "0011D9", "TiVo" },
    { "0011DA", "Vivaas Technology Inc." },
    { "0011DB", "Land-Cellular Corporation" },
    { "0011DC", "Glunz & Jensen" },
    { "0011DD", "FROMUS TEC. Co., Ltd." },
    { "0011DE", "EURILOGIC" },
    { "0011DF", "Current Energy" },
    { "0011E0", "U-MEDIA Communications, Inc." },
    { "0011E1", "Arcelik A.S" },
    { "0011E2", "Hua Jung Components Co., Ltd." },
    { "0011E3", "Thomson, Inc." },
    { "0011E4", "Danelec Electronics A/S" },
    { "0011E5", "KCodes Corporation" },
    { "0011E6", "Scientific Atlanta" },
    { "0011E7", "WORLDSAT - Texas de France" },
    { "0011E8", "Tixi.Com" },
    { "0011E9", "STARNEX CO., LTD." },
    { "0011EA", "IWICS Inc." },
    { "0011EB", "Innovative Integration" },
    { "0011EC", "AVIX INC." },
    { "0011ED", "802 Global" },
    { "0011EE", "Estari, Inc." },
    { "0011EF", "Conitec Datensysteme GmbH" },
    { "0011F0", "Wideful Limited" },
    { "0011F1", "QinetiQ Ltd" },
    { "0011F2", "Institute of Network Technologies" },
    { "0011F3", "NeoMedia Europe AG" },
    { "0011F4", "woori-net" },
    { "0011F5", "ASKEY COMPUTER CORP" },
    { "0011F6", "Asia Pacific Microsystems , Inc." },
    { "0011F7", "Shenzhen Forward Industry Co., Ltd" },
    { "0011F8", "AIRAYA Corp" },
    { "0011F9", "Nortel Networks" },
    { "0011FA", "Rane Corporation" },
    { "0011FB", "Heidelberg Engineering GmbH" },
    { "0011FC", "HARTING Electronics GmbH" },
    { "0011FD", "KORG INC." },
    { "0011FE", "Keiyo System Research, Inc." },
    { "0011FF", "Digitro Tecnologia Ltda" },
    { "001200", "Cisco Systems, Inc" },
    { "001201", "Cisco Systems, Inc" },
    { "001202", "Decrane Aerospace - Audio International Inc." },
    { "001203", "ActivNetworks" },
    { "001204", "u10 Networks, Inc." },
    { "001205", "Terrasat Communications, Inc." },
    { "001206", "iQuest (NZ) Ltd" },
    { "001207", "Head Strong International Limited" },
    { "001208", "Gantner Instruments GmbH" },
    { "001209", "Fastrax Ltd" },
    { "00120A", "Emerson Climate Technologies GmbH" },
    { "00120B", "Chinasys Technologies Limited" },
    { "00120C", "CE-Infosys Pte Ltd" },
    { "00120D", "Advanced Telecommunication Technologies, Inc." },
    { "00120E", "AboCom" },
    { "00120F", "IEEE 802.3" },
    { "001210", "WideRay Corp" },
    { "001211", "Protechna Herbst GmbH & Co. KG" },
    { "001212", "PLUS Corporation" },
    { "001213", "Metrohm AG" },
    { "001214", "Koenig & Bauer AG" },
    { "001215", "iStor Networks, Inc." },
    { "001216", "ICP Internet Communication Payment AG" },
    { "001217", "Cisco-Linksys, LLC" },
    { "001218", "ARUZE Corporation" },
    { "001219", "General Datacomm LLC" },
    { "00121A", "Techno Soft Systemnics Inc." },
    { "00121B", "Sound Devices, LLC" },
    { "00121C", "PARROT SA" },
    { "00121D", "Netfabric Corporation" },
    { "00121E", "Juniper Networks" },
    { "00121F", "Harding Instruments" },
    { "001220", "Cadco Systems" },
    { "001221", "B.Braun Melsungen AG" },
    { "001222", "Skardin (UK) Ltd" },
    { "001223", "Pixim" },
    { "001224", "NexQL Corporation" },
    { "001225", "Commscope" },
    { "001226", "Japan Direx Corporation" },
    { "001227", "Franklin Electric Co., Inc." },
    { "001228", "Data Ltd." },
    { "001229", "BroadEasy Technologies Co.,Ltd" },
    { "00122A", "VTech Telecommunications Ltd." },
    { "00122B", "Virbiage Pty Ltd" },
    { "00122C", "Soenen Controls N.V." },
    { "00122D", "SiNett Corporation" },
    { "00122E", "Signal Technology - AISD" },
    { "00122F", "Sanei Electric Inc." },
    { "001230", "Picaso Infocommunication CO., LTD." },
    { "001231", "Motion Control Systems, Inc." },
    { "001232", "LeWiz Communications Inc." },
    { "001233", "JRC TOKKI Co.,Ltd." },
    { "001234", "Camille Bauer" },
    { "001235", "Andrew Corporation" },
    { "001236", "ConSentry Networks" },
    { "001237", "Texas Instruments" },
    { "001238", "SetaBox Technology Co., Ltd." },
    { "001239", "S Net Systems Inc." },
    { "00123A", "Posystech Inc., Co." },
    { "00123B", "KeRo Systems ApS" },
    { "00123C", "Second Rule LLC" },
    { "00123D", "GES Co, Ltd" },
    { "00123E", "ERUNE technology Co., Ltd." },
    { "00123F", "Dell Inc." },
    { "001240", "AMOI ELECTRONICS CO.,LTD" },
    { "001241", "a2i marketing center" },
    { "001242", "Millennial Net" },
    { "001243", "Cisco Systems, Inc" },
    { "001244", "Cisco Systems, Inc" },
    { "001245", "Zellweger Analytics, Inc." },
    { "001246", "T.O.M TECHNOLOGY INC.." },
    { "001247", "Samsung Electronics Co.,Ltd" },
    { "001248", "Dell EMC" },
    { "001249", "Delta Elettronica S.p.A." },
    { "00124A", "Dedicated Devices, Inc." },
    { "00124B", "Texas Instruments" },
    { "00124C", "BBWM Corporation" },
    { "00124D", "Inducon BV" },
    { "00124E", "XAC AUTOMATION CORP." },
    { "00124F", "nVent" },
    { "001250", "Tokyo Aircaft Instrument Co., Ltd." },
    { "001251", "SILINK" },
    { "001252", "Citronix, LLC" },
    { "001253", "AudioDev AB" },
    { "001254", "Spectra Technologies Holdings Company Ltd" },
    { "001255", "NetEffect Incorporated" },
    { "001256", "LG INFORMATION & COMM." },
    { "001257", "LeapComm Communication Technologies Inc." },
    { "001258", "TechVoIP Sp z o.o." },
    { "001259", "THERMO ELECTRON KARLSRUHE" },
    { "00125A", "Microsoft Corporation" },
    { "00125B", "KAIMEI ELECTRONI" },
    { "00125C", "Green Hills Software, Inc." },
    { "00125D", "CyberNet Inc." },
    { "00125E", "CAEN" },
    { "00125F", "AWIND Inc." },
    { "001260", "Stanton Magnetics,inc." },
    { "001261", "Adaptix, Inc" },
    { "001262", "Nokia Danmark A/S" },
    { "001263", "Data Voice Technologies GmbH" },
    { "001264", "daum electronic gmbh" },
    { "001265", "Enerdyne Technologies, Inc." },
    { "001266", "Swisscom Hospitality Services SA" },
    { "001267", "Panasonic Corporation" },
    { "001268", "IPS d.o.o." },
    { "001269", "Value Electronics" },
    { "00126A", "OPTOELECTRONICS Co., Ltd." },
    { "00126B", "Ascalade Communications Limited" },
    { "00126C", "Visonic Technologies 1993 Ltd." },
    { "00126D", "University of California, Berkeley" },
    { "00126E", "Seidel Elektronik GmbH Nfg.KG" },
    { "00126F", "Rayson Technology Co., Ltd." },
    { "001270", "NGES Denro Systems" },
    { "001271", "Measurement Computing Corp" },
    { "001272", "Redux Communications Ltd." },
    { "001273", "Stoke Inc" },
    { "001274", "NIT lab" },
    { "001275", "Sentilla Corporation" },
    { "001276", "CG Power Systems Ireland Limited" },
    { "001277", "Beijer Electronics Corp." },
    { "001278", "International Bar Code" },
    { "001279", "Hewlett Packard" },
    { "00127A", "Sanyu Industry Co.,Ltd." },
    { "00127B", "VIA Networking Technologies, Inc." },
    { "00127C", "SWEGON AB" },
    { "00127D", "MobileAria" },
    { "00127E", "Digital Lifestyles Group, Inc." },
    { "00127F", "Cisco Systems, Inc" },
    { "001280", "Cisco Systems, Inc" },
    { "001281", "March Networks S.p.A." },
    { "001282", "Qovia" },
    { "001283", "Nortel Networks" },
    { "001284", "Lab33 Srl" },
    { "001285", "Gizmondo Europe Ltd" },
    { "001286", "ENDEVCO CORP" },
    { "001287", "Digital Everywhere Unterhaltungselektronik GmbH" },
    { "001288", "2Wire Inc" },
    { "001289", "Advance Sterilization Products" },
    { "00128A", "Commscope" },
    { "00128B", "Sensory Networks Inc" },
    { "00128C", "Woodward Governor" },
    { "00128D", "STB Datenservice GmbH" },
    { "00128E", "Q-Free ASA" },
    { "00128F", "Montilio" },
    { "001290", "KYOWA Electric & Machinery Corp." },
    { "001291", "KWS Computersysteme GmbH" },
    { "001292", "Griffin Technology" },
    { "001293", "ABB Switzerland Ltd." },
    { "001294", "SUMITOMO ELECTRIC DEVICE INNOVATIONS, INC" },
    { "001295", "Aiware Inc." },
    { "001296", "Addlogix" },
    { "001297", "O2Micro, Inc." },
    { "001298", "MICO ELECTRIC(SHENZHEN) LIMITED" },
    { "001299", "Ktech Telecommunications Inc" },
    { "00129A", "IRT Electronics Pty Ltd" },
    { "00129B", "E2S Electronic Engineering Solutions, S.L." },
    { "00129C", "Yulinet" },
    { "00129D", "First International Computer do Brasil" },
    { "00129E", "Surf Communications Inc." },
    { "00129F", "RAE Systems" },
    { "0012A0", "NeoMeridian Sdn Bhd" },
    { "0012A1", "BluePacket Communications Co., Ltd." },
    { "0012A2", "VITA" },
    { "0012A3", "Trust International B.V." },
    { "0012A4", "ThingMagic, LLC" },
    { "0012A5", "Dolphin Interconnect Solutions AS" },
    { "0012A6", "Dolby Australia" },
    { "0012A7", "ISR TECHNOLOGIES Inc" },
    { "0012A8", "intec GmbH" },
    { "0012A9", "3Com Ltd" },
    { "0012AA", "IEE, Inc." },
    { "0012AB", "WiLife, Inc." },
    { "0012AC", "ONTIMETEK INC." },
    { "0012AD", "VIVAVIS AG" },
    { "0012AE", "HLS HARD-LINE Solutions Inc." },
    { "0012AF", "ELPRO Technologies" },
    { "0012B0", "Efore Oyj (Plc)" },
    { "0012B1", "Dai Nippon Printing Co., Ltd" },
    { "0012B2", "AVOLITES LTD." },
    { "0012B3", "Advance Wireless Technology Corp." },
    { "0012B4", "Work Microwave GmbH" },
    { "0012B5", "Vialta, Inc." },
    { "0012B6", "Santa Barbara Infrared, Inc." },
    { "0012B7", "PTW Freiburg" },
    { "0012B8", "G2 Microsystems" },
    { "0012B9", "Fusion Digital Technology" },
    { "0012BA", "FSI Systems, Inc." },
    { "0012BB", "Telecommunications Industry Association TR-41 Committee" },
    { "0012BC", "Echolab LLC" },
    { "0012BD", "Avantec Manufacturing Limited" },
    { "0012BE", "Astek Corporation" },
    { "0012BF", "Arcadyan Technology Corporation" },
    { "0012C0", "HotLava Systems, Inc." },
    { "0012C1", "Check Point Software Technologies" },
    { "0012C2", "Apex Electronics Factory" },
    { "0012C3", "WIT S.A." },
    { "0012C4", "Viseon, Inc." },
    { "0012C5", "V-Show Technology (China) Co.,Ltd" },
    { "0012C6", "TGC America, Inc" },
    { "0012C7", "SECURAY Technologies Ltd.Co." },
    { "0012C8", "Perfect tech" },
    { "0012C9", "Commscope" },
    { "0012CA", "Mechatronic Brick Aps" },
    { "0012CB", "CSS Inc." },
    { "0012CC", "Bitatek CO., LTD" },
    { "0012CD", "ASEM SpA" },
    { "0012CE", "Advanced Cybernetics Group" },
    { "0012CF", "Accton Technology Corp" },
    { "0012D0", "Gossen-Metrawatt-GmbH" },
    { "0012D1", "Texas Instruments" },
    { "0012D2", "Texas Instruments" },
    { "0012D3", "Zetta Systems, Inc." },
    { "0012D4", "Princeton Technology, Ltd" },
    { "0012D5", "Motion Reality Inc." },
    { "0012D6", "Jiangsu Yitong High-Tech Co.,Ltd" },
    { "0012D7", "Invento Networks, Inc." },
    { "0012D8", "International Games System Co., Ltd." },
    { "0012D9", "Cisco Systems, Inc" },
    { "0012DA", "Cisco Systems, Inc" },
    { "0012DB", "ZIEHL industrie-elektronik GmbH + Co KG" },
    { "0012DC", "SunCorp Industrial Limited" },
    { "0012DD", "Shengqu Information Technology (Shanghai) Co., Ltd." },
    { "0012DE", "Radio Components Sweden AB" },
    { "0012DF", "Novomatic AG" },
    { "0012E0", "Codan Limited" },
    { "0012E1", "Alliant Networks, Inc" },
    { "0012E2", "ALAXALA Networks Corporation" },
    { "0012E3", "Agat Soft LLC" },
    { "0012E4", "ZIEHL industrie-electronik GmbH + Co KG" },
    { "0012E5", "Time America, Inc." },
    { "0012E6", "SPECTEC COMPUTER CO., LTD." },
    { "0012E7", "Projectek Networking Electronics Corp." },
    { "0012E8", "Fraunhofer IMS" },
    { "0012E9", "Abbey Systems Ltd" },
    { "0012EA", "Trane" },
    { "0012EB", "PDH Solutions, LLC" },
    { "0012EC", "Movacolor b.v." },
    { "0012ED", "AVG Advanced Technologies" },
    { "0012EE", "Sony Corporation" },
    { "0012EF", "OneAccess SA" },
    { "0012F0", "Intel Corporate" },
    { "0012F1", "IFOTEC" },
    { "0012F2", "Brocade Communications Systems LLC" },
    { "0012F3", "u-blox AG" },
    { "0012F4", "Belco International Co.,Ltd." },
    { "0012F5", "Imarda New Zealand Limited" },
    { "0012F6", "MDK CO.,LTD." },
    { "0012F7", "Xiamen Xinglian Electronics Co., Ltd." },
    { "0012F8", "WNI Resources, LLC" },
    { "0012F9", "URYU SEISAKU, LTD." },
    { "0012FA", "THX LTD" },
    { "0012FB", "Samsung Electronics Co.,Ltd" },
    { "0012FC", "PLANET System Co.,LTD" },
    { "0012FD", "OPTIMUS IC S.A." },
    { "0012FE", "Lenovo Mobile Communication Technology Ltd." },
    { "0012FF", "Lely Industries N.V." },
    { "001300", "IT-FACTORY, INC." },
    { "001301", "IronGate S.L." },
    { "001302", "Intel Corporate" },
    { "001303", "GateConnect" },
    { "001304", "Flaircomm Technologies Co. LTD" },
    { "001305", "Epicom, Inc." },
    { "001306", "Always On Wireless" },
    { "001307", "Paravirtual Corporation" },
    { "001308", "Nuvera Fuel Cells" },
    { "001309", "Ocean Broadband Networks" },
    { "00130A", "Nortel Networks" },
    { "00130B", "Mextal B.V." },
    { "00130C", "HF System Corporation" },
    { "00130D", "GALILEO AVIONICA" },
    { "00130E", "Focusrite Audio Engineering Limited" },
    { "00130F", "EGEMEN Bilgisayar Muh San ve Tic LTD STI" },
    { "001310", "Cisco-Linksys, LLC" },
    { "001311", "Commscope" },
    { "001312", "Amedia Networks Inc." },
    { "001313", "GuangZhou Post & Telecom Equipment ltd" },
    { "001314", "Asiamajor Inc." },
    { "001315", "Sony Interactive Entertainment Inc." },
    { "001316", "L-S-B Broadcast Technologies GmbH" },
    { "001317", "GN Netcom A/S" },
    { "001318", "DGSTATION Co., Ltd." },
    { "001319", "Cisco Systems, Inc" },
    { "00131A", "Cisco Systems, Inc" },
    { "00131B", "BeCell Innovations Corp." },
    { "00131C", "LiteTouch, Inc." },
    { "00131D", "Scanvaegt International A/S" },
    { "00131E", "peiker acustic GmbH" },
    { "00131F", "NxtPhase T&D, Corp." },
    { "001320", "Intel Corporate" },
    { "001321", "Hewlett Packard" },
    { "001322", "DAQ Electronics, Inc." },
    { "001323", "Cap Co., Ltd." },
    { "001324", "Schneider Electric Ultra Terminal" },
    { "001325", "Cortina Systems Inc" },
    { "001326", "ECM Systems Ltd" },
    { "001327", "Data Acquisitions limited" },
    { "001328", "Westech Korea Inc.," },
    { "001329", "VSST Co., LTD" },
    { "00132A", "Sitronics Telecom Solutions" },
    { "00132B", "Phoenix Digital" },
    { "00132C", "MAZ Brandenburg GmbH" },
    { "00132D", "iWise Communications" },
    { "00132E", "ITian Coporation" },
    { "00132F", "Interactek" },
    { "001330", "EURO PROTECTION SURVEILLANCE" },
    { "001331", "CellPoint Connect" },
    { "001332", "Beijing Topsec Network Security Technology Co., Ltd." },
    { "001333", "BaudTec Corporation" },
    { "001334", "Arkados, Inc." },
    { "001335", "VS Industry Berhad" },
    { "001336", "Tianjin 712 Communication Broadcasting co., ltd." },
    { "001337", "Orient Power Home Network Ltd." },
    { "001338", "FRESENIUS-VIAL" },
    { "001339", "CCV Deutschland GmbH" },
    { "00133A", "VadaTech Inc." },
    { "00133B", "Speed Dragon Multimedia Limited" },
    { "00133C", "QUINTRON SYSTEMS INC." },
    { "00133D", "Micro Memory Curtiss Wright Co" },
    { "00133E", "MetaSwitch" },
    { "00133F", "Eppendorf Instrumente GmbH" },
    { "001340", "AD.EL s.r.l." },
    { "001341", "Shandong New Beiyang Information Technology Co.,Ltd" },
    { "001342", "Vision Research, Inc." },
    { "001343", "Matsushita Electronic Components (Europe) GmbH" },
    { "001344", "Fargo Electronics Inc." },
    { "001345", "Eaton Corporation" },
    { "001346", "D-Link Corporation" },
    { "001347", "Red Lion Controls, LP" },
    { "001348", "Artila Electronics Co., Ltd." },
    { "001349", "Zyxel Communications Corporation" },
    { "00134A", "Engim, Inc." },
    { "00134B", "ToGoldenNet Technology Inc." },
    { "00134C", "YDT Technology International" },
    { "00134D", "Inepro BV" },
    { "00134E", "Valox Systems, Inc." },
    { "00134F", "Rapidus Wireless Networks Inc." },
    { "001350", "Silver Spring Networks, Inc" },
    { "001351", "Niles Audio Corporation" },
    { "001352", "Naztec, Inc." },
    { "001353", "HYDAC Filtertechnik GMBH" },
    { "001354", "Zcomax Technologies, Inc." },
    { "001355", "TOMEN Cyber-business Solutions, Inc." },
    { "001356", "FLIR Radiation Inc" },
    { "001357", "Soyal Technology Co., Ltd." },
    { "001358", "Realm Systems, Inc." },
    { "001359", "ProTelevision Technologies A/S" },
    { "00135A", "Project T&E Limited" },
    { "00135B", "PanelLink Cinema, LLC" },
    { "00135C", "OnSite Systems, Inc." },
    { "00135D", "NTTPC Communications, Inc." },
    { "00135E", "EAB/RWI/K" },
    { "00135F", "Cisco Systems, Inc" },
    { "001360", "Cisco Systems, Inc" },
    { "001361", "Biospace Co., Ltd." },
    { "001362", "ShinHeung Precision Co., Ltd." },
    { "001363", "Verascape, Inc." },
    { "001364", "Paradigm Technology Inc.." },
    { "001365", "Nortel Networks" },
    { "001366", "Neturity Technologies Inc." },
    { "001367", "Narayon. Co., Ltd." },
    { "001368", "Saab Danmark A/S" },
    { "001369", "Honda Electron Co., LED." },
    { "00136A", "Hach Lange Sarl" },
    { "00136B", "E-TEC" },
    { "00136C", "TomTom" },
    { "00136D", "Tentaculus AB" },
    { "00136E", "Techmetro Corp." },
    { "00136F", "PacketMotion, Inc." },
    { "001370", "Nokia Danmark A/S" },
    { "001371", "Commscope" },
    { "001372", "Dell Inc." },
    { "001373", "BLwave Electronics Co., Ltd" },
    { "001374", "Atheros Communications, Inc." },
    { "001375", "American Security Products Co." },
    { "001376", "Tabor Electronics Ltd." },
    { "001377", "Samsung Electronics Co.,Ltd" },
    { "001378", "Qsan Technology, Inc." },
    { "001379", "PONDER INFORMATION INDUSTRIES LTD." },
    { "00137A", "Netvox Technology Co., Ltd." },
    { "00137B", "Movon Corporation" },
    { "00137C", "Kaicom co., Ltd." },
    { "00137D", "Dynalab, Inc." },
    { "00137E", "CorEdge Networks, Inc." },
    { "00137F", "Cisco Systems, Inc" },
    { "001380", "Cisco Systems, Inc" },
    { "001381", "CHIPS & Systems, Inc." },
    { "001382", "Cetacea Networks Corporation" },
    { "001383", "Application Technologies and Engineering Research Laboratory" },
    { "001384", "Advanced Motion Controls" },
    { "001385", "Add-On Technology Co., LTD." },
    { "001386", "ABB Inc/Totalflow" },
    { "001387", "27M Technologies AB" },
    { "001388", "WiMedia Alliance" },
    { "001389", "Redes de Telefonía Móvil S.A." },
    { "00138A", "Qingdao GoerTek Technology Co., Ltd." },
    { "00138B", "Phantom Technologies LLC" },
    { "00138C", "Kumyoung.Co.Ltd" },
    { "00138D", "Kinghold" },
    { "00138E", "FOAB Elektronik AB" },
    { "00138F", "Asiarock Technology Limited" },
    { "001390", "Termtek Computer Co., Ltd" },
    { "001391", "OUEN CO.,LTD." },
    { "001392", "Ruckus Wireless" },
    { "001393", "Panta Systems, Inc." },
    { "001394", "Infohand Co.,Ltd" },
    { "001395", "congatec GmbH" },
    { "001396", "Acbel Polytech Inc." },
    { "001397", "Oracle Corporation " },
    { "001398", "TrafficSim Co.,Ltd" },
    { "001399", "STAC Corporation." },
    { "00139A", "K-ubique ID Corp." },
    { "00139B", "ioIMAGE Ltd." },
    { "00139C", "Exavera Technologies, Inc." },
    { "00139D", "MaxLinear Hispania S.L.U." },
    { "00139E", "Ciara Technologies Inc." },
    { "00139F", "Electronics Design Services, Co., Ltd." },
    { "0013A0", "ALGOSYSTEM Co., Ltd." },
    { "0013A1", "Crow Electronic Engeneering" },
    { "0013A2", "MaxStream, Inc" },
    { "0013A3", "Siemens Home & Office Comm. Devices" },
    { "0013A4", "KeyEye Communications" },
    { "0013A5", "General Solutions, LTD." },
    { "0013A6", "Extricom Ltd" },
    { "0013A7", "BATTELLE MEMORIAL INSTITUTE" },
    { "0013A8", "Tanisys Technology" },
    { "0013A9", "Sony Corporation" },
    { "0013AA", "ALS & TEC Ltd." },
    { "0013AB", "Telemotive AG" },
    { "0013AC", "Sunmyung Electronics Co., LTD" },
    { "0013AD", "Sendo Ltd" },
    { "0013AE", "Radiance Technologies, Inc." },
    { "0013AF", "NUMA Technology,Inc." },
    { "0013B0", "Jablotron" },
    { "0013B1", "Intelligent Control Systems (Asia) Pte Ltd" },
    { "0013B2", "Carallon Limited" },
    { "0013B3", "Ecom Communications Technology Co., Ltd." },
    { "0013B4", "Appear AS" },
    { "0013B5", "Wavesat" },
    { "0013B6", "Sling Media, Inc." },
    { "0013B7", "Scantech ID" },
    { "0013B8", "RyCo Electronic Systems Limited" },
    { "0013B9", "BM SPA" },
    { "0013BA", "ReadyLinks Inc" },
    { "0013BB", "Smartvue Corporation" },
    { "0013BC", "Artimi Ltd" },
    { "0013BD", "HYMATOM SA" },
    { "0013BE", "Virtual Conexions" },
    { "0013BF", "Media System Planning Corp." },
    { "0013C0", "Trix Tecnologia Ltda." },
    { "0013C1", "Asoka USA Corporation" },
    { "0013C2", "WACOM Co.,Ltd" },
    { "0013C3", "Cisco Systems, Inc" },
    { "0013C4", "Cisco Systems, Inc" },
    { "0013C5", "LIGHTRON FIBER-OPTIC DEVICES INC." },
    { "0013C6", "OpenGear, Inc" },
    { "0013C7", "IONOS Co.,Ltd." },
    { "0013C8", "ADB Broadband Italia" },
    { "0013C9", "Beyond Achieve Enterprises Ltd." },
    { "0013CA", "ATX" },
    { "0013CB", "Zenitel Norway AS" },
    { "0013CC", "Tall Maple Systems" },
    { "0013CD", "MTI co. LTD" },
    { "0013CE", "Intel Corporate" },
    { "0013CF", "4Access Communications" },
    { "0013D0", "t+ Medical Ltd" },
    { "0013D1", "KIRK telecom A/S" },
    { "0013D2", "PAGE IBERICA, S.A." },
    { "0013D3", "MICRO-STAR INTERNATIONAL CO., LTD." },
    { "0013D4", "ASUSTek COMPUTER INC." },
    { "0013D5", "RuggedCom" },
    { "0013D6", "TII NETWORK TECHNOLOGIES, INC." },
    { "0013D7", "SPIDCOM Technologies SA" },
    { "0013D8", "Princeton Instruments" },
    { "0013D9", "Matrix Product Development, Inc." },
    { "0013DA", "Diskware Co., Ltd" },
    { "0013DB", "SHOEI Electric Co.,Ltd" },
    { "0013DC", "IBTEK INC." },
    { "0013DD", "Abbott Diagnostics" },
    { "0013DE", "Adapt4, LLC" },
    { "0013DF", "Ryvor Corp." },
    { "0013E0", "Murata Manufacturing Co., Ltd." },
    { "0013E1", "Iprobe AB" },
    { "0013E2", "GeoVision Inc." },
    { "0013E3", "CoVi Technologies, Inc." },
    { "0013E4", "YANGJAE SYSTEMS CORP." },
    { "0013E5", "TENOSYS, INC." },
    { "0013E6", "Technolution" },
    { "0013E7", "Halcro" },
    { "0013E8", "Intel Corporate" },
    { "0013E9", "VeriWave, Inc." },
    { "0013EA", "Kamstrup A/S" },
    { "0013EB", "Sysmaster Corporation" },
    { "0013EC", "Netsnapper Technologies SARL" },
    { "0013ED", "PSIA" },
    { "0013EE", "JBX Designs Inc." },
    { "0013EF", "Kingjon Digital Technology Co.,Ltd" },
    { "0013F0", "Wavefront Semiconductor" },
    { "0013F1", "AMOD Technology Co., Ltd." },
    { "0013F2", "Klas Ltd" },
    { "0013F3", "Giga-byte Communications Inc." },
    { "0013F4", "Psitek (Pty) Ltd" },
    { "0013F5", "Akimbi Systems" },
    { "0013F6", "Cintech" },
    { "0013F7", "SMC Networks, Inc." },
    { "0013F8", "Dex Security Solutions" },
    { "0013F9", "Cavera Systems" },
    { "0013FA", "LifeSize Communications, Inc" },
    { "0013FB", "RKC INSTRUMENT INC." },
    { "0013FC", "SiCortex, Inc" },
    { "0013FD", "Nokia Danmark A/S" },
    { "0013FE", "GRANDTEC ELECTRONIC CORP." },
    { "0013FF", "Dage-MTI of MC, Inc." },
    { "001400", "MINERVA KOREA CO., LTD" },
    { "001401", "Rivertree Networks Corp." },
    { "001402", "kk-electronic a/s" },
    { "001403", "Renasis, LLC" },
    { "001404", "Commscope" },
    { "001405", "OpenIB, Inc." },
    { "001406", "Go Networks" },
    { "001407", "Sperian Protection Instrumentation" },
    { "001408", "Eka Systems Inc." },
    { "001409", "MAGNETI MARELLI S.E. S.p.A." },
    { "00140A", "WEPIO Co., Ltd." },
    { "00140B", "FIRST INTERNATIONAL COMPUTER, INC." },
    { "00140C", "GKB CCTV CO., LTD." },
    { "00140D", "Nortel Networks" },
    { "00140E", "Nortel Networks" },
    { "00140F", "Federal State Unitary Enterprise Leningrad R&D Institute of" },
    { "001410", "Suzhou Keda Technology CO.,Ltd" },
    { "001411", "Deutschmann Automation GmbH & Co. KG" },
    { "001412", "S-TEC electronics AG" },
    { "001413", "Trebing & Himstedt Prozeßautomation GmbH & Co. KG" },
    { "001414", "Jumpnode Systems LLC." },
    { "001415", "Intec Automation inc." },
    { "001416", "Scosche Industries, Inc." },
    { "001417", "RSE Informations Technologie GmbH" },
    { "001418", "C4Line" },
    { "001419", "SIDSA" },
    { "00141A", "DEICY CORPORATION" },
    { "00141B", "Cisco Systems, Inc" },
    { "00141C", "Cisco Systems, Inc" },
    { "00141D", "KEBA Industrial Automation Germany GmbH" },
    { "00141E", "P.A. Semi, Inc." },
    { "00141F", "SunKwang Electronics Co., Ltd" },
    { "001420", "G-Links networking company" },
    { "001421", "Total Wireless Technologies Pte. Ltd." },
    { "001422", "Dell Inc." },
    { "001423", "J-S Co. NEUROCOM" },
    { "001424", "Merry Electrics CO., LTD." },
    { "001425", "Galactic Computing Corp." },
    { "001426", "NL Technology" },
    { "001427", "JazzMutant" },
    { "001428", "Vocollect Inc" },
    { "001429", "V Center Technologies Co., Ltd." },
    { "00142A", "Elitegroup Computer Systems Co.,Ltd." },
    { "00142B", "Edata Communication Inc." },
    { "00142C", "Koncept International, Inc." },
    { "00142D", "Toradex AG" },
    { "00142E", "77 Elektronika Kft." },
    { "00142F", "Savvius" },
    { "001430", "ViPowER, Inc" },
    { "001431", "PDL Electronics Ltd" },
    { "001432", "Tarallax Wireless, Inc." },
    { "001433", "Empower Technologies(Canada) Inc." },
    { "001434", "Keri Systems, Inc" },
    { "001435", "CityCom Corp." },
    { "001436", "Qwerty Elektronik AB" },
    { "001437", "GSTeletech Co.,Ltd." },
    { "001438", "Hewlett Packard Enterprise " },
    { "001439", "Blonder Tongue Laboratories, Inc" },
    { "00143A", "RAYTALK INTERNATIONAL SRL" },
    { "00143B", "Sensovation AG" },
    { "00143C", "Rheinmetall Canada Inc." },
    { "00143D", "Aevoe Inc." },
    { "00143E", "AirLink Communications, Inc." },
    { "00143F", "Hotway Technology Corporation" },
    { "001440", "ATOMIC Corporation" },
    { "001441", "Innovation Sound Technology Co., LTD." },
    { "001442", "ATTO CORPORATION" },
    { "001443", "Consultronics Europe Ltd" },
    { "001444", "Grundfos Holding" },
    { "001445", "Telefon-Gradnja d.o.o." },
    { "001446", "SuperVision Solutions LLC" },
    { "001447", "BOAZ Inc." },
    { "001448", "Inventec Multimedia & Telecom Corporation" },
    { "001449", "Sichuan Changhong Electric Ltd." },
    { "00144A", "Taiwan Thick-Film Ind. Corp." },
    { "00144B", "Hifn, Inc." },
    { "00144C", "General Meters Corp." },
    { "00144D", "Intelligent Systems" },
    { "00144E", "SRISA" },
    { "00144F", "Oracle Corporation " },
    { "001450", "Heim Systems GmbH" },
    { "001451", "Apple, Inc." },
    { "001452", "CALCULEX,INC." },
    { "001453", "ADVANTECH TECHNOLOGIES CO.,LTD" },
    { "001454", "Symwave" },
    { "001455", "Coder Electronics Corporation" },
    { "001456", "Edge Products" },
    { "001457", "Nevion" },
    { "001458", "HS Automatic ApS" },
    { "001459", "Moram Co., Ltd." },
    { "00145A", "Westermo Neratec AG" },
    { "00145B", "SeekerNet Inc." },
    { "00145C", "Intronics B.V." },
    { "00145D", "WJ Communications, Inc." },
    { "00145E", "IBM Corp" },
    { "00145F", "ADITEC CO. LTD" },
    { "001460", "Kyocera Wireless Corp." },
    { "001461", "CORONA CORPORATION" },
    { "001462", "Digiwell Technology, inc" },
    { "001463", "IDCS N.V." },
    { "001464", "Cryptosoft" },
    { "001465", "Novo Nordisk A/S" },
    { "001466", "Kleinhenz Elektronik GmbH" },
    { "001467", "ArrowSpan Inc." },
    { "001468", "CelPlan International, Inc." },
    { "001469", "Cisco Systems, Inc" },
    { "00146A", "Cisco Systems, Inc" },
    { "00146B", "Anagran, Inc." },
    { "00146C", "NETGEAR" },
    { "00146D", "RF Technologies" },
    { "00146E", "H. Stoll GmbH & Co. KG" },
    { "00146F", "Kohler Co" },
    { "001470", "Prokom Software SA" },
    { "001471", "Eastern Asia Technology Limited" },
    { "001472", "China Broadband Wireless IP Standard group(ChinaBWIPS)" },
    { "001473", "Bookham Inc" },
    { "001474", "K40 Electronics" },
    { "001475", "Wiline Networks, Inc." },
    { "001476", "MultiCom Industries Limited" },
    { "001477", "Trilliant" },
    { "001478", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "001479", "NEC Magnus Communications,Ltd." },
    { "00147A", "Eubus GmbH" },
    { "00147B", "Iteris, Inc." },
    { "00147C", "3Com Ltd" },
    { "00147D", "Aeon Digital International" },
    { "00147E", "InnerWireless" },
    { "00147F", "Thomson Telecom Belgium" },
    { "001480", "Hitachi-LG Data Storage Korea, Inc" },
    { "001481", "Multilink Inc" },
    { "001482", "Aurora Networks" },
    { "001483", "eXS Inc." },
    { "001484", "Cermate Technologies Inc." },
    { "001485", "Giga-Byte" },
    { "001486", "Echo Digital Audio Corporation" },
    { "001487", "American Technology Integrators" },
    { "001488", "Akorri" },
    { "001489", "B15402100 - JANDEI, S.L." },
    { "00148A", "Elin Ebg Traction Gmbh" },
    { "00148B", "Globo Electronic GmbH & Co. KG" },
    { "00148C", "General Dynamics Mission Systems" },
    { "00148D", "Cubic Defense Simulation Systems" },
    { "00148E", "Tele Power Inc." },
    { "00148F", "Protronic (Far East) Ltd." },
    { "001490", "ASP Corporation" },
    { "001491", "Daniels Electronics Ltd. dba Codan Radio Communications" },
    { "001492", "Liteon, Mobile Media Solution SBU" },
    { "001493", "Systimax Solutions" },
    { "001494", "ESU AG" },
    { "001495", "2Wire Inc" },
    { "001496", "Phonic Corp." },
    { "001497", "ZHIYUAN Eletronics co.,ltd." },
    { "001498", "Viking Design Technology" },
    { "001499", "Helicomm Inc" },
    { "00149A", "Commscope" },
    { "00149B", "Nokota Communications, LLC" },
    { "00149C", "HF Company" },
    { "00149D", "Sound ID Inc." },
    { "00149E", "UbONE Co., Ltd" },
    { "00149F", "System and Chips, Inc." },
    { "0014A0", "Accsense, Inc." },
    { "0014A1", "Synchronous Communication Corp" },
    { "0014A2", "Core Micro Systems Inc." },
    { "0014A3", "Vitelec BV" },
    { "0014A4", "Hon Hai Precision Ind. Co.,Ltd." },
    { "0014A5", "Gemtek Technology Co., Ltd." },
    { "0014A6", "Teranetics, Inc." },
    { "0014A7", "Nokia Danmark A/S" },
    { "0014A8", "Cisco Systems, Inc" },
    { "0014A9", "Cisco Systems, Inc" },
    { "0014AA", "Ashly Audio, Inc." },
    { "0014AB", "Senhai Electronic Technology Co., Ltd." },
    { "0014AC", "Bountiful WiFi" },
    { "0014AD", "Gassner Wiege- und Meßtechnik GmbH" },
    { "0014AE", "Wizlogics Co., Ltd." },
    { "0014AF", "Datasym POS Inc." },
    { "0014B0", "Naeil Community" },
    { "0014B1", "Axell Wireless Limited" },
    { "0014B2", "mCubelogics Corporation" },
    { "0014B3", "CoreStar International Corp" },
    { "0014B4", "General Dynamics United Kingdom Ltd" },
    { "0014B5", "PHYSIOMETRIX,INC" },
    { "0014B6", "Enswer Technology Inc." },
    { "0014B7", "AR Infotek Inc." },
    { "0014B8", "Hill-Rom" },
    { "0014B9", "MSTAR SEMICONDUCTOR" },
    { "0014BA", "Carvers SA de CV" },
    { "0014BB", "Open Interface North America" },
    { "0014BC", "SYNECTIC TELECOM EXPORTS PVT. LTD." },
    { "0014BD", "incNETWORKS, Inc" },
    { "0014BE", "Wink communication technology CO.LTD" },
    { "0014BF", "Cisco-Linksys, LLC" },
    { "0014C0", "Symstream Technology Group Ltd" },
    { "0014C1", "U.S. Robotics Corporation" },
    { "0014C2", "Hewlett Packard" },
    { "0014C3", "Seagate Technology" },
    { "0014C4", "Vitelcom Mobile Technology" },
    { "0014C5", "Alive Technologies Pty Ltd" },
    { "0014C6", "Quixant Ltd" },
    { "0014C7", "Nortel Networks" },
    { "0014C8", "Contemporary Research Corp" },
    { "0014C9", "Brocade Communications Systems LLC" },
    { "0014CA", "Key Radio Systems Limited" },
    { "0014CB", "LifeSync Corporation" },
    { "0014CC", "Zetec, Inc." },
    { "0014CD", "DigitalZone Co., Ltd." },
    { "0014CE", "NF CORPORATION" },
    { "0014CF", "INVISIO Communications" },
    { "0014D0", "BTI Systems Inc." },
    { "0014D1", "TRENDnet, Inc." },
    { "0014D2", "Kyuden Technosystems Corporation" },
    { "0014D3", "SEPSA" },
    { "0014D4", "K Technology Corporation" },
    { "0014D5", "Datang Telecom Technology CO. , LCD,Optical Communication Br" },
    { "0014D6", "Jeongmin Electronics Co.,Ltd." },
    { "0014D7", "Datastore Technology Corp" },
    { "0014D8", "bio-logic SA" },
    { "0014D9", "IP Fabrics, Inc." },
    { "0014DA", "Huntleigh Healthcare" },
    { "0014DB", "Elma Trenew Electronic GmbH" },
    { "0014DC", "Communication System Design & Manufacturing (CSDM)" },
    { "0014DD", "Covergence Inc." },
    { "0014DE", "Sage Instruments Inc." },
    { "0014DF", "HI-P Tech Corporation" },
    { "0014E0", "LET'S Corporation" },
    { "0014E1", "Data Display AG" },
    { "0014E2", "datacom systems inc." },
    { "0014E3", "mm-lab GmbH" },
    { "0014E4", "infinias, LLC" },
    { "0014E5", "Alticast" },
    { "0014E6", "AIM Infrarotmodule GmbH" },
    { "0014E7", "Stolinx,. Inc" },
    { "0014E8", "Commscope" },
    { "0014E9", "Nortech International" },
    { "0014EA", "S Digm Inc. (Safe Paradigm Inc.)" },
    { "0014EB", "AwarePoint Corporation" },
    { "0014EC", "Acro Telecom" },
    { "0014ED", "Airak, Inc." },
    { "0014EE", "Western Digital Technologies, Inc." },
    { "0014EF", "TZero Technologies, Inc." },
    { "0014F0", "Business Security OL AB" },
    { "0014F1", "Cisco Systems, Inc" },
    { "0014F2", "Cisco Systems, Inc" },
    { "0014F3", "ViXS Systems Inc" },
    { "0014F4", "DekTec Digital Video B.V." },
    { "0014F5", "OSI Security Devices" },
    { "0014F6", "Juniper Networks" },
    { "0014F7", "CREVIS Co., LTD" },
    { "0014F8", "Scientific Atlanta" },
    { "0014F9", "Vantage Controls" },
    { "0014FA", "AsGa S.A." },
    { "0014FB", "Technical Solutions Inc." },
    { "0014FC", "Extandon, Inc." },
    { "0014FD", "Thecus Technology Corp." },
    { "0014FE", "Artech Electronics" },
    { "0014FF", "Precise Automation, Inc." },
    { "001500", "Intel Corporate" },
    { "001501", "LexBox" },
    { "001502", "BETA tech" },
    { "001503", "PROFIcomms s.r.o." },
    { "001504", "GAME PLUS CO., LTD." },
    { "001505", "Actiontec Electronics, Inc" },
    { "001506", "Neo Photonics" },
    { "001507", "Renaissance Learning Inc" },
    { "001508", "Global Target Enterprise Inc" },
    { "001509", "Plus Technology Co., Ltd" },
    { "00150A", "Sonoa Systems, Inc" },
    { "00150B", "SAGE INFOTECH LTD." },
    { "00150C", "AVM GmbH" },
    { "00150D", "Hoana Medical, Inc." },
    { "00150E", "OPENBRAIN TECHNOLOGIES CO., LTD." },
    { "00150F", "mingjong" },
    { "001510", "Techsphere Co., Ltd" },
    { "001511", "Data Center Systems" },
    { "001512", "Zurich University of Applied Sciences" },
    { "001513", "EFS sas" },
    { "001514", "Hu Zhou NAVA Networks&Electronics Ltd." },
    { "001515", "Leipold+Co.GmbH" },
    { "001516", "URIEL SYSTEMS INC." },
    { "001517", "Intel Corporate" },
    { "001518", "Shenzhen 10MOONS Technology Development CO.,Ltd" },
    { "001519", "StoreAge Networking Technologies" },
    { "00151A", "Hunter Engineering Company" },
    { "00151B", "Isilon Systems Inc." },
    { "00151C", "LENECO" },
    { "00151D", "M2I CORPORATION" },
    { "00151E", "ETHERNET Powerlink Standarization Group (EPSG)" },
    { "00151F", "Multivision Intelligent Surveillance (Hong Kong) Ltd" },
    { "001520", "Radiocrafts AS" },
    { "001521", "Horoquartz" },
    { "001522", "Dea Security" },
    { "001523", "Meteor Communications Corporation" },
    { "001524", "Numatics, Inc." },
    { "001525", "Chamberlain Access Solutions" },
    { "001526", "Remote Technologies Inc" },
    { "001527", "Balboa Instruments" },
    { "001528", "Beacon Medical Products LLC d.b.a. BeaconMedaes" },
    { "001529", "N3 Corporation" },
    { "00152A", "Nokia Corporation" },
    { "00152B", "Cisco Systems, Inc" },
    { "00152C", "Cisco Systems, Inc" },
    { "00152D", "TenX Networks, LLC" },
    { "00152E", "PacketHop, Inc." },
    { "00152F", "Commscope" },
    { "001530", "Dell EMC" },
    { "001531", "KOCOM" },
    { "001532", "Consumer Technologies Group, LLC" },
    { "001533", "NADAM.CO.,LTD" },
    { "001534", "A Beltrónica-Companhia de Comunicações, Lda" },
    { "001535", "OTE Spa" },
    { "001536", "Powertech co.,Ltd" },
    { "001537", "Ventus Networks" },
    { "001538", "RFID, Inc." },
    { "001539", "Technodrive srl" },
    { "00153A", "Shenzhen Syscan Technology Co.,Ltd." },
    { "00153B", "EMH Metering GmbH & Co. KG" },
    { "00153C", "Kprotech Co., Ltd." },
    { "00153D", "ELIM PRODUCT CO." },
    { "00153E", "Q-Matic Sweden AB" },
    { "00153F", "Alcatel Alenia Space Italia" },
    { "001540", "Nortel Networks" },
    { "001541", "StrataLight Communications, Inc." },
    { "001542", "MICROHARD S.R.L." },
    { "001543", "Aberdeen Test Center" },
    { "001544", "coM.s.a.t. AG" },
    { "001545", "SEECODE Co., Ltd." },
    { "001546", "ITG Worldwide Sdn Bhd" },
    { "001547", "AiZen Solutions Inc." },
    { "001548", "CUBE TECHNOLOGIES" },
    { "001549", "Dixtal Biomedica Ind. Com. Ltda" },
    { "00154A", "WANSHIH ELECTRONIC CO., LTD" },
    { "00154B", "Wonde Proud Technology Co., Ltd" },
    { "00154C", "Saunders Electronics" },
    { "00154D", "Netronome Systems, Inc." },
    { "00154E", "IEC" },
    { "00154F", "one RF Technology" },
    { "001550", "Nits Technology Inc" },
    { "001551", "RadioPulse Inc." },
    { "001552", "Wi-Gear Inc." },
    { "001553", "Cytyc Corporation" },
    { "001554", "Atalum Wireless S.A." },
    { "001555", "DFM GmbH" },
    { "001556", "Sagemcom Broadband SAS" },
    { "001557", "Olivetti" },
    { "001558", "FOXCONN" },
    { "001559", "Securaplane Technologies, Inc." },
    { "00155A", "DAINIPPON PHARMACEUTICAL CO., LTD." },
    { "00155B", "Sampo Corporation" },
    { "00155C", "Dresser Wayne" },
    { "00155D", "Microsoft Corporation" },
    { "00155E", "Morgan Stanley" },
    { "00155F", "GreenPeak Technologies" },
    { "001560", "Hewlett Packard" },
    { "001561", "JJPlus Corporation" },
    { "001562", "Cisco Systems, Inc" },
    { "001563", "Cisco Systems, Inc" },
    { "001564", "BEHRINGER Spezielle Studiotechnik GmbH" },
    { "001565", "XIAMEN YEALINK NETWORK TECHNOLOGY CO.,LTD" },
    { "001566", "A-First Technology Co., Ltd." },
    { "001567", "RADWIN Inc." },
    { "001568", "Dilithium Networks" },
    { "001569", "PECO II, Inc." },
    { "00156A", "DG2L Technologies Pvt. Ltd." },
    { "00156B", "Perfisans Networks Corp." },
    { "00156C", "SANE SYSTEM CO., LTD" },
    { "00156D", "Ubiquiti Inc" },
    { "00156E", "A. W. Communication Systems Ltd" },
    { "00156F", "Xiranet Communications GmbH" },
    { "001570", "Zebra Technologies Inc" },
    { "001571", "Nolan Systems" },
    { "001572", "Red-Lemon" },
    { "001573", "NewSoft Technology Corporation" },
    { "001574", "Horizon Semiconductors Ltd." },
    { "001575", "Nevis Networks Inc." },
    { "001576", "LABiTec - Labor Biomedical Technologies GmbH" },
    { "001577", "Allied Telesis, Inc." },
    { "001578", "Audio / Video Innovations" },
    { "001579", "Lunatone Industrielle Elektronik GmbH" },
    { "00157A", "Telefin S.p.A." },
    { "00157B", "Leuze electronic GmbH + Co. KG" },
    { "00157C", "Dave Networks, Inc." },
    { "00157D", "POSDATA" },
    { "00157E", "Weidmüller Interface GmbH & Co. KG" },
    { "00157F", "ChuanG International Holding CO.,LTD." },
    { "001580", "U-WAY CORPORATION" },
    { "001581", "MAKUS Inc." },
    { "001582", "Pulse Eight Limited" },
    { "001583", "IVT corporation" },
    { "001584", "Schenck Process GmbH" },
    { "001585", "Aonvision Technolopy Corp." },
    { "001586", "Xiamen Overseas Chinese Electronic Co., Ltd." },
    { "001587", "Takenaka Seisakusho Co.,Ltd" },
    { "001588", "Salutica Allied Solutions Sdn Bhd" },
    { "001589", "D-MAX Technology Co.,Ltd" },
    { "00158A", "SURECOM Technology Corp." },
    { "00158B", "Park Air Systems Ltd" },
    { "00158C", "Liab ApS" },
    { "00158D", "Jennic Ltd" },
    { "00158E", "Plustek.INC" },
    { "00158F", "NTT Advanced Technology Corporation" },
    { "001590", "Hectronic GmbH" },
    { "001591", "RLW Inc." },
    { "001592", "Facom UK Ltd (Melksham)" },
    { "001593", "U4EA Technologies Inc." },
    { "001594", "BIXOLON CO.,LTD" },
    { "001595", "Quester Tangent Corporation" },
    { "001596", "Commscope" },
    { "001597", "AETA AUDIO SYSTEMS" },
    { "001598", "Kolektor group" },
    { "001599", "Samsung Electronics Co.,Ltd" },
    { "00159A", "Commscope" },
    { "00159B", "Nortel Networks" },
    { "00159C", "B-KYUNG SYSTEM Co.,Ltd." },
    { "00159D", "Tripp Lite " },
    { "00159E", "Mad Catz Interactive Inc" },
    { "00159F", "Terascala, Inc." },
    { "0015A0", "Nokia Danmark A/S" },
    { "0015A1", "ECA-SINTERS" },
    { "0015A2", "Commscope" },
    { "0015A3", "Commscope" },
    { "0015A4", "Commscope" },
    { "0015A5", "DCI Co., Ltd." },
    { "0015A6", "Digital Electronics Products Ltd." },
    { "0015A7", "Robatech AG" },
    { "0015A8", "Commscope" },
    { "0015A9", "KWANG WOO I&C CO.,LTD" },
    { "0015AA", "Rextechnik International Co.," },
    { "0015AB", "PRO CO SOUND INC" },
    { "0015AC", "Capelon AB" },
    { "0015AD", "Accedian Networks" },
    { "0015AE", "kyung il" },
    { "0015AF", "AzureWave Technology Inc." },
    { "0015B0", "AUTOTELENET CO.,LTD" },
    { "0015B1", "Ambient Corporation" },
    { "0015B2", "Advanced Industrial Computer, Inc." },
    { "0015B3", "Caretech AB" },
    { "0015B4", "Polymap Wireless LLC" },
    { "0015B5", "CI Network Corp." },
    { "0015B6", "ShinMaywa Industries, Ltd." },
    { "0015B7", "Toshiba" },
    { "0015B8", "Tahoe" },
    { "0015B9", "Samsung Electronics Co.,Ltd" },
    { "0015BA", "iba AG" },
    { "0015BB", "SMA Solar Technology AG" },
    { "0015BC", "Develco" },
    { "0015BD", "Group 4 Technology Ltd" },
    { "0015BE", "Iqua Ltd." },
    { "0015BF", "technicob" },
    { "0015C0", "DIGITAL TELEMEDIA CO.,LTD." },
    { "0015C1", "Sony Interactive Entertainment Inc." },
    { "0015C2", "3M Germany" },
    { "0015C3", "Ruf Telematik AG" },
    { "0015C4", "FLOVEL CO., LTD." },
    { "0015C5", "Dell Inc." },
    { "0015C6", "Cisco Systems, Inc" },
    { "0015C7", "Cisco Systems, Inc" },
    { "0015C8", "FlexiPanel Ltd" },
    { "0015C9", "Gumstix, Inc" },
    { "0015CA", "TeraRecon, Inc." },
    { "0015CB", "Surf Communication Solutions Ltd." },
    { "0015CC", "UQUEST, LTD." },
    { "0015CD", "Exartech International Corp." },
    { "0015CE", "Commscope" },
    { "0015CF", "Commscope" },
    { "0015D0", "Commscope" },
    { "0015D1", "Commscope" },
    { "0015D2", "Xantech Corporation" },
    { "0015D3", "Pantech&Curitel Communications, Inc." },
    { "0015D4", "Emitor AB" },
    { "0015D5", "NICEVT" },
    { "0015D6", "OSLiNK Sp. z o.o." },
    { "0015D7", "Reti Corporation" },
    { "0015D8", "Interlink Electronics" },
    { "0015D9", "PKC Electronics Oy" },
    { "0015DA", "IRITEL A.D." },
    { "0015DB", "Canesta Inc." },
    { "0015DC", "KT&C Co., Ltd." },
    { "0015DD", "IP Control Systems Ltd." },
    { "0015DE", "Nokia Danmark A/S" },
    { "0015DF", "Clivet S.p.A." },
    { "0015E0", "Ericsson" },
    { "0015E1", "Picochip Ltd" },
    { "0015E2", "Dr.Ing. Herbert Knauer GmbH" },
    { "0015E3", "Dream Technologies Corporation" },
    { "0015E4", "Zimmer Elektromedizin" },
    { "0015E5", "Cheertek Inc." },
    { "0015E6", "MOBILE TECHNIKA Inc." },
    { "0015E7", "Quantec Tontechnik" },
    { "0015E8", "Nortel Networks" },
    { "0015E9", "D-Link Corporation" },
    { "0015EA", "Tellumat (Pty) Ltd" },
    { "0015EB", "zte corporation" },
    { "0015EC", "Boca Devices LLC" },
    { "0015ED", "Fulcrum Microsystems, Inc." },
    { "0015EE", "Omnex Control Systems" },
    { "0015EF", "NEC TOKIN Corporation" },
    { "0015F0", "EGO BV" },
    { "0015F1", "KYLINK Communications Corp." },
    { "0015F2", "ASUSTek COMPUTER INC." },
    { "0015F3", "PELTOR AB" },
    { "0015F4", "Eventide" },
    { "0015F5", "Sustainable Energy Systems" },
    { "0015F6", "SCIENCE AND ENGINEERING SERVICES, INC." },
    { "0015F7", "Wintecronics Ltd." },
    { "0015F8", "Kingtronics Industrial Co. Ltd." },
    { "0015F9", "Cisco Systems, Inc" },
    { "0015FA", "Cisco Systems, Inc" },
    { "0015FB", "setex schermuly textile computer gmbh" },
    { "0015FC", "Littelfuse Startco" },
    { "0015FD", "Complete Media Systems" },
    { "0015FE", "SCHILLING ROBOTICS LLC" },
    { "0015FF", "Inseego Wireless, Inc" },
    { "001600", "CelleBrite Mobile Synchronization" },
    { "001601", "BUFFALO.INC" },
    { "001602", "CEYON TECHNOLOGY CO.,LTD." },
    { "001603", "COOLKSKY Co., LTD" },
    { "001604", "Sigpro" },
    { "001605", "YORKVILLE SOUND INC." },
    { "001606", "Ideal Industries" },
    { "001607", "Curves International Inc." },
    { "001608", "Sequans Communications" },
    { "001609", "Unitech electronics co., ltd." },
    { "00160A", "SWEEX Europe BV" },
    { "00160B", "TVWorks LLC" },
    { "00160C", "LPL DEVELOPMENT S.A. DE C.V" },
    { "00160D", "Be Here Corporation" },
    { "00160E", "Optica Technologies Inc." },
    { "00160F", "BADGER METER INC" },
    { "001610", "Carina Technology" },
    { "001611", "Altecon Srl" },
    { "001612", "Otsuka Electronics Co., Ltd." },
    { "001613", "LibreStream Technologies Inc." },
    { "001614", "Picosecond Pulse Labs" },
    { "001615", "Nittan Company, Limited" },
    { "001616", "BROWAN COMMUNICATIONS INCORPORATION" },
    { "001617", "MSI" },
    { "001618", "HIVION Co., Ltd." },
    { "001619", "Lancelan Technologies S.L." },
    { "00161A", "Dametric AB" },
    { "00161B", "Micronet Corporation" },
    { "00161C", "e:cue" },
    { "00161D", "Innovative Wireless Technologies, Inc." },
    { "00161E", "Woojinnet" },
    { "00161F", "SUNWAVETEC Co., Ltd." },
    { "001620", "Sony Corporation" },
    { "001621", "Colorado Vnet" },
    { "001622", "BBH SYSTEMS GMBH" },
    { "001623", "Interval Media" },
    { "001624", "Teneros, Inc." },
    { "001625", "Impinj, Inc." },
    { "001626", "Commscope" },
    { "001627", "embedded-logic DESIGN AND MORE GmbH" },
    { "001628", "Magicard Ltd" },
    { "001629", "Nivus GmbH" },
    { "00162A", "Antik computers & communications s.r.o." },
    { "00162B", "Togami Electric Mfg.co.,Ltd." },
    { "00162C", "Xanboo" },
    { "00162D", "STNet Co., Ltd." },
    { "00162E", "Space Shuttle Hi-Tech Co., Ltd." },
    { "00162F", "Geutebrück GmbH" },
    { "001630", "Vativ Technologies" },
    { "001631", "Xteam" },
    { "001632", "Samsung Electronics Co.,Ltd" },
    { "001633", "Oxford Diagnostics Ltd." },
    { "001634", "Mathtech, Inc." },
    { "001635", "Hewlett Packard" },
    { "001636", "Quanta Computer Inc." },
    { "001637", "CITEL SpA" },
    { "001638", "TECOM Co., Ltd." },
    { "001639", "Ubiquam Co., Ltd." },
    { "00163A", "YVES TECHNOLOGY CO., LTD." },
    { "00163B", "Communications & Power Industries" },
    { "00163C", "Rebox B.V." },
    { "00163D", "Tsinghua Tongfang Legend Silicon Tech. Co., Ltd." },
    { "00163E", "Xensource, Inc." },
    { "00163F", "CReTE SYSTEMS Inc." },
    { "001640", "Asmobile Communication Inc." },
    { "001641", "Universal Global Scientific Industrial Co., Ltd." },
    { "001642", "Pangolin" },
    { "001643", "Sunhillo Corporation" },
    { "001644", "LITE-ON Technology Corp." },
    { "001645", "Eaton Corporation" },
    { "001646", "Cisco Systems, Inc" },
    { "001647", "Cisco Systems, Inc" },
    { "001648", "SSD Company Limited" },
    { "001649", "SetOne GmbH" },
    { "00164A", "Vibration Technology Limited" },
    { "00164B", "Quorion Data Systems GmbH" },
    { "00164C", "PLANET INT Co., Ltd" },
    { "00164D", "Alcatel-Lucent IPD" },
    { "00164E", "Nokia Danmark A/S" },
    { "00164F", "World Ethnic Broadcastin Inc." },
    { "001650", "Kratos EPD " },
    { "001651", "Exeo Systems" },
    { "001652", "Hoatech Technologies, Inc." },
    { "001653", "LEGO System A/S IE Electronics Division" },
    { "001654", "Flex-P Industries Sdn. Bhd." },
    { "001655", "FUHO TECHNOLOGY Co., LTD" },
    { "001656", "Nintendo Co., Ltd." },
    { "001657", "Aegate Ltd" },
    { "001658", "Fusiontech Technologies Inc." },
    { "001659", "Z.M.P. RADWAG" },
    { "00165A", "Harman Specialty Group" },
    { "00165B", "Grip Audio" },
    { "00165C", "Trackflow Ltd." },
    { "00165D", "AirDefense, Inc." },
    { "00165E", "Precision I/O" },
    { "00165F", "Fairmount Automation" },
    { "001660", "Nortel Networks" },
    { "001661", "Novatium Solutions (P) Ltd" },
    { "001662", "Liyuh Technology Ltd." },
    { "001663", "KBT Mobile" },
    { "001664", "Prod-El SpA" },
    { "001665", "Cellon France" },
    { "001666", "Quantier Communication Inc." },
    { "001667", "A-TEC Subsystem INC." },
    { "001668", "Eishin Electronics" },
    { "001669", "MRV Communication (Networks) LTD" },
    { "00166A", "TPS" },
    { "00166B", "Samsung Electronics Co.,Ltd" },
    { "00166C", "Samsung Electronics Co.,Ltd" },
    { "00166D", "Yulong Computer Telecommunication Scientific (Shenzhen) Co.,Ltd" },
    { "00166E", "Arbitron Inc." },
    { "00166F", "Intel Corporate" },
    { "001670", "SKNET Corporation" },
    { "001671", "Symphox Information Co." },
    { "001672", "Zenway enterprise ltd" },
    { "001673", "Bury GmbH & Co. KG" },
    { "001674", "EuroCB (Phils.), Inc." },
    { "001675", "Commscope" },
    { "001676", "Intel Corporate" },
    { "001677", "Bihl + Wiedemann GmbH" },
    { "001678", "SHENZHEN BAOAN GAOKE ELECTRONICS CO., LTD" },
    { "001679", "eOn Communications" },
    { "00167A", "Skyworth Overseas Development Ltd." },
    { "00167B", "Haver&Boecker" },
    { "00167C", "iRex Technologies BV" },
    { "00167D", "Sky-Line Information Co., Ltd." },
    { "00167E", "DIBOSS.CO.,LTD" },
    { "00167F", "Bluebird Soft Inc." },
    { "001680", "Bally Gaming + Systems" },
    { "001681", "Vector Informatik GmbH" },
    { "001682", "OMS Motion" },
    { "001683", "WEBIO International Co.,.Ltd." },
    { "001684", "Donjin Co.,Ltd." },
    { "001685", "Elisa Oyj" },
    { "001686", "Karl Storz Imaging" },
    { "001687", "Chubb CSC-Vendor AP" },
    { "001688", "ServerEngines LLC" },
    { "001689", "Pilkor Electronics Co., Ltd" },
    { "00168A", "id-Confirm Inc" },
    { "00168B", "Paralan Corporation" },
    { "00168C", "DSL Partner AS" },
    { "00168D", "KORWIN CO., Ltd." },
    { "00168E", "Vimicro corporation" },
    { "00168F", "GN Netcom A/S" },
    { "001690", "J-TEK INCORPORATION" },
    { "001691", "Moser-Baer AG" },
    { "001692", "Scientific-Atlanta, Inc." },
    { "001693", "PowerLink Technology Inc." },
    { "001694", "Sennheiser Communications A/S" },
    { "001695", "AVC Technology (International) Limited" },
    { "001696", "QDI Technology (H.K.) Limited" },
    { "001697", "NEC Corporation" },
    { "001698", "T&A Mobile Phones" },
    { "001699", "Tonic DVB Marketing Ltd" },
    { "00169A", "Quadrics Ltd" },
    { "00169B", "Alstom Transport" },
    { "00169C", "Cisco Systems, Inc" },
    { "00169D", "Cisco Systems, Inc" },
    { "00169E", "TV One Ltd" },
    { "00169F", "Vimtron Electronics Co., Ltd." },
    { "0016A0", "Auto-Maskin" },
    { "0016A1", "3Leaf Networks" },
    { "0016A2", "CentraLite Systems, Inc." },
    { "0016A3", "INGETEAM" },
    { "0016A4", "Ezurio Ltd" },
    { "0016A5", "Tandberg Storage ASA" },
    { "0016A6", "Dovado FZ-LLC" },
    { "0016A7", "AWETA G&P" },
    { "0016A8", "CWT CO., LTD." },
    { "0016A9", "2EI" },
    { "0016AA", "Kei Communication Technology Inc." },
    { "0016AB", "Dansensor A/S" },
    { "0016AC", "Toho Technology Corp." },
    { "0016AD", "BT-Links Company Limited" },
    { "0016AE", "INVENTEL" },
    { "0016AF", "Shenzhen Union Networks Equipment Co.,Ltd." },
    { "0016B0", "VK Corporation" },
    { "0016B1", "KBS" },
    { "0016B2", "DriveCam Inc" },
    { "0016B3", "Photonicbridges (China) Co., Ltd." },
    { "0016B4", "Private" },
    { "0016B5", "Commscope" },
    { "0016B6", "Cisco-Linksys, LLC" },
    { "0016B7", "Seoul Commtech" },
    { "0016B8", "Sony Corporation" },
    { "0016B9", "ProCurve Networking by HP" },
    { "0016BA", "WEATHERNEWS INC." },
    { "0016BB", "Law-Chain Computer Technology Co Ltd" },
    { "0016BC", "Nokia Danmark A/S" },
    { "0016BD", "ATI Industrial Automation" },
    { "0016BE", "INFRANET, Inc." },
    { "0016BF", "PaloDEx Group Oy" },
    { "0016C0", "Semtech Corporation" },
    { "0016C1", "Eleksen Ltd" },
    { "0016C2", "Avtec Systems Inc" },
    { "0016C3", "BA Systems Inc" },
    { "0016C4", "SiRF Technology, Inc." },
    { "0016C5", "Shenzhen Xing Feng Industry Co.,Ltd" },
    { "0016C6", "North Atlantic Industries" },
    { "0016C7", "Cisco Systems, Inc" },
    { "0016C8", "Cisco Systems, Inc" },
    { "0016C9", "NAT Seattle, Inc." },
    { "0016CA", "Nortel Networks" },
    { "0016CB", "Apple, Inc." },
    { "0016CC", "Xcute Mobile Corp." },
    { "0016CD", "HIJI HIGH-TECH CO., LTD." },
    { "0016CE", "Hon Hai Precision Ind. Co.,Ltd." },
    { "0016CF", "Hon Hai Precision Ind. Co.,Ltd." },
    { "0016D0", "ATech elektronika d.o.o." },
    { "0016D1", "ZAT a.s." },
    { "0016D2", "Caspian" },
    { "0016D3", "Wistron Corporation" },
    { "0016D4", "Compal Communications, Inc." },
    { "0016D5", "Synccom Co., Ltd" },
    { "0016D6", "TDA Tech Pty Ltd" },
    { "0016D7", "Sunways AG" },
    { "0016D8", "Senea AB" },
    { "0016D9", "NINGBO BIRD CO.,LTD." },
    { "0016DA", "Futronic Technology Co. Ltd." },
    { "0016DB", "Samsung Electronics Co.,Ltd" },
    { "0016DC", "ARCHOS" },
    { "0016DD", "Gigabeam Corporation" },
    { "0016DE", "FAST Inc" },
    { "0016DF", "Lundinova AB" },
    { "0016E0", "3Com Ltd" },
    { "0016E1", "SiliconStor, Inc." },
    { "0016E2", "American Fibertek, Inc." },
    { "0016E3", "ASKEY COMPUTER CORP" },
    { "0016E4", "VANGUARD SECURITY ENGINEERING CORP." },
    { "0016E5", "FORDLEY DEVELOPMENT LIMITED" },
    { "0016E6", "GIGA-BYTE TECHNOLOGY CO.,LTD." },
    { "0016E7", "Dynamix Promotions Limited" },
    { "0016E8", "Lumissil Microsystems" },
    { "0016E9", "Tiba Medical Inc" },
    { "0016EA", "Intel Corporate" },
    { "0016EB", "Intel Corporate" },
    { "0016EC", "Elitegroup Computer Systems Co.,Ltd." },
    { "0016ED", "Utility, Inc" },
    { "0016EE", "Royaldigital Inc." },
    { "0016EF", "Koko Fitness, Inc." },
    { "0016F0", "Dell" },
    { "0016F1", "OmniSense, LLC" },
    { "0016F2", "Dmobile System Co., Ltd." },
    { "0016F3", "CAST Information Co., Ltd" },
    { "0016F4", "Eidicom Co., Ltd." },
    { "0016F5", "Dalian Golden Hualu Digital Technology Co.,Ltd" },
    { "0016F6", "Nevion" },
    { "0016F7", "L-3 Communications, Aviation Recorders" },
    { "0016F8", "AVIQTECH TECHNOLOGY CO., LTD." },
    { "0016F9", "CETRTA POT, d.o.o., Kranj" },
    { "0016FA", "ECI Telecom Ltd." },
    { "0016FB", "SHENZHEN MTC CO LTD" },
    { "0016FC", "TOHKEN CO.,LTD." },
    { "0016FD", "Jaty Electronics" },
    { "0016FE", "ALPSALPINE CO,.LTD" },
    { "0016FF", "Wamin Optocomm Mfg Corp" },
    { "001700", "Commscope" },
    { "001701", "KDE, Inc." },
    { "001702", "Osung Midicom Co., Ltd" },
    { "001703", "MOSDAN Internation Co.,Ltd" },
    { "001704", "Shinco Electronics Group Co.,Ltd" },
    { "001705", "Methode Electronics" },
    { "001706", "Techfaithwireless Communication Technology Limited." },
    { "001707", "InGrid, Inc" },
    { "001708", "Hewlett Packard" },
    { "001709", "Exalt Communications" },
    { "00170A", "INEW DIGITAL COMPANY" },
    { "00170B", "Contela, Inc." },
    { "00170C", "Twig Com Ltd." },
    { "00170D", "Dust Networks Inc." },
    { "00170E", "Cisco Systems, Inc" },
    { "00170F", "Cisco Systems, Inc" },
    { "001710", "AxyomCore Inc." },
    { "001711", "Cytiva Sweden AB" },
    { "001712", "ISCO International" },
    { "001713", "Tiger NetCom" },
    { "001714", "BR Controls Nederland bv" },
    { "001715", "Qstik" },
    { "001716", "Qno Technology Inc." },
    { "001717", "Leica Geosystems AG" },
    { "001718", "Vansco Electronics Oy" },
    { "001719", "Audiocodes USA, Inc" },
    { "00171A", "Winegard Company" },
    { "00171B", "Innovation Lab Corp." },
    { "00171C", "NT MicroSystems, Inc." },
    { "00171D", "DIGIT" },
    { "00171E", "Theo Benning GmbH & Co. KG" },
    { "00171F", "IMV Corporation" },
    { "001720", "Image Sensing Systems, Inc." },
    { "001721", "FITRE S.p.A." },
    { "001722", "Hanazeder Electronic GmbH" },
    { "001723", "Summit Data Communications" },
    { "001724", "Studer Professional Audio GmbH" },
    { "001725", "Liquid Computing" },
    { "001726", "m2c Electronic Technology Ltd." },
    { "001727", "Thermo Ramsey Italia s.r.l." },
    { "001728", "Selex Communications" },
    { "001729", "Ubicod Co.LTD" },
    { "00172A", "Proware Technology Corp.(By Unifosa)" },
    { "00172B", "Global Technologies Inc." },
    { "00172C", "TAEJIN INFOTECH" },
    { "00172D", "Axcen Photonics Corporation" },
    { "00172E", "FXC Inc." },
    { "00172F", "NeuLion Incorporated" },
    { "001730", "Automation Electronics" },
    { "001731", "ASUSTek COMPUTER INC." },
    { "001732", "Science-Technical Center RISSA" },
    { "001733", "SFR" },
    { "001734", "ADC Telecommunications" },
    { "001735", "Intel Wireless Network Group" },
    { "001736", "iiTron Inc." },
    { "001737", "Industrie Dial Face S.p.A." },
    { "001738", "International Business Machines" },
    { "001739", "Bright Headphone Electronics Company" },
    { "00173A", "Cloudastructure Inc" },
    { "00173B", "Cisco Systems, Inc" },
    { "00173C", "Extreme Engineering Solutions" },
    { "00173D", "Neology" },
    { "00173E", "LeucotronEquipamentos Ltda." },
    { "00173F", "Belkin International Inc." },
    { "001740", "Bluberi Gaming Technologies Inc" },
    { "001741", "DEFIDEV" },
    { "001742", "FUJITSU LIMITED" },
    { "001743", "Deck Srl" },
    { "001744", "Araneo Ltd." },
    { "001745", "INNOTZ CO., Ltd" },
    { "001746", "Freedom9 Inc." },
    { "001747", "Trimble" },
    { "001748", "Neokoros Brasil Ltda" },
    { "001749", "HYUNDAE YONG-O-SA CO.,LTD" },
    { "00174A", "SOCOMEC" },
    { "00174B", "Nokia Danmark A/S" },
    { "00174C", "Millipore" },
    { "00174D", "DYNAMIC NETWORK FACTORY, INC." },
    { "00174E", "Parama-tech Co.,Ltd." },
    { "00174F", "iCatch Inc." },
    { "001750", "GSI Group, MicroE Systems" },
    { "001751", "Online Corporation" },
    { "001752", "DAGS, Inc" },
    { "001753", "nFore Technology Inc." },
    { "001754", "Arkino HiTOP Corporation Limited" },
    { "001755", "GE Security" },
    { "001756", "Vinci Labs Oy" },
    { "001757", "RIX TECHNOLOGY LIMITED" },
    { "001758", "ThruVision Ltd" },
    { "001759", "Cisco Systems, Inc" },
    { "00175A", "Cisco Systems, Inc" },
    { "00175B", "ACS Solutions Switzerland Ltd." },
    { "00175C", "SHARP CORPORATION" },
    { "00175D", "Dongseo system." },
    { "00175E", "Zed-3" },
    { "00175F", "XENOLINK Communications Co., Ltd." },
    { "001760", "Naito Densei Machida MFG.CO.,LTD" },
    { "001761", "Private" },
    { "001762", "Solar Technology, Inc." },
    { "001763", "Essentia S.p.A." },
    { "001764", "ATMedia GmbH" },
    { "001765", "Nortel Networks" },
    { "001766", "Accense Technology, Inc." },
    { "001767", "Earforce AS" },
    { "001768", "Zinwave Ltd" },
    { "001769", "Cymphonix Corp" },
    { "00176A", "Avago Technologies" },
    { "00176B", "Kiyon, Inc." },
    { "00176C", "Pivot3, Inc." },
    { "00176D", "CORE CORPORATION" },
    { "00176E", "DUCATI SISTEMI" },
    { "00176F", "PAX Computer Technology(Shenzhen) Ltd." },
    { "001770", "Arti Industrial Electronics Ltd." },
    { "001771", "APD Communications Ltd" },
    { "001772", "ASTRO Strobel Kommunikationssysteme GmbH" },
    { "001773", "Laketune Technologies Co. Ltd" },
    { "001774", "Elesta GmbH" },
    { "001775", "TTE Germany GmbH" },
    { "001776", "Meso Scale Diagnostics, LLC" },
    { "001777", "Obsidian Research Corporation" },
    { "001778", "Central Music Co." },
    { "001779", "QuickTel" },
    { "00177A", "ASSA ABLOY AB" },
    { "00177B", "Azalea Networks inc" },
    { "00177C", "Smartlink Network Systems Limited" },
    { "00177D", "IDT Technology Limited" },
    { "00177E", "Meshcom Technologies Inc." },
    { "00177F", "Worldsmart Retech" },
    { "001780", "Applied Biosystems B.V." },
    { "001781", "Greystone Data System, Inc." },
    { "001782", "LoBenn Inc." },
    { "001783", "Texas Instruments" },
    { "001784", "Commscope" },
    { "001785", "Sparr Electronics Ltd" },
    { "001786", "wisembed" },
    { "001787", "Brother, Brother & Sons ApS" },
    { "001788", "Philips Lighting BV" },
    { "001789", "Zenitron Corporation" },
    { "00178A", "DARTS TECHNOLOGIES CORP." },
    { "00178B", "Teledyne Technologies Incorporated" },
    { "00178C", "Independent Witness, Inc" },
    { "00178D", "Checkpoint Systems, Inc." },
    { "00178E", "Gunnebo Cash Automation AB" },
    { "00178F", "NINGBO YIDONG ELECTRONIC CO.,LTD." },
    { "001790", "HYUNDAI DIGITECH Co, Ltd." },
    { "001791", "LinTech GmbH" },
    { "001792", "Falcom Wireless Comunications Gmbh" },
    { "001793", "Tigi Corporation" },
    { "001794", "Cisco Systems, Inc" },
    { "001795", "Cisco Systems, Inc" },
    { "001796", "Rittmeyer AG" },
    { "001797", "Telsy Elettronica S.p.A." },
    { "001798", "Azonic Technology Co., LTD" },
    { "001799", "SmarTire Systems Inc." },
    { "00179A", "D-Link Corporation" },
    { "00179B", "CHANT SINCERE CO.,LTD" },
    { "00179C", "DEPRAG SCHULZ GMBH u. CO." },
    { "00179D", "Kelman Limited" },
    { "00179E", "Sirit Inc" },
    { "00179F", "Apricorn" },
    { "0017A0", "RoboTech srl" },
    { "0017A1", "3soft inc." },
    { "0017A2", "Camrivox Ltd." },
    { "0017A3", "MIX s.r.l." },
    { "0017A4", "Hewlett Packard" },
    { "0017A5", "MediaTek Inc" },
    { "0017A6", "YOSIN ELECTRONICS CO., LTD." },
    { "0017A7", "Mobile Computing Promotion Consortium" },
    { "0017A8", "EDM Corporation" },
    { "0017A9", "Sentivision" },
    { "0017AA", "elab-experience inc." },
    { "0017AB", "Nintendo Co., Ltd." },
    { "0017AC", "O'Neil Product Development Inc." },
    { "0017AD", "AceNet Corporation" },
    { "0017AE", "GAI-Tronics" },
    { "0017AF", "Enermet" },
    { "0017B0", "Nokia Danmark A/S" },
    { "0017B1", "ACIST Medical Systems, Inc." },
    { "0017B2", "SK Telesys" },
    { "0017B3", "Aftek Infosys Limited" },
    { "0017B4", "Remote Security Systems, LLC" },
    { "0017B5", "Peerless Systems Corporation" },
    { "0017B6", "Aquantia Corporation" },
    { "0017B7", "Tonze Technology Co." },
    { "0017B8", "NOVATRON CO., LTD." },
    { "0017B9", "Gambro Lundia AB" },
    { "0017BA", "SEDO CO., LTD." },
    { "0017BB", "Syrinx Industrial Electronics" },
    { "0017BC", "Touchtunes Music Corporation" },
    { "0017BD", "Tibetsystem" },
    { "0017BE", "Tratec Telecom B.V." },
    { "0017BF", "Coherent Research Limited" },
    { "0017C0", "PureTech Systems, Inc." },
    { "0017C1", "CM Precision Technology LTD." },
    { "0017C2", "ADB Broadband Italia" },
    { "0017C3", "KTF Technologies Inc." },
    { "0017C4", "Quanta Microsystems, INC." },
    { "0017C5", "SonicWALL" },
    { "0017C6", "Cross Match Technologies Inc" },
    { "0017C7", "MARA Systems Consulting AB" },
    { "0017C8", "KYOCERA Display Corporation" },
    { "0017C9", "Samsung Electronics Co.,Ltd" },
    { "0017CA", "Qisda Corporation" },
    { "0017CB", "Juniper Networks" },
    { "0017CC", "Alcatel-Lucent" },
    { "0017CD", "CEC Wireless R&D Ltd." },
    { "0017CE", "Screen Service Spa" },
    { "0017CF", "iMCA-GmbH" },
    { "0017D0", "Opticom Communications, LLC" },
    { "0017D1", "Nortel Networks" },
    { "0017D2", "THINLINX PTY LTD" },
    { "0017D3", "Etymotic Research, Inc." },
    { "0017D4", "Monsoon Multimedia, Inc" },
    { "0017D5", "Samsung Electronics Co.,Ltd" },
    { "0017D6", "Bluechips Microhouse Co.,Ltd." },
    { "0017D7", "ION Geophysical Corporation Inc." },
    { "0017D8", "Magnum Semiconductor, Inc." },
    { "0017D9", "AAI Corporation" },
    { "0017DA", "Spans Logic" },
    { "0017DB", "CANKO TECHNOLOGIES INC." },
    { "0017DC", "DAEMYUNG ZERO1" },
    { "0017DD", "Clipsal Australia" },
    { "0017DE", "Advantage Six Ltd" },
    { "0017DF", "Cisco Systems, Inc" },
    { "0017E0", "Cisco Systems, Inc" },
    { "0017E1", "DACOS Technologies Co., Ltd." },
    { "0017E2", "Commscope" },
    { "0017E3", "Texas Instruments" },
    { "0017E4", "Texas Instruments" },
    { "0017E5", "Texas Instruments" },
    { "0017E6", "Texas Instruments" },
    { "0017E7", "Texas Instruments" },
    { "0017E8", "Texas Instruments" },
    { "0017E9", "Texas Instruments" },
    { "0017EA", "Texas Instruments" },
    { "0017EB", "Texas Instruments" },
    { "0017EC", "Texas Instruments" },
    { "0017ED", "WooJooIT Ltd." },
    { "0017EE", "Commscope" },
    { "0017EF", "IBM Corp" },
    { "0017F0", "SZCOM Broadband Network Technology Co.,Ltd" },
    { "0017F1", "Renu Electronics Pvt Ltd" },
    { "0017F2", "Apple, Inc." },
    { "0017F3", "Harris Corporation" },
    { "0017F4", "ZERON ALLIANCE" },
    { "0017F5", "LIG NEOPTEK" },
    { "0017F6", "Pyramid Meriden Inc." },
    { "0017F7", "CEM Solutions Pvt Ltd" },
    { "0017F8", "Motech Industries Inc." },
    { "0017F9", "Forcom Sp. z o.o." },
    { "0017FA", "Microsoft Corporation" },
    { "0017FB", "FA" },
    { "0017FC", "Suprema Inc." },
    { "0017FD", "Amulet Hotkey" },
    { "0017FE", "TALOS SYSTEM INC." },
    { "0017FF", "PLAYLINE Co.,Ltd." },
    { "001800", "UNIGRAND LTD" },
    { "001801", "Actiontec Electronics, Inc" },
    { "001802", "Alpha Networks Inc." },
    { "001803", "ArcSoft Shanghai Co. LTD" },
    { "001804", "E-TEK DIGITAL TECHNOLOGY LIMITED" },
    { "001805", "Beijing InHand Networking Technology Co.,Ltd." },
    { "001806", "Hokkei Industries Co., Ltd." },
    { "001807", "Fanstel Corp." },
    { "001808", "SightLogix, Inc." },
    { "001809", "CRESYN" },
    { "00180A", "Cisco Meraki" },
    { "00180B", "Brilliant Telecommunications" },
    { "00180C", "Zhone Technologies, Inc." },
    { "00180D", "Terabytes Server Storage Tech Corp" },
    { "00180E", "Avega Systems" },
    { "00180F", "Nokia Danmark A/S" },
    { "001810", "IPTrade S.A." },
    { "001811", "Neuros Technology International, LLC." },
    { "001812", "Beijing Xinwei Telecom Technology Co., Ltd." },
    { "001813", "Sony Corporation" },
    { "001814", "Mitutoyo Corporation" },
    { "001815", "GZ Technologies, Inc." },
    { "001816", "Ubixon Co., Ltd." },
    { "001817", "D. E. Shaw Research, LLC" },
    { "001818", "Cisco Systems, Inc" },
    { "001819", "Cisco Systems, Inc" },
    { "00181A", "AVerMedia Information Inc." },
    { "00181B", "TaiJin Metal Co., Ltd." },
    { "00181C", "VITEC" },
    { "00181D", "ASIA ELECTRONICS CO.,LTD" },
    { "00181E", "GDX Technologies Ltd." },
    { "00181F", "Palmmicro Communications" },
    { "001820", "w5networks" },
    { "001821", "SINDORICOH" },
    { "001822", "CEC TELECOM CO.,LTD." },
    { "001823", "Delta Electronics, Inc." },
    { "001824", "Kimaldi Electronics, S.L." },
    { "001825", "Private" },
    { "001826", "Cale Access AB" },
    { "001827", "NEC UNIFIED SOLUTIONS NEDERLAND B.V." },
    { "001828", "e2v technologies (UK) ltd." },
    { "001829", "Gatsometer" },
    { "00182A", "Taiwan Video & Monitor" },
    { "00182B", "Softier" },
    { "00182C", "Ascend Networks, Inc." },
    { "00182D", "Artec Design" },
    { "00182E", "XStreamHD" },
    { "00182F", "Texas Instruments" },
    { "001830", "Texas Instruments" },
    { "001831", "Texas Instruments" },
    { "001832", "Texas Instruments" },
    { "001833", "Texas Instruments" },
    { "001834", "Texas Instruments" },
    { "001835", "Thoratec / ITC" },
    { "001836", "REJ Co.,Ltd" },
    { "001837", "Universal ABIT Co., Ltd." },
    { "001838", "PanAccess Communications,Inc." },
    { "001839", "Cisco-Linksys, LLC" },
    { "00183A", "Westell Technologies Inc." },
    { "00183B", "CENITS Co., Ltd." },
    { "00183C", "Encore Software Limited" },
    { "00183D", "Vertex Link Corporation" },
    { "00183E", "Digilent, Inc" },
    { "00183F", "2Wire Inc" },
    { "001840", "3 Phoenix, Inc." },
    { "001841", "High Tech Computer Corp" },
    { "001842", "Nokia Danmark A/S" },
    { "001843", "Dawevision Ltd" },
    { "001844", "Heads Up Technologies, Inc." },
    { "001845", "Pulsar-Telecom LLC." },
    { "001846", "Crypto S.A." },
    { "001847", "AceNet Technology Inc." },
    { "001848", "Vecima Networks Inc." },
    { "001849", "nVent, Schroff GmbH" },
    { "00184A", "Catcher, Inc." },
    { "00184B", "Las Vegas Gaming, Inc." },
    { "00184C", "Bogen Communications" },
    { "00184D", "NETGEAR" },
    { "00184E", "Lianhe Technologies, Inc." },
    { "00184F", "8 Ways Technology Corp." },
    { "001850", "Secfone Kft" },
    { "001851", "SWsoft" },
    { "001852", "StorLink Semiconductors, Inc." },
    { "001853", "Atera Networks LTD." },
    { "001854", "Argard Co., Ltd" },
    { "001855", "Aeromaritime Systembau GmbH" },
    { "001856", "EyeFi, Inc" },
    { "001857", "Unilever R&D" },
    { "001858", "TagMaster AB" },
    { "001859", "Strawberry Linux Co.,Ltd." },
    { "00185A", "uControl, Inc." },
    { "00185B", "Network Chemistry, Inc" },
    { "00185C", "EDSLAB Technologies" },
    { "00185D", "TAIGUEN TECHNOLOGY (SHEN-ZHEN) CO., LTD." },
    { "00185E", "Nexterm Inc." },
    { "00185F", "TAC Inc." },
    { "001860", "SIM Technology Group Shanghai Simcom Ltd.," },
    { "001861", "Ooma, Inc." },
    { "001862", "Seagate Technology" },
    { "001863", "Veritech Electronics Limited" },
    { "001864", "Eaton Corporation" },
    { "001865", "Siemens Healthcare Diagnostics Manufacturing Ltd" },
    { "001866", "Leutron Vision" },
    { "001867", "Datalogic ADC" },
    { "001868", "Cisco SPVTG" },
    { "001869", "KINGJIM" },
    { "00186A", "Global Link Digital Technology Co,.LTD" },
    { "00186B", "Sambu Communics CO., LTD." },
    { "00186C", "Neonode AB" },
    { "00186D", "Zhenjiang Sapphire Electronic Industry CO." },
    { "00186E", "3Com Ltd" },
    { "00186F", "Setha Industria Eletronica LTDA" },
    { "001870", "E28 Shanghai Limited" },
    { "001871", "Hewlett Packard" },
    { "001872", "Expertise Engineering" },
    { "001873", "Cisco Systems, Inc" },
    { "001874", "Cisco Systems, Inc" },
    { "001875", "AnaCise Testnology Pte Ltd" },
    { "001876", "WowWee Ltd." },
    { "001877", "Amplex A/S" },
    { "001878", "Mackware GmbH" },
    { "001879", "dSys" },
    { "00187A", "Wiremold" },
    { "00187B", "4NSYS Co. Ltd." },
    { "00187C", "INTERCROSS, LLC" },
    { "00187D", "Armorlink Co .Ltd" },
    { "00187E", "RGB Spectrum" },
    { "00187F", "ZODIANET" },
    { "001880", "Maxim Integrated Products" },
    { "001881", "Buyang Electronics Industrial Co., Ltd" },
    { "001882", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "001883", "FORMOSA21 INC." },
    { "001884", "Fon Technology S.L." },
    { "001885", "Motorola Solutions Inc." },
    { "001886", "EL-TECH, INC." },
    { "001887", "Metasystem SpA" },
    { "001888", "GOTIVE a.s." },
    { "001889", "WinNet Solutions Limited" },
    { "00188A", "Infinova LLC" },
    { "00188B", "Dell Inc." },
    { "00188C", "Mobile Action Technology Inc." },
    { "00188D", "Nokia Danmark A/S" },
    { "00188E", "Ekahau, Inc." },
    { "00188F", "Montgomery Technology, Inc." },
    { "001890", "RadioCOM, s.r.o." },
    { "001891", "Zhongshan General K-mate Electronics Co., Ltd" },
    { "001892", "ads-tec GmbH" },
    { "001893", "SHENZHEN PHOTON BROADBAND TECHNOLOGY CO.,LTD" },
    { "001894", "NPCore, Inc." },
    { "001895", "Hansun Technologies Inc." },
    { "001896", "Great Well Electronic LTD" },
    { "001897", "JESS-LINK PRODUCTS Co., LTD" },
    { "001898", "KINGSTATE ELECTRONICS CORPORATION" },
    { "001899", "ShenZhen jieshun Science&Technology Industry CO,LTD." },
    { "00189A", "HANA Micron Inc." },
    { "00189B", "Thomson Inc." },
    { "00189C", "Weldex Corporation" },
    { "00189D", "Navcast Inc." },
    { "00189E", "OMNIKEY GmbH." },
    { "00189F", "Lenntek Corporation" },
    { "0018A0", "Cierma Ascenseurs" },
    { "0018A1", "Tiqit Computers, Inc." },
    { "0018A2", "XIP Technology AB" },
    { "0018A3", "ZIPPY TECHNOLOGY CORP." },
    { "0018A4", "Commscope" },
    { "0018A5", "ADigit Technologies Corp." },
    { "0018A6", "Persistent Systems, LLC" },
    { "0018A7", "Yoggie Security Systems LTD." },
    { "0018A8", "AnNeal Technology Inc." },
    { "0018A9", "Ethernet Direct Corporation" },
    { "0018AA", "Protec Fire Detection plc" },
    { "0018AB", "BEIJING LHWT MICROELECTRONICS INC." },
    { "0018AC", "Shanghai Jiao Da HISYS Technology Co. Ltd." },
    { "0018AD", "NIDEC INSTRUMENTS CORPORATION" },
    { "0018AE", "TVT CO.,LTD" },
    { "0018AF", "Samsung Electronics Co.,Ltd" },
    { "0018B0", "Nortel Networks" },
    { "0018B1", "IBM Corp" },
    { "0018B2", "ADEUNIS RF" },
    { "0018B3", "TEC WizHome Co., Ltd." },
    { "0018B4", "Dawon Media Inc." },
    { "0018B5", "Magna Carta" },
    { "0018B6", "S3C, Inc." },
    { "0018B7", "D3 LED, LLC" },
    { "0018B8", "New Voice International AG" },
    { "0018B9", "Cisco Systems, Inc" },
    { "0018BA", "Cisco Systems, Inc" },
    { "0018BB", "Eliwell Controls srl" },
    { "0018BC", "ZAO NVP Bolid" },
    { "0018BD", "SHENZHEN DVBWORLD TECHNOLOGY CO., LTD." },
    { "0018BE", "ANSA Corporation" },
    { "0018BF", "Essence Technology Solution, Inc." },
    { "0018C0", "Commscope" },
    { "0018C1", "Almitec Informática e Comércio" },
    { "0018C2", "Firetide, Inc" },
    { "0018C3", "CS Corporation" },
    { "0018C4", "Raba Technologies LLC" },
    { "0018C5", "Nokia Danmark A/S" },
    { "0018C6", "OPW Fuel Management Systems" },
    { "0018C7", "Real Time Automation" },
    { "0018C8", "ISONAS Inc." },
    { "0018C9", "EOps Technology Limited" },
    { "0018CA", "Viprinet GmbH" },
    { "0018CB", "Tecobest Technology Limited" },
    { "0018CC", "AXIOHM SAS" },
    { "0018CD", "Erae Electronics Industry Co., Ltd" },
    { "0018CE", "Dreamtech Co., Ltd" },
    { "0018CF", "Baldor Electric Company" },
    { "0018D0", "AtRoad, A Trimble Company" },
    { "0018D1", "Siemens Home & Office Comm. Devices" },
    { "0018D2", "High-Gain Antennas LLC" },
    { "0018D3", "TEAMCAST" },
    { "0018D4", "Unified Display Interface SIG" },
    { "0018D5", "REIGNCOM" },
    { "0018D6", "Swirlnet A/S" },
    { "0018D7", "JAVAD GNSS, Inc." },
    { "0018D8", "ARCH METER Corporation" },
    { "0018D9", "Santosha Internatonal, Inc" },
    { "0018DA", "Würth Elektronik eiSos GmbH & Co. KG" },
    { "0018DB", "EPL Technology Ltd" },
    { "0018DC", "Prostar Co., Ltd." },
    { "0018DD", "Silicondust Engineering Ltd" },
    { "0018DE", "Intel Corporate" },
    { "0018DF", "The Morey Corporation" },
    { "0018E0", "ANAVEO" },
    { "0018E1", "Verkerk Service Systemen" },
    { "0018E2", "Topdata Sistemas de Automacao Ltda" },
    { "0018E3", "Visualgate Systems, Inc." },
    { "0018E4", "YIGUANG" },
    { "0018E5", "Adhoco AG" },
    { "0018E6", "Computer Hardware Design SIA" },
    { "0018E7", "Cameo Communications, INC." },
    { "0018E8", "Hacetron Corporation" },
    { "0018E9", "Numata Corporation" },
    { "0018EA", "Alltec GmbH" },
    { "0018EB", "Blue Zen Enterprises Private Limited" },
    { "0018EC", "Welding Technology Corporation" },
    { "0018ED", "Accutech Ultrasystems Co., Ltd." },
    { "0018EE", "Videology Imaging Solutions, Inc." },
    { "0018EF", "Escape Communications, Inc." },
    { "0018F0", "JOYTOTO Co., Ltd." },
    { "0018F1", "Chunichi Denshi Co.,LTD." },
    { "0018F2", "Beijing Tianyu Communication Equipment Co., Ltd" },
    { "0018F3", "ASUSTek COMPUTER INC." },
    { "0018F4", "EO TECHNICS Co., Ltd." },
    { "0018F5", "Shenzhen Streaming Video Technology Company Limited" },
    { "0018F6", "Thomson Telecom Belgium" },
    { "0018F7", "Kameleon Technologies" },
    { "0018F8", "Cisco-Linksys, LLC" },
    { "0018F9", "VVOND, Inc." },
    { "0018FA", "Yushin Precision Equipment Co.,Ltd." },
    { "0018FB", "Compro Technology" },
    { "0018FC", "Altec Electronic AG" },
    { "0018FD", "Optimal Technologies International Inc." },
    { "0018FE", "Hewlett Packard" },
    { "0018FF", "PowerQuattro Co." },
    { "001900", "Intelliverese - DBA Voicecom" },
    { "001901", "F1MEDIA" },
    { "001902", "Cambridge Consultants Ltd" },
    { "001903", "Bigfoot Networks Inc" },
    { "001904", "WB Electronics Sp. z o.o." },
    { "001905", "SCHRACK Seconet AG" },
    { "001906", "Cisco Systems, Inc" },
    { "001907", "Cisco Systems, Inc" },
    { "001908", "Duaxes Corporation" },
    { "001909", "DEVI - Danfoss A/S" },
    { "00190A", "HASWARE INC." },
    { "00190B", "Southern Vision Systems, Inc." },
    { "00190C", "Encore Electronics, Inc." },
    { "00190D", "IEEE 1394c" },
    { "00190E", "Atech Technology Co., Ltd." },
    { "00190F", "Advansus Corp." },
    { "001910", "Knick Elektronische Messgeraete GmbH & Co. KG" },
    { "001911", "Just In Mobile Information Technologies (Shanghai) Co., Ltd." },
    { "001912", "Welcat Inc" },
    { "001913", "Chuang-Yi Network Equipment Co.Ltd." },
    { "001914", "Winix Co., Ltd" },
    { "001915", "TECOM Co., Ltd." },
    { "001916", "PayTec AG" },
    { "001917", "Posiflex Inc." },
    { "001918", "Interactive Wear AG" },
    { "001919", "ASTEL Inc." },
    { "00191A", "IRLINK" },
    { "00191B", "Sputnik Engineering AG" },
    { "00191C", "Sensicast Systems" },
    { "00191D", "Nintendo Co., Ltd." },
    { "00191E", "Beyondwiz Co., Ltd." },
    { "00191F", "Microlink communications Inc." },
    { "001920", "KUME electric Co.,Ltd." },
    { "001921", "Elitegroup Computer Systems Co.,Ltd." },
    { "001922", "CM Comandos Lineares" },
    { "001923", "Phonex Korea Co., LTD." },
    { "001924", "LBNL Engineering" },
    { "001925", "Intelicis Corporation" },
    { "001926", "BitsGen Co., Ltd." },
    { "001927", "ImCoSys Ltd" },
    { "001928", "Siemens AG, Transportation Systems" },
    { "001929", "2M2B Montadora de Maquinas Bahia Brasil LTDA" },
    { "00192A", "Antiope Associates" },
    { "00192B", "Aclara RF Systems Inc." },
    { "00192C", "Commscope" },
    { "00192D", "Nokia Corporation" },
    { "00192E", "Spectral Instruments, Inc." },
    { "00192F", "Cisco Systems, Inc" },
    { "001930", "Cisco Systems, Inc" },
    { "001931", "Balluff GmbH" },
    { "001932", "Gude Systems GmbH" },
    { "001933", "Strix Systems, Inc." },
    { "001934", "TRENDON TOUCH TECHNOLOGY CORP." },
    { "001935", "DUERR DENTAL AG" },
    { "001936", "STERLITE OPTICAL TECHNOLOGIES LIMITED" },
    { "001937", "CommerceGuard AB" },
    { "001938", "UMB Communications Co., Ltd." },
    { "001939", "Gigamips" },
    { "00193A", "OESOLUTIONS" },
    { "00193B", "LigoWave" },
    { "00193C", "HighPoint Technologies Incorporated" },
    { "00193D", "GMC Guardian Mobility Corp." },
    { "00193E", "ADB Broadband Italia" },
    { "00193F", "RDI technology(Shenzhen) Co.,LTD" },
    { "001940", "Rackable Systems" },
    { "001941", "Pitney Bowes, Inc" },
    { "001942", "ON SOFTWARE INTERNATIONAL LIMITED" },
    { "001943", "Belden" },
    { "001944", "Fossil Partners, L.P." },
    { "001945", "RF COncepts, LLC" },
    { "001946", "Cianet Industria e Comercio S/A" },
    { "001947", "Cisco SPVTG" },
    { "001948", "AireSpider Networks" },
    { "001949", "TENTEL COMTECH CO., LTD." },
    { "00194A", "TESTO AG" },
    { "00194B", "Sagemcom Broadband SAS" },
    { "00194C", "Fujian Stelcom information & Technology CO.,Ltd" },
    { "00194D", "Avago Technologies Sdn Bhd" },
    { "00194E", "Ultra Electronics - TCS (Tactical Communication Systems)" },
    { "00194F", "Nokia Danmark A/S" },
    { "001950", "Harman Multimedia" },
    { "001951", "NETCONS, s.r.o." },
    { "001952", "ACOGITO Co., Ltd" },
    { "001953", "Chainleader Communications Corp." },
    { "001954", "Leaf Corporation." },
    { "001955", "Cisco Systems, Inc" },
    { "001956", "Cisco Systems, Inc" },
    { "001957", "Saafnet Canada Inc." },
    { "001958", "Bluetooth SIG, Inc." },
    { "001959", "Staccato Communications Inc." },
    { "00195A", "Jenaer Antriebstechnik GmbH" },
    { "00195B", "D-Link Corporation" },
    { "00195C", "Innotech Corporation" },
    { "00195D", "ShenZhen XinHuaTong Opto Electronics Co.,Ltd" },
    { "00195E", "Commscope" },
    { "00195F", "Valemount Networks Corporation" },
    { "001960", "DoCoMo Systems, Inc." },
    { "001961", "Blaupunkt Embedded Systems GmbH" },
    { "001962", "Commerciant, LP" },
    { "001963", "Sony Corporation" },
    { "001964", "Doorking Inc." },
    { "001965", "YuHua TelTech (ShangHai) Co., Ltd." },
    { "001966", "Asiarock Technology Limited" },
    { "001967", "TELDAT Sp.J." },
    { "001968", "Digital Video Networks(Shanghai) CO. LTD." },
    { "001969", "Nortel Networks" },
    { "00196A", "MikroM GmbH" },
    { "00196B", "Danpex Corporation" },
    { "00196C", "ETROVISION TECHNOLOGY" },
    { "00196D", "Raybit Systems Korea, Inc" },
    { "00196E", "Metacom (Pty) Ltd." },
    { "00196F", "SensoPart GmbH" },
    { "001970", "Z-Com, Inc." },
    { "001971", "Guangzhou Unicomp Technology Co.,Ltd" },
    { "001972", "Plexus (Xiamen) Co.,ltd." },
    { "001973", "Zeugma Systems" },
    { "001974", "16063" },
    { "001975", "Beijing Huisen networks technology Inc" },
    { "001976", "Xipher Technologies, LLC" },
    { "001977", "Extreme Networks Headquarters" },
    { "001978", "Datum Systems, Inc." },
    { "001979", "Nokia Danmark A/S" },
    { "00197A", "MAZeT GmbH" },
    { "00197B", "Picotest Corp." },
    { "00197C", "Riedel Communications GmbH" },
    { "00197D", "Hon Hai Precision Ind. Co.,Ltd." },
    { "00197E", "Hon Hai Precision Ind. Co.,Ltd." },
    { "00197F", "PLANTRONICS, INC." },
    { "001980", "Gridpoint Systems" },
    { "001981", "Vivox Inc" },
    { "001982", "SmarDTV Corporation" },
    { "001983", "CCT R&D Limited" },
    { "001984", "ESTIC Corporation" },
    { "001985", "IT Watchdogs, Inc" },
    { "001986", "Cheng Hongjian" },
    { "001987", "Panasonic Mobile Communications Co.,Ltd." },
    { "001988", "Wi2Wi, Inc" },
    { "001989", "Sonitrol Corporation" },
    { "00198A", "Northrop Grumman Systems Corp." },
    { "00198B", "Novera Optics Korea, Inc." },
    { "00198C", "iXSea" },
    { "00198D", "Ocean Optics, Inc." },
    { "00198E", "Demant A/S" },
    { "00198F", "Nokia Bell N.V." },
    { "001990", "ELM DATA Co., Ltd." },
    { "001991", "avinfo" },
    { "001992", "Adtran Inc" },
    { "001993", "Changshu Switchgear MFG. Co.,Ltd. (Former Changshu Switchgea" },
    { "001994", "Jorjin Technologies Inc." },
    { "001995", "Jurong Hi-Tech (Suzhou)Co.ltd" },
    { "001996", "TurboChef Technologies Inc." },
    { "001997", "Soft Device Sdn Bhd" },
    { "001998", "SATO CORPORATION" },
    { "001999", "Fujitsu Technology Solutions GmbH" },
    { "00199A", "EDO-EVI" },
    { "00199B", "Diversified Technical Systems, Inc." },
    { "00199C", "CTRING" },
    { "00199D", "Vizio, Inc" },
    { "00199E", "Nifty" },
    { "00199F", "DKT A/S" },
    { "0019A0", "NIHON DATA SYSTENS, INC." },
    { "0019A1", "LG INFORMATION & COMM." },
    { "0019A2", "ORDYN TECHNOLOGIES" },
    { "0019A3", "asteel electronique atlantique" },
    { "0019A4", "Austar Technology (hang zhou) Co.,Ltd" },
    { "0019A5", "RadarFind Corporation" },
    { "0019A6", "Commscope" },
    { "0019A7", "ITU-T" },
    { "0019A8", "WiQuest Communications" },
    { "0019A9", "Cisco Systems, Inc" },
    { "0019AA", "Cisco Systems, Inc" },
    { "0019AB", "Raycom CO ., LTD" },
    { "0019AC", "GSP SYSTEMS Inc." },
    { "0019AD", "BOBST SA" },
    { "0019AE", "Hopling Technologies b.v." },
    { "0019AF", "Rigol Technologies, Inc." },
    { "0019B0", "HanYang System" },
    { "0019B1", "Arrow7 Corporation" },
    { "0019B2", "XYnetsoft Co.,Ltd" },
    { "0019B3", "Stanford Research Systems" },
    { "0019B4", "Intellio Ltd" },
    { "0019B5", "Famar Fueguina S.A." },
    { "0019B6", "Euro Emme s.r.l." },
    { "0019B7", "Nokia Danmark A/S" },
    { "0019B8", "Boundary Devices" },
    { "0019B9", "Dell Inc." },
    { "0019BA", "Paradox Security Systems Ltd" },
    { "0019BB", "Hewlett Packard" },
    { "0019BC", "ELECTRO CHANCE SRL" },
    { "0019BD", "New Media Life" },
    { "0019BE", "Altai Technologies Limited" },
    { "0019BF", "Citiway technology Co.,ltd" },
    { "0019C0", "Commscope" },
    { "0019C1", "ALPSALPINE CO,.LTD" },
    { "0019C2", "Equustek Solutions, Inc." },
    { "0019C3", "Qualitrol" },
    { "0019C4", "Infocrypt Inc." },
    { "0019C5", "Sony Interactive Entertainment Inc." },
    { "0019C6", "zte corporation" },
    { "0019C7", "Cambridge Industries(Group) Co.,Ltd." },
    { "0019C8", "AnyDATA Corporation" },
    { "0019C9", "S&C ELECTRIC COMPANY" },
    { "0019CA", "Broadata Communications, Inc" },
    { "0019CB", "Zyxel Communications Corporation" },
    { "0019CC", "RCG (HK) Ltd" },
    { "0019CD", "Chengdu ethercom information technology Ltd." },
    { "0019CE", "Progressive Gaming International" },
    { "0019CF", "SALICRU, S.A." },
    { "0019D0", "Cathexis" },
    { "0019D1", "Intel Corporate" },
    { "0019D2", "Intel Corporate" },
    { "0019D3", "TRAK Microwave" },
    { "0019D4", "ICX Technologies" },
    { "0019D5", "IP Innovations, Inc." },
    { "0019D6", "LS Cable and System Ltd." },
    { "0019D7", "FORTUNETEK CO., LTD" },
    { "0019D8", "MAXFOR" },
    { "0019D9", "Zeutschel GmbH" },
    { "0019DA", "Welltrans O&E Technology Co. , Ltd." },
    { "0019DB", "MICRO-STAR INTERNATIONAL CO., LTD." },
    { "0019DC", "ENENSYS Technologies" },
    { "0019DD", "FEI-Zyfer, Inc." },
    { "0019DE", "MOBITEK" },
    { "0019DF", "Thomson Inc." },
    { "0019E0", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "0019E1", "Nortel Networks" },
    { "0019E2", "Juniper Networks" },
    { "0019E3", "Apple, Inc." },
    { "0019E4", "2Wire Inc" },
    { "0019E5", "Lynx Studio Technology, Inc." },
    { "0019E6", "TOYO MEDIC CO.,LTD." },
    { "0019E7", "Cisco Systems, Inc" },
    { "0019E8", "Cisco Systems, Inc" },
    { "0019E9", "S-Information Technolgy, Co., Ltd." },
    { "0019EA", "TeraMage Technologies Co., Ltd." },
    { "0019EB", "Pyronix Ltd" },
    { "0019EC", "Sagamore Systems, Inc." },
    { "0019ED", "Axesstel Inc." },
    { "0019EE", "CARLO GAVAZZI CONTROLS SPA-Controls Division" },
    { "0019EF", "SHENZHEN LINNKING ELECTRONICS CO.,LTD" },
    { "0019F0", "UNION MAN TECHNOLOGY CO.,LTD" },
    { "0019F1", "Star Communication Network Technology Co.,Ltd" },
    { "0019F2", "Teradyne K.K." },
    { "0019F3", "Cetis, Inc" },
    { "0019F4", "Convergens Oy Ltd" },
    { "0019F5", "Imagination Technologies Ltd" },
    { "0019F6", "Acconet (PTE) Ltd" },
    { "0019F7", "Onset Computer Corporation" },
    { "0019F8", "Embedded Systems Design, Inc." },
    { "0019F9", "TDK-Lambda" },
    { "0019FA", "Cable Vision Electronics CO., LTD." },
    { "0019FB", "SKY UK LIMITED" },
    { "0019FC", "PT. Ufoakses Sukses Luarbiasa" },
    { "0019FD", "Nintendo Co., Ltd." },
    { "0019FE", "SHENZHEN SEECOMM TECHNOLOGY CO.,LTD." },
    { "0019FF", "Finnzymes" },
    { "001A00", "MATRIX INC." },
    { "001A01", "ICU Medical, Inc." },
    { "001A02", "SECURE CARE PRODUCTS, INC" },
    { "001A03", "Angel Electronics Co., Ltd." },
    { "001A04", "Interay Solutions BV" },
    { "001A05", "OPTIBASE LTD" },
    { "001A06", "OpVista, Inc." },
    { "001A07", "Arecont Vision" },
    { "001A08", "Simoco Ltd." },
    { "001A09", "Wayfarer Transit Systems Ltd" },
    { "001A0A", "Adaptive Micro-Ware Inc." },
    { "001A0B", "BONA TECHNOLOGY INC." },
    { "001A0C", "Swe-Dish Satellite Systems AB" },
    { "001A0D", "HandHeld entertainment, Inc." },
    { "001A0E", "Cheng Uei Precision Industry Co.,Ltd" },
    { "001A0F", "ARTECHE GROUP" },
    { "001A10", "LUCENT TRANS ELECTRONICS CO.,LTD" },
    { "001A11", "Google, Inc." },
    { "001A12", "Essilor" },
    { "001A13", "Wanlida Group Co., LTD" },
    { "001A14", "Xin Hua Control Engineering Co.,Ltd." },
    { "001A15", "gemalto e-Payment" },
    { "001A16", "Nokia Danmark A/S" },
    { "001A17", "Teak Technologies, Inc." },
    { "001A18", "Advanced Simulation Technology inc." },
    { "001A19", "Computer Engineering Limited" },
    { "001A1A", "Gentex Corporation/Electro-Acoustic Products" },
    { "001A1B", "Commscope" },
    { "001A1C", "GT&T Engineering Pte Ltd" },
    { "001A1D", "PChome Online Inc." },
    { "001A1E", "Hewlett Packard Enterprise" },
    { "001A1F", "Coastal Environmental Systems" },
    { "001A20", "CMOTECH Co. Ltd." },
    { "001A21", "Brookhuis Applied Technologies BV" },
    { "001A22", "eQ-3 Entwicklung GmbH" },
    { "001A23", "Ice Qube, Inc" },
    { "001A24", "Galaxy Telecom Technologies Ltd" },
    { "001A25", "DELTA DORE" },
    { "001A26", "Deltanode Solutions AB" },
    { "001A27", "Ubistar" },
    { "001A28", "ASWT Co., LTD. Taiwan Branch H.K." },
    { "001A29", "Johnson Outdoors Marine Electronics d/b/a Minnkota" },
    { "001A2A", "Arcadyan Technology Corporation" },
    { "001A2B", "Ayecom Technology Co., Ltd." },
    { "001A2C", "SATEC Co.,LTD" },
    { "001A2D", "The Navvo Group" },
    { "001A2E", "Ziova Coporation" },
    { "001A2F", "Cisco Systems, Inc" },
    { "001A30", "Cisco Systems, Inc" },
    { "001A31", "SCAN COIN AB" },
    { "001A32", "ACTIVA MULTIMEDIA" },
    { "001A33", "ASI Communications, Inc." },
    { "001A34", "Konka Group Co., Ltd." },
    { "001A35", "BARTEC GmbH" },
    { "001A36", "Aipermon GmbH & Co. KG" },
    { "001A37", "Lear Corporation" },
    { "001A38", "Sanmina-SCI" },
    { "001A39", "Merten GmbH&CoKG" },
    { "001A3A", "Dongahelecomm" },
    { "001A3B", "Doah Elecom Inc." },
    { "001A3C", "Technowave Ltd." },
    { "001A3D", "Ajin Vision Co.,Ltd" },
    { "001A3E", "Faster Technology LLC" },
    { "001A3F", "Intelbras" },
    { "001A40", "A-FOUR TECH CO., LTD." },
    { "001A41", "INOCOVA Co.,Ltd" },
    { "001A42", "Techcity Technology co., Ltd." },
    { "001A43", "Logical Link Communications" },
    { "001A44", "JWTrading Co., Ltd" },
    { "001A45", "GN Netcom A/S" },
    { "001A46", "Digital Multimedia Technology Co., Ltd" },
    { "001A47", "Agami Systems, Inc." },
    { "001A48", "Takacom Corporation" },
    { "001A49", "Micro Vision Co.,LTD" },
    { "001A4A", "Qumranet Inc." },
    { "001A4B", "Hewlett Packard" },
    { "001A4C", "Crossbow Technology, Inc" },
    { "001A4D", "GIGA-BYTE TECHNOLOGY CO.,LTD." },
    { "001A4E", "NTI AG / LinMot" },
    { "001A4F", "AVM GmbH" },
    { "001A50", "PheeNet Technology Corp." },
    { "001A51", "Alfred Mann Foundation" },
    { "001A52", "Meshlinx Wireless Inc." },
    { "001A53", "Zylaya" },
    { "001A54", "Hip Shing Electronics Ltd." },
    { "001A55", "ACA-Digital Corporation" },
    { "001A56", "ViewTel Co,. Ltd." },
    { "001A57", "Matrix Design Group, LLC" },
    { "001A58", "CCV Deutschland GmbH - Celectronic eHealth Div." },
    { "001A59", "Ircona" },
    { "001A5A", "Korea Electric Power Data Network (KDN) Co., Ltd" },
    { "001A5B", "NetCare Service Co., Ltd." },
    { "001A5C", "Euchner GmbH+Co. KG" },
    { "001A5D", "Mobinnova Corp." },
    { "001A5E", "Thincom Technology Co.,Ltd" },
    { "001A5F", "KitWorks.fi Ltd." },
    { "001A60", "Wave Electronics Co.,Ltd." },
    { "001A61", "PacStar Corp." },
    { "001A62", "Data Robotics, Incorporated" },
    { "001A63", "Elster Solutions, LLC," },
    { "001A64", "IBM Corp" },
    { "001A65", "Seluxit" },
    { "001A66", "Commscope" },
    { "001A67", "Infinite QL Sdn Bhd" },
    { "001A68", "Weltec Enterprise Co., Ltd." },
    { "001A69", "Wuhan Yangtze Optical Technology CO.,Ltd." },
    { "001A6A", "Tranzas, Inc." },
    { "001A6B", "Universal Global Scientific Industrial Co., Ltd." },
    { "001A6C", "Cisco Systems, Inc" },
    { "001A6D", "Cisco Systems, Inc" },
    { "001A6E", "Impro Technologies" },
    { "001A6F", "MI.TEL s.r.l." },
    { "001A70", "Cisco-Linksys, LLC" },
    { "001A71", "Diostech Co., Ltd." },
    { "001A72", "Mosart Semiconductor Corp." },
    { "001A73", "Gemtek Technology Co., Ltd." },
    { "001A74", "Procare International Co" },
    { "001A75", "Sony Corporation" },
    { "001A76", "SDT information Technology Co.,LTD." },
    { "001A77", "Commscope" },
    { "001A78", "ubtos" },
    { "001A79", "TELECOMUNICATION TECHNOLOGIES LTD." },
    { "001A7A", "Lismore Instruments Limited" },
    { "001A7B", "Teleco, Inc." },
    { "001A7C", "Hirschmann Multimedia B.V." },
    { "001A7D", "cyber-blue(HK)Ltd" },
    { "001A7E", "LN Srithai Comm Ltd." },
    { "001A7F", "GCI Science & Technology Co.,LTD" },
    { "001A80", "Sony Corporation" },
    { "001A81", "Zelax" },
    { "001A82", "PROBA Building Automation Co.,LTD" },
    { "001A83", "Pegasus Technologies Inc." },
    { "001A84", "V One Multimedia Pte Ltd" },
    { "001A85", "NV Michel Van de Wiele" },
    { "001A86", "New Wave Design & Verification" },
    { "001A87", "Canhold International Limited" },
    { "001A88", "Venergy,Co,Ltd" },
    { "001A89", "Nokia Danmark A/S" },
    { "001A8A", "Samsung Electronics Co.,Ltd" },
    { "001A8B", "CHUNIL ELECTRIC IND., CO." },
    { "001A8C", "Sophos Ltd" },
    { "001A8D", "AVECS Bergen GmbH" },
    { "001A8E", "3Way Networks Ltd" },
    { "001A8F", "Nortel Networks" },
    { "001A90", "Trópico Sistemas e Telecomunicações da Amazônia LTDA. " },
    { "001A91", "FusionDynamic Ltd." },
    { "001A92", "ASUSTek COMPUTER INC." },
    { "001A93", "ERCO Leuchten GmbH" },
    { "001A94", "Votronic GmbH" },
    { "001A95", "Hisense Mobile Communications Technoligy Co.,Ltd." },
    { "001A96", "ECLER S.A." },
    { "001A97", "fitivision technology Inc." },
    { "001A98", "Asotel Communication Limited Taiwan Branch" },
    { "001A99", "Smarty (HZ) Information Electronics Co., Ltd" },
    { "001A9A", "Skyworth Digital Technology(Shenzhen) Co.,Ltd" },
    { "001A9B", "ADEC & Parter AG" },
    { "001A9C", "RightHand Technologies, Inc." },
    { "001A9D", "Skipper Wireless, Inc." },
    { "001A9E", "ICON Digital International Limited" },
    { "001A9F", "A-Link Ltd" },
    { "001AA0", "Dell Inc." },
    { "001AA1", "Cisco Systems, Inc" },
    { "001AA2", "Cisco Systems, Inc" },
    { "001AA3", "DELORME" },
    { "001AA4", "Future University-Hakodate" },
    { "001AA5", "BRN Phoenix" },
    { "001AA6", "Elbit Systems Deutschland GmbH & Co. KG" },
    { "001AA7", "Torian Wireless" },
    { "001AA8", "Mamiya Digital Imaging Co., Ltd." },
    { "001AA9", "FUJIAN STAR-NET COMMUNICATION CO.,LTD" },
    { "001AAA", "Analogic Corp." },
    { "001AAB", "eWings s.r.l." },
    { "001AAC", "Corelatus AB" },
    { "001AAD", "Commscope" },
    { "001AAE", "Savant Systems LLC" },
    { "001AAF", "BLUSENS TECHNOLOGY" },
    { "001AB0", "Signal Networks Pvt. Ltd.," },
    { "001AB1", "Asia Pacific Satellite Industries Co., Ltd." },
    { "001AB2", "Cyber Solutions Inc." },
    { "001AB3", "VISIONITE INC." },
    { "001AB4", "FFEI Ltd." },
    { "001AB5", "Home Network System" },
    { "001AB6", "Texas Instruments" },
    { "001AB7", "Ethos Networks LTD." },
    { "001AB8", "Anseri Corporation" },
    { "001AB9", "PMC" },
    { "001ABA", "Caton Overseas Limited" },
    { "001ABB", "Fontal Technology Incorporation" },
    { "001ABC", "U4EA Technologies Ltd" },
    { "001ABD", "Impatica Inc." },
    { "001ABE", "COMPUTER HI-TECH INC." },
    { "001ABF", "TRUMPF Laser Marking Systems AG" },
    { "001AC0", "JOYBIEN TECHNOLOGIES CO., LTD." },
    { "001AC1", "3Com Ltd" },
    { "001AC2", "YEC Co.,Ltd." },
    { "001AC3", "Scientific-Atlanta, Inc" },
    { "001AC4", "2Wire Inc" },
    { "001AC5", "Keysight Technologies, Inc." },
    { "001AC6", "Micro Control Designs" },
    { "001AC7", "UNIPOINT" },
    { "001AC8", "ISL (Instrumentation Scientifique de Laboratoire)" },
    { "001AC9", "SUZUKEN CO.,LTD" },
    { "001ACA", "Tilera Corporation" },
    { "001ACB", "Autocom Products Ltd" },
    { "001ACC", "Celestial Semiconductor, Ltd" },
    { "001ACD", "Tidel Engineering LP" },
    { "001ACE", "YUPITERU CORPORATION" },
    { "001ACF", "C.T. ELETTRONICA" },
    { "001AD0", "Albis Technologies AG" },
    { "001AD1", "FARGO CO., LTD." },
    { "001AD2", "Eletronica Nitron Ltda" },
    { "001AD3", "Vamp Ltd." },
    { "001AD4", "iPOX Technology Co., Ltd." },
    { "001AD5", "KMC CHAIN INDUSTRIAL CO., LTD." },
    { "001AD6", "JIAGNSU AETNA ELECTRIC CO.,LTD" },
    { "001AD7", "Christie Digital Systems, Inc." },
    { "001AD8", "AlsterAero GmbH" },
    { "001AD9", "International Broadband Electric Communications, Inc." },
    { "001ADA", "Biz-2-Me Inc." },
    { "001ADB", "Commscope" },
    { "001ADC", "Nokia Danmark A/S" },
    { "001ADD", "PePWave Ltd" },
    { "001ADE", "Commscope" },
    { "001ADF", "Interactivetv Pty Limited" },
    { "001AE0", "Mythology Tech Express Inc." },
    { "001AE1", "EDGE ACCESS INC" },
    { "001AE2", "Cisco Systems, Inc" },
    { "001AE3", "Cisco Systems, Inc" },
    { "001AE4", "Medicis Technologies Corporation" },
    { "001AE5", "Mvox Technologies Inc." },
    { "001AE6", "Atlanta Advanced Communications Holdings Limited" },
    { "001AE7", "Aztek Networks, Inc." },
    { "001AE8", "Unify Software and Solutions GmbH & Co. KG" },
    { "001AE9", "Nintendo Co., Ltd." },
    { "001AEA", "Radio Terminal Systems Pty Ltd" },
    { "001AEB", "Allied Telesis K.K." },
    { "001AEC", "Keumbee Electronics Co.,Ltd." },
    { "001AED", "INCOTEC GmbH" },
    { "001AEE", "Shenztech Ltd" },
    { "001AEF", "Loopcomm Technology, Inc." },
    { "001AF0", "Alcatel-Lucent IPD" },
    { "001AF1", "Embedded Artists AB" },
    { "001AF2", "Dynavisions Schweiz AG" },
    { "001AF3", "Samyoung Electronics" },
    { "001AF4", "Handreamnet" },
    { "001AF5", "PENTAONE. CO., LTD." },
    { "001AF6", "Woven Systems, Inc." },
    { "001AF7", "dataschalt e+a GmbH" },
    { "001AF8", "Copley Controls Corporation" },
    { "001AF9", "AeroVIronment (AV Inc)" },
    { "001AFA", "Welch Allyn, Inc." },
    { "001AFB", "Joby Inc." },
    { "001AFC", "ModusLink Corporation" },
    { "001AFD", "EVOLIS" },
    { "001AFE", "SOFACREAL" },
    { "001AFF", "Wizyoung Tech." },
    { "001B00", "Neopost Technologies" },
    { "001B01", "Applied Radio Technologies" },
    { "001B02", "ED Co.Ltd" },
    { "001B03", "Action Technology (SZ) Co., Ltd" },
    { "001B04", "Affinity International S.p.a" },
    { "001B05", "YMC AG" },
    { "001B06", "Ateliers R. LAUMONIER" },
    { "001B07", "Mendocino Software" },
    { "001B08", "Danfoss Drives A/S" },
    { "001B09", "MATRIX COMSEC PRIVATE LIMITED" },
    { "001B0A", "Intelligent Distributed Controls Ltd" },
    { "001B0B", "Phidgets Inc." },
    { "001B0C", "Cisco Systems, Inc" },
    { "001B0D", "Cisco Systems, Inc" },
    { "001B0E", "InoTec GmbH Organisationssysteme" },
    { "001B0F", "Petratec" },
    { "001B10", "ShenZhen Kang Hui Technology Co.,ltd" },
    { "001B11", "D-Link Corporation" },
    { "001B12", "Apprion" },
    { "001B13", "Icron Technologies Corporation" },
    { "001B14", "Carex Lighting Equipment Factory" },
    { "001B15", "Voxtel, Inc." },
    { "001B16", "Celtro Ltd." },
    { "001B17", "Palo Alto Networks" },
    { "001B18", "Tsuken Electric Ind. Co.,Ltd" },
    { "001B19", "IEEE I&M Society TC9" },
    { "001B1A", "e-trees Japan, Inc." },
    { "001B1B", "Siemens AG," },
    { "001B1C", "Coherent" },
    { "001B1D", "Phoenix International Co., Ltd" },
    { "001B1E", "HART Communication Foundation" },
    { "001B1F", "FORCE Technology" },
    { "001B20", "TPine Technology" },
    { "001B21", "Intel Corporate" },
    { "001B22", "Palit Microsystems ( H.K.) Ltd." },
    { "001B23", "SimpleComTools" },
    { "001B24", "Quanta Computer Inc." },
    { "001B25", "Nortel Networks" },
    { "001B26", "RON-Telecom ZAO" },
    { "001B27", "Merlin CSI" },
    { "001B28", "POLYGON, JSC" },
    { "001B29", "Avantis.Co.,Ltd" },
    { "001B2A", "Cisco Systems, Inc" },
    { "001B2B", "Cisco Systems, Inc" },
    { "001B2C", "ATRON electronic GmbH" },
    { "001B2D", "Med-Eng Systems Inc." },
    { "001B2E", "Sinkyo Electron Inc" },
    { "001B2F", "NETGEAR" },
    { "001B30", "Solitech Inc." },
    { "001B31", "Neural Image. Co. Ltd." },
    { "001B32", "QLogic Corporation" },
    { "001B33", "Nokia Danmark A/S" },
    { "001B34", "Focus System Inc." },
    { "001B35", "ChongQing JINOU Science & Technology Development CO.,Ltd" },
    { "001B36", "Tsubata Engineering Co.,Ltd. (Head Office)" },
    { "001B37", "Computec Oy" },
    { "001B38", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "001B39", "Proxicast" },
    { "001B3A", "SIMS Corp." },
    { "001B3B", "Yi-Qing CO., LTD" },
    { "001B3C", "Software Technologies Group,Inc." },
    { "001B3D", "EuroTel Spa" },
    { "001B3E", "Curtis, Inc." },
    { "001B3F", "ProCurve Networking by HP" },
    { "001B40", "Network Automation mxc AB" },
    { "001B41", "General Infinity Co.,Ltd." },
    { "001B42", "Wise & Blue" },
    { "001B43", "Beijing DG Telecommunications equipment Co.,Ltd" },
    { "001B44", "SanDisk Corporation" },
    { "001B45", "ABB AS, Division Automation Products" },
    { "001B46", "Blueone Technology Co.,Ltd" },
    { "001B47", "Futarque A/S" },
    { "001B48", "Shenzhen Lantech Electronics Co., Ltd." },
    { "001B49", "Roberts Radio limited" },
    { "001B4A", "W&W Communications, Inc." },
    { "001B4B", "SANION Co., Ltd." },
    { "001B4C", "Signtech" },
    { "001B4D", "Areca Technology Corporation" },
    { "001B4E", "Navman New Zealand" },
    { "001B4F", "Avaya Inc" },
    { "001B50", "Nizhny Novgorod Factory named after M.Frunze, FSUE (NZiF)" },
    { "001B51", "Vector Technology Corp." },
    { "001B52", "Commscope" },
    { "001B53", "Cisco Systems, Inc" },
    { "001B54", "Cisco Systems, Inc" },
    { "001B55", "Hurco Automation Ltd." },
    { "001B56", "Tehuti Networks Ltd." },
    { "001B57", "SEMINDIA SYSTEMS PRIVATE LIMITED" },
    { "001B58", "ACE CAD Enterprise Co., Ltd." },
    { "001B59", "Sony Corporation" },
    { "001B5A", "Apollo Imaging Technologies, Inc." },
    { "001B5B", "2Wire Inc" },
    { "001B5C", "Azuretec Co., Ltd." },
    { "001B5D", "Vololink Pty Ltd" },
    { "001B5E", "BPL Limited" },
    { "001B5F", "Alien Technology" },
    { "001B60", "NAVIGON AG" },
    { "001B61", "Digital Acoustics, LLC" },
    { "001B62", "JHT Optoelectronics Co.,Ltd." },
    { "001B63", "Apple, Inc." },
    { "001B64", "IsaacLandKorea Co., Ltd," },
    { "001B65", "China Gridcom Co., Ltd" },
    { "001B66", "Sennheiser electronic GmbH & Co. KG" },
    { "001B67", "Cisco Systems Inc" },
    { "001B68", "Modnnet Co., Ltd" },
    { "001B69", "Equaline Corporation" },
    { "001B6A", "Powerwave Technologies Sweden AB" },
    { "001B6B", "Swyx Solutions AG" },
    { "001B6C", "LookX Digital Media BV" },
    { "001B6D", "Midtronics, Inc." },
    { "001B6E", "Keysight Technologies, Inc." },
    { "001B6F", "Teletrak Ltd" },
    { "001B70", "IRI Ubiteq, INC." },
    { "001B71", "Telular Corp." },
    { "001B72", "Sicep s.p.a." },
    { "001B73", "DTL Broadcast Ltd" },
    { "001B74", "MiraLink Corporation" },
    { "001B75", "Hypermedia Systems" },
    { "001B76", "Ripcode, Inc." },
    { "001B77", "Intel Corporate" },
    { "001B78", "Hewlett Packard" },
    { "001B79", "FAIVELEY TRANSPORT" },
    { "001B7A", "Nintendo Co., Ltd." },
    { "001B7B", "The Tintometer Ltd" },
    { "001B7C", "A & R Cambridge" },
    { "001B7D", "CXR Anderson Jacobson" },
    { "001B7E", "Beckmann GmbH" },
    { "001B7F", "TMN Technologies Telecomunicacoes Ltda" },
    { "001B80", "LORD Corporation" },
    { "001B81", "DATAQ Instruments, Inc." },
    { "001B82", "Taiwan Semiconductor Co., Ltd." },
    { "001B83", "Finsoft Ltd" },
    { "001B84", "Scan Engineering Telecom" },
    { "001B85", "MAN Energy Solutions" },
    { "001B86", "Bosch Access Systems GmbH" },
    { "001B87", "Deepsound Tech. Co., Ltd" },
    { "001B88", "Divinet Access Technologies Ltd" },
    { "001B89", "EMZA Visual Sense Ltd." },
    { "001B8A", "2M Electronic A/S" },
    { "001B8B", "NEC Platforms, Ltd." },
    { "001B8C", "JMicron Technology Corp." },
    { "001B8D", "Electronic Computer Systems, Inc." },
    { "001B8E", "Hulu Sweden AB" },
    { "001B8F", "Cisco Systems, Inc" },
    { "001B90", "Cisco Systems, Inc" },
    { "001B91", "EFKON AG" },
    { "001B92", "l-acoustics" },
    { "001B93", "JC Decaux SA DNT" },
    { "001B94", "T.E.M.A. S.p.A." },
    { "001B95", "VIDEO SYSTEMS SRL" },
    { "001B96", "General Sensing" },
    { "001B97", "Violin Technologies" },
    { "001B98", "Samsung Electronics Co.,Ltd" },
    { "001B99", "KS System GmbH" },
    { "001B9A", "Apollo Fire Detectors Ltd" },
    { "001B9B", "Hose-McCann Communications" },
    { "001B9C", "SATEL sp. z o.o." },
    { "001B9D", "Novus Security Sp. z o.o." },
    { "001B9E", "ASKEY COMPUTER CORP" },
    { "001B9F", "Calyptech Pty Ltd" },
    { "001BA0", "Awox" },
    { "001BA1", "Åmic AB" },
    { "001BA2", "IDS Imaging Development Systems GmbH" },
    { "001BA3", "Flexit Group GmbH" },
    { "001BA4", "S.A.E Afikim" },
    { "001BA5", "MyungMin Systems, Inc." },
    { "001BA6", "intotech inc." },
    { "001BA7", "Lorica Solutions" },
    { "001BA8", "UBI&MOBI,.Inc" },
    { "001BA9", "Brother industries, LTD." },
    { "001BAA", "XenICs nv" },
    { "001BAB", "Telchemy, Incorporated" },
    { "001BAC", "Curtiss Wright Controls Embedded Computing" },
    { "001BAD", "iControl Incorporated" },
    { "001BAE", "Micro Control Systems, Inc" },
    { "001BAF", "Nokia Danmark A/S" },
    { "001BB0", "Bharat Electronics Limited" },
    { "001BB1", "Wistron Neweb Corporation" },
    { "001BB2", "Intellect International NV" },
    { "001BB3", "Condalo GmbH" },
    { "001BB4", "Airvod Limited" },
    { "001BB5", "Cherry GmbH" },
    { "001BB6", "Bird Electronic Corp." },
    { "001BB7", "Alta Heights Technology Corp." },
    { "001BB8", "BLUEWAY ELECTRONIC CO;LTD" },
    { "001BB9", "Elitegroup Computer Systems Co.,Ltd." },
    { "001BBA", "Nortel Networks" },
    { "001BBB", "RFTech Co.,Ltd" },
    { "001BBC", "Silver Peak Systems, Inc." },
    { "001BBD", "FMC Kongsberg Subsea AS" },
    { "001BBE", "ICOP Digital" },
    { "001BBF", "Sagemcom Broadband SAS" },
    { "001BC0", "Juniper Networks" },
    { "001BC1", "HOLUX Technology, Inc." },
    { "001BC2", "Integrated Control Technology Limitied" },
    { "001BC3", "Mobisolution Co.,Ltd" },
    { "001BC4", "Ultratec, Inc." },
    { "001BC5", "IEEE Registration Authority" },
    { "001BC6", "Strato Rechenzentrum AG" },
    { "001BC7", "StarVedia Technology Inc." },
    { "001BC8", "MIURA CO.,LTD" },
    { "001BC9", "FSN DISPLAY INC" },
    { "001BCA", "Beijing Run Technology LTD. Company" },
    { "001BCB", "PEMPEK SYSTEMS PTY LTD" },
    { "001BCC", "KINGTEK CCTV ALLIANCE CO., LTD." },
    { "001BCD", "DAVISCOMMS (S) PTE LTD" },
    { "001BCE", "Measurement Devices Ltd" },
    { "001BCF", "Dataupia Corporation" },
    { "001BD0", "IDENTEC SOLUTIONS" },
    { "001BD1", "SOGESTMATIC" },
    { "001BD2", "ULTRA-X ASIA PACIFIC Inc." },
    { "001BD3", "Panasonic Corporation AVC Networks Company" },
    { "001BD4", "Cisco Systems, Inc" },
    { "001BD5", "Cisco Systems, Inc" },
    { "001BD6", "Kelvin Hughes Ltd" },
    { "001BD7", "Cisco SPVTG" },
    { "001BD8", "FLIR Systems Inc" },
    { "001BD9", "Edgewater Wireless Systems Inc" },
    { "001BDA", "UTStarcom Inc" },
    { "001BDB", "Valeo VECS" },
    { "001BDC", "Vencer Co., Ltd." },
    { "001BDD", "Commscope" },
    { "001BDE", "Renkus-Heinz, Inc." },
    { "001BDF", "Iskra Sistemi d.d." },
    { "001BE0", "TELENOT ELECTRONIC GmbH" },
    { "001BE1", "ViaLogy" },
    { "001BE2", "AhnLab,Inc." },
    { "001BE3", "Health Hero Network, Inc." },
    { "001BE4", "TOWNET SRL" },
    { "001BE5", "802automation Limited" },
    { "001BE6", "VR AG" },
    { "001BE7", "Postek Electronics Co., Ltd." },
    { "001BE8", "Ultratronik GmbH" },
    { "001BE9", "Broadcom" },
    { "001BEA", "Nintendo Co., Ltd." },
    { "001BEB", "DMP Electronics INC." },
    { "001BEC", "Netio Technologies Co., Ltd" },
    { "001BED", "Brocade Communications Systems LLC" },
    { "001BEE", "Nokia Danmark A/S" },
    { "001BEF", "Blossoms Digital Technology Co.,Ltd." },
    { "001BF0", "Value Platforms Limited" },
    { "001BF1", "Nanjing SilverNet Software Co., Ltd." },
    { "001BF2", "KWORLD COMPUTER CO., LTD" },
    { "001BF3", "TRANSRADIO SenderSysteme Berlin AG" },
    { "001BF4", "KENWIN INDUSTRIAL(HK) LTD." },
    { "001BF5", "Tellink Sistemas de Telecomunicación S.L." },
    { "001BF6", "CONWISE Technology Corporation Ltd." },
    { "001BF7", "Lund IP Products AB" },
    { "001BF8", "Digitrax Inc." },
    { "001BF9", "Intellitect Water Ltd" },
    { "001BFA", "G.i.N. mbH" },
    { "001BFB", "ALPSALPINE CO,.LTD" },
    { "001BFC", "ASUSTek COMPUTER INC." },
    { "001BFD", "Dignsys Inc." },
    { "001BFE", "Zavio Inc." },
    { "001BFF", "Millennia Media inc." },
    { "001C00", "Entry Point, LLC" },
    { "001C01", "ABB Oy Drives" },
    { "001C02", "Pano Logic" },
    { "001C03", "Betty TV Technology AG" },
    { "001C04", "Airgain, Inc." },
    { "001C05", "Nonin Medical Inc." },
    { "001C06", "Siemens Numerical Control Ltd., Nanjing" },
    { "001C07", "Cwlinux Limited" },
    { "001C08", "Echo360, Inc." },
    { "001C09", "SAE Electronic Co.,Ltd." },
    { "001C0A", "Shenzhen AEE Technology Co.,Ltd." },
    { "001C0B", "SmartAnt Telecom" },
    { "001C0C", "TANITA Corporation" },
    { "001C0D", "G-Technology, Inc." },
    { "001C0E", "Cisco Systems, Inc" },
    { "001C0F", "Cisco Systems, Inc" },
    { "001C10", "Cisco-Linksys, LLC" },
    { "001C11", "Commscope" },
    { "001C12", "Commscope" },
    { "001C13", "OPTSYS TECHNOLOGY CO., LTD." },
    { "001C14", "VMware, Inc." },
    { "001C15", "iPhotonix LLC" },
    { "001C16", "ThyssenKrupp Elevator" },
    { "001C17", "Nortel Networks" },
    { "001C18", "Sicert S.r.L." },
    { "001C19", "secunet Security Networks AG" },
    { "001C1A", "Thomas Instrumentation, Inc" },
    { "001C1B", "Hyperstone GmbH" },
    { "001C1C", "Center Communication Systems GmbH" },
    { "001C1D", "CHENZHOU GOSPELL DIGITAL TECHNOLOGY CO.,LTD" },
    { "001C1E", "emtrion GmbH" },
    { "001C1F", "Quest Retail Technology Pty Ltd" },
    { "001C20", "CLB Benelux" },
    { "001C21", "Nucsafe Inc." },
    { "001C22", "Aeris Elettronica s.r.l." },
    { "001C23", "Dell Inc." },
    { "001C24", "Formosa Wireless Systems Corp." },
    { "001C25", "Hon Hai Precision Ind. Co.,Ltd." },
    { "001C26", "Hon Hai Precision Ind. Co.,Ltd." },
    { "001C27", "Sunell Electronics Co." },
    { "001C28", "Sphairon Technologies GmbH " },
    { "001C29", "CORE DIGITAL ELECTRONICS CO., LTD" },
    { "001C2A", "Envisacor Technologies Inc." },
    { "001C2B", "Hive" },
    { "001C2C", "Synapse" },
    { "001C2D", "FlexRadio Systems" },
    { "001C2E", "HPN Supply Chain" },
    { "001C2F", "Pfister GmbH" },
    { "001C30", "Mode Lighting (UK ) Ltd." },
    { "001C31", "Mobile XP Technology Co., LTD" },
    { "001C32", "Telian Corporation" },
    { "001C33", "Sutron" },
    { "001C34", "HUEY CHIAO INTERNATIONAL CO., LTD." },
    { "001C35", "Nokia Danmark A/S" },
    { "001C36", "iNEWiT NV" },
    { "001C37", "Callpod, Inc." },
    { "001C38", "Bio-Rad Laboratories, Inc." },
    { "001C39", "S Netsystems Inc." },
    { "001C3A", "Element Labs, Inc." },
    { "001C3B", "AmRoad Technology Inc." },
    { "001C3C", "Seon Design Inc." },
    { "001C3D", "WaveStorm" },
    { "001C3E", "ECKey Corporation" },
    { "001C3F", "International Police Technologies, Inc." },
    { "001C40", "VDG-Security bv" },
    { "001C41", "scemtec Transponder Technology GmbH" },
    { "001C42", "Parallels, Inc." },
    { "001C43", "Samsung Electronics Co.,Ltd" },
    { "001C44", "Bosch Security Systems BV" },
    { "001C45", "Chenbro Micom Co., Ltd." },
    { "001C46", "QTUM" },
    { "001C47", "Hangzhou Hollysys Automation Co., Ltd" },
    { "001C48", "WiDeFi, Inc." },
    { "001C49", "Zoltan Technology Inc." },
    { "001C4A", "AVM GmbH" },
    { "001C4B", "Gener8, Inc." },
    { "001C4C", "Petrotest Instruments" },
    { "001C4D", "Aplix IP Holdings Corporation" },
    { "001C4E", "TASA International Limited" },
    { "001C4F", "MACAB AB" },
    { "001C50", "TCL Technoly Electronics (Huizhou) Co., Ltd." },
    { "001C51", "Celeno Communications" },
    { "001C52", "VISIONEE SRL" },
    { "001C53", "Synergy Lighting Controls" },
    { "001C54", "Hillstone Networks Inc" },
    { "001C55", "Shenzhen Kaifa Technology Co." },
    { "001C56", "Pado Systems, Inc." },
    { "001C57", "Cisco Systems, Inc" },
    { "001C58", "Cisco Systems, Inc" },
    { "001C59", "DEVON IT" },
    { "001C5A", "Advanced Relay Corporation" },
    { "001C5B", "Chubb Electronic Security Systems Ltd" },
    { "001C5C", "Integrated Medical Systems, Inc." },
    { "001C5D", "Leica Microsystems" },
    { "001C5E", "ASTON France" },
    { "001C5F", "Winland Electronics, Inc." },
    { "001C60", "CSP Frontier Technologies,Inc." },
    { "001C61", "Galaxy Microsystems LImited" },
    { "001C62", "LG Electronics (Mobile Communications)" },
    { "001C63", "TRUEN" },
    { "001C64", "Landis+Gyr" },
    { "001C65", "JoeScan, Inc." },
    { "001C66", "UCAMP CO.,LTD" },
    { "001C67", "Pumpkin Networks, Inc." },
    { "001C68", "Anhui Sun Create Electronics Co., Ltd" },
    { "001C69", "Packet Vision Ltd" },
    { "001C6A", "Weiss Engineering Ltd." },
    { "001C6B", "COVAX Co. Ltd" },
    { "001C6C", "30805" },
    { "001C6D", "KYOHRITSU ELECTRONIC INDUSTRY CO., LTD." },
    { "001C6E", "Newbury Networks, Inc." },
    { "001C6F", "Emfit Ltd" },
    { "001C70", "NOVACOMM LTDA" },
    { "001C71", "Emergent Electronics" },
    { "001C72", "Mayer & Cie GmbH & Co KG" },
    { "001C73", "Arista Networks" },
    { "001C74", "Syswan Technologies Inc." },
    { "001C75", "Segnet Ltd." },
    { "001C76", "The Wandsworth Group Ltd" },
    { "001C77", "Prodys" },
    { "001C78", "WYPLAY SAS" },
    { "001C79", "Cohesive Financial Technologies LLC" },
    { "001C7A", "Perfectone Netware Company Ltd" },
    { "001C7B", "Castlenet Technology Inc." },
    { "001C7C", "PERQ SYSTEMS CORPORATION" },
    { "001C7D", "Excelpoint Manufacturing Pte Ltd" },
    { "001C7E", "Toshiba" },
    { "001C7F", "Check Point Software Technologies" },
    { "001C80", "New Business Division/Rhea-Information CO., LTD." },
    { "001C81", "NextGen Venturi LTD" },
    { "001C82", "Genew Technologies" },
    { "001C83", "New Level Telecom Co., Ltd." },
    { "001C84", "STL Solution Co.,Ltd." },
    { "001C85", "Eunicorn" },
    { "001C86", "Cranite Systems, Inc." },
    { "001C87", "Uriver Inc." },
    { "001C88", "TRANSYSTEM INC." },
    { "001C89", "Force Communications, Inc." },
    { "001C8A", "Cirrascale Corporation" },
    { "001C8B", "MJ Innovations Ltd." },
    { "001C8C", "DIAL TECHNOLOGY LTD." },
    { "001C8D", "Mesa Imaging" },
    { "001C8E", "Alcatel-Lucent IPD" },
    { "001C8F", "Advanced Electronic Design, Inc." },
    { "001C90", "Empacket Corporation" },
    { "001C91", "Gefen LLC" },
    { "001C92", "Tervela" },
    { "001C93", "ExaDigm Inc" },
    { "001C94", "LI-COR Biosciences" },
    { "001C95", "Opticomm Corporation" },
    { "001C96", "Linkwise Technology Pte Ltd" },
    { "001C97", "Enzytek Technology Inc.," },
    { "001C98", "LUCKY TECHNOLOGY (HK) COMPANY LIMITED" },
    { "001C99", "Shunra Software Ltd." },
    { "001C9A", "Nokia Danmark A/S" },
    { "001C9B", "FEIG ELECTRONIC GmbH" },
    { "001C9C", "Nortel Networks" },
    { "001C9D", "Liecthi AG" },
    { "001C9E", "Dualtech IT AB" },
    { "001C9F", "Razorstream, LLC" },
    { "001CA0", "Production Resource Group, LLC" },
    { "001CA1", "Akamai Technologies Inc." },
    { "001CA2", "ADB Broadband Italia" },
    { "001CA3", "Terra" },
    { "001CA4", "Sony Corporation" },
    { "001CA5", "Zygo Corporation" },
    { "001CA6", "Win4NET" },
    { "001CA7", "International Quartz Limited" },
    { "001CA8", "AirTies Wireless Networks" },
    { "001CA9", "Audiomatica Srl" },
    { "001CAA", "Bellon Pty Ltd" },
    { "001CAB", "Meyer Sound Laboratories, Inc." },
    { "001CAC", "Qniq Technology Corp." },
    { "001CAD", "Wuhan Telecommunication Devices Co.,Ltd" },
    { "001CAE", "WiChorus, Inc." },
    { "001CAF", "Plato Networks Inc." },
    { "001CB0", "Cisco Systems, Inc" },
    { "001CB1", "Cisco Systems, Inc" },
    { "001CB2", "BPT SPA" },
    { "001CB3", "Apple, Inc." },
    { "001CB4", "Iridium Satellite LLC" },
    { "001CB5", "Neihua Network Technology Co.,LTD.(NHN)" },
    { "001CB6", "Duzon CNT Co., Ltd." },
    { "001CB7", "USC DigiArk Corporation" },
    { "001CB8", "CBC Co., Ltd" },
    { "001CB9", "KWANG SUNG ELECTRONICS CO., LTD." },
    { "001CBA", "VerScient, Inc." },
    { "001CBB", "MusicianLink" },
    { "001CBC", "CastGrabber, LLC" },
    { "001CBD", "Ezze Mobile Tech., Inc." },
    { "001CBE", "Nintendo Co., Ltd." },
    { "001CBF", "Intel Corporate" },
    { "001CC0", "Intel Corporate" },
    { "001CC1", "Commscope" },
    { "001CC2", "Part II Research, Inc." },
    { "001CC3", "Commscope" },
    { "001CC4", "Hewlett Packard" },
    { "001CC5", "3Com Ltd" },
    { "001CC6", "ProStor Systems" },
    { "001CC7", "Rembrandt Technologies, LLC d/b/a REMSTREAM" },
    { "001CC8", "INDUSTRONIC Industrie-Electronic GmbH & Co. KG" },
    { "001CC9", "Kaise Electronic Technology Co., Ltd." },
    { "001CCA", "Shanghai Gaozhi Science & Technology Development Co." },
    { "001CCB", "Forth Corporation Public Company Limited" },
    { "001CCC", "BlackBerry RTS" },
    { "001CCD", "Alektrona Corporation" },
    { "001CCE", "By Techdesign" },
    { "001CCF", "LIMETEK" },
    { "001CD0", "Circleone Co.,Ltd." },
    { "001CD1", "Waves Audio LTD" },
    { "001CD2", "King Champion (Hong Kong) Limited" },
    { "001CD3", "ZP Engineering SEL" },
    { "001CD4", "Nokia Danmark A/S" },
    { "001CD5", "ZeeVee, Inc." },
    { "001CD6", "Nokia Danmark A/S" },
    { "001CD7", "Harman/Becker Automotive Systems GmbH" },
    { "001CD8", "BlueAnt Wireless" },
    { "001CD9", "GlobalTop Technology Inc." },
    { "001CDA", "Exegin Technologies Limited" },
    { "001CDB", "CARPOINT CO.,LTD" },
    { "001CDC", "Custom Computer Services, Inc." },
    { "001CDD", "COWBELL ENGINEERING CO., LTD." },
    { "001CDE", "Interactive Multimedia eXchange Inc." },
    { "001CDF", "Belkin International Inc." },
    { "001CE0", "DASAN TPS" },
    { "001CE1", "INDRA SISTEMAS, S.A." },
    { "001CE2", "Attero Tech, LLC." },
    { "001CE3", "Optimedical Systems" },
    { "001CE4", "EleSy JSC" },
    { "001CE5", "MBS Electronic Systems GmbH" },
    { "001CE6", "INNES" },
    { "001CE7", "Rocon PLC Research Centre" },
    { "001CE8", "Cummins Inc" },
    { "001CE9", "Galaxy Technology Limited" },
    { "001CEA", "Scientific-Atlanta, Inc" },
    { "001CEB", "Nortel Networks" },
    { "001CEC", "Mobilesoft (Aust.) Pty Ltd" },
    { "001CED", "ENVIRONNEMENT SA" },
    { "001CEE", "SHARP Corporation" },
    { "001CEF", "Primax Electronics Ltd." },
    { "001CF0", "D-Link Corporation" },
    { "001CF1", "SUPoX Technology Co. , LTD." },
    { "001CF2", "Tenlon Technology Co.,Ltd." },
    { "001CF3", "EVS Broadcast Equipment" },
    { "001CF4", "Media Technology Systems Inc" },
    { "001CF5", "Wiseblue Technology Limited" },
    { "001CF6", "Cisco Systems, Inc" },
    { "001CF7", "AudioScience" },
    { "001CF8", "Parade Technologies, Ltd." },
    { "001CF9", "Cisco Systems, Inc" },
    { "001CFA", "Alarm.com" },
    { "001CFB", "Commscope" },
    { "001CFC", "Sumitomo Electric Industries, Ltd" },
    { "001CFD", "Universal Electronics, Inc." },
    { "001CFE", "Quartics Inc" },
    { "001CFF", "Napera Networks Inc" },
    { "001D00", "Brivo Systems, LLC" },
    { "001D01", "Neptune Digital" },
    { "001D02", "Cybertech Telecom Development" },
    { "001D03", "Design Solutions Inc." },
    { "001D04", "Zipit Wireless, Inc." },
    { "001D05", "Cooper Lighting Solutions" },
    { "001D06", "HM Electronics, Inc." },
    { "001D07", "Shenzhen Sang Fei Consumer Communications Co.,Ltd" },
    { "001D08", "Jiangsu Yinhe Electronics Co.,Ltd." },
    { "001D09", "Dell Inc." },
    { "001D0A", "Davis Instruments, Inc." },
    { "001D0B", "Power Standards Lab" },
    { "001D0C", "MobileCompia" },
    { "001D0D", "Sony Interactive Entertainment Inc." },
    { "001D0E", "Agapha Technology co., Ltd." },
    { "001D0F", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "001D10", "LightHaus Logic, Inc." },
    { "001D11", "Analogue & Micro Ltd" },
    { "001D12", "ROHM CO., LTD." },
    { "001D13", "NextGTV" },
    { "001D14", "SPERADTONE INFORMATION TECHNOLOGY LIMITED" },
    { "001D15", "Shenzhen Dolphin Electronic Co., Ltd" },
    { "001D16", "SFR" },
    { "001D17", "Digital Sky Corporation" },
    { "001D18", "Power Innovation GmbH" },
    { "001D19", "Arcadyan Technology Corporation" },
    { "001D1A", "OvisLink S.A." },
    { "001D1B", "Sangean Electronics Inc." },
    { "001D1C", "Gennet s.a." },
    { "001D1D", "Inter-M Corporation" },
    { "001D1E", "KYUSHU TEN CO.,LTD" },
    { "001D1F", "Siauliu Tauro Televizoriai, JSC" },
    { "001D20", "Comtrend Corporation" },
    { "001D21", "Alcad SL" },
    { "001D22", "Foss Analytical A/S" },
    { "001D23", "SENSUS " },
    { "001D24", "Aclara Power-Line Systems Inc." },
    { "001D25", "Samsung Electronics Co.,Ltd" },
    { "001D26", "Rockridgesound Technology Co." },
    { "001D27", "NAC-INTERCOM" },
    { "001D28", "Sony Corporation" },
    { "001D29", "Doro AB" },
    { "001D2A", "SHENZHEN BUL-TECH CO.,LTD." },
    { "001D2B", "Wuhan Pont Technology CO. , LTD" },
    { "001D2C", "Wavetrend Technologies (Pty) Limited" },
    { "001D2D", "Pylone, Inc." },
    { "001D2E", "Ruckus Wireless" },
    { "001D2F", "QuantumVision Corporation" },
    { "001D30", "YX Wireless S.A." },
    { "001D31", "HIGHPRO INTERNATIONAL R&D CO,.LTD." },
    { "001D32", "Longkay Communication & Technology (Shanghai) Co. Ltd" },
    { "001D33", "Maverick Systems Inc." },
    { "001D34", "SYRIS Technology Corp" },
    { "001D35", "Viconics Electronics Inc." },
    { "001D36", "ELECTRONICS CORPORATION OF INDIA LIMITED" },
    { "001D37", "Thales-Panda Transportation System" },
    { "001D38", "Seagate Technology" },
    { "001D39", "MOOHADIGITAL CO., LTD" },
    { "001D3A", "mh acoustics LLC" },
    { "001D3B", "Nokia Danmark A/S" },
    { "001D3C", "Muscle Corporation" },
    { "001D3D", "Avidyne Corporation" },
    { "001D3E", "SAKA TECHNO SCIENCE CO.,LTD" },
    { "001D3F", "Mitron Pty Ltd" },
    { "001D40", "Intel – GE Care Innovations LLC" },
    { "001D41", "Hardy Instruments" },
    { "001D42", "Nortel Networks" },
    { "001D43", "Shenzhen G-link Digital Technology Co., Ltd." },
    { "001D44", "Krohne" },
    { "001D45", "Cisco Systems, Inc" },
    { "001D46", "Cisco Systems, Inc" },
    { "001D47", "Covote GmbH & Co KG" },
    { "001D48", "Sensor-Technik Wiedemann GmbH" },
    { "001D49", "Innovation Wireless Inc." },
    { "001D4A", "Carestream Health, Inc." },
    { "001D4B", "Grid Connect Inc." },
    { "001D4C", "Alcatel-Lucent" },
    { "001D4D", "Adaptive Recognition Hungary, Inc" },
    { "001D4E", "TCM Mobile LLC" },
    { "001D4F", "Apple, Inc." },
    { "001D50", "SPINETIX SA" },
    { "001D51", "Babcock & Wilcox Power Generation Group, Inc" },
    { "001D52", "Defzone B.V." },
    { "001D53", "S&O Electronics (Malaysia) Sdn. Bhd." },
    { "001D54", "Sunnic Technology & Merchandise INC." },
    { "001D55", "ZANTAZ, Inc" },
    { "001D56", "Kramer Electronics Ltd." },
    { "001D57", "CAETEC Messtechnik" },
    { "001D58", "CQ Inc" },
    { "001D59", "Mitra Energy & Infrastructure" },
    { "001D5A", "2Wire Inc" },
    { "001D5B", "Tecvan Informatica Ltda" },
    { "001D5C", "Tom Communication Industrial Co.,Ltd." },
    { "001D5D", "Control Dynamics Pty. Ltd." },
    { "001D5E", "COMING MEDIA CORP." },
    { "001D5F", "OverSpeed SARL" },
    { "001D60", "ASUSTek COMPUTER INC." },
    { "001D61", "BIJ Corporation" },
    { "001D62", "InPhase Technologies" },
    { "001D63", "Miele & Cie. KG" },
    { "001D64", "Adam Communications Systems Int Ltd" },
    { "001D65", "Microwave Radio Communications" },
    { "001D66", "Hyundai Telecom" },
    { "001D67", "AMEC" },
    { "001D68", "Thomson Telecom Belgium" },
    { "001D69", "Knorr-Bremse IT-Services GmbH" },
    { "001D6A", "Alpha Networks Inc." },
    { "001D6B", "Commscope" },
    { "001D6C", "ClariPhy Communications, Inc." },
    { "001D6D", "Confidant International LLC" },
    { "001D6E", "Nokia Danmark A/S" },
    { "001D6F", "Chainzone Technology Co., Ltd" },
    { "001D70", "Cisco Systems, Inc" },
    { "001D71", "Cisco Systems, Inc" },
    { "001D72", "Wistron Corporation" },
    { "001D73", "BUFFALO.INC" },
    { "001D74", "Tianjin China-Silicon Microelectronics Co., Ltd." },
    { "001D75", "Radioscape PLC" },
    { "001D76", "Eyeheight Ltd." },
    { "001D77", "NSGate" },
    { "001D78", "Invengo Information Technology Co.,Ltd" },
    { "001D79", "SIGNAMAX LLC" },
    { "001D7A", "Wideband Semiconductor, Inc." },
    { "001D7B", "Ice Energy, Inc." },
    { "001D7C", "ABE Elettronica S.p.A." },
    { "001D7D", "GIGA-BYTE TECHNOLOGY CO.,LTD." },
    { "001D7E", "Cisco-Linksys, LLC" },
    { "001D7F", "Tekron International Ltd" },
    { "001D80", "Beijing Huahuan Eletronics Co.,Ltd" },
    { "001D81", "GUANGZHOU GATEWAY ELECTRONICS CO., LTD" },
    { "001D82", "GN Netcom A/S" },
    { "001D83", "Emitech Corporation" },
    { "001D84", "Gateway, Inc." },
    { "001D85", "Call Direct Cellular Solutions" },
    { "001D86", "Shinwa Industries(China) Ltd." },
    { "001D87", "VigTech Labs Sdn Bhd" },
    { "001D88", "Clearwire" },
    { "001D89", "VaultStor Corporation" },
    { "001D8A", "TechTrex Inc" },
    { "001D8B", "ADB Broadband Italia" },
    { "001D8C", "La Crosse Technology LTD" },
    { "001D8D", "Fluke Process Instruments GmbH" },
    { "001D8E", "Alereon, Inc." },
    { "001D8F", "PureWave Networks" },
    { "001D90", "EMCO Flow Systems" },
    { "001D91", "Digitize, Inc" },
    { "001D92", "MICRO-STAR INT'L CO.,LTD." },
    { "001D93", "Modacom" },
    { "001D94", "Climax Technology Co., Ltd" },
    { "001D95", "Flash, Inc." },
    { "001D96", "WatchGuard Video" },
    { "001D97", "Alertus Technologies LLC" },
    { "001D98", "Nokia Danmark A/S" },
    { "001D99", "Cyan Optic, Inc." },
    { "001D9A", "GODEX INTERNATIONAL CO., LTD" },
    { "001D9B", "Hokuyo Automatic Co., Ltd." },
    { "001D9C", "Rockwell Automation" },
    { "001D9D", "ARTJOY INTERNATIONAL LIMITED" },
    { "001D9E", "AXION TECHNOLOGIES" },
    { "001D9F", "MATT R.P.Traczynscy Sp.J." },
    { "001DA0", "Heng Yu Electronic Manufacturing Company Limited" },
    { "001DA1", "Cisco Systems, Inc" },
    { "001DA2", "Cisco Systems, Inc" },
    { "001DA3", "SabiOso" },
    { "001DA4", "Hangzhou System Technology CO., LTD" },
    { "001DA5", "WB Electronics" },
    { "001DA6", "Media Numerics Limited" },
    { "001DA7", "Seamless Internet" },
    { "001DA8", "Takahata Electronics Co.,Ltd" },
    { "001DA9", "Castles Technology, Co., LTD" },
    { "001DAA", "DrayTek Corp." },
    { "001DAB", "SwissQual License AG" },
    { "001DAC", "Gigamon Systems LLC" },
    { "001DAD", "Sinotech Engineering Consultants, Inc. Geotechnical Enginee" },
    { "001DAE", "CHANG TSENG TECHNOLOGY CO., LTD" },
    { "001DAF", "Nortel Networks" },
    { "001DB0", "FuJian HengTong Information Technology Co.,Ltd" },
    { "001DB1", "Crescendo Networks" },
    { "001DB2", "HOKKAIDO ELECTRIC ENGINEERING CO.,LTD." },
    { "001DB3", "HPN Supply Chain" },
    { "001DB4", "KUMHO ENG CO.,LTD" },
    { "001DB5", "Juniper Networks" },
    { "001DB6", "BestComm Networks, Inc." },
    { "001DB7", "Tendril Networks, Inc." },
    { "001DB8", "Intoto Inc." },
    { "001DB9", "Wellspring Wireless" },
    { "001DBA", "Sony Corporation" },
    { "001DBB", "Dynamic System Electronics Corp." },
    { "001DBC", "Nintendo Co., Ltd." },
    { "001DBD", "Versamed Inc." },
    { "001DBE", "Commscope" },
    { "001DBF", "Radiient Technologies, Inc." },
    { "001DC0", "Enphase Energy" },
    { "001DC1", "Audinate Pty L" },
    { "001DC2", "XORTEC OY" },
    { "001DC3", "RIKOR TV, Ltd" },
    { "001DC4", "AIOI Systems Co., Ltd." },
    { "001DC5", "Beijing Jiaxun Feihong Electricial Co., Ltd." },
    { "001DC6", "SNR Inc." },
    { "001DC7", "L-3 Communications Geneva Aerospace" },
    { "001DC8", "Navionics Research Inc., dba SCADAmetrics" },
    { "001DC9", "GainSpan Corp." },
    { "001DCA", "PAV Electronics Limited" },
    { "001DCB", "Exéns Development Oy" },
    { "001DCC", "Ayon Cyber Security, Inc" },
    { "001DCD", "Commscope" },
    { "001DCE", "Commscope" },
    { "001DCF", "Commscope" },
    { "001DD0", "Commscope" },
    { "001DD1", "Commscope" },
    { "001DD2", "Commscope" },
    { "001DD3", "Commscope" },
    { "001DD4", "Commscope" },
    { "001DD5", "Commscope" },
    { "001DD6", "Commscope" },
    { "001DD7", "Algolith" },
    { "001DD8", "Microsoft Corporation" },
    { "001DD9", "Hon Hai Precision Ind. Co.,Ltd." },
    { "001DDA", "Mikroelektronika spol. s r. o." },
    { "001DDB", "C-BEL Corporation" },
    { "001DDC", "HangZhou DeChangLong Tech&Info Co.,Ltd" },
    { "001DDD", "DAT H.K. LIMITED" },
    { "001DDE", "Zhejiang Broadcast&Television Technology Co.,Ltd." },
    { "001DDF", "Sunitec Enterprise Co.,Ltd" },
    { "001DE0", "Intel Corporate" },
    { "001DE1", "Intel Corporate" },
    { "001DE2", "Radionor Communications" },
    { "001DE3", "Intuicom" },
    { "001DE4", "Visioneered Image Systems" },
    { "001DE5", "Cisco Systems, Inc" },
    { "001DE6", "Cisco Systems, Inc" },
    { "001DE7", "Marine Sonic Technology, Ltd." },
    { "001DE8", "Nikko Denki Tsushin Corporation(NDTC)" },
    { "001DE9", "Nokia Danmark A/S" },
    { "001DEA", "Commtest Instruments Ltd" },
    { "001DEB", "DINEC International" },
    { "001DEC", "Marusys" },
    { "001DED", "Grid Net, Inc." },
    { "001DEE", "NEXTVISION SISTEMAS DIGITAIS DE TELEVISÃO LTDA." },
    { "001DEF", "TRIMM, INC." },
    { "001DF0", "Vidient Systems, Inc." },
    { "001DF1", "Intego Systems, Inc." },
    { "001DF2", "Netflix, Inc." },
    { "001DF3", "SBS Science & Technology Co., Ltd" },
    { "001DF4", "Magellan Technology Pty Limited" },
    { "001DF5", "Sunshine Co,LTD" },
    { "001DF6", "Samsung Electronics Co.,Ltd" },
    { "001DF7", "R. STAHL Schaltgeräte GmbH" },
    { "001DF8", "Webpro Vision Technology Corporation" },
    { "001DF9", "Cybiotronics (Far East) Limited" },
    { "001DFA", "Fujian LANDI Commercial Equipment Co.,Ltd" },
    { "001DFB", "NETCLEUS Systems Corporation" },
    { "001DFC", "KSIC" },
    { "001DFD", "Nokia Danmark A/S" },
    { "001DFE", "Palm, Inc" },
    { "001DFF", "Network Critical Solutions Ltd" },
    { "001E00", "Shantou Institute of Ultrasonic Instruments" },
    { "001E01", "Renesas Technology Sales Co., Ltd." },
    { "001E02", "Sougou Keikaku Kougyou Co.,Ltd." },
    { "001E03", "LiComm Co., Ltd." },
    { "001E04", "Hanson Research Corporation" },
    { "001E05", "Xseed Technologies & Computing" },
    { "001E06", "WIBRAIN" },
    { "001E07", "Winy Technology Co., Ltd." },
    { "001E08", "Centec Networks Inc" },
    { "001E09", "ZEFATEK Co.,LTD" },
    { "001E0A", "Syba Tech Limited" },
    { "001E0B", "Hewlett Packard" },
    { "001E0C", "Sherwood Information Partners, Inc." },
    { "001E0D", "Micran Ltd." },
    { "001E0E", "MAXI VIEW HOLDINGS LIMITED" },
    { "001E0F", "Briot International" },
    { "001E10", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "001E11", "ELELUX INTERNATIONAL LTD" },
    { "001E12", "Ecolab" },
    { "001E13", "Cisco Systems, Inc" },
    { "001E14", "Cisco Systems, Inc" },
    { "001E15", "Beech Hill Electronics" },
    { "001E16", "Keytronix" },
    { "001E17", "STN BV" },
    { "001E18", "Radio Activity srl" },
    { "001E19", "GTRI" },
    { "001E1A", "Best Source Taiwan Inc." },
    { "001E1B", "Digital Stream Technology, Inc." },
    { "001E1C", "SWS Australia Pty Limited" },
    { "001E1D", "East Coast Datacom, Inc." },
    { "001E1E", "Honeywell Life Safety" },
    { "001E1F", "Nortel Networks" },
    { "001E20", "Intertain Inc." },
    { "001E21", "Qisda Corporation" },
    { "001E22", "ARVOO Imaging Products BV" },
    { "001E23", "Electronic Educational Devices, Inc" },
    { "001E24", "Zhejiang Bell Technology Co.,ltd" },
    { "001E25", "INTEK DIGITAL" },
    { "001E26", "Digifriends Co. Ltd" },
    { "001E27", "SBN TECH Co.,Ltd." },
    { "001E28", "Lumexis Corporation" },
    { "001E29", "Hypertherm Inc" },
    { "001E2A", "NETGEAR" },
    { "001E2B", "Radio Systems Design, Inc." },
    { "001E2C", "CyVerse Corporation" },
    { "001E2D", "STIM" },
    { "001E2E", "SIRTI S.p.A." },
    { "001E2F", "DiMoto Pty Ltd" },
    { "001E30", "Shireen Inc" },
    { "001E31", "infomark" },
    { "001E32", "Zensys" },
    { "001E33", "INVENTEC CORPORATION" },
    { "001E34", "CryptoMetrics" },
    { "001E35", "Nintendo Co., Ltd." },
    { "001E36", "IPTE" },
    { "001E37", "Universal Global Scientific Industrial Co., Ltd." },
    { "001E38", "Bluecard Software Technology Co., Ltd." },
    { "001E39", "Comsys Communication Ltd." },
    { "001E3A", "Nokia Danmark A/S" },
    { "001E3B", "Nokia Danmark A/S" },
    { "001E3C", "Lyngbox Media AB" },
    { "001E3D", "ALPSALPINE CO,.LTD" },
    { "001E3E", "KMW Inc." },
    { "001E3F", "TrellisWare Technologies, Inc." },
    { "001E40", "Shanghai DareGlobal Technologies Co.,Ltd" },
    { "001E41", "Microwave Communication & Component, Inc." },
    { "001E42", "Teltonika" },
    { "001E43", "AISIN CORPORATION" },
    { "001E44", "SANTEC" },
    { "001E45", "Sony Corporation" },
    { "001E46", "Commscope" },
    { "001E47", "PT. Hariff Daya Tunggal Engineering" },
    { "001E48", "Wi-Links" },
    { "001E49", "Cisco Systems, Inc" },
    { "001E4A", "Cisco Systems, Inc" },
    { "001E4B", "City Theatrical" },
    { "001E4C", "Hon Hai Precision Ind. Co.,Ltd." },
    { "001E4D", "Welkin Sciences, LLC" },
    { "001E4E", "DAKO EDV-Ingenieur- und Systemhaus GmbH" },
    { "001E4F", "Dell Inc." },
    { "001E50", "BATTISTONI RESEARCH" },
    { "001E51", "Converter Industry Srl" },
    { "001E52", "Apple, Inc." },
    { "001E53", "Further Tech Co., LTD" },
    { "001E54", "TOYO ELECTRIC Corporation" },
    { "001E55", "COWON SYSTEMS,Inc." },
    { "001E56", "Bally Wulff Entertainment GmbH" },
    { "001E57", "ALCOMA, spol. s r.o." },
    { "001E58", "D-Link Corporation" },
    { "001E59", "Silicon Turnkey Express, LLC" },
    { "001E5A", "Commscope" },
    { "001E5B", "Unitron Company, Inc." },
    { "001E5C", "RB GeneralEkonomik" },
    { "001E5D", "Holosys d.o.o." },
    { "001E5E", "COmputime Ltd." },
    { "001E5F", "KwikByte, LLC" },
    { "001E60", "Digital Lighting Systems, Inc" },
    { "001E61", "ITEC GmbH" },
    { "001E62", "Siemon" },
    { "001E63", "Vibro-Meter SA" },
    { "001E64", "Intel Corporate" },
    { "001E65", "Intel Corporate" },
    { "001E66", "RESOL Elektronische Regelungen GmbH" },
    { "001E67", "Intel Corporate" },
    { "001E68", "Quanta Computer Inc." },
    { "001E69", "Thomson Inc." },
    { "001E6A", "Beijing Bluexon Technology Co.,Ltd" },
    { "001E6B", "Cisco SPVTG" },
    { "001E6C", "Opaque Systems" },
    { "001E6D", "IT R&D Center" },
    { "001E6E", "Shenzhen First Mile Communications Ltd" },
    { "001E6F", "Magna-Power Electronics, Inc." },
    { "001E70", "Chelton Limited" },
    { "001E71", "MIrcom Group of Companies" },
    { "001E72", "PCS" },
    { "001E73", "zte corporation" },
    { "001E74", "Sagemcom Broadband SAS" },
    { "001E75", "LG Electronics (Mobile Communications)" },
    { "001E76", "Thermo Fisher Scientific" },
    { "001E77", "Air2App" },
    { "001E78", "Owitek Technology Ltd.," },
    { "001E79", "Cisco Systems, Inc" },
    { "001E7A", "Cisco Systems, Inc" },
    { "001E7B", "R.I.CO. S.r.l." },
    { "001E7C", "Taiwick Limited" },
    { "001E7D", "Samsung Electronics Co.,Ltd" },
    { "001E7E", "Nortel Networks" },
    { "001E7F", "CBM of America" },
    { "001E80", "Icotera A/S" },
    { "001E81", "CNB Technology Inc." },
    { "001E82", "SanDisk Corporation" },
    { "001E83", "LAN/MAN Standards Association (LMSC)" },
    { "001E84", "Pika Technologies Inc." },
    { "001E85", "Lagotek Corporation" },
    { "001E86", "MEL Co.,Ltd." },
    { "001E87", "Realease Limited" },
    { "001E88", "ANDOR SYSTEM SUPPORT CO., LTD." },
    { "001E89", "CRFS Limited" },
    { "001E8A", "eCopy, Inc" },
    { "001E8B", "Infra Access Korea Co., Ltd." },
    { "001E8C", "ASUSTek COMPUTER INC." },
    { "001E8D", "Commscope" },
    { "001E8E", "Hunkeler AG" },
    { "001E8F", "CANON INC." },
    { "001E90", "Elitegroup Computer Systems Co.,Ltd." },
    { "001E91", "KIMIN Electronic Co., Ltd." },
    { "001E92", "JEULIN S.A." },
    { "001E93", "CiriTech Systems Inc" },
    { "001E94", "SUPERCOM TECHNOLOGY CORPORATION" },
    { "001E95", "SIGMALINK" },
    { "001E96", "Sepura Limited" },
    { "001E97", "Medium Link System Technology CO., LTD," },
    { "001E98", "GreenLine Communications" },
    { "001E99", "Vantanol Industrial Corporation" },
    { "001E9A", "HAMILTON Bonaduz AG" },
    { "001E9B", "San-Eisha, Ltd." },
    { "001E9C", "Fidustron INC" },
    { "001E9D", "Recall Technologies, Inc." },
    { "001E9E", "ddm hopt + schuler Gmbh + Co. KG" },
    { "001E9F", "Visioneering Systems, Inc." },
    { "001EA0", "XLN-t" },
    { "001EA1", "Brunata a/s" },
    { "001EA2", "Symx Systems, Inc." },
    { "001EA3", "Nokia Danmark A/S" },
    { "001EA4", "Nokia Danmark A/S" },
    { "001EA5", "ROBOTOUS, Inc." },
    { "001EA6", "Best IT World (India) Pvt. Ltd." },
    { "001EA7", "Actiontec Electronics, Inc" },
    { "001EA8", "Datang Mobile Communications Equipment CO.,LTD" },
    { "001EA9", "Nintendo Co., Ltd." },
    { "001EAA", "E-Senza Technologies GmbH" },
    { "001EAB", "TeleWell Oy" },
    { "001EAC", "Armadeus Systems" },
    { "001EAD", "Wingtech Group Limited" },
    { "001EAE", "Continental Automotive Systems Inc." },
    { "001EAF", "Ophir Optronics Ltd" },
    { "001EB0", "ImesD Electronica S.L." },
    { "001EB1", "Cryptsoft Pty Ltd" },
    { "001EB2", "LG Innotek" },
    { "001EB3", "Primex Wireless" },
    { "001EB4", "UNIFAT TECHNOLOGY LTD." },
    { "001EB5", "Ever Sparkle Technologies Ltd" },
    { "001EB6", "TAG Heuer SA" },
    { "001EB7", "TBTech, Co., Ltd." },
    { "001EB8", "Aloys, Inc" },
    { "001EB9", "Sing Fai Technology Limited" },
    { "001EBA", "High Density Devices AS" },
    { "001EBB", "BLUELIGHT TECHNOLOGY INC." },
    { "001EBC", "WINTECH AUTOMATION CO.,LTD." },
    { "001EBD", "Cisco Systems, Inc" },
    { "001EBE", "Cisco Systems, Inc" },
    { "001EBF", "Haas Automation Inc." },
    { "001EC0", "Microchip Technology Inc." },
    { "001EC1", "3COM EUROPE LTD" },
    { "001EC2", "Apple, Inc." },
    { "001EC3", "Kozio, Inc." },
    { "001EC4", "Celio Corp" },
    { "001EC5", "Middle Atlantic Products Inc" },
    { "001EC6", "Leviton Manufacturing Co., Inc" },
    { "001EC7", "2Wire Inc" },
    { "001EC8", "Rapid Mobile (Pty) Ltd" },
    { "001EC9", "Dell Inc." },
    { "001ECA", "Nortel Networks" },
    { "001ECB", "RPC Energoautomatika Ltd" },
    { "001ECC", "CDVI" },
    { "001ECD", "KYLAND Technology Co. LTD" },
    { "001ECE", "BISA Technologies (Hong Kong) Limited" },
    { "001ECF", "PHILIPS ELECTRONICS UK LTD" },
    { "001ED0", "Ingespace" },
    { "001ED1", "Keyprocessor B.V." },
    { "001ED2", "Ray Shine Video Technology Inc" },
    { "001ED3", "Dot Technology Int'l Co., Ltd." },
    { "001ED4", "Doble Engineering" },
    { "001ED5", "Tekon-Automatics" },
    { "001ED6", "Alentec & Orion AB" },
    { "001ED7", "H-Stream Wireless, Inc." },
    { "001ED8", "Digital United Inc." },
    { "001ED9", "Mitsubishi Precision Co.,LTd." },
    { "001EDA", "Wesemann Elektrotechniek B.V." },
    { "001EDB", "Giken Trastem Co., Ltd." },
    { "001EDC", "Sony Corporation" },
    { "001EDD", "WASKO S.A." },
    { "001EDE", "BYD COMPANY LIMITED" },
    { "001EDF", "Master Industrialization Center Kista" },
    { "001EE0", "Urmet SpA" },
    { "001EE1", "Samsung Electronics Co.,Ltd" },
    { "001EE2", "Samsung Electronics Co.,Ltd" },
    { "001EE3", "T&W Electronics (ShenZhen) Co.,Ltd" },
    { "001EE4", "ACS Solutions France" },
    { "001EE5", "Cisco-Linksys, LLC" },
    { "001EE6", "Shenzhen Advanced Video Info-Tech Co., Ltd." },
    { "001EE7", "Epic Systems Inc" },
    { "001EE8", "Mytek" },
    { "001EE9", "Stoneridge Electronics AB" },
    { "001EEA", "Sensor Switch, Inc." },
    { "001EEB", "Talk-A-Phone Co." },
    { "001EEC", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "001EED", "Adventiq Ltd." },
    { "001EEE", "ETL Systems Ltd" },
    { "001EEF", "Cantronic International Limited" },
    { "001EF0", "Gigafin Networks" },
    { "001EF1", "Servimat" },
    { "001EF2", "Micro Motion Inc" },
    { "001EF3", "From2" },
    { "001EF4", "L-3 Communications Display Systems" },
    { "001EF5", "Hitek Automated Inc." },
    { "001EF6", "Cisco Systems, Inc" },
    { "001EF7", "Cisco Systems, Inc" },
    { "001EF8", "Emfinity Inc." },
    { "001EF9", "Pascom Kommunikations systeme GmbH." },
    { "001EFA", "PROTEI Ltd." },
    { "001EFB", "Trio Motion Technology Ltd" },
    { "001EFC", "JSC MASSA-K" },
    { "001EFD", "Microbit 2.0 AB" },
    { "001EFE", "LEVEL s.r.o." },
    { "001EFF", "Mueller-Elektronik GmbH & Co. KG" },
    { "001F00", "Nokia Danmark A/S" },
    { "001F01", "Nokia Danmark A/S" },
    { "001F02", "Pixelmetrix Corporation Pte Ltd" },
    { "001F03", "NUM AG" },
    { "001F04", "Granch Ltd." },
    { "001F05", "iTAS Technology Corp." },
    { "001F06", "Integrated Dispatch Solutions" },
    { "001F07", "AZTEQ Mobile" },
    { "001F08", "RISCO LTD" },
    { "001F09", "Jastec" },
    { "001F0A", "Nortel Networks" },
    { "001F0B", "Federal State Unitary Enterprise Industrial UnionElectropribor" },
    { "001F0C", "Intelligent Digital Services GmbH" },
    { "001F0D", "L3 Communications - Telemetry West" },
    { "001F0E", "Japan Kyastem Co., Ltd" },
    { "001F0F", "Select Engineered Systems" },
    { "001F10", "TOLEDO DO BRASIL INDUSTRIA DE BALANCAS LTDA" },
    { "001F11", "OPENMOKO, INC." },
    { "001F12", "Juniper Networks" },
    { "001F13", "S.& A.S. Ltd." },
    { "001F14", "NexG" },
    { "001F15", "Bioscrypt Inc" },
    { "001F16", "Wistron Corporation" },
    { "001F17", "IDX Company, Ltd." },
    { "001F18", "Hakusan.Mfg.Co,.Ltd" },
    { "001F19", "BEN-RI ELECTRONICA S.A." },
    { "001F1A", "Prominvest" },
    { "001F1B", "RoyalTek Company Ltd." },
    { "001F1C", "KOBISHI ELECTRIC Co.,Ltd." },
    { "001F1D", "Atlas Material Testing Technology LLC" },
    { "001F1E", "Astec Technology Co., Ltd" },
    { "001F1F", "Edimax Technology Co. Ltd." },
    { "001F20", "Logitech Europe SA" },
    { "001F21", "Inner Mongolia Yin An Science & Technology Development Co.,L" },
    { "001F22", "Source Photonics, Inc." },
    { "001F23", "DGS Diagnostics A/S" },
    { "001F24", "DIGITVIEW TECHNOLOGY CO., LTD." },
    { "001F25", "MBS GmbH" },
    { "001F26", "Cisco Systems, Inc" },
    { "001F27", "Cisco Systems, Inc" },
    { "001F28", "HPN Supply Chain" },
    { "001F29", "Hewlett Packard" },
    { "001F2A", "ACCM" },
    { "001F2B", "Orange Logic" },
    { "001F2C", "Starbridge Networks" },
    { "001F2D", "Electro-Optical Imaging, Inc." },
    { "001F2E", "Triangle Research Int'l Pte Ltd" },
    { "001F2F", "Berker GmbH & Co. KG" },
    { "001F30", "Travelping" },
    { "001F31", "Radiocomp" },
    { "001F32", "Nintendo Co., Ltd." },
    { "001F33", "NETGEAR" },
    { "001F34", "Lung Hwa Electronics Co., Ltd." },
    { "001F35", "AIR802 LLC" },
    { "001F36", "Bellwin Information Co. Ltd.," },
    { "001F37", "Genesis I&C" },
    { "001F38", "POSITRON" },
    { "001F39", "Construcciones y Auxiliar de Ferrocarriles, S.A." },
    { "001F3A", "Hon Hai Precision Ind. Co.,Ltd." },
    { "001F3B", "Intel Corporate" },
    { "001F3C", "Intel Corporate" },
    { "001F3D", "Qbit GmbH" },
    { "001F3E", "RP-Technik e.K." },
    { "001F3F", "AVM GmbH" },
    { "001F40", "Speakercraft Inc." },
    { "001F41", "Ruckus Wireless" },
    { "001F42", "Etherstack plc" },
    { "001F43", "ENTES ELEKTRONIK" },
    { "001F44", "GE Transportation Systems" },
    { "001F45", "Enterasys" },
    { "001F46", "Nortel Networks" },
    { "001F47", "MCS Logic Inc." },
    { "001F48", "Mojix Inc." },
    { "001F49", "Manhattan TV Ltd" },
    { "001F4A", "Albentia Systems S.A." },
    { "001F4B", "Lineage Power" },
    { "001F4C", "Roseman Engineering Ltd" },
    { "001F4D", "Segnetics LLC" },
    { "001F4E", "ConMed Linvatec" },
    { "001F4F", "Thinkware Co. Ltd." },
    { "001F50", "Swissdis AG" },
    { "001F51", "HD Communications Corp" },
    { "001F52", "UVT Unternehmensberatung fur Verkehr und Technik GmbH" },
    { "001F53", "GEMAC Chemnitz GmbH" },
    { "001F54", "Lorex Technology Inc." },
    { "001F55", "Honeywell Security (China) Co., Ltd." },
    { "001F56", "DIGITAL FORECAST" },
    { "001F57", "Phonik Innovation Co.,LTD" },
    { "001F58", "EMH Energiemesstechnik GmbH" },
    { "001F59", "Kronback Tracers" },
    { "001F5A", "Beckwith Electric Co." },
    { "001F5B", "Apple, Inc." },
    { "001F5C", "Nokia Danmark A/S" },
    { "001F5D", "Nokia Danmark A/S" },
    { "001F5E", "Dyna Technology Co.,Ltd." },
    { "001F5F", "Blatand GmbH" },
    { "001F60", "COMPASS SYSTEMS CORP." },
    { "001F61", "Talent Communication Networks Inc." },
    { "001F62", "JSC Stilsoft" },
    { "001F63", "JSC Goodwin-Europa" },
    { "001F64", "Beijing Autelan Technology Inc." },
    { "001F65", "KOREA ELECTRIC TERMINAL CO., LTD." },
    { "001F66", "PLANAR LLC" },
    { "001F67", "Hitachi,Ltd." },
    { "001F68", "Martinsson Elektronik AB" },
    { "001F69", "Pingood Technology Co., Ltd." },
    { "001F6A", "PacketFlux Technologies, Inc." },
    { "001F6B", "LG Electronics (Mobile Communications)" },
    { "001F6C", "Cisco Systems, Inc" },
    { "001F6D", "Cisco Systems, Inc" },
    { "001F6E", "Vtech Engineering Corporation" },
    { "001F6F", "Fujian Sunnada Communication Co.,Ltd." },
    { "001F70", "Botik Technologies LTD" },
    { "001F71", "xG Technology, Inc." },
    { "001F72", "QingDao Hiphone Technology Co,.Ltd" },
    { "001F73", "Teraview Technology Co., Ltd." },
    { "001F74", "Eigen Development" },
    { "001F75", "GiBahn Media" },
    { "001F76", "AirLogic Systems Inc." },
    { "001F77", "HEOL DESIGN" },
    { "001F78", "Blue Fox Porini Textile" },
    { "001F79", "Lodam Electronics A/S" },
    { "001F7A", "WiWide Inc." },
    { "001F7B", "TechNexion Ltd." },
    { "001F7C", "Witelcom AS" },
    { "001F7D", "Embedded Wireless GmbH" },
    { "001F7E", "Commscope" },
    { "001F7F", "Phabrix Limited" },
    { "001F80", "Lucas Holding bv" },
    { "001F81", "Accel Semiconductor Corp" },
    { "001F82", "Cal-Comp Electronics & Communications Company Ltd." },
    { "001F83", "Teleplan Technology Services Sdn Bhd" },
    { "001F84", "Gigle Semiconductor" },
    { "001F85", "Apriva ISS, LLC" },
    { "001F86", "digEcor" },
    { "001F87", "Skydigital Inc." },
    { "001F88", "FMS Force Measuring Systems AG" },
    { "001F89", "Signalion GmbH" },
    { "001F8A", "Ellion Digital Inc." },
    { "001F8B", "Cache IQ" },
    { "001F8C", "CCS Inc." },
    { "001F8D", "Ingenieurbuero Stark GmbH und Ko. KG" },
    { "001F8E", "Metris USA Inc." },
    { "001F8F", "Shanghai Bellmann Digital Source Co.,Ltd." },
    { "001F90", "Actiontec Electronics, Inc" },
    { "001F91", "DBS Lodging Technologies, LLC" },
    { "001F92", "Motorola Solutions Inc." },
    { "001F93", "Xiotech Corporation" },
    { "001F94", "Lascar Electronics Ltd" },
    { "001F95", "Sagemcom Broadband SAS" },
    { "001F96", "APROTECH CO.LTD" },
    { "001F97", "BERTANA srl" },
    { "001F98", "DAIICHI-DENTSU LTD." },
    { "001F99", "SERONICS co.ltd" },
    { "001F9A", "Nortel Networks" },
    { "001F9B", "POSBRO" },
    { "001F9C", "Havis Inc." },
    { "001F9D", "Cisco Systems, Inc" },
    { "001F9E", "Cisco Systems, Inc" },
    { "001F9F", "Thomson Telecom Belgium" },
    { "001FA0", "A10 Networks" },
    { "001FA1", "Gtran Inc" },
    { "001FA2", "Datron World Communications, Inc." },
    { "001FA3", "T&W Electronics(Shenzhen)Co.,Ltd." },
    { "001FA4", "SHENZHEN GONGJIN ELECTRONICS CO.,LT" },
    { "001FA5", "Blue-White Industries" },
    { "001FA6", "Stilo srl" },
    { "001FA7", "Sony Interactive Entertainment Inc." },
    { "001FA8", "Smart Energy Instruments Inc." },
    { "001FA9", "Atlanta DTH, Inc." },
    { "001FAA", "Taseon, Inc." },
    { "001FAB", "I.S HIGH TECH.INC" },
    { "001FAC", "Goodmill Systems Ltd" },
    { "001FAD", "Brown Innovations, Inc" },
    { "001FAE", "Blick South Africa (Pty) Ltd" },
    { "001FAF", "NextIO, Inc." },
    { "001FB0", "TimeIPS, Inc." },
    { "001FB1", "Cybertech Inc." },
    { "001FB2", "Sontheim Industrie Elektronik GmbH" },
    { "001FB3", "2Wire Inc" },
    { "001FB4", "SmartShare Systems" },
    { "001FB5", "I/O Interconnect Inc." },
    { "001FB6", "Chi Lin Technology Co., Ltd." },
    { "001FB7", "WiMate Technologies Corp." },
    { "001FB8", "Universal Remote Control, Inc." },
    { "001FB9", "Paltronics" },
    { "001FBA", "Boyoung Tech" },
    { "001FBB", "Xenatech Co.,LTD" },
    { "001FBC", "EVGA Corporation" },
    { "001FBD", "Kyocera Wireless Corp." },
    { "001FBE", "Shenzhen Mopnet Industrial Co.,Ltd" },
    { "001FBF", "Fulhua Microelectronics Corp. Taiwan Branch" },
    { "001FC0", "Control Express Finland Oy" },
    { "001FC1", "Hanlong Technology Co.,LTD" },
    { "001FC2", "Jow Tong Technology Co Ltd" },
    { "001FC3", "SmartSynch, Inc" },
    { "001FC4", "Commscope" },
    { "001FC5", "Nintendo Co., Ltd." },
    { "001FC6", "ASUSTek COMPUTER INC." },
    { "001FC7", "Casio Hitachi Mobile Communications Co., Ltd." },
    { "001FC8", "Up-Today Industrial Co., Ltd." },
    { "001FC9", "Cisco Systems, Inc" },
    { "001FCA", "Cisco Systems, Inc" },
    { "001FCB", "NIW Solutions" },
    { "001FCC", "Samsung Electronics Co.,Ltd" },
    { "001FCD", "Samsung Electronics Co.,Ltd" },
    { "001FCE", "QTECH LLC" },
    { "001FCF", "MSI Technology GmbH" },
    { "001FD0", "GIGA-BYTE TECHNOLOGY CO.,LTD." },
    { "001FD1", "OPTEX CO.,LTD." },
    { "001FD2", "COMMTECH TECHNOLOGY MACAO COMMERCIAL OFFSHORE LTD." },
    { "001FD3", "RIVA Networks Inc." },
    { "001FD4", "4IPNET, INC." },
    { "001FD5", "MICRORISC s.r.o." },
    { "001FD6", "Shenzhen Allywll" },
    { "001FD7", "TELERAD SA" },
    { "001FD8", "A-TRUST COMPUTER CORPORATION" },
    { "001FD9", "RSD Communications Ltd" },
    { "001FDA", "Nortel Networks" },
    { "001FDB", "Network Supply Corp.," },
    { "001FDC", "Mobile Safe Track Ltd" },
    { "001FDD", "GDI LLC" },
    { "001FDE", "Nokia Danmark A/S" },
    { "001FDF", "Nokia Danmark A/S" },
    { "001FE0", "EdgeVelocity Corp" },
    { "001FE1", "Hon Hai Precision Ind. Co.,Ltd." },
    { "001FE2", "Hon Hai Precision Ind. Co.,Ltd." },
    { "001FE3", "LG Electronics (Mobile Communications)" },
    { "001FE4", "Sony Corporation" },
    { "001FE5", "In-Circuit GmbH" },
    { "001FE6", "Alphion Corporation" },
    { "001FE7", "Simet" },
    { "001FE8", "KURUSUGAWA Electronics Industry Inc,." },
    { "001FE9", "Printrex, Inc." },
    { "001FEA", "Applied Media Technologies Corporation" },
    { "001FEB", "Trio Datacom Pty Ltd" },
    { "001FEC", "Synapse Électronique" },
    { "001FED", "Tecan Systems Inc." },
    { "001FEE", "ubisys technologies GmbH" },
    { "001FEF", "SHINSEI INDUSTRIES CO.,LTD" },
    { "001FF0", "Audio Partnership" },
    { "001FF1", "Paradox Hellas S.A." },
    { "001FF2", "VIA Technologies, Inc." },
    { "001FF3", "Apple, Inc." },
    { "001FF4", "Power Monitors, Inc." },
    { "001FF5", "Kongsberg Defence & Aerospace" },
    { "001FF6", "PS Audio International" },
    { "001FF7", "Nakajima All Precision Co., Ltd." },
    { "001FF8", "Siemens AG, Sector Industry, Drive Technologies, Motion Control Systems" },
    { "001FF9", "Advanced Knowledge Associates" },
    { "001FFA", "Coretree, Co, Ltd" },
    { "001FFB", "Green Packet Bhd" },
    { "001FFC", "Riccius+Sohn GmbH" },
    { "001FFD", "Indigo Mobile Technologies Corp." },
    { "001FFE", "HPN Supply Chain" },
    { "001FFF", "Respironics, Inc." },
    { "002000", "LEXMARK INTERNATIONAL, INC." },
    { "002001", "DSP SOLUTIONS, INC." },
    { "002002", "SERITECH ENTERPRISE CO., LTD." },
    { "002003", "PIXEL POWER LTD." },
    { "002004", "YAMATAKE-HONEYWELL CO., LTD." },
    { "002005", "SIMPLE TECHNOLOGY" },
    { "002006", "GARRETT COMMUNICATIONS, INC." },
    { "002007", "SFA, INC." },
    { "002008", "CABLE & COMPUTER TECHNOLOGY" },
    { "002009", "PACKARD BELL ELEC., INC." },
    { "00200A", "SOURCE-COMM CORP." },
    { "00200B", "OCTAGON SYSTEMS CORP." },
    { "00200C", "ADASTRA SYSTEMS CORP." },
    { "00200D", "CARL ZEISS" },
    { "00200E", "NSSLGlobal Technologies AS" },
    { "00200F", "EBRAINS Inc" },
    { "002010", "JEOL SYSTEM TECHNOLOGY CO. LTD" },
    { "002011", "CANOPUS CO., LTD." },
    { "002012", "CAMTRONICS MEDICAL SYSTEMS" },
    { "002013", "DIVERSIFIED TECHNOLOGY, INC." },
    { "002014", "GLOBAL VIEW CO., LTD." },
    { "002015", "ACTIS COMPUTER SA" },
    { "002016", "SHOWA ELECTRIC WIRE & CABLE CO" },
    { "002017", "ORBOTECH" },
    { "002018", "CIS TECHNOLOGY INC." },
    { "002019", "OHLER GMBH" },
    { "00201A", "MRV Communications, Inc." },
    { "00201B", "NORTHERN TELECOM/NETWORK" },
    { "00201C", "EXCEL, INC." },
    { "00201D", "KATANA PRODUCTS" },
    { "00201E", "NETQUEST CORPORATION" },
    { "00201F", "BEST POWER TECHNOLOGY, INC." },
    { "002020", "MEGATRON COMPUTER INDUSTRIES PTY, LTD." },
    { "002021", "ALGORITHMS SOFTWARE PVT. LTD." },
    { "002022", "NMS Communications" },
    { "002023", "T.C. TECHNOLOGIES PTY. LTD" },
    { "002024", "PACIFIC COMMUNICATION SCIENCES" },
    { "002025", "CONTROL TECHNOLOGY, INC." },
    { "002026", "AMKLY SYSTEMS, INC." },
    { "002027", "MING FORTUNE INDUSTRY CO., LTD" },
    { "002028", "WEST EGG SYSTEMS, INC." },
    { "002029", "TELEPROCESSING PRODUCTS, INC." },
    { "00202A", "N.V. DZINE" },
    { "00202B", "ADVANCED TELECOMMUNICATIONS MODULES, LTD." },
    { "00202C", "WELLTRONIX CO., LTD." },
    { "00202D", "TAIYO CORPORATION" },
    { "00202E", "DAYSTAR DIGITAL" },
    { "00202F", "ZETA COMMUNICATIONS, LTD." },
    { "002030", "ANALOG & DIGITAL SYSTEMS" },
    { "002031", "Tattile SRL " },
    { "002032", "ALCATEL TAISEL" },
    { "002033", "SYNAPSE TECHNOLOGIES, INC." },
    { "002034", "ROTEC INDUSTRIEAUTOMATION GMBH" },
    { "002035", "IBM Corp" },
    { "002036", "BMC SOFTWARE" },
    { "002037", "Seagate Technology" },
    { "002038", "VME MICROSYSTEMS INTERNATIONAL CORPORATION" },
    { "002039", "SCINETS" },
    { "00203A", "DIGITAL BI0METRICS INC." },
    { "00203B", "WISDM LTD." },
    { "00203C", "EUROTIME AB" },
    { "00203D", "Honeywell Environmental & Combustion Controls" },
    { "00203E", "LogiCan Technologies, Inc." },
    { "00203F", "JUKI CORPORATION" },
    { "002040", "Commscope" },
    { "002041", "DATA NET" },
    { "002042", "DATAMETRICS CORP." },
    { "002043", "NEURON COMPANY LIMITED" },
    { "002044", "GENITECH PTY LTD" },
    { "002045", "ION Networks, Inc." },
    { "002046", "CIPRICO, INC." },
    { "002047", "STEINBRECHER CORP." },
    { "002048", "Marconi Communications" },
    { "002049", "COMTRON, INC." },
    { "00204A", "PRONET GMBH" },
    { "00204B", "AUTOCOMPUTER CO., LTD." },
    { "00204C", "MITRON COMPUTER PTE LTD." },
    { "00204D", "INOVIS GMBH" },
    { "00204E", "NETWORK SECURITY SYSTEMS, INC." },
    { "00204F", "DEUTSCHE AEROSPACE AG" },
    { "002050", "KOREA COMPUTER INC." },
    { "002051", "Verilink Corporation" },
    { "002052", "RAGULA SYSTEMS" },
    { "002053", "HUNTSVILLE MICROSYSTEMS, INC." },
    { "002054", "Sycamore Networks" },
    { "002055", "ALTECH CO., LTD." },
    { "002056", "NEOPRODUCTS" },
    { "002057", "TITZE DATENTECHNIK GmbH" },
    { "002058", "ALLIED SIGNAL INC." },
    { "002059", "MIRO COMPUTER PRODUCTS AG" },
    { "00205A", "COMPUTER IDENTICS" },
    { "00205B", "Kentrox, LLC" },
    { "00205C", "InterNet Systems of Florida, Inc." },
    { "00205D", "NANOMATIC OY" },
    { "00205E", "CASTLE ROCK, INC." },
    { "00205F", "GAMMADATA COMPUTER GMBH" },
    { "002060", "ALCATEL ITALIA S.p.A." },
    { "002061", "GarrettCom, Inc." },
    { "002062", "SCORPION LOGIC, LTD." },
    { "002063", "WIPRO INFOTECH LTD." },
    { "002064", "PROTEC MICROSYSTEMS, INC." },
    { "002065", "SUPERNET NETWORKING INC." },
    { "002066", "GENERAL MAGIC, INC." },
    { "002067", "Private" },
    { "002068", "ISDYNE" },
    { "002069", "ISDN SYSTEMS CORPORATION" },
    { "00206A", "OSAKA COMPUTER CORP." },
    { "00206B", "KONICA MINOLTA HOLDINGS, INC." },
    { "00206C", "EVERGREEN TECHNOLOGY CORP." },
    { "00206D", "DATA RACE, INC." },
    { "00206E", "XACT, INC." },
    { "00206F", "FLOWPOINT CORPORATION" },
    { "002070", "HYNET, LTD." },
    { "002071", "IBR GMBH" },
    { "002072", "WORKLINK INNOVATIONS" },
    { "002073", "FUSION SYSTEMS CORPORATION" },
    { "002074", "SUNGWOON SYSTEMS" },
    { "002075", "MOTOROLA COMMUNICATION ISRAEL" },
    { "002076", "REUDO CORPORATION" },
    { "002077", "KARDIOS SYSTEMS CORP." },
    { "002078", "RUNTOP, INC." },
    { "002079", "MIKRON GMBH" },
    { "00207A", "WiSE Communications, Inc." },
    { "00207B", "Intel Corporation" },
    { "00207C", "AUTEC GMBH" },
    { "00207D", "ADVANCED COMPUTER APPLICATIONS" },
    { "00207E", "FINECOM CO., LTD." },
    { "00207F", "KYOEI SANGYO CO., LTD." },
    { "002080", "SYNERGY (UK) LTD." },
    { "002081", "TITAN ELECTRONICS" },
    { "002082", "ONEAC CORPORATION" },
    { "002083", "PRESTICOM INCORPORATED" },
    { "002084", "OCE PRINTING SYSTEMS, GMBH" },
    { "002085", "Eaton Corporation" },
    { "002086", "MICROTECH ELECTRONICS LIMITED" },
    { "002087", "MEMOTEC, INC." },
    { "002088", "GLOBAL VILLAGE COMMUNICATION" },
    { "002089", "T3PLUS NETWORKING, INC." },
    { "00208A", "SONIX COMMUNICATIONS, LTD." },
    { "00208B", "LAPIS TECHNOLOGIES, INC." },
    { "00208C", "GALAXY NETWORKS, INC." },
    { "00208D", "CMD TECHNOLOGY" },
    { "00208E", "CHEVIN SOFTWARE ENG. LTD." },
    { "00208F", "ECI Telecom Ltd." },
    { "002090", "ADVANCED COMPRESSION TECHNOLOGY, INC." },
    { "002091", "J125, NATIONAL SECURITY AGENCY" },
    { "002092", "CHESS ENGINEERING B.V." },
    { "002093", "LANDINGS TECHNOLOGY CORP." },
    { "002094", "CUBIX CORPORATION" },
    { "002095", "RIVA ELECTRONICS" },
    { "002096", "Invensys" },
    { "002097", "APPLIED SIGNAL TECHNOLOGY" },
    { "002098", "HECTRONIC AB" },
    { "002099", "BON ELECTRIC CO., LTD." },
    { "00209A", "THE 3DO COMPANY" },
    { "00209B", "ERSAT ELECTRONIC GMBH" },
    { "00209C", "PRIMARY ACCESS CORP." },
    { "00209D", "LIPPERT AUTOMATIONSTECHNIK" },
    { "00209E", "BROWN'S OPERATING SYSTEM SERVICES, LTD." },
    { "00209F", "MERCURY COMPUTER SYSTEMS, INC." },
    { "0020A0", "OA LABORATORY CO., LTD." },
    { "0020A1", "DOVATRON" },
    { "0020A2", "GALCOM NETWORKING LTD." },
    { "0020A3", "Harmonic, Inc" },
    { "0020A4", "MULTIPOINT NETWORKS" },
    { "0020A5", "API ENGINEERING" },
    { "0020A6", "Proxim Wireless" },
    { "0020A7", "PAIRGAIN TECHNOLOGIES, INC." },
    { "0020A8", "SAST TECHNOLOGY CORP." },
    { "0020A9", "WHITE HORSE INDUSTRIAL" },
    { "0020AA", "Ericsson Television Limited" },
    { "0020AB", "MICRO INDUSTRIES CORP." },
    { "0020AC", "INTERFLEX DATENSYSTEME GMBH" },
    { "0020AD", "LINQ SYSTEMS" },
    { "0020AE", "ORNET DATA COMMUNICATION TECH." },
    { "0020AF", "3COM" },
    { "0020B0", "GATEWAY DEVICES, INC." },
    { "0020B1", "COMTECH RESEARCH INC." },
    { "0020B2", "GKD Gesellschaft Fur Kommunikation Und Datentechnik" },
    { "0020B3", "Tattile SRL " },
    { "0020B4", "TERMA ELEKTRONIK AS" },
    { "0020B5", "YASKAWA ELECTRIC CORPORATION" },
    { "0020B6", "AGILE NETWORKS, INC." },
    { "0020B7", "NAMAQUA COMPUTERWARE" },
    { "0020B8", "PRIME OPTION, INC." },
    { "0020B9", "METRICOM, INC." },
    { "0020BA", "CENTER FOR HIGH PERFORMANCE" },
    { "0020BB", "ZAX CORPORATION" },
    { "0020BC", "Long Reach Networks Pty Ltd" },
    { "0020BD", "NIOBRARA R & D CORPORATION" },
    { "0020BE", "LAN ACCESS CORP." },
    { "0020BF", "AEHR TEST SYSTEMS" },
    { "0020C0", "PULSE ELECTRONICS, INC." },
    { "0020C1", "SAXA, Inc." },
    { "0020C2", "TEXAS MEMORY SYSTEMS, INC." },
    { "0020C3", "COUNTER SOLUTIONS LTD." },
    { "0020C4", "INET,INC." },
    { "0020C5", "EAGLE TECHNOLOGY" },
    { "0020C6", "NECTEC" },
    { "0020C7", "AKAI Professional M.I. Corp." },
    { "0020C8", "LARSCOM INCORPORATED" },
    { "0020C9", "VICTRON BV" },
    { "0020CA", "DIGITAL OCEAN" },
    { "0020CB", "PRETEC ELECTRONICS CORP." },
    { "0020CC", "DIGITAL SERVICES, LTD." },
    { "0020CD", "HYBRID NETWORKS, INC." },
    { "0020CE", "LOGICAL DESIGN GROUP, INC." },
    { "0020CF", "TEST & MEASUREMENT SYSTEMS INC" },
    { "0020D0", "VERSALYNX CORPORATION" },
    { "0020D1", "MICROCOMPUTER SYSTEMS (M) SDN." },
    { "0020D2", "RAD DATA COMMUNICATIONS, LTD." },
    { "0020D3", "OST (OUEST STANDARD TELEMATIQU" },
    { "0020D4", "Cabletron Systems, Inc." },
    { "0020D5", "VIPA GMBH" },
    { "0020D6", "Breezecom, Ltd." },
    { "0020D7", "JAPAN MINICOMPUTER SYSTEMS CO., Ltd." },
    { "0020D8", "Nortel Networks" },
    { "0020D9", "PANASONIC TECHNOLOGIES, INC./MIECO-US" },
    { "0020DA", "Alcatel-Lucent Enterprise" },
    { "0020DB", "XNET TECHNOLOGY, INC." },
    { "0020DC", "DENSITRON TAIWAN LTD." },
    { "0020DD", "Cybertec Pty Ltd" },
    { "0020DE", "JAPAN DIGITAL LABORAT'Y CO.LTD" },
    { "0020DF", "KYOSAN ELECTRIC MFG. CO., LTD." },
    { "0020E0", "Actiontec Electronics, Inc" },
    { "0020E1", "ALAMAR ELECTRONICS" },
    { "0020E2", "INFORMATION RESOURCE ENGINEERING" },
    { "0020E3", "MCD KENCOM CORPORATION" },
    { "0020E4", "HSING TECH ENTERPRISE CO., LTD" },
    { "0020E5", "APEX DATA, INC." },
    { "0020E6", "LIDKOPING MACHINE TOOLS AB" },
    { "0020E7", "B&W NUCLEAR SERVICE COMPANY" },
    { "0020E8", "DATATREK CORPORATION" },
    { "0020E9", "DANTEL" },
    { "0020EA", "EFFICIENT NETWORKS, INC." },
    { "0020EB", "CINCINNATI MICROWAVE, INC." },
    { "0020EC", "TECHWARE SYSTEMS CORP." },
    { "0020ED", "GIGA-BYTE TECHNOLOGY CO., LTD." },
    { "0020EE", "GTECH CORPORATION" },
    { "0020EF", "USC CORPORATION" },
    { "0020F0", "UNIVERSAL MICROELECTRONICS CO." },
    { "0020F1", "ALTOS INDIA LIMITED" },
    { "0020F2", "Oracle Corporation " },
    { "0020F3", "RAYNET CORPORATION" },
    { "0020F4", "SPECTRIX CORPORATION" },
    { "0020F5", "PANDATEL AG" },
    { "0020F6", "NET TEK AND KARLNET, INC." },
    { "0020F7", "CYBERDATA CORPORATION" },
    { "0020F8", "CARRERA COMPUTERS, INC." },
    { "0020F9", "PARALINK NETWORKS, INC." },
    { "0020FA", "GDE SYSTEMS, INC." },
    { "0020FB", "OCTEL COMMUNICATIONS CORP." },
    { "0020FC", "Matrox Central Services Inc" },
    { "0020FD", "ITV TECHNOLOGIES, INC." },
    { "0020FE", "TOPWARE INC. / GRAND COMPUTER" },
    { "0020FF", "SYMMETRICAL TECHNOLOGIES" },
    { "002100", "Gemtek Technology Co., Ltd." },
    { "002101", "Aplicaciones Electronicas Quasar (AEQ)" },
    { "002102", "UpdateLogic Inc." },
    { "002103", "GHI Electronics, LLC" },
    { "002104", "Gigaset Communications GmbH" },
    { "002105", "Alcatel-Lucent IPD" },
    { "002106", "RIM Testing Services" },
    { "002107", "Seowonintech Co Ltd." },
    { "002108", "Nokia Danmark A/S" },
    { "002109", "Nokia Danmark A/S" },
    { "00210A", "byd:sign Corporation" },
    { "00210B", "GEMINI TRAZE RFID PVT. LTD." },
    { "00210C", "Cymtec Systems, Inc." },
    { "00210D", "SAMSIN INNOTEC" },
    { "00210E", "Gilbarco Inc. " },
    { "00210F", "Cernium Corp" },
    { "002110", "Clearbox Systems" },
    { "002111", "Uniphone Inc." },
    { "002112", "WISCOM SYSTEM CO.,LTD" },
    { "002113", "Padtec S/A" },
    { "002114", "Hylab Technology Inc." },
    { "002115", "PHYWE Systeme GmbH & Co. KG" },
    { "002116", "Transcon Electronic Systems, spol. s r. o." },
    { "002117", "Tellord" },
    { "002118", "Athena Tech, Inc." },
    { "002119", "SAMSUNG ELECTRO MECHANICS CO., LTD." },
    { "00211A", "LInTech Corporation" },
    { "00211B", "Cisco Systems, Inc" },
    { "00211C", "Cisco Systems, Inc" },
    { "00211D", "Dataline AB" },
    { "00211E", "Commscope" },
    { "00211F", "SHINSUNG DELTATECH CO.,LTD." },
    { "002120", "Sequel Technologies" },
    { "002121", "VRmagic GmbH" },
    { "002122", "Chip-pro Ltd." },
    { "002123", "Aerosat Avionics" },
    { "002124", "Optos Plc" },
    { "002125", "KUK JE TONG SHIN Co.,LTD" },
    { "002126", "Shenzhen Torch Equipment Co., Ltd." },
    { "002127", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "002128", "Oracle Corporation" },
    { "002129", "Cisco-Linksys, LLC" },
    { "00212A", "Audiovox Corporation" },
    { "00212B", "MSA Auer" },
    { "00212C", "SemIndia System Private Limited" },
    { "00212D", "SCIMOLEX CORPORATION" },
    { "00212E", "dresden-elektronik" },
    { "00212F", "Phoebe Micro Inc." },
    { "002130", "Keico Hightech Inc." },
    { "002131", "Blynke Inc." },
    { "002132", "Masterclock, Inc." },
    { "002133", "Building B, Inc" },
    { "002134", "Brandywine Communications" },
    { "002135", "ALCATEL-LUCENT" },
    { "002136", "Commscope" },
    { "002137", "Bay Controls, LLC" },
    { "002138", "Cepheid" },
    { "002139", "Escherlogic Inc." },
    { "00213A", "Winchester Systems Inc." },
    { "00213B", "Berkshire Products, Inc" },
    { "00213C", "AliphCom" },
    { "00213D", "Cermetek Microelectronics, Inc." },
    { "00213E", "TomTom International BV" },
    { "00213F", "A-Team Technology Ltd." },
    { "002140", "EN Technologies Inc." },
    { "002141", "RADLIVE" },
    { "002142", "Advanced Control Systems doo" },
    { "002143", "Commscope" },
    { "002144", "SS Telecoms" },
    { "002145", "Semptian Technologies Ltd." },
    { "002146", "Sanmina-SCI" },
    { "002147", "Nintendo Co., Ltd." },
    { "002148", "Kaco Solar Korea" },
    { "002149", "China Daheng Group ,Inc." },
    { "00214A", "Pixel Velocity, Inc" },
    { "00214B", "Shenzhen HAMP Science & Technology Co.,Ltd" },
    { "00214C", "Samsung Electronics Co.,Ltd" },
    { "00214D", "Guangzhou Skytone Transmission Technology Com. Ltd." },
    { "00214E", "GS Yuasa Power Supply Ltd." },
    { "00214F", "ALPSALPINE CO,.LTD" },
    { "002150", "EYEVIEW ELECTRONICS" },
    { "002151", "Millinet Co., Ltd." },
    { "002152", "General Satellite Research & Development Limited" },
    { "002153", "SeaMicro Inc." },
    { "002154", "D-TACQ Solutions Ltd" },
    { "002155", "Cisco Systems, Inc" },
    { "002156", "Cisco Systems, Inc" },
    { "002157", "National Datacast, Inc." },
    { "002158", "Style Flying Technology Co." },
    { "002159", "Juniper Networks" },
    { "00215A", "Hewlett Packard" },
    { "00215B", "SenseAnywhere" },
    { "00215C", "Intel Corporate" },
    { "00215D", "Intel Corporate" },
    { "00215E", "IBM Corp" },
    { "00215F", "IHSE GmbH" },
    { "002160", "Hidea Solutions Co. Ltd." },
    { "002161", "Yournet Inc." },
    { "002162", "Nortel Networks" },
    { "002163", "ASKEY COMPUTER CORP" },
    { "002164", "Special Design Bureau for Seismic Instrumentation" },
    { "002165", "Presstek Inc." },
    { "002166", "NovAtel Inc." },
    { "002167", "HWA JIN T&I Corp." },
    { "002168", "iVeia, LLC" },
    { "002169", "Prologix, LLC." },
    { "00216A", "Intel Corporate" },
    { "00216B", "Intel Corporate" },
    { "00216C", "ODVA" },
    { "00216D", "Soltech Co., Ltd." },
    { "00216E", "Function ATI (Huizhou) Telecommunications Co., Ltd." },
    { "00216F", "SymCom, Inc." },
    { "002170", "Dell Inc." },
    { "002171", "Wesung TNC Co., Ltd." },
    { "002172", "Seoultek Valley" },
    { "002173", "Ion Torrent Systems, Inc." },
    { "002174", "AvaLAN Wireless" },
    { "002175", "Pacific Satellite International Ltd." },
    { "002176", "YMax Telecom Ltd." },
    { "002177", "W. L. Gore & Associates" },
    { "002178", "Matuschek Messtechnik GmbH" },
    { "002179", "IOGEAR, Inc." },
    { "00217A", "Sejin Electron, Inc." },
    { "00217B", "Bastec AB" },
    { "00217C", "2Wire Inc" },
    { "00217D", "PYXIS S.R.L." },
    { "00217E", "Telit Communication s.p.a" },
    { "00217F", "Intraco Technology Pte Ltd" },
    { "002180", "Commscope" },
    { "002181", "Si2 Microsystems Limited" },
    { "002182", "SandLinks Systems, Ltd." },
    { "002183", "ANDRITZ HYDRO GmbH" },
    { "002184", "POWERSOFT SRL" },
    { "002185", "MICRO-STAR INT'L CO.,LTD." },
    { "002186", "Universal Global Scientific Industrial Co., Ltd." },
    { "002187", "Imacs GmbH" },
    { "002188", "EMC Corporation" },
    { "002189", "AppTech, Inc." },
    { "00218A", "Electronic Design and Manufacturing Company" },
    { "00218B", "Wescon Technology, Inc." },
    { "00218C", "TopControl GMBH" },
    { "00218D", "AP Router Ind. Eletronica LTDA" },
    { "00218E", "MEKICS CO., LTD." },
    { "00218F", "Avantgarde Acoustic Lautsprechersysteme GmbH" },
    { "002190", "Goliath Solutions" },
    { "002191", "D-Link Corporation" },
    { "002192", "Baoding Galaxy Electronic Technology Co.,Ltd" },
    { "002193", "Videofon MV" },
    { "002194", "Ping Communication" },
    { "002195", "GWD Media Limited" },
    { "002196", "Telsey S.p.A." },
    { "002197", "Elitegroup Computer Systems Co.,Ltd." },
    { "002198", "Thai Radio Co, LTD" },
    { "002199", "Vacon Plc" },
    { "00219A", "Cambridge Visual Networks Ltd" },
    { "00219B", "Dell Inc." },
    { "00219C", "Honeywld Technology Corp." },
    { "00219D", "Adesys BV" },
    { "00219E", "Sony Corporation" },
    { "00219F", "SATEL OY" },
    { "0021A0", "Cisco Systems, Inc" },
    { "0021A1", "Cisco Systems, Inc" },
    { "0021A2", "EKE-Electronics Ltd." },
    { "0021A3", "Micromint" },
    { "0021A4", "Dbii Networks" },
    { "0021A5", "ERLPhase Power Technologies Ltd." },
    { "0021A6", "Videotec Spa" },
    { "0021A7", "Hantle System Co., Ltd." },
    { "0021A8", "Telephonics Corporation" },
    { "0021A9", "Mobilink Telecom Co.,Ltd" },
    { "0021AA", "Nokia Danmark A/S" },
    { "0021AB", "Nokia Danmark A/S" },
    { "0021AC", "Infrared Integrated Systems Ltd" },
    { "0021AD", "Nordic ID Oy" },
    { "0021AE", "ALCATEL-LUCENT FRANCE - WTD" },
    { "0021AF", "Radio Frequency Systems" },
    { "0021B0", "Tyco Telecommunications" },
    { "0021B1", "DIGITAL SOLUTIONS LTD" },
    { "0021B2", "Fiberblaze A/S" },
    { "0021B3", "Ross Controls" },
    { "0021B4", "APRO MEDIA CO., LTD" },
    { "0021B5", "Galvanic Ltd" },
    { "0021B6", "Triacta Power Technologies Inc." },
    { "0021B7", "LEXMARK INTERNATIONAL, INC." },
    { "0021B8", "Inphi Corporation" },
    { "0021B9", "Universal Devices Inc." },
    { "0021BA", "Texas Instruments" },
    { "0021BB", "Riken Keiki Co., Ltd." },
    { "0021BC", "ZALA COMPUTER" },
    { "0021BD", "Nintendo Co., Ltd." },
    { "0021BE", "Cisco SPVTG" },
    { "0021BF", "Hitachi High-Tech Control Systems Corporation" },
    { "0021C0", "Mobile Appliance, Inc." },
    { "0021C1", "ABB Oy / Medium Voltage Products" },
    { "0021C2", "GL Communications Inc" },
    { "0021C3", "CORNELL Communications, Inc." },
    { "0021C4", "Consilium AB" },
    { "0021C5", "3DSP Corp" },
    { "0021C6", "CSJ Global, Inc." },
    { "0021C7", "Russound" },
    { "0021C8", "LOHUIS Networks" },
    { "0021C9", "Wavecom Asia Pacific Limited" },
    { "0021CA", "ART System Co., Ltd." },
    { "0021CB", "SMS TECNOLOGIA ELETRONICA LTDA" },
    { "0021CC", "Flextronics International" },
    { "0021CD", "LiveTV" },
    { "0021CE", "NTC-Metrotek" },
    { "0021CF", "The Crypto Group" },
    { "0021D0", "Global Display Solutions Spa" },
    { "0021D1", "Samsung Electronics Co.,Ltd" },
    { "0021D2", "Samsung Electronics Co.,Ltd" },
    { "0021D3", "BOCOM SECURITY(ASIA PACIFIC) LIMITED" },
    { "0021D4", "Vollmer Werke GmbH" },
    { "0021D5", "X2E GmbH" },
    { "0021D6", "LXI Consortium" },
    { "0021D7", "Cisco Systems, Inc" },
    { "0021D8", "Cisco Systems, Inc" },
    { "0021D9", "SEKONIC CORPORATION" },
    { "0021DA", "Automation Products Group Inc." },
    { "0021DB", "Santachi Video Technology (Shenzhen) Co., Ltd." },
    { "0021DC", "TECNOALARM S.r.l." },
    { "0021DD", "Northstar Systems Corp" },
    { "0021DE", "Firepro Wireless" },
    { "0021DF", "Martin Christ GmbH" },
    { "0021E0", "CommAgility Ltd" },
    { "0021E1", "Nortel Networks" },
    { "0021E2", "visago Systems & Controls GmbH & Co. KG" },
    { "0021E3", "SerialTek LLC" },
    { "0021E4", "I-WIN" },
    { "0021E5", "Display Solution AG" },
    { "0021E6", "Starlight Video Limited" },
    { "0021E7", "Informatics Services Corporation" },
    { "0021E8", "Murata Manufacturing Co., Ltd." },
    { "0021E9", "Apple, Inc." },
    { "0021EA", "Bystronic Laser AG" },
    { "0021EB", "ESP SYSTEMS, LLC" },
    { "0021EC", "Solutronic GmbH" },
    { "0021ED", "Telegesis" },
    { "0021EE", "Full Spectrum Inc." },
    { "0021EF", "Kapsys" },
    { "0021F0", "EW3 Technologies LLC" },
    { "0021F1", "Tutus Data AB" },
    { "0021F2", "EASY3CALL Technology Limited" },
    { "0021F3", "Si14 SpA" },
    { "0021F4", "INRange Systems, Inc" },
    { "0021F5", "Western Engravers Supply, Inc." },
    { "0021F6", "Oracle Corporation" },
    { "0021F7", "HPN Supply Chain" },
    { "0021F8", "Enseo, Inc." },
    { "0021F9", "WIRECOM Technologies" },
    { "0021FA", "A4SP Technologies Ltd." },
    { "0021FB", "LG Electronics (Mobile Communications)" },
    { "0021FC", "Nokia Danmark A/S" },
    { "0021FD", "LACROIX TRAFFIC S.A.U" },
    { "0021FE", "Nokia Danmark A/S" },
    { "0021FF", "Cyfrowy Polsat SA" },
    { "002200", "IBM Corp" },
    { "002201", "Aksys Networks Inc" },
    { "002202", "Excito Elektronik i Skåne AB" },
    { "002203", "Glensound Electronics Ltd" },
    { "002204", "KORATEK" },
    { "002205", "WeLink Solutions, Inc." },
    { "002206", "Cyberdyne Inc." },
    { "002207", "Inteno Broadband Technology AB" },
    { "002208", "Certicom Corp" },
    { "002209", "Omron Healthcare Co., Ltd" },
    { "00220A", "OnLive, Inc" },
    { "00220B", "National Source Coding Center" },
    { "00220C", "Cisco Systems, Inc" },
    { "00220D", "Cisco Systems, Inc" },
    { "00220E", "Indigo Security Co., Ltd." },
    { "00220F", "MoCA (Multimedia over Coax Alliance)" },
    { "002210", "Commscope" },
    { "002211", "Rohati Systems" },
    { "002212", "CAI Networks, Inc." },
    { "002213", "PCI CORPORATION" },
    { "002214", "RINNAI KOREA" },
    { "002215", "ASUSTek COMPUTER INC." },
    { "002216", "SHIBAURA VENDING MACHINE CORPORATION" },
    { "002217", "Neat Electronics" },
    { "002218", "Akamai Technologies Inc." },
    { "002219", "Dell Inc." },
    { "00221A", "Audio Precision" },
    { "00221B", "Morega Systems" },
    { "00221C", "Private" },
    { "00221D", "Freegene Technology LTD" },
    { "00221E", "Media Devices Co., Ltd." },
    { "00221F", "eSang Technologies Co., Ltd." },
    { "002220", "Mitac Technology Corp" },
    { "002221", "ITOH DENKI CO,LTD." },
    { "002222", "Schaffner Deutschland GmbH" },
    { "002223", "TimeKeeping Systems, Inc." },
    { "002224", "Good Will Instrument Co., Ltd." },
    { "002225", "Thales Avionics Ltd" },
    { "002226", "Avaak, Inc." },
    { "002227", "uv-electronic GmbH" },
    { "002228", "Breeze Innovations Ltd." },
    { "002229", "Compumedics Ltd" },
    { "00222A", "SoundEar A/S" },
    { "00222B", "Nucomm, Inc." },
    { "00222C", "Ceton Corp" },
    { "00222D", "SMC Networks Inc." },
    { "00222E", "maintech GmbH" },
    { "00222F", "Open Grid Computing, Inc." },
    { "002230", "FutureLogic Inc." },
    { "002231", "SMT&C Co., Ltd." },
    { "002232", "Design Design Technology Ltd" },
    { "002233", "ADB Broadband Italia" },
    { "002234", "Corventis Inc." },
    { "002235", "Strukton Systems bv" },
    { "002236", "VECTOR SP. Z O.O." },
    { "002237", "Shinhint Group" },
    { "002238", "LOGIPLUS" },
    { "002239", "Indiana Life Sciences Incorporated" },
    { "00223A", "Cisco SPVTG" },
    { "00223B", "Communication Networks, LLC" },
    { "00223C", "RATIO Entwicklungen GmbH" },
    { "00223D", "JumpGen Systems, LLC" },
    { "00223E", "IRTrans GmbH" },
    { "00223F", "NETGEAR" },
    { "002240", "Universal Telecom S/A" },
    { "002241", "Apple, Inc." },
    { "002242", "Alacron Inc." },
    { "002243", "AzureWave Technology Inc." },
    { "002244", "Chengdu Linkon Communications Device Co., Ltd" },
    { "002245", "Leine & Linde AB" },
    { "002246", "Evoc Intelligent Technology Co.,Ltd." },
    { "002247", "DAC ENGINEERING CO., LTD." },
    { "002248", "Microsoft Corporation" },
    { "002249", "HOME MULTIENERGY SL" },
    { "00224A", "RAYLASE AG" },
    { "00224B", "AIRTECH TECHNOLOGIES, INC." },
    { "00224C", "Nintendo Co., Ltd." },
    { "00224D", "MITAC INTERNATIONAL CORP." },
    { "00224E", "SEEnergy Corp." },
    { "00224F", "Byzoro Networks Ltd." },
    { "002250", "Point Six Wireless, LLC" },
    { "002251", "Lumasense Technologies" },
    { "002252", "ZOLL Lifecor Corporation" },
    { "002253", "Entorian Technologies" },
    { "002254", "Bigelow Aerospace" },
    { "002255", "Cisco Systems, Inc" },
    { "002256", "Cisco Systems, Inc" },
    { "002257", "3COM EUROPE LTD" },
    { "002258", "Taiyo Yuden Co., Ltd." },
    { "002259", "Guangzhou New Postcom Equipment Co.,Ltd." },
    { "00225A", "Garde Security AB" },
    { "00225B", "Teradici Corporation" },
    { "00225C", "Multimedia & Communication Technology" },
    { "00225D", "Digicable Network India Pvt. Ltd." },
    { "00225E", "Uwin Technologies Co.,LTD" },
    { "00225F", "Liteon Technology Corporation" },
    { "002260", "AFREEY Inc." },
    { "002261", "Frontier Silicon Ltd" },
    { "002262", "BEP Marine" },
    { "002263", "Koos Technical Services, Inc." },
    { "002264", "Hewlett Packard" },
    { "002265", "Nokia Danmark A/S" },
    { "002266", "Nokia Danmark A/S" },
    { "002267", "Nortel Networks" },
    { "002268", "Hon Hai Precision Ind. Co.,Ltd." },
    { "002269", "Hon Hai Precision Ind. Co.,Ltd." },
    { "00226A", "Honeywell" },
    { "00226B", "Cisco-Linksys, LLC" },
    { "00226C", "LinkSprite Technologies, Inc." },
    { "00226D", "Shenzhen GIEC Electronics Co., Ltd." },
    { "00226E", "Gowell Electronic Limited" },
    { "00226F", "3onedata Technology Co. Ltd." },
    { "002270", "ABK North America, LLC" },
    { "002271", "Jäger Computergesteuerte Meßtechnik GmbH." },
    { "002272", "American Micro-Fuel Device Corp." },
    { "002273", "Techway" },
    { "002274", "FamilyPhone AB" },
    { "002275", "Belkin International Inc." },
    { "002276", "Triple EYE B.V." },
    { "002277", "NEC Australia Pty Ltd" },
    { "002278", "Shenzhen Tongfang Multimedia Technology Co.,Ltd." },
    { "002279", "Nippon Conlux Co., Ltd." },
    { "00227A", "Telecom Design" },
    { "00227B", "Apogee Labs, Inc." },
    { "00227C", "Woori SMT Co.,ltd" },
    { "00227D", "YE DATA INC." },
    { "00227E", "Chengdu 30Kaitian Communication Industry Co.Ltd" },
    { "00227F", "Ruckus Wireless" },
    { "002280", "A2B Electronics AB" },
    { "002281", "Daintree Networks Pty" },
    { "002282", "8086 Consultancy" },
    { "002283", "Juniper Networks" },
    { "002284", "DESAY A&V SCIENCE AND TECHNOLOGY CO.,LTD" },
    { "002285", "NOMUS COMM SYSTEMS" },
    { "002286", "ASTRON" },
    { "002287", "Titan Wireless LLC" },
    { "002288", "Sagrad, Inc." },
    { "002289", "Vanderlande APC inc." },
    { "00228A", "Teratronik elektronische systeme gmbh" },
    { "00228B", "Kensington Computer Products Group" },
    { "00228C", "Photon Europe GmbH" },
    { "00228D", "GBS Laboratories LLC" },
    { "00228E", "TV-NUMERIC" },
    { "00228F", "CNRS" },
    { "002290", "Cisco Systems, Inc" },
    { "002291", "Cisco Systems, Inc" },
    { "002292", "Cinetal" },
    { "002293", "zte corporation" },
    { "002294", "KYOCERA CORPORATION " },
    { "002295", "SGM Technology for lighting spa" },
    { "002296", "LinoWave Corporation" },
    { "002297", "XMOS Semiconductor" },
    { "002298", "Sony Corporation" },
    { "002299", "SeaMicro Inc." },
    { "00229A", "Lastar, Inc." },
    { "00229B", "AverLogic Technologies, Inc." },
    { "00229C", "Verismo Networks Inc" },
    { "00229D", "PYUNG-HWA IND.CO.,LTD" },
    { "00229E", "Social Aid Research Co., Ltd." },
    { "00229F", "Sensys Traffic AB" },
    { "0022A0", "APTIV SERVICES US, LLC" },
    { "0022A1", "Huawei Symantec Technologies Co.,Ltd." },
    { "0022A2", "Xtramus Technologies" },
    { "0022A3", "California Eastern Laboratories" },
    { "0022A4", "2Wire Inc" },
    { "0022A5", "Texas Instruments" },
    { "0022A6", "Sony Computer Entertainment America" },
    { "0022A7", "Tyco Electronics AMP GmbH" },
    { "0022A8", "Ouman Oy" },
    { "0022A9", "LG Electronics (Mobile Communications)" },
    { "0022AA", "Nintendo Co., Ltd." },
    { "0022AB", "Shenzhen Turbosight Technology Ltd" },
    { "0022AC", "Hangzhou Siyuan Tech. Co., Ltd" },
    { "0022AD", "TELESIS TECHNOLOGIES, INC." },
    { "0022AE", "Mattel Inc." },
    { "0022AF", "Safety Vision, LLC" },
    { "0022B0", "D-Link Corporation" },
    { "0022B1", "Elbit Systems Ltd." },
    { "0022B2", "4RF Communications Ltd" },
    { "0022B3", "Sei S.p.A." },
    { "0022B4", "Commscope" },
    { "0022B5", "NOVITA" },
    { "0022B6", "Superflow Technologies Group" },
    { "0022B7", "GSS Grundig SAT-Systems GmbH" },
    { "0022B8", "Norcott" },
    { "0022B9", "Analogix Seminconductor, Inc" },
    { "0022BA", "HUTH Elektronik Systeme GmbH" },
    { "0022BB", "beyerdynamic GmbH & Co. KG" },
    { "0022BC", "JDSU France SAS" },
    { "0022BD", "Cisco Systems, Inc" },
    { "0022BE", "Cisco Systems, Inc" },
    { "0022BF", "SieAmp Group of Companies" },
    { "0022C0", "Shenzhen Forcelink Electronic Co, Ltd" },
    { "0022C1", "Active Storage Inc." },
    { "0022C2", "Proview Eletrônica do Brasil LTDA" },
    { "0022C3", "Zeeport Technology Inc." },
    { "0022C4", "epro GmbH" },
    { "0022C5", "INFORSON Co,Ltd." },
    { "0022C6", "Sutus Inc" },
    { "0022C7", "SEGGER Microcontroller GmbH & Co. KG" },
    { "0022C8", "ModuVision Technologies" },
    { "0022C9", "Lenord, Bauer & Co GmbH" },
    { "0022CA", "Anviz Biometric Tech. Co., Ltd." },
    { "0022CB", "IONODES Inc." },
    { "0022CC", "SciLog, Inc." },
    { "0022CD", "Ared Technology Co., Ltd." },
    { "0022CE", "Cisco SPVTG" },
    { "0022CF", "PLANEX COMMUNICATIONS INC." },
    { "0022D0", "Polar Electro Oy" },
    { "0022D1", "Albrecht Jung GmbH & Co. KG" },
    { "0022D2", "All Earth Comércio de Eletrônicos LTDA." },
    { "0022D3", "Hub-Tech" },
    { "0022D4", "ComWorth Co., Ltd." },
    { "0022D5", "Eaton Corp. Electrical Group Data Center Solutions - Pulizzi" },
    { "0022D6", "Cypak AB" },
    { "0022D7", "Nintendo Co., Ltd." },
    { "0022D8", "Shenzhen GST Security and Safety Technology Limited" },
    { "0022D9", "Fortex Industrial Ltd." },
    { "0022DA", "ANATEK, LLC" },
    { "0022DB", "Translogic Corporation" },
    { "0022DC", "Vigil Health Solutions Inc." },
    { "0022DD", "Protecta Electronics Ltd" },
    { "0022DE", "OPPO Digital, Inc." },
    { "0022DF", "TAMUZ Monitors" },
    { "0022E0", "Atlantic Software Technologies S.r.L." },
    { "0022E1", "ZORT Labs, LLC." },
    { "0022E2", "WABTEC Transit Division" },
    { "0022E3", "Amerigon" },
    { "0022E4", "APASS TECHNOLOGY CO., LTD." },
    { "0022E5", "Fisher-Rosemount Systems Inc." },
    { "0022E6", "Intelligent Data" },
    { "0022E7", "WPS Parking Systems" },
    { "0022E8", "Applition Co., Ltd." },
    { "0022E9", "ProVision Communications" },
    { "0022EA", "Rustelcom Inc." },
    { "0022EB", "Data Respons A/S" },
    { "0022EC", "IDEALBT TECHNOLOGY CORPORATION" },
    { "0022ED", "TSI Power Corporation" },
    { "0022EE", "Algo Communication Products Ltd" },
    { "0022EF", "iWDL Technologies" },
    { "0022F0", "3 Greens Aviation Limited" },
    { "0022F1", "Private" },
    { "0022F2", "SunPower Corp" },
    { "0022F3", "SHARP Corporation" },
    { "0022F4", "AMPAK Technology, Inc." },
    { "0022F5", "Advanced Realtime Tracking GmbH" },
    { "0022F6", "Syracuse Research Corporation" },
    { "0022F7", "Conceptronic" },
    { "0022F8", "PIMA Electronic Systems Ltd." },
    { "0022F9", "Pollin Electronic GmbH" },
    { "0022FA", "Intel Corporate" },
    { "0022FB", "Intel Corporate" },
    { "0022FC", "Nokia Danmark A/S" },
    { "0022FD", "Nokia Danmark A/S" },
    { "0022FE", "Advanced Illumination" },
    { "0022FF", "NIVIS LLC" },
    { "002300", "Cayee Computer Ltd." },
    { "002301", "Witron Technology Limited" },
    { "002302", "Cobalt Digital, Inc." },
    { "002303", "LITE-ON IT Corporation" },
    { "002304", "Cisco Systems, Inc" },
    { "002305", "Cisco Systems, Inc" },
    { "002306", "ALPSALPINE CO,.LTD" },
    { "002307", "FUTURE INNOVATION TECH CO.,LTD" },
    { "002308", "Arcadyan Technology Corporation" },
    { "002309", "Janam Technologies LLC" },
    { "00230A", "ARBURG GmbH & Co KG" },
    { "00230B", "Commscope" },
    { "00230C", "CLOVER ELECTRONICS CO.,LTD." },
    { "00230D", "Nortel Networks" },
    { "00230E", "Gorba AG" },
    { "00230F", "Hirsch Electronics Corporation" },
    { "002310", "LNC Technology Co., Ltd." },
    { "002311", "Gloscom Co., Ltd." },
    { "002312", "Apple, Inc." },
    { "002313", "Qool Technologies Ltd." },
    { "002314", "Intel Corporate" },
    { "002315", "Intel Corporate" },
    { "002316", "KISAN ELECTRONICS CO" },
    { "002317", "Lasercraft Inc" },
    { "002318", "Toshiba" },
    { "002319", "Sielox LLC" },
    { "00231A", "ITF Co., Ltd." },
    { "00231B", "Danaher Motion - Kollmorgen" },
    { "00231C", "Fourier Systems Ltd." },
    { "00231D", "Deltacom Electronics Ltd" },
    { "00231E", "Cezzer Multimedia Technologies" },
    { "00231F", "Guangda Electronic & Telecommunication Technology Development Co., Ltd." },
    { "002320", "Nicira Networks" },
    { "002321", "Avitech International Corp" },
    { "002322", "KISS Teknical Solutions, Inc." },
    { "002323", "Zylin AS" },
    { "002324", "G-PRO COMPUTER" },
    { "002325", "IOLAN Holding" },
    { "002326", "FUJITSU LIMITED" },
    { "002327", "Shouyo Electronics CO., LTD" },
    { "002328", "ALCON TELECOMMUNICATIONS CO., LTD." },
    { "002329", "DDRdrive LLC" },
    { "00232A", "eonas IT-Beratung und -Entwicklung GmbH" },
    { "00232B", "IRD A/S" },
    { "00232C", "Senticare" },
    { "00232D", "SandForce" },
    { "00232E", "Kedah Electronics Engineering, LLC" },
    { "00232F", "Advanced Card Systems Ltd." },
    { "002330", "DIZIPIA, INC." },
    { "002331", "Nintendo Co., Ltd." },
    { "002332", "Apple, Inc." },
    { "002333", "Cisco Systems, Inc" },
    { "002334", "Cisco Systems, Inc" },
    { "002335", "Linkflex Co.,Ltd" },
    { "002336", "METEL s.r.o." },
    { "002337", "Global Star Solutions ULC" },
    { "002338", "OJ-Electronics A/S" },
    { "002339", "Samsung Electronics Co.,Ltd" },
    { "00233A", "Samsung Electronics Co.,Ltd" },
    { "00233B", "C-Matic Systems Ltd" },
    { "00233C", "Alflex" },
    { "00233D", "Laird Technologies" },
    { "00233E", "Alcatel-Lucent IPD" },
    { "00233F", "Purechoice Inc" },
    { "002340", "MiXTelematics" },
    { "002341", "Vanderbilt International (SWE) AB " },
    { "002342", "Coffee Equipment Company" },
    { "002343", "TEM AG" },
    { "002344", "Objective Interface Systems, Inc." },
    { "002345", "Sony Corporation" },
    { "002346", "Vestac" },
    { "002347", "ProCurve Networking by HP" },
    { "002348", "Sagemcom Broadband SAS" },
    { "002349", "Helmholtz Centre Berlin for Material and Energy" },
    { "00234A", "Private" },
    { "00234B", "Inyuan Technology Inc." },
    { "00234C", "KTC AB" },
    { "00234D", "Hon Hai Precision Ind. Co.,Ltd." },
    { "00234E", "Hon Hai Precision Ind. Co.,Ltd." },
    { "00234F", "Luminous Power Technologies Pvt. Ltd." },
    { "002350", "RDC, Inc. dba LynTec" },
    { "002351", "2Wire Inc" },
    { "002352", "DATASENSOR S.p.A." },
    { "002353", "F E T Elettronica snc" },
    { "002354", "ASUSTek COMPUTER INC." },
    { "002355", "Kinco Automation(Shanghai) Ltd." },
    { "002356", "Packet Forensics LLC" },
    { "002357", "Pitronot Technologies and Engineering P.T.E. Ltd." },
    { "002358", "SYSTEL SA" },
    { "002359", "Benchmark Electronics ( Thailand ) Public Company Limited" },
    { "00235A", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "00235B", "Gulfstream" },
    { "00235C", "Aprius, Inc." },
    { "00235D", "Cisco Systems, Inc" },
    { "00235E", "Cisco Systems, Inc" },
    { "00235F", "Silicon Micro Sensors GmbH" },
    { "002360", "Lookit Technology Co., Ltd" },
    { "002361", "Unigen Corporation" },
    { "002362", "Goldline Controls" },
    { "002363", "Zhuhai Raysharp Technology Co.,Ltd" },
    { "002364", "Power Instruments Pte Ltd" },
    { "002365", "Insta Elektro GmbH" },
    { "002366", "Beijing Siasun Electronic System Co.,Ltd." },
    { "002367", "UniControls a.s." },
    { "002368", "Zebra Technologies Inc" },
    { "002369", "Cisco-Linksys, LLC" },
    { "00236A", "SmartRG Inc" },
    { "00236B", "Xembedded, Inc." },
    { "00236C", "Apple, Inc." },
    { "00236D", "ResMed Ltd" },
    { "00236E", "Burster GmbH & Co KG" },
    { "00236F", "DAQ System" },
    { "002370", "Snell" },
    { "002371", "SOAM Systel" },
    { "002372", "MORE STAR INDUSTRIAL GROUP LIMITED" },
    { "002373", "GridIron Systems, Inc." },
    { "002374", "Commscope" },
    { "002375", "Commscope" },
    { "002376", "HTC Corporation" },
    { "002377", "Isotek Electronics Ltd" },
    { "002378", "GN Netcom A/S" },
    { "002379", "Union Business Machines Co. Ltd." },
    { "00237A", "RIM" },
    { "00237B", "WHDI LLC" },
    { "00237C", "NEOTION" },
    { "00237D", "Hewlett Packard" },
    { "00237E", "ELSTER GMBH" },
    { "00237F", "PLANTRONICS, INC." },
    { "002380", "Nanoteq" },
    { "002381", "Lengda Technology(Xiamen) Co.,Ltd." },
    { "002382", "Lih Rong electronic Enterprise Co., Ltd." },
    { "002383", "InMage Systems Inc" },
    { "002384", "GGH Engineering s.r.l." },
    { "002385", "ANTIPODE" },
    { "002386", "IMI Hydronic Engineering international SA" },
    { "002387", "ThinkFlood, Inc." },
    { "002388", "V.T. Telematica S.p.a." },
    { "002389", "Hangzhou H3C Technologies Co., Limited" },
    { "00238A", "Ciena Corporation" },
    { "00238B", "Quanta Computer Inc." },
    { "00238C", "Private" },
    { "00238D", "Techno Design Co., Ltd." },
    { "00238E", "ADB Broadband Italia" },
    { "00238F", "NIDEC COPAL CORPORATION" },
    { "002390", "Algolware Corporation" },
    { "002391", "Maxian" },
    { "002392", "Proteus Industries Inc." },
    { "002393", "AJINEXTEK" },
    { "002394", "Samjeon" },
    { "002395", "Commscope" },
    { "002396", "ANDES TECHNOLOGY CORPORATION" },
    { "002397", "Westell Technologies Inc." },
    { "002398", "Vutlan sro" },
    { "002399", "Samsung Electronics Co.,Ltd" },
    { "00239A", "EasyData Hardware GmbH" },
    { "00239B", "Elster Solutions, LLC" },
    { "00239C", "Juniper Networks" },
    { "00239D", "Mapower Electronics Co., Ltd" },
    { "00239E", "Jiangsu Lemote Technology Corporation Limited" },
    { "00239F", "Institut für Prüftechnik" },
    { "0023A0", "Hana CNS Co., LTD." },
    { "0023A1", "Trend Electronics Ltd" },
    { "0023A2", "Commscope" },
    { "0023A3", "Commscope" },
    { "0023A4", "New Concepts Development Corp." },
    { "0023A5", "SageTV, LLC" },
    { "0023A6", "E-Mon" },
    { "0023A7", "Redpine Signals, Inc." },
    { "0023A8", "Marshall Electronics" },
    { "0023A9", "Beijing Detianquan Electromechanical Equipment Co., Ltd" },
    { "0023AA", "HFR, Inc." },
    { "0023AB", "Cisco Systems, Inc" },
    { "0023AC", "Cisco Systems, Inc" },
    { "0023AD", "Xmark Corporation" },
    { "0023AE", "Dell Inc." },
    { "0023AF", "Commscope" },
    { "0023B0", "COMXION Technology Inc." },
    { "0023B1", "Longcheer Technology (Singapore) Pte Ltd" },
    { "0023B2", "Intelligent Mechatronic Systems Inc" },
    { "0023B3", "Lyyn AB" },
    { "0023B4", "Nokia Danmark A/S" },
    { "0023B5", "ORTANA LTD" },
    { "0023B6", "SECURITE COMMUNICATIONS / HONEYWELL" },
    { "0023B7", "Q-Light Co., Ltd." },
    { "0023B8", "Sichuan Jiuzhou Electronic Technology Co.,Ltd" },
    { "0023B9", "Airbus Defence and Space Deutschland GmbH" },
    { "0023BA", "Chroma" },
    { "0023BB", "Accretech SBS, Inc." },
    { "0023BC", "EQ-SYS GmbH" },
    { "0023BD", "Digital Ally, Inc." },
    { "0023BE", "Cisco SPVTG" },
    { "0023BF", "Mainpine, Inc." },
    { "0023C0", "Broadway Networks" },
    { "0023C1", "Securitas Direct AB" },
    { "0023C2", "SAMSUNG Electronics. Co. LTD" },
    { "0023C3", "LogMeIn, Inc." },
    { "0023C4", "Lux Lumen" },
    { "0023C5", "Radiation Safety and Control Services Inc" },
    { "0023C6", "SMC Corporation" },
    { "0023C7", "AVSystem sp. z o. o." },
    { "0023C8", "TEAM-R" },
    { "0023C9", "Sichuan Tianyi Information Science & Technology Stock CO.,LTD" },
    { "0023CA", "Behind The Set, LLC" },
    { "0023CB", "Shenzhen Full-join Technology Co.,Ltd" },
    { "0023CC", "Nintendo Co., Ltd." },
    { "0023CD", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "0023CE", "KITA DENSHI CORPORATION" },
    { "0023CF", "CUMMINS-ALLISON CORP." },
    { "0023D0", "Uniloc USA Inc." },
    { "0023D1", "TRG" },
    { "0023D2", "Inhand Electronics, Inc." },
    { "0023D3", "AirLink WiFi Networking Corp." },
    { "0023D4", "Texas Instruments" },
    { "0023D5", "WAREMA Renkhoff SE" },
    { "0023D6", "Samsung Electronics Co.,Ltd" },
    { "0023D7", "Samsung Electronics Co.,Ltd" },
    { "0023D8", "Ball-It Oy" },
    { "0023D9", "Banner Engineering" },
    { "0023DA", "Industrial Computer Source (Deutschland)GmbH" },
    { "0023DB", "saxnet gmbh" },
    { "0023DC", "Benein, Inc" },
    { "0023DD", "ELGIN S.A." },
    { "0023DE", "Ansync Inc." },
    { "0023DF", "Apple, Inc." },
    { "0023E0", "INO Therapeutics LLC" },
    { "0023E1", "Cavena Image Products AB" },
    { "0023E2", "SEA Signalisation" },
    { "0023E3", "Microtronic AG" },
    { "0023E4", "IPnect co. ltd." },
    { "0023E5", "IPaXiom Networks" },
    { "0023E6", "Innovation Farm, Inc." },
    { "0023E7", "Hinke A/S" },
    { "0023E8", "Demco Corp." },
    { "0023E9", "F5 Inc." },
    { "0023EA", "Cisco Systems, Inc" },
    { "0023EB", "Cisco Systems, Inc" },
    { "0023EC", "Algorithmix GmbH" },
    { "0023ED", "Commscope" },
    { "0023EE", "Commscope" },
    { "0023EF", "Zuend Systemtechnik AG" },
    { "0023F0", "Shanghai Jinghan Weighing Apparatus Co. Ltd." },
    { "0023F1", "Sony Corporation" },
    { "0023F2", "TVLogic" },
    { "0023F3", "Glocom, Inc." },
    { "0023F4", "Masternaut" },
    { "0023F5", "WILO SE" },
    { "0023F6", "Softwell Technology Co., Ltd." },
    { "0023F7", "Private" },
    { "0023F8", "Zyxel Communications Corporation" },
    { "0023F9", "Double-Take Software, INC." },
    { "0023FA", "RG Nets, Inc." },
    { "0023FB", "IP Datatel, LLC." },
    { "0023FC", "Ultra Stereo Labs, Inc" },
    { "0023FD", "AFT Atlas Fahrzeugtechnik GmbH" },
    { "0023FE", "Biodevices, SA" },
    { "0023FF", "Beijing HTTC Technology Ltd." },
    { "002400", "Nortel Networks" },
    { "002401", "D-Link Corporation" },
    { "002402", "Op-Tection GmbH" },
    { "002403", "Nokia Danmark A/S" },
    { "002404", "Nokia Danmark A/S" },
    { "002405", "Dilog Nordic AB" },
    { "002406", "Pointmobile" },
    { "002407", "TELEM SAS" },
    { "002408", "Pacific Biosciences" },
    { "002409", "The Toro Company" },
    { "00240A", "US Beverage Net" },
    { "00240B", "Virtual Computer Inc." },
    { "00240C", "DELEC GmbH" },
    { "00240D", "OnePath Networks LTD." },
    { "00240E", "Inventec Besta Co., Ltd." },
    { "00240F", "Ishii Tool & Engineering Corporation" },
    { "002410", "NUETEQ Technology,Inc." },
    { "002411", "PharmaSmart LLC" },
    { "002412", "Benign Technologies Co, Ltd." },
    { "002413", "Cisco Systems, Inc" },
    { "002414", "Cisco Systems, Inc" },
    { "002415", "Magnetic Autocontrol GmbH" },
    { "002416", "Any Use" },
    { "002417", "Thomson Telecom Belgium" },
    { "002418", "Nextwave Semiconductor" },
    { "002419", "Private" },
    { "00241A", "Red Beetle Inc." },
    { "00241B", "iWOW Communications Pte Ltd" },
    { "00241C", "FuGang Electronic (DG) Co.,Ltd" },
    { "00241D", "GIGA-BYTE TECHNOLOGY CO.,LTD." },
    { "00241E", "Nintendo Co., Ltd." },
    { "00241F", "DCT-Delta GmbH" },
    { "002420", "NetUP Inc." },
    { "002421", "MICRO-STAR INT'L CO., LTD." },
    { "002422", "Knapp Logistik Automation GmbH" },
    { "002423", "AzureWave Technologies (Shanghai) Inc." },
    { "002424", "Ace Axis Limited" },
    { "002425", "Shenzhenshi chuangzhicheng Technology Co.,Ltd" },
    { "002426", "NOHMI BOSAI LTD." },
    { "002427", "SSI COMPUTER CORP" },
    { "002428", "EnergyICT" },
    { "002429", "MK MASTER INC." },
    { "00242A", "Hittite Microwave Corporation" },
    { "00242B", "Hon Hai Precision Ind. Co.,Ltd." },
    { "00242C", "Hon Hai Precision Ind. Co.,Ltd." },
    { "00242E", "Datastrip Inc." },
    { "00242F", "Micron" },
    { "002430", "Ruby Tech Corp." },
    { "002431", "Uni-v co.,ltd" },
    { "002432", "Neostar Technology Co.,LTD" },
    { "002433", "ALPSALPINE CO,.LTD" },
    { "002434", "Lectrosonics, Inc." },
    { "002435", "WIDE CORPORATION" },
    { "002436", "Apple, Inc." },
    { "002437", "Motorola - BSG" },
    { "002438", "Brocade Communications Systems LLC" },
    { "002439", "Digital Barriers Advanced Technologies" },
    { "00243A", "Ludl Electronic Products" },
    { "00243B", "CSSI (S) Pte Ltd" },
    { "00243C", "S.A.A.A." },
    { "00243D", "Emerson Appliance Motors and Controls" },
    { "00243F", "Storwize, Inc." },
    { "002440", "Halo Monitoring, Inc." },
    { "002441", "Wanzl Metallwarenfabrik GmbH" },
    { "002442", "Axona Limited" },
    { "002443", "Nortel Networks" },
    { "002444", "Nintendo Co., Ltd." },
    { "002445", "Adtran Inc" },
    { "002446", "MMB Research Inc." },
    { "002447", "Kaztek Systems" },
    { "002448", "SpiderCloud Wireless, Inc" },
    { "002449", "Shen Zhen Lite Star Electronics Technology Co., Ltd" },
    { "00244A", "Voyant International" },
    { "00244B", "PERCEPTRON INC" },
    { "00244C", "Solartron Metrology Ltd" },
    { "00244D", "Hokkaido Electronics Corporation" },
    { "00244E", "RadChips, Inc." },
    { "00244F", "Asantron Technologies Ltd." },
    { "002450", "Cisco Systems, Inc" },
    { "002451", "Cisco Systems, Inc" },
    { "002452", "Silicon Software GmbH" },
    { "002453", "Initra d.o.o." },
    { "002454", "Samsung Electronics Co.,Ltd" },
    { "002455", "MuLogic BV" },
    { "002456", "2Wire Inc" },
    { "002458", "PA Bastion CC" },
    { "002459", "ABB Automation products GmbH" },
    { "00245A", "Nanjing Panda Electronics Company Limited" },
    { "00245B", "RAIDON TECHNOLOGY, INC." },
    { "00245C", "Design-Com Technologies Pty. Ltd." },
    { "00245D", "Terberg besturingstechniek B.V." },
    { "00245E", "Hivision Co.,ltd" },
    { "00245F", "Vine Telecom CO.,Ltd." },
    { "002460", "Giaval Science Development Co. Ltd." },
    { "002461", "Shin Wang Tech." },
    { "002462", "Rayzone Corporation" },
    { "002463", "Phybridge Inc" },
    { "002464", "Bridge Technologies Co AS" },
    { "002465", "Elentec" },
    { "002466", "Unitron nv" },
    { "002467", "AOC International (Europe) GmbH" },
    { "002468", "Sumavision Technologies Co.,Ltd" },
    { "002469", "Fasttel - Smart Doorphones" },
    { "00246A", "Solid Year Co., Ltd." },
    { "00246B", "Covia, Inc." },
    { "00246C", "Hewlett Packard Enterprise" },
    { "00246D", "Weinzierl Engineering GmbH" },
    { "00246E", "Phihong USA Corp." },
    { "00246F", "Onda Communication spa" },
    { "002470", "AUROTECH ultrasound AS." },
    { "002471", "Fusion MultiSystems dba Fusion-io" },
    { "002472", "ReDriven Power Inc." },
    { "002473", "3COM EUROPE LTD" },
    { "002474", "Autronica Fire And Securirty" },
    { "002475", "Compass System(Embedded Dept.)" },
    { "002476", "TAP.tv" },
    { "002477", "Tibbo Technology" },
    { "002478", "Mag Tech Electronics Co Limited" },
    { "002479", "Optec Displays, Inc." },
    { "00247A", "FU YI CHENG Technology Co., Ltd." },
    { "00247B", "Actiontec Electronics, Inc" },
    { "00247C", "Nokia Danmark A/S" },
    { "00247D", "Nokia Danmark A/S" },
    { "00247E", "Universal Global Scientific Industrial Co., Ltd." },
    { "00247F", "Nortel Networks" },
    { "002480", "Meteocontrol GmbH" },
    { "002481", "Hewlett Packard" },
    { "002482", "Ruckus Wireless" },
    { "002483", "LG Electronics (Mobile Communications)" },
    { "002484", "Bang and Olufsen Medicom a/s" },
    { "002485", "ConteXtream Ltd" },
    { "002486", "DesignArt Networks" },
    { "002487", "Transact Campus, Inc." },
    { "002488", "Centre For Development Of Telematics" },
    { "002489", "Vodafone Omnitel N.V." },
    { "00248A", "Kaga Electronics Co., Ltd." },
    { "00248B", "HYBUS CO., LTD." },
    { "00248C", "ASUSTek COMPUTER INC." },
    { "00248D", "Sony Interactive Entertainment Inc." },
    { "00248E", "Infoware ZRt." },
    { "00248F", "DO-MONIX" },
    { "002490", "Samsung Electronics Co.,Ltd" },
    { "002491", "Samsung Electronics Co.,Ltd" },
    { "002492", "Motorola, Broadband Solutions Group" },
    { "002493", "Commscope" },
    { "002494", "Shenzhen Baoxin Tech CO., Ltd." },
    { "002495", "Commscope" },
    { "002496", "Ginzinger electronic systems" },
    { "002497", "Cisco Systems, Inc" },
    { "002498", "Cisco Systems, Inc" },
    { "002499", "Aquila Technologies" },
    { "00249A", "Beijing Zhongchuang Telecommunication Test Co., Ltd." },
    { "00249B", "Action Star Enterprise Co., Ltd." },
    { "00249C", "Bimeng Comunication System Co. Ltd" },
    { "00249D", "NES Technology Inc." },
    { "00249E", "ADC-Elektronik GmbH" },
    { "00249F", "RIM Testing Services" },
    { "0024A0", "Commscope" },
    { "0024A1", "Commscope" },
    { "0024A2", "Hong Kong Middleware Technology Limited" },
    { "0024A3", "Sonim Technologies Inc" },
    { "0024A4", "Siklu Communication" },
    { "0024A5", "BUFFALO.INC" },
    { "0024A6", "TELESTAR DIGITAL GmbH" },
    { "0024A7", "Advanced Video Communications Inc." },
    { "0024A8", "ProCurve Networking by HP" },
    { "0024A9", "Ag Leader Technology" },
    { "0024AA", "Dycor Technologies Ltd." },
    { "0024AB", "A7 Engineering, Inc." },
    { "0024AC", "Hangzhou DPtech Technologies Co., Ltd." },
    { "0024AD", "Adolf Thies Gmbh & Co. KG" },
    { "0024AE", "IDEMIA" },
    { "0024AF", "Dish Technologies Corp" },
    { "0024B0", "ESAB AB" },
    { "0024B1", "Coulomb Technologies" },
    { "0024B2", "NETGEAR" },
    { "0024B3", "Graf-Syteco GmbH & Co. KG" },
    { "0024B4", "ESCATRONIC GmbH" },
    { "0024B5", "Nortel Networks" },
    { "0024B6", "Seagate Technology" },
    { "0024B7", "GridPoint, Inc." },
    { "0024B8", "free alliance sdn bhd" },
    { "0024B9", "Wuhan Higheasy Electronic Technology Development Co.Ltd" },
    { "0024BA", "Texas Instruments" },
    { "0024BB", "CENTRAL Corporation" },
    { "0024BC", "HuRob Co.,Ltd" },
    { "0024BD", "Hainzl Industriesysteme GmbH" },
    { "0024BE", "Sony Corporation" },
    { "0024BF", "Carrier Culoz SA" },
    { "0024C0", "NTI COMODO INC" },
    { "0024C1", "Commscope" },
    { "0024C2", "Asumo Co.,Ltd." },
    { "0024C3", "Cisco Systems, Inc" },
    { "0024C4", "Cisco Systems, Inc" },
    { "0024C5", "Meridian Audio Limited" },
    { "0024C6", "Hager Electro SAS" },
    { "0024C7", "Mobilarm Ltd" },
    { "0024C8", "Broadband Solutions Group" },
    { "0024C9", "Broadband Solutions Group" },
    { "0024CA", "Tobii Technology AB" },
    { "0024CB", "Autonet Mobile" },
    { "0024CC", "Fascinations Toys and Gifts, Inc." },
    { "0024CD", "Willow Garage, Inc." },
    { "0024CE", "Exeltech Inc" },
    { "0024CF", "Inscape Data Corporation" },
    { "0024D0", "Shenzhen SOGOOD Industry CO.,LTD." },
    { "0024D1", "Thomson Inc." },
    { "0024D2", "ASKEY COMPUTER CORP" },
    { "0024D3", "QUALICA Inc." },
    { "0024D4", "FREEBOX SAS" },
    { "0024D5", "Winward Industrial Limited" },
    { "0024D6", "Intel Corporate" },
    { "0024D7", "Intel Corporate" },
    { "0024D8", "IlSung Precision" },
    { "0024D9", "BICOM, Inc." },
    { "0024DA", "Innovar Systems Limited" },
    { "0024DB", "Alcohol Monitoring Systems" },
    { "0024DC", "Juniper Networks" },
    { "0024DD", "Centrak, Inc." },
    { "0024DE", "GLOBAL Technology Inc." },
    { "0024DF", "Digitalbox Europe GmbH" },
    { "0024E0", "DS Tech, LLC" },
    { "0024E1", "Convey Computer Corp." },
    { "0024E2", "HASEGAWA ELECTRIC CO.,LTD." },
    { "0024E3", "CAO Group" },
    { "0024E4", "Withings" },
    { "0024E5", "Seer Technology, Inc" },
    { "0024E6", "In Motion Technology Inc." },
    { "0024E7", "Plaster Networks" },
    { "0024E8", "Dell Inc." },
    { "0024E9", "Samsung Electronics Co.,Ltd" },
    { "0024EA", "iris-GmbH infrared & intelligent sensors" },
    { "0024EB", "ClearPath Networks, Inc." },
    { "0024EC", "United Information Technology Co.,Ltd." },
    { "0024ED", "YT Elec. Co,.Ltd." },
    { "0024EE", "Wynmax Inc." },
    { "0024EF", "Sony Corporation" },
    { "0024F0", "Seanodes" },
    { "0024F1", "Shenzhen Fanhai Sanjiang Electronics Co., Ltd." },
    { "0024F2", "Uniphone Telecommunication Co., Ltd." },
    { "0024F3", "Nintendo Co., Ltd." },
    { "0024F4", "Kaminario, Ltd." },
    { "0024F5", "NDS Surgical Imaging" },
    { "0024F6", "MIYOSHI ELECTRONICS CORPORATION" },
    { "0024F7", "Cisco Systems, Inc" },
    { "0024F8", "Technical Solutions Company Ltd." },
    { "0024F9", "Cisco Systems, Inc" },
    { "0024FA", "Hilger u. Kern GMBH" },
    { "0024FB", "Private" },
    { "0024FC", "QuoPin Co., Ltd." },
    { "0024FD", "Accedian Networks Inc" },
    { "0024FE", "AVM GmbH" },
    { "0024FF", "QLogic Corporation" },
    { "002500", "Apple, Inc." },
    { "002501", "JSC Supertel" },
    { "002502", "NaturalPoint" },
    { "002503", "IBM Corp" },
    { "002504", "Valiant Communications Limited" },
    { "002505", "eks Engel GmbH & Co. KG" },
    { "002506", "A.I. ANTITACCHEGGIO ITALIA SRL" },
    { "002507", "ASTAK Inc." },
    { "002508", "Maquet Cardiopulmonary AG" },
    { "002509", "SHARETRONIC Group LTD" },
    { "00250A", "Security Expert Co. Ltd" },
    { "00250B", "CENTROFACTOR INC" },
    { "00250C", "Senet Inc" },
    { "00250D", "GZT Telkom-Telmor sp. z o.o." },
    { "00250E", "gt german telematics gmbh" },
    { "00250F", "On-Ramp Wireless, Inc." },
    { "002510", "Pico-Tesla Magnetic Therapies" },
    { "002511", "Elitegroup Computer Systems Co.,Ltd." },
    { "002512", "zte corporation" },
    { "002513", "CXP DIGITAL BV" },
    { "002514", "PC Worth Int'l Co., Ltd." },
    { "002515", "SFR" },
    { "002516", "Integrated Design Tools, Inc." },
    { "002517", "Venntis, LLC" },
    { "002518", "Power PLUS Communications AG" },
    { "002519", "Viaas Inc" },
    { "00251A", "Psiber Data Systems Inc." },
    { "00251B", "Philips CareServant" },
    { "00251C", "EDT" },
    { "00251D", "DSA Encore, LLC" },
    { "00251E", "ROTEL TECHNOLOGIES" },
    { "00251F", "ZYNUS VISION INC." },
    { "002520", "SMA Railway Technology GmbH" },
    { "002521", "Logitek Electronic Systems, Inc." },
    { "002522", "ASRock Incorporation" },
    { "002523", "OCP Inc." },
    { "002524", "Lightcomm Technology Co., Ltd" },
    { "002525", "CTERA Networks Ltd." },
    { "002526", "Genuine Technologies Co., Ltd." },
    { "002527", "Bitrode Corp." },
    { "002528", "Daido Signal Co., Ltd." },
    { "002529", "COMELIT GROUP S.P.A" },
    { "00252A", "Chengdu GeeYa Technology Co.,LTD" },
    { "00252B", "Stirling Energy Systems" },
    { "00252C", "Entourage Systems, Inc." },
    { "00252D", "Kiryung Electronics" },
    { "00252E", "Cisco SPVTG" },
    { "00252F", "Energy, Inc." },
    { "002530", "Aetas Systems Inc." },
    { "002531", "Cloud Engines, Inc." },
    { "002532", "Digital Recorders" },
    { "002533", "WITTENSTEIN AG" },
    { "002535", "Minimax GmbH & Co KG" },
    { "002536", "Oki Electric Industry Co., Ltd." },
    { "002537", "Runcom Technologies Ltd." },
    { "002538", "Samsung Electronics Co., Ltd., Memory Division" },
    { "002539", "IfTA GmbH" },
    { "00253A", "CEVA, Ltd." },
    { "00253B", "din Dietmar Nocker Facilitymanagement GmbH" },
    { "00253C", "2Wire Inc" },
    { "00253D", "DRS Consolidated Controls" },
    { "00253E", "Sensus Metering Systems" },
    { "002540", "Quasar Technologies, Inc." },
    { "002541", "Maquet Critical Care AB" },
    { "002542", "Pittasoft" },
    { "002543", "MONEYTECH" },
    { "002544", "LoJack Corporation" },
    { "002545", "Cisco Systems, Inc" },
    { "002546", "Cisco Systems, Inc" },
    { "002547", "Nokia Danmark A/S" },
    { "002548", "Nokia Danmark A/S" },
    { "002549", "Jeorich Tech. Co.,Ltd." },
    { "00254A", "RingCube Technologies, Inc." },
    { "00254B", "Apple, Inc." },
    { "00254C", "Videon Central, Inc." },
    { "00254D", "Singapore Technologies Electronics Limited" },
    { "00254E", "Vertex Wireless Co., Ltd." },
    { "00254F", "ELETTROLAB Srl" },
    { "002550", "Riverbed Technology, Inc." },
    { "002551", "SE-Elektronic GmbH" },
    { "002552", "VXi Corporation" },
    { "002553", "ADB Broadband Italia" },
    { "002554", "Pixel8 Networks" },
    { "002555", "Visonic Technologies 1993 Ltd." },
    { "002556", "Hon Hai Precision Ind. Co.,Ltd." },
    { "002557", "BlackBerry RTS" },
    { "002558", "MPEDIA" },
    { "002559", "Syphan Technologies Ltd" },
    { "00255A", "Tantalus Systems Corp." },
    { "00255B", "CoachComm, LLC" },
    { "00255C", "NEC Corporation" },
    { "00255D", "Morningstar Corporation" },
    { "00255E", "Shanghai Dare Technologies Co.,Ltd." },
    { "00255F", "SenTec AG" },
    { "002560", "Ibridge Networks & Communications Ltd." },
    { "002561", "ProCurve Networking by HP" },
    { "002562", "interbro Co. Ltd." },
    { "002563", "Luxtera Inc" },
    { "002564", "Dell Inc." },
    { "002565", "Vizimax Inc." },
    { "002566", "Samsung Electronics Co.,Ltd" },
    { "002567", "Samsung Electronics Co.,Ltd" },
    { "002568", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "002569", "Sagemcom Broadband SAS" },
    { "00256A", "inIT - Institut Industrial IT" },
    { "00256B", "ATENIX E.E. s.r.l." },
    { "00256C", "Azimut Production Association JSC" },
    { "00256D", "Broadband Forum" },
    { "00256E", "Van Breda B.V." },
    { "00256F", "Dantherm Power" },
    { "002570", "Eastern Communications Company Limited" },
    { "002571", "Zhejiang Tianle Digital Electric Co.,Ltd" },
    { "002572", "Nemo-Q International AB" },
    { "002573", "ST Electronics (Info-Security) Pte Ltd" },
    { "002574", "KUNIMI MEDIA DEVICE Co., Ltd." },
    { "002575", "FiberPlex Technologies, LLC" },
    { "002576", "NELI TECHNOLOGIES" },
    { "002577", "D-BOX Technologies" },
    { "002578", "JSC Concern Sozvezdie" },
    { "002579", "J & F Labs" },
    { "00257A", "CAMCO Produktions- und Vertriebs-GmbH für Beschallungs- und Beleuchtungsanlagen" },
    { "00257B", "STJ ELECTRONICS PVT LTD" },
    { "00257C", "Huachentel Technology Development Co., Ltd" },
    { "00257D", "PointRed Telecom Private Ltd." },
    { "00257E", "NEW POS TECHNOLOGY LIMITED" },
    { "00257F", "CallTechSolution Co.,Ltd" },
    { "002580", "Equipson S.A." },
    { "002581", "x-star networks Inc." },
    { "002582", "Maksat Technologies (P) Ltd" },
    { "002583", "Cisco Systems, Inc" },
    { "002584", "Cisco Systems, Inc" },
    { "002585", "KOKUYO S&T Co., Ltd." },
    { "002586", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "002587", "Vitality, Inc." },
    { "002588", "Genie Industries, Inc." },
    { "002589", "Hills Industries Limited" },
    { "00258A", "Pole/Zero Corporation" },
    { "00258B", "Mellanox Technologies, Inc." },
    { "00258C", "ESUS ELEKTRONIK SAN. VE DIS. TIC. LTD. STI." },
    { "00258D", "Haier" },
    { "00258E", "The Weather Channel" },
    { "00258F", "Trident Microsystems, Inc." },
    { "002590", "Super Micro Computer, Inc." },
    { "002591", "NEXTEK, Inc." },
    { "002592", "Guangzhou Shirui Electronic Co., Ltd" },
    { "002593", "DatNet Informatikai Kft." },
    { "002594", "Eurodesign BG LTD" },
    { "002595", "Northwest Signal Supply, Inc" },
    { "002596", "GIGAVISION srl" },
    { "002597", "Kalki Communication Technologies" },
    { "002598", "Zhong Shan City Litai Electronic Industrial Co. Ltd" },
    { "002599", "Hedon e.d. B.V." },
    { "00259A", "CEStronics GmbH" },
    { "00259B", "Beijing PKUNITY Microsystems Technology Co., Ltd" },
    { "00259C", "Cisco-Linksys, LLC" },
    { "00259D", "Private" },
    { "00259E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "00259F", "TechnoDigital Technologies GmbH" },
    { "0025A0", "Nintendo Co., Ltd." },
    { "0025A1", "Enalasys" },
    { "0025A2", "Alta Definicion LINCEO S.L." },
    { "0025A3", "Trimax Wireless, Inc." },
    { "0025A4", "EuroDesign embedded technologies GmbH" },
    { "0025A5", "Walnut Media Network" },
    { "0025A6", "Central Network Solution Co., Ltd." },
    { "0025A7", "itron" },
    { "0025A8", "Kontron (BeiJing) Technology Co.,Ltd" },
    { "0025A9", "Shanghai Embedway Information Technologies Co.,Ltd" },
    { "0025AA", "Beijing Soul Technology Co.,Ltd." },
    { "0025AB", "AIO LCD PC BU / TPV" },
    { "0025AC", "I-Tech corporation" },
    { "0025AD", "Manufacturing Resources International" },
    { "0025AE", "Microsoft Corporation" },
    { "0025AF", "COMFILE Technology" },
    { "0025B0", "Schmartz Inc" },
    { "0025B1", "Maya-Creation Corporation" },
    { "0025B2", "MBDA Deutschland GmbH" },
    { "0025B3", "Hewlett Packard" },
    { "0025B4", "Cisco Systems, Inc" },
    { "0025B5", "Cisco Systems, Inc" },
    { "0025B6", "Telecom FM" },
    { "0025B7", "Costar electronics, inc.," },
    { "0025B8", "Agile Communications, Inc." },
    { "0025B9", "Cypress Solutions Inc" },
    { "0025BA", "Alcatel-Lucent IPD" },
    { "0025BB", "INNERINT Co., Ltd." },
    { "0025BC", "Apple, Inc." },
    { "0025BD", "Italdata Ingegneria dell'Idea S.p.A." },
    { "0025BE", "Tektrap Systems Inc." },
    { "0025BF", "Wireless Cables Inc." },
    { "0025C0", "ZillionTV Corporation" },
    { "0025C1", "Nawoo Korea Corp." },
    { "0025C2", "RingBell Co.,Ltd." },
    { "0025C3", "21168" },
    { "0025C4", "Ruckus Wireless" },
    { "0025C5", "Star Link Communication Pvt. Ltd." },
    { "0025C6", "kasercorp, ltd" },
    { "0025C7", "altek Corporation" },
    { "0025C8", "S-Access GmbH" },
    { "0025C9", "SHENZHEN HUAPU DIGITAL CO., LTD" },
    { "0025CA", "Laird Connectivity" },
    { "0025CB", "Reiner SCT" },
    { "0025CC", "Mobile Communications Korea Incorporated" },
    { "0025CD", "Skylane Optics" },
    { "0025CE", "InnerSpace" },
    { "0025CF", "Nokia Danmark A/S" },
    { "0025D0", "Nokia Danmark A/S" },
    { "0025D1", "Eastern Asia Technology Limited" },
    { "0025D2", "InpegVision Co., Ltd" },
    { "0025D3", "AzureWave Technology Inc." },
    { "0025D4", "General Dynamics Mission Systems" },
    { "0025D5", "Robonica (Pty) Ltd" },
    { "0025D6", "The Kroger Co." },
    { "0025D7", "CEDO" },
    { "0025D8", "KOREA MAINTENANCE" },
    { "0025D9", "DataFab Systems Inc." },
    { "0025DA", "Secura Key" },
    { "0025DB", "ATI Electronics(Shenzhen) Co., LTD" },
    { "0025DC", "Sumitomo Electric Industries, Ltd" },
    { "0025DD", "SUNNYTEK INFORMATION CO., LTD." },
    { "0025DE", "Probits Co., LTD." },
    { "0025DF", "Axon Enterprise, Inc." },
    { "0025E0", "CeedTec Sdn Bhd" },
    { "0025E1", "SHANGHAI SEEYOO ELECTRONIC & TECHNOLOGY CO., LTD" },
    { "0025E2", "Everspring Industry Co., Ltd." },
    { "0025E3", "Hanshinit Inc." },
    { "0025E4", "OMNI-WiFi, LLC" },
    { "0025E5", "LG Electronics (Mobile Communications)" },
    { "0025E6", "Belgian Monitoring Systems bvba" },
    { "0025E7", "Sony Corporation" },
    { "0025E8", "Idaho Technology" },
    { "0025E9", "i-mate Development, Inc." },
    { "0025EA", "Iphion BV" },
    { "0025EB", "Reutech Radar Systems (PTY) Ltd" },
    { "0025EC", "Humanware" },
    { "0025ED", "NuVo Technologies LLC" },
    { "0025EE", "Avtex Ltd" },
    { "0025EF", "I-TEC Co., Ltd." },
    { "0025F0", "Suga Electronics Limited" },
    { "0025F1", "Commscope" },
    { "0025F2", "Commscope" },
    { "0025F3", "Nordwestdeutsche Zählerrevision" },
    { "0025F4", "KoCo Connector AG" },
    { "0025F5", "DVS Korea, Co., Ltd" },
    { "0025F6", "netTALK.com, Inc." },
    { "0025F7", "Ansaldo STS USA" },
    { "0025F9", "GMK electronic design GmbH" },
    { "0025FA", "J&M Analytik AG" },
    { "0025FB", "Tunstall Healthcare A/S" },
    { "0025FC", "ENDA" },
    { "0025FD", "OBR Centrum Techniki Morskiej S.A." },
    { "0025FE", "Pilot Electronics Corporation" },
    { "0025FF", "CreNova Multimedia Co., Ltd" },
    { "002600", "TEAC Australia Pty Ltd." },
    { "002601", "Cutera Inc" },
    { "002602", "SMART Temps LLC" },
    { "002603", "Shenzhen Wistar Technology Co., Ltd" },
    { "002604", "WorldCast Systems" },
    { "002605", "CC Systems AB" },
    { "002606", "RAUMFELD GmbH" },
    { "002607", "Enabling Technology Pty Ltd" },
    { "002608", "Apple, Inc." },
    { "002609", "Phyllis Co., Ltd." },
    { "00260A", "Cisco Systems, Inc" },
    { "00260B", "Cisco Systems, Inc" },
    { "00260C", "Dataram" },
    { "00260D", "Mercury Systems, Inc." },
    { "00260E", "Ablaze Systems, LLC" },
    { "00260F", "Linn Products Ltd" },
    { "002610", "Apacewave Technologies" },
    { "002611", "Licera AB" },
    { "002612", "Space Exploration Technologies" },
    { "002613", "Engel Axil S.L." },
    { "002614", "KTNF" },
    { "002615", "Teracom Limited" },
    { "002616", "Rosemount Inc." },
    { "002617", "OEM Worldwide" },
    { "002618", "ASUSTek COMPUTER INC." },
    { "002619", "FRC" },
    { "00261A", "Femtocomm System Technology Corp." },
    { "00261B", "LAUREL BANK MACHINES CO., LTD." },
    { "00261C", "NEOVIA INC." },
    { "00261D", "COP SECURITY SYSTEM CORP." },
    { "00261E", "QINGBANG ELEC(SZ) CO., LTD" },
    { "00261F", "SAE Magnetics (H.K.) Ltd." },
    { "002620", "ISGUS GmbH" },
    { "002621", "InteliCloud Technology Inc." },
    { "002622", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "002623", "JRD Communication Inc" },
    { "002624", "Thomson Inc." },
    { "002625", "MediaSputnik" },
    { "002626", "Geophysical Survey Systems, Inc." },
    { "002627", "Truesell" },
    { "002628", "companytec automação e controle ltda." },
    { "002629", "Juphoon System Software Inc." },
    { "00262A", "Proxense, LLC" },
    { "00262B", "Wongs Electronics Co. Ltd." },
    { "00262C", "IKT Advanced Technologies s.r.o." },
    { "00262D", "Wistron Corporation" },
    { "00262E", "Chengdu Jiuzhou Electronic Technology Inc" },
    { "00262F", "HAMAMATSU TOA ELECTRONICS" },
    { "002630", "ACOREL S.A.S" },
    { "002631", "COMMTACT LTD" },
    { "002632", "Instrumentation Technologies d.d." },
    { "002633", "MIR - Medical International Research" },
    { "002634", "Infineta Systems, Inc" },
    { "002635", "Bluetechnix GmbH" },
    { "002636", "Commscope" },
    { "002637", "SAMSUNG ELECTRO MECHANICS CO., LTD." },
    { "002638", "Xia Men Joyatech Co., Ltd." },
    { "002639", "T.M. Electronics, Inc." },
    { "00263A", "Digitec Systems" },
    { "00263B", "Onbnetech" },
    { "00263C", "Bachmann Technology GmbH & Co. KG" },
    { "00263D", "MIA Corporation" },
    { "00263E", "Trapeze Networks" },
    { "00263F", "LIOS Technology GmbH" },
    { "002640", "Baustem Broadband Technologies, Ltd." },
    { "002641", "Commscope" },
    { "002642", "Commscope" },
    { "002643", "ALPSALPINE CO,.LTD" },
    { "002644", "Thomson Telecom Belgium" },
    { "002645", "Circontrol S.A." },
    { "002646", "SHENYANG TONGFANG MULTIMEDIA TECHNOLOGY COMPANY LIMITED" },
    { "002647", "WFE TECHNOLOGY CORP." },
    { "002648", "Emitech Corp." },
    { "00264A", "Apple, Inc." },
    { "00264C", "Shanghai DigiVision Technology Co., Ltd." },
    { "00264D", "Arcadyan Technology Corporation" },
    { "00264E", "r2p GmbH" },
    { "00264F", "Krüger &Gothe GmbH" },
    { "002650", "2Wire Inc" },
    { "002651", "Cisco Systems, Inc" },
    { "002652", "Cisco Systems, Inc" },
    { "002653", "DaySequerra Corporation" },
    { "002654", "3COM" },
    { "002655", "Hewlett Packard" },
    { "002656", "Sansonic Electronics USA" },
    { "002657", "OOO NPP EKRA" },
    { "002658", "T-Platforms (Cyprus) Limited" },
    { "002659", "Nintendo Co., Ltd." },
    { "00265A", "D-Link Corporation" },
    { "00265B", "Hitron Technologies. Inc" },
    { "00265C", "Hon Hai Precision Ind. Co.,Ltd." },
    { "00265D", "Samsung Electronics Co.,Ltd" },
    { "00265E", "Hon Hai Precision Ind. Co.,Ltd." },
    { "00265F", "Samsung Electronics Co.,Ltd" },
    { "002660", "Logiways" },
    { "002661", "Irumtek Co., Ltd." },
    { "002662", "Actiontec Electronics, Inc" },
    { "002663", "Shenzhen Huitaiwei Tech. Ltd, co." },
    { "002664", "Core System Japan" },
    { "002665", "ProtectedLogic Corporation" },
    { "002666", "EFM Networks" },
    { "002667", "CARECOM CO.,LTD." },
    { "002668", "Nokia Danmark A/S" },
    { "002669", "Nokia Danmark A/S" },
    { "00266A", "ESSENSIUM NV" },
    { "00266B", "SHINE UNION ENTERPRISE LIMITED" },
    { "00266C", "INVENTEC CORPORATION" },
    { "00266D", "MobileAccess Networks" },
    { "00266E", "Nissho-denki Co.,LTD." },
    { "00266F", "Coordiwise Technology Corp." },
    { "002670", "Cinch Connectors" },
    { "002671", "AUTOVISION Co., Ltd" },
    { "002672", "AAMP of America" },
    { "002673", "RICOH COMPANY,LTD." },
    { "002674", "Hunter Douglas" },
    { "002675", "Aztech Electronics Pte Ltd" },
    { "002676", "COMMidt AS" },
    { "002677", "DEIF A/S" },
    { "002678", "Logic Instrument SA" },
    { "002679", "Euphonic Technologies, Inc." },
    { "00267A", "wuhan hongxin telecommunication technologies co.,ltd" },
    { "00267B", "GSI Helmholtzzentrum für Schwerionenforschung GmbH" },
    { "00267C", "Metz-Werke GmbH & Co KG" },
    { "00267D", "A-Max Technology Macao Commercial Offshore Company Limited" },
    { "00267E", "PARROT SA" },
    { "00267F", "Oregan Networks Ltd." },
    { "002680", "SIL3 Pty.Ltd" },
    { "002681", "Interspiro AB" },
    { "002682", "Gemtek Technology Co., Ltd." },
    { "002683", "Ajoho Enterprise Co., Ltd." },
    { "002684", "KISAN SYSTEM" },
    { "002685", "Digital Innovation" },
    { "002686", "Quantenna Communcations, Inc." },
    { "002687", "corega K.K" },
    { "002688", "Juniper Networks" },
    { "002689", "General Dynamics Robotic Systems" },
    { "00268A", "Terrier SC Ltd" },
    { "00268B", "Guangzhou Escene Computer Technology Limited" },
    { "00268C", "StarLeaf Ltd." },
    { "00268D", "CellTel S.p.A." },
    { "00268E", "Alta Solutions, Inc." },
    { "00268F", "MTA SpA" },
    { "002690", "I DO IT" },
    { "002691", "Sagemcom Broadband SAS" },
    { "002692", "Mitsubishi Electric Corporation" },
    { "002693", "QVidium Technologies, Inc." },
    { "002694", "Senscient Ltd" },
    { "002695", "ZT Group Int'l Inc" },
    { "002696", "NOOLIX Co., Ltd" },
    { "002697", "Alpha Technologies Inc." },
    { "002698", "Cisco Systems, Inc" },
    { "002699", "Cisco Systems, Inc" },
    { "00269A", "Carina System Co., Ltd." },
    { "00269B", "SOKRAT Ltd." },
    { "00269C", "ITUS JAPAN CO. LTD" },
    { "00269D", "M2Mnet Co., Ltd." },
    { "00269E", "Quanta Computer Inc." },
    { "00269F", "Private" },
    { "0026A0", "moblic" },
    { "0026A1", "Megger" },
    { "0026A2", "Instrumentation Technology Systems" },
    { "0026A3", "FQ Ingenieria Electronica S.A." },
    { "0026A4", "Novus Produtos Eletronicos Ltda" },
    { "0026A5", "MICROROBOT.CO.,LTD" },
    { "0026A6", "TRIXELL" },
    { "0026A7", "CONNECT SRL" },
    { "0026A8", "DAEHAP HYPER-TECH" },
    { "0026A9", "Strong Technologies Pty Ltd" },
    { "0026AA", "Kenmec Mechanical Engineering Co., Ltd." },
    { "0026AB", "Seiko Epson Corporation" },
    { "0026AC", "Shanghai LUSTER Teraband photonic Co., Ltd." },
    { "0026AD", "Arada Systems, Inc." },
    { "0026AE", "Wireless Measurement Ltd" },
    { "0026AF", "Duelco A/S" },
    { "0026B0", "Apple, Inc." },
    { "0026B1", "Navis Auto Motive Systems, Inc." },
    { "0026B2", "Setrix GmbH" },
    { "0026B3", "Thales Communications Inc" },
    { "0026B4", "Ford Motor Company" },
    { "0026B5", "ICOMM Tele Ltd" },
    { "0026B6", "ASKEY COMPUTER CORP" },
    { "0026B7", "Kingston Technology Company, Inc." },
    { "0026B8", "Actiontec Electronics, Inc" },
    { "0026B9", "Dell Inc." },
    { "0026BA", "Commscope" },
    { "0026BB", "Apple, Inc." },
    { "0026BC", "General Jack Technology Ltd." },
    { "0026BD", "JTEC Card &amp; Communication Co., Ltd" },
    { "0026BE", "Schoonderbeek Elektronica Systemen B.V." },
    { "0026BF", "ShenZhen Temobi Science&Tech Development Co.,Ltd" },
    { "0026C0", "EnergyHub" },
    { "0026C1", "ARTRAY CO., LTD." },
    { "0026C2", "SCDI Co. LTD" },
    { "0026C3", "Insightek Corp." },
    { "0026C4", "Cadmos microsystems S.r.l." },
    { "0026C5", "Guangdong Gosun Telecommunications Co.,Ltd" },
    { "0026C6", "Intel Corporate" },
    { "0026C7", "Intel Corporate" },
    { "0026C8", "System Sensor" },
    { "0026C9", "Proventix Systems, Inc." },
    { "0026CA", "Cisco Systems, Inc" },
    { "0026CB", "Cisco Systems, Inc" },
    { "0026CC", "Nokia Danmark A/S" },
    { "0026CD", "PurpleComm, Inc." },
    { "0026CE", "Kozumi USA Corp." },
    { "0026CF", "DEKA R&D" },
    { "0026D0", "Semihalf" },
    { "0026D1", "S Squared Innovations Inc." },
    { "0026D2", "Pcube Systems, Inc." },
    { "0026D3", "Zeno Information System" },
    { "0026D4", "IRCA SpA" },
    { "0026D5", "Ory Solucoes em Comercio de Informatica Ltda." },
    { "0026D6", "Ningbo Andy Optoelectronic Co., Ltd." },
    { "0026D7", "KM Electornic Technology Co., Ltd." },
    { "0026D8", "Magic Point Inc." },
    { "0026D9", "Commscope" },
    { "0026DA", "Universal Media Corporation /Slovakia/ s.r.o." },
    { "0026DB", "Ionics EMS Inc." },
    { "0026DC", "Optical Systems Design" },
    { "0026DD", "Fival Science & Technology Co.,Ltd." },
    { "0026DE", "FDI MATELEC" },
    { "0026DF", "TaiDoc Technology Corp." },
    { "0026E0", "ASITEQ" },
    { "0026E1", "Stanford University, OpenFlow Group" },
    { "0026E2", "LG Electronics (Mobile Communications)" },
    { "0026E3", "DTI" },
    { "0026E4", "Canal +" },
    { "0026E5", "AEG Power Solutions" },
    { "0026E6", "Visionhitech Co., Ltd." },
    { "0026E7", "Shanghai ONLAN Communication Tech. Co., Ltd." },
    { "0026E8", "Murata Manufacturing Co., Ltd." },
    { "0026E9", "SP Corp" },
    { "0026EA", "Cheerchip Electronic Technology (ShangHai) Co., Ltd." },
    { "0026EB", "Advanced Spectrum Technology Co., Ltd." },
    { "0026EC", "Legrand Home Systems, Inc" },
    { "0026ED", "zte corporation" },
    { "0026EE", "TKM GmbH" },
    { "0026EF", "Technology Advancement Group, Inc." },
    { "0026F0", "cTrixs International GmbH." },
    { "0026F1", "ProCurve Networking by HP" },
    { "0026F2", "NETGEAR" },
    { "0026F3", "SMC Networks" },
    { "0026F4", "Nesslab" },
    { "0026F5", "XRPLUS Inc." },
    { "0026F6", "Military Communication Institute" },
    { "0026F7", "Nivetti Systems Pvt. Ltd." },
    { "0026F8", "Golden Highway Industry Development Co., Ltd." },
    { "0026F9", "S.E.M. srl" },
    { "0026FA", "BandRich Inc." },
    { "0026FB", "AirDio Wireless, Inc." },
    { "0026FC", "AcSiP Technology Corp." },
    { "0026FD", "Interactive Intelligence" },
    { "0026FE", "MKD Technology Inc." },
    { "0026FF", "BlackBerry RTS" },
    { "002700", "Shenzhen Siglent Technology Co., Ltd." },
    { "002701", "INCOstartec GmbH" },
    { "002702", "SolarEdge Technologies" },
    { "002703", "Testech Electronics Pte Ltd" },
    { "002704", "Accelerated Concepts, Inc" },
    { "002705", "Sectronic" },
    { "002706", "YOISYS" },
    { "002707", "Lift Complex DS, JSC" },
    { "002708", "Nordiag ASA" },
    { "002709", "Nintendo Co., Ltd." },
    { "00270A", "IEE S.A." },
    { "00270B", "Adura Technologies" },
    { "00270C", "Cisco Systems, Inc" },
    { "00270D", "Cisco Systems, Inc" },
    { "00270E", "Intel Corporate" },
    { "00270F", "Envisionnovation Inc" },
    { "002710", "Intel Corporate" },
    { "002711", "LanPro Inc" },
    { "002712", "MaxVision LLC" },
    { "002713", "Universal Global Scientific Industrial Co., Ltd." },
    { "002714", "Grainmustards, Co,ltd." },
    { "002715", "Rebound Telecom. Co., Ltd" },
    { "002716", "Adachi-Syokai Co., Ltd." },
    { "002717", "CE Digital(Zhenjiang)Co.,Ltd" },
    { "002718", "Suzhou NEW SEAUNION Video Technology Co.,Ltd" },
    { "002719", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "00271A", "Geenovo Technology Ltd." },
    { "00271B", "Alec Sicherheitssysteme GmbH" },
    { "00271C", "MERCURY CORPORATION" },
    { "00271D", "Comba Telecom Systems (China) Ltd." },
    { "00271E", "Xagyl Communications" },
    { "00271F", "MIPRO Electronics Co., Ltd" },
    { "002720", "NEW-SOL COM" },
    { "002721", "Shenzhen Baoan Fenda Industrial Co., Ltd" },
    { "002722", "Ubiquiti Inc" },
    { "002790", "Cisco Systems, Inc" },
    { "0027E3", "Cisco Systems, Inc" },
    { "0027F8", "Brocade Communications Systems LLC" },
    { "00289F", "Semptian Co., Ltd." },
    { "0028F8", "Intel Corporate" },
    { "002926", "Applied Optoelectronics, Inc Taiwan Branch" },
    { "0029C2", "Cisco Systems, Inc" },
    { "002A10", "Cisco Systems, Inc" },
    { "002A6A", "Cisco Systems, Inc" },
    { "002AAF", "LARsys-Automation GmbH" },
    { "002B67", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "002B70", "Samsung Electronics Co.,Ltd" },
    { "002B90", "Zelus(Shenzhen) Technology Ltd." },
    { "002BF5", "BUFFALO.INC" },
    { "002CC8", "Cisco Systems, Inc" },
    { "002D76", "TITECH GmbH" },
    { "002DB3", "AMPAK Technology,Inc." },
    { "002EC7", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "002F5C", "Cisco Systems, Inc" },
    { "002FD9", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "003000", "ALLWELL TECHNOLOGY CORP." },
    { "003001", "SMP" },
    { "003002", "Expand Networks" },
    { "003003", "Phasys Ltd." },
    { "003004", "LEADTEK RESEARCH INC." },
    { "003005", "Fujitsu Siemens Computers" },
    { "003006", "SUPERPOWER COMPUTER" },
    { "003007", "OPTI, INC." },
    { "003008", "AVIO DIGITAL, INC." },
    { "003009", "Tachion Networks, Inc." },
    { "00300A", "Aztech Electronics Pte Ltd" },
    { "00300B", "mPHASE Technologies, Inc." },
    { "00300C", "CONGRUENCY, LTD." },
    { "00300D", "MMC Technology, Inc." },
    { "00300E", "Klotz Digital AG" },
    { "00300F", "IMT - Information Management T" },
    { "003010", "VISIONETICS INTERNATIONAL" },
    { "003011", "HMS Industrial Networks " },
    { "003012", "DIGITAL ENGINEERING LTD." },
    { "003013", "NEC Corporation" },
    { "003014", "DIVIO, INC." },
    { "003015", "CP CLARE CORP." },
    { "003016", "ISHIDA CO., LTD." },
    { "003017", "BlueArc UK Ltd" },
    { "003018", "Jetway Information Co., Ltd." },
    { "003019", "Cisco Systems, Inc" },
    { "00301A", "SMARTBRIDGES PTE. LTD." },
    { "00301B", "SHUTTLE, INC." },
    { "00301C", "ALTVATER AIRDATA SYSTEMS" },
    { "00301D", "SKYSTREAM, INC." },
    { "00301E", "3COM EUROPE LTD" },
    { "00301F", "OPTICAL NETWORKS, INC." },
    { "003020", "TSI, Inc.." },
    { "003021", "HSING TECH. ENTERPRISE CO.,LTD" },
    { "003022", "Fong Kai Industrial Co., Ltd." },
    { "003023", "COGENT COMPUTER SYSTEMS, INC." },
    { "003024", "Cisco Systems, Inc" },
    { "003025", "CHECKOUT COMPUTER SYSTEMS, LTD" },
    { "003026", "HeiTel Digital Video GmbH" },
    { "003027", "KERBANGO, INC." },
    { "003028", "FASE Saldatura srl" },
    { "003029", "OPICOM" },
    { "00302A", "SOUTHERN INFORMATION" },
    { "00302B", "Inalp Solutions AG" },
    { "00302C", "SYLANTRO SYSTEMS CORPORATION" },
    { "00302D", "QUANTUM BRIDGE COMMUNICATIONS" },
    { "00302E", "Hoft & Wessel AG" },
    { "00302F", "GE Aviation System" },
    { "003030", "HARMONIX CORPORATION" },
    { "003031", "LIGHTWAVE COMMUNICATIONS, INC." },
    { "003032", "MagicRam, Inc." },
    { "003033", "ORIENT TELECOM CO., LTD." },
    { "003034", "SET ENGINEERING" },
    { "003035", "Corning Incorporated" },
    { "003036", "RMP ELEKTRONIKSYSTEME GMBH" },
    { "003037", "Packard Bell Nec Services" },
    { "003038", "XCP, INC." },
    { "003039", "SOFTBOOK PRESS" },
    { "00303A", "MAATEL" },
    { "00303B", "PowerCom Technology" },
    { "00303C", "ONNTO CORP." },
    { "00303D", "IVA CORPORATION" },
    { "00303E", "Radcom Ltd." },
    { "00303F", "TurboComm Tech Inc." },
    { "003040", "Cisco Systems, Inc" },
    { "003041", "SAEJIN T & M CO., LTD." },
    { "003042", "DeTeWe-Deutsche Telephonwerke" },
    { "003043", "IDREAM TECHNOLOGIES, PTE. LTD." },
    { "003044", "CradlePoint, Inc" },
    { "003045", "Village Networks, Inc. (VNI)" },
    { "003046", "Controlled Electronic Manageme" },
    { "003047", "NISSEI ELECTRIC CO., LTD." },
    { "003048", "Super Micro Computer, Inc." },
    { "003049", "BRYANT TECHNOLOGY, LTD." },
    { "00304A", "Fraunhofer IPMS" },
    { "00304B", "ORBACOM SYSTEMS, INC." },
    { "00304C", "APPIAN COMMUNICATIONS, INC." },
    { "00304D", "ESI" },
    { "00304E", "BUSTEC PRODUCTION LTD." },
    { "00304F", "PLANET Technology Corporation" },
    { "003050", "Versa Technology" },
    { "003051", "ORBIT AVIONIC & COMMUNICATION" },
    { "003052", "Zhone Technologies, Inc." },
    { "003053", "Basler AG" },
    { "003054", "Castlenet Technology Inc." },
    { "003055", "Renesas Technology America, Inc." },
    { "003056", "HMS Industrial Networks" },
    { "003057", "QTelNet, Inc." },
    { "003058", "API MOTION" },
    { "003059", "Kontron Europe GmbH" },
    { "00305A", "TELGEN CORPORATION" },
    { "00305B", "Toko Inc." },
    { "00305C", "SMAR Laboratories Corp." },
    { "00305D", "DIGITRA SYSTEMS, INC." },
    { "00305E", "Abelko Innovation" },
    { "00305F", "Hasselblad" },
    { "003060", "Powerfile, Inc." },
    { "003061", "MobyTEL" },
    { "003062", "IP Video Networks Inc" },
    { "003063", "SANTERA SYSTEMS, INC." },
    { "003064", "ADLINK TECHNOLOGY, INC." },
    { "003065", "Apple, Inc." },
    { "003066", "RFM" },
    { "003067", "BIOSTAR Microtech Int'l Corp." },
    { "003068", "CYBERNETICS TECH. CO., LTD." },
    { "003069", "IMPACCT TECHNOLOGY CORP." },
    { "00306A", "PENTA MEDIA CO., LTD." },
    { "00306B", "CMOS SYSTEMS, INC." },
    { "00306C", "Hitex Holding GmbH" },
    { "00306D", "LUCENT TECHNOLOGIES" },
    { "00306E", "Hewlett Packard" },
    { "00306F", "SEYEON TECH. CO., LTD." },
    { "003070", "1Net Corporation" },
    { "003071", "Cisco Systems, Inc" },
    { "003072", "Intellibyte Inc." },
    { "003073", "International Microsystems, In" },
    { "003074", "EQUIINET LTD." },
    { "003075", "ADTECH" },
    { "003076", "Akamba Corporation" },
    { "003077", "ONPREM NETWORKS" },
    { "003078", "Cisco Systems, Inc" },
    { "003079", "CQOS, INC." },
    { "00307A", "Advanced Technology & Systems" },
    { "00307B", "Cisco Systems, Inc" },
    { "00307C", "ADID SA" },
    { "00307D", "GRE AMERICA, INC." },
    { "00307E", "Redflex Communication Systems" },
    { "00307F", "IRLAN LTD." },
    { "003080", "Cisco Systems, Inc" },
    { "003081", "ALTOS C&C" },
    { "003082", "TAIHAN ELECTRIC WIRE CO., LTD." },
    { "003083", "Ivron Systems" },
    { "003084", "ALLIED TELESYN INTERNAIONAL" },
    { "003085", "Cisco Systems, Inc" },
    { "003086", "Transistor Devices, Inc." },
    { "003087", "VEGA GRIESHABER KG" },
    { "003088", "Ericsson" },
    { "003089", "Spectrapoint Wireless, LLC" },
    { "00308A", "NICOTRA SISTEMI S.P.A" },
    { "00308B", "Brix Networks" },
    { "00308C", "Quantum Corporation" },
    { "00308D", "Pinnacle Systems, Inc." },
    { "00308E", "Crossmatch Technologies/HID Global" },
    { "00308F", "MICRILOR, Inc." },
    { "003090", "CYRA TECHNOLOGIES, INC." },
    { "003091", "TAIWAN FIRST LINE ELEC. CORP." },
    { "003092", "Kontron Electronics AG" },
    { "003093", "Sonnet Technologies, Inc" },
    { "003094", "Cisco Systems, Inc" },
    { "003095", "Procomp Informatics, Ltd." },
    { "003096", "Cisco Systems, Inc" },
    { "003097", "AB Regin" },
    { "003098", "Global Converging Technologies" },
    { "003099", "BOENIG UND KALLENBACH OHG" },
    { "00309A", "ASTRO TERRA CORP." },
    { "00309B", "Smartware" },
    { "00309C", "Timing Applications, Inc." },
    { "00309D", "Nimble Microsystems, Inc." },
    { "00309E", "WORKBIT CORPORATION." },
    { "00309F", "AMBER NETWORKS" },
    { "0030A0", "TYCO SUBMARINE SYSTEMS, LTD." },
    { "0030A1", "WEBGATE Inc." },
    { "0030A2", "Lightner Engineering" },
    { "0030A3", "Cisco Systems, Inc" },
    { "0030A4", "Woodwind Communications System" },
    { "0030A5", "ACTIVE POWER" },
    { "0030A6", "VIANET TECHNOLOGIES, LTD." },
    { "0030A7", "SCHWEITZER ENGINEERING" },
    { "0030A8", "OL'E COMMUNICATIONS, INC." },
    { "0030A9", "Netiverse, Inc." },
    { "0030AA", "AXUS MICROSYSTEMS, INC." },
    { "0030AB", "DELTA NETWORKS, INC." },
    { "0030AC", "Systeme Lauer GmbH & Co., Ltd." },
    { "0030AD", "SHANGHAI COMMUNICATION" },
    { "0030AE", "Times N System, Inc." },
    { "0030AF", "Honeywell GmbH" },
    { "0030B0", "Convergenet Technologies" },
    { "0030B1", "TrunkNet" },
    { "0030B2", "L-3 Sonoma EO" },
    { "0030B3", "San Valley Systems, Inc." },
    { "0030B4", "INTERSIL CORP." },
    { "0030B5", "Tadiran Microwave Networks" },
    { "0030B6", "Cisco Systems, Inc" },
    { "0030B7", "Teletrol Systems, Inc." },
    { "0030B8", "RiverDelta Networks" },
    { "0030B9", "ECTEL" },
    { "0030BA", "AC&T SYSTEM CO., LTD." },
    { "0030BB", "CacheFlow, Inc." },
    { "0030BC", "Optronic AG" },
    { "0030BD", "BELKIN COMPONENTS" },
    { "0030BE", "City-Net Technology, Inc." },
    { "0030BF", "MULTIDATA GMBH" },
    { "0030C0", "Lara Technology, Inc." },
    { "0030C1", "Hewlett Packard" },
    { "0030C2", "COMONE" },
    { "0030C3", "FLUECKIGER ELEKTRONIK AG" },
    { "0030C4", "Canon Imaging Systems Inc." },
    { "0030C5", "CADENCE DESIGN SYSTEMS, INC." },
    { "0030C6", "CONTROL SOLUTIONS, INC." },
    { "0030C7", "Macromate Corp." },
    { "0030C8", "GAD LINE, LTD." },
    { "0030C9", "LuxN, N" },
    { "0030CA", "Discovery Com" },
    { "0030CB", "OMNI FLOW COMPUTERS, INC." },
    { "0030CC", "Tenor Networks, Inc." },
    { "0030CD", "CONEXANT SYSTEMS, INC." },
    { "0030CE", "Zaffire" },
    { "0030CF", "TWO TECHNOLOGIES, INC." },
    { "0030D0", "Tellabs" },
    { "0030D1", "INOVA CORPORATION" },
    { "0030D2", "WIN TECHNOLOGIES, CO., LTD." },
    { "0030D3", "Agilent Technologies, Inc." },
    { "0030D4", "AAE Systems, Inc." },
    { "0030D5", "DResearch GmbH" },
    { "0030D6", "MSC VERTRIEBS GMBH" },
    { "0030D7", "Innovative Systems, L.L.C." },
    { "0030D8", "SITEK" },
    { "0030D9", "DATACORE SOFTWARE CORP." },
    { "0030DA", "Comtrend Corporation" },
    { "0030DB", "Mindready Solutions, Inc." },
    { "0030DC", "RIGHTECH CORPORATION" },
    { "0030DD", "INDIGITA CORPORATION" },
    { "0030DE", "WAGO Kontakttechnik GmbH" },
    { "0030DF", "KB/TEL TELECOMUNICACIONES" },
    { "0030E0", "OXFORD SEMICONDUCTOR LTD." },
    { "0030E1", "Network Equipment Technologies, Inc." },
    { "0030E2", "GARNET SYSTEMS CO., LTD." },
    { "0030E3", "SEDONA NETWORKS CORP." },
    { "0030E4", "CHIYODA SYSTEM RIKEN" },
    { "0030E5", "Amper Datos S.A." },
    { "0030E6", "Draeger Medical Systems, Inc." },
    { "0030E7", "CNF MOBILE SOLUTIONS, INC." },
    { "0030E8", "ENSIM CORP." },
    { "0030E9", "GMA COMMUNICATION MANUFACT'G" },
    { "0030EA", "TeraForce Technology Corporation" },
    { "0030EB", "TURBONET COMMUNICATIONS, INC." },
    { "0030EC", "BORGARDT" },
    { "0030ED", "Expert Magnetics Corp." },
    { "0030EE", "DSG Technology, Inc." },
    { "0030EF", "NEON TECHNOLOGY, INC." },
    { "0030F0", "Uniform Industrial Corp." },
    { "0030F1", "Accton Technology Corp" },
    { "0030F2", "Cisco Systems, Inc" },
    { "0030F3", "At Work Computers" },
    { "0030F4", "STARDOT TECHNOLOGIES" },
    { "0030F5", "Wild Lab. Ltd." },
    { "0030F6", "SECURELOGIX CORPORATION" },
    { "0030F7", "RAMIX INC." },
    { "0030F8", "Dynapro Systems, Inc." },
    { "0030F9", "Sollae Systems Co., Ltd." },
    { "0030FA", "TELICA, INC." },
    { "0030FB", "AZS Technology AG" },
    { "0030FC", "Terawave Communications, Inc." },
    { "0030FD", "INTEGRATED SYSTEMS DESIGN" },
    { "0030FE", "DSA GmbH" },
    { "0030FF", "DataFab Systems Inc." },
    { "003126", "Nokia" },
    { "003146", "Juniper Networks" },
    { "003192", "TP-Link Systems Inc" },
    { "003217", "Cisco Systems, Inc" },
    { "00323A", "so-logic" },
    { "003358", "Ruckus Wireless" },
    { "00336C", "SynapSense Corporation" },
    { "00337A", "Tuya Smart Inc." },
    { "0034A1", "RF-LAMBDA USA INC." },
    { "0034DA", "LG Electronics (Mobile Communications)" },
    { "0034F1", "Radicom Research, Inc." },
    { "0034FE", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "00351A", "Cisco Systems, Inc" },
    { "003532", "Electro-Metrics Corporation" },
    { "003560", "Rosen Aviation" },
    { "0035FF", "Texas Instruments" },
    { "003676", "Commscope" },
    { "0036BE", "Northwest Towers" },
    { "0036D7", "Keltron IOT Corp." },
    { "0036F8", "Conti Temic microelectronic GmbH" },
    { "0036FE", "SuperVision" },
    { "00376D", "Murata Manufacturing Co., Ltd." },
    { "0037B7", "Sagemcom Broadband SAS" },
    { "0038DF", "Cisco Systems, Inc" },
    { "003969", "Air-Weigh Incorporated" },
    { "003A7D", "Cisco Systems, Inc" },
    { "003A98", "Cisco Systems, Inc" },
    { "003A99", "Cisco Systems, Inc" },
    { "003A9A", "Cisco Systems, Inc" },
    { "003A9B", "Cisco Systems, Inc" },
    { "003A9C", "Cisco Systems, Inc" },
    { "003A9D", "NEC Platforms, Ltd." },
    { "003AAF", "BlueBit Ltd." },
    { "003C10", "Cisco Systems, Inc" },
    { "003C84", "Silicon Laboratories" },
    { "003CC5", "WONWOO Engineering Co., Ltd" },
    { "003D41", "Hatteland Computer AS" },
    { "003DE1", "Huawei Device Co., Ltd." },
    { "003DE8", "LG Electronics (Mobile Communications)" },
    { "003E73", "Mist Systems, Inc." },
    { "003EE1", "Apple, Inc." },
    { "003F10", "Shenzhen GainStrong Technology Co., Ltd." },
    { "004000", "PCI COMPONENTES DA AMZONIA LTD" },
    { "004001", "Zero One Technology Co. Ltd." },
    { "004002", "PERLE SYSTEMS LIMITED" },
    { "004003", "Emerson Process Management Power & Water Solutions, Inc." },
    { "004004", "ICM CO. LTD." },
    { "004005", "ANI COMMUNICATIONS INC." },
    { "004006", "SAMPO TECHNOLOGY CORPORATION" },
    { "004007", "TELMAT INFORMATIQUE" },
    { "004008", "A PLUS INFO CORPORATION" },
    { "004009", "TACHIBANA TECTRON CO., LTD." },
    { "00400A", "PIVOTAL TECHNOLOGIES, INC." },
    { "00400B", "Cisco Systems, Inc" },
    { "00400C", "GENERAL MICRO SYSTEMS, INC." },
    { "00400D", "LANNET DATA COMMUNICATIONS,LTD" },
    { "00400E", "MEMOTEC, INC." },
    { "00400F", "DATACOM TECHNOLOGIES" },
    { "004010", "SONIC SYSTEMS, INC." },
    { "004011", "ANDOVER CONTROLS CORPORATION" },
    { "004012", "WINDATA, INC." },
    { "004013", "NTT DATA COMM. SYSTEMS CORP." },
    { "004014", "COMSOFT GMBH" },
    { "004015", "ASCOM INFRASYS AG" },
    { "004016", "ADC - Global Connectivity Solutions Division" },
    { "004017", "Silex Technology America" },
    { "004018", "ADOBE SYSTEMS, INC." },
    { "004019", "AEON SYSTEMS, INC." },
    { "00401A", "FUJI ELECTRIC CO., LTD." },
    { "00401B", "PRINTER SYSTEMS CORP." },
    { "00401C", "AST RESEARCH, INC." },
    { "00401D", "INVISIBLE SOFTWARE, INC." },
    { "00401E", "ICC" },
    { "00401F", "COLORGRAPH LTD" },
    { "004020", "CommScope Inc" },
    { "004021", "RASTER GRAPHICS" },
    { "004022", "KLEVER COMPUTERS, INC." },
    { "004023", "LOGIC CORPORATION" },
    { "004024", "COMPAC INC." },
    { "004025", "MOLECULAR DYNAMICS" },
    { "004026", "BUFFALO.INC" },
    { "004027", "SMC MASSACHUSETTS, INC." },
    { "004028", "NETCOMM LIMITED" },
    { "004029", "Compex" },
    { "00402A", "Canoga Perkins Corporation" },
    { "00402B", "TRIGEM COMPUTER, INC." },
    { "00402C", "ISIS DISTRIBUTED SYSTEMS, INC." },
    { "00402D", "HARRIS ADACOM CORPORATION" },
    { "00402E", "PRECISION SOFTWARE, INC." },
    { "00402F", "XLNT DESIGNS INC." },
    { "004030", "GK COMPUTER" },
    { "004031", "KOKUSAI DENKI Electric Inc." },
    { "004032", "DIGITAL COMMUNICATIONS" },
    { "004033", "ADDTRON TECHNOLOGY CO., LTD." },
    { "004034", "BUSTEK CORPORATION" },
    { "004035", "OPCOM" },
    { "004036", "Minim Inc." },
    { "004037", "SEA-ILAN, INC." },
    { "004038", "TALENT ELECTRIC INCORPORATED" },
    { "004039", "OPTEC DAIICHI DENKO CO., LTD." },
    { "00403A", "IMPACT TECHNOLOGIES" },
    { "00403B", "SYNERJET INTERNATIONAL CORP." },
    { "00403C", "FORKS, INC." },
    { "00403D", "Teradata Corporation" },
    { "00403E", "RASTER OPS CORPORATION" },
    { "00403F", "SSANGYONG COMPUTER SYSTEMS" },
    { "004040", "RING ACCESS, INC." },
    { "004041", "FUJIKURA LTD." },
    { "004042", "N.A.T. GMBH" },
    { "004043", "Nokia Siemens Networks GmbH & Co. KG." },
    { "004044", "QNIX COMPUTER CO., LTD." },
    { "004045", "TWINHEAD CORPORATION" },
    { "004046", "UDC RESEARCH LIMITED" },
    { "004047", "WIND RIVER SYSTEMS" },
    { "004048", "SMD INFORMATICA S.A." },
    { "004049", "Roche Diagnostics International Ltd." },
    { "00404A", "WEST AUSTRALIAN DEPARTMENT" },
    { "00404B", "MAPLE COMPUTER SYSTEMS" },
    { "00404C", "HYPERTEC PTY LTD." },
    { "00404D", "TELECOMMUNICATIONS TECHNIQUES" },
    { "00404E", "FLUENT, INC." },
    { "00404F", "SPACE & NAVAL WARFARE SYSTEMS" },
    { "004050", "IRONICS, INCORPORATED" },
    { "004051", "Garbee and Garbee" },
    { "004052", "STAR TECHNOLOGIES, INC." },
    { "004053", "AMPRO COMPUTERS" },
    { "004054", "CONNECTION MACHINES SERVICES" },
    { "004055", "METRONIX GMBH" },
    { "004056", "MCM JAPAN LTD." },
    { "004057", "LOCKHEED - SANDERS" },
    { "004058", "UKG" },
    { "004059", "YOSHIDA KOGYO K. K." },
    { "00405A", "GOLDSTAR INFORMATION & COMM." },
    { "00405B", "FUNASSET LIMITED" },
    { "00405C", "FUTURE SYSTEMS, INC." },
    { "00405D", "STAR-TEK, INC." },
    { "00405E", "NORTH HILLS ISRAEL" },
    { "00405F", "AFE COMPUTERS LTD." },
    { "004060", "COMENDEC LTD" },
    { "004061", "DATATECH ENTERPRISES CO., LTD." },
    { "004062", "E-SYSTEMS, INC./GARLAND DIV." },
    { "004063", "VIA TECHNOLOGIES, INC." },
    { "004064", "KLA INSTRUMENTS CORPORATION" },
    { "004065", "GTE SPACENET" },
    { "004066", "APRESIA Systems Ltd" },
    { "004067", "OMNIBYTE CORPORATION" },
    { "004068", "EXTENDED SYSTEMS" },
    { "004069", "LEMCOM SYSTEMS, INC." },
    { "00406A", "KENTEK INFORMATION SYSTEMS,INC" },
    { "00406B", "SYSGEN" },
    { "00406C", "COPERNIQUE" },
    { "00406D", "LANCO, INC." },
    { "00406E", "COROLLARY, INC." },
    { "00406F", "SYNC RESEARCH INC." },
    { "004070", "INTERWARE CO., LTD." },
    { "004071", "ATM COMPUTER GMBH" },
    { "004072", "Applied Innovation Inc." },
    { "004073", "BASS ASSOCIATES" },
    { "004074", "CABLE AND WIRELESS" },
    { "004075", "Tattile SRL " },
    { "004076", "Sun Conversion Technologies" },
    { "004077", "MAXTON TECHNOLOGY CORPORATION" },
    { "004078", "WEARNES AUTOMATION PTE LTD" },
    { "004079", "JUKO MANUFACTURE COMPANY, LTD." },
    { "00407A", "SOCIETE D'EXPLOITATION DU CNIT" },
    { "00407B", "SCIENTIFIC ATLANTA" },
    { "00407C", "QUME CORPORATION" },
    { "00407D", "EXTENSION TECHNOLOGY CORP." },
    { "00407E", "EVERGREEN SYSTEMS, INC." },
    { "00407F", "FLIR Systems" },
    { "004080", "ATHENIX CORPORATION" },
    { "004081", "MANNESMANN SCANGRAPHIC GMBH" },
    { "004082", "LABORATORY EQUIPMENT CORP." },
    { "004083", "TDA INDUSTRIA DE PRODUTOS" },
    { "004084", "Honeywell" },
    { "004085", "SAAB INSTRUMENTS AB" },
    { "004086", "MICHELS & KLEBERHOFF COMPUTER" },
    { "004087", "UBITREX CORPORATION" },
    { "004088", "MOBIUS TECHNOLOGIES, INC." },
    { "004089", "MEIDENSHA CORPORATION" },
    { "00408A", "TPS TELEPROCESSING SYS. GMBH" },
    { "00408B", "RAYLAN CORPORATION" },
    { "00408C", "Axis Communications AB" },
    { "00408D", "THE GOODYEAR TIRE & RUBBER CO." },
    { "00408E", "Tattile SRL " },
    { "00408F", "WM-DATA MINFO AB" },
    { "004090", "ANSEL COMMUNICATIONS" },
    { "004091", "PROCOMP INDUSTRIA ELETRONICA" },
    { "004092", "ASP COMPUTER PRODUCTS, INC." },
    { "004093", "PAXDATA NETWORKS LTD." },
    { "004094", "SHOGRAPHICS, INC." },
    { "004095", "R.P.T. INTERGROUPS INT'L LTD." },
    { "004096", "Cisco Systems, Inc" },
    { "004097", "DATEX DIVISION OF" },
    { "004098", "DRESSLER GMBH & CO." },
    { "004099", "NEWGEN SYSTEMS CORP." },
    { "00409A", "NETWORK EXPRESS, INC." },
    { "00409B", "HAL COMPUTER SYSTEMS INC." },
    { "00409C", "TRANSWARE" },
    { "00409D", "DigiBoard" },
    { "00409E", "Concurrent Technologies Ltd." },
    { "00409F", "Telco Systems, Inc. " },
    { "0040A0", "GOLDSTAR CO., LTD." },
    { "0040A1", "ERGO COMPUTING" },
    { "0040A2", "KINGSTAR TECHNOLOGY INC." },
    { "0040A3", "MICROUNITY SYSTEMS ENGINEERING" },
    { "0040A4", "ROSE ELECTRONICS" },
    { "0040A5", "CLINICOMP INTL." },
    { "0040A6", "Cray, Inc." },
    { "0040A7", "ITAUTEC PHILCO S.A." },
    { "0040A8", "IMF INTERNATIONAL LTD." },
    { "0040A9", "DATACOM INC." },
    { "0040AA", "Valmet Automation" },
    { "0040AB", "ROLAND DG CORPORATION" },
    { "0040AC", "SUPER WORKSTATION, INC." },
    { "0040AD", "SMA REGELSYSTEME GMBH" },
    { "0040AE", "DELTA CONTROLS, INC." },
    { "0040AF", "DIGITAL PRODUCTS, INC." },
    { "0040B0", "BYTEX CORPORATION, ENGINEERING" },
    { "0040B1", "CODONICS INC." },
    { "0040B2", "SYSTEMFORSCHUNG" },
    { "0040B3", "ParTech Inc." },
    { "0040B4", "NEXTCOM K.K." },
    { "0040B5", "VIDEO TECHNOLOGY COMPUTERS LTD" },
    { "0040B6", "COMPUTERM CORPORATION" },
    { "0040B7", "STEALTH COMPUTER SYSTEMS" },
    { "0040B8", "IDEA ASSOCIATES" },
    { "0040B9", "MACQ ELECTRONIQUE SA" },
    { "0040BA", "ALLIANT COMPUTER SYSTEMS CORP." },
    { "0040BB", "GOLDSTAR CABLE CO., LTD." },
    { "0040BC", "ALGORITHMICS LTD." },
    { "0040BD", "STARLIGHT NETWORKS, INC." },
    { "0040BE", "BOEING DEFENSE & SPACE" },
    { "0040BF", "CHANNEL SYSTEMS INTERN'L INC." },
    { "0040C0", "VISTA CONTROLS CORPORATION" },
    { "0040C1", "BIZERBA-WERKE WILHEIM KRAUT" },
    { "0040C2", "APPLIED COMPUTING DEVICES" },
    { "0040C3", "FISCHER AND PORTER CO." },
    { "0040C4", "KINKEI SYSTEM CORPORATION" },
    { "0040C5", "MICOM COMMUNICATIONS INC." },
    { "0040C6", "FIBERNET RESEARCH, INC." },
    { "0040C7", "RUBY TECH CORPORATION" },
    { "0040C8", "MILAN TECHNOLOGY CORPORATION" },
    { "0040C9", "NCUBE" },
    { "0040CA", "FIRST INTERNAT'L COMPUTER, INC" },
    { "0040CB", "LANWAN TECHNOLOGIES" },
    { "0040CC", "SILCOM MANUF'G TECHNOLOGY INC." },
    { "0040CD", "TERA MICROSYSTEMS, INC." },
    { "0040CE", "NET-SOURCE, INC." },
    { "0040CF", "STRAWBERRY TREE, INC." },
    { "0040D0", "MITAC INTERNATIONAL CORP." },
    { "0040D1", "FUKUDA DENSHI CO., LTD." },
    { "0040D2", "PAGINE CORPORATION" },
    { "0040D3", "KIMPSION INTERNATIONAL CORP." },
    { "0040D4", "GAGE TALKER CORP." },
    { "0040D5", "Sartorius Mechatronics T&H GmbH " },
    { "0040D6", "LOCAMATION B.V." },
    { "0040D7", "STUDIO GEN INC." },
    { "0040D8", "OCEAN OFFICE AUTOMATION LTD." },
    { "0040D9", "AMERICAN MEGATRENDS INC." },
    { "0040DA", "TELSPEC LTD" },
    { "0040DB", "ADVANCED TECHNICAL SOLUTIONS" },
    { "0040DC", "TRITEC ELECTRONIC GMBH" },
    { "0040DD", "HONG TECHNOLOGIES" },
    { "0040DE", "Elsag Datamat spa" },
    { "0040DF", "DIGALOG SYSTEMS, INC." },
    { "0040E0", "ATOMWIDE LTD." },
    { "0040E1", "MARNER INTERNATIONAL, INC." },
    { "0040E2", "MESA RIDGE TECHNOLOGIES, INC." },
    { "0040E3", "QUIN SYSTEMS LTD" },
    { "0040E4", "E-M TECHNOLOGY, INC." },
    { "0040E5", "SYBUS CORPORATION" },
    { "0040E6", "C.A.E.N." },
    { "0040E7", "ARNOS INSTRUMENTS & COMPUTER" },
    { "0040E8", "CHARLES RIVER DATA SYSTEMS,INC" },
    { "0040E9", "ACCORD SYSTEMS, INC." },
    { "0040EA", "PLAIN TREE SYSTEMS INC" },
    { "0040EB", "MARTIN MARIETTA CORPORATION" },
    { "0040EC", "MIKASA SYSTEM ENGINEERING" },
    { "0040ED", "NETWORK CONTROLS INT'NATL INC." },
    { "0040EE", "OPTIMEM" },
    { "0040EF", "HYPERCOM, INC." },
    { "0040F0", "MicroBrain,Inc." },
    { "0040F1", "CHUO ELECTRONICS CO., LTD." },
    { "0040F2", "JANICH & KLASS COMPUTERTECHNIK" },
    { "0040F3", "NETCOR" },
    { "0040F4", "CAMEO COMMUNICATIONS, INC." },
    { "0040F5", "OEM ENGINES" },
    { "0040F6", "KATRON COMPUTERS INC." },
    { "0040F7", "Polaroid Corporation" },
    { "0040F8", "SYSTEMHAUS DISCOM" },
    { "0040F9", "COMBINET" },
    { "0040FA", "MICROBOARDS, INC." },
    { "0040FB", "CASCADE COMMUNICATIONS" },
    { "0040FC", "IBR COMPUTER TECHNIK GMBH" },
    { "0040FD", "LXE" },
    { "0040FE", "SYMPLEX COMMUNICATIONS" },
    { "0040FF", "TELEBIT CORPORATION" },
    { "00410E", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "0041B4", "Wuxi Zhongxing Optoelectronics Technology Co.,Ltd." },
    { "0041D2", "Cisco Systems, Inc" },
    { "004238", "Intel Corporate" },
    { "004252", "RLX Technologies" },
    { "00425A", "Cisco Systems, Inc" },
    { "004268", "Cisco Systems, Inc" },
    { "004279", "Sunitec Enterprise Co.,Ltd" },
    { "0043FF", "KETRON S.R.L." },
    { "004501", "Midmark RTLS" },
    { "00451D", "Cisco Systems, Inc" },
    { "0045E2", "CyberTAN Technology Inc." },
    { "00464B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "004A77", "zte corporation" },
    { "004B0D", "Huawei Device Co., Ltd." },
    { "004B12", "Espressif Inc." },
    { "004BF3", "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "004CE5", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "004D32", "Andon Health Co.,Ltd." },
    { "004E01", "Dell Inc." },
    { "004E35", "Hewlett Packard Enterprise " },
    { "004F1A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "005000", "NEXO COMMUNICATIONS, INC." },
    { "005001", "YAMASHITA SYSTEMS CORP." },
    { "005002", "OMNISEC AG" },
    { "005003", "Xrite Inc" },
    { "005004", "3COM" },
    { "005006", "TAC AB" },
    { "005007", "SIEMENS TELECOMMUNICATION SYSTEMS LIMITED" },
    { "005008", "TIVA MICROCOMPUTER CORP. (TMC)" },
    { "005009", "PHILIPS BROADBAND NETWORKS" },
    { "00500A", "IRIS TECHNOLOGIES, INC." },
    { "00500B", "Cisco Systems, Inc" },
    { "00500C", "e-Tek Labs, Inc." },
    { "00500D", "SATORI ELECTORIC CO., LTD." },
    { "00500E", "CHROMATIS NETWORKS, INC." },
    { "00500F", "Cisco Systems, Inc" },
    { "005010", "NovaNET Learning, Inc." },
    { "005012", "CBL - GMBH" },
    { "005013", "Seagate Cloud Systems Inc" },
    { "005014", "Cisco Systems, Inc" },
    { "005015", "BRIGHT STAR ENGINEERING" },
    { "005016", "Molex Canada Ltd" },
    { "005017", "RSR S.R.L." },
    { "005018", "AMIT, Inc." },
    { "005019", "SPRING TIDE NETWORKS, INC." },
    { "00501A", "IQinVision" },
    { "00501B", "ABL CANADA, INC." },
    { "00501C", "JATOM SYSTEMS, INC." },
    { "00501E", "Grass Valley, A Belden Brand" },
    { "00501F", "MRG SYSTEMS, LTD." },
    { "005020", "MEDIASTAR CO., LTD." },
    { "005021", "EIS INTERNATIONAL, INC." },
    { "005022", "ZONET TECHNOLOGY, INC." },
    { "005023", "PG DESIGN ELECTRONICS, INC." },
    { "005024", "NAVIC SYSTEMS, INC." },
    { "005026", "COSYSTEMS, INC." },
    { "005027", "GENICOM CORPORATION" },
    { "005028", "AVAL COMMUNICATIONS" },
    { "005029", "1394 PRINTER WORKING GROUP" },
    { "00502A", "Cisco Systems, Inc" },
    { "00502B", "GENRAD LTD." },
    { "00502C", "SOYO COMPUTER, INC." },
    { "00502D", "ACCEL, INC." },
    { "00502E", "CAMBEX CORPORATION" },
    { "00502F", "TollBridge Technologies, Inc." },
    { "005030", "FUTURE PLUS SYSTEMS" },
    { "005031", "AEROFLEX LABORATORIES, INC." },
    { "005032", "PICAZO COMMUNICATIONS, INC." },
    { "005033", "MAYAN NETWORKS" },
    { "005036", "NETCAM, LTD." },
    { "005037", "KOGA ELECTRONICS CO." },
    { "005038", "DAIN TELECOM CO., LTD." },
    { "005039", "MARINER NETWORKS" },
    { "00503A", "DATONG ELECTRONICS LTD." },
    { "00503B", "MEDIAFIRE CORPORATION" },
    { "00503C", "TSINGHUA NOVEL ELECTRONICS" },
    { "00503E", "Cisco Systems, Inc" },
    { "00503F", "ANCHOR GAMES" },
    { "005040", "Panasonic Electric Works Co., Ltd." },
    { "005041", "Coretronic Corporation" },
    { "005042", "SCI MANUFACTURING SINGAPORE PTE, LTD." },
    { "005043", "MARVELL SEMICONDUCTOR, INC." },
    { "005044", "ASACA CORPORATION" },
    { "005045", "RIOWORKS SOLUTIONS, INC." },
    { "005046", "MENICX INTERNATIONAL CO., LTD." },
    { "005047", "Private" },
    { "005048", "INFOLIBRIA" },
    { "005049", "Arbor Networks Inc" },
    { "00504A", "ELTECO A.S." },
    { "00504B", "BARCONET N.V." },
    { "00504C", "Galil Motion Control" },
    { "00504D", "Tokyo Electron Device Limited" },
    { "00504E", "SIERRA MONITOR CORP." },
    { "00504F", "OLENCOM ELECTRONICS" },
    { "005050", "Cisco Systems, Inc" },
    { "005051", "IWATSU ELECTRIC CO., LTD." },
    { "005052", "TIARA NETWORKS, INC." },
    { "005053", "Cisco Systems, Inc" },
    { "005054", "Cisco Systems, Inc" },
    { "005055", "DOMS A/S" },
    { "005056", "VMware, Inc." },
    { "005057", "BROADBAND ACCESS SYSTEMS" },
    { "005058", "Sangoma Technologies" },
    { "005059", "iBAHN" },
    { "00505A", "NETWORK ALCHEMY, INC." },
    { "00505B", "KAWASAKI LSI U.S.A., INC." },
    { "00505C", "TUNDO CORPORATION" },
    { "00505E", "DIGITEK MICROLOGIC S.A." },
    { "00505F", "BRAND INNOVATORS" },
    { "005060", "TANDBERG TELECOM AS" },
    { "005062", "KOUWELL ELECTRONICS CORP. **" },
    { "005063", "OY COMSEL SYSTEM AB" },
    { "005064", "CAE ELECTRONICS" },
    { "005065", "TDK-Lambda Corporation" },
    { "005066", "AtecoM GmbH advanced telecomunication modules" },
    { "005067", "AEROCOMM, INC." },
    { "005068", "ELECTRONIC INDUSTRIES ASSOCIATION" },
    { "005069", "PixStream Incorporated" },
    { "00506A", "EDEVA, INC." },
    { "00506B", "SPX-ATEG" },
    { "00506C", "Beijer Electronics Products AB" },
    { "00506D", "VIDEOJET SYSTEMS" },
    { "00506E", "CORDER ENGINEERING CORPORATION" },
    { "00506F", "G-CONNECT" },
    { "005070", "CHAINTECH COMPUTER CO., LTD." },
    { "005071", "AIWA CO., LTD." },
    { "005072", "CORVIS CORPORATION" },
    { "005073", "Cisco Systems, Inc" },
    { "005074", "ADVANCED HI-TECH CORP." },
    { "005075", "KESTREL SOLUTIONS" },
    { "005076", "IBM Corp" },
    { "005077", "PROLIFIC TECHNOLOGY, INC." },
    { "005078", "MEGATON HOUSE, LTD." },
    { "005079", "Private" },
    { "00507A", "XPEED, INC." },
    { "00507B", "MERLOT COMMUNICATIONS" },
    { "00507C", "VIDEOCON AG" },
    { "00507D", "IFP" },
    { "00507E", "NEWER TECHNOLOGY" },
    { "00507F", "DrayTek Corp." },
    { "005080", "Cisco Systems, Inc" },
    { "005081", "MURATA MACHINERY, LTD." },
    { "005082", "FORESSON CORPORATION" },
    { "005083", "GILBARCO, INC." },
    { "005084", "Quantum Corp." },
    { "005086", "TELKOM SA, LTD." },
    { "005087", "TERASAKI ELECTRIC CO., LTD." },
    { "005088", "AMANO CORPORATION" },
    { "005089", "SAFETY MANAGEMENT SYSTEMS" },
    { "00508B", "Hewlett Packard" },
    { "00508C", "RSI SYSTEMS" },
    { "00508D", "ABIT COMPUTER CORPORATION" },
    { "00508E", "OPTIMATION, INC." },
    { "00508F", "ASITA TECHNOLOGIES INT'L LTD." },
    { "005090", "DCTRI" },
    { "005091", "NETACCESS, INC." },
    { "005092", "Rigaku Corporation Osaka Plant" },
    { "005093", "BOEING" },
    { "005094", "Commscope" },
    { "005095", "PERACOM NETWORKS" },
    { "005096", "SALIX TECHNOLOGIES, INC." },
    { "005097", "MMC-EMBEDDED COMPUTERTECHNIK GmbH" },
    { "005098", "GLOBALOOP, LTD." },
    { "005099", "3COM EUROPE LTD" },
    { "00509A", "TAG ELECTRONIC SYSTEMS" },
    { "00509B", "SWITCHCORE AB" },
    { "00509C", "BETA RESEARCH" },
    { "00509D", "THE INDUSTREE B.V." },
    { "00509E", "Les Technologies SoftAcoustik Inc." },
    { "00509F", "HORIZON COMPUTER" },
    { "0050A0", "DELTA COMPUTER SYSTEMS, INC." },
    { "0050A1", "CARLO GAVAZZI, INC." },
    { "0050A2", "Cisco Systems, Inc" },
    { "0050A3", "TransMedia Communications, Inc." },
    { "0050A4", "IO TECH, INC." },
    { "0050A5", "CAPITOL BUSINESS SYSTEMS, LTD." },
    { "0050A6", "OPTRONICS" },
    { "0050A7", "Cisco Systems, Inc" },
    { "0050A8", "OpenCon Systems, Inc." },
    { "0050A9", "MOLDAT WIRELESS TECHNOLGIES" },
    { "0050AA", "KONICA MINOLTA HOLDINGS, INC." },
    { "0050AB", "NALTEC, Inc." },
    { "0050AC", "MAPLE COMPUTER CORPORATION" },
    { "0050AD", "CommUnique Wireless Corp." },
    { "0050AE", "FDK Co., Ltd" },
    { "0050AF", "INTERGON, INC." },
    { "0050B0", "TECHNOLOGY ATLANTA CORPORATION" },
    { "0050B1", "GIDDINGS & LEWIS" },
    { "0050B2", "BRODEL GmbH" },
    { "0050B3", "VOICEBOARD CORPORATION" },
    { "0050B4", "SATCHWELL CONTROL SYSTEMS, LTD" },
    { "0050B5", "FICHET SECURITE ELECTRONIQUE" },
    { "0050B6", "GOOD WAY IND. CO., LTD." },
    { "0050B7", "BOSER TECHNOLOGY CO., LTD." },
    { "0050B8", "INOVA COMPUTERS GMBH & CO. KG" },
    { "0050B9", "XITRON TECHNOLOGIES, INC." },
    { "0050BA", "D-Link Corporation" },
    { "0050BB", "CMS TECHNOLOGIES" },
    { "0050BC", "HAMMER STORAGE SOLUTIONS" },
    { "0050BD", "Cisco Systems, Inc" },
    { "0050BE", "FAST MULTIMEDIA AG" },
    { "0050BF", "Metalligence Technology Corp." },
    { "0050C0", "GATAN, INC." },
    { "0050C1", "GEMFLEX NETWORKS, LTD." },
    { "0050C2", "IEEE Registration Authority" },
    { "0050C4", "IMD" },
    { "0050C5", "ADS Technologies, Inc" },
    { "0050C6", "LOOP TELECOMMUNICATION INTERNATIONAL, INC." },
    { "0050C7", "Private" },
    { "0050C8", "Addonics Technologies, Inc." },
    { "0050C9", "MASPRO DENKOH CORP." },
    { "0050CA", "Zhone Technologies, Inc." },
    { "0050CB", "Bucher Automation AG" },
    { "0050CC", "Seagate Cloud Systems Inc" },
    { "0050CD", "DIGIANSWER A/S" },
    { "0050CE", "LG INTERNATIONAL CORP." },
    { "0050CF", "VANLINK COMMUNICATION TECHNOLOGY RESEARCH INSTITUTE" },
    { "0050D0", "MINERVA SYSTEMS" },
    { "0050D1", "Cisco Systems, Inc" },
    { "0050D2", "CMC Electronics Inc" },
    { "0050D3", "DIGITAL AUDIO PROCESSING PTY. LTD." },
    { "0050D4", "JOOHONG INFORMATION &" },
    { "0050D5", "AD SYSTEMS CORP." },
    { "0050D6", "ATLAS COPCO TOOLS AB" },
    { "0050D7", "TELSTRAT" },
    { "0050D8", "UNICORN COMPUTER CORP." },
    { "0050D9", "ENGETRON-ENGENHARIA ELETRONICA IND. e COM. LTDA" },
    { "0050DA", "3COM" },
    { "0050DB", "CONTEMPORARY CONTROL" },
    { "0050DC", "TAS TELEFONBAU A. SCHWABE GMBH & CO. KG" },
    { "0050DD", "SERRA SOLDADURA, S.A." },
    { "0050DE", "SIGNUM SYSTEMS CORP." },
    { "0050DF", "AirFiber, Inc." },
    { "0050E1", "NS TECH ELECTRONICS SDN BHD" },
    { "0050E2", "Cisco Systems, Inc" },
    { "0050E3", "Commscope" },
    { "0050E4", "Apple, Inc." },
    { "0050E6", "HAKUSAN CORPORATION" },
    { "0050E7", "PARADISE INNOVATIONS (ASIA)" },
    { "0050E8", "Nomadix, Inc" },
    { "0050EA", "XEL COMMUNICATIONS, INC." },
    { "0050EB", "ALPHA-TOP CORPORATION" },
    { "0050EC", "OLICOM A/S" },
    { "0050ED", "ANDA NETWORKS" },
    { "0050EE", "TEK DIGITEL CORPORATION" },
    { "0050EF", "SPE Systemhaus GmbH" },
    { "0050F0", "Cisco Systems, Inc" },
    { "0050F1", "Maxlinear, Inc" },
    { "0050F2", "MICROSOFT CORP." },
    { "0050F3", "GLOBAL NET INFORMATION CO., Ltd." },
    { "0050F4", "SIGMATEK GMBH & CO. KG" },
    { "0050F6", "PAN-INTERNATIONAL INDUSTRIAL CORP." },
    { "0050F7", "VENTURE MANUFACTURING (SINGAPORE) LTD." },
    { "0050F8", "ENTREGA TECHNOLOGIES, INC." },
    { "0050F9", "Sensormatic Electronics LLC" },
    { "0050FA", "OXTEL, LTD." },
    { "0050FB", "VSK ELECTRONICS" },
    { "0050FC", "Edimax Technology Co. Ltd." },
    { "0050FD", "VISIONCOMM CO., LTD." },
    { "0050FE", "PCTVnet ASA" },
    { "0050FF", "HAKKO ELECTRONICS CO., LTD." },
    { "0051ED", "LG Innotek" },
    { "005218", "Wuxi Keboda Electron Co.Ltd" },
    { "005245", "GANATECHWIN" },
    { "0052C2", "peiker acustic GmbH" },
    { "0052C8", "Made Studio Design Ltd." },
    { "00549F", "Avaya Inc" },
    { "0054AF", "Continental Automotive Systems Inc." },
    { "0054BD", "Swelaser AB" },
    { "0055B1", "Shanghai Baud Data Communication Co.,Ltd." },
    { "0055DA", "IEEE Registration Authority" },
    { "00562B", "Cisco Systems, Inc" },
    { "00566D", "Huawei Device Co., Ltd." },
    { "0056CD", "Apple, Inc." },
    { "0056F1", "zte corporation" },
    { "0057C1", "LG Electronics (Mobile Communications)" },
    { "0057D2", "Cisco Systems, Inc" },
    { "005828", "Axon Networks Inc." },
    { "00583F", "PC Aquarius" },
    { "005907", "LenovoEMC Products USA, LLC" },
    { "00596C", "Cisco Systems, Inc" },
    { "005979", "Networked Energy Services" },
    { "0059AC", "KPN. B.V." },
    { "0059DC", "Cisco Systems, Inc" },
    { "005A13", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "005A39", "SHENZHEN FAST TECHNOLOGIES CO.,LTD" },
    { "005B94", "Apple, Inc." },
    { "005BA1", "shanghai huayuan chuangxin software CO., LTD." },
    { "005C86", "SHENZHEN FAST TECHNOLOGIES CO.,LTD" },
    { "005CB1", "Gospell DIGITAL TECHNOLOGY CO., LTD" },
    { "005CC2", "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "005D03", "Xilinx, Inc" },
    { "005D73", "Cisco Systems, Inc" },
    { "005E0C", "HMD Global Oy" },
    { "005F67", "TP-Link Systems Inc" },
    { "005F86", "Cisco Systems, Inc" },
    { "005FBF", "Toshiba Corp." },
    { "006000", "XYCOM INC." },
    { "006001", "InnoSys, Inc." },
    { "006002", "SCREEN SUBTITLING SYSTEMS, LTD" },
    { "006003", "TERAOKA WEIGH SYSTEM PTE, LTD." },
    { "006004", "COMPUTADORES MODULARES SA" },
    { "006005", "Touchstar ATC Limited" },
    { "006006", "SOTEC CO., LTD" },
    { "006007", "ACRES GAMING, INC." },
    { "006008", "3COM" },
    { "006009", "Cisco Systems, Inc" },
    { "00600A", "SORD COMPUTER CORPORATION" },
    { "00600B", "LOGWARE GmbH" },
    { "00600C", "Eurotech Inc." },
    { "00600D", "Digital Logic GmbH" },
    { "00600E", "WAVENET INTERNATIONAL, INC." },
    { "00600F", "Westell Technologies Inc." },
    { "006010", "NETWORK MACHINES, INC." },
    { "006011", "CRYSTAL SEMICONDUCTOR CORP." },
    { "006012", "POWER COMPUTING CORPORATION" },
    { "006013", "NETSTAL MASCHINEN AG" },
    { "006014", "EDEC CO., LTD." },
    { "006015", "NET2NET CORPORATION" },
    { "006016", "CLARIION" },
    { "006017", "TOKIMEC INC." },
    { "006018", "STELLAR ONE CORPORATION" },
    { "006019", "Roche Diagnostics" },
    { "00601A", "KEITHLEY INSTRUMENTS" },
    { "00601B", "MESA ELECTRONICS" },
    { "00601C", "TELXON CORPORATION" },
    { "00601D", "LUCENT TECHNOLOGIES" },
    { "00601E", "SOFTLAB, INC." },
    { "00601F", "STALLION TECHNOLOGIES" },
    { "006020", "PIVOTAL NETWORKING, INC." },
    { "006021", "DSC CORPORATION" },
    { "006022", "VICOM SYSTEMS, INC." },
    { "006023", "PERICOM SEMICONDUCTOR CORP." },
    { "006024", "GRADIENT TECHNOLOGIES, INC." },
    { "006025", "ACTIVE IMAGING PLC" },
    { "006026", "VIKING Modular Solutions" },
    { "006027", "Superior Modular Products" },
    { "006028", "MACROVISION CORPORATION" },
    { "006029", "CARY PERIPHERALS INC." },
    { "00602A", "SYMICRON COMPUTER COMMUNICATIONS, LTD." },
    { "00602B", "PEAK AUDIO" },
    { "00602C", "LINX Data Terminals, Inc." },
    { "00602D", "ALERTON TECHNOLOGIES, INC." },
    { "00602E", "CYCLADES CORPORATION" },
    { "00602F", "Cisco Systems, Inc" },
    { "006030", "VILLAGE TRONIC ENTWICKLUNG" },
    { "006031", "HRK SYSTEMS" },
    { "006032", "I-CUBE, INC." },
    { "006033", "ACUITY IMAGING, INC." },
    { "006034", "ROBERT BOSCH GmbH" },
    { "006035", "DALLAS SEMICONDUCTOR, INC." },
    { "006036", "AIT Austrian Institute of Technology GmbH" },
    { "006037", "NXP Semiconductors" },
    { "006038", "Nortel Networks" },
    { "006039", "SanCom Technology, Inc." },
    { "00603A", "QUICK CONTROLS LTD." },
    { "00603B", "AMTEC spa" },
    { "00603C", "HAGIWARA SYS-COM CO., LTD." },
    { "00603D", "3CX" },
    { "00603E", "Cisco Systems, Inc" },
    { "00603F", "PATAPSCO DESIGNS" },
    { "006040", "NETRO CORP." },
    { "006041", "Yokogawa Digital Computer Corporation" },
    { "006042", "TKS (USA), INC." },
    { "006043", "iDirect, INC." },
    { "006044", "LITTON/POLY-SCIENTIFIC" },
    { "006045", "PATHLIGHT TECHNOLOGIES" },
    { "006046", "VMETRO, INC." },
    { "006047", "Cisco Systems, Inc" },
    { "006048", "Dell EMC" },
    { "006049", "VINA TECHNOLOGIES" },
    { "00604A", "SAIC IDEAS GROUP" },
    { "00604B", "Safe-com GmbH & Co. KG" },
    { "00604C", "Sagemcom Broadband SAS" },
    { "00604D", "MMC NETWORKS, INC." },
    { "00604E", "CYCLE COMPUTER CORPORATION, INC." },
    { "00604F", "Tattile SRL " },
    { "006050", "INTERNIX INC." },
    { "006051", "QUALITY SEMICONDUCTOR" },
    { "006052", "PERIPHERALS ENTERPRISE CO., Ltd." },
    { "006053", "TOYODA MACHINE WORKS, LTD." },
    { "006054", "CONTROLWARE GMBH" },
    { "006055", "CORNELL UNIVERSITY" },
    { "006056", "NETWORK TOOLS, INC." },
    { "006057", "Murata Manufacturing Co., Ltd." },
    { "006058", "COPPER MOUNTAIN COMMUNICATIONS, INC." },
    { "006059", "TECHNICAL COMMUNICATIONS CORP." },
    { "00605A", "CELCORE, INC." },
    { "00605B", "IntraServer Technology, Inc." },
    { "00605C", "Cisco Systems, Inc" },
    { "00605D", "SCANIVALVE CORP." },
    { "00605E", "LIBERTY TECHNOLOGY NETWORKING" },
    { "00605F", "NIPPON UNISOFT CORPORATION" },
    { "006060", "Data Innovations North America" },
    { "006061", "WHISTLE COMMUNICATIONS CORP." },
    { "006062", "TELESYNC, INC." },
    { "006063", "PSION DACOM PLC." },
    { "006064", "NETCOMM LIMITED" },
    { "006065", "B&R Industrial Automation GmbH " },
    { "006066", "LACROIX Trafic" },
    { "006067", "ACER NETXUS INC." },
    { "006068", "Dialogic Corporation" },
    { "006069", "Brocade Communications Systems LLC" },
    { "00606A", "MITSUBISHI WIRELESS COMMUNICATIONS. INC." },
    { "00606B", "Synclayer Inc." },
    { "00606C", "ARESCOM" },
    { "00606D", "DIGITAL EQUIPMENT CORP." },
    { "00606E", "DAVICOM SEMICONDUCTOR, INC." },
    { "00606F", "CLARION CORPORATION OF AMERICA" },
    { "006070", "Cisco Systems, Inc" },
    { "006071", "MIDAS LAB, INC." },
    { "006072", "VXL INSTRUMENTS, LIMITED" },
    { "006073", "REDCREEK COMMUNICATIONS, INC." },
    { "006074", "QSC LLC" },
    { "006075", "PENTEK, INC." },
    { "006076", "SCHLUMBERGER TECHNOLOGIES RETAIL PETROLEUM SYSTEMS" },
    { "006077", "PRISA NETWORKS" },
    { "006078", "POWER MEASUREMENT LTD." },
    { "006079", "Mainstream Data, Inc." },
    { "00607A", "DVS GMBH" },
    { "00607B", "FORE SYSTEMS, INC." },
    { "00607C", "WaveAccess, Ltd." },
    { "00607D", "SENTIENT NETWORKS INC." },
    { "00607E", "GIGALABS, INC." },
    { "00607F", "AURORA TECHNOLOGIES, INC." },
    { "006080", "MICROTRONIX DATACOM LTD." },
    { "006081", "TV/COM INTERNATIONAL" },
    { "006082", "NOVALINK TECHNOLOGIES, INC." },
    { "006083", "Cisco Systems, Inc" },
    { "006084", "DIGITAL VIDEO" },
    { "006085", "Storage Concepts" },
    { "006086", "LOGIC REPLACEMENT TECH. LTD." },
    { "006087", "KANSAI ELECTRIC CO., LTD." },
    { "006088", "Analog Devices, Inc." },
    { "006089", "XATA" },
    { "00608A", "CITADEL COMPUTER" },
    { "00608B", "ConferTech International" },
    { "00608C", "3COM" },
    { "00608D", "UNIPULSE CORP." },
    { "00608E", "HE ELECTRONICS, TECHNOLOGIE & SYSTEMTECHNIK GmbH" },
    { "00608F", "TEKRAM TECHNOLOGY CO., LTD." },
    { "006090", "Artiza Networks Inc" },
    { "006091", "FIRST PACIFIC NETWORKS, INC." },
    { "006092", "MICRO/SYS, INC." },
    { "006093", "VARIAN" },
    { "006094", "IBM Corp" },
    { "006095", "ACCU-TIME SYSTEMS, INC." },
    { "006096", "T.S. MICROTECH INC." },
    { "006097", "3COM" },
    { "006098", "HT COMMUNICATIONS" },
    { "006099", "SBE, Inc." },
    { "00609A", "NJK TECHNO CO." },
    { "00609B", "AstroNova, Inc" },
    { "00609C", "Perkin-Elmer Incorporated" },
    { "00609D", "PMI FOOD EQUIPMENT GROUP" },
    { "00609E", "ASC X3 - INFORMATION TECHNOLOGY STANDARDS SECRETARIATS" },
    { "00609F", "PHAST CORPORATION" },
    { "0060A0", "SWITCHED NETWORK TECHNOLOGIES, INC." },
    { "0060A1", "VPNet, Inc." },
    { "0060A2", "NIHON UNISYS LIMITED CO." },
    { "0060A3", "CONTINUUM TECHNOLOGY CORP." },
    { "0060A4", "GEW Technologies (PTY)Ltd" },
    { "0060A5", "PERFORMANCE TELECOM CORP." },
    { "0060A6", "PARTICLE MEASURING SYSTEMS" },
    { "0060A7", "MICROSENS GmbH & CO. KG" },
    { "0060A8", "TIDOMAT AB" },
    { "0060A9", "GESYTEC MBH" },
    { "0060AA", "INTELLIGENT DEVICES INC. (IDI)" },
    { "0060AB", "LARSCOM INCORPORATED" },
    { "0060AC", "RESILIENCE CORPORATION" },
    { "0060AD", "MegaChips Corporation" },
    { "0060AE", "TRIO INFORMATION SYSTEMS AB" },
    { "0060AF", "PACIFIC MICRO DATA, INC." },
    { "0060B0", "Hewlett Packard" },
    { "0060B1", "Input/Output, Inc." },
    { "0060B2", "PROCESS CONTROL CORP." },
    { "0060B3", "Z-COM, INC." },
    { "0060B4", "GLENAYRE R&D INC." },
    { "0060B5", "KEBA GmbH" },
    { "0060B6", "LAND COMPUTER CO., LTD." },
    { "0060B7", "CHANNELMATIC, INC." },
    { "0060B8", "CORELIS Inc." },
    { "0060B9", "NEC Platforms, Ltd" },
    { "0060BA", "SAHARA NETWORKS, INC." },
    { "0060BB", "Cabletron Systems, Inc." },
    { "0060BC", "KeunYoung Electronics & Communication Co., Ltd." },
    { "0060BD", "Enginuity Communications" },
    { "0060BE", "WEBTRONICS" },
    { "0060BF", "MACRAIGOR SYSTEMS, INC." },
    { "0060C0", "Nera Networks AS" },
    { "0060C1", "WaveSpan Corporation" },
    { "0060C2", "MPL AG" },
    { "0060C3", "NETVISION CORPORATION" },
    { "0060C4", "SOLITON SYSTEMS K.K." },
    { "0060C5", "ANCOT CORP." },
    { "0060C6", "DCS AG" },
    { "0060C7", "AMATI COMMUNICATIONS CORP." },
    { "0060C8", "KUKA WELDING SYSTEMS & ROBOTS" },
    { "0060C9", "ControlNet, Inc." },
    { "0060CA", "HARMONIC SYSTEMS INCORPORATED" },
    { "0060CB", "HITACHI ZOSEN CORPORATION" },
    { "0060CC", "EMTRAK, INCORPORATED" },
    { "0060CD", "VideoServer, Inc." },
    { "0060CE", "ACCLAIM COMMUNICATIONS" },
    { "0060CF", "ALTEON NETWORKS, INC." },
    { "0060D0", "SNMP RESEARCH INCORPORATED" },
    { "0060D1", "CASCADE COMMUNICATIONS" },
    { "0060D2", "LUCENT TECHNOLOGIES TAIWAN TELECOMMUNICATIONS CO., LTD." },
    { "0060D3", "AT&T" },
    { "0060D4", "ELDAT COMMUNICATION LTD." },
    { "0060D5", "AMADA CO., LTD" },
    { "0060D6", "NovAtel Inc." },
    { "0060D7", "ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE (EPFL)" },
    { "0060D8", "ELMIC SYSTEMS, INC." },
    { "0060D9", "TRANSYS NETWORKS INC." },
    { "0060DA", "Red Lion Controls, LP" },
    { "0060DB", "NTP ELEKTRONIK A/S" },
    { "0060DC", "NEC Magnus Communications,Ltd." },
    { "0060DD", "MYRICOM, INC." },
    { "0060DE", "Kayser-Threde GmbH" },
    { "0060DF", "Brocade Communications Systems LLC" },
    { "0060E0", "AXIOM TECHNOLOGY CO., LTD." },
    { "0060E1", "ORCKIT COMMUNICATIONS LTD." },
    { "0060E2", "QUEST ENGINEERING & DEVELOPMENT" },
    { "0060E3", "ARBIN INSTRUMENTS" },
    { "0060E4", "COMPUSERVE, INC." },
    { "0060E5", "FUJI AUTOMATION CO., LTD." },
    { "0060E6", "SHOMITI SYSTEMS INCORPORATED" },
    { "0060E7", "RANDATA" },
    { "0060E8", "HITACHI COMPUTER PRODUCTS (AMERICA), INC." },
    { "0060E9", "ATOP TECHNOLOGIES, INC." },
    { "0060EA", "StreamLogic" },
    { "0060EB", "FOURTHTRACK SYSTEMS" },
    { "0060EC", "HERMARY OPTO ELECTRONICS INC." },
    { "0060ED", "RICARDO TEST AUTOMATION LTD." },
    { "0060EE", "APOLLO" },
    { "0060EF", "FLYTECH TECHNOLOGY CO., LTD." },
    { "0060F0", "JOHNSON & JOHNSON MEDICAL, INC" },
    { "0060F1", "EXP COMPUTER, INC." },
    { "0060F2", "LASERGRAPHICS, INC." },
    { "0060F3", "Performance Analysis Broadband, Spirent plc" },
    { "0060F4", "ADVANCED COMPUTER SOLUTIONS, Inc." },
    { "0060F5", "ICON WEST, INC." },
    { "0060F6", "NEXTEST COMMUNICATIONS PRODUCTS, INC." },
    { "0060F7", "DATAFUSION SYSTEMS" },
    { "0060F8", "Loran International Technologies Inc." },
    { "0060F9", "DIAMOND LANE COMMUNICATIONS" },
    { "0060FA", "EDUCATIONAL TECHNOLOGY RESOURCES, INC." },
    { "0060FB", "PACKETEER, INC." },
    { "0060FC", "CONSERVATION THROUGH INNOVATION LTD." },
    { "0060FD", "NetICs, Inc." },
    { "0060FE", "LYNX SYSTEM DEVELOPERS, INC." },
    { "0060FF", "QuVis, Inc." },
    { "006151", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "006171", "Apple, Inc." },
    { "006201", "Motorola Mobility LLC, a Lenovo Company" },
    { "00620B", "Broadcom Limited" },
    { "0062EC", "Cisco Systems, Inc" },
    { "0063DE", "CLOUDWALK TECHNOLOGY CO.,LTD" },
    { "006440", "Cisco Systems, Inc" },
    { "0064A6", "Maquet CardioVascular" },
    { "0064AF", "Dish Technologies Corp" },
    { "00651E", "Amcrest Technologies" },
    { "006619", "Huawei Device Co., Ltd." },
    { "00664B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "006762", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "00682B", "Huawei Device Co., Ltd." },
    { "0068EB", "HP Inc." },
    { "00692D", "Sunnovo International Limited" },
    { "006967", "IEEE Registration Authority" },
    { "006A5E", "IEEE Registration Authority" },
    { "006B6F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "006B8E", "Shanghai Feixun Communication Co.,Ltd." },
    { "006B9E", "Vizio, Inc" },
    { "006BA0", "SHENZHEN UNIVERSAL INTELLISYS PTE LTD" },
    { "006BF1", "Cisco Systems, Inc" },
    { "006CBC", "Cisco Systems, Inc" },
    { "006CFD", "Sichuan Changhong Electric Ltd." },
    { "006D52", "Apple, Inc." },
    { "006D61", "Guangzhou V-SOLUTION Electronic Technology Co., Ltd." },
    { "006DFB", "Vutrix Technologies Ltd" },
    { "006E02", "Xovis AG" },
    { "006F64", "Samsung Electronics Co.,Ltd" },
    { "006FF2", "MITSUMI ELECTRIC CO.,LTD." },
    { "007007", "Espressif Inc." },
    { "0070B0", "M/A-COM INC. COMPANIES" },
    { "0070B3", "DATA RECALL LTD." },
    { "007147", "Amazon Technologies Inc." },
    { "0071C2", "PEGATRON CORPORATION" },
    { "0071CC", "Hon Hai Precision Ind. Co.,Ltd." },
    { "007204", "Samsung Electronics Co., Ltd. ARTIK" },
    { "007263", "Netis Technology Co., Ltd." },
    { "007278", "Cisco Systems, Inc" },
    { "0072EE", "Intel Corporate" },
    { "00738D", "Shenzhen TINNO Mobile Technology Corp." },
    { "0073E0", "Samsung Electronics Co.,Ltd" },
    { "00749C", "Ruijie Networks Co.,LTD" },
    { "007532", "Integrated Engineering BV" },
    { "0075E1", "Ampt, LLC" },
    { "00763D", "Veea" },
    { "007686", "Cisco Systems, Inc" },
    { "0076B1", "Somfy-Protect By Myfox SAS" },
    { "00778D", "Cisco Systems, Inc" },
    { "0077E4", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "007839", "Nokia" },
    { "007888", "Cisco Systems, Inc" },
    { "00789E", "Sagemcom Broadband SAS" },
    { "0078CD", "Ignition Design Labs" },
    { "007B18", "SENTRY Co., LTD." },
    { "007C2D", "Samsung Electronics Co.,Ltd" },
    { "007D3B", "Samsung Electronics Co.,Ltd" },
    { "007D60", "Apple, Inc." },
    { "007DFA", "Volkswagen Group of America" },
    { "007E56", "China Dragon Technology Limited" },
    { "007E95", "Cisco Systems, Inc" },
    { "007F28", "Actiontec Electronics, Inc" },
    { "008000", "MULTITECH SYSTEMS, INC." },
    { "008001", "PERIPHONICS CORPORATION" },
    { "008002", "SATELCOM (UK) LTD" },
    { "008003", "HYTEC ELECTRONICS LTD." },
    { "008004", "ANTLOW COMMUNICATIONS, LTD." },
    { "008005", "CACTUS COMPUTER INC." },
    { "008006", "COMPUADD CORPORATION" },
    { "008007", "DLOG NC-SYSTEME" },
    { "008008", "DYNATECH COMPUTER SYSTEMS" },
    { "008009", "JUPITER SYSTEMS, INC." },
    { "00800A", "JAPAN COMPUTER CORP." },
    { "00800B", "CSK CORPORATION" },
    { "00800C", "VIDECOM LIMITED" },
    { "00800D", "VOSSWINKEL F.U." },
    { "00800E", "ATLANTIX CORPORATION" },
    { "00800F", "STANDARD MICROSYSTEMS" },
    { "008010", "COMMODORE INTERNATIONAL" },
    { "008011", "DIGITAL SYSTEMS INT'L. INC." },
    { "008012", "INTEGRATED MEASUREMENT SYSTEMS" },
    { "008013", "THOMAS-CONRAD CORPORATION" },
    { "008014", "ESPRIT SYSTEMS" },
    { "008015", "SEIKO SYSTEMS, INC." },
    { "008016", "WANDEL AND GOLTERMANN" },
    { "008017", "PFU LIMITED" },
    { "008018", "KOBE STEEL, LTD." },
    { "008019", "DAYNA COMMUNICATIONS, INC." },
    { "00801A", "BELL ATLANTIC" },
    { "00801B", "KODIAK TECHNOLOGY" },
    { "00801C", "NEWPORT SYSTEMS SOLUTIONS" },
    { "00801D", "INTEGRATED INFERENCE MACHINES" },
    { "00801E", "XINETRON, INC." },
    { "00801F", "KRUPP ATLAS ELECTRONIK GMBH" },
    { "008020", "NETWORK PRODUCTS" },
    { "008021", "Alcatel Canada Inc." },
    { "008022", "SCAN-OPTICS" },
    { "008023", "INTEGRATED BUSINESS NETWORKS" },
    { "008024", "KALPANA, INC." },
    { "008025", "Telit Wireless Solutions GmbH" },
    { "008026", "NETWORK PRODUCTS CORPORATION" },
    { "008027", "ADAPTIVE SYSTEMS, INC." },
    { "008028", "TRADPOST (HK) LTD" },
    { "008029", "EAGLE TECHNOLOGY, INC." },
    { "00802A", "TEST SYSTEMS & SIMULATIONS INC" },
    { "00802B", "INTEGRATED MARKETING CO" },
    { "00802C", "THE SAGE GROUP PLC" },
    { "00802D", "XYLOGICS INC" },
    { "00802E", "CASTLE ROCK COMPUTING" },
    { "00802F", "NATIONAL INSTRUMENTS CORP." },
    { "008030", "NEXUS ELECTRONICS" },
    { "008031", "BASYS, CORP." },
    { "008032", "ACCESS CO., LTD." },
    { "008033", "EMS Aviation, Inc." },
    { "008034", "SMT GOUPIL" },
    { "008035", "TECHNOLOGY WORKS, INC." },
    { "008036", "REFLEX MANUFACTURING SYSTEMS" },
    { "008037", "Ericsson Group" },
    { "008038", "DATA RESEARCH & APPLICATIONS" },
    { "008039", "ALCATEL STC AUSTRALIA" },
    { "00803A", "VARITYPER, INC." },
    { "00803B", "APT COMMUNICATIONS, INC." },
    { "00803C", "TVS ELECTRONICS LTD" },
    { "00803D", "SURIGIKEN CO., LTD." },
    { "00803E", "SYNERNETICS" },
    { "00803F", "TATUNG COMPANY" },
    { "008040", "JOHN FLUKE MANUFACTURING CO." },
    { "008041", "VEB KOMBINAT ROBOTRON" },
    { "008042", "Artesyn Embedded Technologies" },
    { "008043", "NETWORLD, INC." },
    { "008044", "SYSTECH COMPUTER CORP." },
    { "008045", "MATSUSHITA ELECTRIC IND. CO" },
    { "008046", "Tattile SRL " },
    { "008047", "IN-NET CORP." },
    { "008048", "COMPEX INCORPORATED" },
    { "008049", "NISSIN ELECTRIC CO., LTD." },
    { "00804A", "PRO-LOG" },
    { "00804B", "EAGLE TECHNOLOGIES PTY.LTD." },
    { "00804C", "CONTEC CO., LTD." },
    { "00804D", "CYCLONE MICROSYSTEMS, INC." },
    { "00804E", "APEX COMPUTER COMPANY" },
    { "00804F", "DAIKIN INDUSTRIES, LTD." },
    { "008050", "ZIATECH CORPORATION" },
    { "008051", "FIBERMUX" },
    { "008052", "TECHNICALLY ELITE CONCEPTS" },
    { "008053", "INTELLICOM, INC." },
    { "008054", "FRONTIER TECHNOLOGIES CORP." },
    { "008055", "FERMILAB" },
    { "008056", "SPHINX Electronics GmbH & Co KG" },
    { "008057", "ADSOFT, LTD." },
    { "008058", "PRINTER SYSTEMS CORP." },
    { "008059", "STANLEY ELECTRIC CO., LTD" },
    { "00805A", "TULIP COMPUTERS INTERNAT'L B.V" },
    { "00805B", "CONDOR SYSTEMS, INC." },
    { "00805C", "AGILIS CORPORATION" },
    { "00805D", "CANSTAR" },
    { "00805E", "LSI LOGIC CORPORATION" },
    { "00805F", "Hewlett Packard" },
    { "008060", "NETWORK INTERFACE CORPORATION" },
    { "008061", "LITTON SYSTEMS, INC." },
    { "008062", "INTERFACE CO." },
    { "008063", "Hirschmann Automation and Control GmbH" },
    { "008064", "WYSE TECHNOLOGY LLC" },
    { "008065", "CYBERGRAPHIC SYSTEMS PTY LTD." },
    { "008066", "Eurotech S.p.A." },
    { "008067", "SQUARE D COMPANY" },
    { "008068", "YAMATECH SCIENTIFIC LTD." },
    { "008069", "COMPUTONE SYSTEMS" },
    { "00806A", "ERI (EMPAC RESEARCH INC.)" },
    { "00806B", "SCHMID TELECOMMUNICATION" },
    { "00806C", "Secure Systems & Services" },
    { "00806D", "CENTURY SYSTEMS CORP." },
    { "00806E", "NIPPON STEEL CORPORATION" },
    { "00806F", "ONELAN LTD." },
    { "008070", "COMPUTADORAS MICRON" },
    { "008071", "SAI TECHNOLOGY" },
    { "008072", "MICROPLEX SYSTEMS LTD." },
    { "008073", "DWB ASSOCIATES" },
    { "008074", "FISHER CONTROLS" },
    { "008075", "PARSYTEC GMBH" },
    { "008076", "MCNC" },
    { "008077", "Brother industries, LTD." },
    { "008078", "PRACTICAL PERIPHERALS, INC." },
    { "008079", "MICROBUS DESIGNS LTD." },
    { "00807A", "AITECH SYSTEMS LTD." },
    { "00807B", "ARTEL COMMUNICATIONS CORP." },
    { "00807C", "FIBERCOM, INC." },
    { "00807D", "EQUINOX SYSTEMS INC." },
    { "00807E", "SOUTHERN PACIFIC LTD." },
    { "00807F", "DY-4 INCORPORATED" },
    { "008080", "DATAMEDIA CORPORATION" },
    { "008081", "KENDALL SQUARE RESEARCH CORP." },
    { "008082", "PEP MODULAR COMPUTERS GMBH" },
    { "008083", "AMDAHL" },
    { "008084", "THE CLOUD INC." },
    { "008085", "H-THREE SYSTEMS CORPORATION" },
    { "008086", "COMPUTER GENERATION INC." },
    { "008087", "OKI ELECTRIC INDUSTRY CO., LTD" },
    { "008088", "VICTOR COMPANY OF JAPAN, LTD." },
    { "008089", "TECNETICS (PTY) LTD." },
    { "00808A", "SUMMIT MICROSYSTEMS CORP." },
    { "00808B", "DACOLL LIMITED" },
    { "00808C", "NETSCOUT SYSTEMS INC" },
    { "00808D", "WESTCOAST TECHNOLOGY B.V." },
    { "00808E", "RADSTONE TECHNOLOGY" },
    { "00808F", "C. ITOH ELECTRONICS, INC." },
    { "008090", "MICROTEK INTERNATIONAL, INC." },
    { "008091", "TOKYO ELECTRIC CO.,LTD" },
    { "008092", "Silex Technology, Inc." },
    { "008093", "XYRON CORPORATION" },
    { "008094", "ALFA LAVAL AUTOMATION AB" },
    { "008095", "BASIC MERTON HANDELSGES.M.B.H." },
    { "008096", "HUMAN DESIGNED SYSTEMS, INC." },
    { "008097", "CENTRALP AUTOMATISMES" },
    { "008098", "TDK CORPORATION" },
    { "008099", "Eaton Industries GmbH" },
    { "00809A", "NOVUS NETWORKS LTD" },
    { "00809B", "JUSTSYSTEM CORPORATION" },
    { "00809C", "LUXCOM, INC." },
    { "00809D", "Commscraft Ltd." },
    { "00809E", "DATUS GMBH" },
    { "00809F", "ALE International" },
    { "0080A0", "Hewlett Packard" },
    { "0080A1", "MICROTEST, INC." },
    { "0080A2", "CREATIVE ELECTRONIC SYSTEMS" },
    { "0080A3", "Lantronix" },
    { "0080A4", "LIBERTY ELECTRONICS" },
    { "0080A5", "SPEED INTERNATIONAL" },
    { "0080A6", "REPUBLIC TECHNOLOGY, INC." },
    { "0080A7", "Honeywell International Inc" },
    { "0080A8", "VITACOM CORPORATION" },
    { "0080A9", "CLEARPOINT RESEARCH" },
    { "0080AA", "MAXPEED" },
    { "0080AB", "DUKANE NETWORK INTEGRATION" },
    { "0080AC", "IMLOGIX, DIVISION OF GENESYS" },
    { "0080AD", "CNET TECHNOLOGY, INC." },
    { "0080AE", "HUGHES NETWORK SYSTEMS" },
    { "0080AF", "ALLUMER CO., LTD." },
    { "0080B0", "ADVANCED INFORMATION" },
    { "0080B1", "SOFTCOM A/S" },
    { "0080B2", "NETWORK EQUIPMENT TECHNOLOGIES" },
    { "0080B3", "AVAL DATA CORPORATION" },
    { "0080B4", "SOPHIA SYSTEMS" },
    { "0080B5", "UNITED NETWORKS INC." },
    { "0080B6", "Mercury Systems – Trusted Mission Solutions, Inc. " },
    { "0080B7", "STELLAR COMPUTER" },
    { "0080B8", "DMG MORI Digital Co., LTD" },
    { "0080B9", "ARCHE TECHNOLIGIES INC." },
    { "0080BA", "SPECIALIX (ASIA) PTE, LTD" },
    { "0080BB", "HUGHES LAN SYSTEMS" },
    { "0080BC", "HITACHI ENGINEERING CO., LTD" },
    { "0080BD", "THE FURUKAWA ELECTRIC CO., LTD" },
    { "0080BE", "ARIES RESEARCH" },
    { "0080BF", "TAKAOKA ELECTRIC MFG. CO. LTD." },
    { "0080C0", "PENRIL DATACOMM" },
    { "0080C1", "LANEX CORPORATION" },
    { "0080C2", "IEEE 802.1 Chair" },
    { "0080C3", "BICC INFORMATION SYSTEMS & SVC" },
    { "0080C4", "DOCUMENT TECHNOLOGIES, INC." },
    { "0080C5", "NOVELLCO DE MEXICO" },
    { "0080C6", "NATIONAL DATACOMM CORPORATION" },
    { "0080C7", "XIRCOM" },
    { "0080C8", "D-LINK SYSTEMS, INC." },
    { "0080C9", "ALBERTA MICROELECTRONIC CENTRE" },
    { "0080CA", "NETCOM RESEARCH INCORPORATED" },
    { "0080CB", "FALCO DATA PRODUCTS" },
    { "0080CC", "MICROWAVE BYPASS SYSTEMS" },
    { "0080CD", "MICRONICS COMPUTER, INC." },
    { "0080CE", "BROADCAST TELEVISION SYSTEMS" },
    { "0080CF", "EMBEDDED PERFORMANCE INC." },
    { "0080D0", "COMPUTER PERIPHERALS, INC." },
    { "0080D1", "KIMTRON CORPORATION" },
    { "0080D2", "SHINNIHONDENKO CO., LTD." },
    { "0080D3", "SHIVA CORP." },
    { "0080D4", "CHASE RESEARCH LTD." },
    { "0080D5", "CADRE TECHNOLOGIES" },
    { "0080D6", "NUVOTECH, INC." },
    { "0080D7", "Fantum Engineering" },
    { "0080D8", "NETWORK PERIPHERALS INC." },
    { "0080D9", "EMK Elektronik GmbH & Co. KG" },
    { "0080DA", "Hottinger Brüel & Kjær A/S" },
    { "0080DB", "GRAPHON CORPORATION" },
    { "0080DC", "PICKER INTERNATIONAL" },
    { "0080DD", "GMX INC/GIMIX" },
    { "0080DE", "GIPSI S.A." },
    { "0080DF", "ADC CODENOLL TECHNOLOGY CORP." },
    { "0080E0", "XTP SYSTEMS, INC." },
    { "0080E1", "STMicroelectronics SRL" },
    { "0080E2", "T.D.I. CO., LTD." },
    { "0080E3", "CORAL NETWORK CORPORATION" },
    { "0080E4", "NORTHWEST DIGITAL SYSTEMS, INC" },
    { "0080E5", "NetApp" },
    { "0080E6", "PEER NETWORKS, INC." },
    { "0080E7", "Leonardo UK Ltd" },
    { "0080E8", "CUMULUS CORPORATIION" },
    { "0080E9", "Madge Ltd." },
    { "0080EA", "ADVA Optical Networking Ltd." },
    { "0080EB", "COMPCONTROL B.V." },
    { "0080EC", "SUPERCOMPUTING SOLUTIONS, INC." },
    { "0080ED", "IQ TECHNOLOGIES, INC." },
    { "0080EE", "THOMSON CSF" },
    { "0080EF", "RATIONAL" },
    { "0080F0", "Panasonic Communications Co., Ltd." },
    { "0080F1", "OPUS SYSTEMS" },
    { "0080F2", "RAYCOM SYSTEMS INC" },
    { "0080F3", "SUN ELECTRONICS CORP." },
    { "0080F4", "TELEMECANIQUE ELECTRIQUE" },
    { "0080F5", "Quantel Ltd" },
    { "0080F6", "SYNERGY MICROSYSTEMS" },
    { "0080F7", "Zenith Electronics Corporation" },
    { "0080F8", "MIZAR, INC." },
    { "0080F9", "HEURIKON CORPORATION" },
    { "0080FA", "RWT GMBH" },
    { "0080FB", "BVM LIMITED" },
    { "0080FC", "AVATAR CORPORATION" },
    { "0080FD", "EXSCEED CORPRATION" },
    { "0080FE", "AZURE TECHNOLOGIES, INC." },
    { "0080FF", "SOC. DE TELEINFORMATIQUE RTC" },
    { "00812A", "Apple, Inc." },
    { "0081C4", "Cisco Systems, Inc" },
    { "0081F9", "Texas Instruments" },
    { "008320", "Huawei Device Co., Ltd." },
    { "00841E", "Cisco Meraki" },
    { "008497", "Shenzhen MiaoMing Intelligent Technology Co.,Ltd" },
    { "0084ED", "LEXMARK INTERNATIONAL, INC." },
    { "008621", "Amazon Technologies Inc." },
    { "008667", "LG Innotek" },
    { "00869C", "Palo Alto Networks" },
    { "0086A0", "Private" },
    { "008701", "Samsung Electronics Co.,Ltd" },
    { "008731", "Cisco Systems, Inc" },
    { "008764", "Cisco Systems, Inc" },
    { "008865", "Apple, Inc." },
    { "0088BA", "NC&C" },
    { "008A55", "Huawei Device Co., Ltd." },
    { "008A76", "Apple, Inc." },
    { "008A96", "Cisco Systems, Inc" },
    { "008B43", "RFTECH" },
    { "008BFC", "mixi,Inc." },
    { "008C10", "Black Box Corp." },
    { "008C54", "ADB Broadband Italia" },
    { "008CFA", "INVENTEC CORPORATION" },
    { "008D4E", "CJSC NII STT" },
    { "008DDA", "Link One Co., Ltd." },
    { "008DF4", "IEEE Registration Authority" },
    { "008E73", "Cisco Systems, Inc" },
    { "008EF2", "NETGEAR" },
    { "009000", "DIAMOND MULTIMEDIA" },
    { "009001", "NISHIMU ELECTRONICS INDUSTRIES CO., LTD." },
    { "009002", "ALLGON AB" },
    { "009003", "APLIO" },
    { "009004", "3COM EUROPE LTD" },
    { "009005", "PROTECH SYSTEMS CO., LTD." },
    { "009006", "Hamamatsu Photonics K.K." },
    { "009007", "DOMEX TECHNOLOGY CORP." },
    { "009008", "HanA Systems Inc." },
    { "009009", "I Controls, Inc." },
    { "00900A", "PROTON ELECTRONIC INDUSTRIAL CO., LTD." },
    { "00900B", "LANNER ELECTRONICS, INC." },
    { "00900C", "Cisco Systems, Inc" },
    { "00900D", "Overland Storage Inc." },
    { "00900E", "HANDLINK TECHNOLOGIES, INC." },
    { "00900F", "KAWASAKI HEAVY INDUSTRIES, LTD" },
    { "009010", "SIMULATION LABORATORIES, INC." },
    { "009011", "WAVTrace, Inc." },
    { "009012", "GLOBESPAN SEMICONDUCTOR, INC." },
    { "009013", "SAMSAN CORP." },
    { "009014", "ROTORK INSTRUMENTS, LTD." },
    { "009015", "CENTIGRAM COMMUNICATIONS CORP." },
    { "009016", "ZAC" },
    { "009017", "Zypcom, Inc" },
    { "009018", "ITO ELECTRIC INDUSTRY CO, LTD." },
    { "009019", "HERMES ELECTRONICS CO., LTD." },
    { "00901A", "UNISPHERE SOLUTIONS" },
    { "00901B", "DIGITAL CONTROLS" },
    { "00901C", "mps Software Gmbh" },
    { "00901D", "PEC (NZ) LTD." },
    { "00901E", "Selesta Ingegneria S.p.A." },
    { "00901F", "ADTEC PRODUCTIONS, INC." },
    { "009020", "PHILIPS ANALYTICAL X-RAY B.V." },
    { "009021", "Cisco Systems, Inc" },
    { "009022", "IVEX" },
    { "009023", "ZILOG INC." },
    { "009024", "PIPELINKS, INC." },
    { "009025", "BAE Systems Australia (Electronic Systems) Pty Ltd" },
    { "009026", "ADVANCED SWITCHING COMMUNICATIONS, INC." },
    { "009027", "Intel Corporation" },
    { "009028", "NIPPON SIGNAL CO., LTD." },
    { "009029", "CRYPTO AG" },
    { "00902A", "COMMUNICATION DEVICES, INC." },
    { "00902B", "Cisco Systems, Inc" },
    { "00902C", "DATA & CONTROL EQUIPMENT LTD." },
    { "00902D", "DATA ELECTRONICS (AUST.) PTY, LTD." },
    { "00902E", "NAMCO LIMITED" },
    { "00902F", "NETCORE SYSTEMS, INC." },
    { "009030", "HONEYWELL-DATING" },
    { "009031", "MYSTICOM, LTD." },
    { "009032", "PELCOMBE GROUP LTD." },
    { "009033", "INNOVAPHONE AG" },
    { "009034", "IMAGIC, INC." },
    { "009035", "ALPHA TELECOM, INC." },
    { "009036", "ens, inc." },
    { "009037", "ACUCOMM, INC." },
    { "009038", "FOUNTAIN TECHNOLOGIES, INC." },
    { "009039", "SHASTA NETWORKS" },
    { "00903A", "NIHON MEDIA TOOL INC." },
    { "00903B", "TriEMS Research Lab, Inc." },
    { "00903C", "ATLANTIC NETWORK SYSTEMS" },
    { "00903D", "BIOPAC SYSTEMS, INC." },
    { "00903E", "N.V. PHILIPS INDUSTRIAL ACTIVITIES" },
    { "00903F", "WorldCast Systems" },
    { "009040", "Siemens Network Convergence LLC" },
    { "009041", "APPLIED DIGITAL ACCESS" },
    { "009042", "ECCS, Inc." },
    { "009043", "Tattile SRL " },
    { "009044", "ASSURED DIGITAL, INC." },
    { "009045", "Marconi Communications" },
    { "009046", "DEXDYNE, LTD." },
    { "009047", "GIGA FAST E. LTD." },
    { "009048", "ZEAL CORPORATION" },
    { "009049", "ENTRIDIA CORPORATION" },
    { "00904A", "CONCUR SYSTEM TECHNOLOGIES" },
    { "00904B", "Gemtek Technology Co., Ltd." },
    { "00904C", "Epigram, Inc." },
    { "00904D", "SPEC S.A." },
    { "00904E", "DELEM BV" },
    { "00904F", "ABB POWER T&D COMPANY, INC." },
    { "009050", "Teleste Corporation" },
    { "009051", "ULTIMATE TECHNOLOGY CORP." },
    { "009052", "SELCOM ELETTRONICA S.R.L." },
    { "009053", "DAEWOO ELECTRONICS CO., LTD." },
    { "009054", "INNOVATIVE SEMICONDUCTORS, INC" },
    { "009055", "PARKER HANNIFIN CORPORATION COMPUMOTOR DIVISION" },
    { "009056", "TELESTREAM, INC." },
    { "009057", "AANetcom, Inc." },
    { "009058", "Ultra Electronics Command & Control Systems" },
    { "009059", "TELECOM DEVICE K.K." },
    { "00905A", "DEARBORN GROUP, INC." },
    { "00905B", "RAYMOND AND LAE ENGINEERING" },
    { "00905C", "EDMI" },
    { "00905D", "NETCOM SICHERHEITSTECHNIK GMBH" },
    { "00905E", "RAULAND-BORG CORPORATION" },
    { "00905F", "Cisco Systems, Inc" },
    { "009060", "SYSTEM CREATE CORP." },
    { "009061", "PACIFIC RESEARCH & ENGINEERING CORPORATION" },
    { "009062", "ICP VORTEX COMPUTERSYSTEME GmbH" },
    { "009063", "COHERENT COMMUNICATIONS SYSTEMS CORPORATION" },
    { "009064", "Thomson Inc." },
    { "009065", "FINISAR CORPORATION" },
    { "009066", "Troika Networks, Inc." },
    { "009067", "WalkAbout Computers, Inc." },
    { "009068", "DVT CORP." },
    { "009069", "Juniper Networks" },
    { "00906A", "TURNSTONE SYSTEMS, INC." },
    { "00906B", "APPLIED RESOURCES, INC." },
    { "00906C", "Sartorius Hamburg GmbH" },
    { "00906D", "Cisco Systems, Inc" },
    { "00906E", "PRAXON, INC." },
    { "00906F", "Cisco Systems, Inc" },
    { "009070", "NEO NETWORKS, INC." },
    { "009071", "Applied Innovation Inc." },
    { "009072", "SIMRAD AS" },
    { "009073", "GAIO TECHNOLOGY" },
    { "009074", "ARGON NETWORKS, INC." },
    { "009075", "NEC DO BRASIL S.A." },
    { "009076", "FMT AIRCRAFT GATE SUPPORT SYSTEMS AB" },
    { "009077", "ADVANCED FIBRE COMMUNICATIONS" },
    { "009078", "MER TELEMANAGEMENT SOLUTIONS, LTD." },
    { "009079", "ClearOne, Inc." },
    { "00907A", "Spectralink, Inc" },
    { "00907B", "E-TECH, INC." },
    { "00907C", "DIGITALCAST, INC." },
    { "00907D", "Lake Communications" },
    { "00907E", "VETRONIX CORP." },
    { "00907F", "WatchGuard Technologies, Inc." },
    { "009080", "NOT LIMITED, INC." },
    { "009081", "ALOHA NETWORKS, INC." },
    { "009082", "FORCE INSTITUTE" },
    { "009083", "TURBO COMMUNICATION, INC." },
    { "009084", "ATECH SYSTEM" },
    { "009085", "GOLDEN ENTERPRISES, INC." },
    { "009086", "Cisco Systems, Inc" },
    { "009087", "ITIS" },
    { "009088", "BAXALL SECURITY LTD." },
    { "009089", "SOFTCOM MICROSYSTEMS, INC." },
    { "00908A", "BAYLY COMMUNICATIONS, INC." },
    { "00908B", "Tattile SRL" },
    { "00908C", "ETREND ELECTRONICS, INC." },
    { "00908D", "VICKERS ELECTRONICS SYSTEMS" },
    { "00908E", "Nortel Networks Broadband Access" },
    { "00908F", "AUDIO CODES LTD." },
    { "009090", "I-BUS" },
    { "009091", "DigitalScape, Inc." },
    { "009092", "Cisco Systems, Inc" },
    { "009093", "EIZO Corporation" },
    { "009094", "OSPREY TECHNOLOGIES, INC." },
    { "009095", "UNIVERSAL AVIONICS" },
    { "009096", "ASKEY COMPUTER CORP" },
    { "009097", "Sycamore Networks" },
    { "009098", "SBC DESIGNS, INC." },
    { "009099", "ALLIED TELESIS, K.K." },
    { "00909A", "ONE WORLD SYSTEMS, INC." },
    { "00909B", "MARKEM-IMAJE" },
    { "00909C", "Commscope" },
    { "00909D", "NovaTech, LLC" },
    { "00909E", "Critical IO, LLC" },
    { "00909F", "DIGI-DATA CORPORATION" },
    { "0090A0", "8X8 INC." },
    { "0090A1", "Flying Pig Systems/High End Systems Inc." },
    { "0090A2", "CyberTAN Technology Inc." },
    { "0090A3", "Corecess Inc." },
    { "0090A4", "ALTIGA NETWORKS" },
    { "0090A5", "SPECTRA LOGIC" },
    { "0090A6", "Cisco Systems, Inc" },
    { "0090A7", "CLIENTEC CORPORATION" },
    { "0090A8", "NineTiles Networks, Ltd." },
    { "0090A9", "WESTERN DIGITAL" },
    { "0090AA", "INDIGO ACTIVE VISION SYSTEMS LIMITED" },
    { "0090AB", "Cisco Systems, Inc" },
    { "0090AC", "OPTIVISION, INC." },
    { "0090AD", "ASPECT ELECTRONICS, INC." },
    { "0090AE", "ITALTEL S.p.A/RF-UP-I" },
    { "0090AF", "J. MORITA MFG. CORP." },
    { "0090B0", "VADEM" },
    { "0090B1", "Cisco Systems, Inc" },
    { "0090B2", "AVICI SYSTEMS INC." },
    { "0090B3", "AGRANAT SYSTEMS" },
    { "0090B4", "WILLOWBROOK TECHNOLOGIES" },
    { "0090B5", "NIKON CORPORATION" },
    { "0090B6", "FIBEX SYSTEMS" },
    { "0090B7", "DIGITAL LIGHTWAVE, INC." },
    { "0090B8", "ROHDE & SCHWARZ GMBH & CO. KG" },
    { "0090B9", "BERAN INSTRUMENTS LTD." },
    { "0090BA", "VALID NETWORKS, INC." },
    { "0090BB", "TAINET COMMUNICATION SYSTEM Corp." },
    { "0090BC", "TELEMANN CO., LTD." },
    { "0090BD", "OMNIA COMMUNICATIONS, INC." },
    { "0090BE", "IBC/INTEGRATED BUSINESS COMPUTERS" },
    { "0090BF", "Cisco Systems, Inc" },
    { "0090C0", "K.J. LAW ENGINEERS, INC." },
    { "0090C1", "Peco II, Inc." },
    { "0090C2", "JK microsystems, Inc." },
    { "0090C3", "TOPIC SEMICONDUCTOR CORP." },
    { "0090C4", "JAVELIN SYSTEMS, INC." },
    { "0090C5", "INTERNET MAGIC, INC." },
    { "0090C6", "OPTIM SYSTEMS, INC." },
    { "0090C7", "ICOM INC." },
    { "0090C8", "WAVERIDER COMMUNICATIONS (CANADA) INC." },
    { "0090C9", "DPAC Technologies" },
    { "0090CA", "ACCORD VIDEO TELECOMMUNICATIONS, LTD." },
    { "0090CB", "Wireless OnLine, Inc." },
    { "0090CC", "PLANEX COMMUNICATIONS INC." },
    { "0090CD", "ENT-EMPRESA NACIONAL DE TELECOMMUNICACOES, S.A." },
    { "0090CE", "avateramedical Mechatronics GmbH" },
    { "0090CF", "NORTEL" },
    { "0090D0", "Thomson Telecom Belgium" },
    { "0090D1", "LEICHU ENTERPRISE CO., LTD." },
    { "0090D2", "Artel Video Systems" },
    { "0090D3", "GIESECKE & DEVRIENT GmbH" },
    { "0090D4", "BindView Development Corp." },
    { "0090D5", "EUPHONIX, INC." },
    { "0090D6", "Crystal Group, Inc." },
    { "0090D7", "NetBoost Corp." },
    { "0090D8", "WHITECROSS SYSTEMS" },
    { "0090D9", "Cisco Systems, Inc" },
    { "0090DA", "DYNARC, INC." },
    { "0090DB", "NEXT LEVEL COMMUNICATIONS" },
    { "0090DC", "TECO INFORMATION SYSTEMS" },
    { "0090DD", "MIHARU COMMUNICATIONS Inc" },
    { "0090DE", "CARDKEY SYSTEMS, INC." },
    { "0090DF", "MITSUBISHI CHEMICAL AMERICA, INC." },
    { "0090E0", "SYSTRAN CORP." },
    { "0090E1", "TELENA S.P.A." },
    { "0090E2", "DISTRIBUTED PROCESSING TECHNOLOGY" },
    { "0090E3", "AVEX ELECTRONICS INC." },
    { "0090E4", "NEC AMERICA, INC." },
    { "0090E5", "TEKNEMA, INC." },
    { "0090E6", "ALi Corporation" },
    { "0090E7", "HORSCH ELEKTRONIK AG" },
    { "0090E8", "MOXA TECHNOLOGIES CORP., LTD." },
    { "0090E9", "JANZ COMPUTER AG" },
    { "0090EA", "ALPHA TECHNOLOGIES, INC." },
    { "0090EB", "SENTRY TELECOM SYSTEMS" },
    { "0090EC", "PYRESCOM" },
    { "0090ED", "CENTRAL SYSTEM RESEARCH CO., LTD." },
    { "0090EE", "PERSONAL COMMUNICATIONS TECHNOLOGIES" },
    { "0090EF", "INTEGRIX, INC." },
    { "0090F0", "Harmonic Video Systems Ltd." },
    { "0090F1", "Seagate Cloud Systems Inc" },
    { "0090F2", "Cisco Systems, Inc" },
    { "0090F3", "ASPECT COMMUNICATIONS" },
    { "0090F4", "LIGHTNING INSTRUMENTATION" },
    { "0090F5", "CLEVO CO." },
    { "0090F6", "ESCALATE NETWORKS, INC." },
    { "0090F7", "NBASE COMMUNICATIONS LTD." },
    { "0090F8", "MEDIATRIX TELECOM" },
    { "0090F9", "Imagine Communications" },
    { "0090FA", "Emulex Corporation" },
    { "0090FB", "PORTWELL, INC." },
    { "0090FC", "NETWORK COMPUTING DEVICES" },
    { "0090FD", "CopperCom, Inc." },
    { "0090FE", "ELECOM CO., LTD. (LANEED DIV.)" },
    { "0090FF", "TELLUS TECHNOLOGY INC." },
    { "00919E", "Intel Corporate" },
    { "0091D6", "Crystal Group, Inc." },
    { "0091EB", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "0091FA", "Synapse Product Development" },
    { "00927D", "Ficosa Internationa(Taicang) C0.,Ltd." },
    { "0092A5", "LG Innotek" },
    { "0092FA", "SHENZHEN WISKY TECHNOLOGY CO.,LTD" },
    { "009337", "Intel Corporate" },
    { "009363", "Uni-Link Technology Co., Ltd." },
    { "0094A1", "F5 Inc." },
    { "0094EC", "Huawei Device Co., Ltd." },
    { "009569", "LSD Science and Technology Co.,Ltd." },
    { "0097F1", "Apple, Inc." },
    { "0097FF", "Heimann Sensor GmbH" },
    { "00991D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "009ACD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "009AD2", "Cisco Systems, Inc" },
    { "009B08", "Quectel Wireless Solutions Co.,Ltd." },
    { "009C02", "Hewlett Packard" },
    { "009C17", "Quectel Wireless Solutions Co.,Ltd." },
    { "009CC0", "vivo Mobile Communication Co., Ltd." },
    { "009D6B", "Murata Manufacturing Co., Ltd." },
    { "009D85", "Sigmastar Technology Ltd." },
    { "009D8E", "CARDIAC RECORDERS, INC." },
    { "009E1E", "Cisco Systems, Inc" },
    { "009EC8", "Xiaomi Communications Co Ltd" },
    { "009EEE", "Positivo Tecnologia S.A." },
    { "00A000", "CENTILLION NETWORKS, INC." },
    { "00A001", "DRS Signal Solutions" },
    { "00A002", "LEEDS & NORTHRUP AUSTRALIA PTY LTD" },
    { "00A003", "Siemens Switzerland Ltd., I B T HVP" },
    { "00A004", "NETPOWER, INC." },
    { "00A005", "DANIEL INSTRUMENTS, LTD." },
    { "00A006", "IMAGE DATA PROCESSING SYSTEM GROUP" },
    { "00A007", "APEXX TECHNOLOGY, INC." },
    { "00A008", "NETCORP" },
    { "00A009", "WHITETREE NETWORK" },
    { "00A00A", "Airspan" },
    { "00A00B", "COMPUTEX CO., LTD." },
    { "00A00C", "KINGMAX TECHNOLOGY, INC." },
    { "00A00D", "THE PANDA PROJECT" },
    { "00A00E", "NETSCOUT SYSTEMS INC" },
    { "00A00F", "Broadband Technologies" },
    { "00A010", "SYSLOGIC DATENTECHNIK AG" },
    { "00A011", "MUTOH INDUSTRIES LTD." },
    { "00A012", "Telco Systems, Inc. " },
    { "00A013", "TELTREND LTD." },
    { "00A014", "CSIR" },
    { "00A015", "WYLE" },
    { "00A016", "MICROPOLIS CORP." },
    { "00A017", "J B M CORPORATION" },
    { "00A018", "CREATIVE CONTROLLERS, INC." },
    { "00A019", "NEBULA CONSULTANTS, INC." },
    { "00A01A", "BINAR ELEKTRONIK AB" },
    { "00A01B", "Zhone Technologies, Inc." },
    { "00A01C", "NASCENT NETWORKS CORPORATION" },
    { "00A01D", "Red Lion Controls, LP" },
    { "00A01E", "EST CORPORATION" },
    { "00A01F", "TRICORD SYSTEMS, INC." },
    { "00A020", "CITICORP/TTI" },
    { "00A021", "General Dynamics Mission Systems" },
    { "00A022", "CENTRE FOR DEVELOPMENT OF ADVANCED COMPUTING" },
    { "00A023", "APPLIED CREATIVE TECHNOLOGY, INC." },
    { "00A024", "3COM" },
    { "00A025", "REDCOM LABS INC." },
    { "00A026", "TELDAT, S.A." },
    { "00A027", "FIREPOWER SYSTEMS, INC." },
    { "00A028", "CONNER PERIPHERALS" },
    { "00A029", "COULTER CORPORATION" },
    { "00A02A", "TRANCELL SYSTEMS" },
    { "00A02B", "TRANSITIONS RESEARCH CORP." },
    { "00A02C", "interWAVE Communications" },
    { "00A02D", "1394 Trade Association" },
    { "00A02E", "BRAND COMMUNICATIONS, LTD." },
    { "00A02F", "ADB Broadband Italia" },
    { "00A030", "CAPTOR NV/SA" },
    { "00A031", "HAZELTINE CORPORATION, MS 1-17" },
    { "00A032", "GES SINGAPORE PTE. LTD." },
    { "00A033", "imc MeBsysteme GmbH" },
    { "00A034", "AXEL" },
    { "00A035", "CYLINK CORPORATION" },
    { "00A036", "APPLIED NETWORK TECHNOLOGY" },
    { "00A037", "Mindray DS USA, Inc." },
    { "00A038", "EMAIL ELECTRONICS" },
    { "00A039", "ROSS TECHNOLOGY, INC." },
    { "00A03A", "KUBOTEK CORPORATION" },
    { "00A03B", "TOSHIN ELECTRIC CO., LTD." },
    { "00A03C", "EG&G NUCLEAR INSTRUMENTS" },
    { "00A03D", "OPTO-22" },
    { "00A03E", "ATM FORUM" },
    { "00A03F", "COMPUTER SOCIETY MICROPROCESSOR & MICROPROCESSOR STANDARDS C" },
    { "00A040", "Apple, Inc." },
    { "00A041", "INFICON" },
    { "00A042", "SPUR PRODUCTS CORP." },
    { "00A043", "AMERICAN TECHNOLOGY LABS, INC." },
    { "00A044", "NTT IT CO., LTD." },
    { "00A045", "Phoenix Contact GmbH & Co. KG" },
    { "00A046", "SCITEX CORP. LTD." },
    { "00A047", "INTEGRATED FITNESS CORP." },
    { "00A048", "QUESTECH, LTD." },
    { "00A049", "DIGITECH INDUSTRIES, INC." },
    { "00A04A", "NISSHIN ELECTRIC CO., LTD." },
    { "00A04B", "TFL LAN INC." },
    { "00A04C", "INNOVATIVE SYSTEMS & TECHNOLOGIES, INC." },
    { "00A04D", "EDA INSTRUMENTS, INC." },
    { "00A04E", "VOELKER TECHNOLOGIES, INC." },
    { "00A04F", "AMERITEC CORP." },
    { "00A050", "CYPRESS SEMICONDUCTOR" },
    { "00A051", "ANGIA COMMUNICATIONS. INC." },
    { "00A052", "STANILITE ELECTRONICS PTY. LTD" },
    { "00A053", "COMPACT DEVICES, INC." },
    { "00A054", "Private" },
    { "00A055", "Data Device Corporation" },
    { "00A056", "MICROPROSS" },
    { "00A057", "LANCOM Systems GmbH" },
    { "00A058", "GLORY, LTD." },
    { "00A059", "HAMILTON HALLMARK" },
    { "00A05A", "KOFAX IMAGE PRODUCTS" },
    { "00A05B", "MARQUIP, INC." },
    { "00A05C", "INVENTORY CONVERSION, INC./" },
    { "00A05D", "CS COMPUTER SYSTEME GmbH" },
    { "00A05E", "MYRIAD LOGIC INC." },
    { "00A05F", "BTG Electronics Design BV" },
    { "00A060", "ACER PERIPHERALS, INC." },
    { "00A061", "PURITAN BENNETT" },
    { "00A062", "AES PRODATA" },
    { "00A063", "JRL SYSTEMS, INC." },
    { "00A064", "KVB/ANALECT" },
    { "00A065", "Symantec Corporation" },
    { "00A066", "ISA CO., LTD." },
    { "00A067", "NETWORK SERVICES GROUP" },
    { "00A068", "BHP LIMITED" },
    { "00A069", "Symmetricom, Inc." },
    { "00A06A", "Verilink Corporation" },
    { "00A06B", "DMS DORSCH MIKROSYSTEM GMBH" },
    { "00A06C", "SHINDENGEN ELECTRIC MFG. CO., LTD." },
    { "00A06D", "MANNESMANN TALLY CORPORATION" },
    { "00A06E", "AUSTRON, INC." },
    { "00A06F", "Color Sentinel Systems, LLC" },
    { "00A070", "COASTCOM" },
    { "00A071", "VIDEO LOTTERY TECHNOLOGIES,INC" },
    { "00A072", "OVATION SYSTEMS LTD." },
    { "00A073", "COM21, INC." },
    { "00A074", "PERCEPTION TECHNOLOGY" },
    { "00A075", "MICRON TECHNOLOGY, INC." },
    { "00A076", "CARDWARE LAB, INC." },
    { "00A077", "FUJITSU NEXION, INC." },
    { "00A078", "Marconi Communications" },
    { "00A079", "ALPS ELECTRIC (USA), INC." },
    { "00A07A", "ADVANCED PERIPHERALS TECHNOLOGIES, INC." },
    { "00A07B", "DAWN COMPUTER INCORPORATION" },
    { "00A07C", "TONYANG NYLON CO., LTD." },
    { "00A07D", "SEEQ TECHNOLOGY, INC." },
    { "00A07E", "AVID TECHNOLOGY, INC." },
    { "00A07F", "GSM-SYNTEL, LTD." },
    { "00A080", "Tattile SRL " },
    { "00A081", "ALCATEL DATA NETWORKS" },
    { "00A082", "NKT ELEKTRONIK A/S" },
    { "00A083", "ASIMMPHONY TURKEY" },
    { "00A084", "Dataplex Pty Ltd" },
    { "00A085", "Private" },
    { "00A086", "AMBER WAVE SYSTEMS, INC." },
    { "00A087", "Microsemi Corporation" },
    { "00A088", "ESSENTIAL COMMUNICATIONS" },
    { "00A089", "XPOINT TECHNOLOGIES, INC." },
    { "00A08A", "BROOKTROUT TECHNOLOGY, INC." },
    { "00A08B", "ASTON ELECTRONIC DESIGNS LTD." },
    { "00A08C", "MultiMedia LANs, Inc." },
    { "00A08D", "JACOMO CORPORATION" },
    { "00A08E", "Check Point Software Technologies" },
    { "00A08F", "DESKNET SYSTEMS, INC." },
    { "00A090", "TimeStep Corporation" },
    { "00A091", "APPLICOM INTERNATIONAL" },
    { "00A092", "H. BOLLMANN MANUFACTURERS, LTD" },
    { "00A093", "B/E AEROSPACE, Inc." },
    { "00A094", "COMSAT CORPORATION" },
    { "00A095", "ACACIA NETWORKS, INC." },
    { "00A096", "MITSUMI ELECTRIC CO.,LTD." },
    { "00A097", "JC INFORMATION SYSTEMS" },
    { "00A098", "NetApp" },
    { "00A099", "K-NET LTD." },
    { "00A09A", "NIHON KOHDEN AMERICA" },
    { "00A09B", "QPSX COMMUNICATIONS, LTD." },
    { "00A09C", "Xyplex, Inc." },
    { "00A09D", "JOHNATHON FREEMAN TECHNOLOGIES" },
    { "00A09E", "ICTV" },
    { "00A09F", "COMMVISION CORP." },
    { "00A0A0", "COMPACT DATA, LTD." },
    { "00A0A1", "EPIC DATA INC." },
    { "00A0A2", "B810 S.R.L." },
    { "00A0A3", "RELIABLE POWER METERS" },
    { "00A0A4", "Oracle Corporation " },
    { "00A0A5", "TEKNOR MICROSYSTEME, INC." },
    { "00A0A6", "M.I. SYSTEMS, K.K." },
    { "00A0A7", "VORAX CORPORATION" },
    { "00A0A8", "RENEX CORPORATION" },
    { "00A0A9", "NAVTEL COMMUNICATIONS INC." },
    { "00A0AA", "SPACELABS MEDICAL" },
    { "00A0AB", "NETCS INFORMATIONSTECHNIK GMBH" },
    { "00A0AC", "GILAT SATELLITE NETWORKS, LTD." },
    { "00A0AD", "MARCONI SPA" },
    { "00A0AE", "NUCOM SYSTEMS, INC." },
    { "00A0AF", "WMS INDUSTRIES" },
    { "00A0B0", "I-O DATA DEVICE,INC." },
    { "00A0B1", "FIRST VIRTUAL CORPORATION" },
    { "00A0B2", "SHIMA SEIKI" },
    { "00A0B3", "ZYKRONIX" },
    { "00A0B4", "TEXAS MICROSYSTEMS, INC." },
    { "00A0B5", "3H TECHNOLOGY" },
    { "00A0B6", "SANRITZ AUTOMATION CO., LTD." },
    { "00A0B7", "CORDANT, INC." },
    { "00A0B8", "NetApp" },
    { "00A0B9", "EAGLE TECHNOLOGY, INC." },
    { "00A0BA", "PATTON ELECTRONICS CO." },
    { "00A0BB", "HILAN GMBH" },
    { "00A0BC", "VIASAT, INCORPORATED" },
    { "00A0BD", "I-TECH CORP." },
    { "00A0BE", "INTEGRATED CIRCUIT SYSTEMS, INC. COMMUNICATIONS GROUP" },
    { "00A0BF", "WIRELESS DATA GROUP MOTOROLA" },
    { "00A0C0", "DIGITAL LINK CORP." },
    { "00A0C1", "ORTIVUS MEDICAL AB" },
    { "00A0C2", "R.A. SYSTEMS CO., LTD." },
    { "00A0C3", "UNICOMPUTER GMBH" },
    { "00A0C4", "CRISTIE ELECTRONICS LTD." },
    { "00A0C5", "Zyxel Communications Corporation" },
    { "00A0C6", "Qualcomm Inc." },
    { "00A0C7", "TADIRAN TELECOMMUNICATIONS" },
    { "00A0C8", "Adtran Inc" },
    { "00A0C9", "Intel Corporation" },
    { "00A0CA", "FUJITSU DENSO LTD." },
    { "00A0CB", "ARK TELECOMMUNICATIONS, INC." },
    { "00A0CC", "LITE-ON COMMUNICATIONS, INC." },
    { "00A0CD", "DR. JOHANNES HEIDENHAIN GmbH" },
    { "00A0CE", "Ecessa" },
    { "00A0CF", "SOTAS, INC." },
    { "00A0D0", "TEN X TECHNOLOGY, INC." },
    { "00A0D1", "INVENTEC CORPORATION" },
    { "00A0D2", "ALLIED TELESIS INTERNATIONAL CORPORATION" },
    { "00A0D3", "INSTEM COMPUTER SYSTEMS, LTD." },
    { "00A0D4", "RADIOLAN, INC." },
    { "00A0D5", "Sierra Wireless, ULC" },
    { "00A0D6", "SBE, Inc." },
    { "00A0D7", "KASTEN CHASE APPLIED RESEARCH" },
    { "00A0D8", "SPECTRA - TEK" },
    { "00A0D9", "CONVEX COMPUTER CORPORATION" },
    { "00A0DA", "INTEGRATED SYSTEMS Technology, Inc." },
    { "00A0DB", "FISHER & PAYKEL PRODUCTION" },
    { "00A0DC", "O.N. ELECTRONIC CO., LTD." },
    { "00A0DD", "AZONIX CORPORATION" },
    { "00A0DE", "YAMAHA CORPORATION" },
    { "00A0DF", "STS TECHNOLOGIES, INC." },
    { "00A0E0", "TENNYSON TECHNOLOGIES PTY LTD" },
    { "00A0E1", "WESTPORT RESEARCH ASSOCIATES, INC." },
    { "00A0E2", "Keisokugiken Corporation" },
    { "00A0E3", "XKL SYSTEMS CORP." },
    { "00A0E4", "OPTIQUEST" },
    { "00A0E5", "NHC COMMUNICATIONS" },
    { "00A0E6", "DIALOGIC CORPORATION" },
    { "00A0E7", "CENTRAL DATA CORPORATION" },
    { "00A0E8", "REUTERS HOLDINGS PLC" },
    { "00A0E9", "ELECTRONIC RETAILING SYSTEMS INTERNATIONAL" },
    { "00A0EA", "ETHERCOM CORP." },
    { "00A0EB", "Encore Networks, Inc." },
    { "00A0EC", "TRANSMITTON LTD." },
    { "00A0ED", "Brooks Automation, Inc." },
    { "00A0EE", "NASHOBA NETWORKS" },
    { "00A0EF", "LUCIDATA LTD." },
    { "00A0F0", "TORONTO MICROELECTRONICS INC." },
    { "00A0F1", "MTI" },
    { "00A0F2", "INFOTEK COMMUNICATIONS, INC." },
    { "00A0F3", "STAUBLI" },
    { "00A0F4", "GE" },
    { "00A0F5", "RADGUARD LTD." },
    { "00A0F6", "AutoGas Systems Inc." },
    { "00A0F7", "V.I COMPUTER CORP." },
    { "00A0F8", "Zebra Technologies Inc" },
    { "00A0F9", "BINTEC COMMUNICATIONS GMBH" },
    { "00A0FA", "Marconi Communication GmbH" },
    { "00A0FB", "Toray Engineering D Solutions Co., Ltd." },
    { "00A0FC", "IMAGE SCIENCES, INC." },
    { "00A0FD", "SCITEX DIGITAL PRINTING, INC." },
    { "00A0FE", "BOSTON TECHNOLOGY, INC." },
    { "00A0FF", "TELLABS OPERATIONS, INC." },
    { "00A159", "LG Electronics" },
    { "00A1DE", "ShenZhen ShiHua Technology CO.,LTD" },
    { "00A265", "M2Motive Technology Inc." },
    { "00A289", "Cisco Systems, Inc" },
    { "00A2DA", "INAT GmbH" },
    { "00A2EE", "Cisco Systems, Inc" },
    { "00A2F5", "Guangzhou Yuanyun Network Technology Co.,Ltd" },
    { "00A2FF", "abatec group AG" },
    { "00A388", "SKY UK LIMITED" },
    { "00A38E", "Cisco Systems, Inc" },
    { "00A3D1", "Cisco Systems, Inc" },
    { "00A41C", "Tonly Technology Co. Ltd " },
    { "00A45F", "Huawei Device Co., Ltd." },
    { "00A509", "WigWag Inc." },
    { "00A554", "Intel Corporate" },
    { "00A5BF", "Cisco Systems, Inc" },
    { "00A62B", "Shanghai High-Flying Electronics Technology Co.,Ltd" },
    { "00A6CA", "Cisco Systems, Inc" },
    { "00A742", "Cisco Systems, Inc" },
    { "00A784", "ITX security" },
    { "00A91D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "00AA00", "Intel Corporation" },
    { "00AA01", "Intel Corporation" },
    { "00AA02", "Intel Corporation" },
    { "00AA3C", "OLIVETTI TELECOM SPA (OLTECO)" },
    { "00AA6E", "Cisco Systems, Inc" },
    { "00AA70", "LG Electronics (Mobile Communications)" },
    { "00AAFD", "Texas Instruments" },
    { "00AB48", "eero inc." },
    { "00ACE0", "Commscope" },
    { "00AD24", "D-Link International" },
    { "00AD63", "Dedicated Micros Malta LTD" },
    { "00ADD5", "Huawei Device Co., Ltd." },
    { "00AECD", "Pensando Systems" },
    { "00AEF7", "Dreame Technology (Suzhou) Limited" },
    { "00AEFA", "Murata Manufacturing Co., Ltd." },
    { "00AF1F", "Cisco Systems, Inc" },
    { "00B009", "Grass Valley, A Belden Brand" },
    { "00B017", "InfoGear Technology Corp." },
    { "00B019", "UTC CCS" },
    { "00B01C", "Westport Technologies" },
    { "00B01E", "Rantic Labs, Inc." },
    { "00B02A", "ORSYS GmbH" },
    { "00B02D", "ViaGate Technologies, Inc." },
    { "00B033", "OAO Izhevskiy radiozavod" },
    { "00B03B", "HiQ Networks" },
    { "00B048", "Marconi Communications Inc." },
    { "00B04A", "Cisco Systems, Inc" },
    { "00B052", "Atheros Communications" },
    { "00B064", "Cisco Systems, Inc" },
    { "00B069", "Honewell Oy" },
    { "00B06D", "Jones Futurex Inc." },
    { "00B080", "Mannesmann Ipulsys B.V." },
    { "00B086", "LocSoft Limited" },
    { "00B08E", "Cisco Systems, Inc" },
    { "00B091", "Transmeta Corp." },
    { "00B094", "Alaris, Inc." },
    { "00B09A", "Morrow Technologies Corp." },
    { "00B09D", "Point Grey Research Inc." },
    { "00B0AC", "SIAE-Microelettronica S.p.A." },
    { "00B0AE", "Symmetricom" },
    { "00B0B3", "XSTREAMIS PLC" },
    { "00B0C2", "Cisco Systems, Inc" },
    { "00B0C7", "Tellabs Operations, Inc." },
    { "00B0CE", "Viveris Technologies" },
    { "00B0D0", "Dell Inc." },
    { "00B0DB", "Nextcell, Inc." },
    { "00B0DF", "Starboard Storage Systems" },
    { "00B0E1", "Cisco Systems, Inc" },
    { "00B0E7", "British Federal Ltd." },
    { "00B0EC", "EACEM" },
    { "00B0EE", "Ajile Systems, Inc." },
    { "00B0F0", "CALY NETWORKS" },
    { "00B0F5", "NetWorth Technologies, Inc." },
    { "00B1E3", "Cisco Systems, Inc" },
    { "00B338", "Kontron Asia Pacific Design Sdn. Bhd" },
    { "00B342", "MacroSAN Technologies Co., Ltd." },
    { "00B362", "Apple, Inc." },
    { "00B4F5", "DongGuan Siyoto Electronics Co., Ltd " },
    { "00B56D", "David Electronics Co., LTD." },
    { "00B5D0", "Samsung Electronics Co.,Ltd" },
    { "00B5D6", "Omnibit Inc." },
    { "00B600", "VOIM Co., Ltd." },
    { "00B670", "Cisco Systems, Inc" },
    { "00B69F", "Latch" },
    { "00B771", "Cisco Systems, Inc" },
    { "00B78D", "Nanjing Shining Electric Automation Co., Ltd" },
    { "00B7A8", "Heinzinger electronic GmbH" },
    { "00B810", "Yichip Microelectronics (Hangzhou) Co.,Ltd" },
    { "00B881", "New platforms LLC" },
    { "00B8B3", "Cisco Systems, Inc" },
    { "00B8B6", "Motorola Mobility LLC, a Lenovo Company" },
    { "00B8C2", "Heights Telecom T ltd" },
    { "00B9F6", "Shenzhen Super Rich Electronics Co.,Ltd" },
    { "00BAC0", "Biometric Access Company" },
    { "00BB01", "OCTOTHORPE CORP." },
    { "00BB1C", "Huawei Device Co., Ltd." },
    { "00BB3A", "Amazon Technologies Inc." },
    { "00BB43", "Tiinlab Corporation" },
    { "00BB60", "Intel Corporate" },
    { "00BB8E", "HME Co., Ltd." },
    { "00BBC1", "CANON INC." },
    { "00BBF0", "UNGERMANN-BASS INC." },
    { "00BC2F", "Actiontec Electronics Inc." },
    { "00BC60", "Cisco Systems, Inc" },
    { "00BD27", "Exar Corp." },
    { "00BD3A", "Nokia Corporation" },
    { "00BD3E", "Vizio, Inc" },
    { "00BD82", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "00BE3B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "00BE43", "Dell Inc." },
    { "00BE44", "Silicon Laboratories" },
    { "00BE75", "Cisco Systems, Inc" },
    { "00BE9E", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "00BED5", "New H3C Technologies Co., Ltd" },
    { "00BF15", "Genetec Inc." },
    { "00BF61", "Samsung Electronics Co.,Ltd" },
    { "00BF77", "Cisco Systems, Inc" },
    { "00BFAF", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "00C000", "LANOPTICS, LTD." },
    { "00C001", "DIATEK PATIENT MANAGMENT" },
    { "00C002", "Sercomm Corporation." },
    { "00C003", "GLOBALNET COMMUNICATIONS" },
    { "00C004", "JAPAN BUSINESS COMPUTER CO.LTD" },
    { "00C005", "LIVINGSTON ENTERPRISES, INC." },
    { "00C006", "NIPPON AVIONICS CO., LTD." },
    { "00C007", "PINNACLE DATA SYSTEMS, INC." },
    { "00C008", "SECO SRL" },
    { "00C009", "KT TECHNOLOGY (S) PTE LTD" },
    { "00C00A", "MICRO CRAFT" },
    { "00C00B", "NORCONTROL A.S." },
    { "00C00C", "RELIA TECHNOLGIES" },
    { "00C00D", "ADVANCED LOGIC RESEARCH, INC." },
    { "00C00E", "PSITECH, INC." },
    { "00C00F", "QUANTUM SOFTWARE SYSTEMS LTD." },
    { "00C010", "HIRAKAWA HEWTECH CORP." },
    { "00C011", "INTERACTIVE COMPUTING DEVICES" },
    { "00C012", "NETSPAN CORPORATION" },
    { "00C013", "NETRIX" },
    { "00C014", "TELEMATICS CALABASAS INT'L,INC" },
    { "00C015", "NEW MEDIA CORPORATION" },
    { "00C016", "ELECTRONIC THEATRE CONTROLS" },
    { "00C017", "NetAlly" },
    { "00C018", "LANART CORPORATION" },
    { "00C019", "LEAP TECHNOLOGY, INC." },
    { "00C01A", "COROMETRICS MEDICAL SYSTEMS" },
    { "00C01B", "SOCKET COMMUNICATIONS, INC." },
    { "00C01C", "INTERLINK COMMUNICATIONS LTD." },
    { "00C01D", "GRAND JUNCTION NETWORKS, INC." },
    { "00C01E", "LA FRANCAISE DES JEUX" },
    { "00C01F", "S.E.R.C.E.L." },
    { "00C020", "ARCO ELECTRONIC, CONTROL LTD." },
    { "00C021", "NETEXPRESS" },
    { "00C022", "LASERMASTER TECHNOLOGIES, INC." },
    { "00C023", "TUTANKHAMON ELECTRONICS" },
    { "00C024", "EDEN SISTEMAS DE COMPUTACAO SA" },
    { "00C025", "DATAPRODUCTS CORPORATION" },
    { "00C026", "LANS TECHNOLOGY CO., LTD." },
    { "00C027", "CIPHER SYSTEMS, INC." },
    { "00C028", "JASCO CORPORATION" },
    { "00C029", "Aginode Germany GmbH" },
    { "00C02A", "OHKURA ELECTRIC CO., LTD." },
    { "00C02B", "GERLOFF GESELLSCHAFT FUR" },
    { "00C02C", "CENTRUM COMMUNICATIONS, INC." },
    { "00C02D", "FUJI PHOTO FILM CO., LTD." },
    { "00C02E", "NETWIZ" },
    { "00C02F", "OKUMA CORPORATION" },
    { "00C030", "INTEGRATED ENGINEERING B. V." },
    { "00C031", "DESIGN RESEARCH SYSTEMS, INC." },
    { "00C032", "I-CUBED LIMITED" },
    { "00C033", "TELEBIT COMMUNICATIONS APS" },
    { "00C034", "TRANSACTION NETWORK" },
    { "00C035", "QUINTAR COMPANY" },
    { "00C036", "RAYTECH ELECTRONIC CORP." },
    { "00C037", "DYNATEM" },
    { "00C038", "RASTER IMAGE PROCESSING SYSTEM" },
    { "00C039", "Teridian Semiconductor Corporation" },
    { "00C03A", "duagon Germany GmbH" },
    { "00C03B", "MULTIACCESS COMPUTING CORP." },
    { "00C03C", "TOWER TECH S.R.L." },
    { "00C03D", "WIESEMANN & THEIS GMBH" },
    { "00C03E", "FA. GEBR. HELLER GMBH" },
    { "00C03F", "STORES AUTOMATED SYSTEMS, INC." },
    { "00C040", "ECCI" },
    { "00C041", "DIGITAL TRANSMISSION SYSTEMS" },
    { "00C042", "DATALUX CORP." },
    { "00C043", "STRATACOM" },
    { "00C044", "EMCOM CORPORATION" },
    { "00C045", "ISOLATION SYSTEMS, LTD." },
    { "00C046", "Blue Chip Technology Ltd" },
    { "00C047", "UNIMICRO SYSTEMS, INC." },
    { "00C048", "BAY TECHNICAL ASSOCIATES" },
    { "00C049", "U.S. ROBOTICS, INC." },
    { "00C04A", "GROUP 2000 AG" },
    { "00C04B", "CREATIVE MICROSYSTEMS" },
    { "00C04C", "DEPARTMENT OF FOREIGN AFFAIRS" },
    { "00C04D", "MITEC, INC." },
    { "00C04E", "COMTROL CORPORATION" },
    { "00C04F", "Dell Inc." },
    { "00C050", "TOYO DENKI SEIZO K.K." },
    { "00C051", "ADVANCED INTEGRATION RESEARCH" },
    { "00C052", "BURR-BROWN" },
    { "00C053", "Aspect Software Inc." },
    { "00C054", "NETWORK PERIPHERALS, LTD." },
    { "00C055", "MODULAR COMPUTING TECHNOLOGIES" },
    { "00C056", "SOMELEC" },
    { "00C057", "MYCO ELECTRONICS" },
    { "00C058", "DATAEXPERT CORP." },
    { "00C059", "DENSO CORPORATION" },
    { "00C05A", "SEMAPHORE COMMUNICATIONS CORP." },
    { "00C05B", "NETWORKS NORTHWEST, INC." },
    { "00C05C", "ELONEX PLC" },
    { "00C05D", "L&N TECHNOLOGIES" },
    { "00C05E", "VARI-LITE, INC." },
    { "00C05F", "FINE-PAL COMPANY LIMITED" },
    { "00C060", "ID SCANDINAVIA AS" },
    { "00C061", "SOLECTEK CORPORATION" },
    { "00C062", "IMPULSE TECHNOLOGY" },
    { "00C063", "MORNING STAR TECHNOLOGIES, INC" },
    { "00C064", "General Datacomm LLC" },
    { "00C065", "SCOPE COMMUNICATIONS, INC." },
    { "00C066", "DOCUPOINT, INC." },
    { "00C067", "UNITED BARCODE INDUSTRIES" },
    { "00C068", "HME Clear-Com LTD." },
    { "00C069", "Axxcelera Broadband Wireless" },
    { "00C06A", "Zahner-Elektrik Ingeborg Zahner-Schiller GmbH & Co. KG." },
    { "00C06B", "OSI PLUS CORPORATION" },
    { "00C06C", "SVEC COMPUTER CORP." },
    { "00C06D", "BOCA RESEARCH, INC." },
    { "00C06E", "HAFT TECHNOLOGY, INC." },
    { "00C06F", "KOMATSU LTD." },
    { "00C070", "SECTRA SECURE-TRANSMISSION AB" },
    { "00C071", "AREANEX COMMUNICATIONS, INC." },
    { "00C072", "KNX LTD." },
    { "00C073", "XEDIA CORPORATION" },
    { "00C074", "TOYODA AUTOMATIC LOOM" },
    { "00C075", "XANTE CORPORATION" },
    { "00C076", "I-DATA INTERNATIONAL A-S" },
    { "00C077", "DAEWOO TELECOM LTD." },
    { "00C078", "COMPUTER SYSTEMS ENGINEERING" },
    { "00C079", "FONSYS CO.,LTD." },
    { "00C07A", "PRIVA B.V." },
    { "00C07B", "ASCEND COMMUNICATIONS, INC." },
    { "00C07C", "HIGHTECH INFORMATION" },
    { "00C07D", "RISC DEVELOPMENTS LTD." },
    { "00C07E", "KUBOTA CORPORATION ELECTRONIC" },
    { "00C07F", "NUPON COMPUTING CORP." },
    { "00C080", "NETSTAR, INC." },
    { "00C081", "METRODATA LTD." },
    { "00C082", "MOORE PRODUCTS CO." },
    { "00C083", "TRACE MOUNTAIN PRODUCTS, INC." },
    { "00C084", "DATA LINK CORP. LTD." },
    { "00C085", "ELECTRONICS FOR IMAGING, INC." },
    { "00C086", "THE LYNK CORPORATION" },
    { "00C087", "UUNET TECHNOLOGIES, INC." },
    { "00C088", "EKF ELEKTRONIK GMBH" },
    { "00C089", "TELINDUS DISTRIBUTION" },
    { "00C08A", "Lauterbach GmbH" },
    { "00C08B", "RISQ MODULAR SYSTEMS, INC." },
    { "00C08C", "PERFORMANCE TECHNOLOGIES, INC." },
    { "00C08D", "TRONIX PRODUCT DEVELOPMENT" },
    { "00C08E", "NETWORK INFORMATION TECHNOLOGY" },
    { "00C08F", "Panasonic Electric Works Co., Ltd." },
    { "00C090", "PRAIM S.R.L." },
    { "00C091", "JABIL CIRCUIT, INC." },
    { "00C092", "MENNEN MEDICAL INC." },
    { "00C093", "ALTA RESEARCH CORP." },
    { "00C094", "VMX INC." },
    { "00C095", "ZNYX Networks, Inc." },
    { "00C096", "TAMURA CORPORATION" },
    { "00C097", "ARCHIPEL SA" },
    { "00C098", "CHUNTEX ELECTRONIC CO., LTD." },
    { "00C099", "YOSHIKI INDUSTRIAL CO.,LTD." },
    { "00C09A", "PHOTONICS CORPORATION" },
    { "00C09B", "Tellabs Enterprise, Inc." },
    { "00C09C", "HIOKI E.E. CORPORATION" },
    { "00C09D", "DISTRIBUTED SYSTEMS INT'L, INC" },
    { "00C09E", "CACHE COMPUTERS, INC." },
    { "00C09F", "Quanta Computer Inc." },
    { "00C0A0", "ADVANCE MICRO RESEARCH, INC." },
    { "00C0A1", "TOKYO DENSHI SEKEI CO." },
    { "00C0A2", "INTERMEDIUM A/S" },
    { "00C0A3", "DUAL ENTERPRISES CORPORATION" },
    { "00C0A4", "UNIGRAF OY" },
    { "00C0A5", "DICKENS DATA SYSTEMS" },
    { "00C0A6", "EXICOM AUSTRALIA PTY. LTD" },
    { "00C0A7", "SEEL LTD." },
    { "00C0A8", "GVC CORPORATION" },
    { "00C0A9", "BARRON MCCANN LTD." },
    { "00C0AA", "SILICON VALLEY COMPUTER" },
    { "00C0AB", "Telco Systems, Inc. " },
    { "00C0AC", "GAMBIT COMPUTER COMMUNICATIONS" },
    { "00C0AD", "MARBEN COMMUNICATION SYSTEMS" },
    { "00C0AE", "TOWERCOM CO. INC. DBA PC HOUSE" },
    { "00C0AF", "TEKLOGIX INC." },
    { "00C0B0", "GCC TECHNOLOGIES,INC." },
    { "00C0B1", "GENIUS NET CO." },
    { "00C0B2", "NORAND CORPORATION" },
    { "00C0B3", "COMSTAT DATACOMM CORPORATION" },
    { "00C0B4", "MYSON TECHNOLOGY, INC." },
    { "00C0B5", "CORPORATE NETWORK SYSTEMS,INC." },
    { "00C0B6", "HVE, Inc. " },
    { "00C0B7", "AMERICAN POWER CONVERSION CORP" },
    { "00C0B8", "FRASER'S HILL LTD." },
    { "00C0B9", "FUNK SOFTWARE, INC." },
    { "00C0BA", "NETVANTAGE" },
    { "00C0BB", "FORVAL CREATIVE, INC." },
    { "00C0BC", "TELECOM AUSTRALIA/CSSC" },
    { "00C0BD", "INEX TECHNOLOGIES, INC." },
    { "00C0BE", "ALCATEL - SEL" },
    { "00C0BF", "TECHNOLOGY CONCEPTS, LTD." },
    { "00C0C0", "SHORE MICROSYSTEMS, INC." },
    { "00C0C1", "QUAD/GRAPHICS, INC." },
    { "00C0C2", "INFINITE NETWORKS LTD." },
    { "00C0C3", "ACUSON COMPUTED SONOGRAPHY" },
    { "00C0C4", "COMPUTER OPERATIONAL" },
    { "00C0C5", "SID INFORMATICA" },
    { "00C0C6", "PERSONAL MEDIA CORP." },
    { "00C0C7", "SPARKTRUM MICROSYSTEMS, INC." },
    { "00C0C8", "MICRO BYTE PTY. LTD." },
    { "00C0C9", "ELSAG BAILEY PROCESS" },
    { "00C0CA", "ALFA, INC." },
    { "00C0CB", "CONTROL TECHNOLOGY CORPORATION" },
    { "00C0CC", "TELESCIENCES CO SYSTEMS, INC." },
    { "00C0CD", "COMELTA, S.A." },
    { "00C0CE", "CEI SYSTEMS & ENGINEERING PTE" },
    { "00C0CF", "IMATRAN VOIMA OY" },
    { "00C0D0", "RATOC SYSTEM INC." },
    { "00C0D1", "COMTREE TECHNOLOGY CORPORATION" },
    { "00C0D2", "SYNTELLECT, INC." },
    { "00C0D3", "OLYMPUS IMAGE SYSTEMS, INC." },
    { "00C0D4", "AXON NETWORKS, INC." },
    { "00C0D5", "Werbeagentur Jürgen Siebert" },
    { "00C0D6", "J1 SYSTEMS, INC." },
    { "00C0D7", "TAIWAN TRADING CENTER DBA" },
    { "00C0D8", "UNIVERSAL DATA SYSTEMS" },
    { "00C0D9", "QUINTE NETWORK CONFIDENTIALITY" },
    { "00C0DA", "NICE SYSTEMS LTD." },
    { "00C0DB", "IPC CORPORATION (PTE) LTD." },
    { "00C0DC", "EOS TECHNOLOGIES, INC." },
    { "00C0DD", "QLogic Corporation" },
    { "00C0DE", "ZCOMM, INC." },
    { "00C0DF", "KYE Systems Corp." },
    { "00C0E0", "DSC COMMUNICATION CORP." },
    { "00C0E1", "SONIC SOLUTIONS" },
    { "00C0E2", "CALCOMP, INC." },
    { "00C0E3", "OSITECH COMMUNICATIONS, INC." },
    { "00C0E4", "SIEMENS BUILDING" },
    { "00C0E5", "GESPAC, S.A." },
    { "00C0E6", "Verilink Corporation" },
    { "00C0E7", "FIBERDATA AB" },
    { "00C0E8", "PLEXCOM, INC." },
    { "00C0E9", "OAK SOLUTIONS, LTD." },
    { "00C0EA", "ARRAY TECHNOLOGY LTD." },
    { "00C0EB", "SEH COMPUTERTECHNIK GMBH" },
    { "00C0EC", "DAUPHIN TECHNOLOGY" },
    { "00C0ED", "US ARMY ELECTRONIC" },
    { "00C0EE", "KYOCERA Display Corporation" },
    { "00C0EF", "ABIT CORPORATION" },
    { "00C0F0", "Kingston Technology Company, Inc." },
    { "00C0F1", "SHINKO ELECTRIC CO., LTD." },
    { "00C0F2", "Lantronix" },
    { "00C0F3", "NETWORK COMMUNICATIONS CORP." },
    { "00C0F4", "INTERLINK SYSTEM CO., LTD." },
    { "00C0F5", "METACOMP, INC." },
    { "00C0F6", "CELAN TECHNOLOGY INC." },
    { "00C0F7", "ENGAGE COMMUNICATION, INC." },
    { "00C0F8", "ABOUT COMPUTING INC." },
    { "00C0F9", "Artesyn Embedded Technologies" },
    { "00C0FA", "CANARY COMMUNICATIONS, INC." },
    { "00C0FB", "ADVANCED TECHNOLOGY LABS" },
    { "00C0FC", "ELASTIC REALITY, INC." },
    { "00C0FD", "PROSUM" },
    { "00C0FE", "APTEC COMPUTER SYSTEMS, INC." },
    { "00C0FF", "Seagate Cloud Systems Inc" },
    { "00C14F", "DDL Co,.ltd." },
    { "00C164", "Cisco Systems, Inc" },
    { "00C1B1", "Cisco Systems, Inc" },
    { "00C28F", "Allied Telesis K.K." },
    { "00C2C6", "Intel Corporate" },
    { "00C30A", "Xiaomi Communications Co Ltd" },
    { "00C343", "E-T-A Circuit Breakers Ltd" },
    { "00C3F4", "Samsung Electronics Co.,Ltd" },
    { "00C52C", "Juniper Networks" },
    { "00C585", "Apple, Inc." },
    { "00C5DB", "Datatech Sistemas Digitales Avanzados SL" },
    { "00C610", "Apple, Inc." },
    { "00C711", "ITEL MOBILE LIMITED" },
    { "00C84E", "Hewlett Packard Enterprise" },
    { "00C88B", "Cisco Systems, Inc" },
    { "00C896", "CIG SHANGHAI CO LTD" },
    { "00CAE0", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "00CAE5", "Cisco Systems, Inc" },
    { "00CB00", "Private" },
    { "00CB51", "Sagemcom Broadband SAS" },
    { "00CB7A", "Vantiva USA LLC" },
    { "00CBB4", "SHENZHEN ATEKO PHOTOELECTRICITY CO.,LTD" },
    { "00CBBD", "Cambridge Broadband Networks Group" },
    { "00CC34", "Juniper Networks" },
    { "00CC3F", "Universal Electronics, Inc." },
    { "00CCFC", "Cisco Systems, Inc" },
    { "00CD90", "MAS Elektronik AG" },
    { "00CDFE", "Apple, Inc." },
    { "00CE30", "Express LUCK Industrial Ltd." },
    { "00CF1C", "Communication Machinery Corporation" },
    { "00CFC0", "China Mobile Group Device Co.,Ltd." },
    { "00D000", "FERRAN SCIENTIFIC, INC." },
    { "00D001", "VST TECHNOLOGIES, INC." },
    { "00D002", "DITECH CORPORATION" },
    { "00D003", "COMDA ENTERPRISES CORP." },
    { "00D004", "PENTACOM LTD." },
    { "00D005", "ZHS ZEITMANAGEMENTSYSTEME" },
    { "00D006", "Cisco Systems, Inc" },
    { "00D007", "MIC ASSOCIATES, INC." },
    { "00D008", "MACTELL CORPORATION" },
    { "00D009", "HSING TECH. ENTERPRISE CO. LTD" },
    { "00D00A", "LANACCESS TELECOM S.A." },
    { "00D00B", "RHK TECHNOLOGY, INC." },
    { "00D00C", "SNIJDER MICRO SYSTEMS" },
    { "00D00D", "MICROMERITICS INSTRUMENT" },
    { "00D00E", "PLURIS, INC." },
    { "00D00F", "SPEECH DESIGN GMBH" },
    { "00D010", "CONVERGENT NETWORKS, INC." },
    { "00D011", "PRISM VIDEO, INC." },
    { "00D012", "GATEWORKS CORP." },
    { "00D013", "PRIMEX AEROSPACE COMPANY" },
    { "00D014", "ROOT, INC." },
    { "00D015", "UNIVEX MICROTECHNOLOGY CORP." },
    { "00D016", "SCM MICROSYSTEMS, INC." },
    { "00D017", "SYNTECH INFORMATION CO., LTD." },
    { "00D018", "QWES. COM, INC." },
    { "00D019", "DAINIPPON SCREEN CORPORATE" },
    { "00D01A", "URMET TLC S.P.A." },
    { "00D01B", "MIMAKI ENGINEERING CO., LTD." },
    { "00D01C", "SBS TECHNOLOGIES," },
    { "00D01D", "FURUNO ELECTRIC CO., LTD." },
    { "00D01E", "PINGTEL CORP." },
    { "00D01F", "Senetas Corporation Ltd" },
    { "00D020", "AIM SYSTEM, INC." },
    { "00D021", "REGENT ELECTRONICS CORP." },
    { "00D022", "INCREDIBLE TECHNOLOGIES, INC." },
    { "00D023", "INFORTREND TECHNOLOGY, INC." },
    { "00D024", "Cognex Corporation" },
    { "00D025", "XROSSTECH, INC." },
    { "00D026", "HIRSCHMANN AUSTRIA GMBH" },
    { "00D027", "APPLIED AUTOMATION, INC." },
    { "00D028", "Harmonic, Inc" },
    { "00D029", "WAKEFERN FOOD CORPORATION" },
    { "00D02A", "Voxent Systems Ltd." },
    { "00D02B", "JETCELL, INC." },
    { "00D02C", "CAMPBELL SCIENTIFIC, INC." },
    { "00D02D", "Resideo" },
    { "00D02E", "COMMUNICATION AUTOMATION CORP." },
    { "00D02F", "VLSI TECHNOLOGY INC." },
    { "00D030", "Safetran Systems Corp" },
    { "00D031", "INDUSTRIAL LOGIC CORPORATION" },
    { "00D032", "YANO ELECTRIC CO., LTD." },
    { "00D033", "DALIAN DAXIAN NETWORK" },
    { "00D034", "ORMEC SYSTEMS CORP." },
    { "00D035", "BEHAVIOR TECH. COMPUTER CORP." },
    { "00D036", "TECHNOLOGY ATLANTA CORP." },
    { "00D037", "Commscope" },
    { "00D038", "FIVEMERE, LTD." },
    { "00D039", "UTILICOM, INC." },
    { "00D03A", "ZONEWORX, INC." },
    { "00D03B", "VISION PRODUCTS PTY. LTD." },
    { "00D03C", "Vieo, Inc." },
    { "00D03D", "GALILEO TECHNOLOGY, LTD." },
    { "00D03E", "ROCKETCHIPS, INC." },
    { "00D03F", "AMERICAN COMMUNICATION" },
    { "00D040", "SYSMATE CO., LTD." },
    { "00D041", "AMIGO TECHNOLOGY CO., LTD." },
    { "00D042", "MAHLO GMBH & CO. UG" },
    { "00D043", "ZONAL RETAIL DATA SYSTEMS" },
    { "00D044", "ALIDIAN NETWORKS, INC." },
    { "00D045", "KVASER AB" },
    { "00D046", "DOLBY LABORATORIES, INC." },
    { "00D047", "XN TECHNOLOGIES" },
    { "00D048", "ECTON, INC." },
    { "00D049", "IMPRESSTEK CO., LTD." },
    { "00D04A", "PRESENCE TECHNOLOGY GMBH" },
    { "00D04B", "LA CIE GROUP S.A." },
    { "00D04C", "Eseye Design Ltd" },
    { "00D04D", "DIV OF RESEARCH & STATISTICS" },
    { "00D04E", "LOGIBAG" },
    { "00D04F", "BITRONICS, INC." },
    { "00D050", "Iskratel d.o.o." },
    { "00D051", "O2 MICRO, INC." },
    { "00D052", "ASCEND COMMUNICATIONS, INC." },
    { "00D053", "CONNECTED SYSTEMS" },
    { "00D054", "SAS INSTITUTE INC." },
    { "00D055", "KATHREIN-WERKE KG" },
    { "00D056", "SOMAT CORPORATION" },
    { "00D057", "ULTRAK, INC." },
    { "00D058", "Cisco Systems, Inc" },
    { "00D059", "AMBIT MICROSYSTEMS CORP." },
    { "00D05A", "SYMBIONICS, LTD." },
    { "00D05B", "ACROLOOP MOTION CONTROL" },
    { "00D05C", "KATHREIN TechnoTrend GmbH" },
    { "00D05D", "INTELLIWORXX, INC." },
    { "00D05E", "STRATABEAM TECHNOLOGY, INC." },
    { "00D05F", "VALCOM, INC." },
    { "00D060", "Panasonic Europe Ltd." },
    { "00D061", "TREMON ENTERPRISES CO., LTD." },
    { "00D062", "DIGIGRAM" },
    { "00D063", "Cisco Systems, Inc" },
    { "00D064", "MULTITEL" },
    { "00D065", "TOKO ELECTRIC" },
    { "00D066", "WINTRISS ENGINEERING CORP." },
    { "00D067", "CAMPIO COMMUNICATIONS" },
    { "00D068", "IWILL CORPORATION" },
    { "00D069", "TECHNOLOGIC SYSTEMS" },
    { "00D06A", "LINKUP SYSTEMS CORPORATION" },
    { "00D06B", "SR TELECOM INC." },
    { "00D06C", "SHAREWAVE, INC." },
    { "00D06D", "ACRISON, INC." },
    { "00D06E", "TRENDVIEW RECORDERS LTD." },
    { "00D06F", "KMC CONTROLS" },
    { "00D070", "LONG WELL ELECTRONICS CORP." },
    { "00D071", "ECHELON CORP." },
    { "00D072", "BROADLOGIC" },
    { "00D073", "ACN ADVANCED COMMUNICATIONS" },
    { "00D074", "TAQUA SYSTEMS, INC." },
    { "00D075", "ALARIS MEDICAL SYSTEMS, INC." },
    { "00D076", "Bank of America" },
    { "00D077", "LUCENT TECHNOLOGIES" },
    { "00D078", "Eltex of Sweden AB" },
    { "00D079", "Cisco Systems, Inc" },
    { "00D07A", "AMAQUEST COMPUTER CORP." },
    { "00D07B", "COMCAM INTERNATIONAL INC" },
    { "00D07C", "JTEKT ELECTRONICS CORPORATION" },
    { "00D07D", "COSINE COMMUNICATIONS" },
    { "00D07E", "KEYCORP LTD." },
    { "00D07F", "STRATEGY & TECHNOLOGY, LIMITED" },
    { "00D080", "EXABYTE CORPORATION" },
    { "00D081", "RTD Embedded Technologies, Inc." },
    { "00D082", "IOWAVE INC." },
    { "00D083", "INVERTEX, INC." },
    { "00D084", "NEXCOMM SYSTEMS, INC." },
    { "00D085", "OTIS ELEVATOR COMPANY" },
    { "00D086", "FOVEON, INC." },
    { "00D087", "MICROFIRST INC." },
    { "00D088", "Commscope" },
    { "00D089", "DYNACOLOR, INC." },
    { "00D08A", "PHOTRON USA" },
    { "00D08B", "ADVA Optical Networking Ltd." },
    { "00D08C", "GENOA TECHNOLOGY, INC." },
    { "00D08D", "PHOENIX GROUP, INC." },
    { "00D08E", "Grass Valley, A Belden Brand" },
    { "00D08F", "ARDENT TECHNOLOGIES, INC." },
    { "00D090", "Cisco Systems, Inc" },
    { "00D091", "SMARTSAN SYSTEMS, INC." },
    { "00D092", "GLENAYRE WESTERN MULTIPLEX" },
    { "00D093", "TQ - COMPONENTS GMBH" },
    { "00D094", "Seeion Control LLC" },
    { "00D095", "Alcatel-Lucent Enterprise" },
    { "00D096", "3COM EUROPE LTD" },
    { "00D097", "Cisco Systems, Inc" },
    { "00D098", "Photon Dynamics Canada Inc." },
    { "00D099", "Elcard Wireless Systems Oy" },
    { "00D09A", "FILANET CORPORATION" },
    { "00D09B", "SPECTEL LTD." },
    { "00D09C", "KAPADIA COMMUNICATIONS" },
    { "00D09D", "VERIS INDUSTRIES" },
    { "00D09E", "2Wire Inc" },
    { "00D09F", "NOVTEK TEST SYSTEMS" },
    { "00D0A0", "MIPS DENMARK" },
    { "00D0A1", "OSKAR VIERLING GMBH + CO. KG" },
    { "00D0A2", "INTEGRATED DEVICE" },
    { "00D0A3", "VOCAL DATA, INC." },
    { "00D0A4", "ALANTRO COMMUNICATIONS" },
    { "00D0A5", "AMERICAN ARIUM" },
    { "00D0A6", "LANBIRD TECHNOLOGY CO., LTD." },
    { "00D0A7", "TOKYO SOKKI KENKYUJO CO., LTD." },
    { "00D0A8", "NETWORK ENGINES, INC." },
    { "00D0A9", "SHINANO KENSHI CO., LTD." },
    { "00D0AA", "CHASE COMMUNICATIONS" },
    { "00D0AB", "DELTAKABEL TELECOM CV" },
    { "00D0AC", "Commscope, Inc" },
    { "00D0AD", "TL INDUSTRIES" },
    { "00D0AE", "ORESIS COMMUNICATIONS, INC." },
    { "00D0AF", "CUTLER-HAMMER, INC." },
    { "00D0B0", "BITSWITCH LTD." },
    { "00D0B1", "OMEGA ELECTRONICS SA" },
    { "00D0B2", "Xiotech Corporation" },
    { "00D0B3", "DRS Technologies Canada Ltd" },
    { "00D0B4", "KATSUJIMA CO., LTD." },
    { "00D0B5", "IPricot formerly DotCom" },
    { "00D0B6", "CRESCENT NETWORKS, INC." },
    { "00D0B7", "Intel Corporation" },
    { "00D0B8", "Iomega Corporation" },
    { "00D0B9", "MICROTEK INTERNATIONAL, INC." },
    { "00D0BA", "Cisco Systems, Inc" },
    { "00D0BB", "Cisco Systems, Inc" },
    { "00D0BC", "Cisco Systems, Inc" },
    { "00D0BD", "Lattice Semiconductor Corp. (LPA)" },
    { "00D0BE", "EMUTEC INC." },
    { "00D0BF", "PIVOTAL TECHNOLOGIES" },
    { "00D0C0", "Cisco Systems, Inc" },
    { "00D0C1", "HARMONIC DATA SYSTEMS, LTD." },
    { "00D0C2", "BALTHAZAR TECHNOLOGY AB" },
    { "00D0C3", "VIVID TECHNOLOGY PTE, LTD." },
    { "00D0C4", "TERATECH CORPORATION" },
    { "00D0C5", "COMPUTATIONAL SYSTEMS, INC." },
    { "00D0C6", "THOMAS & BETTS CORP." },
    { "00D0C7", "PATHWAY, INC." },
    { "00D0C8", "Prevas A/S" },
    { "00D0C9", "ADVANTECH CO., LTD." },
    { "00D0CA", "Intrinsyc Software International Inc." },
    { "00D0CB", "DASAN CO., LTD." },
    { "00D0CC", "TECHNOLOGIES LYRE INC." },
    { "00D0CD", "ATAN TECHNOLOGY INC." },
    { "00D0CE", "TASKING Labs" },
    { "00D0CF", "MORETON BAY" },
    { "00D0D0", "ZHONGXING TELECOM LTD." },
    { "00D0D1", "Sycamore Networks" },
    { "00D0D2", "EPILOG CORPORATION" },
    { "00D0D3", "Cisco Systems, Inc" },
    { "00D0D4", "V-BITS, INC." },
    { "00D0D5", "GRUNDIG AG" },
    { "00D0D6", "AETHRA TELECOMUNICAZIONI" },
    { "00D0D7", "B2C2, INC." },
    { "00D0D8", "3COM" },
    { "00D0D9", "DEDICATED MICROCOMPUTERS" },
    { "00D0DA", "TAICOM DATA SYSTEMS CO., LTD." },
    { "00D0DB", "MCQUAY INTERNATIONAL" },
    { "00D0DC", "MODULAR MINING SYSTEMS, INC." },
    { "00D0DD", "SUNRISE TELECOM, INC." },
    { "00D0DE", "PHILIPS MULTIMEDIA NETWORK" },
    { "00D0DF", "KUZUMI ELECTRONICS, INC." },
    { "00D0E0", "DOOIN ELECTRONICS CO." },
    { "00D0E1", "AVIONITEK ISRAEL INC." },
    { "00D0E2", "MRT MICRO, INC." },
    { "00D0E3", "ELE-CHEM ENGINEERING CO., LTD." },
    { "00D0E4", "Cisco Systems, Inc" },
    { "00D0E5", "SOLIDUM SYSTEMS CORP." },
    { "00D0E6", "IBOND INC." },
    { "00D0E7", "VCON TELECOMMUNICATION LTD." },
    { "00D0E8", "MAC SYSTEM CO., LTD." },
    { "00D0E9", "Advantage Century Telecommunication Corp." },
    { "00D0EA", "NEXTONE COMMUNICATIONS, INC." },
    { "00D0EB", "LIGHTERA NETWORKS, INC." },
    { "00D0EC", "NAKAYO Inc" },
    { "00D0ED", "XIOX" },
    { "00D0EE", "DICTAPHONE CORPORATION" },
    { "00D0EF", "IGT" },
    { "00D0F0", "CONVISION TECHNOLOGY GMBH" },
    { "00D0F1", "SEGA ENTERPRISES, LTD." },
    { "00D0F2", "MONTEREY NETWORKS" },
    { "00D0F3", "SOLARI DI UDINE SPA" },
    { "00D0F4", "CARINTHIAN TECH INSTITUTE" },
    { "00D0F5", "ORANGE MICRO, INC." },
    { "00D0F6", "Nokia" },
    { "00D0F7", "NEXT NETS CORPORATION" },
    { "00D0F8", "FUJIAN STAR TERMINAL" },
    { "00D0F9", "ACUTE COMMUNICATIONS CORP." },
    { "00D0FA", "Thales e-Security Ltd." },
    { "00D0FB", "TEK MICROSYSTEMS, INCORPORATED" },
    { "00D0FC", "GRANITE MICROSYSTEMS" },
    { "00D0FD", "OPTIMA TELE.COM, INC." },
    { "00D0FE", "ASTRAL POINT" },
    { "00D0FF", "Cisco Systems, Inc" },
    { "00D11C", "ACETEL" },
    { "00D279", "VINGROUP JOINT STOCK COMPANY" },
    { "00D2B1", "TPV Display Technology (Xiamen) Co.,Ltd." },
    { "00D318", "SPG Controls" },
    { "00D38D", "Hotel Technology Next Generation" },
    { "00D49E", "Intel Corporate" },
    { "00D598", "BOPEL MOBILE TECHNOLOGY CO.,LIMITED" },
    { "00D626", "Mist Systems, Inc." },
    { "00D632", "GE Energy" },
    { "00D6CB", "Quectel Wireless Solutions Co.,Ltd." },
    { "00D6FE", "Cisco Systems, Inc" },
    { "00D76D", "Intel Corporate" },
    { "00D78F", "Cisco Systems, Inc" },
    { "00D861", "Micro-Star INTL CO., LTD." },
    { "00D8A2", "Huawei Device Co., Ltd." },
    { "00D9D1", "Sony Interactive Entertainment Inc." },
    { "00DA27", "Palo Alto Networks" },
    { "00DA55", "Cisco Systems, Inc" },
    { "00DB1E", "Albedo Telecom SL" },
    { "00DB45", "THAMWAY CO.,LTD." },
    { "00DB70", "Apple, Inc." },
    { "00DBDF", "Intel Corporate" },
    { "00DCB2", "Extreme Networks Headquarters" },
    { "00DD00", "UNGERMANN-BASS INC." },
    { "00DD01", "UNGERMANN-BASS INC." },
    { "00DD02", "UNGERMANN-BASS INC." },
    { "00DD03", "UNGERMANN-BASS INC." },
    { "00DD04", "UNGERMANN-BASS INC." },
    { "00DD05", "UNGERMANN-BASS INC." },
    { "00DD06", "UNGERMANN-BASS INC." },
    { "00DD07", "UNGERMANN-BASS INC." },
    { "00DD08", "UNGERMANN-BASS INC." },
    { "00DD09", "UNGERMANN-BASS INC." },
    { "00DD0A", "UNGERMANN-BASS INC." },
    { "00DD0B", "UNGERMANN-BASS INC." },
    { "00DD0C", "UNGERMANN-BASS INC." },
    { "00DD0D", "UNGERMANN-BASS INC." },
    { "00DD0E", "UNGERMANN-BASS INC." },
    { "00DD0F", "UNGERMANN-BASS INC." },
    { "00DD25", "Shenzhen hechengdong Technology Co., Ltd" },
    { "00DDB6", "New H3C Technologies Co., Ltd" },
    { "00DEFB", "Cisco Systems, Inc" },
    { "00DF1D", "Cisco Systems, Inc" },
    { "00E000", "FUJITSU LIMITED" },
    { "00E001", "STRAND LIGHTING LIMITED" },
    { "00E002", "CROSSROADS SYSTEMS, INC." },
    { "00E003", "NOKIA WIRELESS BUSINESS COMMUN" },
    { "00E004", "PMC-SIERRA, INC." },
    { "00E005", "TECHNICAL CORP." },
    { "00E006", "SILICON INTEGRATED SYS. CORP." },
    { "00E007", "Avaya ECS Ltd" },
    { "00E008", "AMAZING CONTROLS! INC." },
    { "00E009", "Stratus Technologies" },
    { "00E00A", "DIBA, INC." },
    { "00E00B", "ROOFTOP COMMUNICATIONS CORP." },
    { "00E00C", "MOTOROLA" },
    { "00E00D", "RADIANT SYSTEMS" },
    { "00E00E", "AVALON IMAGING SYSTEMS, INC." },
    { "00E00F", "Shanghai Baud Data Communication Co.,Ltd." },
    { "00E010", "HESS SB-AUTOMATENBAU GmbH" },
    { "00E011", "UNIDEN CORPORATION" },
    { "00E012", "PLUTO TECHNOLOGIES INTERNATIONAL INC." },
    { "00E013", "EASTERN ELECTRONIC CO., LTD." },
    { "00E014", "Cisco Systems, Inc" },
    { "00E015", "HEIWA CORPORATION" },
    { "00E016", "RAPID CITY COMMUNICATIONS" },
    { "00E017", "EXXACT GmbH" },
    { "00E018", "ASUSTek COMPUTER INC." },
    { "00E019", "ING. GIORDANO ELETTRONICA" },
    { "00E01A", "COMTEC SYSTEMS. CO., LTD." },
    { "00E01B", "SPHERE COMMUNICATIONS, INC." },
    { "00E01C", "CradlePoint, Inc" },
    { "00E01D", "WebTV NETWORKS, INC." },
    { "00E01E", "Cisco Systems, Inc" },
    { "00E01F", "AVIDIA Systems, Inc." },
    { "00E020", "TECNOMEN OY" },
    { "00E021", "FREEGATE CORP." },
    { "00E022", "Analog Devices, Inc." },
    { "00E023", "TELRAD" },
    { "00E024", "GADZOOX NETWORKS" },
    { "00E025", "dit Co., Ltd." },
    { "00E026", "Redlake MASD LLC" },
    { "00E027", "DUX, INC." },
    { "00E028", "APTIX CORPORATION" },
    { "00E029", "STANDARD MICROSYSTEMS CORP." },
    { "00E02A", "TANDBERG TELEVISION AS" },
    { "00E02B", "Extreme Networks Headquarters" },
    { "00E02C", "AST COMPUTER" },
    { "00E02D", "InnoMediaLogic, Inc." },
    { "00E02E", "SPC ELECTRONICS CORPORATION" },
    { "00E02F", "MCNS HOLDINGS, L.P." },
    { "00E030", "MELITA INTERNATIONAL CORP." },
    { "00E031", "HAGIWARA ELECTRIC CO., LTD." },
    { "00E032", "MISYS FINANCIAL SYSTEMS, LTD." },
    { "00E033", "E.E.P.D. GmbH" },
    { "00E034", "Cisco Systems, Inc" },
    { "00E035", "Artesyn Embedded Technologies" },
    { "00E036", "PIONEER CORPORATION" },
    { "00E037", "CENTURY CORPORATION" },
    { "00E038", "PROXIMA CORPORATION" },
    { "00E039", "PARADYNE CORP." },
    { "00E03A", "Cabletron Systems, Inc." },
    { "00E03B", "PROMINET CORPORATION" },
    { "00E03C", "AdvanSys" },
    { "00E03D", "FOCON ELECTRONIC SYSTEMS A/S" },
    { "00E03E", "ALFATECH, INC." },
    { "00E03F", "JATON CORPORATION" },
    { "00E040", "DeskStation Technology, Inc." },
    { "00E041", "CSPI" },
    { "00E042", "Pacom Systems Ltd." },
    { "00E043", "VitalCom" },
    { "00E044", "LSICS CORPORATION" },
    { "00E045", "TOUCHWAVE, INC." },
    { "00E046", "BENTLY NEVADA CORP." },
    { "00E047", "InFocus Corporation" },
    { "00E048", "SDL COMMUNICATIONS, INC." },
    { "00E049", "MICROWI ELECTRONIC GmbH" },
    { "00E04A", "ZX Technologies, Inc" },
    { "00E04B", "JUMP INDUSTRIELLE COMPUTERTECHNIK GmbH" },
    { "00E04C", "REALTEK SEMICONDUCTOR CORP." },
    { "00E04D", "INTERNET INITIATIVE JAPAN, INC" },
    { "00E04E", "SANYO DENKI CO., LTD." },
    { "00E04F", "Cisco Systems, Inc" },
    { "00E050", "EXECUTONE INFORMATION SYSTEMS, INC." },
    { "00E051", "TALX CORPORATION" },
    { "00E052", "Brocade Communications Systems LLC" },
    { "00E053", "CELLPORT LABS, INC." },
    { "00E054", "KODAI HITEC CO., LTD." },
    { "00E055", "INGENIERIA ELECTRONICA COMERCIAL INELCOM S.A." },
    { "00E056", "HOLONTECH CORPORATION" },
    { "00E057", "HAN MICROTELECOM. CO., LTD." },
    { "00E058", "PHASE ONE DENMARK A/S" },
    { "00E059", "CONTROLLED ENVIRONMENTS, LTD." },
    { "00E05A", "GALEA NETWORK SECURITY" },
    { "00E05B", "WEST END SYSTEMS CORP." },
    { "00E05C", "PHC Corporation" },
    { "00E05D", "UNITEC CO., LTD." },
    { "00E05E", "JAPAN AVIATION ELECTRONICS INDUSTRY, LTD." },
    { "00E05F", "e-Net, Inc." },
    { "00E060", "SHERWOOD" },
    { "00E061", "EdgePoint Networks, Inc." },
    { "00E062", "HOST ENGINEERING" },
    { "00E063", "Cabletron Systems, Inc." },
    { "00E064", "SAMSUNG ELECTRONICS" },
    { "00E065", "OPTICAL ACCESS INTERNATIONAL" },
    { "00E066", "ProMax Systems, Inc." },
    { "00E067", "eac AUTOMATION-CONSULTING GmbH" },
    { "00E068", "MERRIMAC SYSTEMS INC." },
    { "00E069", "JAYCOR" },
    { "00E06A", "KAPSCH AG" },
    { "00E06B", "W&G SPECIAL PRODUCTS" },
    { "00E06C", "Ultra Electronics Command & Control Systems" },
    { "00E06D", "COMPUWARE CORPORATION" },
    { "00E06E", "FAR SYSTEMS S.p.A." },
    { "00E06F", "Commscope" },
    { "00E070", "DH TECHNOLOGY" },
    { "00E071", "EPIS MICROCOMPUTER" },
    { "00E072", "LYNK" },
    { "00E073", "NATIONAL AMUSEMENT NETWORK, INC." },
    { "00E074", "TIERNAN COMMUNICATIONS, INC." },
    { "00E075", "Verilink Corporation" },
    { "00E076", "DEVELOPMENT CONCEPTS, INC." },
    { "00E077", "WEBGEAR, INC." },
    { "00E078", "BERKELEY NETWORKS" },
    { "00E079", "A.T.N.R." },
    { "00E07A", "MIKRODIDAKT AB" },
    { "00E07B", "BAY NETWORKS" },
    { "00E07C", "METTLER-TOLEDO, INC." },
    { "00E07D", "NETRONIX, INC." },
    { "00E07E", "WALT DISNEY IMAGINEERING" },
    { "00E07F", "LOGISTISTEM s.r.l." },
    { "00E080", "CONTROL RESOURCES CORPORATION" },
    { "00E081", "TYAN COMPUTER CORP." },
    { "00E082", "ANERMA" },
    { "00E083", "JATO TECHNOLOGIES, INC." },
    { "00E084", "COMPULITE R&D" },
    { "00E085", "GLOBAL MAINTECH, INC." },
    { "00E086", "Emerson Network Power, Avocent Division" },
    { "00E087", "LeCroy - Networking Productions Division" },
    { "00E088", "LTX-Credence CORPORATION" },
    { "00E089", "ION Networks, Inc." },
    { "00E08A", "GEC AVERY, LTD." },
    { "00E08B", "QLogic Corporation" },
    { "00E08C", "NEOPARADIGM LABS, INC." },
    { "00E08D", "PRESSURE SYSTEMS, INC." },
    { "00E08E", "UTSTARCOM" },
    { "00E08F", "Cisco Systems, Inc" },
    { "00E090", "BECKMAN LAB. AUTOMATION DIV." },
    { "00E091", "LG Electronics" },
    { "00E092", "ADMTEK INCORPORATED" },
    { "00E093", "ACKFIN NETWORKS" },
    { "00E094", "OSAI SRL" },
    { "00E095", "ADVANCED-VISION TECHNOLGIES CORP." },
    { "00E096", "SHIMADZU CORPORATION" },
    { "00E097", "CARRIER ACCESS CORPORATION" },
    { "00E098", "AboCom" },
    { "00E099", "SAMSON AG" },
    { "00E09A", "Positron Inc." },
    { "00E09B", "ENGAGE NETWORKS, INC." },
    { "00E09C", "MII" },
    { "00E09D", "SARNOFF CORPORATION" },
    { "00E09E", "Quantum Corporation" },
    { "00E09F", "PIXEL VISION" },
    { "00E0A0", "WILTRON CO." },
    { "00E0A1", "HIMA PAUL HILDEBRANDT GmbH Co. KG" },
    { "00E0A2", "MICROSLATE INC." },
    { "00E0A3", "Cisco Systems, Inc" },
    { "00E0A4", "ESAOTE S.p.A." },
    { "00E0A5", "ComCore Semiconductor, Inc." },
    { "00E0A6", "TELOGY NETWORKS, INC." },
    { "00E0A7", "IPC INFORMATION SYSTEMS, INC." },
    { "00E0A8", "SAT GmbH & Co." },
    { "00E0A9", "FUNAI ELECTRIC CO., LTD." },
    { "00E0AA", "ELECTROSONIC LTD." },
    { "00E0AB", "DIMAT S.A." },
    { "00E0AC", "MIDSCO, INC." },
    { "00E0AD", "EES TECHNOLOGY, LTD." },
    { "00E0AE", "XAQTI CORPORATION" },
    { "00E0AF", "GENERAL DYNAMICS INFORMATION SYSTEMS" },
    { "00E0B0", "Cisco Systems, Inc" },
    { "00E0B1", "Alcatel-Lucent Enterprise" },
    { "00E0B2", "TELMAX COMMUNICATIONS CORP." },
    { "00E0B3", "EtherWAN Systems, Inc." },
    { "00E0B4", "TECHNO SCOPE CO., LTD." },
    { "00E0B5", "ARDENT COMMUNICATIONS CORP." },
    { "00E0B6", "Entrada Networks" },
    { "00E0B7", "Cosworth Electronics Ltd" },
    { "00E0B8", "GATEWAY 2000" },
    { "00E0B9", "BYAS SYSTEMS" },
    { "00E0BA", "BERGHOF AUTOMATIONSTECHNIK GmbH" },
    { "00E0BB", "NBX CORPORATION" },
    { "00E0BC", "SYMON COMMUNICATIONS, INC." },
    { "00E0BD", "INTERFACE SYSTEMS, INC." },
    { "00E0BE", "GENROCO INTERNATIONAL, INC." },
    { "00E0BF", "TORRENT NETWORKING TECHNOLOGIES CORP." },
    { "00E0C0", "SEIWA ELECTRIC MFG. CO., LTD." },
    { "00E0C1", "MEMOREX TELEX JAPAN, LTD." },
    { "00E0C2", "NECSY S.p.A." },
    { "00E0C3", "SAKAI SYSTEM DEVELOPMENT CORP." },
    { "00E0C4", "HORNER ELECTRIC, INC." },
    { "00E0C5", "BCOM ELECTRONICS INC." },
    { "00E0C6", "LINK2IT, L.L.C." },
    { "00E0C7", "EUROTECH SRL" },
    { "00E0C8", "VIRTUAL ACCESS, LTD." },
    { "00E0C9", "AutomatedLogic Corporation" },
    { "00E0CA", "BEST DATA PRODUCTS" },
    { "00E0CB", "RESON, INC." },
    { "00E0CC", "HERO SYSTEMS, LTD." },
    { "00E0CD", "SAAB SENSIS CORPORATION" },
    { "00E0CE", "ARN" },
    { "00E0CF", "INTEGRATED DEVICE" },
    { "00E0D0", "NETSPEED, INC." },
    { "00E0D1", "TELSIS LIMITED" },
    { "00E0D2", "VERSANET COMMUNICATIONS, INC." },
    { "00E0D3", "DATENTECHNIK GmbH" },
    { "00E0D4", "EXCELLENT COMPUTER" },
    { "00E0D5", "Emulex Corporation" },
    { "00E0D6", "COMPUTER & COMMUNICATION RESEARCH LAB." },
    { "00E0D7", "SUNSHINE ELECTRONICS, INC." },
    { "00E0D8", "LANBit Computer, Inc." },
    { "00E0D9", "TAZMO CO., LTD." },
    { "00E0DA", "Alcatel-Lucent Enterprise" },
    { "00E0DB", "ViaVideo Communications, Inc." },
    { "00E0DC", "NEXWARE CORP." },
    { "00E0DD", "Zenith Electronics Corporation" },
    { "00E0DE", "DATAX NV" },
    { "00E0DF", "Zhone Technologies, Inc." },
    { "00E0E0", "SI ELECTRONICS, LTD." },
    { "00E0E1", "G2 NETWORKS, INC." },
    { "00E0E2", "INNOVA CORP." },
    { "00E0E3", "SK-ELEKTRONIK GMBH" },
    { "00E0E4", "FANUC ROBOTICS NORTH AMERICA, Inc." },
    { "00E0E5", "CINCO NETWORKS, INC." },
    { "00E0E6", "INCAA Computers" },
    { "00E0E7", "RAYTHEON E-SYSTEMS, INC." },
    { "00E0E8", "GRETACODER Data Systems AG" },
    { "00E0E9", "DATA LABS, INC." },
    { "00E0EA", "INNOVAT COMMUNICATIONS, INC." },
    { "00E0EB", "DIGICOM SYSTEMS, INCORPORATED" },
    { "00E0EC", "CELESTICA INC." },
    { "00E0ED", "SILICOM, LTD." },
    { "00E0EE", "MAREL HF" },
    { "00E0EF", "DIONEX" },
    { "00E0F0", "ABLER TECHNOLOGY, INC." },
    { "00E0F1", "THAT CORPORATION" },
    { "00E0F2", "ARLOTTO COMNET, INC." },
    { "00E0F3", "WebSprint Communications, Inc." },
    { "00E0F4", "INSIDE Technology A/S" },
    { "00E0F5", "TELES AG" },
    { "00E0F6", "DECISION EUROPE" },
    { "00E0F7", "Cisco Systems, Inc" },
    { "00E0F8", "DICNA CONTROL AB" },
    { "00E0F9", "Cisco Systems, Inc" },
    { "00E0FA", "TRL TECHNOLOGY, LTD." },
    { "00E0FB", "LEIGHTRONIX, INC." },
    { "00E0FC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "00E0FD", "A-TREND TECHNOLOGY CO., LTD." },
    { "00E0FE", "Cisco Systems, Inc" },
    { "00E0FF", "SECURITY DYNAMICS TECHNOLOGIES, Inc." },
    { "00E12F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "00E16D", "Cisco Systems, Inc" },
    { "00E175", "AK-Systems Ltd" },
    { "00E18C", "Intel Corporate" },
    { "00E22C", "China Mobile Group Device Co.,Ltd." },
    { "00E3B2", "Samsung Electronics Co.,Ltd" },
    { "00E400", "Sichuan Changhong Electric Ltd." },
    { "00E406", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "00E421", "Sony Interactive Entertainment Inc." },
    { "00E5E4", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "00E5F1", "BUFFALO.INC" },
    { "00E60E", "Extreme Networks Headquarters" },
    { "00E63A", "Ruckus Wireless" },
    { "00E666", "ARIMA Communications Corp." },
    { "00E6D3", "NIXDORF COMPUTER CORP." },
    { "00E6E8", "Netzin Technology Corporation,.Ltd." },
    { "00E7E3", "zte corporation" },
    { "00E8AB", "Meggitt Training Systems, Inc." },
    { "00E93A", "AzureWave Technology Inc." },
    { "00EABD", "Cisco Systems, Inc" },
    { "00EB2D", "Sony Corporation" },
    { "00EBD5", "Cisco Systems, Inc" },
    { "00EBD8", "MERCUSYS TECHNOLOGIES CO., LTD." },
    { "00EC0A", "Xiaomi Communications Co Ltd" },
    { "00EDB8", "KYOCERA Corporation " },
    { "00EE01", "Enablers Solucoes e Consultoria em Dispositivos" },
    { "00EEAB", "Cisco Systems, Inc" },
    { "00EEBD", "HTC Corporation" },
    { "00F051", "KWB Gmbh" },
    { "00F22C", "Shanghai B-star Technology Co.,Ltd." },
    { "00F28B", "Cisco Systems, Inc" },
    { "00F361", "Amazon Technologies Inc." },
    { "00F39F", "Apple, Inc." },
    { "00F3DB", "WOO Sports" },
    { "00F403", "Orbis Systems Oy" },
    { "00F46F", "Samsung Electronics Co.,Ltd" },
    { "00F48D", "Liteon Technology Corporation" },
    { "00F4B9", "Apple, Inc." },
    { "00F5FD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "00F620", "Google, Inc." },
    { "00F663", "Cisco Systems, Inc" },
    { "00F76F", "Apple, Inc." },
    { "00F7AD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "00F81C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "00F82C", "Cisco Systems, Inc" },
    { "00F860", "PT. Panggung Electric Citrabuana" },
    { "00F871", "Demant A/S" },
    { "00F8CC", "Sagemcom Broadband SAS" },
    { "00F952", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "00FA21", "Samsung Electronics Co.,Ltd" },
    { "00FA3B", "CLOOS ELECTRONIC GMBH" },
    { "00FAB6", "Kontakt Micro-Location Sp z o.o." },
    { "00FADE", "Motorola Mobility LLC, a Lenovo Company" },
    { "00FBF9", "Axiado Corporation" },
    { "00FC58", "WebSilicon Ltd." },
    { "00FC70", "Intrepid Control Systems, Inc." },
    { "00FC8B", "Amazon Technologies Inc." },
    { "00FC8D", "Hitron Technologies. Inc" },
    { "00FCBA", "Cisco Systems, Inc" },
    { "00FD22", "Cisco Systems, Inc" },
    { "00FD45", "Hewlett Packard Enterprise " },
    { "00FD4C", "NEVATEC" },
    { "00FEC8", "Cisco Systems, Inc" },
    { "020701", "RACAL-DATACOM" },
    { "021C7C", "PERQ SYSTEMS CORPORATION" },
    { "026086", "LOGIC REPLACEMENT TECH. LTD." },
    { "02608C", "3COM" },
    { "027001", "RACAL-DATACOM" },
    { "0270B0", "M/A-COM INC. COMPANIES" },
    { "0270B3", "DATA RECALL LTD." },
    { "029D8E", "CARDIAC RECORDERS, INC." },
    { "02AA3C", "OLIVETTI TELECOMM SPA (OLTECO)" },
    { "02BB01", "OCTOTHORPE CORP." },
    { "02C08C", "3COM" },
    { "02CF1C", "Communication Machinery Corporation" },
    { "02E6D3", "NIXDORF COMPUTER CORP." },
    { "040067", "Stanley Black & Decker" },
    { "04006E", "Google, Inc." },
    { "0401A1", "Fortinet, Inc." },
    { "0401BB", "TECNO MOBILE LIMITED" },
    { "04021F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0402CA", "Shenzhen Vtsonic Co.,ltd" },
    { "040312", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "0403D6", "Nintendo Co.,Ltd" },
    { "0404B8", "China Hualu Panasonic AVC Networks Co., LTD." },
    { "0404EA", "Valens Semiconductor Ltd." },
    { "0405DD", "Shenzhen Cultraview Digital Technology Co., Ltd" },
    { "04072E", "VTech Electronics Ltd." },
    { "040973", "Hewlett Packard Enterprise " },
    { "040986", "Arcadyan Corporation" },
    { "0409A5", "HFR, Inc." },
    { "040A83", "Alcatel-Lucent" },
    { "040AE0", "XMIT AG COMPUTER NETWORKS" },
    { "040CCE", "Apple, Inc." },
    { "040D84", "Silicon Laboratories" },
    { "040E3C", "HP Inc." },
    { "040EC2", "ViewSonic Mobile China Limited" },
    { "040F66", "TP-Link Systems Inc." },
    { "04106B", "Xiaomi Communications Co Ltd" },
    { "041119", "IEEE Registration Authority" },
    { "04137A", "Apple, Inc." },
    { "041471", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "041552", "Apple, Inc." },
    { "0415D9", "Viwone" },
    { "0417B6", "Smart Innovation LLC" },
    { "04180F", "Samsung Electronics Co.,Ltd" },
    { "041892", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0418B6", "Private" },
    { "0418D6", "Ubiquiti Inc" },
    { "04197F", "Grasphere Japan" },
    { "041A04", "WaveIP" },
    { "041B6D", "LG Electronics (Mobile Communications)" },
    { "041B94", "Host Mobility AB" },
    { "041BBA", "Samsung Electronics Co.,Ltd" },
    { "041D10", "Dream Ware Inc." },
    { "041DC7", "zte corporation" },
    { "041E64", "Apple, Inc." },
    { "041E7A", "DSPWorks" },
    { "041EFA", "BISSELL Homecare, Inc." },
    { "041FB8", "vivo Mobile Communication Co., Ltd." },
    { "042084", "zte corporation" },
    { "04208A", "浙江路川科技有限公司" },
    { "04209A", "Panasonic Corporation AVC Networks Company" },
    { "042144", "Sunitec Enterprise Co.,Ltd" },
    { "04214C", "Insight Energy Ventures LLC" },
    { "042234", "Wireless Standard Extensions" },
    { "042322", "Texas Instruments" },
    { "0425C5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0425E0", "Taicang T&W Electronics" },
    { "0425E8", "Texas Instruments" },
    { "0425F0", "Nokia" },
    { "042605", "Bosch Building Automation GmbH" },
    { "042665", "Apple, Inc." },
    { "042728", "Microsoft Corporation" },
    { "042758", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "04292E", "Samsung Electronics Co.,Ltd" },
    { "042AE2", "Cisco Systems, Inc" },
    { "042B58", "Shenzhen Hanzsung Technology Co.,Ltd" },
    { "042BBB", "PicoCELA, Inc." },
    { "042DAD", "Areus GmbH" },
    { "042DB4", "First Property (Beijing) Co., Ltd Modern MOMA Branch" },
    { "042F56", "ATOCS (Shenzhen) LTD" },
    { "043110", "Inspur Group Co., Ltd." },
    { "043201", "Broadcom Limited" },
    { "0432F4", "Partron" },
    { "04331F", "Huawei Device Co., Ltd." },
    { "043385", "Nanchang BlackShark Co.,Ltd." },
    { "043389", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0433C2", "Intel Corporate" },
    { "0434C3", "Qingdao Goertek Horizons Tecnology Co.,LTD" },
    { "0434F6", "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd." },
    { "04359B", "WuLu Networks Pty Ltd" },
    { "043604", "Gyeyoung I&T" },
    { "0436B8", "I&C Technology" },
    { "043855", "Scopus International Pvt. Ltd." },
    { "0438DC", "China Unicom Online Information Technology Co.,Ltd" },
    { "043926", "China Dragon Technology Limited" },
    { "043A0D", "SM Optics S.r.l." },
    { "043CE8", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "043D6E", "Nokia" },
    { "043D98", "ChongQing QingJia Electronics CO.,LTD" },
    { "043F72", "Mellanox Technologies, Inc." },
    { "0440A9", "New H3C Technologies Co., Ltd" },
    { "044169", "GoPro" },
    { "0441A5", "Apple, Inc." },
    { "04421A", "ASUSTek COMPUTER INC." },
    { "0443FD", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "0444A1", "TELECON GALICIA,S.A." },
    { "044562", "ANDRA Sp. z o. o." },
    { "0445A1", "NIRIT- Xinwei Telecom Technology Co., Ltd." },
    { "044665", "Murata Manufacturing Co., Ltd." },
    { "0446CF", "Beijing Venustech Cybervision Co.,Ltd." },
    { "044707", "Texas Instruments" },
    { "04472A", "Palo Alto Networks" },
    { "0447CA", "GREE ELECTRIC APPLIANCES, INC. OF ZHUHAI" },
    { "04489A", "Apple, Inc." },
    { "04495D", "Huawei Device Co., Ltd." },
    { "044A50", "Ramaxel Technology (Shenzhen) limited company" },
    { "044A69", "Shenzhen Phaten Tech. LTD" },
    { "044A6A", "niliwi nanjing big data Co,.Ltd" },
    { "044A6C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "044AC6", "Aipon Electronics Co., Ltd" },
    { "044BA5", "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "044BB1", "Huawei Device Co., Ltd." },
    { "044BED", "Apple, Inc." },
    { "044BFF", "GuangZhou Hedy Digital Technology Co., Ltd" },
    { "044CEF", "Fujian Sanao Technology Co.,Ltd" },
    { "044E06", "Ericsson AB" },
    { "044E5A", "Commscope" },
    { "044EAF", "LG Innotek" },
    { "044F17", "HUMAX Co., Ltd." },
    { "044F4C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "044F7A", "China Mobile Group Device Co.,Ltd." },
    { "044F8B", "Adapteva, Inc." },
    { "044FAA", "Ruckus Wireless" },
    { "0450DA", "Qiku Internet Network Scientific (Shenzhen) Co., Ltd" },
    { "045170", "Zhongshan K-mate General Electronics Co.,Ltd" },
    { "0452C7", "Bose Corporation" },
    { "0452F3", "Apple, Inc." },
    { "0453D5", "Sysorex Global Holdings" },
    { "045453", "Apple, Inc." },
    { "0455B2", "Huaqin Technology Co.,Ltd" },
    { "0455B8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0455CA", "BriView (Xiamen) Corp." },
    { "045604", "Gionee Communication Equipment Co.,Ltd." },
    { "045665", "Nokia Shanghai Bell Co., Ltd." },
    { "0456E5", "Intel Corporate" },
    { "04572F", "Sertel Electronics UK Ltd" },
    { "045747", "GoPro" },
    { "045791", "Shenzhenshi Xinzhongxin Technology Co.Ltd" },
    { "04585D", "IEEE Registration Authority" },
    { "04586F", "Sichuan Whayer information industry Co.,LTD" },
    { "045A95", "Nokia Corporation" },
    { "045C06", "Zmodo Technology Corporation" },
    { "045C6C", "Juniper Networks" },
    { "045C8E", "gosund GROUP CO.,LTD" },
    { "045D4B", "Sony Corporation" },
    { "045D56", "camtron industrial inc." },
    { "045EA4", "Netis Technology Co., Ltd." },
    { "045FA7", "Shenzhen Yichen Technology Development Co.,LTD" },
    { "045FB9", "Cisco Systems, Inc" },
    { "046169", "MEDIA GLOBAL LINKS CO., LTD." },
    { "046273", "Cisco Systems, Inc" },
    { "0462D7", "ALSTOM HYDRO FRANCE" },
    { "0463D0", "Huawei Device Co., Ltd." },
    { "0463E0", "Nome Oy" },
    { "046565", "Testop" },
    { "046761", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "046785", "scemtec Hard- und Software fuer Mess- und Steuerungstechnik GmbH" },
    { "046865", "Apple, Inc." },
    { "046874", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "04698F", "Juniper Networks" },
    { "0469F8", "Apple, Inc." },
    { "046B1B", "SYSDINE Co., Ltd." },
    { "046B25", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "046C59", "Intel Corporate" },
    { "046C9D", "Cisco Systems, Inc" },
    { "046D42", "Bryston Ltd." },
    { "046E02", "OpenRTLS Group" },
    { "046E49", "TaiYear Electronic Technology (Suzhou) Co., Ltd" },
    { "046ECB", "zte corporation" },
    { "047056", "Arcadyan Corporation" },
    { "0470BC", "Globalstar Inc." },
    { "04714B", "IEEE Registration Authority" },
    { "047153", "SERNET (SUZHOU) TECHNOLOGIES CORPORATION" },
    { "047295", "Apple, Inc." },
    { "04749E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0474A1", "Aligera Equipamentos Digitais Ltda" },
    { "047503", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0475F5", "CSST" },
    { "0475F9", "Taicang T&W Electronics" },
    { "04766E", "ALPSALPINE CO,.LTD" },
    { "0476B0", "Cisco Systems, Inc" },
    { "047863", "Shanghai MXCHIP Information Technology Co., Ltd." },
    { "047970", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "047975", "Honor Device Co., Ltd." },
    { "0479B7", "Texas Instruments" },
    { "0479FD", "Ciena Corporation" },
    { "047A0B", "Beijing Xiaomi Electronics Co., Ltd." },
    { "047AAE", "Huawei Device Co., Ltd." },
    { "047BCB", "Universal Global Scientific Industrial Co., Ltd." },
    { "047C16", "Micro-Star INTL CO., LTD." },
    { "047D50", "Shenzhen Kang Ying Technology Co.Ltd." },
    { "047D7B", "Quanta Computer Inc." },
    { "047E23", "China Mobile IOT Company Limited" },
    { "047E4A", "moobox CO., Ltd." },
    { "047F0E", "Barrot Technology Co.,LTD" },
    { "0480A7", "ShenZhen TianGang Micro Technology CO.LTD" },
    { "04819B", "SKY UK LIMITED" },
    { "0481AE", "Clack Corporation" },
    { "048308", "Espressif Inc." },
    { "04848A", "7INOVA TECHNOLOGY LIMITED" },
    { "048680", "Quectel Wireless Solutions Co.,Ltd." },
    { "048727", "Silicon Laboratories" },
    { "04885F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "04888C", "Eifelwerk Butler Systeme GmbH" },
    { "0488E2", "Beats Electronics LLC" },
    { "048A15", "Avaya Inc" },
    { "048AE1", "FLEXTRONICS MANUFACTURING(ZHUHAI)CO.,LTD." },
    { "048B42", "Skspruce Technologies" },
    { "048C03", "ThinPAD Technology (Shenzhen)CO.,LTD" },
    { "048C16", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "048C9A", "Huawei Device Co., Ltd." },
    { "048D38", "Netis Technology Co., Ltd." },
    { "048F00", "Rong-Paisa Electronics Co., Ltd." },
    { "049081", "Pensando Systems, Inc." },
    { "0490C0", "Forvia" },
    { "049162", "Microchip Technology Inc." },
    { "049226", "ASUSTek COMPUTER INC." },
    { "0492EE", "iway AG" },
    { "04946B", "TECNO MOBILE LIMITED" },
    { "0494A1", "CATCH THE WIND INC" },
    { "0494E9", "FAXedge Technologies, LLC" },
    { "049573", "zte corporation" },
    { "0495E6", "Tenda Technology Co.,Ltd.Dongguan branch" },
    { "049645", "WUXI SKY CHIP INTERCONNECTION TECHNOLOGY CO.,LTD." },
    { "049790", "Lartech telecom LLC" },
    { "04981C", "Ningbo Zhixiang Technology Co., Ltd" },
    { "0498F3", "ALPSALPINE CO,.LTD" },
    { "0499B9", "Apple, Inc." },
    { "0499BB", "Apple, Inc." },
    { "0499E6", "Shenzhen Yoostar Technology Co., Ltd" },
    { "049B9C", "Eadingcore Intelligent Technology Co., Ltd." },
    { "049C62", "BMT Medical Technology s.r.o." },
    { "049D05", "Apple, Inc." },
    { "049DFE", "Hivesystem" },
    { "049F06", "Smobile Co., Ltd." },
    { "049F15", "Humane" },
    { "049F81", "NETSCOUT SYSTEMS INC" },
    { "049FCA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "04A151", "NETGEAR" },
    { "04A16F", "IEEE Registration Authority" },
    { "04A222", "Arcadyan Corporation" },
    { "04A2F3", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "04A316", "Texas Instruments" },
    { "04A3F3", "Emicon" },
    { "04A526", "Nokia" },
    { "04A741", "Cisco Systems, Inc" },
    { "04A81C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "04A82A", "Nokia Corporation" },
    { "04A85A", "SZ DJI TECHNOLOGY CO.,LTD" },
    { "04A924", "Mist Systems, Inc." },
    { "04A959", "New H3C Technologies Co., Ltd" },
    { "04AAE1", "BEIJING MICROVISION TECHNOLOGY CO.,LTD" },
    { "04AB08", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "04AB18", "ELECOM CO.,LTD." },
    { "04AB6A", "Chun-il Co.,Ltd." },
    { "04AC44", "Holtek Semiconductor Inc." },
    { "04AE47", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "04AEC7", "Marquardt" },
    { "04B066", "Private" },
    { "04B0E7", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "04B167", "Xiaomi Communications Co Ltd" },
    { "04B1A1", "Samsung Electronics Co.,Ltd" },
    { "04B3B6", "Seamap (UK) Ltd" },
    { "04B429", "Samsung Electronics Co.,Ltd" },
    { "04B466", "BSP Co., Ltd." },
    { "04B4FE", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "04B5C1", "ITEL MOBILE LIMITED" },
    { "04B648", "ZENNER" },
    { "04B6BE", "CIG SHANGHAI CO LTD" },
    { "04B86A", "SKY UK LIMITED" },
    { "04B97D", "AiVIS Co., Itd." },
    { "04B9E3", "Samsung Electronics Co.,Ltd" },
    { "04BA1C", "Huawei Device Co., Ltd." },
    { "04BA36", "Li Seng Technology Ltd" },
    { "04BA8D", "Samsung Electronics Co.,Ltd" },
    { "04BAD6", "D-Link Corporation" },
    { "04BBF9", "Pavilion Data Systems Inc" },
    { "04BC6D", "Apple, Inc." },
    { "04BC87", "Shenzhen JustLink Technology Co., LTD" },
    { "04BC9F", "Calix Inc." },
    { "04BD70", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "04BD88", "Hewlett Packard Enterprise" },
    { "04BD97", "Cisco Systems, Inc" },
    { "04BDBF", "Samsung Electronics Co.,Ltd" },
    { "04BE58", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "04BF1B", "Dell Inc." },
    { "04BF6D", "Zyxel Communications Corporation" },
    { "04BFA8", "ISB Corporation" },
    { "04BFD5", "Apple, Inc." },
    { "04C05B", "Tigo Energy" },
    { "04C06F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "04C09C", "Tellabs Inc." },
    { "04C103", "Clover Network, Inc." },
    { "04C1B9", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "04C1D8", "Huawei Device Co., Ltd." },
    { "04C23E", "HTC Corporation" },
    { "04C241", "Nokia" },
    { "04C29B", "Aura Home, Inc." },
    { "04C3E6", "IEEE Registration Authority" },
    { "04C461", "Murata Manufacturing Co., Ltd." },
    { "04C5A4", "Cisco Systems, Inc" },
    { "04C807", "Xiaomi Communications Co Ltd" },
    { "04C845", "TP-Link Systems Inc." },
    { "04C880", "Samtec Inc" },
    { "04C98B", "IEEE Registration Authority" },
    { "04C991", "Phistek INC." },
    { "04C9D9", "Dish Technologies Corp" },
    { "04CA8D", "Enfabrica" },
    { "04CAED", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "04CB01", "Samsung Electronics Co.,Ltd" },
    { "04CB1D", "Traka plc" },
    { "04CB88", "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd" },
    { "04CCBC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "04CD15", "Silicon Laboratories" },
    { "04CDC0", "Mist Systems, Inc." },
    { "04CE09", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "04CE14", "Wilocity LTD." },
    { "04CE7E", "NXP France Semiconductors France" },
    { "04CF25", "MANYCOLORS, INC." },
    { "04CF4B", "Intel Corporate" },
    { "04CF8C", "XIAOMI Electronics,CO.,LTD" },
    { "04D13A", "Xiaomi Communications Co Ltd" },
    { "04D168", "Sunplus Technology Co., Ltd." },
    { "04D16E", "IEEE Registration Authority" },
    { "04D190", "Kaon Group Co., Ltd." },
    { "04D320", "ITEL MOBILE LIMITED" },
    { "04D395", "Motorola Mobility LLC, a Lenovo Company" },
    { "04D3B0", "Intel Corporate" },
    { "04D3B5", "Huawei Device Co., Ltd." },
    { "04D3CF", "Apple, Inc." },
    { "04D437", "ZNV" },
    { "04D442", "GUANGDONG GENIUS TECHNOLOGY CO., LTD." },
    { "04D4C4", "ASUSTek COMPUTER INC." },
    { "04D590", "Fortinet, Inc." },
    { "04D60E", "FUNAI ELECTRIC CO., LTD." },
    { "04D6AA", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "04D6F4", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "04D783", "Y&H E&C Co.,LTD." },
    { "04D7A5", "New H3C Technologies Co., Ltd" },
    { "04D921", "Occuspace" },
    { "04D9C8", "Hon Hai Precision Industry Co., Ltd." },
    { "04D9F5", "ASUSTek COMPUTER INC." },
    { "04DA28", "Chongqing Zhouhai Intelligent Technology Co., Ltd" },
    { "04DAD2", "Cisco Systems, Inc" },
    { "04DB56", "Apple, Inc." },
    { "04DB8A", "Suntech International Ltd." },
    { "04DD4C", "Velocytech" },
    { "04DEDB", "Rockport Networks Inc" },
    { "04DEF2", "Shenzhen ECOM Technology Co. Ltd" },
    { "04DF69", "Car Connectivity Consortium" },
    { "04E0B0", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "04E0C4", "TRIUMPH-ADLER AG" },
    { "04E1C8", "IMS Soluções em Energia Ltda." },
    { "04E229", "Qingdao Haier Technology Co.,Ltd" },
    { "04E2F8", "AEP Ticketing solutions srl" },
    { "04E31A", "Sagemcom Broadband SAS" },
    { "04E387", "Cisco Systems, Inc" },
    { "04E3C8", "FUJIAN STAR-NET COMMUNICATION CO.,LTD" },
    { "04E3E5", "Silicon Laboratories" },
    { "04E451", "Texas Instruments" },
    { "04E4B6", "Samsung Electronics Co.,Ltd" },
    { "04E536", "Apple, Inc." },
    { "04E548", "Cohda Wireless Pty Ltd" },
    { "04E56E", "THUB Co., ltd." },
    { "04E598", "Xiaomi Communications Co Ltd" },
    { "04E662", "Acroname Inc." },
    { "04E676", "AMPAK Technology, Inc." },
    { "04E69E", "ZHONGGUANCUN XINHAIZEYOU TECHNOLOGY CO.,LTD" },
    { "04E77E", "We Corporation Inc." },
    { "04E795", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "04E892", "SHENNAN CIRCUITS CO.,LTD" },
    { "04E8B9", "Intel Corporate" },
    { "04E9E5", "PJRC.COM, LLC" },
    { "04EA56", "Intel Corporate" },
    { "04EB40", "Cisco Systems, Inc" },
    { "04ECBB", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "04ECD8", "Intel Corporate" },
    { "04ED33", "Intel Corporate" },
    { "04ED62", "Daikin Europe NV" },
    { "04EE03", "Texas Instruments" },
    { "04EE91", "x-fabric GmbH" },
    { "04EEE8", "IEEE Registration Authority" },
    { "04EEEE", "Laplace System Co., Ltd." },
    { "04F021", "Compex Systems Pte Ltd" },
    { "04F03E", "Huawei Device Co., Ltd." },
    { "04F0E4", "ShenZhen Hosecom Electronic Technology Co.,LTD" },
    { "04F0EE", "Intel Corporate" },
    { "04F128", "HMD Global Oy" },
    { "04F13E", "Apple, Inc." },
    { "04F169", "Huawei Device Co., Ltd." },
    { "04F17D", "Tarana Wireless" },
    { "04F352", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "04F41C", "Routerboard.com" },
    { "04F4BC", "Xena Networks" },
    { "04F4D8", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "04F5F4", "Proxim Wireless" },
    { "04F778", "Sony Interactive Entertainment Inc." },
    { "04F7E4", "Apple, Inc." },
    { "04F8C2", "Flaircomm Microelectronics, Inc." },
    { "04F8F8", "Edgecore Networks Corporation" },
    { "04F938", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "04F993", "Infinix mobility limited" },
    { "04F9D9", "Speaker Electronic(Jiashan) Co.,Ltd" },
    { "04F9F8", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "04FA3F", "OptiCore Inc." },
    { "04FA83", "Qingdao Haier Technology Co.,Ltd" },
    { "04FDE8", "Technoalpin" },
    { "04FE31", "Samsung Electronics Co.,Ltd" },
    { "04FE7F", "Cisco Systems, Inc" },
    { "04FE8D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "04FEA1", "Fihonest communication co.,Ltd" },
    { "04FF08", "Huawei Device Co., Ltd." },
    { "04FF51", "NOVAMEDIA INNOVISION SP. Z O.O." },
    { "080001", "COMPUTERVISION CORPORATION" },
    { "080002", "BRIDGE COMMUNICATIONS INC." },
    { "080003", "ADVANCED COMPUTER COMM." },
    { "080004", "CROMEMCO INCORPORATED" },
    { "080005", "SYMBOLICS INC." },
    { "080006", "SIEMENS AG" },
    { "080007", "Apple, Inc." },
    { "080008", "BOLT BERANEK AND NEWMAN INC." },
    { "080009", "Hewlett Packard" },
    { "08000A", "NESTAR SYSTEMS INCORPORATED" },
    { "08000B", "UNISYS CORPORATION" },
    { "08000C", "MIKLYN DEVELOPMENT CO." },
    { "08000D", "International Computers, Ltd" },
    { "08000E", "NCR CORPORATION" },
    { "08000F", "MITEL CORPORATION" },
    { "080011", "TEKTRONIX INC." },
    { "080012", "BELL ATLANTIC INTEGRATED SYST." },
    { "080013", "Exxon" },
    { "080014", "EXCELAN" },
    { "080015", "STC BUSINESS SYSTEMS" },
    { "080016", "BARRISTER INFO SYS CORP" },
    { "080017", "NATIONAL SEMICONDUCTOR" },
    { "080018", "PIRELLI FOCOM NETWORKS" },
    { "080019", "GENERAL ELECTRIC CORPORATION" },
    { "08001A", "TIARA/ 10NET" },
    { "08001B", "Dell EMC" },
    { "08001C", "KDD-KOKUSAI DEBNSIN DENWA CO." },
    { "08001D", "ABLE COMMUNICATIONS INC." },
    { "08001E", "APOLLO COMPUTER INC." },
    { "08001F", "SHARP CORPORATION" },
    { "080020", "Oracle Corporation" },
    { "080021", "3M COMPANY" },
    { "080022", "NBI INC." },
    { "080023", "Panasonic Communications Co., Ltd." },
    { "080024", "10NET COMMUNICATIONS/DCA" },
    { "080025", "CONTROL DATA" },
    { "080026", "NORSK DATA A.S." },
    { "080027", "PCS Systemtechnik GmbH" },
    { "080028", "Texas Instruments" },
    { "080029", "Megatek Corporation" },
    { "08002A", "MOSAIC TECHNOLOGIES INC." },
    { "08002B", "DIGITAL EQUIPMENT CORPORATION" },
    { "08002C", "BRITTON LEE INC." },
    { "08002D", "LAN-TEC INC." },
    { "08002E", "METAPHOR COMPUTER SYSTEMS" },
    { "08002F", "PRIME COMPUTER INC." },
    { "080030", "CERN" },
    { "080030", "NETWORK RESEARCH CORPORATION" },
    { "080030", "ROYAL MELBOURNE INST OF TECH" },
    { "080031", "LITTLE MACHINES INC." },
    { "080032", "TIGAN INCORPORATED" },
    { "080033", "BAUSCH & LOMB" },
    { "080034", "FILENET CORPORATION" },
    { "080035", "MICROFIVE CORPORATION" },
    { "080036", "INTERGRAPH CORPORATION" },
    { "080037", "FUJIFILM Business Innovation Corp." },
    { "080038", "BULL S.A.S." },
    { "080039", "SPIDER SYSTEMS LIMITED" },
    { "08003A", "ORCATECH INC." },
    { "08003B", "TORUS SYSTEMS LIMITED" },
    { "08003C", "SCHLUMBERGER WELL SERVICES" },
    { "08003D", "CADNETIX CORPORATIONS" },
    { "08003E", "CODEX CORPORATION" },
    { "08003F", "FRED KOSCHARA ENTERPRISES" },
    { "080040", "FERRANTI COMPUTER SYS. LIMITED" },
    { "080041", "RACAL-MILGO INFORMATION SYS.." },
    { "080042", "MACNICA, Inc." },
    { "080043", "PIXEL COMPUTER INC." },
    { "080044", "DAVID SYSTEMS INC." },
    { "080045", "CONCURRENT COMPUTER CORP." },
    { "080046", "Sony Corporation" },
    { "080047", "SEQUENT COMPUTER SYSTEMS INC." },
    { "080048", "EUROTHERM GAUGING SYSTEMS" },
    { "080049", "UNIVATION" },
    { "08004A", "BANYAN SYSTEMS INC." },
    { "08004B", "Planning Research Corp." },
    { "08004C", "HYDRA COMPUTER SYSTEMS INC." },
    { "08004D", "CORVUS SYSTEMS INC." },
    { "08004E", "3COM EUROPE LTD" },
    { "08004F", "CYGNET SYSTEMS" },
    { "080050", "DAISY SYSTEMS CORP." },
    { "080051", "ExperData" },
    { "080052", "INSYSTEC" },
    { "080053", "MIDDLE EAST TECH. UNIVERSITY" },
    { "080055", "STANFORD TELECOMM. INC." },
    { "080056", "STANFORD LINEAR ACCEL. CENTER" },
    { "080057", "Evans & Sutherland" },
    { "080058", "SYSTEMS CONCEPTS" },
    { "080059", "A/S MYCRON" },
    { "08005A", "IBM Corp" },
    { "08005B", "VTA TECHNOLOGIES INC." },
    { "08005C", "FOUR PHASE SYSTEMS" },
    { "08005D", "GOULD INC." },
    { "08005E", "COUNTERPOINT COMPUTER INC." },
    { "08005F", "SABER TECHNOLOGY CORP." },
    { "080060", "INDUSTRIAL NETWORKING INC." },
    { "080061", "JAROGATE LTD." },
    { "080062", "General Dynamics" },
    { "080063", "PLESSEY" },
    { "080064", "Sitasys AG" },
    { "080065", "GENRAD INC." },
    { "080066", "AGFA CORPORATION" },
    { "080067", "ComDesign" },
    { "080068", "RIDGE COMPUTERS" },
    { "080069", "Silicon Graphics" },
    { "08006A", "AT&T" },
    { "08006B", "ACCEL TECHNOLOGIES INC." },
    { "08006C", "SUNTEK TECHNOLOGY INT'L" },
    { "08006D", "WHITECHAPEL COMPUTER WORKS" },
    { "08006E", "MASSCOMP" },
    { "08006F", "PHILIPS APELDOORN B.V." },
    { "080070", "Mitsubishi Precision Co.,LTd." },
    { "080071", "MATRA (DSIE)" },
    { "080072", "XEROX CORP UNIV GRANT PROGRAM" },
    { "080073", "TECMAR INC." },
    { "080074", "CASIO COMPUTER CO. LTD." },
    { "080075", "DANSK DATA ELECTRONIK" },
    { "080076", "PC LAN TECHNOLOGIES" },
    { "080077", "TSL COMMUNICATIONS LTD." },
    { "080078", "ACCELL CORPORATION" },
    { "080079", "THE DROID WORKS" },
    { "08007A", "INDATA" },
    { "08007B", "SANYO ELECTRIC CO. LTD." },
    { "08007C", "VITALINK COMMUNICATIONS CORP." },
    { "08007E", "AMALGAMATED WIRELESS(AUS) LTD" },
    { "08007F", "CARNEGIE-MELLON UNIVERSITY" },
    { "080080", "AES DATA INC." },
    { "080081", "ASTECH INC." },
    { "080082", "VERITAS SOFTWARE" },
    { "080083", "Seiko Instruments Inc." },
    { "080084", "TOMEN ELECTRONICS CORP." },
    { "080085", "ELXSI" },
    { "080086", "KONICA MINOLTA HOLDINGS, INC." },
    { "080087", "Xyplex, Inc." },
    { "080088", "Brocade Communications Systems LLC" },
    { "080089", "Kinetics" },
    { "08008A", "PerfTech, Inc." },
    { "08008B", "PYRAMID TECHNOLOGY CORP." },
    { "08008C", "NETWORK RESEARCH CORPORATION" },
    { "08008D", "XYVISION INC." },
    { "08008E", "Tandem Computers" },
    { "08008F", "CHIPCOM CORPORATION" },
    { "080090", "SONOMA SYSTEMS" },
    { "08010F", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "080205", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "08023C", "Samsung Electronics Co.,Ltd" },
    { "08028E", "NETGEAR" },
    { "080342", "Palo Alto Networks" },
    { "080371", "KRG CORPORATE" },
    { "0804B4", "Texas Instruments" },
    { "080581", "Roku, Inc." },
    { "0805CD", "DongGuang EnMai Electronic Product Co.Ltd." },
    { "0805E2", "Juniper Networks" },
    { "08085C", "Luna Products" },
    { "0808C2", "Samsung Electronics Co.,Ltd" },
    { "0808EA", "AMSC" },
    { "0809B6", "Masimo Corp" },
    { "0809C7", "Zhuhai Unitech Power Technology Co., Ltd." },
    { "080A4E", "Planet Bingo® — 3rd Rock Gaming®" },
    { "080C0B", "SysMik GmbH Dresden" },
    { "080CC9", "Mission Technology Group, dba Magma" },
    { "080D84", "GECO, Inc." },
    { "080EA8", "Velex s.r.l." },
    { "080FE5", "Cisco Systems, Inc" },
    { "080FFA", "KSP INC." },
    { "081031", "Lithiunal Energy" },
    { "081086", "NEC Platforms, Ltd." },
    { "081093", "Samsung Electronics Co.,Ltd" },
    { "08115E", "Bitel Co., Ltd." },
    { "081196", "Intel Corporate" },
    { "081287", "Jiangxi Risound Electronics Co.,LTD" },
    { "0812A5", "Amazon Technologies Inc." },
    { "081443", "UNIBRAIN S.A." },
    { "08152F", "Samsung Electronics Co., Ltd. ARTIK" },
    { "0815AE", "China Mobile Group Device Co.,Ltd." },
    { "081605", "Vodafone Italia S.p.A." },
    { "081651", "SHENZHEN SEA STAR TECHNOLOGY CO.,LTD" },
    { "0816D5", "GOERTEK INC." },
    { "081735", "Cisco Systems, Inc" },
    { "0817F4", "IBM Corp" },
    { "08181A", "zte corporation" },
    { "08184C", "A. S. Thomas, Inc." },
    { "0819A6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "081A1E", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "081AFD", "Huawei Device Co., Ltd." },
    { "081C6E", "Xiaomi Communications Co Ltd" },
    { "081DC4", "Thermo Fisher Scientific Messtechnik GmbH" },
    { "081DFB", "Shanghai Mexon Communication Technology Co.,Ltd" },
    { "081F3F", "WondaLink Inc." },
    { "081F71", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "081FEB", "BinCube" },
    { "081FF3", "Cisco Systems, Inc" },
    { "0821EF", "Samsung Electronics Co.,Ltd" },
    { "0823B2", "vivo Mobile Communication Co., Ltd." },
    { "0823C6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "082522", "ADVANSEE" },
    { "082525", "Xiaomi Communications Co Ltd" },
    { "082573", "Apple, Inc." },
    { "082697", "Zyxel Communications Corporation" },
    { "0826AE", "IEEE Registration Authority" },
    { "082719", "APS systems/electronic AG" },
    { "08276B", "Huawei Device Co., Ltd." },
    { "0827A8", "Arcadyan Corporation" },
    { "0827CE", "NAGANO KEIKI CO., LTD." },
    { "0827F0", "Accton Technology Co., Ltd." },
    { "082802", "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD" },
    { "082AD0", "SRD Innovations Inc." },
    { "082CB0", "Network Instruments" },
    { "082CB6", "Apple, Inc." },
    { "082CED", "Technity Solutions Inc." },
    { "082E36", "Huawei Device Co., Ltd." },
    { "082E5F", "Hewlett Packard" },
    { "082FE9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "08306B", "Palo Alto Networks" },
    { "0830CE", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "08318B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0831A4", "Huawei Device Co., Ltd." },
    { "0833ED", "ASKEY COMPUTER CORP" },
    { "08351B", "Shenzhen Jialihua Electronic Technology Co., Ltd" },
    { "083571", "CASwell INC." },
    { "0835B2", "CoreEdge Networks Co., Ltd" },
    { "0836C9", "NETGEAR" },
    { "08373D", "Samsung Electronics Co.,Ltd" },
    { "08379C", "Topaz Co. LTD." },
    { "083869", "Hong Kong AMobile Intelligent Corp. Limited Taiwan Branch" },
    { "0838A5", "Funkwerk plettac electronic GmbH" },
    { "0838E6", "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd." },
    { "083A2F", "Guangzhou Juan Intelligent Tech Joint Stock Co.,Ltd" },
    { "083A38", "New H3C Technologies Co., Ltd" },
    { "083A5C", "Junilab, Inc." },
    { "083A88", "Universal Global Scientific Industrial Co., Ltd." },
    { "083A8D", "Espressif Inc." },
    { "083AB8", "Shinoda Plasma Co., Ltd." },
    { "083AF2", "Espressif Inc." },
    { "083BE9", "New H3C Technologies Co., Ltd" },
    { "083D88", "Samsung Electronics Co.,Ltd" },
    { "083E0C", "Commscope" },
    { "083E5D", "Sagemcom Broadband SAS" },
    { "083E8E", "Hon Hai Precision Ind. Co.,Ltd." },
    { "083F21", "Motorola Mobility LLC, a Lenovo Company" },
    { "083F3E", "WSH GmbH" },
    { "083F76", "Intellian Technologies, Inc." },
    { "083FBC", "zte corporation" },
    { "084027", "Gridstore Inc." },
    { "0840F3", "Tenda Technology Co.,Ltd.Dongguan branch" },
    { "084218", "Asyril SA" },
    { "084296", "Mobile Technology Solutions LLC" },
    { "084473", "zte corporation" },
    { "0845D1", "Cisco Systems, Inc" },
    { "084656", "VEO-LABS" },
    { "08468B", "Guangdong NanGuang Photo & Video Systems Co., Ltd" },
    { "0846C7", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "08474C", "Nokia" },
    { "0847D0", "Nokia Shanghai Bell Co., Ltd." },
    { "08482C", "Raycore Taiwan Co., LTD." },
    { "084857", "Suteng Innovation Technology Co., Ltd." },
    { "084ACF", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "084B44", "Robert Bosch Elektronika Kft. " },
    { "084E1C", "H2A Systems, LLC" },
    { "084EBF", "Sumitomo Electric Industries, Ltd" },
    { "084F0A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "084F66", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "084FA9", "Cisco Systems, Inc" },
    { "084FF9", "Cisco Systems, Inc" },
    { "085104", "Huawei Device Co., Ltd." },
    { "085114", "QINGDAO TOPSCOMM COMMUNICATION CO., LTD" },
    { "08512E", "Orion Diagnostica Oy" },
    { "085240", "EbV Elektronikbau- und Vertriebs GmbH" },
    { "08524E", "Shenzhen Fangcheng Baiyi Technology Co., Ltd." },
    { "085411", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "0854BB", "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD" },
    { "085531", "Routerboard.com" },
    { "08569B", "WiZ" },
    { "085700", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "0857FB", "Amazon Technologies Inc." },
    { "0858A5", "Beijing Vrv Software Corpoaration Limited." },
    { "085A11", "D-Link International" },
    { "085AE0", "Recovision Technology Co., Ltd." },
    { "085B0E", "Fortinet, Inc." },
    { "085BD6", "Intel Corporate" },
    { "085BDA", "CliniCare LTD" },
    { "085C1B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "085D53", "Apple, Inc." },
    { "085DDD", "MERCURY CORPORATION" },
    { "08606E", "ASUSTek COMPUTER INC." },
    { "086083", "zte corporation" },
    { "086195", "Rockwell Automation" },
    { "086202", "Apple, Inc." },
    { "086266", "ASUSTek COMPUTER INC." },
    { "086332", "IEEE Registration Authority" },
    { "086361", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "086518", "Apple, Inc." },
    { "0865F0", "JM Zengge Co., Ltd" },
    { "08661F", "Palo Alto Networks" },
    { "086698", "Apple, Inc." },
    { "08674E", "Hisense broadband multimedia technology Co.,Ltd" },
    { "08688D", "New H3C Technologies Co., Ltd" },
    { "0868D0", "Japan System Design" },
    { "0868EA", "EITO ELECTRONICS CO., LTD." },
    { "086A0A", "ASKEY COMPUTER CORP" },
    { "086AC5", "Intel Corporate" },
    { "086AE5", "Amazon Technologies Inc." },
    { "086BD1", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "086BD7", "Silicon Laboratories" },
    { "086D41", "Apple, Inc." },
    { "086DF2", "Shenzhen MIMOWAVE Technology Co.,Ltd" },
    { "086E9C", "Huawei Device Co., Ltd." },
    { "086F48", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "087045", "Apple, Inc." },
    { "087073", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "087158", "HANSHOW TECHNOLOGY CO.,LTD." },
    { "087190", "Intel Corporate" },
    { "08736F", "EM Microelectronic" },
    { "087402", "Apple, Inc." },
    { "087458", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "0874F6", "Winterhalter Gastronom GmbH" },
    { "087572", "Obelux Oy" },
    { "087618", "ViE Technologies Sdn. Bhd." },
    { "087671", "Juniper Networks" },
    { "087695", "Auto Industrial Co., Ltd." },
    { "0876FF", "Thomson Telecom Belgium" },
    { "087808", "Samsung Electronics Co.,Ltd" },
    { "08798C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "087999", "AIM GmbH" },
    { "087A4C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "087B12", "Sagemcom Broadband SAS" },
    { "087B87", "Cisco Systems, Inc" },
    { "087BAA", "SVYAZKOMPLEKTSERVICE, LLC" },
    { "087C39", "Amazon Technologies Inc." },
    { "087CBE", "Quintic Corp." },
    { "087D21", "Altasec technology corporation" },
    { "087E64", "Vantiva USA LLC" },
    { "087F98", "vivo Mobile Communication Co., Ltd." },
    { "088039", "Cisco SPVTG" },
    { "0881B2", "Logitech (China) Technology Co., Ltd" },
    { "0881BC", "HongKong Ipro Technology Co., Limited" },
    { "0881F4", "Juniper Networks" },
    { "088466", "Novartis Pharma AG" },
    { "08849D", "Amazon Technologies Inc." },
    { "0884FB", "Honor Device Co., Ltd." },
    { "08855B", "Kontron Europe GmbH" },
    { "088620", "TECNO MOBILE LIMITED" },
    { "08863B", "Belkin International Inc." },
    { "0887C6", "INGRAM MICRO SERVICES" },
    { "0887C7", "Apple, Inc." },
    { "088AF1", "MERCUSYS TECHNOLOGIES CO., LTD." },
    { "088BC8", "Google, Inc." },
    { "088C2C", "Samsung Electronics Co.,Ltd" },
    { "088DC8", "Ryowa Electronics Co.,Ltd" },
    { "088E4F", "SF Software Solutions" },
    { "088E90", "Intel Corporate" },
    { "088EDC", "Apple, Inc." },
    { "088F2C", "Amber Technology Ltd." },
    { "088FC3", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "0890BA", "Danlaw Inc" },
    { "089115", "Amazon Technologies Inc." },
    { "0891A3", "Amazon Technologies Inc." },
    { "089204", "Dell Inc." },
    { "089272", "Espressif Inc." },
    { "089356", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0894EF", "Wistron Infocomm (Zhongshan) Corporation" },
    { "08952A", "Vantiva USA LLC" },
    { "089542", "Apple, Inc." },
    { "0896AD", "Cisco Systems, Inc" },
    { "0896D7", "AVM GmbH" },
    { "089707", "Cisco Systems, Inc" },
    { "089734", "Hewlett Packard Enterprise " },
    { "089758", "Shenzhen Strong Rising Electronics Co.,Ltd DongGuan Subsidiary" },
    { "089798", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "0899E8", "KEMAS GmbH" },
    { "089AC7", "zte corporation" },
    { "089B4B", "iKuai Networks" },
    { "089BB9", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "089BF1", "eero inc." },
    { "089C86", "Nokia Shanghai Bell Co., Ltd." },
    { "089DF4", "Intel Corporate" },
    { "089E01", "Quanta Computer Inc." },
    { "089E08", "Google, Inc." },
    { "089E84", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "089F97", "LEROY AUTOMATION" },
    { "08A12B", "ShenZhen EZL Technology Co., Ltd" },
    { "08A136", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "08A189", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "08A5C8", "Sunnovo International Limited" },
    { "08A5DF", "Samsung Electronics Co.,Ltd" },
    { "08A6BC", "Amazon Technologies Inc." },
    { "08A6F7", "Espressif Inc." },
    { "08A7C0", "Vantiva USA LLC" },
    { "08A842", "Huawei Device Co., Ltd." },
    { "08A8A1", "Cyclotronics Power Concepts, Inc" },
    { "08A95A", "AzureWave Technology Inc." },
    { "08AA55", "Motorola Mobility LLC, a Lenovo Company" },
    { "08AA89", "zte corporation" },
    { "08ACA5", "Benu Video, Inc." },
    { "08ACC4", "FMTech" },
    { "08AED6", "Samsung Electronics Co.,Ltd" },
    { "08AF78", "Totus Solutions, Inc." },
    { "08B055", "ASKEY COMPUTER CORP" },
    { "08B0A7", "Truebeyond Co., Ltd" },
    { "08B258", "Juniper Networks" },
    { "08B2A3", "Cynny Italia S.r.L." },
    { "08B3AF", "vivo Mobile Communication Co., Ltd." },
    { "08B3D6", "Huawei Device Co., Ltd." },
    { "08B49D", "TECNO MOBILE LIMITED" },
    { "08B4B1", "Google, Inc." },
    { "08B4CF", "Abicom International" },
    { "08B4D2", "Intel Corporate" },
    { "08B61F", "Espressif Inc." },
    { "08B657", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "08B738", "Lite-On Technogy Corp." },
    { "08B7EC", "Wireless Seismic" },
    { "08B8D0", "Chipsea Technologies (Shenzhen) Corp." },
    { "08B95F", "Silicon Laboratories" },
    { "08BA22", "Swaive Corporation" },
    { "08BA5F", "Qingdao Hisense Electronics Co.,Ltd." },
    { "08BAB7", "Ceragon Networks Ltd." },
    { "08BB3C", "Flextronics Tech.(Ind) Pvt Ltd" },
    { "08BBCC", "AK-NORD EDV VERTRIEBSGES. mbH" },
    { "08BC20", "Hangzhou Royal Cloud Technology Co., Ltd" },
    { "08BD43", "NETGEAR" },
    { "08BE09", "Astrol Electronic AG" },
    { "08BE77", "Green Electronics" },
    { "08BEAC", "Edimax Technology Co. Ltd." },
    { "08BFA0", "Samsung Electronics Co.,Ltd" },
    { "08BFB8", "ASUSTek COMPUTER INC." },
    { "08C021", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "08C06C", "Huawei Device Co., Ltd." },
    { "08C0EB", "Mellanox Technologies, Inc." },
    { "08C224", "Amazon Technologies Inc." },
    { "08C3B3", "TCL King Electrical Appliances(Huizhou)Co.,Ltd" },
    { "08C5E1", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "08C6B3", "QTECH LLC" },
    { "08C729", "Apple, Inc." },
    { "08C7B5", "Apple, Inc." },
    { "08C7F5", "Vantiva Connected Home - Technologies Telco" },
    { "08C8C2", "GN Audio A/S" },
    { "08CA45", "Toyou Feiji Electronics Co., Ltd." },
    { "08CBE5", "R3 Solutions GmbH" },
    { "08CC27", "Motorola Mobility LLC, a Lenovo Company" },
    { "08CC68", "Cisco Systems, Inc" },
    { "08CC81", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "08CCA7", "Cisco Systems, Inc" },
    { "08CD9B", "samtec automotive electronics & software GmbH" },
    { "08CE94", "EM Microelectronic" },
    { "08D09F", "Cisco Systems, Inc" },
    { "08D0B7", "Qingdao Hisense Communications Co.,Ltd." },
    { "08D1F9", "Espressif Inc." },
    { "08D23E", "Intel Corporate" },
    { "08D29A", "Proformatique" },
    { "08D34B", "Techman Electronics (Changshu) Co., Ltd." },
    { "08D40C", "Intel Corporate" },
    { "08D42B", "Samsung Electronics Co.,Ltd" },
    { "08D46A", "LG Electronics (Mobile Communications)" },
    { "08D593", "Texas Instruments" },
    { "08D59D", "Sagemcom Broadband SAS" },
    { "08D5C0", "Seers Technology Co., Ltd" },
    { "08D833", "Shenzhen RF Technology Co., Ltd" },
    { "08D945", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "08DA33", "IEEE Registration Authority" },
    { "08DD03", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "08DDEB", "Silicon Laboratories" },
    { "08DF1F", "Bose Corporation" },
    { "08DFCB", "Systrome Networks" },
    { "08E021", "Honor Device Co., Ltd." },
    { "08E342", "Cear, Inc." },
    { "08E4DF", "Shenzhen Sande Dacom Electronics Co., Ltd" },
    { "08E5DA", "NANJING FUJITSU COMPUTER PRODUCTS CO.,LTD. " },
    { "08E63B", "zte corporation" },
    { "08E672", "JEBSEE ELECTRONICS CO.,LTD." },
    { "08E689", "Apple, Inc." },
    { "08E6C9", "Business-intelligence of Oriental Nations Corporation Ltd." },
    { "08E7E5", "Huawei Device Co., Ltd." },
    { "08E84F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "08E9F6", "AMPAK Technology,Inc." },
    { "08EA40", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "08EA44", "Extreme Networks Headquarters" },
    { "08EB21", "Intel Corporate" },
    { "08EB29", "Jiangsu Huitong Group Co.,Ltd." },
    { "08EB74", "HUMAX Co., Ltd." },
    { "08EBED", "World Elite Technology Co.,LTD" },
    { "08EBF6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "08ECA9", "Samsung Electronics Co.,Ltd" },
    { "08ECF5", "Cisco Systems, Inc" },
    { "08ED02", "IEEE Registration Authority" },
    { "08ED9D", "TECNO MOBILE LIMITED" },
    { "08EDB9", "Hon Hai Precision Ind. Co.,Ltd." },
    { "08EDED", "Zhejiang Dahua Technology Co., Ltd." },
    { "08EE8B", "Samsung Electronics Co.,Ltd" },
    { "08EF3B", "MCS Logic Inc." },
    { "08EFAB", "SAYME WIRELESS SENSOR NETWORK" },
    { "08F01E", "eero inc." },
    { "08F0B6", "Edifier International" },
    { "08F1B3", "Cisco Meraki" },
    { "08F1B7", "Towerstream Corpration" },
    { "08F1EA", "Hewlett Packard Enterprise " },
    { "08F2F4", "Net One Partners Co.,Ltd." },
    { "08F3FB", "Cisco Systems, Inc" },
    { "08F458", "Huawei Device Co., Ltd." },
    { "08F4AB", "Apple, Inc." },
    { "08F4F0", "Cisco Systems, Inc" },
    { "08F606", "zte corporation" },
    { "08F69C", "Apple, Inc." },
    { "08F6F8", "GET Engineering" },
    { "08F728", "GLOBO Multimedia Sp. z o.o. Sp.k." },
    { "08F7E9", "HRCP Research and Development Partnership" },
    { "08F80D", "IEEE Registration Authority" },
    { "08F8BC", "Apple, Inc." },
    { "08F9E0", "Espressif Inc." },
    { "08FA28", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "08FA79", "vivo Mobile Communication Co., Ltd." },
    { "08FAE0", "Fohhn Audio AG" },
    { "08FBEA", "AMPAK Technology,Inc." },
    { "08FC52", "OpenXS BV" },
    { "08FC88", "Samsung Electronics Co.,Ltd" },
    { "08FD0E", "Samsung Electronics Co.,Ltd" },
    { "08FD52", "Silicon Laboratories" },
    { "08FF24", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "08FF44", "Apple, Inc." },
    { "0C014B", "zte corporation" },
    { "0C01A5", "zte corporation" },
    { "0C01C8", "DENSO Co.,Ltd" },
    { "0C01DB", "Infinix mobility limited" },
    { "0C0227", "Vantiva USA LLC" },
    { "0C02BD", "Samsung Electronics Co.,Ltd" },
    { "0C0400", "Jantar d.o.o." },
    { "0C0535", "Juniper Systems" },
    { "0C07DF", "Xiaomi Communications Co Ltd" },
    { "0C07F3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0C08B4", "HUMAX Co., Ltd." },
    { "0C0ADF", "Texas Instruments" },
    { "0C0E76", "D-Link International" },
    { "0C1105", "AKUVOX (XIAMEN) NETWORKS CO., LTD" },
    { "0C1167", "Cisco Systems, Inc" },
    { "0C1262", "zte corporation" },
    { "0C130B", "Uniqoteq Ltd." },
    { "0C1420", "Samsung Electronics Co.,Ltd" },
    { "0C14D2", "China Mobile Group Device Co.,Ltd." },
    { "0C1539", "Apple, Inc." },
    { "0C1563", "Apple, Inc." },
    { "0C15C5", "SDTEC Co., Ltd." },
    { "0C1773", "Huawei Device Co., Ltd." },
    { "0C17F1", "TELECSYS" },
    { "0C184E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0C191F", "Inform Electronik" },
    { "0C19F8", "Apple, Inc." },
    { "0C1A10", "Acoustic Stream" },
    { "0C1A61", "Neox FZCO" },
    { "0C1BCC", "IFLYTEK CO.,LTD." },
    { "0C1C19", "LONGCONN ELECTRONICS(SHENZHEN) CO.,LTD" },
    { "0C1C1A", "eero inc." },
    { "0C1C20", "Kakao Corp" },
    { "0C1C31", "MERCUSYS TECHNOLOGIES CO., LTD." },
    { "0C1C57", "Texas Instruments" },
    { "0C1DAF", "Xiaomi Communications Co Ltd" },
    { "0C1DC2", "SeAH Networks" },
    { "0C1EF7", "Omni-ID" },
    { "0C2026", "noax Technologies AG" },
    { "0C20D3", "vivo Mobile Communication Co., Ltd." },
    { "0C2138", "Hengstler GmbH" },
    { "0C2369", "Honeywell SPS" },
    { "0C238D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0C2576", "LONGCHEER TELECOMMUNICATION LIMITED" },
    { "0C2724", "Cisco Systems, Inc" },
    { "0C2755", "Valuable Techologies Limited" },
    { "0C2756", "RONGCHEENG GOER TECHNOLOGY CO.,LTD." },
    { "0C2779", "New H3C Technologies Co., Ltd" },
    { "0C298F", "Tesla,Inc." },
    { "0C29EF", "Dell Inc." },
    { "0C2A69", "electric imp, incorporated" },
    { "0C2A6F", "Silicon Laboratories" },
    { "0C2A86", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "0C2AE7", "Beijing General Research Institute of Mining and Metallurgy" },
    { "0C2C54", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0C2C7C", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "0C2D89", "QiiQ Communications Inc." },
    { "0C2E57", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0C2FB0", "Samsung Electronics Co.,Ltd" },
    { "0C3021", "Apple, Inc." },
    { "0C31DC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0C323A", "Samsung Electronics Co.,Ltd" },
    { "0C331B", "TydenBrooks" },
    { "0C3526", "Microsoft Corporation" },
    { "0C354F", "Nokia" },
    { "0C35FE", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "0C3747", "zte corporation" },
    { "0C3796", "BIZLINK TECHNOLOGY, INC." },
    { "0C37DC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0C383E", "Fanvil Technology Co., Ltd." },
    { "0C3956", "Observator instruments" },
    { "0C3AFA", "New H3C Technologies Co., Ltd" },
    { "0C3B50", "Apple, Inc." },
    { "0C3C65", "Dome Imaging Inc" },
    { "0C3CCD", "Universal Global Scientific Industrial Co., Ltd." },
    { "0C3E9F", "Apple, Inc." },
    { "0C4101", "Ruichi Auto Technology (Guangzhou) Co., Ltd." },
    { "0C413E", "Microsoft Corporation" },
    { "0C41E9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0C42A1", "Mellanox Technologies, Inc." },
    { "0C4314", "Silicon Laboratories" },
    { "0C43F9", "Amazon Technologies Inc." },
    { "0C44C0", "zte corporation" },
    { "0C45BA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0C469D", "MS Sedco" },
    { "0C473D", "Hitron Technologies. Inc" },
    { "0C47A9", "IEEE Registration Authority" },
    { "0C47C9", "Amazon Technologies Inc." },
    { "0C4885", "LG Electronics (Mobile Communications)" },
    { "0C48C6", "CELESTICA INC." },
    { "0C4933", "Sichuan Jiuzhou Electronic Technology Co., Ltd." },
    { "0C4B48", "Nokia" },
    { "0C4B54", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "0C4BEE", "Texas Instruments" },
    { "0C4C39", "MitraStar Technology Corp." },
    { "0C4DE9", "Apple, Inc." },
    { "0C4EA0", "Espressif Inc." },
    { "0C4EC0", "Maxlinear Inc" },
    { "0C4F5A", "ASA-RT s.r.l." },
    { "0C4F9B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0C5101", "Apple, Inc." },
    { "0C517E", "Apple, Inc." },
    { "0C51F7", "CHAUVIN ARNOUX" },
    { "0C5203", "AGM GROUP LIMITED" },
    { "0C5331", "ETH Zurich" },
    { "0C53B7", "Apple, Inc." },
    { "0C5415", "Intel Corporate" },
    { "0C54A5", "PEGATRON CORPORATION" },
    { "0C54B9", "Nokia" },
    { "0C5521", "Axiros GmbH" },
    { "0C565C", "HyBroad Vision (Hong Kong) Technology Co Ltd" },
    { "0C57EB", "Mueller Systems" },
    { "0C5842", "DME Micro" },
    { "0C587B", "Quectel Wireless Solutions Co.,Ltd." },
    { "0C599C", "Juniper Networks" },
    { "0C5A19", "Axtion Sdn Bhd" },
    { "0C5A9E", "Wi-SUN Alliance" },
    { "0C5CB5", "IEEE Registration Authority" },
    { "0C5CD8", "DOLI Elektronik GmbH" },
    { "0C5F35", "Niagara Video Corporation" },
    { "0C6046", "vivo Mobile Communication Co., Ltd." },
    { "0C6076", "Hon Hai Precision Ind. Co.,Ltd." },
    { "0C6111", "Anda Technologies SAC" },
    { "0C6127", "Actiontec Electronics, Inc" },
    { "0C61CF", "Texas Instruments" },
    { "0C62A6", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "0C63FC", "Nanjing Signway Technology Co., Ltd" },
    { "0C6422", "Beijing Wiseasy Technology Co.,Ltd." },
    { "0C659A", "Panasonic Automotive Systems Company of America" },
    { "0C6714", "SERNET (SUZHOU) TECHNOLOGIES CORPORATION" },
    { "0C6743", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0C6803", "Cisco Systems, Inc" },
    { "0C6825", "Suzhou HYC technology Co., Ltd." },
    { "0C6ABC", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "0C6AC4", "Apple, Inc." },
    { "0C6AE6", "Stanley Security Solutions" },
    { "0C6E4F", "PrimeVOLT Co., Ltd." },
    { "0C6F9C", "Shaw Communications Inc." },
    { "0C7043", "Sony Interactive Entertainment Inc." },
    { "0C704A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0C715D", "Samsung Electronics Co.,Ltd" },
    { "0C718C", "TCT mobile ltd" },
    { "0C722C", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "0C7274", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "0C72D9", "zte corporation" },
    { "0C7329", "Sercomm Corporation." },
    { "0C73BE", "Dongguan Haimai Electronie Technology Co.,Ltd" },
    { "0C73EB", "IEEE Registration Authority" },
    { "0C74C2", "Apple, Inc." },
    { "0C7512", "Shenzhen Kunlun TongTai Technology Co.,Ltd." },
    { "0C7523", "BEIJING GEHUA CATV NETWORK CO.,LTD" },
    { "0C756C", "Anaren Microwave, Inc." },
    { "0C75BD", "Cisco Systems, Inc" },
    { "0C75D2", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "0C771A", "Apple, Inc." },
    { "0C7955", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "0C7A15", "Intel Corporate" },
    { "0C7BC8", "Cisco Meraki" },
    { "0C7C28", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "0C7D7C", "Kexiang Information Technology Co, Ltd." },
    { "0C7DB0", "Motorola Mobility LLC, a Lenovo Company" },
    { "0C7E24", "Garmin International" },
    { "0C7FB2", "Commscope" },
    { "0C7FED", "IEEE Registration Authority" },
    { "0C802F", "Murata Manufacturing Co., Ltd." },
    { "0C8063", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "0C8112", "Private" },
    { "0C8126", "Juniper Networks" },
    { "0C817D", "EEP Elektro-Elektronik Pranjic GmbH" },
    { "0C8230", "SHENZHEN MAGNUS TECHNOLOGIES CO.,LTD" },
    { "0C8247", "CIG SHANGHAI CO LTD" },
    { "0C8268", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "0C826A", "Wuhan Huagong Genuine Optics Technology Co., Ltd" },
    { "0C82D5", "Maxio Technology Hangzhou Co., Ltd." },
    { "0C8306", "Huawei Device Co., Ltd." },
    { "0C839A", "Huawei Device Co., Ltd." },
    { "0C83CC", "Alpha Networks Inc." },
    { "0C8408", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0C8411", "A.O. Smith Water Products" },
    { "0C8447", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "0C8484", "Zenovia Electronics Inc." },
    { "0C84DC", "Hon Hai Precision Ind. Co.,Ltd." },
    { "0C8525", "Cisco Systems, Inc" },
    { "0C85E1", "Apple, Inc." },
    { "0C8610", "Juniper Networks" },
    { "0C8629", "IEEE Registration Authority" },
    { "0C86C7", "Jabil Circuit (Guangzhou) Limited" },
    { "0C8772", "FUJIAN STAR-NET COMMUNICATION CO.,LTD" },
    { "0C8910", "Samsung Electronics Co.,Ltd" },
    { "0C8A87", "AgLogica Holdings, Inc" },
    { "0C8B7D", "Vizio, Inc" },
    { "0C8B95", "Espressif Inc." },
    { "0C8BA2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0C8BD3", "ITEL MOBILE LIMITED" },
    { "0C8BFD", "Intel Corporate" },
    { "0C8C24", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "0C8C69", "Shenzhen elink smart Co., ltd" },
    { "0C8C8F", "Kamo Technology Limited" },
    { "0C8CDC", "Suunto Oy" },
    { "0C8D7A", "RADiflow" },
    { "0C8D98", "TOP EIGHT IND CORP" },
    { "0C8DCA", "Samsung Electronics Co.,Ltd" },
    { "0C8DDB", "Cisco Meraki" },
    { "0C8E29", "Arcadyan Corporation" },
    { "0C8FFF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0C9043", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "0C9160", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "0C9192", "Intel Corporate" },
    { "0C924E", "Rice Lake Weighing Systems" },
    { "0C9301", "PT. Prasimax Inovasi Teknologi" },
    { "0C938F", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "0C93A5", "eero inc." },
    { "0C93FB", "BNS Solutions" },
    { "0C9505", "The Chamberlain Group, Inc" },
    { "0C9515", "Palltronics, Inc." },
    { "0C9541", "CHIPSEA TECHNOLOGIES (SHENZHEN) CORP." },
    { "0C96BF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0C96CD", "MERCURY CORPORATION" },
    { "0C96E6", "Cloud Network Technology (Samoa) Limited" },
    { "0C975F", "Hewlett Packard Enterprise" },
    { "0C979B", "FUJIAN STAR-NET COMMUNICATION CO.,LTD" },
    { "0C9838", "Xiaomi Communications Co Ltd" },
    { "0C9A3C", "Intel Corporate" },
    { "0C9A42", "FN-LINK TECHNOLOGY LIMITED" },
    { "0C9AE6", "SZ DJI TECHNOLOGY CO.,LTD" },
    { "0C9B13", "Shanghai Magic Mobile Telecommunication Co.Ltd." },
    { "0C9B78", "Extreme Networks Headquarters" },
    { "0C9D56", "Consort Controls Ltd" },
    { "0C9D92", "ASUSTek COMPUTER INC." },
    { "0C9E91", "Sankosha Corporation" },
    { "0C9F71", "Dolphin Electronics (DongGuan) Co., Ltd." },
    { "0CA06C", "Industrial Cyber Sensing Inc." },
    { "0CA138", "BLiNQ Networks Inc." },
    { "0CA2F4", "Chameleon Technology (UK) Limited" },
    { "0CA402", "Alcatel-Lucent IPD" },
    { "0CA42A", "OB Telecom Electronic Technology Co., Ltd" },
    { "0CA64C", "Hangzhou Ezviz Software Co.,Ltd." },
    { "0CA694", "Sunitec Enterprise Co.,Ltd" },
    { "0CA8A7", "Samsung Electronics Co.,Ltd" },
    { "0CA94A", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "0CAAEE", "Ansjer Electronics Co., Ltd." },
    { "0CAC05", "Unitend Technologies Inc." },
    { "0CAC8A", "Sagemcom Broadband SAS" },
    { "0CAE5F", "Silicon Laboratories" },
    { "0CAE7D", "Texas Instruments" },
    { "0CAEBD", "Edifier International" },
    { "0CAF31", "Cisco Systems, Inc" },
    { "0CAF5A", "GENUS POWER INFRASTRUCTURES LIMITED" },
    { "0CB088", "AITelecom" },
    { "0CB2B7", "Texas Instruments" },
    { "0CB319", "Samsung Electronics Co.,Ltd" },
    { "0CB34F", "Shenzhen Xiaoqi Intelligent Technology Co., Ltd." },
    { "0CB459", "Marketech International Corp." },
    { "0CB4A4", "Xintai Automobile Intelligent Network Technology" },
    { "0CB4EF", "Digience Co.,Ltd." },
    { "0CB527", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0CB5B3", "Huawei Device Co., Ltd." },
    { "0CB5DE", "Alcatel Lucent" },
    { "0CB6D2", "D-Link International" },
    { "0CB771", "Commscope" },
    { "0CB787", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0CB789", "Honor Device Co., Ltd." },
    { "0CB78E", "Huawei Device Co., Ltd." },
    { "0CB815", "Espressif Inc." },
    { "0CB8E8", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "0CB912", "JM-DATA GmbH" },
    { "0CB937", "Ubee Interactive Co., Limited" },
    { "0CB983", "Honor Device Co., Ltd." },
    { "0CBC9F", "Apple, Inc." },
    { "0CBD51", "TCT mobile ltd" },
    { "0CBD75", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "0CBEF1", "Huawei Device Co., Ltd." },
    { "0CBF15", "Genetec Inc." },
    { "0CBF3F", "Shenzhen Lencotion Technology Co.,Ltd" },
    { "0CBF74", "Morse Micro" },
    { "0CC0C0", "MAGNETI MARELLI SISTEMAS ELECTRONICOS MEXICO" },
    { "0CC119", "Shenzhen Phaten Tech. LTD" },
    { "0CC3A7", "Meritec" },
    { "0CC3B8", "Shenzhen Jiahua Zhongli Technology Co., LTD" },
    { "0CC413", "Google, Inc." },
    { "0CC47A", "Super Micro Computer, Inc." },
    { "0CC47E", "EUCAST Co., Ltd." },
    { "0CC56C", "Apple, Inc." },
    { "0CC655", "Wuxi YSTen Technology Co.,Ltd." },
    { "0CC66A", "Nokia Corporation" },
    { "0CC6AC", "DAGS" },
    { "0CC6CC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0CC6FD", "Xiaomi Communications Co Ltd" },
    { "0CC731", "Currant, Inc." },
    { "0CC81F", "Summer Infant, Inc." },
    { "0CC844", "Cambridge Mobile Telematics, Inc." },
    { "0CC9C6", "Samwin Hong Kong Limited" },
    { "0CCAFB", "TPVision Europe B.V" },
    { "0CCB0C", "iSYS RTS GmbH" },
    { "0CCB85", "Motorola Mobility LLC, a Lenovo Company" },
    { "0CCB8D", "ASCO Numatics GmbH" },
    { "0CCC26", "Airenetworks" },
    { "0CCC47", "IEEE Registration Authority" },
    { "0CCC5D", "Apple, Inc." },
    { "0CCDB4", "Sichuan AI-Link Technology Co., Ltd." },
    { "0CCDD3", "EASTRIVER TECHNOLOGY CO., LTD." },
    { "0CCDFB", "EDIC Systems Inc." },
    { "0CCEF6", "Guizhou Fortuneship Technology Co., Ltd" },
    { "0CCF89", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "0CCFD1", "SPRINGWAVE Co., Ltd" },
    { "0CD0F8", "Cisco Systems, Inc" },
    { "0CD292", "Intel Corporate" },
    { "0CD2B5", "Binatone Telecommunication Pvt. Ltd" },
    { "0CD3A1", "Monthly Kitchen" },
    { "0CD502", "Westell Technologies Inc." },
    { "0CD5D3", "Cisco Systems, Inc" },
    { "0CD696", "Amimon Ltd" },
    { "0CD6BD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0CD746", "Apple, Inc." },
    { "0CD7C2", "Axium Technologies, Inc." },
    { "0CD86C", "SHENZHEN FAST TECHNOLOGIES CO.,LTD" },
    { "0CD923", "GOCLOUD Networks(GAOKE Networks)" },
    { "0CD996", "Cisco Systems, Inc" },
    { "0CD9C1", "Visteon Corporation" },
    { "0CDA41", "Hangzhou H3C Technologies Co., Limited" },
    { "0CDBEA", "Apple, Inc." },
    { "0CDC7E", "Espressif Inc." },
    { "0CDC91", "Amazon Technologies Inc." },
    { "0CDCCC", "Inala Technologies" },
    { "0CDD24", "Intel Corporate" },
    { "0CDDEF", "Nokia Corporation" },
    { "0CDFA4", "Samsung Electronics Co.,Ltd" },
    { "0CE041", "iDruide" },
    { "0CE0DC", "Samsung Electronics Co.,Ltd" },
    { "0CE0E4", "PLANTRONICS, INC." },
    { "0CE0FC", "Edgecore Americas Networking Corporation" },
    { "0CE159", "Shenzhen iStartek Technology Co., Ltd." },
    { "0CE441", "Apple, Inc." },
    { "0CE4A0", "Huawei Device Co., Ltd." },
    { "0CE5A3", "SharkNinja" },
    { "0CE5B5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0CE5D3", "DH electronics GmbH" },
    { "0CE67C", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "0CE709", "Fox Crypto B.V." },
    { "0CE725", "Microsoft Corporation" },
    { "0CE82F", "Bonfiglioli Vectron GmbH" },
    { "0CE936", "ELIMOS srl" },
    { "0CE99A", "ATLS ALTEC" },
    { "0CEA14", "Ubiquiti Inc" },
    { "0CEAC9", "Commscope" },
    { "0CEC80", "Texas Instruments" },
    { "0CEC84", "Shenzhen TINNO Mobile Technology Corp." },
    { "0CEC8D", "Motorola Mobility LLC, a Lenovo Company" },
    { "0CED71", "Extreme Networks Headquarters" },
    { "0CEDC8", "Xiaomi Communications Co Ltd" },
    { "0CEE20", "FBC" },
    { "0CEE99", "Amazon Technologies Inc." },
    { "0CEEE6", "Hon Hai Precision Ind. Co.,Ltd." },
    { "0CEF15", "TP-Link Systems Inc." },
    { "0CEF7C", "AnaCom Inc" },
    { "0CEFAF", "IEEE Registration Authority" },
    { "0CEFF6", "Silicon Laboratories" },
    { "0CF019", "Malgn Technology Co., Ltd." },
    { "0CF07B", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "0CF0B4", "Globalsat International Technology Ltd" },
    { "0CF346", "Xiaomi Communications Co Ltd" },
    { "0CF361", "Java Information" },
    { "0CF3EE", "EM Microelectronic" },
    { "0CF405", "Beijing Signalway Technologies Co.,Ltd" },
    { "0CF475", "Zliide Technologies ApS" },
    { "0CF4D5", "Ruckus Wireless" },
    { "0CF5A4", "Cisco Systems, Inc" },
    { "0CF893", "Commscope" },
    { "0CF9C0", "SKY UK LIMITED" },
    { "0CFA22", "FLIPPER DEVICES INC" },
    { "0CFC18", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "0CFC83", "Airoha Technology Corp.," },
    { "0CFD37", "SUSE Linux GmbH" },
    { "0CFE45", "Sony Interactive Entertainment Inc." },
    { "0CFE5D", "IEEE Registration Authority" },
    { "0CFE7B", "Vantiva USA LLC" },
    { "100000", "Private" },
    { "100020", "Apple, Inc." },
    { "10003B", "Espressif Inc." },
    { "10005A", "IBM Corp" },
    { "1000E8", "NATIONAL SEMICONDUCTOR" },
    { "1000FD", "LaonPeople" },
    { "100177", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1001CA", "Ashley Butterworth" },
    { "1002B5", "Intel Corporate" },
    { "1004C1", "JD Cloud Computing Co., Ltd." },
    { "100501", "PEGATRON CORPORATION" },
    { "1005B1", "Commscope" },
    { "1005CA", "Cisco Systems, Inc" },
    { "1005E1", "Nokia" },
    { "10061C", "Espressif Inc." },
    { "100645", "Sagemcom Broadband SAS" },
    { "100648", "IEEE Registration Authority" },
    { "1006ED", "Cisco Systems, Inc" },
    { "10071D", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "100723", "IEEE Registration Authority" },
    { "1007B6", "Samsung Electronics Co.,Ltd" },
    { "10082C", "Texas Instruments" },
    { "1008B1", "Hon Hai Precision Ind. Co.,Ltd." },
    { "10090C", "JANOME Corporation" },
    { "1009F9", "Amazon Technologies Inc." },
    { "100BA9", "Intel Corporate" },
    { "100C24", "pomdevices, LLC" },
    { "100C29", "Shenzhen NORCO lntelligent Technology Co.,Ltd" },
    { "100C6B", "NETGEAR" },
    { "100D2F", "Online Security Pty. Ltd." },
    { "100D32", "Embedian, Inc." },
    { "100D7F", "NETGEAR" },
    { "100D8C", "Huawei Device Co., Ltd." },
    { "100E2B", "NEC CASIO Mobile Communications" },
    { "100E7E", "Juniper Networks" },
    { "100F18", "Fu Gang Electronic(KunShan)CO.,LTD" },
    { "101081", "zte corporation" },
    { "1010B6", "McCain Inc" },
    { "101212", "Vivo International Corporation Pty Ltd" },
    { "101218", "Korins Inc." },
    { "101248", "ITG, Inc." },
    { "101250", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "1012B4", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "1012D0", "zte corporation" },
    { "1012FB", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "101331", "Technicolor Delivery Technologies Belgium NV" },
    { "1013EE", "Justec International Technology INC." },
    { "1015C1", "Zhanzuo (Beijing) Technology Co., Ltd." },
    { "101849", "WEIFANG GOERTEK ELECTRONICS CO.,LTD" },
    { "10189E", "Elmo Motion Control" },
    { "101965", "New H3C Technologies Co., Ltd" },
    { "101A92", "AKEBONO BRAKE INDUSTRY CO.,LTD." },
    { "101B54", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "101C0C", "Apple, Inc." },
    { "101D51", "8Mesh Networks Limited" },
    { "101D6E", "Hewlett Packard Enterprise" },
    { "101DC0", "Samsung Electronics Co.,Ltd" },
    { "101EDA", "INGENICO TERMINALS SAS" },
    { "101F74", "Hewlett Packard" },
    { "1020BA", "Espressif Inc." },
    { "102279", "ZeroDesktop, Inc." },
    { "102381", "Barrot Technology Co.,LTD" },
    { "102407", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1025CE", "ELKA - Torantriebe GmbH u. Co. Betriebs KG" },
    { "102779", "Sadel S.p.A." },
    { "1027BE", "TVIP" },
    { "1027F5", "TP-Link Systems Inc" },
    { "102831", "Morion Inc." },
    { "102834", "SALZ Automation GmbH" },
    { "102874", "Shenzhen Jingxun Technology Co., Ltd." },
    { "102959", "Apple, Inc." },
    { "1029AB", "Samsung Electronics Co.,Ltd" },
    { "102AB3", "Xiaomi Communications Co Ltd" },
    { "102B41", "Samsung Electronics Co.,Ltd" },
    { "102BAA", "Sagemcom Broadband SAS" },
    { "102C6B", "AMPAK Technology, Inc." },
    { "102C83", "XIMEA" },
    { "102C8D", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "102CB1", "Smart Innovation LLC" },
    { "102CEF", "EMU Electronic AG" },
    { "102D31", "Shenzhen Americas Trading Company LLC" },
    { "102D41", "Sichuan AI-Link Technology Co., Ltd." },
    { "102D96", "Looxcie Inc." },
    { "102E00", "Intel Corporate" },
    { "102EAF", "Texas Instruments" },
    { "102F6B", "Microsoft Corporation" },
    { "102F6E", "Shenzhen Sundray Technologies company Limited" },
    { "102FA3", "Shenzhen Uvision-tech Technology Co.Ltd" },
    { "102FCA", "Apple, Inc." },
    { "102FF8", "Vicoretek (Nanjing) Co.,Ltd." },
    { "103025", "Apple, Inc." },
    { "103034", "Cara Systems" },
    { "103047", "Samsung Electronics Co.,Ltd" },
    { "10321D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "10322C", "Murata Manufacturing Co., Ltd." },
    { "10327E", "Huawei Device Co., Ltd." },
    { "103378", "FLECTRON Co., LTD" },
    { "1033BF", "Vantiva USA LLC" },
    { "10341B", "Spacelink" },
    { "103597", "Qorvo Utrecht B.V." },
    { "10364A", "Boston Dynamics" },
    { "1036AA", "Vantiva - Connected Home" },
    { "103711", "NORBIT ITS" },
    { "10381F", "Sichuan AI-Link Technology Co., Ltd." },
    { "103917", "Samsung Electronics Co.,Ltd" },
    { "10394E", "Hisense broadband multimedia technology Co.,Ltd" },
    { "1039E9", "Juniper Networks" },
    { "103B59", "Samsung Electronics Co.,Ltd" },
    { "103C59", "zte corporation" },
    { "103D0A", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "103D1C", "Intel Corporate" },
    { "103D3E", "China Mobile Group Device Co.,Ltd." },
    { "103DEA", "HFC Technology (Beijing) Ltd. Co." },
    { "103F44", "Xiaomi Communications Co Ltd" },
    { "103F8C", "New H3C Technologies Co., Ltd" },
    { "1040F3", "Apple, Inc." },
    { "104121", "TELLESCOM INDUSTRIA E COMERCIO EM TELECOMUNICACAO " },
    { "10417F", "Apple, Inc." },
    { "104210", "Apple, Inc." },
    { "104369", "Soundmax Electronic Limited " },
    { "104400", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "10445A", "Shaanxi Hitech Electronic Co., LTD" },
    { "1045BE", "Norphonic AS" },
    { "1045F8", "LNT-Automation GmbH" },
    { "1046B4", "FormericaOE" },
    { "104738", "Nokia Shanghai Bell Co., Ltd." },
    { "104780", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1047E7", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "1048B1", "Beijing Duokan Technology Limited" },
    { "104963", "HARTING K.K." },
    { "104A7D", "Intel Corporate" },
    { "104B46", "Mitsubishi Electric Corporation" },
    { "104C43", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "104D15", "Viaanix Inc" },
    { "104D77", "Innovative Computer Engineering" },
    { "104E07", "Shanghai Genvision Industries Co.,Ltd" },
    { "104E20", "HSE SMART" },
    { "104E89", "Garmin International" },
    { "104F58", "Hewlett Packard Enterprise" },
    { "104FA8", "Sony Corporation" },
    { "105072", "Sercomm Corporation." },
    { "105107", "Intel Corporate" },
    { "105172", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1051DB", "Espressif Inc." },
    { "10521C", "Espressif Inc." },
    { "1052BD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "105403", "INTARSO GmbH" },
    { "1054D2", "IEEE Registration Authority" },
    { "1055E4", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "105611", "Commscope" },
    { "1056CA", "Peplink International Ltd." },
    { "105725", "Cisco Systems, Inc" },
    { "105887", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "105917", "Tonal" },
    { "105932", "Roku, Inc" },
    { "105A17", "Tuya Smart Inc." },
    { "105A95", "TP-Link Systems Inc." },
    { "105AF7", "ADB Italia " },
    { "105BAD", "Mega Well Limited" },
    { "105C3B", "Perma-Pipe, Inc." },
    { "105CBF", "DuroByte Inc" },
    { "105DDC", "Huawei Device Co., Ltd." },
    { "105EAE", "New H3C Technologies Co., Ltd" },
    { "105F02", "Ruijie Networks Co.,LTD" },
    { "105F06", "Actiontec Electronics, Inc" },
    { "105F49", "Cisco SPVTG" },
    { "105F81", "INTENTSECURE Inc.," },
    { "105FAD", "Intel Corporate" },
    { "105FD4", "Tendyron Corporation" },
    { "10604B", "Hewlett Packard" },
    { "1062C9", "Adatis GmbH & Co. KG" },
    { "1062D0", "Vantiva USA LLC" },
    { "1062E5", "Hewlett Packard" },
    { "1062EB", "D-Link International" },
    { "10634B", "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "1063A3", "IEEE Registration Authority" },
    { "1063C8", "Liteon Technology Corporation" },
    { "1064E2", "ADFweb.com s.r.l." },
    { "106519", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "106530", "Dell Inc." },
    { "1065A3", "Panamax LLC" },
    { "1065CF", "IQSIM" },
    { "106650", "Robert Bosch JuP1" },
    { "10666A", "Zabbly" },
    { "106682", "NEC Platforms, Ltd." },
    { "1067A3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "106838", "AzureWave Technology Inc." },
    { "10683F", "LG Electronics (Mobile Communications)" },
    { "106F3F", "BUFFALO.INC" },
    { "106FD9", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "106FEF", "Ad-Sol Nissin Corp" },
    { "1070FD", "Mellanox Technologies, Inc." },
    { "107100", "Huawei Device Co., Ltd." },
    { "1071B3", "Zyxel Communications Corporation" },
    { "1071F9", "Cloud Telecomputers, LLC" },
    { "1071FA", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "107223", "TELLESCOM INDUSTRIA E COMERCIO EM TELECOMUNICACAO " },
    { "1073C6", "August Internet Limited" },
    { "1073EB", "Infiniti Electro-Optics" },
    { "10746F", "MOTOROLA SOLUTIONS MALAYSIA SDN. BHD." },
    { "1074C5", "Calix Inc." },
    { "107636", "Earda Technologies co Ltd" },
    { "10768A", "EoCell" },
    { "107717", "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD" },
    { "1077B0", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "1077B1", "Samsung Electronics Co.,Ltd" },
    { "10785B", "Actiontec Electronics, Inc" },
    { "107873", "Shenzhen Jinkeyi Communication Co., Ltd." },
    { "1078CE", "Hanvit SI, Inc." },
    { "1078D2", "Elitegroup Computer Systems Co.,Ltd." },
    { "107A86", "U&U ENGINEERING INC." },
    { "107B44", "ASUSTek COMPUTER INC." },
    { "107B93", "Zhen Shi Information Technology (Shanghai) Co., Ltd." },
    { "107BA4", "Olive & Dove Co.,Ltd." },
    { "107BCE", "Nokia" },
    { "107BEF", "Zyxel Communications Corporation" },
    { "107C61", "ASUSTek COMPUTER INC." },
    { "107D1A", "Dell Inc." },
    { "1081B4", "Hunan Greatwall Galaxy Science and Technology Co.,Ltd." },
    { "10823D", "Ruijie Networks Co.,LTD" },
    { "108286", "Luxshare Precision Industry Co.,Ltd" },
    { "1082D7", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "1083B4", "Sidora Srl" },
    { "1083D2", "Microseven Systems, LLC" },
    { "10868C", "Commscope" },
    { "1086F4", "Huawei Device Co., Ltd." },
    { "10880F", "Daruma Telecomunicações e Informática S.A." },
    { "1088CE", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "1089FB", "Samsung Electronics Co.,Ltd" },
    { "108A1B", "RAONIX Inc." },
    { "108A7B", "Nokia" },
    { "108B6A", "Antailiye Technology Co.,Ltd" },
    { "108CCF", "Cisco Systems, Inc" },
    { "108EBA", "Molekule" },
    { "108EE0", "Samsung Electronics Co.,Ltd" },
    { "108FFE", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "10907D", "Funshion Online Technologies Co.,Ltd" },
    { "1090FA", "New H3C Technologies Co., Ltd" },
    { "1090FC", "Shenzhen DOOGEE Hengtong Technology CO.,LTD" },
    { "109166", "Shenzhen Yinwang Intelligent Technologies Co.,Ltd." },
    { "1091A8", "Espressif Inc." },
    { "1091D1", "Intel Corporate" },
    { "109266", "Samsung Electronics Co.,Ltd" },
    { "109397", "Commscope" },
    { "1093E9", "Apple, Inc." },
    { "109497", "Logitech Hong Kong" },
    { "1094BB", "Apple, Inc." },
    { "10954B", "Megabyte Ltd." },
    { "10961A", "CHIPSEA TECHNOLOGIES (SHENZHEN) CORP." },
    { "109693", "Amazon Technologies Inc." },
    { "1096C6", "Cisco Systems, Inc" },
    { "1097BD", "Espressif Inc." },
    { "109819", "Dell Inc." },
    { "109826", "Nokia" },
    { "109836", "Dell Inc." },
    { "10985F", "Inventus Power Eletronica do Brasil LTDA" },
    { "1098C3", "Murata Manufacturing Co., Ltd." },
    { "109AB9", "Tosibox Oy" },
    { "109ADD", "Apple, Inc." },
    { "109C70", "Prusa Research s.r.o." },
    { "109D7A", "Huawei Device Co., Ltd." },
    { "109D9C", "EM Microelectronic" },
    { "109E3A", "Zhejiang Tmall Technology Co., Ltd." },
    { "109F41", "Apple, Inc." },
    { "109F47", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "109F4F", "New H3C Intelligence Terminal Co., Ltd." },
    { "109FA9", "Actiontec Electronics, Inc" },
    { "10A13B", "FUJIKURA RUBBER LTD." },
    { "10A145", "nexzo india pvt ltd " },
    { "10A1DA", "Apple, Inc." },
    { "10A24E", "GOLD3LINK ELECTRONICS CO., LTD" },
    { "10A2D3", "Apple, Inc." },
    { "10A30F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "10A3B8", "Iskratel d.o.o." },
    { "10A450", "Kwikset" },
    { "10A4B9", "Baidu Online Network Technology (Beijing) Co., Ltd" },
    { "10A4BE", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "10A4DA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "10A51D", "Intel Corporate" },
    { "10A562", "Iton Technology Corp." },
    { "10A5D0", "Murata Manufacturing Co., Ltd." },
    { "10A659", "Mobile Create Co.,Ltd." },
    { "10A743", "SK Mtek Limited" },
    { "10A793", "Vantiva USA LLC" },
    { "10A829", "Cisco Systems, Inc" },
    { "10A932", "Beijing Cyber Cloud Technology Co. ,Ltd." },
    { "10ABC9", "Samsung Electronics Co.,Ltd" },
    { "10AE60", "Amazon Technologies Inc." },
    { "10AEA5", "Duskrise inc." },
    { "10AF78", "Shenzhen ATUE Technology Co., Ltd" },
    { "10B1DF", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "10B1F8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "10B232", "Qingdao Intelligent&Precise Electronics Co.,Ltd." },
    { "10B26B", "base Co.,Ltd." },
    { "10B36F", "Bowei Technology Company Limited " },
    { "10B3C6", "Cisco Systems, Inc" },
    { "10B3D5", "Cisco Systems, Inc" },
    { "10B3D6", "Cisco Systems, Inc" },
    { "10B41D", "Espressif Inc." },
    { "10B588", "Apple, Inc." },
    { "10B65E", "New H3C Technologies Co., Ltd" },
    { "10B676", "HP Inc." },
    { "10B713", "Private" },
    { "10B7A8", "CableFree Networks Limited" },
    { "10B7F6", "Plastoform Industries Ltd." },
    { "10B9C4", "Apple, Inc." },
    { "10B9F7", "Niko-Servodan" },
    { "10B9FE", "Lika srl" },
    { "10BA1A", "SHENZHEN IK WORLD Technology Co., Ltd" },
    { "10BAA5", "GANA I&C CO., LTD" },
    { "10BBF3", "HUNAN FN-LINK TECHNOLOGY LIMITED" },
    { "10BC97", "vivo Mobile Communication Co., Ltd." },
    { "10BD18", "Cisco Systems, Inc" },
    { "10BD3A", "Apple, Inc." },
    { "10BD55", "Q-Lab Corporation" },
    { "10BE99", "Netberg" },
    { "10BEF5", "D-Link International" },
    { "10BF48", "ASUSTek COMPUTER INC." },
    { "10BF67", "Amazon Technologies Inc." },
    { "10C07C", "Blu-ray Disc Association" },
    { "10C0D5", "HOLOEYE Photonics AG" },
    { "10C172", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "10C22F", "China Entropy Co., Ltd." },
    { "10C25A", "Vantiva USA LLC" },
    { "10C2BA", "UTT Co., Ltd." },
    { "10C37B", "ASUSTek COMPUTER INC." },
    { "10C3AB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "10C4CA", "HUMAX Co., Ltd." },
    { "10C586", "BIO SOUND LAB CO., LTD." },
    { "10C595", "Lenovo" },
    { "10C60C", "Domino UK Ltd" },
    { "10C61F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "10C65E", "Adapt-IP" },
    { "10C67E", "SHENZHEN JUCHIN TECHNOLOGY CO., LTD" },
    { "10C6FC", "Garmin International" },
    { "10C735", "Microsoft Corporation" },
    { "10C73F", "Midas Klark Teknik Ltd" },
    { "10C753", "Qingdao Intelligent&Precise Electronics Co.,Ltd." },
    { "10C9CA", "Ace Technology Corp." },
    { "10CA81", "PRECIA" },
    { "10CABF", "Texas Instruments" },
    { "10CC1B", "Liverock technologies,INC" },
    { "10CCDB", "AXIMUM PRODUITS ELECTRONIQUES" },
    { "10CD6E", "FISYS" },
    { "10CDAE", "Avaya Inc" },
    { "10CDB6", "Essential Products, Inc." },
    { "10CE02", "Amazon Technologies Inc." },
    { "10CE45", "Miromico AG" },
    { "10CEA9", "Texas Instruments" },
    { "10CEE9", "Apple, Inc." },
    { "10CF0F", "Apple, Inc." },
    { "10D07A", "AMPAK Technology, Inc." },
    { "10D0AB", "zte corporation" },
    { "10D1DC", "INSTAR Deutschland GmbH" },
    { "10D38A", "Samsung Electronics Co.,Ltd" },
    { "10D542", "Samsung Electronics Co.,Ltd" },
    { "10D561", "Tuya Smart Inc." },
    { "10D657", "Siemens Industrial Automation Products Ltd., Chengdu" },
    { "10D680", "Tendyron Corporation" },
    { "10D7B0", "Sagemcom Broadband SAS" },
    { "10D9A2", "Google, Inc." },
    { "10DA43", "NETGEAR" },
    { "10DA49", "Huawei Device Co., Ltd." },
    { "10DA63", "Apple, Inc." },
    { "10DC4A", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "10DCB6", "IEEE Registration Authority" },
    { "10DDB1", "Apple, Inc." },
    { "10DDF4", "Maxway Electronics CO.,LTD" },
    { "10DEE4", "automationNEXT GmbH" },
    { "10DF8B", "Shenzhen CareDear Communication Technology Co.,Ltd" },
    { "10DFFC", "Siemens AG" },
    { "10E177", "Commscope" },
    { "10E18E", "Universal Global Scientific Industrial., Ltd" },
    { "10E2C9", "Apple, Inc." },
    { "10E2D5", "Qi Hardware Inc." },
    { "10E376", "Cisco Systems, Inc" },
    { "10E3C7", "Seohwa Telecom" },
    { "10E4AF", "APR, LLC" },
    { "10E4C2", "Samsung Electronics Co.,Ltd" },
    { "10E68F", "KWANGSUNG ELECTRONICS KOREA CO.,LTD." },
    { "10E6AE", "Source Technologies, LLC" },
    { "10E77A", "STMicrolectronics International NV" },
    { "10E7C6", "Hewlett Packard" },
    { "10E83A", "FIBERX DISTRIBUIDORA DE PRODUTOS DE TELECOMUNICACAO LTDA" },
    { "10E840", "ZOWEE TECHNOLOGY(HEYUAN) CO., LTD." },
    { "10E878", "Nokia" },
    { "10E8A7", "Wistron Neweb Corporation" },
    { "10E8EE", "PhaseSpace" },
    { "10E953", "Huawei Device Co., Ltd." },
    { "10E992", "INGRAM MICRO SERVICES" },
    { "10EA59", "Cisco SPVTG" },
    { "10EC81", "Samsung Electronics Co.,Ltd" },
    { "10EED9", "Canoga Perkins Corporation" },
    { "10F005", "Intel Corporate" },
    { "10F068", "Ruckus Wireless" },
    { "10F163", "TNK CO.,LTD" },
    { "10F1F2", "LG Electronics (Mobile Communications)" },
    { "10F311", "Cisco Systems, Inc" },
    { "10F3DB", "Gridco Systems, Inc." },
    { "10F49A", "T3 Innovation" },
    { "10F605", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "10F60A", "Intel Corporate" },
    { "10F681", "vivo Mobile Communication Co., Ltd." },
    { "10F920", "Cisco Systems, Inc" },
    { "10F96F", "LG Electronics (Mobile Communications)" },
    { "10F9EB", "Industria Fueguina de Relojería Electrónica s.a." },
    { "10F9EE", "Nokia Corporation" },
    { "10FACE", "Reacheng Communication Technology Co.,Ltd" },
    { "10FBF0", "KangSheng LTD." },
    { "10FC33", "Huawei Device Co., Ltd." },
    { "10FC54", "Shany Electronic Co., Ltd. " },
    { "10FCB6", "mirusystems CO.,LTD" },
    { "10FEED", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "10FFE0", "GIGA-BYTE TECHNOLOGY CO.,LTD." },
    { "1100AA", "Private" },
    { "111111", "Private" },
    { "140020", "LongSung Technology (Shanghai) Co.,Ltd. " },
    { "14007D", "zte corporation" },
    { "1400E9", "Mitel Networks Corporation" },
    { "140152", "Samsung Electronics Co.,Ltd" },
    { "14019C", "Ubyon Inc." },
    { "1402EC", "Hewlett Packard Enterprise " },
    { "140467", "SNK Technologies Co.,Ltd." },
    { "140589", "Motorola Mobility LLC, a Lenovo Company" },
    { "14064C", "Vogl Electronic GmbH" },
    { "1406A7", "CRESYN" },
    { "140708", "CP PLUS GMBH & CO. KG" },
    { "1407E0", "Abrantix AG" },
    { "140805", "SKY UK LIMITED" },
    { "140808", "Espressif Inc." },
    { "1409B4", "zte corporation" },
    { "1409DC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "140A29", "Tiinlab Corporation" },
    { "140AC5", "Amazon Technologies Inc." },
    { "140C5B", "PLNetworks" },
    { "140C76", "FREEBOX SAS" },
    { "140D4F", "Flextronics International" },
    { "140F42", "Nokia" },
    { "140FA6", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "14109F", "Apple, Inc." },
    { "141114", "TECNO MOBILE LIMITED" },
    { "14115D", "Skyworth Digital Technology(Shenzhen) Co.,Ltd" },
    { "14130B", "Garmin International" },
    { "141330", "Anakreon UK LLP" },
    { "141333", "AzureWave Technology Inc." },
    { "141346", "Skyworth Digital Technology(Shenzhen) Co.,Ltd" },
    { "141357", "ATP Electronics, Inc." },
    { "1413FB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "141416", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "14144B", "Ruijie Networks Co.,LTD" },
    { "141459", "Vodafone Italia S.p.A." },
    { "14147D", "Apple, Inc." },
    { "1414E6", "Ningbo Sanhe Digital Co.,Ltd" },
    { "14157C", "TOKYO COSMOS ELECTRIC CO.,LTD." },
    { "14169D", "Cisco Systems, Inc" },
    { "14169E", "Wingtech Group (HongKong）Limited" },
    { "14172A", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "141844", "Xenon Smart Teknoloji Ltd." },
    { "141877", "Dell Inc." },
    { "1418C3", "Intel Corporate" },
    { "141973", "Beijing Yunyi Times Technology Co.,Ltd" },
    { "141A51", "Treetech Sistemas Digitais" },
    { "141A97", "Apple, Inc." },
    { "141AA3", "Motorola Mobility LLC, a Lenovo Company" },
    { "141AAA", "Metal Work SpA" },
    { "141B30", "Shenzhen Yipingfang Network Technology Co., Ltd." },
    { "141BA0", "Apple, Inc." },
    { "141BBD", "Volex Inc." },
    { "141BF0", "Intellimedia Systems Ltd" },
    { "141F78", "Samsung Electronics Co.,Ltd" },
    { "141FBA", "IEEE Registration Authority" },
    { "14205E", "Apple, Inc." },
    { "142103", "Calix Inc." },
    { "142233", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "14223B", "Google, Inc." },
    { "1422DB", "eero inc." },
    { "14230A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1423D7", "EUTRONIX CO., LTD." },
    { "1423F2", "Broadcom Limited" },
    { "1423F3", "Broadcom Limited" },
    { "142475", "4DReplay, Inc" },
    { "142876", "Apple, Inc." },
    { "142882", "MIDICOM ELECTRONICS CO.LTD" },
    { "142971", "NEMOA ELECTRONICS (HK) CO. LTD" },
    { "142A14", "ShenZhen Selenview Digital Technology Co.,Ltd" },
    { "142B2F", "Espressif Inc." },
    { "142BD2", "Armtel Ltd." },
    { "142BD6", "Guangdong Appscomm Co.,Ltd" },
    { "142C78", "GooWi Wireless Technology Co., Limited" },
    { "142D27", "Hon Hai Precision Ind. Co.,Ltd." },
    { "142D41", "Silicon Laboratories" },
    { "142D4D", "Apple, Inc." },
    { "142D79", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "142D8B", "Incipio Technologies, Inc" },
    { "142DF5", "Amphitech" },
    { "142E5E", "Sercomm Corporation." },
    { "142FFD", "LT SECURITY INC" },
    { "143004", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "14307A", "Avermetrics" },
    { "1430C6", "Motorola Mobility LLC, a Lenovo Company" },
    { "1432D1", "Samsung Electronics Co.,Ltd" },
    { "14335C", "Espressif Inc." },
    { "143365", "TEM Mobile Limited" },
    { "143375", "Zyxel Communications Corporation" },
    { "1434F6", "LV SOLUTION SDN. BHD." },
    { "14358B", "Mediabridge Products, LLC." },
    { "1435B3", "Future Designs, Inc." },
    { "1435B7", "Apple, Inc." },
    { "143605", "Nokia Corporation" },
    { "14360E", "Zyxel Communications Corporation" },
    { "1436C6", "Lenovo Mobile Communication Technology Ltd." },
    { "143719", "PT Prakarsa Visi Valutama" },
    { "14373B", "PROCOM Systems" },
    { "14375E", "Symbotic LLC" },
    { "14392F", "LEAR" },
    { "143A9A", "Hon Hai Precision Industry Co.,LTD" },
    { "143AEA", "Dynapower Company LLC" },
    { "143B42", "Realfit(Shenzhen) Intelligent Technology Co., Ltd" },
    { "143B51", "Huawei Device Co., Ltd." },
    { "143CC3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "143DF2", "Beijing Shidai Hongyuan Network Communication Co.,Ltd" },
    { "143E60", "Nokia" },
    { "143EBF", "zte corporation" },
    { "143F27", "Noccela Oy" },
    { "143FA6", "Sony Home Entertainment&Sound Products Inc" },
    { "143FC3", "SnapAV" },
    { "144146", "Honeywell (China) Co., LTD" },
    { "1441E2", "Monaco Enterprises, Inc." },
    { "1442FC", "Texas Instruments" },
    { "144319", "Creative&Link Technology Limited" },
    { "14444A", "Apollo Seiko Ltd." },
    { "14448F", "Edgecore Networks Corporation" },
    { "144658", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1446E4", "AVISTEL" },
    { "14472D", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "144802", "THE YEOLRIM Co.,Ltd." },
    { "14488B", "Shenzhen Doov Technology Co.,Ltd" },
    { "144920", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "144978", "Digital Control Incorporated" },
    { "1449BC", "DrayTek Corp." },
    { "1449D4", "Xiaomi Communications Co Ltd" },
    { "1449E0", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "144C1A", "Max Communication GmbH" },
    { "144D67", "Zioncom Electronics (Shenzhen) Ltd." },
    { "144E2A", "Ciena Corporation" },
    { "144E34", "Remote Solution" },
    { "144F8A", "Intel Corporate" },
    { "144FD7", "IEEE Registration Authority" },
    { "145051", "SHARP Corporation" },
    { "145120", "Huawei Device Co., Ltd." },
    { "14517E", "New H3C Technologies Co., Ltd" },
    { "145290", "KNS Group LLC (YADRO Company)" },
    { "145412", "Entis Co., Ltd." },
    { "145594", "Huawei Device Co., Ltd." },
    { "1455B9", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "14563A", "Huawei Device Co., Ltd." },
    { "145645", "Savitech Corp." },
    { "14568E", "Samsung Electronics Co.,Ltd" },
    { "145790", "Qingdao Haier Technology Co.,Ltd" },
    { "14579F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "145808", "Taicang T&W Electronics" },
    { "1458D0", "Hewlett Packard" },
    { "1459C0", "NETGEAR" },
    { "1459C3", "Creative Chips GmbH" },
    { "145A05", "Apple, Inc." },
    { "145A41", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "145A83", "Logi-D inc" },
    { "145AFC", "Liteon Technology Corporation" },
    { "145BB9", "ConMet" },
    { "145BD1", "Commscope" },
    { "145BE1", "nyantec GmbH" },
    { "145D34", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "145E45", "Bamboo Systems Group" },
    { "145E69", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "145F94", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "146080", "zte corporation" },
    { "1460CB", "Apple, Inc." },
    { "146102", "Alps Alpine" },
    { "14612F", "Avaya Inc" },
    { "1461A4", "Honor Device Co., Ltd." },
    { "146308", "JABIL CIRCUIT (SHANGHAI) LTD." },
    { "14656A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1466B7", "Advanced Design Technology Pty Ltd" },
    { "1469A2", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "146A0B", "Cypress Electronics Limited" },
    { "146B72", "Shenzhen Fortune Ship Technology Co., Ltd." },
    { "146B9A", "zte corporation" },
    { "146B9C", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "146C27", "Dongguan Liesheng Electronic Co., Ltd." },
    { "146E0A", "Private" },
    { "147057", "SHENNAN CIRCUITS CO.,LTD" },
    { "147373", "TUBITAK UEKAE" },
    { "147411", "RIM" },
    { "14755B", "Intel Corporate" },
    { "147590", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "1475E5", "ELMAX Srl" },
    { "147740", "Huawei Device Co., Ltd." },
    { "14780B", "Varex Imaging Deutschland AG" },
    { "1479F3", "China Mobile Group Device Co.,Ltd." },
    { "147AE4", "Apple, Inc." },
    { "147BAC", "Nokia" },
    { "147D05", "SERCOMM PHILIPPINES INC" },
    { "147DB3", "JOA TELECOM.CO.,LTD" },
    { "147DC5", "Murata Manufacturing Co., Ltd." },
    { "147DDA", "Apple, Inc." },
    { "147E19", "Hewlett Packard Enterprise" },
    { "147EA1", "Britania Eletrônicos S.A." },
    { "147F0F", "Texas Instruments" },
    { "147F67", "LG Innotek" },
    { "147FCE", "Apple, Inc." },
    { "14801F", "Sunnovo International Limited" },
    { "1480CC", "Quectel Wireless Solutions Co.,Ltd." },
    { "148121", "TOP WING Corporation" },
    { "14825B", "Hefei Radio Communication Technology Co., Ltd " },
    { "148430", "MITAC COMPUTING TECHNOLOGY CORPORATION" },
    { "148473", "Cisco Systems, Inc" },
    { "148477", "New H3C Technologies Co., Ltd" },
    { "148501", "Rivos Inc." },
    { "148509", "Apple, Inc." },
    { "148554", "Earda Technologies co Ltd" },
    { "14857F", "Intel Corporate" },
    { "148692", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "14876A", "Apple, Inc." },
    { "1488E6", "Apple, Inc." },
    { "148919", "2bps" },
    { "14893E", "VIXTEL TECHNOLOGIES LIMTED" },
    { "1489CB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1489FD", "Samsung Electronics Co.,Ltd" },
    { "148A70", "ADS GmbH" },
    { "148C4A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "148F21", "Garmin International" },
    { "148F34", "TECNO MOBILE LIMITED" },
    { "148F79", "Apple, Inc." },
    { "148FC6", "Apple, Inc." },
    { "149090", "KongTop industrial(shen zhen)CO.,LTD" },
    { "149138", "Amazon Technologies Inc." },
    { "149182", "Belkin International Inc." },
    { "1492F9", "Sichuan AI-Link Technology Co., Ltd." },
    { "149346", "PNI sensor corporation" },
    { "14942F", "USYS CO.,LTD." },
    { "149448", "BLU CASTLE S.A." },
    { "14946C", "Apple, Inc." },
    { "1495CE", "Apple, Inc." },
    { "14962D", "New H3C Technologies Co., Ltd" },
    { "1496E5", "Samsung Electronics Co.,Ltd" },
    { "149877", "Apple, Inc." },
    { "14987D", "Vantiva USA LLC" },
    { "14993E", "Xiaomi Communications Co Ltd" },
    { "1499E2", "Apple, Inc." },
    { "149A10", "Microsoft Corporation" },
    { "149AA3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "149B2F", "JiangSu ZhongXie Intelligent Technology co., LTD" },
    { "149BD7", "MULI MUWAI FURNITURE QIDONG CO., LTD" },
    { "149BF3", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "149CEF", "Texas Instruments" },
    { "149D09", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "149D99", "Apple, Inc." },
    { "149E5D", "JSC IB Reform" },
    { "149ECF", "Dell Inc." },
    { "149F3C", "Samsung Electronics Co.,Ltd" },
    { "149F43", "Cisco Meraki" },
    { "149FB6", "GUANGDONG GENIUS TECHNOLOGY CO., LTD." },
    { "149FE8", "Lenovo Mobile Communication Technology Ltd." },
    { "14A0F8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "14A1BF", "ASSA ABLOY Korea Co., Ltd Unilock" },
    { "14A1DF", "China Mobile Group Device Co.,Ltd." },
    { "14A2A0", "Cisco Systems, Inc" },
    { "14A32F", "Huawei Device Co., Ltd." },
    { "14A364", "Samsung Electronics Co.,Ltd" },
    { "14A3B4", "Huawei Device Co., Ltd." },
    { "14A417", "Shenzhen Belon Technology CO.,LTD" },
    { "14A51A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "14A62C", "S.M. Dezac S.A." },
    { "14A72B", "currentoptronics Pvt.Ltd" },
    { "14A78B", "Zhejiang Dahua Technology Co., Ltd." },
    { "14A86B", "ShenZhen Telacom Science&Technology Co., Ltd" },
    { "14A9D0", "F5 Inc." },
    { "14A9E3", "MST CORPORATION" },
    { "14AB02", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "14AB56", "WUXI FUNIDE DIGITAL CO.,LTD" },
    { "14ABC5", "Intel Corporate" },
    { "14ABEC", "Hewlett Packard Enterprise" },
    { "14ABF0", "Commscope" },
    { "14AC60", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "14ADCA", "China Mobile Iot Limited company" },
    { "14AE68", "KLG Smartec" },
    { "14AE85", "IEEE Registration Authority" },
    { "14AEDB", "VTech Telecommunications Ltd." },
    { "14B126", "Industrial Software Co" },
    { "14B1C8", "InfiniWing, Inc." },
    { "14B2E5", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "14B31F", "Dell Inc." },
    { "14B370", "Gigaset Digital Technology (Shenzhen) Co., Ltd." },
    { "14B3A1", "Juniper Networks" },
    { "14B457", "Silicon Laboratories" },
    { "14B484", "Samsung Electronics Co.,Ltd" },
    { "14B5CD", "Liteon Technology Corporation" },
    { "14B73D", "ARCHEAN Technologies" },
    { "14B7F8", "Vantiva USA LLC" },
    { "14B837", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "14B968", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "14BAAF", "BKS GmbH" },
    { "14BB6E", "Samsung Electronics Co.,Ltd" },
    { "14BC68", "Cisco Systems, Inc" },
    { "14BD61", "Apple, Inc." },
    { "14BEFC", "Nanjing Jiahao Technology Co., Ltd." },
    { "14C03E", "Commscope" },
    { "14C050", "GUANGDONG GENIUS TECHNOLOGY CO., LTD." },
    { "14C089", "DUNE HD LTD" },
    { "14C0A1", "UCloud Technology Co., Ltd." },
    { "14C126", "Nokia Corporation" },
    { "14C14E", "Google, Inc." },
    { "14C1FF", "ShenZhen QianHai Comlan communication Co.,LTD" },
    { "14C213", "Apple, Inc." },
    { "14C21D", "Sabtech Industries" },
    { "14C24D", "ATW TECHNOLOGY, INC." },
    { "14C35E", "FibRSol Global Network Limited" },
    { "14C3C2", "K.A. Schmersal GmbH & Co. KG" },
    { "14C67D", "Vizio, Inc" },
    { "14C697", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "14C88B", "Apple, Inc." },
    { "14C913", "LG Electronics" },
    { "14C9CF", "Sigmastar Technology Ltd." },
    { "14CA56", "zte corporation" },
    { "14CAA0", "Hu&Co" },
    { "14CB19", "HP Inc." },
    { "14CB49", "Habolink Technology Co.,LTD" },
    { "14CB65", "Microsoft Corporation" },
    { "14CC20", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "14CCB3", "AO GK NATEKS" },
    { "14CF8D", "OHSUNG" },
    { "14CF92", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "14CFE2", "Commscope" },
    { "14D00D", "Apple, Inc." },
    { "14D11F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "14D169", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "14D19E", "Apple, Inc." },
    { "14D424", "AzureWave Technology Inc." },
    { "14D4FE", "Commscope" },
    { "14D64D", "D-Link International" },
    { "14D725", "Barrot Technology Co.,Ltd." },
    { "14D76E", "CONCH ELECTRONIC Co.,Ltd" },
    { "14D864", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "14D881", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "14DAB9", "Huawei Device Co., Ltd." },
    { "14DAE9", "ASUSTek COMPUTER INC." },
    { "14DB85", "S NET MEDIA" },
    { "14DC51", "Xiamen Cheerzing IOT Technology Co.,Ltd." },
    { "14DCE2", "THALES AVS France" },
    { "14DD02", "Liangang Optoelectronic Technology CO., Ltd." },
    { "14DD9C", "vivo Mobile Communication Co., Ltd." },
    { "14DDA9", "ASUSTek COMPUTER INC." },
    { "14DDE5", "MPMKVVCL" },
    { "14DE39", "Huawei Device Co., Ltd." },
    { "14E01D", "Samsung Electronics Co.,Ltd" },
    { "14E289", "Abietec Inc." },
    { "14E4EC", "mLogic LLC" },
    { "14E6E4", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "14E7C8", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "14E9B2", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "14EA63", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "14EAA1", "Micronet union Technology (chengdu) co., Ltd" },
    { "14EB08", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "14EB33", "BSMediasoft Co., Ltd." },
    { "14EBB6", "TP-Link Systems Inc" },
    { "14EDA5", "Wächter GmbH Sicherheitssysteme" },
    { "14EDBB", "2Wire Inc" },
    { "14EDE4", "Kaiam Corporation" },
    { "14EE9D", "AirNav Systems LLC" },
    { "14EFCF", "SCHREDER" },
    { "14F0C5", "Xtremio Ltd." },
    { "14F287", "Apple, Inc." },
    { "14F28E", "ShenYang ZhongKe-Allwin Technology Co.LTD" },
    { "14F42A", "Samsung Electronics Co.,Ltd" },
    { "14F592", "Shenzhen SDG DONZHI Technology Co., Ltd" },
    { "14F5F9", "HUNAN FN-LINK TECHNOLOGY LIMITED" },
    { "14F65A", "Xiaomi Communications Co Ltd" },
    { "14F6D8", "Intel Corporate" },
    { "14F893", "Wuhan FiberHome Digital Technology Co.,Ltd." },
    { "14FB70", "Huawei Device Co., Ltd." },
    { "14FEAF", "SAGITTAR LIMITED" },
    { "14FEB5", "Dell Inc." },
    { "18002D", "Sony Corporation" },
    { "1800DB", "Fitbit Inc." },
    { "18017D", "Harbin Arteor technology co., LTD" },
    { "1801E3", "Bittium Wireless Ltd" },
    { "1801F1", "Xiaomi Communications Co Ltd" },
    { "18022D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1802AE", "vivo Mobile Communication Co., Ltd." },
    { "180373", "Dell Inc." },
    { "1803FA", "IBT Interfaces" },
    { "180403", "vivo Mobile Communication Co., Ltd." },
    { "1804ED", "Texas Instruments" },
    { "180675", "Dilax Intelcom GmbH" },
    { "1806F5", "RAD Data Communications, Ltd." },
    { "1806FF", "Acer Computer(Shanghai) Limited." },
    { "180712", "Shenzhen Dazoo Technologies CO.,Ltd" },
    { "180B1B", "Amazon Technologies Inc." },
    { "180B52", "Nanotron Technologies GmbH" },
    { "180BD0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "180C14", "iSonea Limited" },
    { "180C77", "Westinghouse Electric Company, LLC" },
    { "180C7A", "Sagemcom Broadband SAS" },
    { "180CAC", "CANON INC." },
    { "180D2C", "Intelbras" },
    { "180DF9", "Silicon Laboratories" },
    { "180EAC", "SHENZHEN FAST TECHNOLOGIES CO.,LTD" },
    { "180F76", "D-Link International" },
    { "18104E", "CEDINT-UPM" },
    { "181171", "Guangzhou Doctorpai Education & Technology Co.,Ltd" },
    { "181212", "Cepton Technologies" },
    { "18132D", "zte corporation" },
    { "181420", "TEB SAS" },
    { "181456", "Nokia Corporation" },
    { "18146C", "Zhejiang Tmall Technology Co., Ltd." },
    { "1814AE", "Nokia" },
    { "181628", "SharkNinja Operating LLC" },
    { "1816C9", "Samsung Electronics Co.,Ltd" },
    { "1816E8", "Siliconware Precision Industries Co., Ltd." },
    { "181714", "DAEWOOIS" },
    { "181725", "Cameo Communications, Inc." },
    { "18188B", "FCNT LLC" },
    { "18193F", "Tamtron Oy" },
    { "1819D6", "Samsung Electronics Co.,Ltd" },
    { "181BEB", "Actiontec Electronics, Inc" },
    { "181DEA", "Intel Corporate" },
    { "181E78", "Sagemcom Broadband SAS" },
    { "181E95", "AuVerte" },
    { "181EB0", "Samsung Electronics Co.,Ltd" },
    { "182012", "Aztech Associates Inc." },
    { "182032", "Apple, Inc." },
    { "18204C", "Kummler+Matter AG" },
    { "1820A6", "Sage Co., Ltd." },
    { "1820D5", "Commscope" },
    { "182195", "Samsung Electronics Co.,Ltd" },
    { "18227E", "Samsung Electronics Co.,Ltd" },
    { "182649", "Intel Corporate" },
    { "182654", "Samsung Electronics Co.,Ltd" },
    { "182666", "Samsung Electronics Co.,Ltd" },
    { "182861", "AirTies Wireless Networks" },
    { "182A44", "HIROSE ELECTRONIC SYSTEM" },
    { "182A57", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "182A7B", "Nintendo Co., Ltd." },
    { "182AD3", "Juniper Networks" },
    { "182B05", "8D Technologies" },
    { "182C65", "Texas Instruments" },
    { "182C91", "Concept Development, Inc." },
    { "182CA9", "DASAN Networks, Inc. " },
    { "182CB4", "Nectarsoft Co., Ltd." },
    { "182D98", "Jinwoo Industrial system" },
    { "182DF7", "JY COMPANY" },
    { "183009", "Woojin Industrial Systems Co., Ltd." },
    { "18300C", "Hisense Electric Co.,Ltd" },
    { "18314F", "AIDIN ROBOTICS" },
    { "1831BF", "ASUSTek COMPUTER INC." },
    { "183219", "EM Microelectronic" },
    { "1832A2", "LAON TECHNOLOGY CO., LTD." },
    { "183386", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "18339D", "Cisco Systems, Inc" },
    { "183451", "Apple, Inc." },
    { "1834AF", "Kaon Group Co., Ltd." },
    { "1835D1", "Commscope" },
    { "183672", "Shaoxing ShunChuang Technology CO.,LTD" },
    { "1836FC", "Elecsys International Corporation" },
    { "183825", "Wuhan Lingjiu High-tech Co.,Ltd." },
    { "183864", "CAP-TECH INTERNATIONAL CO., LTD." },
    { "1838AE", "CONSPIN SOLUTION" },
    { "183919", "Unicoi Systems" },
    { "18396E", "SUNSEA TELECOMMUNICATIONS CO.,LTD." },
    { "18399C", "Skorpios Technologies" },
    { "183A2D", "Samsung Electronics Co.,Ltd" },
    { "183A48", "VostroNet" },
    { "183BD2", "BYD Precision Manufacture Company Ltd." },
    { "183C98", "Shenzhen Hengyi Technology Co., LTD" },
    { "183CB7", "Huawei Device Co., Ltd." },
    { "183D2D", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "183D5E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "183DA2", "Intel Corporate" },
    { "183EEF", "Apple, Inc." },
    { "183F47", "Samsung Electronics Co.,Ltd" },
    { "183F70", "Apple, Inc." },
    { "1840A4", "Shenzhen Trylong Smart Science and Technology Co., Ltd." },
    { "1841C3", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "1841FE", "Digital 14" },
    { "18421D", "Private" },
    { "18422F", "Alcatel Lucent" },
    { "1842D4", "Wuhan Hosan Telecommunication Technology Co.,Ltd" },
    { "184462", "Riava Networks, Inc." },
    { "1844CF", "B+L Industrial Measurements GmbH" },
    { "1844E6", "zte corporation" },
    { "184516", "Texas Instruments" },
    { "184593", "Taicang T&W Electronics" },
    { "1845B3", "IEEE Registration Authority" },
    { "184617", "Samsung Electronics Co.,Ltd" },
    { "184644", "Home Control Singapore Pte Ltd" },
    { "18473D", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "184859", "Castlenet Technology Inc." },
    { "1848BE", "Amazon Technologies Inc." },
    { "1848CA", "Murata Manufacturing Co., Ltd." },
    { "1848D8", "Fastback Networks" },
    { "1849F8", "Extreme Networks Headquarters" },
    { "184A53", "Apple, Inc." },
    { "184A6F", "Alcatel-Lucent Shanghai Bell Co., Ltd" },
    { "184B0D", "Ruckus Wireless" },
    { "184BDF", "Caavo Inc" },
    { "184C08", "Rockwell Automation" },
    { "184CAE", "CONTINENTAL" },
    { "184E03", "HMD Global Oy" },
    { "184E16", "Samsung Electronics Co.,Ltd" },
    { "184E94", "MESSOA TECHNOLOGIES INC." },
    { "184ECB", "Samsung Electronics Co.,Ltd" },
    { "184F32", "Hon Hai Precision Ind. Co.,Ltd." },
    { "184F43", "UNIONMAN TECHNOLOGY CO.,LTD" },
    { "184F5D", "JRC Mobility Inc." },
    { "18502A", "SOARNEX" },
    { "185073", "Tianjin HuaLai Technology CO., Ltd." },
    { "185111", "Universal Electronics, Inc." },
    { "185207", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "18523D", "Xiamen Jiwu Technology CO.,Ltd" },
    { "185253", "Pixord Corporation" },
    { "185282", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "185345", "Nokia" },
    { "1853E0", "Hanyang Digitech Co.Ltd" },
    { "1854CF", "Samsung Electronics Co.,Ltd" },
    { "18550F", "Cisco SPVTG" },
    { "1855E3", "Apple, Inc." },
    { "185644", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "185680", "Intel Corporate" },
    { "1856C3", "Apple, Inc." },
    { "185869", "Sailer Electronic Co., Ltd" },
    { "185880", "Arcadyan Corporation" },
    { "185933", "Cisco SPVTG" },
    { "185936", "Xiaomi Communications Co Ltd" },
    { "1859F5", "Cisco Systems, Inc" },
    { "185A58", "Dell Inc." },
    { "185AE8", "Zenotech.Co.,Ltd" },
    { "185B00", "Nokia" },
    { "185BB3", "Samsung Electronics Co.,Ltd" },
    { "185D6F", "N3com" },
    { "185D9A", "BobjGear LLC" },
    { "185E0B", "zte corporation" },
    { "185E0F", "Intel Corporate" },
    { "186024", "Hewlett Packard" },
    { "186041", "Arcadyan Corporation" },
    { "1861C7", "lemonbeat GmbH" },
    { "18622C", "Sagemcom Broadband SAS" },
    { "1862E4", "Texas Instruments" },
    { "186472", "Hewlett Packard Enterprise" },
    { "186571", "Top Victory Electronics (Taiwan) Co., Ltd." },
    { "186590", "Apple, Inc." },
    { "1865C7", "Dongguan YIMO Technology Co.LTD" },
    { "186696", "IEEE Registration Authority" },
    { "1866C7", "Shenzhen Libre Technology Co., Ltd" },
    { "1866DA", "Dell Inc." },
    { "1866E3", "Veros Systems, Inc." },
    { "1866F0", "Jupiter Systems" },
    { "18673F", "Hanover Displays Limited" },
    { "186751", "KOMEG Industrielle Messtechnik GmbH" },
    { "1867B0", "Samsung Electronics Co.,Ltd" },
    { "18686A", "zte corporation" },
    { "186882", "Beward R&D Co., Ltd." },
    { "1868CB", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "1869D4", "Samsung Electronics Co.,Ltd" },
    { "1869D8", "Tuya Smart Inc." },
    { "1869DA", "China Mobile Group Device Co.,Ltd." },
    { "186A81", "Sagemcom Broadband SAS" },
    { "186BE2", "LYLINK LIMITED" },
    { "186D99", "Adanis Inc." },
    { "186F2D", "Shenzhen Sundray Technologies company Limited" },
    { "18703B", "Huawei Device Co., Ltd." },
    { "187117", "eta plus electronic gmbh" },
    { "1871D5", "Hazens Automotive Electronics(SZ)Co.,Ltd." },
    { "18742E", "Amazon Technologies Inc." },
    { "1874E2", "IEEE Registration Authority" },
    { "187532", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "187758", "Audoo Limited (UK)" },
    { "1878D4", "Verizon " },
    { "1879A2", "GMJ ELECTRIC LIMITED" },
    { "1879FD", "zte corporation" },
    { "187A3B", "Hewlett Packard Enterprise" },
    { "187A3E", "Silicon Laboratories" },
    { "187A93", "AMICCOM Electronics Corporation" },
    { "187C0B", "Ruckus Wireless" },
    { "187C81", "Valeo Vision Systems" },
    { "187CAA", "China Mobile Group Device Co.,Ltd." },
    { "187E20", "FUJIAN STAR-NET COMMUNICATION CO.,LTD" },
    { "187EB9", "Apple, Inc." },
    { "187ED5", "shenzhen kaism technology Co. Ltd" },
    { "187F88", "Ring LLC" },
    { "188025", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "188090", "Cisco Systems, Inc" },
    { "1880CE", "Barberry Solutions Ltd" },
    { "1880F5", "Alcatel-Lucent Shanghai Bell Co., Ltd" },
    { "18810E", "Apple, Inc." },
    { "1881ED", "TERACOM TELEMATICA S.A" },
    { "188219", "Alibaba Cloud Computing Ltd." },
    { "18828C", "Arcadyan Corporation" },
    { "188331", "Samsung Electronics Co.,Ltd" },
    { "1883BF", "Arcadyan Technology Corporation" },
    { "188410", "CoreTrust Inc." },
    { "1884C1", "Guangzhou Shiyuan Electronic Technology Company Limited" },
    { "188637", "INGRAM MICRO SERVICES" },
    { "18863A", "DIGITAL ART SYSTEM" },
    { "1886AC", "Nokia Danmark A/S" },
    { "1886C3", "Nokia" },
    { "188740", "Xiaomi Communications Co Ltd" },
    { "188796", "HTC Corporation" },
    { "188857", "Beijing Jinhong Xi-Dian Information Technology Corp." },
    { "18895B", "Samsung Electronics Co.,Ltd" },
    { "1889A0", "Funshion Online Technologies Co.,Ltd" },
    { "1889CF", "TECNO MOBILE LIMITED" },
    { "1889DF", "OMNIVISION" },
    { "188A6A", "AVPro Global Hldgs" },
    { "188B0E", "Espressif Inc." },
    { "188B15", "ShenZhen ZhongRuiJing Technology co.,LTD" },
    { "188B45", "Cisco Systems, Inc" },
    { "188B9D", "Cisco Systems, Inc" },
    { "188ED5", "TP Vision Belgium N.V. - innovation site Brugge" },
    { "188EF9", "G2C Co. Ltd." },
    { "189067", "Shenzhen Jingxun Technology Co., Ltd." },
    { "189088", "eero inc." },
    { "1890D8", "Sagemcom Broadband SAS" },
    { "18922C", "Virtual Instruments" },
    { "1892A4", "Ciena Corporation" },
    { "189341", "Intel Corporate" },
    { "18937F", "AMPAK Technology, Inc." },
    { "1893D7", "Texas Instruments" },
    { "1894A3", "Wistron Service(Kunshan) Co., Ltd." },
    { "1894C6", "ShenZhen Chenyee Technology Co., Ltd." },
    { "189552", "1MORE" },
    { "189578", "DENSO CORPORATION" },
    { "1897F1", "KOSTAL (Shanghai) Management Co., Ltd." },
    { "1897FF", "TechFaith Wireless Technology Limited" },
    { "1899F5", "Sichuan Changhong Electric Ltd." },
    { "189A67", "CSE-Servelec Limited" },
    { "189BA5", "IEEE Registration Authority" },
    { "189C27", "Commscope" },
    { "189C2C", "Dongguan Huayin Electronic Technology Co., Ltd." },
    { "189C5D", "Cisco Systems, Inc" },
    { "189CE1", "Arista Networks" },
    { "189E2C", "Huawei Device Co., Ltd." },
    { "189E2D", "Allwinner Technology Co., Ltd" },
    { "189EAD", "Shenzhen Chengqian Information Technology Co., Ltd" },
    { "189EFC", "Apple, Inc." },
    { "18A28A", "Essel-T Co., Ltd" },
    { "18A3E8", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "18A4A9", "Vanu Inc." },
    { "18A59C", "IEEE Registration Authority" },
    { "18A5FF", "Arcadyan Corporation" },
    { "18A6F7", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "18A788", "Shenzhen MEK Intellisys Pte Ltd" },
    { "18A7F1", "Qingdao Haier Technology Co.,Ltd" },
    { "18A905", "Hewlett Packard" },
    { "18A958", "PROVISION THAI CO., LTD." },
    { "18A99B", "Dell Inc." },
    { "18A9A6", "Nebra Ltd" },
    { "18A9ED", "eero inc." },
    { "18AA0F", "Huawei Device Co., Ltd." },
    { "18AA1E", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "18AA45", "Fon Technology" },
    { "18AACA", "Sichuan tianyi kanghe communications co., LTD" },
    { "18AB1D", "Samsung Electronics Co.,Ltd" },
    { "18ABF5", "Ultra Electronics Electrics" },
    { "18AC9E", "ITEL MOBILE LIMITED" },
    { "18AD4D", "Polostar Technology Corporation" },
    { "18AEBB", "Siemens Convergence Creators GmbH&Co.KG" },
    { "18AF61", "Apple, Inc." },
    { "18AF8F", "Apple, Inc." },
    { "18AF9F", "DIGITRONIC Automationsanlagen GmbH" },
    { "18AFA1", "Shenzhen Yifang Network Technology Co., Ltd." },
    { "18B169", "Sonicwall" },
    { "18B185", "Qiao Information Technology (Zhengzhou) Co., Ltd." },
    { "18B209", "Torrey Pines Logic, Inc" },
    { "18B3BA", "Netlogic AB" },
    { "18B430", "Nest Labs Inc." },
    { "18B591", "I-Storm" },
    { "18B6CC", "We Corporation Inc." },
    { "18B6F7", "NEW POS TECHNOLOGY LIMITED" },
    { "18B79E", "Invoxia" },
    { "18B81F", "Commscope" },
    { "18B905", "Hong Kong Bouffalo Lab Limited" },
    { "18B96E", "Dongguan Liesheng Electronic Co., Ltd." },
    { "18BB1C", "Huawei Device Co., Ltd." },
    { "18BB26", "FN-LINK TECHNOLOGY LIMITED" },
    { "18BB41", "Huawei Device Co., Ltd." },
    { "18BC57", "ADVA Optical Networking Ltd." },
    { "18BC5A", "Zhejiang Tmall Technology Co., Ltd." },
    { "18BDAD", "L-TECH CORPORATION" },
    { "18BE92", "Delta Networks, Inc." },
    { "18BF1C", "Jiangsu Huitong Group Co.,Ltd." },
    { "18BFB3", "Samsung Electronics Co., Ltd., Memory Division" },
    { "18C007", "Huawei Device Co., Ltd." },
    { "18C009", "New H3C Technologies Co., Ltd" },
    { "18C04D", "GIGA-BYTE TECHNOLOGY CO.,LTD." },
    { "18C086", "Broadcom" },
    { "18C19D", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "18C1E2", "Qolsys Inc." },
    { "18C23C", "Lumi United Technology Co., Ltd" },
    { "18C241", "SonicWall" },
    { "18C293", "Laird Connectivity" },
    { "18C2BF", "BUFFALO.INC" },
    { "18C300", "Nokia" },
    { "18C3F4", "IEEE Registration Authority" },
    { "18C451", "Tucson Embedded Systems" },
    { "18C501", "SHENZHEN GONGJIN ELECTRONICS CO.,LT" },
    { "18C58A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "18C8E7", "Shenzhen Hualistone Technology Co.,Ltd" },
    { "18CAA7", "zte corporation" },
    { "18CC18", "Intel Corporate" },
    { "18CC23", "Philio Technology Corporation" },
    { "18CC88", "Hitachi Air Conditioning Shimizu, Inc." },
    { "18CE94", "Samsung Electronics Co.,Ltd" },
    { "18CEDF", "Quectel Wireless Solutions Co.,Ltd." },
    { "18CF24", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "18CF5E", "Liteon Technology Corporation" },
    { "18D071", "DASAN CO., LTD." },
    { "18D0C5", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "18D225", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "18D276", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "18D5B6", "SMG Holdings LLC" },
    { "18D61C", "Shenzhen TINNO Mobile Technology Corp." },
    { "18D66A", "Inmarsat" },
    { "18D6C7", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "18D6CF", "Kurth Electronic GmbH" },
    { "18D6DD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "18D717", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "18D793", "IEEE Registration Authority" },
    { "18D949", "Qvis Labs, LLC" },
    { "18D98F", "Huawei Device Co., Ltd." },
    { "18D9EF", "Shuttle Inc." },
    { "18DBF2", "Dell Inc." },
    { "18DC56", "Yulong Computer Telecommunication Scientific (Shenzhen) Co.,Ltd" },
    { "18DE50", "Tuya Smart Inc." },
    { "18DED7", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "18DF26", "INGRAM MICRO SERVICES" },
    { "18DFB4", "BOSUNG POWERTEC CO.,LTD." },
    { "18DFC1", "Aetheros" },
    { "18E1CA", "wanze" },
    { "18E1DE", "Chengdu ChipIntelli Technology Co., Ltd" },
    { "18E204", "BEIJING COOLSHARK TECHNOLOGY CO.,LTD." },
    { "18E215", "Nokia" },
    { "18E288", "STT Condigi" },
    { "18E29F", "vivo Mobile Communication Co., Ltd." },
    { "18E2C2", "Samsung Electronics Co.,Ltd" },
    { "18E3BC", "TCT mobile ltd" },
    { "18E671", "Apple, Inc." },
    { "18E728", "Cisco Systems, Inc" },
    { "18E777", "vivo Mobile Communication Co., Ltd." },
    { "18E7B0", "Apple, Inc." },
    { "18E7F4", "Apple, Inc." },
    { "18E80F", "Viking Electronics Inc." },
    { "18E829", "Ubiquiti Inc" },
    { "18E83B", "Citadel Wallet LLC" },
    { "18E8DD", "MODULETEK" },
    { "18E8EC", "STMicrolectronics International NV" },
    { "18E91D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "18EBD4", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "18ECE7", "BUFFALO.INC" },
    { "18EE69", "Apple, Inc." },
    { "18EE86", "Inseego Wireless, Inc" },
    { "18EF3A", "Sichuan AI-Link Technology Co., Ltd." },
    { "18EF63", "Cisco Systems, Inc" },
    { "18EFC0", "Sercomm Japan Corporation" },
    { "18F0E4", "Xiaomi Communications Co Ltd" },
    { "18F145", "NetComm Wireless Limited" },
    { "18F18E", "ChipER Technology co. ltd" },
    { "18F1D8", "Apple, Inc." },
    { "18F22C", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "18F292", "Shannon Systems" },
    { "18F46A", "Hon Hai Precision Ind. Co.,Ltd." },
    { "18F46B", "Telenor Connexion AB" },
    { "18F643", "Apple, Inc." },
    { "18F650", "Multimedia Pacific Limited" },
    { "18F697", "Axiom Memory Solutions, Inc." },
    { "18F76B", "Zhejiang Winsight Technology CO.,LTD" },
    { "18F87A", "i3 International Inc." },
    { "18F87F", "Wha Yu Industrial Co., Ltd." },
    { "18F935", "Cisco Systems, Inc" },
    { "18F9C4", "BAE Systems" },
    { "18FA6F", "ISC applied systems corp" },
    { "18FAB7", "Apple, Inc." },
    { "18FB7B", "Dell Inc." },
    { "18FB8E", "VusionGroup" },
    { "18FC26", "Qorvo International Pte. Ltd." },
    { "18FC9F", "Changhe Electronics Co., Ltd." },
    { "18FD74", "Routerboard.com" },
    { "18FDCB", "IEEE Registration Authority" },
    { "18FE34", "Espressif Inc." },
    { "18FF0F", "Intel Corporate" },
    { "18FF2E", "Shenzhen Rui Ying Da Technology Co., Ltd" },
    { "1C0042", "NARI Technology Co., Ltd." },
    { "1C012D", "Ficer Technology" },
    { "1C0219", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "1C05B7", "Chongqing Trantor Technology Co., Ltd." },
    { "1C0656", "IDY Corporation" },
    { "1C08C1", "LG Innotek" },
    { "1C0B52", "EPICOM S.A" },
    { "1C0B8B", "Ubiquiti Inc" },
    { "1C0D7D", "Apple, Inc." },
    { "1C0EAF", "Huawei Device Co., Ltd." },
    { "1C0EC2", "Apple, Inc." },
    { "1C0ED3", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "1C0FAF", "Lucid Vision Labs" },
    { "1C0FCF", "Sypro Optics GmbH" },
    { "1C112F", "vivo Mobile Communication Co., Ltd." },
    { "1C1161", "Ciena Corporation" },
    { "1C11E1", "Wartsila Finland Oy" },
    { "1C129D", "IEEE PES PSRC/SUB " },
    { "1C12B0", "Amazon Technologies Inc." },
    { "1C1338", "Kimball Electronics Group, LLC" },
    { "1C1386", "Huawei Device Co., Ltd." },
    { "1C13FA", "Huawei Device Co., Ltd." },
    { "1C1448", "Commscope" },
    { "1C14B3", "Airwire Technologies" },
    { "1C151F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1C17D3", "Cisco Systems, Inc" },
    { "1C184A", "ShenZhen RicherLink Technologies Co.,LTD" },
    { "1C19DE", "eyevis GmbH" },
    { "1C1A1B", "Shanghai Sunmi Technology Co.,Ltd." },
    { "1C1AC0", "Apple, Inc." },
    { "1C1ADF", "Microsoft Corporation" },
    { "1C1B0D", "GIGA-BYTE TECHNOLOGY CO.,LTD." },
    { "1C1B68", "Commscope" },
    { "1C1BB5", "Intel Corporate" },
    { "1C1CFD", "Dalian Hi-Think Computer Technology, Corp" },
    { "1C1D67", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1C1D86", "Cisco Systems, Inc" },
    { "1C1DD3", "Apple, Inc." },
    { "1C1E38", "PCCW Global, Inc." },
    { "1C1EE3", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "1C1FD4", "LifeBEAM Technologies LTD" },
    { "1C1FF1", "Huawei Device Co., Ltd." },
    { "1C20DB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1C2156", "Smappee NV" },
    { "1C21D1", "IEEE Registration Authority" },
    { "1C2285", "Serrature Meroni SpA" },
    { "1C232C", "Samsung Electronics Co.,Ltd" },
    { "1C234F", "EDMI Europe Ltd" },
    { "1C24CD", "ASKEY COMPUTER CORP" },
    { "1C24EB", "Burlywood" },
    { "1C25E1", "China Mobile IOT Company Limited" },
    { "1C2704", "zte corporation" },
    { "1C27DD", "Datang Gohighsec(zhejiang)Information Technology Co.,Ltd." },
    { "1C28AF", "Hewlett Packard Enterprise" },
    { "1C2A8B", "Nokia" },
    { "1C2AA3", "Shenzhen HongRui Optical Technology Co., Ltd." },
    { "1C2AB0", "Beijing Xiaomi Electronics Co.,Ltd" },
    { "1C2CE0", "Shanghai Mountain View Silicon" },
    { "1C2E1B", "Suzhou Tremenet Communication Technology Co., Ltd." },
    { "1C2FA2", "Guangzhou Shiyuan Electronic Technology Company Limited" },
    { "1C3003", "Hewlett Packard Enterprise" },
    { "1C3008", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "1C3283", "COMTTI Intelligent Technology(Shenzhen) Co., Ltd." },
    { "1C32AC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1C330E", "PernixData" },
    { "1C334D", "ITS Telecom" },
    { "1C3477", "Innovation Wireless" },
    { "1C34DA", "Mellanox Technologies, Inc." },
    { "1C34F1", "Silicon Laboratories" },
    { "1C35F1", "NEW Lift Neue Elektronische Wege Steuerungsbau GmbH" },
    { "1C36BB", "Apple, Inc." },
    { "1C37BF", "Cloudium Systems Ltd." },
    { "1C3929", "OHSUNG" },
    { "1C3947", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "1C398A", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "1C3A4F", "AccuSpec Electronics, LLC" },
    { "1C3A60", "Ruckus Wireless" },
    { "1C3ADE", "Samsung Electronics Co.,Ltd" },
    { "1C3B01", "Shanghai Xiaodu Technology Limited" },
    { "1C3B62", "HMD Global Oy" },
    { "1C3B8F", "Selve GmbH & Co. KG" },
    { "1C3BF3", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "1C3C78", "Apple, Inc." },
    { "1C3CD4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1C3D2F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1C3DE7", "Sigma Koki Co.,Ltd." },
    { "1C3E84", "Hon Hai Precision Ind. Co.,Ltd." },
    { "1C4024", "Dell Inc." },
    { "1C40E8", "SHENZHEN PROGRESS&WIN TECHNOLOGY CO.,LTD" },
    { "1C4158", "Gemalto M2M GmbH" },
    { "1C4176", "China Mobile Group Device Co.,Ltd." },
    { "1C4190", "Universal Electronics, Inc." },
    { "1C427D", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "1C4363", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1C43EC", "JAPAN CIRCUIT CO.,LTD" },
    { "1C4419", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "1C4455", "Sieb & Meyer AG" },
    { "1C4586", "Nintendo Co.,Ltd" },
    { "1C4593", "Texas Instruments" },
    { "1C45C2", "Huizhou City Sunsin lntelligent Technology Co.,Ltd" },
    { "1C46D1", "SKY UK LIMITED" },
    { "1C472F", "Huawei Device Co., Ltd." },
    { "1C47F6", "Zhidao Network Technology(Shenzhen) Co.,Ltd" },
    { "1C4840", "IMS Messsysteme GmbH" },
    { "1C48CE", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "1C48F9", "GN Netcom A/S" },
    { "1C497B", "Gemtek Technology Co., Ltd." },
    { "1C4AF7", "AMON INC" },
    { "1C4BB9", "SMG ENTERPRISE, LLC" },
    { "1C4BD6", "AzureWave Technology Inc." },
    { "1C4C27", "World WLAN Application Alliance" },
    { "1C4C48", "ITEL MOBILE LIMITED" },
    { "1C4D66", "Amazon Technologies Inc." },
    { "1C4D70", "Intel Corporate" },
    { "1C4D89", "Hangzhou Huacheng Network Technology Co.,Ltd" },
    { "1C4EA2", "Shenzhen V-Link Technology CO., LTD." },
    { "1C501E", "Sunplus Technology Co., Ltd." },
    { "1C51B5", "Techaya LTD" },
    { "1C5216", "DONGGUAN HELE ELECTRONICS CO., LTD" },
    { "1C52A7", "Coram AI, Inc" },
    { "1C52D6", "FLAT DISPLAY TECHNOLOGY CORPORATION" },
    { "1C53F9", "Google, Inc." },
    { "1C549E", "Universal Electronics, Inc." },
    { "1C54E6", "Shenzhen Yisheng Technology Co.,Ltd" },
    { "1C553A", "QianGua Corp." },
    { "1C568E", "Zioncom Electronics (Shenzhen) Ltd." },
    { "1C56FE", "Motorola Mobility LLC, a Lenovo Company" },
    { "1C573E", "Altice Labs" },
    { "1C57D8", "Kraftway Corporation PLC" },
    { "1C57DC", "Apple, Inc." },
    { "1C5974", "IEEE Registration Authority" },
    { "1C599B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1C5A0B", "Tegile Systems" },
    { "1C5A3E", "Samsung Electronics Co.,Ltd" },
    { "1C5A6B", "Philips Electronics Nederland BV" },
    { "1C5C55", "PRIMA Cinema, Inc" },
    { "1C5C60", "Shenzhen Belzon Technology Co.,LTD." },
    { "1C5CF2", "Apple, Inc." },
    { "1C5D80", "Mitubishi Hitachi Power Systems Industries Co., Ltd." },
    { "1C5EE6", "SHENZHEN TWOWING TECHNOLOGIES CO.,LTD." },
    { "1C5F2B", "D-Link International" },
    { "1C5FFF", "Beijing Ereneben Information Technology Co.,Ltd Shenzhen Branch" },
    { "1C6066", "TEJAS NETWORKS LTD" },
    { "1C60D2", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "1C60DE", "MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "1C61B4", "TP-Link Systems Inc" },
    { "1C61BF", "Apple, Inc." },
    { "1C627E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1C62B8", "Samsung Electronics Co.,Ltd" },
    { "1C6349", "Texas Instruments" },
    { "1C63A5", "securityplatform" },
    { "1C63B7", "OpenProducts 237 AB" },
    { "1C63BF", "SHENZHEN BROADTEL TELECOM CO.,LTD" },
    { "1C6499", "Comtrend Corporation" },
    { "1C64F0", "Motorola Mobility LLC, a Lenovo Company" },
    { "1C659D", "Liteon Technology Corporation" },
    { "1C666D", "Hon Hai Precision Ind. Co.,Ltd." },
    { "1C66AA", "Samsung Electronics Co.,Ltd" },
    { "1C674A", "zte corporation" },
    { "1C6758", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1C6760", "Phonesuite" },
    { "1C687E", "Shenzhen Qihu Intelligent Technology Company Limited" },
    { "1C6920", "Espressif Inc." },
    { "1C697A", "EliteGroup Computer Systems Co., LTD" },
    { "1C69A5", "BlackBerry RTS" },
    { "1C6A1B", "Ubiquiti Inc" },
    { "1C6A76", "Apple, Inc." },
    { "1C6A7A", "Cisco Systems, Inc" },
    { "1C6BCA", "Mitsunami Co., Ltd." },
    { "1C6E4C", "Logistic Service & Engineering Co.,Ltd" },
    { "1C6E74", "EnOcean Edge Inc." },
    { "1C6E76", "Quarion Technology Inc" },
    { "1C6EE6", "NHNETWORKS" },
    { "1C6F65", "GIGA-BYTE TECHNOLOGY CO.,LTD." },
    { "1C7022", "Murata Manufacturing Co., Ltd." },
    { "1C70C9", "Jiangsu Aisida Electronic Co., Ltd" },
    { "1C7125", "Apple, Inc." },
    { "1C7126", "snom technology GmbH" },
    { "1C721D", "Dell Inc." },
    { "1C7328", "Connected Home" },
    { "1C7370", "Neotech" },
    { "1C73E2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1C740D", "Zyxel Communications Corporation" },
    { "1C7508", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "1C76CA", "Terasic Technologies Inc." },
    { "1C76F2", "Samsung Electronics Co.,Ltd" },
    { "1C7754", "Apple, Inc." },
    { "1C77F6", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "1C7839", "Shenzhen Tencent Computer System Co., Ltd." },
    { "1C784B", "Bouffalo Lab (Nanjing) Co., Ltd." },
    { "1C784E", "China Mobile Iot Limited company" },
    { "1C792D", "CHINA DRAGON TECHNOLOGY LIMITED" },
    { "1C7A43", "vivo Mobile Communication Co., Ltd." },
    { "1C7ACF", "vivo Mobile Communication Co., Ltd." },
    { "1C7B21", "Sony Corporation" },
    { "1C7B23", "Qingdao Hisense Communications Co.,Ltd." },
    { "1C7C11", "EID " },
    { "1C7C45", "Vitek Industrial Video Products, Inc." },
    { "1C7C98", "NEC Platforms, Ltd." },
    { "1C7CC7", "Coriant GmbH" },
    { "1C7D22", "FUJIFILM Business Innovation Corp." },
    { "1C7E51", "3bumen.com" },
    { "1C7EE5", "D-Link International" },
    { "1C7F2C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1C8259", "IEEE Registration Authority" },
    { "1C8341", "Hefei Bitland Information Technology Co.Ltd" },
    { "1C83B0", "Linked IP GmbH" },
    { "1C83EC", "Ubee Interactive co, Limited." },
    { "1C8464", "FORMOSA WIRELESS COMMUNICATION CORP." },
    { "1C860B", "Guangdong Taiying Technology Co.,Ltd" },
    { "1C8682", "Apple, Inc." },
    { "1C869A", "Samsung Electronics Co.,Ltd" },
    { "1C86AD", "MCT CO., LTD." },
    { "1C872C", "ASUSTek COMPUTER INC." },
    { "1C87E3", "TECNO MOBILE LIMITED" },
    { "1C880C", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "1C8B76", "Calix Inc." },
    { "1C8BEF", "Beijing Xiaomi Electronics Co.,Ltd" },
    { "1C8E5C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1C8E8E", "DB Communication & Systems Co., ltd." },
    { "1C8F8A", "Phase Motion Control SpA" },
    { "1C90BE", "Ericsson AB" },
    { "1C90FF", "Tuya Smart Inc." },
    { "1C9148", "Apple, Inc." },
    { "1C9179", "Integrated System Technologies Ltd" },
    { "1C9180", "Apple, Inc." },
    { "1C919D", "Dongguan Liesheng Electronic Co., Ltd." },
    { "1C937C", "Commscope" },
    { "1C93C4", "Amazon Technologies Inc." },
    { "1C9468", "New H3C Technologies Co., Ltd" },
    { "1C9492", "RUAG Schweiz AG" },
    { "1C955D", "I-LAX ELECTRONICS INC." },
    { "1C959F", "Veethree Electronics And Marine LLC" },
    { "1C965A", "WEIFANG GOERTEK ELECTRONICS CO.,LTD" },
    { "1C973D", "PRICOM Design" },
    { "1C97C5", "Ynomia Pty Ltd" },
    { "1C97FB", "CoolBitX Ltd." },
    { "1C98C1", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "1C98EC", "Hewlett Packard Enterprise " },
    { "1C994C", "Murata Manufacturing Co., Ltd." },
    { "1C9957", "Intel Corporate" },
    { "1C99DB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1C9C26", "Zoovel Technologies" },
    { "1C9C8C", "Juniper Networks" },
    { "1C9D3E", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "1C9D72", "Vantiva USA LLC" },
    { "1C9DC2", "Espressif Inc." },
    { "1C9E46", "Apple, Inc." },
    { "1C9ECB", "Beijing Nari Smartchip Microelectronics Company Limited" },
    { "1C9ECC", "Vantiva USA LLC" },
    { "1C9F4E", "COOSEA GROUP (HK) COMPANY LIMITED" },
    { "1CA0B8", "Hon Hai Precision Industry Co., Ltd." },
    { "1CA0D3", "IEEE Registration Authority" },
    { "1CA0EF", "IEEE Registration Authority" },
    { "1CA2B1", "ruwido austria gmbh" },
    { "1CA410", "Amlogic, Inc." },
    { "1CA532", "SHENZHEN GONGJIN ELECTRONICS CO.,LT" },
    { "1CA681", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1CA770", "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD" },
    { "1CA852", "SENSAIO PTE LTD" },
    { "1CAA07", "Cisco Systems, Inc" },
    { "1CAB01", "Innovolt" },
    { "1CAB34", "New H3C Technologies Co., Ltd" },
    { "1CAB48", "TECNO MOBILE LIMITED" },
    { "1CABA7", "Apple, Inc." },
    { "1CABC0", "Hitron Technologies. Inc" },
    { "1CADD1", "Bosung Electronics Co., Ltd." },
    { "1CAE3E", "IEEE Registration Authority" },
    { "1CAECB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1CAF05", "Samsung Electronics Co.,Ltd" },
    { "1CAF4A", "Samsung Electronics Co.,Ltd" },
    { "1CAFF7", "D-Link International" },
    { "1CB044", "ASKEY COMPUTER CORP" },
    { "1CB094", "HTC Corporation" },
    { "1CB17F", "NEC Platforms, Ltd." },
    { "1CB243", "TDC A/S" },
    { "1CB3C9", "Apple, Inc." },
    { "1CB3E9", "Shenzhen Zhongke United Communication Technology " },
    { "1CB46C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1CB72C", "ASUSTek COMPUTER INC." },
    { "1CB796", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1CB857", "Becon Technologies Co,.Ltd." },
    { "1CB8BA", "XIAMEN LEELEN TECHNOLOGY CO., LTD" },
    { "1CB9C4", "Ruckus Wireless" },
    { "1CBA8C", "Texas Instruments" },
    { "1CBBA8", "OJSC Ufimskiy Zavod Promsvyaz" },
    { "1CBCEC", "silex technology, Inc." },
    { "1CBD0E", "Amplified Engineering Pty Ltd" },
    { "1CBDB9", "D-Link International" },
    { "1CBFC0", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "1CBFCE", "Shenzhen Century Xinyang Technology Co., Ltd" },
    { "1CC035", "PLANEX COMMUNICATIONS INC." },
    { "1CC089", "Silicon Laboratories" },
    { "1CC0E1", "IEEE Registration Authority" },
    { "1CC10C", "Intel Corporate" },
    { "1CC11A", "Wavetronix" },
    { "1CC1BC", "Yichip Microelectronics (Hangzhou) Co.,Ltd" },
    { "1CC1DE", "Hewlett Packard" },
    { "1CC316", "Xiamen Milesight IoT Co., Ltd." },
    { "1CC3EB", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "1CC586", "Absolute Acoustics" },
    { "1CC63C", "Arcadyan Technology Corporation" },
    { "1CC72D", "Shenzhen Huapu Digital CO.,Ltd" },
    { "1CC8C1", "HongKong YiTong Technology Ltd." },
    { "1CC992", "Honor Device Co., Ltd." },
    { "1CCA41", "AO " },
    { "1CCAE3", "IEEE Registration Authority" },
    { "1CCB99", "TCT mobile ltd" },
    { "1CCCD6", "Xiaomi Communications Co Ltd" },
    { "1CCDE5", "Shanghai Wind Technologies Co.,Ltd" },
    { "1CCE51", "AzureWave Technology Inc." },
    { "1CD107", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "1CD1BA", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "1CD1D7", "Hangzhou BroadLink Technology Co., Ltd" },
    { "1CD1E0", "Cisco Systems, Inc" },
    { "1CD3AF", "LG Innotek" },
    { "1CD40C", "Kriwan Industrie-Elektronik GmbH" },
    { "1CD5E2", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "1CD6BD", "LEEDARSON LIGHTING CO., LTD." },
    { "1CD6BE", "Wistron Neweb Corporation" },
    { "1CDA27", "vivo Mobile Communication Co., Ltd." },
    { "1CDBD4", "Espressif Inc." },
    { "1CDDEA", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "1CDE57", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "1CDEA7", "Cisco Systems, Inc" },
    { "1CDF0F", "Cisco Systems, Inc" },
    { "1CDF52", "Texas Instruments" },
    { "1CE165", "Marshal Corporation" },
    { "1CE192", "Qisda Corporation" },
    { "1CE209", "Apple, Inc." },
    { "1CE2CC", "Texas Instruments" },
    { "1CE504", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1CE57F", "Samsung Electronics Co.,Ltd" },
    { "1CE61D", "Samsung Electronics Co.,Ltd" },
    { "1CE62B", "Apple, Inc." },
    { "1CE639", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1CE6AD", "Huawei Device Co., Ltd." },
    { "1CE6C7", "Cisco Systems, Inc" },
    { "1CE85D", "Cisco Systems, Inc" },
    { "1CE89E", "SJIT" },
    { "1CEA0B", "Edgecore Networks Corporation" },
    { "1CEA1B", "Nokia" },
    { "1CEAAC", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "1CEC72", "Allradio Co., Ltd" },
    { "1CED6F", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "1CEEC9", "Elo touch solutions" },
    { "1CEEE8", "Ilshin Elecom" },
    { "1CEF03", "Guangzhou V-SOLUTION Electronic Technology Co., Ltd." },
    { "1CEFCE", "bebro electronic GmbH" },
    { "1CF03E", "Wearhaus Inc." },
    { "1CF061", "SCAPS GmbH" },
    { "1CF29A", "Google, Inc." },
    { "1CF42B", "Huawei Device Co., Ltd." },
    { "1CF43F", "Arcadyan Corporation" },
    { "1CF4CA", "Private" },
    { "1CF5E7", "Turtle Industry Co., Ltd." },
    { "1CF64C", "Apple, Inc." },
    { "1CF8D0", "Samsung Electronics Co.,Ltd" },
    { "1CF9D5", "Apple, Inc." },
    { "1CFA68", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "1CFC17", "Cisco Systems, Inc" },
    { "1CFC2A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "1CFCBB", "Realfiction ApS" },
    { "1CFD08", "IEEE Registration Authority" },
    { "1CFE2B", "Amazon Technologies Inc." },
    { "1CFEA7", "IDentytech Solutins Ltd." },
    { "1CFF59", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "1CFFAD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "20014F", "Linea Research Ltd" },
    { "20019C", "Bigleaf Networks Inc." },
    { "2002AF", "Murata Manufacturing Co., Ltd." },
    { "2002C9", "Zhejiang Huayi IOT Technology Co.,Ltd" },
    { "2002FE", "Hangzhou Dangbei Network Technology Co., Ltd" },
    { "20040F", "Dell Inc." },
    { "200484", "Apple, Inc." },
    { "2004F3", "Honor Device Co., Ltd." },
    { "200505", "RADMAX COMMUNICATION PRIVATE LIMITED" },
    { "2005B6", "OpenWrt" },
    { "2005E8", "OOO InProMedia" },
    { "200889", "zte corporation" },
    { "2008ED", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "200A0D", "IEEE Registration Authority" },
    { "200A5E", "Xiangshan Giant Eagle Technology Developing Co., Ltd." },
    { "200A87", "Guangzhou On-Bright Electronics Co., Ltd." },
    { "200B16", "Texas Instruments" },
    { "200B74", "AzureWave Technology Inc." },
    { "200BC5", "Cisco Systems, Inc" },
    { "200BC7", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "200BCF", "Nintendo Co.,Ltd" },
    { "200C86", "GX India Pvt Ltd" },
    { "200CC8", "NETGEAR" },
    { "200DB0", "Shenzhen Four Seas Global Link Network Technology Co., Ltd." },
    { "200E0F", "Panasonic Marketing Middle East & Africa FZE" },
    { "200E2B", "Apple, Inc." },
    { "200E95", "IEC – TC9 WG43" },
    { "200F70", "FOXTECH" },
    { "200F92", "STK Technology Co., Ltd." },
    { "20107A", "Gemtek Technology Co., Ltd." },
    { "20108A", "zte corporation" },
    { "2010B1", "Amazon Technologies Inc." },
    { "20114E", "MeteRSit S.R.L." },
    { "201257", "Most Lucky Trading Ltd" },
    { "2012D5", "Scientech Materials Corporation" },
    { "2013E0", "Samsung Electronics Co.,Ltd" },
    { "2014C4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "201582", "Apple, Inc." },
    { "2015DE", "Samsung Electronics Co.,Ltd" },
    { "20163D", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "201642", "Microsoft Corporation" },
    { "2016B9", "Intel Corporate" },
    { "2016D8", "Liteon Technology Corporation" },
    { "201742", "LG Electronics" },
    { "201746", "Paradromics, Inc." },
    { "20180E", "Shenzhen Sunchip Technology Co., Ltd" },
    { "20185B", "Shenzhen Jingxun Technology Co., Ltd." },
    { "2019F3", "WavTek Technologies, Inc" },
    { "201A06", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "201A94", "Apple, Inc." },
    { "201B88", "Dongguan Liesheng Electronic Co., Ltd." },
    { "201BA5", "Vizio, Inc" },
    { "201BC9", "Juniper Networks" },
    { "201C3A", "Nintendo Co.,Ltd" },
    { "201D03", "Elatec GmbH" },
    { "201E1D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "201E88", "Intel Corporate" },
    { "201F31", "Inteno Broadband Technology AB" },
    { "201F3B", "Google, Inc." },
    { "201F54", "Raisecom Technology CO., LTD" },
    { "202027", "Shenzhen Sundray Technologies company Limited" },
    { "202051", "zte corporation" },
    { "202141", "Universal Electronics BV" },
    { "2021A5", "LG Electronics (Mobile Communications)" },
    { "202351", "TP-Link Systems Inc" },
    { "202564", "PEGATRON CORPORATION" },
    { "202598", "Teleview" },
    { "2025CC", "Xiaomi Communications Co Ltd" },
    { "2025D2", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "202681", "TECNO MOBILE LIMITED" },
    { "20283E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2028BC", "Visionscape Co,. Ltd." },
    { "2029B9", "Ikotek technology SH Co., Ltd" },
    { "202AC5", "Petite-En" },
    { "202B20", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "202BC1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "202CB7", "Kong Yue Electronics & Information Industry (Xinhui) Ltd." },
    { "202D07", "Samsung Electronics Co.,Ltd" },
    { "202D23", "Collinear Networks Inc." },
    { "202DF6", "Apple, Inc." },
    { "202DF8", "Digital Media Cartridge Ltd." },
    { "20311C", "vivo Mobile Communication Co., Ltd." },
    { "20318D", "Giax GmbH" },
    { "2031EB", "HDSN" },
    { "203233", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "20326C", "Samsung Electronics Co.,Ltd" },
    { "2032C6", "Apple, Inc." },
    { "203462", "Xiaomi Communications Co Ltd" },
    { "2034FB", "Xiaomi Communications Co Ltd" },
    { "203543", "Sagemcom Broadband SAS" },
    { "203626", "TP-Link Systems Inc" },
    { "20365B", "Megafone Limited" },
    { "2036D7", "Shanghai Reacheng Communication Technology Co.,Ltd" },
    { "203706", "Cisco Systems, Inc" },
    { "2037A5", "Apple, Inc." },
    { "2037BC", "Kuipers Electronic Engineering BV" },
    { "2037F0", "Arcadyan Corporation" },
    { "203956", "HMD Global Oy" },
    { "203A07", "Cisco Systems, Inc" },
    { "203A0C", "eero inc." },
    { "203A43", "Intel Corporate" },
    { "203AEB", "zte corporation" },
    { "203AEF", "Sivantos GmbH" },
    { "203B34", "Xiaomi Communications Co Ltd" },
    { "203B67", "Samsung Electronics Co.,Ltd" },
    { "203B69", "vivo Mobile Communication Co., Ltd." },
    { "203CAE", "Apple, Inc." },
    { "203CC0", "Beijing Tosee Technology Co., Ltd." },
    { "203D66", "Commscope" },
    { "203DB2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "203DBD", "LG Innotek" },
    { "204005", "feno GmbH" },
    { "20406A", "AMPAK Technology,Inc." },
    { "20415A", "Smarteh d.o.o." },
    { "204181", "ESYSE GmbH Embedded Systems Engineering" },
    { "2043A8", "Espressif Inc." },
    { "20443A", "Schneider Electric Asia Pacific Ltd" },
    { "204441", "Remote Solution" },
    { "204569", "ITEL MOBILE LIMITED" },
    { "2046A1", "VECOW Co., Ltd" },
    { "2046F9", "Advanced Network Devices (dba:AND)" },
    { "204747", "Dell Inc." },
    { "2047B5", "Sagemcom Broadband SAS" },
    { "2047DA", "Xiaomi Communications Co Ltd" },
    { "2047ED", "SKY UK LIMITED" },
    { "204AAA", "Hanscan Spain S.A." },
    { "204B22", "Sunnovo International Limited" },
    { "204C03", "Hewlett Packard Enterprise" },
    { "204C6D", "Hugo Brennenstuhl Gmbh & Co. KG." },
    { "204C9E", "Cisco Systems, Inc" },
    { "204D52", "Mellanox Technologies, Inc." },
    { "204E6B", "Axxana(israel) ltd" },
    { "204E71", "Juniper Networks" },
    { "204E7F", "NETGEAR" },
    { "204EF6", "AzureWave Technology Inc." },
    { "20500F", "Fiber Groep B.V." },
    { "2050E7", "AMPAK Technology,Inc." },
    { "2051F5", "Earda Technologies co Ltd" },
    { "205383", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "20538D", "Hon Hai Precision Industry Co., Ltd." },
    { "2053CA", "Risk Technology Ltd" },
    { "205476", "Sony Corporation" },
    { "2054FA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "205531", "Samsung Electronics Co.,Ltd" },
    { "205532", "Gotech International Technology Limited" },
    { "205721", "Salix Technology CO., Ltd." },
    { "20579E", "HUNAN FN-LINK TECHNOLOGY LIMITED" },
    { "2057AF", "Shenzhen FH-NET OPTOELECTRONICS CO.,LTD" },
    { "205869", "Ruckus Wireless" },
    { "2059A0", "Paragon Technologies Inc." },
    { "205A00", "Coval" },
    { "205A1D", "zte corporation" },
    { "205A8F", "Shenzhen Hikeen Technology Co.,LTD" },
    { "205B2A", "Private" },
    { "205B5E", "Shenzhen Wonhe Technology Co., Ltd" },
    { "205CFA", "Yangzhou ChangLian Network Technology Co,ltd." },
    { "205D0D", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "205D47", "vivo Mobile Communication Co., Ltd." },
    { "205E64", "Huawei Device Co., Ltd." },
    { "205E97", "Nokia" },
    { "205EF7", "Samsung Electronics Co.,Ltd" },
    { "205F3D", "Adtran Inc" },
    { "206274", "Microsoft Corporation" },
    { "206296", "Shenzhen Malio Technology Co.,Ltd" },
    { "20635F", "Abeeway" },
    { "206432", "SAMSUNG ELECTRO MECHANICS CO., LTD." },
    { "2064CB", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "2064DE", "Sunitec Enterprise Co.,Ltd" },
    { "20658E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2066CF", "FREEBOX SAS" },
    { "2066FD", "CONSTELL8 NV" },
    { "20677C", "Hewlett Packard Enterprise " },
    { "2067B1", "Pluto inc." },
    { "2067E0", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "20689D", "Liteon Technology Corporation" },
    { "206980", "Apple, Inc." },
    { "206A8A", "Wistron Infocomm (Zhongshan) Corporation" },
    { "206A94", "Hitron Technologies. Inc" },
    { "206AFF", "Atlas Elektronik UK Limited" },
    { "206BD5", "vivo Mobile Communication Co., Ltd." },
    { "206BE7", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "206BF4", "Huawei Device Co., Ltd." },
    { "206C8A", "Extreme Networks Headquarters" },
    { "206D31", "FIREWALLA INC" },
    { "206E9C", "Samsung Electronics Co.,Ltd" },
    { "206EF1", "Espressif Inc." },
    { "206FEC", "Braemac CA LLC" },
    { "20719E", "SF Technology Co.,Ltd" },
    { "2072A9", "Beijing Xiaomi Electronics Co.,Ltd" },
    { "207355", "Commscope" },
    { "207454", "vivo Mobile Communication Co., Ltd." },
    { "2074CF", "Shenzhen Voxtech Co.,Ltd" },
    { "207600", "Actiontec Electronics, Inc" },
    { "20768F", "Apple, Inc." },
    { "207693", "Lenovo (Beijing) Limited." },
    { "207759", "OPTICAL NETWORK VIDEO TECHNOLOGIES (SHENZHEN) CO., LTD." },
    { "20780B", "Delta Faucet Company" },
    { "207852", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "2078CD", "Apple, Inc." },
    { "2078F0", "Apple, Inc." },
    { "207918", "Intel Corporate" },
    { "207BD2", "ASIX Electronics Corporation" },
    { "207C14", "Qotom" },
    { "207C8F", "Quanta Microsystems,Inc." },
    { "207D74", "Apple, Inc." },
    { "208058", "Ciena Corporation" },
    { "208097", "Shenzhen OXO Technology limited" },
    { "20826A", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "2082C0", "Xiaomi Communications Co Ltd" },
    { "2083F8", "Advanced Digital Broadcast SA" },
    { "2084F5", "Yufei Innovation Software(Shenzhen) Co., Ltd." },
    { "20858C", "Assa" },
    { "208593", "IEEE Registration Authority" },
    { "208756", "SIEMENS AG" },
    { "2087AC", "AES motomation" },
    { "2087EC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "208810", "Dell Inc." },
    { "20896F", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "208984", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "208986", "zte corporation" },
    { "20898A", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "208B37", "Skyworth Digital Technology(Shenzhen) Co.,Ltd" },
    { "208BD1", "NXP Semiconductor (Tianjin) LTD." },
    { "208C47", "Tenstorrent Inc" },
    { "208C86", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "20906F", "Shenzhen Tencent Computer System Co., Ltd." },
    { "209148", "Texas Instruments" },
    { "20918A", "PROFALUX" },
    { "2091D9", "I'M SPA" },
    { "2091DF", "Apple, Inc." },
    { "209339", "Juniper Networks" },
    { "20934D", "FUJIAN STAR-NET COMMUNICATION CO.,LTD" },
    { "20968A", "China Mobile (Hangzhou) Information Technology Co., Ltd." },
    { "209727", "TELTONIKA NETWORKS UAB" },
    { "2098D8", "Shenzhen Yingdakang Technology CO., LTD" },
    { "2098ED", "AltoBeam Inc." },
    { "209952", "Xiaomi Communications Co Ltd" },
    { "209A7D", "Sagemcom Broadband SAS" },
    { "209AE9", "Volacomm Co., Ltd" },
    { "209BA5", "JIAXING GLEAD Electronics Co.,Ltd" },
    { "209BCD", "Apple, Inc." },
    { "209BE6", "Guangzhou Shiyuan Electronic Technology Company Limited" },
    { "209CB4", "Hewlett Packard Enterprise" },
    { "209E79", "Universal Electronics, Inc." },
    { "209EF7", "Extreme Networks Headquarters" },
    { "20A171", "Amazon Technologies Inc." },
    { "20A200", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "20A2E4", "Apple, Inc." },
    { "20A2E7", "Lee-Dickens Ltd" },
    { "20A5CB", "Apple, Inc." },
    { "20A60C", "Xiaomi Communications Co Ltd" },
    { "20A680", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "20A6CD", "Hewlett Packard Enterprise " },
    { "20A716", "Silicon Laboratories" },
    { "20A766", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "20A783", "miControl GmbH" },
    { "20A787", "Bointec Taiwan Corporation Limited" },
    { "20A7F9", "SHENZHEN OLANBOA TECHNOLOGY CO., LTD" },
    { "20A8B9", "SIEMENS AG" },
    { "20A8BF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "20A90E", "TCT mobile ltd" },
    { "20A99B", "Microsoft Corporation" },
    { "20AA25", "IP-NET LLC" },
    { "20AA4B", "Cisco-Linksys, LLC" },
    { "20AB37", "Apple, Inc." },
    { "20AB48", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "20AC9C", "China Telecom Corporation Limited" },
    { "20AD56", "Continental Automotive Systems Inc." },
    { "20AF1B", "SteelSeries ApS" },
    { "20B001", "Technicolor Delivery Technologies Belgium NV" },
    { "20B0F7", "Enclustra GmbH" },
    { "20B399", "Enterasys" },
    { "20B5C6", "Mimosa Networks" },
    { "20B730", "TeconGroup, Inc" },
    { "20B780", "Toshiba Visual Solutions Corporation Co.,Ltd" },
    { "20B7C0", "OMICRON electronics GmbH" },
    { "20B82B", "Sagemcom Broadband SAS" },
    { "20B868", "Motorola Mobility LLC, a Lenovo Company" },
    { "20BA36", "u-blox AG" },
    { "20BB76", "COL GIOVANNI PAOLO SpA" },
    { "20BBBC", "Hangzhou Ezviz Software Co.,Ltd." },
    { "20BBC0", "Cisco Systems, Inc" },
    { "20BBC6", "Jabil Circuit Hungary Ltd." },
    { "20BD1D", "Intel Corporate" },
    { "20BEB4", "Hisense broadband multimedia technology Co.,Ltd" },
    { "20BEB8", "Amazon Technologies Inc." },
    { "20BECD", "eero inc." },
    { "20BFDB", "DVL" },
    { "20C047", "Verizon " },
    { "20C06D", "SHENZHEN SPACETEK TECHNOLOGY CO.,LTD" },
    { "20C19B", "Intel Corporate" },
    { "20C1AF", "i Wit Digital Co., Limited" },
    { "20C2B0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "20C38F", "Texas Instruments" },
    { "20C3A4", "RetailNext" },
    { "20C60D", "Shanghai annijie Information technology Co.,LTD" },
    { "20C6EB", "Panasonic Corporation AVC Networks Company" },
    { "20C74F", "SensorPush" },
    { "20C792", "Wuhan Maiwe communication Co.,Ltd" },
    { "20C8B3", "SHENZHEN BUL-TECH CO.,LTD." },
    { "20C9D0", "Apple, Inc." },
    { "20CC27", "Cisco Systems, Inc" },
    { "20CD39", "Texas Instruments" },
    { "20CD6E", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "20CE2A", "IEEE Registration Authority" },
    { "20CEC4", "Peraso Technologies" },
    { "20CF30", "ASUSTek COMPUTER INC." },
    { "20CFAE", "Cisco Systems, Inc" },
    { "20D160", "Private" },
    { "20D21F", "Wincal Technology Corp." },
    { "20D25F", "SmartCap Technologies" },
    { "20D276", "ITEL MOBILE LIMITED" },
    { "20D390", "Samsung Electronics Co.,Ltd" },
    { "20D5AB", "Korea Infocom Co.,Ltd." },
    { "20D5BF", "Samsung Electronics Co.,Ltd" },
    { "20D607", "Nokia Corporation" },
    { "20D75A", "Posh Mobile Limited" },
    { "20D778", "Texas Instruments" },
    { "20D80B", "Juniper Networks" },
    { "20D906", "Iota, Inc." },
    { "20DA22", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "20DBAB", "Samsung Electronics Co., Ltd." },
    { "20DBEA", "Cisco Systems, Inc" },
    { "20DC93", "Cheetah Hi-Tech, Inc." },
    { "20DCE6", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "20DCFD", "Huawei Device Co., Ltd." },
    { "20DE1E", "Nokia" },
    { "20DE88", "IC Realtime LLC" },
    { "20DF3F", "Nanjing SAC Power Grid Automation Co., Ltd." },
    { "20DF73", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "20DFB9", "Google, Inc." },
    { "20E09C", "Nokia" },
    { "20E2A8", "Apple, Inc." },
    { "20E407", "Spark srl" },
    { "20E46F", "vivo Mobile Communication Co., Ltd." },
    { "20E52A", "NETGEAR" },
    { "20E564", "Commscope" },
    { "20E6DF", "eero inc." },
    { "20E791", "Siemens Healthcare Diagnostics, Inc" },
    { "20E7B6", "Universal Electronics, Inc." },
    { "20E7C8", "Espressif Inc." },
    { "20E874", "Apple, Inc." },
    { "20E882", "zte corporation" },
    { "20EAC7", "SHENZHEN RIOPINE ELECTRONICS CO., LTD" },
    { "20ED47", "Juniper Networks" },
    { "20ED74", "Ability enterprise co.,Ltd." },
    { "20EE28", "Apple, Inc." },
    { "20EEC6", "Elefirst Science & Tech Co ., ltd" },
    { "20EFBD", "Roku, Inc" },
    { "20F002", "MTData Developments Pty. Ltd." },
    { "20F094", "Google, Inc." },
    { "20F120", "Cisco Systems, Inc" },
    { "20F17C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "20F19E", "Commscope" },
    { "20F307", "zte corporation" },
    { "20F375", "Commscope" },
    { "20F3A3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "20F41B", "Shenzhen Bilian electronic CO.,LTD" },
    { "20F44F", "Nokia" },
    { "20F452", "Shanghai IUV Software Development Co. Ltd" },
    { "20F478", "Xiaomi Communications Co Ltd" },
    { "20F4D4", "Apple, Inc." },
    { "20F510", "Codex Digital Limited" },
    { "20F543", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "20F597", "Maasiv, LLC" },
    { "20F77C", "vivo Mobile Communication Co., Ltd." },
    { "20F83B", "Nabu Casa, Inc." },
    { "20F85E", "Delta Electronics" },
    { "20FA85", "Apple, Inc." },
    { "20FABB", "Cambridge Executive Limited" },
    { "20FADB", "Huahao Kunpeng Technology (chengDu) Co.,Ltd." },
    { "20FDF1", "3COM EUROPE LTD" },
    { "20FE00", "Amazon Technologies Inc." },
    { "20FECD", "System In Frontier Inc." },
    { "20FEDB", "M2M Solution S.A.S." },
    { "20FF36", "IFLYTEK CO.,LTD." },
    { "2400BA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2400FA", "China Mobile (Hangzhou) Information Technology Co., Ltd" },
    { "24016F", "Huawei Device Co., Ltd." },
    { "2401C7", "Cisco Systems, Inc" },
    { "240462", "Siemens Energy Global GmbH & Co.KG - GT PRM" },
    { "24050F", "MTN Electronic Co. Ltd" },
    { "240588", "Google, Inc." },
    { "2405F5", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "2406AA", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "2406F2", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "24085D", "Continental Aftermarket & Services GmbH" },
    { "240917", "Devlin Electronics Limited" },
    { "240935", "Samsung Electronics Co.,Ltd" },
    { "240995", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "240A11", "TCT mobile ltd" },
    { "240A3F", "Samsung Electronics Co.,Ltd" },
    { "240A63", "Commscope" },
    { "240A64", "AzureWave Technology Inc." },
    { "240AC4", "Espressif Inc." },
    { "240B0A", "Palo Alto Networks" },
    { "240B2A", "Viettel Group" },
    { "240B88", "Taicang T&W Electronics" },
    { "240BB1", "KOSTAL Industrie Elektrik GmbH " },
    { "240D65", "Shenzhen Vsun Communication Technology Co., Ltd." },
    { "240D6C", "SMND" },
    { "240DC2", "TCT mobile ltd" },
    { "240F5E", "Shenzhen z-router Technology Co., Ltd" },
    { "240F9B", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "241064", "Shenzhen Ecsino Tecnical Co. Ltd" },
    { "241125", "Hutek Co., Ltd." },
    { "241145", "Xiaomi Communications Co Ltd" },
    { "241148", "Entropix, LLC" },
    { "241153", "Samsung Electronics Co.,Ltd" },
    { "2411D0", "Chongqing Ehs Science and Technology Development Co.,Ltd." },
    { "241281", "China Mobile Group Device Co.,Ltd." },
    { "241407", "Xiamen Sigmastar Technology Ltd." },
    { "241510", "IEEE Registration Authority" },
    { "241551", "Huawei Device Co., Ltd." },
    { "24161B", "Cisco Systems, Inc" },
    { "241651", "Chipsea Technologies (Shenzhen) Corp." },
    { "24166D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "24169D", "Cisco Systems, Inc" },
    { "24181D", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "2418C0", "E. Wehrle GmbH" },
    { "2418C6", "HUNAN FN-LINK TECHNOLOGY LIMITED" },
    { "241972", "Quectel Wireless Solutions Co.,Ltd." },
    { "241A8C", "Squarehead Technology AS" },
    { "241AE6", "Huawei Device Co., Ltd." },
    { "241B13", "Shanghai Nutshell Electronic Co., Ltd." },
    { "241B44", "Hangzhou Tuners Electronics Co., Ltd" },
    { "241B7A", "Apple, Inc." },
    { "241C04", "SHENZHEN JEHE TECHNOLOGY DEVELOPMENT CO., LTD." },
    { "241D48", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "241E2B", "Zhejiang Cainiao Supply Chain Management Co., Ltd" },
    { "241EEB", "Apple, Inc." },
    { "241F2C", "Calsys, Inc." },
    { "241FA0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "241FBD", "Extreme Networks Headquarters" },
    { "2420C7", "Sagemcom Broadband SAS" },
    { "242124", "Nokia" },
    { "24215E", "Quectel Wireless Solutions Co.,Ltd." },
    { "2421AB", "Sony Corporation" },
    { "242361", "vivo Mobile Communication Co., Ltd." },
    { "24240E", "Apple, Inc." },
    { "2424B7", "Samsung Electronics Co.,Ltd" },
    { "242642", "SHARP Corporation." },
    { "2426BA", "Shenzhen Toptel Technology Co., Ltd." },
    { "2426D6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "242730", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "2427E5", "Huawei Device Co., Ltd." },
    { "242856", "Beijing Gctech Technology Co.,LTD" },
    { "2428FD", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "242934", "Google, Inc." },
    { "2429B0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2429FE", "KYOCERA Corporation " },
    { "242A04", "Cisco Systems, Inc" },
    { "242BD6", "Ring LLC" },
    { "242CFE", "Zhejiang Tmall Technology Co., Ltd." },
    { "242D4B", "F5 Inc." },
    { "242D6C", "eero inc." },
    { "242E02", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "242E90", "PALIT MICROSYSTEMS, LTD" },
    { "242FD0", "TP-Link Systems Inc" },
    { "242FFA", "Toshiba Global Commerce Solutions" },
    { "2430F8", "Huawei Device Co., Ltd." },
    { "243154", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "243184", "SHARP Corporation" },
    { "2432AE", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "24336C", "Private" },
    { "243408", "Edgecore Americas Networking Corporation" },
    { "2435CC", "Zhongshan Scinan Internet of Things Co.,Ltd." },
    { "2436DA", "Cisco Systems, Inc" },
    { "24374C", "Cisco SPVTG" },
    { "2437EF", "EMC Electronic Media Communication SA" },
    { "243A82", "IRTS" },
    { "243C20", "Dynamode Group" },
    { "243CB0", "Dongguan Mentech Optical & Magnetic Co., Ltd." },
    { "243F30", "Oxygen Broadband s.a." },
    { "243F75", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "243FAA", "Huawei Device Co., Ltd." },
    { "2440AE", "NIIC Technology Co., Ltd." },
    { "24418C", "Intel Corporate" },
    { "2441FE", "Wistron Neweb Corporation" },
    { "2442BC", "Alinco,incorporated" },
    { "2442E3", "Shenzhen Ai-Thinker Technology Co.,Ltd" },
    { "2443E2", "DASAN Network Solutions" },
    { "244427", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "24456B", "Huawei Device Co., Ltd." },
    { "244597", "GEMUE Gebr. Mueller Apparatebau" },
    { "2446C8", "Motorola Mobility LLC, a Lenovo Company" },
    { "2446E4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "24470E", "PentronicAB" },
    { "244845", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "244885", "Huawei Device Co., Ltd." },
    { "24497B", "Innovative Converged Devices Inc" },
    { "244B03", "Samsung Electronics Co.,Ltd" },
    { "244B81", "Samsung Electronics Co.,Ltd" },
    { "244BF1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "244BFE", "ASUSTek COMPUTER INC." },
    { "244C07", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "244CAB", "Espressif Inc." },
    { "244CE3", "Amazon Technologies Inc." },
    { "244E7B", "IEEE Registration Authority" },
    { "244F1D", "iRule LLC" },
    { "24506F", "THINKCAR TECH CO.,LTD." },
    { "24526A", "Zhejiang Dahua Technology Co., Ltd." },
    { "2453BF", "Enernet" },
    { "24586E", "zte corporation" },
    { "24587C", "Espressif Inc." },
    { "245880", "VIZEO" },
    { "24590B", "White Sky Inc. Limited" },
    { "2459E5", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "245A4C", "Ubiquiti Inc" },
    { "245A5F", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "245AB5", "Samsung Electronics Co.,Ltd" },
    { "245B83", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "245BA7", "Apple, Inc." },
    { "245BF0", "Liteon, Inc." },
    { "245CBF", "NCSE" },
    { "245CC5", "Huawei Device Co., Ltd." },
    { "245CCB", "AXIe Consortium, Inc." },
    { "245D92", "Juniper Networks" },
    { "245DFC", "IEEE Registration Authority" },
    { "245E48", "Apple, Inc." },
    { "245EBE", "QNAP Systems, Inc." },
    { "245EE1", "United Automotive Electronic Systems Co.,Ltd." },
    { "245F9F", "Huawei Device Co., Ltd." },
    { "245FDF", "KYOCERA CORPORATION " },
    { "246078", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "246081", "razberi technologies" },
    { "2460B3", "Samsung Electronics Co.,Ltd" },
    { "24615A", "China Mobile Group Device Co.,Ltd." },
    { "246278", "sysmocom - s.f.m.c. GmbH" },
    { "2462AB", "Espressif Inc." },
    { "2462CE", "Hewlett Packard Enterprise" },
    { "24649F", "Huawei Device Co., Ltd." },
    { "2464EF", "CYG SUNRI CO.,LTD." },
    { "246511", "AVM GmbH" },
    { "2465E1", "Ciena Corporation" },
    { "246830", "Shenzhen Shokzhear Co., Ltd" },
    { "246880", "Braveridge.co.,ltd." },
    { "2468B0", "Samsung Electronics Co.,Ltd" },
    { "24693E", "innodisk Corporation" },
    { "24694A", "Jasmine Systems Inc." },
    { "246968", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "24698E", "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "2469A5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "246A0E", "HP Inc." },
    { "246AAB", "IT-IS International" },
    { "246C60", "Huawei Device Co., Ltd." },
    { "246C84", "Cisco Systems, Inc" },
    { "246C8A", "YUKAI Engineering" },
    { "246E96", "Dell Inc." },
    { "246F28", "Espressif Inc." },
    { "246F8C", "Huawei Device Co., Ltd." },
    { "247152", "Dell Inc." },
    { "247189", "Texas Instruments" },
    { "24724A", "Nile Global Inc" },
    { "247260", "IOTTECH Corp" },
    { "2474F7", "GoPro" },
    { "24753A", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "2475B3", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "2475FC", "zte corporation" },
    { "247625", "Texas Instruments" },
    { "247645", "Huawei Device Co., Ltd." },
    { "247656", "Shanghai Net Miles Fiber Optics Technology Co., LTD." },
    { "24767D", "Cisco SPVTG" },
    { "247703", "Intel Corporate" },
    { "247823", "Panasonic Entertainment & Communication Co., Ltd." },
    { "2478EF", "Nokia" },
    { "24792A", "Ruckus Wireless" },
    { "2479EF", "Greenpacket Berhad, Taiwan" },
    { "2479F3", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "2479F8", "KUPSON spol. s r.o." },
    { "247BA4", "China Mobile Group Device Co.,Ltd." },
    { "247C46", "FLEXTRONICS TECHNOLOGIES MEXICO S DE RL DE CV" },
    { "247C4C", "Herman Miller" },
    { "247D4D", "Texas Instruments" },
    { "247E12", "Cisco Systems, Inc" },
    { "247E51", "zte corporation" },
    { "247F20", "Sagemcom Broadband SAS" },
    { "247F3C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "248000", "Westcontrol AS" },
    { "24813B", "Cisco Systems, Inc" },
    { "2481AA", "KSH International Co., Ltd." },
    { "2481C7", "Huawei Device Co., Ltd." },
    { "24828A", "Prowave Technologies Ltd." },
    { "248498", "Beijing Jiaoda Microunion Tech.Co.,Ltd." },
    { "248602", "Bouffalo Lab (Nanjing) Co., Ltd." },
    { "248625", "IEEE Registration Authority" },
    { "2486F4", "Ctek, Inc." },
    { "248707", "SEnergy Corporation" },
    { "248894", "shenzhen lensun Communication Technology LTD" },
    { "248A07", "Mellanox Technologies, Inc." },
    { "248BE0", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "249038", "Universal Biosensors Pty Ltd" },
    { "2491BB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "24920E", "Samsung Electronics Co.,Ltd" },
    { "2493CA", "Voxtronic Austria" },
    { "249442", "OPEN ROAD SOLUTIONS , INC." },
    { "249493", "FibRSol Global Network Limited" },
    { "249494", "Hong Kong Bouffalo Lab Limited" },
    { "2494CB", "Commscope" },
    { "249504", "SFR" },
    { "24952F", "Google, Inc." },
    { "2496D5", "NEXCON Technology Co.,ltd." },
    { "249745", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2497ED", "Techvision Intelligent Technology Limited" },
    { "249AC8", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "249AD8", "YEALINK(XIAMEN) NETWORK TECHNOLOGY CO.,LTD." },
    { "249D2A", "LinkData Technology (Tianjin) Co., LTD" },
    { "249E7D", "Beijing Roborock Technology Co., Ltd." },
    { "249EAB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "249F89", "Texas Instruments" },
    { "24A074", "Apple, Inc." },
    { "24A160", "Espressif Inc." },
    { "24A186", "Commscope" },
    { "24A2E1", "Apple, Inc." },
    { "24A3F0", "IEEE Registration Authority" },
    { "24A42C", "NETIO products a.s." },
    { "24A43C", "Ubiquiti Inc" },
    { "24A452", "Samsung Electronics Co.,Ltd" },
    { "24A487", "Huawei Device Co., Ltd." },
    { "24A495", "Thales Canada Inc." },
    { "24A52C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "24A534", "SynTrust Tech International Ltd. " },
    { "24A65E", "zte corporation" },
    { "24A6FA", "WEIFANG GOERTEK ELECTRONICS CO.,LTD" },
    { "24A799", "Huawei Device Co., Ltd." },
    { "24A7DC", "SKY UK LIMITED" },
    { "24A87D", "Panasonic Automotive Systems Asia Pacific(Thailand)Co.,Ltd." },
    { "24A937", "PURE Storage" },
    { "24AB81", "Apple, Inc." },
    { "24ACAC", "Polar Electro Oy" },
    { "24AECC", "Honor Device Co., Ltd." },
    { "24AF4A", "Alcatel-Lucent IPD" },
    { "24AF54", "NEXGEN Mediatech Inc." },
    { "24B0A9", "Shanghai Mobiletek Communication Ltd." },
    { "24B105", "Prama Hikvision India Private Limited" },
    { "24B209", "Avaya Inc" },
    { "24B231", "Dongguan Liesheng Electronic Co., Ltd." },
    { "24B2B9", "Liteon Technology Corporation" },
    { "24B2DE", "Espressif Inc." },
    { "24B339", "Apple, Inc." },
    { "24B5B9", "Motorola Mobility LLC, a Lenovo Company" },
    { "24B5F2", "Shanghai Ingeek Technology Co., Ltd" },
    { "24B657", "Cisco Systems, Inc" },
    { "24B6B8", "FRIEM SPA" },
    { "24B6FD", "Dell Inc." },
    { "24B72A", "China Dragon Technology Limited" },
    { "24B7DA", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "24B88C", "Crenus Co.,Ltd." },
    { "24B8D2", "Opzoon Technology Co.,Ltd." },
    { "24BA13", "RISO KAGAKU CORPORATION" },
    { "24BA30", "Technical Consumer Products, Inc." },
    { "24BBC1", "Absolute Analysis" },
    { "24BBC9", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "24BC82", "Dali Wireless, Inc." },
    { "24BCF8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "24BE05", "Hewlett Packard" },
    { "24BE18", "DADOUTEK COMPANY LIMITED" },
    { "24BF74", "Hamamatsu Photonics K.K." },
    { "24C0B3", "RSF" },
    { "24C17A", "BEIJING IACTIVE NETWORK CO.,LTD" },
    { "24C1BD", "CRRC DALIAN R&D CO.,LTD." },
    { "24C3F9", "Securitas Direct AB" },
    { "24C406", "SUNITEC TECHNOLOGY CO.,LIMITED" },
    { "24C42F", "Philips Lifeline" },
    { "24C44A", "zte corporation" },
    { "24C613", "Samsung Electronics Co.,Ltd" },
    { "24C696", "Samsung Electronics Co.,Ltd" },
    { "24C848", "mywerk Portal GmbH" },
    { "24C86E", "Chaney Instrument Co." },
    { "24C8D3", "McWane India Pvt Ltd" },
    { "24C9A1", "Ruckus Wireless" },
    { "24C9DE", "Genoray" },
    { "24CACB", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "24CBE1", "Ericsson AB" },
    { "24CBE7", "MYK, Inc." },
    { "24CD8D", "Murata Manufacturing Co., Ltd." },
    { "24CE33", "Amazon Technologies Inc." },
    { "24CF21", "Shenzhen State Micro Technology Co., Ltd" },
    { "24CF24", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "24D0DF", "Apple, Inc." },
    { "24D13F", "MEXUS CO.,LTD" },
    { "24D1A1", "Shenzhen Cultraview Digital Technology Co., Ltd" },
    { "24D208", "Sensata Technologies Inc." },
    { "24D2CC", "SmartDrive Systems Inc." },
    { "24D337", "Xiaomi Communications Co Ltd" },
    { "24D3F2", "zte corporation" },
    { "24D51C", "Zhongtian broadband technology co., LTD " },
    { "24D5E4", "Cisco Systems, Inc" },
    { "24D76B", "Syntronic AB" },
    { "24D79C", "Cisco Systems, Inc" },
    { "24D7EB", "Espressif Inc." },
    { "24D81E", "MirWifi,Joint-Stock Company" },
    { "24D904", "Sichuan Changhong Network Technologies Co., Ltd." },
    { "24D921", "Avaya Inc" },
    { "24DA11", "NO NDA Inc" },
    { "24DA33", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "24DA9B", "Motorola Mobility LLC, a Lenovo Company" },
    { "24DAB6", "Sistemas de Gestión Energética S.A. de C.V" },
    { "24DBAC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "24DBAD", "ShopperTrak RCT Corporation" },
    { "24DBED", "Samsung Electronics Co.,Ltd" },
    { "24DC0F", "Phytium Technology Co.,Ltd." },
    { "24DCC3", "Espressif Inc." },
    { "24DD1B", "Qingdao Hi-image Technologies Co., Ltd" },
    { "24DE8A", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "24DEC6", "Hewlett Packard Enterprise" },
    { "24DEEB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "24DF17", "Shenzhen Sande Dacom Electronics Co., Ltd" },
    { "24DF6A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "24DFA7", "Hangzhou BroadLink Technology Co.,Ltd" },
    { "24E124", "Xiamen Milesight IoT Co., Ltd." },
    { "24E271", "Qingdao Hisense Communications Co.,Ltd." },
    { "24E29D", "Huawei Device Co., Ltd." },
    { "24E314", "Apple, Inc." },
    { "24E3A4", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "24E3DE", "China Telecom Fufu Information Technology Co., Ltd." },
    { "24E43F", "Wenzhou Kunmei Communication Technology Co.,Ltd." },
    { "24E4C8", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "24E4CE", "Kaon Group Co., Ltd." },
    { "24E50F", "Google, Inc." },
    { "24E5AA", "Philips Oral Healthcare, Inc." },
    { "24E6BA", "JSC Zavod im. Kozitsky" },
    { "24E853", "LG Innotek" },
    { "24E8CE", "Qingdao Haier Technology Co.,Ltd" },
    { "24E8E5", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "24E927", "TomTom International BV" },
    { "24E9B3", "Cisco Systems, Inc" },
    { "24E9CA", "Huawei Device Co., Ltd." },
    { "24EA40", "Helmholz GmbH & Co. KG" },
    { "24EB16", "Intel Corporate" },
    { "24EB65", "SAET I.S. S.r.l." },
    { "24EBED", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "24EC4A", "Espressif Inc." },
    { "24EC51", "ADF Technologies Sdn Bhd" },
    { "24EC99", "ASKEY COMPUTER CORP" },
    { "24ECD6", "CSG Science & Technology Co.,Ltd.Hefei" },
    { "24EDFD", "Siemens Canada Limited" },
    { "24EE3A", "Chengdu Yingji Electronic Hi-tech Co Ltd" },
    { "24EE9A", "Intel Corporate" },
    { "24EFB4", "Shanghai Neardi Technologies Co. Ltd." },
    { "24F094", "Apple, Inc." },
    { "24F0D3", "Samsung Electronics Co.,Ltd" },
    { "24F0FF", "GHT Co., Ltd." },
    { "24F128", "Telstra" },
    { "24F150", "Guangzhou Qi'an Technology Co., Ltd." },
    { "24F27F", "Hewlett Packard Enterprise " },
    { "24F2DD", "Radiant Zemax LLC" },
    { "24F306", "ITEL MOBILE LIMITED" },
    { "24F3E3", "eero inc." },
    { "24F40A", "Samsung Electronics Co.,Ltd" },
    { "24F57E", "HWH CO., LTD." },
    { "24F5A2", "Belkin International Inc." },
    { "24F5AA", "Samsung Electronics Co.,Ltd" },
    { "24F603", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "24F677", "Apple, Inc." },
    { "24F68D", "Nokia" },
    { "24FAF3", "Shanghai Flexem Technology Co.,Ltd." },
    { "24FB65", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "24FBE3", "HP Inc." },
    { "24FC4E", "Juniper Networks" },
    { "24FCE5", "Samsung Electronics Co.,Ltd" },
    { "24FD0D", "Intelbras" },
    { "24FD52", "Liteon Technology Corporation" },
    { "24FD5B", "SmartThings, Inc." },
    { "24FDFA", "Private" },
    { "24FE9A", "CyberTAN Technology Inc." },
    { "2800AF", "Dell Inc." },
    { "28011C", "zte corporation" },
    { "2801CD", "Mellanox Technologies, Inc." },
    { "28022E", "Apple, Inc." },
    { "280244", "Apple, Inc." },
    { "280245", "Konze System Technology Co.,Ltd." },
    { "2802D8", "Samsung Electronics Co.,Ltd" },
    { "2804C6", "Wanan Hongsheng Electronic Co.Ltd" },
    { "2804E0", "FERMAX ELECTRONICA S.A.U." },
    { "28052E", "Dematic Corp" },
    { "28061E", "NINGBO GLOBAL USEFUL ELECTRIC CO.,LTD" },
    { "28068D", "ITL, LLC" },
    { "280708", "Samsung Electronics Co.,Ltd" },
    { "28070D", "GUANGZHOU WINSOUND INFORMATION TECHNOLOGY CO.,LTD." },
    { "280AEE", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "280B5C", "Apple, Inc." },
    { "280C28", "Unigen DataStorage Corporation" },
    { "280C2D", "QUALVISION TECHNOLOGY CO.,LTD" },
    { "280C50", "Intel Corporate" },
    { "280CB8", "Mikrosay Yazilim ve Elektronik A.S." },
    { "280DFC", "Sony Interactive Entertainment Inc." },
    { "280E8B", "Beijing Spirit Technology Development Co., Ltd." },
    { "280FC5", "Beijing Leadsec Technology Co., Ltd." },
    { "280FEB", "LG Innotek" },
    { "28101B", "MagnaCom" },
    { "28107B", "D-Link International" },
    { "2811A5", "Bose Corporation" },
    { "2811A8", "Intel Corporate" },
    { "2811EC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "281293", "Honor Device Co., Ltd." },
    { "281471", "Lantis co., LTD." },
    { "2815A4", "SHENZHEN PINSU ZHILIAN INFORMATION TECHNOLOGY CO.,LTD." },
    { "28162E", "2Wire Inc" },
    { "28167F", "Xiaomi Communications Co Ltd" },
    { "2816A8", "Microsoft Corporation" },
    { "2816AD", "Intel Corporate" },
    { "281709", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "28172E", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "2817CB", "Software Freedom Conservancy" },
    { "2817CE", "Omnisense Ltd" },
    { "281878", "Microsoft Corporation" },
    { "2818FD", "Aditya Infotech Ltd." },
    { "281B04", "Zalliant LLC" },
    { "281D21", "IN ONE SMART TECHNOLOGY(H,K,)LIMITED" },
    { "281DFB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "28221E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "282246", "Beijing Sinoix Communication Co., LTD" },
    { "282373", "Digita" },
    { "2823F5", "China Mobile (Hangzhou) Information Technology Co., Ltd." },
    { "2824C9", "Amazon Technologies Inc." },
    { "2824FF", "Wistron Neweb Corporation" },
    { "282536", "SHENZHEN HOLATEK CO.,LTD" },
    { "28255F", "HUMAX NETWORKS" },
    { "2826A6", "PBR electronics GmbH" },
    { "2827BF", "Samsung Electronics Co.,Ltd" },
    { "28285D", "Zyxel Communications Corporation" },
    { "282947", "Chipsea Technologies (Shenzhen) Corp." },
    { "282986", "APC by Schneider Electric" },
    { "2829CC", "Corsa Technology Incorporated" },
    { "2829D9", "GlobalBeiMing technology (Beijing)Co. Ltd" },
    { "282A87", "ITEL MOBILE LIMITED" },
    { "282B96", "Huawei Device Co., Ltd." },
    { "282BB9", "Shenzhen Xiongxin Technology Co.,Ltd" },
    { "282C02", "IEEE Registration Authority" },
    { "282CB2", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "282CC4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "282D06", "AMPAK Technology,Inc." },
    { "282D7F", "Apple, Inc." },
    { "282E30", "MECHATRONICS INNOVATION TECHNOLOGIES, S.L.U." },
    { "282E89", "Wistron Neweb Corporation" },
    { "282FC2", "Automotive Data Solutions" },
    { "2830AC", "Frontiir Co. Ltd." },
    { "283152", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "283166", "vivo Mobile Communication Co., Ltd." },
    { "28317E", "Hongkong Nano IC Technologies Co., Ltd" },
    { "2831F8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2832C5", "HUMAX Co., Ltd." },
    { "283334", "Huawei Device Co., Ltd." },
    { "283410", "Enigma Diagnostics Limited" },
    { "2834A2", "Cisco Systems, Inc" },
    { "2834FF", "Apple, Inc." },
    { "28353A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "283545", "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD" },
    { "283613", "IEEE Registration Authority" },
    { "283638", "IEEE Registration Authority" },
    { "2836F0", "Huawei Device Co., Ltd." },
    { "283713", "Shenzhen 3Nod Digital Technology Co., Ltd." },
    { "28372F", "Espressif Inc." },
    { "283737", "Apple, Inc." },
    { "28385C", "FLEXTRONICS" },
    { "2838CF", "Gen2wave" },
    { "283926", "CyberTAN Technology Inc." },
    { "28395E", "Samsung Electronics Co.,Ltd" },
    { "283984", "Qidi Technology (shanghai) Co.,Ltd." },
    { "2839E7", "Preceno Technology Pte.Ltd." },
    { "283A4D", "Cloud Network Technology (Samoa) Limited" },
    { "283B82", "D-Link International" },
    { "283B96", "Cool Control LTD" },
    { "283C90", "Texas Instruments" },
    { "283CE4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "283DC2", "Samsung Electronics Co.,Ltd" },
    { "283DE8", "Guangzhou Shiyuan Electronic Technology Company Limited" },
    { "283E0C", "Preferred Robotics, Inc." },
    { "283E76", "Common Networks" },
    { "283F69", "Sony Corporation" },
    { "28401A", "C8 MediSensors, Inc." },
    { "2840DD", "Sony Interactive Entertainment Inc." },
    { "284121", "OptiSense Network, LLC" },
    { "2841C6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2841EC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2843DC", "United Memory Technology (Jiangsu) Limited" },
    { "284430", "Arcade Communications Ltd." },
    { "2844F4", "Honor Device Co., Ltd." },
    { "2845AC", "Huawei Device Co., Ltd." },
    { "2847AA", "Nokia Corporation" },
    { "284846", "GridCentric Inc." },
    { "2848E7", "Huawei Device Co., Ltd." },
    { "284992", "Luminator Technology Group Global LLC" },
    { "284C53", "Intune Networks" },
    { "284D7D", "zte corporation" },
    { "284D92", "Luminator Technology Group Global LLC" },
    { "284E44", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "284ED7", "OutSmart Power Systems, Inc." },
    { "284EE9", "mercury corperation" },
    { "284FCE", "Liaoning Wontel Science and Technology Development Co.,Ltd." },
    { "285132", "Shenzhen Prayfly Technology Co.,Ltd" },
    { "285261", "Cisco Systems, Inc" },
    { "2852E0", "Layon international Electronic & Telecom Co.,Ltd" },
    { "2852F9", "Zhongxin Intelligent Times (Shenzhen) Co., Ltd." },
    { "28534E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2853E0", "Sintela Ltd" },
    { "285471", "Huawei Device Co., Ltd." },
    { "28562F", "Espressif Inc." },
    { "28563A", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "28565A", "Hon Hai Precision Ind. Co.,Ltd." },
    { "2856C1", "Harman/Becker Automotive Systems GmbH" },
    { "28575D", "Apple, Inc." },
    { "285767", "Dish Technologies Corp" },
    { "2857BE", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "285923", "Xiaomi Communications Co Ltd" },
    { "285AEB", "Apple, Inc." },
    { "285B0C", "Sichuan Jiuzhou Electronic Technology Co., Ltd." },
    { "285F2F", "RNware Co.,Ltd." },
    { "285FDB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "286046", "Lantech Communications Global, Inc." },
    { "286094", "CAPELEC" },
    { "286336", "Siemens AG" },
    { "2863BD", "APTIV SERVICES US, LLC" },
    { "2864B0", "Huawei Device Co., Ltd." },
    { "2864EF", "Shenzhen Fsan Intelligent Technology Co.,Ltd" },
    { "28656B", "Keystone Microtech Corporation" },
    { "2866E3", "AzureWave Technology Inc." },
    { "286847", "Silicon Laboratories" },
    { "2868D2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "286AB8", "Apple, Inc." },
    { "286ABA", "Apple, Inc." },
    { "286B35", "Intel Corporate" },
    { "286B5C", "Cisco Systems, Inc" },
    { "286BB4", "SJIT Co., Ltd." },
    { "286C07", "XIAOMI Electronics,CO.,LTD" },
    { "286D97", "SAMJIN Co., Ltd." },
    { "286DCD", "Beijing Winner Microelectronics Co.,Ltd. " },
    { "286DDA", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "286ED4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "286F40", "Tonly Technology Co. Ltd " },
    { "286F7F", "Cisco Systems, Inc" },
    { "286FB9", "Nokia Shanghai Bell Co., Ltd." },
    { "28704E", "Ubiquiti Inc" },
    { "287184", "Spire Payments" },
    { "2872C5", "Smartmatic Corp" },
    { "2872F0", "ATHENA" },
    { "2873F6", "Amazon Technologies Inc." },
    { "2874F5", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "2875D8", "FUJIAN STAR-NET COMMUNICATION CO.,LTD" },
    { "287610", "IgniteNet" },
    { "287681", "Silicon Laboratories" },
    { "2876CD", "Funshion Online Technologies Co.,Ltd" },
    { "287777", "zte corporation" },
    { "2877B1", "Tri plus grupa d.o.o." },
    { "2877F1", "Apple, Inc." },
    { "287994", "Realplay Digital Technology(Shenzhen) Co.,Ltd" },
    { "287AB4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "287AEE", "Commscope" },
    { "287B09", "zte corporation" },
    { "287B11", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "287CDB", "Hefei Toycloud Technology Co.,ltd" },
    { "287E80", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "287FCF", "Intel Corporate" },
    { "288023", "Hewlett Packard" },
    { "288088", "NETGEAR" },
    { "28808A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2880A2", "Inseego Wireless, Inc" },
    { "28827C", "Bosch Automative products(Suzhou)Co.,Ltd Changzhou Branch" },
    { "288335", "Samsung Electronics Co.,Ltd" },
    { "2883C9", "Apple, Inc." },
    { "28840E", "silicon valley immigration service " },
    { "2884FA", "SHARP Corporation" },
    { "28852D", "Touch Networks" },
    { "2885BB", "Zen Exim Pvt. Ltd." },
    { "288761", "LG Innotek" },
    { "2887BA", "TP-Link Systems Inc" },
    { "288915", "CashGuard Sverige AB" },
    { "288A1C", "Juniper Networks" },
    { "288CB8", "zte corporation" },
    { "288EB9", "Wacom Co.,Ltd." },
    { "288EEC", "Apple, Inc." },
    { "288FF6", "Apple, Inc." },
    { "289176", "Indyme Solutions, LLC" },
    { "2891D0", "Stage Tec Entwicklungsgesellschaft für professionelle Audiotechnik mbH" },
    { "289200", "Intel Corporate" },
    { "28924A", "Hewlett Packard" },
    { "28937D", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "2893FE", "Cisco Systems, Inc" },
    { "289401", "NETGEAR" },
    { "28940F", "Cisco Systems, Inc" },
    { "2894AF", "Samhwa Telecom" },
    { "289529", "Intel Corporate" },
    { "2897B8", "myenergi Ltd" },
    { "28987B", "Samsung Electronics Co.,Ltd" },
    { "28993A", "Arista Networks" },
    { "2899C7", "LINDSAY BROADBAND INC" },
    { "289A4B", "SteelSeries ApS" },
    { "289AF7", "ADVA Optical Networking Ltd." },
    { "289AFA", "TCT mobile ltd" },
    { "289C6E", "Shanghai High-Flying Electronics Technology Co., Ltd" },
    { "289E97", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "289EDF", "Danfoss Turbocor Compressors, Inc" },
    { "289EFC", "Sagemcom Broadband SAS" },
    { "289F04", "Samsung Electronics Co.,Ltd" },
    { "28A02B", "Apple, Inc." },
    { "28A06B", "Intel Corporate" },
    { "28A183", "ALPSALPINE CO,.LTD" },
    { "28A186", "enblink" },
    { "28A192", "GERP Solution" },
    { "28A1EB", "ETEK TECHNOLOGY (SHENZHEN) CO.,LTD" },
    { "28A241", "exlar corp" },
    { "28A24B", "Juniper Networks" },
    { "28A331", "Sierra Wireless, ULC" },
    { "28A44A", "Intel Corporate" },
    { "28A53F", "vivo Mobile Communication Co., Ltd." },
    { "28A574", "Miller Electric Mfg. Co." },
    { "28A5EE", "Shenzhen SDGI CATV Co., Ltd" },
    { "28A6AC", "seca gmbh & co. kg" },
    { "28A6DB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "28A915", "SERVERCOM (INDIA) PRIVATE LIMITED" },
    { "28AC67", "Mach Power, Rappresentanze Internazionali s.r.l." },
    { "28AC9E", "Cisco Systems, Inc" },
    { "28AD18", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "28AD3E", "Shenzhen TONG BO WEI Technology CO.,LTD" },
    { "28AF0A", "Sirius XM Radio Inc" },
    { "28AF21", "zte corporation" },
    { "28AF42", "Samsung Electronics Co.,Ltd" },
    { "28AFFD", "Cisco Systems, Inc" },
    { "28B0CC", "Xenya d.o.o." },
    { "28B133", "SHINEMAN(SHENZHEN) Tech. Cor., Ltd." },
    { "28B221", "Sienda Multimedia Ltd" },
    { "28B2BD", "Intel Corporate" },
    { "28B371", "Ruckus Wireless" },
    { "28B3AB", "Genmark Automation" },
    { "28B448", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "28B4FB", "Sprocomm Technologies CO.,LTD." },
    { "28B591", "Cisco Systems, Inc" },
    { "28B5E8", "Texas Instruments" },
    { "28B77C", "IEEE Registration Authority" },
    { "28B829", "Juniper Networks" },
    { "28B9D9", "Radisys Corporation" },
    { "28BA18", "NextNav, LLC" },
    { "28BAB5", "Samsung Electronics Co.,Ltd" },
    { "28BB59", "RNET Technologies, Inc." },
    { "28BBED", "Bouffalo Lab (Nanjing) Co., Ltd." },
    { "28BC05", "BLU Products Inc" },
    { "28BC18", "SourcingOverseas Co. Ltd" },
    { "28BC56", "EMAC, Inc." },
    { "28BD89", "Google, Inc." },
    { "28BE03", "TCT mobile ltd" },
    { "28BE43", "vivo Mobile Communication Co., Ltd." },
    { "28BE9B", "Vantiva USA LLC" },
    { "28BEF3", "FUJIAN STAR-NET COMMUNICATION CO.,LTD" },
    { "28BF89", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "28C01B", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "28C0DA", "Juniper Networks" },
    { "28C13C", "Hon Hai Precision Industry Co., Ltd." },
    { "28C1A0", "Apple, Inc." },
    { "28C21F", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "28C2DD", "AzureWave Technology Inc." },
    { "28C538", "Apple, Inc." },
    { "28C5C8", "HP Inc." },
    { "28C5D2", "Intel Corporate" },
    { "28C63F", "Intel Corporate" },
    { "28C671", "Yota Devices OY" },
    { "28C68E", "NETGEAR" },
    { "28C709", "Apple, Inc." },
    { "28C718", "Altierre" },
    { "28C7CE", "Cisco Systems, Inc" },
    { "28C825", "DellKing Industrial Co., Ltd" },
    { "28C87A", "Commscope" },
    { "28C87C", "zte corporation" },
    { "28C914", "Taimag Corporation" },
    { "28C97A", "New H3C Technologies Co., Ltd" },
    { "28CA09", "ThyssenKrupp Elevators (Shanghai) Co.,Ltd" },
    { "28CB5C", "Shenzhen CPETEK Technology Co.,Ltd." },
    { "28CBEB", "One" },
    { "28CC01", "Samsung Electronics Co.,Ltd" },
    { "28CCFF", "Corporacion Empresarial Altra SL" },
    { "28CD1C", "Espotel Oy" },
    { "28CD4C", "Individual Computers GmbH" },
    { "28CD9C", "Shenzhen Dynamax Software Development Co.,Ltd." },
    { "28CDC1", "Raspberry Pi Trading Ltd" },
    { "28CDC4", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "28CE15", "Shenzhen Xinwei Intelligent Co., Ltd" },
    { "28CF08", "ESSYS" },
    { "28CF51", "Nintendo Co.,Ltd" },
    { "28CFDA", "Apple, Inc." },
    { "28CFE9", "Apple, Inc." },
    { "28D043", "AzureWave Technology Inc." },
    { "28D044", "Shenzhen Xinyin technology company" },
    { "28D0CB", "Adtran Inc" },
    { "28D0EA", "Intel Corporate" },
    { "28D0F5", "Ruijie Networks Co.,LTD" },
    { "28D127", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "28D1AF", "Nokia Corporation" },
    { "28D1B7", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "28D244", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "28D25A", "Infinix mobility limited" },
    { "28D3EA", "Huawei Device Co., Ltd." },
    { "28D41E", "Barrot Technology Co.,Ltd." },
    { "28D436", "Jiangsu dewosi electric co., LTD" },
    { "28D576", "Premier Wireless, Inc." },
    { "28D5B1", "Apple, Inc." },
    { "28D93E", "Telecor Inc." },
    { "28D98A", "Hangzhou Konke Technology Co.,Ltd." },
    { "28D997", "Yuduan Mobile Co., Ltd." },
    { "28DB02", "zte corporation" },
    { "28DB81", "Shanghai Guao Electronic Technology Co., Ltd" },
    { "28DBA7", "Silicon Laboratories" },
    { "28DE1C", "Samsung Electronics Co.,Ltd" },
    { "28DE59", "Domus NTW CORP." },
    { "28DE65", "Hewlett Packard Enterprise" },
    { "28DEA8", "zte corporation" },
    { "28DEE5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "28DEF6", "bioMerieux Inc." },
    { "28DFEB", "Intel Corporate" },
    { "28E02C", "Apple, Inc." },
    { "28E14C", "Apple, Inc." },
    { "28E297", "Shanghai InfoTM Microelectronics Co.,Ltd" },
    { "28E31F", "Xiaomi Communications Co Ltd" },
    { "28E347", "Liteon Technology Corporation" },
    { "28E34E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "28E424", "New H3C Technologies Co., Ltd" },
    { "28E476", "Pi-Coral" },
    { "28E5B0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "28E608", "Tokheim" },
    { "28E6A9", "Samsung Electronics Co.,Ltd" },
    { "28E6E9", "SIS Sat Internet Services GmbH" },
    { "28E71D", "Arista Networks" },
    { "28E794", "Microtime Computer Inc." },
    { "28E7CF", "Apple, Inc." },
    { "28E98E", "Mitsubishi Electric Corporation" },
    { "28EA0B", "Microsoft Corporation" },
    { "28EA2D", "Apple, Inc." },
    { "28EB0A", "Rolling Wireless S.a.r.l. Luxembourg" },
    { "28EBA6", "Nex-T LLC" },
    { "28EBB7", "ambie corporation" },
    { "28EC22", "eero inc." },
    { "28EC95", "Apple, Inc." },
    { "28EC9A", "Texas Instruments" },
    { "28ED58", "JAG Jakob AG" },
    { "28ED6A", "Apple, Inc." },
    { "28EDE0", "AMPAK Technology, Inc." },
    { "28EE2C", "Frontline Test Equipment" },
    { "28EE52", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "28EED3", "Shenzhen Super D Technology Co., Ltd" },
    { "28EF01", "Amazon Technologies Inc." },
    { "28F033", "Apple, Inc." },
    { "28F076", "Apple, Inc." },
    { "28F10E", "Dell Inc." },
    { "28F358", "2C - Trifonov & Co" },
    { "28F366", "Shenzhen Bilian electronic CO.,LTD" },
    { "28F49B", "LEETEK" },
    { "28F52B", "FN-LINK TECHNOLOGY Ltd." },
    { "28F532", "ADD-Engineering BV" },
    { "28F537", "IEEE Registration Authority" },
    { "28F5D1", "Commscope" },
    { "28F606", "Syes srl" },
    { "28F7D6", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "28F8C6", "IEEE Registration Authority" },
    { "28FA19", "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd" },
    { "28FA7A", "Zhejiang Tmall Technology Co., Ltd." },
    { "28FAA0", "vivo Mobile Communication Co., Ltd." },
    { "28FBAE", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "28FBD3", "Ragentek Technology Group" },
    { "28FC51", "The Electric Controller and Manufacturing Co., LLC" },
    { "28FCF6", "Shenzhen Xin KingBrand enterprises Co.,Ltd" },
    { "28FD80", "IEEE Registration Authority" },
    { "28FE65", "DongGuan Siyoto Electronics Co., Ltd " },
    { "28FECD", "Lemobile Information Technology (Beijing) Co., Ltd." },
    { "28FEDE", "COMESTA, Inc." },
    { "28FF3C", "Apple, Inc." },
    { "28FF3E", "zte corporation" },
    { "28FF5F", "HG Genuine Intelligent Terminal (Xiaogan) Co.,Ltd." },
    { "28FFB2", "Toshiba Corp." },
    { "2C002A", "Shenzhen TINNO Mobile Technology Corp." },
    { "2C002C", "UNOWHY" },
    { "2C0033", "EControls, LLC" },
    { "2C00AB", "Commscope" },
    { "2C00F7", "XOS" },
    { "2C010B", "NASCENT Technology, LLC - RemKon" },
    { "2C01B5", "Cisco Systems, Inc" },
    { "2C029F", "3ALogics" },
    { "2C0369", "ACCTON TECHNOLOGY CORPORATION" },
    { "2C0547", "Shenzhen Phaten Tech. LTD" },
    { "2C0623", "Win Leader Inc." },
    { "2C073C", "DEVLINE LIMITED" },
    { "2C0786", "Huawei Device Co., Ltd." },
    { "2C07F6", "SKG Health Technologies Co., Ltd. " },
    { "2C081C", "OVH" },
    { "2C0823", "Sercomm France Sarl" },
    { "2C088C", "HUMAX Co., Ltd." },
    { "2C08B4", "Huawei Device Co., Ltd." },
    { "2C094D", "Raptor Engineering, LLC" },
    { "2C09CB", "COBS AB" },
    { "2C0B97", "Xiaomi Communications Co Ltd" },
    { "2C0BAB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2C0BE9", "Cisco Systems, Inc" },
    { "2C0D27", "Huawei Device Co., Ltd." },
    { "2C0DA7", "Intel Corporate" },
    { "2C0DCF", "Xiaomi Communications Co Ltd" },
    { "2C0E3D", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "2C10C1", "Nintendo Co., Ltd." },
    { "2C1165", "Silicon Laboratories" },
    { "2C157E", "RADIODATA GmbH" },
    { "2C15BF", "Samsung Electronics Co.,Ltd" },
    { "2C15D9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2C15E1", "Phicomm (Shanghai) Co., Ltd." },
    { "2C16BD", "IEEE Registration Authority" },
    { "2C17E0", "SYSTEMES ET TECHNOLOGIES IDENTIFICATION (STid)" },
    { "2C1809", "Apple, Inc." },
    { "2C1875", "Skyworth Digital Technology(Shenzhen) Co.,Ltd" },
    { "2C18AE", "Trend Electronics Co., Ltd." },
    { "2C195C", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "2C1984", "IDN Telecom, Inc." },
    { "2C1A01", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2C1A05", "Cisco Systems, Inc" },
    { "2C1A31", "Electronics Company Limited" },
    { "2C1B3A", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "2C1BC8", "Hunan Topview Network System CO.,LTD" },
    { "2C1CF6", "Alien Green LLC" },
    { "2C1DB8", "Commscope" },
    { "2C1E4F", "Chengdu Qianli Network Technology Co., Ltd." },
    { "2C1EEA", "AERODEV" },
    { "2C1F23", "Apple, Inc." },
    { "2C200B", "Apple, Inc." },
    { "2C2080", "Huawei Device Co., Ltd." },
    { "2C2131", "Juniper Networks" },
    { "2C2172", "Juniper Networks" },
    { "2C21D7", "IMAX Corporation" },
    { "2C228B", "CTR SRL" },
    { "2C233A", "Hewlett Packard" },
    { "2C245F", "Babolat VS" },
    { "2C2617", "Oculus VR, LLC" },
    { "2C265F", "IEEE Registration Authority" },
    { "2C26C5", "zte corporation" },
    { "2C2768", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2C279E", "IEEE Registration Authority" },
    { "2C27D7", "Hewlett Packard" },
    { "2C282D", "BBK EDUCATIONAL ELECTRONICS CORP.,LTD." },
    { "2C28B7", "Hangzhou Ruiying technology co., LTD" },
    { "2C2997", "Microsoft Corporation" },
    { "2C2BF9", "LG Innotek" },
    { "2C2D48", "Commend International GmbH" },
    { "2C301A", "Technicolor CH USA Inc for Telus" },
    { "2C3033", "NETGEAR" },
    { "2C3068", "Pantech Co.,Ltd" },
    { "2C3124", "Cisco Systems, Inc" },
    { "2C326A", "Apple, Inc." },
    { "2C3311", "Cisco Systems, Inc" },
    { "2C3341", "China Mobile IOT Company Limited" },
    { "2C3358", "Intel Corporate" },
    { "2C3361", "Apple, Inc." },
    { "2C337A", "Hon Hai Precision Ind. Co.,Ltd." },
    { "2C3427", "ERCO & GENER" },
    { "2C347B", "SHENZHEN JUNGE TECHNOLOGY CO.,LTD" },
    { "2C3557", "ELIIY Power CO., Ltd." },
    { "2C36A0", "Capisco Limited" },
    { "2C36F2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2C36F8", "Cisco Systems, Inc" },
    { "2C3731", "SHENZHEN YIFANG DIGITAL TECHNOLOGY CO.,LTD." },
    { "2C3796", "CYBO CO.,LTD." },
    { "2C37C5", "Qingdao Haier Intelligent Home Appliance Technology Co.,Ltd" },
    { "2C3996", "Sagemcom Broadband SAS" },
    { "2C39C1", "Ciena Corporation" },
    { "2C3A28", "Fagor Electrónica" },
    { "2C3A91", "Huawei Device Co., Ltd." },
    { "2C3AE8", "Espressif Inc." },
    { "2C3AFD", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "2C3B70", "AzureWave Technology Inc." },
    { "2C3BFD", "Netstor Technology Co., Ltd." },
    { "2C3C05", "Marinesync Corp" },
    { "2C3EBF", "HOSIN Global Electronics Co., Ltd." },
    { "2C3ECF", "Cisco Systems, Inc" },
    { "2C3F0B", "Cisco Meraki" },
    { "2C3F38", "Cisco Systems, Inc" },
    { "2C3F3E", "Alge-Timing GmbH" },
    { "2C402B", "Smart iBlue Technology Limited" },
    { "2C4053", "Samsung Electronics Co.,Ltd" },
    { "2C4138", "Hewlett Packard" },
    { "2C41A1", "Bose Corporation" },
    { "2C4205", "Lytx" },
    { "2C431A", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "2C43BE", "Sunnovo International Limited" },
    { "2C4401", "Samsung Electronics Co.,Ltd" },
    { "2C441B", "Spectrum Medical Limited" },
    { "2C44FD", "Hewlett Packard" },
    { "2C459A", "Dixon Technologies (India) Limited" },
    { "2C4759", "Beijing MEGA preponderance Science & Technology Co. Ltd" },
    { "2C4835", "IEEE Registration Authority" },
    { "2C4881", "vivo Mobile Communication Co., Ltd." },
    { "2C4A11", "Ciena Corporation" },
    { "2C4C15", "Juniper Networks" },
    { "2C4C7D", "New H3C Technologies Co., Ltd" },
    { "2C4CC6", "Murata Manufacturing Co., Ltd." },
    { "2C4D54", "ASUSTek COMPUTER INC." },
    { "2C4D79", "WEIFANG GOERTEK ELECTRONICS CO.,LTD" },
    { "2C4DDE", "TECNO MOBILE LIMITED" },
    { "2C4E7D", "Chunghua Intelligent Network Equipment Inc." },
    { "2C4F52", "Cisco Systems, Inc" },
    { "2C5089", "Shenzhen Kaixuan Visual Technology Co.,Limited" },
    { "2C52AF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2C532B", "TCT mobile ltd" },
    { "2C534A", "Shenzhen Winyao Electronic Limited" },
    { "2C53D7", "Sonova AG" },
    { "2C542D", "Cisco Systems, Inc" },
    { "2C5491", "Microsoft Corporation" },
    { "2C54CF", "LG Electronics (Mobile Communications)" },
    { "2C553C", "Vecima Networks Inc." },
    { "2C557C", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "2C55D3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2C5683", "China Mobile Group Device Co.,Ltd." },
    { "2C56DC", "ASUSTek COMPUTER INC." },
    { "2C572C", "Allwinner Technology Co., Ltd" },
    { "2C5731", "Wingtech Group (HongKong）Limited" },
    { "2C5741", "Cisco Systems, Inc" },
    { "2C57CE", "Apple, Inc." },
    { "2C584F", "Commscope" },
    { "2C58B9", "HP Inc." },
    { "2C58E8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2C598A", "LG Electronics (Mobile Communications)" },
    { "2C59E5", "Hewlett Packard" },
    { "2C5A05", "Nokia Corporation" },
    { "2C5A0F", "Cisco Systems, Inc" },
    { "2C5A8D", "SYSTRONIK Elektronik u. Systemtechnik GmbH" },
    { "2C5AA3", "PROMATE ELECTRONIC CO.LTD" },
    { "2C5BB8", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "2C5BE1", "Centripetal Networks, Inc" },
    { "2C5D34", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "2C5D93", "Ruckus Wireless" },
    { "2C5EAB", "Mellanox Technologies, Inc." },
    { "2C5FF3", "Pertronic Industries" },
    { "2C600C", "Quanta Computer Inc." },
    { "2C60CD", "NR ELECTRIC CO., LTD" },
    { "2C6104", "SHENZHEN FENGLIAN TECHNOLOGY CO., LTD." },
    { "2C61F6", "Apple, Inc." },
    { "2C625A", "Finest Security Systems Co., Ltd" },
    { "2C6289", "Regenersis (Glenrothes) Ltd" },
    { "2C6373", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "2C641F", "Vizio, Inc" },
    { "2C64F6", "Wu Qi Technologies,Inc." },
    { "2C66AD", "NimbleTech Digital Inc." },
    { "2C66F5", "SHENZHEN ELECTRICAL APPLIANCES CO." },
    { "2C6798", "InTalTech Ltd." },
    { "2C67AB", "EZELINK TELECOM" },
    { "2C67BE", "DWnet Technologies(Suzhou) Corporation" },
    { "2C67FB", "ShenZhen Zhengjili Electronics Co., LTD" },
    { "2C691D", "IEEE Registration Authority" },
    { "2C693E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2C69BA", "RF Controls, LLC" },
    { "2C69CC", "Valeo Detection Systems " },
    { "2C6A6F", "IEEE Registration Authority" },
    { "2C6B7D", "Texas Instruments" },
    { "2C6BF5", "Juniper Networks" },
    { "2C6DC1", "Intel Corporate" },
    { "2C6E85", "Intel Corporate" },
    { "2C6F37", "Nokia" },
    { "2C6F4E", "Hubei Yuan Times Technology Co.,Ltd." },
    { "2C6F51", "Herospeed Digital Technology Limited" },
    { "2C6FC9", "Hon Hai Precision Ind. Co.,Ltd." },
    { "2C704F", "zte corporation" },
    { "2C7155", "HiveMotion" },
    { "2C71FF", "Amazon Technologies Inc." },
    { "2C72C3", "Soundmatters" },
    { "2C7360", "Earda Technologies co Ltd" },
    { "2C73A0", "Cisco Systems, Inc" },
    { "2C750F", "Shanghai Dongzhou-Lawton Communication Technology Co. Ltd." },
    { "2C75CB", "Novitec Co., Ltd." },
    { "2C7600", "Apple, Inc." },
    { "2C768A", "Hewlett Packard" },
    { "2C780E", "Huawei Device Co., Ltd." },
    { "2C784C", "Iton Technology Corp." },
    { "2C793D", "Boditech Med" },
    { "2C79D7", "Sagemcom Broadband SAS" },
    { "2C7AF4", "IEEE Registration Authority" },
    { "2C7B5A", "Milper Ltd" },
    { "2C7B84", "OOO Petr Telegin" },
    { "2C7BA0", "Intel Corporate" },
    { "2C7CE4", "Wuhan Tianyu Information Industry Co., Ltd." },
    { "2C7CF2", "Apple, Inc." },
    { "2C7E81", "Commscope" },
    { "2C7ECF", "Onzo Ltd" },
    { "2C8065", "HARTING Inc. of North America" },
    { "2C8158", "Hon Hai Precision Ind. Co.,Ltd." },
    { "2C81BF", "Apple, Inc." },
    { "2C8217", "Apple, Inc." },
    { "2C86D2", "Cisco Systems, Inc" },
    { "2C8A72", "HTC Corporation" },
    { "2C8AC7", "Ubee Interactive Co., Limited" },
    { "2C8BF2", "Hitachi Metals America Ltd" },
    { "2C8D37", "Virtium" },
    { "2C8DB1", "Intel Corporate" },
    { "2C9127", "Eintechno Corporation" },
    { "2C91AB", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "2C922C", "Kishu Giken Kogyou Company Ltd,." },
    { "2C93FB", "Sercomm France Sarl" },
    { "2C9452", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2C9464", "Cincoze Co., Ltd." },
    { "2C9520", "Apple, Inc." },
    { "2C9569", "Commscope" },
    { "2C957F", "zte corporation" },
    { "2C9662", "Invenit BV" },
    { "2C9682", "MitraStar Technology Corp." },
    { "2C9717", "I.C.Y. B.V." },
    { "2C97B1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2C97ED", "Sony Imaging Products & Solutions Inc." },
    { "2C9811", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "2C9924", "Commscope" },
    { "2C9975", "Samsung Electronics Co.,Ltd" },
    { "2C9AA4", "Eolo SpA" },
    { "2C9C58", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "2C9D1E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2C9D4B", "Lavelle Networks Private Limited" },
    { "2C9D5A", "Flaircomm Microelectronics,Inc." },
    { "2C9D65", "vivo Mobile Communication Co., Ltd." },
    { "2C9E00", "Sony Interactive Entertainment Inc." },
    { "2C9E5F", "Commscope" },
    { "2C9EE0", "Cavli Inc." },
    { "2C9EEC", "Jabil Circuit Penang" },
    { "2C9EFC", "CANON INC." },
    { "2C9FFB", "Wistron Neweb Corporation" },
    { "2CA02F", "Veroguard Systems Pty Ltd" },
    { "2CA042", "Huawei Device Co., Ltd." },
    { "2CA157", "acromate, Inc." },
    { "2CA17D", "Commscope" },
    { "2CA2B4", "Fortify Technologies, LLC" },
    { "2CA30E", "POWER DRAGON DEVELOPMENT LIMITED" },
    { "2CA327", "Oraimo Technology Limited" },
    { "2CA539", "Parallel Wireless, Inc" },
    { "2CA59C", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "2CA774", "Texas Instruments" },
    { "2CA780", "True Technologies Inc." },
    { "2CA797", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2CA79E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2CA7EF", "OnePlus Technology (Shenzhen) Co., Ltd" },
    { "2CA835", "RIM" },
    { "2CA89C", "Creatz inc." },
    { "2CA9F0", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "2CAA8E", "Wyze Labs Inc" },
    { "2CAB00", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2CAB25", "SHENZHEN GONGJIN ELECTRONICS CO.,LT" },
    { "2CAB33", "Texas Instruments" },
    { "2CAB46", "Ruckus Wireless" },
    { "2CABA4", "Cisco SPVTG" },
    { "2CABEB", "Cisco Systems, Inc" },
    { "2CAC44", "CONEXTOP" },
    { "2CAD13", "SHENZHEN ZHILU TECHNOLOGY CO.,LTD" },
    { "2CAE2B", "Samsung Electronics Co.,Ltd" },
    { "2CB05D", "NETGEAR" },
    { "2CB0DF", "Soliton Technologies Pvt Ltd" },
    { "2CB0FD", "Shenzhen MiaoMing Intelligent Technology Co.,Ltd" },
    { "2CB115", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "2CB21A", "Phicomm (Shanghai) Co., Ltd." },
    { "2CB301", "Honor Device Co., Ltd." },
    { "2CB43A", "Apple, Inc." },
    { "2CB68F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2CB693", "Radware" },
    { "2CB69D", "RED Digital Cinema" },
    { "2CB6C2", "zte corporation" },
    { "2CB6C8", "Raisecom Technology CO., LTD" },
    { "2CB7A1", "Huawei Device Co., Ltd." },
    { "2CB8ED", "SonicWall" },
    { "2CBABA", "Samsung Electronics Co.,Ltd" },
    { "2CBACA", "Cosonic Electroacoustic Technology Co., Ltd." },
    { "2CBC87", "Apple, Inc." },
    { "2CBCBB", "Espressif Inc." },
    { "2CBE08", "Apple, Inc." },
    { "2CBE97", "Ingenieurbuero Bickele und Buehler GmbH" },
    { "2CBEEB", "Nothing Technology Limited" },
    { "2CBEEE", "Nothing Technology Limited" },
    { "2CC1F4", "Nokia Solutions and Networks India Private Limited" },
    { "2CC253", "Apple, Inc." },
    { "2CC260", "Oracle Corporation " },
    { "2CC3E6", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "2CC407", "machineQ" },
    { "2CC44F", "IEEE Registration Authority" },
    { "2CC546", "Huawei Device Co., Ltd." },
    { "2CC548", "IAdea Corporation" },
    { "2CC5D3", "Ruckus Wireless" },
    { "2CC682", "Quectel Wireless Solutions Co.,Ltd." },
    { "2CC6A0", "Lumacron Technology Ltd." },
    { "2CC81B", "Routerboard.com" },
    { "2CC8F5", "Huawei Device Co., Ltd." },
    { "2CCA0C", "WITHUS PLANET" },
    { "2CCA16", "Apple, Inc." },
    { "2CCA75", "Robert Bosch GmbH AnP" },
    { "2CCC15", "Nokia Corporation" },
    { "2CCC44", "Sony Interactive Entertainment Inc." },
    { "2CCC7A", "AltoBeam Inc." },
    { "2CCCE6", "Skyworth Digital Technology(Shenzhen) Co.,Ltd" },
    { "2CCD27", "Precor Inc" },
    { "2CCD43", "Summit Technology Group" },
    { "2CCD69", "Aqavi.com" },
    { "2CCE1E", "Cloudtronics Pty Ltd" },
    { "2CCF58", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2CCF67", "Raspberry Pi (Trading) Ltd" },
    { "2CD02D", "Cisco Systems, Inc" },
    { "2CD05A", "Liteon Technology Corporation" },
    { "2CD066", "Xiaomi Communications Co Ltd" },
    { "2CD141", "IEEE Registration Authority" },
    { "2CD1C6", "Murata Manufacturing Co., Ltd." },
    { "2CD1DA", "Keysight Technologies, Inc." },
    { "2CD26B", "FN-LINK TECHNOLOGY LIMITED" },
    { "2CD2E3", "Guangzhou Aoshi Electronic Co.,Ltd" },
    { "2CD2E7", "Nokia Corporation" },
    { "2CD3AD", "Texas Instruments" },
    { "2CD444", "FUJITSU LIMITED" },
    { "2CD7FF", "LANCOM Systems GmbH" },
    { "2CD8AE", "Shenzhen SEI Robotics Co.,Ltd" },
    { "2CD8DE", "AltoBeam Inc." },
    { "2CD974", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "2CDA3F", "DongGuan Ramaxel Memory Technology Limited" },
    { "2CDA46", "Samsung Electronics Co.,Ltd" },
    { "2CDB07", "Intel Corporate" },
    { "2CDC78", "Descartes Systems (USA) LLC" },
    { "2CDCAD", "Wistron Neweb Corporation" },
    { "2CDCC1", "EM Microelectronic" },
    { "2CDCD7", "AzureWave Technology Inc." },
    { "2CDD0C", "Discovergy GmbH" },
    { "2CDD5F", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "2CDD95", "Taicang T&W Electronics" },
    { "2CDDA3", "Point Grey Research Inc." },
    { "2CDDE9", "Arista Networks" },
    { "2CDEDF", "Guangxi Konaixin Precision Technology Co., Ltd" },
    { "2CDF68", "Apple, Inc." },
    { "2CDFE6", "Raisecom Technology CO., LTD" },
    { "2CE032", "TCL King Electrical Appliances(Huizhou)Co.,Ltd" },
    { "2CE099", "FUJIAN STAR-NET COMMUNICATION CO.,LTD" },
    { "2CE2A8", "DeviceDesign" },
    { "2CE2D9", "Huawei Device Co., Ltd." },
    { "2CE310", "Stratacache" },
    { "2CE38E", "Cisco Systems, Inc" },
    { "2CE412", "Sagemcom Broadband SAS" },
    { "2CE6CC", "Ruckus Wireless" },
    { "2CE871", "Alert Metalguard ApS" },
    { "2CEA7F", "Dell Inc." },
    { "2CEADA", "ICC Intelligent Platforms GmbH" },
    { "2CEADC", "ASKEY COMPUTER CORP" },
    { "2CECA6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2CECF7", "Shenzhen SDMC Technology CO.,Ltd." },
    { "2CED89", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2CEDB0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "2CEDEB", "Alpheus Digital Company Limited" },
    { "2CEE26", "Petroleum Geo-Services" },
    { "2CF05D", "Micro-Star INTL CO., LTD." },
    { "2CF0A2", "Apple, Inc." },
    { "2CF0EE", "Apple, Inc." },
    { "2CF1BB", "zte corporation" },
    { "2CF203", "EMKO ELEKTRONIK SAN VE TIC AS" },
    { "2CF295", "Huawei Device Co., Ltd." },
    { "2CF2A5", "Sagemcom Broadband SAS" },
    { "2CF432", "Espressif Inc." },
    { "2CF4C5", "Avaya Inc" },
    { "2CF7F1", "Seeed Technology Inc." },
    { "2CF814", "Cisco Systems, Inc" },
    { "2CF89B", "Cisco Systems, Inc" },
    { "2CFAA2", "Alcatel-Lucent Enterprise" },
    { "2CFB0F", "Sagemcom Broadband SAS" },
    { "2CFC8B", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "2CFCE4", "CTEK Sweden AB" },
    { "2CFD37", "Blue Calypso, Inc." },
    { "2CFDA1", "ASUSTek COMPUTER INC." },
    { "2CFDAB", "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd." },
    { "2CFDB3", "Tonly Technology Co. Ltd " },
    { "2CFDB4", "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd" },
    { "2CFE4F", "Xiaomi Communications Co Ltd" },
    { "2CFE8B", "Microchip Technologies Inc" },
    { "2CFEE2", "Qingdao Hisense Communications Co.,Ltd." },
    { "2CFF65", "Oki Electric Industry Co., Ltd." },
    { "2CFFEE", "vivo Mobile Communication Co., Ltd." },
    { "3000FC", "Nokia" },
    { "3001AF", "Cisco Systems, Inc" },
    { "3003C8", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "30045C", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "300475", "QBIC COMMUNICATIONS DMCC" },
    { "300505", "Intel Corporate" },
    { "30053F", "JTI Co.,Ltd." },
    { "30055C", "Brother industries, LTD." },
    { "30074D", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "3009C0", "Motorola Mobility LLC, a Lenovo Company" },
    { "3009F9", "IEEE Registration Authority" },
    { "300A60", "IEEE Registration Authority" },
    { "300A9D", "Axino Solutions AG" },
    { "300AC5", "Ruio telecommunication technologies Co., Limited" },
    { "300B9C", "Delta Mobile Systems, Inc." },
    { "300C23", "zte corporation" },
    { "300D2A", "Zhejiang Wellcom Technology Co.,Ltd." },
    { "300D43", "Microsoft Mobile Oy" },
    { "300D9E", "Ruijie Networks Co.,LTD" },
    { "300EB8", "LG Electronics" },
    { "300ED5", "Hon Hai Precision Ind. Co.,Ltd." },
    { "300EE3", "Aquantia Corporation" },
    { "3010B3", "Liteon Technology Corporation" },
    { "3010E4", "Apple, Inc." },
    { "301389", "Siemens AG, Automations & Drives," },
    { "30138B", "HP Inc." },
    { "30142D", "Piciorgros GmbH" },
    { "30144A", "Wistron Neweb Corporation" },
    { "301518", "Ubiquitous Communication Co. ltd." },
    { "30168D", "ProLon" },
    { "30169D", "MERCUSYS TECHNOLOGIES CO., LTD." },
    { "3017C8", "Sony Corporation" },
    { "3018CF", "DEOS control systems GmbH" },
    { "301966", "Samsung Electronics Co.,Ltd" },
    { "301984", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "301A28", "Mako Networks Ltd" },
    { "301A30", "Mako Networks Ltd" },
    { "301ABA", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "301B97", "Lierda Science & Technology Group Co.,Ltd" },
    { "301D49", "Firmus Technologies Pty Ltd" },
    { "301F48", "zte corporation" },
    { "301F9A", "IEEE Registration Authority" },
    { "30215B", "Shenzhen Ostar Display Electronic Co.,Ltd" },
    { "302303", "Belkin International Inc." },
    { "302364", "Nokia Shanghai Bell Co., Ltd." },
    { "3023BA", "Accelerated Memory Production Inc." },
    { "3023CD", "FUJIAN STAR-NET COMMUNICATION CO.,LTD" },
    { "302432", "Intel Corporate" },
    { "302450", "Hangzhou Huacheng Network Technology Co.,Ltd" },
    { "302478", "Sagemcom Broadband SAS" },
    { "3024A9", "HP Inc." },
    { "3027CF", "Canopy Growth Corp" },
    { "30292B", "eero inc." },
    { "30294B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "302952", "Hillstone Networks Inc" },
    { "3029BE", "Shanghai MRDcom Co.,Ltd" },
    { "302BDC", "Top-Unum Electronics Co., LTD" },
    { "302DE8", "JDA, LLC (JDA Systems)" },
    { "302F1E", "SIEMENS AG" },
    { "302FAC", "Zhejiang HuaRay Technology Co.,Ltd" },
    { "30305F", "Valeo Schalter und Sensoren GmbH" },
    { "3030D0", "Texas Instruments" },
    { "3030F9", "Espressif Inc." },
    { "30317D", "Hosiden Corporation" },
    { "303180", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "303235", "Qingdao Intelligent&Precise Electronics Co.,Ltd." },
    { "303294", "W-IE-NE-R Plein & Baus GmbH" },
    { "3032D4", "Hanilstm Co., Ltd." },
    { "303335", "Boosty" },
    { "303422", "eero inc." },
    { "3034D2", "Availink, Inc." },
    { "3034DB", "LG Innotek" },
    { "3034F6", "Vantiva Connected Home - Subcomponents" },
    { "3035AD", "Apple, Inc." },
    { "3035C5", "Huawei Device Co., Ltd." },
    { "3037A6", "Cisco Systems, Inc" },
    { "3037B3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "303855", "Nokia Corporation" },
    { "303926", "Sony Corporation" },
    { "303955", "Shenzhen Jinhengjia Electronic Co., Ltd." },
    { "3039A9", "Hongshan Information Science and Technology (HangZhou) Co.,Ltd." },
    { "3039F2", "ADB Broadband Italia" },
    { "303A4A", "eero inc." },
    { "303A64", "Intel Corporate" },
    { "303ABA", "Guangzhou BaoLun Electronics Co., Ltd" },
    { "303B49", "Cisco Meraki" },
    { "303B7C", "Apple, Inc." },
    { "303D08", "GLINTT TES S.A." },
    { "303D51", "IEEE Registration Authority" },
    { "303EA7", "Intel Corporate" },
    { "303EAD", "Sonavox Canada Inc" },
    { "303F5D", "PT HAN SUNG ELECTORONICS INDONESIA" },
    { "303F7B", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "303FBB", "Hewlett Packard Enterprise " },
    { "304074", "zte corporation" },
    { "304174", "ALTEC LANSING LLC" },
    { "304225", "BURG-WÄCHTER KG" },
    { "304240", "zte corporation" },
    { "3042A1", "ilumisys Inc. DBA Toggled" },
    { "3043D7", "IEEE Registration Authority" },
    { "304449", "PLATH Signal Products GmbH & Co. KG" },
    { "304487", "Hefei Radio Communication Technology Co., Ltd " },
    { "3044A1", "Shanghai Nanchao Information Technology" },
    { "304511", "Texas Instruments" },
    { "304596", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "30469A", "NETGEAR" },
    { "30493B", "Nanjing Z-Com Wireless Co.,Ltd" },
    { "304950", "IEEE Registration Authority" },
    { "30499E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "304A26", "Shenzhen Trolink Technology CO, LTD" },
    { "304AC4", "Barrot Technology Co.,LTD" },
    { "304B07", "Motorola Mobility LLC, a Lenovo Company" },
    { "304C7E", "Panasonic Electric Works Automation Controls Techno Co.,Ltd." },
    { "304E1B", "Huawei Device Co., Ltd." },
    { "304EC3", "Tianjin Techua Technology Co., Ltd." },
    { "304F00", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "304F75", "Zhone Technologies, Inc." },
    { "305075", "GN Audio A/S" },
    { "3050CE", "Xiaomi Communications Co Ltd" },
    { "3050F1", "Ennoconn Corporation." },
    { "3050FD", "Skyworth Digital Technology(Shenzhen) Co.,Ltd" },
    { "3051F8", "BYK-Gardner GmbH" },
    { "305223", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "305253", "BuildJet, Inc." },
    { "30525A", "NST Co., LTD" },
    { "3052CB", "Liteon Technology Corporation" },
    { "3053C1", "CRESYN" },
    { "3055ED", "Trex Network LLC" },
    { "30560F", "GIGA-BYTE TECHNOLOGY CO. , Ltd." },
    { "305684", "SHENZHEN YUNJI INTELLIGENT TECHNOLOGY CO.,LTD" },
    { "305696", "Infinix mobility limited" },
    { "305714", "Apple, Inc." },
    { "30578E", "eero inc." },
    { "3057AC", "IRLAB LTD." },
    { "305890", "Frontier Silicon Ltd" },
    { "3058EB", "zte corporation" },
    { "30595B", "streamnow AG" },
    { "3059B7", "Microsoft" },
    { "305A3A", "ASUSTek COMPUTER INC." },
    { "305A99", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "305D38", "Beissbarth " },
    { "305DA6", "ADVALY SYSTEM Inc." },
    { "305F77", "New H3C Technologies Co., Ltd" },
    { "30600A", "CIG SHANGHAI CO LTD" },
    { "306023", "Commscope" },
    { "306112", "PAV GmbH" },
    { "306118", "Paradom Inc." },
    { "3061A2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "30636B", "Apple, Inc." },
    { "306371", "Shenzhenshi Xinzhongxin Technology Co.Ltd" },
    { "3063CF", "Zhejiang Cainiao Supply Chain Management Co., Ltd" },
    { "3063EA", "Juniper Networks" },
    { "3065EC", "Wistron (ChongQing)" },
    { "3066D0", "Huawei Device Co., Ltd." },
    { "3067A1", "Sagemcom Broadband SAS" },
    { "30688C", "Reach Technology Inc." },
    { "306893", "TP-Link Systems Inc." },
    { "30694B", "RIM" },
    { "306A85", "Samsung Electronics Co.,Ltd" },
    { "306CBE", "Skymotion Technology (HK) Limited" },
    { "306DF9", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "306E5C", "Validus Technologies" },
    { "306F07", "Nations Technologies Inc." },
    { "3071B2", "Hangzhou Prevail Optoelectronic Equipment Co.,LTD." },
    { "307350", "Inpeco SA" },
    { "307467", "Samsung Electronics Co.,Ltd" },
    { "307496", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "307512", "Sony Corporation" },
    { "30766F", "LG Electronics (Mobile Communications)" },
    { "3077CB", "Maike Industry(Shenzhen)CO.,LTD" },
    { "30785C", "Partow Tamas Novin (Parman)" },
    { "30786B", "TIANJIN Golden Pentagon Electronics Co., Ltd." },
    { "3078C2", "Innowireless / QUCELL Networks" },
    { "3078D3", "Virgilant Technologies Ltd." },
    { "307A57", "Accuenergy (CANADA) Inc" },
    { "307BAC", "New H3C Technologies Co., Ltd" },
    { "307BC9", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "307C30", "RIM" },
    { "307C4A", "Huawei Device Co., Ltd." },
    { "307C5E", "Juniper Networks" },
    { "307CB2", "ANOV FRANCE" },
    { "307ECB", "SFR" },
    { "307F10", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "30809B", "New H3C Technologies Co., Ltd" },
    { "308216", "Apple, Inc." },
    { "308398", "Espressif Inc." },
    { "3083D2", "Motorola Mobility LLC, a Lenovo Company" },
    { "308454", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "3085A9", "ASUSTek COMPUTER INC." },
    { "3085EB", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "30862D", "Arista Network, Inc." },
    { "3086F1", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "308730", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3087D9", "Ruckus Wireless" },
    { "308841", "Sichuan AI-Link Technology Co., Ltd." },
    { "308944", "DEVA Broadcast Ltd." },
    { "30894A", "Intel Corporate" },
    { "308976", "DALIAN LAMBA TECHNOLOGY CO.,LTD" },
    { "308999", "Guangdong East Power Co.," },
    { "3089A6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3089D3", "HONGKONG UCLOUDLINK NETWORK TECHNOLOGY LIMITED" },
    { "308AF7", "Huawei Device Co., Ltd." },
    { "308BB2", "Cisco Systems, Inc" },
    { "308CFB", "Dropcam" },
    { "308D99", "Hewlett Packard" },
    { "308DD4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "308E7A", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "308ECF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "309048", "Apple, Inc." },
    { "3090AB", "Apple, Inc." },
    { "309176", "Skyworth Digital Technology(Shenzhen) Co.,Ltd" },
    { "30918F", "Technicolor Delivery Technologies Belgium NV" },
    { "3092F6", "SHANGHAI SUNMON COMMUNICATION TECHNOGY CO.,LTD" },
    { "3093BC", "Sagemcom Broadband SAS" },
    { "309435", "vivo Mobile Communication Co., Ltd." },
    { "309587", "HUNAN FN-LINK TECHNOLOGY LIMITED" },
    { "3095E3", "SHANGHAI SIMCOM LIMITED" },
    { "309610", "Huawei Device Co., Ltd." },
    { "30963B", "Huawei Device Co., Ltd." },
    { "3096FB", "Samsung Electronics Co.,Ltd" },
    { "309935", "zte corporation" },
    { "309BAD", "BBK EDUCATIONAL ELECTRONICS CORP.,LTD." },
    { "309C23", "Micro-Star INTL CO., LTD." },
    { "309E1D", "OHSUNG" },
    { "309E62", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "309FFB", "Ardomus Networks Corporation" },
    { "30A023", "ROCK PATH S.R.L" },
    { "30A176", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "30A1FA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "30A220", "ARG Telecom" },
    { "30A243", "Shenzhen Prifox Innovation Technology Co., Ltd." },
    { "30A2C2", "Huawei Device Co., Ltd." },
    { "30A30F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "30A3B5", "Jiangsu Best Tone Information Service Co., Ltd" },
    { "30A452", "Arrival Elements BV" },
    { "30A612", "ShenZhen Hugsun Technology Co.,Ltd." },
    { "30A889", "DECIMATOR DESIGN" },
    { "30A8DB", "Sony Corporation" },
    { "30A998", "Huawei Device Co., Ltd." },
    { "30A9DE", "LG Innotek" },
    { "30AABD", "Shanghai Reallytek Information Technology Co.,Ltd" },
    { "30AAE4", "Huawei Device Co., Ltd." },
    { "30AB6A", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "30ACED", "Packet Clearing House" },
    { "30AE7B", "Deqing Dusun Electron CO., LTD" },
    { "30AEA4", "Espressif Inc." },
    { "30AEF6", "Radio Mobile Access" },
    { "30AF7E", "Texas Instruments" },
    { "30AFCE", "vivo Mobile Communication Co., Ltd." },
    { "30B037", "New H3C Technologies Co., Ltd" },
    { "30B0EA", "Shenzhen Chuangxin Internet Communication Technology Co., Ltd" },
    { "30B164", "Power Electronics International Inc." },
    { "30B1B5", "Arcadyan Corporation" },
    { "30B216", "Hitachi Energy Germany AG" },
    { "30B237", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "30B29F", "EVIDENT CORPORATION" },
    { "30B346", "CJSC NORSI-TRANS" },
    { "30B3A2", "Shenzhen Heguang Measurement & Control Technology Co.,Ltd" },
    { "30B49E", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "30B4B8", "LG Electronics" },
    { "30B5C2", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "30B5F1", "Aitexin Technology Co., Ltd" },
    { "30B62D", "Mojo Networks, Inc." },
    { "30B64F", "Juniper Networks" },
    { "30B7D4", "Hitron Technologies. Inc" },
    { "30B851", "Siemens AG" },
    { "30B930", "zte corporation" },
    { "30B9B0", "Intracom Asia Co., Ltd" },
    { "30BB43", "Sixi Networks Co., Ltd" },
    { "30BB7D", "OnePlus Technology (Shenzhen) Co., Ltd" },
    { "30BD13", "Zyxel Communications Corporation" },
    { "30BDFE", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "30BE29", "AltoBeam Inc." },
    { "30BE3B", "Mitsubishi Electric Corporation" },
    { "30C01B", "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd" },
    { "30C0AE", "Apple, Inc." },
    { "30C3D9", "ALPSALPINE CO,.LTD" },
    { "30C507", "ECI Telecom Ltd." },
    { "30C50F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "30C599", "ASUSTek COMPUTER INC." },
    { "30C6AB", "zte corporation" },
    { "30C6D7", "New H3C Technologies Co., Ltd" },
    { "30C6F7", "Espressif Inc." },
    { "30C750", "MIC Technology Group" },
    { "30C7AE", "Samsung Electronics Co.,Ltd" },
    { "30C82A", "WI-BIZ srl" },
    { "30C91B", "Zhen Shi Information Technology（Shanghai）Co.,Ltd." },
    { "30C922", "Espressif Inc." },
    { "30C9AB", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "30C9CC", "Samsung Electronics Co.,Ltd" },
    { "30CB36", "Belden Singapore Pte. Ltd." },
    { "30CBC7", "Cambium Networks Limited" },
    { "30CBF8", "Samsung Electronics Co.,Ltd" },
    { "30CC21", "zte corporation" },
    { "30CDA7", "Samsung Electronics Co.,Ltd" },
    { "30D042", "Dell Inc." },
    { "30D16B", "Liteon Technology Corporation" },
    { "30D17E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "30D32D", "devolo AG" },
    { "30D357", "Logosol, Inc." },
    { "30D386", "zte corporation" },
    { "30D46A", "Autosales Incorporated" },
    { "30D4E2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "30D51F", "Prolights" },
    { "30D53E", "Apple, Inc." },
    { "30D587", "Samsung Electronics Co.,Ltd" },
    { "30D659", "Merging Technologies SA" },
    { "30D6C9", "Samsung Electronics Co.,Ltd" },
    { "30D7A1", "Apple, Inc." },
    { "30D875", "Apple, Inc." },
    { "30D941", "Raydium Semiconductor Corp." },
    { "30D959", "Shanghai Longcheer Technology Co., Ltd." },
    { "30D97F", "Tech4home, Lda" },
    { "30D9D9", "Apple, Inc." },
    { "30DCE7", "zte corporation" },
    { "30DDAA", "ZHEJIANG DAHUA TECHNOLOGYCO.,LTD" },
    { "30DE4B", "TP-Link Systems Inc" },
    { "30DE86", "Cedac Software S.r.l." },
    { "30DF17", "ALPSALPINE CO,.LTD" },
    { "30DF8D", "SHENZHEN GONGJIN ELECTRONICS CO.,LT" },
    { "30E04F", "Apple, Inc." },
    { "30E090", "Genevisio Ltd." },
    { "30E171", "Hewlett Packard" },
    { "30E1F1", "Intelbras" },
    { "30E271", "Fsas Technologies Inc." },
    { "30E283", "Texas Instruments" },
    { "30E37A", "Intel Corporate" },
    { "30E396", "Huawei Device Co., Ltd." },
    { "30E3A4", "Intel Corporate" },
    { "30E3D6", "Spotify USA Inc." },
    { "30E48E", "Vodafone UK" },
    { "30E4D8", "Huawei Device Co., Ltd." },
    { "30E4DB", "Cisco Systems, Inc" },
    { "30E7BC", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "30E8E4", "Qorvo International Pte. Ltd." },
    { "30E98E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "30EA26", "Sycada BV" },
    { "30EB15", "Huawei Device Co., Ltd." },
    { "30EB1F", "Skylab M&C Technology Co.,Ltd" },
    { "30EB25", "INTEK DIGITAL" },
    { "30EB5A", "LANDIS + GYR" },
    { "30EC7C", "Shenzhen Along Electronics Co., Ltd" },
    { "30ED96", "LS Mecapion" },
    { "30EDA0", "Espressif Inc." },
    { "30EFD1", "Alstom Strongwish (Shenzhen) Co., Ltd." },
    { "30F028", "Bosch Sicherheitssysteme GmbH" },
    { "30F23C", "TECNO MOBILE LIMITED" },
    { "30F31D", "zte corporation" },
    { "30F335", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "30F33A", "+plugg srl" },
    { "30F42F", "ESP" },
    { "30F527", "New H3C Technologies Co., Ltd" },
    { "30F600", "Sagemcom Broadband SAS" },
    { "30F6B9", "Ecocentric Energy" },
    { "30F6EF", "Intel Corporate" },
    { "30F70D", "Cisco Systems, Inc" },
    { "30F772", "Hon Hai Precision Ind. Co.,Ltd." },
    { "30F77F", "S Mobile Devices Limited" },
    { "30F7C5", "Apple, Inc." },
    { "30F7D7", "Thread Technology Co., Ltd" },
    { "30F856", "Extreme Networks Headquarters" },
    { "30F947", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "30F94B", "Universal Electronics, Inc." },
    { "30F9ED", "Sony Corporation" },
    { "30FAB7", "Tunai Creative" },
    { "30FB10", "Silicon Laboratories" },
    { "30FB68", "Wuhan Zmvision Technology Co. Ltd." },
    { "30FB94", "Shanghai Fangzhiwei Information Technology CO.,Ltd." },
    { "30FBB8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "30FC68", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "30FC8C", "Vantiva - Connected Home" },
    { "30FCEB", "LG Electronics (Mobile Communications)" },
    { "30FD11", "MACROTECH (USA) INC." },
    { "30FD38", "Google, Inc." },
    { "30FD65", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "30FE31", "Nokia" },
    { "30FE6C", "Apple, Inc." },
    { "30FEFA", "Cisco Systems, Inc" },
    { "30FFF6", "HangZhou KuoHeng Technology Co.,ltd" },
    { "30FFFD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "34008A", "IEEE Registration Authority" },
    { "3400A3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "340286", "Intel Corporate" },
    { "34029B", "Plexonics Technologies LImited" },
    { "3403DE", "Texas Instruments" },
    { "34049E", "IEEE Registration Authority" },
    { "340557", "vivo Mobile Communication Co., Ltd." },
    { "34074F", "AccelStor, Inc." },
    { "3407AC", "PRONYX TRADING LLC" },
    { "3407FB", "Ericsson AB" },
    { "340804", "D-Link Corporation" },
    { "3408BC", "Apple, Inc." },
    { "3408E1", "Texas Instruments" },
    { "340962", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "3409C9", "Dongguan Huayin Electronic Technology Co., Ltd." },
    { "340A22", "TOP-ACCESS ELECTRONICS CO LTD" },
    { "340A33", "D-Link International" },
    { "340A98", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "340AFF", "Qingdao Hisense Communications Co.,Ltd." },
    { "340B40", "MIOS ELETTRONICA SRL" },
    { "340CED", "Moduel AB" },
    { "340E22", "Apple, Inc." },
    { "340F66", "Web Sensing LLC" },
    { "34105D", "Texas Instruments" },
    { "3410BE", "Apple, Inc." },
    { "3410F4", "Silicon Laboratories" },
    { "341290", "Treeview Co.,Ltd." },
    { "341298", "Apple, Inc." },
    { "3412F9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "341343", "GE Lighting" },
    { "3413A8", "Mediplan Limited" },
    { "3413E8", "Intel Corporate" },
    { "341453", "Gantner Electronic GmbH" },
    { "34145F", "Samsung Electronics Co.,Ltd" },
    { "3414B5", "Texas Instruments" },
    { "341513", "Texas Instruments" },
    { "341593", "Ruckus Wireless" },
    { "34159E", "Apple, Inc." },
    { "341736", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "3417EB", "Dell Inc." },
    { "34194D", "Arcadyan Corporation" },
    { "341A35", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "341A4C", "SHENZHEN WEIBU ELECTRONICS CO.,LTD." },
    { "341B22", "Grandbeing Technology Co., Ltd" },
    { "341B2D", "Cisco Systems, Inc" },
    { "341CF0", "Xiaomi Communications Co Ltd" },
    { "341E6B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "341FE4", "Commscope" },
    { "342003", "Shenzhen Feitengyun Technology Co.,LTD" },
    { "3420E3", "Ruckus Wireless" },
    { "342109", "Jensen Scandinavia AS" },
    { "342387", "Hon Hai Precision Ind. Co.,Ltd." },
    { "3423BA", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "34243E", "zte corporation" },
    { "34255D", "Shenzhen Loadcom Technology Co.,Ltd" },
    { "3425B4", "Silicon Laboratories" },
    { "3425BE", "Amazon Technologies Inc." },
    { "342606", "CarePredict, Inc." },
    { "342792", "FREEBOX SAS" },
    { "342840", "Apple, Inc." },
    { "342865", "Juniper Networks" },
    { "3428F0", "ATN International Limited" },
    { "342912", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "34298F", "IEEE Registration Authority" },
    { "3429EA", "MCD ELECTRONICS SP. Z O.O." },
    { "3429EF", "Qingdao Haier Technology Co.,Ltd" },
    { "342AF1", "Texas Instruments" },
    { "342B6E", "Apple, Inc." },
    { "342B70", "Arris" },
    { "342C8E", "Ufispace Co., LTD." },
    { "342CC4", "Compal Broadband Networks, Inc." },
    { "342D0D", "Samsung Electronics Co.,Ltd" },
    { "342DA3", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "342EB6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "342EB7", "Intel Corporate" },
    { "342F6E", "Anywire corporation" },
    { "342FBD", "Nintendo Co.,Ltd" },
    { "343111", "Samsung Electronics Co.,Ltd" },
    { "34317F", "Panasonic Appliances Company" },
    { "34318F", "Apple, Inc." },
    { "3431C4", "AVM GmbH" },
    { "3432E6", "Panasonic Industrial Devices Europe GmbH" },
    { "343607", "PINEWAVE PTE. LTD." },
    { "34363B", "Apple, Inc." },
    { "343654", "zte corporation" },
    { "343759", "zte corporation" },
    { "343794", "Hamee Corp." },
    { "343839", "NEC Platforms, Ltd." },
    { "3438AF", "Inlab Networks GmbH" },
    { "3438B7", "HUMAX Co., Ltd." },
    { "343916", "Google, Inc." },
    { "343A20", "Hewlett Packard Enterprise" },
    { "343C30", "NXP Semiconductor (Tianjin) LTD." },
    { "343D7F", "Klipsch Group, Inc." },
    { "343D98", "JinQianMao Technology Co.,Ltd." },
    { "343DC4", "BUFFALO.INC" },
    { "343E25", "OHSUNG" },
    { "343EA4", "Ring LLC" },
    { "3440B5", "IBM" },
    { "34415D", "Intel Corporate" },
    { "3441A8", "ER-Telecom" },
    { "344262", "Apple, Inc." },
    { "344663", "IEEE Registration Authority" },
    { "34466F", "HiTEM Engineering" },
    { "3446EC", "Huawei Device Co., Ltd." },
    { "34479A", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "3447D4", "Chengdu Quanjing Intelligent Technology Co.,Ltd" },
    { "3448ED", "Dell Inc." },
    { "344951", "Eliyan Corp." },
    { "34495B", "Sagemcom Broadband SAS" },
    { "344A1B", "zte corporation" },
    { "344AC3", "HuNan ZiKun Information Technology CO., Ltd" },
    { "344B3D", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "344B50", "zte corporation" },
    { "344CA4", "amazipoint technology Ltd." },
    { "344CC8", "Echodyne Corp" },
    { "344DEA", "zte corporation" },
    { "344DF7", "LG Electronics (Mobile Communications)" },
    { "344E2F", "LEAR" },
    { "344F3F", "IO-Power Technology Co., Ltd." },
    { "344F5C", "R&amp;M AG" },
    { "344F69", "EKINOPS SAS" },
    { "34516F", "Skychers Creations ShenZhen Limited" },
    { "345180", "TCL King Electrical Appliances (Huizhou) Co., Ltd" },
    { "345184", "Huawei Device Co., Ltd." },
    { "3451AA", "JID GLOBAL" },
    { "3451C9", "Apple, Inc." },
    { "3453D2", "Sagemcom Broadband SAS" },
    { "34543C", "TAKAOKA TOKO CO.,LTD." },
    { "345506", "GUANGDONG GENIUS TECHNOLOGY CO., LTD." },
    { "345594", "FUJIAN STAR-NET COMMUNICATION CO.,LTD" },
    { "3456FE", "Cisco Meraki" },
    { "345760", "MitraStar Technology Corp." },
    { "345840", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "34587C", "MIRAE INFORMATION TECHNOLOGY CO., LTD." },
    { "34588A", "Cisco Systems, Inc" },
    { "345A06", "SHARP Corporation" },
    { "345A18", "Alignment Engine Inc." },
    { "345A60", "Micro-Star INTL CO., LTD." },
    { "345ABA", "tcloud intelligence" },
    { "345B11", "EVI HEAT AB" },
    { "345B98", "EM Microelectronic" },
    { "345BBB", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "345C40", "Cargt Holdings LLC" },
    { "345CF3", "Chipsea Technologies (Shenzhen) Corp." },
    { "345D10", "Wytek" },
    { "345D9E", "Sagemcom Broadband SAS" },
    { "345DA8", "Cisco Systems, Inc" },
    { "345E08", "Roku, Inc" },
    { "345EE7", "Hangzhou ChengFengErLai Digial Technology Co.,Ltd." },
    { "345F45", "Espressif Inc." },
    { "3460F9", "TP-Link Systems Inc" },
    { "346178", "The Boeing Company" },
    { "346288", "Cisco Systems, Inc" },
    { "3462B4", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "3463D4", "BIONIX SUPPLYCHAIN TECHNOLOGIES SLU" },
    { "3464A9", "Hewlett Packard" },
    { "346679", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "346691", "Apple, Inc." },
    { "3466EA", "VERTU INTERNATIONAL CORPORATION LIMITED" },
    { "34684A", "Teraworks Co., Ltd." },
    { "346893", "Tecnovideo Srl" },
    { "346895", "Hon Hai Precision Ind. Co.,Ltd." },
    { "3468B5", "Texas Instruments" },
    { "346987", "zte corporation" },
    { "346AC2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "346B46", "Sagemcom Broadband SAS" },
    { "346B5B", "New H3C Technologies Co., Ltd" },
    { "346BD3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "346C0F", "Pramod Telecom Pvt. Ltd" },
    { "346D9C", "Carrier Corporation" },
    { "346E68", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "346E8A", "Ecosense" },
    { "346E9D", "Ericsson AB" },
    { "346F11", "Qingdao Zhipai Information Technology Co., Ltd." },
    { "346F24", "AzureWave Technology Inc." },
    { "346F71", "TenaFe Inc. " },
    { "346F90", "Cisco Systems, Inc" },
    { "346F92", "White Rodgers Division" },
    { "346FED", "Enovation Controls" },
    { "347069", "Cisco Systems, Inc" },
    { "347146", "Huawei Device Co., Ltd." },
    { "34732D", "Cisco Systems, Inc" },
    { "34735A", "Dell Inc." },
    { "347379", "xFusion Digital Technologies Co., Limited" },
    { "347563", "SHENZHEN RF-LINK TECHNOLOGY CO.,LTD." },
    { "3475C7", "Avaya Inc" },
    { "3476C5", "I-O DATA DEVICE,INC." },
    { "347839", "zte corporation" },
    { "347877", "O-Net Communications (Shenzhen) Limited" },
    { "3478D7", "Gionee Communication Equipment Co.,Ltd." },
    { "347916", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "347A60", "Commscope" },
    { "347C25", "Apple, Inc." },
    { "347DE4", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "347DF6", "Intel Corporate" },
    { "347E00", "Huawei Device Co., Ltd." },
    { "347E39", "Nokia Danmark A/S" },
    { "347E5C", "Sonos, Inc." },
    { "347ECA", "NEXTWILL" },
    { "34800D", "Cavium Inc" },
    { "3480B3", "Xiaomi Communications Co Ltd" },
    { "348137", "UNICARD SA" },
    { "3481C4", "AVM GmbH" },
    { "3481F4", "SST Taiwan Ltd." },
    { "3482C5", "Samsung Electronics Co.,Ltd" },
    { "3482DE", "Kiio Inc" },
    { "348302", "iFORCOM Co., Ltd" },
    { "3483D5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "348446", "Ericsson AB" },
    { "3484E4", "Texas Instruments" },
    { "348511", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "348518", "Espressif Inc." },
    { "348584", "Extreme Networks Headquarters" },
    { "34862A", "Heinz Lackmann GmbH & Co KG" },
    { "34865D", "Espressif Inc." },
    { "3486DA", "Honor Device Co., Ltd." },
    { "34873D", "Quectel Wireless Solutions Co.,Ltd." },
    { "348818", "Cisco Systems, Inc" },
    { "34885D", "Logitech Far East" },
    { "348A12", "Hewlett Packard Enterprise" },
    { "348A7B", "Samsung Electronics Co.,Ltd" },
    { "348AAE", "Sagemcom Broadband SAS" },
    { "348B75", "LAVA INTERNATIONAL(H.K) LIMITED" },
    { "348C5E", "Apple, Inc." },
    { "348D13", "Silicon Laboratories" },
    { "348D52", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "348E89", "Hangzhou BroadLink Technology Co., Ltd" },
    { "348F27", "Ruckus Wireless" },
    { "3490EA", "Murata Manufacturing Co., Ltd." },
    { "34916F", "UserGate Ltd." },
    { "3492C2", "Square Route Co., Ltd." },
    { "349342", "TTE Corporation" },
    { "34936F", "Juniper Networks" },
    { "349454", "Espressif Inc." },
    { "3495DB", "Logitec Corporation" },
    { "349671", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "349672", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "349677", "zte corporation" },
    { "34976F", "Rootech, Inc." },
    { "3497D7", "YEALINK(XIAMEN) NETWORK TECHNOLOGY CO.,LTD." },
    { "3497F6", "ASUSTek COMPUTER INC." },
    { "3497FB", "ADVANCED RF TECHNOLOGIES INC" },
    { "34987A", "Espressif Inc." },
    { "3498B5", "NETGEAR" },
    { "34996F", "VPI Engineering" },
    { "349971", "Quanta Storage Inc." },
    { "3499D7", "Universal Flow Monitors, Inc." },
    { "349A0D", "ZBD Displays Ltd" },
    { "349B5B", "Maquet GmbH" },
    { "349D90", "Heinzmann GmbH & CO. KG" },
    { "349E34", "Evervictory Electronic Co.Ltd" },
    { "349F7B", "CANON INC." },
    { "34A183", "AWare, Inc" },
    { "34A2A2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "34A34E", "NevadaNano" },
    { "34A395", "Apple, Inc." },
    { "34A3BF", "Terewave. Inc." },
    { "34A55D", "TECHNOSOFT INTERNATIONAL SRL" },
    { "34A5B4", "NAVTECH PTE LTD" },
    { "34A5E1", "Sensorist ApS" },
    { "34A68C", "Shine Profit Development Limited" },
    { "34A6EF", "Shenzhen Phaten Tech. LTD" },
    { "34A709", "Trevil srl" },
    { "34A7BA", "Fischer International Systems Corporation" },
    { "34A843", "KYOCERA Display Corporation" },
    { "34A84E", "Cisco Systems, Inc" },
    { "34A8DB", "SenArch ApS" },
    { "34A8EB", "Apple, Inc." },
    { "34AA31", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "34AA8B", "Samsung Electronics Co.,Ltd" },
    { "34AA99", "Nokia" },
    { "34AAEE", "Mikrovisatos Servisas UAB" },
    { "34AB37", "Apple, Inc." },
    { "34AB95", "Espressif Inc." },
    { "34AC11", "China Mobile Group Device Co.,Ltd." },
    { "34AD61", "CELESTICA INC." },
    { "34ADE4", "Shanghai Chint Power Systems Co., Ltd." },
    { "34AF2C", "Nintendo Co., Ltd." },
    { "34AFA3", "Recogni Inc" },
    { "34AFB3", "Amazon Technologies Inc." },
    { "34B1EB", "Apple, Inc." },
    { "34B1F7", "Texas Instruments" },
    { "34B20A", "Huawei Device Co., Ltd." },
    { "34B354", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "34B472", "Espressif Inc." },
    { "34B571", "PLDS" },
    { "34B5A3", "CIG SHANGHAI CO LTD" },
    { "34B5F3", "IEEE Registration Authority" },
    { "34B7DA", "Espressif Inc." },
    { "34B7FD", "Guangzhou Younghead Electronic Technology Co.,Ltd" },
    { "34B883", "Cisco Systems, Inc" },
    { "34B98D", "Xiaomi Communications Co Ltd" },
    { "34BA38", "PAL MOHAN ELECTRONICS PVT LTD" },
    { "34BA51", "Se-Kure Controls, Inc." },
    { "34BA75", "Everest Networks, Inc" },
    { "34BA9A", "Asiatelco Technologies Co." },
    { "34BB1F", "BlackBerry RTS" },
    { "34BB26", "Motorola Mobility LLC, a Lenovo Company" },
    { "34BC5E", "eero inc." },
    { "34BCA6", "Beijing Ding Qing Technology, Ltd." },
    { "34BD20", "Hangzhou Hikrobot Technology Co., Ltd." },
    { "34BDC8", "Cisco Systems, Inc" },
    { "34BDF9", "Shanghai WDK Industrial Co.,Ltd." },
    { "34BDFA", "Cisco SPVTG" },
    { "34BE00", "Samsung Electronics Co.,Ltd" },
    { "34BF90", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "34C059", "Apple, Inc." },
    { "34C0F9", "Rockwell Automation" },
    { "34C103", "Hangzhou Huamu Technology Co.,Ltd." },
    { "34C1E9", "Ulak Communications Inc." },
    { "34C3AC", "Samsung Electronics Co.,Ltd" },
    { "34C3D2", "FN-LINK TECHNOLOGY LIMITED" },
    { "34C459", "Texas Instruments" },
    { "34C515", "Hewlett Packard Enterprise" },
    { "34C5D0", "Hagleitner Hygiene International GmbH" },
    { "34C69A", "Enecsys Ltd" },
    { "34C6DD", "Hangzhou Ezviz Software Co.,Ltd." },
    { "34C731", "ALPSALPINE CO,.LTD" },
    { "34C7E9", "Google, Inc." },
    { "34C803", "Nokia Corporation" },
    { "34C8D6", "IEEE Registration Authority" },
    { "34C93D", "Intel Corporate" },
    { "34C99D", "EIDOLON COMMUNICATIONS TECHNOLOGY CO. LTD." },
    { "34C9F0", "LM Technologies Ltd" },
    { "34CA81", "New H3C Intelligence Terminal Co., Ltd." },
    { "34CB1A", "Procter & Gamble Company" },
    { "34CC28", "Nexpring Co. LTD.," },
    { "34CD6D", "CommSky Technologies" },
    { "34CDB0", "Espressif Inc." },
    { "34CDBE", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "34CE00", "XIAOMI Electronics,CO.,LTD" },
    { "34CE69", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "34CE94", "Parsec (Pty) Ltd" },
    { "34CF6C", "Hangzhou Taili wireless communication equipment Co.,Ltd" },
    { "34CFB5", "Robotic d.o.o." },
    { "34CFF6", "Intel Corporate" },
    { "34D09B", "MobilMAX Technology Inc." },
    { "34D0B8", "IEEE Registration Authority" },
    { "34D262", "SZ DJI TECHNOLOGY CO.,LTD" },
    { "34D270", "Amazon Technologies Inc." },
    { "34D2C4", "RENA GmbH Print Systeme" },
    { "34D4E3", "Atom Power, Inc." },
    { "34D509", "Genexis B.V." },
    { "34D693", "Huawei Device Co., Ltd." },
    { "34D712", "Smartisan Digital Co., Ltd" },
    { "34D72F", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "34D737", "IBG Industriebeteiligungsgesellschaft mbH &b Co. KG" },
    { "34D772", "Xiamen Yudian Automation Technology Co., Ltd " },
    { "34D7B4", "Tributary Systems, Inc." },
    { "34D856", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "34D868", "Broadcom Limited" },
    { "34D954", "WiBotic Inc." },
    { "34DAA1", "Apple, Inc." },
    { "34DAB7", "zte corporation" },
    { "34DAC1", "SAE Technologies Development(Dongguan) Co., Ltd." },
    { "34DB9C", "Sagemcom Broadband SAS" },
    { "34DBFD", "Cisco Systems, Inc" },
    { "34DC99", "New H3C Technologies Co., Ltd" },
    { "34DD04", "Minut AB" },
    { "34DD7E", "Umeox Innovations Co.,Ltd" },
    { "34DE1A", "Intel Corporate" },
    { "34DE34", "zte corporation" },
    { "34DF20", "Shenzhen Comstar .Technology Co.,Ltd" },
    { "34DF2A", "Fujikon Industrial Co.,Limited" },
    { "34E0CF", "zte corporation" },
    { "34E0D7", "DONGGUAN QISHENG ELECTRONICS INDUSTRIAL CO., LTD" },
    { "34E12D", "Intel Corporate" },
    { "34E1A9", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "34E1D1", "IEEE Registration Authority" },
    { "34E2FD", "Apple, Inc." },
    { "34E380", "Genexis B.V." },
    { "34E3DA", "Hoval Aktiengesellschaft" },
    { "34E3FB", "Samsung Electronics Co.,Ltd" },
    { "34E42A", "Automatic Bar Controls Inc." },
    { "34E5EC", "Palo Alto Networks" },
    { "34E6AD", "Intel Corporate" },
    { "34E6D7", "Dell Inc." },
    { "34E6E6", "LG Innotek" },
    { "34E70B", "HAN Networks Co., Ltd" },
    { "34E71C", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "34E894", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "34E911", "vivo Mobile Communication Co., Ltd." },
    { "34E9FE", "Metis Co., Ltd." },
    { "34EA10", "Orbic North America" },
    { "34EA34", "HangZhou Gubei Electronics Technology Co.,Ltd" },
    { "34EAE7", "Shanghai High-Flying Electronics Technology Co., Ltd" },
    { "34ECB6", "Phyplus Microelectronics Limited" },
    { "34ED0B", "Shanghai XZ-COM.CO.,Ltd." },
    { "34ED1B", "Cisco Systems, Inc" },
    { "34EE16", "Apple, Inc." },
    { "34EE2A", "ConMet" },
    { "34EF44", "2Wire Inc" },
    { "34EF8B", "NTT Communications Corporation" },
    { "34EFB6", "Edgecore Networks Corporation" },
    { "34F043", "Samsung Electronics Co.,Ltd" },
    { "34F0CA", "Shenzhen Linghangyuan Digital Technology Co.,Ltd." },
    { "34F150", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "34F223", "Fujian Newland Communication Science Technology Co.,Ltd." },
    { "34F39A", "Intel Corporate" },
    { "34F39B", "WizLAN Ltd." },
    { "34F5D7", "Huawei Device Co., Ltd." },
    { "34F62D", "SHARP Corporation" },
    { "34F64B", "Intel Corporate" },
    { "34F68D", "Apple, Inc." },
    { "34F6D2", "Panasonic Taiwan Co.,Ltd." },
    { "34F716", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "34F86E", "Parker Hannifin Corporation" },
    { "34F8E7", "Cisco Systems, Inc" },
    { "34F968", "ATEK Products, LLC" },
    { "34FA1C", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "34FA40", "Guangzhou Robustel Technologies Co., Limited" },
    { "34FA9F", "Ruckus Wireless" },
    { "34FC6F", "ALCEA" },
    { "34FC99", "SJIT Co., Ltd." },
    { "34FCA1", "Micronet union Technology(Chengdu)Co., Ltd." },
    { "34FCB9", "Hewlett Packard Enterprise " },
    { "34FCEF", "LG Electronics (Mobile Communications)" },
    { "34FD6A", "Apple, Inc." },
    { "34FD70", "Intel Corporate" },
    { "34FE1C", "CHOUNG HWA TECH CO.,LTD" },
    { "34FE77", "Apple, Inc." },
    { "34FE9E", "Fujitsu Limited" },
    { "34FEC5", "Shenzhen Sunwoda intelligent hardware Co.,Ltd" },
    { "34FFF3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "380025", "Intel Corporate" },
    { "380118", "ULVAC,Inc." },
    { "380146", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "380195", "Samsung Electronics Co.,Ltd" },
    { "380197", "TSST Global,Inc" },
    { "38019F", "SHENZHEN FAST TECHNOLOGIES CO.,LTD" },
    { "3802DE", "Sercomm Corporation." },
    { "3802E3", "YICHEN (SHENZHEN) TECHNOLOGY CO.,LTD" },
    { "380525", "IEEE Registration Authority" },
    { "380546", "Foctek Photonics, Inc." },
    { "3805AC", "Piller Group GmbH" },
    { "3806B4", "A.D.C. GmbH" },
    { "3806E6", "Arcadyan Corporation" },
    { "380716", "FREEBOX SAS" },
    { "3807D4", "Zeppelin Systems GmbH" },
    { "3808FD", "Silca Spa" },
    { "3809A4", "Firefly Integrations" },
    { "3809FB", "Apple, Inc." },
    { "380A0A", "Sky-City Communication and Electronics Limited Company" },
    { "380A4F", "PRACHI ENTERPRISES" },
    { "380A94", "Samsung Electronics Co.,Ltd" },
    { "380AAB", "Formlabs" },
    { "380B26", "Mindray Co., Ltd." },
    { "380B3C", "Texas Instruments" },
    { "380B40", "Samsung Electronics Co.,Ltd" },
    { "380DD4", "Primax Electronics Ltd." },
    { "380E4D", "Cisco Systems, Inc" },
    { "380E7B", "V.P.S. Thai Co., Ltd" },
    { "380F4A", "Apple, Inc." },
    { "380FAD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "380FE4", "Dedicated Network Partners Oy" },
    { "3810D5", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "3810F0", "Hewlett Packard Enterprise" },
    { "38127B", "Crenet Labs Co., Ltd." },
    { "38141B", "Secure Letter Inc." },
    { "381428", "Dell Inc." },
    { "38144E", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "38165A", "zte corporation" },
    { "381672", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "3816D1", "Samsung Electronics Co.,Ltd" },
    { "381730", "Ulrich Lippert GmbH & Co KG" },
    { "381766", "PROMZAKAZ LTD." },
    { "3817B1", "Sagemcom Broadband SAS" },
    { "3817C3", "Hewlett Packard Enterprise " },
    { "3817E1", "Vantiva USA LLC" },
    { "38182B", "Espressif Inc." },
    { "38184C", "Sony Home Entertainment&Sound Products Inc" },
    { "381868", "Intel Corporate" },
    { "38192F", "Nokia Corporation" },
    { "381A52", "Seiko Epson Corporation" },
    { "381B9E", "Qingdao Intelligent&Precise Electronics Co.,Ltd." },
    { "381C1A", "Cisco Systems, Inc" },
    { "381C23", "Hilan Technology CO.,LTD" },
    { "381C4A", "SIMCom Wireless Solutions Co.,Ltd." },
    { "381D14", "Skydio Inc." },
    { "381DD9", "FN-LINK TECHNOLOGY LIMITED" },
    { "381EC7", "Chipsea Technologies(Shenzhen) Corp." },
    { "381F26", "IEEE Registration Authority" },
    { "381F8D", "Tuya Smart Inc." },
    { "382028", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "382056", "Cisco Systems, Inc" },
    { "3820A8", "ColorTokens, Inc." },
    { "382187", "Midea Group Co., Ltd." },
    { "3821C7", "Hewlett Packard Enterprise" },
    { "382228", "Telink Micro LLC" },
    { "38229D", "ADB Broadband Italia" },
    { "3822D6", "Hangzhou H3C Technologies Co., Limited" },
    { "3822E2", "HP Inc." },
    { "3822F4", "Huawei Device Co., Ltd." },
    { "3824F1", "Private" },
    { "38256B", "Microsoft Mobile Oy" },
    { "3825F3", "Mellanox Technologies, Inc." },
    { "38262B", "UTran Technology" },
    { "382656", "TCL King Electrical Appliances(Huizhou)Co.,Ltd" },
    { "3826CD", "ANDTEK" },
    { "382835", "zte corporation" },
    { "3828EA", "Fujian Netcom Technology Co., LTD" },
    { "38295A", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "3829DD", "ONvocal Inc" },
    { "382A19", "Technica Engineering GmbH" },
    { "382A8B", "nFore Technology Co., Ltd." },
    { "382A8C", "ALL Winner (Hong Kong) Limited" },
    { "382B78", "ECO PLUGS ENTERPRISE CO., LTD" },
    { "382C4A", "ASUSTek COMPUTER INC." },
    { "382CE5", "Tuya Smart Inc." },
    { "382DD1", "Samsung Electronics Co.,Ltd" },
    { "382DE8", "Samsung Electronics Co.,Ltd" },
    { "3830F9", "LG Electronics (Mobile Communications)" },
    { "38315A", "Rinnai" },
    { "3831AC", "WEG" },
    { "3835FB", "Sagemcom Broadband SAS" },
    { "38378B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "38384B", "vivo Mobile Communication Co., Ltd." },
    { "3838A6", "Arista Networks" },
    { "38396C", "Huawei Device Co., Ltd." },
    { "38398F", "Silicon Laboratories" },
    { "3839CD", "vivo Mobile Communication Co., Ltd." },
    { "383A21", "IEEE Registration Authority" },
    { "383B26", "Jiangsu Qinheng Co., Ltd." },
    { "383BC8", "2Wire Inc" },
    { "383C9C", "Fujian Newland Payment Technology Co.,Ltd." },
    { "383D5B", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "383F10", "DBL Technology Ltd." },
    { "383FB3", "Vantiva USA LLC" },
    { "383FE8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "38420B", "Sonos, Inc." },
    { "384233", "Wildeboer Bauteile GmbH" },
    { "3842A6", "Ingenieurbuero Stahlkopf" },
    { "384369", "Patrol Products Consortium LLC" },
    { "38437D", "Compal Broadband Networks, Inc." },
    { "3843E5", "Grotech Inc" },
    { "38453B", "Ruckus Wireless" },
    { "38454C", "Light Labs, Inc." },
    { "384554", "Harman/Becker Automotive Systems GmbH" },
    { "38458C", "MyCloud Technology corporation" },
    { "384608", "zte corporation" },
    { "3847BC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3847F2", "Recogni Inc" },
    { "38484C", "Apple, Inc." },
    { "384A80", "Samsung Electronics Co.,Ltd" },
    { "384B24", "SIEMENS AG" },
    { "384B5B", "ZTRON TECHNOLOGY LIMITED" },
    { "384B76", "AIRTAME ApS" },
    { "384C4F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "384C90", "Commscope" },
    { "384DD2", "Huawei Device Co., Ltd." },
    { "384F49", "Juniper Networks" },
    { "384FF0", "AzureWave Technology Inc." },
    { "38521A", "Nokia" },
    { "385247", "Huawei Device Co., Ltd." },
    { "385319", "34ED LLC DBA Centegix" },
    { "38539C", "Apple, Inc." },
    { "385439", "Guangzhou Shiyuan Electronic Technology Company Limited" },
    { "38549B", "zte corporation" },
    { "3854F5", "AltoBeam Inc." },
    { "385610", "CANDY HOUSE, Inc." },
    { "38563D", "Microsoft Corporation" },
    { "3856B5", "Peerbridge Health Inc" },
    { "38580C", "Panaccess Systems GmbH" },
    { "3859F8", "MindMade Sp. z o.o." },
    { "3859F9", "Hon Hai Precision Ind. Co.,Ltd." },
    { "385AA8", "Beijing Zhongdun Security Technology Development Co." },
    { "385B44", "Silicon Laboratories" },
    { "385C76", "PLANTRONICS, INC." },
    { "385CFB", "Silicon Laboratories" },
    { "385F66", "Cisco SPVTG" },
    { "385FC3", "Yu Jeong System, Co.Ltd" },
    { "386077", "PEGATRON CORPORATION" },
    { "3861A5", "Grabango Co" },
    { "386233", "Apple, Inc." },
    { "38637F", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "3863BB", "Hewlett Packard" },
    { "3863F6", "3NOD MULTIMEDIA(SHENZHEN)CO.,LTD" },
    { "386407", "Qingdao Intelligent&Precise Electronics Co.,Ltd." },
    { "386504", "Honor Device Co., Ltd." },
    { "3865B2", "Apple, Inc." },
    { "386645", "OOSIC Technology CO.,Ltd" },
    { "3866F0", "Apple, Inc." },
    { "386793", "Asia Optical Co., Inc." },
    { "386893", "Intel Corporate" },
    { "3868A4", "Samsung Electronics Co.,Ltd" },
    { "3868BE", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "3868DD", "INVENTEC CORPORATION" },
    { "386A77", "Samsung Electronics Co.,Ltd" },
    { "386B1C", "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "386BBB", "Commscope" },
    { "386C9B", "Ivy Biomedical" },
    { "386E21", "Wasion Group Ltd." },
    { "386E88", "zte corporation" },
    { "386EA2", "vivo Mobile Communication Co., Ltd." },
    { "386EB2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "386F6B", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "38700C", "Commscope" },
    { "3870F2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3871DE", "Apple, Inc." },
    { "3872C0", "Comtrend Corporation" },
    { "3873EA", "IEEE Registration Authority" },
    { "387605", "Inogeni" },
    { "3876CA", "Shenzhen Smart Intelligent Technology Co.Ltd" },
    { "3876D1", "Euronda SpA" },
    { "387707", "AltoBeam Inc." },
    { "3877CD", "KOKUSAI ELECTRIC CORPORATION" },
    { "387862", "Sony Corporation" },
    { "387A0E", "Intel Corporate" },
    { "387A3C", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "387ACC", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "387B01", "Shenzhen MiaoMing Intelligent Technology Co.,Ltd" },
    { "387B47", "AKELA, Inc." },
    { "387C76", "Universal Global Scientific Industrial Co., Ltd." },
    { "387F8B", "Apple, Inc." },
    { "3880DF", "Motorola Mobility LLC, a Lenovo Company" },
    { "3881D7", "Texas Instruments" },
    { "388345", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "38839A", "SHENZHEN RF-LINK TECHNOLOGY CO.,LTD." },
    { "388479", "Cisco Meraki" },
    { "388602", "Flexoptix GmbH" },
    { "3886F7", "Google, Inc." },
    { "3887D5", "Intel Corporate" },
    { "38881E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "388871", "ASKEY COMPUTER CORP" },
    { "3888A4", "Apple, Inc." },
    { "38892C", "Apple, Inc." },
    { "3889DC", "Opticon Sensors Europe B.V." },
    { "388A06", "Samsung Electronics Co.,Ltd" },
    { "388A21", "UAB Teltonika Telematics" },
    { "388AB7", "ITC Networks" },
    { "388ABE", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "388B59", "Google, Inc." },
    { "388C50", "LG Electronics" },
    { "388D3D", "Wistron Neweb Corporation" },
    { "388E7A", "AUTOIT" },
    { "388EE7", "Fanhattan LLC" },
    { "388F30", "Samsung Electronics Co.,Ltd" },
    { "389052", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3890A5", "Cisco Systems, Inc" },
    { "3890AF", "zte corporation" },
    { "389148", "zte corporation" },
    { "3891B7", "Cisco Systems, Inc" },
    { "3891D5", "Hangzhou H3C Technologies Co., Limited" },
    { "3891FB", "Xenox Holding BV" },
    { "389201", "Tianyi Telecom Terminals Company Limited" },
    { "38922E", "ArrayComm" },
    { "389231", "TECNO MOBILE LIMITED" },
    { "389461", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "389496", "Samsung Electronics Co.,Ltd" },
    { "3894E0", "Syrotech Networks. Ltd." },
    { "3894ED", "NETGEAR" },
    { "389592", "Tendyron Corporation" },
    { "3897A4", "ELECOM CO.,LTD." },
    { "3897D6", "Hangzhou H3C Technologies Co., Limited" },
    { "3898D8", "MERITECH CO.,LTD" },
    { "3898E9", "Huawei Device Co., Ltd." },
    { "389AF6", "Samsung Electronics Co.,Ltd" },
    { "389CB2", "Apple, Inc." },
    { "389D92", "Seiko Epson Corporation" },
    { "389E4C", "Hewlett Packard Enterprise" },
    { "389E80", "zte corporation" },
    { "389F5A", "C-Kur TV Inc." },
    { "389F83", "OTN Systems N.V." },
    { "38A067", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "38A28C", "SHENZHEN RF-LINK TECHNOLOGY CO.,LTD." },
    { "38A44B", "Huawei Device Co., Ltd." },
    { "38A4ED", "Xiaomi Communications Co Ltd" },
    { "38A53C", "COMECER Netherlands" },
    { "38A5B6", "SHENZHEN MEGMEET ELECTRICAL CO.,LTD" },
    { "38A5C9", "Tuya Smart Inc." },
    { "38A659", "Sagemcom Broadband SAS" },
    { "38A6CE", "SKY UK LIMITED" },
    { "38A746", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "38A851", "Quickset Defense Technologies, LLC" },
    { "38A86B", "Orga BV" },
    { "38A89B", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "38A8CD", "IEEE Registration Authority" },
    { "38A91C", "New H3C Technologies Co., Ltd" },
    { "38A95F", "Actifio Inc" },
    { "38A9EA", "HK DAPU ELECTRONIC TECHNOLOGY CO., LIMITED" },
    { "38AA09", "Cisco Systems, Inc" },
    { "38AA20", "zte corporation" },
    { "38AA3C", "SAMSUNG ELECTRO MECHANICS CO., LTD." },
    { "38AB16", "NPO RTT LLC" },
    { "38AB41", "Texas Instruments" },
    { "38AC3D", "Nephos Inc" },
    { "38ACDD", "Valenco GmbH" },
    { "38AD2B", "Hitron Technologies. Inc" },
    { "38AD8E", "New H3C Technologies Co., Ltd" },
    { "38ADBE", "New H3C Technologies Co., Ltd" },
    { "38AF29", "Zhejiang Dahua Technology Co., Ltd." },
    { "38AFD0", "Nevro" },
    { "38AFD7", "FUJITSU LIMITED" },
    { "38B12D", "Sonotronic Nagel GmbH" },
    { "38B19E", "IEEE Registration Authority" },
    { "38B1DB", "Hon Hai Precision Ind. Co.,Ltd." },
    { "38B3F7", "Huawei Device Co., Ltd." },
    { "38B4D3", "BSH Hausgeraete GmbH" },
    { "38B54D", "Apple, Inc." },
    { "38B5BD", "E.G.O. Elektro-Ger" },
    { "38B5C9", "INGRAM MICRO SERVICES" },
    { "38B5D3", "SecuWorks" },
    { "38B725", "Wistron Infocomm (Zhongshan) Corporation" },
    { "38B74D", "Fijowave Limited" },
    { "38B800", "Wistron Neweb Corporation" },
    { "38B8EB", "IEEE Registration Authority" },
    { "38B9AF", "NXP Semiconductors Taiwan Ltd." },
    { "38BAB0", "Broadcom" },
    { "38BAF8", "Intel Corporate" },
    { "38BB23", "OzVision America LLC" },
    { "38BB3C", "Avaya Inc" },
    { "38BC01", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "38BC1A", "MEIZU Technology Co., Ltd." },
    { "38BC61", "Starkoff Co., Ltd." },
    { "38BD7A", "Hewlett Packard Enterprise" },
    { "38BEAB", "AltoBeam (China) Inc." },
    { "38BF2F", "Espec Corp." },
    { "38BF33", "NEC CASIO Mobile Communications" },
    { "38C096", "ALPSALPINE CO,.LTD" },
    { "38C0EA", "Fortinet, Inc." },
    { "38C2BA", "CCTV NEOTECH" },
    { "38C43A", "Apple, Inc." },
    { "38C4E8", "NSS Sp. z o.o." },
    { "38C6BD", "Xiaomi Communications Co Ltd" },
    { "38C6CE", "Nintendo Co.,Ltd" },
    { "38C70A", "WiFiSong" },
    { "38C7BA", "CS Services Co.,Ltd." },
    { "38C804", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "38C85C", "Cisco SPVTG" },
    { "38C986", "Apple, Inc." },
    { "38C9A9", "SMART High Reliability Solutions, Inc." },
    { "38CA73", "Shenzhen MiaoMing Intelligent Technology Co.,Ltd" },
    { "38CA84", "HP Inc." },
    { "38CA97", "Contour Design LLC" },
    { "38CADA", "Apple, Inc." },
    { "38CD07", "Beijing FaceCam Technology Co., Ltd." },
    { "38D09C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "38D135", "EasyIO Corporation Sdn. Bhd." },
    { "38D269", "Texas Instruments" },
    { "38D2CA", "Zhejiang Tmall Technology Co., Ltd." },
    { "38D40B", "Samsung Electronics Co.,Ltd" },
    { "38D518", "GooWi Wireless Technology Co., Limited" },
    { "38D547", "ASUSTek COMPUTER INC." },
    { "38D57A", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "38D620", "Limidea Concept Pte. Ltd." },
    { "38D6E0", "TOPDON TECHNOLOGY Co.,Ltd." },
    { "38D7CA", "7HUGS LABS" },
    { "38D82F", "zte corporation" },
    { "38D9A5", "Mikotek Information Inc. " },
    { "38DBBB", "Sunbow Telecom Co., Ltd." },
    { "38DE35", "GUANGZHOU YUANDIANHE COMMUNICATION TECHNOLOGY CO.,LTD" },
    { "38DE60", "Mohlenhoff GmbH" },
    { "38DEAD", "Intel Corporate" },
    { "38E08E", "Mitsubishi Electric Corporation" },
    { "38E13D", "Apple, Inc." },
    { "38E1AA", "zte corporation" },
    { "38E1F4", "Sagemcom Broadband SAS" },
    { "38E26E", "ShenZhen Sweet Rain Electronics Co.,Ltd." },
    { "38E2C4", "Texas Instruments" },
    { "38E2CA", "Katun Corporation" },
    { "38E2DD", "zte corporation" },
    { "38E39F", "Motorola Mobility LLC, a Lenovo Company" },
    { "38E3C5", "Taicang T&W Electronics" },
    { "38E563", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "38E595", "SHENZHEN GONGJIN ELECTRONICS CO.,LT" },
    { "38E60A", "Xiaomi Communications Co Ltd" },
    { "38E7C0", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "38E7D8", "HTC Corporation" },
    { "38E8DF", "b gmbh medien + datenbanken" },
    { "38E8EE", "Nanjing Youkuo Electric Technology Co., Ltd" },
    { "38E98C", "Reco S.p.A." },
    { "38EAA7", "Hewlett Packard" },
    { "38EB47", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "38EC0D", "Apple, Inc." },
    { "38EC11", "Novatek Microelectronics Corp." },
    { "38ECE4", "Samsung Electronics Co.,Ltd" },
    { "38ED18", "Cisco Systems, Inc" },
    { "38EE9D", "Anedo Ltd." },
    { "38EFE3", "INGENICO TERMINALS SAS" },
    { "38F098", "Vapor Stone Rail Systems" },
    { "38F0BB", "CompuSoft A/S" },
    { "38F0C8", "Logitech" },
    { "38F135", "SensorTec-Canada" },
    { "38F18F", "Technicolor (China) Technology Co., Ltd." },
    { "38F195", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "38F20D", "Juniper Networks" },
    { "38F23E", "Microsoft Mobile Oy" },
    { "38F32E", "Skullcandy" },
    { "38F33F", "TATSUNO CORPORATION" },
    { "38F3AB", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "38F3FB", "Asperiq" },
    { "38F45E", "H1-Radio co.,ltd" },
    { "38F554", "HISENSE ELECTRIC CO.,LTD" },
    { "38F557", "JOLATA, INC." },
    { "38F597", "home2net GmbH" },
    { "38F601", "Solid State Storage Technology Corporation" },
    { "38F6CF", "zte corporation" },
    { "38F6ED", "EVK DI Kerschhaggl GmbH" },
    { "38F708", "National Resource Management, Inc." },
    { "38F73D", "Amazon Technologies Inc." },
    { "38F7B2", "SEOJUN ELECTRIC" },
    { "38F7CD", "IEEE Registration Authority" },
    { "38F7F1", "Huawei Device Co., Ltd." },
    { "38F85E", "HUMAX Co., Ltd." },
    { "38F889", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "38F8B7", "V2COM PARTICIPACOES S.A." },
    { "38F8CA", "OWIN Inc." },
    { "38F8F6", "Adtran Inc" },
    { "38F9D3", "Apple, Inc." },
    { "38F9F5", "Garmin International" },
    { "38FACA", "Skyworth Digital Technology(Shenzhen) Co.,Ltd" },
    { "38FB14", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "38FC34", "Huawei Device Co., Ltd." },
    { "38FC98", "Intel Corporate" },
    { "38FDF5", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "38FDF8", "Cisco Systems, Inc" },
    { "38FDFE", "IEEE Registration Authority" },
    { "38FEC5", "Ellips B.V." },
    { "38FF13", "Joint Stock Company Research Instinite Masshtab" },
    { "38FF36", "Ruckus Wireless" },
    { "3C01EF", "Sony Corporation" },
    { "3C0268", "Infinera, Inc." },
    { "3C02B1", "Creation Technologies LP" },
    { "3C0461", "Commscope" },
    { "3C04BF", "PRAVIS SYSTEMS Co.Ltd.," },
    { "3C0518", "Samsung Electronics Co.,Ltd" },
    { "3C058E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3C05AB", "Product Creation Studio" },
    { "3C0630", "Apple, Inc." },
    { "3C0664", "Beijing Leagrid Technology Co.,Ltd." },
    { "3C06A7", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "3C0754", "Apple, Inc." },
    { "3C0771", "Sony Corporation" },
    { "3C07D7", "Apple, Inc." },
    { "3C081E", "Beijing Yupont Electric Power Technology Co.,Ltd" },
    { "3C0868", "Power Plus Communications AG" },
    { "3C08CD", "Juniper Networks" },
    { "3C08F6", "Cisco Systems, Inc" },
    { "3C096D", "Powerhouse Dynamics" },
    { "3C0A7A", "Samsung Electronics Co.,Ltd" },
    { "3C0AF3", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "3C0B4F", "Intertech Services AG" },
    { "3C0B59", "Tuya Smart Inc." },
    { "3C0C48", "Servergy, Inc." },
    { "3C0C7D", "Tiny Mesh AS" },
    { "3C0CDB", "UNION MAN TECHNOLOGY CO.,LTD" },
    { "3C0D2C", "Liquid-Markets GmbH" },
    { "3C0E23", "Cisco Systems, Inc" },
    { "3C0FC1", "KBC Networks" },
    { "3C1040", "daesung network" },
    { "3C1060", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "3C106F", "ALBAHITH TECHNOLOGIES " },
    { "3C10E6", "PHAZR Inc." },
    { "3C11B2", "Fraunhofer FIT" },
    { "3C135A", "Xiaomi Communications Co Ltd" },
    { "3C13BB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3C13CC", "Cisco Systems, Inc" },
    { "3C1512", "Shenzhen Huanhu Technology Co.,Ltd" },
    { "3C15C2", "Apple, Inc." },
    { "3C15EA", "TESCOM CO., LTD." },
    { "3C15FB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3C1640", "Qingdao Haier Technology Co.,Ltd" },
    { "3C1710", "Sagemcom Broadband SAS" },
    { "3C189F", "Nokia Corporation" },
    { "3C18A0", "Luxshare Precision Industry Company Limited" },
    { "3C1915", "GFI Chrono Time" },
    { "3C195E", "Samsung Electronics Co.,Ltd" },
    { "3C197D", "Ericsson AB" },
    { "3C19CB", "TECNO MOBILE LIMITED" },
    { "3C1A0F", "ClearSky Data" },
    { "3C1A57", "Cardiopulmonary Corp" },
    { "3C1A65", "Nokia" },
    { "3C1A79", "Huayuan Technology CO.,LTD" },
    { "3C1A9E", "VitalThings AS" },
    { "3C1BF8", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "3C1CBE", "JADAK LLC" },
    { "3C1E04", "D-Link International" },
    { "3C1E13", "HANGZHOU SUNRISE TECHNOLOGY CO., LTD" },
    { "3C1EB5", "Apple, Inc." },
    { "3C2093", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "3C20F6", "Samsung Electronics Co.,Ltd" },
    { "3C219C", "Intel Corporate" },
    { "3C22FB", "Apple, Inc." },
    { "3C24F0", "IEEE Registration Authority" },
    { "3C25D7", "Nokia Corporation" },
    { "3C25F8", "Dell Inc." },
    { "3C26D5", "Sotera Wireless" },
    { "3C26E4", "Cisco Systems, Inc" },
    { "3C2763", "SLE quality engineering GmbH & Co. KG" },
    { "3C286D", "Google, Inc." },
    { "3C28A6", "Alcatel-Lucent Enterprise (China)" },
    { "3C2AF4", "Brother Industries, LTD." },
    { "3C2C30", "Dell Inc." },
    { "3C2C94", "杭州德澜科技有限公司（HangZhou Delan Technology Co.,Ltd）" },
    { "3C2C99", "Edgecore Networks Corporation" },
    { "3C2CA6", "Beijing Xiaomi Electronics Co.,Ltd" },
    { "3C2D9E", "Vantiva - Connected Home" },
    { "3C2DB7", "Texas Instruments" },
    { "3C2EF5", "Silicon Laboratories" },
    { "3C2EF9", "Apple, Inc." },
    { "3C2EFF", "Apple, Inc." },
    { "3C2F3A", "SFORZATO Corp." },
    { "3C300C", "Dewar Electronics Pty Ltd" },
    { "3C306F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3C3174", "Google, Inc." },
    { "3C3178", "Qolsys Inc." },
    { "3C318A", "Samsung Electronics Co.,Ltd" },
    { "3C32B9", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "3C3300", "Shenzhen Bilian electronic CO.,LTD" },
    { "3C3332", "D-Link Corporation" },
    { "3C3464", "Apple, Inc." },
    { "3C3556", "Cognitec Systems GmbH" },
    { "3C3576", "ITEL MOBILE LIMITED" },
    { "3C363D", "Nokia Corporation" },
    { "3C366A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3C36E4", "Commscope" },
    { "3C3712", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "3C3786", "NETGEAR" },
    { "3C3824", "Xiaomi Communications Co Ltd" },
    { "3C3888", "ConnectQuest, llc" },
    { "3C38F4", "Sony Corporation" },
    { "3C39A8", "Shenzhen Taichi Technology Limited" },
    { "3C39C3", "JW Electronics Co., Ltd." },
    { "3C39C8", "Apple, Inc." },
    { "3C39E7", "IEEE Registration Authority" },
    { "3C3A73", "Avaya Inc" },
    { "3C3B4D", "Toyo Seisakusho Kaisha, Limited" },
    { "3C3B77", "Apple, Inc." },
    { "3C3B99", "ITEL MOBILE LIMITED" },
    { "3C3BAD", "CHINA DRAGON TECHNOLOGY LIMITED" },
    { "3C3F51", "2CRSI" },
    { "3C4015", "12mm Health Technology (Hainan) Co., Ltd." },
    { "3C404F", "GUANGDONG PISEN ELECTRONICS CO.,LTD" },
    { "3C410E", "Cisco Systems, Inc" },
    { "3C427E", "IEEE Registration Authority" },
    { "3C438E", "Commscope" },
    { "3C450B", "Sentry Equipment Corp." },
    { "3C457A", "SKY UK LIMITED" },
    { "3C4645", "Shanghai Infinity Wireless Technologies Co.,Ltd." },
    { "3C46A1", "Ruckus Wireless" },
    { "3C46D8", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "3C4711", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3C479B", "Theissen Training Systems, Inc." },
    { "3C4937", "ASSMANN Electronic GmbH" },
    { "3C4A92", "Hewlett Packard" },
    { "3C4AC9", "Huawei Device Co., Ltd." },
    { "3C4C69", "Infinity System S.L." },
    { "3C4CD0", "CERAGON NETWORKS" },
    { "3C4DBE", "Apple, Inc." },
    { "3C4E47", "Etronic A/S" },
    { "3C4E56", "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD" },
    { "3C5002", "Apple, Inc." },
    { "3C510E", "Cisco Systems, Inc" },
    { "3C5282", "Hewlett Packard" },
    { "3C52A1", "TP-Link Systems Inc" },
    { "3C53D7", "CEDES AG" },
    { "3C5447", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3C5576", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "3C55DB", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "3C566E", "Infinix mobility limited" },
    { "3C5731", "Cisco Systems, Inc" },
    { "3C574F", "China Mobile Group Device Co.,Ltd." },
    { "3C5765", "UNIONMAN TECHNOLOGY CO.,LTD" },
    { "3C576C", "Samsung Electronics Co.,Ltd" },
    { "3C57BD", "Kessler Crane Inc." },
    { "3C57D5", "FiveCo" },
    { "3C5836", "Sagemcom Broadband SAS" },
    { "3C585D", "Sagemcom Broadband SAS" },
    { "3C58C2", "Intel Corporate" },
    { "3C591E", "TCL King Electrical Appliances (Huizhou) Co., Ltd" },
    { "3C59C0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3C5A37", "Samsung Electronics Co.,Ltd" },
    { "3C5AB4", "Google, Inc." },
    { "3C5CC3", "Shenzhen First Blue Chip Technology Ltd" },
    { "3C5CC4", "Amazon Technologies Inc." },
    { "3C5CF1", "eero inc." },
    { "3C5D29", "Zhejiang Tmall Technology Co., Ltd." },
    { "3C5EC3", "Cisco Systems, Inc" },
    { "3C5F01", "Synerchip Co., Ltd." },
    { "3C6104", "Juniper Networks" },
    { "3C6105", "Espressif Inc." },
    { "3C6200", "Samsung Electronics Co.,Ltd" },
    { "3C6278", "SHENZHEN JETNET TECHNOLOGY CO.,LTD." },
    { "3C62F0", "Sercomm Corporation." },
    { "3C64CF", "TP-Link Systems Inc" },
    { "3C6716", "Lily Robotics" },
    { "3C672C", "Sciovid Inc." },
    { "3C678C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3C6816", "VXi Corporation" },
    { "3C69D1", "ADC Automotive Distance Control System GmbH" },
    { "3C6A2C", "IEEE Registration Authority" },
    { "3C6A48", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "3C6A7D", "Niigata Power Systems Co., Ltd." },
    { "3C6A9D", "Dexatek Technology LTD." },
    { "3C6AA7", "Intel Corporate" },
    { "3C6AD2", "TP-Link Systems Inc." },
    { "3C6D66", "NVIDIA Corporation" },
    { "3C6D89", "Apple, Inc." },
    { "3C6E63", "Mitron OY" },
    { "3C6F45", "Fiberpro Inc." },
    { "3C6F9B", "zte corporation" },
    { "3C6FEA", "Panasonic India Pvt. Ltd." },
    { "3C6FF7", "EnTek Systems, Inc." },
    { "3C7059", "MakerBot Industries" },
    { "3C71BF", "Espressif Inc." },
    { "3C7437", "RIM" },
    { "3C754A", "Commscope" },
    { "3C7625", "zte corporation" },
    { "3C7787", "Huawei Device Co., Ltd." },
    { "3C77E6", "Hon Hai Precision Ind. Co.,Ltd." },
    { "3C7843", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3C7873", "Airsonics" },
    { "3C7895", "TP-Link Systems Inc." },
    { "3C792B", "Dongguan Auklink TechnologyCo.,Ltd" },
    { "3C7A8A", "Commscope" },
    { "3C7AAA", "China Dragon Technology Limited" },
    { "3C7AC4", "Chemtronics" },
    { "3C7AF0", "ITEL MOBILE LIMITED" },
    { "3C7C3F", "ASUSTek COMPUTER INC." },
    { "3C7D0A", "Apple, Inc." },
    { "3C7DB1", "Texas Instruments" },
    { "3C7F6F", "Telechips, Inc." },
    { "3C806B", "Hunan Voc Acoustics Technology Co., Ltd." },
    { "3C80AA", "Ransnet Singapore Pte Ltd" },
    { "3C81D8", "Sagemcom Broadband SAS" },
    { "3C82C0", "Vantiva USA LLC" },
    { "3C831E", "CKD Corporation" },
    { "3C8375", "Microsoft Corporation" },
    { "3C83B5", "Advance Vision Electronics Co. Ltd." },
    { "3C8427", "Espressif Inc." },
    { "3C846A", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "3C869A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3C86A8", "Sangshin elecom .co,, LTD" },
    { "3C86D1", "vivo Mobile Communication Co., Ltd." },
    { "3C894D", "Dr. Ing. h.c. F. Porsche AG" },
    { "3C8970", "Neosfar" },
    { "3C8994", "SKY UK LIMITED" },
    { "3C89A6", "KAPELSE" },
    { "3C8A1F", "Espressif Inc." },
    { "3C8AB0", "Juniper Networks" },
    { "3C8AE5", "Tensun Information Technology(Hangzhou) Co.,LTD" },
    { "3C8B7F", "Cisco Systems, Inc" },
    { "3C8BCD", "Alcatel-Lucent Shanghai Bell Co., Ltd" },
    { "3C8BFE", "Samsung Electronics Co.,Ltd" },
    { "3C8C40", "Hangzhou H3C Technologies Co., Limited" },
    { "3C8C93", "Juniper Networks" },
    { "3C8CF8", "TRENDnet, Inc." },
    { "3C8D20", "Google, Inc." },
    { "3C8F06", "Shenzhen Libtor Technology Co.,Ltd" },
    { "3C9066", "SmartRG, Inc." },
    { "3C90E0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3C912B", "Vexata Inc" },
    { "3C9157", "Yulong Computer Telecommunication Scientific (Shenzhen) Co.,Ltd" },
    { "3C9174", "ALONG COMMUNICATION TECHNOLOGY" },
    { "3C9180", "Liteon Technology Corporation" },
    { "3C92DC", "Octopod Technology Co. Ltd." },
    { "3C93F4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3C94D5", "Juniper Networks" },
    { "3C94FD", "Mist Systems, Inc." },
    { "3C9509", "Liteon Technology Corporation" },
    { "3C970E", "Wistron InfoComm(Kunshan)Co.,Ltd." },
    { "3C9722", "shenzhen worldelite electronics co., LTD" },
    { "3C977E", "IPS Technology Limited" },
    { "3C9872", "Sercomm Corporation." },
    { "3C98BF", "Quest Controls, Inc." },
    { "3C996D", "Marelli Europe s.p.a." },
    { "3C998C", "Houwa System Design Corp." },
    { "3C99F7", "Lansentechnology AB" },
    { "3C9A77", "Vantiva USA LLC" },
    { "3C9BC6", "Huawei Device Co., Ltd." },
    { "3C9BD6", "Vizio, Inc" },
    { "3C9C0F", "Intel Corporate" },
    { "3C9D56", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3C9EC7", "SKY UK LIMITED" },
    { "3C9F81", "Shenzhen CATIC Bit Communications Technology Co.,Ltd" },
    { "3C9FC3", "Beijing Sinead Technology Co., Ltd. " },
    { "3C9FCD", "Shenzhen Neoway Technology Co.,Ltd." },
    { "3CA067", "Liteon Technology Corporation" },
    { "3CA070", "Blink by Amazon " },
    { "3CA10D", "Samsung Electronics Co.,Ltd" },
    { "3CA161", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3CA2C3", "vivo Mobile Communication Co., Ltd." },
    { "3CA308", "Texas Instruments" },
    { "3CA315", "Bless Information & Communications Co., Ltd" },
    { "3CA31A", "Oilfind International LLC" },
    { "3CA348", "vivo Mobile Communication Co., Ltd." },
    { "3CA37E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3CA581", "vivo Mobile Communication Co., Ltd." },
    { "3CA616", "vivo Mobile Communication Co., Ltd." },
    { "3CA62F", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "3CA6F6", "Apple, Inc." },
    { "3CA72B", "MRV Communications (Networks) LTD" },
    { "3CA7AE", "zte corporation" },
    { "3CA80A", "vivo Mobile Communication Co., Ltd." },
    { "3CA82A", "Hewlett Packard" },
    { "3CA8ED", "smart light technology" },
    { "3CA916", "Huawei Device Co., Ltd." },
    { "3CA9AB", "Nintendo Co.,Ltd" },
    { "3CA9F4", "Intel Corporate" },
    { "3CAA3F", "iKey, Ltd." },
    { "3CAB72", "Nanjing Qinheng Microelectronics Co., Ltd." },
    { "3CAB8E", "Apple, Inc." },
    { "3CAE69", "ESA Elektroschaltanlagen Grimma GmbH" },
    { "3CAFB7", "Xiaomi Communications Co Ltd" },
    { "3CB07E", "Arounds Intelligent Equipment Co., Ltd." },
    { "3CB0ED", "Nothing Technology Limited" },
    { "3CB15B", "Avaya Inc" },
    { "3CB17F", "Wattwatchers Pty Ld" },
    { "3CB233", "Huawei Device Co., Ltd." },
    { "3CB43D", "SZ Tenveo video technology co., Ltd" },
    { "3CB53D", "HUNAN GOKE MICROELECTRONICS CO.,LTD" },
    { "3CB6B7", "vivo Mobile Communication Co., Ltd." },
    { "3CB6E7", "Handheld Scientific, Inc." },
    { "3CB72B", "PLUMgrid Inc" },
    { "3CB74B", "Vantiva USA LLC" },
    { "3CB792", "Hitachi Maxell, Ltd., Optronics Division" },
    { "3CB87A", "Private" },
    { "3CB8D6", "Bluebank Communication Technology Co.,Ltd." },
    { "3CB9A6", "Belden Deutschland GmbH" },
    { "3CBB73", "Shenzhen Xinguodu Technology Co., Ltd." },
    { "3CBBFD", "Samsung Electronics Co.,Ltd" },
    { "3CBCD0", "zte corporation" },
    { "3CBD14", "Diebold Nixdorf" },
    { "3CBD3E", "Beijing Xiaomi Electronics Co., Ltd." },
    { "3CBD69", "Nokia Shanghai Bell Co., Ltd." },
    { "3CBDC5", "Arcadyan Corporation" },
    { "3CBDD8", "LG ELECTRONICS INC" },
    { "3CBEE1", "NIKON CORPORATION" },
    { "3CBF60", "Apple, Inc." },
    { "3CC03E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3CC079", "Shenzhen One-Nine Intelligent Electronic Science and Technology Co., Ltd" },
    { "3CC0C6", "d&b audiotechnik GmbH" },
    { "3CC12C", "AES Corporation" },
    { "3CC1F6", "Melange Systems Pvt. Ltd." },
    { "3CC243", "Nokia Corporation" },
    { "3CC2E1", "XINHUA CONTROL ENGINEERING CO.,LTD" },
    { "3CC5C7", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3CC5DD", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "3CC683", "Sichuan AI-Link Technology Co., Ltd." },
    { "3CC786", "DONGGUAN HUARONG COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "3CC99E", "Huiyang Technology Co., Ltd" },
    { "3CCA61", "TECNO MOBILE LIMITED" },
    { "3CCA87", "Iders Incorporated" },
    { "3CCB4D", "Avikus Co., Ltd" },
    { "3CCB7C", "TCT mobile ltd" },
    { "3CCD36", "Apple, Inc." },
    { "3CCD40", "Apple, Inc." },
    { "3CCD57", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "3CCD5A", "Technische Alternative GmbH" },
    { "3CCD5D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3CCD73", "Nebula Electronic Technology Corporation" },
    { "3CCD93", "LG ELECTRONICS INC" },
    { "3CCE0D", "Shenzhen juduoping Technology Co.,Ltd" },
    { "3CCE15", "Mercedes-Benz USA, LLC" },
    { "3CCE73", "Cisco Systems, Inc" },
    { "3CCF5B", "ICOMM HK LIMITED" },
    { "3CCFB4", "Telink Semiconductor (Shanghai) Co., Ltd." },
    { "3CD0F8", "Apple, Inc." },
    { "3CD16E", "Telepower Communication Co., Ltd" },
    { "3CD1C9", "Groupe SEB" },
    { "3CD2E5", "New H3C Technologies Co., Ltd" },
    { "3CD4D6", "WirelessWERX, Inc" },
    { "3CD7DA", "SK Mtek microelectronics(shenzhen)limited" },
    { "3CD92B", "Hewlett Packard" },
    { "3CD9CE", "Eclipse WiFi" },
    { "3CDA2A", "zte corporation" },
    { "3CDA6D", "Tiandy Technologies CO.,LTD" },
    { "3CDC03", "IEEE Registration Authority" },
    { "3CDC75", "Espressif Inc." },
    { "3CDCBC", "Samsung Electronics Co.,Ltd" },
    { "3CDD57", "Apple, Inc." },
    { "3CDD89", "SOMO HOLDINGS & TECH. CO.,LTD." },
    { "3CDF1E", "Cisco Systems, Inc" },
    { "3CDFA9", "Commscope" },
    { "3CDFBD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3CE002", "Texas Instruments" },
    { "3CE038", "Omnifi Inc." },
    { "3CE064", "Texas Instruments" },
    { "3CE072", "Apple, Inc." },
    { "3CE1A1", "Universal Global Scientific Industrial Co., Ltd." },
    { "3CE36B", "Zhejiang Dahua Technology Co., Ltd." },
    { "3CE3E7", "China Mobile Group Device Co.,Ltd." },
    { "3CE441", "Amazon Technologies Inc." },
    { "3CE4B0", "Texas Instruments" },
    { "3CE5A6", "Hangzhou H3C Technologies Co., Limited" },
    { "3CE5B4", "KIDASEN INDUSTRIA E COMERCIO DE ANTENAS LTDA" },
    { "3CE624", "LG Display " },
    { "3CE824", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3CE86E", "Hewlett Packard Enterprise" },
    { "3CE90E", "Espressif Inc." },
    { "3CE9F7", "Intel Corporate" },
    { "3CEA4F", "2Wire Inc" },
    { "3CEAF9", "JUBIXCOLTD" },
    { "3CEAFB", "NSE AG" },
    { "3CECDE", "FUJIAN STAR-NET COMMUNICATION CO.,LTD" },
    { "3CECEF", "Super Micro Computer, Inc." },
    { "3CEF42", "TCT mobile ltd" },
    { "3CEF8C", "Zhejiang Dahua Technology Co., Ltd." },
    { "3CF011", "Intel Corporate" },
    { "3CF083", "Arcadyan Corporation" },
    { "3CF341", "Hosenso GmbH & Co. KG" },
    { "3CF392", "Virtualtek. Co. Ltd" },
    { "3CF4F9", "Moda-InnoChips" },
    { "3CF52C", "DSPECIALISTS GmbH" },
    { "3CF591", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "3CF5CC", "New H3C Technologies Co., Ltd" },
    { "3CF652", "zte corporation" },
    { "3CF692", "Huawei Device Co., Ltd." },
    { "3CF72A", "Nokia Corporation" },
    { "3CF748", "Shenzhen Linsn Technology Development Co.,Ltd" },
    { "3CF7A4", "Samsung Electronics Co.,Ltd" },
    { "3CF7D1", "OMRON Corporation" },
    { "3CF808", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3CF862", "Intel Corporate" },
    { "3CF9F0", "zte corporation" },
    { "3CFA06", "Microsoft Corporation" },
    { "3CFA30", "Palo Alto Networks" },
    { "3CFA43", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3CFA80", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "3CFAD3", "IEEE Registration Authority" },
    { "3CFB5C", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "3CFB96", "Emcraft Systems LLC" },
    { "3CFDFE", "Intel Corporate" },
    { "3CFEAC", "Cisco Systems, Inc" },
    { "3CFFD8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "4000E0", "Derek(Shaoguan)Limited" },
    { "400107", "Arista Corp" },
    { "40017A", "Cisco Systems, Inc" },
    { "4001C6", "3COM EUROPE LTD" },
    { "400326", "NXP Semiconductor (Tianjin) LTD." },
    { "40040C", "A&T" },
    { "400589", "T-Mobile, USA" },
    { "400634", "Huawei Device Co., Ltd." },
    { "4006A0", "Texas Instruments" },
    { "4006D5", "Cisco Systems, Inc" },
    { "4007C0", "Railtec Systems GmbH" },
    { "400D10", "Commscope" },
    { "400E67", "Tremol Ltd." },
    { "400E85", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "400EF3", "zte corporation" },
    { "400FC1", "Vantiva USA LLC" },
    { "4010ED", "G.Tech Technology Ltd." },
    { "401175", "IEEE Registration Authority" },
    { "4011C3", "Samsung Electronics Co.,Ltd" },
    { "4011DC", "Sonance" },
    { "4012E4", "Compass-EOS" },
    { "4013D9", "Global ES" },
    { "401482", "Cisco Systems, Inc" },
    { "4014AD", "Huawei Device Co., Ltd." },
    { "401597", "Protect America, Inc." },
    { "40163B", "Samsung Electronics Co.,Ltd" },
    { "40167E", "ASUSTek COMPUTER INC." },
    { "40169F", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "4016FA", "EKM Metering" },
    { "4017E2", "INTAI TECHNOLOGY CORP." },
    { "4017F6", "TKH SECURITY,S.L.U." },
    { "4018B1", "Extreme Networks Headquarters" },
    { "4018D7", "Smartronix, Inc." },
    { "401920", "Movon Corporation" },
    { "401A58", "Wistron Neweb Corporation" },
    { "401B5F", "WEIFANG GOERTEK ELECTRONICS CO.,LTD" },
    { "401C83", "Intel Corporate" },
    { "401CD4", "Huawei Device Co., Ltd." },
    { "401D59", "Biometric Associates, LP" },
    { "402230", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "4022D8", "Espressif Inc." },
    { "4022ED", "Digital Projection Ltd" },
    { "402343", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "4024B2", "Sichuan AI-Link Technology Co., Ltd." },
    { "4024D2", "Huawei Device Co., Ltd." },
    { "402508", "Highway 9 Networks, Inc." },
    { "4025C2", "Intel Corporate" },
    { "402619", "Apple, Inc." },
    { "40268E", "Shenzhen Photon Leap Technology Co., Ltd." },
    { "40270B", "Mobileeco Co., Ltd" },
    { "4027A8", "Cisco Meraki" },
    { "402814", "RFI Engineering" },
    { "402A8F", "Shanghai High-Flying Electronics Technology Co., Ltd" },
    { "402B50", "Commscope" },
    { "402B69", "Kumho Electric Inc." },
    { "402BA1", "Sony Corporation" },
    { "402C76", "IEEE Registration Authority" },
    { "402CF4", "Universal Global Scientific Industrial Co., Ltd." },
    { "402E28", "MiXTelematics" },
    { "402E71", "Texas Instruments" },
    { "402F51", "Maxtek Optoelectronics Ltd" },
    { "402F86", "LG Innotek" },
    { "403004", "Apple, Inc." },
    { "403059", "Silicon Laboratories" },
    { "403067", "Conlog (Pty) Ltd" },
    { "40311B", "Genbyte Technology Inc." },
    { "40313C", "XIAOMI Electronics,CO.,LTD" },
    { "40329D", "Union Image Co.,Ltd" },
    { "403306", "Taicang T&W Electronics" },
    { "40331A", "Apple, Inc." },
    { "40336C", "Godrej & Boyce Mfg. co. ltd" },
    { "4035E6", "Samsung Electronics Co.,Ltd" },
    { "403668", "E&B TELECOM" },
    { "4036B7", "Juniper Networks" },
    { "4037AD", "Macro Image Technology, Inc." },
    { "403B7B", "Huawei Device Co., Ltd." },
    { "403CFC", "Apple, Inc." },
    { "403DEC", "HUMAX Co., Ltd." },
    { "403F43", "Extreme Networks Headquarters" },
    { "403F8C", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "404022", "ZIV" },
    { "404028", "ZIV" },
    { "40406B", "Icomera" },
    { "40406C", "Icomera" },
    { "4040A7", "Sony Corporation" },
    { "404101", "Rockwell Automation" },
    { "40410D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "404229", "Layer3TV, Inc" },
    { "404244", "Cisco Systems, Inc" },
    { "4044CE", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "4044FD", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "4045A0", "vivo Mobile Communication Co., Ltd." },
    { "4045C4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "4045DA", "Spreadtrum Communications (Shanghai) Co., Ltd." },
    { "40475E", "eero inc." },
    { "40476A", "Astro Gaming" },
    { "40486E", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "4048FD", "IEEE Registration Authority" },
    { "40490F", "Hon Hai Precision Ind. Co.,Ltd." },
    { "40497C", "eero inc." },
    { "40498A", "Synapticon GmbH" },
    { "404A03", "Zyxel Communications Corporation" },
    { "404A18", "Addrek Smart Solutions" },
    { "404AD4", "Widex A/S" },
    { "404C77", "Commscope" },
    { "404CCA", "Espressif Inc." },
    { "404D7F", "Apple, Inc." },
    { "404D8E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "404E36", "HTC Corporation" },
    { "404EEB", "Higher Way Electronic Co., Ltd." },
    { "404F42", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "4050B5", "Shenzhen New Species Technology Co., Ltd." },
    { "4050E0", "Milton Security Group LLC" },
    { "40516C", "Grandex International Corporation" },
    { "40520D", "Pico Technology" },
    { "4054E4", "Wearsafe Labs Inc" },
    { "405539", "Cisco Systems, Inc" },
    { "405548", "Quectel Wireless Solutions Co.,Ltd." },
    { "405582", "Nokia" },
    { "40560C", "In Home Displays Ltd" },
    { "40562D", "Smartron India Pvt ltd" },
    { "405662", "GuoTengShengHua Electronics LTD." },
    { "405846", "vivo Mobile Communication Co., Ltd." },
    { "405899", "Logitech Far East" },
    { "405A9B", "ANOVO" },
    { "405B7F", "Broadcom Limited" },
    { "405BD8", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "405CFD", "Dell Inc." },
    { "405D82", "NETGEAR" },
    { "405ECF", "Esconet Technologies Limited" },
    { "405EE1", "Shenzhen H&T Intelligent Control Co.,Ltd." },
    { "405EF6", "Samsung Electronics Co.,Ltd" },
    { "405F7D", "TCT mobile ltd" },
    { "405FBE", "RIM" },
    { "405FC2", "Texas Instruments" },
    { "40605A", "Hawkeye Tech Co. Ltd" },
    { "406186", "MICRO-STAR INT'L CO.,LTD" },
    { "40618E", "Stella-Green Co" },
    { "406231", "GIFA" },
    { "406234", "Telink Semiconductor (Shanghai) Co., Ltd." },
    { "4062B6", "Tele system communication" },
    { "4062EA", "China Mobile Group Device Co.,Ltd." },
    { "4062EE", "Chicony Electronics Co., Ltd." },
    { "4064A4", "THE FURUKAWA ELECTRIC CO., LTD" },
    { "4064DC", "X-speed lnformation Technology Co.,Ltd" },
    { "4065A3", "Sagemcom Broadband SAS" },
    { "40667A", "mediola - connected living AG" },
    { "40679B", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "406826", "Thales UK Limited" },
    { "406918", "EM Microelectronic" },
    { "406A8E", "Hangzhou Puwell OE Tech Ltd." },
    { "406AAB", "RIM" },
    { "406C8F", "Apple, Inc." },
    { "406F27", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "406F2A", "BlackBerry RTS" },
    { "407009", "Commscope" },
    { "40704A", "Power Idea Technology Limited" },
    { "407074", "Life Technology (China) Co., Ltd" },
    { "4070A5", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "4070F5", "Apple, Inc." },
    { "407183", "Juniper Networks" },
    { "407218", "Tonly Technology Co. Ltd " },
    { "40734D", "New H3C Technologies Co., Ltd" },
    { "407496", "aFUN TECHNOLOGY INC." },
    { "4074E0", "Intel Corporate" },
    { "4075C3", "Vantiva USA LLC" },
    { "4076A9", "Huawei Device Co., Ltd." },
    { "4077A9", "New H3C Technologies Co., Ltd" },
    { "40786A", "Motorola Mobility LLC, a Lenovo Company" },
    { "407875", "IMBEL - Industria de Material Belico do Brasil" },
    { "407911", "Apple, Inc." },
    { "407912", "Texas Instruments" },
    { "407A80", "Nokia Corporation" },
    { "407AA4", "ZHEJIANG DAHUA TECHNOLOGYCO.,LTD" },
    { "407B1B", "Mettle Networks Inc." },
    { "407C7D", "Nokia" },
    { "407D0F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "407F5F", "Juniper Networks" },
    { "407FE0", "Glory Star Technics (ShenZhen) Limited" },
    { "4080E1", "FN-LINK TECHNOLOGY Ltd." },
    { "408256", "Continental Automotive GmbH" },
    { "40827B", "STMicroelectronics Rousset SAS" },
    { "40831D", "Apple, Inc." },
    { "4083DE", "Zebra Technologies Inc" },
    { "408432", "Microchip Technology Inc." },
    { "408493", "Clavister AB" },
    { "408556", "Continental Automotive Romania SLR" },
    { "40862E", "JDM MOBILE INTERNET SOLUTION CO., LTD." },
    { "4086CB", "D-Link Corporation" },
    { "4087E5", "Wu Qi Technologies,Inc." },
    { "408805", "Motorola Mobility LLC, a Lenovo Company" },
    { "40882F", "Extreme Networks Headquarters" },
    { "4088E0", "Beijing Ereneben Information Technology Limited Shenzhen Branch" },
    { "4089A8", "WiredIQ, LLC" },
    { "4089C6", "Amazon Technologies Inc." },
    { "408A9A", "TITENG CO., Ltd." },
    { "408B07", "Actiontec Electronics, Inc" },
    { "408BF6", "Shenzhen TCL New Technology Co., Ltd" },
    { "408C1F", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "408C4C", "Shenzhen MiaoMing Intelligent Technology Co.,Ltd" },
    { "408D5C", "GIGA-BYTE TECHNOLOGY CO.,LTD." },
    { "408E2C", "Microsoft Corporation" },
    { "408EDF", "Huawei Device Co., Ltd." },
    { "408EF6", "Infinix mobility limited" },
    { "408F9A", "KanEL Sweden AB" },
    { "408F9D", "Juniper Networks" },
    { "409151", "Espressif Inc." },
    { "40921A", "Apple, Inc." },
    { "409249", "Shanghai Baud Data Communication Co.,Ltd." },
    { "409505", "ACOINFO TECHNOLOGY CO.,LTD" },
    { "409558", "Aisino Corporation" },
    { "409595", "TP-Link Systems Inc." },
    { "4095BD", "NTmore.Co.,Ltd" },
    { "4097D1", "BK Electronics cc" },
    { "40984C", "Casacom Solutions AG" },
    { "40984E", "Texas Instruments" },
    { "40987B", "Aisino Corporation" },
    { "4098AD", "Apple, Inc." },
    { "409922", "AzureWave Technology Inc." },
    { "4099E3", "Guangzhou Mudi Information Technology Co., Ltd" },
    { "4099F6", "Telink Semiconductor(Shanghai) Co.,Ltd" },
    { "409A30", "TECNO MOBILE LIMITED" },
    { "409B0D", "Shenzhen Yourf Kwan Industrial Co., Ltd" },
    { "409B21", "Nokia" },
    { "409BCD", "D-Link International" },
    { "409C28", "Apple, Inc." },
    { "409CA6", "Curvalux" },
    { "409CA7", "CHINA DRAGON TECHNOLOGY LIMITED" },
    { "409EA4", "Juniper Networks" },
    { "409F38", "AzureWave Technology Inc." },
    { "409F87", "Jide Technology (Hong Kong) Limited" },
    { "409FC7", "BAEKCHUN I&C Co., Ltd." },
    { "40A108", "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd." },
    { "40A2DB", "Amazon Technologies Inc." },
    { "40A3CC", "Intel Corporate" },
    { "40A53B", "Nokia" },
    { "40A5EF", "Shenzhen Four Seas Global Link Network Technology Co., Ltd." },
    { "40A63D", "SignalFire Telemetry" },
    { "40A677", "Juniper Networks" },
    { "40A6A4", "PassivSystems Ltd" },
    { "40A6B7", "Intel Corporate" },
    { "40A6D9", "Apple, Inc." },
    { "40A6E8", "Cisco Systems, Inc" },
    { "40A786", "TECNO MOBILE LIMITED" },
    { "40A8F0", "Hewlett Packard" },
    { "40A93F", "Pivotal Commware, Inc." },
    { "40A9CF", "Amazon Technologies Inc." },
    { "40AA56", "China Dragon Technology Limited" },
    { "40AC8D", "Data Management, Inc." },
    { "40ACBF", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "40AE30", "TP-Link Systems Inc" },
    { "40B02F", "Miele & Cie. KG" },
    { "40B034", "Hewlett Packard" },
    { "40B076", "ASUSTek COMPUTER INC." },
    { "40B0A1", "VALCOM CO.,LTD." },
    { "40B0FA", "LG Electronics (Mobile Communications)" },
    { "40B15C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "40B215", "Extreme Networks Headquarters" },
    { "40B2C8", "Nortel Networks" },
    { "40B30E", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "40B31E", "Universal Electronics, Inc." },
    { "40B395", "Apple, Inc." },
    { "40B3CD", "Chiyoda Electronics Co.,Ltd." },
    { "40B3FA", "Apple, Inc." },
    { "40B3FC", "Logital Co. Limited " },
    { "40B4CD", "Amazon Technologies Inc." },
    { "40B4F0", "Juniper Networks" },
    { "40B5C1", "Cisco Systems, Inc" },
    { "40B607", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "40B688", "LEGIC Identsystems AG" },
    { "40B6B1", "SUNGSAM CO,.Ltd" },
    { "40B6E7", "Huawei Device Co., Ltd." },
    { "40B70E", "Huawei Device Co., Ltd." },
    { "40B7F3", "Commscope" },
    { "40B7FC", "Phyplus Microelectronics Limited" },
    { "40B82D", "Ruckus Wireless" },
    { "40B837", "Sony Corporation" },
    { "40B89A", "Hon Hai Precision Ind. Co.,Ltd." },
    { "40B8C2", "OSMOZIS" },
    { "40B93C", "Hewlett Packard Enterprise " },
    { "40BA61", "ARIMA Communications Corp." },
    { "40BB56", "TeraNXT Global India Pvt Ltd." },
    { "40BC60", "Apple, Inc." },
    { "40BC68", "Funshion Online Technologies Co.,Ltd" },
    { "40BC73", "Cronoplast S.L." },
    { "40BC8B", "itelio GmbH" },
    { "40BD32", "Texas Instruments" },
    { "40BD9E", "Physio-Control, Inc" },
    { "40BEEE", "Shenzhen Yunding Information Technology Co.,Ltd" },
    { "40BF17", "Digistar Telecom. SA" },
    { "40C02F", "Actiontec Electronics Inc." },
    { "40C0EE", "365mesh Pty Ltd" },
    { "40C1F6", "Shenzhen Jingxun Technology Co., Ltd." },
    { "40C245", "Shenzhen Hexicom Technology Co., Ltd." },
    { "40C2BA", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "40C3BC", "Huawei Device Co., Ltd." },
    { "40C3C6", "SnapRoute" },
    { "40C48C", "N-iTUS CO.,LTD." },
    { "40C4D6", "ChongQing Camyu Technology Development Co.,Ltd." },
    { "40C62A", "Shanghai Jing Ren Electronic Technology Co., Ltd." },
    { "40C711", "Apple, Inc." },
    { "40C729", "Sagemcom Broadband SAS" },
    { "40C73C", "Intel Corporate" },
    { "40C7C9", "Naviit Inc." },
    { "40C81F", "Shenzhen Xinguodu Technology Co., Ltd." },
    { "40C8CB", "AM Telecom co., Ltd." },
    { "40CA63", "Seongji Industry Company" },
    { "40CBA8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "40CBC0", "Apple, Inc." },
    { "40CD3A", "Z3 Technology" },
    { "40CD7A", "Qingdao Hisense Communications Co.,Ltd." },
    { "40CE24", "Cisco Systems, Inc" },
    { "40D133", "Intel Corporate" },
    { "40D160", "Apple, Inc." },
    { "40D25F", "ITEL MOBILE LIMITED" },
    { "40D28A", "Nintendo Co., Ltd." },
    { "40D32D", "Apple, Inc." },
    { "40D357", "Ison Technology Co., Ltd." },
    { "40D3AE", "Samsung Electronics Co.,Ltd" },
    { "40D40E", "Biodata Ltd" },
    { "40D4BD", "SK Networks Service CO., LTD." },
    { "40D4F6", "Honor Device Co., Ltd." },
    { "40D521", "LG Innotek" },
    { "40D559", "MICRO S.E.R.I." },
    { "40D563", "HANA Electronics" },
    { "40D63C", "Equitech Industrial(DongGuan)Co.,Ltd" },
    { "40D855", "IEEE Registration Authority" },
    { "40D95A", "AMPAK Technology,Inc." },
    { "40DA5C", "Apple, Inc." },
    { "40DC9D", "HAJEN" },
    { "40DCA5", "Huawei Device Co., Ltd." },
    { "40DDD1", "Beautiful Card Corporation" },
    { "40DE17", "Shenzhen Lanfeng Times Industrial Co.,Ltd." },
    { "40DE24", "Samsung Electronics Co.,Ltd" },
    { "40DEAD", "Juniper Networks" },
    { "40DF02", "LINE BIZ Plus" },
    { "40E11C", "shenzhen Cloud High Communication Technology Co.,Ltd" },
    { "40E171", "Jiangsu Huitong Group Co.,Ltd." },
    { "40E1E4", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "40E230", "AzureWave Technology Inc." },
    { "40E317", "Extreme Networks Headquarters" },
    { "40E3D6", "Hewlett Packard Enterprise" },
    { "40E64B", "Apple, Inc." },
    { "40E730", "DEY Storage Systems, Inc." },
    { "40E793", "Shenzhen Siviton Technology Co.,Ltd" },
    { "40E99B", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "40EACE", "FOUNDER BROADBAND NETWORK SERVICE CO.,LTD" },
    { "40EB21", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "40EC99", "Intel Corporate" },
    { "40ECF8", "Siemens AG" },
    { "40ED00", "TP-Link Systems Inc" },
    { "40ED98", "IEEE Registration Authority" },
    { "40EDCF", "Apple, Inc." },
    { "40EE15", "Zioncom Electronics (Shenzhen) Ltd." },
    { "40EEBE", "NXP Semiconductor (Tianjin) LTD." },
    { "40EEDD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "40EF4C", "Fihonest communication co.,Ltd" },
    { "40F02F", "Liteon Technology Corporation" },
    { "40F04E", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "40F078", "Cisco Systems, Inc" },
    { "40F14C", "ISE Europe SPRL" },
    { "40F201", "Sagemcom Broadband SAS" },
    { "40F21C", "Zhone Technologies, Inc." },
    { "40F2E9", "IBM" },
    { "40F308", "Murata Manufacturing Co., Ltd." },
    { "40F385", "IEEE Registration Authority" },
    { "40F3B0", "Texas Instruments" },
    { "40F407", "Nintendo Co., Ltd." },
    { "40F413", "Rubezh" },
    { "40F420", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "40F49F", "Cisco Systems, Inc" },
    { "40F4C9", "FN-LINK TECHNOLOGY Ltd." },
    { "40F4EC", "Cisco Systems, Inc" },
    { "40F4FD", "UNION MAN TECHNOLOGY CO.,LTD" },
    { "40F520", "Espressif Inc." },
    { "40F52E", "Leica Microsystems (Schweiz) AG" },
    { "40F6BC", "Amazon Technologies Inc." },
    { "40F8DF", "CANON INC." },
    { "40F946", "Apple, Inc." },
    { "40F9D5", "Tecore Networks" },
    { "40FA7F", "Preh Car Connect GmbH" },
    { "40FAFE", "Motorola Mobility LLC, a Lenovo Company" },
    { "40FC89", "Commscope" },
    { "40FDF3", "AMPAK Technology,Inc." },
    { "40FE0D", "MAXIO" },
    { "40FE95", "New H3C Technologies Co., Ltd" },
    { "40FF40", "GloquadTech" },
    { "440010", "Apple, Inc." },
    { "440049", "Amazon Technologies Inc." },
    { "44004D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "4401BB", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "44032C", "Intel Corporate" },
    { "440377", "IEEE Registration Authority" },
    { "4403A7", "Cisco Systems, Inc" },
    { "440444", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "44053F", "Sagemcom Broadband SAS" },
    { "4405B8", "Huawei Device Co., Ltd." },
    { "4405E8", "twareLAB" },
    { "44070B", "Google, Inc." },
    { "4409B8", "Salcomp (Shenzhen) CO., LTD." },
    { "4409C6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "4409DA", "Apple, Inc." },
    { "440BAB", "Positivo Tecnologia S.A." },
    { "440CEE", "Robert Bosch Elektronikai Kft." },
    { "440CFD", "NetMan Co., Ltd." },
    { "4410FE", "Huizhou Foryou General Electronics Co., Ltd." },
    { "441102", "EDMI Europe Ltd" },
    { "441146", "The Chamberlain Group, Inc" },
    { "4411C2", "Telegartner Karl Gartner GmbH" },
    { "441244", "Hewlett Packard Enterprise" },
    { "441319", "WKK TECHNOLOGY LTD." },
    { "4413D0", "zte corporation" },
    { "441441", "AudioControl Inc." },
    { "441524", "Sagemcom Broadband SAS" },
    { "441622", "Microsoft Corporation" },
    { "441690", "Wuxi Ranke Technology Co., Ltd." },
    { "4416FA", "Samsung Electronics Co.,Ltd" },
    { "441793", "Espressif Inc." },
    { "441847", "HUNAN SCROWN ELECTRONIC INFORMATION TECH.CO.,LTD" },
    { "44184F", "Fitview" },
    { "4418FD", "Apple, Inc." },
    { "4419B6", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "441A4C", "xFusion Digital Technologies Co.,Ltd." },
    { "441A84", "Quectel Wireless Solutions Co.,Ltd." },
    { "441AAC", "Elektrik Uretim AS EOS" },
    { "441AFA", "New H3C Technologies Co., Ltd" },
    { "441B88", "Apple, Inc." },
    { "441C12", "Vantiva USA LLC" },
    { "441C7F", "Motorola Mobility LLC, a Lenovo Company" },
    { "441CA8", "Hon Hai Precision Ind. Co.,Ltd." },
    { "441D64", "Espressif Inc." },
    { "441DB1", "APTIV SERVICES US, LLC" },
    { "441E91", "ARVIDA Intelligent Electronics Technology Co.,Ltd." },
    { "441E98", "Ruckus Wireless" },
    { "441EA1", "Hewlett Packard" },
    { "442063", "Continental Automotive Technologies GmbH" },
    { "44227C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "442295", "China Mobile Iot Limited company" },
    { "4422F1", "S.FAC, INC" },
    { "44237C", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "4423AA", "Farmage Co., Ltd." },
    { "4425BB", "Bamboo Entertainment Corporation" },
    { "44272E", "Huawei Device Co., Ltd." },
    { "442745", "LG Innotek" },
    { "4427F3", "70mai Co.,Ltd." },
    { "4428A3", "Jiangsu fulian Communication Technology Co., Ltd." },
    { "44291E", "AltoBeam (China) Inc." },
    { "442938", "NietZsche enterprise Co.Ltd." },
    { "442A60", "Apple, Inc." },
    { "442AFF", "E3 Technology, Inc." },
    { "442B03", "Cisco Systems, Inc" },
    { "442C05", "AMPAK Technology, Inc." },
    { "44303F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "443192", "Hewlett Packard" },
    { "44322A", "Avaya Inc" },
    { "443262", "zte corporation" },
    { "4432C2", "GOAL Co., Ltd." },
    { "4432C8", "Vantiva USA LLC" },
    { "44334C", "Shenzhen Bilian electronic CO.,LTD" },
    { "44348F", "MXT INDUSTRIAL LTDA" },
    { "4434A7", "Commscope" },
    { "44356F", "Neterix Ltd" },
    { "443583", "Apple, Inc." },
    { "4435B9", "NetComm Wireless Pty Ltd" },
    { "4435D3", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "443659", "Robert Bosch GmbH" },
    { "44365D", "Shenzhen HippStor Technology Co., Ltd" },
    { "443708", "MRV Comunications" },
    { "44370B", "Guangzhou Shiyuan Electronic Technology Company Limited" },
    { "443719", "2 Save Energy Ltd" },
    { "44376F", "Young Electric Sign Co" },
    { "4437E6", "Hon Hai Precision Ind. Co.,Ltd." },
    { "443839", "Cumulus Networks, inc" },
    { "44388C", "Sumitomo Electric Industries, Ltd" },
    { "4438E8", "Intel Corporate" },
    { "4438F3", "EM Microelectronic" },
    { "4439C4", "Universal Global Scientific Industrial Co., Ltd." },
    { "443B14", "MitraStar Technology Corp." },
    { "443B32", "Intelbras" },
    { "443C88", "FICOSA MAROC INTERNATIONAL" },
    { "443C9C", "Pintsch GmbH" },
    { "443D21", "Nuvolt" },
    { "443D54", "Amazon Technologies Inc." },
    { "443E07", "Electrolux" },
    { "443E8A", "Texas Instruments" },
    { "443EB2", "DEOTRON Co., LTD." },
    { "4441F0", "zte corporation" },
    { "444201", "Amazon Technologies Inc." },
    { "44422F", "TESTOP CO.,LTD." },
    { "444450", "OttoQ" },
    { "44456F", "SHENZHEN ONEGA TECHNOLOGY CO.,LTD" },
    { "444648", "Hon Hai Precision Industry Co.,LTD" },
    { "444687", "Realme Chongqing MobileTelecommunications Corp Ltd" },
    { "4447CC", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "444891", "HDMI Licensing, LLC" },
    { "4448B9", "MitraStar Technology Corp." },
    { "4448C1", "Hewlett Packard Enterprise " },
    { "4448FF", "Qingdao Haier Technology Co.,Ltd" },
    { "444963", "Woven By Toyota U.S., Inc." },
    { "444988", "Intel Corporate" },
    { "444A37", "Xiaomi Communications Co Ltd" },
    { "444A65", "Silverflare Ltd." },
    { "444AB0", "Zhejiang Moorgen Intelligence Technology Co., Ltd" },
    { "444AD6", "Shenzhen Rinocloud Technology Co.,Ltd." },
    { "444ADB", "Apple, Inc." },
    { "444B5D", "GE Healthcare" },
    { "444B7E", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "444C0C", "Apple, Inc." },
    { "444CA8", "Arista Networks" },
    { "444E1A", "Samsung Electronics Co.,Ltd" },
    { "444E6D", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "444F5E", "Pan Studios Co.,Ltd." },
    { "444F8E", "WiZ" },
    { "4451DB", "Raytheon BBN Technologies" },
    { "4454C0", "Thompson Aerospace" },
    { "4455B1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "4455C4", "Huawei Device Co., Ltd." },
    { "44568D", "PNC Technologies Co., Ltd." },
    { "4456B7", "Spawn Labs, Inc" },
    { "4456E2", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "445829", "Cisco SPVTG" },
    { "445925", "Square Inc." },
    { "445943", "zte corporation" },
    { "44599F", "Criticare Systems, Inc" },
    { "4459E3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "445ADF", "MIKAMI & CO., LTD." },
    { "445BED", "Hewlett Packard Enterprise" },
    { "445CE9", "Samsung Electronics Co.,Ltd" },
    { "445D5E", "SHENZHEN Coolkit Technology CO.,LTD" },
    { "445ECD", "Razer Inc" },
    { "445EF3", "Tonalite Holding B.V." },
    { "445F7A", "Shihlin Electric & Engineering Corp." },
    { "445F8C", "Intercel Group Limited" },
    { "446132", "ecobee inc" },
    { "44619C", "FONsystem co. ltd." },
    { "446246", "Comat AG" },
    { "446370", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "44643C", "Cisco Systems, Inc" },
    { "44650D", "Amazon Technologies Inc." },
    { "44656A", "Mega Video Electronic(HK) Industry Co., Ltd" },
    { "44657F", "Calix Inc." },
    { "4465E0", "Merlion Consulting Services (Shenzhen) Co., Ltd" },
    { "44666E", "IP-LINE" },
    { "4466FC", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "446747", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "446752", "Wistron INFOCOMM (Zhongshan) CORPORATION" },
    { "446755", "Orbit Irrigation" },
    { "44680C", "Wacom Co.,Ltd." },
    { "4468AB", "JUIN COMPANY, LIMITED" },
    { "446A2E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "446AB7", "Commscope" },
    { "446B1F", "Texas Instruments" },
    { "446C24", "Reallin Electronic Co.,Ltd" },
    { "446D05", "NoTraffic" },
    { "446D57", "Liteon Technology Corporation" },
    { "446D6C", "Samsung Electronics Co.,Ltd" },
    { "446D7F", "Amazon Technologies Inc." },
    { "446EE5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "446FD8", "IEEE Registration Authority" },
    { "446FF8", "Dyson Limited" },
    { "44700B", "IFFU" },
    { "447098", "MING HONG TECHNOLOGY (SHEN ZHEN) LIMITED" },
    { "447147", "Beijing Xiaomi Electronics Co.,Ltd" },
    { "4471B3", "Logitech Far East" },
    { "4472AC", "AltoBeam Inc." },
    { "4473D6", "Logitech" },
    { "44746C", "Sony Corporation" },
    { "447609", "New H3C Technologies Co., Ltd" },
    { "447654", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "4476E7", "TECNO MOBILE LIMITED" },
    { "44783E", "Samsung Electronics Co.,Ltd" },
    { "447BBB", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "447BC4", "DualShine Technology(SZ)Co.,Ltd" },
    { "447C7F", "Innolight Technology Corporation" },
    { "447DA5", "VTION INFORMATION TECHNOLOGY (FUJIAN) CO.,LTD" },
    { "447E76", "Trek Technology (S) Pte Ltd" },
    { "447E95", "Alpha and Omega, Inc" },
    { "447F77", "Connected Home" },
    { "4480EB", "Motorola Mobility LLC, a Lenovo Company" },
    { "4482E5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "448312", "Star-Net" },
    { "448500", "Intel Corporate" },
    { "448502", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "4485DA", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "4486C1", "Siemens Low Voltage & Products" },
    { "448723", "HOYA SERVICE CORPORATION" },
    { "448763", "FN-LINK TECHNOLOGY Ltd." },
    { "4487DB", "Tymphany Acoustic Technology (Huizhou) Co., Ltd." },
    { "4487FC", "Elitegroup Computer Systems Co.,Ltd." },
    { "448816", "Cisco Systems, Inc" },
    { "4488BE", "Texas Instruments" },
    { "4488CB", "Camco Technologies NV" },
    { "44896D", "TELLESCOM INDUSTRIA E COMERCIO EM TELECOMUNICACAO " },
    { "448A5B", "Micro-Star INT'L CO., LTD." },
    { "448C00", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "448C52", "KTIS CO., Ltd" },
    { "448CAB", "Beijing Flitlink Vientiane Technology Co., LTD" },
    { "448DBF", "Rhino Mobility LLC" },
    { "448DD5", "Cisco Systems, Inc" },
    { "448E12", "DT Research, Inc." },
    { "448E81", "VIG" },
    { "448EEC", "China Mobile Group Device Co.,Ltd." },
    { "448F17", "Samsung Electronics Co., Ltd. ARTIK" },
    { "449046", "Honor Device Co., Ltd." },
    { "4490BB", "Apple, Inc." },
    { "449160", "Murata Manufacturing Co., Ltd." },
    { "44917C", "HMD Global Oy" },
    { "4491DB", "Shanghai Huaqin Telecom Technology Co.,Ltd" },
    { "4494FC", "NETGEAR" },
    { "44953B", "RLTech India Private Limited" },
    { "4495FA", "Qingdao Santong Digital Technology Co.Ltd" },
    { "44962B", "Aidon Oy" },
    { "44975A", "SHENZHEN FAST TECHNOLOGIES CO.,LTD" },
    { "449B78", "The Now Factory" },
    { "449BC1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "449CB5", "Alcomp, Inc" },
    { "449E8B", "Apple, Inc." },
    { "449EF9", "vivo Mobile Communication Co., Ltd." },
    { "449F46", "Huawei Device Co., Ltd." },
    { "449F7F", "DataCore Software Corporation" },
    { "449FDA", "Silicon Laboratories" },
    { "44A038", "Huawei Device Co., Ltd." },
    { "44A10E", "Apple, Inc." },
    { "44A191", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "44A3BB", "Intel Corporate" },
    { "44A3C7", "zte corporation" },
    { "44A42D", "TCT mobile ltd" },
    { "44A466", "GROUPE LDLC" },
    { "44A54E", "Qorvo International Pte. Ltd." },
    { "44A56E", "NETGEAR" },
    { "44A61E", "INGRAM MICRO SERVICES" },
    { "44A642", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "44A689", "PROMAX ELECTRONICA SA" },
    { "44A6E5", "THINKING TECHNOLOGY CO.,LTD" },
    { "44A7CF", "Murata Manufacturing Co., Ltd." },
    { "44A7F4", "Apple, Inc." },
    { "44A842", "Dell Inc." },
    { "44A8C2", "SEWOO TECH CO., LTD" },
    { "44A8FC", "Apple, Inc." },
    { "44A92C", "IEEE Registration Authority" },
    { "44AA27", "udworks Co., Ltd." },
    { "44AA50", "Juniper Networks" },
    { "44AAE8", "Nanotec Electronic GmbH & Co. KG" },
    { "44AAF5", "Commscope" },
    { "44AC85", "eero inc." },
    { "44AD19", "XINGFEI （H.K）LIMITED " },
    { "44ADB1", "Sagemcom Broadband SAS" },
    { "44ADD9", "Cisco Systems, Inc" },
    { "44AE25", "Cisco Systems, Inc" },
    { "44AE44", "Huawei Device Co., Ltd." },
    { "44AEAB", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "44AF28", "Intel Corporate" },
    { "44B295", "Sichuan AI-Link Technology Co., Ltd." },
    { "44B32D", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "44B382", "Kuang-chi Institute of Advanced Technology" },
    { "44B3C5", "Huawei Device Co., Ltd." },
    { "44B412", "SIUS AG" },
    { "44B423", "HANWHA VISION VIETNAM COMPANY LIMITED" },
    { "44B433", "tide.co.,ltd" },
    { "44B462", "Flextronics Tech.(Ind) Pvt Ltd" },
    { "44B4B2", "Amazon Technologies Inc." },
    { "44B59C", "Tenet Networks Private Limited" },
    { "44B6BE", "Cisco Systems, Inc" },
    { "44B7D0", "Microchip Technology Inc." },
    { "44B994", "Douglas Lighting Controls" },
    { "44BA46", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "44BB3B", "Google, Inc." },
    { "44BDDE", "BHTC GmbH" },
    { "44BE0B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "44BFE3", "Shenzhen Longtech Electronics Co.,Ltd" },
    { "44C15C", "Texas Instruments" },
    { "44C20C", "Cisco Systems, Inc" },
    { "44C233", "Guangzhou Comet Technology Development Co.Ltd" },
    { "44C306", "SIFROM Inc." },
    { "44C346", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "44C39B", "OOO RUBEZH NPO" },
    { "44C3B6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "44C4A9", "Opticom Communication, LLC" },
    { "44C532", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "44C56F", "NGN Easy Satfinder (Tianjin) Electronic Co., Ltd" },
    { "44C63C", "Samsung Electronics Co.,Ltd" },
    { "44C65D", "Apple, Inc." },
    { "44C69B", "Wuhan Feng Tian Information Network CO.,LTD" },
    { "44C7FC", "Huawei Device Co., Ltd." },
    { "44C874", "China Mobile Group Device Co.,Ltd." },
    { "44C9A2", "Greenwald Industries" },
    { "44CB8B", "LG Innotek" },
    { "44CBAD", "Xiaomi Communications Co Ltd" },
    { "44CD0E", "FLEXTRONICS MANUFACTURING(ZHUHAI)CO.,LTD." },
    { "44CE3A", "Jiangsu Huacun Electronic Technology Co., Ltd." },
    { "44CE7D", "SFR" },
    { "44D15E", "Shanghai Kingto Information Technology Ltd" },
    { "44D1FA", "Shenzhen Yunlink Technology Co., Ltd" },
    { "44D244", "Seiko Epson Corporation" },
    { "44D267", "Snorble" },
    { "44D2CA", "Anvia TV Oy" },
    { "44D3AD", "Shenzhen TINNO Mobile Technology Corp." },
    { "44D3CA", "Cisco Systems, Inc" },
    { "44D437", "Inteno Broadband Technology AB" },
    { "44D453", "Sagemcom Broadband SAS" },
    { "44D454", "Sagemcom Broadband SAS" },
    { "44D465", "NXP Semiconductors Taiwan Ltd." },
    { "44D47F", "INGENICO TERMINALS SAS" },
    { "44D4E0", "Sony Corporation" },
    { "44D506", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "44D5A5", "AddOn Computer" },
    { "44D5C1", "EM Microelectronic" },
    { "44D5CC", "Amazon Technologies Inc." },
    { "44D5F2", "IEEE Registration Authority" },
    { "44D63D", "Talari Networks" },
    { "44D6E1", "Snuza International Pty. Ltd." },
    { "44D77E", "Robert Bosch GmbH" },
    { "44D791", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "44D832", "AzureWave Technology Inc." },
    { "44D878", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "44D884", "Apple, Inc." },
    { "44D980", "EVERYBOT INC." },
    { "44D9E7", "Ubiquiti Inc" },
    { "44DA30", "Apple, Inc." },
    { "44DB60", "Nanjing Baihezhengliu Technology Co., Ltd" },
    { "44DBD2", "YEALINK(XIAMEN) NETWORK TECHNOLOGY CO.,LTD." },
    { "44DC4E", "ITEL MOBILE LIMITED" },
    { "44DC91", "PLANEX COMMUNICATIONS INC." },
    { "44DCCB", "SEMINDIA SYSTEMS PVT LTD" },
    { "44DF65", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "44E08E", "Cisco SPVTG" },
    { "44E137", "Commscope" },
    { "44E2F1", "NewRadio Technologies Co. , Ltd." },
    { "44E2F8", "Silicon Laboratories" },
    { "44E49A", "OMNITRONICS PTY LTD" },
    { "44E4D9", "Cisco Systems, Inc" },
    { "44E4E6", "Extreme Networks Headquarters" },
    { "44E4EE", "Wistron Neweb Corporation" },
    { "44E517", "Intel Corporate" },
    { "44E59B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "44E64A", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "44E66E", "Apple, Inc." },
    { "44E6B0", "China Mobile IOT Company Limited" },
    { "44E761", "Infinix mobility limited" },
    { "44E8A5", "Myreka Technologies Sdn. Bhd." },
    { "44E968", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "44E9DD", "Sagemcom Broadband SAS" },
    { "44EA30", "Samsung Electronics Co.,Ltd" },
    { "44EA4B", "Actlas Inc." },
    { "44EAD8", "Texas Instruments" },
    { "44EB2E", "ALPSALPINE CO,.LTD" },
    { "44ECCE", "Juniper Networks" },
    { "44ED57", "Longicorn, inc." },
    { "44EE02", "MTI Ltd." },
    { "44EE14", "Texas Instruments" },
    { "44EE30", "Budelmann Elektronik GmbH" },
    { "44EFBF", "China Dragon Technology Limited" },
    { "44EFCF", "UGENE SOLUTION inc." },
    { "44F034", "Kaon Group Co., Ltd." },
    { "44F09E", "Apple, Inc." },
    { "44F21B", "Apple, Inc." },
    { "44F436", "zte corporation" },
    { "44F459", "Samsung Electronics Co.,Ltd" },
    { "44F477", "Juniper Networks" },
    { "44F4E7", "Cohesity Inc" },
    { "44F53E", "Earda Technologies co Ltd" },
    { "44F770", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "44F849", "Union Pacific Railroad" },
    { "44F971", "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "44FA66", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "44FB42", "Apple, Inc." },
    { "44FB5A", "zte corporation" },
    { "44FDA3", "Everysight LTD." },
    { "44FE3B", "Arcadyan Corporation" },
    { "44FEEF", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "44FFBA", "zte corporation" },
    { "480020", "Hewlett Packard Enterprise" },
    { "480031", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "480033", "Vantiva USA LLC" },
    { "48007D", "DTS ELEKTRONIK SAN. TIC. LTD. STI." },
    { "4800B3", "Cisco Systems, Inc" },
    { "4801C5", "OnePlus Technology (Shenzhen) Co., Ltd" },
    { "48022A", "B-Link Electronic Limited" },
    { "480234", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "480286", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "4802AF", "Telit Communication s.p.a" },
    { "480362", "DESAY ELECTRONICS(HUIZHOU)CO.,LTD" },
    { "48049F", "ELECOM CO., LTD" },
    { "480560", "Meta Platforms, Inc." },
    { "4805E2", "Huawei Device Co., Ltd." },
    { "48062B", "Private" },
    { "48066A", "Tempered Networks, Inc." },
    { "480BB2", "IEEE Registration Authority" },
    { "480C49", "NAKAYO Inc" },
    { "480E13", "ittim" },
    { "480EEC", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "480FCF", "Hewlett Packard" },
    { "481063", "NTT Innovation Institute, Inc." },
    { "481249", "Luxcom Technologies Inc." },
    { "481258", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "48128F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "48137E", "Samsung Electronics Co.,Ltd" },
    { "4813F3", "BBK EDUCATIONAL ELECTRONICS CORP.,LTD." },
    { "481693", "Lear Corporation GmbH" },
    { "48174C", "MicroPower technologies" },
    { "481842", "Shanghai Winaas Co. Equipment Co. Ltd." },
    { "48188D", "WEIFANG GOERTEK ELECTRONICS CO.,LTD" },
    { "4818FA", "Nocsys" },
    { "481A84", "Pointer Telocation Ltd" },
    { "481B40", "Vantiva USA LLC" },
    { "481BA4", "Cisco Systems, Inc" },
    { "481BD2", "Intron Scientific co., ltd." },
    { "481CB9", "SZ DJI TECHNOLOGY CO.,LTD" },
    { "481D70", "Cisco SPVTG" },
    { "481F2D", "Shenzhen Jie Shi Lian Industrial Co.,LTD" },
    { "481F66", "China Mobile Group Device Co.,Ltd." },
    { "48210B", "PEGATRON CORPORATION" },
    { "48216C", "China Mobile IOT Company Limited" },
    { "482218", "Shenzhen Yipingfang Network Technology Co., Ltd." },
    { "482254", "TP-Link Systems Inc" },
    { "482335", "Dialog Semiconductor Hellas SA" },
    { "482567", "Poly" },
    { "4825F3", "Huawei Device Co., Ltd." },
    { "48262C", "Apple, Inc." },
    { "48264C", "BSH Electrical Appliances (Jiangsu) Co., Ltd." },
    { "4826E8", "Tek-Air Systems, Inc." },
    { "482759", "Levven Electronics Ltd." },
    { "4827C5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "4827E2", "Espressif Inc." },
    { "4827EA", "Samsung Electronics Co.,Ltd" },
    { "48282F", "zte corporation" },
    { "482952", "Sagemcom Broadband SAS" },
    { "4829D6", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "4829E4", "AO " },
    { "482AE3", "Wistron InfoComm(Kunshan)Co.,Ltd." },
    { "482CA0", "Xiaomi Communications Co Ltd" },
    { "482CD0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "482CEA", "Motorola Inc Business Light Radios" },
    { "482D63", "Wavarts Technologies Co., Ltd" },
    { "482E72", "Cisco Systems, Inc" },
    { "482F6B", "Hewlett Packard Enterprise" },
    { "482FD7", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "483133", "Robert Bosch Elektronika Kft." },
    { "483177", "Nintendo Co.,Ltd" },
    { "4831B7", "Espressif Inc." },
    { "4831DB", "Huawei Device Co., Ltd." },
    { "4833DD", "ZENNIO AVANCE Y TECNOLOGIA, S.L." },
    { "48343D", "IEP GmbH" },
    { "48352B", "Apple, Inc." },
    { "48352E", "Shenzhen Wolck Network Product Co.,LTD" },
    { "483543", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "483584", "Huawei Device Co., Ltd." },
    { "48365F", "Wintecronics Ltd." },
    { "483871", "Huawei Device Co., Ltd." },
    { "4838B6", "Auhui Taoyun Technology Co., Ltd" },
    { "483974", "Proware Technologies Co., Ltd." },
    { "483A02", "Fortinet, Inc." },
    { "483B38", "Apple, Inc." },
    { "483C0C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "483D32", "Syscor Controls &amp; Automation" },
    { "483E5E", "SERNET (SUZHOU) TECHNOLOGIES CORPORATION" },
    { "483FDA", "Espressif Inc." },
    { "483FE9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "48417B", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "48435A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "48437C", "Apple, Inc." },
    { "4843DD", "Amazon Technologies Inc." },
    { "484487", "Cisco SPVTG" },
    { "4844F7", "Samsung Electronics Co.,Ltd" },
    { "484520", "Intel Corporate" },
    { "4845CF", "LLC Proizvodstvennaya Kompania TransService" },
    { "4845E6", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "48468D", "Zepcam B.V." },
    { "4846C1", "FN-LINK TECHNOLOGY LIMITED" },
    { "4846F1", "Uros Oy" },
    { "4846FB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "48474B", "Huawei Device Co., Ltd." },
    { "484982", "Huawei Device Co., Ltd." },
    { "484996", "Huawei Device Co., Ltd." },
    { "4849C7", "Samsung Electronics Co.,Ltd" },
    { "484A30", "George Robotics Limited" },
    { "484AE9", "Hewlett Packard Enterprise " },
    { "484BAA", "Apple, Inc." },
    { "484BD4", "Vantiva USA LLC" },
    { "484C29", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "484C86", "Huawei Device Co., Ltd." },
    { "484D7E", "Dell Inc." },
    { "484EFC", "Commscope" },
    { "485073", "Microsoft Corporation" },
    { "485169", "Samsung Electronics Co.,Ltd" },
    { "4851B7", "Intel Corporate" },
    { "4851C5", "Intel Corporate" },
    { "4851CF", "Intelbras" },
    { "4851D0", "Jiangsu Xinsheng Intelligent Technology Co., Ltd. " },
    { "485261", "SOREEL" },
    { "485415", "NET RULES TECNOLOGIA EIRELI" },
    { "485519", "Espressif Inc." },
    { "485541", "Iskratel d.o.o." },
    { "48555C", "Wu Qi Technologies,Inc." },
    { "48555E", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "48555F", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "485702", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "4857D2", "Broadcom Limited" },
    { "4857DD", "Facebook Inc" },
    { "485929", "LG Electronics (Mobile Communications)" },
    { "4859A4", "zte corporation" },
    { "485A0D", "Juniper Networks" },
    { "485A3F", "WISOL" },
    { "485A67", "Shaanxi Ruixun Electronic Information Technology Co., Ltd" },
    { "485AB6", "Hon Hai Precision Ind. Co.,Ltd." },
    { "485AEA", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "485B39", "ASUSTek COMPUTER INC." },
    { "485C2C", "Earda Technologies co Ltd" },
    { "485D35", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "485D36", "Verizon " },
    { "485D60", "AzureWave Technology Inc." },
    { "485DEB", "Just Add Power" },
    { "485DED", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "485E0E", "IEEE Registration Authority" },
    { "485F08", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "485F2D", "Amazon Technologies Inc." },
    { "485F99", "Cloud Network Technology (Samoa) Limited" },
    { "485FDF", "zte corporation" },
    { "48605F", "LG Electronics (Mobile Communications)" },
    { "4860BC", "Apple, Inc." },
    { "4861A3", "Concern Axion JSC" },
    { "4861EE", "Samsung Electronics Co.,Ltd" },
    { "486264", "Arlo Technology" },
    { "486276", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "486345", "Huawei Device Co., Ltd." },
    { "4865EE", "IEEE Registration Authority" },
    { "486834", "Silicon Motion, Inc." },
    { "48684A", "Intel Corporate" },
    { "486B2C", "BBK EDUCATIONAL ELECTRONICS CORP.,LTD." },
    { "486B91", "Fleetwood Group Inc." },
    { "486DBB", "Vestel Elektronik San ve Tic. A.S." },
    { "486E70", "Zhejiang Tmall Technology Co., Ltd." },
    { "486E73", "Pica8, Inc." },
    { "486EFB", "Davit System Technology Co., Ltd." },
    { "486F33", "KYUNGWOO.SYSTEM, INC." },
    { "486FD2", "StorSimple Inc" },
    { "48701E", "Texas Instruments" },
    { "48706F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "487119", "SGB GROUP LTD." },
    { "487310", "Juniper Networks" },
    { "487397", "New H3C Technologies Co., Ltd" },
    { "4873CB", "Tiinlab Corporation" },
    { "487410", "Cisco Systems, Inc" },
    { "487412", "OnePlus Technology (Shenzhen) Co., Ltd" },
    { "48746E", "Apple, Inc." },
    { "487583", "Intellion AG" },
    { "487604", "Private" },
    { "487706", "NXP Semiconductor (Tianjin) LTD." },
    { "487746", "Calix Inc." },
    { "4877BD", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "48785E", "Amazon Technologies Inc." },
    { "48794D", "Samsung Electronics Co.,Ltd" },
    { "487A55", "ALE International" },
    { "487ADA", "Hangzhou H3C Technologies Co., Limited" },
    { "487AF6", "NCS ELECTRICAL SDN BHD" },
    { "487AFF", "ESSYS" },
    { "487B2F", "Microsoft Corporation" },
    { "487B5E", "SMT TELECOMM HK" },
    { "487B6B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "487D2E", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "487E48", "Earda Technologies co Ltd" },
    { "488002", "Cisco Systems, Inc" },
    { "48814E", "E&M SOLUTION CO,.Ltd" },
    { "4881D4", "Ruijie Networks Co.,LTD" },
    { "488244", "Life Fitness / Div. of Brunswick" },
    { "4882F2", "Appel Elektronik GmbH" },
    { "48836F", "EM Microelectronic" },
    { "4883B4", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "4883C7", "Sagemcom Broadband SAS" },
    { "48849D", "Texas Instruments" },
    { "4886E8", "Microsoft Corporation" },
    { "48872D", "SHEN ZHEN DA XIA LONG QUE TECHNOLOGY CO.,LTD" },
    { "488759", "Xiaomi Communications Co Ltd" },
    { "488764", "vivo Mobile Communication Co., Ltd." },
    { "4887B8", "TCL King Electrical Appliances(Huizhou)Co.,Ltd" },
    { "488803", "ManTechnology Inc." },
    { "48881E", "EthoSwitch LLC" },
    { "488899", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "4888CA", "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd." },
    { "4889E7", "Intel Corporate" },
    { "488AD2", "MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "488AE8", "vivo Mobile Communication Co., Ltd." },
    { "488B0A", "Cisco Systems, Inc" },
    { "488C63", "Huawei Device Co., Ltd." },
    { "488C78", "Alpha Networks Inc." },
    { "488D36", "Arcadyan Corporation" },
    { "488E42", "DIGALOG GmbH" },
    { "488EB7", "Zebra Technologies Inc." },
    { "488EEF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "488F4C", "shenzhen trolink Technology Co.,Ltd" },
    { "488F5A", "Routerboard.com" },
    { "48902F", "LG Electronics (Mobile Communications)" },
    { "489153", "Weinmann Geräte für Medizin GmbH + Co. KG" },
    { "4891D5", "Cisco Systems, Inc" },
    { "4891F6", "Shenzhen Reach software technology CO.,LTD" },
    { "4893DC", "UNIWAY INFOCOM PVT LTD" },
    { "489507", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "4896D9", "zte corporation" },
    { "4898CA", "Sichuan AI-Link Technology Co., Ltd." },
    { "489A42", "Technomate Ltd" },
    { "489A5B", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "489BD5", "Extreme Networks Headquarters" },
    { "489BE0", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "489BE2", "SCI Innovations Ltd" },
    { "489D18", "Flashbay Limited" },
    { "489D24", "BlackBerry RTS" },
    { "489DD1", "Samsung Electronics Co.,Ltd" },
    { "489E9D", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "489EBD", "HP Inc." },
    { "489ECB", "Hewlett Packard Enterprise " },
    { "48A0F8", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "48A170", "Cisco Systems, Inc" },
    { "48A195", "Apple, Inc." },
    { "48A22D", "Shenzhen Huaxuchang Telecom Technology Co.,Ltd" },
    { "48A2B7", "Kodofon JSC" },
    { "48A2B8", "Chengdu Vision-Zenith Tech.Co,.Ltd" },
    { "48A2E6", "Resideo" },
    { "48A380", "Gionee Communication Equipment Co.,Ltd." },
    { "48A3BD", "Texas Instruments" },
    { "48A472", "Intel Corporate" },
    { "48A48C", "Shanghai Zenchant Electornics Co.,LTD" },
    { "48A493", "TAIYO YUDEN CO.,LTD" },
    { "48A4FD", "AltoBeam Inc." },
    { "48A516", "Huawei Device Co., Ltd." },
    { "48A5E7", "Nintendo Co.,Ltd" },
    { "48A6B8", "Sonos, Inc." },
    { "48A6D2", "GJsun Optical Science and Tech Co.,Ltd." },
    { "48A73C", "Sichuan tianyi kanghe communications co., LTD" },
    { "48A74E", "zte corporation" },
    { "48A91C", "Apple, Inc." },
    { "48A964", "APEXSHA SMARTTECH PRIVATE LIMITED" },
    { "48A98A", "Routerboard.com" },
    { "48A9D2", "Wistron Neweb Corporation" },
    { "48AA5D", "Store Electronic Systems" },
    { "48AD08", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "48AD9A", "Intel Corporate" },
    { "48B02D", "NVIDIA Corporation" },
    { "48B253", "Marketaxess Corporation" },
    { "48B25D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "48B313", "Idesco Oy" },
    { "48B423", "Amazon Technologies Inc." },
    { "48B424", "eero inc." },
    { "48B4C3", "Hewlett Packard Enterprise" },
    { "48B5A7", "Glory Horse Industries Ltd." },
    { "48B620", "ROLI Ltd." },
    { "48B8A3", "Apple, Inc." },
    { "48B8DE", "HOMEWINS TECHNOLOGY CO.,LTD." },
    { "48B977", "PulseOn Oy" },
    { "48B9C2", "Teletics Inc." },
    { "48BA4E", "Hewlett Packard" },
    { "48BC0E", "Sichuan AI-Link Technology Co., Ltd." },
    { "48BCA6", "​ASUNG TECHNO CO.,Ltd" },
    { "48BCE1", "Samsung Electronics Co.,Ltd" },
    { "48BD0E", "Quanta Storage Inc." },
    { "48BD3D", "New H3C Technologies Co., Ltd" },
    { "48BD4A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "48BDA7", "Honor Device Co., Ltd." },
    { "48BDCE", "Vantiva USA LLC" },
    { "48BE2D", "Symanitron" },
    { "48BF6B", "Apple, Inc." },
    { "48BF74", "Baicells Technologies Co.,LTD" },
    { "48C030", "Kogniza Inc." },
    { "48C049", "Broad Telecom SA" },
    { "48C093", "Xirrus, Inc." },
    { "48C1AC", "PLANTRONICS, INC." },
    { "48C1EE", "Honor Device Co., Ltd." },
    { "48C35A", "LENOVO(BEIJING)CO., LTD." },
    { "48C381", "TP-Link Systems Inc." },
    { "48C3B0", "Pharos Co.Ltd" },
    { "48C461", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "48C58D", "Lear Corporation GmbH" },
    { "48C663", "GTO Access Systems LLC" },
    { "48C796", "Samsung Electronics Co.,Ltd" },
    { "48C862", "Simo Wireless,Inc." },
    { "48C8B6", "SysTec GmbH" },
    { "48CA43", "Espressif Inc." },
    { "48CA68", "Apple, Inc." },
    { "48CAC6", "UNION MAN TECHNOLOGY CO.,LTD" },
    { "48CB6E", "Cello Electronics (UK) Ltd" },
    { "48CDD3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "48CFA9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "48D017", "Telecom Infra Project" },
    { "48D01C", "AltoBeam Inc." },
    { "48D0CF", "Universal Electronics, Inc." },
    { "48D18E", "Metis Communication Co.,Ltd" },
    { "48D224", "Liteon Technology Corporation" },
    { "48D24F", "Sagemcom Broadband SAS" },
    { "48D343", "Commscope" },
    { "48D35D", "Private" },
    { "48D475", "Lampuga GmbH" },
    { "48D539", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "48D54C", "Jeda Networks" },
    { "48D682", "zte corporation" },
    { "48D6D5", "Google, Inc." },
    { "48D705", "Apple, Inc." },
    { "48D7FF", "BLANKOM Antennentechnik GmbH" },
    { "48D845", "Shenzhen Mainuoke Electronics Co., Ltd" },
    { "48D855", "Telvent" },
    { "48D875", "China TransInfo Technology Co., Ltd" },
    { "48D890", "FN-LINK TECHNOLOGY LIMITED" },
    { "48D8FE", "ClarIDy Solutions, Inc." },
    { "48DA35", "IEEE Registration Authority" },
    { "48DA96", "Eddy Smart Home Solutions Inc." },
    { "48DB50", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "48DC2D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "48DC9D", "Grandprint(Beijing) Technology Co., LTD." },
    { "48DCFB", "Nokia Corporation" },
    { "48DD0C", "eero inc." },
    { "48DD9D", "ITEL MOBILE LIMITED" },
    { "48DF1C", "Wuhan NEC Fibre Optic Communications industry Co. Ltd" },
    { "48DF37", "Hewlett Packard Enterprise " },
    { "48E150", "Intel Corporate" },
    { "48E15C", "Apple, Inc." },
    { "48E1AF", "Vity" },
    { "48E1E9", "Chengdu Meross Technology Co., Ltd." },
    { "48E244", "Hon Hai Precision Ind. Co.,Ltd." },
    { "48E2AD", "HUMAX NETWORKS" },
    { "48E3C3", "JENOPTIK Advanced Systems GmbH" },
    { "48E533", "Sunnovo International Limited" },
    { "48E663", "IEEE Registration Authority" },
    { "48E695", "Insigma Inc" },
    { "48E6C0", "SIMCom Wireless Solutions Co.,Ltd." },
    { "48E6C6", "IEEE Registration Authority" },
    { "48E729", "Espressif Inc." },
    { "48E7DA", "AzureWave Technology Inc." },
    { "48E9CA", "creoline GmbH" },
    { "48E9F1", "Apple, Inc." },
    { "48EA62", "HP Inc." },
    { "48EA63", "Zhejiang Uniview Technologies Co., Ltd." },
    { "48EB30", "ETERNA TECHNOLOGY, INC." },
    { "48EB62", "Murata Manufacturing Co., Ltd." },
    { "48EB65", "Henan KunLun Technologies CO.,Ltd." },
    { "48EC5B", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "48ED80", "daesung eltec" },
    { "48EDE6", "Zyxel Communications Corporation" },
    { "48EE07", "Silver Palm Technologies LLC" },
    { "48EE0C", "D-Link International" },
    { "48EE86", "UTStarcom (China) Co.,Ltd" },
    { "48EEE2", "ROAMWIFI TECHNOLOGY(HK) LIMITED" },
    { "48EF1C", "Samsung Electronics Co.,Ltd" },
    { "48EF61", "Huawei Device Co., Ltd." },
    { "48F027", "Chengdu newifi Co.,Ltd" },
    { "48F07B", "ALPSALPINE CO,.LTD" },
    { "48F17F", "Intel Corporate" },
    { "48F1EB", "Nintendo Co.,Ltd" },
    { "48F230", "Ubizcore Co.,LTD" },
    { "48F317", "Private" },
    { "48F3F3", "Baidu Online Network Technology (Beijing) Co., Ltd" },
    { "48F47D", "TechVision Holding Internation Limited" },
    { "48F6EE", "Espressif Inc." },
    { "48F7BC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "48F7C0", "Vantiva USA LLC" },
    { "48F7F1", "Nokia" },
    { "48F8B3", "Cisco-Linksys, LLC" },
    { "48F8DB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "48F8E1", "Nokia" },
    { "48F8FF", "CHENGDU KT ELECTRONIC HI-TECH CO.,LTD" },
    { "48F925", "Maestronic" },
    { "48F97C", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "48FC07", "Huawei Device Co., Ltd." },
    { "48FCB6", "LAVA INTERNATIONAL(H.K) LIMITED" },
    { "48FCB8", "Woodstream Corporation" },
    { "48FD8E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "48FDA3", "Xiaomi Communications Co Ltd" },
    { "48FEEA", "HOMA B.V." },
    { "4C0082", "Cisco Systems, Inc" },
    { "4C0143", "eero inc." },
    { "4C01F7", "Cisco Systems, Inc" },
    { "4C0220", "Xiaomi Communications Co Ltd" },
    { "4C022E", "CMR KOREA CO., LTD" },
    { "4C0289", "LEX COMPUTECH CO., LTD" },
    { "4C034F", "Intel Corporate" },
    { "4C0617", "Taicang T&W Electronics" },
    { "4C068A", "Basler Electric Company" },
    { "4C06B7", "ProDVX Europe B.V." },
    { "4C07C9", "COMPUTER OFFICE Co.,Ltd." },
    { "4C09B4", "zte corporation" },
    { "4C09D4", "Arcadyan Technology Corporation" },
    { "4C09FA", "FRONTIER SMART TECHNOLOGIES LTD" },
    { "4C0A3D", "ADNACOM INC." },
    { "4C0A4E", "Extreme Networks Headquarters" },
    { "4C0B3A", "TCT mobile ltd" },
    { "4C0BBE", "Microsoft" },
    { "4C0DEE", "JABIL CIRCUIT (SHANGHAI) LTD." },
    { "4C0F3E", "Intel Corporate" },
    { "4C0F6E", "Hon Hai Precision Ind. Co.,Ltd." },
    { "4C0FC7", "Earda Technologies co Ltd" },
    { "4C10D5", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "4C1154", "Mobiwire Mobiles (NingBo) Co., LTD" },
    { "4C1159", "Vision Information & Communications" },
    { "4C11AE", "Espressif Inc." },
    { "4C11BF", "Zhejiang Dahua Technology Co., Ltd." },
    { "4C1265", "Commscope" },
    { "4C12E8", "VIETNAM POST AND TELECOMMUNICATION INDUSTRY TECHNOLOGY JOIN STOCK COMPANY" },
    { "4C1365", "Emplus Technologies" },
    { "4C1480", "NOREGON SYSTEMS, INC" },
    { "4C14A3", "TCL Technoly Electronics (Huizhou) Co., Ltd." },
    { "4C1694", "shenzhen sibituo Technology Co., Ltd" },
    { "4C16F1", "zte corporation" },
    { "4C16FC", "Juniper Networks" },
    { "4C1744", "Amazon Technologies Inc." },
    { "4C17EB", "Sagemcom Broadband SAS" },
    { "4C189A", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "4C195D", "Sagemcom Broadband SAS" },
    { "4C1A3A", "PRIMA Research And Production Enterprise Ltd." },
    { "4C1A3D", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "4C1A95", "Novakon Co., Ltd." },
    { "4C1B86", "Arcadyan Corporation" },
    { "4C1D96", "Intel Corporate" },
    { "4C1F86", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "4C1FCC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "4C20B8", "Apple, Inc." },
    { "4C2113", "Nokia Shanghai Bell Co., Ltd." },
    { "4C218C", "Panasonic India Private limited" },
    { "4C21D0", "Sony Corporation" },
    { "4C2219", "YUANFUDAO HK LIMTED" },
    { "4C2258", "cozybit, Inc." },
    { "4C22C9", "zte corporation" },
    { "4C22F3", "Arcadyan Corporation" },
    { "4C231A", "Extreme Networks Headquarters" },
    { "4C2338", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "4C2498", "Texas Instruments" },
    { "4C24CE", "Sichuan AI-Link Technology Co., Ltd." },
    { "4C2578", "Nokia Corporation" },
    { "4C26E7", "Welgate Co., Ltd." },
    { "4C2B3B", "Huawei Device Co., Ltd." },
    { "4C2C80", "Beijing Skyway Technologies Co.,Ltd " },
    { "4C2C83", "Zhejiang KaNong Network Technology Co.,Ltd." },
    { "4C2E5E", "Samsung Electronics Co.,Ltd" },
    { "4C2EB4", "Apple, Inc." },
    { "4C2EFE", "Shenzhen Comnect Technology Co.,LTD" },
    { "4C2F7B", "AltoBeam Inc." },
    { "4C2F9D", "ICM Controls" },
    { "4C2FD7", "Huawei Device Co., Ltd." },
    { "4C3089", "Hitachi Rail GTS Deutschland GmbH" },
    { "4C312D", "Sichuan AI-Link Technology Co., Ltd." },
    { "4C322D", "TELEDATA NETWORKS" },
    { "4C3275", "Apple, Inc." },
    { "4C32D9", "M Rutty Holdings Pty. Ltd." },
    { "4C3329", "Sweroam" },
    { "4C334E", "HIGHTECH" },
    { "4C3488", "Intel Corporate" },
    { "4C364E", "Panasonic Connect Co., Ltd." },
    { "4C37DE", "AltoBeam Inc." },
    { "4C38D5", "MITAC COMPUTING TECHNOLOGY CORPORATION" },
    { "4C38D8", "Commscope" },
    { "4C3909", "HPL Electric & Power Private Limited" },
    { "4C3910", "Newtek Electronics co., Ltd." },
    { "4C3946", "Samsung Electronics Co.,Ltd" },
    { "4C3B6C", "GARO AB" },
    { "4C3B74", "VOGTEC(H.K.) Co., Ltd" },
    { "4C3BDF", "Microsoft Corporation" },
    { "4C3C16", "Samsung Electronics Co.,Ltd" },
    { "4C3FA7", "uGrid Network Inc." },
    { "4C3FD3", "Texas Instruments" },
    { "4C4088", "SANSHIN ELECTRONICS CO.,LTD." },
    { "4C421E", "Cisco Systems, Inc" },
    { "4C4341", "Calix Inc." },
    { "4C445B", "Intel Corporate" },
    { "4C4576", "China Mobile(Hangzhou) Information Technology Co.,Ltd." },
    { "4C46D1", "Guangzhou V-Solution Telecommunication Technology Co.,Ltd." },
    { "4C48DA", "Beijing Autelan Technology Co.,Ltd" },
    { "4C4929", "TCL King Electrical Appliances(Huizhou)Co.,Ltd" },
    { "4C494F", "zte corporation" },
    { "4C4968", "Ruijie Networks Co.,LTD" },
    { "4C496C", "Intel Corporate" },
    { "4C49E3", "Xiaomi Communications Co Ltd" },
    { "4C4B68", "Mobile Device, Inc. " },
    { "4C4BF9", "IEEE Registration Authority" },
    { "4C4CD8", "zte corporation" },
    { "4C4D66", "Nanjing Jiahao Technology Co., Ltd." },
    { "4C4E03", "TCT mobile ltd" },
    { "4C4E35", "Cisco Systems, Inc" },
    { "4C4FEE", "OnePlus Technology (Shenzhen) Co., Ltd" },
    { "4C5077", "Huawei Device Co., Ltd." },
    { "4C50DD", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "4C50F1", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "4C5262", "Fujitsu Technology Solutions GmbH" },
    { "4C52EC", "SOLARWATT GmbH" },
    { "4C5369", "YanFeng Visteon(ChongQing) Automotive Electronic Co.,Ltd" },
    { "4C53FD", "Amazon Technologies Inc." },
    { "4C5427", "Linepro Sp. z o.o." },
    { "4C5499", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "4C5585", "Hamilton Systems" },
    { "4C55B8", "Turkcell Teknoloji" },
    { "4C55CC", "Zentri Pty Ltd" },
    { "4C569D", "Apple, Inc." },
    { "4C56DF", "Targus US LLC" },
    { "4C5739", "Samsung Electronics Co.,Ltd" },
    { "4C57CA", "Apple, Inc." },
    { "4C5BB3", "Silicon Laboratories" },
    { "4C5CDF", "ITEL MOBILE LIMITED" },
    { "4C5D3C", "Cisco Systems, Inc" },
    { "4C5D6A", "Apple, Inc." },
    { "4C5DCD", "Oy Finnish Electric Vehicle Technologies Ltd" },
    { "4C5E0C", "Routerboard.com" },
    { "4C5ED3", "Unisyue Technologies Co; LTD." },
    { "4C5F70", "Intel Corporate" },
    { "4C5FD2", "Alcatel-Lucent" },
    { "4C60AD", "Amazon Technologies Inc." },
    { "4C60BA", "AltoBeam Inc." },
    { "4C60D5", "airPointe of New Hampshire" },
    { "4C60DE", "NETGEAR" },
    { "4C617E", "Huawei Device Co., Ltd." },
    { "4C6255", "SANMINA-SCI SYSTEM DE MEXICO S.A. DE C.V." },
    { "4C627B", "SmartCow AI Technologies Taiwan Ltd." },
    { "4C62CD", "Nokia" },
    { "4C62DF", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "4C6371", "Xiaomi Communications Co Ltd" },
    { "4C63AD", "Huawei Device Co., Ltd." },
    { "4C63EB", "Application Solutions (Electronics and Vision) Ltd" },
    { "4C6460", "ITEL MOBILE LIMITED" },
    { "4C64D9", "Guangdong Leawin Group Co., Ltd" },
    { "4C65A8", "IEEE Registration Authority" },
    { "4C6641", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "4C66A6", "Samsung Electronics Co.,Ltd" },
    { "4C6AF6", "HMD Global Oy" },
    { "4C6BB8", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "4C6BE8", "Apple, Inc." },
    { "4C6C13", "IoT Company Solucoes Tecnologicas Ltda" },
    { "4C6D58", "Juniper Networks" },
    { "4C6E6E", "Comnect Technology CO.,LTD" },
    { "4C6F9C", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "4C70CC", "Blyott NV" },
    { "4C710C", "Cisco Systems, Inc" },
    { "4C710D", "Cisco Systems, Inc" },
    { "4C7167", "PoLabs d.o.o." },
    { "4C7274", "Shenzhenshi Xinzhongxin Technology Co.Ltd" },
    { "4C72B9", "PEGATRON CORPORATION" },
    { "4C734F", "Juniper Networks" },
    { "4C7367", "Genius Bytes Software Solutions GmbH" },
    { "4C73A5", "KOVE" },
    { "4C7403", "BQ" },
    { "4C7487", "Leader Phone Communication Technology Co., Ltd." },
    { "4C74A7", "IEEE Registration Authority" },
    { "4C74BF", "Apple, Inc." },
    { "4C7525", "Espressif Inc." },
    { "4C7625", "Dell Inc." },
    { "4C7713", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "4C774F", "Embedded Wireless Labs " },
    { "4C7766", "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "4C776D", "Cisco Systems, Inc" },
    { "4C77CB", "Intel Corporate" },
    { "4C7872", "Cav. Uff. Giacomo Cimberio S.p.A. " },
    { "4C7897", "Arrowhead Alarm Products Ltd" },
    { "4C796E", "Intel Corporate" },
    { "4C7975", "Apple, Inc." },
    { "4C79BA", "Intel Corporate" },
    { "4C7A48", "Nippon Seiki (Europe) B.V." },
    { "4C7B35", "UNIONMAN TECHNOLOGY CO.,LTD" },
    { "4C7C5F", "Apple, Inc." },
    { "4C7CD9", "Apple, Inc." },
    { "4C7F62", "Nokia Corporation" },
    { "4C804F", "Armstrong Monitoring Corp" },
    { "4C8093", "Intel Corporate" },
    { "4C80BA", "Wuhan Tianyu Information Industry Co., Ltd." },
    { "4C8120", "Taicang T&W Electronics" },
    { "4C8125", "ZOWEE TECHNOLOGY(HEYUAN)Co.,Ltd" },
    { "4C820C", "Apple, Inc." },
    { "4C8237", "Telink Micro LLC" },
    { "4C82A9", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "4C82CF", "Dish Technologies Corp" },
    { "4C83DE", "Cisco SPVTG" },
    { "4C858A", "BUFFALO.INC" },
    { "4C875D", "Bose Corporation" },
    { "4C889E", "Huawei Device Co., Ltd." },
    { "4C8B30", "Actiontec Electronics, Inc" },
    { "4C8B55", "Grupo Digicon" },
    { "4C8BEF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "4C8D53", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "4C8D79", "Apple, Inc." },
    { "4C8ECC", "SILKAN SA" },
    { "4C8FA5", "Jastec" },
    { "4C90DB", "JL Audio" },
    { "4C910C", "Lanix Internacional, S.A. de C.V." },
    { "4C9157", "Fujian LANDI Commercial Equipment Co.,Ltd" },
    { "4C917A", "IEEE Registration Authority" },
    { "4C92D2", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "4C93A6", "IEEE Registration Authority" },
    { "4C9614", "Juniper Networks" },
    { "4C962D", "Fresh AB" },
    { "4C968A", "Wacom Co.,Ltd." },
    { "4C97A1", "Silicon Laboratories" },
    { "4C97CC", "Apple, Inc." },
    { "4C98EF", "Zeo" },
    { "4C9992", "vivo Mobile Communication Co., Ltd." },
    { "4C99E8", "ZHEJIANG DAHUA TECHNOLOGYCO.,LTD" },
    { "4C9B63", "LG Innotek" },
    { "4C9D22", "ACES Co.,Ltd" },
    { "4C9E6C", "BROADEX TECHNOLOGIES CO.LTD" },
    { "4C9E80", "KYOKKO ELECTRIC Co., Ltd." },
    { "4C9EE4", "Hanyang Navicom Co.,Ltd." },
    { "4C9EFF", "Zyxel Communications Corporation" },
    { "4C9FF1", "Apple, Inc." },
    { "4CA003", "VITEC" },
    { "4CA0D4", "Telink Semiconductor (Shanghai) Co., Ltd." },
    { "4CA161", "Rain Bird Corporation" },
    { "4CA38F", "shenzhen trolink Technology Co.,Ltd" },
    { "4CA3A7", "TECNO MOBILE LIMITED" },
    { "4CA515", "Baikal Electronics JSC" },
    { "4CA56D", "Samsung Electronics Co.,Ltd" },
    { "4CA64D", "Cisco Systems, Inc" },
    { "4CA74B", "Alcatel Lucent" },
    { "4CA919", "Tuya Smart Inc." },
    { "4CA928", "Insensi" },
    { "4CA954", "Intel Corporate" },
    { "4CAA16", "AzureWave Technologies (Shanghai) Inc." },
    { "4CAB33", "KST technology" },
    { "4CAB4F", "Apple, Inc." },
    { "4CABF3", "Universal Electronics, Inc." },
    { "4CABF8", "ASKEY COMPUTER CORP" },
    { "4CABFC", "zte corporation" },
    { "4CAC0A", "zte corporation" },
    { "4CAD35", "Apple, Inc." },
    { "4CADA8", "PANOPTICS CORP." },
    { "4CADDF", "Công ty Cổ phần Thiết bị Công nghiệp GEIC" },
    { "4CAE13", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "4CAE1C", "SaiNXT Technologies LLP" },
    { "4CAE31", "ShengHai Electronics (Shenzhen) Ltd" },
    { "4CAEA3", "Hewlett Packard Enterprise " },
    { "4CAEEC", "Guangzhou limee technology co.,LTD" },
    { "4CB008", "Shenzhen Gwelltimes Technology Co.,Ltd" },
    { "4CB04A", "Intel Corporate" },
    { "4CB087", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "4CB0E8", "Beijing RongZhi xinghua technology co., LTD" },
    { "4CB16C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "4CB199", "Apple, Inc." },
    { "4CB1CD", "Ruckus Wireless" },
    { "4CB21C", "Maxphotonics Co.,Ltd" },
    { "4CB44A", "NANOWAVE Technologies Inc." },
    { "4CB4EA", "HRD (S) PTE., LTD." },
    { "4CB76D", "Novi Security" },
    { "4CB7E0", "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "4CB81C", "SAM Electronics GmbH" },
    { "4CB82C", "Cambridge Mobile Telematics, Inc." },
    { "4CB8B5", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "4CB910", "Apple, Inc." },
    { "4CB911", "Raisecom Technology CO., LTD" },
    { "4CB99B", "WEIFANG GOERTEK ELECTRONICS CO.,LTD" },
    { "4CB9C8", "CONET CO., LTD." },
    { "4CB9EA", "iRobot Corporation" },
    { "4CBA7D", "Gemtek Technology Co., Ltd." },
    { "4CBAA3", "Bison Electronics Inc." },
    { "4CBAD7", "LG Innotek" },
    { "4CBB47", "NVIDIA Corporation" },
    { "4CBB58", "Chicony Electronics Co., Ltd." },
    { "4CBB6F", "Infinix mobility limited" },
    { "4CBC42", "Shenzhen Hangsheng Electronics Co.,Ltd." },
    { "4CBC48", "Cisco Systems, Inc" },
    { "4CBC72", "Primex Wireless" },
    { "4CBC98", "IEEE Registration Authority" },
    { "4CBCA5", "Samsung Electronics Co.,Ltd" },
    { "4CBCB4", "ABB SpA - DIN Rail" },
    { "4CBCE9", "LG Innotek" },
    { "4CBD8F", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "4CC00A", "vivo Mobile Communication Co., Ltd." },
    { "4CC096", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "4CC206", "Somfy" },
    { "4CC382", "Espressif Inc." },
    { "4CC449", "Icotera A/S" },
    { "4CC452", "Shang Hai Tyd. Electon Technology Ltd." },
    { "4CC53E", "Zyxel Communications Corporation" },
    { "4CC5D9", "Dell Inc." },
    { "4CC602", "Radios, Inc." },
    { "4CC64C", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "4CC681", "Shenzhen Aisat Electronic Co., Ltd." },
    { "4CC7D6", "FLEXTRONICS MANUFACTURING(ZHUHAI)CO.,LTD." },
    { "4CC844", "Maipu Communication Technology Co.,Ltd." },
    { "4CC8A1", "Cisco Meraki" },
    { "4CC94F", "Nokia" },
    { "4CC95E", "Samsung Electronics Co.,Ltd" },
    { "4CCA53", "Skyera, Inc." },
    { "4CCA95", "Huawei Device Co., Ltd." },
    { "4CCBF5", "zte corporation" },
    { "4CCC34", "Motorola Solutions Inc." },
    { "4CCC6A", "Micro-Star INTL CO., LTD." },
    { "4CCDB6", "Apple, Inc." },
    { "4CCE2D", "Danlaw Inc" },
    { "4CCF7C", "HP Inc." },
    { "4CD08A", "HUMAX Co., Ltd." },
    { "4CD0CB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "4CD0DD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "4CD0F9", "Cisco Systems, Inc" },
    { "4CD1A1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "4CD2FB", "UNIONMAN TECHNOLOGY CO.,LTD" },
    { "4CD3AF", "HMD Global Oy" },
    { "4CD546", "Hewlett Packard Enterprise" },
    { "4CD577", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "4CD587", "Hewlett Packard Enterprise" },
    { "4CD629", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "4CD637", "Qsono Electronics Co., Ltd" },
    { "4CD717", "Dell Inc." },
    { "4CD7B6", "Helmer Scientific" },
    { "4CD7C8", "Guangzhou V-Solution Telecommunication Technology Co.,Ltd." },
    { "4CD98F", "Dell Inc." },
    { "4CD9C4", "Magneti Marelli Automotive Electronics (Guangzhou) Co. Ltd" },
    { "4CDA38", "Texas Instruments" },
    { "4CDC0D", "Coral Telecom Limited" },
    { "4CDD31", "Samsung Electronics Co.,Ltd" },
    { "4CDD7D", "LHP Telematics LLC" },
    { "4CDE48", "Huawei Device Co., Ltd." },
    { "4CDF3D", "TEAM ENGINEERS ADVANCE TECHNOLOGIES INDIA PVT LTD" },
    { "4CE0DB", "Xiaomi Communications Co Ltd" },
    { "4CE136", "Private" },
    { "4CE173", "IEEE Registration Authority" },
    { "4CE175", "Cisco Systems, Inc" },
    { "4CE176", "Cisco Systems, Inc" },
    { "4CE19E", "TECNO MOBILE LIMITED" },
    { "4CE1BB", "Zhuhai HiFocus Technology Co., Ltd." },
    { "4CE2F1", "Udino srl" },
    { "4CE5AE", "Tianjin Beebox Intelligent Technology Co.,Ltd." },
    { "4CE676", "BUFFALO.INC" },
    { "4CE6C0", "Apple, Inc." },
    { "4CE705", "Siemens Industrial Automation Products Ltd., Chengdu" },
    { "4CE933", "RailComm, LLC" },
    { "4CE9E4", "New H3C Technologies Co., Ltd" },
    { "4CEA41", "IEEE Registration Authority" },
    { "4CEAAE", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "4CEB42", "Intel Corporate" },
    { "4CEB76", "Murrelektronik GmbH" },
    { "4CEBBD", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "4CEBD6", "Espressif Inc." },
    { "4CEC0F", "Cisco Systems, Inc" },
    { "4CECEF", "Soraa, Inc." },
    { "4CEDDE", "ASKEY COMPUTER CORP" },
    { "4CEDFB", "ASUSTek COMPUTER INC." },
    { "4CEEB0", "SHC Netzwerktechnik GmbH" },
    { "4CEF56", "Shenzhen Sundray Technologies company Limited" },
    { "4CEFC0", "Amazon Technologies Inc." },
    { "4CF02E", "Vifa Denmark A/S" },
    { "4CF19E", "Groupe Atlantic" },
    { "4CF202", "Xiaomi Communications Co Ltd" },
    { "4CF2BF", "Cambridge Industries(Group) Co.,Ltd." },
    { "4CF45B", "Blue Clover Devices" },
    { "4CF475", "Huawei Device Co., Ltd." },
    { "4CF55B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "4CF5A0", "Scalable Network Technologies Inc" },
    { "4CF5DC", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "4CF737", "SamJi Electronics Co., Ltd" },
    { "4CF95D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "4CFA9A", "Shenzhen Quanxing Technology Co., Ltd" },
    { "4CFAC9", "BWS IoT" },
    { "4CFACA", "Cambridge Industries(Group) Co.,Ltd." },
    { "4CFB45", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "4CFBF4", "Optimal Audio Ltd" },
    { "4CFBFE", "Sercomm Japan Corporation" },
    { "4CFC22", "SHANGHAI HI-TECH CONTROL SYSTEM CO.,LTD." },
    { "4CFCAA", "Tesla,Inc." },
    { "4CFE2E", "DongGuan Siyoto Electronics Co., Ltd " },
    { "4CFF12", "Fuze Entertainment Co., ltd" },
    { "500084", "Siemens Canada" },
    { "50008C", "Hong Kong Telecommunications (HKT) Limited" },
    { "5000E0", "Cisco Systems, Inc" },
    { "5000E6", "Mellanox Technologies, Inc." },
    { "50016B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5001BB", "Samsung Electronics Co.,Ltd" },
    { "5001D9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "500238", "Nokia Shanghai Bell Co., Ltd." },
    { "500291", "Espressif Inc." },
    { "5003CF", "CANON INC." },
    { "500401", "TelHi Corporation" },
    { "5004B8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "50053D", "CyWee Group Ltd" },
    { "50056E", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "500604", "Cisco Systems, Inc" },
    { "5006AB", "Cisco Systems, Inc" },
    { "5006F5", "Roku, Inc" },
    { "5007C3", "Amazon Technologies Inc." },
    { "500959", "Vantiva USA LLC" },
    { "5009E5", "Drimsys,Inc" },
    { "500A52", "Huiwan Technologies Co. Ltd" },
    { "500A9C", "Extreme Networks Headquarters" },
    { "500B26", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "500B32", "Foxda Technology Industrial(ShenZhen)Co.,LTD" },
    { "500B88", "Moxa.Inc" },
    { "500B91", "IEEE Registration Authority" },
    { "500E6D", "TrafficCast International" },
    { "500F59", "STMicrolectronics International NV" },
    { "500F80", "Cisco Systems, Inc" },
    { "500FF5", "Tenda Technology Co.,Ltd.Dongguan branch" },
    { "5011EB", "SilverNet Ltd" },
    { "50131D", "Motorola Mobility LLC, a Lenovo Company" },
    { "501365", "Vola Networks Inc." },
    { "501395", "Sichuan AI-Link Technology Co., Ltd." },
    { "50139D", "Sierra Wireless, ULC" },
    { "501408", "AiNET" },
    { "501479", "iRobot Corporation " },
    { "5014B5", "Richfit Information Technology Co., Ltd" },
    { "5014C1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5016F4", "Motorola Mobility LLC, a Lenovo Company" },
    { "5017FF", "Cisco Systems, Inc" },
    { "50184C", "Platina Systems Inc." },
    { "501AA5", "GN Netcom A/S" },
    { "501AC5", "Microsoft" },
    { "501B32", "Taicang T&W Electronics" },
    { "501B6A", "SUNITEC TECHNOLOGY CO.,LIMITED" },
    { "501CB0", "Cisco Systems, Inc" },
    { "501CBF", "Cisco Systems, Inc" },
    { "501D93", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "501E2D", "StreamUnlimited Engineering GmbH" },
    { "501FC6", "Apple, Inc." },
    { "502065", "Quectel Wireless Solutions Co.,Ltd." },
    { "50206B", "Copeland - Transportation Solutions ApS" },
    { "5021EC", "Huawei Device Co., Ltd." },
    { "502267", "PixeLINK" },
    { "50236D", "Nintendo Co.,Ltd" },
    { "5023A2", "Apple, Inc." },
    { "50252B", "Nethra Imaging Incorporated" },
    { "502690", "FUJITSU LIMITED" },
    { "5026D2", "AVIRE Trading Limited" },
    { "5026EF", "Murata Manufacturing Co., Ltd." },
    { "5027A9", "eero inc." },
    { "5027C7", "TECHNART Co.,Ltd" },
    { "50284A", "Intel Corporate" },
    { "502873", "Huawei Device Co., Ltd." },
    { "50294D", "NANJING IOT SENSOR TECHNOLOGY CO,LTD" },
    { "50297B", "China Mobile Group Device Co.,Ltd." },
    { "5029F5", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "502A7E", "Smart electronic GmbH" },
    { "502A8B", "Telekom Research and Development Sdn Bhd" },
    { "502B73", "Tenda Technology Co.,Ltd.Dongguan branch" },
    { "502B98", "Es-tech International" },
    { "502CC6", "GREE ELECTRIC APPLIANCES, INC. OF ZHUHAI" },
    { "502D1D", "Nokia Corporation" },
    { "502DA2", "Intel Corporate" },
    { "502DBB", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "502DF4", "Phytec Messtechnik GmbH" },
    { "502DFB", "IGShare Co., Ltd." },
    { "502E5C", "HTC Corporation" },
    { "502E66", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "502E91", "AzureWave Technology Inc." },
    { "502ECE", "Asahi Electronics Co.,Ltd" },
    { "502F54", "CIG SHANGHAI CO LTD" },
    { "502F9B", "Intel Corporate" },
    { "502FA8", "Cisco Systems, Inc" },
    { "502FBB", "Motorola Mobility LLC, a Lenovo Company" },
    { "5030F4", "Exascend, Inc." },
    { "5031AD", "ABB Global Industries and Services Private Limited" },
    { "503237", "Apple, Inc." },
    { "50325F", "Silicon Laboratories" },
    { "503275", "Samsung Electronics Co.,Ltd" },
    { "50338B", "Texas Instruments" },
    { "5033F0", "YICHEN (SHENZHEN) TECHNOLOGY CO.LTD" },
    { "50382F", "ASE Group Chung-Li" },
    { "5038AB", "PROVE" },
    { "50392F", "INGRAM MICRO SERVICES" },
    { "503955", "Cisco SPVTG" },
    { "503A0F", "ALL Winner (Hong Kong) Limited" },
    { "503A7D", "AlphaTech PLC Int’l Co., Ltd." },
    { "503AA0", "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "503CC4", "Lenovo Mobile Communication Technology Ltd." },
    { "503CCA", "TECNO MOBILE LIMITED" },
    { "503CEA", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "503D7F", "Nokia Shanghai Bell Co., Ltd." },
    { "503DA1", "Samsung Electronics Co.,Ltd" },
    { "503DC6", "Xiaomi Communications Co Ltd" },
    { "503DD1", "TP-Link Systems Inc." },
    { "503DE5", "Cisco Systems, Inc" },
    { "503DEB", "Zhejiang Tmall Technology Co., Ltd." },
    { "503E7C", "LeiShen Intelligent System Co.Ltd" },
    { "503EAA", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "503F50", "Huawei Device Co., Ltd." },
    { "503F56", "Syncmold Enterprise Corp" },
    { "503F98", "CMITECH" },
    { "504061", "Nokia" },
    { "504074", "Alarm.com" },
    { "50411C", "AMPAK Technology,Inc." },
    { "504172", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5041B9", "I-O DATA DEVICE,INC." },
    { "504289", "zte corporation" },
    { "504348", "ThingsMatrix Inc." },
    { "5043B9", "OktoInform RUS" },
    { "504594", "Radisys" },
    { "5045F7", "Liuhe Intelligence Technology Ltd." },
    { "50464A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "50465D", "ASUSTek COMPUTER INC." },
    { "5046AE", "MERCURY CORPORATION" },
    { "50482C", "IEEE Registration Authority" },
    { "504877", "Honor Device Co., Ltd." },
    { "5048EB", "BEIJING HAIHEJINSHENG NETWORK TECHNOLOGY CO. LTD." },
    { "504921", "Cisco Systems, Inc" },
    { "5049B0", "Samsung Electronics Co.,Ltd" },
    { "504A5E", "Masimo Corporation" },
    { "504A6E", "NETGEAR" },
    { "504B5B", "CONTROLtronic GmbH" },
    { "504B9E", "Huawei Device Co., Ltd." },
    { "504C7E", "THE 41ST INSTITUTE OF CETC" },
    { "504EDC", "Ping Communication" },
    { "504F3B", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "504F94", "Loxone Electronics GmbH" },
    { "50502A", "Egardia" },
    { "505065", "TAKT Corporation" },
    { "5050A4", "Samsung Electronics Co.,Ltd" },
    { "5050CE", "Hangzhou Dianyixia Communication Technology Co. Ltd." },
    { "50514F", "Sapphire Brands Limited" },
    { "5051A9", "Texas Instruments" },
    { "50523B", "Nokia" },
    { "5052D2", "Hangzhou Telin Technologies Co., Limited" },
    { "50547B", "Nanjing Qinheng Microelectronics Co., Ltd." },
    { "505527", "LG Electronics (Mobile Communications)" },
    { "50558D", "China Mobile IOT Company Limited" },
    { "505663", "Texas Instruments" },
    { "5056A8", "Jollyboys Ltd" },
    { "5056BF", "Samsung Electronics Co.,Ltd" },
    { "50578A", "Apple, Inc." },
    { "50579C", "Seiko Epson Corporation" },
    { "5057A8", "Cisco Systems, Inc" },
    { "505800", "WyTec International, Inc." },
    { "50584F", "waytotec,Inc." },
    { "50586F", "Huawei Device Co., Ltd." },
    { "5058B0", "Hunan Greatwall Computer System Co., Ltd." },
    { "505967", "Intent Solutions Inc" },
    { "505A65", "AzureWave Technology Inc." },
    { "505AC6", "GUANGDONG SUPER TELECOM CO.,LTD." },
    { "505B1D", "Shenzhen C-Data Technology Co., Ltd." },
    { "505BC2", "Liteon Technology Corporation" },
    { "505C88", "Cisco Systems, Inc" },
    { "505D7A", "zte corporation" },
    { "505DAC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "505E24", "zte corporation" },
    { "505E5C", "SUNITEC TECHNOLOGY CO.,LIMITED" },
    { "505FB5", "ASKEY COMPUTER CORP" },
    { "506028", "Xirrus Inc." },
    { "506184", "Avaya Inc" },
    { "5061BF", "Cisco Systems, Inc" },
    { "5061D6", "Indu-Sol GmbH" },
    { "5061F6", "Universal Electronics, Inc." },
    { "506245", "Annapurna labs" },
    { "506255", "IEEE Registration Authority" },
    { "506313", "Hon Hai Precision Ind. Co.,Ltd." },
    { "506391", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "50642B", "XIAOMI Electronics,CO.,LTD" },
    { "506441", "Greenlee" },
    { "506583", "Texas Instruments" },
    { "5065F3", "Hewlett Packard" },
    { "5066E5", "Huawei Device Co., Ltd." },
    { "506787", "Planet Networks" },
    { "5067AE", "Cisco Systems, Inc" },
    { "5067E0", "Altice Labs" },
    { "5067F0", "Zyxel Communications Corporation" },
    { "50680A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5068AC", "Huawei Device Co., Ltd." },
    { "50695A", "AiFamous（shenzhen）Technology Co.,Ltd" },
    { "506A03", "NETGEAR" },
    { "506B4B", "Mellanox Technologies, Inc." },
    { "506B8D", "Nutanix" },
    { "506CBE", "InnosiliconTechnology Ltd" },
    { "506E92", "Innocent Technology Co., Ltd." },
    { "506F0C", "Sagemcom Broadband SAS" },
    { "506F77", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "506F98", "Sehaj Synergy Technologies Private Limited" },
    { "506F9A", "Wi-Fi Alliance" },
    { "507043", "SKY UK LIMITED" },
    { "507097", "China Mobile Group Device Co.,Ltd." },
    { "5070E5", "He Shan World Fair Electronics Technology Limited" },
    { "507224", "Texas Instruments" },
    { "50724D", "BEG Brueck Electronic GmbH" },
    { "5075F1", "Commscope" },
    { "507691", "Tekpea, Inc." },
    { "5076A6", "Ecil Informatica Ind. Com. Ltda" },
    { "5076AF", "Intel Corporate" },
    { "507705", "Samsung Electronics Co.,Ltd" },
    { "50787D", "Espressif Inc." },
    { "5078B0", "Huawei Device Co., Ltd." },
    { "5078B3", "zte corporation" },
    { "50795B", "Interexport Telecomunicaciones S.A." },
    { "507973", "Inagile Electronic Technology Co.,LTD." },
    { "507A55", "Apple, Inc." },
    { "507AC5", "Apple, Inc." },
    { "507B91", "Sigmastar Technology Ltd." },
    { "507B9D", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "507C6F", "Intel Corporate" },
    { "507D02", "BIODIT" },
    { "507E5D", "Arcadyan Technology Corporation" },
    { "50804A", "Quectel Wireless Solutions Co.,Ltd." },
    { "508140", "HP Inc." },
    { "5082D5", "Apple, Inc." },
    { "508492", "Intel Corporate" },
    { "508569", "Samsung Electronics Co.,Ltd" },
    { "50874D", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "508789", "Cisco Systems, Inc" },
    { "5087B8", "Nuvyyo Inc" },
    { "508811", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "5088C7", "Taicang T&W Electronics" },
    { "508965", "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "5089D1", "Huawei Device Co., Ltd." },
    { "508A06", "Tuya Smart Inc." },
    { "508A0F", "SHENZHEN FISE TECHNOLOGY HOLDING CO.,LTD." },
    { "508A42", "Uptmate Technology Co., LTD" },
    { "508A7F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "508ACB", "SHENZHEN MAXMADE TECHNOLOGY CO., LTD." },
    { "508BB9", "Tuya Smart Inc." },
    { "508C77", "DIRMEIER Schanktechnik GmbH &Co KG" },
    { "508CB1", "Texas Instruments" },
    { "508CC9", "zte corporation" },
    { "508CF5", "China Mobile Group Device Co.,Ltd." },
    { "508D62", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "508D6F", "CHAHOO Limited" },
    { "508E49", "Xiaomi Communications Co Ltd" },
    { "508F4C", "Xiaomi Communications Co Ltd" },
    { "5091E3", "TP-Link Systems Inc" },
    { "5092B9", "Samsung Electronics Co.,Ltd" },
    { "50934F", "Gradual Tecnologia Ltda." },
    { "509551", "Commscope" },
    { "509707", "Xiamen Paperang Technology Co.,Ltd." },
    { "509744", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "509772", "Westinghouse Digital" },
    { "509839", "Xiaomi Communications Co Ltd" },
    { "509871", "Inventum Technologies Private Limited" },
    { "509893", "Texas Instruments" },
    { "5098B8", "New H3C Technologies Co., Ltd" },
    { "5098F3", "Rheem Australia Pty Ltd" },
    { "509903", "Meta Platforms, Inc." },
    { "50995A", "Amazon Technologies Inc." },
    { "509A46", "Safetrust Inc" },
    { "509A4C", "Dell Inc." },
    { "509A88", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "509B94", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "509EA7", "Samsung Electronics Co.,Ltd" },
    { "509F27", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "509F3B", "OI ELECTRIC CO.,LTD" },
    { "50A009", "Xiaomi Communications Co Ltd" },
    { "50A015", "Shenzhen Yipingfang Network Technology Co., Ltd." },
    { "50A030", "IEEE Registration Authority" },
    { "50A054", "Actineon" },
    { "50A0A4", "Nokia" },
    { "50A0BF", "Alba Fiber Systems Inc." },
    { "50A132", "Shenzhen MiaoMing Intelligent Technology Co.,Ltd" },
    { "50A1F3", "Huawei Device Co., Ltd." },
    { "50A4C8", "Samsung Electronics Co.,Ltd" },
    { "50A4D0", "IEEE Registration Authority" },
    { "50A5DC", "Commscope" },
    { "50A67F", "Apple, Inc." },
    { "50A6D8", "Apple, Inc." },
    { "50A6E3", "David Clark Company" },
    { "50A715", "Aboundi, Inc." },
    { "50A72B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "50A733", "Ruckus Wireless" },
    { "50A83A", "S Mobile Devices Limited" },
    { "50A9DE", "Smartcom - Bulgaria AD" },
    { "50AB3E", "Qibixx AG" },
    { "50ABBF", "Hoseo Telecom" },
    { "50ACB9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "50AD71", "Tessolve Semiconductor Private Limited" },
    { "50AD92", "NX Technologies" },
    { "50ADD5", "Dynalec Corporation" },
    { "50AE86", "Linkintec Co., Ltd" },
    { "50AF4D", "zte corporation" },
    { "50AF73", "Shenzhen Bitland Information Technology Co., Ltd." },
    { "50B03B", "Sony Interactive Entertainment Inc." },
    { "50B127", "Apple, Inc." },
    { "50B140", "ELPROMA ELEKTRONIKA SP Z O O" },
    { "50B363", "Digitron da Amazonia S/A" },
    { "50B3B4", "Shenzhen Furuilian Electronic Co.,Ltd." },
    { "50B695", "Micropoint Biotechnologies,Inc." },
    { "50B7C3", "Samsung Electronics Co.,Ltd" },
    { "50B888", "wi2be Tecnologia S/A" },
    { "50B8A2", "ImTech Technologies LLC," },
    { "50BA02", "Qingdao Intelligent&Precise Electronics Co.,Ltd." },
    { "50BB9F", "Vantiva USA LLC" },
    { "50BBB5", "AzureWave Technology Inc." },
    { "50BC96", "Apple, Inc." },
    { "50BD5F", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "50C006", "Carmanah Signs" },
    { "50C0F0", "Artek Microelectronics Co.,Ltd." },
    { "50C1F0", "NXP Semiconductor (Tianjin) LTD." },
    { "50C271", "SECURETECH INC" },
    { "50C275", "GN Audio A/S" },
    { "50C2E8", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "50C2ED", "GN Audio A/S" },
    { "50C3A2", "nFore Technology Co.,Ltd." },
    { "50C4DD", "BUFFALO.INC" },
    { "50C58D", "Juniper Networks" },
    { "50C68E", "Biwin Semiconductor (HK) Company Limted" },
    { "50C6AD", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "50C709", "Juniper Networks" },
    { "50C7BF", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "50C8E5", "Samsung Electronics Co.,Ltd" },
    { "50C971", "GN Netcom A/S" },
    { "50C9A0", "SKIPPER AS" },
    { "50CCF8", "SAMSUNG ELECTRO MECHANICS CO., LTD." },
    { "50CD22", "Avaya Inc" },
    { "50CD32", "NanJing Chaoran Science & Technology Co.,Ltd." },
    { "50CE75", "Measy Electronics Co., Ltd." },
    { "50CEE3", "Gigafirm.co.LTD" },
    { "50CF14", "Quectel Wireless Solutions Co.,Ltd." },
    { "50CF56", "China Mobile Group Device Co.,Ltd." },
    { "50D065", "ESYLUX GmbH" },
    { "50D213", "CviLux Corporation" },
    { "50D274", "Steffes Corporation" },
    { "50D2F5", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "50D33B", "cloudnineinfo" },
    { "50D37F", "Yu Fly Mikly Way Science and Technology Co., Ltd." },
    { "50D448", "Private" },
    { "50D45C", "Amazon Technologies Inc." },
    { "50D4F7", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "50D59C", "Thai Habel Industrial Co., Ltd." },
    { "50D6D7", "Takahata Precision" },
    { "50D753", "CONELCOM GmbH" },
    { "50DA00", "Hangzhou H3C Technologies Co., Limited" },
    { "50DAD6", "Xiaomi Communications Co Ltd" },
    { "50DB3F", "SHENZHEN GONGJIN ELECTRONICS CO.,LT" },
    { "50DCD0", "Observint Technologies, Inc." },
    { "50DCE7", "Amazon Technologies Inc." },
    { "50DCFC", "ECOCOM" },
    { "50DD4F", "Automation Components, Inc" },
    { "50DE06", "Apple, Inc." },
    { "50DE19", "IEEE Registration Authority" },
    { "50DF95", "Lytx" },
    { "50E039", "Zyxel Communications Corporation" },
    { "50E085", "Intel Corporate" },
    { "50E099", "HangZhou Atuo Future Technology Co., Ltd" },
    { "50E0C7", "TurControlSystme AG" },
    { "50E0EF", "Nokia" },
    { "50E14A", "Private" },
    { "50E24E", "zte corporation" },
    { "50E452", "Chipsea Technologies (Shenzhen) Corp." },
    { "50E478", "Sichuan AI-Link Technology Co., Ltd." },
    { "50E4E0", "Hewlett Packard Enterprise" },
    { "50E538", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "50E549", "GIGA-BYTE TECHNOLOGY CO.,LTD." },
    { "50E636", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "50E666", "Shenzhen Techtion Electronics Co., Ltd." },
    { "50E7A0", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "50E7B7", "vivo Mobile Communication Co., Ltd." },
    { "50E971", "Jibo, Inc." },
    { "50E9DF", "Quectel Wireless Solutions Co.,Ltd." },
    { "50EAD6", "Apple, Inc." },
    { "50EB1A", "Brocade Communications Systems LLC" },
    { "50EB71", "Intel Corporate" },
    { "50EBF6", "ASUSTek COMPUTER INC." },
    { "50EC50", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "50ED3C", "Apple, Inc." },
    { "50ED78", "Changzhou Yongse Infotech Co.,Ltd" },
    { "50ED94", "EGATEL SL" },
    { "50EE32", "Hon Hai Precision Industry Co.,LTD" },
    { "50F003", "Open Stack, Inc." },
    { "50F0D3", "Samsung Electronics Co.,Ltd" },
    { "50F14A", "Texas Instruments" },
    { "50F222", "EM Microelectronic" },
    { "50F261", "Photon Sail Technologies" },
    { "50F265", "Apple, Inc." },
    { "50F351", "Apple, Inc." },
    { "50F43C", "Leeo Inc" },
    { "50F4EB", "Apple, Inc." },
    { "50F520", "Samsung Electronics Co.,Ltd" },
    { "50F5DA", "Amazon Technologies Inc." },
    { "50F61A", "Kunshan JADE Technologies co., Ltd." },
    { "50F722", "Cisco Systems, Inc" },
    { "50F7ED", "Huawei Device Co., Ltd." },
    { "50F8A5", "eWBM Co., Ltd." },
    { "50F908", "Wizardlab Co., Ltd." },
    { "50F958", "Huawei Device Co., Ltd." },
    { "50FA84", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "50FAAB", "L-tek d.o.o." },
    { "50FB19", "CHIPSEA TECHNOLOGIES (SHENZHEN) CORP." },
    { "50FC30", "Treehouse Labs" },
    { "50FC9F", "Samsung Electronics Co.,Ltd" },
    { "50FDD5", "SJI Industry Company" },
    { "50FE0C", "AzureWave Technology Inc." },
    { "50FEF2", "Sify Technologies Ltd" },
    { "50FF20", "Keenetic Limited" },
    { "50FF99", "IEEE Registration Authority" },
    { "540237", "Teltronic AG" },
    { "540295", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "540384", "Hongkong Nano IC Technologies Co., Ltd" },
    { "5403F5", "EBN Technology Corp." },
    { "540463", "CIG SHANGHAI CO LTD" },
    { "540496", "Gigawave LTD" },
    { "5404A6", "ASUSTek COMPUTER INC." },
    { "540536", "Vivago Oy" },
    { "54055F", "Alcatel Lucent" },
    { "540593", "WOORI ELEC Co.,Ltd" },
    { "5405DB", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "54068B", "Ningbo Deli Kebei Technology Co.LTD" },
    { "540764", "Huawei Device Co., Ltd." },
    { "54077D", "NETGEAR" },
    { "54083B", "IEEE Registration Authority" },
    { "540853", "Qingdao Haier Technology Co.,Ltd" },
    { "5408D3", "Tianyi Telecom Terminals Company Limited" },
    { "540910", "Apple, Inc." },
    { "540929", "Inventus Power Eletronica do Brasil LTDA" },
    { "540955", "zte corporation" },
    { "54098D", "deister electronic GmbH" },
    { "540A77", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "540A8A", "Jlztlink Industry(ShenZhen)Co.,Ltd. " },
    { "540BB6", "Variscite LTD" },
    { "540DF9", "Huawei Device Co., Ltd." },
    { "540E2D", "vivo Mobile Communication Co., Ltd." },
    { "540E58", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "540F2C", "Arista Networks" },
    { "540F57", "Silicon Laboratories" },
    { "54102E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "541031", "SMARTO" },
    { "54104F", "Samsung Electronics Co.,Ltd" },
    { "54107B", "Guangdong Jeton International Tech Ltd." },
    { "5410EC", "Microchip Technology Inc." },
    { "54112F", "Sulzer Pump Solutions Finland Oy" },
    { "541149", "vivo Mobile Communication Co., Ltd." },
    { "541159", "Nettrix Information Industry co.LTD" },
    { "54115F", "Atamo Pty Ltd" },
    { "5412CB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "541310", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "541379", "Hon Hai Precision Ind. Co.,Ltd." },
    { "5413CA", "ITEL MOBILE LIMITED" },
    { "541473", "Wingtech Group (HongKong）Limited" },
    { "5414A7", "Nanjing Qinheng Microelectronics Co., Ltd." },
    { "5414F3", "Intel Corporate" },
    { "5414FD", "Orbbec 3D Technology International" },
    { "541589", "MCS Logic Inc." },
    { "541651", "Ruijie Networks Co.,LTD" },
    { "5419C8", "vivo Mobile Communication Co., Ltd." },
    { "541B5D", "Techno-Innov" },
    { "541D61", "YEESTOR Microelectronics Co., Ltd" },
    { "541DFB", "Freestyle Energy Ltd" },
    { "541E56", "Juniper Networks" },
    { "541F8D", "zte corporation" },
    { "541FD5", "Advantage Electronics" },
    { "542018", "Tely Labs" },
    { "542097", "TTTech Auto AG" },
    { "54211D", "Huawei Device Co., Ltd." },
    { "542160", "Alula" },
    { "54219D", "Samsung Electronics Co.,Ltd" },
    { "5421A9", "ITEL MOBILE LIMITED" },
    { "542259", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5422E0", "Adtran Inc" },
    { "5422F8", "zte corporation" },
    { "5425EA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "54263D", "Sony Corporation" },
    { "542696", "Apple, Inc." },
    { "54271E", "AzureWave Technology Inc." },
    { "542722", "Lacroix" },
    { "542758", "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd." },
    { "54276C", "Jiangsu Houge Technology Corp." },
    { "54278D", "NXP (China) Management Ltd." },
    { "542906", "Apple, Inc." },
    { "542A1B", "Sonos, Inc." },
    { "542A9C", "LSY Defense, LLC." },
    { "542AA2", "Alpha Networks Inc." },
    { "542B1C", "Amazon Technologies Inc." },
    { "542B57", "Night Owl SP" },
    { "542B76", "zte corporation" },
    { "542B8D", "Apple, Inc." },
    { "542BDE", "New H3C Technologies Co., Ltd" },
    { "542CEA", "PROTECTRON" },
    { "542F04", "Shanghai Longcheer Technology Co., Ltd." },
    { "542F2B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "542F89", "Euclid Laboratories, Inc." },
    { "542F8A", "TELLESCOM INDUSTRIA E COMERCIO EM TELECOMUNICACAO " },
    { "543131", "Raster Vision Ltd" },
    { "5431D4", "TGW Mechanics GmbH" },
    { "543204", "Espressif Inc." },
    { "5432C7", "Apple, Inc." },
    { "5433C6", "Mist Systems, Inc." },
    { "5433CB", "Apple, Inc." },
    { "5434EF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "543530", "Hon Hai Precision Ind. Co.,Ltd." },
    { "5435DF", "Symeo GmbH" },
    { "5435E9", "Feitian Technologies Co., Ltd" },
    { "54369B", "1Verge Internet Technology (Beijing) Co., Ltd." },
    { "5437BB", "Taicang T&W Electronics" },
    { "543968", "Edgewater Networks Inc" },
    { "5439DF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "543AD6", "Samsung Electronics Co.,Ltd" },
    { "543ADF", "Qualfiber Technology Co.,Ltd" },
    { "543B30", "duagon AG" },
    { "543D37", "Ruckus Wireless" },
    { "543D60", "Arcadyan Corporation" },
    { "543D92", "WIRELESS-TEK TECHNOLOGY LIMITED" },
    { "543E64", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "5440AD", "Samsung Electronics Co.,Ltd" },
    { "544249", "Sony Corporation" },
    { "5443B2", "Espressif Inc." },
    { "544408", "Nokia Corporation" },
    { "54443B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5444A3", "Samsung Electronics Co.,Ltd" },
    { "544538", "Texas Instruments" },
    { "544617", "zte corporation" },
    { "54466B", "Shenzhen CZTIC Electronic Technology Co., Ltd " },
    { "544741", "XCHENG HOLDING" },
    { "5447CC", "Sagemcom Broadband SAS" },
    { "5447D3", "TSAT AS" },
    { "5447E8", "Syrotech Networks. Ltd." },
    { "544810", "Dell Inc." },
    { "54489C", "CDOUBLES ELECTRONICS CO. LTD." },
    { "5448E6", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "5449DF", "Peloton Interactive, Inc" },
    { "5449FC", "Ubee Interactive Co., Limited" },
    { "544A00", "Cisco Systems, Inc" },
    { "544A05", "wenglor sensoric gmbh" },
    { "544A16", "Texas Instruments" },
    { "544B8C", "Juniper Networks" },
    { "544C8A", "Microsoft Corporation" },
    { "544DD4", "China Mobile Group Device Co.,Ltd." },
    { "544E45", "Private" },
    { "544E90", "Apple, Inc." },
    { "544EF0", "Roku, Inc" },
    { "54511B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "545146", "AMG Systems Ltd." },
    { "5451DE", "Cisco Systems, Inc" },
    { "545284", "Huawei Device Co., Ltd." },
    { "5453ED", "Sony Corporation" },
    { "545414", "Digital RF Corea, Inc" },
    { "5454CF", "PROBEDIGITAL CO.,LTD" },
    { "5455D5", "Huawei Device Co., Ltd." },
    { "545AA6", "Espressif Inc." },
    { "545B86", "Panasonic Automotive Systems Co.,Ltd" },
    { "545DD9", "EDISTEC" },
    { "545EBD", "NL Technologies" },
    { "545FA7", "Jibaiyou Technology Co.,Ltd." },
    { "545FA9", "Teracom Limited" },
    { "546009", "Google, Inc." },
    { "54606D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "546172", "ZODIAC AEROSPACE SAS" },
    { "5461EA", "Zaplox AB" },
    { "5462E2", "Apple, Inc." },
    { "5464BC", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "5464D9", "Sagemcom Broadband SAS" },
    { "5464DE", "u-blox AG" },
    { "546503", "Quectel Wireless Solutions Co.,Ltd." },
    { "5465DE", "Commscope" },
    { "54666C", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "5466F9", "ConMet" },
    { "546706", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "546749", "Google, Inc." },
    { "546751", "Compal Broadband Networks, Inc." },
    { "5467E6", "SHENZHEN MTC CO LTD" },
    { "546925", "PS INODIC CO., LTD." },
    { "546990", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "546AD8", "Elster Water Metering" },
    { "546C0E", "Texas Instruments" },
    { "546CAC", "Intelbras" },
    { "546CEB", "Intel Corporate" },
    { "546D52", "TOPVIEW OPTRONICS CORP." },
    { "546F71", "uAvionix Corporation" },
    { "547068", "VTech Communications Limited" },
    { "5471DD", "Huawei Device Co., Ltd." },
    { "54724F", "Apple, Inc." },
    { "54725E", "UNION MAN TECHNOLOGY CO.,LTD" },
    { "54726E", "Daimler Truck AG" },
    { "547398", "Toyo Electronics Corporation" },
    { "5474E6", "Webtech Wireless" },
    { "547595", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "5475D0", "Cisco Systems, Inc" },
    { "5476B2", "Raisecom Technology CO., LTD" },
    { "547787", "Earda Technologies co Ltd" },
    { "54778A", "Hewlett Packard Enterprise " },
    { "54781A", "Cisco Systems, Inc" },
    { "547885", "SHENZHEN GIEC DIGITAL CO.,LTD" },
    { "5478C9", "AMPAK Technology,Inc." },
    { "5478F0", "zte corporation" },
    { "547975", "Nokia Corporation" },
    { "547A52", "CTE International srl" },
    { "547C69", "Cisco Systems, Inc" },
    { "547D40", "Powervision Tech Inc." },
    { "547DCD", "Texas Instruments" },
    { "547E1A", "Kaon Group Co., Ltd." },
    { "547F54", "INGENICO" },
    { "547FA8", "TELCO systems, s.r.o." },
    { "547FBC", "iodyne" },
    { "547FEE", "Cisco Systems, Inc" },
    { "548028", "Hewlett Packard Enterprise " },
    { "54808A", "PT. BIZLINK TECHNOLOGY INDONESIA" },
    { "54812D", "PAX Computer Technology(Shenzhen) Ltd." },
    { "5481AD", "Eagle Research Corporation" },
    { "54833A", "Zyxel Communications Corporation" },
    { "548450", "Tiinlab Corporation" },
    { "54847B", "Digital Devices GmbH" },
    { "5484DC", "zte corporation" },
    { "5485C1", "Siliconwaves Technologies Co.,Ltd" },
    { "5486BC", "Cisco Systems, Inc" },
    { "54880E", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "5488DE", "Cisco Systems, Inc" },
    { "5488FE", "Xiaoniu network technology (Shanghai) Co., Ltd." },
    { "548922", "Zelfy Inc" },
    { "548998", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "548ABA", "Cisco Systems, Inc" },
    { "548C81", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "548CA0", "Liteon Technology Corporation" },
    { "548D5A", "Intel Corporate" },
    { "5491AF", "IEEE Registration Authority" },
    { "5491E1", "Vitalacy Inc." },
    { "549209", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "54926A", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "5492BE", "Samsung Electronics Co.,Ltd" },
    { "549359", "SHENZHEN TWOWING TECHNOLOGIES CO.,LTD." },
    { "549478", "Silvershore Technology Partners" },
    { "549963", "Apple, Inc." },
    { "549A11", "IEEE Registration Authority" },
    { "549A16", "Uzushio Electric Co.,Ltd." },
    { "549A4C", "GUANGDONG HOMECARE TECHNOLOGY CO.,LTD. " },
    { "549A8F", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "549B12", "Samsung Electronics Co.,Ltd" },
    { "549B24", "Mellanox Technologies, Inc." },
    { "549B49", "NEC Platforms, Ltd." },
    { "549B72", "Ericsson AB" },
    { "549C27", "Plasma Cloud Limited" },
    { "549D85", "EnerAccess inc" },
    { "549F06", "Nokia Shanghai Bell Co., Ltd." },
    { "549F13", "Apple, Inc." },
    { "549F35", "Dell Inc." },
    { "549FAE", "iBASE Gaming Inc" },
    { "549FC6", "Cisco Systems, Inc" },
    { "54A04F", "t-mac Technologies Ltd" },
    { "54A050", "ASUSTek COMPUTER INC." },
    { "54A104", "OPTOWL Co.,Ltd" },
    { "54A245", "Digisol Systems Limited" },
    { "54A274", "Cisco Systems, Inc" },
    { "54A31B", "Shenzhen Linkworld Technology Co,.LTD" },
    { "54A3FA", "BQT Solutions (Australia)Pty Ltd" },
    { "54A493", "IEEE Registration Authority" },
    { "54A51B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "54A54B", "NSC Communications Siberia Ltd" },
    { "54A552", "Shenzhen WeSing Interactive Entertainment Technology Co., Ltd" },
    { "54A619", "Alcatel-Lucent Shanghai Bell Co., Ltd" },
    { "54A637", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "54A65C", "Vantiva USA LLC" },
    { "54A6DB", "Huawei Device Co., Ltd." },
    { "54A703", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "54A7A0", "HUNAN AIMAG INTELLIGENT TECHNOLOGY CO.,LTD" },
    { "54A9C8", "Home Control Singapore Pte Ltd" },
    { "54A9D4", "Minibar Systems" },
    { "54AB3A", "Quanta Computer Inc." },
    { "54ACFC", "LIZN ApS" },
    { "54AE27", "Apple, Inc." },
    { "54AED0", "DASAN Networks, Inc. " },
    { "54AED2", "CSL Dualcom Ltd" },
    { "54AF97", "TP-Link Systems Inc" },
    { "54B121", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "54B203", "PEGATRON CORPORATION" },
    { "54B29D", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "54B56C", "Xi'an NovaStar Tech Co., Ltd" },
    { "54B620", "SUHDOL E&C Co.Ltd." },
    { "54B753", "Hunan Fenghui Yinjia Science And Technology Co.,Ltd" },
    { "54B7BD", "Arcadyan Corporation" },
    { "54B7E5", "Rayson Technology Co., Ltd." },
    { "54B802", "Samsung Electronics Co.,Ltd" },
    { "54B80A", "D-Link International" },
    { "54B874", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "54B8DB", "Apple, Inc." },
    { "54BAD6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "54BD79", "Samsung Electronics Co.,Ltd" },
    { "54BE53", "zte corporation" },
    { "54BEF7", "PEGATRON CORPORATION" },
    { "54BF64", "Dell Inc." },
    { "54C078", "Infinix mobility limited" },
    { "54C250", "Iskratel d.o.o." },
    { "54C33E", "Ciena Corporation" },
    { "54C415", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "54C45B", "Arcadyan Corporation" },
    { "54C480", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "54C57A", "Sunnovo International Limited" },
    { "54C6A6", "Hubei Yangtze Mason Semiconductor Technology Co., Ltd." },
    { "54C6FF", "New H3C Technologies Co., Ltd" },
    { "54C80F", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "54C8CC", "Shenzhen SDG Telecom Equipment Co.,Ltd." },
    { "54C9DF", "FN-LINK TECHNOLOGY LIMITED" },
    { "54CD10", "Panasonic Mobile Communications Co.,Ltd." },
    { "54CDA7", "Fujian Shenzhou Electronic Co.,Ltd" },
    { "54CDEE", "ShenZhen Apexis Electronic Co.,Ltd" },
    { "54CE69", "Hikari Trading Co.,Ltd." },
    { "54CE82", "zte corporation" },
    { "54CF8D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "54D0B4", "Xiamen Four-Faith Communication Technology Co.,Ltd" },
    { "54D0ED", "AXIM Communications" },
    { "54D163", "MAX-TECH,INC" },
    { "54D17D", "Samsung Electronics Co.,Ltd" },
    { "54D1B0", "Universal Laser Systems, Inc" },
    { "54D272", "Nuki Home Solutions GmbH" },
    { "54D46F", "Cisco SPVTG" },
    { "54D60D", "Hangzhou Ezviz Software Co.,Ltd." },
    { "54D751", "Proximus" },
    { "54D7E3", "Hewlett Packard Enterprise" },
    { "54D9C6", "Huawei Device Co., Ltd." },
    { "54D9E4", "BRILLIANTTS CO., LTD" },
    { "54DBA2", "Fibrain" },
    { "54DC1D", "Yulong Computer Telecommunication Scientific (Shenzhen) Co.,Ltd" },
    { "54DCE9", "Silicon Laboratories" },
    { "54DD21", "Huawei Device Co., Ltd." },
    { "54DD4F", "Samsung Electronics Co.,Ltd" },
    { "54DED0", "Sevio Srl" },
    { "54DED3", "zte corporation" },
    { "54DF00", "Ulterius Technologies, LLC" },
    { "54DF1B", "Vestel Elektronik San ve Tic. A.S." },
    { "54DF24", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "54DF63", "Intrakey technologies GmbH" },
    { "54E005", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "54E019", "Ring LLC" },
    { "54E032", "Juniper Networks" },
    { "54E061", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "54E140", "INGENICO" },
    { "54E15B", "Huawei Device Co., Ltd." },
    { "54E1AD", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "54E1B6", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "54E2C8", "Dongguan Aoyuan Electronics Technology Co., Ltd" },
    { "54E2E0", "Commscope" },
    { "54E3B0", "JVL Industri Elektronik" },
    { "54E3F6", "Alcatel-Lucent" },
    { "54E43A", "Apple, Inc." },
    { "54E4A9", "BHR Tech GmbH" },
    { "54E4BD", "FN-LINK TECHNOLOGY LIMITED" },
    { "54E4ED", "Intel Corporate" },
    { "54E61B", "Apple, Inc." },
    { "54E63F", "ShenZhen LingKeWeiEr Technology Co., Ltd." },
    { "54E6FC", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "54E7D5", "Sun Cupid Technology (HK) LTD" },
    { "54EAA8", "Apple, Inc." },
    { "54EAE1", "Honor Device Co., Ltd." },
    { "54EBE9", "Apple, Inc." },
    { "54EC2F", "Ruckus Wireless" },
    { "54ECB0", "Sercomm France Sarl" },
    { "54EDA3", "Navdy, Inc." },
    { "54EE75", "Wistron InfoComm(Kunshan)Co.,Ltd." },
    { "54EF33", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "54EF43", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "54EF44", "Lumi United Technology Co., Ltd" },
    { "54EF5B", "Science Corporation" },
    { "54EF92", "Shenzhen Elink Technology Co., LTD" },
    { "54EFFE", "Fullpower Technologies, Inc." },
    { "54F0B1", "Hewlett Packard Enterprise" },
    { "54F15F", "Sichuan AI-Link Technology Co., Ltd." },
    { "54F201", "Samsung Electronics Co.,Ltd" },
    { "54F294", "Huawei Device Co., Ltd." },
    { "54F29F", "HUNAN FN-LINK TECHNOLOGY LIMITED" },
    { "54F5B6", "ORIENTAL PACIFIC INTERNATIONAL LIMITED" },
    { "54F607", "Huawei Device Co., Ltd." },
    { "54F666", "Berthold Technologies GmbH and Co.KG" },
    { "54F6C5", "FUJIAN STAR-NET COMMUNICATION CO.,LTD" },
    { "54F6E2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "54F82A", "u-blox AG" },
    { "54F876", "ABB AG" },
    { "54F8F0", "Tesla Inc" },
    { "54FA3E", "Samsung Electronics Co.,Ltd" },
    { "54FA89", "Medtronic CRM" },
    { "54FA96", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "54FB58", "WISEWARE, Lda" },
    { "54FB5A", "Optomind Inc." },
    { "54FB66", "ASRock Incorporation" },
    { "54FCF0", "Samsung Electronics Co.,Ltd" },
    { "54FDBF", "Scheidt & Bachmann GmbH" },
    { "54FEEB", "Texas Instruments" },
    { "54FF82", "Davit Solution co." },
    { "54FFCF", "Mopria Alliance" },
    { "580032", "Genexis B.V." },
    { "5800BB", "Juniper Networks" },
    { "5800E3", "Liteon Technology Corporation" },
    { "580205", "AzureWave Technology Inc." },
    { "5803FB", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "580454", "ICOMM HK LIMITED" },
    { "5804CB", "Tianjin Huisun Technology Co.,Ltd." },
    { "580528", "LABRIS NETWORKS" },
    { "580556", "Elettronica GF S.r.L." },
    { "5805D9", "Seiko Epson Corporation" },
    { "5807F8", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "5808FA", "Fiber Optic & telecommunication INC." },
    { "580943", "Private" },
    { "580987", "Amazon Technologies Inc." },
    { "5809E5", "Kivic Inc." },
    { "580A20", "Cisco Systems, Inc" },
    { "580AD4", "Apple, Inc." },
    { "580D0D", "GREE ELECTRIC APPLIANCES, INC. OF ZHUHAI" },
    { "581031", "Hon Hai Precision IND.CO.,LTD" },
    { "58108C", "Intelbras" },
    { "5810B7", "Infinix mobility limited" },
    { "581122", "ASUSTek COMPUTER INC." },
    { "581243", "AcSiP Technology Corp." },
    { "5813D3", "Gemtek Technology Co., Ltd." },
    { "581626", "Avaya Inc" },
    { "5816D7", "ALPSALPINE CO,.LTD" },
    { "58170C", "Sony Corporation" },
    { "581862", "Sony Corporation" },
    { "5819F8", "Commscope" },
    { "581CBD", "Affinegy" },
    { "581CF8", "Intel Corporate" },
    { "581D91", "Advanced Mobile Telecom co.,ltd." },
    { "581DC9", "MSE CO.,LTD." },
    { "581DD8", "Sagemcom Broadband SAS" },
    { "581F19", "EMH Metering GmbH & Co. KG" },
    { "581F28", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "581F67", "Open-m technology limited" },
    { "581FAA", "Apple, Inc." },
    { "581FEF", "Tuttnaer LTD" },
    { "582059", "Xiaomi Communications Co Ltd" },
    { "582071", "Samsung Electronics Co.,Ltd" },
    { "58208A", "IEEE Registration Authority" },
    { "5820B1", "Hewlett Packard" },
    { "582136", "KMB systems, s.r.o." },
    { "5821E9", "TWPI" },
    { "58238C", "Vantiva USA LLC" },
    { "58239B", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "5823BC", "IEEE Registration Authority" },
    { "582429", "Google, Inc." },
    { "582575", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "58257A", "MOBIWIRE MOBILES(NINGBO) CO.,LTD" },
    { "58263A", "Silicon Laboratories" },
    { "58278C", "BUFFALO.INC" },
    { "582AF7", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "582B0A", "Texas Instruments" },
    { "582BDB", "Pax AB" },
    { "582D34", "Qingping Electronics (Suzhou) Co., Ltd" },
    { "582EFE", "Lighting Science Group" },
    { "582F40", "Nintendo Co.,Ltd" },
    { "582F42", "Universal Electric Corporation" },
    { "582FF7", "Sagemcom Broadband SAS" },
    { "58305B", "Shanghai Junqian Sensing Technology Co.,LTD" },
    { "58306E", "Nokia" },
    { "583112", "DRUST" },
    { "583277", "Reliance Communications LLC" },
    { "58343B", "Glovast Technology Ltd." },
    { "58350F", "EM Microelectronic" },
    { "583526", "DEEPLET TECHNOLOGY CORP" },
    { "58355D", "Huawei Device Co., Ltd." },
    { "58356B", "TECNO MOBILE LIMITED" },
    { "5835D9", "Cisco Systems, Inc" },
    { "583653", "Apple, Inc." },
    { "583879", "RICOH COMPANY, LTD." },
    { "583BC2", "Silicon Laboratories" },
    { "583BD9", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "583CC6", "Omneality Ltd." },
    { "583F54", "LG Electronics (Mobile Communications)" },
    { "58404E", "Apple, Inc." },
    { "584120", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "584146", "Guangzhou Shiyuan Electronic Technology Company Limited" },
    { "5842E4", "Baxter International Inc" },
    { "5843AB", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "584498", "Xiaomi Communications Co Ltd" },
    { "58454C", "Ericsson AB" },
    { "58468F", "Koncar Electronics and Informatics" },
    { "5846E1", "Baxter International Inc" },
    { "584704", "Shenzhen Webridge Technology Co.,Ltd" },
    { "5847CA", "IEEE Registration Authority" },
    { "584822", "Sony Corporation" },
    { "584849", "IEEE Registration Authority" },
    { "5848C0", "COFLEC" },
    { "584925", "E3 Enterprise" },
    { "58493B", "Palo Alto Networks" },
    { "5849BA", "Chitai Electronic Corp." },
    { "584BBC", "zte corporation" },
    { "584C19", "Chongqing Guohong Technology Development Company Limited" },
    { "584CEE", "Digital One Technologies, Limited" },
    { "584D42", "Dragos, Inc." },
    { "585076", "Linear Equipamentos Eletronicos SA" },
    { "5850AB", "TLS Corporation" },
    { "5850E6", "Best Buy Corporation" },
    { "5850ED", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "58528A", "Mitsubishi Electric Corporation" },
    { "5853C0", "Beijing Guang Runtong Technology Development Company co.,Ltd" },
    { "585595", "Apple, Inc." },
    { "5855CA", "Apple, Inc." },
    { "58569F", "Cisco Systems, Inc" },
    { "5856C2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5856E8", "Commscope" },
    { "58570D", "Danfoss Solar Inverters" },
    { "5858CD", "Extreme Networks Headquarters" },
    { "585924", "Nanjing Simon Info Tech Co.,Ltd." },
    { "5859C2", "Extreme Networks Headquarters" },
    { "585B69", "TVT CO., LTD" },
    { "585FF6", "zte corporation" },
    { "58605F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5860D8", "Commscope" },
    { "586163", "Quantum Networks (SG) Pte. Ltd." },
    { "586356", "FN-LINK TECHNOLOGY LIMITED" },
    { "58639A", "TPL SYSTEMES" },
    { "5864C4", "Apple, Inc." },
    { "5865E6", "infomark" },
    { "58666D", "Apple, Inc." },
    { "5866BA", "Hangzhou H3C Technologies Co., Limited" },
    { "58671A", "Barnes&Noble" },
    { "58677F", "Clare Controls Inc." },
    { "58685D", "Tempo Australia Pty Ltd" },
    { "586861", "VIASAT, INCORPORATED" },
    { "58687A", "Sagemcom Broadband SAS" },
    { "58696C", "Ruijie Networks Co.,LTD" },
    { "5869F9", "Fusion Transactive Ltd." },
    { "586AB1", "Hangzhou H3C Technologies Co., Limited" },
    { "586B14", "Apple, Inc." },
    { "586C25", "Intel Corporate" },
    { "586D67", "Intel Corporate" },
    { "586D8F", "Cisco-Linksys, LLC" },
    { "586ED6", "Private" },
    { "58707F", "Ericsson AB" },
    { "5870C6", "Shanghai Xiaoyi Technology Co., Ltd." },
    { "5872C9", "zte corporation" },
    { "5873D1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5873D8", "Apple, Inc." },
    { "587521", "CJSC RTSoft" },
    { "587675", "Beijing ECHO Technologies Co.,Ltd" },
    { "58769C", "Palo Alto Networks" },
    { "5876AC", "SERNET (SUZHOU) TECHNOLOGIES CORPORATION" },
    { "5876B3", "Ubee Interactive Co., Limited" },
    { "5876C5", "DIGI I'S LTD" },
    { "587961", "Microsoft Corporation" },
    { "5879E0", "Samsung Electronics Co.,Ltd" },
    { "587A4D", "Stonesoft Corporation" },
    { "587A62", "Texas Instruments" },
    { "587A6A", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "587AB1", "Shanghai Lixun Information Technology Co., Ltd." },
    { "587BE9", "AirPro Technology India Pvt. Ltd" },
    { "587DB6", "Northern Data AG" },
    { "587E61", "Qingdao Hisense Communications Co.,Ltd." },
    { "587F57", "Apple, Inc." },
    { "587F66", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "587FB7", "SONAR INDUSTRIAL CO., LTD." },
    { "587FC8", "S2M" },
    { "58821D", "H. Schomäcker GmbH" },
    { "5882A8", "Microsoft" },
    { "588336", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5884E4", "IP500 Alliance e.V." },
    { "58856E", "QSC AG" },
    { "5885A2", "Realme Chongqing MobileTelecommunications Corp Ltd" },
    { "5885E9", "Realme Chongqing MobileTelecommunications Corp Ltd" },
    { "588670", "Juniper Networks" },
    { "588694", "EFM Networks" },
    { "58874C", "LITE-ON CLEAN ENERGY TECHNOLOGY CORP." },
    { "588785", "Adtran Inc" },
    { "58879F", "Huawei Device Co., Ltd." },
    { "5887E2", "Shenzhen Coship Electronics Co., Ltd." },
    { "588A5A", "Dell Inc." },
    { "588B1C", "Cisco Systems, Inc" },
    { "588BF3", "Zyxel Communications Corporation" },
    { "588C81", "Espressif Inc." },
    { "588D09", "Cisco Systems, Inc" },
    { "588D39", "MITSUBISHI ELECTRIC AUTOMATION (CHINA) LTD." },
    { "588D64", "Xi'an Clevbee Technology Co.,Ltd" },
    { "588E81", "Silicon Laboratories" },
    { "588FCF", "Hangzhou Ezviz Software Co.,Ltd." },
    { "589043", "Sagemcom Broadband SAS" },
    { "589153", "China Mobile IOT Company Limited" },
    { "5891CF", "Intel Corporate" },
    { "58920D", "Kinetic Avionics Limited" },
    { "589351", "Huawei Device Co., Ltd." },
    { "589396", "Ruckus Wireless" },
    { "5893D8", "Texas Instruments" },
    { "5893E8", "Apple, Inc." },
    { "58946B", "Intel Corporate" },
    { "5894A2", "KETEK GmbH" },
    { "5894AE", "Huawei Device Co., Ltd." },
    { "5894B2", "BrainCo" },
    { "5894CF", "Vertex Standard LMR, Inc." },
    { "58957E", "Huawei Device Co., Ltd." },
    { "5895D8", "IEEE Registration Authority" },
    { "58960A", "LG Electronics" },
    { "58961D", "Intel Corporate" },
    { "589630", "Vantiva USA LLC" },
    { "589671", "Wistron Neweb Corporation" },
    { "58971E", "Cisco Systems, Inc" },
    { "5897BD", "Cisco Systems, Inc" },
    { "589835", "Technicolor Delivery Technologies Belgium NV" },
    { "58986F", "Revolution Display" },
    { "589A3E", "Amazon Technologies Inc." },
    { "589B0B", "Shineway Technologies, Inc." },
    { "589B4A", "DWnet Technologies(Suzhou) Corporation" },
    { "589BF7", "Hefei Radio Communication Technology Co., Ltd " },
    { "589CFC", "FreeBSD Foundation" },
    { "589EC6", "Gigaset Communications GmbH" },
    { "58A023", "Intel Corporate" },
    { "58A0CB", "TrackNet, Inc" },
    { "58A15F", "Texas Instruments" },
    { "58A2B5", "LG Electronics (Mobile Communications)" },
    { "58A2E1", "Mellanox Technologies, Inc." },
    { "58A48E", "PixArt Imaging Inc." },
    { "58A639", "Samsung Electronics Co.,Ltd" },
    { "58A748", "Lytx" },
    { "58A76F", "iD corporation" },
    { "58A839", "Intel Corporate" },
    { "58A87B", "Fitbit, Inc." },
    { "58A8E8", "Amazon Technologies Inc." },
    { "58AC78", "Cisco Systems, Inc" },
    { "58AD12", "Apple, Inc." },
    { "58AE2B", "Huawei Device Co., Ltd." },
    { "58AEA8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "58AEF1", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "58B035", "Apple, Inc." },
    { "58B03E", "Nintendo Co.,Ltd" },
    { "58B0D4", "ZuniData Systems Inc." },
    { "58B0FE", "Team EPS GmbH" },
    { "58B10F", "Samsung Electronics Co.,Ltd" },
    { "58B18F", "Huawei Device Co., Ltd." },
    { "58B38F", "New H3C Technologies Co., Ltd" },
    { "58B3FC", "SHENZHEN RF-LINK TECHNOLOGY CO.,LTD." },
    { "58B42D", "YSTen Technology Co.,Ltd" },
    { "58B4BB", "Ruijie Networks Co.,LTD" },
    { "58B568", "SECURITAS DIRECT ESPAÑA, SAU" },
    { "58B623", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "58B633", "Ruckus Wireless" },
    { "58B858", "SZ DJI TECHNOLOGY CO.,LTD" },
    { "58B961", "SOLEM Electronique" },
    { "58B965", "Apple, Inc." },
    { "58B9E1", "Crystalfontz America, Inc." },
    { "58BAD3", "NANJING CASELA TECHNOLOGIES CORPORATION LIMITED" },
    { "58BAD4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "58BC27", "Cisco Systems, Inc" },
    { "58BC8F", "Cognitive Systems Corp." },
    { "58BDA3", "Nintendo Co., Ltd." },
    { "58BDF9", "Sigrand" },
    { "58BE72", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "58BF25", "Espressif Inc." },
    { "58BFEA", "Cisco Systems, Inc" },
    { "58C17A", "Cambium Networks Limited" },
    { "58C232", "NEC Corporation" },
    { "58C356", "EM Microelectronic" },
    { "58C38B", "Samsung Electronics Co.,Ltd" },
    { "58C41E", "IEEE Registration Authority" },
    { "58C57E", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "58C583", "ITEL MOBILE LIMITED" },
    { "58C587", "AltoBeam Inc." },
    { "58C5CB", "Samsung Electronics Co.,Ltd" },
    { "58C6F0", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "58C7AC", "New H3C Technologies Co., Ltd" },
    { "58C876", "China Mobile (Hangzhou) Information Technology Co., Ltd." },
    { "58C935", "Chiun Mai Communication System, Inc" },
    { "58CB52", "Google, Inc." },
    { "58CDC9", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "58CE2A", "Intel Corporate" },
    { "58CF4B", "Lufkin Industries" },
    { "58CF79", "Espressif Inc." },
    { "58D061", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "58D071", "BW Broadcast" },
    { "58D08F", "IEEE 1904.1 Working Group" },
    { "58D15A", "Texas Instruments" },
    { "58D237", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "58D312", "zte corporation" },
    { "58D349", "Apple, Inc." },
    { "58D391", "Quectel Wireless Solutions Co.,Ltd." },
    { "58D50A", "Murata Manufacturing Co., Ltd." },
    { "58D533", "Huaqin Technology Co.,Ltd" },
    { "58D56E", "D-Link International" },
    { "58D61F", "Ubiquiti Inc" },
    { "58D67A", "TCPlink" },
    { "58D697", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "58D6D3", "Dairy Cheq Inc" },
    { "58D759", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "58D8A7", "Bird Home Automation GmbH" },
    { "58D9C3", "Motorola Mobility LLC, a Lenovo Company" },
    { "58D9D5", "Tenda Technology Co.,Ltd.Dongguan branch" },
    { "58DB09", "Quectel Wireless Solutions Co.,Ltd." },
    { "58DB15", "TECNO MOBILE LIMITED" },
    { "58DB8D", "Fast Co., Ltd." },
    { "58DC6D", "Exceptional Innovation, Inc." },
    { "58DF59", "Cisco Systems, Inc" },
    { "58E02C", "Micro Technic A/S" },
    { "58E16C", "Ying Hua Information Technology (Shanghai)Co., LTD" },
    { "58E28F", "Apple, Inc." },
    { "58E326", "Compass Technologies Inc." },
    { "58E359", "Interroll Software & Electronics GmbH" },
    { "58E403", "Wistron Neweb Corporation" },
    { "58E434", "Juniper Networks" },
    { "58E476", "CENTRON COMMUNICATIONS TECHNOLOGIES FUJIAN CO.,LTD" },
    { "58E488", "Amazon Technologies Inc." },
    { "58E4EB", "FN-LINK TECHNOLOGY Ltd." },
    { "58E636", "EVRsafe Technologies" },
    { "58E6BA", "Apple, Inc." },
    { "58E6C5", "Espressif Inc." },
    { "58E747", "Deltanet AG" },
    { "58E808", "AUTONICS CORPORATION" },
    { "58E873", "HANGZHOU DANGBEI NETWORK TECH.Co.,Ltd" },
    { "58E876", "IEEE Registration Authority" },
    { "58EA1F", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "58EAFC", "ELL-IoT Inc" },
    { "58EB14", "Proteus Digital Health" },
    { "58ECE1", "Newport Corporation" },
    { "58ECED", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "58ED99", "zte corporation" },
    { "58EECE", "Icon Time Systems" },
    { "58EF68", "Belkin International Inc." },
    { "58F102", "BLU Products Inc." },
    { "58F2FC", "Huawei Device Co., Ltd." },
    { "58F387", "Airios" },
    { "58F39C", "Cisco Systems, Inc" },
    { "58F496", "Source Chain" },
    { "58F67B", "Xia Men UnionCore Technology LTD." },
    { "58F6BF", "Kyoto University" },
    { "58F85C", "LLC Proizvodstvennaya Kompania TransService" },
    { "58F8D7", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "58F987", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "58F98E", "SECUDOS GmbH" },
    { "58FB3E", "Huawei Device Co., Ltd." },
    { "58FB84", "Intel Corporate" },
    { "58FB96", "Ruckus Wireless" },
    { "58FC20", "Altice Labs" },
    { "58FC73", "Arria Live Media, Inc." },
    { "58FCC6", "TOZO INC" },
    { "58FCC8", "Honeywell" },
    { "58FCDB", "IEEE Registration Authority" },
    { "58FCE3", "Funshion Online Technologies Co.,Ltd" },
    { "58FD20", "Systemhouse Solutions AB" },
    { "58FD5D", "Hangzhou Xinyun technology Co., Ltd." },
    { "58FDB1", "LG Electronics" },
    { "58FDBE", "Shenzhen Taikaida Technology Co., Ltd" },
    { "58FE7E", "zte corporation" },
    { "58FFA1", "zte corporation" },
    { "5C0038", "Viasat Group S.p.A." },
    { "5C013B", "Espressif Inc." },
    { "5C0214", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "5C026A", "Applied Vision Corporation" },
    { "5C0272", "Silicon Laboratories" },
    { "5C0339", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5C045A", "Company NA Stage & Light" },
    { "5C0610", "Cisco Meraki" },
    { "5C0758", "Ufispace Co., LTD." },
    { "5C076F", "Thought Creator" },
    { "5C07A4", "Ciena Corporation" },
    { "5C07A6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5C0947", "Apple, Inc." },
    { "5C0979", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5C0A5B", "SAMSUNG ELECTRO MECHANICS CO., LTD." },
    { "5C0B3B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5C0BCA", "Tunstall Nordic AB" },
    { "5C0C0E", "Guizhou Huaxintong Semiconductor Technology Co Ltd" },
    { "5C0CBB", "CELIZION Inc." },
    { "5C0CE6", "Nintendo Co.,Ltd" },
    { "5C0E8B", "Extreme Networks Headquarters" },
    { "5C0FFB", "Amino Communications Ltd" },
    { "5C101E", "zte corporation" },
    { "5C10C5", "Samsung Electronics Co.,Ltd" },
    { "5C1193", "Seal One AG" },
    { "5C13CC", "Apple, Inc." },
    { "5C1437", "Thyssenkrupp Aufzugswerke GmbH" },
    { "5C14EB", "Trident IoT" },
    { "5C1515", "ADVAN" },
    { "5C15C5", "Shenzhen SSC Technology Co. Ltd" },
    { "5C15E1", "AIDC TECHNOLOGY (S) PTE LTD" },
    { "5C1648", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "5C167D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5C16C7", "Arista Networks" },
    { "5C1720", "Huawei Device Co., Ltd." },
    { "5C1737", "I-View Now, LLC." },
    { "5C1783", "Edgecore Americas Networking Corporation" },
    { "5C17CF", "OnePlus Technology (Shenzhen) Co., Ltd" },
    { "5C17D3", "LGE " },
    { "5C18B5", "Talon Communications" },
    { "5C18DD", "CIG SHANGHAI CO LTD" },
    { "5C1923", "Hangzhou Lanly Technology Co., Ltd." },
    { "5C1A6F", "Cambridge Industries(Group) Co.,Ltd." },
    { "5C1BF4", "Apple, Inc." },
    { "5C1CB9", "vivo Mobile Communication Co., Ltd." },
    { "5C1DD9", "Apple, Inc." },
    { "5C20D0", "Asoni Communication Co., Ltd." },
    { "5C2167", "Rockwell Automation" },
    { "5C22C4", "DAE EUN ELETRONICS CO., LTD" },
    { "5C22DA", "Vantiva USA LLC" },
    { "5C2316", "Squirrels Research Labs LLC" },
    { "5C241F", "Qingdao Haier Technology Co.,Ltd" },
    { "5C2443", "O-Sung Telecom Co., Ltd." },
    { "5C2479", "Baltech AG" },
    { "5C24E2", "Suzhou Denbom Electronic S&T Co., Ltd" },
    { "5C254C", "Avire Global Pte Ltd" },
    { "5C2573", "Mellanox Technologies, Inc." },
    { "5C260A", "Dell Inc." },
    { "5C2623", "WaveLynx Technologies Corporation" },
    { "5C2763", "Itibia Technologies" },
    { "5C27D4", "Shenzhen Qihu Intelligent Technology Company Limited" },
    { "5C2886", "Inventec(Chongqing) Corporation" },
    { "5C2AEF", "r2p Asia-Pacific Pty Ltd" },
    { "5C2BF5", "Vivint Wireless Inc. " },
    { "5C2D08", "Subeca" },
    { "5C2E59", "Samsung Electronics Co.,Ltd" },
    { "5C2ED2", "ABC(XiSheng) Electronics Co.,Ltd" },
    { "5C2FAF", "HomeWizard B.V." },
    { "5C313E", "Texas Instruments" },
    { "5C3192", "Cisco Systems, Inc" },
    { "5C32C5", "Teracom Ltd." },
    { "5C3327", "Spazio Italia srl" },
    { "5C335C", "Swissphone Telecom AG" },
    { "5C337B", "Google, Inc." },
    { "5C338E", "Alpha Networks Inc." },
    { "5C3400", "Hisense Electric Co.,Ltd" },
    { "5C345B", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "5C353B", "Compal Broadband Networks, Inc." },
    { "5C3548", "Aditya Infotech Ltd." },
    { "5C35DA", "There Corporation Oy" },
    { "5C35FC", "Actiontec Electronics Inc." },
    { "5C36B8", "TCL King Electrical Appliances (Huizhou) Co., Ltd" },
    { "5C38E0", "Shanghai Super Electronics Technology Co.,LTD" },
    { "5C3A3D", "zte corporation" },
    { "5C3A45", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "5C3B35", "Gehirn Inc." },
    { "5C3C27", "Samsung Electronics Co.,Ltd" },
    { "5C3E06", "Cisco Systems, Inc" },
    { "5C3E1B", "Apple, Inc." },
    { "5C4058", "Jefferson Audio Video Systems, Inc." },
    { "5C4071", "Xiaomi Communications Co Ltd" },
    { "5C40E3", "NOVAON" },
    { "5C415A", "Amazon.com, LLC" },
    { "5C41E7", "Wiatec International Ltd." },
    { "5C43D2", "HAZEMEYER" },
    { "5C443E", "Skullcandy" },
    { "5C4527", "Juniper Networks" },
    { "5C46B0", "SIMCom Wireless Solutions Limited" },
    { "5C475E", "Ring LLC" },
    { "5C4842", "Hangzhou Anysoft Information Technology Co. , Ltd" },
    { "5C4979", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "5C497D", "Samsung Electronics Co.,Ltd" },
    { "5C49FA", "Shenzhen Guowei Shidai Communication Equipement Co., Ltd" },
    { "5C4A1F", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "5C4A26", "Enguity Technology Corp" },
    { "5C4CA9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5C4DBF", "zte corporation" },
    { "5C4EEE", "AltoBeam Inc." },
    { "5C5015", "Cisco Systems, Inc" },
    { "5C50D9", "Apple, Inc." },
    { "5C514F", "Intel Corporate" },
    { "5C5181", "Samsung Electronics Co.,Ltd" },
    { "5C5188", "Motorola Mobility LLC, a Lenovo Company" },
    { "5C521E", "Nintendo Co.,Ltd" },
    { "5C5230", "Apple, Inc." },
    { "5C5284", "Apple, Inc." },
    { "5C5310", "Nanjing Qinheng Microelectronics Co., Ltd." },
    { "5C53B4", "EM Microelectronic" },
    { "5C53C3", "Ubee Interactive Co., Limited" },
    { "5C546D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5C5578", "iryx corp" },
    { "5C56A4", "Wanan Hongsheng Electronic Co.Ltd" },
    { "5C56ED", "3pleplay Electronics Private Limited" },
    { "5C571A", "Commscope" },
    { "5C579E", "Chiun Mai Communication System, Inc" },
    { "5C57C8", "Nokia Corporation" },
    { "5C5819", "Jingsheng Technology Co., Ltd." },
    { "5C58E6", "Palo Alto Networks" },
    { "5C5948", "Apple, Inc." },
    { "5C5A4C", "IEEE Registration Authority" },
    { "5C5AC7", "Cisco Systems, Inc" },
    { "5C5AEA", "FORD" },
    { "5C5B35", "Mist Systems, Inc." },
    { "5C5BC2", "YIK Corporation" },
    { "5C5DEC", "JiangSu Newcom Optical&Electrical Communication CO Ltd" },
    { "5C5E0A", "Samsung Electronics Co.,Ltd" },
    { "5C5EAB", "Juniper Networks" },
    { "5C5EBB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5C5F67", "Intel Corporate" },
    { "5C60BA", "HP Inc." },
    { "5C6152", "NXP Semiconductor (Tianjin) LTD." },
    { "5C6199", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "5C625A", "CANON INC." },
    { "5C628B", "TP-Link Systems Inc" },
    { "5C63BF", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "5C63C9", "Intellithings Ltd." },
    { "5C640F", "Sage Technologies Inc." },
    { "5C647A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5C648E", "Zyxel Communications Corporation" },
    { "5C64F1", "Cisco Systems, Inc" },
    { "5C64F3", "sywinkey HongKong Co,. Limited?" },
    { "5C666C", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "5C6776", "IDS Imaging Development Systems GmbH" },
    { "5C6783", "Intel Corporate" },
    { "5C68D0", "Aurora Innovation Inc." },
    { "5C6984", "NUVICO" },
    { "5C6A7D", "KENTKART EGE ELEKTRONIK SAN. VE TIC. LTD. STI. " },
    { "5C6A80", "Zyxel Communications Corporation" },
    { "5C6AEC", "IEEE Registration Authority" },
    { "5C6B32", "Texas Instruments" },
    { "5C6B4F", "Hello Inc." },
    { "5C6BD7", "Foshan VIOMI Electric Appliance Technology Co. Ltd." },
    { "5C6D20", "Hon Hai Precision Ind. Co.,Ltd." },
    { "5C6F4F", "S.A. SISTEL" },
    { "5C6F69", "Broadcom Limited" },
    { "5C7017", "Apple, Inc." },
    { "5C7075", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5C70A3", "LG Electronics (Mobile Communications)" },
    { "5C710D", "Cisco Systems, Inc" },
    { "5C7545", "Wayties, Inc." },
    { "5C75AF", "Fitbit, Inc." },
    { "5C75C6", "China Mobile Group Device Co.,Ltd." },
    { "5C7695", "Vantiva USA LLC" },
    { "5C76D5", "Nokia" },
    { "5C7757", "Haivision Network Video" },
    { "5C7776", "TCT mobile ltd" },
    { "5C78F8", "Huawei Device Co., Ltd." },
    { "5C7B5C", "Shenzhen SDMC Technology CO.,Ltd." },
    { "5C7B6C", "Tradit Co., Ltd " },
    { "5C7D5E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5C7D7D", "Vantiva USA LLC" },
    { "5C7DAE", "zte corporation" },
    { "5C7DF3", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "5C80B6", "Intel Corporate" },
    { "5C81A7", "Network Devices Pty Ltd" },
    { "5C836C", "Ruckus Wireless" },
    { "5C8382", "Nokia" },
    { "5C838F", "Cisco Systems, Inc" },
    { "5C83CD", "New platforms" },
    { "5C843C", "Sony Interactive Entertainment Inc." },
    { "5C8486", "Brightsource Industries Israel LTD" },
    { "5C857E", "IEEE Registration Authority" },
    { "5C85F8", "SHENZHEN KAIFA TECHNOLOGY CO.,LTD." },
    { "5C8613", "Beijing Zhoenet Technology Co., Ltd" },
    { "5C864A", "Secret Labs LLC" },
    { "5C865C", "Samsung Electronics Co.,Ltd" },
    { "5C86C1", "DONGGUAN SOLUM ELECTRONICS CO.,LTD" },
    { "5C8730", "Apple, Inc." },
    { "5C8778", "Cybertelbridge co.,ltd" },
    { "5C879C", "Intel Corporate" },
    { "5C87D8", "IEEE Registration Authority" },
    { "5C8816", "Rockwell Automation" },
    { "5C899A", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "5C89D4", "Beijing Banner Electric Co.,Ltd" },
    { "5C89E6", "Richard Wolf GmbH" },
    { "5C8A38", "Hewlett Packard" },
    { "5C8AAE", "CHINA DRAGON TECHNOLOGY LIMITED" },
    { "5C8B6B", "Amazon Technologies Inc." },
    { "5C8C30", "Taicang T&W Electronics" },
    { "5C8D2D", "Shanghai Wellpay Information Technology Co., Ltd" },
    { "5C8D4E", "Apple, Inc." },
    { "5C8DE5", "Delta Electronics, Inc." },
    { "5C8E10", "TimeWatch Infocom Pvt. Ltd. " },
    { "5C8E8B", "Shenzhen Linghai Electronics Co.,Ltd" },
    { "5C8F40", "TECNO MOBILE LIMITED" },
    { "5C8FE0", "Commscope" },
    { "5C9012", "Owl Cyber Defense Solutions, LLC" },
    { "5C9157", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5C9175", "Apple, Inc." },
    { "5C91FD", "Jaewoncnc" },
    { "5C925E", "Zioncom Electronics (Shenzhen) Ltd." },
    { "5C93A2", "Liteon Technology Corporation" },
    { "5C9462", "Shenzhen Jiuzhou Electric Co.,LTD" },
    { "5C95AE", "Apple, Inc." },
    { "5C9656", "AzureWave Technology Inc." },
    { "5C9666", "Sony Interactive Entertainment Inc." },
    { "5C966A", "RTNET" },
    { "5C969D", "Apple, Inc." },
    { "5C97F3", "Apple, Inc." },
    { "5C9960", "Samsung Electronics Co.,Ltd" },
    { "5C9977", "Apple, Inc." },
    { "5C9AA1", "Huawei Device Co., Ltd." },
    { "5C9AD8", "FUJITSU LIMITED" },
    { "5C9BA6", "Apple, Inc." },
    { "5CA06C", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "5CA176", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "5CA178", "TableTop Media (dba Ziosk)" },
    { "5CA1E0", "EmbedWay Technologies" },
    { "5CA39D", "SAMSUNG ELECTRO MECHANICS CO., LTD." },
    { "5CA3EB", "SKODA DIGITAL s.r.o." },
    { "5CA436", "Shenzhen G-world Technology Incorporated Company" },
    { "5CA47D", "Hewlett Packard Enterprise" },
    { "5CA48A", "Cisco Systems, Inc" },
    { "5CA4A4", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "5CA4F4", "zte corporation" },
    { "5CA5BC", "eero inc." },
    { "5CA62D", "Cisco Systems, Inc" },
    { "5CA64F", "TP-Link Systems Inc." },
    { "5CA6E6", "TP-Link Systems Inc" },
    { "5CA721", "New H3C Technologies Co., Ltd" },
    { "5CA86A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5CA933", "Luma Home" },
    { "5CAAFD", "Sonos, Inc." },
    { "5CAC3D", "Samsung Electronics Co.,Ltd" },
    { "5CAC4C", "Hon Hai Precision Ind. Co.,Ltd." },
    { "5CAD76", "Shenzhen TCL New Technology Co., Ltd" },
    { "5CADBA", "Apple, Inc." },
    { "5CADCF", "Apple, Inc." },
    { "5CAF06", "LG Electronics (Mobile Communications)" },
    { "5CB00A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5CB066", "Commscope" },
    { "5CB12E", "Cisco Systems, Inc" },
    { "5CB13E", "Sagemcom Broadband SAS" },
    { "5CB15F", "Oceanblue Cloud Technology Limited" },
    { "5CB260", "EM Microelectronic" },
    { "5CB26D", "Intel Corporate" },
    { "5CB29E", "ASCO Power Technologies" },
    { "5CB395", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5CB3F6", "Human, Incorporated" },
    { "5CB43E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5CB47E", "Intel Corporate" },
    { "5CB4E2", "Inspur Software Group Ltd." },
    { "5CB524", "Sony Corporation" },
    { "5CB559", "CNEX Labs" },
    { "5CB6CC", "NovaComm Technologies Inc." },
    { "5CB8CB", "Allis Communications" },
    { "5CB901", "Hewlett Packard" },
    { "5CBA2C", "Hewlett Packard Enterprise " },
    { "5CBA37", "Microsoft Corporation" },
    { "5CBAEF", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "5CBBEE", "zte corporation" },
    { "5CBD9A", "Huawei Device Co., Ltd." },
    { "5CBD9E", "HONGKONG MIRACLE EAGLE TECHNOLOGY(GROUP) LIMITED" },
    { "5CBE05", "ISPEC" },
    { "5CBE69", "Oraimo Technology Limited" },
    { "5CC0A0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5CC1D7", "Samsung Electronics Co.,Ltd" },
    { "5CC1F2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5CC213", "Fr. Sauter AG" },
    { "5CC307", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5CC336", "ittim" },
    { "5CC563", "HUNAN FN-LINK TECHNOLOGY LIMITED" },
    { "5CC5D4", "Intel Corporate" },
    { "5CC6D0", "Skyworth Digital Technology(Shenzhen) Co.,Ltd" },
    { "5CC6E9", "Edifier International" },
    { "5CC787", "Huawei Device Co., Ltd." },
    { "5CC7C1", "Silicon Laboratories" },
    { "5CC7D7", "AZROAD TECHNOLOGY COMPANY LIMITED" },
    { "5CC8E3", "Shintec Hozumi co.ltd." },
    { "5CC999", "New H3C Technologies Co., Ltd" },
    { "5CC9C0", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "5CC9D3", "PALLADIUM ENERGY ELETRONICA DA AMAZONIA LTDA" },
    { "5CCA1A", "Microsoft Mobile Oy" },
    { "5CCA32", "Theben AG" },
    { "5CCAD3", "CHIPSEA TECHNOLOGIES (SHENZHEN) CORP." },
    { "5CCB99", "Samsung Electronics Co.,Ltd" },
    { "5CCBCA", "FUJIAN STAR-NET COMMUNICATION CO.,LTD" },
    { "5CCCA0", "Gridwiz Inc." },
    { "5CCCFF", "Techroutes Network Pvt Ltd" },
    { "5CCD5B", "Intel Corporate" },
    { "5CCD7C", "MEIZU Technology Co.,Ltd." },
    { "5CCDA8", "ITEL MOBILE LIMITED" },
    { "5CCEAD", "CDYNE Corporation" },
    { "5CCF7F", "Espressif Inc." },
    { "5CD06E", "Xiaomi Communications Co Ltd" },
    { "5CD135", "Xtreme Power Systems" },
    { "5CD20B", "Yytek Co., Ltd." },
    { "5CD2E4", "Intel Corporate" },
    { "5CD41B", "UCZOON Technology Co., LTD" },
    { "5CD4AB", "Zektor" },
    { "5CD5B5", "Shenzhen WiSiYiLink Technology Co.,Ltd" },
    { "5CD61F", "Qardio, Inc" },
    { "5CD89E", "Huawei Device Co., Ltd." },
    { "5CD998", "D-Link Corporation" },
    { "5CDAD4", "Murata Manufacturing Co., Ltd." },
    { "5CDB36", "Calix Inc." },
    { "5CDC49", "Samsung Electronics Co.,Ltd" },
    { "5CDC96", "Arcadyan Technology Corporation" },
    { "5CDD70", "Hangzhou H3C Technologies Co., Limited" },
    { "5CDE34", "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "5CDF89", "Ruckus Wireless" },
    { "5CDFB8", "Shenzhen Unionmemory Information System Limited" },
    { "5CE0C5", "Intel Corporate" },
    { "5CE0CA", "FeiTian United (Beijing) System Technology Co., Ltd." },
    { "5CE0F6", "NIC.br- Nucleo de Informacao e Coordenacao do Ponto BR" },
    { "5CE176", "Cisco Systems, Inc" },
    { "5CE1A4", "Pleneo" },
    { "5CE223", "Delphin Technology AG" },
    { "5CE286", "Nortel Networks" },
    { "5CE28C", "Zyxel Communications Corporation" },
    { "5CE2F4", "AcSiP Technology Corp." },
    { "5CE30E", "Commscope" },
    { "5CE3B6", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "5CE42A", "Intel Corporate" },
    { "5CE50C", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "5CE688", "VECOS Europe B.V." },
    { "5CE747", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5CE753", "Private" },
    { "5CE7A0", "Nokia" },
    { "5CE7BF", "New Singularity International Technical Development Co.,Ltd" },
    { "5CE883", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5CE8B7", "Oraimo Technology Limited" },
    { "5CE8D3", "Signalinks Communication Technology Co., Ltd" },
    { "5CE8EB", "Samsung Electronics Co.,Ltd" },
    { "5CE91E", "Apple, Inc." },
    { "5CE931", "TP-Link Systems Inc" },
    { "5CEA1D", "Hon Hai Precision Ind. Co.,Ltd." },
    { "5CEB4E", "R. STAHL HMI Systems GmbH" },
    { "5CEB52", "zte corporation" },
    { "5CEB68", "Cheerstar Technology Co., Ltd" },
    { "5CED8C", "Hewlett Packard Enterprise " },
    { "5CEDF4", "Samsung Electronics Co.,Ltd" },
    { "5CEE79", "Global Digitech Co LTD" },
    { "5CF207", "Speco Technologies" },
    { "5CF286", "IEEE Registration Authority" },
    { "5CF370", "CC&C Technologies, Inc" },
    { "5CF3FC", "IBM Corp" },
    { "5CF4AB", "Zyxel Communications Corporation" },
    { "5CF50D", "Institute of microelectronic applications" },
    { "5CF51A", "Zhejiang Dahua Technology Co., Ltd." },
    { "5CF5DA", "Apple, Inc." },
    { "5CF6DC", "Samsung Electronics Co.,Ltd" },
    { "5CF796", "New H3C Technologies Co., Ltd" },
    { "5CF7C3", "SYNTECH (HK) TECHNOLOGY LIMITED" },
    { "5CF7E6", "Apple, Inc." },
    { "5CF821", "Texas Instruments" },
    { "5CF838", "IEEE Registration Authority" },
    { "5CF8A1", "Murata Manufacturing Co., Ltd." },
    { "5CF938", "Apple, Inc." },
    { "5CF96A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "5CF9DD", "Dell Inc." },
    { "5CF9F0", "Atomos Engineering P/L" },
    { "5CF9FD", "Taicang T&W Electronics" },
    { "5CFA25", "Sagemcom Broadband SAS" },
    { "5CFA5A", "Sinepower Lda" },
    { "5CFAFB", "Acubit" },
    { "5CFB3A", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "5CFB7C", "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd" },
    { "5CFC66", "Cisco Systems, Inc" },
    { "5CFC6E", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "5CFCE1", "Resideo" },
    { "5CFE9E", "Wiwynn Corporation Tainan Branch" },
    { "5CFF35", "Wistron Corporation" },
    { "5CFFFF", "Shenzhen Kezhonglong Optoelectronic Technology Co., Ltd" },
    { "600194", "Espressif Inc." },
    { "6001B1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "600292", "PEGATRON CORPORATION" },
    { "6002B4", "Wistron Neweb Corporation" },
    { "600308", "Apple, Inc." },
    { "60030C", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "600347", "Billion Electric Co. Ltd." },
    { "6003A6", "Inteno Broadband Technology AB" },
    { "600417", "POSBANK CO.,LTD" },
    { "60045C", "NXP Semiconductor (Tianjin) LTD." },
    { "60058A", "Hitachi Metals, Ltd." },
    { "6006E3", "Apple, Inc." },
    { "60077C", "Jala Group" },
    { "6007C4", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "600810", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "600837", "ivvi Scientific(Nanchang)Co.Ltd" },
    { "6009C3", "u-blox AG" },
    { "600A8C", "Shenzhen Sundray Technologies company Limited" },
    { "600B03", "Hangzhou H3C Technologies Co., Limited" },
    { "600F6B", "Apple, Inc." },
    { "600F77", "SilverPlus, Inc" },
    { "60109E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6010A2", "Crompton Instruments" },
    { "601199", "Siama Systems Inc" },
    { "60123C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "601283", "TSB REAL TIME LOCATION SYSTEMS S.L." },
    { "60128B", "CANON INC." },
    { "601466", "zte corporation" },
    { "6014B3", "CyberTAN Technology Inc." },
    { "601521", "Redarc Electronics" },
    { "60152B", "Palo Alto Networks" },
    { "601592", "IEEE Registration Authority" },
    { "6015C7", "IdaTech" },
    { "601803", "Daikin Air-conditioning (Shanghai) Co., Ltd." },
    { "60182E", "ShenZhen Protruly Electronic Ltd co." },
    { "60183A", "Huawei Device Co., Ltd." },
    { "601888", "zte corporation" },
    { "601895", "Dell Inc." },
    { "60190C", "RRAMAC" },
    { "601929", "VOLTRONIC POWER TECHNOLOGY(SHENZHEN) CORP." },
    { "601970", "HUIZHOU QIAOXING ELECTRONICS TECHNOLOGY CO., LTD." },
    { "601971", "Commscope" },
    { "601A4F", "Beijing China Electronics Intelligent Acoustics Technology Co.,Ltd" },
    { "601AC7", "Nintendo Co.,Ltd" },
    { "601B52", "Vodafone Italia S.p.A." },
    { "601D0F", "Midnite Solar" },
    { "601D16", "Med-Eng Holdings ULC" },
    { "601D56", "Extreme Networks Headquarters" },
    { "601D91", "Motorola Mobility LLC, a Lenovo Company" },
    { "601D9D", "Sichuan AI-Link Technology Co., Ltd." },
    { "601E02", "EltexAlatau" },
    { "601E98", "Axevast Technology" },
    { "602101", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "602103", "I4VINE, INC" },
    { "6021C0", "Murata Manufacturing Co., Ltd." },
    { "602232", "Ubiquiti Inc" },
    { "6023A4", "Sichuan AI-Link Technology Co., Ltd." },
    { "6024C1", "Jiangsu Zhongxun Electronic Technology Co., Ltd" },
    { "602602", "Texas Instruments" },
    { "6026AA", "Cisco Systems, Inc" },
    { "6026EF", "Hewlett Packard Enterprise" },
    { "60271C", "VIDEOR E. Hartig GmbH" },
    { "60292B", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "6029D5", "DAVOLINK Inc." },
    { "602A1B", "JANCUS" },
    { "602A54", "CardioTek B.V." },
    { "602AD0", "Cisco SPVTG" },
    { "602B58", "EM Microelectronic" },
    { "602D74", "Extreme Networks Headquarters" },
    { "602E20", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6030B3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6030D4", "Apple, Inc." },
    { "60313B", "Sunnovo International Limited" },
    { "603192", "OVT India pvt Ltd " },
    { "603197", "Zyxel Communications Corporation" },
    { "60323B", "Quectel Wireless Solutions Co.,Ltd." },
    { "6032B1", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "6032F0", "Mplus technology" },
    { "60334B", "Apple, Inc." },
    { "603457", "HP Tuners LLC" },
    { "603553", "Buwon Technology" },
    { "603573", "Earda Technologies co Ltd" },
    { "6035C0", "SFR" },
    { "603696", "The Sapling Company" },
    { "6036DD", "Intel Corporate" },
    { "60380E", "ALPSALPINE CO,.LTD" },
    { "6038E0", "Belkin International Inc." },
    { "60391F", "ABB Ltd" },
    { "603A7C", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "603AAF", "Samsung Electronics Co.,Ltd" },
    { "603C0E", "Guizhou Huaxin Information Technology Co.,Ltd" },
    { "603C68", "Garmin International" },
    { "603CEE", "LG Electronics (Mobile Communications)" },
    { "603D26", "Vantiva USA LLC" },
    { "603D29", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "603D61", "Private" },
    { "603E5F", "Apple, Inc." },
    { "603E7B", "Gafachi, Inc." },
    { "603ECA", "Cambridge Medical Robotics Ltd" },
    { "603FC5", "COX CO., LTD" },
    { "60427F", "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD" },
    { "60447A", "Water-i.d. GmbH" },
    { "6044F5", "Easy Digital Ltd." },
    { "60452E", "Intel Corporate" },
    { "60455E", "Liptel s.r.o." },
    { "6045BD", "Microsoft" },
    { "6045CB", "ASUSTek COMPUTER INC." },
    { "6045CD", "Sagemcom Broadband SAS" },
    { "6045E8", "Arcadyan Corporation" },
    { "604616", "XIAMEN VANN INTELLIGENT CO., LTD" },
    { "604762", "Beijing Sensoro Technology Co.,Ltd." },
    { "6047D4", "FORICS Electronic Technology Co., Ltd." },
    { "604826", "Newbridge Technologies Int. Ltd." },
    { "60489C", "YIPPEE ELECTRONICS CO.,LIMITED" },
    { "604966", "Shenzhen Dingsheng Technology Co., Ltd." },
    { "6049C1", "Avaya Inc" },
    { "604A1C", "SUYIN Corporation" },
    { "604A77", "Finder SpA" },
    { "604BAA", "Magic Leap, Inc." },
    { "604DE1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "604F5B", "Huawei Device Co., Ltd." },
    { "6050C1", "Kinetek Sports" },
    { "60512C", "TCT mobile ltd" },
    { "6052D0", "FACTS Engineering " },
    { "605317", "Sandstone Technologies" },
    { "605375", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "605464", "Eyedro Green Solutions Inc." },
    { "605556", "Jiangxi Risound Electronics Co.,LTD" },
    { "6055F9", "Espressif Inc." },
    { "605661", "IXECLOUD Tech" },
    { "60567D", "AM Telecom co., Ltd." },
    { "605699", "MAGNETI MARELLI S.E. S.p.A." },
    { "6056B1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6056EE", "AltoBeam Inc." },
    { "605718", "Intel Corporate" },
    { "605747", "CIG SHANGHAI CO LTD" },
    { "60577D", "eero inc." },
    { "6057C8", "Apple, Inc." },
    { "605801", "Shandong ZTop Microelectronics Co., Ltd." },
    { "605B30", "Dell Inc." },
    { "605BB4", "AzureWave Technology Inc." },
    { "605E4F", "Huawei Device Co., Ltd." },
    { "605F8D", "eero inc." },
    { "605FAA", "Huawei Device Co., Ltd." },
    { "60601F", "SZ DJI TECHNOLOGY CO.,LTD" },
    { "606134", "Arcade Communications Ltd." },
    { "6061DF", "Z-meta Research LLC" },
    { "60634C", "D-Link International" },
    { "6063F9", "Ciholas, Inc." },
    { "6063FD", "Transcend Communication Beijing Co.,Ltd." },
    { "606405", "Texas Instruments" },
    { "606453", "AOD Co.,Ltd." },
    { "6064A1", "RADiflow Ltd." },
    { "606525", "Apple, Inc." },
    { "606682", "SHENZHEN ATEKO PHOTOELECTRICITY CO.,LTD" },
    { "606720", "Intel Corporate" },
    { "606832", "Guangdong Seneasy Intelligent Technology Co., Ltd." },
    { "60684E", "Samsung Electronics Co.,Ltd" },
    { "606944", "Apple, Inc." },
    { "60699B", "isepos GmbH" },
    { "606B5B", "Arista Network, Inc." },
    { "606BB3", "zte corporation" },
    { "606BBD", "Samsung Electronics Co.,Ltd" },
    { "606BFF", "Nintendo Co.,Ltd" },
    { "606C63", "Hitron Technologies. Inc" },
    { "606C66", "Intel Corporate" },
    { "606D3C", "Luxshare Precision Industry Company Limited" },
    { "606D9D", "Otto Bock Healthcare Products GmbH" },
    { "606DC7", "Hon Hai Precision Ind. Co.,Ltd." },
    { "606E41", "Barrot Technology Co.,LTD" },
    { "606E53", "Beijing Wisdomstar Technology Co., Ltd" },
    { "606ED0", "SEAL AG" },
    { "606EE8", "Xiaomi Communications Co Ltd" },
    { "60706C", "Google, Inc." },
    { "607072", "SHENZHEN HONGDE SMART LINK TECHNOLOGY CO., LTD" },
    { "6070C0", "Apple, Inc." },
    { "60720B", "BLU Products Inc" },
    { "60735C", "Cisco Systems, Inc" },
    { "6073BC", "zte corporation" },
    { "6073C8", "Voyetra Turtle Beach, Inc." },
    { "60748D", "Atmaca Elektronik" },
    { "6074B1", "Home Control Singapore Pte Ltd" },
    { "6074F4", "Private" },
    { "60756C", "LG Electronics" },
    { "607623", "Shenzhen E-Superlink Technology Co., Ltd" },
    { "607688", "Velodyne" },
    { "607771", "Texas Instruments" },
    { "6077E2", "Samsung Electronics Co.,Ltd" },
    { "607D09", "Luxshare Precision Industry Co., Ltd" },
    { "607DDD", "Shenzhen Shichuangyi Electronics Co.,Ltd" },
    { "607EA4", "Shanghai Imilab Technology Co.Ltd" },
    { "607EC9", "Apple, Inc." },
    { "607ECD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "607EDD", "Microsoft Mobile Oy" },
    { "607FCB", "Samsung Electronics Co.,Ltd" },
    { "608110", "Apple, Inc." },
    { "60812B", "Astronics Custom Control Concepts" },
    { "6081F9", "Helium Systems, Inc" },
    { "608246", "Apple, Inc." },
    { "608334", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "608373", "Apple, Inc." },
    { "6083B2", "GkWare e.K." },
    { "6083E2", "Shanghai Notion Information Technology Co., Ltd" },
    { "6083E7", "TP-Link Systems Inc" },
    { "6083F8", "SICHUAN HUAKUN ZHENYU INTELLIGENT TECHNOLOGY CO.,LTD" },
    { "60843B", "Soladigm, Inc." },
    { "6084BD", "BUFFALO.INC" },
    { "608645", "Avery Weigh-Tronix, LLC" },
    { "60893C", "Thermo Fisher Scientific P.O.A." },
    { "6089B1", "Key Digital Systems" },
    { "6089B7", "KAEL MÜHENDİSLİK ELEKTRONİK TİCARET SANAYİ LİMİTED ŞİRKETİ" },
    { "608A10", "Microchip Technology Inc." },
    { "608B0E", "Apple, Inc." },
    { "608C2B", "Hanson Technology" },
    { "608C4A", "Apple, Inc." },
    { "608CDF", "Beamtrail-Sole Proprietorship" },
    { "608CE6", "Commscope" },
    { "608D17", "Sentrus Government Systems Division, Inc" },
    { "608D26", "Arcadyan Corporation" },
    { "608E08", "Samsung Electronics Co.,Ltd" },
    { "608F5C", "Samsung Electronics Co.,Ltd" },
    { "608FA4", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "609084", "DSSD Inc" },
    { "6091F3", "vivo Mobile Communication Co., Ltd." },
    { "609217", "Apple, Inc." },
    { "6092C8", "Roku, Inc" },
    { "6092F5", "Commscope" },
    { "609316", "Apple, Inc." },
    { "609532", "Zebra Technologies Inc." },
    { "6095BD", "Apple, Inc." },
    { "6095CE", "IEEE Registration Authority" },
    { "609620", "Private" },
    { "6096A4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6097DD", "MicroSys Electronics GmbH" },
    { "609813", "Shanghai Visking Digital Technology Co. LTD" },
    { "609849", "Nokia Solutions and Networks India Private Limited" },
    { "609866", "Texas Instruments" },
    { "6099D1", "Vuzix / Lenovo" },
    { "609AA4", "GVI SECURITY INC." },
    { "609AC1", "Apple, Inc." },
    { "609B2D", "JMACS Japan Co., Ltd." },
    { "609BB4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "609BC8", "Hipad Intelligent Technology Co., Ltd." },
    { "609C9F", "Brocade Communications Systems LLC" },
    { "609E64", "Vivonic GmbH" },
    { "609F9D", "CloudSwitch" },
    { "60A10A", "Samsung Electronics Co.,Ltd" },
    { "60A11E", "Wuhan Maxsine Electric Co.,Ltd." },
    { "60A2C6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "60A37D", "Apple, Inc." },
    { "60A3E3", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "60A423", "Silicon Laboratories" },
    { "60A434", "IEEE Registration Authority" },
    { "60A44C", "ASUSTek COMPUTER INC." },
    { "60A4B7", "TP-Link Systems Inc" },
    { "60A4D0", "Samsung Electronics Co.,Ltd" },
    { "60A5E2", "Intel Corporate" },
    { "60A6C5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "60A730", "Shenzhen Yipinfang Internet Technology Co.,Ltd" },
    { "60A751", "Huawei Device Co., Ltd." },
    { "60A8FE", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "60A9B0", "Merchandising Technologies, Inc" },
    { "60AAEF", "Huawei Device Co., Ltd." },
    { "60AB14", "LG Innotek" },
    { "60AB67", "Xiaomi Communications Co Ltd" },
    { "60ABD2", "Bose Corporation" },
    { "60ACC8", "KunTeng Inc." },
    { "60AF6D", "Samsung Electronics Co.,Ltd" },
    { "60B02B", "Qingdao Haier Technology Co.,Ltd" },
    { "60B0E8", "Huawei Device Co., Ltd." },
    { "60B185", "ATH system" },
    { "60B387", "Synergics Technologies GmbH" },
    { "60B3C4", "Elber Srl" },
    { "60B4F7", "Plume Design Inc" },
    { "60B58D", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "60B606", "Phorus" },
    { "60B617", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "60B647", "Silicon Laboratories" },
    { "60B6E1", "Texas Instruments" },
    { "60B76E", "Google, Inc." },
    { "60B933", "Deutron Electronics Corp." },
    { "60B982", "RO.VE.R. Laboratories S.p.A." },
    { "60B9C0", "Cisco Systems, Inc" },
    { "60BA18", "nextLAP GmbH" },
    { "60BB0C", "Beijing HuaqinWorld Technology Co,Ltd" },
    { "60BC4C", "EWM Hightec Welding GmbH" },
    { "60BD2C", "Taicang T&W Electronics" },
    { "60BD83", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "60BD91", "Move Innovation" },
    { "60BEB4", "S-Bluetech co., limited" },
    { "60BEB5", "Motorola Mobility LLC, a Lenovo Company" },
    { "60BEC4", "Apple, Inc." },
    { "60C01E", "V&G Information System Co.,Ltd" },
    { "60C0BF", "ON Semiconductor" },
    { "60C1CB", "Fujian Great Power PLC Equipment Co.,Ltd" },
    { "60C22A", "Allwinner Technology Co., Ltd" },
    { "60C397", "2Wire Inc" },
    { "60C547", "Apple, Inc." },
    { "60C5A8", "Beijing LT Honway Technology Co.,Ltd" },
    { "60C5AD", "Samsung Electronics Co.,Ltd" },
    { "60C5E6", "Skullcandy" },
    { "60C658", "PHYTRONIX Co.,Ltd." },
    { "60C727", "Digiboard Eletronica da Amazonia Ltda" },
    { "60C78D", "Juniper Networks" },
    { "60C798", "Verifone, Inc." },
    { "60C7BE", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "60C980", "Trymus" },
    { "60C9AA", "Nokia" },
    { "60CBFB", "AirScape Inc." },
    { "60CDA9", "Abloomy" },
    { "60CDC5", "Taiwan Carol Electronics., Ltd" },
    { "60CE41", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "60CE86", "Sercomm Corporation." },
    { "60CE92", "The Refined Industry Company Limited" },
    { "60CF69", "meerecompany" },
    { "60CF84", "ASUSTek COMPUTER INC." },
    { "60D02C", "Ruckus Wireless" },
    { "60D039", "Apple, Inc." },
    { "60D0A9", "Samsung Electronics Co.,Ltd" },
    { "60D1AA", "Vishal Telecommunications Pvt Ltd" },
    { "60D21C", "Sunnovo International Limited" },
    { "60D248", "Commscope" },
    { "60D262", "Tzukuri Pty Ltd" },
    { "60D2B9", "REALAND BIO CO., LTD." },
    { "60D2DD", "Shenzhen Baitong Putian Technology Co.,Ltd." },
    { "60D30A", "Quatius Limited" },
    { "60D4E9", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "60D51B", "Fujitsu Limited" },
    { "60D561", "Shenzhen Glazero Technology Co., Ltd." },
    { "60D755", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "60D7E3", "IEEE Registration Authority" },
    { "60D819", "Hon Hai Precision Ind. Co.,Ltd." },
    { "60D89C", "HMD Global Oy" },
    { "60D8A4", "Vantiva Connected Home - Technologies Telco" },
    { "60D9A0", "Lenovo Mobile Communication Technology Ltd." },
    { "60D9C7", "Apple, Inc." },
    { "60DA23", "Estech Co.,Ltd" },
    { "60DA83", "Hangzhou H3C Technologies Co., Limited" },
    { "60DB15", "New H3C Technologies Co., Ltd" },
    { "60DB2A", "HNS" },
    { "60DB98", "Calix Inc." },
    { "60DBEF", "Unify Software and Solutions GmbH & Co. KG" },
    { "60DC0D", "TAIWAN SHIN KONG SECURITY CO., LTD" },
    { "60DC81", "AltoBeam Inc." },
    { "60DD70", "Apple, Inc." },
    { "60DD8E", "Intel Corporate" },
    { "60DE35", "GITSN, Inc." },
    { "60DE44", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "60DE94", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "60DEF3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "60DEF4", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "60E00E", "SHINSEI ELECTRONICS CO LTD" },
    { "60E327", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "60E32B", "Intel Corporate" },
    { "60E3AC", "LG Electronics (Mobile Communications)" },
    { "60E5D8", "zte corporation" },
    { "60E6BC", "Sino-Telecom Technology Co.,Ltd." },
    { "60E6F0", "Wistron Neweb Corporation" },
    { "60E701", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "60E78A", "UNISEM" },
    { "60E85B", "Texas Instruments" },
    { "60E956", "Ayla Networks, Inc" },
    { "60E9AA", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "60EB5A", "Asterfusion Data Technologies Co.,Ltd" },
    { "60EB69", "Quanta Computer Inc." },
    { "60EE5C", "SHENZHEN FAST TECHNOLOGIES CO.,LTD" },
    { "60EFAB", "Silicon Laboratories" },
    { "60EFC6", "Shenzhen Chima Technologies Co Limited" },
    { "60F04D", "Honor Device Co., Ltd." },
    { "60F13D", "JABLOCOM s.r.o." },
    { "60F189", "Murata Manufacturing Co., Ltd." },
    { "60F18A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "60F262", "Intel Corporate" },
    { "60F281", "TRANWO TECHNOLOGY CO., LTD." },
    { "60F2EF", "VisionVera International Co., Ltd." },
    { "60F3DA", "Logic Way GmbH" },
    { "60F419", "eero inc." },
    { "60F43A", "Edifier International" },
    { "60F445", "Apple, Inc." },
    { "60F494", "Hon Hai Precision Ind. Co.,Ltd." },
    { "60F549", "Apple, Inc." },
    { "60F59C", "CRU-Dataport" },
    { "60F673", "TERUMO CORPORATION" },
    { "60F677", "Intel Corporate" },
    { "60F81D", "Apple, Inc." },
    { "60F8F2", "Synaptec" },
    { "60FA9D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "60FAB1", "Kempower Oyj" },
    { "60FACD", "Apple, Inc." },
    { "60FB00", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "60FB42", "Apple, Inc." },
    { "60FCF1", "Private" },
    { "60FD56", "WOORISYSTEMS CO., Ltd" },
    { "60FDA6", "Apple, Inc." },
    { "60FE1E", "China Palms Telecom.Ltd" },
    { "60FE20", "2Wire Inc" },
    { "60FEC5", "Apple, Inc." },
    { "60FEF9", "Thomas & Betts" },
    { "60FF12", "Samsung Electronics Co.,Ltd" },
    { "60FF9E", "AzureWave Technology Inc." },
    { "60FFDD", "C.E. ELECTRONICS, INC" },
    { "64002D", "Powerlinq Co., LTD" },
    { "64006A", "Dell Inc." },
    { "64009C", "Insulet Corporation" },
    { "6400F1", "Cisco Systems, Inc" },
    { "6401FB", "Landis+Gyr GmbH" },
    { "64028F", "Silicon Laboratories" },
    { "6402CB", "Commscope" },
    { "64037F", "Samsung Electronics Co.,Ltd" },
    { "640552", "China Post Communication Equipment Co.," },
    { "6405BE", "NEW LIGHT LED" },
    { "6405E4", "ALPSALPINE CO,.LTD" },
    { "6405E9", "Shenzhen WayOS Technology Crop., Ltd." },
    { "64078C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6407F6", "Samsung Electronics Co.,Ltd" },
    { "640864", "Cisco Systems, Inc" },
    { "64094C", "Beijing Superbee Wireless Technology Co.,Ltd" },
    { "640980", "Xiaomi Communications Co Ltd" },
    { "6409AC", "TCT mobile ltd" },
    { "640B4A", "Digital Telecom Technology Limited" },
    { "640BD7", "Apple, Inc." },
    { "640C91", "Apple, Inc." },
    { "640D22", "LG Electronics (Mobile Communications)" },
    { "640DCE", "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "640DE6", "Petra Systems" },
    { "640E36", "TAZTAG" },
    { "640E6A", "SECO-LARM USA Inc" },
    { "640E94", "Pluribus Networks, Inc." },
    { "640E9B", "ISHIDA MEDICAL CO., LTD." },
    { "640F28", "2Wire Inc" },
    { "641084", "HEXIUM Technical Development Co., Ltd." },
    { "6411A4", "Motorola Mobility LLC, a Lenovo Company" },
    { "641225", "Cisco Systems, Inc" },
    { "641236", "Vantiva USA LLC" },
    { "641269", "Commscope" },
    { "641331", "Bosch Car Multimedia (Wuhu) Co. Ltd." },
    { "64135A", "Itectra A/S" },
    { "64136C", "zte corporation" },
    { "6413AB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "641666", "Nest Labs Inc." },
    { "64167F", "Polycom" },
    { "64168D", "Cisco Systems, Inc" },
    { "6416F0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "641759", "Intellivision Holdings, LLC" },
    { "6417CD", "Samsung Electronics Co.,Ltd" },
    { "641A22", "Heliospectra AB" },
    { "641ABA", "Dryad Networks GmbH" },
    { "641B2F", "Samsung Electronics Co.,Ltd" },
    { "641C10", "Texas Instruments" },
    { "641C67", "DIGIBRAS INDUSTRIA DO BRASILS/A" },
    { "641CAE", "Samsung Electronics Co.,Ltd" },
    { "641CB0", "Samsung Electronics Co.,Ltd" },
    { "641E81", "Dowslake Microsystems" },
    { "64200C", "Apple, Inc." },
    { "64209F", "Tilgin AB" },
    { "6420E0", "T3 Technology Co., Ltd." },
    { "642184", "Nippon Denki Kagaku Co.,LTD" },
    { "642216", "Shandong Taixin Electronic co.,Ltd" },
    { "642315", "Huawei Device Co., Ltd." },
    { "642400", "Xorcom Ltd." },
    { "64255E", "Observint Technologies, Inc." },
    { "6425EC", "guangdong kesheng zhixun technology " },
    { "642656", "Shenzhen Fanweitai Technology Service Co.,Ltd" },
    { "642677", "BKM-Micronic Richtfunkanlagen GmbH" },
    { "642737", "Hon Hai Precision Ind. Co.,Ltd." },
    { "642753", "Huawei Device Co., Ltd." },
    { "642943", "D-Link Corporation" },
    { "6429ED", "AO PKK Milandr" },
    { "6429FF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "642B8A", "ALL BEST Industrial Co., Ltd." },
    { "642C0F", "vivo Mobile Communication Co., Ltd." },
    { "642CAC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "642DB7", "SEUNGIL ELECTRONICS" },
    { "642E41", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "642FC7", "New H3C Technologies Co., Ltd" },
    { "643135", "Apple, Inc." },
    { "643139", "IEEE Registration Authority" },
    { "643150", "Hewlett Packard" },
    { "643172", "ZHEJIANG HISING TECHNOLOGY CO.,LTD" },
    { "64317E", "Dexin Corporation" },
    { "643216", "Weidu Technology (Beijing) Co., Ltd." },
    { "6432A8", "Intel Corporate" },
    { "6433AA", "Mellanox Technologies, Inc." },
    { "6433B5", "IEEE Registration Authority" },
    { "6433DB", "Texas Instruments" },
    { "643409", "BITwave Pte Ltd" },
    { "64351C", "e-CON SYSTEMS INDIA PVT LTD" },
    { "6437A4", "TOKYOSHUHA CO.,LTD." },
    { "643AB1", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "643AEA", "Cisco Systems, Inc" },
    { "643E0A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "643E8C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "643F5F", "Exablaze" },
    { "6441E6", "Apple, Inc." },
    { "644212", "Shenzhen Water World Information Co.,Ltd." },
    { "644214", "Swisscom Energy Solutions AG" },
    { "644346", "GuangDong Quick Network Computer CO.,LTD" },
    { "64447B", "vivo Mobile Communication Co., Ltd." },
    { "6444D5", "TD Tech" },
    { "6447E0", "Feitian Technologies Co., Ltd" },
    { "644842", "Apple, Inc." },
    { "64497D", "Intel Corporate" },
    { "644A7D", "Intel Corporate" },
    { "644BC3", "Shanghai WOASiS Telecommunications Ltd., Co." },
    { "644BF0", "CalDigit, Inc" },
    { "644C36", "Intel Corporate" },
    { "644C69", "CONPROVE" },
    { "644D70", "dSPACE GmbH" },
    { "644ED7", "HP Inc." },
    { "644EEB", "Daikin Holdings Singapore Pte Ltd" },
    { "644F42", "JETTER CO., Ltd." },
    { "644F56", "YEALINK(XIAMEN) NETWORK TECHNOLOGY CO.,LTD." },
    { "644F74", "LENUS Co., Ltd." },
    { "644FB0", "Hyunjin.com" },
    { "6450D6", "Liquidtool Systems" },
    { "645106", "Hewlett Packard" },
    { "64517E", "LONG BEN (DONGGUAN) ELECTRONIC TECHNOLOGY CO.,LTD." },
    { "6451F4", "Huawei Device Co., Ltd." },
    { "645234", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "645299", "The Chamberlain Group, Inc" },
    { "64535D", "Frauscher Sensortechnik" },
    { "6453E0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "645422", "Equinox Payments" },
    { "645563", "Q-Free America, Inc." },
    { "64557F", "NSFOCUS Information Technology Co., Ltd." },
    { "6455B1", "Commscope" },
    { "645601", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "645725", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "6457E5", "Beijing Royaltech Co.,Ltd" },
    { "6458AD", "China Mobile IOT Company Limited" },
    { "6459F8", "Vodafone Omnitel B.V." },
    { "645A04", "Chicony Electronics Co., Ltd." },
    { "645A36", "Apple, Inc." },
    { "645AED", "Apple, Inc." },
    { "645CF3", "ParanTek Inc." },
    { "645D86", "Intel Corporate" },
    { "645D92", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "645DD7", "Shenzhen Lifesense Medical Electronics Co., Ltd. " },
    { "645DF4", "Samsung Electronics Co.,Ltd" },
    { "645E10", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "645E2C", "IRay Technology Co., Ltd." },
    { "645EBE", "Yahoo! JAPAN" },
    { "645FFF", "Nicolet Neuro" },
    { "646038", "Hirschmann Automation and Control GmbH" },
    { "646140", "Huawei Device Co., Ltd." },
    { "646184", "VELUX" },
    { "646223", "Cellient Co., Ltd." },
    { "646266", "IEEE Registration Authority" },
    { "64628A", "evon GmbH" },
    { "646306", "Xiaomi Communications Co Ltd" },
    { "64644A", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "64649B", "Juniper Networks" },
    { "6465C0", "Nuvon, Inc" },
    { "646624", "Sagemcom Broadband SAS" },
    { "6466B3", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "6466D8", "Samsung Electronics Co.,Ltd" },
    { "646707", "Beijing Omnific Technology, Ltd." },
    { "646772", "SERCOMM PHILIPPINES INC" },
    { "6467CD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "64680C", "Comtrend Corporation" },
    { "64681A", "DASAN Network Solutions" },
    { "646876", "Edifier International" },
    { "64694E", "Texas Instruments" },
    { "6469BC", "Hytera Communications Co .,ltd" },
    { "646A52", "Avaya Inc" },
    { "646A74", "AUTH-SERVERS, LLC" },
    { "646C80", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "646CB2", "Samsung Electronics Co.,Ltd" },
    { "646D2F", "Apple, Inc." },
    { "646D4E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "646D6C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "646E60", "zte corporation" },
    { "646E69", "Liteon Technology Corporation" },
    { "646E6C", "Radio Datacom LLC" },
    { "646E97", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "646EE0", "Intel Corporate" },
    { "646EEA", "Iskratel d.o.o." },
    { "647002", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "647033", "Apple, Inc." },
    { "647060", "Texas Instruments" },
    { "6472D8", "GooWi Technology Co.,Limited" },
    { "647366", "Shenzhen Siera Technology Ltd" },
    { "6473E2", "Arbiter Systems, Inc." },
    { "6474F6", "Shooter Detection Systems" },
    { "647520", "zte corporation" },
    { "6475DA", "Arcadyan Corporation" },
    { "647657", "Innovative Security Designs" },
    { "6476BA", "Apple, Inc." },
    { "64777D", "Hitron Technologies. Inc" },
    { "647791", "Samsung Electronics Co.,Ltd" },
    { "647924", "Huawei Device Co., Ltd." },
    { "6479A7", "Phison Electronics Corp. " },
    { "6479F0", "Intel Corporate" },
    { "647B1E", "Sagemcom Broadband SAS" },
    { "647B40", "Sichuan AI-Link Technology Co., Ltd." },
    { "647BCE", "Samsung Electronics Co.,Ltd" },
    { "647BD4", "Texas Instruments" },
    { "647C34", "Ubee Interactive Co., Limited" },
    { "647CE8", "Palo Alto Networks" },
    { "647D81", "YOKOTA INDUSTRIAL CO,.LTD" },
    { "647FDA", "TEKTELIC Communications Inc." },
    { "64808B", "VG Controls, Inc." },
    { "648099", "Intel Corporate" },
    { "648125", "Alphatron Marine BV" },
    { "648214", "FN-LINK TECHNOLOGY Ltd." },
    { "648505", "zte corporation" },
    { "648624", "Beijing Global Safety Technology Co., LTD." },
    { "648788", "Juniper Networks" },
    { "6487D7", "ADB Broadband Italia" },
    { "6488FF", "Sichuan Changhong Electric Ltd." },
    { "64899A", "LG Electronics (Mobile Communications)" },
    { "6489F1", "Samsung Electronics Co.,Ltd" },
    { "648B9B", "ALWAYS ON TECH PTE.LTD." },
    { "648CBB", "Texas Instruments" },
    { "648D9E", "IVT Electronic Co.,Ltd" },
    { "648F3E", "Cisco Systems, Inc" },
    { "648FDB", "Huaqin Technology Co.LTD" },
    { "6490C1", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "64956C", "LG Electronics" },
    { "649714", "eero inc." },
    { "649829", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "64989E", "TRINNOV AUDIO" },
    { "64995D", "LGE " },
    { "649968", "Elentec" },
    { "6499A0", "AG Elektronik AB" },
    { "649A08", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "649A12", "P2 Mobile Technologies Limited" },
    { "649A63", "Ring LLC" },
    { "649ABE", "Apple, Inc." },
    { "649B24", "V Technology Co., Ltd." },
    { "649C81", "Qualcomm Inc." },
    { "649C8E", "Texas Instruments" },
    { "649CF3", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "649D38", "Google, Inc." },
    { "649D99", "FS COM INC" },
    { "649E31", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "649EF3", "Cisco Systems, Inc" },
    { "649FF7", "Kone OYj" },
    { "64A0AC", "Adtran Inc" },
    { "64A0E7", "Cisco Systems, Inc" },
    { "64A198", "Huawei Device Co., Ltd." },
    { "64A200", "Xiaomi Communications Co Ltd" },
    { "64A232", "OOO Samlight" },
    { "64A28A", "Huawei Device Co., Ltd." },
    { "64A2F9", "OnePlus Technology (Shenzhen) Co., Ltd" },
    { "64A341", "Wonderlan (Beijing) Technology Co., Ltd." },
    { "64A3CB", "Apple, Inc." },
    { "64A3CC", "LeoLabs" },
    { "64A40E", "Tianyi Telecom Terminals Company Limited" },
    { "64A444", "Loongson Technology Corporation Limited" },
    { "64A5C3", "Apple, Inc." },
    { "64A651", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "64A68F", "Zhongshan Readboy Electronics Co.,Ltd" },
    { "64A769", "HTC Corporation" },
    { "64A7DD", "Avaya Inc" },
    { "64A837", "Juni Korea Co., Ltd" },
    { "64A965", "Linkflow Co., Ltd." },
    { "64AC2B", "Juniper Networks" },
    { "64AE0C", "Cisco Systems, Inc" },
    { "64AE88", "Polytec GmbH" },
    { "64AEF1", "Qingdao Hisense Electronics Co.,Ltd." },
    { "64B0A6", "Apple, Inc." },
    { "64B0E8", "Huawei Device Co., Ltd." },
    { "64B21D", "Chengdu Phycom Tech Co., Ltd." },
    { "64B2B4", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "64B310", "Samsung Electronics Co.,Ltd" },
    { "64B370", "PowerComm Solutions LLC" },
    { "64B379", "Jiangsu Viscore Technologies Co.,Ltd" },
    { "64B473", "Xiaomi Communications Co Ltd" },
    { "64B5C6", "Nintendo Co.,Ltd" },
    { "64B5F2", "Samsung Electronics Co.,Ltd" },
    { "64B623", "Schrack Seconet Care Communication GmbH" },
    { "64B64A", "ViVOtech, Inc." },
    { "64B708", "Espressif Inc." },
    { "64B853", "Samsung Electronics Co.,Ltd" },
    { "64B94E", "Dell Technologies" },
    { "64B9E8", "Apple, Inc." },
    { "64BAA4", "zte corporation" },
    { "64BABD", "SDJ Technologies, Inc." },
    { "64BB1E", "Earda Technologies co Ltd" },
    { "64BC0C", "LG Electronics (Mobile Communications)" },
    { "64BC11", "CombiQ AB" },
    { "64BC58", "Intel Corporate" },
    { "64BE63", "STORDIS GmbH" },
    { "64BF6B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "64C01A", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "64C17E", "cheilelectric" },
    { "64C269", "eero inc." },
    { "64C2DE", "LG Electronics (Mobile Communications)" },
    { "64C354", "Avaya Inc" },
    { "64C394", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "64C3D6", "Juniper Networks" },
    { "64C403", "Quectel Wireless Solutions Co.,Ltd." },
    { "64C582", "China Mobile Group Device Co.,Ltd." },
    { "64C5AA", "South African Broadcasting Corporation" },
    { "64C667", "Barnes&Noble" },
    { "64C6AF", "AXERRA Networks Ltd" },
    { "64C6D2", "Seiko Epson Corporation" },
    { "64C753", "Apple, Inc." },
    { "64C901", "INVENTEC Corporation" },
    { "64C944", "LARK Technologies, Inc" },
    { "64CB5D", "SIA TeleSet" },
    { "64CB9F", "TECNO MOBILE LIMITED" },
    { "64CBA3", "Pointmobile" },
    { "64CBE9", "LG Innotek" },
    { "64CC22", "Arcadyan Corporation" },
    { "64CC2E", "Xiaomi Communications Co Ltd" },
    { "64CDC2", "Amazon Technologies Inc." },
    { "64CDF1", "KO & AL Co., Ltd" },
    { "64CE6E", "Sierra Wireless, ULC" },
    { "64CF13", "Weigao Nikkiso(Weihai)Dialysis Equipment Co.,Ltd" },
    { "64CFD9", "Texas Instruments" },
    { "64D02D", "NEXT GENERATION INTEGRATION LIMITED (NGI)" },
    { "64D0D6", "Samsung Electronics Co.,Ltd" },
    { "64D154", "Routerboard.com" },
    { "64D1A3", "Sitecom Europe BV" },
    { "64D241", "Keith & Koep GmbH" },
    { "64D2C4", "Apple, Inc." },
    { "64D315", "HMD Global Oy" },
    { "64D4BD", "ALPSALPINE CO,.LTD" },
    { "64D4DA", "Intel Corporate" },
    { "64D69A", "Intel Corporate" },
    { "64D7C0", "Huawei Device Co., Ltd." },
    { "64D814", "Cisco Systems, Inc" },
    { "64D81B", "Vestel Elektronik San ve Tic. A.S." },
    { "64D912", "Solidica, Inc." },
    { "64D954", "Taicang T&W Electronics" },
    { "64D989", "Cisco Systems, Inc" },
    { "64D9C2", "eero inc." },
    { "64DAA0", "Robert Bosch Smart Home GmbH" },
    { "64DAED", "eero inc." },
    { "64DB18", "OpenPattern" },
    { "64DB38", "zte corporation" },
    { "64DB43", "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd." },
    { "64DB81", "Syszone Co., Ltd." },
    { "64DB8B", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "64DBA0", "Select Comfort" },
    { "64DBF7", "Nokia Shanghai Bell Co., Ltd." },
    { "64DC01", "Static Systems Group PLC" },
    { "64DCDE", "ZheJiang FuChunJiang Information Technology Co.,Ltd" },
    { "64DD68", "Zyxel Communications Corporation" },
    { "64DDE9", "Xiaomi Communications Co Ltd" },
    { "64DE1C", "Kingnetic Pte Ltd" },
    { "64DE6D", "Intel Corporate" },
    { "64DF10", "JingLue Semiconductor(SH) Ltd." },
    { "64DFE9", "ATEME" },
    { "64E003", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "64E0AB", "UNION MAN TECHNOLOGY CO.,LTD" },
    { "64E161", "DEP Corp." },
    { "64E172", "Shenzhen Qihoo Intelligent Technology Co.,Ltd" },
    { "64E204", "NTN Technical Service Corporation" },
    { "64E220", "Qisda Corporation" },
    { "64E4A5", "LG Electronics" },
    { "64E599", "EFM Networks" },
    { "64E625", "Woxu Wireless Co., Ltd" },
    { "64E682", "Apple, Inc." },
    { "64E738", "Zhejiang SUPCON Technology Co., Ltd." },
    { "64E7D8", "Samsung Electronics Co.,Ltd" },
    { "64E815", "Arcelik A.S" },
    { "64E833", "Espressif Inc." },
    { "64E84F", "Serialway Communication Technology Co. Ltd" },
    { "64E881", "Hewlett Packard Enterprise" },
    { "64E892", "Morio Denki Co., Ltd." },
    { "64E8E6", "global moisture management system" },
    { "64E950", "Cisco Systems, Inc" },
    { "64EAC5", "SiboTech Automation Co., Ltd." },
    { "64EB8C", "Seiko Epson Corporation" },
    { "64EC65", "vivo Mobile Communication Co., Ltd." },
    { "64ED57", "Commscope" },
    { "64ED62", "WOORI SYSTEMS Co., Ltd" },
    { "64EEB7", "Netis Technology Co., Ltd." },
    { "64F0AD", "Zhejiang Tmall Technology Co., Ltd." },
    { "64F242", "Gerdes Aktiengesellschaft" },
    { "64F2FB", "Hangzhou Ezviz Software Co.,Ltd." },
    { "64F50E", "Kinion Technology Company Limited" },
    { "64F54E", "EM Microelectronic" },
    { "64F69D", "Cisco Systems, Inc" },
    { "64F6BB", "Fibocom Wireless Inc." },
    { "64F6F7", "Anhui Dynamic Power Co., Ltd." },
    { "64F705", "Huawei Device Co., Ltd." },
    { "64F81C", "Huawei Technologies Co., Ltd." },
    { "64F88A", "China Mobile IOT Company Limited" },
    { "64F947", "Senscomm Semiconductor Co., Ltd." },
    { "64F970", "Kenade Electronics Technology Co.,LTD." },
    { "64F987", "Avvasi Inc." },
    { "64F9C0", "ANALOG DEVICES" },
    { "64FB01", "Zhongshan Camry Electronic Company Limited" },
    { "64FB50", "RoomReady/Zdi, Inc." },
    { "64FB81", "IEEE Registration Authority" },
    { "64FB92", "PPC Broadband Inc." },
    { "64FC8C", "Zonar Systems" },
    { "64FD29", "Zhejiang Dahua Technology Co., Ltd." },
    { "64FD96", "Sagemcom Broadband SAS" },
    { "64FE15", "Flaircomm Microelectronics,Inc." },
    { "64FF0A", "Wistron Neweb Corporation" },
    { "680235", "Konten Networks Inc." },
    { "6802B8", "Compal Broadband Networks, Inc." },
    { "680489", "Cisco Systems, Inc" },
    { "680571", "Samsung Electronics Co.,Ltd" },
    { "6805CA", "Intel Corporate" },
    { "68070A", "TPVision Europe B.V" },
    { "680715", "Intel Corporate" },
    { "680927", "Apple, Inc." },
    { "680AD7", "Yancheng Kecheng Optoelectronic Technology Co., Ltd" },
    { "680AE2", "Silicon Laboratories" },
    { "68122D", "Special Instrument Development Co., Ltd." },
    { "681295", "Lupine Lighting Systems GmbH" },
    { "681324", "Huawei Device Co., Ltd." },
    { "6813E2", "Eltex Enterprise LTD" },
    { "6813F3", "Amazon Technologies Inc." },
    { "681401", "Hon Hai Precision Ind. Co.,Ltd." },
    { "681590", "Sagemcom Broadband SAS" },
    { "6815D3", "Zaklady Elektroniki i Mechaniki Precyzyjnej R&G S.A." },
    { "681605", "Systems And Electronic Development FZCO" },
    { "681729", "Intel Corporate" },
    { "6818D9", "Hill AFB - CAPRE Group" },
    { "68193F", "Digital Airways" },
    { "6819AC", "Guangzhou Xianyou Intelligent Technogoly CO., LTD" },
    { "681A7C", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "681AA4", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "681AB2", "zte corporation" },
    { "681BEF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "681CA2", "Rosewill Inc." },
    { "681D4C", "eSystems MTG GmbH" },
    { "681D64", "Sunwave Communications Co., Ltd" },
    { "681DEF", "Shenzhen CYX Technology Co., Ltd." },
    { "681E8B", "InfoSight Corporation" },
    { "681F40", "Blu Wireless Technology Ltd" },
    { "681FD8", "Siemens Industry, Inc." },
    { "68215F", "Edgecore Networks Corporation" },
    { "68228E", "Juniper Networks" },
    { "6822E5", "Huawei Device Co., Ltd." },
    { "68234B", "Nihon Dengyo Kousaku" },
    { "6823B0", "Texas Instruments" },
    { "6823F4", "Shenzhen Jinlangxin Technology Co., Ltd" },
    { "682499", "Quectel Wireless Solutions Co.,Ltd." },
    { "6825DD", "Espressif Inc." },
    { "682624", "Ergatta" },
    { "68262A", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "682719", "Microchip Technology Inc." },
    { "682737", "Samsung Electronics Co.,Ltd" },
    { "68275F", "zte corporation" },
    { "68286C", "Sony Interactive Entertainment Inc." },
    { "6828BA", "Dejai" },
    { "6828CF", "Hewlett Packard Enterprise" },
    { "6828F6", "Vubiq Networks, Inc." },
    { "6829DC", "Ficosa Electronics S.L.U." },
    { "682C4F", "leerang corporation" },
    { "682C7B", "Cisco Systems, Inc" },
    { "682D83", "SHENZHEN DINGHE COMMUNICATION COMPANY" },
    { "682DDC", "Wuhan Changjiang Electro-Communication Equipment CO.,LTD" },
    { "682F67", "Apple, Inc." },
    { "6831FE", "Teladin Co.,Ltd." },
    { "68332C", "KENSTEL NETWORKS LIMITED" },
    { "6833EE", "Commscope" },
    { "683421", "Intel Corporate" },
    { "683489", "LEA Professional" },
    { "683563", "SHENZHEN LIOWN ELECTRONICS CO.,LTD." },
    { "6836B5", "DriveScale, Inc." },
    { "6837E9", "Amazon Technologies Inc." },
    { "6838E3", "EYESON SOLUTION CO.,Ltd" },
    { "683943", "ittim" },
    { "683A1E", "Cisco Meraki" },
    { "683A48", "SAMJIN Co., Ltd." },
    { "683B1E", "Countwise LTD" },
    { "683B78", "Cisco Systems, Inc" },
    { "683C7D", "Magic Intelligence Technology Limited" },
    { "683E02", "SIEMENS AG, Digital Factory, Motion Control System" },
    { "683E26", "Intel Corporate" },
    { "683E34", "MEIZU Technology Co., Ltd." },
    { "683EC0", "Apple, Inc." },
    { "683EEC", "ERECA" },
    { "683F1E", "EFFECT Photonics B.V." },
    { "683F7D", "INGRAM MICRO SERVICES" },
    { "68403C", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "68418F", "Telechips, Inc." },
    { "684216", "Steplock Access AB" },
    { "684352", "Bhuu Limited" },
    { "6843D7", "Agilecom Photonics Solutions Guangdong Limited" },
    { "684406", "Texas Instruments" },
    { "684465", "Apple, Inc." },
    { "684571", "Huawei Device Co., Ltd." },
    { "6845CC", "Apple, Inc." },
    { "6845F1", "TOSHIBA CLIENT SOLUTIONS CO., LTD." },
    { "684724", "EM Microelectronic" },
    { "684749", "Texas Instruments" },
    { "684898", "Samsung Electronics Co.,Ltd" },
    { "684983", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "684992", "Cisco Meraki" },
    { "6849B2", "CARLO GAVAZZI LTD" },
    { "684A5F", "Apple, Inc." },
    { "684A76", "eero inc." },
    { "684AAE", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "684AE9", "Samsung Electronics Co.,Ltd" },
    { "684B88", "Galtronics Telemetry Inc." },
    { "684C25", "Huawei Device Co., Ltd." },
    { "684CA8", "Shenzhen Herotel Tech. Co., Ltd." },
    { "684DB6", "Xiaomi Communications Co Ltd" },
    { "684E05", "HUNAN FN-LINK TECHNOLOGY LIMITED" },
    { "684F64", "Dell Inc." },
    { "68505D", "Halo Technologies" },
    { "68508C", "Shanghai Sunmi Technology Co.,Ltd." },
    { "685134", "Hewlett Packard Enterprise" },
    { "6851B7", "PowerCloud Systems, Inc." },
    { "685210", "MCS Logic" },
    { "6852D6", "UGame Technology Co.,Ltd" },
    { "68536C", "SPnS Co.,Ltd" },
    { "685377", "Bouffalo Lab (Nanjing) Co., Ltd." },
    { "685388", "P&S Technology" },
    { "68539D", "EM Microelectronic" },
    { "68545A", "Intel Corporate" },
    { "6854C1", "ColorTokens, Inc." },
    { "6854ED", "Alcatel-Lucent" },
    { "6854F5", "enLighted Inc" },
    { "6854FD", "Amazon Technologies Inc." },
    { "6855D4", "Seiko Epson Corporation" },
    { "68572D", "Tuya Smart Inc." },
    { "685811", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "6858A0", "Honor Device Co., Ltd." },
    { "6858C5", "ZF TRW Automotive" },
    { "685932", "Sunitec Enterprise Co.,Ltd" },
    { "68597F", "Alcatel Lucent" },
    { "685ACF", "Samsung Electronics Co.,Ltd" },
    { "685B35", "Apple, Inc." },
    { "685B36", "POWERTECH INDUSTRIAL CO., LTD." },
    { "685D43", "Intel Corporate" },
    { "685E1C", "Texas Instruments" },
    { "685E6B", "PowerRay Co., Ltd." },
    { "685EDD", "Apple, Inc." },
    { "68628A", "vivo Mobile Communication Co., Ltd." },
    { "686350", "Hella India Automotive Pvt Ltd" },
    { "686359", "Advanced Digital Broadcast SA" },
    { "686372", "Huawei Device Co., Ltd." },
    { "68644B", "Apple, Inc." },
    { "6865B7", "Zhishang Chuanglian Technology Co., Ltd" },
    { "686725", "Espressif Inc." },
    { "6867C7", "Sagemcom Broadband SAS" },
    { "68692E", "Zycoo Co.,Ltd" },
    { "686975", "Angler Labs Inc" },
    { "6869CA", "Hitachi, Ltd." },
    { "6869F2", "ComAp s.r.o." },
    { "686B6A", "Phytium Technology Co.,Ltd." },
    { "686CE6", "Microsoft Corporation" },
    { "686DBC", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "686E23", "Wi3 Inc." },
    { "686E48", "Prophet Electronic Technology Corp.,Ltd" },
    { "687161", "Cisco Systems, Inc" },
    { "687251", "Ubiquiti Inc" },
    { "6872C3", "Samsung Electronics Co.,Ltd" },
    { "6872DC", "CETORY.TV Company Limited" },
    { "687627", "Zhuhai Dingzhi Electronic Technology Co., Ltd" },
    { "68764F", "Sony Corporation" },
    { "687724", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "6877DA", "zte corporation" },
    { "687848", "WESTUNITIS CO., LTD." },
    { "68784C", "Nortel Networks" },
    { "687909", "Cisco Systems, Inc" },
    { "687912", "IEEE Registration Authority" },
    { "687924", "ELS-GmbH & Co. KG" },
    { "6879C4", "Shanghai MXCHIP Information Technology Co., Ltd." },
    { "6879DD", "Omnipless Manufacturing (PTY) Ltd" },
    { "6879ED", "SHARP Corporation" },
    { "687A64", "Intel Corporate" },
    { "687CC8", "Measurement Systems S. de R.L." },
    { "687CD5", "Y Soft Corporation, a.s." },
    { "687D00", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "687D6B", "Samsung Electronics Co.,Ltd" },
    { "687DB4", "Cisco Systems, Inc" },
    { "687F74", "Cisco-Linksys, LLC" },
    { "687FF0", "TP-Link Systems Inc" },
    { "6881E0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6882F2", "FIXME GmbH" },
    { "68831A", "Pandora Mobility Corporation" },
    { "6883CB", "Apple, Inc." },
    { "688470", "eSSys Co.,Ltd" },
    { "68847E", "FUJITSU LIMITED" },
    { "688540", "IGI Mobile, Inc." },
    { "68856A", "OuterLink Corporation" },
    { "6885A4", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "6886A7", "Cisco Systems, Inc" },
    { "6886E7", "Orbotix, Inc." },
    { "68871C", "Motorola Mobility LLC, a Lenovo Company" },
    { "68876B", "INQ Mobile Limited" },
    { "6887BD", "zte corporation" },
    { "6887C6", "Cisco Systems, Inc" },
    { "6888A1", "Universal Electronics, Inc." },
    { "688975", "nuoxc" },
    { "6889C1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "688AB5", "EDP Servicos" },
    { "688AF0", "zte corporation" },
    { "688B0F", "China Mobile IOT Company Limited" },
    { "688BF4", "Arista Network, Inc." },
    { "688DB6", "AETEK INC." },
    { "688F2E", "Hitron Technologies. Inc" },
    { "688F84", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "688FC9", "Zhuolian (Shenzhen) Communication Co., Ltd" },
    { "6891D0", "IEEE Registration Authority" },
    { "689234", "Ruckus Wireless" },
    { "689320", "New H3C Technologies Co., Ltd" },
    { "68932E", "Habana Labs LTD." },
    { "689361", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "689423", "Hon Hai Precision Ind. Co.,Ltd." },
    { "68944A", "zte corporation" },
    { "68951B", "Huawei Device Co., Ltd." },
    { "689575", "Zhejiang Bodyguard Electronic Co., Ltd" },
    { "68962E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "68966A", "OHSUNG" },
    { "68967B", "Apple, Inc." },
    { "68974B", "Shenzhen Costar Electronics Co. Ltd." },
    { "6897E8", "Society of Motion Picture &amp; Television Engineers" },
    { "689861", "Beacon Inc" },
    { "6899CD", "Cisco Systems, Inc" },
    { "689A21", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "689A87", "Amazon Technologies Inc." },
    { "689AB7", "Atelier Vision Corporation" },
    { "689B43", "Huawei Device Co., Ltd." },
    { "689C5E", "AcSiP Technology Corp." },
    { "689C70", "Apple, Inc." },
    { "689CE2", "Cisco Systems, Inc" },
    { "689E0B", "Cisco Systems, Inc" },
    { "689E19", "Texas Instruments" },
    { "689E29", "zte corporation" },
    { "689E6A", "Huawei Device Co., Ltd." },
    { "689FF0", "zte corporation" },
    { "68A03E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "68A0F6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "68A1B7", "Honghao Mingchuan Technology (Beijing) CO.,Ltd." },
    { "68A2AA", "Acres Manufacturing" },
    { "68A378", "FREEBOX SAS" },
    { "68A3C4", "Liteon Technology Corporation" },
    { "68A40E", "BSH Hausgeräte GmbH" },
    { "68A46A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "68A47D", "Sun Cupid Technology (HK) LTD" },
    { "68A593", "Apple, Inc." },
    { "68A682", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "68A7B4", "Honor Device Co., Ltd." },
    { "68A828", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "68A86D", "Apple, Inc." },
    { "68A878", "GeoWAN Pty Ltd" },
    { "68A8E1", "Wacom Co.,Ltd." },
    { "68AAC4", "Altice Labs" },
    { "68AAD2", "DATECS LTD.," },
    { "68AB09", "Nokia" },
    { "68AB1E", "Apple, Inc." },
    { "68AB8A", "RF IDeas" },
    { "68ABBC", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "68AE04", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "68AE20", "Apple, Inc." },
    { "68AF13", "Futura Mobility" },
    { "68AFFF", "Shanghai Cambricon Information Technology Co., Ltd." },
    { "68B094", "INESA ELECTRON CO.,LTD" },
    { "68B1C9", "IYO, Inc" },
    { "68B35E", "Shenzhen Neostra Technology Co.Ltd" },
    { "68B41E", "ZEASN TECHNOLOGY PRIVATE LIMITED" },
    { "68B43A", "WaterFurnace International, Inc." },
    { "68B599", "Hewlett Packard" },
    { "68B691", "Amazon Technologies Inc." },
    { "68B6B3", "Espressif Inc." },
    { "68B6FC", "Hitron Technologies. Inc" },
    { "68B76B", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "68B8BB", "Beijing Xiaomi Electronics Co.,Ltd" },
    { "68B8D9", "Act KDE, Inc." },
    { "68B983", "b-plus GmbH" },
    { "68B9C2", "Earda Technologies co Ltd" },
    { "68B9D3", "Shenzhen Trolink Technology CO, LTD" },
    { "68BC0C", "Cisco Systems, Inc" },
    { "68BDAB", "Cisco Systems, Inc" },
    { "68BE49", "Nebula Matrix" },
    { "68BF6C", "Arista Networks" },
    { "68BFC4", "Samsung Electronics Co.,Ltd" },
    { "68C44C", "Xiaomi Communications Co Ltd" },
    { "68C44D", "Motorola Mobility LLC, a Lenovo Company" },
    { "68C63A", "Espressif Inc." },
    { "68C6AC", "Intel Corporate" },
    { "68C8EB", "Rockwell Automation" },
    { "68C90B", "Texas Instruments" },
    { "68C95D", "SZ Knowact Robot Technology Co., Ltd" },
    { "68CA00", "Octopus Systems Limited" },
    { "68CAC4", "Apple, Inc." },
    { "68CAE4", "Cisco Systems, Inc" },
    { "68CC6E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "68CC9C", "Mine Site Technologies" },
    { "68CCBA", "Dense Air Networks US LLC" },
    { "68CD0F", "U Tek Company Limited" },
    { "68CE4E", "L-3 Communications Infrared Products" },
    { "68D1BA", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "68D1FD", "Shenzhen Trimax Technology Co.,Ltd" },
    { "68D247", "Portalis LC" },
    { "68D40C", "TELLESCOM INDUSTRIA E COMERCIO EM TELECOMUNICACAO " },
    { "68D482", "SHENZHEN GONGJIN ELECTRONICS CO.,LT" },
    { "68D48B", "Hailo Technologies Ltd." },
    { "68D6ED", "GooWi Wireless Technology Co., Limited" },
    { "68D79A", "Ubiquiti Inc" },
    { "68D925", "ProSys Development Services" },
    { "68D927", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "68D93C", "Apple, Inc." },
    { "68D972", "Cisco Systems, Inc" },
    { "68D976", "EM Microelectronic" },
    { "68DA73", "IEEE Registration Authority" },
    { "68DB54", "Phicomm (Shanghai) Co., Ltd." },
    { "68DB67", "Nantong Coship Electronics Co., Ltd." },
    { "68DB96", "OPWILL Technologies CO .,LTD" },
    { "68DBCA", "Apple, Inc." },
    { "68DBF5", "Amazon Technologies Inc." },
    { "68DCE8", "PacketStorm Communications" },
    { "68DD26", "Shanghai Focus Vision Security Technology Co.,Ltd" },
    { "68DDB7", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "68DDD9", "HMD Global Oy" },
    { "68DECE", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "68DFDD", "Xiaomi Communications Co Ltd" },
    { "68DFE4", "Samsung Electronics Co.,Ltd" },
    { "68E154", "SiMa.ai" },
    { "68E166", "Private" },
    { "68E1DC", "BUFFALO.INC" },
    { "68E209", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "68E41F", "Unglaube Identech GmbH" },
    { "68E478", "Qingdao Haier Technology Co.,Ltd" },
    { "68E580", "Apple, Inc." },
    { "68E59E", "Cisco Systems, Inc" },
    { "68E74A", "Texas Instruments" },
    { "68E7C2", "Samsung Electronics Co.,Ltd" },
    { "68E8EB", "Linktel Technologies Co.,Ltd" },
    { "68E905", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "68EBAE", "Samsung Electronics Co.,Ltd" },
    { "68EBC5", "Angstrem Telecom" },
    { "68EC62", "YODO Technology Corp. Ltd." },
    { "68EC8A", "IKEA of Sweden AB" },
    { "68ECC5", "Intel Corporate" },
    { "68ED43", "BlackBerry RTS" },
    { "68ED57", "Juniper Networks" },
    { "68EDA4", "Shenzhen Seavo Technology Co.,Ltd" },
    { "68EE4B", "Sharetronic Data Technology Co.,Ltd" },
    { "68EE88", "Shenzhen TINNO Mobile Technology Corp." },
    { "68EE96", "Cisco SPVTG" },
    { "68EF43", "Apple, Inc." },
    { "68EFAB", "Vention" },
    { "68EFBD", "Cisco Systems, Inc" },
    { "68EFDC", "Apple, Inc." },
    { "68F06D", "ALONG INDUSTRIAL CO., LIMITED" },
    { "68F0B5", "Honor Device Co., Ltd." },
    { "68F0BC", "Shenzhen LiWiFi Technology Co., Ltd" },
    { "68F0D0", "SkyBell Technologies Inc." },
    { "68F125", "Data Controls Inc." },
    { "68F21F", "Bose Corporation" },
    { "68F38E", "Juniper Networks" },
    { "68F44B", "Home Control Singapore Pte Ltd" },
    { "68F543", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "68F62B", "ITEL MOBILE LIMITED" },
    { "68F63B", "Amazon Technologies Inc." },
    { "68F728", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "68F7D8", "Microsoft Corporation" },
    { "68F895", "Redflow Limited" },
    { "68F956", "Objetivos y Servicio de Valor Añadido" },
    { "68FB7E", "Apple, Inc." },
    { "68FB95", "Generalplus Technology Inc." },
    { "68FCB3", "Next Level Security Systems, Inc." },
    { "68FCB6", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "68FCCA", "Samsung Electronics Co.,Ltd" },
    { "68FE71", "Espressif Inc." },
    { "68FEDA", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "68FEF7", "Apple, Inc." },
    { "68FF7B", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "6C006B", "Samsung Electronics Co.,Ltd" },
    { "6C0273", "Shenzhen Jin Yun Video Equipment Co., Ltd." },
    { "6C02E0", "HP Inc." },
    { "6C0309", "Cisco Systems, Inc" },
    { "6C0370", "Extreme Networks Headquarters" },
    { "6C03B5", "Cisco Systems, Inc" },
    { "6C0460", "RBH Access Technologies Inc." },
    { "6C047A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6C05D3", "CHINA DRAGON TECHNOLOGY LIMITED" },
    { "6C05D5", "Ethertronics Inc" },
    { "6C06D6", "Huawei Device Co., Ltd." },
    { "6C0831", "ANALOG SYSTEMS" },
    { "6C090A", "GEMATICA SRL" },
    { "6C09BF", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "6C09D6", "Digiquest Electronics LTD" },
    { "6C0B5E", "HP Inc." },
    { "6C0B84", "Universal Global Scientific Industrial Co., Ltd." },
    { "6C0C9A", "Amazon Technologies Inc." },
    { "6C0D34", "Nokia" },
    { "6C0DC4", "Beijing Xiaomi Electronics Co., Ltd." },
    { "6C0DE1", "Dongguan Cannice Precision Manufacturing Co., Ltd." },
    { "6C0E0D", "Sony Corporation" },
    { "6C0EE6", "Chengdu Xiyida Electronic Technology Co,.Ltd" },
    { "6C0F0B", "China Mobile Group Device Co.,Ltd." },
    { "6C0F61", "Hypervolt Ltd" },
    { "6C0F6A", "JDC Tech Co., Ltd." },
    { "6C108B", "WeLink Communications" },
    { "6C11B3", "Wu Qi Technologies,Inc." },
    { "6C1270", "Apple, Inc." },
    { "6C13D5", "Cisco Systems, Inc" },
    { "6C1414", "BUJEON ELECTRONICS Co,.Ltd" },
    { "6C146E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6C14F7", "Erhardt+Leimer GmbH" },
    { "6C1524", "IEEE Registration Authority" },
    { "6C1544", "Microsoft Corporation" },
    { "6C15DB", "Arcadyan Corporation" },
    { "6C15F9", "Nautronix Limited" },
    { "6C160E", "ShotTracker" },
    { "6C1629", "Edifier International" },
    { "6C1632", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6C1811", "Decatur Electronics" },
    { "6C198F", "D-Link International" },
    { "6C19C0", "Apple, Inc." },
    { "6C1A75", "Huawei Device Co., Ltd." },
    { "6C1AEA", "Texas Instruments" },
    { "6C1B3F", "MiraeSignal Co., Ltd" },
    { "6C1C71", "Zhejiang Dahua Technology Co., Ltd." },
    { "6C1D2C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6C1DEB", "u-blox AG" },
    { "6C1E70", "Guangzhou YBDS IT Co.,Ltd" },
    { "6C1E90", "Hansol Technics Co., Ltd." },
    { "6C1ED7", "vivo Mobile Communication Co., Ltd." },
    { "6C1F8A", "Apple, Inc." },
    { "6C1FF7", "Ugreen Group Limited" },
    { "6C2056", "Cisco Systems, Inc" },
    { "6C21A2", "AMPAK Technology, Inc." },
    { "6C221A", "AltoBeam Inc." },
    { "6C22AB", "Ainsworth Game Technology" },
    { "6C22F7", "Nokia Shanghai Bell Co., Ltd." },
    { "6C2316", "TATUNG Technology Inc.," },
    { "6C23B9", "Sony Corporation" },
    { "6C23CB", "Wattty Corporation" },
    { "6C2408", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "6C2483", "Microsoft Mobile Oy" },
    { "6C24A6", "vivo Mobile Communication Co., Ltd." },
    { "6C25B9", "BBK EDUCATIONAL ELECTRONICS CORP.,LTD." },
    { "6C2636", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6C2779", "Microsoft Mobile Oy" },
    { "6C27C8", "IEEE Registration Authority" },
    { "6C2990", "WiZ Connected Lighting Company Limited" },
    { "6C2995", "Intel Corporate" },
    { "6C29D2", "Cisco Systems, Inc" },
    { "6C2ACB", "Paxton Access Ltd" },
    { "6C2ADF", "IEEE Registration Authority" },
    { "6C2B59", "Dell Inc." },
    { "6C2C06", "OOO NPP Systemotechnika-NN" },
    { "6C2CDC", "Skyworth Digital Technology(Shenzhen) Co.,Ltd" },
    { "6C2D24", "Zhen Shi Information Technology (Shanghai) Co., Ltd." },
    { "6C2E33", "Accelink Technologies Co.,Ltd." },
    { "6C2E72", "B&B EXPORTING LIMITED" },
    { "6C2E85", "Sagemcom Broadband SAS" },
    { "6C2F1C", "Nexus Raytek Pty Ltd" },
    { "6C2F2C", "Samsung Electronics Co.,Ltd" },
    { "6C2F80", "Intel Corporate" },
    { "6C2F8A", "Samsung Electronics Co.,Ltd" },
    { "6C302A", "Texas Instruments" },
    { "6C310E", "Cisco Systems, Inc" },
    { "6C32DE", "Indieon Technologies Pvt. Ltd." },
    { "6C33A9", "Magicjack LP" },
    { "6C3491", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6C3838", "Marking System Technology Co., Ltd." },
    { "6C3845", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "6C38A1", "Ubee Interactive Co., Limited" },
    { "6C391D", "Beijing ZhongHuaHun Network Information center" },
    { "6C3A36", "Glowforge Inc" },
    { "6C3A84", "Shenzhen Aero-Startech. Co.Ltd" },
    { "6C3AFF", "Apple, Inc." },
    { "6C3B6B", "Routerboard.com" },
    { "6C3BE5", "Hewlett Packard" },
    { "6C3C53", "SoundHawk Corp" },
    { "6C3C7C", "CANON INC." },
    { "6C3C8C", "Dell Inc." },
    { "6C3E6D", "Apple, Inc." },
    { "6C3E9C", "KE Knestel Elektronik GmbH" },
    { "6C4008", "Apple, Inc." },
    { "6C40C6", "Nimbus Data, Inc." },
    { "6C40E8", "vivo Mobile Communication Co., Ltd." },
    { "6C410E", "Cisco Systems, Inc" },
    { "6C416A", "Cisco Systems, Inc" },
    { "6C41DE", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6C42AB", "Subscriber Networks, Inc." },
    { "6C433C", "TECNO MOBILE LIMITED" },
    { "6C4418", "Zappware" },
    { "6C442A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6C4598", "Antex Electronic Corp." },
    { "6C45C4", "Cloudflare, Inc." },
    { "6C4760", "Sunitec Enterprise Co.,Ltd" },
    { "6C483F", "Xiaomi Communications Co Ltd" },
    { "6C48A6", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "6C49C1", "o2ones Co., Ltd." },
    { "6C4A39", "BITA" },
    { "6C4A74", "AERODISK LLC" },
    { "6C4A85", "Apple, Inc." },
    { "6C4B7F", "Vossloh-Schwabe Deutschland GmbH" },
    { "6C4B90", "LiteON" },
    { "6C4BB4", "HUMAX Co., Ltd." },
    { "6C4CE2", "Intel Corporate" },
    { "6C4D51", "Shenzhen Ceres Technology Co., Ltd." },
    { "6C4D73", "Apple, Inc." },
    { "6C4E86", "Third Millennium Systems Ltd." },
    { "6C4EF6", "Cisco Systems, Inc" },
    { "6C4F89", "Zyxel Communications Corporation" },
    { "6C4FA1", "Cisco Systems, Inc" },
    { "6C504D", "Cisco Systems, Inc" },
    { "6C51BF", "Huawei Device Co., Ltd." },
    { "6C51E4", "Huawei Device Co., Ltd." },
    { "6C54CD", "LAMPEX ELECTRONICS LIMITED" },
    { "6C5563", "Samsung Electronics Co.,Ltd" },
    { "6C558D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6C55B1", "Amazon Technologies Inc." },
    { "6C55E8", "Vantiva USA LLC" },
    { "6C5640", "BLU Products Inc" },
    { "6C5697", "Amazon Technologies Inc." },
    { "6C5779", "Aclima, Inc." },
    { "6C5940", "MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "6C5976", "Shanghai Tricheer Technology Co.,Ltd." },
    { "6C5A34", "Shenzhen Haitianxiong Electronic Co., Ltd." },
    { "6C5AB0", "TP-Link Systems Inc" },
    { "6C5AB5", "TCL Technoly Electronics (Huizhou) Co., Ltd." },
    { "6C5C14", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "6C5C3D", "IEEE Registration Authority" },
    { "6C5CB1", "Silicon Laboratories" },
    { "6C5CDE", "SunReports, Inc." },
    { "6C5D3A", "Microsoft Corporation" },
    { "6C5D63", "ShenZhen Rapoo Technology Co., Ltd." },
    { "6C5E3B", "Cisco Systems, Inc" },
    { "6C5E7A", "Ubiquitous Internet Telecom Co., Ltd" },
    { "6C5F1C", "Lenovo Mobile Communication Technology Ltd." },
    { "6C6016", "OnePlus Technology (Shenzhen) Co., Ltd" },
    { "6C60D0", "Huawei Device Co., Ltd." },
    { "6C60EB", "ZHI YUAN ELECTRONICS CO., LIMITED" },
    { "6C6126", "Rinicom Holdings" },
    { "6C61F4", "SFR" },
    { "6C626D", "Micro-Star INT'L CO., LTD" },
    { "6C6286", "Nokia" },
    { "6C62FE", "Juniper Networks" },
    { "6C639C", "Commscope" },
    { "6C63F8", "Ubiquiti Inc" },
    { "6C641A", "Penguin Computing" },
    { "6C6567", "BELIMO Automation AG" },
    { "6C67EF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6C68A4", "Guangzhou V-Solution Telecommunication Technology Co.,Ltd." },
    { "6C6A77", "Intel Corporate" },
    { "6C6C0F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6C6CD3", "Cisco Systems, Inc" },
    { "6C6D09", "Kyowa Electronics Co.,Ltd." },
    { "6C6E07", "CE LINK LIMITED" },
    { "6C6EFE", "Core Logic Inc." },
    { "6C6F18", "Stereotaxis, Inc." },
    { "6C7039", "Novar GmbH" },
    { "6C709F", "Apple, Inc." },
    { "6C70CB", "Samsung Electronics Co.,Ltd" },
    { "6C710D", "Cisco Systems, Inc" },
    { "6C71BD", "EZELINK TELECOM" },
    { "6C71D2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6C71D9", "AzureWave Technology Inc." },
    { "6C7220", "D-Link International" },
    { "6C724A", "Onkyo Technology K.K." },
    { "6C72E2", "amitek" },
    { "6C72E7", "Apple, Inc." },
    { "6C750D", "WiFiSONG" },
    { "6C7637", "Huawei Device Co., Ltd." },
    { "6C7660", "KYOCERA CORPORATION " },
    { "6C78C1", "Juniper Networks" },
    { "6C79B8", "Texas Instruments" },
    { "6C7DB7", "Cisco Meraki" },
    { "6C7E67", "Apple, Inc." },
    { "6C7F0C", "Cisco Meraki" },
    { "6C80AB", "ifanr Inc" },
    { "6C81FE", "Mitsuba Corporation" },
    { "6C8243", "Huawei Device Co., Ltd." },
    { "6C8336", "Samsung Electronics Co.,Ltd" },
    { "6C8338", "Ubihere" },
    { "6C8366", "Nanjing SAC Power Grid Automation Co., Ltd." },
    { "6C8375", "Broadcom Limited" },
    { "6C8686", "Technonia" },
    { "6C8720", "New H3C Technologies Co., Ltd" },
    { "6C8814", "Intel Corporate" },
    { "6C8AEC", "Nantong Coship Electronics Co., Ltd." },
    { "6C8B2F", "zte corporation" },
    { "6C8BD3", "Cisco Systems, Inc" },
    { "6C8CDB", "Otus Technologies Ltd" },
    { "6C8D65", "Wireless Glue Networks, Inc." },
    { "6C8D77", "Cisco Systems, Inc" },
    { "6C8DC1", "Apple, Inc." },
    { "6C8F4E", "Chongqing Yipingfang Technology Co., Ltd." },
    { "6C8FB5", "Microsoft Mobile Oy" },
    { "6C90B1", "SanLogic Inc" },
    { "6C9106", "Katena Computing Technologies" },
    { "6C92BF", "IEIT SYSTEMS Co., Ltd." },
    { "6C92CF", "Broadcom Limited" },
    { "6C9308", "IEEE Registration Authority" },
    { "6C9354", "Yaojin Technology (Shenzhen) Co., LTD." },
    { "6C9392", "BEKO Technologies GmbH" },
    { "6C9466", "Intel Corporate" },
    { "6C94F8", "Apple, Inc." },
    { "6C9522", "Scalys" },
    { "6C96CF", "Apple, Inc." },
    { "6C976D", "Motorola Mobility LLC, a Lenovo Company" },
    { "6C97AA", "AI TECHNOLOGY CO.,LTD." },
    { "6C98EB", "Riverbed Technology, Inc." },
    { "6C9961", "Sagemcom Broadband SAS" },
    { "6C9989", "Cisco Systems, Inc" },
    { "6C999D", "Amazon Technologies Inc." },
    { "6C9AB4", "Brodersen A/S" },
    { "6C9AC9", "Valentine Research, Inc." },
    { "6C9B02", "Nokia Corporation" },
    { "6C9BC0", "Chemoptics Inc." },
    { "6C9CE9", "Nimble Storage" },
    { "6C9CED", "Cisco Systems, Inc" },
    { "6C9E7C", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "6CA0B4", "SKY UK LIMITED" },
    { "6CA100", "Intel Corporate" },
    { "6CA31E", "ITEL MOBILE LIMITED" },
    { "6CA367", "Avlinkpro" },
    { "6CA401", "essensys plc" },
    { "6CA4D1", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "6CA604", "Commscope" },
    { "6CA682", "EDAM information & communications" },
    { "6CA75F", "zte corporation" },
    { "6CA780", "Nokia Corporation" },
    { "6CA7FA", "YOUNGBO ENGINEERING INC." },
    { "6CA849", "Avaya Inc" },
    { "6CA858", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "6CA906", "Telefield Ltd" },
    { "6CA928", "HMD Global Oy" },
    { "6CA936", "DisplayLink (UK) Ltd" },
    { "6CA96F", "TransPacket AS" },
    { "6CAAB3", "Ruckus Wireless" },
    { "6CAB05", "Cisco Systems, Inc" },
    { "6CAB31", "Apple, Inc." },
    { "6CAB4D", "Digital Payment Technologies" },
    { "6CAC60", "Venetex Corp" },
    { "6CACC2", "Samsung Electronics Co.,Ltd" },
    { "6CAD3F", "Hubbell Building Automation, Inc." },
    { "6CADAD", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "6CADEF", "KZ Broadband Technologies, Ltd. " },
    { "6CADF8", "AzureWave Technology Inc." },
    { "6CAE8B", "IBM Corporation" },
    { "6CAEE3", "Nokia" },
    { "6CAEF6", "eero inc." },
    { "6CAF15", "Webasto SE" },
    { "6CB077", "ALL Winner (Hong Kong) Limited" },
    { "6CB0CE", "NETGEAR" },
    { "6CB0FD", "Shenzhen Xinghai Iot Technology Co.,Ltd" },
    { "6CB133", "Apple, Inc." },
    { "6CB158", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "6CB227", "Sony Video & Sound Products Inc." },
    { "6CB2AE", "Cisco Systems, Inc" },
    { "6CB2FD", "Texas Instruments" },
    { "6CB311", "Shenzhen Lianrui Electronics Co.,Ltd" },
    { "6CB34D", "SharkNinja Operating LLC" },
    { "6CB350", "Anhui comhigher tech co.,ltd" },
    { "6CB456", "Espressif Inc." },
    { "6CB4A7", "Landauer, Inc." },
    { "6CB4FD", "Huawei Device Co., Ltd." },
    { "6CB56B", "HUMAX Co., Ltd." },
    { "6CB6CA", "DIVUS GmbH" },
    { "6CB749", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6CB7E2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6CB7F4", "Samsung Electronics Co.,Ltd" },
    { "6CB881", "zte corporation" },
    { "6CB9C5", "Delta Networks, Inc." },
    { "6CBAB8", "Sagemcom Broadband SAS" },
    { "6CBEE9", "Alcatel-Lucent IPD" },
    { "6CBFB5", "Noon Technology Co., Ltd" },
    { "6CC147", "Xiamen Hanin Electronic Technology Co., Ltd" },
    { "6CC1D2", "Commscope" },
    { "6CC217", "Hewlett Packard" },
    { "6CC242", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "6CC26B", "Apple, Inc." },
    { "6CC36A", "vivo Mobile Communication Co., Ltd." },
    { "6CC374", "Texas Instruments" },
    { "6CC3B2", "Cisco Meraki" },
    { "6CC41E", "NEXSEC Incorporated" },
    { "6CC49F", "Hewlett Packard Enterprise" },
    { "6CC4D5", "HMD Global Oy" },
    { "6CC63B", "Taicang T&W Electronics" },
    { "6CC7EC", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "6CC840", "Espressif Inc." },
    { "6CCA08", "Commscope" },
    { "6CCDD6", "NETGEAR" },
    { "6CCE44", "1MORE" },
    { "6CCF39", "Shanghai StarFive Semiconductor Co., Ltd." },
    { "6CD008", "zte corporation" },
    { "6CD032", "LG Electronics" },
    { "6CD146", "FRAMOS GmbH" },
    { "6CD199", "vivo Mobile Communication Co., Ltd." },
    { "6CD1B0", "WING SING ELECTRONICS HONG KONG LIMITED" },
    { "6CD1E5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6CD2BA", "zte corporation" },
    { "6CD3EE", "ZIMI CORPORATION" },
    { "6CD630", "Rootous System Co.,Ltd" },
    { "6CD63F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6CD68A", "LG Electronics (Mobile Communications)" },
    { "6CD6E3", "Cisco Systems, Inc" },
    { "6CD704", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6CD719", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "6CD71F", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "6CD7A0", "WIKO Terminal Technology (Dongguan) Co., Ltd." },
    { "6CD869", "Guangzhou Sat Infrared Co.,LTD" },
    { "6CD94C", "vivo Mobile Communication Co., Ltd." },
    { "6CDC6A", "Promethean Limited" },
    { "6CDD30", "Cisco Systems, Inc" },
    { "6CDDBC", "Samsung Electronics Co.,Ltd" },
    { "6CDDEF", "EPCOMM Inc." },
    { "6CDEA9", "Cisco Meraki" },
    { "6CDFFB", "IEEE Registration Authority" },
    { "6CE01E", "Modcam AB" },
    { "6CE0B0", "SOUND4" },
    { "6CE2D3", "New H3C Technologies Co., Ltd" },
    { "6CE3B6", "Nera Telecommunications Ltd." },
    { "6CE4CE", "Villiger Security Solutions AG" },
    { "6CE4DA", "NEC Platforms, Ltd." },
    { "6CE5C9", "Apple, Inc." },
    { "6CE5F7", "New H3C Technologies Co., Ltd" },
    { "6CE85C", "Apple, Inc." },
    { "6CE873", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "6CE874", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6CE8C6", "Earda Technologies co Ltd" },
    { "6CE907", "Nokia Corporation" },
    { "6CE983", "Gastron Co., LTD." },
    { "6CEBB2", "Dongguan Sen DongLv Electronics Co.,Ltd" },
    { "6CEBB6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "6CEC5A", "Hon Hai Precision Ind. CO.,Ltd." },
    { "6CECA1", "SHENZHEN CLOU ELECTRONICS CO. LTD." },
    { "6CECEB", "Texas Instruments" },
    { "6CED51", "NEXCONTROL Co.,Ltd" },
    { "6CEEF7", "shenzhen scodeno technology co., Ltd." },
    { "6CEFBD", "Cisco Meraki" },
    { "6CEFC6", "SHENZHEN TWOWING TECHNOLOGIES CO.,LTD." },
    { "6CF049", "GIGA-BYTE TECHNOLOGY CO.,LTD." },
    { "6CF17E", "Zhejiang Uniview Technologies Co.,Ltd." },
    { "6CF2D8", "CANON INC." },
    { "6CF373", "Samsung Electronics Co.,Ltd" },
    { "6CF37F", "Hewlett Packard Enterprise" },
    { "6CF5E8", "Mooredoll Inc." },
    { "6CF6DA", "Intel Corporate" },
    { "6CF712", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "6CF784", "Xiaomi Communications Co Ltd" },
    { "6CF97C", "Nanoptix Inc." },
    { "6CF9D2", "CHENGDU POVODO ELECTRONIC TECHNOLOGY CO., LTD" },
    { "6CFA58", "Avaya Inc" },
    { "6CFA89", "Cisco Systems, Inc" },
    { "6CFAA7", "AMPAK Technology, Inc." },
    { "6CFBED", "GN Audio A/S" },
    { "6CFD22", "Silicon Laboratories" },
    { "6CFDB9", "Proware Technologies Co Ltd." },
    { "6CFE54", "Intel Corporate" },
    { "6CFFBE", "MPB Communications Inc." },
    { "6CFFCE", "Sagemcom Broadband SAS" },
    { "700136", "FATEK Automation Corporation" },
    { "7001B5", "Cisco Systems, Inc" },
    { "700258", "01DB-METRAVIB" },
    { "70033F", "Pimax Technology(ShangHai)Co.,Ltd" },
    { "70037E", "Vantiva USA LLC" },
    { "70039F", "Espressif Inc." },
    { "70041D", "Espressif Inc." },
    { "700433", "California Things Inc." },
    { "700514", "LG Electronics (Mobile Communications)" },
    { "700692", "IEEE Registration Authority" },
    { "7006AC", "Eastcompeace Technology Co., Ltd" },
    { "700777", "OnTarget Technologies, Inc" },
    { "700810", "Intel Corporate" },
    { "700894", "Liteon Technology Corporation" },
    { "700971", "Samsung Electronics Co.,Ltd" },
    { "700B01", "Sagemcom Broadband SAS" },
    { "700B4F", "Cisco Systems, Inc" },
    { "700BC0", "Dewav Technology Company" },
    { "700F6A", "Cisco Systems, Inc" },
    { "700FC7", "SHENZHEN IKINLOOP TECHNOLOGY CO.,LTD." },
    { "700FEC", "Poindus Systems Corp." },
    { "70105C", "Cisco Systems, Inc" },
    { "70106F", "Hewlett Packard Enterprise " },
    { "70110E", "zte corporation" },
    { "701124", "Apple, Inc." },
    { "701135", "Livesecu co., Ltd" },
    { "7011AE", "Music Life LTD" },
    { "701301", "Vantiva - Connected Home" },
    { "701384", "Apple, Inc." },
    { "701404", "Limited Liability Company" },
    { "7014A6", "Apple, Inc." },
    { "7015FB", "Intel Corporate" },
    { "70169F", "EtherCAT Technology Group" },
    { "7017D7", "Shanghai Enflame Technology Co., Ltd." },
    { "70188B", "Hon Hai Precision Ind. Co.,Ltd." },
    { "7018A7", "Cisco Systems, Inc" },
    { "70192F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "701988", "Nanjing Qinheng Microelectronics Co., Ltd." },
    { "701A04", "Liteon Technology Corporation" },
    { "701AB8", "Intel Corporate" },
    { "701AD5", "Avigilon Alta" },
    { "701AED", "ADVAS CO., LTD." },
    { "701BFB", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "701CE7", "Intel Corporate" },
    { "701D08", "99IOT Shenzhen co.,ltd" },
    { "701D7F", "Comtech Technology Co., Ltd." },
    { "701DC4", "NorthStar Battery Company, LLC" },
    { "701E68", "Hanna Instruments, Inc." },
    { "701F0B", "WILOGY SRL" },
    { "701F3C", "Samsung Electronics Co.,Ltd" },
    { "701F53", "Cisco Systems, Inc" },
    { "702084", "Hon Hai Precision Industry Co., Ltd." },
    { "70217F", "Xiaomi Communications Co Ltd" },
    { "7022FE", "Apple, Inc." },
    { "702393", "fos4X GmbH" },
    { "702526", "Nokia" },
    { "702559", "CyberTAN Technology Inc." },
    { "702605", "SONY Visual Products Inc." },
    { "702804", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "70288B", "Samsung Electronics Co.,Ltd" },
    { "702900", "Shenzhen ChipTrip Technology Co,Ltd" },
    { "702A7D", "EpSpot AB" },
    { "702AD5", "Samsung Electronics Co.,Ltd" },
    { "702AD7", "New H3C Intelligence Terminal Co., Ltd." },
    { "702B1D", "E-Domus International Limited" },
    { "702C09", "Nintendo Co.,Ltd" },
    { "702C1F", "Wisol" },
    { "702D84", "i4C Innovations" },
    { "702DD1", "Newings Communication CO., LTD." },
    { "702E22", "zte corporation" },
    { "702E80", "DIEHL Connectivity Solutions" },
    { "702ED9", "Guangzhou Shiyuan Electronics Co., Ltd." },
    { "702F35", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "702F4B", "Steelcase Inc." },
    { "702F86", "Marquardt GmbH" },
    { "702F97", "Aava Mobile Oy" },
    { "703018", "Avaya Inc" },
    { "70305D", "Ubiquoss Inc" },
    { "70305E", "Nanjing Zhongke Menglian Information Technology Co.,LTD" },
    { "70317F", "Apple, Inc." },
    { "703187", "ACX GmbH" },
    { "703217", "Intel Corporate" },
    { "7032D5", "Athena Wireless Communications Inc" },
    { "703509", "Cisco Systems, Inc" },
    { "70357B", "vivo Mobile Communication Co., Ltd." },
    { "7036B2", "Focusai Corp" },
    { "70378E", "Tianyi Telecom Terminals Company Limited" },
    { "703811", "Siemens Mobility Limited" },
    { "7038B4", "Low Tech Solutions" },
    { "7038EE", "Avaya Inc" },
    { "703A0E", "Hewlett Packard Enterprise" },
    { "703A2D", "Shenzhen V-Link Technology CO., LTD." },
    { "703A51", "Xiaomi Communications Co Ltd" },
    { "703A73", "Shenzhen Sundray Technologies company Limited" },
    { "703AA6", "New H3C Technologies Co., Ltd" },
    { "703ACB", "Google, Inc." },
    { "703AD8", "Shenzhen Afoundry Electronic Co., Ltd" },
    { "703C03", "RadiAnt Co.,Ltd" },
    { "703C39", "SEAWING Kft" },
    { "703C69", "Apple, Inc." },
    { "703D15", "Hangzhou H3C Technologies Co., Limited" },
    { "703E97", "Iton Technology Corp." },
    { "703EAC", "Apple, Inc." },
    { "7040FF", "Huawei Device Co., Ltd." },
    { "7041B7", "Edwards Lifesciences LLC" },
    { "7042D3", "Ruijie Networks Co.,LTD" },
    { "70441C", "SHENZHEN KAIFA TECHNOLOGY CO.,LTD." },
    { "704642", "CHYNG HONG ELECTRONIC CO., LTD." },
    { "704698", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "704777", "Ruckus Wireless" },
    { "7047E9", "vivo Mobile Communication Co., Ltd." },
    { "70480F", "Apple, Inc." },
    { "7048B5", "CTS System Co., LTD." },
    { "7048F7", "Nintendo Co.,Ltd" },
    { "7049A2", "Zyxel Communications Corporation" },
    { "704A0E", "AMPAK Technology,Inc." },
    { "704AAE", "Xstream Flow (Pty) Ltd" },
    { "704AE4", "Rinstrum Pty Ltd" },
    { "704CA2", "Apple, Inc." },
    { "704CA5", "Fortinet, Inc." },
    { "704CB6", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "704CED", "TMRG, Inc." },
    { "704D7B", "ASUSTek COMPUTER INC." },
    { "704DE7", "TECNO MOBILE LIMITED" },
    { "704E01", "KWANGWON TECH CO., LTD." },
    { "704E66", "SHENZHEN FAST TECHNOLOGIES CO.,LTD" },
    { "704E6B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "704F08", "Shenzhen Huisheng Information Technology Co., Ltd." },
    { "704F57", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "704FB8", "Commscope" },
    { "7050AF", "SKY UK LIMITED" },
    { "7050E7", "IEEE Registration Authority" },
    { "7052C5", "Avaya Inc" },
    { "7052D8", "ITEL MOBILE LIMITED" },
    { "70533F", "Alfa Instrumentos Eletronicos Ltda." },
    { "705425", "Commscope" },
    { "705464", "Silicon Laboratories" },
    { "7054B4", "Vestel Elektronik San ve Tic. A.S." },
    { "7054D2", "PEGATRON CORPORATION" },
    { "7054F5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7055F8", "Cerebras Systems Inc" },
    { "705681", "Apple, Inc." },
    { "7057BF", "New H3C Technologies Co., Ltd" },
    { "705812", "Panasonic Corporation AVC Networks Company" },
    { "705846", "Trig Avionics Limited" },
    { "705896", "InShow Technology" },
    { "7058A4", "Actiontec Electronics Inc." },
    { "705957", "Medallion Instrumentation Systems" },
    { "705986", "OOO TTV" },
    { "705A0F", "Hewlett Packard" },
    { "705A6F", "IEEE Registration Authority" },
    { "705A9E", "Vantiva USA LLC" },
    { "705AAC", "Samsung Electronics Co.,Ltd" },
    { "705AB6", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "705B2E", "M2Communication Inc." },
    { "705CAD", "Konami Gaming Inc" },
    { "705DCC", "EFM Networks" },
    { "705E55", "Realme Chongqing MobileTelecommunications Corp Ltd" },
    { "705EAA", "Action Target, Inc." },
    { "705FA3", "Xiaomi Communications Co Ltd" },
    { "7060DE", "LaVision GmbH" },
    { "706173", "Calantec GmbH" },
    { "70617B", "Cisco Systems, Inc" },
    { "7061BE", "Wistron Neweb Corporation" },
    { "7061EE", "Sunwoda Electronic Co.,Ltd" },
    { "7062B8", "D-Link International" },
    { "706417", "ORBIS TECNOLOGIA ELECTRICA S.A." },
    { "706582", "Suzhou Hanming Technologies Co., Ltd." },
    { "7065A3", "Kandao lightforge Co., Ltd." },
    { "70661B", "Sonova AG" },
    { "70662A", "Sony Interactive Entertainment Inc." },
    { "706655", "AzureWave Technology Inc." },
    { "7066B9", "Huawei Device Co., Ltd." },
    { "7066E1", "dnt Innovation GmbH" },
    { "706871", "FN-LINK TECHNOLOGY Ltd." },
    { "706879", "Saijo Denki International Co., Ltd." },
    { "70695A", "Cisco Systems, Inc" },
    { "706979", "IEEE Registration Authority" },
    { "706AC9", "zte corporation" },
    { "706BB9", "Cisco Systems, Inc" },
    { "706D15", "Cisco Systems, Inc" },
    { "706DEC", "Wifi-soft LLC" },
    { "706E10", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "706E6D", "Cisco Systems, Inc" },
    { "706F81", "Private" },
    { "70700D", "Apple, Inc." },
    { "707013", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "70704C", "Purple Communications, Inc" },
    { "70708B", "Cisco Systems, Inc" },
    { "7070AA", "Amazon Technologies Inc." },
    { "7070FC", "GOLD&WATER INDUSTRIAL LIMITED" },
    { "7071B3", "Brain Corporation" },
    { "7071BC", "PEGATRON CORPORATION" },
    { "70720D", "Lenovo Mobile Communication Technology Ltd." },
    { "70723C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7072CF", "EdgeCore Networks" },
    { "7072FE", "Apple, Inc." },
    { "707362", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7073CB", "Apple, Inc." },
    { "707414", "Murata Manufacturing Co., Ltd." },
    { "707630", "Commscope" },
    { "7076DD", "OxyGuard Internation A/S" },
    { "7076F0", "LevelOne Communications (India) Private Limited" },
    { "7076FF", "KERLINK" },
    { "707781", "Hon Hai Precision Ind. Co.,Ltd." },
    { "70788B", "vivo Mobile Communication Co., Ltd." },
    { "707938", "Wuxi Zhanrui Electronic Technology Co.,LTD" },
    { "707990", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7079B3", "Cisco Systems, Inc" },
    { "707BE8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "707C18", "ADATA Technology Co., Ltd" },
    { "707C69", "Avaya Inc" },
    { "707CE3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "707D95", "Shenzhen City LinwlanTechnology Co. Ltd." },
    { "707DA1", "Sagemcom Broadband SAS" },
    { "707DAF", "Plucent AB" },
    { "707DB9", "Cisco Systems, Inc" },
    { "707E43", "Commscope" },
    { "707EDE", "NASTEC LTD." },
    { "707FF2", "Telechips, Inc." },
    { "708105", "Cisco Systems, Inc" },
    { "708185", "New H3C Technologies Co., Ltd" },
    { "7081EB", "Apple, Inc." },
    { "70820E", "as electronics GmbH" },
    { "70828E", "OleumTech Corporation" },
    { "708540", "Skyworth Digital Technology(Shenzhen) Co.,Ltd" },
    { "70856C", "Ruijie Networks Co.,LTD" },
    { "7085C2", "ASRock Incorporation" },
    { "7085C4", "Ruijie Networks Co.,LTD" },
    { "7085C6", "Commscope" },
    { "7086C1", "Texas Instruments" },
    { "7086CE", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "70879E", "Beken Corporation" },
    { "7087A7", "Murata Manufacturing Co., Ltd." },
    { "70884D", "JAPAN RADIO CO., LTD." },
    { "708976", "Tuya Smart Inc." },
    { "7089CC", "China Mobile Group Device Co.,Ltd." },
    { "7089F5", "Dongguan Lingjie IOT Co., LTD" },
    { "708A09", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "708B78", "citygrow technology co., ltd" },
    { "708B97", "INSYS icom GmbH" },
    { "708BCD", "ASUSTek COMPUTER INC." },
    { "708CB6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "708CBB", "MIMODISPLAYKOREA" },
    { "708CF2", "Apple, Inc." },
    { "708D09", "Nokia Corporation" },
    { "708F47", "vivo Mobile Communication Co., Ltd." },
    { "709041", "Mist Systems, Inc." },
    { "7090B7", "Huawei Device Co., Ltd." },
    { "70918F", "Weber-Stephen Products LLC" },
    { "7091F3", "Universal Electronics, Inc." },
    { "709383", "Intelligent Optical Network High Tech CO.,LTD." },
    { "7093C1", "eero inc." },
    { "7093F8", "Space Monkey, Inc." },
    { "709741", "Arcadyan Corporation" },
    { "709756", "Happyelectronics Co.,Ltd" },
    { "709883", "SHENZHEN KAYAN ELECTRONICS., LTD." },
    { "70991C", "Shenzhen Honesty Electronics Co.,Ltd" },
    { "709A0B", "Italian Institute of Technology" },
    { "709AC4", "Huawei Device Co., Ltd." },
    { "709BA5", "Shenzhen Y&D Electronics Co.,LTD." },
    { "709BFC", "Bryton Inc." },
    { "709C45", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "709C8F", "Nero AG" },
    { "709CD1", "Intel Corporate" },
    { "709E29", "Sony Interactive Entertainment Inc." },
    { "709E86", "X6D Limited" },
    { "709F2D", "zte corporation" },
    { "709FA9", "TECNO MOBILE LIMITED" },
    { "70A191", "Trendsetter Medical, LLC" },
    { "70A2B3", "Apple, Inc." },
    { "70A41C", "Advanced Wireless Dynamics S.L." },
    { "70A56A", "Shenzhen C-Data Technology Co., Ltd." },
    { "70A66A", "Prox Dynamics AS" },
    { "70A6BD", "Honor Device Co., Ltd." },
    { "70A6CC", "Intel Corporate" },
    { "70A703", "Honor Device Co., Ltd." },
    { "70A741", "Ubiquiti Inc" },
    { "70A84C", "MONAD., Inc." },
    { "70A8D3", "Intel Corporate" },
    { "70A8E3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "70A983", "Cisco Systems, Inc" },
    { "70AAB2", "BlackBerry RTS" },
    { "70AC08", "Silicon Laboratories" },
    { "70ACD7", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "70AD54", "Malvern Instruments Ltd" },
    { "70AED5", "Apple, Inc." },
    { "70AF24", "TP Vision Belgium NV" },
    { "70AF25", "Nishiyama Industry Co.,LTD." },
    { "70AF6A", "SHENZHEN FENGLIAN TECHNOLOGY CO., LTD." },
    { "70B035", "Shenzhen Zowee Technology Co., Ltd" },
    { "70B08C", "Shenou Communication Equipment Co.,Ltd" },
    { "70B13D", "Samsung Electronics Co.,Ltd" },
    { "70B14E", "Commscope" },
    { "70B258", "Ruckus Wireless" },
    { "70B265", "Hiltron s.r.l." },
    { "70B306", "Apple, Inc." },
    { "70B317", "Cisco Systems, Inc" },
    { "70B3D5", "IEEE Registration Authority" },
    { "70B51A", "Huawei Device Co., Ltd." },
    { "70B599", "Embedded Technologies s.r.o." },
    { "70B5E8", "Dell Inc." },
    { "70B64F", "Guangzhou V-SOLUTION Electronic Technology Co., Ltd." },
    { "70B651", "Eight Sleep" },
    { "70B7AA", "vivo Mobile Communication Co., Ltd." },
    { "70B7E2", "Jiangsu Miter Technology Co.,Ltd." },
    { "70B7E4", "Broadcom Limited" },
    { "70B8F6", "Espressif Inc." },
    { "70B921", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "70B950", "Texas Instruments" },
    { "70B9BB", "Shenzhen Hankvision Technology CO.,LTD" },
    { "70BAEF", "Hangzhou H3C Technologies Co., Limited" },
    { "70BB5B", "Apple, Inc." },
    { "70BBE9", "Xiaomi Communications Co Ltd" },
    { "70BC10", "Microsoft Corporation" },
    { "70BC48", "Cisco Systems, Inc" },
    { "70BD96", "Cisco Systems, Inc" },
    { "70BDD2", "Adva Network Security GmbH" },
    { "70BF3E", "Charles River Laboratories" },
    { "70BF92", "GN Audio A/S" },
    { "70C59C", "Silicon Laboratories" },
    { "70C6AC", "Bosch Automotive Aftermarket" },
    { "70C6DD", "New H3C Technologies Co., Ltd" },
    { "70C76F", "INNO S" },
    { "70C7F2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "70C833", "Wirepas Oy" },
    { "70C912", "Sichuan AI-Link Technology Co., Ltd." },
    { "70C932", "Dreame Technology (Suzhou) Limited" },
    { "70C94E", "Liteon Technology Corporation" },
    { "70C9C6", "Cisco Systems, Inc" },
    { "70CA4D", "Shenzhen lnovance Technology Co.,Ltd." },
    { "70CA97", "Ruckus Wireless" },
    { "70CA9B", "Cisco Systems, Inc" },
    { "70CD0D", "Intel Corporate" },
    { "70CD60", "Apple, Inc." },
    { "70CD91", "TERACOM TELEMATICA S.A" },
    { "70CE8C", "Samsung Electronics Co.,Ltd" },
    { "70CF49", "Intel Corporate" },
    { "70D081", "Beijing Netpower Technologies Inc." },
    { "70D313", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "70D379", "Cisco Systems, Inc" },
    { "70D4F2", "RIM" },
    { "70D51E", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "70D57E", "Scalar Corporation" },
    { "70D5E7", "Wellcore Corporation" },
    { "70D6B6", "Metrum Technologies" },
    { "70D823", "Intel Corporate" },
    { "70D880", "Upos System sp. z o.o." },
    { "70D8C2", "Intel Corporate" },
    { "70D923", "vivo Mobile Communication Co., Ltd." },
    { "70D931", "Cambridge Industries(Group) Co.,Ltd." },
    { "70D983", "Shanghai JINXVM Microelectronics Co.,Ltd." },
    { "70DA17", "Austrian Audio GmbH" },
    { "70DA48", "Cisco Systems, Inc" },
    { "70DA9C", "TECSEN" },
    { "70DB98", "Cisco Systems, Inc" },
    { "70DDA1", "Tellabs" },
    { "70DDA8", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "70DDEF", "Huawei Device Co., Ltd." },
    { "70DEE2", "Apple, Inc." },
    { "70DEF9", "FAI WAH INTERNATIONAL (HONG KONG) LIMITED" },
    { "70DF2F", "Cisco Systems, Inc" },
    { "70DFF7", "Commscope" },
    { "70E027", "HONGYU COMMUNICATION TECHNOLOGY LIMITED" },
    { "70E139", "3view Ltd" },
    { "70E1FD", "FLEXTRONICS" },
    { "70E24C", "SAE IT-systems GmbH & Co. KG" },
    { "70E284", "Wistron Infocomm (Zhongshan) Corporation" },
    { "70E422", "Cisco Systems, Inc" },
    { "70E46E", "Lytx" },
    { "70E56E", "Texas Instruments" },
    { "70E72C", "Apple, Inc." },
    { "70E843", "Beijing C&W Optical Communication Technology Co.,Ltd." },
    { "70EA1A", "Cisco Systems, Inc" },
    { "70EA5A", "Apple, Inc." },
    { "70EB74", "Ningbo Goneo Electric Appliance Co., Ltd." },
    { "70ECE4", "Apple, Inc." },
    { "70EDFA", "imperix Ltd" },
    { "70EE50", "Netatmo" },
    { "70EEA3", "Eoptolink Technology Inc. Ltd," },
    { "70EF00", "Apple, Inc." },
    { "70F087", "Apple, Inc." },
    { "70F088", "Nintendo Co.,Ltd" },
    { "70F096", "Cisco Systems, Inc" },
    { "70F11C", "Shenzhen Ogemray Technology Co.,Ltd" },
    { "70F176", "Data Modul AG" },
    { "70F196", "Actiontec Electronics, Inc" },
    { "70F1A1", "Liteon Technology Corporation" },
    { "70F1E5", "Xetawave LLC" },
    { "70F220", "Actiontec Electronics, Inc" },
    { "70F35A", "Cisco Systems, Inc" },
    { "70F395", "Universal Global Scientific Industrial Co., Ltd." },
    { "70F6CF", "Relay, Inc." },
    { "70F74F", "Robert Bosch JuP1" },
    { "70F754", "AMPAK Technology,Inc." },
    { "70F82B", "DWnet Technologies(Suzhou) Corporation" },
    { "70F8AE", "Microsoft Corporation" },
    { "70F8E7", "IEEE Registration Authority" },
    { "70F927", "Samsung Electronics Co.,Ltd" },
    { "70F94A", "Apple, Inc." },
    { "70F96D", "Hangzhou H3C Technologies Co., Limited" },
    { "70FC8C", "OneAccess SA" },
    { "70FC8F", "FREEBOX SAS" },
    { "70FD45", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "70FD46", "Samsung Electronics Co.,Ltd" },
    { "70FD88", "Nanjing Jiahao Technology Co., Ltd." },
    { "70FF5C", "Cheerzing Communication(Xiamen)Technology Co.,Ltd" },
    { "70FF76", "Texas Instruments" },
    { "7400E8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7402E1", "Texas Instruments" },
    { "7403BD", "BUFFALO.INC" },
    { "74042B", "Lenovo Mobile Communication (Wuhan) Company Limited" },
    { "7404F0", "Mobiwire Mobiles (NingBo) Co., LTD" },
    { "7404F1", "Intel Corporate" },
    { "74051D", "Jiangxi Risound Electronics Co.,LTD" },
    { "74057C", "Qorvo International Pte. Ltd." },
    { "7405A5", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "740635", "SERNET (SUZHOU) TECHNOLOGIES CORPORATION" },
    { "74077E", "Quectel Wireless Solutions Co.,Ltd." },
    { "7408DE", "Fujian Landi Commercial Technology Co., Ltd." },
    { "7409AC", "Quext, LLC" },
    { "740ABC", "LightwaveRF Technology Ltd" },
    { "740AE1", "Huawei Device Co., Ltd." },
    { "740BB0", "Hirschmann Automation and Control GmbH" },
    { "740CEE", "Huawei Device Co., Ltd." },
    { "740EA4", "Apple, Inc." },
    { "740EDB", "Optowiz Co., Ltd" },
    { "7410E0", "Broadcom Limited" },
    { "7411B2", "Cisco Systems, Inc" },
    { "741213", "Linksys USA, Inc" },
    { "7412B3", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "7412BB", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "7413EA", "Intel Corporate" },
    { "741489", "SRT Wireless" },
    { "741575", "Xiaomi Communications Co Ltd" },
    { "7415E2", "Tri-Sen Systems Corporation" },
    { "7415F5", "Apple, Inc." },
    { "741865", "Shanghai DareGlobal Technologies Co.,Ltd" },
    { "74190A", "Samsung Electronics Co.,Ltd" },
    { "7419F8", "IEEE Registration Authority" },
    { "741AE0", "IEEE Registration Authority" },
    { "741B39", "CONVEY INDIA PRIVATE LIMITED" },
    { "741BB2", "Apple, Inc." },
    { "741C27", "ITEL MOBILE LIMITED" },
    { "741E93", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "741EB1", "Samsung Electronics Co.,Ltd" },
    { "741F4A", "Hangzhou H3C Technologies Co., Limited" },
    { "741F79", "YOUNGKOOK ELECTRONICS CO.,LTD" },
    { "74205F", "Shenzhen Zhongruixin Intelligent Technology Co., Ltd." },
    { "74220D", "CHENGDU XUGUANG TECHNOLOGY CO,LTD" },
    { "7422BB", "Huawei Device Co., Ltd." },
    { "742344", "Xiaomi Communications Co Ltd" },
    { "74238D", "zte corporation" },
    { "74249F", "TIBRO Corp." },
    { "7424CA", "Guangzhou Shiyuan Electronic Technology Company Limited" },
    { "742584", "IEEE Registration Authority" },
    { "74258A", "Hangzhou H3C Technologies Co., Limited" },
    { "7426AC", "Cisco Systems, Inc" },
    { "7426FF", "zte corporation" },
    { "74272C", "Advanced Micro Devices, Inc." },
    { "74273C", "ChangYang Technology (Nanjing) Co., LTD" },
    { "7427EA", "Elitegroup Computer Systems Co.,Ltd." },
    { "742857", "Mayfield Robotics" },
    { "742869", "Huawei Device Co., Ltd." },
    { "742972", "Juniper Networks" },
    { "742981", "Texas Instruments" },
    { "7429AF", "Hon Hai Precision Ind. Co.,Ltd." },
    { "742A8A", "shenzhen worldelite electronics co., LTD" },
    { "742B0F", "Infinidat Ltd." },
    { "742B62", "FUJITSU LIMITED" },
    { "742D0A", "Norfolk Elektronik AG" },
    { "742E4F", "Stienen Group" },
    { "742EC1", "Dixon Electro Appliances Pvt Ltd" },
    { "742EDB", "Perinet GmbH" },
    { "742EFC", "DirectPacket Research, Inc," },
    { "742F68", "AzureWave Technology Inc." },
    { "74309D", "Infinix mobility limited" },
    { "7430AF", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "743170", "Arcadyan Technology Corporation" },
    { "743174", "Apple, Inc." },
    { "743256", "NT-ware Systemprg GmbH" },
    { "7432C2", "KYOLIS" },
    { "743357", "vivo Mobile Communication Co., Ltd." },
    { "7433A6", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "7433E9", "zte corporation" },
    { "743400", "MTG Co., Ltd." },
    { "74342B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "743491", "Shenzhen Kings IoT Co., Ltd" },
    { "7434AE", "this is engineering Inc." },
    { "74366D", "Vodafone Italia S.p.A." },
    { "74372F", "Tongfang Shenzhen Cloudcomputing Technology Co.,Ltd" },
    { "74373B", "UNINET Co.,Ltd." },
    { "74375F", "SERCOMM PHILIPPINES INC" },
    { "743822", "Xiaomi Communications Co Ltd" },
    { "743889", "ANNAX Anzeigesysteme GmbH" },
    { "7438B7", "CANON INC." },
    { "743989", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "743A20", "New H3C Technologies Co., Ltd" },
    { "743A65", "NEC Corporation" },
    { "743AEF", "Kaon Group Co., Ltd." },
    { "743AF4", "Intel Corporate" },
    { "743C18", "Taicang T&W Electronics" },
    { "743C24", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "743E2B", "Ruckus Wireless" },
    { "743E39", "YUSUR Technology Co., Ltd." },
    { "743ECB", "Gentrice tech" },
    { "743FC2", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "7440BB", "Hon Hai Precision Ind. Co.,Ltd." },
    { "7440BE", "LG Innotek" },
    { "744218", "Apple, Inc." },
    { "74427F", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "74428B", "Apple, Inc." },
    { "744401", "NETGEAR" },
    { "74452D", "Huawei Device Co., Ltd." },
    { "74458A", "Samsung Electronics Co.,Ltd" },
    { "7445CE", "CRESYN" },
    { "744687", "Kingsignal Technology Co., Ltd." },
    { "7446A0", "Hewlett Packard" },
    { "7446B3", "Texas Instruments" },
    { "7449D2", "New H3C Technologies Co., Ltd" },
    { "744AA4", "zte corporation" },
    { "744BE9", "EXPLORER HYPERTECH CO.,LTD" },
    { "744CA1", "Liteon Technology Corporation" },
    { "744D28", "Routerboard.com" },
    { "744D6D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "744D79", "Arrive Systems Inc." },
    { "744DBD", "Espressif Inc." },
    { "744DDC", "Sonim Technologies, Inc" },
    { "74504E", "New H3C Technologies Co., Ltd" },
    { "7450CD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7451BA", "Xiaomi Communications Co Ltd" },
    { "745327", "COMMSEN CO., LIMITED" },
    { "7453A8", "ACL Airshop BV" },
    { "745427", "SHENZHEN FAST TECHNOLOGIES CO.,LTD" },
    { "74546B", "hangzhou zhiyi communication co., ltd" },
    { "74547D", "Cisco SPVTG" },
    { "745612", "Commscope" },
    { "74563C", "GIGA-BYTE TECHNOLOGY CO.,LTD." },
    { "745798", "TRUMPF Laser GmbH + Co. KG" },
    { "745889", "Multilaser Industrial S.A." },
    { "7458F3", "Amazon Technologies Inc." },
    { "745909", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "745933", "Danal Entertainment" },
    { "745AAA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "745BC5", "IEEE Registration Authority" },
    { "745C4B", "GN Audio A/S" },
    { "745C9F", "TCT mobile ltd" },
    { "745CFA", "Shenzhen Shunrui Gaojie Technology Co., Ltd." },
    { "745D22", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "745D43", "BSH Hausgeraete GmbH" },
    { "745D68", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "745E1C", "PIONEER CORPORATION" },
    { "745F00", "Samsung Semiconductor Inc." },
    { "745F90", "LAM Technologies" },
    { "745FAE", "TSL PPL" },
    { "74604C", "RODE" },
    { "7460FA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "74614B", "Chongqing Huijiatong Information Technology Co., Ltd." },
    { "7463C2", "Huawei Device Co., Ltd." },
    { "7463DF", "VTS GmbH" },
    { "74650C", "Apple, Inc." },
    { "7465D1", "Atlinks" },
    { "746630", "T:mi Ytti" },
    { "74675F", "COMPAL INFORMATION(KUNSHAN)CO.,LTD." },
    { "7467F7", "Extreme Networks Headquarters" },
    { "74694A", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "746A3A", "Aperi Corporation" },
    { "746A89", "Rezolt Corporation" },
    { "746A8F", "VS Vision Systems GmbH" },
    { "746AB3", "MICIUS Laboratory" },
    { "746B82", "MOVEK " },
    { "746BAB", "GUANGDONG ENOK COMMUNICATION CO., LTD" },
    { "746DFA", "Samsung Electronics Co.,Ltd" },
    { "746EE4", "Asia Vital Components Co.,Ltd." },
    { "746F19", "ICARVISIONS (SHENZHEN) TECHNOLOGY CO., LTD." },
    { "746F3D", "Contec GmbH" },
    { "746F88", "zte corporation" },
    { "746FF7", "Wistron Neweb Corporation" },
    { "747069", "Huawei Device Co., Ltd." },
    { "7470FD", "Intel Corporate" },
    { "74718B", "Apple, Inc." },
    { "74721E", "Edison Labs Inc." },
    { "7472B0", "Guangzhou Shiyuan Electronics Co., Ltd. " },
    { "7472F2", "Chipsip Technology Co., Ltd." },
    { "74731D", "ifm electronic gmbh" },
    { "747336", "MICRODIGTAL Inc" },
    { "7473B4", "Apple, Inc." },
    { "7473E2", "Hillstone Networks Corp." },
    { "747446", "Google, Inc." },
    { "747548", "Amazon Technologies Inc." },
    { "7475DF", "TECLINK" },
    { "74761F", "Microsoft Corporation" },
    { "74765B", "Quectel Wireless Solutions Co.,Ltd." },
    { "74767D", "shenzhen kexint technology co.,ltd" },
    { "747786", "Apple, Inc." },
    { "747818", "Jurumani Solutions" },
    { "747827", "Dell Inc." },
    { "747847", "Interdisciplinary Consulting Corporation" },
    { "7478A6", "Fortinet, Inc." },
    { "747A90", "Murata Manufacturing Co., Ltd." },
    { "747B7A", "ETH Inc." },
    { "747D24", "Phicomm (Shanghai) Co., Ltd." },
    { "747DB6", "Aliwei Communications, Inc" },
    { "747E1A", "Red Embedded Design Limited" },
    { "747E2D", "Beijing Thomson CITIC Digital Technology Co. LTD." },
    { "74803F", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "748114", "Apple, Inc." },
    { "74819A", "PT. Hartono Istana Teknologi" },
    { "7483C2", "Ubiquiti Inc" },
    { "7483EF", "Arista Networks" },
    { "748469", "Nintendo Co.,Ltd" },
    { "7484E1", "Dongguan Haoyuan Electronics Co.,Ltd" },
    { "74852A", "PEGATRON CORPORATION" },
    { "7485C4", "New H3C Technologies Co., Ltd" },
    { "74860B", "Cisco Systems, Inc" },
    { "748669", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "74866F", "zte corporation" },
    { "74867A", "Dell Inc." },
    { "7486E2", "Dell Inc." },
    { "74872E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7487A9", "OCT Technology Co., Ltd." },
    { "7487BB", "Ciena Corporation" },
    { "74882A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "74888B", "ADB Broadband Italia" },
    { "7488A8", "Bouffalo Lab (Nanjing) Co., Ltd." },
    { "7488BB", "Cisco Systems, Inc" },
    { "748A0D", "Commscope" },
    { "748A28", "HMD Global Oy" },
    { "748A69", "Korea Image Technology Co., Ltd" },
    { "748B29", "Micobiomed" },
    { "748B34", "Shanghai Smart System Technology Co., Ltd" },
    { "748D08", "Apple, Inc." },
    { "748E08", "Bestek Corp." },
    { "748EF8", "Brocade Communications Systems LLC" },
    { "748F1B", "MasterImage 3D" },
    { "748F3C", "Apple, Inc." },
    { "748F4D", "duagon Germany GmbH" },
    { "748FC2", "Cisco Systems, Inc" },
    { "74901F", "Ragile Networks Inc." },
    { "749050", "Renesas Electronics Corporation" },
    { "7490BC", "Arcadyan Corporation" },
    { "74911A", "Ruckus Wireless" },
    { "7491BD", "Four systems Co.,Ltd." },
    { "7492BA", "Movesense Ltd" },
    { "7493A4", "Zebra Technologies Corp." },
    { "7493DA", "ASKEY COMPUTER CORP" },
    { "74943D", "AgJunction" },
    { "749533", "Westala Technologies Inc." },
    { "749552", "Xuzhou WIKA Electronics Control Technology Co., Ltd." },
    { "7495A7", "Keyence Corporation" },
    { "7495EC", "ALPSALPINE CO,.LTD" },
    { "749637", "Todaair Electronic Co., Ltd" },
    { "749779", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "749781", "zte corporation" },
    { "74978E", "Nova Labs" },
    { "749975", "IBM Corporation" },
    { "749AC0", "Cachengo, Inc." },
    { "749B89", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "749BE8", "Hitron Technologies. Inc" },
    { "749C52", "Huizhou Desay SV Automotive Co., Ltd." },
    { "749CE3", "KodaCloud Canada, Inc" },
    { "749D79", "Sercomm Corporation." },
    { "749D8F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "749DDC", "2Wire Inc" },
    { "749E75", "Hewlett Packard Enterprise" },
    { "749EA5", "OHSUNG" },
    { "749EAF", "Apple, Inc." },
    { "749EF5", "Samsung Electronics Co.,Ltd" },
    { "74A02F", "Cisco Systems, Inc" },
    { "74A063", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "74A2E6", "Cisco Systems, Inc" },
    { "74A34A", "ZIMI CORPORATION" },
    { "74A4A7", "QRS Music Technologies, Inc." },
    { "74A4B5", "Powerleader Science and Technology Co. Ltd." },
    { "74A528", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "74A57E", "Panasonic Ecology Systems" },
    { "74A58C", "Texas Instruments" },
    { "74A5C2", "KYOCERA Corporation " },
    { "74A6CD", "Apple, Inc." },
    { "74A722", "LG Electronics (Mobile Communications)" },
    { "74A78E", "zte corporation" },
    { "74A7EA", "Amazon Technologies Inc." },
    { "74AB93", "Blink by Amazon " },
    { "74AC5F", "Qiku Internet Network Scientific (Shenzhen) Co., Ltd." },
    { "74ACB9", "Ubiquiti Inc" },
    { "74AD45", "Valeo Auto- Electric Hungary Ltd" },
    { "74AD98", "Cisco Systems, Inc" },
    { "74ADB7", "China Mobile Group Device Co.,Ltd." },
    { "74ADCB", "New H3C Technologies Co., Ltd" },
    { "74AE76", "iNovo Broadband, Inc." },
    { "74B00C", "Network Video Technologies, Inc" },
    { "74B059", "Motorola Mobility LLC, a Lenovo Company" },
    { "74B3EA", "EK INC." },
    { "74B472", "CIESSE" },
    { "74B57E", "zte corporation" },
    { "74B587", "Apple, Inc." },
    { "74B6B6", "eero inc." },
    { "74B725", "Huawei Device Co., Ltd." },
    { "74B7B3", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "74B7E6", "Zegna-Daidong Limited" },
    { "74B80F", "Zipline International Inc." },
    { "74B839", "Texas Instruments" },
    { "74B8A8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "74B91E", "Nanjing Bestway Automation System Co., Ltd" },
    { "74B9EB", "JinQianMao Technology Co.,Ltd." },
    { "74BADB", "Longconn Electornics(shenzhen)Co.,Ltd" },
    { "74BBD3", "Shenzhen xeme Communication Co., Ltd." },
    { "74BE08", "ATEK Products, LLC" },
    { "74BEF3", "Motorola Mobility LLC, a Lenovo Company" },
    { "74BFA1", "HYUNTECK" },
    { "74BFB7", "Nusoft Corporation" },
    { "74BFC0", "CANON INC." },
    { "74C14F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "74C17D", "Infinix mobility limited" },
    { "74C17E", "LG Electronics" },
    { "74C246", "Amazon Technologies Inc." },
    { "74C330", "SHENZHEN FAST TECHNOLOGIES CO.,LTD" },
    { "74C412", "Microsoft Corporation" },
    { "74C530", "vivo Mobile Communication Co., Ltd." },
    { "74C621", "Zhejiang Hite Renewable Energy Co.,LTD" },
    { "74C63B", "AzureWave Technology Inc." },
    { "74C64A", "AGOS Co.,Ltd" },
    { "74C76E", "RTK-TECHNOLOGIES, LLC" },
    { "74C90F", "Microchip Technologies Inc" },
    { "74C929", "Zhejiang Dahua Technology Co., Ltd." },
    { "74C99A", "Ericsson AB" },
    { "74C9A3", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "74CA25", "Calxeda, Inc." },
    { "74CA60", "Sonos, Inc." },
    { "74CBF3", "Lava international limited" },
    { "74CC39", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "74CC40", "Apple, Inc." },
    { "74CD0C", "Smith Myers Communications Ltd." },
    { "74CE56", "Packet Force Technology Limited Company" },
    { "74CF00", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "74D02B", "ASUSTek COMPUTER INC." },
    { "74D0DC", "Ericsson AB" },
    { "74D21D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "74D285", "Texas Instruments" },
    { "74D423", "Amazon Technologies Inc." },
    { "74D435", "GIGA-BYTE TECHNOLOGY CO.,LTD." },
    { "74D4DD", "Quanta Computer Inc." },
    { "74D558", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "74D5B8", "Infraeo Inc" },
    { "74D5C6", "Microchip Technologies Inc" },
    { "74D637", "Amazon Technologies Inc." },
    { "74D654", "GINT" },
    { "74D675", "WYMA Tecnologia" },
    { "74D6CB", "New H3C Technologies Co., Ltd" },
    { "74D6E5", "Huawei Device Co., Ltd." },
    { "74D6EA", "Texas Instruments" },
    { "74D713", "Huaqin Technology Co. LTD" },
    { "74D7CA", "Panasonic Automotive Systems Co.,Ltd" },
    { "74D83E", "Intel Corporate" },
    { "74D850", "Evrisko Systems" },
    { "74D873", "GUANGDONG GENIUS TECHNOLOGY CO., LTD." },
    { "74D9EB", "Petabit Scale, Inc." },
    { "74DA38", "Edimax Technology Co. Ltd." },
    { "74DA88", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "74DADA", "D-Link International" },
    { "74DAEA", "Texas Instruments" },
    { "74DBD1", "Ebay Inc" },
    { "74DDCB", "China Leadshine Technology Co.,Ltd" },
    { "74DE2B", "Liteon Technology Corporation" },
    { "74DFBF", "Liteon Technology Corporation" },
    { "74E06E", "Ergophone GmbH" },
    { "74E147", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "74E14A", "IEEE Registration Authority" },
    { "74E182", "Texas Instruments" },
    { "74E19A", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "74E1B6", "Apple, Inc." },
    { "74E20C", "Amazon Technologies Inc." },
    { "74E277", "Vizmonet Pte Ltd" },
    { "74E28C", "Microsoft Corporation" },
    { "74E2E7", "Cisco Systems, Inc" },
    { "74E2F5", "Apple, Inc." },
    { "74E336", "FUJIAN STAR-NET COMMUNICATION CO.,LTD" },
    { "74E424", "APISTE CORPORATION" },
    { "74E50B", "Intel Corporate" },
    { "74E537", "RADSPIN" },
    { "74E543", "Liteon Technology Corporation" },
    { "74E5F9", "Intel Corporate" },
    { "74E60F", "TECNO MOBILE LIMITED" },
    { "74E6B8", "LG Electronics" },
    { "74E6E2", "Dell Inc." },
    { "74E798", "Juniper Networks" },
    { "74E7C6", "Commscope" },
    { "74E987", "Apple, Inc." },
    { "74E9BF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "74E9D8", "Shanghai High-Flying Electronics Technology Co.,Ltd" },
    { "74EA3A", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "74EAC8", "New H3C Technologies Co., Ltd" },
    { "74EACB", "New H3C Technologies Co., Ltd" },
    { "74EAE8", "Commscope" },
    { "74EB80", "Samsung Electronics Co.,Ltd" },
    { "74EC42", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "74ECB2", "Amazon Technologies Inc." },
    { "74ECF1", "Acumen" },
    { "74EE2A", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "74EE8D", "Apollo Intelligent Connectivity (Beijing) Technology Co., Ltd." },
    { "74EF4B", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "74F06D", "AzureWave Technology Inc." },
    { "74F07D", "BnCOM Co.,Ltd" },
    { "74F102", "Beijing HCHCOM Technology Co., Ltd" },
    { "74F2FA", "Xiaomi Communications Co Ltd" },
    { "74F413", "Maxwell Forest" },
    { "74F441", "Samsung Electronics Co.,Ltd" },
    { "74F612", "Commscope" },
    { "74F61C", "HTC Corporation" },
    { "74F661", "Schneider Electric Fire & Security Oy" },
    { "74F67A", "Samsung Electronics Co.,Ltd" },
    { "74F726", "Neuron Robotics" },
    { "74F737", "KCE" },
    { "74F7F6", "Shanghai Sunmi Technology Co.,Ltd." },
    { "74F85D", "Berkeley Nucleonics Corp" },
    { "74F8DB", "IEEE Registration Authority" },
    { "74F90F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "74F91A", "Onface" },
    { "74F9CA", "Nintendo Co.,Ltd" },
    { "74FB17", "Qingdao Goertek Horizons Tecnology Co.,LTD" },
    { "74FC45", "SIEMENS AG" },
    { "74FDA0", "Compupal (Group) Corporation " },
    { "74FE48", "ADVANTECH CO., LTD." },
    { "74FECE", "TP-Link Systems Inc" },
    { "74FF4C", "Skyworth Digital Technology(Shenzhen) Co.,Ltd" },
    { "74FF7D", "Wren Sound Systems, LLC" },
    { "78009E", "Samsung Electronics Co.,Ltd" },
    { "7800A8", "PT HAN SUNG ELECTORONICS INDONESIA" },
    { "78028B", "Apple, Inc." },
    { "78028F", "Adaptive Spectrum and Signal Alignment (ASSIA), Inc." },
    { "7802B1", "Cisco Systems, Inc" },
    { "7802B7", "ShenZhen Ultra Easy Technology CO.,LTD" },
    { "7802F8", "Xiaomi Communications Co Ltd" },
    { "78034F", "Nokia" },
    { "780473", "Texas Instruments" },
    { "78047A", "Edge Networks LLC" },
    { "7804E3", "Huawei Device Co., Ltd." },
    { "780541", "Queclink Wireless Solutions Co., Ltd" },
    { "78055F", "Shenzhen WYC Technology Co., Ltd." },
    { "78058C", "mMax Communications, Inc." },
    { "7806C9", "Huawei Device Co., Ltd." },
    { "78071C", "Green Energy Options Ltd" },
    { "780738", "Z.U.K. Elzab S.A." },
    { "78078F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "78084D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "780AC7", "Baofeng TV Co., Ltd." },
    { "780B8C", "Private" },
    { "780C71", "Inseego Wireless, Inc" },
    { "780CB8", "Intel Corporate" },
    { "780CF0", "Cisco Systems, Inc" },
    { "780ED1", "TRUMPF Werkzeugmaschinen GmbH+Co.KG" },
    { "780F77", "HangZhou Gubei Electronics Technology Co.,Ltd" },
    { "780F81", "Cisco Meraki" },
    { "781053", "China Mobile Group Device Co.,Ltd." },
    { "781100", "Quantumsolution" },
    { "781185", "NBS Payment Solutions Inc." },
    { "7811DC", "XIAOMI Electronics,CO.,LTD" },
    { "78123E", "TECNO MOBILE LIMITED" },
    { "7812B8", "ORANTEK LIMITED" },
    { "781305", "IEEE Registration Authority" },
    { "7813E0", "FUJIAN STAR-NET COMMUNICATION CO.,LTD" },
    { "78152D", "UNION CHIP TECHNOLOGY LIMITED" },
    { "781699", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "781735", "Nokia Shanghai Bell Co., Ltd." },
    { "7817BE", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "781881", "AzureWave Technology Inc." },
    { "7818A8", "Huawei Device Co., Ltd." },
    { "7818EC", "Fortinet, Inc." },
    { "78192E", "NASCENT Technology" },
    { "7819F7", "Juniper Networks" },
    { "781C1E", "Chongqing Yipingfang Technology Co., Ltd." },
    { "781C3C", "Espressif Inc." },
    { "781C5A", "SHARP Corporation" },
    { "781C9D", "Silicon Laboratories" },
    { "781D4A", "zte corporation" },
    { "781DBA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "781DFD", "Jabil Inc" },
    { "781EB8", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "781F11", "RAB Lighting" },
    { "781F7C", "Nokia" },
    { "781FDB", "Samsung Electronics Co.,Ltd" },
    { "78202E", "Skychers Creations ShenZhen Limited" },
    { "782051", "TP-Link Systems Inc." },
    { "782079", "ID Tech" },
    { "7820A5", "Nintendo Co.,Ltd" },
    { "7820BD", "Polysense (Beijing) Technologies Co. Ltd" },
    { "782184", "Espressif Inc." },
    { "78223D", "Affirmed Networks" },
    { "782288", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "782327", "Samsung Electronics Co.,Ltd" },
    { "7823AE", "Commscope" },
    { "782459", "Alcatel-Lucent Enterprise" },
    { "7824AF", "ASUSTek COMPUTER INC." },
    { "7824BE", "Cisco Systems, Inc" },
    { "782544", "Omnima Limited" },
    { "78257A", "LEO Innovation Lab" },
    { "782599", "Huawei Device Co., Ltd." },
    { "7825AD", "Samsung Electronics Co.,Ltd" },
    { "7826A6", "zte corporation" },
    { "7828CA", "Sonos, Inc." },
    { "7829AD", "NINGBO QIXIANG INFORMATION TECHNOLOGY CO., LTD" },
    { "7829ED", "ASKEY COMPUTER CORP" },
    { "782A79", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "782AF8", "IETHCOM INFORMATION TECHNOLOGY CO., LTD." },
    { "782B46", "Intel Corporate" },
    { "782B60", "Huawei Device Co., Ltd." },
    { "782B64", "Bose Corporation" },
    { "782BCB", "Dell Inc." },
    { "782C29", "New H3C Technologies Co., Ltd" },
    { "782D7E", "TRENDnet, Inc." },
    { "782DAD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "782E03", "GUANGDONG GENIUS TECHNOLOGY CO., LTD." },
    { "782E56", "China Mobile Group Device Co.,Ltd." },
    { "782EEF", "Nokia Corporation" },
    { "782F17", "Xlab Co.,Ltd" },
    { "78303B", "Stephen Technologies Co.,Limited" },
    { "78305D", "zte corporation" },
    { "7830E1", "UltraClenz, LLC" },
    { "7830F5", "TBT Inc." },
    { "78312B", "zte corporation" },
    { "7831C1", "Apple, Inc." },
    { "78321B", "D-Link International" },
    { "78324F", "Millennium Group, Inc." },
    { "7833C6", "Samsung Electronics Co.,Ltd" },
    { "783409", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "783486", "Nokia" },
    { "7834FD", "vivo Mobile Communication Co., Ltd." },
    { "7835A0", "Zurn Industries LLC" },
    { "783607", "Cermate Technologies Inc." },
    { "783690", "Yulong Computer Telecommunication Scientific (Shenzhen) Co.,Ltd" },
    { "7836CC", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "783716", "Samsung Electronics Co.,Ltd" },
    { "78392D", "IEEE Registration Authority" },
    { "783A6C", "TECNO MOBILE LIMITED" },
    { "783A84", "Apple, Inc." },
    { "783CE3", "Kai-EE" },
    { "783D5B", "TELNET Redes Inteligentes S.A." },
    { "783E53", "SKY UK LIMITED" },
    { "783EA1", "Nokia Shanghai Bell Co., Ltd." },
    { "783F15", "EasySYNC Ltd." },
    { "783F4D", "Apple, Inc." },
    { "7840E4", "Samsung Electronics Co.,Ltd" },
    { "78421C", "Espressif Inc." },
    { "784405", "FUJITU(HONG KONG) ELECTRONIC Co.,LTD." },
    { "78444A", "Shenzhen Aiwinn information Technology Co., Ltd." },
    { "784476", "Zioncom Electronics (Shenzhen) Ltd." },
    { "7844FD", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "784501", "Biamp Systems" },
    { "784558", "Ubiquiti Inc" },
    { "784561", "CyberTAN Technology Inc." },
    { "7845B3", "Huawei Device Co., Ltd." },
    { "7845C4", "Dell Inc." },
    { "78465C", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "78465F", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "78467D", "SKAIChips" },
    { "7846C4", "DAEHAP HYPER-TECH" },
    { "7846D4", "Samsung Electronics Co.,Ltd" },
    { "78471D", "Samsung Electronics Co.,Ltd" },
    { "7847E3", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "78482C", "START USA, INC." },
    { "784859", "Hewlett Packard" },
    { "78491D", "The Will-Burt Company" },
    { "784946", "Cambridge Mobile Telematics, Inc." },
    { "784B08", "f.robotics acquisitions ltd" },
    { "784B87", "Murata Manufacturing Co., Ltd." },
    { "784F24", "Taicang T&W Electronics" },
    { "784F43", "Apple, Inc." },
    { "784F9B", "Juniper Networks" },
    { "785005", "MOKO TECHNOLOGY Ltd" },
    { "78507C", "Juniper Networks" },
    { "78510C", "LiveU Ltd." },
    { "78521A", "Samsung Electronics Co.,Ltd" },
    { "785237", "zte corporation" },
    { "78524A", "Optonic GmbH" },
    { "785262", "Shenzhen Hojy Software Co., Ltd." },
    { "78530D", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "785333", "Beijing Xiaomi Electronics Co.,Ltd" },
    { "785364", "SHIFT GmbH" },
    { "7853F2", "Roxton Systems Ltd." },
    { "78542E", "D-Link International" },
    { "785517", "SankyuElectronics" },
    { "785536", "shenzhen AZW Technology(Group) Co.,Ltd" },
    { "785712", "Mobile Integration Workgroup" },
    { "785773", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7857B0", "GERTEC BRASIL LTDA" },
    { "785844", "Hangzhou Sciener Smart Technology Co., Ltd." },
    { "785860", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7858F3", "Vachen Co.,Ltd" },
    { "78593E", "RAFI GmbH & Co.KG" },
    { "78595E", "Samsung Electronics Co.,Ltd" },
    { "785968", "Hon Hai Precision Ind. Co.,Ltd." },
    { "785994", "Alif Semiconductor, Inc." },
    { "785B64", "Huawei Device Co., Ltd." },
    { "785C28", "Prime Motion Inc." },
    { "785C5E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "785C72", "Hioso Technology Co., Ltd." },
    { "785DC8", "LG Electronics" },
    { "785EA2", "Sunitec Enterprise Co.,Ltd" },
    { "785ECC", "Apple, Inc." },
    { "785EE8", "IEEE Registration Authority" },
    { "785F28", "EM Microelectronic" },
    { "785F36", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "785F4C", "Argox Information Co., Ltd." },
    { "785F6C", "Arista Networks" },
    { "78605B", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "78617C", "MITSUMI ELECTRIC CO.,LTD." },
    { "786256", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "786299", "BITSTREAM sp. z o.o." },
    { "7864A0", "Cisco Systems, Inc" },
    { "7864C0", "Apple, Inc." },
    { "7864E6", "Green Motive Technology Limited" },
    { "78653B", "Shaoxing Ourten Electronics Co., Ltd." },
    { "786559", "Sagemcom Broadband SAS" },
    { "78669D", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "7866AE", "ZTEC Instruments, Inc." },
    { "7866D7", "GENSTORAIGE TECHNOLOGY CO.LTD." },
    { "7866F3", "shenzhen worldelite electronics co., LTD" },
    { "78670E", "Wistron Neweb Corporation" },
    { "7867D7", "Apple, Inc." },
    { "786829", "eero inc." },
    { "7868F7", "YSTen Technology Co.,Ltd" },
    { "7869D4", "Shenyang Vibrotech Instruments Inc." },
    { "786A1F", "Commscope" },
    { "786A89", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "786C1C", "Apple, Inc." },
    { "786C84", "Amazon Technologies Inc." },
    { "786CAB", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "786D94", "Palo Alto Networks" },
    { "786DEB", "GE Lighting" },
    { "787052", "Welotec GmbH" },
    { "787104", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "78719C", "Commscope" },
    { "78725D", "Cisco Systems, Inc" },
    { "787264", "IEEE Registration Authority" },
    { "78753E", "Apple, Inc." },
    { "787689", "eero inc." },
    { "7876D9", "EXARA Group" },
    { "787835", "IEEE Registration Authority" },
    { "787A6F", "Juice Technology AG" },
    { "787B8A", "Apple, Inc." },
    { "787D48", "ITEL MOBILE LIMITED" },
    { "787D53", "Extreme Networks Headquarters" },
    { "787DF3", "Sterlite Technologies Limited" },
    { "787E42", "zte corporation" },
    { "787E61", "Apple, Inc." },
    { "787F62", "GiK mbH" },
    { "788038", "FUNAI ELECTRIC CO., LTD." },
    { "788102", "Sercomm Corporation." },
    { "78818C", "Nintendo Co.,Ltd" },
    { "78818F", "Server Racks Australia Pty Ltd" },
    { "7881CE", "China Mobile Iot Limited company" },
    { "788371", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "78843C", "Sony Corporation" },
    { "7884EE", "INDRA ESPACIO S.A." },
    { "788517", "Cisco Systems, Inc" },
    { "7885F4", "Huawei Device Co., Ltd." },
    { "78862E", "Microsoft Corporation" },
    { "7886B6", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "78870D", "Unifiedgateways India Private Limited" },
    { "78886D", "Apple, Inc." },
    { "78888A", "CDR Sp. z o.o. Sp. k." },
    { "788973", "CMC" },
    { "788A20", "Ubiquiti Inc" },
    { "788A86", "China Dragon Technology Limited" },
    { "788B2A", "Zhen Shi Information Technology (Shanghai) Co., Ltd." },
    { "788B77", "Standar Telecom" },
    { "788C4D", "Indyme Solutions, LLC" },
    { "788C54", "Ping Communication" },
    { "788C77", "LEXMARK INTERNATIONAL, INC." },
    { "788CB5", "TP-Link Systems Inc" },
    { "788DAF", "Sagemcom Broadband SAS" },
    { "788DF7", "Hitron Technologies. Inc" },
    { "788E33", "Jiangsu SEUIC Technology Co.,Ltd" },
    { "788E45", "Jizaie inc." },
    { "7890A2", "zte corporation" },
    { "7891DE", "Guangdong ACIGA Science&Technology Co.,Ltd" },
    { "7891E9", "Raisecom Technology CO., LTD" },
    { "78923E", "Nokia Corporation" },
    { "78929C", "Intel Corporate" },
    { "7893C3", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "7894B4", "Sercomm Corporation." },
    { "7894E8", "Radio Bridge" },
    { "7895EB", "ITEL MOBILE LIMITED" },
    { "789682", "zte corporation" },
    { "789684", "Commscope" },
    { "7896A3", "Extreme Networks Headquarters" },
    { "7897C3", "DINGXIN INFORMATION TECHNOLOGY CO.,LTD" },
    { "7898E8", "D-Link International" },
    { "7898FD", "Q9 Networks Inc." },
    { "789912", "Flyingvoice (HongKong) Technologies Limited" },
    { "78995C", "Nationz Technologies Inc" },
    { "789966", "Musilab Electronics (DongGuan)Co.,Ltd." },
    { "789987", "Xiaomi Communications Co Ltd" },
    { "78998F", "MEDILINE ITALIA SRL" },
    { "789A18", "Routerboard.com" },
    { "789C85", "August Home, Inc." },
    { "789CE7", "Shenzhen Aikede Technology Co., Ltd" },
    { "789ED0", "Samsung Electronics Co.,Ltd" },
    { "789F38", "Shenzhen Feasycom Co., Ltd" },
    { "789F4C", "HOERBIGER Elektronik GmbH" },
    { "789F6A", "Ruckus Wireless" },
    { "789F70", "Apple, Inc." },
    { "789F87", "Siemens AG I IA PP PRM" },
    { "789FAA", "Huawei Device Co., Ltd." },
    { "78A03F", "Amazon Technologies Inc." },
    { "78A051", "iiNet Labs Pty Ltd " },
    { "78A106", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "78A13E", "New H3C Technologies Co., Ltd" },
    { "78A183", "Advidia" },
    { "78A1D8", "ShenzhenEnjoyTechnologyCo.,Ltd" },
    { "78A2A0", "Nintendo Co., Ltd." },
    { "78A351", "SHENZHEN ZHIBOTONG ELECTRONICS CO.,LTD" },
    { "78A3E4", "Apple, Inc." },
    { "78A4BA", "Marquardt India Pvt Ltd" },
    { "78A504", "Texas Instruments" },
    { "78A5DD", "Shenzhen Smarteye Digital Electronics Co., Ltd" },
    { "78A683", "Precidata" },
    { "78A6A0", "Hangzhou Ezviz Software Co.,Ltd." },
    { "78A6BD", "DAEYEON Control&Instrument Co,.Ltd" },
    { "78A6E1", "Brocade Communications Systems LLC" },
    { "78A714", "Amphenol" },
    { "78A7C7", "Apple, Inc." },
    { "78A7EB", "1MORE" },
    { "78A873", "Samsung Electronics Co.,Ltd" },
    { "78AA82", "New H3C Technologies Co., Ltd" },
    { "78AB60", "ABB Australia" },
    { "78ABBB", "Samsung Electronics Co.,Ltd" },
    { "78AC44", "Dell Inc." },
    { "78ACBF", "Igneous Systems" },
    { "78ACC0", "Hewlett Packard" },
    { "78AE0C", "Far South Networks" },
    { "78AF08", "Intel Corporate" },
    { "78AF58", "GIMASI SA" },
    { "78AFE4", "Comau S.p.A" },
    { "78B213", "DWnet Technologies(Suzhou) Corporation" },
    { "78B28D", "Beijing Tengling Technology CO.Ltd " },
    { "78B39F", "SERNET (SUZHOU) TECHNOLOGIES CORPORATION" },
    { "78B3B9", "ShangHai sunup lighting CO.,LTD" },
    { "78B3CE", "Elo touch solutions" },
    { "78B46A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "78B554", "Huawei Device Co., Ltd." },
    { "78B5D2", "Ever Treasure Industrial Limited" },
    { "78B6C1", "AOBO Telecom Co.,Ltd" },
    { "78B6EC", "Scuf Gaming International LLC" },
    { "78B6FE", "Samsung Electronics Co.,Ltd" },
    { "78B81A", "INTER SALES A/S" },
    { "78B84B", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "78B8D6", "Zebra Technologies Inc." },
    { "78BAD0", "Shinybow Technology Co. Ltd." },
    { "78BAF9", "Cisco Systems, Inc" },
    { "78BB88", "Maxio Technology (Hangzhou) Ltd." },
    { "78BBC1", "SERVERCOM (INDIA) PRIVATE LIMITED" },
    { "78BC1A", "Cisco Systems, Inc" },
    { "78BDBC", "Samsung Electronics Co.,Ltd" },
    { "78BE81", "FN-LINK TECHNOLOGY Ltd." },
    { "78BEB6", "Enhanced Vision" },
    { "78BEBD", "STULZ GmbH" },
    { "78C11D", "Samsung Electronics Co.,Ltd" },
    { "78C1A7", "zte corporation" },
    { "78C1AE", "Hangzhou Ezviz Software Co.,Ltd." },
    { "78C213", "Sagemcom Broadband SAS" },
    { "78C2C0", "IEEE Registration Authority" },
    { "78C313", "China Mobile Group Device Co.,Ltd." },
    { "78C3E9", "Samsung Electronics Co.,Ltd" },
    { "78C40E", "H&D Wireless" },
    { "78C4AB", "Shenzhen Runsil Technology Co.,Ltd" },
    { "78C4FA", "Meta Platforms, Inc." },
    { "78C57D", "Zyxel Communications Corporation" },
    { "78C5E5", "Texas Instruments" },
    { "78C5F8", "Huawei Device Co., Ltd." },
    { "78C62B", "FUJIAN STAR-NET COMMUNICATION CO.,LTD" },
    { "78C6BB", "Analog Devices, Inc." },
    { "78C881", "Sony Interactive Entertainment Inc." },
    { "78C884", "Huawei Device Co., Ltd." },
    { "78C95E", "Midmark RTLS " },
    { "78CA04", "Nokia Corporation" },
    { "78CA39", "Apple, Inc." },
    { "78CA5E", "ELNO" },
    { "78CA83", "IEEE Registration Authority" },
    { "78CB2C", "Join Digital, Inc." },
    { "78CB33", "DHC Software Co.,Ltd" },
    { "78CB68", "DAEHAP HYPER-TECH" },
    { "78CC2B", "SINEWY TECHNOLOGY CO., LTD" },
    { "78CD55", "Texas Instruments" },
    { "78CD8E", "SMC Networks Inc" },
    { "78CEA5", "Vital link vietnam company limited" },
    { "78CF2F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "78CFF9", "Huawei Device Co., Ltd." },
    { "78D004", "Neousys Technology Inc." },
    { "78D129", "Vicos" },
    { "78D162", "Apple, Inc." },
    { "78D294", "NETGEAR" },
    { "78D347", "Ericsson AB" },
    { "78D34F", "Pace-O-Matic, Inc." },
    { "78D38D", "HONGKONG YUNLINK TECHNOLOGY LIMITED" },
    { "78D3ED", "NORMA" },
    { "78D4F1", "IEEE Registration Authority" },
    { "78D5B5", "NAVIELEKTRO KY" },
    { "78D66F", "Aristocrat Technologies Australia Pty. Ltd." },
    { "78D6B2", "Toshiba" },
    { "78D6D6", "eero inc." },
    { "78D6DC", "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd." },
    { "78D6F0", "SAMSUNG ELECTRO MECHANICS CO., LTD." },
    { "78D71A", "Ciena Corporation" },
    { "78D752", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "78D75F", "Apple, Inc." },
    { "78D800", "IEEE Registration Authority" },
    { "78D840", "Xiaomi Communications Co Ltd" },
    { "78D99F", "NuCom HK Ltd." },
    { "78D9E9", "MOMENTUM IOT" },
    { "78DA07", "Zhejiang Tmall Technology Co., Ltd." },
    { "78DA6E", "Cisco Systems, Inc" },
    { "78DAA2", "Cynosure Technologies Co.,Ltd" },
    { "78DAAF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "78DAB3", "GBO Technology" },
    { "78DB2F", "Texas Instruments" },
    { "78DC87", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "78DD08", "Hon Hai Precision Ind. Co.,Ltd." },
    { "78DD12", "Arcadyan Corporation" },
    { "78DD33", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "78DDD6", "c-scape" },
    { "78DDD9", "Guangzhou Shiyuan Electronics Co., Ltd. " },
    { "78DEE4", "Texas Instruments" },
    { "78DF72", "Shanghai Imilab Technology Co.Ltd" },
    { "78E103", "Amazon Technologies Inc." },
    { "78E167", "Launch Tech Co., Ltd." },
    { "78E22C", "Huawei Device Co., Ltd." },
    { "78E2BD", "Vodafone Automotive S.p.A." },
    { "78E36D", "Espressif Inc." },
    { "78E3B5", "Hewlett Packard" },
    { "78E3DE", "Apple, Inc." },
    { "78E400", "Hon Hai Precision Ind. Co.,Ltd." },
    { "78E61C", "Honor Device Co., Ltd." },
    { "78E7D1", "Hewlett Packard" },
    { "78E8B6", "zte corporation" },
    { "78E980", "RainUs Co.,Ltd" },
    { "78E996", "IEEE Registration Authority" },
    { "78E9CF", "TELLESCOM INDUSTRIA E COMERCIO EM TELECOMUNICACAO " },
    { "78EB14", "SHENZHEN FAST TECHNOLOGIES CO.,LTD" },
    { "78EB39", "Instituto Nacional de Tecnología Industrial" },
    { "78EB46", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "78EC22", "Shanghai Qihui Telecom Technology Co., LTD" },
    { "78EC74", "Kyland-USA" },
    { "78ED25", "New H3C Technologies Co., Ltd" },
    { "78EDBC", "OnePlus Technology (Shenzhen) Co., Ltd" },
    { "78EE4C", "Espressif Inc." },
    { "78EF4C", "Unetconvergence Co., Ltd." },
    { "78F09B", "Huawei Device Co., Ltd." },
    { "78F1C6", "Cisco Systems, Inc" },
    { "78F235", "Sichuan AI-Link Technology Co., Ltd." },
    { "78F238", "Samsung Electronics Co.,Ltd" },
    { "78F276", "Cyklop Fastjet Technologies (Shanghai) Inc." },
    { "78F29E", "PEGATRON CORPORATION" },
    { "78F505", "Murata Manufacturing Co., Ltd." },
    { "78F557", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "78F5E5", "BEGA Gantenbrink-Leuchten KG" },
    { "78F5FD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "78F7A3", "Opentext" },
    { "78F7BE", "Samsung Electronics Co.,Ltd" },
    { "78F7D0", "Silverbrook Research" },
    { "78F882", "LG Electronics (Mobile Communications)" },
    { "78F8B8", "Rako Controls Ltd" },
    { "78F944", "Private" },
    { "78F9B4", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "78FBD8", "Apple, Inc." },
    { "78FC14", "Family Zone Cyber Safety Ltd " },
    { "78FD94", "Apple, Inc." },
    { "78FE3D", "Juniper Networks" },
    { "78FE41", "Socus networks" },
    { "78FEE2", "Shanghai Diveo Technology Co., Ltd" },
    { "78FF57", "Intel Corporate" },
    { "78FFCA", "TECNO MOBILE LIMITED" },
    { "7C004D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7C010A", "Texas Instruments" },
    { "7C0187", "Curtis Instruments, Inc." },
    { "7C0191", "Apple, Inc." },
    { "7C02BC", "Hansung Electronics Co. LTD" },
    { "7C034C", "Sagemcom Broadband SAS" },
    { "7C035E", "Xiaomi Communications Co Ltd" },
    { "7C03AB", "Xiaomi Communications Co Ltd" },
    { "7C03C9", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "7C03D8", "Sagemcom Broadband SAS" },
    { "7C04D0", "Apple, Inc." },
    { "7C0507", "PEGATRON CORPORATION" },
    { "7C051E", "RAFAEL LTD." },
    { "7C0623", "Ultra Electronics Sonar System Division" },
    { "7C08D9", "Shanghai B-Star Technology Co" },
    { "7C092B", "Bekey A/S" },
    { "7C0A3F", "Samsung Electronics Co.,Ltd" },
    { "7C0A50", "J-MEX Inc." },
    { "7C0BC6", "Samsung Electronics Co.,Ltd" },
    { "7C0C92", "Suzhou Mobydata Smart System Co.,Ltd." },
    { "7C0CF6", "Guangdong Huiwei High-tech Co., Ltd." },
    { "7C0CFA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7C0ECE", "Cisco Systems, Inc" },
    { "7C1015", "Brilliant Home Technology, Inc." },
    { "7C10C9", "ASUSTek COMPUTER INC." },
    { "7C11BE", "Apple, Inc." },
    { "7C11CB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7C11CD", "QianTang Technology" },
    { "7C131D", "SERNET (SUZHOU) TECHNOLOGIES CORPORATION" },
    { "7C1476", "Damall Technologies SAS" },
    { "7C152D", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "7C160D", "Saia-Burgess Controls AG" },
    { "7C1689", "Sagemcom Broadband SAS" },
    { "7C1779", "EM Microelectronic" },
    { "7C18CD", "E-TRON Co.,Ltd." },
    { "7C1A03", "8Locations Co., Ltd." },
    { "7C1AC0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7C1AFC", "Dalian Co-Edifice Video Technology Co., Ltd" },
    { "7C1B93", "Huawei Device Co., Ltd." },
    { "7C1C4E", "LG Innotek" },
    { "7C1C68", "Samsung Electronics Co.,Ltd" },
    { "7C1CF1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7C1DD9", "Xiaomi Communications Co Ltd" },
    { "7C1E06", "New H3C Technologies Co., Ltd" },
    { "7C1E4A", "FORTUNE MARKETING PRIVATE LIMITED" },
    { "7C1E52", "Microsoft" },
    { "7C1EB3", "2N TELEKOMUNIKACE a.s." },
    { "7C2048", "KoamTac" },
    { "7C2064", "Alcatel-Lucent IPD" },
    { "7C210D", "Cisco Systems, Inc" },
    { "7C210E", "Cisco Systems, Inc" },
    { "7C214A", "Intel Corporate" },
    { "7C21D8", "Shenzhen Think Will Communication Technology co., LTD." },
    { "7C2302", "Samsung Electronics Co.,Ltd" },
    { "7C240C", "Telechips, Inc." },
    { "7C2499", "Apple, Inc." },
    { "7C2586", "Juniper Networks" },
    { "7C2587", "chaowifi.com" },
    { "7C25DA", "FN-LINK TECHNOLOGY LIMITED" },
    { "7C2634", "Commscope" },
    { "7C2664", "Sagemcom Broadband SAS" },
    { "7C273C", "Shenzhen Yunlink Technology Co., Ltd" },
    { "7C27BC", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "7C296F", "Apple, Inc." },
    { "7C2A31", "Intel Corporate" },
    { "7C2ACA", "Apple, Inc." },
    { "7C2ADB", "Xiaomi Communications Co Ltd" },
    { "7C2BE1", "Shenzhen Ferex Electrical Co.,Ltd" },
    { "7C2C67", "Espressif Inc." },
    { "7C2CF3", "Secure Electrans Ltd" },
    { "7C2E0D", "Blackmagic Design" },
    { "7C2EBD", "Google, Inc." },
    { "7C2EDD", "Samsung Electronics Co.,Ltd" },
    { "7C2F80", "Gigaset Communications GmbH" },
    { "7C310E", "Cisco Systems, Inc" },
    { "7C3180", "SMK corporation" },
    { "7C31FA", "Silicon Laboratories" },
    { "7C336E", "MEG Electronics Inc." },
    { "7C33F9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7C3548", "Transcend Information" },
    { "7C35F8", "Zhejiang Tmall Technology Co., Ltd." },
    { "7C3626", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7C3866", "Texas Instruments" },
    { "7C386C", "Real Time Logic" },
    { "7C38AD", "Samsung Electronics Co.,Ltd" },
    { "7C3920", "SSOMA SECURITY" },
    { "7C3953", "zte corporation" },
    { "7C3985", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7C3B2D", "Apple, Inc." },
    { "7C3BD5", "Imago Group" },
    { "7C3CB6", "Shenzhen Homecare Technology Co.,Ltd." },
    { "7C3D2B", "Huawei Device Co., Ltd." },
    { "7C3E74", "Huawei Device Co., Ltd." },
    { "7C3E82", "Bouffalo Lab (Nanjing) Co., Ltd." },
    { "7C3E9D", "PATECH" },
    { "7C41A2", "Nokia" },
    { "7C438F", "E-Band Communications Corp." },
    { "7C444C", "Entertainment Solutions, S.L." },
    { "7C45D0", "Shenzhen Wewins Wireless Co., ltd" },
    { "7C45F9", "IEEE Registration Authority" },
    { "7C4685", "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd." },
    { "7C477C", "IEEE Registration Authority" },
    { "7C48B2", "Vida Resources Lte Ltd" },
    { "7C49B9", "Plexus Manufacturing Sdn Bhd" },
    { "7C49CF", "eero inc." },
    { "7C49EB", "XIAOMI Electronics,CO.,LTD" },
    { "7C4A82", "Portsmith LLC" },
    { "7C4AA8", "MindTree Wireless PVT Ltd" },
    { "7C4B26", "Apple, Inc." },
    { "7C4B78", "Red Sun Synthesis Pte Ltd" },
    { "7C4C58", "Scale Computing, Inc." },
    { "7C4CA5", "SKY UK LIMITED" },
    { "7C4D8F", "HP Inc." },
    { "7C4E09", "Shenzhen Skyworth Wireless Technology Co.,Ltd" },
    { "7C4F7D", "Sawwave" },
    { "7C4FB5", "Arcadyan Technology Corporation" },
    { "7C5049", "Apple, Inc." },
    { "7C5079", "Intel Corporate" },
    { "7C50DA", "E.J Ward" },
    { "7C5184", "Unis Flash Memory Technology(Chengdu)Co.,Ltd." },
    { "7C5189", "SG Wireless Limited" },
    { "7C5259", "Sichuan Jiuzhou Electronic Technology Co., Ltd." },
    { "7C534A", "Metamako" },
    { "7C55A7", "Kastle Systems" },
    { "7C55E7", "YSI, Inc." },
    { "7C573C", "Hewlett Packard Enterprise" },
    { "7C574E", "COBI GmbH" },
    { "7C5758", "HP Inc." },
    { "7C5A1C", "Sophos Ltd" },
    { "7C5A67", "JNC Systems, Inc." },
    { "7C5CF8", "Intel Corporate" },
    { "7C5E98", "eero inc." },
    { "7C604A", "Avelon" },
    { "7C6097", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7C60DB", "zte corporation" },
    { "7C6130", "Apple, Inc." },
    { "7C6166", "Amazon Technologies Inc." },
    { "7C6193", "HTC Corporation" },
    { "7C6305", "Amazon Technologies Inc." },
    { "7C6456", "Samsung Electronics Co.,Ltd" },
    { "7C646C", "LG Electronics" },
    { "7C669A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7C669D", "Texas Instruments" },
    { "7C66EF", "Hon Hai Precision IND.CO.,LTD" },
    { "7C67A2", "Intel Corporate" },
    { "7C67AB", "Roku, Inc" },
    { "7C68B9", "Huawei Device Co., Ltd." },
    { "7C692B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7C696B", "Atmosic Technologies" },
    { "7C69F6", "Cisco Systems, Inc" },
    { "7C6A60", "China Mobile Group Device Co.,Ltd." },
    { "7C6A8A", "SINOBONDER Technology Co., Ltd." },
    { "7C6AB3", "IBC TECHNOLOGIES INC." },
    { "7C6AC3", "GatesAir, Inc" },
    { "7C6ADB", "SafeTone Technology Co.,Ltd" },
    { "7C6AF3", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "7C6B33", "Tenyu Tech Co. Ltd." },
    { "7C6B52", "Tigaro Wireless" },
    { "7C6B9C", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "7C6BF7", "NTI co., ltd." },
    { "7C6C39", "PIXSYS SRL" },
    { "7C6C8F", "AMS NEVE LTD" },
    { "7C6CF0", "Shenzhen TINNO Mobile Technology Corp." },
    { "7C6D62", "Apple, Inc." },
    { "7C6DA6", "Superwave Group LLC" },
    { "7C6DF8", "Apple, Inc." },
    { "7C6F06", "Caterpillar Trimble Control Technologies" },
    { "7C6FF8", "ShenZhen ACTO Digital Video Technology Co.,Ltd." },
    { "7C70BC", "IEEE Registration Authority" },
    { "7C70DB", "Intel Corporate" },
    { "7C7176", "Wuxi iData Technology Company Ltd." },
    { "7C726E", "Ericsson AB" },
    { "7C72E4", "Unikey Technologies" },
    { "7C72E7", "Texas Instruments" },
    { "7C738B", "Cocoon Alarm Ltd" },
    { "7C7398", "Espressif Inc." },
    { "7C73EB", "Huawei Device Co., Ltd." },
    { "7C752D", "Samsung Electronics Co.,Ltd" },
    { "7C7630", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "7C7635", "Intel Corporate" },
    { "7C7668", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7C7673", "ENMAS GmbH" },
    { "7C7716", "Zyxel Communications Corporation" },
    { "7C787E", "Samsung Electronics Co.,Ltd" },
    { "7C78B2", "Wyze Labs Inc" },
    { "7C79E8", "PayRange Inc." },
    { "7C7A3C", "New H3C Technologies Co., Ltd" },
    { "7C7A53", "Phytrex Technology Corp." },
    { "7C7A91", "Intel Corporate" },
    { "7C7B1C", "Motorola Mobility LLC, a Lenovo Company" },
    { "7C7B68", "VIETNAM POST AND TELECOMMUNICATION INDUSTRY TECHNOLOGY JOIN STOCK COMPANY" },
    { "7C7B8B", "Control Concepts, Inc." },
    { "7C7BBF", "Samsung Electronics Co.,Ltd" },
    { "7C7BE4", "Z'SEDAI KENKYUSHO CORPORATION" },
    { "7C7D21", "zte corporation" },
    { "7C7D3D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7C7D41", "Jinmuyu Electronics Co., Ltd." },
    { "7C7EF9", "eero inc." },
    { "7C822D", "Nortec" },
    { "7C8274", "Shenzhen Hikeen Technology CO.,LTD" },
    { "7C8306", "Glen Dimplex Nordic as" },
    { "7C8334", "IEEE Registration Authority" },
    { "7C8437", "China Post Communications Equipment Co., Ltd." },
    { "7C8530", "Nokia" },
    { "7C87CE", "Espressif Inc." },
    { "7C8899", "FN-LINK TECHNOLOGY Ltd." },
    { "7C8931", "Huawei Device Co., Ltd." },
    { "7C8956", "Samsung Electronics Co.,Ltd" },
    { "7C89C1", "Palo Alto Networks" },
    { "7C8AE1", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "7C8BB5", "Samsung Electronics Co.,Ltd" },
    { "7C8BC1", "Infinix mobility limited" },
    { "7C8BCA", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "7C8C09", "Mellanox Technologies, Inc." },
    { "7C8D91", "Shanghai Hongzhuo Information Technology co.,LTD" },
    { "7C8D9C", "Edgecore Americas Networking Corporation" },
    { "7C8EE4", "Texas Instruments" },
    { "7C8FDE", "DWnet Technologies(Suzhou) Corporation" },
    { "7C9122", "Samsung Electronics Co.,Ltd" },
    { "7C942A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7C949F", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "7C94B2", "Philips Healthcare PCCI" },
    { "7C95B1", "Extreme Networks Headquarters" },
    { "7C95F3", "Cisco Systems, Inc" },
    { "7C96D2", "Fihonest communication co.,Ltd" },
    { "7C9763", "Openmatics s.r.o." },
    { "7C97E1", "Huawei Device Co., Ltd." },
    { "7C992E", "Shanghai Notion lnformatio Technology Co.,Ltd." },
    { "7C9946", "Sector Alarm Tech S.L." },
    { "7C9A1D", "Apple, Inc." },
    { "7C9A54", "Vantiva USA LLC" },
    { "7C9A9B", "VSE valencia smart energy" },
    { "7C9EBD", "Espressif Inc." },
    { "7C9F07", "CIG SHANGHAI CO LTD" },
    { "7CA15D", "GN ReSound A/S" },
    { "7CA177", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7CA1AE", "Apple, Inc." },
    { "7CA237", "King Slide Technology CO., LTD." },
    { "7CA23E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7CA29B", "D.SignT GmbH & Co. KG" },
    { "7CA449", "Xiaomi Communications Co Ltd" },
    { "7CA4F7", "Extreme Networks Headquarters" },
    { "7CA58F", "shenzhen Qikai Electronic Co., Ltd." },
    { "7CA61D", "MHL, LLC" },
    { "7CA62A", "Hewlett Packard Enterprise " },
    { "7CA7B0", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "7CA8EC", "Hewlett Packard Enterprise" },
    { "7CA909", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "7CA96B", "Syrotech Networks. Ltd." },
    { "7CA97D", "Objenious" },
    { "7CAADE", "AltoBeam Inc." },
    { "7CAB25", "MESMO TECHNOLOGY INC." },
    { "7CAB60", "Apple, Inc." },
    { "7CACB2", "Bosch Software Innovations GmbH" },
    { "7CAD4F", "Cisco Systems, Inc" },
    { "7CAD74", "Cisco Systems, Inc" },
    { "7CB03E", "OSRAM GmbH" },
    { "7CB073", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "7CB0C2", "Intel Corporate" },
    { "7CB15D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7CB177", "Satelco AG" },
    { "7CB21B", "Cisco SPVTG" },
    { "7CB232", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "7CB25C", "Acacia Communications" },
    { "7CB27D", "Intel Corporate" },
    { "7CB30A", "zte corporation" },
    { "7CB353", "Cisco Systems, Inc" },
    { "7CB37B", "Qingdao Intelligent&Precise Electronics Co.,Ltd." },
    { "7CB542", "ACES Technology" },
    { "7CB566", "Intel Corporate" },
    { "7CB59B", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "7CB59F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7CB68D", "Mist Systems, Inc." },
    { "7CB733", "ASKEY COMPUTER CORP" },
    { "7CB77B", "Paradigm Electronics Inc" },
    { "7CB8DA", "Murata Manufacturing Co., Ltd." },
    { "7CB8E6", "GREE ELECTRIC APPLIANCES, INC. OF ZHUHAI" },
    { "7CB94C", "Bouffalo Lab (Nanjing) Co., Ltd." },
    { "7CB960", "Shanghai X-Cheng telecom LTD" },
    { "7CBAC0", "EVBox BV" },
    { "7CBAC6", "Solar Manager AG" },
    { "7CBACC", "IEEE Registration Authority" },
    { "7CBB6F", "Cosco Electronics Co., Ltd." },
    { "7CBB8A", "Nintendo Co., Ltd." },
    { "7CBC84", "IEEE Registration Authority" },
    { "7CBD06", "AE REFUsol" },
    { "7CBECE", "Private" },
    { "7CBF77", "SPEEDTECH CORP." },
    { "7CBF88", "Mobilicom LTD" },
    { "7CBFAE", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "7CBFB1", "Commscope" },
    { "7CC025", "Palo Alto Networks" },
    { "7CC06F", "Apple, Inc." },
    { "7CC0AA", "Microsoft Corporation" },
    { "7CC177", "INGRAM MICRO SERVICES" },
    { "7CC178", "Ericsson AB" },
    { "7CC180", "Apple, Inc." },
    { "7CC225", "Samsung Electronics Co.,Ltd" },
    { "7CC255", "Super Micro Computer, Inc." },
    { "7CC294", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "7CC2C6", "TP-Link Systems Inc" },
    { "7CC385", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7CC3A1", "Apple, Inc." },
    { "7CC4EF", "Devialet" },
    { "7CC518", "vivo Mobile Communication Co., Ltd." },
    { "7CC537", "Apple, Inc." },
    { "7CC6B6", "Silicon Laboratories" },
    { "7CC6C4", "Kolff Computer Supplies b.v." },
    { "7CC709", "SHENZHEN RF-LINK TECHNOLOGY CO.,LTD." },
    { "7CC74A", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "7CC77E", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "7CC790", "Palo Alto Networks" },
    { "7CC882", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7CC8AB", "Acro Associates, Inc." },
    { "7CC8D0", "TIANJIN YAAN TECHNOLOGY CO., LTD." },
    { "7CC8D7", "Damalisk" },
    { "7CC8DF", "Apple, Inc." },
    { "7CC926", "Wuhan GreeNet Information Service Co.,Ltd." },
    { "7CC95A", "Dell EMC" },
    { "7CC95E", "Dongguan Liesheng Electronic Co., Ltd." },
    { "7CCB0D", "Antaira Technologies, LLC" },
    { "7CCBE2", "IEEE Registration Authority" },
    { "7CCC1F", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "7CCCB8", "Intel Corporate" },
    { "7CCCFC", "Quectel Wireless Solutions Co.,Ltd." },
    { "7CCD11", "MS-Magnet" },
    { "7CCD3C", "Guangzhou Juzing Technology Co., Ltd" },
    { "7CCFCF", "Shanghai SEARI Intelligent System Co., Ltd" },
    { "7CD1C3", "Apple, Inc." },
    { "7CD2DA", "Apple, Inc." },
    { "7CD30A", "INVENTEC CORPORATION" },
    { "7CD3E5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7CD44D", "Shanghai Moorewatt Energy Technology Co.,Ltd" },
    { "7CD566", "Amazon Technologies Inc." },
    { "7CD661", "Xiaomi Communications Co Ltd" },
    { "7CD762", "Freestyle Technology Pty Ltd" },
    { "7CD844", "Enmotus Inc" },
    { "7CD95C", "Google, Inc." },
    { "7CD9A0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7CD9F4", "UAB Teltonika Telematics" },
    { "7CD9FE", "New Cosmos Electric Co., Ltd." },
    { "7CDA84", "Dongnian Networks Inc." },
    { "7CDAC3", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "7CDB98", "ASKEY COMPUTER CORP" },
    { "7CDC73", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7CDCCC", "BEIJING STARBLAZE TECHNOLOGY CO.,LTD" },
    { "7CDD11", "Chongqing MAS SCI&TECH.Co.,Ltd" },
    { "7CDD20", "IOXOS Technologies S.A." },
    { "7CDD76", "Suzhou Hanming Technologies Co., Ltd." },
    { "7CDD90", "Shenzhen Ogemray Technology Co., Ltd." },
    { "7CDDE9", "ATOM tech Inc." },
    { "7CDE78", "New H3C Technologies Co., Ltd" },
    { "7CDFA1", "Espressif Inc." },
    { "7CE044", "NEON Inc" },
    { "7CE152", "THE GOODYEAR TIRE & RUBBER COMPANY" },
    { "7CE1FF", "Computer Performance, Inc. DBA Digital Loggers, Inc." },
    { "7CE269", "Texas Instruments" },
    { "7CE2CA", "Juniper Networks" },
    { "7CE4AA", "Private" },
    { "7CE524", "Quirky, Inc." },
    { "7CE53F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "7CE56B", "ESEN Optoelectronics Technology Co.,Ltd." },
    { "7CE712", "Quectel Wireless Solutions Co.,Ltd." },
    { "7CE87F", "Sagemcom Broadband SAS" },
    { "7CE913", "Fantasia Trading LLC" },
    { "7CE97C", "ITEL MOBILE LIMITED" },
    { "7CE9D3", "Hon Hai Precision Ind. Co.,Ltd." },
    { "7CEB7F", "Dmet Products Corp." },
    { "7CEBAE", "Ridgeline Instruments" },
    { "7CEBEA", "ASCT" },
    { "7CEC79", "Texas Instruments" },
    { "7CEC9B", "Fuzhou Teraway Information Technology Co.,Ltd" },
    { "7CECB1", "Apple, Inc." },
    { "7CED8D", "Microsoft" },
    { "7CEDC6", "Amazon Technologies Inc." },
    { "7CEE7B", "Logically Us Ltd" },
    { "7CEF18", "Creative Product Design Pty. Ltd." },
    { "7CEF40", "Nextorage Corporation" },
    { "7CEF61", "STR Elektronik Josef Schlechtinger GmbH" },
    { "7CEF8A", "Inhon International Ltd." },
    { "7CF05F", "Apple, Inc." },
    { "7CF098", "Bee Beans Technologies, Inc." },
    { "7CF0BA", "Linkwell Telesystems Pvt Ltd" },
    { "7CF0E5", "OnePlus Technology (Shenzhen) Co., Ltd" },
    { "7CF17E", "TP-Link Systems Inc" },
    { "7CF2DD", "Vence Corp" },
    { "7CF31B", "LG Electronics (Mobile Communications)" },
    { "7CF34D", "Apple, Inc." },
    { "7CF429", "NUUO Inc. " },
    { "7CF462", "BEIJING HUAWOO TECHNOLOGIES CO.LTD" },
    { "7CF666", "Tuya Smart Inc." },
    { "7CF854", "Samsung Electronics Co.,Ltd" },
    { "7CF880", "Cisco Systems, Inc" },
    { "7CF90E", "Samsung Electronics Co.,Ltd" },
    { "7CF95C", "U.I. Lapp GmbH" },
    { "7CF9A0", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "7CFA80", "JiangSu Fulian Communication Technology Co., Ltd" },
    { "7CFAD6", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "7CFADF", "Apple, Inc." },
    { "7CFC16", "Apple, Inc." },
    { "7CFC3C", "Visteon Corporation" },
    { "7CFCFD", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "7CFD6B", "Xiaomi Communications Co Ltd" },
    { "7CFD82", "GUANGDONG GENIUS TECHNOLOGY CO., LTD." },
    { "7CFE28", "Salutron Inc." },
    { "7CFE4E", "Shenzhen Safe vision Technology Co.,LTD" },
    { "7CFE62", "ShenZhen XinZhongXin Technology Co.， Ltd" },
    { "7CFE90", "Mellanox Technologies, Inc." },
    { "7CFF4D", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "7CFF62", "Huizhou Super Electron Technology Co.,Ltd." },
    { "80000B", "Intel Corporate" },
    { "800010", "AT&T" },
    { "80006E", "Apple, Inc." },
    { "80015C", "Synaptics, Inc" },
    { "800184", "HTC Corporation" },
    { "80029C", "Gemtek Technology Co., Ltd." },
    { "8002DF", "ORA Inc." },
    { "8002F4", "IEEE Registration Authority" },
    { "80030D", "CANON INC." },
    { "800384", "Ruckus Wireless" },
    { "80045F", "Apple, Inc." },
    { "800518", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "80051F", "Vizio, Inc" },
    { "80053A", "CHeKT Inc." },
    { "800588", "Ruijie Networks Co.,LTD" },
    { "8005DF", "Montage Technology Group Limited" },
    { "8006D9", "zte corporation" },
    { "80071B", "VSOLUTION TELECOMMUNICATION TECHNOLOGY CO.,LTD." },
    { "800794", "Samsung Electronics Co.,Ltd" },
    { "8007A2", "Esson Technology Inc." },
    { "800902", "Keysight Technologies, Inc." },
    { "800A06", "COMTEC co.,ltd" },
    { "800A80", "IEEE Registration Authority" },
    { "800B51", "Chengdu XGimi Technology Co.,Ltd" },
    { "800C67", "Apple, Inc." },
    { "800CF9", "Amazon Technologies Inc." },
    { "800DD7", "Latticework, Inc" },
    { "800E24", "ForgetBox" },
    { "800EA9", "TCL Yuxin Zhixing Technology (Huizhou) Co.,Ltd" },
    { "801242", "Apple, Inc." },
    { "8012DF", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "801382", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "801440", "Sunlit System Technology Corp" },
    { "8014A8", "Guangzhou V-SOLUTION Electronic Technology Co., Ltd." },
    { "801605", "Vodafone Italia S.p.A." },
    { "801609", "Sleep Number" },
    { "8016B7", "Brunel University" },
    { "80177D", "Nortel Networks" },
    { "801844", "Dell Inc." },
    { "8018A7", "Samsung Electronics Co.,Ltd" },
    { "801934", "Intel Corporate" },
    { "801967", "Shanghai Reallytek Information Technology Co.,Ltd" },
    { "801970", "Samsung Electronics Co.,Ltd" },
    { "8019FE", "JianLing Technology CO., LTD" },
    { "801DAA", "Avaya Inc" },
    { "801F02", "Edimax Technology Co. Ltd." },
    { "801F12", "Microchip Technology Inc." },
    { "8020AF", "Trade FIDES, a.s." },
    { "8020DA", "Sagemcom Broadband SAS" },
    { "8020E1", "BVBA DPTechnics" },
    { "8020FD", "Samsung Electronics Co.,Ltd" },
    { "802275", "Beijing Beny Wave Technology Co Ltd" },
    { "802278", "China Mobile IOT Company Limited" },
    { "8022A7", "NEC Platforms, Ltd." },
    { "8022FA", "ITEL MOBILE LIMITED" },
    { "802395", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "80248F", "Cisco Systems, Inc" },
    { "802511", "ITEL MOBILE LIMITED" },
    { "802689", "D-Link International" },
    { "80276C", "Cisco Systems, Inc" },
    { "80283C", "Sonova AG" },
    { "802994", "Vantiva USA LLC" },
    { "802AA8", "Ubiquiti Inc" },
    { "802AFA", "Germaneers GmbH" },
    { "802BF9", "Hon Hai Precision Ind. Co.,Ltd." },
    { "802D1A", "zte corporation" },
    { "802DBF", "Cisco Systems, Inc" },
    { "802DE1", "Solarbridge Technologies" },
    { "802E14", "azeti Networks AG" },
    { "802EC3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "802EDE", "Huawei Device Co., Ltd." },
    { "802FDE", "Zurich Instruments AG" },
    { "803049", "Liteon Technology Corporation" },
    { "8030DC", "Texas Instruments" },
    { "8030E0", "Hewlett Packard Enterprise " },
    { "8031F0", "Samsung Electronics Co.,Ltd" },
    { "803253", "Intel Corporate" },
    { "803428", "Microchip Technology Inc." },
    { "803457", "OT Systems Limited" },
    { "8035C1", "Xiaomi Communications Co Ltd" },
    { "803773", "NETGEAR" },
    { "803896", "SHARP Corporation" },
    { "8038BC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "8038D4", "Fibercentury Network Technology Co.,Ltd." },
    { "8038FB", "Intel Corporate" },
    { "8038FD", "LeapFrog Enterprises, Inc." },
    { "80398C", "Samsung Electronics Co.,Ltd" },
    { "8039E5", "PATLITE CORPORATION" },
    { "803A0A", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "803A59", "AT&T" },
    { "803AF4", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "803B2A", "ABB Xiamen Low Voltage Equipment Co.,Ltd." },
    { "803B9A", "ghe-ces electronic ag" },
    { "803BF6", "LOOK EASY INTERNATIONAL LIMITED" },
    { "803C04", "TP-Link Systems Inc." },
    { "803C20", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "803E48", "SHENZHEN GONGJIN ELECTRONICS CO.,LT" },
    { "803E4F", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "803F5D", "Winstars Technology Ltd" },
    { "803FD6", "bytes at work AG" },
    { "804005", "Guangdong COROS Sports Technology Co.,Ltd" },
    { "804126", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "80414E", "BBK EDUCATIONAL ELECTRONICS CORP.,LTD." },
    { "80427C", "Adolf Tedsen GmbH & Co. KG" },
    { "80433F", "Juniper Networks" },
    { "8044FD", "China Mobile (Hangzhou) Information Technology Co., Ltd." },
    { "8045DD", "Intel Corporate" },
    { "804731", "Packet Design, Inc." },
    { "804786", "Samsung Electronics Co.,Ltd" },
    { "80482C", "Wyze Labs Inc" },
    { "80489F", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "8048A5", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "804971", "Apple, Inc." },
    { "804A14", "Apple, Inc." },
    { "804AF2", "Sonos, Inc." },
    { "804B20", "Ventilation Control" },
    { "804B50", "Silicon Laboratories" },
    { "804C5D", "NXP Semiconductor (Tianjin) LTD." },
    { "804E70", "Samsung Electronics Co.,Ltd" },
    { "804E81", "Samsung Electronics Co.,Ltd" },
    { "804F58", "ThinkEco, Inc." },
    { "80501B", "Nokia Corporation" },
    { "805067", "W & D TECHNOLOGY CORPORATION" },
    { "8050F6", "ITEL MOBILE LIMITED" },
    { "80542D", "Samsung Electronics Co.,Ltd" },
    { "80546A", "SHENZHEN GONGJIN ELECTRONICS CO.,LT" },
    { "80549C", "Samsung Electronics Co.,Ltd" },
    { "8054D9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "8054E3", "Apple, Inc." },
    { "80563C", "ZF" },
    { "8056F2", "Hon Hai Precision Ind. Co.,Ltd." },
    { "805719", "Samsung Electronics Co.,Ltd" },
    { "8058C5", "NovaTec Kommunikationstechnik GmbH" },
    { "8058F8", "Motorola Mobility LLC, a Lenovo Company" },
    { "8059FD", "Noviga" },
    { "805A04", "LG Electronics (Mobile Communications)" },
    { "805B65", "LG Innotek" },
    { "805E0C", "YEALINK(XIAMEN) NETWORK TECHNOLOGY CO.,LTD." },
    { "805E4F", "FN-LINK TECHNOLOGY LIMITED" },
    { "805EC0", "YEALINK(XIAMEN) NETWORK TECHNOLOGY CO.,LTD." },
    { "805F8E", "Huizhou BYD Electronic Co., Ltd." },
    { "805FC5", "Apple, Inc." },
    { "806007", "RIM" },
    { "806036", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "8060B7", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "80615F", "Beijing Sinead Technology Co., Ltd. " },
    { "80616C", "New H3C Technologies Co., Ltd" },
    { "80618F", "Shenzhen sangfei consumer communications co.,ltd" },
    { "806459", "Nimbus Inc." },
    { "80646F", "Espressif Inc." },
    { "80647A", "Ola Sense Inc" },
    { "80647C", "Tuya Smart Inc." },
    { "806559", "EM Microelectronic" },
    { "80656D", "Samsung Electronics Co.,Ltd" },
    { "80657C", "Apple, Inc." },
    { "806599", "Espressif Inc." },
    { "8065E9", "BenQ Corporation" },
    { "806629", "Prescope Technologies CO.,LTD." },
    { "80691A", "Belkin International Inc." },
    { "806933", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "806940", "LEXAR CO.,LIMITED" },
    { "806A00", "Cisco Systems, Inc" },
    { "806A10", "Whisker Labs - Ting" },
    { "806A34", "Bouffalo Lab (Nanjing) Co., Ltd." },
    { "806AB0", "Shenzhen TINNO Mobile Technology Corp." },
    { "806C1B", "Motorola Mobility LLC, a Lenovo Company" },
    { "806C8B", "KAESER KOMPRESSOREN AG" },
    { "806CBC", "NET New Electronic Technology GmbH" },
    { "806D71", "Amazon Technologies Inc." },
    { "806D97", "Private" },
    { "806F1C", "Huawei Device Co., Ltd." },
    { "806FB0", "Texas Instruments" },
    { "80711F", "Juniper Networks" },
    { "80717A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "807215", "SKY UK LIMITED" },
    { "807264", "Huawei Device Co., Ltd." },
    { "80739F", "KYOCERA CORPORATION " },
    { "807459", "K's Co.,Ltd." },
    { "807484", "ALL Winner (Hong Kong) Limited" },
    { "80751F", "SKY UK LIMITED" },
    { "8075BF", "Samsung Electronics Co.,Ltd" },
    { "807677", "hangzhou puwell cloud tech co., ltd." },
    { "807693", "Newag SA" },
    { "8076C2", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "8077A4", "TECNO MOBILE LIMITED" },
    { "807871", "ASKEY COMPUTER CORP" },
    { "807933", "Aigentec Technology(Zhejiang) Co., Ltd." },
    { "80795D", "Infinix mobility limited" },
    { "8079AE", "ShanDong Tecsunrise Co.,Ltd" },
    { "807A7F", "ABB Genway Xiamen Electrical Equipment CO., LTD" },
    { "807ABF", "HTC Corporation" },
    { "807B1E", "Corsair Memory, Inc." },
    { "807B3E", "Samsung Electronics Co.,Ltd" },
    { "807B85", "IEEE Registration Authority" },
    { "807C0A", "zte corporation" },
    { "807C62", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "807D14", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "807D1B", "Neosystem Co. Ltd." },
    { "807D3A", "Espressif Inc." },
    { "807DE3", "Chongqing Sichuan Instrument Microcircuit Co.LTD." },
    { "807EB4", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "807FF8", "Juniper Networks" },
    { "80802C", "Fortinet, Inc." },
    { "8081A5", "TONGQING COMMUNICATION EQUIPMENT (SHENZHEN) Co.,Ltd" },
    { "808223", "Apple, Inc." },
    { "808287", "ATCOM Technology Co.Ltd." },
    { "8082F5", "STMicrolectronics International NV" },
    { "8082FE", "Arcadyan Corporation" },
    { "8083F6", "Apple, Inc." },
    { "808489", "Intel Corporate" },
    { "8084A9", "oshkosh Corporation" },
    { "808544", "Intelbras" },
    { "808698", "Netronics Technologies Inc." },
    { "8086D9", "Samsung Electronics Co.,Ltd" },
    { "8086F2", "Intel Corporate" },
    { "808800", "zte corporation" },
    { "808917", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "808A8B", "vivo Mobile Communication Co., Ltd." },
    { "808ABD", "Samsung Electronics Co.,Ltd" },
    { "808AF7", "Nanoleaf" },
    { "808B5C", "Shenzhen Runhuicheng Technology Co., Ltd" },
    { "808C97", "Kaon Group Co., Ltd." },
    { "808DB7", "Hewlett Packard Enterprise " },
    { "808F1D", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "808FE8", "Intelbras" },
    { "80912A", "Lih Rong electronic Enterprise Co., Ltd." },
    { "809133", "AzureWave Technology Inc." },
    { "8091C0", "AgileMesh, Inc." },
    { "80929F", "Apple, Inc." },
    { "8092A5", "Valeo Interior Controls (Shenzhen) Co.,Ltd" },
    { "809393", "Xapt GmbH" },
    { "80946C", "TOKYO RADAR CORPORATION" },
    { "80953A", "Apple, Inc." },
    { "809562", "Extreme Networks Headquarters" },
    { "809621", "Lenovo" },
    { "809698", "Apple, Inc." },
    { "8096B1", "Commscope" },
    { "8096CA", "Hon Hai Precision Ind. Co.,Ltd." },
    { "80971B", "Altenergy Power System,Inc." },
    { "809733", "Shenzhen Elebao Technology Co., Ltd" },
    { "8099E7", "Sony Corporation" },
    { "809B20", "Intel Corporate" },
    { "809D65", "FN-LINK TECHNOLOGY Ltd." },
    { "809F9B", "Sichuan AI-Link Technology Co., Ltd." },
    { "809FAB", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "809FF5", "Samsung Electronics Co.,Ltd" },
    { "80A036", "Shanghai MXCHIP Information Technology Co., Ltd." },
    { "80A197", "u-blox AG" },
    { "80A1AB", "Intellisis" },
    { "80A1D7", "Shanghai DareGlobal Technologies Co.,Ltd" },
    { "80A235", "Edgecore Networks Corporation" },
    { "80A579", "IEEE Registration Authority" },
    { "80A589", "AzureWave Technology Inc." },
    { "80A796", "Neuralink Corp." },
    { "80A85D", "Osterhout Design Group" },
    { "80A997", "Apple, Inc." },
    { "80AA1C", "Luxottica Tristar (Dongguan) Optical Co.,Ltd" },
    { "80AAA4", "USAG" },
    { "80AB4D", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "80AC7C", "Sichuan AI-Link Technology Co., Ltd." },
    { "80ACAC", "Juniper Networks" },
    { "80ACC8", "Phyplus Microelectronics Limited" },
    { "80AD16", "Xiaomi Communications Co Ltd" },
    { "80AD67", "Kasda Networks Inc" },
    { "80AE3C", "Taicang T&W Electronics" },
    { "80AE54", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "80AF19", "Apple, Inc." },
    { "80AFCA", "Shenzhen Cudy Technology Co., Ltd." },
    { "80B03D", "Apple, Inc." },
    { "80B07B", "zte corporation" },
    { "80B219", "ELEKTRON TECHNOLOGY UK LIMITED" },
    { "80B234", "Vantiva USA LLC" },
    { "80B289", "Forworld Electronics Ltd." },
    { "80B32A", "UK Grid Solutions Ltd" },
    { "80B54E", "Espressif Inc." },
    { "80B575", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "80B624", "IVS" },
    { "80B655", "Intel Corporate" },
    { "80B686", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "80B708", "Blue Danube Systems, Inc" },
    { "80B709", "Viptela, Inc" },
    { "80B745", "The Silk Technologies ILC LTD" },
    { "80B946", "Nokia" },
    { "80B95C", "ELFTECH Co., Ltd." },
    { "80B97A", "eero inc." },
    { "80B989", "Apple, Inc." },
    { "80BA16", "Micas Networks Inc." },
    { "80BAAC", "TeleAdapt Ltd" },
    { "80BAE6", "Neets" },
    { "80BBEB", "Satmap Systems Ltd" },
    { "80BC37", "Ruckus Wireless" },
    { "80BE05", "Apple, Inc." },
    { "80BEAF", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "80C01E", "Intel Corporate" },
    { "80C16E", "Hewlett Packard" },
    { "80C3BA", "Sonova Consumer Hearing GmbH" },
    { "80C41B", "Texas Instruments" },
    { "80C45D", "IPG Laser GmbH" },
    { "80C501", "OctoGate IT Security Systems GmbH" },
    { "80C548", "Shenzhen Zowee Technology Co.,Ltd" },
    { "80C5E6", "Microsoft Corporation" },
    { "80C5F2", "AzureWave Technology Inc." },
    { "80C63F", "Remec Broadband Wireless , LLC" },
    { "80C6AB", "Vantiva USA LLC" },
    { "80C6CA", "Endian s.r.l." },
    { "80C755", "Panasonic Appliances Company" },
    { "80C7C5", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "80C862", "Openpeak, Inc" },
    { "80C955", "Redpine Signals, Inc." },
    { "80CA4B", "SHENZHEN GONGJIN ELECTRONICS CO.,LTD" },
    { "80CA52", "Wistron InfoComm(Chongqing)Co.,Ltd." },
    { "80CBBC", "Qingdao Intelligent&Precise Electronics Co.,Ltd." },
    { "80CC12", "Huawei Device Co., Ltd." },
    { "80CC9C", "NETGEAR" },
    { "80CE62", "Hewlett Packard" },
    { "80CEB1", "Theissen Training Systems GmbH" },
    { "80CEB9", "Samsung Electronics Co.,Ltd" },
    { "80CF41", "Lenovo Mobile Communication Technology Ltd." },
    { "80CFA2", "Huawei Device Co., Ltd." },
    { "80D019", "Embed, Inc" },
    { "80D04A", "Vantiva USA LLC" },
    { "80D065", "CKS Corporation" },
    { "80D09B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "80D10A", "Sichuan AI-Link Technology Co., Ltd." },
    { "80D160", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "80D18B", "Hangzhou I'converge Technology Co.,Ltd" },
    { "80D1CE", "Apple, Inc." },
    { "80D21D", "AzureWave Technology Inc." },
    { "80D266", "ScaleFlux" },
    { "80D2E5", "Nintendo Co.,Ltd" },
    { "80D336", "CERN" },
    { "80D433", "LzLabs GmbH" },
    { "80D4A5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "80D52C", "Beijing Cheering Networks Technology Co.,Ltd." },
    { "80D605", "Apple, Inc." },
    { "80D733", "QSR Automations, Inc." },
    { "80DA13", "eero inc." },
    { "80DABC", "Megafone Limited" },
    { "80DAC2", "Vantiva USA LLC" },
    { "80DB17", "Juniper Networks" },
    { "80DB31", "Power Quotient International Co., Ltd." },
    { "80DECC", "HYBE Co.,LTD" },
    { "80E01D", "Cisco Systems, Inc" },
    { "80E1BF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "80E455", "New H3C Technologies Co., Ltd" },
    { "80E4BA", "Intel Corporate" },
    { "80E4DA", "IEEE Registration Authority" },
    { "80E540", "Commscope" },
    { "80E650", "Apple, Inc." },
    { "80E769", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "80E82C", "Hewlett Packard" },
    { "80E86F", "Cisco Systems, Inc" },
    { "80E94A", "LEAPS s.r.o." },
    { "80EA07", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "80EA0B", "Zyxel Communications Corporation" },
    { "80EA23", "Wistron Neweb Corporation" },
    { "80EA96", "Apple, Inc." },
    { "80EACA", "Dialog Semiconductor Hellas SA" },
    { "80EB77", "Wistron Corporation" },
    { "80ED2C", "Apple, Inc." },
    { "80EE25", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "80EE73", "Shuttle Inc." },
    { "80F0CF", "Ruckus Wireless" },
    { "80F1A4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "80F1B2", "Espressif Inc." },
    { "80F1F1", "Tech4home, Lda" },
    { "80F25E", "Kyynel" },
    { "80F3DA", "Espressif Inc." },
    { "80F3EF", "Meta Platforms, Inc." },
    { "80F416", "Chipsea Technologies (Shenzhen) Corp." },
    { "80F503", "Commscope" },
    { "80F593", "IRCO Sistemas de Telecomunicación S.A." },
    { "80F5AE", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "80F5B5", "Texas Instruments" },
    { "80F62E", "Hangzhou H3C Technologies Co., Limited" },
    { "80F7A6", "Shenzhen C-Data Technology Co., Ltd." },
    { "80F8EB", "RayTight" },
    { "80FA5B", "CLEVO CO." },
    { "80FB06", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "80FBF0", "Quectel Wireless Solutions Co.,Ltd." },
    { "80FBF1", "Freescale Semiconductor (China) Ltd." },
    { "80FD7A", "BLU Products Inc" },
    { "80FD7B", "BLU Products Inc" },
    { "80FFA8", "UNIDIS" },
    { "84002D", "PEGATRON CORPORATION" },
    { "8400D2", "Sony Corporation" },
    { "840112", "Kaon Group Co., Ltd." },
    { "8401A7", "Greyware Automation Products, Inc" },
    { "840283", "HUMAX Co., Ltd." },
    { "840328", "Juniper Networks" },
    { "8404D2", "Kirale Technologies SL" },
    { "840511", "Apple, Inc." },
    { "8406FA", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "8407C4", "Walter Kidde Portable Equipment, Inc." },
    { "840A9E", "Nexapp Technologies Pvt Ltd" },
    { "840B2D", "SAMSUNG ELECTRO MECHANICS CO., LTD." },
    { "840B7C", "Hitron Technologies. Inc" },
    { "840BBB", "MitraStar Technology Corp." },
    { "840C6D", "Cisco Systems, Inc" },
    { "840D8E", "Espressif Inc." },
    { "840F2A", "Jiangxi Risound Electronics Co.,LTD" },
    { "840F45", "Shanghai GMT Digital Technologies Co., Ltd" },
    { "840F4C", "Apple, Inc." },
    { "84100D", "Motorola Mobility LLC, a Lenovo Company" },
    { "84119E", "Samsung Electronics Co.,Ltd" },
    { "8411C2", "IEEE Registration Authority" },
    { "84139F", "zte corporation" },
    { "84144D", "Intel Corporate" },
    { "841571", "TCT mobile ltd" },
    { "8415D3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "84160C", "Broadcom Limited" },
    { "8416F9", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "841715", "GP Electronics (HK) Ltd." },
    { "841766", "WEIFANG GOERTEK ELECTRONICS CO.,LTD" },
    { "8417EF", "Vantiva USA LLC" },
    { "841826", "Osram GmbH" },
    { "84183A", "Ruckus Wireless" },
    { "841888", "Juniper Networks" },
    { "841A24", "UNIONMAN TECHNOLOGY CO.,LTD" },
    { "841B38", "Shenzhen Excelsecu Data Technology Co.,Ltd" },
    { "841B5E", "NETGEAR" },
    { "841B77", "Intel Corporate" },
    { "841C70", "zte corporation" },
    { "841E26", "KERNEL-I Co.,LTD" },
    { "841EA3", "Sagemcom Broadband SAS" },
    { "841FE8", "Espressif Inc." },
    { "842096", "SHENZHEN RF-LINK TECHNOLOGY CO.,LTD." },
    { "842141", "Shenzhen Ginwave Technologies Ltd." },
    { "8421F1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "84225E", "SHENZHEN TECHNEWCHIP TECHNOLOGY CO.,LTD." },
    { "842289", "Samsung Electronics Co.,Ltd" },
    { "842388", "Ruckus Wireless" },
    { "84248D", "Zebra Technologies Inc" },
    { "842519", "Samsung Electronics" },
    { "84253F", "silex technology, Inc." },
    { "8425A4", "Tariox Limited" },
    { "8425DB", "Samsung Electronics Co.,Ltd" },
    { "842615", "ADB Broadband Italia" },
    { "84262B", "Nokia" },
    { "84267A", "GUANGDONG TAIDE ZHILIAN TECHNOLOGY CO.,LTD" },
    { "842690", "BEIJING THOUGHT SCIENCE CO.,LTD." },
    { "842712", "Silicon Laboratories" },
    { "8427B6", "China Mobile IOT Company Limited" },
    { "8427CE", "Corporation of the Presiding Bishop of The Church of Jesus Christ of Latter-day Saints" },
    { "842859", "Amazon Technologies Inc." },
    { "84285A", "Saffron Solutions Inc" },
    { "8428D6", "SERVERCOM (INDIA) PRIVATE LIMITED" },
    { "842914", "EMPORIA TELECOM Produktions- und VertriebsgesmbH & Co KG" },
    { "842999", "Apple, Inc." },
    { "842AFD", "HP Inc." },
    { "842B2B", "Dell Inc." },
    { "842B50", "Huria Co.,Ltd." },
    { "842BBC", "Modelleisenbahn GmbH" },
    { "842C80", "Sichuan Changhong Electric Ltd." },
    { "842E14", "Silicon Laboratories" },
    { "842E27", "Samsung Electronics Co.,Ltd" },
    { "842F57", "Apple, Inc." },
    { "842F75", "Innokas Group" },
    { "843095", "Hon Hai Precision IND.CO.,LTD" },
    { "8430CE", "Shenzhen Jaguar Microsystems Co., Ltd" },
    { "8430E5", "SkyHawke Technologies, LLC" },
    { "8431A8", "Funshion Online Technologies Co.,Ltd" },
    { "84326F", "GUANGZHOU AVA ELECTRONICS TECHNOLOGY CO.,LTD " },
    { "8432EA", "ANHUI WANZTEN P&T CO., LTD" },
    { "8433F2", "Shenzhen Stellamore Technology Co.,Ltd" },
    { "843497", "Hewlett Packard" },
    { "843611", "hyungseul publishing networks" },
    { "8437D5", "Samsung Electronics Co.,Ltd" },
    { "843835", "Apple, Inc." },
    { "843838", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "84398F", "Fortinet, Inc." },
    { "8439FC", "Nokia" },
    { "843A4B", "Intel Corporate" },
    { "843A5B", "Inventec(Chongqing) Corporation" },
    { "843B10", "LVSWITCHES INC." },
    { "843C4C", "Robert Bosch SRL" },
    { "843C99", "zte corporation" },
    { "843DC6", "Cisco Systems, Inc" },
    { "843E03", "Sagemcom Broadband SAS" },
    { "843E1D", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "843E79", "Shenzhen Belon Technology CO.,LTD" },
    { "843E92", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "843F4E", "Tri-Tech Manufacturing, Inc." },
    { "844076", "Drivenets" },
    { "844167", "Apple, Inc." },
    { "844464", "ServerU Inc" },
    { "8444AF", "Zhejiang Tmall Technology Co., Ltd." },
    { "844693", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "8446FE", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "844709", "Shenzhen IP3 Century Intelligent Technology CO.,Ltd" },
    { "844765", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "844823", "WOXTER TECHNOLOGY Co. Ltd" },
    { "844915", "vArmour Networks, Inc." },
    { "8449EE", "Demant Enterprise A/S" },
    { "844BB7", "Beijing Sankuai Online Technology Co.,Ltd" },
    { "844BF5", "Hon Hai Precision Ind. Co.,Ltd." },
    { "844D4C", "Adtran Inc" },
    { "844DBE", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "844F03", "Ablelink Electronics Ltd" },
    { "845075", "Huawei Device Co., Ltd." },
    { "84509A", "Easy Soft TV Co., Ltd" },
    { "845181", "Samsung Electronics Co.,Ltd" },
    { "845234", "Juniper Networks" },
    { "8453CD", "China Mobile Group Device Co.,Ltd." },
    { "8454DF", "Huawei Device Co., Ltd." },
    { "8455A5", "Samsung Electronics Co.,Ltd" },
    { "84569C", "Coho Data, Inc.," },
    { "845733", "Microsoft Corporation" },
    { "845787", "DVR C&C Co., Ltd." },
    { "8457F7", "Meta Platforms, Inc." },
    { "845A3E", "Cisco Systems, Inc" },
    { "845A81", "ffly4u" },
    { "845B12", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "845C93", "Chabrier Services" },
    { "845CF3", "Intel Corporate" },
    { "845DD7", "Shenzhen Netcom Electronics Co.,Ltd" },
    { "845F04", "Samsung Electronics Co.,Ltd" },
    { "846082", "Hyperloop Technologies, Inc dba Virgin Hyperloop" },
    { "8461A0", "Commscope" },
    { "846223", "Shenzhen Coship Electronics Co., Ltd." },
    { "8462A6", "EuroCB (Phils), Inc." },
    { "8463D6", "Microsoft Corporation" },
    { "8464DD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "84652B", "Donaldson Company" },
    { "846569", "New H3C Technologies Co., Ltd" },
    { "84683E", "Intel Corporate" },
    { "846878", "Apple, Inc." },
    { "8468C8", "TOTOLINK TECHNOLOGY INT‘L LIMITED" },
    { "846991", "Nokia" },
    { "846993", "HP Inc." },
    { "846A66", "Sumitomo Kizai Co.,Ltd." },
    { "846AED", "Wireless Tsukamoto.,co.LTD" },
    { "846B48", "ShenZhen EepuLink Co., Ltd." },
    { "846EB1", "Park Assist LLC" },
    { "846EBC", "Nokia Solutions and Networks India Private Limited" },
    { "846FCE", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "847051", "ALPSALPINE CO,.LTD" },
    { "8470D7", "eero inc." },
    { "847127", "Silicon Laboratories" },
    { "84716A", "Huawei Device Co., Ltd." },
    { "847207", "I&C Technology" },
    { "847293", "Texas Instruments" },
    { "847303", "Letv Mobile and Intelligent Information Technology (Beijing) Corporation Ltd." },
    { "84742A", "zte corporation" },
    { "847460", "zte corporation" },
    { "847616", "Addat s.r.o." },
    { "847637", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "847778", "Cochlear Limited" },
    { "847848", "Ubiquiti Inc" },
    { "84788B", "Apple, Inc." },
    { "8478AC", "Cisco Systems, Inc" },
    { "847933", "profichip GmbH" },
    { "847973", "Shanghai Baud Data Communication Co.,Ltd." },
    { "847A88", "HTC Corporation" },
    { "847AB6", "AltoBeam (China) Inc." },
    { "847ADF", "FUJIAN STAR-NET COMMUNICATION CO.,LTD" },
    { "847B57", "Intel Corporate" },
    { "847BEB", "Dell Inc." },
    { "847C9B", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "847D50", "Holley Metering Limited" },
    { "847E40", "Texas Instruments" },
    { "847F3D", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "84802D", "Cisco Systems, Inc" },
    { "848094", "Meter, Inc." },
    { "848102", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "84821B", "PROX SG Pte Ltd" },
    { "8482F4", "Beijing Huasun Unicreate Technology Co., Ltd" },
    { "848319", "Hangzhou Zero Zero Technology Co., Ltd." },
    { "848336", "Newrun" },
    { "848371", "Avaya Inc" },
    { "848433", "Paradox Engineering SA" },
    { "848506", "Apple, Inc." },
    { "84850A", "Hella Sonnen- und Wetterschutztechnik GmbH" },
    { "848553", "Biznes Systema Telecom, LLC" },
    { "8485E6", "Guangdong Asano Technology CO.,Ltd." },
    { "848687", "weiyuantechnology" },
    { "8486F3", "Greenvity Communications" },
    { "8487FF", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "8488E1", "Apple, Inc." },
    { "8489AD", "Apple, Inc." },
    { "8489EC", "IEEE Registration Authority" },
    { "848A59", "Hisilicon Technologies Co., Ltd" },
    { "848A8D", "Cisco Systems, Inc" },
    { "848BCD", "IEEE Registration Authority" },
    { "848C8D", "Apple, Inc." },
    { "848D84", "Rajant Corporation" },
    { "848DC7", "Cisco SPVTG" },
    { "848DCE", "Ciena Corporation" },
    { "848E0C", "Apple, Inc." },
    { "848E96", "Embertec Pty Ltd" },
    { "848EDF", "Sony Corporation" },
    { "848F69", "Dell Inc." },
    { "849000", "Arnold&Richter Cine Technik GmbH & Co. Betriebs KG" },
    { "84900A", "Arcadyan Corporation" },
    { "8492E5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "84930C", "InCoax Networks Europe AB" },
    { "8493A0", "Huawei Device Co., Ltd." },
    { "8493B2", "zte corporation" },
    { "849437", "Apple, Inc." },
    { "849459", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "84948C", "Hitron Technologies. Inc" },
    { "849681", "Cathay Communication Co.,Ltd" },
    { "849690", "Murata Manufacturing Co., Ltd." },
    { "8496D8", "Commscope" },
    { "8497B8", "Memjet Inc." },
    { "849866", "Samsung Electronics Co.,Ltd" },
    { "849A40", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "849C02", "Druid Software" },
    { "849CA4", "Mimosa Networks" },
    { "849CA6", "Arcadyan Technology Corporation" },
    { "849D4B", "Shenzhen Boomtech Industrial Corporation" },
    { "849D64", "SMC Corporation" },
    { "849DC2", "Shanghai MXCHIP Information Technology Co., Ltd." },
    { "849DC5", "Centera Photonics Inc." },
    { "849E56", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "849FB5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "84A06E", "Sagemcom Broadband SAS" },
    { "84A134", "Apple, Inc." },
    { "84A1B7", "Honor Device Co., Ltd." },
    { "84A1D1", "Sagemcom Broadband SAS" },
    { "84A24D", "Birds Eye Systems Private Limited" },
    { "84A329", "Arcadyan Corporation" },
    { "84A3B5", "Propulsion systems" },
    { "84A423", "Sagemcom Broadband SAS" },
    { "84A466", "Samsung Electronics Co.,Ltd" },
    { "84A6C8", "Intel Corporate" },
    { "84A783", "Alcatel Lucent" },
    { "84A788", "Perples" },
    { "84A824", "Google, Inc." },
    { "84A8E4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "84A938", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "84A93E", "Hewlett Packard" },
    { "84A991", "Cyber Trans Japan Co.,Ltd." },
    { "84A9C4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "84A9EA", "Career Technologies USA" },
    { "84AA9C", "MitraStar Technology Corp." },
    { "84AAA4", "SONoC Corp." },
    { "84AB1A", "Apple, Inc." },
    { "84AB26", "Tiinlab Corporation" },
    { "84AC16", "Apple, Inc." },
    { "84AC60", "Guangxi Hesheng Electronics Co., Ltd." },
    { "84ACA4", "Beijing Novel Super Digital TV Technology Co., Ltd" },
    { "84ACFB", "Crouzet Automatismes" },
    { "84AD58", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "84AD8D", "Apple, Inc." },
    { "84AF1F", "GopherTec Inc." },
    { "84AFEC", "BUFFALO.INC" },
    { "84B153", "Apple, Inc." },
    { "84B1E2", "Microsoft Corporation" },
    { "84B1E4", "Apple, Inc." },
    { "84B261", "Cisco Systems, Inc" },
    { "84B31B", "Kinexon GmbH" },
    { "84B386", "IEEE Registration Authority" },
    { "84B4D2", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "84B4DB", "Silicon Laboratories" },
    { "84B517", "Cisco Systems, Inc" },
    { "84B541", "Samsung Electronics Co.,Ltd" },
    { "84B59C", "Juniper Networks" },
    { "84B630", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "84B802", "Cisco Systems, Inc" },
    { "84B866", "Beijing XiaoLu technology co. LTD" },
    { "84B8B8", "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd." },
    { "84BA20", "Silicon Laboratories" },
    { "84BA3B", "CANON INC." },
    { "84BA59", "Wistron InfoComm(Chongqing)Co.,Ltd." },
    { "84BB26", "Texas Instruments" },
    { "84BB69", "Commscope" },
    { "84BE52", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "84BE8B", "Chengdu Geeker Technology Co., Ltd." },
    { "84C0EF", "Samsung Electronics Co.,Ltd" },
    { "84C1C1", "Juniper Networks" },
    { "84C2E4", "Jiangsu Qinheng Co., Ltd." },
    { "84C3E8", "Vaillant GmbH" },
    { "84C5A6", "Intel Corporate" },
    { "84C692", "Texas Instruments" },
    { "84C727", "Gnodal Ltd" },
    { "84C78F", "APS Networks GmbH" },
    { "84C7A9", "C3PO S.A." },
    { "84C7EA", "Sony Corporation" },
    { "84C807", "ADVA Optical Networking Ltd." },
    { "84C8A0", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "84C8B1", "Incognito Software Systems Inc." },
    { "84C9B2", "D-Link International" },
    { "84C9C6", "SHENZHEN GONGJIN ELECTRONICS CO.,LT" },
    { "84CB85", "EM Microelectronic" },
    { "84CC11", "LG Electornics " },
    { "84CC63", "Huawei Device Co., Ltd." },
    { "84CCA8", "Espressif Inc." },
    { "84CD62", "ShenZhen IDWELL Technology CO.,Ltd" },
    { "84CFBF", "Fairphone" },
    { "84D0DB", "Guangdong Juan Intelligent Technology Joint Stock Co., Ltd." },
    { "84D15A", "TCT mobile ltd" },
    { "84D328", "Apple, Inc." },
    { "84D32A", "IEEE 1905.1" },
    { "84D343", "Calix Inc." },
    { "84D352", "Tonly Technology Co. Ltd " },
    { "84D3D5", "Huawei Device Co., Ltd." },
    { "84D412", "Palo Alto Networks" },
    { "84D47E", "Hewlett Packard Enterprise" },
    { "84D4C8", "Widex A/S" },
    { "84D5A0", "MCOT INC." },
    { "84D608", "Wingtech Mobile Communications Co., Ltd." },
    { "84D6C5", "SolarEdge Technologies" },
    { "84D6D0", "Amazon Technologies Inc." },
    { "84D7DE", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "84D81B", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "84D931", "Hangzhou H3C Technologies Co., Limited" },
    { "84D9C8", "Unipattern Co.," },
    { "84DB2F", "Sierra Wireless, ULC" },
    { "84DB9E", "Pink Nectarine Health AB" },
    { "84DBA4", "Huawei Device Co., Ltd." },
    { "84DBAC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "84DBFC", "Nokia" },
    { "84DD20", "Texas Instruments" },
    { "84DDB7", "Cilag GmbH International" },
    { "84DE3D", "Crystal Vision Ltd" },
    { "84DF0C", "NET2GRID BV" },
    { "84DF19", "Chuango Security Technology Corporation" },
    { "84E058", "Commscope" },
    { "84E0F4", "IEEE Registration Authority" },
    { "84E323", "Green Wave Telecommunication SDN BHD" },
    { "84E327", "TAILYN TECHNOLOGIES INC" },
    { "84E342", "Tuya Smart Inc." },
    { "84E4D9", "Shenzhen NEED technology Ltd." },
    { "84E585", "EM Microelectronic" },
    { "84E5D8", "Guangdong UNIPOE IoT Technology Co.,Ltd." },
    { "84E629", "Bluwan SA" },
    { "84E657", "Sony Interactive Entertainment Inc." },
    { "84E714", "Liang Herng Enterprise,Co.Ltd." },
    { "84E892", "Actiontec Electronics, Inc" },
    { "84E8CB", "BUFFALO.INC" },
    { "84E986", "Huawei Device Co., Ltd." },
    { "84E9C1", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "84EA97", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "84EA99", "Vieworks" },
    { "84EAD2", "KOGANEI CORPORATION" },
    { "84EAED", "Roku, Inc" },
    { "84EB18", "Texas Instruments" },
    { "84EB3E", "Vivint Smart Home" },
    { "84EB3F", "Vivint Inc" },
    { "84EBEF", "Cisco Systems, Inc" },
    { "84ED33", "BBMC Co.,Ltd" },
    { "84EE7F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "84EEE4", "Samsung Electronics Co.,Ltd" },
    { "84EF18", "Intel Corporate" },
    { "84F117", "Newseason" },
    { "84F129", "Metrascale Inc." },
    { "84F147", "Cisco Systems, Inc" },
    { "84F175", "Jiangxi Xunte Intelligent Terminal Co., Ltd" },
    { "84F1D0", "EHOOME IOT PRIVATE LIMITED" },
    { "84F1F7", "NXP Semiconductor (Tianjin) LTD." },
    { "84F2C1", "zte corporation" },
    { "84F3EB", "Espressif Inc." },
    { "84F44C", "International Integrated Systems., Inc." },
    { "84F493", "OMS spol. s.r.o." },
    { "84F5EB", "zte corporation" },
    { "84F64C", "Cross Point BV" },
    { "84F6FA", "Miovision Technologies Incorporated" },
    { "84F703", "Espressif Inc." },
    { "84F758", "Huizhou BYD Electronic Co., Ltd." },
    { "84F883", "Luminar Technologies" },
    { "84FB43", "Central Denshi Seigyo" },
    { "84FCAC", "Apple, Inc." },
    { "84FCE6", "Espressif Inc." },
    { "84FCFE", "Apple, Inc." },
    { "84FD27", "Silicon Laboratories" },
    { "84FDD1", "Intel Corporate" },
    { "84FE40", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "84FE9E", "RTC Industries, Inc." },
    { "84FEDC", "Borqs Beijing Ltd." },
    { "84FFC2", "Cisco Systems, Inc" },
    { "88010C", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "880118", "BLT Co" },
    { "8801F2", "Vitec System Engineering Inc." },
    { "8801F9", "Texas Instruments" },
    { "880264", "Pascal Audio" },
    { "88034C", "WEIFANG GOERTEK ELECTRONICS CO.,LTD" },
    { "880355", "Arcadyan Technology Corporation" },
    { "8803E9", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "88074B", "LG Electronics (Mobile Communications)" },
    { "880894", "Skullcandy" },
    { "880905", "MTMCommunications" },
    { "880907", "MKT Systemtechnik GmbH & Co. KG" },
    { "8809AF", "Masimo Corporation" },
    { "880AA3", "Juniper Networks" },
    { "880CE0", "Texas Instruments" },
    { "880E85", "Shenzhen Boomtech Industrial Corporation" },
    { "880F10", "Huami Information Technology Co.,Ltd." },
    { "880F62", "Silicon Laboratories" },
    { "880FA2", "Sagemcom Broadband SAS" },
    { "880FB6", "Jabil Circuits India Pvt Ltd,-EHTP unit" },
    { "881036", "Panodic(ShenZhen) Electronics Limted" },
    { "88108F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "881196", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "88123D", "Suzhou Aquila Solutions Inc." },
    { "88124E", "Qualcomm Inc." },
    { "88127D", "Shenzhen Melon Electronics Co.,Ltd" },
    { "8812AC", "HUNAN FN-LINK TECHNOLOGY LIMITED" },
    { "8813BF", "Espressif Inc." },
    { "88142B", "Protonic Holland" },
    { "881544", "Cisco Meraki" },
    { "881566", "Huawei Device Co., Ltd." },
    { "8815C5", "Huawei Device Co., Ltd." },
    { "8817A3", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "8818AE", "Tamron Co., Ltd" },
    { "881908", "Apple, Inc." },
    { "881A14", "Silicon Laboratories" },
    { "881B99", "SHENZHEN XIN FEI JIA ELECTRONIC CO. LTD." },
    { "881C95", "ITEL MOBILE LIMITED" },
    { "881DFC", "Cisco Systems, Inc" },
    { "881E59", "Onion Corporation" },
    { "881E5A", "Apple, Inc." },
    { "881FA1", "Apple, Inc." },
    { "88200D", "Apple, Inc." },
    { "882012", "LMI Technologies" },
    { "882067", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "8821E3", "Nebusens, S.L." },
    { "882222", "VusionGroup" },
    { "88225B", "Hewlett Packard Enterprise" },
    { "8822B2", "Chipsea Technologies (Shenzhen) Corp." },
    { "88231F", "Fibocom Wireless Inc." },
    { "882364", "Watchnet DVR Inc" },
    { "88238C", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "8823FE", "TTTech Computertechnik AG" },
    { "882508", "Meta Platforms, Inc." },
    { "882510", "Hewlett Packard Enterprise" },
    { "88252C", "Arcadyan Technology Corporation" },
    { "882593", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "88263F", "Zhejiang Uniview Technologies Co.,Ltd." },
    { "88287D", "AltoBeam (China) Inc." },
    { "8828B3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "8828FB", "Juniper Networks" },
    { "882949", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "882950", "Netmoon Technology Co., Ltd" },
    { "88299C", "Samsung Electronics Co.,Ltd" },
    { "882A5E", "New H3C Technologies Co., Ltd" },
    { "882AE1", "MRC INC." },
    { "882B94", "MADOKA SYSTEM Co.,Ltd." },
    { "882BD7", "ADDÉNERGIE TECHNOLOGIES" },
    { "882D53", "Baidu Online Network Technology (Beijing) Co., Ltd." },
    { "882E5A", "storONE" },
    { "882F64", "BCOM Networks Limited" },
    { "883037", "Juniper Networks" },
    { "88308A", "Murata Manufacturing Co., Ltd." },
    { "88329B", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "883314", "Texas Instruments" },
    { "8833BE", "Ivenix, Inc." },
    { "8834FE", "Bosch Automotive Products (Suzhou) Co. Ltd" },
    { "88354C", "Transics" },
    { "8835C1", "OI ELECTRIC CO.,LTD" },
    { "883612", "SRC Computers, LLC" },
    { "88365F", "LG Electronics (Mobile Communications)" },
    { "88366C", "EFM Networks" },
    { "8836CF", "Huawei Device Co., Ltd." },
    { "883A30", "Hewlett Packard Enterprise" },
    { "883B8B", "Cheering Connection Co. Ltd." },
    { "883C1C", "MERCURY CORPORATION" },
    { "883C93", "Alcatel-Lucent Enterprise" },
    { "883CC5", "IEEE Registration Authority" },
    { "883D24", "Google, Inc." },
    { "883E0D", "HD Hyundai Electric" },
    { "883F0C", "system a.v. co., ltd." },
    { "883F27", "Huawei Device Co., Ltd." },
    { "883F37", "UHTEK CO., LTD." },
    { "883F4A", "Texas Instruments" },
    { "883F99", "Siemens AG" },
    { "883FD3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "884033", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "88403B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "884067", "infomark" },
    { "884157", "Shenzhen Atsmart Technology Co.,Ltd." },
    { "8841C1", "ORBISAT DA AMAZONIA IND E AEROL SA" },
    { "8841FC", "AirTies Wireless Networks" },
    { "8843E1", "Cisco Systems, Inc" },
    { "884477", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "8844F6", "Nokia Corporation" },
    { "8845F0", "GUANGDONG GENIUS TECHNOLOGY CO., LTD." },
    { "884604", "Xiaomi Communications Co Ltd" },
    { "88462A", "Telechips Inc." },
    { "884A18", "Opulinks" },
    { "884A70", "Wacom Co.,Ltd." },
    { "884AEA", "Texas Instruments" },
    { "884B39", "Siemens AG, Healthcare Sector" },
    { "884CCF", "Pulzze Systems, Inc" },
    { "884D7C", "Apple, Inc." },
    { "884F59", "Cisco Systems, Inc" },
    { "885046", "LEAR" },
    { "8850DD", "Infiniband Trade Association" },
    { "8850F6", "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd" },
    { "88517A", "HMD Global Oy" },
    { "8851F2", "Apple, Inc." },
    { "8851FB", "Hewlett Packard" },
    { "8852EB", "Xiaomi Communications Co Ltd" },
    { "88532E", "Intel Corporate" },
    { "885395", "Apple, Inc." },
    { "8853D4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "88541F", "Google, Inc." },
    { "88546B", "Texas Instruments" },
    { "88548E", "vivo Mobile Communication Co., Ltd." },
    { "88571D", "Seongji Industry Company" },
    { "885721", "Espressif Inc." },
    { "88576D", "XTA Electronics Ltd" },
    { "8857EE", "BUFFALO.INC" },
    { "8858BE", "kuosheng.com" },
    { "885A06", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "885A23", "CELESTICA INC." },
    { "885A85", "Wistron Neweb Corporation" },
    { "885A92", "Cisco Systems, Inc" },
    { "885BDD", "Extreme Networks Headquarters" },
    { "885C47", "Alcatel Lucent" },
    { "885D90", "IEEE Registration Authority" },
    { "885DFB", "zte corporation" },
    { "885EBD", "NCKOREA Co.,Ltd." },
    { "885FE8", "IEEE Registration Authority" },
    { "886076", "Sparnex n.v." },
    { "886078", "Sparnex n.v." },
    { "88615A", "Siano Mobile Silicon Ltd." },
    { "88625D", "BITNETWORKS CO.,LTD" },
    { "8863C5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "8863DF", "Apple, Inc." },
    { "886440", "Apple, Inc." },
    { "88659F", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "886639", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "88665A", "Apple, Inc." },
    { "8866A5", "Apple, Inc." },
    { "886746", "eero inc." },
    { "8867DC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "88684B", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "88685C", "Shenzhen ChuangDao & Perpetual Eternal Technology Co.,Ltd" },
    { "88693D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "886AB1", "vivo Mobile Communication Co., Ltd." },
    { "886AE3", "Alpha Networks Inc." },
    { "886B0F", "Bluegiga Technologies OY" },
    { "886B44", "Sunnovo International Limited" },
    { "886B6E", "Apple, Inc." },
    { "886B76", "CHINA HOPEFUL GROUP HOPEFUL ELECTRIC CO.,LTD" },
    { "886BDB", "Apple, Inc." },
    { "886C60", "Xiaomi Communications Co Ltd" },
    { "886D2D", "Huawei Device Co., Ltd." },
    { "886EDD", "Micronet union Technology(Chengdu)Co., Ltd." },
    { "886EE1", "Erbe Elektromedizin GmbH" },
    { "886EEB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "886F29", "Pocketbook International SA" },
    { "886FD4", "Dell Inc." },
    { "887033", "Hangzhou Silan Microelectronic Inc" },
    { "88708C", "Lenovo Mobile Communication Technology Ltd." },
    { "8870EF", "SC Professional Trading Co., Ltd." },
    { "8871B1", "Commscope" },
    { "8871E5", "Amazon Technologies Inc." },
    { "887384", "Toshiba" },
    { "887398", "K2E Tekpoint" },
    { "887477", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "887556", "Cisco Systems, Inc" },
    { "887598", "Samsung Electronics Co.,Ltd" },
    { "8876B9", "D-Link Corporation" },
    { "887873", "Intel Corporate" },
    { "88789C", "Game Technologies SA" },
    { "88795B", "Konka Group Co., Ltd." },
    { "88797E", "Motorola Mobility LLC, a Lenovo Company" },
    { "887A31", "Velankani Electronics Pvt. Ltd." },
    { "887ABC", "Cisco Systems, Inc" },
    { "887B2C", "zte corporation" },
    { "887E25", "Extreme Networks Headquarters" },
    { "887F03", "Comper Technology Investment Limited" },
    { "887FD5", "zte corporation" },
    { "888187", "Umeox Innovations Co.,Ltd" },
    { "8881B9", "Huawei Device Co., Ltd." },
    { "888279", "Shenzhen RB-LINK Intelligent Technology Co.Ltd" },
    { "888322", "Samsung Electronics Co.,Ltd" },
    { "88835D", "FN-LINK TECHNOLOGY LIMITED" },
    { "888603", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "8886A0", "Simton Technologies, Ltd." },
    { "8886C2", "STABILO International GmbH" },
    { "888717", "CANON INC." },
    { "8887DD", "DarbeeVision Inc." },
    { "888914", "All Components Incorporated" },
    { "88892F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "888964", "GSI Electronics Inc." },
    { "888B5D", "Storage Appliance Corporation " },
    { "888C19", "Brady Corp Asia Pacific Ltd" },
    { "888C1B", "EM Microelectronic" },
    { "888E68", "Huawei Device Co., Ltd." },
    { "888E7F", "ATOP CORPORATION" },
    { "888F10", "Shenzhen Max Infinite Technology Co.,Ltd." },
    { "888FA4", "Huawei Device Co., Ltd." },
    { "889009", "Juniper Networks" },
    { "88908D", "Cisco Systems, Inc" },
    { "889166", "Viewcooper Corp." },
    { "8891DD", "Racktivity" },
    { "8892CC", "Tonly Technology Co. Ltd " },
    { "889471", "Brocade Communications Systems LLC" },
    { "88947E", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "88948E", "Max Weishaupt SE" },
    { "88948F", "Xi'an Zhisensor Technologies Co.,Ltd" },
    { "8894F9", "Gemicom Technology, Inc." },
    { "8895B9", "Unified Packet Systems Crop" },
    { "88964E", "Commscope" },
    { "889655", "Zitte corporation" },
    { "889676", "TTC MARCONI s.r.o." },
    { "8896B6", "Global Fire Equipment S.A." },
    { "8896F2", "Valeo Schalter und Sensoren GmbH" },
    { "889746", "Sichuan AI-Link Technology Co., Ltd." },
    { "889765", "exands" },
    { "8897DF", "Entrypass Corporation Sdn. Bhd." },
    { "889821", "TERAON" },
    { "889AFF", "SystemX Co.,Ltd." },
    { "889B39", "Samsung Electronics Co.,Ltd" },
    { "889CA6", "BTB Korea INC" },
    { "889CAD", "Cisco Systems, Inc" },
    { "889D98", "Allied-telesisK.K." },
    { "889E33", "TCT mobile ltd" },
    { "889E68", "Vantiva USA LLC" },
    { "889E96", "zte corporation" },
    { "889F6F", "Samsung Electronics Co.,Ltd" },
    { "889FAA", "Hella Gutmann Solutions GmbH " },
    { "889FFA", "Hon Hai Precision Ind. Co.,Ltd." },
    { "88A084", "Formation Data Systems" },
    { "88A0BE", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "88A25E", "Juniper Networks" },
    { "88A29E", "Raspberry Pi (Trading) Ltd" },
    { "88A2D7", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "88A303", "Samsung Electronics Co.,Ltd" },
    { "88A3CC", "Amatis Controls" },
    { "88A479", "Apple, Inc." },
    { "88A4C2", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "88A5BD", "QPCOM INC." },
    { "88A6C6", "Sagemcom Broadband SAS" },
    { "88A6EF", "IEEE Registration Authority" },
    { "88A73C", "Ragentek Technology Group" },
    { "88A9A7", "IEEE Registration Authority" },
    { "88A9B7", "Apple, Inc." },
    { "88AC9E", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "88ACC0", "Zyxel Communications Corporation" },
    { "88ACC1", "Generiton Co., Ltd. " },
    { "88AD43", "PEGATRON CORPORATION" },
    { "88ADD2", "Samsung Electronics Co.,Ltd" },
    { "88AE07", "Apple, Inc." },
    { "88AE1D", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "88AE35", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "88AEDD", "EliteGroup Computer Systems Co., LTD" },
    { "88AF7B", "Nanjing Powercore Tech Co.,Ltd" },
    { "88B111", "Intel Corporate" },
    { "88B168", "Delta Control GmbH" },
    { "88B1E1", "Mojo Networks, Inc." },
    { "88B291", "Apple, Inc." },
    { "88B2AB", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "88B362", "Nokia Shanghai Bell Co., Ltd." },
    { "88B436", "FUJIFILM Corporation" },
    { "88B4A6", "Motorola Mobility LLC, a Lenovo Company" },
    { "88B4BE", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "88B5FF", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "88B627", "Gembird Europe BV" },
    { "88B66B", "easynetworks" },
    { "88B6BD", "Flaircomm Microelectronics, Inc." },
    { "88B6EE", "Dish Technologies Corp" },
    { "88B7EB", "Apple, Inc." },
    { "88B863", "HISENSE VISUAL TECHNOLOGY CO.,LTD" },
    { "88B86F", "Infinix mobility limited" },
    { "88B8D0", "Dongguan Koppo Electronic Co.,Ltd" },
    { "88B945", "Apple, Inc." },
    { "88B951", "Xiaomi Communications Co Ltd" },
    { "88BA7F", "Qfiednet Co., Ltd." },
    { "88BCAC", "Zebra Technologies Inc." },
    { "88BCC1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "88BD09", "Netis Technology Co., Ltd." },
    { "88BD45", "Samsung Electronics Co.,Ltd" },
    { "88BD78", "Flaircomm Microelectronics,Inc." },
    { "88BF35", "Verifone, Inc." },
    { "88BFD5", "Simple Audio Ltd" },
    { "88BFE4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "88C08B", "Apple, Inc." },
    { "88C174", "zte corporation" },
    { "88C227", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "88C242", "Poynt Co." },
    { "88C255", "Texas Instruments" },
    { "88C36E", "Beijing Ereneben lnformation Technology Limited" },
    { "88C397", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "88C3B3", "SOVICO" },
    { "88C3E5", "Betop Techonologies " },
    { "88C48E", "UNEEVIU TECHNOLOGIES INDIA PRIVATE LIMITED" },
    { "88C626", "Logitech, Inc" },
    { "88C663", "Apple, Inc." },
    { "88C6E8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "88C78F", "zte corporation" },
    { "88C9B3", "IEEE Registration Authority" },
    { "88C9D0", "LG Electronics (Mobile Communications)" },
    { "88C9E8", "Sony Corporation" },
    { "88CB87", "Apple, Inc." },
    { "88CBA5", "Suzhou Torchstar Intelligent Technology Co.,Ltd" },
    { "88CC45", "Skyworth Digital Technology(Shenzhen) Co.,Ltd" },
    { "88CE3F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "88CEFA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "88CF98", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "88CFCD", "Texas Instruments" },
    { "88D039", "Tonly Technology Co. Ltd " },
    { "88D171", "BEGHELLI S.P.A" },
    { "88D199", "Vencer Co., Ltd." },
    { "88D211", "Eko Devices, Inc." },
    { "88D274", "zte corporation" },
    { "88D2BF", "German Autolabs" },
    { "88D37B", "FirmTek, LLC" },
    { "88D50C", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "88D546", "Apple, Inc." },
    { "88D5A8", "ITEL MOBILE LIMITED" },
    { "88D652", "AMERGINT Technologies" },
    { "88D7BC", "DEP Company" },
    { "88D7F6", "ASUSTek COMPUTER INC." },
    { "88D82E", "Intel Corporate" },
    { "88D962", "Canopus Systems US LLC" },
    { "88D98F", "Juniper Networks" },
    { "88DA04", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "88DA18", "China Mobile Group Device Co.,Ltd." },
    { "88DA1A", "Redpine Signals, Inc." },
    { "88DA33", "Beijing Xiaoyuer Network Technology Co., Ltd" },
    { "88DA36", "Calix Inc." },
    { "88DC96", "EnGenius Technologies, Inc." },
    { "88DD79", "Voltaire" },
    { "88DDB8", "Huawei Device Co., Ltd." },
    { "88DE39", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "88DE7C", "ASKEY COMPUTER CORP" },
    { "88DEA9", "Roku, Inc." },
    { "88DF9E", "New H3C Technologies Co., Ltd" },
    { "88E034", "Shinwa industries(China) ltd." },
    { "88E056", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "88E0A0", "Shenzhen VisionSTOR Technologies Co., Ltd" },
    { "88E0F3", "Juniper Networks" },
    { "88E161", "Art Beijing Science and Technology Development Co., Ltd." },
    { "88E3AB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "88E603", "Avotek corporation" },
    { "88E628", "Shenzhen Kezhonglong Optoelectronic Technology Co.,Ltd" },
    { "88E64B", "Juniper Networks" },
    { "88E712", "Whirlpool Corporation" },
    { "88E7A6", "iKnowledge Integration Corp." },
    { "88E87F", "Apple, Inc." },
    { "88E8F8", "YONG TAI ELECTRONIC (DONGGUAN) LTD." },
    { "88E90F", "innomdlelab" },
    { "88E917", "Tamaggo" },
    { "88E9A4", "Hewlett Packard Enterprise " },
    { "88E9FE", "Apple, Inc." },
    { "88ED1C", "Cudo Communication Co., Ltd." },
    { "88EF16", "Commscope" },
    { "88F00F", "Miraeil" },
    { "88F031", "Cisco Systems, Inc" },
    { "88F077", "Cisco Systems, Inc" },
    { "88F2BD", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "88F488", "cellon communications technology(shenzhen)Co.,Ltd." },
    { "88F490", "Jetmobile Pte Ltd" },
    { "88F4DA", "Intel Corporate" },
    { "88F56E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "88F6DC", "Huawei Device Co., Ltd." },
    { "88F715", "Arista Networks" },
    { "88F7BF", "vivo Mobile Communication Co., Ltd." },
    { "88F7C7", "Vantiva USA LLC" },
    { "88F872", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "88F916", "Qingdao Dayu Dance Digital Technology Co.,Ltd" },
    { "88FC5D", "Cisco Systems, Inc" },
    { "88FCA6", "devolo AG" },
    { "88FD15", "LINEEYE CO., LTD" },
    { "88FED6", "ShangHai WangYong Software Co., Ltd." },
    { "8C006D", "Apple, Inc." },
    { "8C019D", "Arista Networks" },
    { "8C02CD", "FUJIAN STAR-NET COMMUNICATION CO.,LTD" },
    { "8C02FA", "COMMANDO Networks Limited" },
    { "8C04BA", "Dell Inc." },
    { "8C04FF", "Vantiva USA LLC" },
    { "8C0551", "Koubachi AG" },
    { "8C0572", "Huawei Device Co., Ltd." },
    { "8C06CB", "Toradex AG" },
    { "8C0734", "Private" },
    { "8C078C", "FLOW DATA INC" },
    { "8C0879", "Texas Instruments" },
    { "8C088B", "Remote Solution" },
    { "8C08AA", "Apple, Inc." },
    { "8C09F4", "Commscope" },
    { "8C0C87", "Nokia" },
    { "8C0C90", "Ruckus Wireless" },
    { "8C0CA3", "Amper" },
    { "8C0D76", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "8C0DD9", "Skullcandy" },
    { "8C0E60", "Nanjing Juplink Intelligent Technologies Co., Ltd." },
    { "8C0EE3", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "8C0F6F", "PEGATRON CORPORATION" },
    { "8C0F83", "Angie Hospitality LLC" },
    { "8C0FA0", "di-soric GmbH & Co. KG" },
    { "8C0FC9", "Huawei Device Co., Ltd." },
    { "8C0FFA", "Hutec co.,ltd" },
    { "8C10D4", "Sagemcom Broadband SAS" },
    { "8C11CB", "ABUS Security-Center GmbH & Co. KG" },
    { "8C12C2", "GLBB Japan" },
    { "8C13E2", "NETLINK ICT " },
    { "8C147D", "IEEE Registration Authority" },
    { "8C14B4", "zte corporation" },
    { "8C1553", "Beijing Memblaze Technology Co Ltd" },
    { "8C15C7", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "8C1645", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "8C1759", "Intel Corporate" },
    { "8C17B6", "Huawei Device Co., Ltd." },
    { "8C1850", "China Mobile (Hangzhou) Information Technology Co., Ltd." },
    { "8C18D9", "Shenzhen RF Technology Co., Ltd" },
    { "8C192D", "IEEE Registration Authority" },
    { "8C19B5", "Arcadyan Corporation" },
    { "8C1A50", "China Mobile Group Device Co.,Ltd." },
    { "8C1ABF", "Samsung Electronics Co.,Ltd" },
    { "8C1AF3", "Shenzhen Gooxi Information Security CO.,Ltd." },
    { "8C1CDA", "IEEE Registration Authority" },
    { "8C1D55", "Hanwha NxMD (Thailand) Co., Ltd." },
    { "8C1D96", "Intel Corporate" },
    { "8C1E80", "Cisco Systems, Inc" },
    { "8C1ECB", "Nanjing Jiahao Technology Co., Ltd." },
    { "8C1ECF", "UNIONMAN TECHNOLOGY CO.,LTD" },
    { "8C1ED9", "Beijing Unigroup Tsingteng Microsystem Co., LTD." },
    { "8C1F64", "IEEE Registration Authority" },
    { "8C1F94", "RF Surgical System Inc. " },
    { "8C210A", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "8C2505", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "8C255E", "VoltServer" },
    { "8C26AA", "Apple, Inc." },
    { "8C271D", "QuantHouse" },
    { "8C278A", "Vocollect Inc" },
    { "8C2937", "Apple, Inc." },
    { "8C2A85", "Amazon Technologies Inc." },
    { "8C2A8E", "DongGuan Ramaxel Memory Technology" },
    { "8C2DAA", "Apple, Inc." },
    { "8C2E72", "Samsung Electronics Co.,Ltd" },
    { "8C2F39", "IBA Dosimetry GmbH" },
    { "8C2FA6", "Solid Optics B.V." },
    { "8C3066", "Ubiquiti Inc" },
    { "8C31E2", "DAYOUPLUS" },
    { "8C3223", "JWIPC Technology Co.,Ltd." },
    { "8C3330", "EmFirst Co., Ltd." },
    { "8C3357", "HiteVision Digital Media Technology Co.,Ltd." },
    { "8C3396", "Apple, Inc." },
    { "8C3401", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "8C3446", "Huawei Device Co., Ltd." },
    { "8C34FD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "8C3579", "QDIQO Sp. z o.o." },
    { "8C3592", "Guangzhou Shiyuan Electronic Technology Company Limited" },
    { "8C367A", "Palo Alto Networks" },
    { "8C395C", "Bit4id Srl" },
    { "8C3A7E", "Universal Electronics, Inc." },
    { "8C3AE3", "LG Electronics (Mobile Communications)" },
    { "8C3B32", "Microfan B.V." },
    { "8C3B4A", "Universal Global Scientific Industrial Co., Ltd." },
    { "8C3BAD", "NETGEAR" },
    { "8C3C07", "Skiva Technologies, Inc." },
    { "8C3C4A", "NAKAYO Inc" },
    { "8C3D16", "Shenzhen Four Seas Global Link Network Technology Co.,Ltd" },
    { "8C3DB1", "Beijing H-IoT Technology Co., Ltd." },
    { "8C3F44", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "8C41F2", "RDA Technologies Ltd." },
    { "8C41F4", "IPmotion GmbH" },
    { "8C426D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "8C4361", "Hailo Digital Hub GmbH & Co. KG" },
    { "8C4435", "Shanghai BroadMobi Communication Technology Co., Ltd." },
    { "8C444F", "HUMAX Co., Ltd." },
    { "8C44A5", "Cisco Systems, Inc" },
    { "8C44BB", "SHEN ZHEN TENDA TECHNOLOGY CO.,LTD" },
    { "8C4500", "Murata Manufacturing Co., Ltd." },
    { "8C476E", "IEEE Registration Authority" },
    { "8C477F", "NambooSolution" },
    { "8C47BE", "Dell Inc." },
    { "8C4962", "Roku, Inc" },
    { "8C497A", "Extreme Networks Headquarters" },
    { "8C49B6", "vivo Mobile Communication Co., Ltd." },
    { "8C4AEE", "GIGA TMS INC" },
    { "8C4B14", "Espressif Inc." },
    { "8C4B59", "3D Imaging & Simulations Corp" },
    { "8C4CAD", "Evoluzn Inc." },
    { "8C4CDC", "PLANEX COMMUNICATIONS INC." },
    { "8C4DB9", "Unmonday Ltd" },
    { "8C4DEA", "Cerio Corporation" },
    { "8C4E46", "Motorola Mobility LLC, a Lenovo Company" },
    { "8C4F00", "Espressif Inc." },
    { "8C501A", "Private" },
    { "8C5105", "Shenzhen ireadygo Information Technology CO.,LTD." },
    { "8C5109", "IEEE Registration Authority" },
    { "8C5219", "SHARP Corporation" },
    { "8C53C3", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "8C53D2", "China Mobile Group Device Co.,Ltd." },
    { "8C53E6", "Wistron Neweb Corporation" },
    { "8C53F7", "A&D ENGINEERING CO., LTD." },
    { "8C541D", "LGE " },
    { "8C554A", "Intel Corporate" },
    { "8C5570", "IEEE Registration Authority" },
    { "8C55BB", "Songwoo Information & Technology Co., Ltd" },
    { "8C5646", "LG Electronics" },
    { "8C569D", "Imaging Solutions Group" },
    { "8C56C5", "Nintendo Co., Ltd." },
    { "8C579B", "Wistron Neweb Corporation" },
    { "8C57FD", "LVX Western" },
    { "8C5823", "SZ DJI TECHNOLOGY CO.,LTD" },
    { "8C5877", "Apple, Inc." },
    { "8C593C", "IEEE Registration Authority" },
    { "8C5973", "Zyxel Communications Corporation" },
    { "8C598B", "C Technologies AB" },
    { "8C59C3", "ADB Italia " },
    { "8C59DC", "ASR Microelectronics (Shanghai) Co., Ltd." },
    { "8C5A25", "Commscope" },
    { "8C5AC1", "Huawei Device Co., Ltd." },
    { "8C5AF0", "Exeltech Solar Products" },
    { "8C5AF8", "Beijing Xiaomi Electronics Co., Ltd." },
    { "8C5BF0", "Commscope" },
    { "8C5C20", "Vantiva USA LLC" },
    { "8C5C53", "AltoBeam Inc." },
    { "8C5CA1", "d-broad,INC" },
    { "8C5D60", "UCI Corporation Co.,Ltd." },
    { "8C5DB2", "IEEE Registration Authority" },
    { "8C5E4D", "DragonWave Technologies DMCC" },
    { "8C5EBD", "Huawei Device Co., Ltd." },
    { "8C5F48", "Continental Intelligent Transportation Systems LLC" },
    { "8C5FAD", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "8C5FDF", "Beijing Railway Signal Factory" },
    { "8C604F", "Cisco Systems, Inc" },
    { "8C6078", "Swissbit AG" },
    { "8C60E7", "MPGIO CO.,LTD" },
    { "8C6102", "Beijing Baofengmojing Technologies Co., Ltd" },
    { "8C6120", "EM Microelectronic" },
    { "8C61A3", "Commscope" },
    { "8C640B", "Beyond Devices d.o.o." },
    { "8C6422", "Sony Corporation" },
    { "8C64A2", "OnePlus Technology (Shenzhen) Co., Ltd" },
    { "8C64D4", "Hyeco Smart Tech Co.,Ltd" },
    { "8C65A3", "Silicon Laboratories" },
    { "8C6794", "vivo Mobile Communication Co., Ltd." },
    { "8C683A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "8C6878", "Nortek-AS" },
    { "8C68C8", "zte corporation" },
    { "8C6A3B", "Samsung Electronics Co.,Ltd" },
    { "8C6A8D", "Vantiva USA LLC" },
    { "8C6AE4", "Viogem Limited" },
    { "8C6BDB", "Huawei Device Co., Ltd." },
    { "8C6D50", "SHENZHEN MTC CO LTD" },
    { "8C6D77", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "8C6DC4", "Megapixel VR" },
    { "8C6FB9", "Silicon Laboratories" },
    { "8C705A", "Intel Corporate" },
    { "8C7086", "Gesellschaft für Sonder-EDV-Anlagen mbH" },
    { "8C7112", "Microchip Technologies Inc" },
    { "8C71F8", "Samsung Electronics Co.,Ltd" },
    { "8C736E", "FUJITSU LIMITED" },
    { "8C73A0", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "8C73DA", "Silicon Laboratories" },
    { "8C763F", "Commscope" },
    { "8C76C1", "Goden Tech Limited" },
    { "8C7712", "Samsung Electronics Co.,Ltd" },
    { "8C7716", "LONGCHEER TELECOMMUNICATION LIMITED" },
    { "8C78D7", "SHENZHEN FAST TECHNOLOGIES CO.,LTD" },
    { "8C7909", "Hewlett Packard Enterprise" },
    { "8C7967", "zte corporation" },
    { "8C79F5", "Samsung Electronics Co.,Ltd" },
    { "8C7A00", "Nokia" },
    { "8C7A15", "Ruckus Wireless" },
    { "8C7A3D", "Xiaomi Communications Co Ltd" },
    { "8C7AAA", "Apple, Inc." },
    { "8C7AB3", "Guangzhou Shiyuan Electronic Technology Company Limited" },
    { "8C7B9D", "Apple, Inc." },
    { "8C7BF0", "Xufeng Development Limited" },
    { "8C7C92", "Apple, Inc." },
    { "8C7CB5", "Hon Hai Precision Ind. Co.,Ltd." },
    { "8C7CFF", "Brocade Communications Systems LLC" },
    { "8C7EB3", "Lytro, Inc." },
    { "8C7F3B", "Commscope" },
    { "8C8126", "ARCOM" },
    { "8C8172", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "8C82A8", "Insigma Technology Co.,Ltd" },
    { "8C8394", "Arcadyan Corporation" },
    { "8C839D", "SHENZHEN XINYUPENG ELECTRONIC TECHNOLOGY CO., LTD" },
    { "8C83DF", "Nokia" },
    { "8C83E1", "Samsung Electronics Co.,Ltd" },
    { "8C83E8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "8C83FC", "Axioma Metering UAB" },
    { "8C8401", "Private" },
    { "8C8442", "Cisco Systems, Inc" },
    { "8C8474", "Broadcom Limited" },
    { "8C8580", "Smart Innovation LLC" },
    { "8C8590", "Apple, Inc." },
    { "8C85C1", "Hewlett Packard Enterprise" },
    { "8C85E6", "Cleondris GmbH" },
    { "8C861E", "Apple, Inc." },
    { "8C862A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "8C86DD", "TP-Link Systems Inc." },
    { "8C8726", "VAST Data Inc" },
    { "8C873B", "Leica Camera AG" },
    { "8C87D0", "Shenzhen Uascent Technology Co.,Ltd" },
    { "8C8881", "Cisco Meraki" },
    { "8C897A", "AUGTEK" },
    { "8C89A5", "Micro-Star INT'L CO., LTD" },
    { "8C89FA", "Zhejiang Hechuan Technology Co., Ltd." },
    { "8C8A6E", "ESTUN AUTOMATION TECHNOLOY CO., LTD" },
    { "8C8ABB", "Beijing Orient View Technology Co., Ltd." },
    { "8C8ACD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "8C8B48", "Silicon Laboratories" },
    { "8C8B5B", "Wistron Neweb Corporation" },
    { "8C8B83", "Texas Instruments" },
    { "8C8CAA", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "8C8D28", "Intel Corporate" },
    { "8C8E0D", "zte corporation" },
    { "8C8E4E", "Baylan Olcu Aletleri San. ve Tic.A.S." },
    { "8C8E76", "taskit GmbH" },
    { "8C8EF2", "Apple, Inc." },
    { "8C8F8B", "China Mobile Chongqing branch" },
    { "8C8FE9", "Apple, Inc." },
    { "8C902D", "TP-Link Systems Inc." },
    { "8C90D3", "Nokia" },
    { "8C9109", "Toyoshima Electric Technoeogy(Suzhou) Co.,Ltd." },
    { "8C913A", "Mellanox Technologies, Inc." },
    { "8C91A4", "Apple, Inc." },
    { "8C9236", "Aus.Linx Technology Co., Ltd." },
    { "8C9246", "Oerlikon Textile Gmbh&Co.KG" },
    { "8C9351", "Jigowatts Inc." },
    { "8C938B", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "8C941F", "Cisco Systems, Inc" },
    { "8C9461", "Cisco Systems, Inc" },
    { "8C946A", "New H3C Technologies Co., Ltd" },
    { "8C94CC", "SFR" },
    { "8C94CF", "Encell Technology, Inc." },
    { "8C965F", "Shandong Zhongan Technology Co., Ltd." },
    { "8C96A5", "New H3C Technologies Co., Ltd" },
    { "8C97EA", "FREEBOX SAS" },
    { "8C9806", "SHENZHEN SEI ROBOTICS CO.,LTD" },
    { "8C986B", "Apple, Inc." },
    { "8C99E6", "TCT mobile ltd" },
    { "8C9A8F", "Sagemcom Broadband SAS" },
    { "8C9B2D", "PLANTRONICS, INC." },
    { "8C9F3B", "Qingdao Hisense Communications Co.,Ltd." },
    { "8CA048", "Beijing NeTopChip Technology Co.,LTD" },
    { "8CA2FD", "Starry, Inc." },
    { "8CA399", "SERVERCOM (INDIA) PRIVATE LIMITED" },
    { "8CA401", "Shenzhen New Chip Intelligence Co.,LTD" },
    { "8CA5A1", "Oregano Systems - Design & Consulting GmbH" },
    { "8CA5CF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "8CA682", "IEEE Registration Authority" },
    { "8CA6DF", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "8CA96D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "8CA96F", "D&M Holdings Inc." },
    { "8CA982", "Intel Corporate" },
    { "8CAAB5", "Espressif Inc." },
    { "8CAACE", "Xiaomi Communications Co Ltd" },
    { "8CAB8E", "Shanghai Feixun Communication Co.,Ltd." },
    { "8CAE49", "IEEE Registration Authority" },
    { "8CAE4C", "Plugable Technologies" },
    { "8CAE89", "Y-cam Solutions Ltd" },
    { "8CAEDB", "NAGTECH LLC" },
    { "8CB094", "Airtech I&C Co., Ltd" },
    { "8CB0E9", "Samsung Electronics.,LTD" },
    { "8CB50E", "Cisco Systems, Inc" },
    { "8CB64F", "Cisco Systems, Inc" },
    { "8CB6C5", "Mimosa Networks" },
    { "8CB7F7", "Shenzhen UniStrong Science & Technology Co., Ltd" },
    { "8CB82C", "IPitomy Communications" },
    { "8CB84A", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "8CB864", "AcSiP Technology Corp." },
    { "8CB87E", "Intel Corporate" },
    { "8CBA25", "UNION MAN TECHNOLOGY CO.,LTD" },
    { "8CBAFC", "JOYNEXT GmbH" },
    { "8CBD37", "Shenzhen Phaten Tech. LTD" },
    { "8CBE24", "Tashang Semiconductor(Shanghai) Co., Ltd." },
    { "8CBE6F", "Tianyi Telecom Terminals Company Limited" },
    { "8CBEBE", "Xiaomi Communications Co Ltd" },
    { "8CBF9D", "Shanghai Xinyou Information Technology Ltd. Co." },
    { "8CBFA6", "Samsung Electronics Co.,Ltd" },
    { "8CBFEA", "Espressif Inc." },
    { "8CC121", "Panasonic Corporation AVC Networks Company" },
    { "8CC573", "Xsight Labs LTD." },
    { "8CC58C", "ShenZhen Elsky Technology Co.,LTD" },
    { "8CC5B4", "Sagemcom Broadband SAS" },
    { "8CC5D0", "Samsung Electronics Co.,Ltd" },
    { "8CC5E1", "ShenZhen Konka Telecommunication Technology Co.,Ltd" },
    { "8CC661", "Current, powered by GE" },
    { "8CC681", "Intel Corporate" },
    { "8CC7AA", "Radinet Communications Inc." },
    { "8CC7C3", "NETLINK ICT " },
    { "8CC7D0", "zhejiang ebang communication co.,ltd" },
    { "8CC84B", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "8CC8CD", "Samsung Electronics Co.,Ltd" },
    { "8CC8F4", "IEEE Registration Authority" },
    { "8CC9E9", "Huawei Device Co., Ltd." },
    { "8CCB14", "TBS GmbH" },
    { "8CCBDF", "FOXCONN INTERCONNECT TECHNOLOGY " },
    { "8CCD55", "EM Microelectronic" },
    { "8CCDA2", "ACTP, Inc." },
    { "8CCDE8", "Nintendo Co., Ltd." },
    { "8CCDFE", "AMPAK Technology,Inc." },
    { "8CCE4E", "Espressif Inc." },
    { "8CCEFD", "Shenzhen zhouhai technology co.,LTD" },
    { "8CCF09", "Dell EMC" },
    { "8CCF5C", "BEFEGA GmbH" },
    { "8CCF8F", "ITC Systems" },
    { "8CD08B", "WuXi Rigosys Technology Co.,LTD" },
    { "8CD0B2", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "8CD17B", "CG Mobile" },
    { "8CD2E9", "YOKOTE SEIKO CO., LTD." },
    { "8CD3A2", "VisSim AS" },
    { "8CD48E", "ITEL MOBILE LIMITED" },
    { "8CD54A", "TAIYO YUDEN CO.,LTD" },
    { "8CD628", "Ikor Metering" },
    { "8CD67F", "EM Microelectronic" },
    { "8CD9D6", "Xiaomi Communications Co Ltd" },
    { "8CDB25", "ESG Solutions" },
    { "8CDC02", "zte corporation" },
    { "8CDCD4", "Hewlett Packard" },
    { "8CDD0B", "eero inc." },
    { "8CDD8D", "Wifly-City System Inc." },
    { "8CDE52", "ISSC Technologies Corp." },
    { "8CDE99", "Comlab Inc." },
    { "8CDEE6", "Samsung Electronics Co.,Ltd" },
    { "8CDEF9", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "8CDF2C", "vivo Mobile Communication Co., Ltd." },
    { "8CDF9D", "NEC Corporation" },
    { "8CE042", "vivo Mobile Communication Co., Ltd." },
    { "8CE081", "zte corporation" },
    { "8CE117", "zte corporation" },
    { "8CE2DA", "Circle Media Inc" },
    { "8CE38E", "Kioxia Corporation" },
    { "8CE468", "Guangzhou Sageran Technology Co., Ltd." },
    { "8CE4DB", "Shenzhen MiaoMing Intelligent Technology Co.,Ltd" },
    { "8CE5C0", "Samsung Electronics Co.,Ltd" },
    { "8CE5EF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "8CE748", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "8CE78C", "DK Networks" },
    { "8CE7B3", "Sonardyne International Ltd" },
    { "8CE9B4", "Zhejiang Dahua Technology Co., Ltd." },
    { "8CE9EE", "Intel Corporate" },
    { "8CE9FF", "Dell Inc." },
    { "8CEA12", "Shenzhen MiaoMing Intelligent Technology Co.,Ltd" },
    { "8CEA1B", "Edgecore Networks Corporation" },
    { "8CEA48", "Samsung Electronics Co.,Ltd" },
    { "8CEA88", "Chengdu Yocto Communication Technology Co.Ltd." },
    { "8CEBC6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "8CEC4B", "Dell Inc." },
    { "8CEC7B", "Apple, Inc." },
    { "8CEDE1", "Ubiquiti Inc" },
    { "8CEEC6", "Precepscion Pty. Ltd." },
    { "8CEEFD", "zte corporation" },
    { "8CF0DF", "Beijing Zhongyuan Yishang Technology Co.,LTD" },
    { "8CF112", "Motorola Mobility LLC, a Lenovo Company" },
    { "8CF228", "MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "8CF319", "Siemens Industrial Automation Products Ltd., Chengdu" },
    { "8CF3E7", "solidotech" },
    { "8CF5A3", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "8CF681", "Silicon Laboratories" },
    { "8CF710", "AMPAK Technology, Inc." },
    { "8CF773", "Nokia" },
    { "8CF813", "ORANGE POLSKA" },
    { "8CF8C5", "Intel Corporate" },
    { "8CF945", "Power Automation pte Ltd" },
    { "8CF957", "RuiXingHengFang Network (Shenzhen) Co.,Ltd" },
    { "8CF9C9", "MESADA Technology Co.,Ltd." },
    { "8CFABA", "Apple, Inc." },
    { "8CFADD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "8CFCA0", "Shenzhen Smart Device Technology Co., LTD." },
    { "8CFD15", "Imagine Marketing Private Limited" },
    { "8CFD18", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "8CFDDE", "Sagemcom Broadband SAS" },
    { "8CFDF0", "Qualcomm Inc." },
    { "8CFE57", "Apple, Inc." },
    { "8CFE74", "Ruckus Wireless" },
    { "8CFEB4", "VSOONTECH ELECTRONICS CO., LIMITED" },
    { "90004E", "Hon Hai Precision Ind. Co.,Ltd." },
    { "9000DB", "Samsung Electronics Co.,Ltd" },
    { "900117", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "90013B", "Sagemcom Broadband SAS" },
    { "900218", "SKY UK LIMITED" },
    { "90027A", "Shenzhen Sworix Techonlogy Co., Ltd" },
    { "90028A", "Shenzhen Shidean Legrand Electronic Products Co.,Ltd" },
    { "9002A9", "Zhejiang Dahua Technology Co., Ltd." },
    { "900325", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "90032E", "Taicang T&W Electronics" },
    { "900371", "Quectel Wireless Solutions Co.,Ltd." },
    { "900372", "Longnan Junya Digital Technology Co. Ltd. " },
    { "9003B7", "PARROT SA" },
    { "900628", "Samsung Electronics Co.,Ltd" },
    { "9006F2", "Texas Instruments" },
    { "900917", "Far-sighted mobile" },
    { "9009D0", "Synology Incorporated" },
    { "9009DF", "Intel Corporate" },
    { "900A1A", "Taicang T&W Electronics" },
    { "900A39", "Wiio, Inc." },
    { "900A3A", "PSG Plastic Service GmbH" },
    { "900A62", "Inventus Power Eletronica do Brasil LTDA" },
    { "900A84", "Mellanox Technologies, Inc." },
    { "900BC1", "Sprocomm Technologies CO.,Ltd" },
    { "900CB4", "Alinket Electronic Technology Co., Ltd" },
    { "900CC8", "Google, Inc." },
    { "900D66", "Digimore Electronics Co., Ltd" },
    { "900DCB", "Commscope" },
    { "900E83", "Monico Monitoring, Inc." },
    { "900E9E", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "900EB3", "Shenzhen Amediatech Technology Co., Ltd." },
    { "900F0C", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "901057", "Intel Corporate" },
    { "901195", "Amazon Technologies Inc." },
    { "901234", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "9012A1", "We Corporation Inc." },
    { "9013DA", "Athom B.V." },
    { "9014AF", "Cambium Networks Limited" },
    { "901506", "Espressif Inc." },
    { "901564", "IEEE Registration Authority" },
    { "9016BA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "901711", "Hagenuk Marinekommunikation GmbH" },
    { "90173F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "90179B", "Nanomegas" },
    { "9017AC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9017C8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "90185E", "Apex Tool Group GmbH & Co OHG" },
    { "90187C", "SAMSUNG ELECTRO MECHANICS CO., LTD." },
    { "9018AE", "Shanghai Meridian Technologies, Co. Ltd." },
    { "901900", "SCS SA" },
    { "901A4F", "EM Microelectronic" },
    { "901ACA", "Commscope" },
    { "901B0E", "Fujitsu Technology Solutions GmbH" },
    { "901D27", "zte corporation" },
    { "901EDD", "GREAT COMPUTER CORPORATION" },
    { "90203A", "BYD Precision Manufacture Co.,Ltd" },
    { "902083", "General Engine Management Systems Ltd." },
    { "9020C2", "Hewlett Packard Enterprise" },
    { "902106", "SKY UK LIMITED" },
    { "90212E", "Apption Labs Ltd" },
    { "902155", "HTC Corporation" },
    { "902181", "Shanghai Huaqin Telecom Technology Co.,Ltd" },
    { "90235B", "Amazon Technologies Inc." },
    { "9023B4", "New H3C Technologies Co., Ltd" },
    { "9023EC", "Availink, Inc." },
    { "9025F2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "90272B", "Algorab S.r.l." },
    { "902759", "Nanjing Jiahao Technology Co., Ltd." },
    { "902778", "Open Infrastructure" },
    { "9027E4", "Apple, Inc." },
    { "9028F6", "Wuhan Yangtze Optical Technology CO.,Ltd." },
    { "902962", "Linkpower Microelectronics Co., Ltd." },
    { "902AEE", "Xiaomi Communications Co Ltd" },
    { "902B34", "GIGA-BYTE TECHNOLOGY CO.,LTD." },
    { "902BD2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "902C09", "Apple, Inc." },
    { "902CC7", "C-MAX Asia Limited" },
    { "902CFB", "CanTops Co,.Ltd." },
    { "902D77", "Edgecore Americas Networking Corporation" },
    { "902E16", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "902E1C", "Intel Corporate" },
    { "902E87", "LabJack" },
    { "90314B", "AltoBeam Inc." },
    { "9031CD", "Onyx Healthcare Inc." },
    { "90324B", "Hon Hai Precision Ind. Co.,Ltd." },
    { "90342B", "Gatekeeper Systems, Inc." },
    { "9034FC", "Hon Hai Precision Ind. Co.,Ltd." },
    { "90356E", "Vodafone Omnitel N.V." },
    { "9035EA", "Silicon Laboratories" },
    { "903809", "Ericsson AB" },
    { "90380C", "Espressif Inc." },
    { "9038DF", "Changzhou Tiannengbo System Co. Ltd." },
    { "90395E", "Silicon Laboratories" },
    { "90395F", "Amazon Technologies Inc." },
    { "903A72", "Ruckus Wireless" },
    { "903AA0", "Nokia" },
    { "903AE6", "PARROT SA" },
    { "903C1D", "HISENSE VISUAL TECHNOLOGY CO.,LTD" },
    { "903C92", "Apple, Inc." },
    { "903CAE", "Yunnan KSEC Digital Technology Co.,Ltd." },
    { "903CB3", "Edgecore Networks Corporation" },
    { "903CDA", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "903D5A", "Shenzhen Wision Technology Holding Limited" },
    { "903D68", "G-Printec, Inc." },
    { "903D6B", "Zicon Technology Corp." },
    { "903DBD", "SECURE METERS LIMITED" },
    { "903E7F", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "903EAB", "Commscope" },
    { "903FC3", "Huawei Device Co., Ltd." },
    { "903FEA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9043E2", "Cornami, Inc" },
    { "904506", "Tokyo Boeki Medisys Inc." },
    { "904528", "Nintendo Co.,Ltd" },
    { "9046A2", "Tedipay UK Ltd" },
    { "9046B7", "Vadaro Pte Ltd" },
    { "904716", "RORZE CORPORATION" },
    { "90473C", "China Mobile Group Device Co.,Ltd." },
    { "904748", "Sony Interactive Entertainment Inc." },
    { "904846", "Texas Instruments" },
    { "90486C", "Ring LLC" },
    { "90489A", "Hon Hai Precision Ind. Co.,Ltd." },
    { "904992", "YSTen Technology Co.,Ltd" },
    { "9049FA", "Intel Corporate" },
    { "904C02", "vivo Mobile Communication Co., Ltd." },
    { "904C81", "Hewlett Packard Enterprise " },
    { "904CC5", "Apple, Inc." },
    { "904CE5", "Hon Hai Precision Ind. Co.,Ltd." },
    { "904D4A", "Sagemcom Broadband SAS" },
    { "904DC3", "Flonidan A/S" },
    { "904E2B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "904E91", "IEEE Registration Authority" },
    { "90505A", "unGlue, Inc" },
    { "90507B", "Advanced PANMOBIL Systems GmbH & Co. KG" },
    { "9050CA", "Hitron Technologies. Inc" },
    { "90513F", "Elettronica Santerno SpA" },
    { "9052BF", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "905446", "TES ELECTRONIC SOLUTIONS" },
    { "9054B7", "Eltex Enterprise LTD" },
    { "9055AE", "Ericsson, EAB/RWI/K" },
    { "9055DE", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "905607", "Sichuan AI-Link Technology Co., Ltd." },
    { "905671", "Cisco Systems, Inc" },
    { "905682", "Lenbrook Industries Limited" },
    { "905692", "Autotalks Ltd." },
    { "9056FC", "TECNO MOBILE LIMITED" },
    { "905851", "Vantiva USA LLC" },
    { "90593C", "AZ-TECHNOLOGY SDN BHD" },
    { "9059AF", "Texas Instruments" },
    { "905A08", "Super Micro Computer, Inc." },
    { "905C34", "Sirius Electronic Systems Srl" },
    { "905C44", "Compal Broadband Networks, Inc." },
    { "905D7C", "New H3C Technologies Co., Ltd" },
    { "905E44", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "905F2E", "TCT mobile ltd" },
    { "905F7A", "Apple, Inc." },
    { "905F8D", "modas GmbH" },
    { "9060F1", "Apple, Inc." },
    { "90610C", "Fida International (S) Pte Ltd" },
    { "9061AE", "Intel Corporate" },
    { "90623F", "Apple, Inc." },
    { "90633B", "Samsung Electronics Co.,Ltd" },
    { "9064AD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "906560", "EM Microelectronic" },
    { "906584", "Intel Corporate" },
    { "906717", "Alphion India Private Limited" },
    { "90671C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9067B5", "Alcatel-Lucent" },
    { "9067F3", "Alcatel Lucent" },
    { "9068C3", "Motorola Mobility LLC, a Lenovo Company" },
    { "906976", "Withrobot Inc." },
    { "906A94", "hangzhou huacheng network technology co., ltd" },
    { "906AEB", "Microsoft Corporation" },
    { "906C4B", "Advance Security Electronics" },
    { "906CAC", "Fortinet, Inc." },
    { "906D05", "BXB ELECTRONICS CO., LTD" },
    { "906D62", "Cambium Networks Limited" },
    { "906DC8", "DLG Automação Industrial Ltda" },
    { "906EBB", "Hon Hai Precision Ind. Co.,Ltd." },
    { "906F18", "Private" },
    { "906FA9", "NANJING PUTIAN TELECOMMUNICATIONS TECHNOLOGY CO.,LTD." },
    { "907025", "Garea Microsys Co.,Ltd." },
    { "907065", "Texas Instruments" },
    { "9070BF", "Mimosa Networks" },
    { "9070D3", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "907240", "Apple, Inc." },
    { "907282", "Sagemcom Broadband SAS" },
    { "90735A", "Motorola Mobility LLC, a Lenovo Company" },
    { "90742E", "New H3C Technologies Co., Ltd" },
    { "90749D", "IRay Technology Co., Ltd." },
    { "9075BC", "Nokia Shanghai Bell Co., Ltd." },
    { "9075DE", "Zebra Technologies Inc." },
    { "90769F", "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "9077EE", "Cisco Systems, Inc" },
    { "907841", "Intel Corporate" },
    { "9078B2", "Xiaomi Communications Co Ltd" },
    { "907910", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "907990", "Benchmark Electronics Romania SRL" },
    { "9079CF", "zte corporation" },
    { "907A0A", "Gebr. Bode GmbH & Co KG" },
    { "907A28", "Beijing Morncloud Information And Technology Co. Ltd." },
    { "907A58", "Zegna-Daidong Limited" },
    { "907AF1", "Wally" },
    { "907BC6", "Texas Instruments" },
    { "907E30", "LARS" },
    { "907E43", "zte corporation" },
    { "907EBA", "UTEK TECHNOLOGY (SHENZHEN) CO.,LTD" },
    { "907F61", "Chicony Electronics Co., Ltd." },
    { "908060", "Nilfisk A/S" },
    { "90808F", "Huawei Device Co., Ltd." },
    { "90812A", "Apple, Inc." },
    { "908158", "Apple, Inc." },
    { "908175", "Samsung Electronics Co.,Ltd" },
    { "908260", "IEEE 1904.1 Working Group" },
    { "9082C3", "Quanta Computer Inc." },
    { "90834B", "BEIJING YUNYI TIMES TECHNOLOGY CO,.LTD" },
    { "90837A", "General Electric Water & Process Technologies" },
    { "90837E", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "90840D", "Apple, Inc." },
    { "90842B", "LEGO System A/S" },
    { "90848B", "HDR10+ Technologies, LLC" },
    { "908674", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "90869B", "zte corporation" },
    { "908855", "Cisco Systems, Inc" },
    { "9088A2", "IONICS TECHNOLOGY ME LTDA" },
    { "9088A9", "IEEE Registration Authority" },
    { "908938", "Hefei Linkin Technology Co., Ltd." },
    { "90895F", "WEIFANG GOERTEK ELECTRONICS CO.,LTD" },
    { "908C09", "Total Phase" },
    { "908C43", "Apple, Inc." },
    { "908C44", "H.K ZONGMU TECHNOLOGY CO., LTD." },
    { "908C63", "GZ Weedong Networks Technology Co. , Ltd" },
    { "908D1D", "GH Technologies" },
    { "908D6C", "Apple, Inc." },
    { "908D6E", "Dell Inc." },
    { "908D78", "D-Link International" },
    { "908FCF", "UNO System Co., Ltd" },
    { "90903C", "TRISON TECHNOLOGY CORPORATION" },
    { "909060", "RSI VIDEO TECHNOLOGIES" },
    { "909164", "ChongQing Lavid Technology Co., Ltd." },
    { "9092B4", "Diehl BGT Defence GmbH & Co. KG" },
    { "90935A", "Commscope" },
    { "90940A", "Analog Devices, Inc" },
    { "909497", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9094E4", "D-Link International" },
    { "909507", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9096F3", "BUFFALO.INC" },
    { "9097D5", "Espressif Inc." },
    { "9097F3", "Samsung Electronics Co.,Ltd" },
    { "909838", "Huawei Device Co., Ltd." },
    { "909864", "Impex-Sat GmbH&amp;Co KG" },
    { "909877", "Vestel Elektronik San ve Tic. A.S." },
    { "909916", "ELVEES NeoTek OJSC" },
    { "909A4A", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "909A77", "Texas Instruments" },
    { "909B6F", "Apple, Inc." },
    { "909C4A", "Apple, Inc." },
    { "909D7D", "Commscope" },
    { "909DAC", "Infinix mobility limited" },
    { "909DE0", "Newland Design + Assoc. Inc." },
    { "909E24", "ekey biometric systems gmbh" },
    { "909F22", "Zyxel Communications Corporation" },
    { "909F33", "EFM Networks" },
    { "909F43", "Accutron Instruments Inc." },
    { "90A0BE", "Cannice" },
    { "90A137", "Beijing Splendidtel Communication Technology Co,. Ltd" },
    { "90A196", "Private" },
    { "90A1BA", "PNetworks Electronics Information " },
    { "90A210", "United Telecoms Ltd" },
    { "90A25B", "Apple, Inc." },
    { "90A2DA", "GHEO SA" },
    { "90A365", "HMD Global Oy" },
    { "90A46A", "SISNET CO., LTD" },
    { "90A4DE", "Wistron Neweb Corporation" },
    { "90A57D", "Huawei Device Co., Ltd." },
    { "90A5AF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "90A62F", "NAVER" },
    { "90A6BF", "Quectel Wireless Solutions Co.,Ltd." },
    { "90A783", "JSW PACIFIC CORPORATION " },
    { "90A7BF", "EM Microelectronic" },
    { "90A7C1", "Pakedge Device and Software Inc." },
    { "90A822", "Amazon Technologies Inc." },
    { "90A935", "JWEntertainment" },
    { "90A9F7", "IEEE Registration Authority" },
    { "90AAC3", "Hitron Technologies. Inc" },
    { "90AB96", "Silicon Laboratories" },
    { "90AC3F", "BrightSign LLC" },
    { "90AC6D", "Apple, Inc." },
    { "90ADF7", "vivo Mobile Communication Co., Ltd." },
    { "90ADFC", "Telechips, Inc." },
    { "90AE1B", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "90AFD1", "netKTI Co., Ltd" },
    { "90B0ED", "Apple, Inc." },
    { "90B11C", "Dell Inc." },
    { "90B134", "Commscope" },
    { "90B144", "Samsung Electronics Co.,Ltd" },
    { "90B1E0", "Beijing Nebula Link Technology Co., Ltd" },
    { "90B21F", "Apple, Inc." },
    { "90B4DD", "ZPT R&D" },
    { "90B57F", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "90B622", "Samsung Electronics Co.,Ltd" },
    { "90B67A", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "90B685", "WEIFANG GOERTEK ELECTRONICS CO.,LTD" },
    { "90B686", "Murata Manufacturing Co., Ltd." },
    { "90B790", "Apple, Inc." },
    { "90B832", "Extreme Networks Headquarters" },
    { "90B8D0", "Joyent, Inc." },
    { "90B8E0", "SHENZHEN YANRAY TECHNOLOGY CO.,LTD" },
    { "90B931", "Apple, Inc." },
    { "90B942", "zte corporation" },
    { "90B97D", "Johnson Outdoors Marine Electronics d/b/a Minnkota" },
    { "90BDE6", "Quectel Wireless Solutions Co.,Ltd." },
    { "90BFD9", "Smart Innovation LLC" },
    { "90C115", "Sony Corporation" },
    { "90C119", "Nokia" },
    { "90C1C6", "Apple, Inc." },
    { "90C35F", "Nanjing Jiahao Technology Co., Ltd." },
    { "90C54A", "vivo Mobile Communication Co., Ltd." },
    { "90C682", "IEEE Registration Authority" },
    { "90C710", "zte corporation" },
    { "90C792", "Commscope" },
    { "90C7D8", "zte corporation" },
    { "90C99B", "Tesorion Nederland B.V." },
    { "90CAFA", "Google, Inc." },
    { "90CBA3", "TECNO MOBILE LIMITED" },
    { "90CC24", "Synaptics, Inc" },
    { "90CC7A", "Huawei Device Co., Ltd." },
    { "90CCDF", "Intel Corporate" },
    { "90CD1F", "Quectel Wireless Solutions Co.,Ltd." },
    { "90CDB6", "Hon Hai Precision Ind. Co.,Ltd." },
    { "90CDE8", "Apple, Inc." },
    { "90CEB8", "Texas Instruments" },
    { "90CF15", "Nokia Corporation" },
    { "90CF6F", "Dlogixs Co Ltd" },
    { "90CF7D", "Qingdao Hisense Communications Co.,Ltd." },
    { "90D092", "HUMAX Co., Ltd." },
    { "90D11B", "Palomar Medical Technologies" },
    { "90D20B", "Nokia Shanghai Bell Co., Ltd." },
    { "90D3CF", "ASKEY COMPUTER CORP" },
    { "90D432", "zte corporation" },
    { "90D473", "vivo Mobile Communication Co., Ltd." },
    { "90D689", "Huahao Fangzhou Technology Co.,Ltd" },
    { "90D74F", "Bookeen" },
    { "90D7BE", "Wavelab Global Inc." },
    { "90D7EB", "Texas Instruments" },
    { "90D852", "Comtec Co., Ltd." },
    { "90D8F3", "zte corporation" },
    { "90D92C", "HUG-WITSCHI AG" },
    { "90DA4E", "AVANU" },
    { "90DA6A", "FOCUS H&S Co., Ltd." },
    { "90DAF9", "Siemens Rail Automation SAU" },
    { "90DB46", "E-LEAD ELECTRONIC CO., LTD" },
    { "90DD5D", "Apple, Inc." },
    { "90DE80", "Shenzhen Century Xinyang Technology Co., Ltd" },
    { "90DF7D", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "90DFB7", "s.m.s smart microwave sensors GmbH" },
    { "90DFFB", "HOMERIDER SYSTEMS" },
    { "90E0F0", "IEEE 1722a Working Group" },
    { "90E17B", "Apple, Inc." },
    { "90E202", "Texas Instruments" },
    { "90E2BA", "Intel Corporate" },
    { "90E2FC", "IEEE Registration Authority" },
    { "90E317", "Mellanox Technologies, Inc." },
    { "90E468", "Guangzhou Shiyuan Electronic Technology Company Limited" },
    { "90E4B0", "SHARP Corporation" },
    { "90E5B1", "Espressif Inc." },
    { "90E643", "Tesla,Inc." },
    { "90E6BA", "ASUSTek COMPUTER INC." },
    { "90E710", "New H3C Technologies Co., Ltd" },
    { "90E7C4", "HTC Corporation" },
    { "90E868", "AzureWave Technology Inc." },
    { "90E95E", "Cisco Systems, Inc" },
    { "90EA60", "SPI Lasers Ltd " },
    { "90EB48", "Shanghai XinMiaoLink Technology Co., Ltd" },
    { "90EB50", "Cisco Systems, Inc" },
    { "90EC50", "C.O.B.O. SPA" },
    { "90EC77", "silicom" },
    { "90ECE3", "Nokia" },
    { "90ECEA", "Apple, Inc." },
    { "90EEC7", "Samsung Electronics Co.,Ltd" },
    { "90EED9", "UNIVERSAL DE DESARROLLOS ELECTRÓNICOS, SA" },
    { "90EF4A", "Dongguan Liesheng Electronic Co., Ltd." },
    { "90EF68", "Zyxel Communications Corporation" },
    { "90F005", "Xi'an Molead Technology Co., Ltd" },
    { "90F052", "MEIZU Technology Co., Ltd." },
    { "90F157", "Garmin International" },
    { "90F1AA", "Samsung Electronics Co.,Ltd" },
    { "90F1B0", "Hangzhou Anheng Info&Tech CO.,LTD" },
    { "90F260", "Shenzhen Honesty Electronics Co.,Ltd." },
    { "90F278", "Radius Gateway" },
    { "90F305", "HUMAX Co., Ltd." },
    { "90F3B7", "Kirisun Communications Co., Ltd." },
    { "90F3B8", "China Mobile Group Device Co.,Ltd." },
    { "90F421", "IEEE Registration Authority" },
    { "90F4C1", "Rand McNally" },
    { "90F644", "Huawei Device Co., Ltd." },
    { "90F652", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "90F721", "IndiNatus (IndiNatus India Private Limited)" },
    { "90F72F", "Phillips Machine & Welding Co., Inc. " },
    { "90F7B2", "New H3C Technologies Co., Ltd" },
    { "90F82E", "Amazon Technologies Inc." },
    { "90F891", "Kaon Group Co., Ltd." },
    { "90F964", "Rawasi Co" },
    { "90F970", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "90F9B7", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "90FB5B", "Avaya Inc" },
    { "90FB5D", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "90FBA6", "Hon Hai Precision Ind. Co.,Ltd." },
    { "90FD61", "Apple, Inc." },
    { "90FD73", "zte corporation" },
    { "90FD9F", "Silicon Laboratories" },
    { "90FF79", "Metro Ethernet Forum" },
    { "90FFD6", "Honor Device Co., Ltd." },
    { "940006", "jinyoung" },
    { "940070", "Nokia Corporation" },
    { "9400B0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "940149", "AutoHotBox" },
    { "94017D", "SHENZHEN SHLINK.CO.,LIMITED" },
    { "9401AC", "Wuhan Qianyang Iotian Technology Co., Ltd" },
    { "9401C2", "Samsung Electronics Co.,Ltd" },
    { "940230", "Logitech" },
    { "94026B", "Optictimes Co.,Ltd" },
    { "94049C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9404E3", "Vantiva USA LLC" },
    { "9405B6", "Liling FullRiver Electronics & Technology Ltd" },
    { "9405BB", "IEEE Registration Authority" },
    { "940853", "Liteon Technology Corporation" },
    { "9408C7", "Huawei Device Co., Ltd." },
    { "940937", "HUMAX Co., Ltd." },
    { "9409C9", "ALPSALPINE CO,.LTD" },
    { "9409D3", "shenzhen maxtopic technology co.,ltd" },
    { "940B19", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "940B2D", "NetView Technologies(Shenzhen) Co., Ltd" },
    { "940B83", "zte corporation" },
    { "940BCD", "Apple, Inc." },
    { "940BD5", "Himax Technologies, Inc" },
    { "940BFA", "EM Microelectronic" },
    { "940C6D", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "940C98", "Apple, Inc." },
    { "940D2D", "Universal Electronics, Inc." },
    { "940D4B", "Cisco Systems, Inc" },
    { "940E2A", "NXP Semiconductors Taiwan Ltd." },
    { "940E6B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "940EE7", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "94103E", "Belkin International Inc." },
    { "941042", "Fanox Electronic S.L." },
    { "9411DA", "ITF Fröschl GmbH" },
    { "941457", "Shenzhen Sundray Technologies company Limited" },
    { "94147A", "vivo Mobile Communication Co., Ltd." },
    { "9415B2", "Huawei Device Co., Ltd." },
    { "941625", "Apple, Inc." },
    { "941673", "Point Core SARL" },
    { "941700", "Xiaomi Communications Co Ltd" },
    { "941724", "Anhui Guoke Ruidian communication technology Co.,Ltd." },
    { "941787", "Nokia" },
    { "941865", "NETGEAR" },
    { "941882", "Hewlett Packard Enterprise " },
    { "94193A", "Elvaco AB" },
    { "941C56", "Actiontec Electronics, Inc" },
    { "941D1C", "TLab West Systems AB" },
    { "941F3A", "Ambiq" },
    { "941FA2", "Wuhan YuXin Semiconductor Co., Ltd." },
    { "942053", "Nokia Corporation" },
    { "942157", "Apple, Inc." },
    { "942197", "Stalmart Technology Limited" },
    { "94224C", "Qingdao Haier Technology Co.,Ltd" },
    { "94236E", "Shenzhen Junlan Electronic Ltd" },
    { "942453", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9424B8", "GREE ELECTRIC APPLIANCES, INC. OF ZHUHAI" },
    { "9424E1", "Alcatel-Lucent Enterprise" },
    { "942533", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "94261D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "942770", "BSH Hausgeräte GmbH" },
    { "942790", "TCT mobile ltd" },
    { "94282E", "New H3C Technologies Co., Ltd" },
    { "94286F", "zte corporation" },
    { "94290C", "Shenyang wisdom Foundation Technology Development Co., Ltd." },
    { "94292F", "New H3C Technologies Co., Ltd" },
    { "942957", "Airpo Networks Technology Co.,Ltd." },
    { "94298D", "Shanghai AdaptComm Technology Co., Ltd." },
    { "942A3F", "Diversey Inc" },
    { "942A6F", "Ubiquiti Inc" },
    { "942B68", "Apple, Inc." },
    { "942CB3", "HUMAX Co., Ltd." },
    { "942DDC", "Samsung Electronics Co.,Ltd" },
    { "942E17", "Schneider Electric Canada Inc" },
    { "942E63", "Finsécur" },
    { "94319B", "Alphatronics BV" },
    { "9431CB", "vivo Mobile Communication Co., Ltd." },
    { "943251", "ASKEY COMPUTER CORP" },
    { "9432C1", "Honor Device Co., Ltd." },
    { "9433D8", "Cisco Systems, Inc" },
    { "9433DD", "Taco Inc" },
    { "94342F", "ITEL MOBILE LIMITED" },
    { "943469", "Silicon Laboratories" },
    { "94350A", "Samsung Electronics Co.,Ltd" },
    { "943589", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9436E0", "Sichuan Bihong Broadcast &amp; Television New Technologies Co.,Ltd" },
    { "9437F7", "Huawei Device Co., Ltd." },
    { "9438AA", "Technology Innovation Institute" },
    { "94390E", "Intel Corporate" },
    { "9439E5", "Hon Hai Precision Ind. Co.,Ltd." },
    { "943A91", "Amazon Technologies Inc." },
    { "943AF0", "Nokia Corporation" },
    { "943BB0", "New H3C Technologies Co., Ltd" },
    { "943BB1", "Kaon Group Co., Ltd." },
    { "943C96", "Sagemcom Broadband SAS" },
    { "943CC6", "Espressif Inc." },
    { "943DC9", "Asahi Net, Inc." },
    { "943EE4", "WiSA Technologies Inc" },
    { "943EFD", "IEEE Registration Authority" },
    { "943F0C", "Genexis B.V." },
    { "943FBB", "JSC RPC Istok named after Shokin" },
    { "943FC2", "Hewlett Packard Enterprise " },
    { "943FD6", "Apple, Inc." },
    { "9440A2", "Anywave Communication Technologies, Inc." },
    { "9440C9", "Hewlett Packard Enterprise " },
    { "9440F3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9441C1", "Mini-Cam Limited" },
    { "94434D", "Ciena Corporation" },
    { "944444", "LG Innotek" },
    { "944452", "Belkin International Inc." },
    { "944560", "Google, Inc." },
    { "944696", "BaudTec Corporation" },
    { "944788", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9447B0", "BEIJING ESWIN COMPUTING TECHNOLOGY CO., LTD" },
    { "944996", "WiSilica Inc" },
    { "944A09", "BitWise Controls" },
    { "944A0C", "Sercomm Corporation." },
    { "944BF8", "TOZO INC" },
    { "944E5B", "Ubee Interactive Co., Limited" },
    { "944F4C", "Sound United LLC" },
    { "944FDB", "Nokia" },
    { "945044", "Texas Instruments" },
    { "945047", "Rechnerbetriebsgruppe" },
    { "945089", "SimonsVoss Technologies GmbH" },
    { "945103", "Samsung Electronics Co.,Ltd" },
    { "94513D", "iSmart Alarm, Inc." },
    { "9451BF", "Hyundai ESG" },
    { "9451DC", "Espressif Inc." },
    { "945244", "Samsung Electronics Co.,Ltd" },
    { "945330", "Hon Hai Precision Ind. Co.,Ltd." },
    { "945493", "Rigado, LLC" },
    { "9454C5", "Espressif Inc." },
    { "9454CE", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "9454DF", "YST CORP." },
    { "945641", "Palo Alto Networks" },
    { "9457A5", "Hewlett Packard" },
    { "9458CB", "Nintendo Co.,Ltd" },
    { "945907", "Shanghai HITE-BELDEN Network Technology Co., Ltd." },
    { "94592D", "EKE Building Technology Systems Ltd" },
    { "945AFC", "Amazon Technologies Inc." },
    { "945B7E", "TRILOBIT LTDA." },
    { "945C9A", "Apple, Inc." },
    { "945F34", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "946010", "Huawei Device Co., Ltd." },
    { "9460D5", "Hewlett Packard Enterprise" },
    { "94611E", "Wata Electronics Co.,Ltd. " },
    { "946124", "Pason Systems" },
    { "946269", "Commscope" },
    { "946372", "vivo Mobile Communication Co., Ltd." },
    { "9463D1", "Samsung Electronics Co.,Ltd" },
    { "946424", "Hewlett Packard Enterprise" },
    { "94652D", "OnePlus Technology (Shenzhen) Co., Ltd" },
    { "94659C", "Intel Corporate" },
    { "9466E7", "WOM Engineering" },
    { "94677E", "Belden India Private Limited" },
    { "946A77", "Vantiva USA LLC" },
    { "946AB0", "Arcadyan Corporation" },
    { "946C04", "EM Microelectronic" },
    { "946DAE", "Mellanox Technologies, Inc." },
    { "94706C", "Quectel Wireless Solutions Co.,Ltd." },
    { "9470D2", "WINFIRM TECHNOLOGY" },
    { "9471AC", "TCT mobile ltd" },
    { "94720F", "Guangdong Nanguang Photo&Video Systems Co., Ltd." },
    { "94742E", "MERCUSYS TECHNOLOGIES CO., LTD." },
    { "94756E", "QinetiQ North America" },
    { "9476B7", "Samsung Electronics Co.,Ltd" },
    { "94772B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "947806", "NINGBO SUNVOT TECHNOLOGY CO.,LTD" },
    { "947918", "ITEL MOBILE LIMITED" },
    { "947AF4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "947BAE", "Xiaomi Communications Co Ltd" },
    { "947BBE", "Ubicquia LLC" },
    { "947BE7", "Samsung Electronics Co.,Ltd" },
    { "947C3E", "Polewall Norge AS" },
    { "947D77", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "947EB9", "National Narrowband Network Communications Pty Ltd" },
    { "947F1D", "Shenzhen Fastrain Technology Co., Ltd." },
    { "947FD8", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "9481A4", "Azuray Technologies" },
    { "9483C4", "GL Technologies (Hong Kong) Limited" },
    { "94857A", "Evantage Industries Corp" },
    { "9486CD", "SEOUL ELECTRONICS&TELECOM" },
    { "9486D4", "Surveillance Pro Corporation" },
    { "94877C", "Commscope" },
    { "9487E0", "Xiaomi Communications Co Ltd" },
    { "948815", "Infinique Worldwide Inc" },
    { "948854", "Texas Instruments" },
    { "94885E", "Surfilter Network Technology Co., Ltd. " },
    { "948978", "Apple, Inc." },
    { "948AC6", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "948B03", "EAGET Innovation and Technology Co., Ltd." },
    { "948B93", "Xiaomi Communications Co Ltd" },
    { "948BC1", "Samsung Electronics Co.,Ltd" },
    { "948CD7", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "948D50", "Beamex Oy Ab" },
    { "948DEF", "Oetiker Schweiz AG" },
    { "948E6D", "Nintendo Co.,Ltd" },
    { "948E89", "INDUSTRIAS UNIDAS SA DE CV" },
    { "948ED3", "Arista Networks" },
    { "948FCF", "Commscope" },
    { "948FEE", "Verizon Telematics" },
    { "949010", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "949034", "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD" },
    { "94917F", "ASKEY COMPUTER CORP" },
    { "9492BC", "SYNTECH(HK) TECHNOLOGY LIMITED" },
    { "9492D2", "KCF Technologies, Inc." },
    { "949386", "Shenzhen SiACRRIER Industry Machines Co.,LTD" },
    { "949426", "Apple, Inc." },
    { "94944A", "Particle Industries Inc." },
    { "9495A0", "Google, Inc." },
    { "94974F", "Liteon Technology Corporation" },
    { "9497AE", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "949869", "zte corporation" },
    { "94988F", "Sagemcom Broadband SAS" },
    { "9498A2", "Shanghai LISTEN TECH.LTD" },
    { "949901", "Shenzhen YITOA Digital Appliance CO.,LTD" },
    { "949990", "VTC Telecommunications" },
    { "949AA9", "Microsoft Corporation" },
    { "949B2C", "Extreme Networks Headquarters" },
    { "949BFD", "Trans New Technology, Inc." },
    { "949C55", "Alta Data Technologies" },
    { "949D57", "Panasonic do Brasil Limitada" },
    { "949F3E", "Sonos, Inc." },
    { "949F3F", "Optek Digital Technology company limited" },
    { "949F8B", "zte corporation" },
    { "949FB4", "ChengDu JiaFaAnTai Technology Co.,Ltd" },
    { "94A04E", "Bostex Technology Co., LTD" },
    { "94A07D", "Huawei Device Co., Ltd." },
    { "94A081", "Silicon Laboratories" },
    { "94A1A2", "AMPAK Technology, Inc." },
    { "94A3CA", "KonnectONE, LLC" },
    { "94A408", "Shenzhen Trolink Technology CO, LTD" },
    { "94A40C", "Diehl Metering GmbH" },
    { "94A4B5", "Mitsubishi Electric Corporation" },
    { "94A4CE", "Sensear Pty Ltd" },
    { "94A4F9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "94A67E", "NETGEAR" },
    { "94A6D8", "New H3C Technologies Co., Ltd" },
    { "94A748", "New H3C Technologies Co., Ltd" },
    { "94A7B7", "zte corporation" },
    { "94A7BC", "BodyMedia, Inc." },
    { "94A990", "Espressif Inc." },
    { "94A9A8", "Texas Instruments" },
    { "94AA0A", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "94AAB8", "Joview(Beijing) Technology Co. Ltd." },
    { "94AB18", "cellXica ltd" },
    { "94ABDE", "OMX Technology - FZE" },
    { "94ABFE", "Nokia" },
    { "94ACCA", "trivum technologies GmbH" },
    { "94AD23", "Apple, Inc." },
    { "94AE61", "Alcatel Lucent" },
    { "94AEE3", "Belden Hirschmann Industries (Suzhou) Ltd." },
    { "94AEF0", "Cisco Systems, Inc" },
    { "94B01F", "Apple, Inc." },
    { "94B10A", "Samsung Electronics Co.,Ltd" },
    { "94B216", "Silicon Laboratories" },
    { "94B271", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "94B2CC", "PIONEER CORPORATION" },
    { "94B34F", "Ruckus Wireless" },
    { "94B3F7", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "94B40F", "Hewlett Packard Enterprise" },
    { "94B555", "Espressif Inc." },
    { "94B609", "Intel Corporate" },
    { "94B819", "Nokia" },
    { "94B86D", "Intel Corporate" },
    { "94B8C5", "RuggedCom Inc." },
    { "94B97E", "Espressif Inc." },
    { "94B9B4", "Aptos Technology" },
    { "94BA06", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "94BA31", "Visiontec da Amazônia Ltda." },
    { "94BA56", "Shenzhen Coship Electronics Co., Ltd." },
    { "94BB43", "AzureWave Technology Inc." },
    { "94BBAE", "Husqvarna AB" },
    { "94BDBE", "TPV Display Technology (Xiamen) Co.,Ltd." },
    { "94BE09", "China Mobile Group Device Co.,Ltd." },
    { "94BE36", "ADOPT NETTECH PVT LTD" },
    { "94BE46", "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd." },
    { "94BE50", "Remote Solution" },
    { "94BF1E", "eflow Inc. / Smart Device Planning and Development Division" },
    { "94BF2D", "Apple, Inc." },
    { "94BF80", "zte corporation" },
    { "94BF94", "Juniper Networks" },
    { "94BF95", "Shenzhen Coship Electronics Co., Ltd" },
    { "94BFC4", "Ruckus Wireless" },
    { "94C014", "Sorter Sp. j. Konrad Grzeszczyk MichaA, Ziomek" },
    { "94C038", "Tallac Networks" },
    { "94C150", "2Wire Inc" },
    { "94C2BD", "TECNOBIT" },
    { "94C2EF", "ASKEY COMPUTER CORP" },
    { "94C36B", "DRD Automation GmbH" },
    { "94C3E4", "Atlas Copco IAS GmbH" },
    { "94C4E9", "PowerLayer Microsystems HongKong Limited" },
    { "94C5A6", "ITEL MOBILE LIMITED" },
    { "94C691", "EliteGroup Computer Systems Co., LTD" },
    { "94C6EB", "NOVA electronics, Inc." },
    { "94C7A8", "Jiangsu Huitong Group Co.,Ltd." },
    { "94C7AF", "Raylios Technology" },
    { "94C960", "Zhongshan B&T technology.co.,ltd" },
    { "94C962", "Teseq AG" },
    { "94C9B7", "IEEE Registration Authority" },
    { "94CA0F", "Honeywell Analytics" },
    { "94CA9A", "Paul Vahle GmbH & Co. KG" },
    { "94CBCD", "zte corporation" },
    { "94CC04", "IEEE Registration Authority" },
    { "94CCB9", "Commscope" },
    { "94CDAC", "Creowave Oy" },
    { "94CDFD", "eero inc." },
    { "94CE0F", "Huawei Device Co., Ltd." },
    { "94CE2C", "Sony Corporation" },
    { "94CE31", "CTS Limited" },
    { "94CFB0", "Huawei Device Co., Ltd." },
    { "94D00D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "94D019", "Cydle Corp." },
    { "94D029", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "94D075", "CIS Crypto" },
    { "94D299", "Techmation Co.,Ltd." },
    { "94D2BC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "94D331", "Xiaomi Communications Co Ltd" },
    { "94D417", "GPI KOREA INC." },
    { "94D469", "Cisco Systems, Inc" },
    { "94D505", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "94D54D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "94D60E", "shenzhen yunmao information technologies co., ltd" },
    { "94D6DB", "NexFi" },
    { "94D723", "Shanghai DareGlobal Technologies Co.,Ltd" },
    { "94D771", "Samsung Electronics Co.,Ltd" },
    { "94D859", "TCT mobile ltd" },
    { "94D86B", "nass magnet Hungária Kft." },
    { "94D93C", "ENELPS" },
    { "94D9B3", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "94DB49", "SITCORP" },
    { "94DB56", "Sony Home Entertainment&Sound Products Inc" },
    { "94DBC9", "AzureWave Technology Inc." },
    { "94DBDA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "94DC4E", "AEV, spol. s r. o." },
    { "94DD3F", "A+V Link Technologies, Corp." },
    { "94DDF8", "Brother Industries, LTD." },
    { "94DE0E", "SmartOptics AS" },
    { "94DE80", "GIGA-BYTE TECHNOLOGY CO.,LTD." },
    { "94DEB8", "Silicon Laboratories" },
    { "94DF34", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "94DF4E", "Wistron InfoComm(Kunshan)Co.,Ltd." },
    { "94DF58", "IJ Electron CO.,Ltd." },
    { "94E0D0", "HealthStream Taiwan Inc." },
    { "94E0D6", "China Dragon Technology Limited" },
    { "94E129", "Samsung Electronics Co.,Ltd" },
    { "94E1AC", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "94E226", "D. ORtiz Consulting, LLC" },
    { "94E23C", "Intel Corporate" },
    { "94E2FD", "Boge Kompressoren OTTO Boge GmbH & Co. KG" },
    { "94E300", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "94E36D", "Texas Instruments" },
    { "94E3EE", "zte corporation" },
    { "94E4BA", "Huawei Device Co., Ltd." },
    { "94E686", "Espressif Inc." },
    { "94E6BA", "Samsung Electronics Co.,Ltd" },
    { "94E6F7", "Intel Corporate" },
    { "94E70B", "Intel Corporate" },
    { "94E711", "Xirka Dama Persada PT" },
    { "94E7EA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "94E7F3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "94E848", "FYLDE MICRO LTD" },
    { "94E8C5", "Commscope" },
    { "94E96A", "Apple, Inc." },
    { "94E979", "Liteon Technology Corporation" },
    { "94E98C", "Nokia" },
    { "94E9EE", "Huawei Device Co., Ltd." },
    { "94EA32", "Apple, Inc." },
    { "94EAEA", "TELLESCOM INDUSTRIA E COMERCIO EM TELECOMUNICACAO " },
    { "94EB2C", "Google, Inc." },
    { "94EBCD", "BlackBerry RTS" },
    { "94EC13", "Hangzhou Ezviz Software Co.,Ltd." },
    { "94EC32", "Silicon Laboratories" },
    { "94EE9F", "HMD Global Oy" },
    { "94EF49", "BDR Thermea Group B.V" },
    { "94EF50", "TP-Link Systems Inc." },
    { "94F128", "Hewlett Packard Enterprise " },
    { "94F19E", "HUIZHOU MAORONG INTELLIGENT TECHNOLOGY CO.,LTD" },
    { "94F278", "Elma Electronic" },
    { "94F2BB", "Valeo Vision Systems" },
    { "94F392", "Fortinet, Inc." },
    { "94F524", "Chengdu BeiZhongWangXin Technology Co.Ltd" },
    { "94F551", "Cadi Scientific Pte Ltd" },
    { "94F665", "Ruckus Wireless" },
    { "94F692", "Geminico co.,Ltd." },
    { "94F6A3", "Apple, Inc." },
    { "94F6D6", "Apple, Inc." },
    { "94F6F2", "Honor Device Co., Ltd." },
    { "94F717", "CIG SHANGHAI CO LTD" },
    { "94F720", "Tianjin Deviser Electronics Instrument Co., Ltd" },
    { "94F7AD", "Juniper Networks" },
    { "94F7BE", "SERCOMM PHILIPPINES INC" },
    { "94F827", "Shanghai Imilab Technology Co.Ltd" },
    { "94F929", "Meta Platforms, Inc." },
    { "94FAE8", "Shenzhen Eycom Technology Co., Ltd " },
    { "94FB29", "Zebra Technologies Inc." },
    { "94FBA7", "IEEE Registration Authority" },
    { "94FBB2", "SHENZHEN GONGJIN ELECTRONICS CO.,LT" },
    { "94FD1D", "WhereWhen Corp" },
    { "94FD2E", "Shanghai Uniscope Technologies Co.,Ltd" },
    { "94FE22", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "94FE9D", "SHENZHEN GONGJIN ELECTRONICS CO.,LT" },
    { "94FEF4", "Sagemcom Broadband SAS" },
    { "94FF06", "Hewlett Packard Enterprise" },
    { "94FF34", "HANSHOW TECHNOLOGY CO.,LTD." },
    { "94FF3C", "Fortinet, Inc." },
    { "94FF61", "China Mobile Group Device Co.,Ltd." },
    { "94FF7D", "AltoBeam Inc." },
    { "98006A", "zte corporation" },
    { "980074", "Raisecom Technology CO., LTD" },
    { "9800C1", "GuangZhou CREATOR Technology Co.,Ltd.(CHINA)" },
    { "9800C6", "Apple, Inc." },
    { "9801A7", "Apple, Inc." },
    { "980284", "Theobroma Systems GmbH" },
    { "9802D8", "IEEE Registration Authority" },
    { "98038A", "Texas Instruments" },
    { "98038E", "TP-Link Systems Inc." },
    { "98039B", "Mellanox Technologies, Inc." },
    { "9803A0", "ABB n.v. Power Quality Products" },
    { "9803CF", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "9803D8", "Apple, Inc." },
    { "980637", "IEEE Registration Authority" },
    { "98063A", "Home Control Singapore Pte Ltd" },
    { "98063C", "Samsung Electronics Co.,Ltd" },
    { "980709", "Huawei Device Co., Ltd." },
    { "98072D", "Texas Instruments" },
    { "980802", "ORBIS BV" },
    { "9809CF", "OnePlus Technology (Shenzhen) Co., Ltd" },
    { "980C33", "Silicon Laboratories" },
    { "980C82", "SAMSUNG ELECTRO MECHANICS CO., LTD." },
    { "980CA5", "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd." },
    { "980D2E", "HTC Corporation" },
    { "980D51", "Huawei Device Co., Ltd." },
    { "980D67", "Zyxel Communications Corporation" },
    { "980D6F", "Samsung Electronics Co.,Ltd" },
    { "980DAF", "Apple, Inc." },
    { "980E24", "Phytium Technology Co.,Ltd." },
    { "980EE4", "Private" },
    { "981082", "Nsolution Co., Ltd." },
    { "981094", "Shenzhen Vsun communication technology Co.,ltd" },
    { "9810E8", "Apple, Inc." },
    { "981223", "Tarmoc Network LTD" },
    { "9812E0", "Xiaomi Communications Co Ltd" },
    { "981333", "zte corporation" },
    { "9814D2", "Avonic" },
    { "9816CD", "leapio" },
    { "9816EC", "IC Intracom" },
    { "98171A", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "98173C", "Private" },
    { "9817F1", "zte corporation" },
    { "981888", "Cisco Meraki" },
    { "98192C", "Edgecore Networks Corporation" },
    { "981A35", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "981BB5", "ASSA ABLOY Korea Co., Ltd iRevo" },
    { "981C42", "LAIIER" },
    { "981CA2", "Apple, Inc." },
    { "981DAC", "Cyviz AS" },
    { "981DFA", "Samsung Electronics Co.,Ltd" },
    { "981E0F", "Jeelan (Shanghai Jeelan Technology Information Inc" },
    { "981E19", "Sagemcom Broadband SAS" },
    { "981E89", "Tianyi Telecom Terminals Company Limited" },
    { "981FB1", "Shenzhen Lemon Network Technology Co.,Ltd" },
    { "982044", "New H3C Technologies Co., Ltd" },
    { "98208E", "Definium Technologies" },
    { "98226E", "Amazon Technologies Inc." },
    { "9822EF", "Liteon Technology Corporation" },
    { "98234E", "Micromedia AG" },
    { "98247B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "98254A", "TP-Link Systems Inc" },
    { "98256E", "Private" },
    { "98262A", "Applied Research Associates, Inc" },
    { "9826AD", "Quectel Wireless Solutions Co.,Ltd." },
    { "982782", "IEEE Registration Authority" },
    { "98288B", "zhejiang Dusun Electron Co.,Ltd" },
    { "9828A6", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "98291D", "Jaguar de Mexico, SA de CV" },
    { "98293F", "Fujian Start Computer Equipment Co.,Ltd" },
    { "9829A6", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "982A0A", "Intelbras" },
    { "982CBC", "Intel Corporate" },
    { "982CBE", "2Wire Inc" },
    { "982D56", "Resolution Audio" },
    { "982D68", "Samsung Electronics Co., Ltd" },
    { "982DBA", "Fibergate Inc." },
    { "982F3C", "Sichuan Changhong Electric Ltd." },
    { "982F86", "vivo Mobile Communication Co., Ltd." },
    { "982FF8", "Huawei Device Co., Ltd." },
    { "983000", "Beijing KEMACOM Technologies Co., Ltd." },
    { "983071", "DAIKYUNG VASCOM" },
    { "983268", "Silicon Laboratories" },
    { "98348C", "Great Talent Technology Limited" },
    { "98349D", "Krauss Maffei Technologies GmbH" },
    { "983571", "Sub10 Systems Ltd" },
    { "9835B8", "Assembled Products Corporation" },
    { "9835ED", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "983713", "PT.Navicom Indonesia" },
    { "98387D", "ITRONIC TECHNOLOGY CO . , LTD ." },
    { "983910", "Kaon Group Co., Ltd." },
    { "98398E", "Samsung Electronics Co.,Ltd" },
    { "9839C0", "FLEXTRONICS" },
    { "983A1F", "Google, Inc." },
    { "983B16", "AMPAK Technology, Inc." },
    { "983B67", "DWnet Technologies(Suzhou) Corporation" },
    { "983B8F", "Intel Corporate" },
    { "983C8C", "Apple, Inc." },
    { "983DAE", "Espressif Inc." },
    { "983F60", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "983F66", "Funshion Online Technologies Co.,Ltd" },
    { "983F9F", "China SSJ (Suzhou) Network Technology Inc." },
    { "983FE8", "Samsung Electronics Co.,Ltd" },
    { "9840BB", "Dell Inc." },
    { "98415C", "Nintendo Co.,Ltd" },
    { "984246", "SOL INDUSTRY PTE., LTD" },
    { "984265", "Sagemcom Broadband SAS" },
    { "9843DA", "INTERTECH" },
    { "9843FA", "Intel Corporate" },
    { "9844B6", "INFRANOR SAS" },
    { "9844CE", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "984562", "Shanghai Baud Data Communication Co.,Ltd." },
    { "98460A", "Apple, Inc." },
    { "98473C", "SHANGHAI SUNMON COMMUNICATION TECHNOGY CO.,LTD" },
    { "984744", "Shenzhen Boomtech Industrial Corporation" },
    { "984827", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "984874", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "984914", "Wistron Neweb Corporation" },
    { "984925", "Juniper Networks" },
    { "98499F", "Domo Tactical Communications" },
    { "9849E1", "Boeing Defence Australia" },
    { "984A47", "CHG Hospital Beds" },
    { "984A6B", "Ruijie Networks Co.,LTD" },
    { "984B06", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "984B4A", "Commscope" },
    { "984BE1", "Hewlett Packard" },
    { "984C04", "Zhangzhou Keneng Electrical Equipment Co Ltd" },
    { "984CD3", "Mantis Deposition" },
    { "984E8A", "Samsung Electronics Co.,Ltd" },
    { "984E97", "Starlight Marketing (H. K.) Ltd." },
    { "984FEE", "Intel Corporate" },
    { "98502E", "Apple, Inc." },
    { "9850A3", "SIGNALTEK JSC" },
    { "985207", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "98523D", "Sunitec Enterprise Co.,Ltd" },
    { "98524A", "Vantiva USA LLC" },
    { "9852B1", "Samsung Electronics Co.,Ltd" },
    { "98541B", "Intel Corporate" },
    { "9857D3", "HON HAI-CCPBG PRECISION IND.CO.,LTD." },
    { "98588A", "SYSGRATION Ltd." },
    { "985945", "Texas Instruments" },
    { "985949", "LUXOTTICA GROUP S.P.A." },
    { "98597A", "Intel Corporate" },
    { "985A98", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "985AEB", "Apple, Inc." },
    { "985BB0", "KMDATA INC." },
    { "985C93", "SBG Systems SAS" },
    { "985D46", "PeopleNet Communication" },
    { "985D82", "Arista Networks" },
    { "985DAD", "Texas Instruments" },
    { "985E1B", "ConversDigital Co., Ltd." },
    { "985F41", "Intel Corporate" },
    { "985F4F", "Tongfang Computer Co.,Ltd." },
    { "985FD3", "Microsoft Corporation" },
    { "986022", "EMW Co., Ltd." },
    { "9860CA", "Apple, Inc." },
    { "986110", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "986297", "Shenzhen Techwinsemi Technology Co., Ltd." },
    { "986610", "zte corporation" },
    { "9866EA", "Industrial Control Communications, Inc." },
    { "98672E", "Skullcandy" },
    { "98698A", "Apple, Inc." },
    { "986B3D", "Commscope" },
    { "986C5C", "Jiangxi Gosun Guard Security Co.,Ltd" },
    { "986CF5", "zte corporation" },
    { "986D35", "IEEE Registration Authority" },
    { "986DC8", "TOSHIBA MITSUBISHI-ELECTRIC INDUSTRIAL SYSTEMS CORPORATION" },
    { "986EE8", "IEEE Registration Authority" },
    { "986F60", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "9870E8", "INNATECH SDN BHD" },
    { "9873C4", "Sage Electronic Engineering LLC" },
    { "98743D", "Shenzhen Jun Kai Hengye Technology Co. Ltd" },
    { "9874DA", "Infinix mobility limited" },
    { "98751A", "Huawei Device Co., Ltd." },
    { "987596", "EM Microelectronic" },
    { "9876B6", "Adafruit" },
    { "987770", "Pep Digital Technology (Guangzhou) Co., Ltd" },
    { "9877CB", "Vorteks ED" },
    { "9877D5", "WiZ" },
    { "9877E7", "Kaon Group Co., Ltd." },
    { "987A10", "Ericsson AB" },
    { "987A14", "Microsoft Corporation" },
    { "987A9B", "TCL MOKA International Limited" },
    { "987BF3", "Texas Instruments" },
    { "987DDD", "China Mobile Group Device Co.,Ltd." },
    { "987E46", "Emizon Networks Limited" },
    { "987EB5", "Huawei Device Co., Ltd." },
    { "987ECA", "Inventus Power Eletronica do Brasil LTDA" },
    { "987EE3", "vivo Mobile Communication Co., Ltd." },
    { "9880BB", "IEEE Registration Authority" },
    { "9880EE", "Samsung Electronics Co.,Ltd" },
    { "98818A", "Huawei Device Co., Ltd." },
    { "988217", "Disruptive Ltd" },
    { "988389", "Samsung Electronics Co.,Ltd" },
    { "9884E3", "Texas Instruments" },
    { "98865D", "Nokia Shanghai Bell Co., Ltd." },
    { "98868B", "Juniper Networks" },
    { "9886B1", "Flyaudio corporation (China)" },
    { "988744", "Wuxi Hongda Science and Technology Co.,LTD" },
    { "98874C", "SERVERCOM (INDIA) PRIVATE LIMITED" },
    { "98876C", "Huawei Device Co., Ltd." },
    { "98886C", "Huawei Device Co., Ltd." },
    { "9888E0", "Espressif Inc." },
    { "988924", "Texas Instruments" },
    { "9889ED", "Anadem Information Inc." },
    { "988B0A", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "988B5D", "Sagemcom Broadband SAS" },
    { "988B69", "Shenzhen hylitech Co.,LTD" },
    { "988BAD", "Corintech Ltd." },
    { "988CB3", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "988D46", "Intel Corporate" },
    { "988E34", "ZHEJIANG BOXSAM ELECTRONIC CO.,LTD" },
    { "988E4A", "NOXUS(BEIJING) TECHNOLOGY CO.,LTD" },
    { "988E79", "Qudelix, Inc." },
    { "988ED4", "ITEL MOBILE LIMITED" },
    { "988EDD", "TE Connectivity Limerick" },
    { "988F00", "Hewlett Packard Enterprise" },
    { "988FE0", "IEEE Registration Authority" },
    { "989080", "Linkpower Network System Inc Ltd." },
    { "989096", "Dell Inc." },
    { "9893CC", "LG ELECTRONICS INC" },
    { "989449", "Skyworth Wireless Technology Ltd." },
    { "9897CC", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "9897D1", "MitraStar Technology Corp." },
    { "9898FB", "Google, Inc." },
    { "989AB9", "zte corporation" },
    { "989BCB", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "989C57", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "989D39", "China Mobile Group Device Co.,Ltd." },
    { "989D40", "MIWA LOCK CO.,LTD." },
    { "989D5D", "Vantiva USA LLC" },
    { "989DB2", "GOIP Global Services Pvt. Ltd." },
    { "989DE5", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "989E63", "Apple, Inc." },
    { "989F1A", "Private" },
    { "989F1E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "98A14A", "Quectel Wireless Solutions Co.,Ltd." },
    { "98A2C0", "Cisco Systems, Inc" },
    { "98A316", "Espressif Inc." },
    { "98A404", "Ericsson AB" },
    { "98A40E", "Snap, Inc." },
    { "98A44E", "IEC Technologies S. de R.L de C.V." },
    { "98A5F9", "Apple, Inc." },
    { "98A7B0", "MCST ZAO" },
    { "98A829", "AltoBeam Inc." },
    { "98A878", "Agnigate Technologies Private Limited" },
    { "98A92D", "New H3C Technologies Co., Ltd" },
    { "98A942", "Guangzhou Tozed Kangwei Intelligent Technology Co., LTD" },
    { "98A965", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "98AA3C", "Will i-tech Co., Ltd." },
    { "98AAD7", "BLUE WAVE NETWORKING CO LTD" },
    { "98AAFC", "IEEE Registration Authority" },
    { "98AB15", "Fujian Youyike Technology Co.,Ltd" },
    { "98ACEF", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "98AD1D", "Huawei Device Co., Ltd." },
    { "98AE71", "VVDN Technologies Pvt Ltd" },
    { "98AF65", "Intel Corporate" },
    { "98B039", "Nokia" },
    { "98B08B", "Samsung Electronics Co.,Ltd" },
    { "98B177", "LANDIS + GYR" },
    { "98B379", "Apple, Inc." },
    { "98B3EF", "Huawei Device Co., Ltd." },
    { "98B6E9", "Nintendo Co.,Ltd" },
    { "98B71E", "Infinix mobility limited" },
    { "98B785", "Shenzhen 10Gtek Transceivers Co., Limited" },
    { "98B8BA", "LG Electronics (Mobile Communications)" },
    { "98B8BC", "Samsung Electronics Co.,Ltd" },
    { "98B8E3", "Apple, Inc." },
    { "98BA39", "Doro AB" },
    { "98BA5F", "TP-Link Systems Inc." },
    { "98BB1E", "BYD Precision Manufacture Company Ltd." },
    { "98BB99", "Phicomm (Sichuan) Co.,Ltd." },
    { "98BC57", "SVA TECHNOLOGIES CO.LTD" },
    { "98BC99", "Edeltech Co.,Ltd." },
    { "98BD80", "Intel Corporate" },
    { "98BE94", "IBM" },
    { "98BEDC", "Honor Device Co., Ltd." },
    { "98BFF4", "MARKIN co., Ltd." },
    { "98C08A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "98C0EB", "Global Regency Ltd" },
    { "98C3D2", "Ningbo Sanxing Medical Electric Co.,Ltd" },
    { "98C5DB", "Ericsson AB" },
    { "98C7A4", "Shenzhen HS Fiber Communication Equipment CO., LTD" },
    { "98C81C", "BAYTEC LIMITED" },
    { "98C845", "PacketAccess" },
    { "98C854", "Chiun Mai Communication System, Inc" },
    { "98C8B8", "vivo Mobile Communication Co., Ltd." },
    { "98C97C", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "98C9BE", "Shenzhen SDMC Technology CO., LTD" },
    { "98CA20", "Shanghai SIMCOM Ltd." },
    { "98CA33", "Apple, Inc." },
    { "98CB27", "Galore Networks Pvt. Ltd." },
    { "98CB38", "Boxin Communications Limited Liability Company" },
    { "98CBA4", "Benchmark Electronics" },
    { "98CC4D", "Shenzhen mantunsci co., LTD" },
    { "98CCD9", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "98CCE4", "Shenzhen Mindray Animal Medical Technology Co.,LTD" },
    { "98CCF3", "Amazon Technologies Inc." },
    { "98CDAC", "Espressif Inc." },
    { "98CDB4", "Virident Systems, Inc." },
    { "98CF53", "BBK EDUCATIONAL ELECTRONICS CORP.,LTD." },
    { "98D293", "Google, Inc." },
    { "98D331", "Shenzhen Bolutek Technology Co.,Ltd." },
    { "98D3D2", "MEKRA Lang GmbH & Co. KG" },
    { "98D3D7", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "98D3E7", "Netafim L" },
    { "98D686", "Chyi Lee industry Co., ltd." },
    { "98D6BB", "Apple, Inc." },
    { "98D6F7", "LG Electronics (Mobile Communications)" },
    { "98D742", "Samsung Electronics Co.,Ltd" },
    { "98D7E1", "Cisco Systems, Inc" },
    { "98D863", "Shanghai High-Flying Electronics Technology Co., Ltd" },
    { "98D88C", "Nortel Networks" },
    { "98D93D", "Demant Enterprise A/S" },
    { "98DA92", "Vuzix Corporation" },
    { "98DAC4", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "98DCD9", "UNITEC Co., Ltd." },
    { "98DD5B", "TAKUMI JAPAN LTD" },
    { "98DD60", "Apple, Inc." },
    { "98DDEA", "Infinix mobility limited" },
    { "98DED0", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "98DF82", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "98E081", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "98E0D9", "Apple, Inc." },
    { "98E165", "Accutome" },
    { "98E255", "Nintendo Co.,Ltd" },
    { "98E476", "Zentan" },
    { "98E55B", "Intelbras" },
    { "98E743", "Dell Inc." },
    { "98E79A", "Foxconn(NanJing) Communication Co.,Ltd." },
    { "98E7D5", "NXP Semiconductor (Tianjin) LTD." },
    { "98E7F4", "Hewlett Packard" },
    { "98E7F5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "98E848", "Axiim" },
    { "98E8FA", "Nintendo Co.,Ltd" },
    { "98EC65", "Cosesy ApS" },
    { "98ED5C", "Tesla,Inc." },
    { "98ED7E", "eero inc." },
    { "98EDCA", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "98EE8C", "zte corporation" },
    { "98EE94", "Xiaomi Communications Co Ltd" },
    { "98EECB", "Wistron Infocomm (Zhongshan) Corporation" },
    { "98EF9B", "OHSUNG" },
    { "98F058", "Lynxspring, Incl." },
    { "98F07B", "Texas Instruments" },
    { "98F083", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "98F0AB", "Apple, Inc." },
    { "98F112", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "98F170", "Murata Manufacturing Co., Ltd." },
    { "98F181", "New H3C Technologies Co., Ltd" },
    { "98F199", "NEC Platforms, Ltd." },
    { "98F1AE", "Senaisc" },
    { "98F217", "Castlenet Technology Inc." },
    { "98F2B3", "Hewlett Packard Enterprise " },
    { "98F3F6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "98F428", "zte corporation" },
    { "98F487", "Texas Instruments" },
    { "98F4AB", "Espressif Inc." },
    { "98F537", "zte corporation" },
    { "98F5A9", "OHSUNG" },
    { "98F621", "Xiaomi Communications Co Ltd" },
    { "98F67A", "Chipsea Technologies (Shenzhen) Crop." },
    { "98F781", "Commscope" },
    { "98F7D7", "Commscope" },
    { "98F8C1", "IDT Technology Limited" },
    { "98F8DB", "Marini Impianti Industriali s.r.l." },
    { "98F9C7", "IEEE Registration Authority" },
    { "98F9CC", "Zhejiang Dahua Technology Co., Ltd." },
    { "98FA2E", "Sony Interactive Entertainment Inc." },
    { "98FA9B", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "98FAA7", "INNONET" },
    { "98FAE3", "Xiaomi Communications Co Ltd" },
    { "98FB12", "Grand Electronics (HK) Ltd" },
    { "98FB27", "Samsung Electronics Co.,Ltd" },
    { "98FBF5", "ATRALTECH" },
    { "98FC11", "Cisco-Linksys, LLC" },
    { "98FC84", "IEEE Registration Authority" },
    { "98FD74", "ACT.CO.LTD" },
    { "98FDB4", "Primax Electronics Ltd." },
    { "98FE03", "Ericsson - North America" },
    { "98FE3E", "Intel Corporate" },
    { "98FE94", "Apple, Inc." },
    { "98FEE1", "Apple, Inc." },
    { "98FF6A", "OTEC(Shanghai)Technology Co.,Ltd." },
    { "98FFD0", "Lenovo Mobile Communication Technology Ltd." },
    { "9C00D3", "SHENZHEN IK WORLD Technology Co., Ltd" },
    { "9C0111", "Shenzhen Newabel Electronic Co., Ltd." },
    { "9C0298", "Samsung Electronics Co.,Ltd" },
    { "9C0351", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9C039E", "Beijing Winchannel Software Technology Co., Ltd" },
    { "9C0473", "Tecmobile (International) Ltd. " },
    { "9C04EB", "Apple, Inc." },
    { "9C0567", "Honor Device Co., Ltd." },
    { "9C0591", "Mellanox Technologies, Inc." },
    { "9C05D6", "Ubiquiti Inc" },
    { "9C061B", "Hangzhou H3C Technologies Co., Limited" },
    { "9C066E", "Hytera Communications Corporation Limited" },
    { "9C06CF", "PLAUD Inc." },
    { "9C0971", "New H3C Technologies Co., Ltd" },
    { "9C098B", "Cisco Systems, Inc" },
    { "9C0B05", "eero inc." },
    { "9C0C35", "Shenzhenshi Xinzhongxin Technology Co.Ltd" },
    { "9C0CDF", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "9C0DAC", "Tymphany HK Limited" },
    { "9C0E4A", "Shenzhen Vastking Electronic Co.,Ltd." },
    { "9C0E51", "Schneider Electric" },
    { "9C1221", "Sichuan AI-Link Technology Co., Ltd." },
    { "9C139E", "Espressif Inc." },
    { "9C13AB", "Chanson Water Co., Ltd." },
    { "9C1463", "Zhejiang Dahua Technology Co., Ltd." },
    { "9C1465", "Edata Elektronik San. ve Tic. A.Ş." },
    { "9C1874", "Nokia Danmark A/S" },
    { "9C19C2", "Dongguan Liesheng Electronic Co., Ltd." },
    { "9C1A25", "Apple, Inc." },
    { "9C1C12", "Hewlett Packard Enterprise" },
    { "9C1C37", "AltoBeam (China) Inc." },
    { "9C1C6D", "HEFEI DATANG STORAGE TECHNOLOGY CO.,LTD" },
    { "9C1D36", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9C1D58", "Texas Instruments" },
    { "9C1E95", "Actiontec Electronics, Inc" },
    { "9C1EA4", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "9C1ECE", "ALT Co., Ltd." },
    { "9C1ECF", "Valeo Telematik und Akustik GmbH" },
    { "9C1FCA", "Hangzhou AlmightyDigit Technology Co., Ltd" },
    { "9C1FDD", "Accupix Inc." },
    { "9C207B", "Apple, Inc." },
    { "9C216A", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "9C2183", "Broadcom Limited" },
    { "9C220E", "TASCAN Systems GmbH" },
    { "9C2472", "Sagemcom Broadband SAS" },
    { "9C2595", "Samsung Electronics Co.,Ltd" },
    { "9C25BE", "Wildlife Acoustics, Inc." },
    { "9C2840", "Discovery Technology,LTD.." },
    { "9C28B3", "Apple, Inc." },
    { "9C28BF", "Continental Automotive Czech Republic s.r.o." },
    { "9C28EF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9C28F7", "Xiaomi Communications Co Ltd" },
    { "9C293F", "Apple, Inc." },
    { "9C2976", "Intel Corporate" },
    { "9C2A70", "Hon Hai Precision Ind. Co.,Ltd." },
    { "9C2A83", "Samsung Electronics Co.,Ltd" },
    { "9C2BA6", "Ruijie Networks Co.,LTD" },
    { "9C2D49", "Nanowell Info Tech Co., Limited" },
    { "9C2DCD", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "9C2DCF", "Shishi Tongyun Technology(Chengdu)Co.,Ltd." },
    { "9C2E7A", "Samsung Electronics Co.,Ltd" },
    { "9C2EA1", "Xiaomi Communications Co Ltd" },
    { "9C2F4E", "zte corporation" },
    { "9C2F73", "Universal Tiancheng Technology (Beijing) Co., Ltd." },
    { "9C2F9D", "Liteon Technology Corporation" },
    { "9C305B", "Hon Hai Precision Ind. Co.,Ltd." },
    { "9C3066", "RWE Effizienz GmbH" },
    { "9C3178", "Foshan Huadian Intelligent Communications Teachnologies Co.,Ltd" },
    { "9C31B6", "Kulite Semiconductor Products Inc" },
    { "9C31C3", "SKY UK LIMITED" },
    { "9C32A9", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "9C32CE", "CANON INC." },
    { "9C3426", "Commscope" },
    { "9C3583", "Nipro Diagnostics, Inc" },
    { "9C35EB", "Apple, Inc." },
    { "9C36F8", "Hyundai Kefico" },
    { "9C3708", "Hewlett Packard Enterprise" },
    { "9C37CB", "Sony Interactive Entertainment Inc." },
    { "9C37F4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9C3818", "Cisco Systems, Inc" },
    { "9C3928", "Samsung Electronics Co.,Ltd" },
    { "9C3A9A", "Shenzhen Sundray Technologies company Limited" },
    { "9C3AAF", "Samsung Electronics Co.,Ltd" },
    { "9C3B91", "VSSL" },
    { "9C3DCF", "NETGEAR" },
    { "9C3E53", "Apple, Inc." },
    { "9C3EAA", "EnvyLogic Co.,Ltd." },
    { "9C40CD", "Synclayer Inc." },
    { "9C417C", "Hame Technology Co., Limited " },
    { "9C431E", "IEEE Registration Authority" },
    { "9C443D", "CHENGDU XUGUANG TECHNOLOGY CO, LTD" },
    { "9C44A6", "SwiftTest, Inc." },
    { "9C4563", "DIMEP Sistemas" },
    { "9C45F0", "SKYLARK ELECTRONICS PVT LTD" },
    { "9C4782", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "9C47F4", "Nokia" },
    { "9C47F9", "LJU Automatisierungstechnik GmbH" },
    { "9C4929", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9C4952", "Dongguan Liesheng Electronic Co., Ltd." },
    { "9C497F", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "9C4A7B", "Nokia Corporation" },
    { "9C4B6B", "iFlight Technology Company Limited" },
    { "9C4CAE", "Mesa Labs" },
    { "9C4E20", "Cisco Systems, Inc" },
    { "9C4E36", "Intel Corporate" },
    { "9C4E8E", "ALT Systems Ltd" },
    { "9C4EBF", "BoxCast" },
    { "9C4F5F", "Google, Inc." },
    { "9C4FAC", "zte corporation" },
    { "9C4FCF", "TCT mobile ltd" },
    { "9C4FDA", "Apple, Inc." },
    { "9C50D1", "Murata Manufacturing Co., Ltd." },
    { "9C50EE", "Cambridge Industries(Group) Co.,Ltd." },
    { "9C52F8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9C5322", "TP-Link Systems Inc" },
    { "9C5385", "PT. Hartono Istana Teknologi" },
    { "9C53CD", "ENGICAM s.r.l." },
    { "9C5416", "Cisco Systems, Inc" },
    { "9C541C", "Shenzhen My-power Technology Co.,Ltd" },
    { "9C5440", "ChengDu TD Tech" },
    { "9C5467", "Nokia" },
    { "9C54C2", "New H3C Technologies Co., Ltd" },
    { "9C54CA", "Zhengzhou VCOM Science and Technology Co.,Ltd" },
    { "9C54DA", "SkyBell Technologies Inc." },
    { "9C558F", "Lockin Technology(Beijing) Co.,Ltd." },
    { "9C55B4", "I.S.E. S.r.l." },
    { "9C5636", "Huawei Device Co., Ltd." },
    { "9C5711", "Feitian Xunda(Beijing) Aeronautical Information Technology Co., Ltd." },
    { "9C5766", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9C57AD", "Cisco Systems, Inc" },
    { "9C57BC", "eero inc." },
    { "9C583C", "Apple, Inc." },
    { "9C5884", "Apple, Inc." },
    { "9C5A44", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "9C5A80", "Juniper Networks" },
    { "9C5A81", "Xiaomi Communications Co Ltd" },
    { "9C5A8A", "DJI BAIWANG TECHNOLOGY CO LTD" },
    { "9C5B96", "NMR Corporation" },
    { "9C5C8D", "FIREMAX INDÚSTRIA E COMÉRCIO DE PRODUTOS ELETRÔNICOS LTDA" },
    { "9C5C8E", "ASUSTek COMPUTER INC." },
    { "9C5CF9", "Sony Corporation" },
    { "9C5D12", "Extreme Networks Headquarters" },
    { "9C5D95", "VTC Electronics Corp." },
    { "9C5E73", "Calibre UK LTD" },
    { "9C5F5A", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "9C5FB0", "Samsung Electronics Co.,Ltd" },
    { "9C6076", "Apple, Inc." },
    { "9C611D", "Panasonic Corporation of North America" },
    { "9C6121", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "9C61D7", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9C62AB", "Sumavision Technologies Co.,Ltd" },
    { "9C635B", "zte corporation" },
    { "9C63C0", "Mellanox Technologies, Inc." },
    { "9C63ED", "zte corporation" },
    { "9C645E", "Harman Consumer Group" },
    { "9C648B", "Apple, Inc." },
    { "9C65B0", "Samsung Electronics Co.,Ltd" },
    { "9C65EB", "Intel Corporate" },
    { "9C65EE", "Zhone Technologies, Inc." },
    { "9C65F9", "AcSiP Technology Corp." },
    { "9C65FA", "AcSiP" },
    { "9C6650", "Glodio Technolies Co.,Ltd Tianjin Branch" },
    { "9C6697", "Cisco Systems, Inc" },
    { "9C67D6", "Intel Corporate" },
    { "9C685B", "Octonion SA" },
    { "9C6865", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "9C6937", "Qorvo International Pte. Ltd." },
    { "9C69B4", "IEEE Registration Authority" },
    { "9C69D1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9C69D3", "ASIX Electronics Corporation" },
    { "9C69ED", "Arista Networks" },
    { "9C6ABE", "QEES ApS." },
    { "9C6B00", "ASRock Incorporation" },
    { "9C6B37", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "9C6B72", "Realme Chongqing MobileTelecommunications Corp Ltd" },
    { "9C6BF0", "Shenzhen Yipingfang Network Technology Co., Ltd." },
    { "9C6C15", "Microsoft Corporation" },
    { "9C6F52", "zte corporation" },
    { "9C713A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9C7370", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9C73B1", "Samsung Electronics Co.,Ltd" },
    { "9C7403", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "9C741A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9C746F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9C7514", "Wildix srl" },
    { "9C756E", "Ajax Systems DMCC" },
    { "9C760E", "Apple, Inc." },
    { "9C7613", "Ring LLC" },
    { "9C77AA", "NADASNV" },
    { "9C79AC", "Suntec Software(Shanghai) Co., Ltd." },
    { "9C79E3", "Apple, Inc." },
    { "9C7A03", "Ciena Corporation" },
    { "9C7BD2", "NEOLAB Convergence" },
    { "9C7BEF", "Hewlett Packard" },
    { "9C7DA3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9C7DC0", "Tech4home, Lda" },
    { "9C7F57", "UNIC Memory Technology Co Ltd" },
    { "9C7F81", "SHENZHEN FAST TECHNOLOGIES CO.,LTD" },
    { "9C807D", "SYSCABLE Korea Inc." },
    { "9C80DF", "Arcadyan Technology Corporation" },
    { "9C823F", "Huawei Device Co., Ltd." },
    { "9C8275", "Yichip Microelectronics (Hangzhou) Co.,Ltd" },
    { "9C8281", "vivo Mobile Communication Co., Ltd." },
    { "9C8306", "Samsung Electronics Co.,Ltd" },
    { "9C83BF", "PRO-VISION, Inc." },
    { "9C84B6", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "9C84BF", "Apple, Inc." },
    { "9C8566", "Wingtech Mobile Communications Co.,Ltd." },
    { "9C86DA", "Phoenix Geophysics Ltd." },
    { "9C8824", "PetroCloud LLC" },
    { "9C8888", "Simac Techniek NV" },
    { "9C88AD", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "9C891E", "FireBrick Ltd" },
    { "9C8ACB", "Juniper Networks" },
    { "9C8BA0", "Apple, Inc." },
    { "9C8BF1", "The Warehouse Limited" },
    { "9C8C6E", "Samsung Electronics Co.,Ltd" },
    { "9C8CD8", "Hewlett Packard Enterprise " },
    { "9C8D1A", "INTEG process group inc" },
    { "9C8D7C", "ALPSALPINE CO,.LTD" },
    { "9C8DD3", "Leonton Technologies" },
    { "9C8E99", "Hewlett Packard" },
    { "9C8E9C", "Huawei Device Co., Ltd." },
    { "9C8ECD", "Amcrest Technologies" },
    { "9C8EDC", "Teracom Limited" },
    { "9C9019", "Beyless" },
    { "9C924F", "Apple, Inc." },
    { "9C934E", "Xerox Corporation" },
    { "9C93B0", "Megatronix (Beijing) Technology Co., Ltd." },
    { "9C93E4", "Private" },
    { "9C9561", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "9C9567", "Huawei Device Co., Ltd." },
    { "9C956E", "Microchip Technology Inc." },
    { "9C95F8", "SmartDoor Systems, LLC" },
    { "9C9613", "Lenovo Future Communication Technology (Chongqing) Company Limited" },
    { "9C971B", "Intel Corporate" },
    { "9C9726", "Technicolor Delivery Technologies Belgium NV" },
    { "9C9789", "1MORE" },
    { "9C9793", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9C9811", "Guangzhou Sunrise Electronics Development Co., Ltd" },
    { "9C99A0", "Xiaomi Communications Co Ltd" },
    { "9C99CD", "Voippartners" },
    { "9C9AC0", "LEGO System A/S" },
    { "9C9C1D", "Starkey Labs Inc." },
    { "9C9C1F", "Espressif Inc." },
    { "9C9C40", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "9C9D5D", "Raden Inc" },
    { "9C9D7E", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "9C9E03", "awayfrom" },
    { "9C9E6E", "Espressif Inc." },
    { "9C9E71", "Huawei Device Co., Ltd." },
    { "9C9ED5", "Xiaomi Communications Co Ltd" },
    { "9CA10A", "SCLE SFE" },
    { "9CA134", "Nike, Inc." },
    { "9CA2F4", "TP-Link Systems Inc" },
    { "9CA389", "Nokia" },
    { "9CA3A9", "Guangzhou Juan Optical and Electronical Tech Joint Stock Co., Ltd" },
    { "9CA3BA", "SAKURA Internet Inc." },
    { "9CA513", "Samsung Electronics Co.,Ltd" },
    { "9CA525", "Shandong USR IOT Technology Limited" },
    { "9CA570", "eero inc." },
    { "9CA577", "Osorno Enterprises Inc." },
    { "9CA5C0", "vivo Mobile Communication Co., Ltd." },
    { "9CA615", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "9CA69D", "Whaley Technology Co.Ltd" },
    { "9CA6D8", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "9CA9B8", "Cisco Systems, Inc" },
    { "9CA9C5", "Apple, Inc." },
    { "9CA9E4", "zte corporation" },
    { "9CAA1B", "Microsoft Corporation" },
    { "9CAA5D", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "9CAC6D", "Universal Electronics, Inc." },
    { "9CAD97", "Hon Hai Precision Ind. Co.,Ltd." },
    { "9CADEF", "Obihai Technology, Inc." },
    { "9CAED3", "Seiko Epson Corporation" },
    { "9CAF6F", "ITEL MOBILE LIMITED" },
    { "9CAFCA", "Cisco Systems, Inc" },
    { "9CB008", "Ubiquitous Computing Technology Corporation" },
    { "9CB150", "Intel Corporate" },
    { "9CB1DC", "Earda Technologies co Ltd" },
    { "9CB206", "HMS Industrial Networks" },
    { "9CB2B2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9CB2E8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9CB400", "zte corporation" },
    { "9CB654", "Hewlett Packard" },
    { "9CB6D0", "Rivet Networks" },
    { "9CB70D", "Liteon Technology Corporation" },
    { "9CB793", "Creatcomm Technology Inc." },
    { "9CB8B4", "AMPAK Technology,Inc." },
    { "9CBAC9", "Telit Communication s.p.a" },
    { "9CBB98", "Shen Zhen RND Electronic Co.,LTD" },
    { "9CBCF0", "Xiaomi Communications Co Ltd" },
    { "9CBD6E", "DERA Co., Ltd" },
    { "9CBD9D", "SkyDisk, Inc." },
    { "9CBEE0", "Biosoundlab Co., Ltd." },
    { "9CBF0D", "Framework Computer Inc." },
    { "9CBFCD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9CC077", "PrintCounts, LLC" },
    { "9CC0D2", "Conductix-Wampfler GmbH" },
    { "9CC12D", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "9CC172", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9CC2C4", "IEIT SYSTEMS Co., Ltd." },
    { "9CC394", "Apple, Inc." },
    { "9CC7A6", "AVM GmbH" },
    { "9CC7B1", "ELITEGROUP COMPUTER SYSTEMS CO.,LTD." },
    { "9CC7D1", "SHARP Corporation" },
    { "9CC7D3", "AzureWave Technology Inc." },
    { "9CC893", "Juniper Networks" },
    { "9CC8AE", "Becton, Dickinson and Company" },
    { "9CC8E9", "Amazon Technologies Inc." },
    { "9CC8FC", "Commscope" },
    { "9CC950", "Baumer Holding" },
    { "9CC9EB", "NETGEAR" },
    { "9CCAD9", "Nokia Corporation" },
    { "9CCBF7", "CLOUD STAR TECHNOLOGY CO., LTD." },
    { "9CCC83", "Juniper Networks" },
    { "9CCD82", "CHENG UEI PRECISION INDUSTRY CO.,LTD" },
    { "9CCE88", "Ruijie Networks Co.,LTD" },
    { "9CD1D0", "Guangzhou Ronsuo Electronic Technology Co.,Ltd" },
    { "9CD21E", "Hon Hai Precision Ind. Co.,Ltd." },
    { "9CD24B", "zte corporation" },
    { "9CD332", "PLC Technology Ltd" },
    { "9CD35B", "Samsung Electronics Co.,Ltd" },
    { "9CD36D", "NETGEAR" },
    { "9CD48B", "Innolux Technology Europe BV" },
    { "9CD4A6", "SERVERCOM (INDIA) PRIVATE LIMITED" },
    { "9CD57D", "Cisco Systems, Inc" },
    { "9CD643", "D-Link International" },
    { "9CD8E3", "Wuhan Huazhong Numerical Control Co., Ltd" },
    { "9CD917", "Motorola Mobility LLC, a Lenovo Company" },
    { "9CD9CB", "Lesira Manufacturing Pty Ltd" },
    { "9CDA36", "TECNO MOBILE LIMITED" },
    { "9CDA3E", "Intel Corporate" },
    { "9CDAA8", "Apple, Inc." },
    { "9CDAB7", "Hewlett Packard Enterprise" },
    { "9CDB07", "Yellowtec GmbH" },
    { "9CDBAF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9CDBCB", "Funshion Online Technologies Co.,Ltd" },
    { "9CDC71", "Hewlett Packard Enterprise " },
    { "9CDD1F", "Intelligent Steward Co.,Ltd" },
    { "9CDE4D", "ML vision Co.,LTD" },
    { "9CDEF0", "Jiangxi Risound Electronics Co.,LTD" },
    { "9CDF03", "Harman/Becker Automotive Systems GmbH" },
    { "9CDFB1", "Shenzhen Crave Communication Co., LTD" },
    { "9CDFB3", "NXP Semiconductor (Tianjin) LTD." },
    { "9CE041", "Nokia" },
    { "9CE063", "Samsung Electronics Co.,Ltd" },
    { "9CE10E", "NCTech Ltd" },
    { "9CE176", "Cisco Systems, Inc" },
    { "9CE1D6", "Junger Audio-Studiotechnik GmbH" },
    { "9CE230", "JULONG CO,.LTD." },
    { "9CE330", "Cisco Meraki" },
    { "9CE33F", "Apple, Inc." },
    { "9CE374", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "9CE549", "IEEE Registration Authority" },
    { "9CE635", "Nintendo Co., Ltd." },
    { "9CE65E", "Apple, Inc." },
    { "9CE6E7", "Samsung Electronics Co.,Ltd" },
    { "9CE7BD", "Winduskorea co., Ltd" },
    { "9CE82B", "vivo Mobile Communication Co., Ltd." },
    { "9CE895", "New H3C Technologies Co., Ltd" },
    { "9CE91C", "zte corporation" },
    { "9CE91E", "TEJAS NETWORKS LTD" },
    { "9CE951", "Shenzhen Sang Fei Consumer Communications Ltd., Co." },
    { "9CEA97", "Honor Device Co., Ltd." },
    { "9CEBE8", "BizLink (Kunshan) Co.,Ltd" },
    { "9CEC61", "Huawei Device Co., Ltd." },
    { "9CEDFA", "EVUlution AG" },
    { "9CEFD5", "Panda Wireless, Inc." },
    { "9CF029", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "9CF155", "Nokia" },
    { "9CF1D4", "Roku, Inc" },
    { "9CF387", "Apple, Inc." },
    { "9CF3AC", "Apple, Inc." },
    { "9CF48E", "Apple, Inc." },
    { "9CF531", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "9CF55F", "Harman/Becker Automotive Systems GmbH" },
    { "9CF61A", "Carrier Fire & Security" },
    { "9CF67D", "Ricardo Prague, s.r.o." },
    { "9CF6DD", "IEEE Registration Authority" },
    { "9CF86B", "AgiTech Distribution Limited - Linki" },
    { "9CF8B8", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "9CF8DB", "shenzhen eyunmei technology co,.ltd" },
    { "9CF938", "AREVA NP GmbH" },
    { "9CFA3C", "Daeyoung Electronics" },
    { "9CFA76", "Apple, Inc." },
    { "9CFB77", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "9CFBD5", "vivo Mobile Communication Co., Ltd." },
    { "9CFBF1", "MESOMATIC GmbH & Co.KG" },
    { "9CFC01", "Apple, Inc." },
    { "9CFC28", "Apple, Inc." },
    { "9CFCD1", "Aetheris Technology (Shanghai) Co., Ltd." },
    { "9CFCE8", "Intel Corporate" },
    { "9CFEA1", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "9CFFBE", "OTSL Inc." },
    { "9CFFC2", "AVI Systems GmbH" },
    { "A0004C", "HISENSE VISUAL TECHNOLOGY CO.,LTD" },
    { "A0024A", "IEEE Registration Authority" },
    { "A002A5", "Intel Corporate" },
    { "A002DC", "Amazon Technologies Inc." },
    { "A00363", "Robert Bosch Healthcare GmbH" },
    { "A0043E", "Parker Hannifin Manufacturing Germany GmbH & Co. KG" },
    { "A00460", "NETGEAR" },
    { "A00627", "NEXPA System" },
    { "A00798", "Samsung Electronics Co.,Ltd" },
    { "A007B6", "Advanced Technical Support, Inc." },
    { "A0086F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A0092E", "zte corporation" },
    { "A0094C", "CenturyLink" },
    { "A009ED", "Avaya Inc" },
    { "A00A9A", "Huawei Device Co., Ltd." },
    { "A00ABF", "Wieson Technologies Co., Ltd." },
    { "A00BBA", "SAMSUNG ELECTRO MECHANICS CO., LTD." },
    { "A00CA1", "SKTB SKiT" },
    { "A00CE2", "Shenzhen Shokz Co., Ltd." },
    { "A00E98", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A00F37", "Cisco Systems, Inc" },
    { "A01077", "zte corporation" },
    { "A01081", "Samsung Electronics Co.,Ltd" },
    { "A01290", "Avaya Inc" },
    { "A012DB", "TABUCHI ELECTRIC CO.,LTD" },
    { "A0133B", "HiTi Digital, Inc." },
    { "A013CB", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "A0143D", "PARROT SA" },
    { "A0165C", "Triteka LTD" },
    { "A017F1", "Allwinner Technology Co., Ltd" },
    { "A01828", "Apple, Inc." },
    { "A01842", "Comtrend Corporation" },
    { "A01859", "Shenzhen Yidashi Electronics Co Ltd" },
    { "A01917", "Bertel S.p.a." },
    { "A019B2", "IEEE Registration Authority" },
    { "A01AE3", "Edgecore Americas Networking Corporation" },
    { "A01B29", "Sagemcom Broadband SAS" },
    { "A01B9E", "Samsung Electronics Co.,Ltd" },
    { "A01BD6", "Nautitech Mining Systems Pty. Ltd." },
    { "A01C05", "NIMAX TELECOM CO.,LTD." },
    { "A01C87", "UNION MAN TECHNOLOGY CO.,LTD" },
    { "A01C8D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A01D48", "Hewlett Packard" },
    { "A01E0B", "MINIX Technology Limited" },
    { "A020A6", "Espressif Inc." },
    { "A0218B", "ACE Antenna Co., ltd" },
    { "A02195", "Samsung Electronics Co.,Ltd" },
    { "A021AA", "China Mobile Group Device Co.,Ltd." },
    { "A021B7", "NETGEAR" },
    { "A0224E", "IEEE Registration Authority" },
    { "A02252", "Astra Wireless Technology FZ-LLC" },
    { "A022DE", "vivo Mobile Communication Co., Ltd." },
    { "A0231B", "TeleComp R&D Corp." },
    { "A0239F", "Cisco Systems, Inc" },
    { "A02442", "Shenzhenshi Xinzhongxin Technology Co.Ltd" },
    { "A024F9", "Chengdu InnovaTest Technology Co., Ltd" },
    { "A025D7", "Hewlett Packard Enterprise" },
    { "A027B6", "Samsung Electronics Co.,Ltd" },
    { "A02833", "IEEE Registration Authority" },
    { "A02884", "Garmin International" },
    { "A028ED", "HMD Global Oy" },
    { "A02919", "Dell Inc." },
    { "A02942", "Intel Corporate" },
    { "A029BD", "Team Group Inc" },
    { "A02B44", "WaveGo Tech LLC" },
    { "A02BB8", "Hewlett Packard" },
    { "A02C36", "FN-LINK TECHNOLOGY LIMITED" },
    { "A02D13", "AirTies Wireless Networks" },
    { "A02DDB", "Sagemcom Broadband SAS" },
    { "A02EF3", "United Integrated Services Co., Led." },
    { "A03131", "Procenne Digital Security" },
    { "A031DB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A031EB", "Semikron Elektronik GmbH & Co. KG" },
    { "A03299", "Lenovo (Beijing) Co., Ltd." },
    { "A0334F", "Cisco Systems, Inc" },
    { "A0341B", "Adero Inc" },
    { "A03679", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A0369F", "Intel Corporate" },
    { "A036BC", "ASUSTek COMPUTER INC." },
    { "A036F0", "Comprehensive Power" },
    { "A036FA", "Ettus Research LLC" },
    { "A03768", "Shenzhen E-Life Intelligence Technology Co.,Ltd." },
    { "A038F8", "OURA Health Oy" },
    { "A03975", "Leo Bodnar Electronics Ltd" },
    { "A039EE", "Sagemcom Broadband SAS" },
    { "A039F7", "LG Electronics (Mobile Communications)" },
    { "A039F9", "Sagemcom Broadband SAS" },
    { "A03A75", "PSS Belgium N.V." },
    { "A03B01", "Kyung In Electronics" },
    { "A03B1B", "Inspire Tech" },
    { "A03BE3", "Apple, Inc." },
    { "A03C20", "Sagemcom Broadband SAS" },
    { "A03C31", "Shenzhen Belon Technology CO.,LTD" },
    { "A03D6E", "Cisco Systems, Inc" },
    { "A03D6F", "Cisco Systems, Inc" },
    { "A03E6B", "IEEE Registration Authority" },
    { "A04025", "Actioncable, Inc." },
    { "A04041", "SAMWONFA Co.,Ltd." },
    { "A0406F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A040A0", "NETGEAR" },
    { "A0412D", "Lansen Systems AB" },
    { "A04147", "Huawei Device Co., Ltd." },
    { "A0415E", "Opsens Solution Inc." },
    { "A041A7", "NL Ministry of Defense" },
    { "A0423F", "Tyan Computer Corp" },
    { "A04246", "IT Telecom Co., Ltd." },
    { "A042D1", "Huawei Device Co., Ltd." },
    { "A043B0", "Hangzhou BroadLink Technology Co.,Ltd" },
    { "A043DB", "Sitael S.p.A." },
    { "A0445C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A04466", "Intellics" },
    { "A044B7", "PAX Computer Technology(Shenzhen) Ltd." },
    { "A044F3", "RafaelMicro" },
    { "A0465A", "Motorola Mobility LLC, a Lenovo Company" },
    { "A0479B", "PROCITEC GmbH" },
    { "A047D7", "Best IT World (India) Pvt Ltd" },
    { "A0481C", "Hewlett Packard" },
    { "A04839", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A04A5E", "Microsoft Corporation" },
    { "A04C0C", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "A04C5B", "Shenzhen TINNO Mobile Technology Corp." },
    { "A04CC1", "Helixtech Corp." },
    { "A04E01", "CENTRAL ENGINEERING co.,ltd." },
    { "A04E04", "Nokia Corporation" },
    { "A04EA7", "Apple, Inc." },
    { "A04ECF", "Apple, Inc." },
    { "A04F52", "Intel Corporate" },
    { "A04F85", "LG Electronics (Mobile Communications)" },
    { "A04FD4", "ADB Broadband Italia" },
    { "A04FE4", "PAX Computer Technology(Shenzhen) Ltd." },
    { "A0510B", "Intel Corporate" },
    { "A051C6", "Avaya Inc" },
    { "A05272", "Apple, Inc." },
    { "A052AB", "AVM ELECTRONICS PTE LTD" },
    { "A05394", "Shenzhen zediel co., Ltd." },
    { "A0551F", "Sagemcom Broadband SAS" },
    { "A0554F", "Cisco Systems, Inc" },
    { "A055DE", "Commscope" },
    { "A0562C", "Samsung Electronics Co.,Ltd" },
    { "A056B2", "Harman/Becker Automotive Systems GmbH" },
    { "A056F3", "Apple, Inc." },
    { "A057E3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A0593A", "V.D.S. Video Display Systems srl" },
    { "A05950", "Intel Corporate" },
    { "A05AA4", "Grand Products Nevada, Inc." },
    { "A05B21", "ENVINET GmbH" },
    { "A05DC1", "TMCT Co., LTD." },
    { "A05DE7", "DIRECTV, Inc." },
    { "A05E6B", "MELPER Co., Ltd." },
    { "A06032", "Amcrest Technologies" },
    { "A06090", "Samsung Electronics Co.,Ltd" },
    { "A06260", "Private" },
    { "A062FB", "HISENSE VISUAL TECHNOLOGY CO.,LTD" },
    { "A06391", "NETGEAR" },
    { "A0648F", "ASKEY COMPUTER CORP" },
    { "A06518", "VNPT TECHNOLOGY" },
    { "A06610", "FUJITSU LIMITED" },
    { "A06636", "Intracom SA Telecom Solutions" },
    { "A06720", "China Dragon Technology Limited" },
    { "A067BE", "Sicon srl" },
    { "A067D6", "Nokia" },
    { "A0681C", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "A0687E", "Commscope" },
    { "A06974", "Honor Device Co., Ltd." },
    { "A06986", "Wellav Technologies Ltd" },
    { "A069D9", "New H3C Technologies Co., Ltd" },
    { "A06A00", "Verilink Corporation" },
    { "A06A44", "Vizio, Inc" },
    { "A06B4A", "TCT mobile ltd" },
    { "A06C65", "Texas Instruments" },
    { "A06CEC", "RIM" },
    { "A06D09", "Intelcan Technosystems Inc." },
    { "A06E50", "Nanotek Elektronik Sistemler Ltd. Sti." },
    { "A06FAA", "LG Innotek" },
    { "A07099", "Beijing Huacan Electronics Co., Ltd" },
    { "A070B7", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A071A9", "Nokia Corporation" },
    { "A0722C", "HUMAX Co., Ltd." },
    { "A072E4", "NJ SYSTEM CO.,LTD" },
    { "A07332", "Cashmaster International Limited" },
    { "A073FC", "Rancore Technologies Private Limited" },
    { "A07591", "Samsung Electronics Co.,Ltd" },
    { "A075EA", "BoxLock, Inc." },
    { "A0764E", "Espressif Inc." },
    { "A07751", "ASMedia Technology Inc." },
    { "A07771", "Vialis BV" },
    { "A0779E", "Chipsea Technologies (Shenzhen) Corp." },
    { "A07817", "Apple, Inc." },
    { "A0782D", "Apple, Inc." },
    { "A078BA", "Pantech Co., Ltd." },
    { "A07D9C", "Samsung Electronics Co.,Ltd" },
    { "A07E16", "EM Microelectronic" },
    { "A07F8A", "Sagemcom Broadband SAS" },
    { "A08069", "Intel Corporate" },
    { "A0821F", "Samsung Electronics Co.,Ltd" },
    { "A08222", "Qingdao Haier Technology Co.,Ltd" },
    { "A082AC", "Linear DMS Solutions Sdn. Bhd." },
    { "A082C7", "P.T.I Co.,LTD" },
    { "A083B4", "Velorum B.V" },
    { "A084CB", "SonicSensory,Inc." },
    { "A08527", "Intel Corporate" },
    { "A085E3", "Espressif Inc." },
    { "A085FC", "Microsoft Corporation" },
    { "A0861D", "Chengdu Fuhuaxin Technology co.,Ltd" },
    { "A086C6", "Xiaomi Communications Co Ltd" },
    { "A086EC", "SAEHAN HITEC Co., Ltd" },
    { "A0885E", "Anhui Xiangyao New Energy Technology Co., Ltd." },
    { "A08869", "Intel Corporate" },
    { "A0889D", "Huawei Device Co., Ltd." },
    { "A088B4", "Intel Corporate" },
    { "A088C2", "Mellanox Technologies, Inc." },
    { "A08966", "CIG SHANGHAI CO LTD" },
    { "A089E4", "Skyworth Digital Technology(Shenzhen) Co.,Ltd" },
    { "A08A06", "ASKEY COMPUTER CORP" },
    { "A08A87", "HuiZhou KaiYue Electronic Co.,Ltd" },
    { "A08C15", "Gerhard D. Wempe KG" },
    { "A08C9B", "Xtreme Technologies Corp" },
    { "A08CF2", "YINUOLINK CO.,LTD" },
    { "A08CF8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A08CFD", "Hewlett Packard" },
    { "A08D16", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A08E24", "eero inc." },
    { "A08E78", "Sagemcom Broadband SAS" },
    { "A090DE", "VEEDIMS,LLC" },
    { "A09169", "LG Electronics (Mobile Communications)" },
    { "A091A2", "OnePlus Electronics (Shenzhen) Co., Ltd." },
    { "A091C8", "zte corporation" },
    { "A091CA", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "A09208", "Tuya Smart Inc." },
    { "A09347", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "A09351", "Cisco Systems, Inc" },
    { "A0941A", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "A0946A", "Shenzhen XGTEC Technology Co,.Ltd." },
    { "A0950C", "China Mobile IOT Company Limited" },
    { "A0957F", "SERNET (SUZHOU) TECHNOLOGIES CORPORATION" },
    { "A09805", "OpenVox Communication Co Ltd" },
    { "A09857", "Shenzhen ELINK Technology Co., Ltd." },
    { "A098ED", "Shandong Intelligent Optical Communication Development Co., Ltd." },
    { "A09921", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "A0999B", "Apple, Inc." },
    { "A09A52", "Shenzhen MoreSense Technology Co., Ltd. " },
    { "A09A5A", "Time Domain" },
    { "A09A8E", "Apple, Inc." },
    { "A09B12", "China Mobile IOT Company Limited" },
    { "A09B17", "Taicang T&W Electronics" },
    { "A09BBD", "Total Aviation Solutions Pty Ltd" },
    { "A09D22", "Apple, Inc." },
    { "A09D86", "Alcatel-Lucent Shanghai Bell Co., Ltd" },
    { "A09D91", "SoundBridge" },
    { "A09DC1", "China Dragon Technology Limited" },
    { "A09E1A", "Polar Electro Oy" },
    { "A09F10", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "A09F7A", "D-Link Middle East FZCO" },
    { "A0A001", "Hewlett Packard Enterprise" },
    { "A0A0DC", "Huawei Device Co., Ltd." },
    { "A0A130", "DLI Taiwan Branch office" },
    { "A0A23C", "GPMS" },
    { "A0A309", "Apple, Inc." },
    { "A0A33B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A0A3B3", "Espressif Inc." },
    { "A0A3B8", "WISCLOUD" },
    { "A0A3E2", "Actiontec Electronics, Inc" },
    { "A0A3F0", "D-Link International" },
    { "A0A47F", "Cisco Systems, Inc" },
    { "A0A4C5", "Intel Corporate" },
    { "A0A65C", "Supercomputing Systems AG" },
    { "A0A763", "Polytron Vertrieb GmbH" },
    { "A0A8CD", "Intel Corporate" },
    { "A0AAFD", "EraThink Technologies Corp." },
    { "A0AB1B", "D-Link International" },
    { "A0AB51", "WEIFANG GOERTEK ELECTRONICS CO.,LTD" },
    { "A0AC69", "Samsung Electronics Co.,Ltd" },
    { "A0AC78", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "A0AD62", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A0AD9F", "ASUSTek COMPUTER INC." },
    { "A0ADA1", "JMR Electronics, Inc" },
    { "A0AF12", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A0AFBD", "Intel Corporate" },
    { "A0B045", "Halong Mining" },
    { "A0B086", "Hirschmann Automation and Control GmbH" },
    { "A0B0BD", "Samsung Electronics Co.,Ltd" },
    { "A0B100", "ShenZhen Cando Electronics Co.,Ltd" },
    { "A0B339", "Intel Corporate" },
    { "A0B3CC", "Hewlett Packard" },
    { "A0B40F", "Apple, Inc." },
    { "A0B437", "GD Mission Systems" },
    { "A0B439", "Cisco Systems, Inc" },
    { "A0B4A5", "Samsung Electronics Co.,Ltd" },
    { "A0B4BF", "InfiNet LLC" },
    { "A0B53C", "Technicolor Delivery Technologies Belgium NV" },
    { "A0B549", "Arcadyan Corporation" },
    { "A0B5DA", "HongKong THTF Co., Ltd" },
    { "A0B662", "Acutvista Innovation Co., Ltd." },
    { "A0B765", "Espressif Inc." },
    { "A0B8F8", "Amgen U.S.A. Inc." },
    { "A0B9ED", "Skytap" },
    { "A0BAB8", "Pixon Imaging" },
    { "A0BB3E", "IEEE Registration Authority" },
    { "A0BC6F", "Cisco Systems, Inc" },
    { "A0BD1D", "Zhejiang Dahua Technology Co., Ltd." },
    { "A0BD71", "QUALCOMM Incorporated" },
    { "A0BDCD", "SKY UK LIMITED" },
    { "A0BF50", "S.C. ADD-PRODUCTION S.R.L." },
    { "A0BFA5", "CORESYS" },
    { "A0C016", "Sichuan Changhong Network Technologies Co., Ltd." },
    { "A0C1C5", "Apple, Inc." },
    { "A0C20D", "Huawei Device Co., Ltd." },
    { "A0C2DE", "Costar Video Systems" },
    { "A0C3DE", "Triton Electronic Systems Ltd." },
    { "A0C4A5", "SYGN HOUSE INC." },
    { "A0C562", "Commscope" },
    { "A0C589", "Intel Corporate" },
    { "A0C5F2", "IEEE Registration Authority" },
    { "A0C6EC", "ShenZhen ANYK Technology Co.,LTD" },
    { "A0C7D2", "Cisco Systems, Inc" },
    { "A0C98B", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "A0C9A0", "Murata Manufacturing Co., Ltd." },
    { "A0CAA5", "INTELLIGENCE TECHNOLOGY OF CEC CO., LTD" },
    { "A0CBFD", "Samsung Electronics Co.,Ltd" },
    { "A0CC2B", "Murata Manufacturing Co., Ltd." },
    { "A0CDF3", "Murata Manufacturing Co., Ltd." },
    { "A0CEC8", "CE LINK LIMITED" },
    { "A0CF5B", "Cisco Systems, Inc" },
    { "A0CFF5", "zte corporation" },
    { "A0D05B", "Samsung Electronics Co.,Ltd" },
    { "A0D0DC", "Amazon Technologies Inc." },
    { "A0D12A", "AXPRO Technology Inc." },
    { "A0D1B3", "Apple, Inc." },
    { "A0D2B1", "Amazon Technologies Inc." },
    { "A0D365", "Intel Corporate" },
    { "A0D37A", "Intel Corporate" },
    { "A0D385", "AUMA Riester GmbH & Co. KG" },
    { "A0D3C1", "Hewlett Packard" },
    { "A0D42D", "G.Tech Technology Ltd." },
    { "A0D635", "WBS Technology" },
    { "A0D722", "Samsung Electronics Co.,Ltd" },
    { "A0D795", "Apple, Inc." },
    { "A0D7A0", "Huawei Device Co., Ltd." },
    { "A0D7F3", "Samsung Electronics Co.,Ltd" },
    { "A0D807", "Huawei Device Co., Ltd." },
    { "A0D83D", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "A0D86F", "ARGO AI, LLC" },
    { "A0D91A", "Texas Instruments" },
    { "A0DA92", "Nanjing Glarun Atten Technology Co. Ltd." },
    { "A0DC04", "Becker-Antriebe GmbH" },
    { "A0DD6C", "Espressif Inc." },
    { "A0DD97", "PolarLink Technologies, Ltd" },
    { "A0DDE5", "SHARP Corporation" },
    { "A0DE05", "JSC Irbis-T" },
    { "A0DE0F", "Huawei Device Co., Ltd." },
    { "A0DF15", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A0E025", "Provision-ISR" },
    { "A0E06D", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "A0E0AF", "Cisco Systems, Inc" },
    { "A0E201", "AVTrace Ltd.(China)" },
    { "A0E25A", "Amicus SK, s.r.o." },
    { "A0E295", "DAT System Co.,Ltd" },
    { "A0E453", "Sony Corporation" },
    { "A0E4CB", "Zyxel Communications Corporation" },
    { "A0E534", "Stratec Biomedical AG" },
    { "A0E5E9", "enimai Inc" },
    { "A0E617", "MATIS" },
    { "A0E6F8", "Texas Instruments" },
    { "A0E70B", "Intel Corporate" },
    { "A0E7AE", "Commscope" },
    { "A0E9DB", "Ningbo FreeWings Technologies Co.,Ltd" },
    { "A0EB76", "AirCUVE Inc." },
    { "A0EC80", "zte corporation" },
    { "A0ECF9", "Cisco Systems, Inc" },
    { "A0ED6D", "Ubee Interactive Co., Limited" },
    { "A0EDCD", "Apple, Inc." },
    { "A0EDFB", "Quectel Wireless Solutions Co.,Ltd." },
    { "A0EE1A", "Apple, Inc." },
    { "A0EEEE", "CIG SHANGHAI CO LTD" },
    { "A0EF84", "Seine Image Int'l Co., Ltd" },
    { "A0F217", "GE Medical System(China) Co., Ltd. " },
    { "A0F3C1", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "A0F3E4", "Alcatel-Lucent IPD" },
    { "A0F419", "Nokia Corporation" },
    { "A0F450", "HTC Corporation" },
    { "A0F459", "FN-LINK TECHNOLOGY LIMITED" },
    { "A0F479", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A0F509", "IEI Integration Corp." },
    { "A0F6FD", "Texas Instruments" },
    { "A0F849", "Cisco Systems, Inc" },
    { "A0F895", "Shenzhen TINNO Mobile Technology Corp." },
    { "A0F9B7", "Ademco Smart Homes Technology(Tianjin)Co.,Ltd." },
    { "A0F9E0", "VIVATEL COMPANY LIMITED" },
    { "A0FA9C", "WEIFANG GOERTEK ELECTRONICS CO.,LTD" },
    { "A0FAC8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A0FB68", "Miba Battery Systems GmbH" },
    { "A0FB83", "Honor Device Co., Ltd." },
    { "A0FBC5", "Apple, Inc." },
    { "A0FC6E", "Telegrafia a.s." },
    { "A0FE61", "Vivint Wireless Inc. " },
    { "A0FE91", "AVAT Automation GmbH" },
    { "A0FF0C", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "A0FF22", "SHENZHEN APICAL TECHNOLOGY CO., LTD" },
    { "A0FF70", "Vantiva USA LLC" },
    { "A4004E", "Cisco Systems, Inc" },
    { "A400E2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A40130", "ABIsystems Co., LTD" },
    { "A401DE", "SERCOMM PHILIPPINES INC" },
    { "A402B7", "Amazon Technologies Inc." },
    { "A402B9", "Intel Corporate" },
    { "A40450", "nFore Technology Inc." },
    { "A4056E", "Tiinlab Corporation" },
    { "A4059E", "STA Infinity LLP" },
    { "A405D6", "Commscope" },
    { "A406E9", "Texas Instruments" },
    { "A407B6", "Samsung Electronics Co.,Ltd" },
    { "A40801", "Amazon Technologies Inc." },
    { "A408EA", "Murata Manufacturing Co., Ltd." },
    { "A408F5", "Sagemcom Broadband SAS" },
    { "A40987", "Apple, Inc." },
    { "A409B3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A409CB", "Alfred Kaercher GmbH &amp; Co KG" },
    { "A40B78", "FAST PHOTONICS HK CO., LIMITED" },
    { "A40BED", "Carry Technology Co.,Ltd" },
    { "A40C66", "Shenzhen Colorful Yugong Technology and Development Co., Ltd." },
    { "A40CC3", "Cisco Systems, Inc" },
    { "A40DBC", "Xiamen Intretech Inc." },
    { "A40E2B", "Facebook Inc" },
    { "A40E75", "Hewlett Packard Enterprise" },
    { "A40F98", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "A410B6", "Cisco Systems, Inc" },
    { "A41115", "Robert Bosch Engineering and Business Solutions pvt. Ltd." },
    { "A41162", "Arlo Technology" },
    { "A41163", "IEEE Registration Authority" },
    { "A41194", "Lenovo" },
    { "A411BB", "Cisco Systems, Inc" },
    { "A41232", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "A41242", "NEC Platforms, Ltd." },
    { "A4134E", "Luxul " },
    { "A41437", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "A41566", "WEIFANG GOERTEK ELECTRONICS CO.,LTD" },
    { "A41588", "Commscope" },
    { "A416C0", "Apple, Inc." },
    { "A416E7", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A41731", "Hon Hai Precision Ind. Co.,Ltd." },
    { "A41752", "Hifocus Electronics India Private Limited" },
    { "A4178B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A41791", "Shenzhen Decnta Technology Co.,LTD." },
    { "A41875", "Cisco Systems, Inc" },
    { "A41894", "Bosch Security Systems B.V." },
    { "A41908", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "A41A3A", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "A41A6E", "zte corporation" },
    { "A41B34", "China Mobile Group Device Co.,Ltd." },
    { "A41BC0", "Fastec Imaging Corporation" },
    { "A41CB4", "DFI Inc" },
    { "A41EE1", "Taicang T&W Electronics" },
    { "A41F72", "Dell Inc." },
    { "A4218A", "Nortel Networks" },
    { "A42249", "Sagemcom Broadband SAS" },
    { "A42305", "Open Networking Laboratory" },
    { "A424B3", "FlatFrog Laboratories AB" },
    { "A424DD", "Cambrionix Ltd" },
    { "A4251B", "Avaya Inc" },
    { "A42618", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "A42655", "LTI Motion (Shanghai) Co., Ltd." },
    { "A427A5", "Palo Alto Networks" },
    { "A428B7", "Yangtze Memory Technologies Co., Ltd." },
    { "A42902", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "A42940", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "A42983", "Boeing Defence Australia" },
    { "A42985", "Sichuan AI-Link Technology Co., Ltd." },
    { "A429B7", "bluesky" },
    { "A42A71", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "A42A95", "D-Link International" },
    { "A42B8C", "NETGEAR" },
    { "A42BB0", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "A42C08", "Masterwork Automodules" },
    { "A4307A", "Samsung Electronics Co.,Ltd" },
    { "A43111", "ZIV" },
    { "A43135", "Apple, Inc." },
    { "A433D1", "Fibrlink Communications Co.,Ltd." },
    { "A433D7", "MitraStar Technology Corp." },
    { "A43412", "Thales Alenia Space" },
    { "A434D9", "Intel Corporate" },
    { "A434F1", "Texas Instruments" },
    { "A43523", "Guangdong Donyan Network Technologies Co.,Ltd." },
    { "A4352D", "TRIZ Networks corp." },
    { "A436C7", "LG Innotek" },
    { "A4373E", "Huawei Device Co., Ltd." },
    { "A43831", "RF elements s.r.o." },
    { "A438CC", "Nintendo Co.,Ltd" },
    { "A438FC", "Plastic Logic" },
    { "A439B3", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "A439B6", "SHENZHEN PEIZHE MICROELECTRONICS CO .LTD" },
    { "A43A69", "Vers Inc" },
    { "A43B0E", "Huawei Device Co., Ltd." },
    { "A43BFA", "IEEE Registration Authority" },
    { "A43CD7", "NTX Electronics YangZhou co.,LTD" },
    { "A43D78", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "A43E51", "ANOV FRANCE" },
    { "A43EA0", "iComm HK LIMITED" },
    { "A43F51", "Shenzhen Benew Technology Co.,Ltd." },
    { "A43F68", "Arista Network, Inc." },
    { "A44027", "zte corporation" },
    { "A4403D", "Shenzhen Baseus Technology Co., Ltd." },
    { "A4423B", "Intel Corporate" },
    { "A44343", "Huawei Device Co., Ltd." },
    { "A44380", "Huawei Device Co., Ltd." },
    { "A4438C", "Commscope" },
    { "A444D1", "Wingtech Group (HongKong）Limited" },
    { "A44519", "Xiaomi Communications Co Ltd" },
    { "A445CD", "IoT Diagnostics" },
    { "A4466B", "EOC Technology" },
    { "A446B4", "Huawei Device Co., Ltd." },
    { "A446FA", "AmTRAN Video Corporation" },
    { "A44AD3", "ST Electronics(Shanghai) Co.,Ltd" },
    { "A44B15", "Sun Cupid Technology (HK) LTD" },
    { "A44BD5", "Xiaomi Communications Co Ltd" },
    { "A44BD9", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "A44C11", "Cisco Systems, Inc" },
    { "A44C62", "Hangzhou Microimage Software Co., Ltd" },
    { "A44CC8", "Dell Inc." },
    { "A44E2D", "Adaptive Wireless Solutions, LLC" },
    { "A44E31", "Intel Corporate" },
    { "A44F29", "IEEE Registration Authority" },
    { "A45006", "SHENZHEN HUACHUANG SHIDAI TECHNOLOGYCO.,LTD" },
    { "A45046", "Xiaomi Communications Co Ltd" },
    { "A45055", "BUSWARE.DE" },
    { "A45129", "XAG" },
    { "A4515E", "Juniper Networks" },
    { "A4516F", "Microsoft Mobile Oy" },
    { "A4526F", "ADB Broadband Italia" },
    { "A4530E", "Cisco Systems, Inc" },
    { "A45385", "WEIFANG GOERTEK ELECTRONICS CO.,LTD" },
    { "A453EE", "IEEE Registration Authority" },
    { "A45590", "Xiaomi Communications Co Ltd" },
    { "A45602", "fenglian Technology Co.,Ltd." },
    { "A4561B", "MCOT Corporation" },
    { "A45630", "Cisco Systems, Inc" },
    { "A456CC", "Vantiva USA LLC" },
    { "A457A0", "SAMJIN Co., Ltd." },
    { "A45802", "SHIN-IL TECH" },
    { "A4580F", "IEEE Registration Authority" },
    { "A459D3", "IEEE Registration Authority" },
    { "A45A1C", "smart-electronic GmbH" },
    { "A45C25", "Texas Instruments" },
    { "A45C27", "Nintendo Co., Ltd." },
    { "A45D36", "Hewlett Packard" },
    { "A45D5E", "Wilk Elektronik S.A." },
    { "A45DA1", "ADB Broadband Italia" },
    { "A45E5A", "ACTIVIO Inc." },
    { "A45E60", "Apple, Inc." },
    { "A45F9B", "Nexell" },
    { "A45FB9", "DreamBig Semiconductor, Inc." },
    { "A46011", "Verifone, Inc." },
    { "A46032", "MRV Communications (Networks) LTD" },
    { "A46191", "NamJunSa" },
    { "A462DF", "DS Global. Co., LTD" },
    { "A463A1", "Inventus Power Eletronica do Brasil LTDA" },
    { "A46706", "Apple, Inc." },
    { "A468BC", "Oakley Inc." },
    { "A46B1F", "eero inc." },
    { "A46BB6", "Intel Corporate" },
    { "A46C24", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A46C2A", "Cisco Systems, Inc" },
    { "A46CC1", "LTi REEnergy GmbH" },
    { "A46CF1", "Samsung Electronics Co.,Ltd" },
    { "A46D33", "Phyplus Technology (Shanghai) Co., Ltd" },
    { "A46DA4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A46DD4", "Silicon Laboratories" },
    { "A46E79", "DFT System Co.Ltd" },
    { "A46EA7", "DX ANTENNA CO.,LTD." },
    { "A470D6", "Motorola Mobility LLC, a Lenovo Company" },
    { "A47174", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A473AB", "Extreme Networks Headquarters" },
    { "A475B9", "Samsung Electronics Co.,Ltd" },
    { "A47733", "Google, Inc." },
    { "A47758", "Ningbo Freewings Technologies Co.,Ltd" },
    { "A47760", "Nokia Corporation" },
    { "A477F3", "Apple, Inc." },
    { "A47806", "Cisco Systems, Inc" },
    { "A47886", "Avaya Inc" },
    { "A47952", "Huawei Device Co., Ltd." },
    { "A479E4", "KLINFO Corp" },
    { "A47A72", "Arista Networks" },
    { "A47AA4", "Commscope" },
    { "A47ACF", "VIBICOM COMMUNICATIONS INC." },
    { "A47B1A", "Huawei Device Co., Ltd." },
    { "A47B2C", "Nokia" },
    { "A47B52", "JoulWatt Technology Co., Ltd" },
    { "A47B85", "ULTIMEDIA Co Ltd," },
    { "A47B9D", "Espressif Inc." },
    { "A47C14", "ChargeStorm AB" },
    { "A47C1F", "Cobham plc" },
    { "A47CC9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A47D78", "Edgecore Americas Networking Corporation" },
    { "A47D9F", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "A47E36", "EM Microelectronic" },
    { "A47E39", "zte corporation" },
    { "A47EFA", "Withings" },
    { "A47F1B", "Juniper Networks" },
    { "A4817A", "CIG SHANGHAI CO LTD" },
    { "A481EE", "Nokia Corporation" },
    { "A48269", "Datrium, Inc." },
    { "A483E7", "Apple, Inc." },
    { "A48431", "Samsung Electronics Co.,Ltd" },
    { "A4856B", "Q Electronics Ltd" },
    { "A486AE", "Quectel Wireless Solutions Co.,Ltd." },
    { "A486DB", "Guangdong Juan Intelligent Technology Joint Stock Co., Ltd." },
    { "A48873", "Cisco Systems, Inc" },
    { "A4895B", "ARK INFOSOLUTIONS PVT LTD" },
    { "A4897E", "Guangzhou Yuhong Technology Co.,Ltd." },
    { "A48CC0", "JLG Industries, Inc." },
    { "A48CDB", "Lenovo" },
    { "A48D3B", "Vizio, Inc" },
    { "A48E0A", "DeLaval International AB" },
    { "A49005", "CHINA GREATWALL COMPUTER SHENZHEN CO.,LTD" },
    { "A490CE", "vivo Mobile Communication Co., Ltd." },
    { "A491B1", "Technicolor Delivery Technologies Belgium NV" },
    { "A492CB", "Nokia" },
    { "A4933F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A49340", "Beijing Supvan Information Technology Co.,Ltd." },
    { "A4934C", "Cisco Systems, Inc" },
    { "A49426", "Elgama-Elektronika Ltd." },
    { "A494DC", "Infinite Clouds" },
    { "A49700", "Cisco Systems, Inc" },
    { "A49733", "ASKEY COMPUTER CORP" },
    { "A4975C", "VTech Telecommunications Ltd." },
    { "A4978A", "LEAR" },
    { "A49791", "Extreme Networks Headquarters" },
    { "A497B1", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "A497BB", "Hitachi Industrial Equipment Systems Co.,Ltd" },
    { "A49813", "Commscope" },
    { "A49947", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A49981", "FuJian Elite Power Tech CO.,LTD." },
    { "A499A8", "eero inc." },
    { "A49A58", "Samsung Electronics Co.,Ltd" },
    { "A49B13", "Digital Check" },
    { "A49B4F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A49BCD", "Cisco Systems, Inc" },
    { "A49BF5", "Hybridserver Tec GmbH" },
    { "A49D49", "Ketra, Inc." },
    { "A49DDD", "Samsung Electronics Co.,Ltd" },
    { "A49E69", "Silicon Laboratories" },
    { "A49EDB", "AutoCrib, Inc." },
    { "A49F85", "Lyve Minds, Inc" },
    { "A49F89", "Shanghai Rui Rui Communication Technology Co.Ltd." },
    { "A49FE7", "Samsung Electronics Co.,Ltd" },
    { "A4A179", "Nanjing dianyan electric power automation co. LTD" },
    { "A4A1C2", "Ericsson AB" },
    { "A4A1E4", "Innotube, Inc." },
    { "A4A24A", "Cisco SPVTG" },
    { "A4A404", "Bubendorff SAS" },
    { "A4A459", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "A4A46B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A4A490", "Samsung Electronics Co.,Ltd" },
    { "A4A4D3", "Bluebank Communication Technology Co.Ltd" },
    { "A4A528", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "A4A584", "Cisco Systems, Inc" },
    { "A4A6A9", "Private" },
    { "A4A80F", "Shenzhen Coship Electronics Co., Ltd." },
    { "A4A930", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "A4AAFE", "Huawei Device Co., Ltd." },
    { "A4AC0F", "Huawei Device Co., Ltd." },
    { "A4AD00", "Ragsdale Technology" },
    { "A4AD9E", "NEXAIOT" },
    { "A4ADB8", "Vitec Group, Camera Dynamics Ltd" },
    { "A4AE11", "Hon Hai Precision Industry Co., Ltd." },
    { "A4AE12", "Hon Hai Precision Industry Co., Ltd." },
    { "A4AE9A", "Maestro Wireless Solutions ltd." },
    { "A4B039", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "A4B0F5", "Texas Instruments" },
    { "A4B121", "Arantia 2010 S.L." },
    { "A4B197", "Apple, Inc." },
    { "A4B1C1", "Intel Corporate" },
    { "A4B1E9", "Technicolor Delivery Technologies Belgium NV" },
    { "A4B1EE", "H. ZANDER GmbH & Co. KG" },
    { "A4B239", "Cisco Systems, Inc" },
    { "A4B2A7", "Adaxys Solutions AG" },
    { "A4B36A", "JSC SDO Chromatec" },
    { "A4B439", "Cisco Systems, Inc" },
    { "A4B52E", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "A4B61E", "Huawei Device Co., Ltd." },
    { "A4B805", "Apple, Inc." },
    { "A4B818", "PENTA Gesellschaft für elektronische Industriedatenverarbeitung mbH" },
    { "A4B980", "Parking BOXX Inc." },
    { "A4BA70", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "A4BA76", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A4BADB", "Dell Inc." },
    { "A4BB6D", "Dell Inc." },
    { "A4BBAF", "Lime Instruments" },
    { "A4BD7E", "HMD Global Oy" },
    { "A4BDC4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A4BE2B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A4BE61", "EutroVision System, Inc." },
    { "A4BF01", "Intel Corporate" },
    { "A4C0C7", "ShenZhen Hitom Communication Technology Co..LTD" },
    { "A4C0E1", "Nintendo Co., Ltd." },
    { "A4C138", "Telink Semiconductor (Taipei) Co. Ltd." },
    { "A4C1E8", "Nintendo Co.,Ltd" },
    { "A4C23E", "Huizhou Speed Wireless Technology Co.,Ltd" },
    { "A4C2AB", "Hangzhou LEAD-IT Information & Technology Co.,Ltd" },
    { "A4C337", "Apple, Inc." },
    { "A4C34E", "Texas Instruments" },
    { "A4C361", "Apple, Inc." },
    { "A4C3BE", "Xiaomi Communications Co Ltd" },
    { "A4C3F0", "Intel Corporate" },
    { "A4C40D", "WAC Lighting" },
    { "A4C494", "Intel Corporate" },
    { "A4C54E", "Huawei Device Co., Ltd." },
    { "A4C64F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A4C69A", "Samsung Electronics Co.,Ltd" },
    { "A4C6F0", "Apple, Inc." },
    { "A4C74B", "Huawei Device Co., Ltd." },
    { "A4C788", "Xiaomi Communications Co Ltd" },
    { "A4C7DE", "Cambridge Industries(Group) Co.,Ltd." },
    { "A4C7F6", "Extreme Networks Headquarters" },
    { "A4C939", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "A4CAA0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A4CC32", "Inficomm Co., Ltd" },
    { "A4CCB3", "Xiaomi Communications Co Ltd" },
    { "A4CCB9", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "A4CD23", "Shenzhenshi Xinzhongxin Co., Ltd" },
    { "A4CEDA", "Arcadyan Corporation" },
    { "A4CF12", "Espressif Inc." },
    { "A4CF99", "Apple, Inc." },
    { "A4CFD2", "Ubee Interactive Co., Limited" },
    { "A4D094", "VIVAVIS AG" },
    { "A4D18C", "Apple, Inc." },
    { "A4D18F", "Shenzhen Skyee Optical Fiber Communication Technology Ltd. " },
    { "A4D1D1", "ECOtality North America" },
    { "A4D1D2", "Apple, Inc." },
    { "A4D23E", "Apple, Inc." },
    { "A4D3B5", "GLITEL Stropkov, s.r.o." },
    { "A4D4B2", "Shenzhen MeiG Smart Technology Co.,Ltd" },
    { "A4D501", "Yucca Technology Company Limited." },
    { "A4D530", "Avaya LLC" },
    { "A4D578", "Texas Instruments" },
    { "A4D5C2", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "A4D73C", "Seiko Epson Corporation" },
    { "A4D795", "Wingtech Mobile Communications Co.,Ltd" },
    { "A4D856", "Gimbal, Inc" },
    { "A4D8CA", "HONG KONG WATER WORLD TECHNOLOGY CO. LIMITED" },
    { "A4D931", "Apple, Inc." },
    { "A4D990", "Samsung Electronics Co.,Ltd" },
    { "A4D9A4", "neXus ID Solutions AB" },
    { "A4DA22", "IEEE Registration Authority" },
    { "A4DA32", "Texas Instruments" },
    { "A4DA3F", "Bionics Corp." },
    { "A4DAD4", "Yamato Denki Co.,Ltd." },
    { "A4DB2E", "Kingspan Environmental Ltd" },
    { "A4DB30", "Liteon Technology Corporation" },
    { "A4DB4C", "RAI Institute" },
    { "A4DCBE", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A4DCD5", "Cisco Systems, Inc" },
    { "A4DD58", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A4DE26", "Sumitomo Electric Industries, Ltd" },
    { "A4DE50", "Total Walther GmbH" },
    { "A4DEC9", "QLove Mobile Intelligence Information Technology (W.H.) Co. Ltd." },
    { "A4E0E6", "FILIZOLA S.A. PESAGEM E AUTOMACAO" },
    { "A4E11A", "Juniper Networks" },
    { "A4E287", "Xiaomi Communications Co Ltd" },
    { "A4E31B", "Nokia" },
    { "A4E32E", "Silicon & Software Systems Ltd." },
    { "A4E391", "DENY FONTAINE" },
    { "A4E4B8", "BlackBerry RTS" },
    { "A4E57C", "Espressif Inc." },
    { "A4E597", "Gessler GmbH" },
    { "A4E615", "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD" },
    { "A4E6B1", "Shanghai Joindata Technology Co.,Ltd." },
    { "A4E731", "Nokia Corporation" },
    { "A4E7E4", "Connex GmbH" },
    { "A4E88D", "FN-LINK TECHNOLOGY Ltd." },
    { "A4E8A3", "Fujian Newland Auto-ID Tech Co.,Ltd." },
    { "A4E975", "Apple, Inc." },
    { "A4E991", "SISTEMAS AUDIOVISUALES ITELSIS S.L." },
    { "A4E9A3", "Honest Technology Co., Ltd" },
    { "A4EA4F", "VusionGroup" },
    { "A4EA8E", "Extreme Networks Headquarters" },
    { "A4EBD3", "Samsung Electronics Co.,Ltd" },
    { "A4ED43", "IEEE Registration Authority" },
    { "A4ED4E", "Commscope" },
    { "A4EE57", "Seiko Epson Corporation" },
    { "A4EF15", "AltoBeam (China) Inc." },
    { "A4EF52", "Telewave Co., Ltd." },
    { "A4F05E", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "A4F1E8", "Apple, Inc." },
    { "A4F33B", "zte corporation" },
    { "A4F3C1", "Open Source Robotics Foundation, Inc." },
    { "A4F3E7", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "A4F465", "ITEL MOBILE LIMITED" },
    { "A4F4C2", "VNPT TECHNOLOGY" },
    { "A4F522", "CHOFU SEISAKUSHO CO.,LTD" },
    { "A4F6E6", "Apple, Inc." },
    { "A4F6E8", "Apple, Inc." },
    { "A4F7D0", "LAN Accessories Co., Ltd." },
    { "A4F841", "Apple, Inc." },
    { "A4F921", "Apple, Inc." },
    { "A4F933", "Intel Corporate" },
    { "A4F9E4", "AirVine Scientific, Inc." },
    { "A4FA76", "New H3C Technologies Co., Ltd" },
    { "A4FB8D", "Hangzhou Dunchong Technology Co.Ltd" },
    { "A4FC14", "Apple, Inc." },
    { "A4FC77", "Mega Well Limited" },
    { "A4FCA1", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "A4FCCE", "Security Expert Ltd." },
    { "A4FF95", "Nokia" },
    { "A4FF9F", "Xiaomi Communications Co Ltd" },
    { "A800E3", "Starkey Labs Inc." },
    { "A8016D", "Aiwa Corporation" },
    { "A80180", "IMAGO Technologies GmbH" },
    { "A802DB", "zte corporation" },
    { "A8032A", "Espressif Inc." },
    { "A80556", "vivo Mobile Communication Co., Ltd." },
    { "A80577", "Netlist, Inc." },
    { "A80600", "Samsung Electronics Co.,Ltd" },
    { "A808CF", "Vestel Elektronik San ve Tic. A.S." },
    { "A809B1", "Huawei Device Co., Ltd." },
    { "A80B6B", "Chipsea Technologies (Shenzhen) Corp." },
    { "A80BFB", "Ruckus Wireless" },
    { "A80C03", "Florawise" },
    { "A80C0D", "Cisco Systems, Inc" },
    { "A80C63", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A80CCA", "Shenzhen Sundray Technologies company Limited" },
    { "A80DE1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A81087", "Texas Instruments" },
    { "A811FC", "Commscope" },
    { "A81306", "vivo Mobile Communication Co., Ltd." },
    { "A8130B", "eero inc." },
    { "A81374", "Panasonic Corporation AVC Networks Company" },
    { "A8154D", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "A81559", "Breathometer, Inc." },
    { "A815D6", "Shenzhen Meione Technology CO., LTD" },
    { "A8169D", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "A816B2", "LG Electronics (Mobile Communications)" },
    { "A816D0", "Samsung Electronics Co.,Ltd" },
    { "A81710", "Bouffalo Lab (Nanjing) Co., Ltd." },
    { "A81758", "Elektronik System i Umeå AB" },
    { "A81AF1", "Apple, Inc." },
    { "A81B18", "XTS CORP" },
    { "A81B5A", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "A81B5D", "Foxtel Management Pty Ltd" },
    { "A81B6A", "Texas Instruments" },
    { "A81D16", "AzureWave Technology Inc." },
    { "A81E84", "Quanta Computer Inc." },
    { "A81F79", "Yingling Innovations Pte. Ltd." },
    { "A81FAF", "KRYPTON POLSKA" },
    { "A82066", "Apple, Inc." },
    { "A82316", "Nokia" },
    { "A823FE", "LG Electronics" },
    { "A82450", "Beijing Huadianzhongxin Tech.Co.,Ltd" },
    { "A824B8", "Nokia" },
    { "A824EB", "ZAO NPO Introtest" },
    { "A825EB", "Cambridge Industries(Group) Co.,Ltd." },
    { "A826D9", "HTC Corporation" },
    { "A827C8", "Edgecore Americas Networking Corporation" },
    { "A82948", "TP-Link Systems Inc." },
    { "A8294C", "Precision Optical Transceivers, Inc." },
    { "A82AD6", "Arthrex Inc." },
    { "A82BB5", "Edgecore Networks Corporation" },
    { "A82BB9", "Samsung Electronics Co.,Ltd" },
    { "A82BCD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A82BD5", "Xiaomi Communications Co Ltd" },
    { "A82BD6", "Shina System Co., Ltd" },
    { "A82BDD", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "A82C3E", "Shenzhen Cultraview Digital Technology Co., Ltd" },
    { "A82C89", "Apple, Inc." },
    { "A8301C", "Qingdao Intelligent&Precise Electronics Co.,Ltd." },
    { "A830AD", "WEIFANG GOERTEK ELECTRONICS CO.,LTD" },
    { "A830BC", "Samsung Electronics Co.,Ltd" },
    { "A83162", "Hangzhou Huacheng Network Technology Co.,Ltd" },
    { "A8329A", "Digicom Futuristic Technologies Ltd." },
    { "A8346A", "Samsung Electronics Co.,Ltd" },
    { "A83512", "Huawei Device Co., Ltd." },
    { "A8367A", "frogblue TECHNOLOGY GmbH" },
    { "A83759", "Huawei Device Co., Ltd." },
    { "A83944", "Actiontec Electronics, Inc" },
    { "A83A48", "Ubiqcom India Pvt Ltd" },
    { "A83A79", "Mist Systems, Inc." },
    { "A83B5C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A83B76", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "A83CA5", "Dell Inc." },
    { "A83CCB", "ROSSMA" },
    { "A83E0E", "HMD Global Oy" },
    { "A83ED3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A83FA1", "IEEE Registration Authority" },
    { "A8400B", "Visteon Corporation" },
    { "A84025", "Oxide Computer Company" },
    { "A84041", "Dragino Technology Co., Limited" },
    { "A8407D", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "A840F8", "HUMAX NETWORKS" },
    { "A84122", "China Mobile (Hangzhou) Information Technology Co.,Ltd." },
    { "A841F4", "AzureWave Technology Inc." },
    { "A842A1", "TP-Link Systems Inc" },
    { "A842A7", "Jiangsu Huitong Group Co.,Ltd." },
    { "A842E3", "Espressif Inc." },
    { "A84397", "Innogrit Corporation" },
    { "A843A4", "China Dragon Technology Limited" },
    { "A84481", "Nokia Corporation" },
    { "A844AA", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "A845CD", "Siselectron Technology LTD." },
    { "A845E9", "Firich Enterprises CO., LTD." },
    { "A84616", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A84674", "Espressif Inc." },
    { "A8469D", "Cisco Meraki" },
    { "A8474A", "Hon Hai Precision Ind. Co.,Ltd." },
    { "A848FA", "Espressif Inc." },
    { "A8494D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A849A5", "Lisantech Co., Ltd." },
    { "A84A28", "Apple, Inc." },
    { "A84A63", "TPV Display Technology(Xiamen) Co.,Ltd." },
    { "A84B4D", "Samsung Electronics Co.,Ltd" },
    { "A84D4A", "Audiowise Technology Inc." },
    { "A84E3F", "Hitron Technologies. Inc" },
    { "A84FA4", "CHINA DRAGON TECHNOLOGY LIMITED" },
    { "A84FB1", "Cisco Systems, Inc" },
    { "A85008", "Felion Technologies Company Limited" },
    { "A85081", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A8515B", "Samsung Electronics Co.,Ltd" },
    { "A851AB", "Apple, Inc." },
    { "A852D4", "Hewlett Packard Enterprise" },
    { "A8537D", "Mist Systems, Inc." },
    { "A854A2", "Heimgard Technologies AS" },
    { "A854B2", "Wistron Neweb Corporation" },
    { "A8556A", "3S System Technology Inc." },
    { "A8574E", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "A85840", "Cambridge Industries(Group) Co.,Ltd." },
    { "A8584E", "PK VEGA" },
    { "A8587C", "Shoogee GmbH & Co. KG" },
    { "A8595F", "Intel Corporate" },
    { "A85AE0", "Huawei Device Co., Ltd." },
    { "A85AF3", "Shanghai Siflower Communication Technology Co., Ltd" },
    { "A85B36", "IEEE Registration Authority" },
    { "A85B6C", "Robert Bosch Gmbh, CM-CI2" },
    { "A85B78", "Apple, Inc." },
    { "A85BB0", "Shenzhen Dehoo Technology Co.,Ltd" },
    { "A85BB7", "Apple, Inc." },
    { "A85BD1", "INVENTEC CORPORATION" },
    { "A85BF3", "Audivo GmbH" },
    { "A85BF7", "Hewlett Packard Enterprise" },
    { "A85C03", "JiangSu Fulian Communication Technology Co., Ltd" },
    { "A85C2C", "Apple, Inc." },
    { "A85E45", "ASUSTek COMPUTER INC." },
    { "A85EE4", "12Sided Technology, LLC" },
    { "A85EF2", "TECNO MOBILE LIMITED" },
    { "A860B6", "Apple, Inc." },
    { "A8610A", "ARDUINO AG" },
    { "A861AA", "Cloudview Limited" },
    { "A861DF", "China Mobile Group Device Co.,Ltd." },
    { "A862A2", "JIWUMEDIA CO., LTD." },
    { "A86308", "AVUTEC" },
    { "A8637D", "D-Link International" },
    { "A863DF", "DISPLAIRE CORPORATION" },
    { "A863F2", "Texas Instruments" },
    { "A86405", "nimbus 9, Inc" },
    { "A864F1", "Intel Corporate" },
    { "A865B2", "DONGGUAN YISHANG ELECTRONIC TECHNOLOGY CO., LIMITED" },
    { "A8667F", "Apple, Inc." },
    { "A8671E", "RATP" },
    { "A8698C", "Oracle Corporation " },
    { "A86A6F", "RIM" },
    { "A86A86", "Xiaomi Communications Co Ltd" },
    { "A86ABB", "Sagemcom Broadband SAS" },
    { "A86AC1", "HanbitEDS Co., Ltd." },
    { "A86B7C", "SHENZHEN FENGLIAN TECHNOLOGY CO., LTD." },
    { "A86BAD", "Hon Hai Precision Ind. Co.,Ltd." },
    { "A86D04", "Siemens AG" },
    { "A86D5F", "Raisecom Technology CO., LTD" },
    { "A86DAA", "Intel Corporate" },
    { "A86E4E", "Huawei Device Co., Ltd." },
    { "A86E84", "TP-Link Systems Inc" },
    { "A86F36", "vivo Mobile Communication Co., Ltd." },
    { "A8705D", "Commscope" },
    { "A870A5", "UniComm Inc." },
    { "A87116", "Earda Technologies co Ltd" },
    { "A8727E", "WISDRI (wuhan) Automation Company Limited" },
    { "A87285", "IDT, INC." },
    { "A8741D", "Phoenix Contact GmbH & Co. KG" },
    { "A87484", "zte corporation" },
    { "A875D6", "FreeTek International Co., Ltd." },
    { "A875E2", "Aventura Technologies, Inc." },
    { "A87650", "Samsung Electronics Co.,Ltd" },
    { "A8776F", "Zonoff" },
    { "A877E5", "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD" },
    { "A87971", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A8798D", "Samsung Electronics Co.,Ltd" },
    { "A87B39", "Nokia Corporation" },
    { "A87C01", "Samsung Electronics Co.,Ltd" },
    { "A87C45", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A87CF8", "Apple, Inc." },
    { "A87D12", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A87E33", "Nokia Danmark A/S" },
    { "A87EEA", "Intel Corporate" },
    { "A88038", "ShenZhen MovingComm Technology Co., Limited" },
    { "A88055", "Tuya Smart Inc." },
    { "A8817E", "Apple, Inc." },
    { "A88195", "Samsung Electronics Co.,Ltd" },
    { "A881F1", "BMEYE B.V." },
    { "A881FE", "Luxul Tech Co., Ltd" },
    { "A88200", "Hisense Electric Co.,Ltd" },
    { "A8827F", "CIBN Oriental Network(Beijing) CO.,Ltd" },
    { "A885D7", "Sangfor Technologies Inc." },
    { "A886DD", "Apple, Inc." },
    { "A88792", "Broadband Antenna Tracking Systems" },
    { "A887B3", "Samsung Electronics Co.,Ltd" },
    { "A887ED", "ARC Wireless LLC" },
    { "A88808", "Apple, Inc." },
    { "A8881F", "SERVERCOM (INDIA) PRIVATE LIMITED" },
    { "A888CE", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "A88940", "Huawei Device Co., Ltd." },
    { "A88B28", "SHENZHEN DIYANG SMART TECHNOLOGY CO.,LTD." },
    { "A88C3E", "Microsoft Corporation" },
    { "A88CEE", "MicroMade Galka i Drozdz sp.j." },
    { "A88D7B", "SunDroid Global limited." },
    { "A88E24", "Apple, Inc." },
    { "A88F99", "Arista Networks" },
    { "A88FD9", "Apple, Inc." },
    { "A89008", "Beijing Yuecheng Technology Co. Ltd." },
    { "A89042", "Beijing Wanwei Intelligent Technology Co., Ltd." },
    { "A8913D", "Apple, Inc." },
    { "A89162", "Sophos Ltd" },
    { "A8922C", "LG Electronics (Mobile Communications)" },
    { "A8934A", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "A89352", "SHANGHAI ZHONGMI COMMUNICATION TECHNOLOGY CO.,LTD" },
    { "A893E6", "JIANGXI JINGGANGSHAN CKING COMMUNICATION TECHNOLOGY CO.,LTD" },
    { "A895B0", "Aker Subsea Ltd" },
    { "A89609", "FN-LINK TECHNOLOGY Ltd." },
    { "A89675", "Motorola Mobility LLC, a Lenovo Company" },
    { "A8968A", "Apple, Inc." },
    { "A897CD", "Commscope" },
    { "A897DC", "IBM" },
    { "A89892", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "A898C6", "Shinbo Co., Ltd." },
    { "A8995C", "aizo ag" },
    { "A89969", "Dell Inc." },
    { "A899AD", "Chaoyue Technology Co., Ltd." },
    { "A899DC", "i-TOP DESING TECHNOLOGY CO.,LTD" },
    { "A89A8C", "zte corporation" },
    { "A89A93", "Sagemcom Broadband SAS" },
    { "A89AD7", "Nokia" },
    { "A89B10", "inMotion Ltd." },
    { "A89C6C", "Ubiquiti Inc" },
    { "A89C78", "Apple, Inc." },
    { "A89CA4", "Furrion Limited" },
    { "A89CED", "Xiaomi Communications Co Ltd" },
    { "A89D21", "Cisco Systems, Inc" },
    { "A89DD2", "Shanghai DareGlobal Technologies Co.,Ltd" },
    { "A89FBA", "Samsung Electronics Co.,Ltd" },
    { "A89FEC", "Commscope" },
    { "A8A089", "Tactical Communications" },
    { "A8A097", "ScioTeq bvba" },
    { "A8A159", "ASRock Incorporation" },
    { "A8A198", "TCT mobile ltd" },
    { "A8A237", "Arcadyan Corporation" },
    { "A8A5E2", "MSF-Vathauer Antriebstechnik GmbH & Co KG " },
    { "A8A648", "Qingdao Hisense Communications Co.,Ltd." },
    { "A8A668", "zte corporation" },
    { "A8A795", "Hon Hai Precision Ind. Co.,Ltd." },
    { "A8A913", "GN Hearing A/S" },
    { "A8AA7C", "Huawei Device Co., Ltd." },
    { "A8ABB5", "Apple, Inc." },
    { "A8AD3D", "Alcatel-Lucent Shanghai Bell Co., Ltd" },
    { "A8B028", "CubePilot Pty Ltd" },
    { "A8B088", "eero inc." },
    { "A8B0AE", "BizLink Special Cables Germany GmbH" },
    { "A8B0D1", "EFUN Display Technology (Shenzhen) Co., Ltd." },
    { "A8B13B", "HP Inc." },
    { "A8B1D4", "Cisco Systems, Inc" },
    { "A8B271", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A8B2DA", "FUJITSU LIMITED" },
    { "A8B456", "Cisco Systems, Inc" },
    { "A8B483", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "A8B57C", "Roku, Inc" },
    { "A8B58E", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "A8B86E", "LG Electronics (Mobile Communications)" },
    { "A8B8E0", "Changwang Technology inc." },
    { "A8B9B3", "ESSYS" },
    { "A8BA25", "Hewlett Packard Enterprise" },
    { "A8BA69", "Samsung Electronics Co.,Ltd" },
    { "A8BB50", "WiZ IoT Company Limited" },
    { "A8BB56", "Apple, Inc." },
    { "A8BBCF", "Apple, Inc." },
    { "A8BC9C", "Cloud Light Technology Limited" },
    { "A8BD1A", "Honey Bee (Hong Kong) Limited" },
    { "A8BD27", "Hewlett Packard Enterprise " },
    { "A8BD3A", "UNION MAN TECHNOLOGY CO.,LTD" },
    { "A8BE27", "Apple, Inc." },
    { "A8BF3C", "HDV Phoelectron Technology Limited" },
    { "A8C092", "Huawei Device Co., Ltd." },
    { "A8C0EA", "Pepwave Limited" },
    { "A8C222", "TM-Research Inc." },
    { "A8C246", "Gemtek Technology Co., Ltd." },
    { "A8C252", "Huawei Device Co., Ltd." },
    { "A8C266", "HUMAX Co., Ltd." },
    { "A8C407", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A8C56F", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "A8C647", "Extreme Networks Headquarters" },
    { "A8C83A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A8C87F", "Roqos, Inc." },
    { "A8C98A", "New H3C Technologies Co., Ltd" },
    { "A8CA77", "Amazon Technologies Inc." },
    { "A8CA7B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A8CAB9", "SAMSUNG ELECTRO MECHANICS CO., LTD." },
    { "A8CB95", "EAST BEST CO., LTD." },
    { "A8CC6F", "HMD Global Oy" },
    { "A8CCC5", "Saab AB (publ)" },
    { "A8CE90", "CVC" },
    { "A8CFE0", "GDN Enterprises Private Limited" },
    { "A8D081", "Huawei Device Co., Ltd." },
    { "A8D0E3", "Systech Electronics Ltd" },
    { "A8D0E5", "Juniper Networks" },
    { "A8D236", "Lightware Visual Engineering" },
    { "A8D3C8", "Wachendorff Automation GmbH & CO.KG" },
    { "A8D3F7", "Arcadyan Technology Corporation" },
    { "A8D409", "USA 111 Inc" },
    { "A8D498", "Avira Operations GmbH & Co. KG" },
    { "A8D4E0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A8D579", "Beijing Chushang Science and Technology Co.,Ltd" },
    { "A8D828", "Ascensia Diabetes Care" },
    { "A8D861", "ITEL MOBILE LIMITED" },
    { "A8D88A", "Wyconn" },
    { "A8DA01", "Shenzhen NUOLIJIA Digital Technology Co.,Ltd" },
    { "A8DA0C", "SERVERCOM (INDIA) PRIVATE LIMITED" },
    { "A8DB03", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "A8DC5A", "Digital Watchdog" },
    { "A8DE68", "Beijing Wide Technology Co.,Ltd" },
    { "A8E018", "Nokia Corporation" },
    { "A8E207", "GOIP Global Services Pvt. Ltd." },
    { "A8E291", "AzureWave Technology Inc." },
    { "A8E2C1", "Texas Instruments" },
    { "A8E2C3", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "A8E3EE", "Sony Interactive Entertainment Inc." },
    { "A8E539", "Nurivoice Co., Ltd" },
    { "A8E544", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A8E552", "JUWEL Aquarium AG & Co. KG" },
    { "A8E5EC", "Nokia" },
    { "A8E621", "Amazon Technologies Inc." },
    { "A8E6E8", "Tonly Technology Co. Ltd " },
    { "A8E705", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "A8E77D", "Texas Instruments" },
    { "A8E81E", "ATW TECHNOLOGY, INC." },
    { "A8E824", "INIM ELECTRONICS S.R.L." },
    { "A8E978", "Huawei Device Co., Ltd." },
    { "A8EA71", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "A8EAE4", "Weiser" },
    { "A8ED71", "Analogue Enterprises Limited" },
    { "A8EE67", "Samsung Electronics Co.,Ltd" },
    { "A8EE6D", "Fine Point-High Export" },
    { "A8EEC6", "Muuselabs NV/SA" },
    { "A8EF26", "Tritonwave" },
    { "A8EF5F", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "A8F038", "SHEN ZHEN SHI JIN HUA TAI ELECTRONICS CO.,LTD" },
    { "A8F059", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A8F1B2", "Allwinner Technology Co., Ltd" },
    { "A8F266", "Huawei Device Co., Ltd." },
    { "A8F274", "Samsung Electronics Co.,Ltd" },
    { "A8F470", "Fujian Newland Communication Science Technologies Co.,Ltd." },
    { "A8F5AC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "A8F5DD", "Commscope" },
    { "A8F5E1", "Shenzhen Shokz Co., Ltd." },
    { "A8F766", "ITE Tech Inc" },
    { "A8F7D9", "Mist Systems, Inc." },
    { "A8F7E0", "PLANET Technology Corporation" },
    { "A8F8C9", "NXP Semiconductor (Tianjin) LTD." },
    { "A8F94B", "Eltex Enterprise Ltd." },
    { "A8FAD8", "Apple, Inc." },
    { "A8FB40", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "A8FB70", "WiseSec L.t.d" },
    { "A8FCB7", "Consolidated Resource Imaging" },
    { "A8FE9D", "Apple, Inc." },
    { "A8FECE", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "A8FFBA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "AA0000", "DIGITAL EQUIPMENT CORPORATION" },
    { "AA0001", "DIGITAL EQUIPMENT CORPORATION" },
    { "AA0002", "DIGITAL EQUIPMENT CORPORATION" },
    { "AA0003", "DIGITAL EQUIPMENT CORPORATION" },
    { "AA0004", "DIGITAL EQUIPMENT CORPORATION" },
    { "AC007A", "Apple, Inc." },
    { "AC00D0", "zte corporation" },
    { "AC00F9", "BizLink Technology (S.E.A) Sdn. Bhd." },
    { "AC0142", "Uriel Technologies SIA" },
    { "AC017A", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "AC02CA", "HI Solutions, Inc." },
    { "AC02CF", "RW Tecnologia Industria e Comercio Ltda" },
    { "AC02EF", "Comsis" },
    { "AC040B", "Peloton Interactive, Inc" },
    { "AC0425", "ball-b GmbH Co KG" },
    { "AC0481", "Jiangsu Huaxing Electronics Co., Ltd." },
    { "AC04AA", "GoPro" },
    { "AC0613", "Senselogix Ltd" },
    { "AC0650", "Shanghai Baosight Software Co., Ltd" },
    { "AC06C7", "ServerNet S.r.l." },
    { "AC075F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "AC0775", "Apple, Inc." },
    { "AC0A61", "Labor S.r.L." },
    { "AC0BFB", "Espressif Inc." },
    { "AC0D1B", "LG Electronics (Mobile Communications)" },
    { "AC0DFE", "Ekon GmbH - myGEKKO" },
    { "AC1007", "Arcadyan Corporation" },
    { "AC1065", "KT Micro, Inc." },
    { "AC11D3", "Suzhou HOTEK Video Technology Co. Ltd" },
    { "AC1203", "Intel Corporate" },
    { "AC122F", "Fantasia Trading LLC" },
    { "AC128E", "Shanghai Baud Data Communication Co.,Ltd." },
    { "AC139C", "Adtran Inc" },
    { "AC1461", "ATAW Co., Ltd." },
    { "AC14D2", "wi-daq, inc." },
    { "AC1518", "Espressif Inc." },
    { "AC1585", "silergy corp" },
    { "AC15A2", "TP-Link Systems Inc" },
    { "AC15F4", "Apple, Inc." },
    { "AC1615", "Apple, Inc." },
    { "AC162D", "Hewlett Packard" },
    { "AC16DE", "Intel Corporate" },
    { "AC1702", "Fibar Group sp. z o.o." },
    { "AC1754", "tiko Energy Solutions AG" },
    { "AC1794", "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD" },
    { "AC17C8", "Cisco Meraki" },
    { "AC1826", "Seiko Epson Corporation" },
    { "AC198E", "Intel Corporate" },
    { "AC199F", "SUNGROW POWER SUPPLY CO.,LTD." },
    { "AC1A3D", "Dell Inc." },
    { "AC1C26", "Hangzhou Ezviz Software Co.,Ltd." },
    { "AC1D06", "Apple, Inc." },
    { "AC1DDF", "IEEE Registration Authority" },
    { "AC1E92", "Samsung Electronics Co.,Ltd" },
    { "AC1E9E", "Xiaomi Communications Co Ltd" },
    { "AC1EA9", "Intelbras" },
    { "AC1ED0", "Temic Automotive Philippines Inc." },
    { "AC1F09", "shenzhen RAKwireless technology Co.,Ltd" },
    { "AC1F0F", "Texas Instruments" },
    { "AC1F6B", "Super Micro Computer, Inc." },
    { "AC1F74", "Apple, Inc." },
    { "AC1FD7", "Real Vision Technology Co.,Ltd." },
    { "AC202E", "Hitron Technologies. Inc" },
    { "AC203E", "Wuhan Tianyu Information Industry Co., Ltd." },
    { "AC20AA", "DMATEK Co., Ltd." },
    { "AC2205", "Compal Broadband Networks, Inc." },
    { "AC220B", "ASUSTek COMPUTER INC." },
    { "AC2316", "Mist Systems, Inc." },
    { "AC2334", "Infinix mobility limited" },
    { "AC233F", "Shenzhen Minew Technologies Co., Ltd." },
    { "AC2929", "Infinix mobility limited" },
    { "AC293A", "Apple, Inc." },
    { "AC2A0C", "CSR ZHUZHOU INSTITUTE CO.,LTD." },
    { "AC2AA1", "Cisco Systems, Inc" },
    { "AC2B6E", "Intel Corporate" },
    { "AC2DA3", "TXTR GmbH" },
    { "AC2DA9", "TECNO MOBILE LIMITED" },
    { "AC2FA8", "Humannix Co.,Ltd." },
    { "AC3184", "Huawei Device Co., Ltd." },
    { "AC319D", "Shenzhen TG-NET Botone Technology Co.,Ltd." },
    { "AC330B", "Japan Computer Vision Corp." },
    { "AC3328", "Huawei Device Co., Ltd." },
    { "AC3351", "Ericsson AB" },
    { "AC34CB", "Shanhai GBCOM Communication Technology Co. Ltd" },
    { "AC35EE", "FN-LINK TECHNOLOGY LIMITED" },
    { "AC3613", "Samsung Electronics Co.,Ltd" },
    { "AC361B", "Hon Hai Precision Industry Co.,LTD" },
    { "AC3651", "Jiangsu Hengtong Terahertz Technology Co., Ltd." },
    { "AC3728", "Taicang T&W Electronics" },
    { "AC3743", "HTC Corporation" },
    { "AC37C9", "RAID Incorporated" },
    { "AC3870", "Lenovo Mobile Communication Technology Ltd." },
    { "AC3971", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "AC3A67", "Cisco Systems, Inc" },
    { "AC3A7A", "Roku, Inc." },
    { "AC3B77", "Sagemcom Broadband SAS" },
    { "AC3B96", "NXP Semiconductor (Tianjin) LTD." },
    { "AC3C0B", "Apple, Inc." },
    { "AC3C8E", "Flextronics Computing(Suzhou)Co.,Ltd." },
    { "AC3CB4", "Nilan A/S" },
    { "AC3D05", "Instorescreen Aisa" },
    { "AC3D75", "HANGZHOU ZHIWAY TECHNOLOGIES CO.,LTD." },
    { "AC3D94", "Arista Networks" },
    { "AC3DCB", "Intel Corporate" },
    { "AC3EB1", "Google, Inc." },
    { "AC3FA4", "TAIYO YUDEN CO.,LTD" },
    { "AC40EA", "C&T Solution Inc. " },
    { "AC4122", "Eclipse Electronic Systems Inc." },
    { "AC416A", "Amazon Technologies Inc." },
    { "AC4228", "Parta Networks" },
    { "AC4330", "Versa Networks" },
    { "AC44F2", "YAMAHA CORPORATION" },
    { "AC4500", "Apple, Inc." },
    { "AC45CA", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "AC45EF", "Intel Corporate" },
    { "AC471B", "Huawei Device Co., Ltd." },
    { "AC4723", "Genelec" },
    { "AC482D", "Ralinwi Nanjing Electronic Technology Co., Ltd." },
    { "AC49DB", "Apple, Inc." },
    { "AC4A56", "Cisco Systems, Inc" },
    { "AC4A67", "Cisco Systems, Inc" },
    { "AC4AFE", "Hisense Broadband Multimedia Technology Co.,Ltd." },
    { "AC4B1E", "Integri-Sys.Com LLC" },
    { "AC4BC8", "Juniper Networks" },
    { "AC4CA5", "Vantiva USA LLC" },
    { "AC4D16", "Texas Instruments" },
    { "AC4DD9", "Extreme Networks Headquarters" },
    { "AC4E2E", "Shenzhen JingHanDa Electronics Co.Ltd" },
    { "AC4E65", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "AC4E91", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "AC4FFC", "SVS-VISTEK GmbH" },
    { "AC5036", "Pi-Coral Inc" },
    { "AC5093", "Magna Electronics Europe GmbH & Co. OHG" },
    { "AC50DE", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "AC50EE", "Tianyi Telecom Terminals Company Limited" },
    { "AC512C", "Infinix mobility limited" },
    { "AC5135", "MPI TECH" },
    { "AC51AB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "AC51EE", "Adtran Inc" },
    { "AC5474", "China Mobile IOT Company Limited" },
    { "AC54EC", "IEEE P1823 Standards Working Group" },
    { "AC562C", "LAVA INTERNATIONAL(H.K) LIMITED" },
    { "AC567B", "Sunnovo International Limited" },
    { "AC5775", "HMD Global Oy" },
    { "AC583B", "Human Assembler, Inc." },
    { "AC587B", "JCT Healthcare" },
    { "AC5A14", "Samsung Electronics Co.,Ltd" },
    { "AC5AEE", "China Mobile Group Device Co.,Ltd." },
    { "AC5AF0", "LG Electronics" },
    { "AC5AFC", "Intel Corporate" },
    { "AC5C2C", "Apple, Inc." },
    { "AC5C80", "Telink Micro LLC" },
    { "AC5D10", "Pace Americas" },
    { "AC5D5C", "FN-LINK TECHNOLOGY LIMITED" },
    { "AC5E14", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "AC5E8C", "Utillink" },
    { "AC5F3E", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "AC5FEA", "OnePlus Technology (Shenzhen) Co., Ltd" },
    { "AC606F", "Nokia Shanghai Bell Co., Ltd." },
    { "AC6089", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "AC60B6", "Ericsson AB" },
    { "AC6123", "Drivven, Inc." },
    { "AC6175", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "AC61B9", "WAMA Technology Limited" },
    { "AC61EA", "Apple, Inc." },
    { "AC620D", "Jabil Circuit(Wuxi) Co.,Ltd" },
    { "AC63BE", "Amazon Technologies Inc." },
    { "AC6417", "Siemens AG" },
    { "AC6462", "zte corporation" },
    { "AC6490", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "AC64CF", "FN-LINK TECHNOLOGY LIMITED" },
    { "AC64DD", "IEEE Registration Authority" },
    { "AC6706", "Ruckus Wireless" },
    { "AC675D", "Intel Corporate" },
    { "AC676F", "Electrocompaniet A.S." },
    { "AC6784", "Google, Inc." },
    { "AC67B2", "Espressif Inc." },
    { "AC69CF", "Cisco Meraki" },
    { "AC6AA3", "Shenzhen Kertong Technology Co.,Ltd" },
    { "AC6B0F", "CADENCE DESIGN SYSTEMS INC" },
    { "AC6BAC", "Jenny Science AG" },
    { "AC6C90", "Samsung Electronics Co.,Ltd" },
    { "AC6E1A", "SHENZHEN GONGJIN ELECTRONICS CO.,LT" },
    { "AC6F4F", "Enspert Inc" },
    { "AC6FBB", "TATUNG Technology Inc." },
    { "AC6FD9", "Valueplus Inc." },
    { "AC710C", "China Mobile Group Device Co.,Ltd." },
    { "AC712E", "Fortinet, Inc." },
    { "AC7236", "Lexking Technology Co., Ltd." },
    { "AC7289", "Intel Corporate" },
    { "AC72DD", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "AC7352", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "AC7409", "Hangzhou H3C Technologies Co., Limited" },
    { "AC74B1", "Intel Corporate" },
    { "AC74C4", "Maytronics Ltd." },
    { "AC751D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "AC764C", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "AC7713", "Honeywell Safety Products (Shanghai) Co.,Ltd" },
    { "AC77B9", "Nanjing Yufei Intelligent Control Technology Co.,LTD" },
    { "AC78D1", "Juniper Networks" },
    { "AC7A42", "iConnectivity" },
    { "AC7A4D", "ALPSALPINE CO,.LTD" },
    { "AC7A56", "Cisco Systems, Inc" },
    { "AC7A94", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "AC7BA1", "Intel Corporate" },
    { "AC7E01", "Huawei Device Co., Ltd." },
    { "AC7E8A", "Cisco Systems, Inc" },
    { "AC7F3E", "Apple, Inc." },
    { "AC7F8D", "Extreme Networks Headquarters" },
    { "AC800A", "Sony Corporation" },
    { "AC80AE", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "AC80D6", "Hexatronic AB" },
    { "AC80FB", "Samsung Electronics Co.,Ltd" },
    { "AC8112", "Gemtek Technology Co., Ltd." },
    { "AC81B5", "Accton Technology Corporation" },
    { "AC81F3", "Nokia Corporation" },
    { "AC8226", "Qingdao Haier Technology Co.,Ltd" },
    { "AC8247", "Intel Corporate" },
    { "AC82F0", "Apple, Inc." },
    { "AC8317", "Shenzhen Furtunetel Communication Co., Ltd" },
    { "AC83E9", "Beijing Zile Technology Co., Ltd" },
    { "AC83F0", "Cobalt Digital Inc." },
    { "AC83F3", "AMPAK Technology, Inc." },
    { "AC84C6", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "AC84C9", "Sagemcom Broadband SAS" },
    { "AC84FA", "Zhejiang Weilai Jingling Artificial Intelligence Technology Co., Ltd." },
    { "AC853D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "AC8674", "Open Mesh, Inc." },
    { "AC867E", "Create New Technology (HK) Limited Company" },
    { "AC86A3", "Apple, Inc." },
    { "AC86D1", "IEEE Registration Authority" },
    { "AC87A3", "Apple, Inc." },
    { "AC8866", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "AC88FD", "Apple, Inc." },
    { "AC8995", "AzureWave Technology Inc." },
    { "AC89D2", "Ciena Corporation" },
    { "AC8ACD", "ROGER D.Wensker, G.Wensker sp.j." },
    { "AC8B6A", "China Mobile IOT Company Limited" },
    { "AC8B9C", "Primera Technology, Inc." },
    { "AC8BA9", "Ubiquiti Inc" },
    { "AC8C46", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "AC8D14", "Smartrove Inc" },
    { "AC8D34", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "AC8EBD", "Microsoft Corporation" },
    { "AC8FA9", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "AC8FF8", "Nokia" },
    { "AC9073", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "AC9085", "Apple, Inc." },
    { "AC915D", "Digital Control Technology Limited" },
    { "AC919B", "Wistron Neweb Corporation" },
    { "AC91A1", "Dell Inc." },
    { "AC9232", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "AC932F", "Nokia Corporation" },
    { "AC936A", "Huawei Device Co., Ltd." },
    { "AC93C4", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "AC9403", "Envision Peripherals Inc" },
    { "AC9572", "Jovision Technology Co., Ltd." },
    { "AC965B", "Lucid Motors" },
    { "AC9738", "Apple, Inc." },
    { "AC976C", "Greenliant" },
    { "AC9929", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "AC9A22", "NXP Semiconductors" },
    { "AC9A96", "Maxlinear, Inc" },
    { "AC9B0A", "Sony Corporation" },
    { "AC9B84", "Smak Tecnologia e Automacao" },
    { "AC9CE4", "Alcatel-Lucent Shanghai Bell Co., Ltd" },
    { "AC9E17", "ASUSTek COMPUTER INC." },
    { "AC9FC3", "Ring LLC" },
    { "ACA016", "Cisco Systems, Inc" },
    { "ACA09D", "Juniper Networks" },
    { "ACA213", "Shenzhen Bilian electronic CO.,LTD" },
    { "ACA22C", "Baycity Technologies Ltd" },
    { "ACA31E", "Hewlett Packard Enterprise" },
    { "ACA32F", "Solidigm Technology" },
    { "ACA430", "Peerless AV" },
    { "ACA46E", "SHENZHEN GONGJIN ELECTRONICS CO.,LT" },
    { "ACA613", "Aivres SYSTEMS INC" },
    { "ACA667", "Electronic Systems Protection, Inc." },
    { "ACA88E", "SHARP Corporation" },
    { "ACA919", "TrekStor GmbH" },
    { "ACA9A0", "Audioengine, Ltd." },
    { "ACAB2E", "Beijing LasNubes Technology Co., Ltd." },
    { "ACAB8D", "Lyngso Marine A/S" },
    { "ACABBF", "AthenTek Inc." },
    { "ACACE2", "CHANGHONG (HONGKONG) TRADING LIMITED" },
    { "ACAD4B", "zte corporation" },
    { "ACAE19", "Roku, Inc" },
    { "ACAFB9", "Samsung Electronics Co.,Ltd" },
    { "ACB181", "Belden Mooresville" },
    { "ACB1EE", "SHENZHEN FENDA TECHNOLOGY CO., LTD" },
    { "ACB313", "Commscope" },
    { "ACB3B5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "ACB480", "Dell Inc." },
    { "ACB566", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "ACB57D", "Liteon Technology Corporation" },
    { "ACB687", "Arcadyan Corporation" },
    { "ACB722", "Qingdao Haier Technology Co.,Ltd" },
    { "ACB74F", "METEL s.r.o." },
    { "ACB859", "Uniband Electronic Corp," },
    { "ACB92F", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "ACBAC0", "Intertech Services AG" },
    { "ACBB61", "YSTen Technology Co.,Ltd" },
    { "ACBC32", "Apple, Inc." },
    { "ACBC5B", "VIVIBIT INC" },
    { "ACBCB5", "Apple, Inc." },
    { "ACBCD9", "Cisco Systems, Inc" },
    { "ACBD0B", "Leimac Ltd." },
    { "ACBD70", "Huawei Device Co., Ltd." },
    { "ACBDF7", "Cisco Meraki" },
    { "ACBE75", "Ufine Technologies Co.,Ltd." },
    { "ACBEB6", "Visualedge Technology Co., Ltd." },
    { "ACBF71", "Bose Corporation" },
    { "ACC048", "OnePlus Technology (Shenzhen) Co., Ltd" },
    { "ACC1EE", "Xiaomi Communications Co Ltd" },
    { "ACC25D", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "ACC2EC", "CLT INT'L IND. CORP." },
    { "ACC33A", "Samsung Electronics Co.,Ltd" },
    { "ACC358", "Continental Automotive Czech Republic s.r.o." },
    { "ACC3E5", "Cisco Meraki" },
    { "ACC4A9", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "ACC4BD", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "ACC51B", "Zhuhai Pantum Electronics Co., Ltd." },
    { "ACC595", "Graphite Systems" },
    { "ACC5B4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "ACC662", "MitraStar Technology Corp." },
    { "ACC698", "Kohzu Precision Co., Ltd." },
    { "ACC73F", "VITSMO CO., LTD." },
    { "ACC906", "Apple, Inc." },
    { "ACC935", "Ness Corporation" },
    { "ACC9FF", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "ACCA0F", "INDISR COMMUNICATION SERVICES AND TECHNOLOGIES INDIA" },
    { "ACCA54", "Telldus Technologies AB" },
    { "ACCA8E", "ODA Technologies" },
    { "ACCAAB", "Virtual Electric Inc" },
    { "ACCABA", "Midokura Co., Ltd. " },
    { "ACCB09", "Hefcom Metering (Pty) Ltd" },
    { "ACCB36", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "ACCB51", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "ACCC8E", "Axis Communications AB" },
    { "ACCCFC", "Amazon Technologies Inc." },
    { "ACCE8F", "HWA YAO TECHNOLOGIES CO., LTD" },
    { "ACCE92", "New H3C Technologies Co., Ltd" },
    { "ACCF23", "Hi-flying electronics technology Co.,Ltd" },
    { "ACCF5C", "Apple, Inc." },
    { "ACCF7B", "INGRAM MICRO SERVICES" },
    { "ACCF85", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "ACD074", "Espressif Inc." },
    { "ACD180", "Crexendo Business Solutions, Inc." },
    { "ACD1B8", "Hon Hai Precision Ind. Co.,Ltd." },
    { "ACD31D", "Cisco Meraki" },
    { "ACD364", "ABB SPA, ABB SACE DIV." },
    { "ACD564", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "ACD618", "OnePlus Technology (Shenzhen) Co., Ltd" },
    { "ACD657", "Shaanxi GuoLian Digital TV Technology Co.,Ltd." },
    { "ACD75B", "Sagemcom Broadband SAS" },
    { "ACD829", "Bouffalo Lab (Nanjing) Co., Ltd." },
    { "ACD8A7", "BELLDESIGN Inc." },
    { "ACD929", "Quectel Wireless Solutions Co.,Ltd." },
    { "ACD9D6", "tci GmbH" },
    { "ACDB22", "Marquardt Schaltsysteme SCS" },
    { "ACDB48", "Commscope" },
    { "ACDBDA", "Shenzhen Geniatech Inc, Ltd" },
    { "ACDCCA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "ACDCE5", "Procter & Gamble Company" },
    { "ACDE48", "Private" },
    { "ACDF9F", "Arcadyan Corporation" },
    { "ACDFA1", "Apple, Inc." },
    { "ACE010", "Liteon Technology Corporation" },
    { "ACE069", "ISAAC Instruments" },
    { "ACE0D6", "koreabts" },
    { "ACE14F", "Autonomic Controls, Inc." },
    { "ACE215", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "ACE2D3", "Hewlett Packard" },
    { "ACE342", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "ACE348", "MadgeTech, Inc" },
    { "ACE403", "Shenzhen Visteng Technology CO.,LTD" },
    { "ACE42E", "SK hynix" },
    { "ACE4B5", "Apple, Inc." },
    { "ACE5F0", "Doppler Labs" },
    { "ACE64B", "Shenzhen Baojia Battery Technology Co., Ltd." },
    { "ACE77B", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "ACE87B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "ACE87E", "Bytemark Computer Consulting Ltd" },
    { "ACE97F", "IoT Tech Limited" },
    { "ACE9AA", "Hay Systems Ltd" },
    { "ACEA6A", "GENIX INFOCOMM CO., LTD." },
    { "ACEAEA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "ACEB51", "Universal Electronics, Inc." },
    { "ACEC80", "Commscope" },
    { "ACEC85", "eero inc." },
    { "ACED32", "Extreme Networks Headquarters" },
    { "ACED5C", "Intel Corporate" },
    { "ACEE3B", "6harmonics Inc" },
    { "ACEE64", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "ACEE70", "Fontem Ventures BV" },
    { "ACEE9E", "Samsung Electronics Co.,Ltd" },
    { "ACEF92", "IEEE Registration Authority" },
    { "ACF0B2", "Becker Electronics Taiwan Ltd." },
    { "ACF108", "LG Innotek" },
    { "ACF1DF", "D-Link International" },
    { "ACF23C", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "ACF2C5", "Cisco Systems, Inc" },
    { "ACF42C", "Earda Technologies co Ltd" },
    { "ACF466", "HP Inc." },
    { "ACF473", "iRobot Corporation" },
    { "ACF5E6", "Cisco Systems, Inc" },
    { "ACF6F7", "LG Electronics (Mobile Communications)" },
    { "ACF70D", "China Mobile Group Device Co.,Ltd." },
    { "ACF7F3", "Xiaomi Communications Co Ltd" },
    { "ACF85C", "Chengdu Higon Integrated Circuit Design Co,. Ltd." },
    { "ACF8CC", "Commscope" },
    { "ACF970", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "ACF97E", "ELESYS INC." },
    { "ACFAA5", "digitron" },
    { "ACFAE4", "Nintendo Co.,Ltd" },
    { "ACFBC2", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "ACFC82", "Shenzhen Sundray Technologies company Limited" },
    { "ACFCE3", "EM Microelectronic" },
    { "ACFD93", "WEIFANG GOERTEK ELECTRONICS CO.,LTD" },
    { "ACFDCE", "Intel Corporate" },
    { "ACFDEC", "Apple, Inc." },
    { "ACFE05", "ITEL MOBILE LIMITED" },
    { "ACFF6B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B00073", "Wistron Neweb Corporation" },
    { "B000B4", "Cisco Systems, Inc" },
    { "B00247", "AMPAK Technology, Inc." },
    { "B0027E", "MULLER SERVICES" },
    { "B00594", "Liteon Technology Corporation" },
    { "B00875", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B008BF", "Vital Connect, Inc." },
    { "B009D3", "Avizia" },
    { "B009DA", "Ring Solutions" },
    { "B00AD5", "zte corporation" },
    { "B00B22", "Huawei Device Co., Ltd." },
    { "B00C9D", "Quectel Wireless Solutions Co.,Ltd." },
    { "B00CD1", "Hewlett Packard" },
    { "B01041", "Hon Hai Precision Ind. Co.,Ltd." },
    { "B0104B", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "B010A0", "Texas Instruments" },
    { "B01203", "Dynamics Hong Kong Limited" },
    { "B01266", "Futaba-Kikaku" },
    { "B01408", "LIGHTSPEED INTERNATIONAL CO." },
    { "B014DF", "MitraStar Technology Corp." },
    { "B01656", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B01743", "EDISON GLOBAL CIRCUITS LLC" },
    { "B01831", "Apple, Inc." },
    { "B01886", "SmarDTV Corporation" },
    { "B01921", "TP-Link Systems Inc" },
    { "B019C6", "Apple, Inc." },
    { "B01B4B", "Invisible Fun Studio Limited" },
    { "B01B7C", "Ontrol A.S." },
    { "B01BD2", "Le Shi Zhi Xin Electronic Technology (Tianjin) Limited" },
    { "B01C0C", "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD" },
    { "B01C91", "Elim Co" },
    { "B01F29", "Helvetia INC." },
    { "B01F47", "Heights Telecom T ltd" },
    { "B01F81", "IEEE Registration Authority" },
    { "B01F8C", "Hewlett Packard Enterprise" },
    { "B0216F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B0227A", "HP Inc." },
    { "B02347", "Shenzhen Giant Microelectronics Company Limited" },
    { "B02491", "Huawei Device Co., Ltd." },
    { "B024F3", "Progeny Systems" },
    { "B025AA", "AIstone Global Limited" },
    { "B02628", "Broadcom Limited" },
    { "B02680", "Cisco Systems, Inc" },
    { "B027CF", "Extreme Networks Headquarters" },
    { "B0285B", "JUHUA Technology Inc." },
    { "B02A1F", "Wingtech Group (HongKong）Limited" },
    { "B02A43", "Google, Inc." },
    { "B02EE0", "Huawei Device Co., Ltd." },
    { "B03055", "China Mobile IOT Company Limited" },
    { "B030C8", "Teal Drones, Inc." },
    { "B03226", "Keheng Information Industry Co., Ltd." },
    { "B03366", "vivo Mobile Communication Co., Ltd." },
    { "B033A6", "Juniper Networks" },
    { "B03495", "Apple, Inc." },
    { "B034FB", "ShenZhen Microtest Automation Co.,Ltd" },
    { "B0350B", "MOBIWIRE MOBILES (NINGBO) CO.,LTD" },
    { "B0358D", "Nokia Corporation" },
    { "B0359F", "Intel Corporate" },
    { "B035B5", "Apple, Inc." },
    { "B03795", "LG Electronics" },
    { "B03829", "Siliconware Precision Industries Co., Ltd." },
    { "B03850", "Nanjing CAS-ZDC IOT SYSTEM CO.,LTD" },
    { "B0386C", "EFM Networks" },
    { "B03893", "Onda TLC Italia S.r.l." },
    { "B038E2", "Wanan Hongsheng Electronic Co.Ltd" },
    { "B03956", "NETGEAR" },
    { "B03ACE", "Huawei Device Co., Ltd." },
    { "B03B1B", "Kontrolnext Technology (Beijing) Ltd." },
    { "B03CDC", "Intel Corporate" },
    { "B03D96", "Vision Valley FZ LLC" },
    { "B03DC2", "Wasp artificial intelligence(Shenzhen) Co.,ltd" },
    { "B03E51", "SKY UK LIMITED" },
    { "B03EB0", "MICRODIA Ltd." },
    { "B03F64", "Apple, Inc." },
    { "B04089", "Senient Systems LTD" },
    { "B0411D", "ITTIM Technologies" },
    { "B0416F", "Shenzhen Maxtang Computer Co.,Ltd" },
    { "B0435D", "NuLEDs, Inc." },
    { "B04414", "New H3C Technologies Co., Ltd" },
    { "B0449C", "Assa Abloy AB - Yale" },
    { "B04502", "Huawei Device Co., Ltd." },
    { "B04515", "mira fitness,LLC." },
    { "B04519", "TCT mobile ltd" },
    { "B04530", "SKY UK LIMITED" },
    { "B04545", "YACOUB Automation GmbH" },
    { "B04692", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "B046FC", "MitraStar Technology Corp." },
    { "B0475E", "IEEE Registration Authority" },
    { "B047BF", "Samsung Electronics Co.,Ltd" },
    { "B047E9", "Intel Corporate" },
    { "B0481A", "Apple, Inc." },
    { "B0487A", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "B0495F", "OMRON HEALTHCARE Co., Ltd." },
    { "B04A39", "Beijing Roborock Technology Co., Ltd." },
    { "B04A6A", "Samsung Electronics Co.,Ltd" },
    { "B04AB4", "Motorola Mobility LLC, a Lenovo Company" },
    { "B04B68", "NAKAYO Inc" },
    { "B04BBF", "PT HAN SUNG ELECTORONICS INDONESIA" },
    { "B04C05", "Fresenius Medical Care Deutschland GmbH" },
    { "B04E26", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "B04F13", "Dell Inc." },
    { "B04F3C", "Genuine Optics" },
    { "B04FA6", "DongGuan Ramaxel Memory Technology" },
    { "B04FC3", "Shenzhen NVC Cloud Technology Co., Ltd." },
    { "B050BC", "SHENZHEN BASICOM ELECTRONIC CO.,LTD." },
    { "B0518E", "Holl technology CO.Ltd." },
    { "B05216", "Hon Hai Precision Ind. Co.,Ltd." },
    { "B05246", "NXP Semiconductor (Tianjin) LTD." },
    { "B05365", "China Mobile IOT Company Limited" },
    { "B05476", "Samsung Electronics Co.,Ltd" },
    { "B05508", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B05706", "Vallox Oy" },
    { "B058C4", "Broadcast Microwave Services, Inc" },
    { "B05947", "Shenzhen Qihu Intelligent Technology Company Limited" },
    { "B05A44", "Fibocom Wireless Inc." },
    { "B05A7B", "Huawei Device Co., Ltd." },
    { "B05ADA", "Hewlett Packard" },
    { "B05B1F", "THERMO FISHER SCIENTIFIC S.P.A." },
    { "B05B67", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B05B99", "Sagemcom Broadband SAS" },
    { "B05C16", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "B05CDA", "HP Inc." },
    { "B05CE5", "Nokia Corporation" },
    { "B05DD4", "Commscope" },
    { "B06088", "Intel Corporate" },
    { "B061C7", "Ericsson-LG Enterprise" },
    { "B0653A", "Murata Manufacturing Co., Ltd." },
    { "B06563", "Shanghai Railway Communication Factory" },
    { "B065BD", "Apple, Inc." },
    { "B065F1", "WIO Manufacturing HK Limited" },
    { "B0672F", "Bowers & Wilkins" },
    { "B067B5", "Apple, Inc." },
    { "B068B6", "Hangzhou OYE Technology Co. Ltd" },
    { "B068E6", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "B06971", "DEI Sales, Inc." },
    { "B06A41", "Google, Inc." },
    { "B06B11", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "B06BB3", "GRT" },
    { "B06CBF", "3ality Digital Systems GmbH" },
    { "B06E72", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "B06EBF", "ASUSTek COMPUTER INC." },
    { "B06FE0", "Samsung Electronics Co.,Ltd" },
    { "B0700D", "Nokia" },
    { "B0702D", "Apple, Inc." },
    { "B07219", "Apple, Inc." },
    { "B072BF", "Murata Manufacturing Co., Ltd." },
    { "B0735D", "Huawei Device Co., Ltd." },
    { "B0739C", "Amazon Technologies Inc." },
    { "B0750C", "QA Cafe" },
    { "B0754D", "Nokia" },
    { "B075D5", "zte corporation" },
    { "B0761B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B077AC", "Commscope" },
    { "B07839", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "B07870", "Wi-NEXT, Inc." },
    { "B078F0", "Beijing HuaqinWorld Technology Co.,Ltd." },
    { "B07908", "Cummings Engineering" },
    { "B0793C", "Revolv Inc" },
    { "B07994", "Motorola Mobility LLC, a Lenovo Company" },
    { "B07ADF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B07B25", "Dell Inc." },
    { "B07C51", "Ruckus Wireless" },
    { "B07C8E", "Brother Industries, LTD." },
    { "B07D47", "Cisco Systems, Inc" },
    { "B07D62", "Dipl.-Ing. H. Horstmann GmbH" },
    { "B07D64", "Intel Corporate" },
    { "B07E11", "Texas Instruments" },
    { "B07E70", "Zadara Storage Ltd." },
    { "B07FB9", "NETGEAR" },
    { "B0808C", "Laser Light Engines" },
    { "B08101", "Honor Device Co., Ltd." },
    { "B08184", "Espressif Inc." },
    { "B081D8", "I-sys Corp" },
    { "B083D6", "Commscope" },
    { "B083FE", "Dell Inc." },
    { "B0869E", "Chloride S.r.L" },
    { "B08807", "Strata Worldwide" },
    { "B08900", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B08991", "LGE " },
    { "B089C2", "Zyptonite" },
    { "B08B92", "zte corporation" },
    { "B08B9E", "Telechips, Inc." },
    { "B08BA8", "Amazon Technologies Inc." },
    { "B08BBE", "ABL GmbH" },
    { "B08BCF", "Cisco Systems, Inc" },
    { "B08BD0", "Cisco Systems, Inc" },
    { "B08C75", "Apple, Inc." },
    { "B08D57", "Cisco Systems, Inc" },
    { "B08E1A", "URadio Systems Co., Ltd" },
    { "B09074", "Fulan Electronics Limited" },
    { "B0907E", "Cisco Systems, Inc" },
    { "B090D4", "Shenzhen Hoin Internet Technology Co., Ltd" },
    { "B09122", "Texas Instruments" },
    { "B09134", "Taleo" },
    { "B09137", "ISis ImageStream Internet Solutions, Inc" },
    { "B09200", "Apple, Inc." },
    { "B0924A", "Sagemcom Broadband SAS" },
    { "B0935B", "Commscope" },
    { "B09575", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "B0958E", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "B0966C", "Lanbowan Technology Ltd." },
    { "B096EA", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "B09738", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "B0973A", "E-Fuel Corporation" },
    { "B0982B", "Sagemcom Broadband SAS" },
    { "B0989F", "LG CNS" },
    { "B098BC", "Huawei Device Co., Ltd." },
    { "B09928", "FUJITSU LIMITED" },
    { "B0995A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B099D7", "Samsung Electronics Co.,Ltd" },
    { "B09AE2", "STEMMER IMAGING GmbH" },
    { "B09BD4", "GNH Software India Private Limited" },
    { "B09C63", "Xiaomi Communications Co Ltd" },
    { "B09E1B", "Butlr Technologies, Inc." },
    { "B09FBA", "Apple, Inc." },
    { "B0A10A", "Pivotal Systems Corporation" },
    { "B0A187", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "B0A2E7", "Shenzhen TINNO Mobile Technology Corp." },
    { "B0A37E", "QING DAO HAIER TELECOM CO.,LTD." },
    { "B0A3F2", "Huaqin Technology Co. LTD" },
    { "B0A454", "Tripwire Inc." },
    { "B0A460", "Intel Corporate" },
    { "B0A4F0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B0A651", "Cisco Systems, Inc" },
    { "B0A6F5", "Xaptum, Inc." },
    { "B0A72A", "Ensemble Designs, Inc." },
    { "B0A732", "Espressif Inc." },
    { "B0A737", "Roku, Inc." },
    { "B0A7B9", "TP-Link Systems Inc" },
    { "B0A7D2", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "B0A86E", "Juniper Networks" },
    { "B0AA36", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "B0AA77", "Cisco Systems, Inc" },
    { "B0AAD2", "Sichuan tianyi kanghe communications co., LTD" },
    { "B0AC82", "CHINA DRAGON TECHNOLOGY LIMITED" },
    { "B0ACD2", "zte corporation" },
    { "B0ACFA", "FUJITSU LIMITED" },
    { "B0ADAA", "Avaya Inc" },
    { "B0AE25", "Varikorea" },
    { "B0AFF7", "Shenzhen Yipingfang Network Technology Co., Ltd." },
    { "B0B113", "Texas Instruments" },
    { "B0B194", "zte corporation" },
    { "B0B21C", "Espressif Inc." },
    { "B0B28F", "Sagemcom Broadband SAS" },
    { "B0B2DC", "Zyxel Communications Corporation" },
    { "B0B32B", "Slican Sp. z o.o." },
    { "B0B353", "IEEE Registration Authority" },
    { "B0B369", "Shenzhen SDMC Technology CO.,Ltd." },
    { "B0B3AD", "HUMAX Co., Ltd." },
    { "B0B448", "Texas Instruments" },
    { "B0B5C3", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "B0B5E8", "Ruroc LTD" },
    { "B0B867", "Hewlett Packard Enterprise " },
    { "B0B8D5", "Nanjing Nengrui Auto Equipment CO.,Ltd" },
    { "B0B98A", "NETGEAR" },
    { "B0BB8B", "WAVETEL TECHNOLOGY LIMITED" },
    { "B0BBE5", "Sagemcom Broadband SAS" },
    { "B0BC7A", "Harman/Becker Automotive Systems GmbH" },
    { "B0BC8E", "SkyMirr" },
    { "B0BD1B", "Dongguan Liesheng Electronic Co., Ltd." },
    { "B0BD6D", "Echostreams Innovative Solutions" },
    { "B0BDA1", "ZAKLAD ELEKTRONICZNY SIMS" },
    { "B0BE76", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "B0BE83", "Apple, Inc." },
    { "B0BF99", "WIZITDONGDO" },
    { "B0C090", "Chicony Electronics Co., Ltd." },
    { "B0C128", "Adler ELREHA GmbH" },
    { "B0C19E", "zte corporation" },
    { "B0C205", "BIONIME" },
    { "B0C287", "Vantiva USA LLC" },
    { "B0C2C7", "Motorola Mobility LLC, a Lenovo Company" },
    { "B0C387", "GOEFER, Inc." },
    { "B0C38E", "Huawei Device Co., Ltd." },
    { "B0C46C", "Senseit" },
    { "B0C4E7", "Samsung Electronics Co.,Ltd" },
    { "B0C53C", "Cisco Systems, Inc" },
    { "B0C554", "D-Link International" },
    { "B0C559", "Samsung Electronics Co.,Ltd" },
    { "B0C5CA", "IEEE Registration Authority" },
    { "B0C61C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B0C69A", "Juniper Networks" },
    { "B0C745", "BUFFALO.INC" },
    { "B0C787", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B0C7DE", "Silicon Laboratories" },
    { "B0C83F", "Jiangsu Cynray IOT Co., Ltd." },
    { "B0C8AD", "People Power Company" },
    { "B0C952", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "B0C95B", "Beijing Symtech CO.,LTD" },
    { "B0CA68", "Apple, Inc." },
    { "B0CAE7", "Huawei Device Co., Ltd." },
    { "B0CCCE", "IEEE Registration Authority" },
    { "B0CCFE", "Huawei Device Co., Ltd." },
    { "B0CE18", "Zhejiang shenghui lighting co.,Ltd" },
    { "B0CF0E", "Mellanox Technologies, Inc." },
    { "B0CF4D", "MI-Zone Technology Ireland" },
    { "B0CFCB", "Amazon Technologies Inc." },
    { "B0D09C", "Samsung Electronics Co.,Ltd" },
    { "B0D1D6", "Nokia Shanghai Bell Co., Ltd." },
    { "B0D278", "Texas Instruments" },
    { "B0D2F5", "Vello Systems, Inc." },
    { "B0D41F", "MOBITITECABSOLUT S.A." },
    { "B0D568", "Shenzhen Cultraview Digital Technology Co., Ltd" },
    { "B0D576", "Apple, Inc." },
    { "B0D59D", "Shenzhen Zowee Technology Co., Ltd" },
    { "B0D5CC", "Texas Instruments" },
    { "B0D5FB", "Google, Inc." },
    { "B0D77E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B0D7C5", "Logipix Ltd" },
    { "B0D7CC", "Tridonic GmbH & Co KG" },
    { "B0D888", "Panasonic Automotive Systems Co.,Ltd" },
    { "B0DA00", "CERA ELECTRONIQUE" },
    { "B0DAF9", "Commscope" },
    { "B0DCEF", "Intel Corporate" },
    { "B0DD74", "Heimgard Technologies AS" },
    { "B0DE28", "Apple, Inc." },
    { "B0DF3A", "Samsung Electronics Co.,Ltd" },
    { "B0DFC1", "Tenda Technology Co.,Ltd.Dongguan branch" },
    { "B0E03C", "TCT mobile ltd" },
    { "B0E17E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B0E235", "Xiaomi Communications Co Ltd" },
    { "B0E2E5", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "B0E39D", "CAT SYSTEM CO.,LTD." },
    { "B0E45C", "Samsung Electronics Co.,Ltd" },
    { "B0E4D5", "Google, Inc." },
    { "B0E50E", "NRG SYSTEMS INC" },
    { "B0E5ED", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B0E5EF", "Apple, Inc." },
    { "B0E5F9", "Apple, Inc." },
    { "B0E71D", "Shanghai Maigantech Co.,Ltd" },
    { "B0E754", "2Wire Inc" },
    { "B0E7DE", "Homa Technologies JSC" },
    { "B0E892", "Seiko Epson Corporation" },
    { "B0E97E", "Advanced Micro Peripherals" },
    { "B0E9FE", "Woan Technology (Shenzhen) Co., Ltd." },
    { "B0EA19", "TPV Audio and Visual Technology (Shenzhen) Co.,Ltd." },
    { "B0EABC", "ASKEY COMPUTER CORP" },
    { "B0EB57", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B0EB7F", "Juniper Networks" },
    { "B0EC71", "Samsung Electronics Co.,Ltd" },
    { "B0EC8F", "GMX SAS" },
    { "B0ECDD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B0ECE1", "Private" },
    { "B0EE45", "AzureWave Technology Inc." },
    { "B0EE7B", "Roku, Inc" },
    { "B0F00C", "Dongguan Wecxw CO.,Ltd." },
    { "B0F1A3", "Fengfan (BeiJing) Technology Co., Ltd. " },
    { "B0F1AE", "eero inc." },
    { "B0F1BC", "Dhemax Ingenieros Ltda" },
    { "B0F1D8", "Apple, Inc." },
    { "B0F1EC", "AMPAK Technology, Inc." },
    { "B0F208", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "B0F2F6", "Samsung Electronics Co.,Ltd" },
    { "B0F3E9", "PATEO CONNECT (Xiamen) Co., Ltd." },
    { "B0F530", "Hitron Technologies. Inc" },
    { "B0F5C8", "AMPAK Technology Inc." },
    { "B0F7C4", "Amazon Technologies Inc." },
    { "B0F893", "Shanghai MXCHIP Information Technology Co., Ltd." },
    { "B0F963", "Hangzhou H3C Technologies Co., Limited" },
    { "B0FA8B", "Huawei Device Co., Ltd." },
    { "B0FA91", "EM Microelectronic" },
    { "B0FAEB", "Cisco Systems, Inc" },
    { "B0FB15", "Laird Connectivity" },
    { "B0FBDD", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "B0FC0D", "Amazon Technologies Inc." },
    { "B0FC36", "CyberTAN Technology Inc." },
    { "B0FC88", "Sagemcom Broadband SAS" },
    { "B0FD0B", "IEEE Registration Authority" },
    { "B0FEBD", "Private" },
    { "B0FEE5", "Huawei Device Co., Ltd." },
    { "B0FF72", "IEEE Registration Authority" },
    { "B40016", "INGENICO TERMINALS SAS" },
    { "B4009C", "CableWorld Ltd." },
    { "B40142", "GCI Science & Technology Co.,LTD" },
    { "B40216", "Cisco Systems, Inc" },
    { "B402F2", "Synaptics, Inc" },
    { "B40418", "Smartchip Integrated Inc." },
    { "B40421", "zte corporation" },
    { "B4055D", "IEIT SYSTEMS Co., Ltd." },
    { "B40566", "SP Best Corporation Co., LTD." },
    { "B405A1", "Xiaomi Communications Co Ltd" },
    { "B407F9", "SAMSUNG ELECTRO MECHANICS CO., LTD." },
    { "B40832", "TC Communications" },
    { "B40931", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B40AC6", "DEXON Systems Ltd." },
    { "B40AD8", "Sony Interactive Entertainment Inc." },
    { "B40B1D", "Samsung Electronics Co.,Ltd" },
    { "B40B44", "Smartisan Technology Co., Ltd." },
    { "B40B78", "Brusa Elektronik AG" },
    { "B40B7A", "Brusa Elektronik AG" },
    { "B40C25", "Palo Alto Networks" },
    { "B40E06", "Third Reality, Inc" },
    { "B40E96", "HERAN " },
    { "B40ECF", "Bouffalo Lab (Nanjing) Co., Ltd." },
    { "B40EDC", "LG-Ericsson Co.,Ltd." },
    { "B40EDE", "Intel Corporate" },
    { "B40F3B", "Tenda Technology Co.,Ltd.Dongguan branch" },
    { "B40FB3", "vivo Mobile Communication Co., Ltd." },
    { "B4107A", "Amazon Technologies Inc." },
    { "B4107B", "Texas Instruments" },
    { "B41324", "Google, Inc." },
    { "B41489", "Cisco Systems, Inc" },
    { "B414E6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B41513", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B4157E", "Celona Inc." },
    { "B41678", "Juniper Networks" },
    { "B4174D", "PROJECT MONITOR INC" },
    { "B41780", "DTI Group Ltd" },
    { "B417A8", "Meta Platforms, Inc." },
    { "B418D1", "Apple, Inc." },
    { "B41974", "Apple, Inc." },
    { "B41A1D", "Samsung Electronics Co.,Ltd" },
    { "B41BB0", "Apple, Inc." },
    { "B41C30", "zte corporation" },
    { "B41CAB", "ICR, inc." },
    { "B41D2B", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "B41DC4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B41DEF", "Internet Laboratories, Inc." },
    { "B41E52", "Flock Safety" },
    { "B41F4D", "Sony Interactive Entertainment Inc." },
    { "B42046", "eero inc." },
    { "B4205B", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "B4211D", "Beijing GuangXin Technology Co., Ltd" },
    { "B4218A", "Dog Hunter LLC" },
    { "B42200", "Brother Industries, LTD." },
    { "B42330", "Itron Inc" },
    { "B423A2", "Google, Inc." },
    { "B424E7", "Codetek Technology Co.,Ltd" },
    { "B4265D", "Taicang T&W Electronics" },
    { "B42875", "Futecho Solutions Private Limited" },
    { "B428F1", "E-Prime Co., Ltd." },
    { "B4293D", "Shenzhen Urovo Technology Co.,Ltd." },
    { "B42A0E", "Vantiva USA LLC" },
    { "B42A39", "ORBIT MERRET, spol. s r. o." },
    { "B42BB9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B42C92", "Zhejiang Weirong Electronic Co., Ltd" },
    { "B42CBE", "Direct Payment Solutions Limited" },
    { "B42D56", "Extreme Networks Headquarters" },
    { "B42E99", "GIGA-BYTE TECHNOLOGY CO.,LTD." },
    { "B42EF8", "Eline Technology co.Ltd" },
    { "B42F03", "Quectel Wireless Solutions Co.,Ltd." },
    { "B43052", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B430C0", "York Instruments Ltd" },
    { "B43161", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "B431B8", "Aviwest" },
    { "B4346C", "MATSUNICHI DIGITAL TECHNOLOGY (HONG KONG) LIMITED" },
    { "B43522", "Silicon Laboratories" },
    { "B43564", "Fujian Tian Cheng Electron Science & Technical Development Co.,Ltd." },
    { "B435F7", "Zhejiang Pearmain Electronics Co.ltd." },
    { "B436A9", "Fibocom Wireless Inc." },
    { "B436D1", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "B436E3", "KBVISION GROUP" },
    { "B43741", "Consert, Inc." },
    { "B437D1", "IEEE Registration Authority" },
    { "B437D8", "D-Link (Shanghai) Limited Corp." },
    { "B43934", "Pen Generations, Inc." },
    { "B43939", "Shenzhen TINNO Mobile Technology Corp." },
    { "B4394C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B439D6", "ProCurve Networking by HP" },
    { "B43A28", "Samsung Electronics Co.,Ltd" },
    { "B43A31", "Silicon Laboratories" },
    { "B43A45", "Espressif Inc." },
    { "B43A96", "Arista Networks" },
    { "B43AE2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B43D08", "GX International BV" },
    { "B43D6B", "NXP Semiconductor (Tianjin) LTD." },
    { "B43DB2", "Degreane Horizon" },
    { "B43E3B", "Viableware, Inc" },
    { "B440A4", "Apple, Inc." },
    { "B440DC", "Samsung Electronics Co.,Ltd" },
    { "B44130", "Jabil Circuit (Guangzhou) Ltd." },
    { "B4417A", "SHENZHEN GONGJIN ELECTRONICS CO.,LT" },
    { "B4430D", "Broadlink Pty Ltd" },
    { "B44326", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B44506", "Dell Inc." },
    { "B4466B", "REALTIMEID AS" },
    { "B4475E", "Avaya Inc" },
    { "B447F5", "Earda Technologies co Ltd" },
    { "B44BD2", "Apple, Inc." },
    { "B44BD6", "IEEE Registration Authority" },
    { "B44C3B", "Zhejiang Dahua Technology Co., Ltd." },
    { "B44C90", "Cisco Systems, Inc" },
    { "B44CC2", "NR ELECTRIC CO., LTD" },
    { "B44D43", "IEEE Registration Authority" },
    { "B44F96", "Zhejiang Xinzailing Technology co., ltd" },
    { "B45062", "EmBestor Technology Inc." },
    { "B451F9", "NB Software" },
    { "B45253", "Seagate Technology" },
    { "B4527D", "Sony Corporation" },
    { "B4527E", "Sony Corporation" },
    { "B452A9", "Texas Instruments" },
    { "B45459", "China Mobile (Hangzhou) Information Technology Co., Ltd." },
    { "B45570", "Borea" },
    { "B4565D", "Chipsea Technologies (Shenzhen) Corp." },
    { "B456B9", "Teraspek Technologies Co.,Ltd" },
    { "B456E3", "Apple, Inc." },
    { "B456FA", "IOPSYS Software Solutions" },
    { "B457E6", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "B45861", "CRemote, LLC" },
    { "B45976", "Apple, Inc." },
    { "B45B86", "Funshion Online Technologies Co.,Ltd" },
    { "B45BD1", "TP-Link Systems Inc." },
    { "B45CA4", "Thing-talk Wireless Communication Technologies Corporation Limited" },
    { "B45D50", "Hewlett Packard Enterprise" },
    { "B45F84", "zte corporation" },
    { "B46077", "Sichuan Changhong Electric Ltd." },
    { "B4608C", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "B460ED", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "B46142", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B461E9", "Sichuan AI-Link Technology Co., Ltd." },
    { "B461FF", "Lumigon A/S" },
    { "B4622E", "Zhong Ge Smart Technology Co., Ltd." },
    { "B46238", "Exablox" },
    { "B46293", "Samsung Electronics Co.,Ltd" },
    { "B462AD", "Elysia Germany GmbH" },
    { "B4636F", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "B46415", "Guangzhou V-Solution Telecommunication Technology Co.,Ltd." },
    { "B46698", "Zealabs srl" },
    { "B467E9", "Qingdao GoerTek Technology Co., Ltd." },
    { "B46921", "Intel Corporate" },
    { "B4695F", "TCT mobile ltd" },
    { "B46AD4", "Edgecore Americas Networking Corporation" },
    { "B46BFC", "Intel Corporate" },
    { "B46C47", "Panasonic Appliances Company" },
    { "B46D35", "Dalian Seasky Automation Co;Ltd" },
    { "B46D83", "Intel Corporate" },
    { "B46DC2", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "B46E08", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B46E10", "vivo Mobile Communication Co., Ltd." },
    { "B46F2D", "Wahoo Fitness" },
    { "B47064", "Samsung Electronics Co.,Ltd" },
    { "B47356", "Hangzhou Treebear Networking Co., Ltd." },
    { "B47443", "Samsung Electronics Co.,Ltd" },
    { "B47447", "CoreOS" },
    { "B4749F", "ASKEY COMPUTER CORP" },
    { "B4750E", "Belkin International Inc." },
    { "B476A4", "Huawei Device Co., Ltd." },
    { "B47748", "Shenzhen Neoway Technology Co.,Ltd." },
    { "B47947", "Nutanix" },
    { "B479A7", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "B479C8", "Ruckus Wireless" },
    { "B47AF1", "Hewlett Packard Enterprise " },
    { "B47B1A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B47C29", "Shenzhen Guzidi Technology Co.,Ltd" },
    { "B47C59", "Jiangsu Hengxin Technology Co.,Ltd." },
    { "B47C9C", "Amazon Technologies Inc." },
    { "B47CA6", "zte corporation" },
    { "B47D76", "KNS Group LLC" },
    { "B47F5E", "Foresight Manufacture (S) Pte Ltd" },
    { "B48107", "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD" },
    { "B481BF", "Meta-Networks, LLC" },
    { "B48255", "Research Products Corporation" },
    { "B4827B", "AKG Acoustics GmbH" },
    { "B482C5", "Relay2, Inc." },
    { "B482FE", "ASKEY COMPUTER CORP" },
    { "B48351", "Intel Corporate" },
    { "B484D5", "GooWi Wireless Technology Co., Limited" },
    { "B48547", "Amptown System Company GmbH" },
    { "B485E1", "Apple, Inc." },
    { "B48618", "GX India Pvt Ltd" },
    { "B48655", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B48901", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B48910", "Coster T.E. S.P.A." },
    { "B48931", "Silicon Laboratories" },
    { "B48970", "IGEN Tech Co., Ltd." },
    { "B48A0A", "Espressif Inc." },
    { "B48A5F", "Juniper Networks" },
    { "B48B19", "Apple, Inc." },
    { "B48C9D", "AzureWave Technology Inc." },
    { "B492FE", "Arista Network, Inc." },
    { "B4944E", "WeTelecom Co., Ltd." },
    { "B49691", "Intel Corporate" },
    { "B496A5", "Apple, Inc." },
    { "B49842", "zte corporation" },
    { "B49882", "Brusa HyPower AG" },
    { "B4994C", "Texas Instruments" },
    { "B499BA", "Hewlett Packard" },
    { "B49A95", "Shenzhen Boomtech Industrial Corporation" },
    { "B49CDF", "Apple, Inc." },
    { "B49D02", "Samsung Electronics Co.,Ltd" },
    { "B49D0B", "BQ" },
    { "B49DB4", "Axion Technologies Inc." },
    { "B49DFD", "Shenzhen SDMC Technology CO.,Ltd." },
    { "B49E80", "Sichuan Changhong Electric Ltd." },
    { "B49EAC", "Imagik Int'l Corp" },
    { "B49EE6", "SHENZHEN TECHNOLOGY CO LTD" },
    { "B49F4D", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "B4A10A", "Huawei Device Co., Ltd." },
    { "B4A25C", "Cambium Networks Limited" },
    { "B4A2EB", "IEEE Registration Authority" },
    { "B4A305", "XIAMEN YAXON NETWORK CO., LTD." },
    { "B4A382", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "B4A3BD", "Extreme Networks Headquarters" },
    { "B4A4B5", "Zen Eye Co.,Ltd" },
    { "B4A4E3", "Cisco Systems, Inc" },
    { "B4A5A9", "MODI GmbH" },
    { "B4A5AC", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "B4A5EF", "Sercomm Corporation." },
    { "B4A678", "Zhejiang Tmall Technology Co., Ltd." },
    { "B4A7C6", "SERVERCOM (INDIA) PRIVATE LIMITED" },
    { "B4A828", "Shenzhen Concox Information Technology Co., Ltd" },
    { "B4A82B", "Histar Digital Electronics Co., Ltd." },
    { "B4A898", "Huawei Device Co., Ltd." },
    { "B4A8B9", "Cisco Systems, Inc" },
    { "B4A94F", "MERCURY CORPORATION" },
    { "B4A95A", "Avaya Inc" },
    { "B4A984", "Symantec Corporation" },
    { "B4A9FC", "Quanta Computer Inc." },
    { "B4A9FE", "GHIA Technology (Shenzhen) LTD" },
    { "B4AA4D", "Ensequence, Inc." },
    { "B4AB2C", "MtM Technology Corporation" },
    { "B4AC8C", "Bern University of Applied Sciences" },
    { "B4AC9D", "Texas Instruments" },
    { "B4ADA3", "Guangzhou Shiyuan Electronic Technology Company Limited" },
    { "B4AE2B", "Microsoft" },
    { "B4AE6F", "Circle Reliance, Inc DBA Cranberry Networks" },
    { "B4AEC1", "Apple, Inc." },
    { "B4B017", "Avaya Inc" },
    { "B4B024", "TP-Link Systems Inc" },
    { "B4B055", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B4B15A", "Siemens AG Energy Management Division" },
    { "B4B265", "DAEHO I&T" },
    { "B4B291", "LG Electronics" },
    { "B4B2E9", "Fortinet, Inc." },
    { "B4B362", "zte corporation" },
    { "B4B384", "ShenZhen Figigantic Electronic Co.,Ltd" },
    { "B4B52F", "Hewlett Packard" },
    { "B4B542", "Hubbell Power Systems, Inc." },
    { "B4B5AF", "Minsung Electronics" },
    { "B4B5B6", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "B4B676", "Intel Corporate" },
    { "B4B686", "Hewlett Packard" },
    { "B4B742", "Amazon Technologies Inc." },
    { "B4B859", "Texa Spa" },
    { "B4B88D", "Thuh Company" },
    { "B4B9E6", "eero inc." },
    { "B4BA02", "Agatel Ltd" },
    { "B4BA12", "China Mobile (Hangzhou) Information Technology Co.,Ltd." },
    { "B4BA6A", "TECNO MOBILE LIMITED" },
    { "B4BA9D", "SKY UK LIMITED" },
    { "B4BC7C", "Texas Instruments" },
    { "B4BFF6", "Samsung Electronics Co.,Ltd" },
    { "B4C0F5", "Shenzhen TINNO Mobile Technology Corp." },
    { "B4C170", "Yi chip Microelectronics (Hangzhou) Co., Ltd" },
    { "B4C26A", "Garmin International" },
    { "B4C2E0", "Bouffalo Lab (Nanjing) Co., Ltd." },
    { "B4C2F7", "Huawei Device Co., Ltd." },
    { "B4C44E", "VXL eTech Pvt Ltd" },
    { "B4C476", "Wuhan Maritime Communication Research Institute" },
    { "B4C4FC", "Xiaomi Communications Co Ltd" },
    { "B4C556", "Shanghai Kenmyond Industrial Network Equipment Co., Ltd" },
    { "B4C62E", "Molex CMS" },
    { "B4C6F8", "Axilspot Communication" },
    { "B4C799", "Extreme Networks Headquarters" },
    { "B4C810", "Umpi srl" },
    { "B4C9B9", "Sichuan AI-Link Technology Co., Ltd." },
    { "B4CADD", "Cisco Systems, Inc" },
    { "B4CB57", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "B4CBB8", "Universal Electronics, Inc." },
    { "B4CC04", "Piranti" },
    { "B4CCE9", "PROSYST" },
    { "B4CD27", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B4CDF5", "CUB ELECPARTS INC." },
    { "B4CE40", "Samsung Electronics Co.,Ltd" },
    { "B4CEF6", "HTC Corporation" },
    { "B4CEFE", "James Czekaj" },
    { "B4CFDB", "Shenzhen Jiuzhou Electric Co.,LTD" },
    { "B4CFE0", "Sichuan tianyi kanghe communications co., LTD" },
    { "B4D0A9", "China Mobile Group Device Co.,Ltd." },
    { "B4D135", "Cloudistics" },
    { "B4D286", "Telechips, Inc." },
    { "B4D31A", "LYSORA TECHNOLOGY INC." },
    { "B4D5BD", "Intel Corporate" },
    { "B4D5E5", "Samsung Electronics Co.,Ltd" },
    { "B4D64E", "Caldero Limited" },
    { "B4D7DB", "New H3C Technologies Co., Ltd" },
    { "B4D8A9", "BetterBots" },
    { "B4D8DE", "iota Computing, Inc." },
    { "B4DB91", "CELESTICA INC." },
    { "B4DC09", "Guangzhou Dawei Communication Co.,Ltd" },
    { "B4DD15", "ControlThings Oy Ab" },
    { "B4DDD0", "Continental Automotive Hungary Kft" },
    { "B4DDE0", "Shanghai Amphenol Airwave Communication Electronics Co.,Ltd." },
    { "B4DE31", "Cisco Systems, Inc" },
    { "B4DEDF", "zte corporation" },
    { "B4DF3B", "Chromlech" },
    { "B4DF91", "Cisco Meraki" },
    { "B4DFFA", "Litemax Electronics Inc." },
    { "B4E01D", "CONCEPTION ELECTRONIQUE" },
    { "B4E025", "ITLook" },
    { "B4E0CD", "Fusion-io, Inc" },
    { "B4E10F", "Dell Inc." },
    { "B4E1C4", "Microsoft Mobile Oy" },
    { "B4E1EB", "Private" },
    { "B4E265", "Shenzhen SDMC Technology CO.,Ltd." },
    { "B4E3D0", "LG Electronics" },
    { "B4E3F9", "Silicon Laboratories" },
    { "B4E454", "Amazon Technologies Inc." },
    { "B4E46B", "China Mobile IOT Company Limited" },
    { "B4E53E", "Ruckus Wireless" },
    { "B4E54C", "LLC Elektra" },
    { "B4E62A", "LG Innotek" },
    { "B4E62D", "Espressif Inc." },
    { "B4E782", "Vivalnk" },
    { "B4E7B3", "Edifier International" },
    { "B4E842", "Hong Kong Bouffalo Lab Limited" },
    { "B4E85C", "fünfeinhalb Funksysteme GmbH" },
    { "B4E8C9", "XADA Technologies" },
    { "B4E9A3", "port industrial automation GmbH" },
    { "B4E9B0", "Cisco Systems, Inc" },
    { "B4E9B8", "Dell Inc." },
    { "B4EC02", "ALPSALPINE CO,.LTD" },
    { "B4ECF2", "Shanghai Listent Medical Tech Co., Ltd." },
    { "B4ECFF", "Wuhan IPG Technologies Co., Ltd." },
    { "B4ED19", "Pie Digital, Inc." },
    { "B4ED54", "Wohler Technologies" },
    { "B4EDD5", "Quectel Wireless Solutions Co.,Ltd." },
    { "B4EE25", "Shenzhen Belon Technology CO.,LTD" },
    { "B4EEB4", "ASKEY COMPUTER CORP" },
    { "B4EED4", "Texas Instruments" },
    { "B4EF04", "DAIHAN Scientific Co., Ltd." },
    { "B4EF1C", "360 AI Technology Co.Ltd" },
    { "B4EF30", "Shanghai SYH Technology CO.,LTD" },
    { "B4EF39", "Samsung Electronics Co.,Ltd" },
    { "B4EFFA", "Lemobile Information Technology (Beijing) Co., Ltd." },
    { "B4F0AB", "Apple, Inc." },
    { "B4F18C", "Huawei Device Co., Ltd." },
    { "B4F1DA", "LG Electronics (Mobile Communications)" },
    { "B4F267", "Compal Broadband Networks, Inc." },
    { "B4F2E8", "Commscope" },
    { "B4F323", "PETATEL INC." },
    { "B4F58E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B4F61C", "Apple, Inc." },
    { "B4F7A1", "LG Electronics (Mobile Communications)" },
    { "B4F81E", "Kinova" },
    { "B4F949", "optilink networks pvt ltd" },
    { "B4F95D", "Juniper Networks" },
    { "B4FA48", "Apple, Inc." },
    { "B4FBE3", "AltoBeam (China) Inc." },
    { "B4FBE4", "Ubiquiti Inc" },
    { "B4FBF9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B4FC75", "SEMA Electronics(HK) CO.,LTD" },
    { "B4FC7D", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "B4FE8C", "Centro Sicurezza Italia SpA" },
    { "B4FF98", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B80018", "Htel" },
    { "B8011F", "Apple, Inc." },
    { "B802A4", "Aeonsemi, Inc." },
    { "B80305", "Intel Corporate" },
    { "B80415", "Bayan Audio" },
    { "B80533", "Apple, Inc." },
    { "B805AB", "zte corporation" },
    { "B8060D", "Tuya Smart Inc." },
    { "B80716", "vivo Mobile Communication Co., Ltd." },
    { "B80756", "Cisco Meraki" },
    { "B808CF", "Intel Corporate" },
    { "B808D7", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B8098A", "Apple, Inc." },
    { "B80B9D", "ROPEX Industrie-Elektronik GmbH" },
    { "B80BDA", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "B810D4", "Masimo Corporation" },
    { "B8114B", "Cisco Systems, Inc" },
    { "B812DA", "LVSWITCHES INC." },
    { "B81332", "AMPAK Technology,Inc." },
    { "B813E9", "Trace Live Network" },
    { "B81413", "Keen High Holding(HK) Ltd." },
    { "B8144D", "Apple, Inc." },
    { "B8145C", "Huawei Device Co., Ltd." },
    { "B814DB", "OHSUNG" },
    { "B81619", "Commscope" },
    { "B8165F", "LG Innotek" },
    { "B816DB", "CHANT SINCERE CO.,LTD" },
    { "B81743", "shenzhen worldelite electronics co., LTD" },
    { "B817C2", "Apple, Inc." },
    { "B8186F", "ORIENTAL MOTOR CO., LTD." },
    { "B81904", "Nokia Shanghai Bell Co., Ltd." },
    { "B81999", "Nesys" },
    { "B81D1F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B81DAA", "LG Electronics (Mobile Communications)" },
    { "B81E9E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B81EA4", "Liteon Technology Corporation" },
    { "B81F5E", "Apption Labs Limited" },
    { "B8208E", "Panasonic Connect Co., Ltd." },
    { "B820E7", "Guangzhou Horizontal Information & Network Integration Co. Ltd" },
    { "B8211C", "Apple, Inc." },
    { "B8220C", "Apple, Inc." },
    { "B8224F", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "B82410", "Magneti Marelli Slovakia s.r.o." },
    { "B8241A", "SWEDA INFORMATICA LTDA" },
    { "B824F0", "SOYO Technology Development Co., Ltd." },
    { "B8259A", "Thalmic Labs " },
    { "B825B5", "Trakm8 Ltd" },
    { "B8266C", "ANOV France" },
    { "B826D4", "Furukawa Industrial S.A. Produtos Elétricos" },
    { "B827C5", "Huawei Device Co., Ltd." },
    { "B827EB", "Raspberry Pi Foundation" },
    { "B8288B", "Parker Hannifin Manufacturing (UK) Ltd" },
    { "B82903", "VIETNAM POST AND TELECOMMUNICATION INDUSTRY TECHNOLOGY JOIN STOCK COMPANY" },
    { "B829F7", "Blaster Tech" },
    { "B82A72", "Dell Inc." },
    { "B82AA9", "Apple, Inc." },
    { "B82ADC", "EFR Europäische Funk-Rundsteuerung GmbH" },
    { "B82B68", "Huawei Device Co., Ltd." },
    { "B82CA0", "Resideo" },
    { "B82D28", "AMPAK Technology,Inc." },
    { "B82FCB", "CMS Electracom" },
    { "B830A8", "Road-Track Telematics Development" },
    { "B831B5", "Microsoft Corporation" },
    { "B83241", "Wuhan Tianyu Information Industry Co., Ltd." },
    { "B836D8", "Videoswitch" },
    { "B8374A", "Apple, Inc." },
    { "B8374B", "Hewlett Packard Enterprise " },
    { "B83765", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "B837B2", "Hewlett Packard Enterprise" },
    { "B83861", "Cisco Systems, Inc" },
    { "B838CA", "Kyokko Tsushin System CO.,LTD" },
    { "B838EF", "ADVA Optical Networking Ltd." },
    { "B83A08", "Tenda Technology Co.,Ltd.Dongguan branch" },
    { "B83A5A", "Hewlett Packard Enterprise" },
    { "B83A7B", "Worldplay (Canada) Inc." },
    { "B83A9D", "Alarm.com" },
    { "B83B8F", "Hangzhou Hylin IoT Techonology Co.,Ltd." },
    { "B83BAB", "Arcadyan Corporation" },
    { "B83BCC", "Xiaomi Communications Co Ltd" },
    { "B83C20", "Huawei Device Co., Ltd." },
    { "B83C28", "Apple, Inc." },
    { "B83D4E", "Shenzhen Cultraview Digital Technology Co.,Ltd Shanghai Branch" },
    { "B83DF6", "Texas Instruments" },
    { "B83DFB", "Bouffalo Lab (Nanjing) Co., Ltd." },
    { "B83E59", "Roku, Inc." },
    { "B83FD2", "Mellanox Technologies, Inc." },
    { "B8415F", "ASP AG" },
    { "B841A4", "Apple, Inc." },
    { "B841D9", "Guangzhou Shiyuan Electronic Technology Company Limited" },
    { "B843E4", "Vlatacom" },
    { "B844AE", "TCT mobile ltd" },
    { "B844D9", "Apple, Inc." },
    { "B845EB", "Apple, Inc." },
    { "B845F4", "New H3C Technologies Co., Ltd" },
    { "B8477A", "Dasan Electron Co., Ltd." },
    { "B847C6", "SanJet Technology Corp." },
    { "B848AA", "EM Microelectronic" },
    { "B848EC", "Private" },
    { "B8496D", "Apple, Inc." },
    { "B84C87", "IEEE Registration Authority" },
    { "B84D43", "HUNAN FN-LINK TECHNOLOGY LIMITED" },
    { "B84DEE", "Hisense broadband multimedia technology Co.,Ltd" },
    { "B84FA7", "Apple, Inc." },
    { "B84FD5", "Microsoft Corporation" },
    { "B85001", "Extreme Networks Headquarters" },
    { "B850D8", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "B851A9", "Nokia" },
    { "B852E0", "Beijing Xiaomi Electronics Co.,Ltd" },
    { "B853AC", "Apple, Inc." },
    { "B85510", "Zioncom Electronics (Shenzhen) Ltd." },
    { "B85600", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B856BD", "ITT LLC" },
    { "B85776", "lignex1" },
    { "B857D8", "Samsung Electronics Co.,Ltd" },
    { "B85810", "NUMERA, INC." },
    { "B8599F", "Mellanox Technologies, Inc." },
    { "B859C8", "70mai Co.,Ltd." },
    { "B859CE", "Earda Technologies co Ltd" },
    { "B85A73", "Samsung Electronics Co.,Ltd" },
    { "B85AF7", "Ouya, Inc" },
    { "B85AFE", "Handaer Communication Technology (Beijing) Co., Ltd" },
    { "B85B6C", "Control Accessories LLC" },
    { "B85C5C", "Microsoft Corporation" },
    { "B85CEE", "Baidu Online Network Technology (Beijing) Co., Ltd" },
    { "B85D0A", "Apple, Inc." },
    { "B85DC3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B85E71", "Vantiva USA LLC" },
    { "B85E7B", "Samsung Electronics Co.,Ltd" },
    { "B85F98", "Amazon Technologies Inc." },
    { "B85FB0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B86061", "China Mobile Group Device Co.,Ltd." },
    { "B86091", "Onnet Technologies and Innovations LLC" },
    { "B86142", "Beijing Tricolor Technology Co., Ltd" },
    { "B8616F", "Accton Technology Corp" },
    { "B8621F", "Cisco Systems, Inc" },
    { "B8634D", "Apple, Inc." },
    { "B86392", "GUANGDONG GENIUS TECHNOLOGY CO., LTD." },
    { "B863BC", "ROBOTIS, Co, Ltd" },
    { "B86491", "CK Telecom Ltd" },
    { "B8653B", "Bolymin, Inc." },
    { "B86685", "Sagemcom Broadband SAS" },
    { "B869C2", "Sunitec Enterprise Co., Ltd." },
    { "B869F4", "Routerboard.com" },
    { "B86A97", "Edgecore Networks Corporation" },
    { "B86AF1", "Sagemcom Broadband SAS" },
    { "B86B23", "Toshiba" },
    { "B86CE0", "Hewlett Packard Enterprise " },
    { "B86CE8", "Samsung Electronics Co.,Ltd" },
    { "B87029", "Shenzhen Ruiyuanchuangxin Technology Co.,Ltd." },
    { "B870F4", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "B87424", "Viessmann Elektronik GmbH" },
    { "B87447", "Convergence Technologies" },
    { "B875C0", "PayPal, Inc." },
    { "B8763F", "Hon Hai Precision Ind. Co.,Ltd." },
    { "B877C3", "METER Group" },
    { "B87826", "Nintendo Co.,Ltd" },
    { "B8782E", "Apple, Inc." },
    { "B87879", "Roche Diagnostics GmbH" },
    { "B8797E", "Secure Meters (UK) Limited" },
    { "B87AC9", "Siemens Ltd." },
    { "B87BC5", "Apple, Inc." },
    { "B87BD4", "Google, Inc." },
    { "B87C6F", "NXP (China) Management Ltd." },
    { "B87CD0", "Huawei Device Co., Ltd." },
    { "B87CF2", "Extreme Networks Headquarters" },
    { "B87E39", "Motorola Mobility LLC, a Lenovo Company" },
    { "B87E40", "Huawei Device Co., Ltd." },
    { "B87EE5", "Intelbras" },
    { "B88035", "Shenzhen Qihu Intelligent Technology Company Limited" },
    { "B8804F", "Texas Instruments" },
    { "B88198", "Intel Corporate" },
    { "B881FA", "Apple, Inc." },
    { "B882F2", "Wistron Neweb Corporation" },
    { "B88303", "Hewlett Packard Enterprise " },
    { "B8857B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B88584", "Dell Inc." },
    { "B88687", "Liteon Technology Corporation" },
    { "B8871E", "Good Mind Industries Co., Ltd." },
    { "B8876E", "Intertech Services AG" },
    { "B887A8", "Step Ahead Innovations Inc." },
    { "B887C6", "Prudential Technology co.,LTD" },
    { "B88880", "Shanghai Imilab Technology Co.Ltd" },
    { "B888E3", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "B88981", "Chengdu InnoThings Technology Co., Ltd." },
    { "B889CA", "ILJIN ELECTRIC Co., Ltd." },
    { "B88A60", "Intel Corporate" },
    { "B88A72", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "B88AEC", "Nintendo Co.,Ltd" },
    { "B88C29", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "B88C2B", "Sagemcom Broadband SAS" },
    { "B88D12", "Apple, Inc." },
    { "B88DF1", "Nanjing BigFish Semiconductor Co., Ltd." },
    { "B88E3A", "Infinite Technologies JLT" },
    { "B88E82", "Huawei Device Co., Ltd." },
    { "B88EB0", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "B88EC6", "Stateless Networks" },
    { "B88EDF", "Zencheer Communication Technology Co., Ltd." },
    { "B88F14", "Analytica GmbH" },
    { "B88F27", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "B88FB4", "JABIL CIRCUIT ITALIA S.R.L" },
    { "B89047", "Apple, Inc." },
    { "B89165", "Zhejiang Tmall Technology Co., Ltd." },
    { "B891C9", "Handreamnet" },
    { "B8921D", "BG T&A" },
    { "B89436", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B89470", "Calix Inc." },
    { "B894D2", "Retail Innovation HTT AB" },
    { "B894D9", "Texas Instruments" },
    { "B894E7", "Xiaomi Communications Co Ltd" },
    { "B89674", "AllDSP GmbH & Co. KG" },
    { "B8975A", "BIOSTAR Microtech Int'l Corp." },
    { "B8977A", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "B898AD", "Motorola Mobility LLC, a Lenovo Company" },
    { "B898B0", "Atlona Inc." },
    { "B898F7", "Gionee Communication Equipment Co,Ltd.ShenZhen" },
    { "B89919", "7signal Solutions, Inc" },
    { "B899AE", "Shenzhen MiaoMing Intelligent Technology Co.,Ltd" },
    { "B899B0", "Cohere Technologies" },
    { "B89A2A", "Intel Corporate" },
    { "B89A9A", "Xin Shi Jia Technology (Beijing) Co.,Ltd" },
    { "B89ACD", "ELITE OPTOELECTRONIC(ASIA)CO.,LTD" },
    { "B89AED", "OceanServer Technology, Inc" },
    { "B89BC9", "SMC Networks Inc" },
    { "B89BE4", "ABB Power Systems Power Generation" },
    { "B89C13", "Alps Alpine" },
    { "B89EA6", "SPBEC-MINING CO.LTD" },
    { "B89F09", "Wistron Neweb Corporation" },
    { "B89FCC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B8A0B8", "Samsung Electronics Co.,Ltd" },
    { "B8A14A", "Raisecom Technology CO., LTD" },
    { "B8A175", "Roku, Inc." },
    { "B8A1B8", "Arista Networks" },
    { "B8A25D", "Motorola Mobility LLC, a Lenovo Company" },
    { "B8A377", "Cisco Systems, Inc" },
    { "B8A386", "D-Link International" },
    { "B8A3E0", "BenRui Technology Co.,Ltd" },
    { "B8A44F", "Axis Communications AB" },
    { "B8A535", "Vantiva USA LLC" },
    { "B8A58D", "Axe Group Holdings Limited" },
    { "B8A75E", "Wuxi Xinjie Electric Co.,Ltd" },
    { "B8A825", "Samsung Electronics Co.,Ltd" },
    { "B8A8AF", "Logic S.p.A." },
    { "B8AB61", "Cisco Meraki" },
    { "B8AB62", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "B8AC6F", "Dell Inc." },
    { "B8AD3E", "BLUECOM" },
    { "B8AE1C", "Smart Cube., Ltd" },
    { "B8AE1D", "Guangzhou Xingyi Electronic Technology Co.,Ltd" },
    { "B8AE6E", "Nintendo Co., Ltd." },
    { "B8AEED", "Elitegroup Computer Systems Co.,Ltd." },
    { "B8AF67", "Hewlett Packard" },
    { "B8B1C7", "BT&COM CO.,LTD" },
    { "B8B1EA", "Honor Device Co., Ltd." },
    { "B8B2EB", "Googol Technology (HK) Limited" },
    { "B8B2F7", "DRIMAES INC." },
    { "B8B2F8", "Apple, Inc." },
    { "B8B3DC", "DEREK (SHAOGUAN) LIMITED" },
    { "B8B409", "Samsung Electronics Co.,Ltd" },
    { "B8B42E", "Gionee Communication Equipment Co,Ltd.ShenZhen" },
    { "B8B4C9", "Cisco Meraki" },
    { "B8B77D", "Guangdong Transtek Medical Electronics CO.,Ltd" },
    { "B8B7D7", "2GIG Technologies" },
    { "B8B7DB", "GOIP Global Services Pvt. Ltd." },
    { "B8B7F1", "Wistron Neweb Corporation" },
    { "B8B81E", "Intel Corporate" },
    { "B8B94E", "Shenzhen iBaby Labs, Inc." },
    { "B8BA68", "Xi'an Jizhong Digital Communication Co.,Ltd" },
    { "B8BA72", "Cynove" },
    { "B8BB23", "Guangdong Nufront CSC Co., Ltd" },
    { "B8BB6D", "ENERES Co.,Ltd." },
    { "B8BBAF", "Samsung Electronics Co.,Ltd" },
    { "B8BC1B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B8BC5B", "Samsung Electronics Co.,Ltd" },
    { "B8BD79", "TrendPoint Systems" },
    { "B8BEBF", "Cisco Systems, Inc" },
    { "B8BEF4", "devolo AG" },
    { "B8BF83", "Intel Corporate" },
    { "B8C007", "tickIoT Inc." },
    { "B8C051", "VusionGroup" },
    { "B8C065", "Universal Electronics, Inc." },
    { "B8C111", "Apple, Inc." },
    { "B8C1A2", "Dragon Path Technologies Co., Limited" },
    { "B8C227", "PSTec" },
    { "B8C253", "Juniper Networks" },
    { "B8C385", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B8C3BF", "Henan Chengshi NetWork Technology Co.，Ltd" },
    { "B8C46F", "PRIMMCON INDUSTRIES INC" },
    { "B8C68E", "Samsung Electronics Co.,Ltd" },
    { "B8C6AA", "Earda Technologies co Ltd" },
    { "B8C716", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "B8C74A", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "B8C75D", "Apple, Inc." },
    { "B8C855", "Shanghai GBCOM Communication Technology Co.,Ltd." },
    { "B8C8EB", "ITEL MOBILE LIMITED" },
    { "B8C9B5", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "B8CA04", "Holtek Semiconductor Inc." },
    { "B8CA3A", "Dell Inc." },
    { "B8CB29", "Dell Inc." },
    { "B8CB93", "IC BOSS.COM TECH INC" },
    { "B8CC5F", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "B8CD93", "Penetek, Inc" },
    { "B8CDA7", "Maxeler Technologies Ltd." },
    { "B8CEF6", "Mellanox Technologies, Inc." },
    { "B8D06F", "GUANGZHOU HKUST FOK YING TUNG RESEARCH INSTITUTE" },
    { "B8D0F0", "FCNT LLC" },
    { "B8D309", "Cox Communications, Inc" },
    { "B8D43E", "vivo Mobile Communication Co., Ltd." },
    { "B8D49D", "M Seven System Ltd." },
    { "B8D4BC", "zte corporation" },
    { "B8D4C3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B8D4E7", "Hewlett Packard Enterprise" },
    { "B8D4F7", "New H3C Technologies Co., Ltd" },
    { "B8D50B", "Sunitec Enterprise Co.,Ltd" },
    { "B8D526", "Zyxel Communications Corporation" },
    { "B8D56B", "Mirka Ltd." },
    { "B8D61A", "Espressif Inc." },
    { "B8D6F6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B8D7AF", "Murata Manufacturing Co., Ltd." },
    { "B8D812", "IEEE Registration Authority" },
    { "B8D82D", "Qingdao Intelligent&Precise Electronics Co.,Ltd." },
    { "B8D94D", "Sagemcom Broadband SAS" },
    { "B8D9CE", "Samsung Electronics Co.,Ltd" },
    { "B8DAE8", "Huawei Device Co., Ltd." },
    { "B8DAF1", "Strahlenschutz- Entwicklungs- und Ausruestungsgesellschaft mbH" },
    { "B8DAF7", "Advanced Photonics, Inc." },
    { "B8DB1C", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "B8DB38", "Google, Inc." },
    { "B8DC87", "IAI Corporation" },
    { "B8DD71", "zte corporation" },
    { "B8DDE8", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "B8DE5E", "LONGCHEER TELECOMMUNICATION LIMITED" },
    { "B8DF6B", "SpotCam Co., Ltd." },
    { "B8DFD4", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "B8E28C", "MOTOROLA SOLUTIONS MALAYSIA SDN. BHD." },
    { "B8E3B1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "B8E3EE", "Universal Electronics, Inc." },
    { "B8E44A", "Xconnect" },
    { "B8E589", "Payter BV" },
    { "B8E60C", "Apple, Inc." },
    { "B8E625", "2Wire Inc" },
    { "B8E779", "9Solutions Oy" },
    { "B8E856", "Apple, Inc." },
    { "B8E924", "Mellanox Technologies, Inc." },
    { "B8E937", "Sonos, Inc." },
    { "B8EA98", "Xiaomi Communications Co Ltd" },
    { "B8EAAA", "ICG NETWORKS CO.,ltd" },
    { "B8ECA3", "Zyxel Communications Corporation" },
    { "B8EE0E", "Sagemcom Broadband SAS" },
    { "B8EE65", "Liteon Technology Corporation" },
    { "B8EE79", "YWire Technologies, Inc." },
    { "B8EF8B", "SHENZHEN CANNICE TECHNOLOGY CO.,LTD" },
    { "B8F009", "Espressif Inc." },
    { "B8F015", "Juniper Networks" },
    { "B8F080", "SPS, INC." },
    { "B8F0B9", "zte corporation" },
    { "B8F12A", "Apple, Inc." },
    { "B8F255", "Universal Electronics, Inc." },
    { "B8F317", "iSun Smasher Communications Private Limited" },
    { "B8F44F", "u-blox AG" },
    { "B8F4D0", "Herrmann Ultraschalltechnik GmbH & Co. Kg" },
    { "B8F5E7", "WayTools, LLC" },
    { "B8F653", "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd" },
    { "B8F6B1", "Apple, Inc." },
    { "B8F732", "Aryaka Networks Inc" },
    { "B8F74A", "RCNTEC" },
    { "B8F774", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "B8F775", "Intel Corporate" },
    { "B8F828", "Changshu Gaoshida Optoelectronic Technology Co. Ltd." },
    { "B8F853", "Arcadyan Corporation" },
    { "B8F862", "Espressif Inc." },
    { "B8F883", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "B8F8BE", "BLUECOM" },
    { "B8F934", "Sony Corporation" },
    { "B8FBAF", "Xiamen IPRT Technology CO.,LTD" },
    { "B8FC28", "Valeo Vision Systems" },
    { "B8FC9A", "Le Shi Zhi Xin Electronic Technology (Tianjin) Limited" },
    { "B8FD32", "Zhejiang ROICX Microelectronics" },
    { "B8FF61", "Apple, Inc." },
    { "B8FF6F", "Shanghai Typrotech Technology Co.Ltd" },
    { "B8FFB3", "MitraStar Technology Corp." },
    { "B8FFFE", "Texas Instruments" },
    { "BC0004", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "BC0200", "Stewart Audio" },
    { "BC024A", "HMD Global Oy" },
    { "BC026E", "Silicon Laboratories" },
    { "BC0358", "Intel Corporate" },
    { "BC03A7", "MFP MICHELIN" },
    { "BC0435", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "BC0543", "AVM GmbH" },
    { "BC062D", "Wacom Co.,Ltd." },
    { "BC071D", "TP-Link Systems Inc." },
    { "BC0866", "Nestle Purina PetCare" },
    { "BC091B", "Intel Corporate" },
    { "BC095C", "FiSens GmbH" },
    { "BC0963", "Apple, Inc." },
    { "BC09EB", "TECNO MOBILE LIMITED" },
    { "BC0DA5", "Texas Instruments" },
    { "BC0EAB", "Samsung Electronics Co.,Ltd" },
    { "BC0F2B", "FORTUNE TECHGROUP CO.,LTD" },
    { "BC0F64", "Intel Corporate" },
    { "BC0F9A", "D-Link International" },
    { "BC0FA7", "Ouster" },
    { "BC0FB7", "sywinkey HongKong Co,. Limited?" },
    { "BC0FF3", "HP Inc." },
    { "BC0FFE", "Juniper Networks" },
    { "BC102F", "SJI Industry Company" },
    { "BC107B", "Samsung Electronics Co.,Ltd" },
    { "BC125E", "Beijing WisVideo INC." },
    { "BC13A8", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "BC1401", "Hitron Technologies. Inc" },
    { "BC1485", "Samsung Electronics Co.,Ltd" },
    { "BC14EF", "ITON Technology Limited" },
    { "BC1541", "Nokia" },
    { "BC15A6", "Taiwan Jantek Electronics,Ltd." },
    { "BC15AC", "Vodafone Italia S.p.A." },
    { "BC1665", "Cisco Systems, Inc" },
    { "BC1695", "zte corporation" },
    { "BC16F5", "Cisco Systems, Inc" },
    { "BC17B8", "Intel Corporate" },
    { "BC1896", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "BC1A67", "YF Technology Co., Ltd" },
    { "BC1AE4", "Huawei Device Co., Ltd." },
    { "BC1C81", "Sichuan iLink Technology Co., Ltd." },
    { "BC1D89", "Motorola Mobility LLC, a Lenovo Company" },
    { "BC1E85", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "BC1FE1", "Ascendent Technology Group" },
    { "BC20A4", "Samsung Electronics Co.,Ltd" },
    { "BC20BA", "Inspur (Shandong) Electronic Information Co., Ltd" },
    { "BC2106", "Adtran Inc" },
    { "BC2228", "D-Link International" },
    { "BC2247", "New H3C Technologies Co., Ltd" },
    { "BC22FB", "RF Industries" },
    { "BC2392", "BYD Precision Manufacture Company Ltd." },
    { "BC2411", "Proxmox Server Solutions GmbH" },
    { "BC25E0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "BC25F0", "3D Display Technologies Co., Ltd." },
    { "BC261D", "HONG KONG TECON TECHNOLOGY" },
    { "BC2643", "Elprotronic Inc." },
    { "BC26A1", "FACTORY FIVE Corporation" },
    { "BC26C7", "Cisco Systems, Inc" },
    { "BC282C", "e-Smart Systems Pvt. Ltd" },
    { "BC2846", "NextBIT Computing Pvt. Ltd." },
    { "BC28D6", "Rowley Associates Limited" },
    { "BC2978", "Prama Hikvision India Private Limited" },
    { "BC2A33", "Quectel Wireless Solutions Co.,Ltd." },
    { "BC2B02", "CHINA DRAGON TECHNOLOGY LIMITED" },
    { "BC2B6B", "Beijing Haier IC Design Co.,Ltd" },
    { "BC2BD7", "Revogi Innovation Co., Ltd." },
    { "BC2C55", "Bear Flag Design, Inc." },
    { "BC2CE6", "Cisco Systems, Inc" },
    { "BC2D98", "ThinGlobal LLC" },
    { "BC2DEF", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "BC2E48", "Commscope" },
    { "BC2EC3", "Shenzhen Tianruixiang Communication Equipment Co.,Ltd" },
    { "BC2EF6", "Huawei Device Co., Ltd." },
    { "BC2F3D", "vivo Mobile Communication Co., Ltd." },
    { "BC305B", "Dell Inc." },
    { "BC307D", "Wistron Neweb Corporation" },
    { "BC307E", "Wistron Neweb Corporation" },
    { "BC30D9", "Arcadyan Corporation" },
    { "BC3198", "IEEE Registration Authority" },
    { "BC31E2", "New H3C Technologies Co., Ltd" },
    { "BC325F", "Zhejiang Dahua Technology Co., Ltd." },
    { "BC32B2", "Samsung Electronics Co.,Ltd" },
    { "BC3329", "Sony Interactive Entertainment Inc." },
    { "BC3340", "Cisco Meraki" },
    { "BC33AC", "Silicon Laboratories" },
    { "BC3400", "IEEE Registration Authority" },
    { "BC34CA", "INOVANCE" },
    { "BC34D6", "Extreme Networks Headquarters" },
    { "BC351E", "Tuya Smart Inc." },
    { "BC35E5", "Hydro Systems Company" },
    { "BC37D3", "Apple, Inc." },
    { "BC3865", "JWCNETWORKS" },
    { "BC3898", "Intel Corporate" },
    { "BC38D2", "Pandachip Limited" },
    { "BC39A6", "CSUN System Technology Co.,LTD" },
    { "BC39D9", "Z-TEC" },
    { "BC3AEA", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "BC3BAF", "Apple, Inc." },
    { "BC3D85", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "BC3E07", "Hitron Technologies. Inc" },
    { "BC3E0B", "Panasonic Connect Co., Ltd." },
    { "BC3E13", "Accordance Systems Inc." },
    { "BC3ECB", "vivo Mobile Communication Co., Ltd." },
    { "BC3F4E", "Great Talent Technology Limited" },
    { "BC3F8F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "BC4100", "CODACO ELECTRONIC s.r.o." },
    { "BC4101", "Shenzhen TINNO Mobile Technology Corp." },
    { "BC41A0", "zte corporation" },
    { "BC428C", "ALPSALPINE CO,.LTD" },
    { "BC4377", "Hang Zhou Huite Technology Co.,ltd." },
    { "BC4434", "Shenzhen TINNO Mobile Technology Corp." },
    { "BC4486", "Samsung Electronics Co.,Ltd" },
    { "BC44B0", "Elastifile" },
    { "BC4529", "zte corporation" },
    { "BC452E", "Knowledge Development for POF S.L." },
    { "BC4548", "Beijing gpthink technology co.,LTD." },
    { "BC455B", "Samsung Electronics Co.,Ltd" },
    { "BC458C", "Shenzhen Topwise Communication Co.,Ltd" },
    { "BC4632", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "BC4699", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "BC4760", "Samsung Electronics Co.,Ltd" },
    { "BC49B2", "SHENZHEN ALONG COMMUNICATION TECH CO., LTD" },
    { "BC4A56", "Cisco Systems, Inc" },
    { "BC4B79", "SensingTek" },
    { "BC4C78", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "BC4CA0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "BC4CC4", "Apple, Inc." },
    { "BC4DFB", "Hitron Technologies. Inc" },
    { "BC4E3C", "CORE STAFF CO., LTD." },
    { "BC4E5D", "ZhongMiao Technology Co., Ltd." },
    { "BC515F", "Nokia Solutions and Networks India Private Limited" },
    { "BC51FE", "Swann communications Pty Ltd" },
    { "BC5274", "Samsung Electronics Co.,Ltd" },
    { "BC52B4", "Nokia" },
    { "BC52B7", "Apple, Inc." },
    { "BC542F", "Intel Corporate" },
    { "BC5436", "Apple, Inc." },
    { "BC5451", "Samsung Electronics Co.,Ltd" },
    { "BC54F9", "Drogoo Technology Co., Ltd." },
    { "BC54FC", "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "BC5729", "Shenzhen KKM Co., Ltd" },
    { "BC5A34", "New H3C Technologies Co., Ltd" },
    { "BC5A56", "Cisco Systems, Inc" },
    { "BC5BD5", "Commscope" },
    { "BC5C17", "Qingdao Intelligent&Precise Electronics Co.,Ltd." },
    { "BC5C4C", "ELECOM CO.,LTD." },
    { "BC5DA3", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "BC5E33", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "BC5E91", "Honor Device Co., Ltd." },
    { "BC5EA1", "PsiKick, Inc." },
    { "BC5FF4", "ASRock Incorporation" },
    { "BC5FF6", "MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "BC6010", "Qingdao Hisense Communications Co.,Ltd." },
    { "BC606B", "Shanghai Baud Data Communication Co.,Ltd." },
    { "BC60A7", "Sony Interactive Entertainment Inc." },
    { "BC6193", "Xiaomi Communications Co Ltd" },
    { "BC620E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "BC629C", "zte corporation" },
    { "BC629F", "Telenet Systems P. Ltd." },
    { "BC62CE", "Netis Technology Co., Ltd." },
    { "BC62D2", "Genexis International B.V." },
    { "BC644B", "Commscope" },
    { "BC64D9", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "BC6641", "IEEE Registration Authority" },
    { "BC66DE", "Shadow Creator Information Technology Co.,Ltd." },
    { "BC671C", "Cisco Systems, Inc" },
    { "BC6778", "Apple, Inc." },
    { "BC6784", "Environics Oy" },
    { "BC69CB", "Panasonic Electric Works Networks Co., Ltd." },
    { "BC6A16", "tdvine" },
    { "BC6A29", "Texas Instruments" },
    { "BC6A2F", "Henge Docks LLC" },
    { "BC6A44", "Commend International GmbH" },
    { "BC6AD1", "Xiaomi Communications Co Ltd" },
    { "BC6B4D", "Nokia" },
    { "BC6BFF", "Guangzhou Shiyuan Electronic Technology Company Limited" },
    { "BC6C21", "Apple, Inc." },
    { "BC6D05", "Dusun Electron Co.,Ltd." },
    { "BC6E64", "Sony Corporation" },
    { "BC6E6D", "EM Microelectronic" },
    { "BC6E76", "Green Energy Options Ltd" },
    { "BC6EE2", "Intel Corporate" },
    { "BC71C1", "XTrillion, Inc." },
    { "BC72B1", "Samsung Electronics Co.,Ltd" },
    { "BC73A4", "ANDA TELECOM PVT LTD" },
    { "BC744B", "Nintendo Co.,Ltd" },
    { "BC74D7", "HangZhou JuRu Technology CO.,LTD" },
    { "BC7536", "ALPSALPINE CO,.LTD" },
    { "BC7574", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "BC7596", "Beijing Broadwit Technology Co., Ltd." },
    { "BC764E", "Rackspace US, Inc." },
    { "BC765E", "Samsung Electronics Co.,Ltd" },
    { "BC7670", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "BC76C5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "BC76F9", "Ubiquoss Inc" },
    { "BC7737", "Intel Corporate" },
    { "BC779F", "SBM Co., Ltd." },
    { "BC79AD", "Samsung Electronics Co.,Ltd" },
    { "BC7ABF", "Samsung Electronics Co.,Ltd" },
    { "BC7B72", "Huawei Device Co., Ltd." },
    { "BC7DD1", "Radio Data Comms" },
    { "BC7E8B", "Samsung Electronics Co.,Ltd" },
    { "BC7EC3", "Zyxel Communications Corporation" },
    { "BC7F7B", "Huawei Device Co., Ltd." },
    { "BC7FA4", "Xiaomi Communications Co Ltd" },
    { "BC804E", "Apple, Inc." },
    { "BC811F", "Ingate Systems" },
    { "BC8199", "BASIC Co.,Ltd." },
    { "BC825D", "MITSUMI ELECTRIC CO.,LTD." },
    { "BC8385", "Microsoft Corporation" },
    { "BC83A7", "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD" },
    { "BC851F", "Samsung Electronics Co.,Ltd" },
    { "BC8529", "Jiangxi Remote lntelligence Technology Co.,Ltd" },
    { "BC8556", "Hon Hai Precision Ind. Co.,Ltd." },
    { "BC85D0", "SKY UK LIMITED" },
    { "BC86A5", "Cambridge Mobile Telematics, Inc." },
    { "BC87FA", "Bose Corporation" },
    { "BC8893", "VILLBAU Ltd." },
    { "BC88C3", "Ningbo Dooya Mechanic & Electronic Technology Co., Ltd" },
    { "BC89A6", "Nintendo Co.,Ltd" },
    { "BC89A7", "Apple, Inc." },
    { "BC8AA3", "NHN Entertainment" },
    { "BC8AE8", "QING DAO HAIER TELECOM CO.,LTD." },
    { "BC8B55", "NPP ELIKS America Inc. DBA T&M Atlantic" },
    { "BC8CCD", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "BC8D0E", "Nokia" },
    { "BC8D1F", "Cisco Systems, Inc" },
    { "BC8D7E", "Silicon Laboratories" },
    { "BC903A", "Robert Bosch GmbH" },
    { "BC91B5", "Infinix mobility limited" },
    { "BC926B", "Apple, Inc." },
    { "BC9307", "Samsung Electronics Co.,Ltd" },
    { "BC9325", "Ningbo Joyson Preh Car Connect Co.,Ltd. " },
    { "BC9424", "TCT mobile ltd" },
    { "BC9680", "SHENZHEN GONGJIN ELECTRONICS CO.,LT" },
    { "BC96E5", "SERCOMM PHILIPPINES INC" },
    { "BC9740", "IEEE Registration Authority" },
    { "BC9789", "Huawei Device Co., Ltd." },
    { "BC97E1", "Broadcom Limited" },
    { "BC9829", "vivo Mobile Communication Co., Ltd." },
    { "BC9889", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "BC98DF", "Motorola Mobility LLC, a Lenovo Company" },
    { "BC9911", "Zyxel Communications Corporation" },
    { "BC9930", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "BC99BC", "FonSee Technology Inc." },
    { "BC9A53", "Huawei Device Co., Ltd." },
    { "BC9A8E", "HUMAX NETWORKS" },
    { "BC9B5E", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "BC9B68", "Vantiva USA LLC" },
    { "BC9C31", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "BC9CC5", "Beijing Huafei Technology Co., Ltd." },
    { "BC9D37", "Telink Micro LLC" },
    { "BC9D42", "SHENZHEN RF-LINK TECHNOLOGY CO.,LTD." },
    { "BC9D4E", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "BC9DA5", "DASCOM Europe GmbH" },
    { "BC9E2C", "China Mobile Group Device Co.,Ltd." },
    { "BC9EBB", "Nintendo Co.,Ltd" },
    { "BC9F58", "Apple, Inc." },
    { "BC9FE4", "Hewlett Packard Enterprise" },
    { "BC9FEF", "Apple, Inc." },
    { "BCA042", "SHANGHAI FLYCO ELECTRICAL APPLIANCE CO.,LTD" },
    { "BCA080", "Samsung Electronics Co.,Ltd" },
    { "BCA0B9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "BCA13A", "VusionGroup" },
    { "BCA231", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "BCA37F", "Rail-Mil Sp. z o.o. Sp. K." },
    { "BCA4E1", "Nabto" },
    { "BCA511", "NETGEAR" },
    { "BCA58B", "Samsung Electronics Co.,Ltd" },
    { "BCA5A9", "Apple, Inc." },
    { "BCA68D", "Continetal Automotive Systems Sibiu" },
    { "BCA8A6", "Intel Corporate" },
    { "BCA920", "Apple, Inc." },
    { "BCA993", "Cambium Networks Limited" },
    { "BCA9D6", "Cyber-Rain, Inc." },
    { "BCAB7C", "TRnP KOREA Co Ltd" },
    { "BCAD28", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "BCAD90", "Kymeta Purchasing" },
    { "BCADAB", "Avaya Inc" },
    { "BCADAE", "AltoBeam Inc." },
    { "BCAEC5", "ASUSTek COMPUTER INC." },
    { "BCAF87", "smartAC.com, Inc." },
    { "BCAF91", "TE Connectivity Sensor Solutions" },
    { "BCB0E7", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "BCB181", "SHARP CORPORATION" },
    { "BCB1D3", "Cisco Meraki" },
    { "BCB1F3", "Samsung Electronics Co.,Ltd" },
    { "BCB22B", "EM-Tech" },
    { "BCB2CC", "Samsung Electronics Co.,Ltd" },
    { "BCB308", "HONGKONG RAGENTEK COMMUNICATION TECHNOLOGY CO.,LIMITED" },
    { "BCB4FD", "NXP Semiconductor (Tianjin) LTD." },
    { "BCB6FB", "P4Q ELECTRONICS, S.L." },
    { "BCB852", "Cybera, Inc." },
    { "BCB863", "Apple, Inc." },
    { "BCB923", "Alta Networks" },
    { "BCBAC2", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "BCBAE1", "AREC Inc." },
    { "BCBB58", "Apple, Inc." },
    { "BCBBC9", "Kellendonk Elektronik GmbH" },
    { "BCBC46", "SKS Welding Systems GmbH" },
    { "BCBD84", "zte corporation" },
    { "BCBD9E", "ITEL MOBILE LIMITED" },
    { "BCBEFB", "ASL Xiamen Technology CO., LTD" },
    { "BCC00F", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "BCC168", "DinBox Sverige AB" },
    { "BCC23A", "Thomson Video Networks" },
    { "BCC2D9", "Private" },
    { "BCC31B", "Kygo Life A" },
    { "BCC342", "Panasonic Communications Co., Ltd." },
    { "BCC427", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "BCC493", "Cisco Systems, Inc" },
    { "BCC61A", "SPECTRA EMBEDDED SYSTEMS" },
    { "BCC6DB", "Nokia Corporation" },
    { "BCC746", "Hon Hai Precision IND.CO.,LTD" },
    { "BCC7DA", "Earda Technologies co Ltd" },
    { "BCC810", "Cisco SPVTG" },
    { "BCCAB5", "Commscope" },
    { "BCCD45", "VOISMART" },
    { "BCCD7F", "Huawei Device Co., Ltd." },
    { "BCCD99", "Intel Corporate" },
    { "BCCE25", "Nintendo Co.,Ltd" },
    { "BCCF4F", "Zyxel Communications Corporation" },
    { "BCCFCC", "HTC Corporation" },
    { "BCD074", "Apple, Inc." },
    { "BCD0EB", "New H3C Technologies Co., Ltd" },
    { "BCD11F", "Samsung Electronics Co.,Ltd" },
    { "BCD165", "Cisco SPVTG" },
    { "BCD177", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "BCD1D3", "Shenzhen TINNO Mobile Technology Corp." },
    { "BCD206", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "BCD22C", "Intel Corporate" },
    { "BCD295", "Cisco Systems, Inc" },
    { "BCD5B6", "d2d technologies" },
    { "BCD5ED", "Sagemcom Broadband SAS" },
    { "BCD713", "Owl Labs" },
    { "BCD767", "BAE Systems Apllied Intelligence" },
    { "BCD7A5", "Hewlett Packard Enterprise" },
    { "BCD7CE", "China Mobile (Hangzhou) Information Technology Co., Ltd." },
    { "BCD7D4", "Roku, Inc" },
    { "BCD940", "ASR Co,.Ltd." },
    { "BCDB09", "Cisco Meraki" },
    { "BCDDC2", "Espressif Inc." },
    { "BCDF58", "Google, Inc." },
    { "BCDFE1", "IEEE Registration Authority" },
    { "BCE001", "Netis Technology Co., Ltd." },
    { "BCE09D", "Eoslink" },
    { "BCE143", "Apple, Inc." },
    { "BCE265", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "BCE59F", "WATERWORLD Technology Co.,LTD" },
    { "BCE63F", "Samsung Electronics Co.,Ltd" },
    { "BCE67C", "Cambium Networks Limited" },
    { "BCE712", "Cisco Systems, Inc" },
    { "BCE767", "Quanzhou TDX Electronics Co., Ltd" },
    { "BCE796", "Wireless CCTV Ltd" },
    { "BCE8FA", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "BCE92F", "HP Inc." },
    { "BCE9E2", "Brocade Communications Systems LLC" },
    { "BCEA2B", "CityCom GmbH" },
    { "BCEA9C", "ITEL MOBILE LIMITED" },
    { "BCEAFA", "Hewlett Packard" },
    { "BCEB5F", "Fujian Beifeng Telecom Technology Co., Ltd." },
    { "BCEC23", "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD" },
    { "BCEC5D", "Apple, Inc." },
    { "BCECA0", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "BCEE7B", "ASUSTek COMPUTER INC." },
    { "BCF105", "Intel Corporate" },
    { "BCF171", "Intel Corporate" },
    { "BCF1F2", "Cisco Systems, Inc" },
    { "BCF212", "Telink Micro LLC" },
    { "BCF292", "PLANTRONICS, INC." },
    { "BCF2AF", "devolo AG" },
    { "BCF2E5", "Powerful Devices " },
    { "BCF310", "Extreme Networks Headquarters" },
    { "BCF45F", "zte corporation" },
    { "BCF499", "Rockwell Automation" },
    { "BCF4D4", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "BCF5AC", "LG Electronics (Mobile Communications)" },
    { "BCF61C", "Geomodeling Wuxi Technology Co. Ltd." },
    { "BCF685", "D-Link International" },
    { "BCF730", "Samsung Electronics Co.,Ltd" },
    { "BCF811", "Xiamen DNAKE Technology Co.,Ltd" },
    { "BCF87E", "Arcadyan Corporation" },
    { "BCF88B", "zte corporation" },
    { "BCF9F2", "TEKO" },
    { "BCFAB8", "Guangzhou Shiyuan Electronic Technology Company Limited" },
    { "BCFAEB", "Cisco Systems, Inc" },
    { "BCFCE7", "ASUSTek COMPUTER INC." },
    { "BCFD0C", "Shenzhen Phaten Tech. LTD" },
    { "BCFE8C", "Altronic, LLC" },
    { "BCFED9", "Apple, Inc." },
    { "BCFF21", "Smart Code(shenzhen)Technology Co.,Ltd" },
    { "BCFF4D", "Espressif Inc." },
    { "BCFF54", "zte corporation" },
    { "BCFFAC", "TOPCON CORPORATION" },
    { "BCFFEB", "Motorola Mobility LLC, a Lenovo Company" },
    { "C0028D", "WINSTAR Display CO.,Ltd" },
    { "C00380", "Juniper Networks" },
    { "C005C2", "Commscope" },
    { "C0060C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C006C3", "TP-Link Systems Inc" },
    { "C0074A", "Brita GmbH" },
    { "C00925", "FN-LINK TECHNOLOGY Ltd." },
    { "C00D7E", "Additech, Inc." },
    { "C010B1", "HMD Global Oy" },
    { "C01173", "Samsung Electronics Co.,Ltd" },
    { "C011A6", "Fort-Telecom ltd." },
    { "C01242", "Alpha Security Products" },
    { "C0132B", "Sichuan Changhong Electric Ltd." },
    { "C0143D", "Hon Hai Precision Ind. Co.,Ltd." },
    { "C014B8", "Nokia" },
    { "C014FE", "Cisco Systems, Inc" },
    { "C0151B", "Sony Interactive Entertainment Inc." },
    { "C01692", "China Mobile Group Device Co.,Ltd." },
    { "C01693", "Xiaomi Communications Co Ltd" },
    { "C0174D", "Samsung Electronics Co.,Ltd" },
    { "C01754", "Apple, Inc." },
    { "C01803", "HP Inc." },
    { "C01850", "Quanta Computer Inc." },
    { "C01885", "Hon Hai Precision Ind. Co.,Ltd." },
    { "C01944", "Juniper Networks" },
    { "C01ADA", "Apple, Inc." },
    { "C01B23", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "C01C30", "Shenzhen WIFI-3L Technology Co.,Ltd" },
    { "C01C6A", "Google, Inc." },
    { "C01E9B", "Pixavi AS" },
    { "C0210D", "SHENZHEN RF-LINK TECHNOLOGY CO.,LTD." },
    { "C02242", "Chauvet" },
    { "C02250", "Koss Corporation" },
    { "C022F1", "IEEE Registration Authority" },
    { "C0238D", "Samsung Electronics Co.,Ltd" },
    { "C02506", "AVM GmbH" },
    { "C0252F", "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "C0255C", "Cisco Systems, Inc" },
    { "C02567", "Nexxt Solutions" },
    { "C025A2", "NEC Platforms, Ltd." },
    { "C025A5", "Dell Inc." },
    { "C025E9", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "C027B9", "Beijing National Railway Research & Design Institute of Signal & Communication Co., Ltd." },
    { "C0280B", "Honor Device Co., Ltd." },
    { "C0288D", "Logitech, Inc" },
    { "C02973", "Audyssey Laboratories Inc." },
    { "C029F3", "XySystem" },
    { "C02B31", "Phytium Technology Co.,Ltd." },
    { "C02B56", "CANDID OPTRONIX PRIVATE LIMITED" },
    { "C02BFC", "iNES. applied informatics GmbH" },
    { "C02C17", "Cisco Systems, Inc" },
    { "C02C5C", "Apple, Inc." },
    { "C02C7A", "Shenzhen Horn Audio Co.,Ltd." },
    { "C02CED", "Silicon Laboratories" },
    { "C02D2E", "China Mobile Group Device Co.,Ltd." },
    { "C02DEE", "Cuff" },
    { "C02E25", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "C02E26", "iRhythm Technologies, Inc." },
    { "C02FF1", "Volta Networks" },
    { "C0335E", "Microsoft" },
    { "C03379", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C033DA", "Shenzhen JRUN Technologies CO., LTD" },
    { "C034B4", "Gigastone Corporation" },
    { "C03532", "Liteon Technology Corporation" },
    { "C03580", "A&R TECH" },
    { "C035BD", "Velocytech Aps" },
    { "C035C5", "Prosoft Systems LTD" },
    { "C03653", "eero inc." },
    { "C03656", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "C03896", "Hon Hai Precision Ind. Co.,Ltd." },
    { "C038F9", "Nokia Danmark A/S" },
    { "C03937", "GREE ELECTRIC APPLIANCES, INC. OF ZHUHAI" },
    { "C0395A", "Zhejiang Dahua Technology Co., Ltd." },
    { "C03B8F", "Minicom Digital Signage" },
    { "C03C04", "Sagemcom Broadband SAS" },
    { "C03C59", "Intel Corporate" },
    { "C03D03", "Samsung Electronics Co.,Ltd" },
    { "C03D46", "Shanghai Sango Network Technology Co.,Ltd" },
    { "C03DD9", "MitraStar Technology Corp." },
    { "C03E0F", "SKY UK LIMITED" },
    { "C03E50", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C03EBA", "Dell Inc." },
    { "C03F0E", "NETGEAR" },
    { "C03F2A", "Biscotti, Inc." },
    { "C03FBB", "Zhongshan Zhiniu Electronics Co.,Ltd." },
    { "C03FD5", "Elitegroup Computer Systems Co.,Ltd." },
    { "C03FDD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C04004", "Medicaroid Corporation" },
    { "C04121", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "C041F6", "LG ELECTRONICS INC" },
    { "C042D0", "Juniper Networks" },
    { "C04301", "Epec Oy" },
    { "C04327", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "C04442", "Apple, Inc." },
    { "C044E3", "Shenzhen Sinkna Electronics Co., LTD" },
    { "C0470E", "Dell Inc." },
    { "C04754", "vivo Mobile Communication Co., Ltd." },
    { "C0482F", "IEEE Registration Authority" },
    { "C04884", "Sigma Bilisim Sist. Tekn. Elk. Enj. ve San. D??. Tic. Ltd. ?ti." },
    { "C048E6", "Samsung Electronics Co.,Ltd" },
    { "C048FB", "Shenzhen JingHanDa Electronics Co.Ltd" },
    { "C0493D", "MAITRISE TECHNOLOGIQUE" },
    { "C04943", "zte corporation" },
    { "C049BD", "EM Microelectronic" },
    { "C049EF", "Espressif Inc." },
    { "C04A00", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "C04A09", "Zhejiang Everbright Communication Equip. Co,. Ltd" },
    { "C04A0E", "Texas Instruments" },
    { "C04B13", "WonderSound Technology Co., Ltd" },
    { "C04DF7", "SERELEC" },
    { "C04E30", "Espressif Inc." },
    { "C04E8A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C05064", "SHENNAN CIRCUITS CO.,LTD" },
    { "C0515C", "zte corporation" },
    { "C0517E", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "C051F3", "CIG SHANGHAI CO LTD" },
    { "C05234", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C05336", "Beijing National Railway Research & Design Institute of Signal & Communication Group Co..Ltd." },
    { "C0544D", "Nokia Shanghai Bell Co., Ltd." },
    { "C0555C", "Impulse Labs" },
    { "C05627", "Belkin International Inc." },
    { "C056E3", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "C057BC", "Avaya Inc" },
    { "C058A7", "Pico Systems Co., Ltd." },
    { "C05B44", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "C05D39", "Jiangsu Huitong Group Co.,Ltd." },
    { "C05D89", "Espressif Inc." },
    { "C05E6F", "V. Stonkaus firma Kodinis Raktas" },
    { "C05E79", "SHENZHEN HUAXUN ARK TECHNOLOGIES CO.,LTD" },
    { "C05F87", "Legrand INTELLIGENT ELECTRICAL(HUIZHOU)CO.,LTD." },
    { "C06118", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "C0613D", "BioIntelliSense, Inc." },
    { "C0619A", "IEEE Registration Authority" },
    { "C0626B", "Cisco Systems, Inc" },
    { "C06369", "BINXIN TECHNOLOGY(ZHEJIANG) LTD." },
    { "C06380", "Texas Instruments" },
    { "C06394", "Apple, Inc." },
    { "C064C6", "Nokia Corporation" },
    { "C064E4", "Cisco Systems, Inc" },
    { "C06599", "Samsung Electronics Co.,Ltd" },
    { "C067AF", "Cisco Systems, Inc" },
    { "C068CC", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "C06911", "Arista Networks" },
    { "C06B55", "Motorola Mobility LLC, a Lenovo Company" },
    { "C06C0C", "Apple, Inc." },
    { "C06C0F", "Dobbs Stanford" },
    { "C06C6D", "MagneMotion, Inc." },
    { "C06D1A", "Tianjin Henxinhuifeng Technology Co.,Ltd." },
    { "C06DED", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "C06F98", "eero inc." },
    { "C07009", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C071AA", "ShenZhen OnMicro Electronics Co.,Ltd." },
    { "C07415", "IntelPro Inc." },
    { "C0742B", "SHENZHEN XUNLONG SOFTWARE CO.,LIMITED" },
    { "C074AD", "Grandstream Networks, Inc." },
    { "C07831", "Huawei Device Co., Ltd." },
    { "C07878", "FLEXTRONICS MANUFACTURING(ZHUHAI)CO.,LTD." },
    { "C07982", "TCL King Electrical Appliances(Huizhou)Co.,Ltd" },
    { "C07AD6", "Samsung Electronics Co.,Ltd" },
    { "C07BBC", "Cisco Systems, Inc" },
    { "C07C90", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "C07CD1", "PEGATRON CORPORATION" },
    { "C07E40", "SHENZHEN XDK COMMUNICATION EQUIPMENT CO.,LTD" },
    { "C08135", "Ningbo Forfan technology Co., LTD" },
    { "C08170", "Effigis GeoSolutions" },
    { "C0830A", "2Wire Inc" },
    { "C08359", "IEEE Registration Authority" },
    { "C083C9", "Huawei Device Co., Ltd." },
    { "C0847A", "Apple, Inc." },
    { "C0847D", "AMPAK Technology, Inc." },
    { "C08488", "Finis Inc" },
    { "C084E0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C084FF", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "C0854C", "Ragentek Technology Group" },
    { "C086B3", "Shenzhen Voxtech Co., Ltd." },
    { "C08706", "Shenzhen Qianfenyi Intelligent Technology Co.,LTD" },
    { "C087EB", "Samsung Electronics Co.,Ltd" },
    { "C08840", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "C0885B", "SnD Tech Co., Ltd." },
    { "C0886D", "Securosys SA" },
    { "C08997", "Samsung Electronics Co.,Ltd" },
    { "C089AB", "Commscope" },
    { "C08A60", "AltoBeam Inc." },
    { "C08ACD", "Guangzhou Shiyuan Electronic Technology Company Limited" },
    { "C08ADE", "Ruckus Wireless" },
    { "C08B05", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C08B2A", "Cisco Systems, Inc" },
    { "C08B6F", "S I Sistemas Inteligentes Eletrônicos Ltda" },
    { "C08C60", "Cisco Systems, Inc" },
    { "C08C71", "Motorola Mobility LLC, a Lenovo Company" },
    { "C08D51", "Amazon Technologies Inc." },
    { "C08F20", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "C09120", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "C09132", "Patriot Memory" },
    { "C09134", "ProCurve Networking by HP" },
    { "C091B9", "Amazon Technologies Inc." },
    { "C09296", "zte corporation" },
    { "C09435", "Commscope" },
    { "C094AD", "zte corporation" },
    { "C0956D", "Apple, Inc." },
    { "C09573", "AIxLink" },
    { "C095CF", "Amazon Technologies Inc." },
    { "C095DA", "NXP India Private Limited" },
    { "C096A4", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "C09727", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "C09879", "Acer Inc." },
    { "C098DA", "China Mobile IOT Company Limited" },
    { "C098E5", "University of Michigan" },
    { "C09A71", "XIAMEN MEITU MOBILE TECHNOLOGY CO.LTD" },
    { "C09AD0", "Apple, Inc." },
    { "C09B63", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C09B9E", "Silicon Laboratories" },
    { "C09BF4", "IEEE Registration Authority" },
    { "C09C04", "Shaanxi GuoLian Digital TV Technology Co.,Ltd." },
    { "C09C92", "COBY" },
    { "C09D26", "Topicon HK Lmd." },
    { "C09F05", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "C09F42", "Apple, Inc." },
    { "C09F51", "SERNET (SUZHOU) TECHNOLOGIES CORPORATION" },
    { "C09FE1", "zte corporation" },
    { "C0A00D", "Commscope" },
    { "C0A0BB", "D-Link International" },
    { "C0A0C7", "FAIRFIELD INDUSTRIES" },
    { "C0A0DE", "Multi Touch Oy" },
    { "C0A0E2", "Eden Innovations" },
    { "C0A1A2", "MarqMetrix" },
    { "C0A26D", "Abbott Point of Care" },
    { "C0A364", "3D Systems Massachusetts" },
    { "C0A36E", "SKY UK LIMITED" },
    { "C0A39E", "EarthCam, Inc." },
    { "C0A3C7", "Telink Micro LLC" },
    { "C0A476", "Ruijie Networks Co.,LTD" },
    { "C0A53E", "Apple, Inc." },
    { "C0A5DD", "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "C0A5E8", "Intel Corporate" },
    { "C0A600", "Apple, Inc." },
    { "C0A66D", "Inspur Group Co., Ltd." },
    { "C0A810", "Intel Corporate" },
    { "C0A8F0", "Adamson Systems Engineering" },
    { "C0A938", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C0AA68", "OSASI Technos Inc." },
    { "C0AB2B", "Huawei Device Co., Ltd." },
    { "C0AC54", "Sagemcom Broadband SAS" },
    { "C0AD97", "TECNO MOBILE LIMITED" },
    { "C0AEFD", "Shenzhen HC-WLAN Technology Co.,Ltd" },
    { "C0AFF2", "Dyson Limited" },
    { "C0B101", "zte corporation" },
    { "C0B22F", "Apple, Inc." },
    { "C0B339", "Comigo Ltd." },
    { "C0B357", "Yoshiki Electronics Industry Ltd." },
    { "C0B3C8", "LLC NTC Rotek" },
    { "C0B47D", "Huawei Device Co., Ltd." },
    { "C0B5CD", "Huawei Device Co., Ltd." },
    { "C0B5D7", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "C0B658", "Apple, Inc." },
    { "C0B6F9", "Intel Corporate" },
    { "C0B713", "Beijing Xiaoyuer Technology Co. Ltd." },
    { "C0B883", "Intel Corporate" },
    { "C0B8B1", "BitBox Ltd" },
    { "C0B8E6", "Ruijie Networks Co.,LTD" },
    { "C0BA1F", "Private" },
    { "C0BAE6", "Zenitel GB Ltd" },
    { "C0BC9A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C0BD42", "ZPA Smart Energy a.s." },
    { "C0BDC8", "Samsung Electronics Co.,Ltd" },
    { "C0BDD1", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "C0BFA7", "Juniper Networks" },
    { "C0BFAC", "Huawei Device Co., Ltd." },
    { "C0BFBE", "AzureWave Technology Inc." },
    { "C0BFC0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C0C170", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "C0C1C0", "Cisco-Linksys, LLC" },
    { "C0C3B6", "Automatic Systems" },
    { "C0C4F9", "Qisda Corporation" },
    { "C0C520", "Ruckus Wireless" },
    { "C0C522", "Commscope" },
    { "C0C569", "SHANGHAI LYNUC CNC TECHNOLOGY CO.,LTD" },
    { "C0C687", "Cisco SPVTG" },
    { "C0C70A", "Ruckus Wireless" },
    { "C0C7DB", "Apple, Inc." },
    { "C0C946", "MITSUYA LABORATORIES INC." },
    { "C0C976", "Shenzhen TINNO Mobile Technology Corp." },
    { "C0C989", "Edgecore Americas Networking Corporation" },
    { "C0C9E3", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "C0CB38", "Hon Hai Precision Ind. Co.,Ltd." },
    { "C0CBF1", "Mobiwire Mobiles (NingBo) Co., LTD" },
    { "C0CC42", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "C0CCF8", "Apple, Inc." },
    { "C0CDD6", "Espressif Inc." },
    { "C0CECD", "Apple, Inc." },
    { "C0CFA3", "Creative Electronics & Software, Inc." },
    { "C0D012", "Apple, Inc." },
    { "C0D026", "Huawei Device Co., Ltd." },
    { "C0D044", "Sagemcom Broadband SAS" },
    { "C0D063", "EM Microelectronic" },
    { "C0D0FF", "China Mobile IOT Company Limited" },
    { "C0D193", "Huawei Device Co., Ltd." },
    { "C0D2DD", "Samsung Electronics Co.,Ltd" },
    { "C0D2F3", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "C0D391", "IEEE Registration Authority" },
    { "C0D3C0", "Samsung Electronics Co.,Ltd" },
    { "C0D46B", "Huawei Device Co., Ltd." },
    { "C0D5E2", "Samsung Electronics Co.,Ltd" },
    { "C0D60A", "Texas Instruments" },
    { "C0D682", "Arista Networks" },
    { "C0D6D5", "Microsoft Corporation" },
    { "C0D7AA", "Arcadyan Corporation" },
    { "C0D834", "xvtec ltd" },
    { "C0D941", "Shenzhen VMAX Software Co., Ltd." },
    { "C0D962", "ASKEY COMPUTER CORP" },
    { "C0D9F7", "ShanDong Domor Intelligent S&T CO.,Ltd" },
    { "C0DA5E", "Huawei Device Co., Ltd." },
    { "C0DA74", "Hangzhou Sunyard Technology Co., Ltd." },
    { "C0DC6A", "Qingdao Eastsoft Communication Technology Co.,LTD" },
    { "C0DCAB", "LG Innotek" },
    { "C0DCD7", "Huawei Device Co., Ltd." },
    { "C0DCDA", "Samsung Electronics Co.,Ltd" },
    { "C0DD8A", "Meta Platforms, Inc." },
    { "C0DF77", "Conrad Electronic SE" },
    { "C0DFED", "Juniper Networks" },
    { "C0E018", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C0E01C", "IoT Security Group, SL" },
    { "C0E1BE", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C0E350", "CHINA DRAGON TECHNOLOGY LIMITED" },
    { "C0E3A0", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "C0E3FB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C0E422", "Texas Instruments" },
    { "C0E42D", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "C0E434", "AzureWave Technology Inc." },
    { "C0E54E", "ARIES Embedded GmbH" },
    { "C0E5DA", "Qingdao Intelligent&Precise Electronics Co.,Ltd." },
    { "C0E7BF", "Sichuan AI-Link Technology Co., Ltd." },
    { "C0E862", "Apple, Inc." },
    { "C0E911", "RealNetworks" },
    { "C0EAC3", "IEEE Registration Authority" },
    { "C0EAE4", "Sonicwall" },
    { "C0EDE5", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "C0EE40", "Laird Connectivity" },
    { "C0EEB5", "Enice Network." },
    { "C0EEFB", "OnePlus Tech (Shenzhen) Ltd" },
    { "C0F1C4", "Pacidal Corporation Ltd." },
    { "C0F2FB", "Apple, Inc." },
    { "C0F4E6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C0F535", "AMPAK Technology,Inc." },
    { "C0F636", "Hangzhou Kuaiyue Technologies, Ltd." },
    { "C0F6C2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C0F6EC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C0F79D", "Powercode" },
    { "C0F827", "Rapidmax Technology Corporation" },
    { "C0F853", "Tuya Smart Inc." },
    { "C0F87F", "Cisco Systems, Inc" },
    { "C0F8DA", "Hon Hai Precision Ind. Co.,Ltd." },
    { "C0F945", "Toshiba Toko Meter Systems Co., LTD." },
    { "C0F991", "GME Standard Communications P/L" },
    { "C0F9B0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C0F9D2", "arkona technologies GmbH" },
    { "C0FBC1", "ITEL MOBILE LIMITED" },
    { "C0FBF9", "IEEE Registration Authority" },
    { "C0FD84", "zte corporation" },
    { "C0FFA8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C0FFD4", "NETGEAR" },
    { "C40006", "Lipi Data Systems Ltd." },
    { "C40049", "Kamama" },
    { "C400AD", "Advantech Technology (CHINA) Co., Ltd." },
    { "C400B5", "HongKong Tenry Technology Co., Ltd. " },
    { "C40142", "MaxMedia Technology Limited" },
    { "C4017C", "Ruckus Wireless" },
    { "C401B1", "SeekTech INC" },
    { "C401CE", "PRESITION (2000) CO., LTD." },
    { "C402E1", "Khwahish Technologies Private Limited" },
    { "C403A8", "Intel Corporate" },
    { "C40415", "NETGEAR" },
    { "C4047B", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "C404D8", "Aviva Links Inc." },
    { "C40528", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C40683", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C4072F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C40778", "New H3C Technologies Co., Ltd" },
    { "C40826", "HISENSE VISUAL TECHNOLOGY CO.,LTD" },
    { "C4084A", "Nokia" },
    { "C40880", "Shenzhen UTEPO Tech Co., Ltd." },
    { "C40898", "Dropbeats Technology Co., Ltd." },
    { "C40938", "FUJIAN STAR-NET COMMUNICATION CO.,LTD" },
    { "C409B7", "Juniper Networks" },
    { "C40ACB", "Cisco Systems, Inc" },
    { "C40B31", "Apple, Inc." },
    { "C40BCB", "Xiaomi Communications Co Ltd" },
    { "C40D96", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C40E45", "ACK Networks,Inc." },
    { "C40F08", "Intel Corporate" },
    { "C40F09", "Hermes electronic GmbH" },
    { "C40FA6", "SERNET (SUZHOU) TECHNOLOGIES CORPORATION" },
    { "C4108A", "Ruckus Wireless" },
    { "C411E0", "Bull Group Co., Ltd" },
    { "C41234", "Apple, Inc." },
    { "C412EC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C412F5", "D-Link International" },
    { "C41375", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "C413E2", "Extreme Networks Headquarters" },
    { "C41411", "Apple, Inc." },
    { "C4143C", "Cisco Systems, Inc" },
    { "C414A2", "Cisco Meraki" },
    { "C41688", "Huawei Device Co., Ltd." },
    { "C416C8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C416FA", "Prysm Inc" },
    { "C4170E", "Huawei Device Co., Ltd." },
    { "C417FE", "Hon Hai Precision Ind. Co.,Ltd." },
    { "C418E9", "Samsung Electronics Co.,Ltd" },
    { "C418FC", "Cisco Systems, Inc" },
    { "C4198B", "Dominion Voting Systems Corporation" },
    { "C419D1", "Telink Semiconductor (Shanghai) Co., Ltd." },
    { "C419EC", "Qualisys AB" },
    { "C41C07", "Samsung Electronics Co.,Ltd" },
    { "C41C9C", "JiQiDao" },
    { "C41CFF", "Vizio, Inc" },
    { "C41ECE", "HMI Sources Ltd." },
    { "C421B9", "zte corporation" },
    { "C421C8", "KYOCERA CORPORATION " },
    { "C4224E", "Telink Micro LLC" },
    { "C42360", "Intel Corporate" },
    { "C4237A", "WhizNets Inc." },
    { "C423A2", "PT. Emsonic Indonesia" },
    { "C4242E", "Galvanic Applied Sciences Inc" },
    { "C42456", "Palo Alto Networks" },
    { "C42628", "Airo Wireless" },
    { "C42728", "zte corporation" },
    { "C4278C", "Huawei Device Co., Ltd." },
    { "C42795", "Vantiva USA LLC" },
    { "C4282D", "Embedded Intellect Pty Ltd" },
    { "C4291D", "KLEMSAN ELEKTRIK ELEKTRONIK SAN.VE TIC.AS." },
    { "C42996", "Signify B.V." },
    { "C42AD0", "Apple, Inc." },
    { "C42B44", "Huawei Device Co., Ltd." },
    { "C42C03", "Apple, Inc." },
    { "C42C4F", "Qingdao Hisense Mobile Communication Technology Co,Ltd" },
    { "C42F90", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "C43018", "MCS Logic Inc." },
    { "C430CA", "SD Biosensor" },
    { "C432D1", "Farlink Technology Limited" },
    { "C43306", "China Mobile Group Device Co.,Ltd." },
    { "C43396", "Dongguan Hele Electronics Co., Ltd." },
    { "C4345B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C4346B", "Hewlett Packard" },
    { "C435D9", "Apple, Inc." },
    { "C43655", "Shenzhen Fenglian Technology Co., Ltd." },
    { "C4366C", "LG Innotek" },
    { "C436C0", "BUFFALO.INC" },
    { "C436DA", "Rusteletech Ltd." },
    { "C43772", "Virtuozzo International GmbH" },
    { "C43875", "Sonos, Inc." },
    { "C438D3", "TAGATEC CO.,LTD" },
    { "C4393A", "SMC Networks Inc" },
    { "C43960", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "C43A35", "FN-LINK TECHNOLOGY LIMITED" },
    { "C43A9F", "Siconix Inc." },
    { "C43ABE", "Sony Corporation" },
    { "C43C3C", "CYBELEC SA" },
    { "C43CB0", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "C43CEA", "BUFFALO.INC" },
    { "C43D1A", "Intel Corporate" },
    { "C43DC7", "NETGEAR" },
    { "C43EAB", "Huawei Device Co., Ltd." },
    { "C44044", "RackTop Systems Inc." },
    { "C440F6", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "C4411E", "Belkin International Inc." },
    { "C44137", "Quectel Wireless Solutions Co.,Ltd." },
    { "C44202", "Samsung Electronics Co.,Ltd" },
    { "C44268", "CRESTRON ELECTRONICS, INC." },
    { "C4438F", "LG Electronics (Mobile Communications)" },
    { "C4447D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C444A0", "Cisco Systems, Inc" },
    { "C44567", "SAMBON PRECISON and ELECTRONICS" },
    { "C445EC", "Shanghai Yali Electron Co.,LTD" },
    { "C44606", "Cisco Systems, Inc" },
    { "C44619", "Hon Hai Precision Ind. Co.,Ltd." },
    { "C4473F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C4474E", "Intel Corporate" },
    { "C44838", "Satcom Direct, Inc." },
    { "C448FA", "Taicang T&W Electronics" },
    { "C4493E", "Motorola Mobility LLC, a Lenovo Company" },
    { "C449BB", "MITSUMI ELECTRIC CO.,LTD." },
    { "C44AD0", "FIREFLIES SYSTEMS" },
    { "C44B44", "Omniprint Inc." },
    { "C44BD1", "Wallys Communications Teachnologies Co.,Ltd." },
    { "C44D84", "Cisco Systems, Inc" },
    { "C44E1F", "BlueN" },
    { "C44EAC", "Shenzhen Shiningworth Technology Co., Ltd." },
    { "C44F33", "Espressif Inc." },
    { "C44F5F", "Huawei Device Co., Ltd." },
    { "C44F96", "Alps Alpine" },
    { "C44FD5", "Vantiva - Connected Home" },
    { "C45006", "Samsung Electronics Co.,Ltd" },
    { "C4509C", "Vantiva - Connected Home" },
    { "C4518D", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "C4524F", "Apple, Inc." },
    { "C45379", "Micronview Limited Liability Company" },
    { "C45444", "Quanta Computer Inc." },
    { "C455A6", "Cadac Holdings Ltd" },
    { "C455C2", "Bach-Simpson" },
    { "C45600", "Galleon Embedded Computing" },
    { "C456FE", "Lava International Ltd." },
    { "C4571F", "June Life Inc" },
    { "C45746", "Texas Instruments" },
    { "C4576E", "Samsung Electronics Co.,Ltd" },
    { "C45781", "Wingtech Group (HongKong) Limited" },
    { "C457CD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C458C2", "Shenzhen TATFOOK Technology Co., Ltd." },
    { "C45976", "Fugoo Coorporation" },
    { "C45A86", "Huawei Device Co., Ltd." },
    { "C45AB1", "Dell Inc." },
    { "C45BBE", "Espressif Inc." },
    { "C45BF7", "ants" },
    { "C45D83", "Samsung Electronics Co.,Ltd" },
    { "C45DD8", "HDMI Forum" },
    { "C45E5C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C4600A", "Huaqin Technology Co.LTD" },
    { "C46026", "SKY UK LIMITED" },
    { "C46044", "Everex Electronics Limited" },
    { "C4618B", "Apple, Inc." },
    { "C461C7", "Microsoft Corporation" },
    { "C46237", "sunweit industrial limited" },
    { "C4626B", "ZPT Vigantice" },
    { "C462EA", "Samsung Electronics Co.,Ltd" },
    { "C46354", "U-Raku, Inc." },
    { "C463C4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C463FB", "Neatframe AS" },
    { "C46413", "Cisco Systems, Inc" },
    { "C464B7", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "C464E3", "Texas Instruments" },
    { "C464F2", "Infinix mobility limited" },
    { "C46516", "Hewlett Packard" },
    { "C46699", "vivo Mobile Communication Co., Ltd." },
    { "C4678B", "Alphabet Capital Sdn Bhd" },
    { "C467B5", "Libratone A/S" },
    { "C467D1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C468D0", "VTech Telecommunications Ltd." },
    { "C4693E", "Turbulence Design Inc." },
    { "C469F0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C46AB7", "Xiaomi Communications Co Ltd" },
    { "C46BB4", "myIDkey" },
    { "C46DF1", "DataGravity" },
    { "C46E1F", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "C46E33", "Zhong Ge Smart Technology Co., Ltd." },
    { "C46E7B", "SHENZHEN RF-LINK TECHNOLOGY CO.,LTD." },
    { "C4700B", "GUANGZHOU CHIP TECHNOLOGIES CO.,LTD" },
    { "C470AB", "Ruijie Networks Co.,LTD" },
    { "C470BD", "Mellanox Technologies, Inc." },
    { "C4710F", "Xiaomi Communications Co Ltd" },
    { "C47130", "Fon Technology S.L." },
    { "C47154", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "C471FE", "Cisco Systems, Inc" },
    { "C47295", "Cisco Systems, Inc" },
    { "C4731E", "Samsung Electronics Co.,Ltd" },
    { "C4741E", "zte corporation" },
    { "C47469", "BT9" },
    { "C474F8", "Hot Pepper, Inc." },
    { "C475AB", "Intel Corporate" },
    { "C475EA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C47764", "Samsung Electronics Co.,Ltd" },
    { "C477AB", "Beijing ASU Tech Co.,Ltd" },
    { "C477AF", "Advanced Digital Broadcast SA" },
    { "C478A2", "Huawei Device Co., Ltd." },
    { "C47905", "Zhejiang Uniview Technologies Co.,Ltd." },
    { "C47981", "Ehya LTD" },
    { "C4799F", "Haiguang Smart Device Co.,Ltd." },
    { "C47B2F", "Beijing JoinHope Image Technology Ltd." },
    { "C47B80", "Protempis, LLC" },
    { "C47BA3", "NAVIS Inc." },
    { "C47C8D", "IEEE Registration Authority" },
    { "C47D46", "FUJITSU LIMITED" },
    { "C47D4F", "Cisco Systems, Inc" },
    { "C47D9F", "Samsung Electronics Co.,Ltd" },
    { "C47DA8", "NXP Semiconductor (Tianjin) LTD." },
    { "C47DCC", "Zebra Technologies Inc" },
    { "C47DFE", "A.N. Solutions GmbH" },
    { "C47EE0", "Cisco Systems, Inc" },
    { "C47F51", "Inventek Systems" },
    { "C48025", "Huawei Device Co., Ltd." },
    { "C4808A", "Cloud Diagnostics Canada ULC" },
    { "C4823F", "Fujian Newland Auto-ID Tech. Co,.Ltd." },
    { "C4824E", "Changzhou Uchip Electronics Co., LTD." },
    { "C482E1", "Tuya Smart Inc." },
    { "C4836F", "Ciena Corporation" },
    { "C48372", "IEEE Registration Authority" },
    { "C48466", "Apple, Inc." },
    { "C484FC", "Apple, Inc." },
    { "C48508", "Intel Corporate" },
    { "C4864F", "Beijing BitIntelligence Information Technology Co. Ltd." },
    { "C486E9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C488E5", "Samsung Electronics Co.,Ltd" },
    { "C489ED", "Solid Optics EU N.V." },
    { "C48A5A", "JFCONTROL" },
    { "C48B66", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "C48BA3", "Cisco Meraki" },
    { "C48E8F", "Hon Hai Precision Ind. Co.,Ltd." },
    { "C48F07", "Shenzhen Yihao Hulian Science and Technology Co., Ltd." },
    { "C48FC1", "DEEPTRACK S.L.U." },
    { "C4910C", "Apple, Inc." },
    { "C4913A", "Shenzhen Sanland Electronic Co., ltd." },
    { "C491CF", "Luxul " },
    { "C4924C", "KEISOKUKI CENTER CO.,LTD." },
    { "C49300", "8Devices" },
    { "C49313", "100fio networks technology llc" },
    { "C49380", "Speedytel technology" },
    { "C493BB", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "C493D9", "Samsung Electronics Co.,Ltd" },
    { "C49500", "Amazon Technologies Inc." },
    { "C4954D", "IEEE Registration Authority" },
    { "C4955F", "Anhui Saida Technology Limited Liability Company" },
    { "C495A2", "SHENZHEN WEIJIU INDUSTRY AND TRADE DEVELOPMENT CO., LTD" },
    { "C49805", "Minieum Networks, Inc" },
    { "C4985C", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "C49878", "SHANGHAI MOAAN INTELLIGENT TECHNOLOGY CO.,LTD" },
    { "C49880", "Apple, Inc." },
    { "C49886", "Qorvo International Pte. Ltd." },
    { "C49894", "IEEE Registration Authority" },
    { "C49A02", "LG Electronics (Mobile Communications)" },
    { "C49A31", "Zyxel Communications Corporation" },
    { "C49A89", "Suzhou K-Hiragawa Electronic Technology Co.,Ltd" },
    { "C49D08", "Huawei Device Co., Ltd." },
    { "C49DED", "Microsoft Corporation" },
    { "C49E41", "G24 Power Limited" },
    { "C49F4C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C49FF3", "Mciao Technologies, Inc." },
    { "C4A052", "Motorola Mobility LLC, a Lenovo Company" },
    { "C4A10E", "IEEE Registration Authority" },
    { "C4A151", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "C4A1AE", "Huawei Device Co., Ltd." },
    { "C4A366", "zte corporation" },
    { "C4A402", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C4A451", "TECNO MOBILE LIMITED" },
    { "C4A559", "IEEE Registration Authority" },
    { "C4A64E", "Quectel Wireless Solutions Co.,Ltd." },
    { "C4A72B", "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD" },
    { "C4A816", "eero inc." },
    { "C4A81D", "D-Link International" },
    { "C4A9B8", "XIAMENSHI C-CHIP TECHNOLOGY CO.,LTD" },
    { "C4AA99", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C4AAA1", "SUMMIT DEVELOPMENT, spol.s r.o." },
    { "C4AAC4", "Zhejiang Dahua Technology Co., Ltd." },
    { "C4AB4D", "Cisco Systems, Inc" },
    { "C4ABB2", "vivo Mobile Communication Co., Ltd." },
    { "C4AC59", "Murata Manufacturing Co., Ltd." },
    { "C4ACAA", "Apple, Inc." },
    { "C4AD21", "MEDIAEDGE Corporation" },
    { "C4AD34", "Routerboard.com" },
    { "C4ADF1", "GOPEACE Inc." },
    { "C4AE12", "Samsung Electronics Co.,Ltd" },
    { "C4B1D9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C4B239", "Cisco Systems, Inc" },
    { "C4B25B", "Ruijie Networks Co.,LTD" },
    { "C4B301", "Apple, Inc." },
    { "C4B349", "Apple, Inc." },
    { "C4B36A", "Cisco Systems, Inc" },
    { "C4B512", "General Electric Digital Energy" },
    { "C4B691", "Angel Robotics" },
    { "C4B757", "ALPSALPINE CO,.LTD" },
    { "C4B8B4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C4B9CD", "Cisco Systems, Inc" },
    { "C4BA99", "I+ME Actia Informatik und Mikro-Elektronik GmbH" },
    { "C4BAA3", "Beijing Winicssec Technologies Co., Ltd." },
    { "C4BB03", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C4BB4C", "Zebra Information Tech Co. Ltd" },
    { "C4BBEA", "Pakedge Device and Software Inc" },
    { "C4BCD7", "New Ryatek" },
    { "C4BD6A", "SKF GmbH" },
    { "C4BD8D", "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD" },
    { "C4BDE5", "Intel Corporate" },
    { "C4BE84", "Texas Instruments" },
    { "C4BED4", "Avaya Inc" },
    { "C4BF60", "TECNO MOBILE LIMITED" },
    { "C4C063", "New H3C Technologies Co., Ltd" },
    { "C4C0AE", "MIDORI ELECTRONIC CO., LTD." },
    { "C4C138", "OWLink Technology Inc" },
    { "C4C17D", "Apple, Inc." },
    { "C4C19F", "National Oilwell Varco Instrumentation, Monitoring, and Optimization (NOV IMO)" },
    { "C4C36B", "Apple, Inc." },
    { "C4C563", "TECNO MOBILE LIMITED" },
    { "C4C603", "Cisco Systems, Inc" },
    { "C4C6E6", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "C4C755", "Beijing HuaqinWorld Technology Co.,Ltd" },
    { "C4C919", "Energy Imports Ltd" },
    { "C4C9EC", "Gugaoo HK Limited" },
    { "C4CA2B", "Arista Networks" },
    { "C4CA67", "Chongqing ZQZER Technology Co., LTD" },
    { "C4CAD9", "Hangzhou H3C Technologies Co., Limited" },
    { "C4CB54", "Fibocom Auto Inc." },
    { "C4CB6B", "Airista Flow, Inc." },
    { "C4CB76", "Microsoft Corporation" },
    { "C4CBBE", "Great Talent Technology Limited" },
    { "C4CBE1", "Dell Inc." },
    { "C4CC37", "IEEE Registration Authority" },
    { "C4CCF9", "zte corporation" },
    { "C4CD45", "Beijing Boomsense Technology CO.,LTD." },
    { "C4CD82", "Hangzhou Lowan Information Technology Co., Ltd." },
    { "C4D0E3", "Intel Corporate" },
    { "C4D197", "Ventia Utility Services" },
    { "C4D36A", "Texas Instruments" },
    { "C4D438", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C4D489", "JiangSu Joyque Information Industry Co.,Ltd" },
    { "C4D496", "Shenzhen Excelsecu Data Technology Co.,Ltd" },
    { "C4D655", "Tercel technology co.,ltd" },
    { "C4D666", "Cisco Meraki" },
    { "C4D6D3", "Dell Inc." },
    { "C4D738", "Huawei Device Co., Ltd." },
    { "C4D7DC", "Zhejiang Weilai Jingling Artificial Intelligence Technology Co., Ltd." },
    { "C4D7FD", "Bouffalo Lab (Nanjing) Co., Ltd." },
    { "C4D8C8", "Silicon Laboratories" },
    { "C4D8D4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C4D8D5", "Espressif Inc." },
    { "C4D8F3", "iZotope" },
    { "C4D987", "Intel Corporate" },
    { "C4DA26", "NOBLEX SA" },
    { "C4DA7D", "Ivium Technologies B.V." },
    { "C4DB04", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C4DBAD", "Ring LLC" },
    { "C4DD57", "Espressif Inc." },
    { "C4DE7B", "Huawei Device Co., Ltd." },
    { "C4DEE2", "Espressif Inc." },
    { "C4DF39", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "C4E032", "IEEE 1904.1 Working Group" },
    { "C4E0DE", "Zhengzhou XindaJiean Information Technology Co.,Ltd." },
    { "C4E17C", "U2S co." },
    { "C4E1A1", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "C4E287", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C4E39F", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "C4E3CE", "EnGenius Technologies, Inc." },
    { "C4E506", "Piper Networks, Inc." },
    { "C4E510", "Mechatro, Inc." },
    { "C4E532", "Arcadyan Corporation" },
    { "C4E5B1", "Suzhou PanKore Integrated Circuit Technology Co. Ltd." },
    { "C4E733", "Clear Align LLC" },
    { "C4E7AE", "Chengdu Meross Technology Co., Ltd." },
    { "C4E7BE", "SCSpro Co.,Ltd" },
    { "C4E90A", "D-Link International" },
    { "C4E92F", "AB Sciex" },
    { "C4E984", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "C4EA1D", "Technicolor Delivery Technologies Belgium NV" },
    { "C4EB39", "Sagemcom Broadband SAS" },
    { "C4EB41", "Sagemcom Broadband SAS" },
    { "C4EB42", "Sagemcom Broadband SAS" },
    { "C4EB43", "Sagemcom Broadband SAS" },
    { "C4EB68", "VusionGroup" },
    { "C4EBE3", "RRCN SAS" },
    { "C4EBFF", "zte corporation" },
    { "C4EDBA", "Texas Instruments" },
    { "C4EEAE", "VSS Monitoring" },
    { "C4EEF5", "II-VI Incorporated" },
    { "C4EF3D", "Samsung Electronics Co.,Ltd" },
    { "C4EF70", "Home Skinovations" },
    { "C4EFBB", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "C4EFDA", "Honeywell" },
    { "C4F035", "Hughes Network Systems, LLC" },
    { "C4F081", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C4F0EC", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "C4F122", "Nexar Ltd." },
    { "C4F174", "eero inc." },
    { "C4F1D1", "BEIJING SOGOU TECHNOLOGY DEVELOPMENT CO., LTD." },
    { "C4F312", "Texas Instruments" },
    { "C4F464", "Spica international" },
    { "C4F57C", "Brocade Communications Systems LLC" },
    { "C4F5A5", "Kumalift Co., Ltd." },
    { "C4F7C1", "Apple, Inc." },
    { "C4F7D5", "Cisco Systems, Inc" },
    { "C4F839", "Actia Automotive" },
    { "C4FBAA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C4FBC8", "Shenzhen Candour Co., Ltd." },
    { "C4FC22", "YEALINK(XIAMEN) NETWORK TECHNOLOGY CO.,LTD." },
    { "C4FCE4", "DishTV NZ Ltd" },
    { "C4FCEF", "SambaNova Systems, Inc." },
    { "C4FDE6", "DRTECH" },
    { "C4FE5B", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "C4FEE2", "AMICCOM Electronics Corporation" },
    { "C4FF1F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C4FF22", "Huawei Device Co., Ltd." },
    { "C4FF84", "IEEE Registration Authority" },
    { "C4FF99", "Intel Corporate" },
    { "C4FFBC", "IEEE Registration Authority" },
    { "C80084", "Cisco Systems, Inc" },
    { "C80210", "LG Innotek" },
    { "C80258", "ITW GSE ApS" },
    { "C8028F", "Nova Electronics (Shanghai) Co., Ltd." },
    { "C802A6", "Beijing Newmine Technology" },
    { "C803F5", "Ruckus Wireless" },
    { "C8059E", "Hefei Symboltek Co.,Ltd" },
    { "C805A4", "Motorola(Wuhan) Mobility Technologies Communication Co.,Ltd" },
    { "C80718", "TDSi" },
    { "C80739", "NAKAYO Inc" },
    { "C80873", "Ruckus Wireless" },
    { "C8088B", "Arista Networks" },
    { "C808E9", "LG Electronics" },
    { "C809A8", "Intel Corporate" },
    { "C80A35", "Qingdao Hisense Smart Life Technology Co., Ltd" },
    { "C80AA9", "Quanta Computer Inc." },
    { "C80C53", "China Mobile Group Device Co.,Ltd." },
    { "C80CC8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C80D32", "Holoplot GmbH" },
    { "C80E14", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "C80E77", "Le Shi Zhi Xin Electronic Technology (Tianjin) Limited" },
    { "C80E95", "OmniLync Inc." },
    { "C8102F", "NETGEAR" },
    { "C81072", "BBPOS Limited" },
    { "C81073", "CENTURY OPTICOMM CO.,LTD" },
    { "C8120B", "Samsung Electronics Co.,Ltd" },
    { "C81337", "Juniper Networks" },
    { "C8138B", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "C81451", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C81479", "Samsung Electronics Co.,Ltd" },
    { "C814B4", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "C8154E", "Intel Corporate" },
    { "C816A5", "Masimo Corporation" },
    { "C816BD", "Qingdao Hisense Communications Co.,Ltd." },
    { "C816DA", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "C81739", "ITEL MOBILE LIMITED" },
    { "C817F5", "Nanjing Qinheng Microelectronics Co., Ltd." },
    { "C819F7", "Samsung Electronics Co.,Ltd" },
    { "C81AFE", "DLOGIC GmbH" },
    { "C81B5C", "BCTech" },
    { "C81B6B", "Innova Security" },
    { "C81CFE", "Zebra Technologies Inc." },
    { "C81E8E", "ADV Security (S) Pte Ltd" },
    { "C81EC2", "ITEL MOBILE LIMITED" },
    { "C81EE7", "Apple, Inc." },
    { "C81F66", "Dell Inc." },
    { "C81FBE", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C81FE8", "Apple, Inc." },
    { "C81FEA", "Avaya Inc" },
    { "C8208E", "Storagedata" },
    { "C82158", "Intel Corporate" },
    { "C821DA", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "C82478", "Edifier International" },
    { "C82496", "Jiangsu Yinhe Electronics Co.,Ltd." },
    { "C825E1", "Lemobile Information Technology (Beijing) Co., Ltd" },
    { "C826E2", "CHINA DRAGON TECHNOLOGY LIMITED" },
    { "C82832", "Beijing Xiaomi Electronics Co., Ltd." },
    { "C828E5", "Cisco Systems, Inc" },
    { "C8292A", "Barun Electronics" },
    { "C829C8", "Palo Alto Networks" },
    { "C82A14", "Apple, Inc." },
    { "C82ADD", "Google, Inc." },
    { "C82AF1", "TCT mobile ltd" },
    { "C82B6B", "shenzhen worldelite electronics co., LTD" },
    { "C82B96", "Espressif Inc." },
    { "C82C2B", "IEEE Registration Authority" },
    { "C82E18", "Espressif Inc." },
    { "C82E47", "Suzhou SmartChip Semiconductor Co., LTD" },
    { "C82E94", "Halfa Enterprise Co., Ltd." },
    { "C83168", "eZEX corporation" },
    { "C83232", "Hunting Innova" },
    { "C8334B", "Apple, Inc." },
    { "C833E5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C8348E", "Intel Corporate" },
    { "C835B8", "Ericsson, EAB/RWI/K" },
    { "C836A3", "GERTEC BRASIL LTDA" },
    { "C83870", "Samsung Electronics Co.,Ltd" },
    { "C839AC", "Huawei Device Co., Ltd." },
    { "C83A1B", "Toshiba TEC Corporation Inc" },
    { "C83A35", "Tenda Technology Co., Ltd." },
    { "C83A6B", "Roku, Inc" },
    { "C83B45", "JRI" },
    { "C83C85", "Apple, Inc." },
    { "C83D97", "Nokia Corporation" },
    { "C83DD4", "CyberTAN Technology Inc." },
    { "C83DDC", "Xiaomi Communications Co Ltd" },
    { "C83DFC", "AlphaTheta Corporation" },
    { "C83E99", "Texas Instruments" },
    { "C83E9E", "Huawei Device Co., Ltd." },
    { "C83EA7", "KUNBUS GmbH" },
    { "C83F26", "Microsoft Corporation" },
    { "C83FB4", "Commscope" },
    { "C84029", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "C84052", "PAX Computer Technology(Shenzhen) Ltd." },
    { "C8418A", "Samsung Electronics.,LTD" },
    { "C84529", "IMK Networks Co.,Ltd" },
    { "C84544", "Asia Pacific CIS (Wuxi) Co, Ltd" },
    { "C8458F", "Wyler AG" },
    { "C84709", "Cisco Systems, Inc" },
    { "C84782", "Areson Technology Corp." },
    { "C8478C", "Beken Corporation" },
    { "C84805", "Nintendo Co.,Ltd" },
    { "C848F5", "MEDISON Xray Co., Ltd" },
    { "C84AA0", "Sony Interactive Entertainment Inc." },
    { "C84BD6", "Dell Inc." },
    { "C84C75", "Cisco Systems, Inc" },
    { "C84C78", "zte corporation" },
    { "C84D34", "LIONS Taiwan Technology Inc." },
    { "C84D44", "Shenzhen Jiapeng Huaxiang Technology Co.,Ltd" },
    { "C84F0E", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "C84F86", "Sophos Ltd" },
    { "C850CE", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C850E9", "Raisecom Technology CO., LTD" },
    { "C85142", "Samsung Electronics Co.,Ltd" },
    { "C85195", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C851FB", "Extreme Networks Headquarters" },
    { "C85261", "Commscope" },
    { "C85309", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "C853E1", "Douyin Vision Co., Ltd" },
    { "C8544B", "Zyxel Communications Corporation" },
    { "C854A4", "Infinix mobility limited" },
    { "C85645", "Intermas France" },
    { "C85663", "Sunflex Europe GmbH" },
    { "C8586A", "Beijing Winner Microelectronics Co.,Ltd. " },
    { "C85895", "Motorola Mobility LLC, a Lenovo Company" },
    { "C858B3", "Intel Corporate" },
    { "C858C0", "Intel Corporate" },
    { "C85A9F", "zte corporation" },
    { "C85ACF", "HP Inc." },
    { "C85B76", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "C85BA0", "Shenzhen Qihu Intelligent Technology Company Limited" },
    { "C85CCC", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "C85CE2", "IEEE Registration Authority" },
    { "C85D38", "HUMAX Co., Ltd." },
    { "C85EA9", "Intel Corporate" },
    { "C86000", "ASUSTek COMPUTER INC." },
    { "C8608F", "Cisco Systems, Inc" },
    { "C861D0", "SHEN ZHEN KTC TECHNOLOGY.,LTD." },
    { "C86314", "IEEE Registration Authority" },
    { "C863F1", "Sony Interactive Entertainment Inc." },
    { "C863FC", "Commscope" },
    { "C864C7", "zte corporation" },
    { "C8662C", "Beijing Haitai Fangyuan High Technology Co,.Ltd." },
    { "C8664B", "Aperion Technologies LLC" },
    { "C8665D", "Extreme Networks Headquarters" },
    { "C8675E", "Extreme Networks Headquarters" },
    { "C868DE", "Huawei Device Co., Ltd." },
    { "C869CD", "Apple, Inc." },
    { "C86BBC", "IEEE Registration Authority" },
    { "C86C1E", "Display Systems Ltd" },
    { "C86C20", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "C86C3D", "Amazon Technologies Inc." },
    { "C86C87", "Zyxel Communications Corporation" },
    { "C86CB6", "Optcom Co., Ltd." },
    { "C86E08", "Intel Corporate" },
    { "C86F1D", "Apple, Inc." },
    { "C87023", "Altice Labs" },
    { "C870D4", "IBO Technology Co,Ltd" },
    { "C8711F", "SUZHOU TESIEN TECHNOLOGY CO., LTD." },
    { "C87125", "Johnson Outdoors Marine Electronics d/b/a Minnkota" },
    { "C87248", "Aplicom Oy" },
    { "C8727E", "Nokia" },
    { "C87324", "Sow Cheng Technology Co. Ltd." },
    { "C8755B", "Quantify Technology Pty. Ltd." },
    { "C875DD", "LG Electronics NV" },
    { "C875F4", "China Mobile Group Device Co.,Ltd." },
    { "C87765", "Tiesse SpA" },
    { "C8778B", "Mercury Systems – Trusted Mission Solutions, Inc. " },
    { "C877F3", "VusionGroup" },
    { "C87867", "Mist Systems, Inc." },
    { "C8787D", "D-Link Corporation" },
    { "C87B23", "Bose Corporation" },
    { "C87B5B", "zte corporation" },
    { "C87CBC", "Valink Co., Ltd. " },
    { "C87CE2", "Infrawaves" },
    { "C87D77", "Shenzhen Kingtech Communication Equipment Co.,Ltd" },
    { "C87E75", "Samsung Electronics Co.,Ltd" },
    { "C87EA1", "TCL MOKA International Limited" },
    { "C87F2B", "INGRAM MICRO SERVICES" },
    { "C87F54", "ASUSTek COMPUTER INC." },
    { "C88234", "Cisco Systems, Inc" },
    { "C88314", "Tempo Communications" },
    { "C88439", "Sunrise Technologies" },
    { "C88447", "Beautiful Enterprise Co., Ltd" },
    { "C8844E", "Flextronics International Kft" },
    { "C8848C", "Ruckus Wireless" },
    { "C884A1", "Cisco Systems, Inc" },
    { "C884CF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C88550", "Apple, Inc." },
    { "C88629", "Shenzhen Duubee Intelligent Technologies Co.,LTD." },
    { "C88722", "Lumenpulse" },
    { "C8873B", "Net Optics" },
    { "C889F3", "Apple, Inc." },
    { "C88A83", "Dongguan HuaHong Electronics Co.,Ltd" },
    { "C88A9A", "Intel Corporate" },
    { "C88AD8", "CHINA DRAGON TECHNOLOGY LIMITED" },
    { "C88B47", "Nolangroup S.P.A con Socio Unico" },
    { "C88BE8", "Masimo Corporation" },
    { "C88D83", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C88DD4", "Markone technology Co., Ltd." },
    { "C88ED1", "IEEE Registration Authority" },
    { "C88F26", "Skyworth Digital Technology(Shenzhen) Co.,Ltd" },
    { "C89009", "Budderfly Inc." },
    { "C8903E", "Pakton Technologies" },
    { "C8908A", "Samsung Electronics Co.,Ltd" },
    { "C890A8", "Bouffalo Lab (Nanjing) Co., Ltd." },
    { "C891F9", "Sagemcom Broadband SAS" },
    { "C89346", "MXCHIP Company Limited" },
    { "C89383", "Embedded Automation, Inc." },
    { "C89402", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "C894BB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C894D2", "Jiangsu Datang Electronic Products Co., Ltd" },
    { "C895CE", "Intel Corporate" },
    { "C8965A", "SKY UK LIMITED" },
    { "C89665", "Microsoft Corporation" },
    { "C8979F", "Nokia Corporation" },
    { "C89828", "zte corporation" },
    { "C899B2", "Arcadyan Corporation" },
    { "C89BAD", "Honor Device Co., Ltd." },
    { "C89BD7", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "C89C13", "Inspiremobile" },
    { "C89C1D", "Cisco Systems, Inc" },
    { "C89CBB", "Taicang T&W Electronics" },
    { "C89CDC", "Elitegroup Computer Systems Co.,Ltd." },
    { "C89D18", "Huawei Device Co., Ltd." },
    { "C89D6D", "ITEL MOBILE LIMITED" },
    { "C89E43", "NETGEAR" },
    { "C89E61", "Lyngsoe Systems LTd" },
    { "C89F0C", "Motorola Mobility LLC, a Lenovo Company" },
    { "C89F1A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C89F1D", "SHENZHEN COMMUNICATION TECHNOLOGIES CO.,LTD" },
    { "C89F42", "VDII Innovation AB" },
    { "C8A030", "Texas Instruments" },
    { "C8A1B6", "Shenzhen Longway Technologies Co., Ltd" },
    { "C8A1BA", "Neul Ltd" },
    { "C8A1DC", "Motorola Mobility LLC, a Lenovo Company" },
    { "C8A23B", "Shenzhen Sundray Technologies company Limited" },
    { "C8A2CE", "Oasis Media Systems LLC" },
    { "C8A362", "ASIX Electronics Corporation" },
    { "C8A3E8", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "C8A40D", "Cooler Master Technology Inc" },
    { "C8A4CD", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "C8A608", "Ruckus Wireless" },
    { "C8A620", "Nebula, Inc" },
    { "C8A6EF", "Samsung Electronics Co.,Ltd" },
    { "C8A702", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "C8A70A", "Verizon Business" },
    { "C8A729", "SYStronics Co., Ltd." },
    { "C8A776", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C8A823", "Samsung Electronics Co.,Ltd" },
    { "C8A913", "Lontium Semiconductor Corporation" },
    { "C8A9FC", "Goyoo Networks Inc." },
    { "C8AA21", "Commscope" },
    { "C8AA55", "Hunan Comtom Electronic Incorporated Co.,Ltd" },
    { "C8AAB2", "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD" },
    { "C8AACC", "Private" },
    { "C8AC35", "PiLink Co., Ltd." },
    { "C8AE9C", "Shanghai TYD Elecronic Technology Co. Ltd" },
    { "C8AF40", "marco Systemanalyse und Entwicklung GmbH" },
    { "C8AFE3", "Hefei Radio Communication Technology Co., Ltd " },
    { "C8B1CD", "Apple, Inc." },
    { "C8B1EE", "Qorvo" },
    { "C8B21E", "CHIPSEA TECHNOLOGIES (SHENZHEN) CORP." },
    { "C8B29B", "Intel Corporate" },
    { "C8B373", "Cisco-Linksys, LLC" },
    { "C8B422", "ASKEY COMPUTER CORP" },
    { "C8B4AB", "Inspur Computer Technology Co.,Ltd." },
    { "C8B5AD", "Hewlett Packard Enterprise " },
    { "C8B5B7", "Apple, Inc." },
    { "C8B6D3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C8B6FE", "Fitbit, Inc." },
    { "C8B82F", "eero inc." },
    { "C8BA94", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "C8BAE9", "QDIS" },
    { "C8BB81", "Huawei Device Co., Ltd." },
    { "C8BBD3", "Embrane" },
    { "C8BC9C", "Huawei Device Co., Ltd." },
    { "C8BCC8", "Apple, Inc." },
    { "C8BCE5", "Sense Things Japan INC." },
    { "C8BD4D", "Samsung Electronics Co.,Ltd" },
    { "C8BD69", "Samsung Electronics Co.,Ltd" },
    { "C8BE19", "D-Link International" },
    { "C8BE35", "Extreme Networks Headquarters" },
    { "C8BF4C", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "C8BFFE", "Huawei Device Co., Ltd." },
    { "C8C126", "ZPM Industria e Comercio Ltda" },
    { "C8C13C", "RuggedTek Hangzhou Co., Ltd" },
    { "C8C2C6", "Shanghai Airm2m Communication Technology Co., Ltd" },
    { "C8C2F5", "FLEXTRONICS MANUFACTURING(ZHUHAI)CO.,LTD." },
    { "C8C2FA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C8C432", "SG Armaturen AS" },
    { "C8C465", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C8C50E", "Shenzhen Primestone Network Technologies.Co., Ltd." },
    { "C8C64A", "Flextronics Tech.(Ind) Pvt Ltd" },
    { "C8C6FE", "eero inc." },
    { "C8C750", "Motorola Mobility LLC, a Lenovo Company" },
    { "C8C791", "Zero1.tv GmbH" },
    { "C8C83F", "Texas Instruments" },
    { "C8C9A3", "Espressif Inc." },
    { "C8CA63", "Huawei Device Co., Ltd." },
    { "C8CA79", "Ciena Corporation" },
    { "C8CB9E", "Intel Corporate" },
    { "C8CBB8", "Hewlett Packard" },
    { "C8CCB5", "Hunter Douglas" },
    { "C8CD55", "Ruijie Networks Co.,LTD" },
    { "C8CD72", "Sagemcom Broadband SAS" },
    { "C8D019", "Shanghai Tigercel Communication Technology Co.,Ltd" },
    { "C8D083", "Apple, Inc." },
    { "C8D10B", "Nokia Corporation" },
    { "C8D12A", "Comtrend Corporation" },
    { "C8D15E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C8D1A9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C8D1D1", "AGAiT Technology Corporation" },
    { "C8D2C1", "Jetlun (Shenzhen) Corporation" },
    { "C8D3A3", "D-Link International" },
    { "C8D3FF", "Hewlett Packard" },
    { "C8D429", "Muehlbauer AG" },
    { "C8D590", "FLIGHT DATA SYSTEMS" },
    { "C8D5FE", "Shenzhen Zowee Technology Co., Ltd" },
    { "C8D69D", "Arab International Optronics" },
    { "C8D6B7", "Solidigm Technology" },
    { "C8D719", "Cisco-Linksys, LLC" },
    { "C8D778", "BSH Hausgeraete GmbH" },
    { "C8D779", "QING DAO HAIER TELECOM CO.,LTD." },
    { "C8D7B0", "Samsung Electronics Co.,Ltd" },
    { "C8D884", "Universal Electronics, Inc." },
    { "C8D959", "Motorola Mobility LLC, a Lenovo Company" },
    { "C8D995", "Juniper Networks" },
    { "C8D9D2", "Hewlett Packard" },
    { "C8DB26", "Logitech" },
    { "C8DD6A", "OHSUNG" },
    { "C8DDC9", "Lenovo Mobile Communication Technology Ltd." },
    { "C8DE41", "SKY UK LIMITED" },
    { "C8DE51", "IntegraOptics" },
    { "C8DEC9", "Coriant" },
    { "C8DF7C", "Nokia Corporation" },
    { "C8DF84", "Texas Instruments" },
    { "C8E07A", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "C8E0EB", "Apple, Inc." },
    { "C8E130", "Milkyway Group Ltd" },
    { "C8E193", "ITEL MOBILE LIMITED" },
    { "C8E1A7", "Vertu Corporation Limited" },
    { "C8E265", "Intel Corporate" },
    { "C8E306", "eero inc." },
    { "C8E31D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C8E42F", "Technical Research Design and Development" },
    { "C8E5E0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C8E600", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "C8E776", "PTCOM Technology" },
    { "C8E7D8", "MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "C8E7F0", "Juniper Networks" },
    { "C8EAF8", "zte corporation" },
    { "C8EBEC", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "C8EDFC", "Shenzhen Ideaform Industrial Product Design Co., Ltd" },
    { "C8EE08", "TANGTOP TECHNOLOGY CO.,LTD" },
    { "C8EE75", "Pishion International Co. Ltd" },
    { "C8EEA6", "Shenzhen SHX Technology Co., Ltd" },
    { "C8EED7", "Lightspeed Technologies Inc." },
    { "C8EF2E", "Beijing Gefei Tech. Co., Ltd " },
    { "C8EFBC", "Inspur Communication Technology Co.,Ltd." },
    { "C8F09E", "Espressif Inc." },
    { "C8F225", "EM Microelectronic" },
    { "C8F230", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "C8F2B4", "Guizhou Huaxin Information Technology Co., Ltd." },
    { "C8F319", "LG Electronics (Mobile Communications)" },
    { "C8F36B", "Yamato Scale Co.,Ltd." },
    { "C8F386", "Shenzhen Xiaoniao Technology Co.,Ltd" },
    { "C8F406", "Avaya Inc" },
    { "C8F5D6", "IEEE Registration Authority" },
    { "C8F650", "Apple, Inc." },
    { "C8F68D", "S.E.TECHNOLOGIES LIMITED" },
    { "C8F6C8", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "C8F704", "Building Block Video" },
    { "C8F733", "Intel Corporate" },
    { "C8F742", "HangZhou Gubei Electronics Technology Co.,Ltd" },
    { "C8F750", "Dell Inc." },
    { "C8F86D", "Alcatel-Lucent Shanghai Bell Co., Ltd" },
    { "C8F946", "LOCOSYS Technology Inc." },
    { "C8F981", "Seneca s.r.l." },
    { "C8F9C8", "NewSharp Technology(SuZhou)Co,Ltd" },
    { "C8F9F9", "Cisco Systems, Inc" },
    { "C8FA84", "Trusonus corp." },
    { "C8FAE1", "ARQ Digital LLC" },
    { "C8FB26", "Cisco SPVTG" },
    { "C8FB54", "iMin Technology Pte. Ltd." },
    { "C8FD19", "Texas Instruments" },
    { "C8FE0F", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "C8FE30", "Bejing DAYO Mobile Communication Technology Ltd." },
    { "C8FE6A", "Juniper Networks" },
    { "C8FF28", "Liteon Technology Corporation" },
    { "C8FF77", "Dyson Limited" },
    { "C8FFBF", "IEEE Registration Authority" },
    { "CC0080", "BETTINI SRL" },
    { "CC00F1", "Sagemcom Broadband SAS" },
    { "CC037B", "Texas Instruments" },
    { "CC0388", "MangoBoost Inc" },
    { "CC03D9", "Cisco Meraki" },
    { "CC03FA", "Vantiva USA LLC" },
    { "CC047C", "G-WAY Microwave" },
    { "CC04B4", "Select Comfort" },
    { "CC051B", "Samsung Electronics Co.,Ltd" },
    { "CC0577", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "CC0677", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "CC07AB", "Samsung Electronics Co.,Ltd" },
    { "CC07E4", "Lenovo Mobile Communication Technology Ltd." },
    { "CC087B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "CC088D", "Apple, Inc." },
    { "CC08E0", "Apple, Inc." },
    { "CC08FA", "Apple, Inc." },
    { "CC08FB", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "CC09C8", "IMAQLIQ LTD" },
    { "CC0CDA", "Miljovakt AS" },
    { "CC0DE7", "B METERS S.R.L." },
    { "CC0DEC", "Cisco SPVTG" },
    { "CC0DF2", "Motorola Mobility LLC, a Lenovo Company" },
    { "CC10A3", "Beijing Nan Bao Technology Co., Ltd." },
    { "CC115A", "Apple, Inc." },
    { "CC1228", "HISENSE VISUAL TECHNOLOGY CO.,LTD" },
    { "CC14A6", "Yichun MyEnergy Domain, Inc" },
    { "CC14BC", "Edifier International" },
    { "CC1531", "Intel Corporate" },
    { "CC167E", "Cisco Systems, Inc" },
    { "CC187B", "Manzanita Systems, Inc." },
    { "CC19A8", "PT Inovação e Sistemas SA" },
    { "CC1AA3", "Arista Networks" },
    { "CC1AFA", "zte corporation" },
    { "CC1B5A", "Ruckus Wireless" },
    { "CC1BE0", "IEEE Registration Authority" },
    { "CC1E56", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "CC1E97", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "CC1EFF", "Metrological Group BV" },
    { "CC1FC4", "InVue" },
    { "CC208C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "CC20AC", "Samsung Electronics Co.,Ltd" },
    { "CC20E8", "Apple, Inc." },
    { "CC2119", "Samsung Electronics Co.,Ltd" },
    { "CC2218", "InnoDigital Co., Ltd." },
    { "CC2237", "IEEE Registration Authority" },
    { "CC2293", "Amazon Technologies Inc." },
    { "CC22DF", "EM Microelectronic" },
    { "CC22FE", "Apple, Inc." },
    { "CC242E", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "CC25EF", "Apple, Inc." },
    { "CC2614", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "CC262D", "Verifi, LLC" },
    { "CC2746", "Apple, Inc." },
    { "CC28AA", "ASUSTek COMPUTER INC." },
    { "CC29BD", "zte corporation" },
    { "CC29F5", "Apple, Inc." },
    { "CC2A80", "Micro-Biz intelligence solutions Co.,Ltd" },
    { "CC2AAC", "Yunjing lntelligent Technology(Dongguan).,Ltd" },
    { "CC2C83", "DarkMatter L.L.C" },
    { "CC2D1B", "SFR" },
    { "CC2D21", "Tenda Technology Co.,Ltd.Dongguan branch" },
    { "CC2D83", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "CC2D8C", "LG ELECTRONICS INC" },
    { "CC2DB7", "Apple, Inc." },
    { "CC2DE0", "Routerboard.com" },
    { "CC2F71", "Intel Corporate" },
    { "CC3080", "VAIO Corporation" },
    { "CC312A", "HUIZHOU TCL COMMUNICATION ELECTRON CO.,LTD" },
    { "CC3296", "Huawei Device Co., Ltd." },
    { "CC32E5", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "CC3331", "Texas Instruments" },
    { "CC33BB", "Sagemcom Broadband SAS" },
    { "CC3429", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "CC34D7", "GEWISS S.P.A." },
    { "CC3540", "Vantiva USA LLC" },
    { "CC355A", "SecuGen Corporation" },
    { "CC36CF", "Cisco Systems, Inc" },
    { "CC37AB", "Edgecore Networks Corporation" },
    { "CC38D0", "Palo Alto Networks" },
    { "CC398C", "Shiningtek" },
    { "CC3A61", "SAMSUNG ELECTRO MECHANICS CO., LTD." },
    { "CC3ADF", "Neptune Technology Group Inc." },
    { "CC3B27", "TECNO MOBILE LIMITED" },
    { "CC3B3E", "Lester Electrical" },
    { "CC3B58", "Curiouser Products Inc" },
    { "CC3BFB", "Ring LLC" },
    { "CC3C3F", "SA.S.S. Datentechnik AG" },
    { "CC3D82", "Intel Corporate" },
    { "CC3DD1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "CC3E5F", "Hewlett Packard" },
    { "CC3E79", "Commscope" },
    { "CC3F1D", "HMS Industrial Networks SLU" },
    { "CC3F36", "Apple, Inc." },
    { "CC3F8A", "KOMATSU LTD." },
    { "CC3FEA", "BAE Systems, Inc" },
    { "CC4085", "WiZ" },
    { "CC40B2", "ECI Telecom Ltd." },
    { "CC40D0", "NETGEAR" },
    { "CC40F3", "Mellanox Technologies, Inc." },
    { "CC418E", "MSA Innovation" },
    { "CC4210", "Xiaomi Communications Co Ltd" },
    { "CC43E3", "Trump s.a." },
    { "CC4460", "Huawei Device Co., Ltd." },
    { "CC4463", "Apple, Inc." },
    { "CC45A5", "Texas Instruments" },
    { "CC4639", "WAAV, Inc." },
    { "CC464E", "Samsung Electronics Co.,Ltd" },
    { "CC46D6", "Cisco Systems, Inc" },
    { "CC4703", "Intercon Systems Co., Ltd." },
    { "CC4740", "AzureWave Technology Inc." },
    { "CC4792", "ASIX Electronics Corporation" },
    { "CC47BD", "Rhombus Systems" },
    { "CC483A", "Dell Inc." },
    { "CC4AE1", "fourtec -Fourier Technologies" },
    { "CC4B04", "Apple, Inc." },
    { "CC4B73", "AMPAK Technology, Inc." },
    { "CC4BFB", "Hellberg Safety AB" },
    { "CC4D38", "Carnegie Technologies" },
    { "CC4D74", "Fujian Newland Payment Technology Co., Ltd." },
    { "CC4D75", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "CC4E24", "Brocade Communications Systems LLC" },
    { "CC4EEC", "HUMAX Co., Ltd." },
    { "CC4F5C", "IEEE Registration Authority" },
    { "CC500A", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "CC501C", "KVH Industries, Inc." },
    { "CC5076", "Ocom Communications, Inc." },
    { "CC50E3", "Espressif Inc." },
    { "CC51B4", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "CC5289", "SHENZHEN OPTFOCUS TECHNOLOGY.,LTD" },
    { "CC52AF", "Universal Global Scientific Industrial Co., Ltd." },
    { "CC53B5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "CC5459", "OnTime Networks AS" },
    { "CC54FE", "Mimosa Networks" },
    { "CC55AD", "RIM" },
    { "CC5763", "Panasonic Automotive Systems Co.,Ltd" },
    { "CC5830", "Sagemcom Broadband SAS" },
    { "CC593E", "Sensium Healthcare Limited" },
    { "CC5A53", "Cisco Systems, Inc" },
    { "CC5B31", "Nintendo Co.,Ltd" },
    { "CC5C61", "Huawei Device Co., Ltd." },
    { "CC5C75", "Weightech Com. Imp. Exp. Equip. Pesagem Ltda" },
    { "CC5CDE", "China Mobile Group Device Co.,Ltd." },
    { "CC5D4E", "Zyxel Communications Corporation" },
    { "CC5D57", "Information System Research Institute,Inc." },
    { "CC5D78", "JTD Consulting" },
    { "CC5EA5", "Palo Alto Networks" },
    { "CC5EF8", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "CC5FBF", "Topwise 3G Communication Co., Ltd." },
    { "CC6023", "Apple, Inc." },
    { "CC60BB", "Empower RF Systems" },
    { "CC60C8", "Microsoft Corporation" },
    { "CC61E5", "Motorola Mobility LLC, a Lenovo Company" },
    { "CC6200", "Honor Device Co., Ltd." },
    { "CC62FE", "UNION MAN TECHNOLOGY CO.,LTD" },
    { "CC641A", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "CC64A6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "CC65AD", "Commscope" },
    { "CC660A", "Apple, Inc." },
    { "CC6618", "Adtran Inc" },
    { "CC66B2", "Nokia" },
    { "CC67D8", "Telin Semiconductor (Wuhan) Co.,Ltd" },
    { "CC68B6", "TP-Link Systems Inc" },
    { "CC68E0", "Apple, Inc." },
    { "CC69B0", "Global Traffic Technologies, LLC" },
    { "CC69FA", "Apple, Inc." },
    { "CC6A10", "The Chamberlain Group, Inc" },
    { "CC6A33", "Cisco Systems, Inc" },
    { "CC6B1E", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "CC6B98", "Minetec Wireless Technologies" },
    { "CC6BF1", "Sound Masking Inc." },
    { "CC6C52", "Zhone Technologies, Inc." },
    { "CC6D55", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "CC6DA0", "Roku, Inc." },
    { "CC6DEF", "TJK Tietolaite Oy" },
    { "CC6E2A", "Cisco Meraki" },
    { "CC6EA4", "Samsung Electronics Co.,Ltd" },
    { "CC70ED", "Cisco Systems, Inc" },
    { "CC7190", "VIETNAM POST AND TELECOMMUNICATION INDUSTRY TECHNOLOGY JOINT STOCK COMPANY" },
    { "CC720F", "Viscount Systems Inc." },
    { "CC7286", "Xi'an Fengyu Information Technology Co., Ltd." },
    { "CC7314", "HONG KONG WHEATEK TECHNOLOGY LIMITED" },
    { "CC7498", "Filmetrics Inc." },
    { "CC75E2", "Commscope" },
    { "CC763A", "zte corporation" },
    { "CC7669", "SEETECH" },
    { "CC77C9", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "CC785F", "Apple, Inc." },
    { "CC78AB", "Texas Instruments" },
    { "CC794A", "BLU Products Inc." },
    { "CC79CF", "SHENZHEN RF-LINK TECHNOLOGY CO.,LTD." },
    { "CC79D7", "Cisco Systems, Inc" },
    { "CC7A30", "CMAX Wireless Co., Ltd." },
    { "CC7B35", "zte corporation" },
    { "CC7B5C", "Espressif Inc." },
    { "CC7B61", "NIKKISO CO., LTD." },
    { "CC7D37", "Commscope" },
    { "CC7D5B", "Telink Semiconductor (Shanghai) Co., Ltd." },
    { "CC7E0F", "Theben AG" },
    { "CC7EE7", "Panasonic Corporation AVC Networks Company" },
    { "CC7F75", "Cisco Systems, Inc" },
    { "CC7F76", "Cisco Systems, Inc" },
    { "CC812A", "vivo Mobile Communication Co., Ltd." },
    { "CC817D", "Apple, Inc." },
    { "CC81DA", "Phicomm (Shanghai) Co., Ltd." },
    { "CC827F", "Advantech Technology (CHINA) Co., Ltd." },
    { "CC82EB", "KYOCERA CORPORATION " },
    { "CC856C", "SHENZHEN MDK DIGITAL TECHNOLOGY CO.,LTD" },
    { "CC86EC", "Silicon Laboratories" },
    { "CC874A", "Nokia" },
    { "CC8826", "LG Innotek" },
    { "CC88C7", "Hewlett Packard Enterprise" },
    { "CC895E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "CC896C", "GN Hearing A/S" },
    { "CC89FD", "Nokia Corporation" },
    { "CC8A84", "Huawei Device Co., Ltd." },
    { "CC8C17", "ITEL MOBILE LIMITED" },
    { "CC8CBF", "Tuya Smart Inc." },
    { "CC8CDA", "Shenzhen Wei Da Intelligent Technology Go.,Ltd" },
    { "CC8CE3", "Texas Instruments" },
    { "CC8DA2", "Espressif Inc." },
    { "CC8DB5", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "CC8E71", "Cisco Systems, Inc" },
    { "CC9070", "Cisco Systems, Inc" },
    { "CC9093", "Hansong Tehnologies" },
    { "CC9096", "Huawei Device Co., Ltd." },
    { "CC90E8", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "CC912B", "TE Connectivity Touch Solutions" },
    { "CC934A", "Sierra Wireless, ULC" },
    { "CC944A", "Pfeiffer Vacuum GmbH" },
    { "CC9470", "Kinestral Technologies, Inc." },
    { "CC95D7", "Vizio, Inc" },
    { "CC9635", "LVS Co.,Ltd." },
    { "CC96A0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "CC96A2", "China Mobile Group Device Co.,Ltd." },
    { "CC96E5", "Dell Inc." },
    { "CC988B", "SONY Visual Products Inc." },
    { "CC9891", "Cisco Systems, Inc" },
    { "CC9916", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "CC9C3E", "Cisco Meraki" },
    { "CC9DA2", "Eltex Enterprise Ltd." },
    { "CC9E00", "Nintendo Co., Ltd." },
    { "CC9EA2", "Amazon Technologies Inc." },
    { "CC9ECA", "HMD Global Oy" },
    { "CC9F35", "Transbit Sp. z o.o." },
    { "CC9F7A", "Chiun Mai Communication System, Inc" },
    { "CCA08F", "zte corporation" },
    { "CCA0E5", "DZG Metering GmbH" },
    { "CCA12B", "TCL King Electrical Appliances (Huizhou) Co., Ltd" },
    { "CCA150", "SystemX Co.,Ltd." },
    { "CCA174", "Meta Platforms, Inc." },
    { "CCA219", "SHENZHEN ALONG INVESTMENT CO.,LTD" },
    { "CCA223", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "CCA260", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "CCA374", "Guangdong Guanglian Electronic Technology Co.Ltd" },
    { "CCA3BD", "ITEL MOBILE LIMITED" },
    { "CCA462", "Commscope" },
    { "CCA4AF", "Shenzhen Sowell Technology Co., LTD" },
    { "CCA614", "AIFA TECHNOLOGY CORP." },
    { "CCA7C1", "Google, Inc." },
    { "CCAB2C", "HUMAX Co., Ltd." },
    { "CCACFE", "Telink Semiconductor (Shanghai) Co., Ltd." },
    { "CCAF78", "Hon Hai Precision Ind. Co.,Ltd." },
    { "CCB071", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "CCB0A8", "Huawei Device Co., Ltd." },
    { "CCB0B3", "Microsoft Corporation" },
    { "CCB0DA", "Liteon Technology Corporation" },
    { "CCB11A", "Samsung Electronics Co.,Ltd" },
    { "CCB148", "Arcadyan Corporation" },
    { "CCB182", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "CCB255", "D-Link International" },
    { "CCB3AB", "shenzhen Biocare Bio-Medical Equipment Co.,Ltd." },
    { "CCB3F8", "FUJITSU ISOTEC LIMITED" },
    { "CCB54C", "Texas Instruments" },
    { "CCB55A", "Fraunhofer ITWM" },
    { "CCB5D1", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "CCB691", "NECMagnusCommunications" },
    { "CCB6C8", "Cisco Systems, Inc" },
    { "CCB777", "zte corporation" },
    { "CCB7C4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "CCB85E", "Shenzhen Phaten Tech. LTD" },
    { "CCB888", "AnB Securite s.a." },
    { "CCB8A8", "AMPAK Technology, Inc." },
    { "CCB8F1", "EAGLE KINGDOM TECHNOLOGIES LIMITED" },
    { "CCBA6F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "CCBA97", "Espressif Inc." },
    { "CCBABD", "TP-Link Systems Inc." },
    { "CCBBFE", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "CCBC2B", "Huawei Device Co., Ltd." },
    { "CCBCE3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "CCBD35", "Steinel GmbH" },
    { "CCBDD3", "Ultimaker B.V." },
    { "CCBE59", "Calix Inc." },
    { "CCBE71", "OptiLogix BV" },
    { "CCBF0C", "SHENZHEN FENDA TECHNOLOGY CO., LTD" },
    { "CCC079", "Murata Manufacturing Co., Ltd." },
    { "CCC104", "Applied Technical Systems" },
    { "CCC261", "IEEE Registration Authority" },
    { "CCC2E0", "Raisecom Technology CO., LTD" },
    { "CCC3EA", "Motorola Mobility LLC, a Lenovo Company" },
    { "CCC50A", "SHENZHEN DAJIAHAO TECHNOLOGY CO.,LTD" },
    { "CCC5E5", "Dell Inc." },
    { "CCC5EF", "Co-Comm Servicios Telecomunicaciones S.L." },
    { "CCC62B", "Tri-Systems Corporation" },
    { "CCC760", "Apple, Inc." },
    { "CCC8D7", "CIAS Elettronica srl" },
    { "CCC92C", "Schindler - PORT Technology" },
    { "CCC95D", "Apple, Inc." },
    { "CCCC4E", "Sun Fountainhead USA. Corp " },
    { "CCCC77", "Zaram Technology. Inc." },
    { "CCCC81", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "CCCCCC", "Silicon Laboratories" },
    { "CCCCEA", "Phoenix Contact GmbH & Co. KG" },
    { "CCCD64", "SM-Electronic GmbH" },
    { "CCCE1E", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "CCCE40", "Janteq Corp" },
    { "CCCF83", "CIG SHANGHAI CO LTD" },
    { "CCCFFE", "Henan Lingyunda Information Technology Co., Ltd" },
    { "CCD083", "Hewlett Packard Enterprise" },
    { "CCD281", "Apple, Inc." },
    { "CCD29B", "Shenzhen Bopengfa Elec&Technology CO.,Ltd" },
    { "CCD31E", "IEEE Registration Authority" },
    { "CCD342", "Cisco Systems, Inc" },
    { "CCD39D", "IEEE Registration Authority" },
    { "CCD3C1", "Vestel Elektronik San ve Tic. A.S." },
    { "CCD3E2", "Jiangsu Yinhe Electronics Co.,Ltd." },
    { "CCD42E", "Arcadyan Corporation" },
    { "CCD4A1", "MitraStar Technology Corp." },
    { "CCD539", "Cisco Systems, Inc" },
    { "CCD73C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "CCD811", "Aiconn Technology Corporation" },
    { "CCD81F", "Maipu Communication Technology Co.,Ltd." },
    { "CCD843", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "CCD8C1", "Cisco Systems, Inc" },
    { "CCD920", "Huizhou Desay SV Automotive Co., Ltd." },
    { "CCD9AC", "Intel Corporate" },
    { "CCD9E9", "SCR Engineers Ltd." },
    { "CCDA20", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "CCDAB5", "Texas Instruments" },
    { "CCDB04", "DataRemote Inc." },
    { "CCDB93", "Cisco Systems, Inc" },
    { "CCDBA7", "Espressif Inc." },
    { "CCDC55", "Dragonchip Limited" },
    { "CCDD58", "Robert Bosch GmbH" },
    { "CCDEDE", "Nokia" },
    { "CCE0C3", "EXTEN Technologies, Inc." },
    { "CCE0DA", "Baidu Online Network Technology (Beijing) Co., Ltd" },
    { "CCE17F", "Juniper Networks" },
    { "CCE194", "Juniper Networks" },
    { "CCE1D5", "BUFFALO.INC" },
    { "CCE236", "Hangzhou Yaguan Technology Co. LTD" },
    { "CCE4D1", "Arista Networks" },
    { "CCE536", "ittim" },
    { "CCE686", "Samsung Electronics Co.,Ltd" },
    { "CCE798", "My Social Stuff" },
    { "CCE7DF", "American Magnetics, Inc." },
    { "CCE8AC", "SOYEA Technology Co.,Ltd." },
    { "CCE9FA", "Samsung Electronics Co.,Ltd" },
    { "CCEA1C", "DCONWORKS Co., Ltd" },
    { "CCEA27", "GE Appliances" },
    { "CCEB18", "OOO TSS" },
    { "CCEB5E", "Xiaomi Communications Co Ltd" },
    { "CCECB7", "ShenZhen Linked-Z Intelligent Display Co., Ltd" },
    { "CCED21", "Nokia Shanghai Bell Co., Ltd." },
    { "CCED4D", "Cisco Systems, Inc" },
    { "CCEDDC", "MitraStar Technology Corp." },
    { "CCEED9", "VAHLE Automation GmbH" },
    { "CCEF03", "Hunan Keyshare Communication Technology Co., Ltd." },
    { "CCEF48", "Cisco Systems, Inc" },
    { "CCF0FD", "China Mobile (Hangzhou) Information Technology Co., Ltd." },
    { "CCF305", "SHENZHEN TIAN XING CHUANG ZHAN ELECTRONIC CO.,LTD" },
    { "CCF3A5", "Chi Mei Communication Systems, Inc" },
    { "CCF3C8", "Vantiva USA LLC" },
    { "CCF407", "EUKREA ELECTROMATIQUE SARL" },
    { "CCF411", "Google, Inc." },
    { "CCF538", "3isysnetworks" },
    { "CCF55F", "E FOCUS INSTRUMENTS INDIA PRIVATE LIMITED" },
    { "CCF67A", "Ayecka Communication Systems LTD" },
    { "CCF735", "Amazon Technologies Inc." },
    { "CCF826", "Samsung Electronics Co.,Ltd" },
    { "CCF841", "Lumewave" },
    { "CCF8F0", "Xi'an HISU Multimedia Technology Co.,Ltd." },
    { "CCF954", "Avaya Inc" },
    { "CCF957", "u-blox AG" },
    { "CCF9E4", "Intel Corporate" },
    { "CCF9E8", "Samsung Electronics Co.,Ltd" },
    { "CCF9F0", "Samsung Electronics Co.,Ltd" },
    { "CCFA00", "LG Electronics (Mobile Communications)" },
    { "CCFA66", "Huawei Device Co., Ltd." },
    { "CCFAF1", "Sagemcom Broadband SAS" },
    { "CCFB65", "Nintendo Co., Ltd." },
    { "CCFC6D", "RIZ TRANSMITTERS" },
    { "CCFCB1", "Wireless Technology, Inc." },
    { "CCFD17", "TCT mobile ltd" },
    { "CCFE3C", "Samsung Electronics Co.,Ltd" },
    { "CCFF90", "Huawei Device Co., Ltd." },
    { "D0034B", "Apple, Inc." },
    { "D003DF", "Samsung Electronics Co.,Ltd" },
    { "D003EB", "Texas Instruments" },
    { "D00401", "Motorola Mobility LLC, a Lenovo Company" },
    { "D00492", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "D004B0", "Samsung Electronics Co.,Ltd" },
    { "D0052A", "Arcadyan Corporation" },
    { "D005E4", "Huawei Device Co., Ltd." },
    { "D0066A", "Cornelis Networks, Inc." },
    { "D00790", "Texas Instruments" },
    { "D007CA", "Juniper Networks" },
    { "D009C8", "Cisco Systems, Inc" },
    { "D009F5", "Hosiden Corporation" },
    { "D00AAB", "Yokogawa Digital Computer Corporation" },
    { "D00B27", "Murata Manufacturing Co., Ltd." },
    { "D00DF7", "Huawei Device Co., Ltd." },
    { "D00EA4", "Porsche Cars North America" },
    { "D00ED9", "Taicang T&W Electronics" },
    { "D00F6D", "T&W Electronics Company" },
    { "D011E5", "Apple, Inc." },
    { "D01242", "BIOS Corporation" },
    { "D01255", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "D012CB", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "D0131E", "Sunrex Technology Corp" },
    { "D013FD", "LG Electronics (Mobile Communications)" },
    { "D01411", "IEEE Registration Authority" },
    { "D0154A", "zte corporation" },
    { "D015A6", "Hewlett Packard Enterprise" },
    { "D015BB", "IEEE Registration Authority" },
    { "D0167C", "eero inc." },
    { "D016B4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D016F0", "IEEE Registration Authority" },
    { "D01769", "Murata Manufacturing Co., Ltd." },
    { "D0176A", "Samsung Electronics Co.,Ltd" },
    { "D017C2", "ASUSTek COMPUTER INC." },
    { "D0196A", "Ciena Corporation" },
    { "D019D3", "ITEL MOBILE LIMITED" },
    { "D01AA7", "UniPrint" },
    { "D01B1F", "OHSUNG" },
    { "D01B49", "Samsung Electronics Co.,Ltd" },
    { "D01BBE", "Onward Brands" },
    { "D01BF4", "Sagemcom Broadband SAS" },
    { "D01C3C", "TECNO MOBILE LIMITED" },
    { "D01CBB", "Beijing Ctimes Digital Technology Co., Ltd." },
    { "D01E1D", "SaiNXT Technologies LLP" },
    { "D020DD", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "D021AC", "Yohana" },
    { "D021F9", "Ubiquiti Inc" },
    { "D02212", "IEEE Registration Authority" },
    { "D022BE", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "D023DB", "Apple, Inc." },
    { "D02516", "MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "D02544", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "D02598", "Apple, Inc." },
    { "D02788", "Hon Hai Precision Ind. Co.,Ltd." },
    { "D028BA", "Realme Chongqing MobileTelecommunications Corp Ltd" },
    { "D02AAA", "Infinix mobility limited" },
    { "D02B20", "Apple, Inc." },
    { "D02C45", "littleBits Electronics, Inc." },
    { "D02DB3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D02EAB", "Texas Instruments" },
    { "D03110", "Ingenic Semiconductor Co.,Ltd" },
    { "D03169", "Samsung Electronics Co.,Ltd" },
    { "D032C3", "D-Link Corporation" },
    { "D03311", "Apple, Inc." },
    { "D035E5", "EM Microelectronic" },
    { "D03742", "Yulong Computer Telecommunication Scientific (Shenzhen) Co.,Ltd" },
    { "D03745", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "D03761", "Texas Instruments" },
    { "D03957", "Liteon Technology Corporation" },
    { "D03972", "Texas Instruments" },
    { "D039B3", "Commscope" },
    { "D039EA", "NetApp" },
    { "D039FA", "Samsung Electronics Co.,Ltd" },
    { "D03BF4", "Shenzhen Kean Digital Co.,Ltd" },
    { "D03C1F", "Intel Corporate" },
    { "D03D52", "Ava Security Limited" },
    { "D03DC3", "AQ Corporation" },
    { "D03E07", "Apple, Inc." },
    { "D03E5C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D03E7D", "CHIPSEA TECHNOLOGIES (SHENZHEN) CORP." },
    { "D03F27", "Wyze Labs Inc" },
    { "D03FAA", "Apple, Inc." },
    { "D040BE", "NPO RPS LLC" },
    { "D040EF", "Murata Manufacturing Co., Ltd." },
    { "D041C9", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "D0431E", "Dell Inc." },
    { "D04433", "Clourney Semiconductor" },
    { "D0460C", "Dell Inc." },
    { "D046DC", "Southwest Research Institute" },
    { "D047C1", "Elma Electronic AG" },
    { "D0484F", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "D048A1", "Juniper Networks" },
    { "D048F3", "DATTUS Inc" },
    { "D0497C", "OnePlus Technology (Shenzhen) Co., Ltd" },
    { "D0498B", "ZOOM SERVER" },
    { "D04CC1", "SINTRONES Technology Corp." },
    { "D04D2C", "Roku, Inc." },
    { "D04D86", "Apple, Inc." },
    { "D04DC6", "Hewlett Packard Enterprise" },
    { "D04E50", "Mobiwire Mobiles (NingBo) Co., LTD" },
    { "D04E99", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D04F58", "Ruckus Wireless" },
    { "D04F7E", "Apple, Inc." },
    { "D04FAB", "Yoqu Technology (Shenzhen) Co., Ltd." },
    { "D05099", "ASRock Incorporation" },
    { "D05157", "LEAX Arkivator Telecom" },
    { "D05162", "Sony Corporation" },
    { "D052A8", "Physical Graph Corporation" },
    { "D05349", "Liteon Technology Corporation" },
    { "D0542D", "Cambridge Industries(Group) Co.,Ltd." },
    { "D05475", "SAVI Controls" },
    { "D05509", "Nintendo Co.,Ltd" },
    { "D055B2", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "D056BF", "AMOSENSE" },
    { "D056FB", "Samsung Electronics Co.,Ltd" },
    { "D0574C", "Cisco Systems, Inc" },
    { "D0577B", "Intel Corporate" },
    { "D0577E", "Intel Corporate" },
    { "D05785", "Pantech Co., Ltd." },
    { "D05794", "Sagemcom Broadband SAS" },
    { "D057A1", "Werma Signaltechnik GmbH & Co. KG" },
    { "D05875", "Active Control Technology Inc." },
    { "D058A5", "Apple, Inc." },
    { "D058A8", "zte corporation" },
    { "D058AB", "Mara Tech LLC" },
    { "D058C0", "Qingdao Haier Multimedia Limited. " },
    { "D058FC", "SKY UK LIMITED" },
    { "D05919", "zte corporation" },
    { "D05995", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "D059C3", "CeraMicro Technology Corporation" },
    { "D059E4", "Samsung Electronics Co.,Ltd" },
    { "D05A00", "Vantiva USA LLC" },
    { "D05A0F", "I-BT DIGITAL CO.,LTD" },
    { "D05AF1", "Shenzhen Pulier Tech CO.,Ltd" },
    { "D05AFD", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "D05BA8", "zte corporation" },
    { "D05BCB", "Tianyi Telecom Terminals Company Limited" },
    { "D05C7A", "Sartura d.o.o." },
    { "D05F64", "IEEE Registration Authority" },
    { "D05FAF", "Shenzhen C-Data Technology Co., Ltd." },
    { "D05FB8", "Texas Instruments" },
    { "D05FCE", "Hitachi Data Systems" },
    { "D0608C", "zte corporation" },
    { "D06158", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D0622C", "Xi'an Yipu Telecom Technology Co.,Ltd." },
    { "D062A0", "China Essence Technology (Zhumadian) Co., Ltd." },
    { "D0634D", "Meiko Maschinenbau GmbH &amp; Co. KG" },
    { "D063B4", "SolidRun Ltd." },
    { "D06544", "Apple, Inc." },
    { "D06578", "Intel Corporate" },
    { "D065B3", "TCL King Electrical Appliances(Huizhou)Co.,Ltd" },
    { "D065CA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D0666D", "Shenzhen Bus-Lan Technology Co., Ltd." },
    { "D0667B", "Samsung Electronics Co.,Ltd" },
    { "D06726", "Hewlett Packard Enterprise " },
    { "D067E5", "Dell Inc." },
    { "D06827", "eero inc." },
    { "D0699E", "LUMINEX Lighting Control Equipment" },
    { "D069C1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D069D0", "Verto Medical Solutions, LLC" },
    { "D069FF", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "D06A1F", "BSE CO.,LTD." },
    { "D06B78", "Apple, Inc." },
    { "D06C37", "ikeja wireless (pty) ltd" },
    { "D06DC8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D06DC9", "Sagemcom Broadband SAS" },
    { "D06EDE", "Sagemcom Broadband SAS" },
    { "D06F4A", "TOPWELL INTERNATIONAL HOLDINGS LIMITED" },
    { "D06F82", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D071C4", "zte corporation" },
    { "D072DC", "Cisco Systems, Inc" },
    { "D0737F", "Mini-Circuits" },
    { "D07380", "Huawei Device Co., Ltd." },
    { "D0738E", "DONG OH PRECISION CO., LTD. " },
    { "D073D5", "LIFI LABS MANAGEMENT PTY LTD" },
    { "D075BE", "Reno A&E" },
    { "D07602", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "D0760C", "Qisda Corporation" },
    { "D07650", "IEEE Registration Authority" },
    { "D0768F", "Calix Inc." },
    { "D076E7", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "D07714", "Motorola Mobility LLC, a Lenovo Company" },
    { "D077CE", "Edgecore Networks Corporation" },
    { "D07880", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "D0789A", "Vantiva - Connected Home" },
    { "D07AB5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D07B6F", "Zhuhai Yunmai Technology Co.,Ltd" },
    { "D07C2D", "Leie IOT technology Co., Ltd" },
    { "D07CB2", "Sigmastar Technology Ltd." },
    { "D07D33", "Huawei Device Co., Ltd." },
    { "D07DE5", "Forward Pay Systems, Inc." },
    { "D07E01", "Huawei Device Co., Ltd." },
    { "D07E28", "Hewlett Packard" },
    { "D07E35", "Intel Corporate" },
    { "D07FA0", "Samsung Electronics Co.,Ltd" },
    { "D07FC4", "Ou Wei Technology Co.，Ltd. of Shenzhen City" },
    { "D0817A", "Apple, Inc." },
    { "D081C5", "Juniper Networks" },
    { "D083D4", "Xtel Wireless ApS" },
    { "D084B0", "Sagemcom Broadband SAS" },
    { "D08543", "Cisco Systems, Inc" },
    { "D087B5", "SAFEMO PTE. LTD." },
    { "D087E2", "Samsung Electronics Co.,Ltd" },
    { "D0880C", "Apple, Inc." },
    { "D08999", "APCON, Inc." },
    { "D08A55", "Skullcandy" },
    { "D08A91", "Vantiva USA LLC" },
    { "D08B7E", "Passif Semiconductor" },
    { "D08CB5", "Texas Instruments" },
    { "D08CFF", "UPWIS AB" },
    { "D08E79", "Dell Inc." },
    { "D09168", "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD" },
    { "D09200", "FiRa Consortium" },
    { "D09288", "Powertek Limited" },
    { "D0929E", "Microsoft Corporation" },
    { "D092FA", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "D09380", "Ducere Technologies Pvt. Ltd." },
    { "D09395", "IEEE Registration Authority" },
    { "D093F8", "Stonestreet One LLC" },
    { "D09466", "Dell Inc." },
    { "D094CF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D095C7", "Pantech Co., Ltd." },
    { "D09686", "IEEE Registration Authority" },
    { "D096FB", "Zhone Technologies, Inc." },
    { "D097FE", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "D0989C", "ConMet" },
    { "D099D5", "Alcatel-Lucent" },
    { "D09B05", "Emtronix" },
    { "D09C30", "Foster Electric Company, Limited" },
    { "D09C7A", "Xiaomi Communications Co Ltd" },
    { "D09CAE", "vivo Mobile Communication Co., Ltd." },
    { "D09D0A", "LINKCOM" },
    { "D09DAB", "TCT mobile ltd" },
    { "D09FD9", "IEEE Registration Authority" },
    { "D0A011", "IEEE Registration Authority" },
    { "D0A0BB", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "D0A0D6", "ChengDu TD Tech" },
    { "D0A311", "Neuberger Gebäudeautomation GmbH" },
    { "D0A46F", "China Dragon Technology Limited" },
    { "D0A4B1", "Sonifex Ltd." },
    { "D0A5A6", "Cisco Systems, Inc" },
    { "D0A637", "Apple, Inc." },
    { "D0A9D3", "EM Microelectronic" },
    { "D0AA5F", "IEEE Registration Authority" },
    { "D0AB4A", "Telink Micro LLC" },
    { "D0ABD5", "Intel Corporate" },
    { "D0AD08", "HP Inc." },
    { "D0AE05", "Xiaomi Communications Co Ltd" },
    { "D0AEEC", "Alpha Networks Inc." },
    { "D0AFB6", "Linktop Technology Co., LTD" },
    { "D0B0CD", "Moen" },
    { "D0B128", "Samsung Electronics Co.,Ltd" },
    { "D0B1CA", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "D0B214", "PoeWit Inc" },
    { "D0B270", "Visteon Portuguesa, Ltd" },
    { "D0B2C4", "Vantiva USA LLC" },
    { "D0B33F", "Shenzhen TINNO Mobile Technology Corp." },
    { "D0B3C2", "Meta Platforms, Inc." },
    { "D0B45D", "Huawei Device Co., Ltd." },
    { "D0B498", "Robert Bosch LLC Automotive Electronics" },
    { "D0B523", "Bestcare Cloucal Corp." },
    { "D0B53D", "SEPRO ROBOTIQUE" },
    { "D0B5C2", "Texas Instruments" },
    { "D0B60A", "Xingluo Technology Company Limited" },
    { "D0B66F", "SERNET (SUZHOU) TECHNOLOGIES CORPORATION" },
    { "D0BAE4", "Shanghai MXCHIP Information Technology Co., Ltd." },
    { "D0BB61", "zte corporation" },
    { "D0BB80", "SHL Telemedicine International Ltd." },
    { "D0BCC1", "WEIFANG GOERTEK ELECTRONICS CO.,LTD" },
    { "D0BD01", "DS International" },
    { "D0BE2C", "CNSLink Co., Ltd." },
    { "D0BF9C", "Hewlett Packard" },
    { "D0C050", "Apple, Inc." },
    { "D0C0BF", "Actions Microelectronics" },
    { "D0C193", "SKYBELL, INC" },
    { "D0C1B1", "Samsung Electronics Co.,Ltd" },
    { "D0C1B5", "Dell Inc." },
    { "D0C1BF", "Xiaomi Communications Co Ltd" },
    { "D0C24E", "Samsung Electronics Co.,Ltd" },
    { "D0C282", "Cisco Systems, Inc" },
    { "D0C31E", "JUNGJIN Electronics Co.,Ltd" },
    { "D0C35A", "Jabil Circuit de Chihuahua" },
    { "D0C42F", "Tamagawa Seiki Co.,Ltd." },
    { "D0C5D3", "AzureWave Technology Inc." },
    { "D0C5D8", "LATECOERE" },
    { "D0C5F3", "Apple, Inc." },
    { "D0C637", "Intel Corporate" },
    { "D0C65B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D0C67F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D0C730", "zte corporation" },
    { "D0C789", "Cisco Systems, Inc" },
    { "D0C7C0", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "D0C857", "IEEE Registration Authority" },
    { "D0C901", "GLA ELECTRONICS PVT LTD" },
    { "D0C907", "Private" },
    { "D0CBDD", "eero inc." },
    { "D0CDE1", "Scientech Electronics" },
    { "D0CEC0", "Xiaomi Communications Co Ltd" },
    { "D0CEC9", "HAN CHANG" },
    { "D0CF0E", "Sagemcom Broadband SAS" },
    { "D0CF13", "Espressif Inc." },
    { "D0CF5E", "Energy Micro AS" },
    { "D0CFD8", "Huizhou Boshijie Technology Co.,Ltd" },
    { "D0D003", "Samsung Electronics Co.,Ltd" },
    { "D0D04B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D0D0FD", "Cisco Systems, Inc" },
    { "D0D212", "K2NET Co.,Ltd." },
    { "D0D23C", "Apple, Inc." },
    { "D0D286", "Beckman Coulter K.K." },
    { "D0D2B0", "Apple, Inc." },
    { "D0D3E0", "Hewlett Packard Enterprise" },
    { "D0D3FC", "Mios, Ltd." },
    { "D0D412", "ADB Broadband Italia" },
    { "D0D471", "MVTECH co., Ltd" },
    { "D0D49F", "Apple, Inc." },
    { "D0D6CC", "Wintop" },
    { "D0D783", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D0D7BE", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D0D94F", "IEEE Registration Authority" },
    { "D0DAD7", "Apple, Inc." },
    { "D0DB32", "Nokia Corporation" },
    { "D0DBB7", "Casa Systems" },
    { "D0DC2C", "Cisco Systems, Inc" },
    { "D0DD49", "Juniper Networks" },
    { "D0DD7C", "zte corporation" },
    { "D0DF9A", "Liteon Technology Corporation" },
    { "D0DFB2", "Genie Networks Limited" },
    { "D0DFC7", "Samsung Electronics Co.,Ltd" },
    { "D0E042", "Cisco Systems, Inc" },
    { "D0E140", "Apple, Inc." },
    { "D0E347", "Yoga" },
    { "D0E40B", "Wearable Inc." },
    { "D0E44A", "Murata Manufacturing Co., Ltd." },
    { "D0E54D", "Commscope" },
    { "D0E581", "Apple, Inc." },
    { "D0E782", "AzureWave Technology Inc." },
    { "D0E828", "Radiant Industries Incorporated" },
    { "D0EB03", "Zhehua technology limited" },
    { "D0EB9E", "Seowoo Inc." },
    { "D0EC35", "Cisco Systems, Inc" },
    { "D0EDFF", "ZF CVCS" },
    { "D0EE47", "Hitron Technologies. Inc" },
    { "D0EF76", "Espressif Inc." },
    { "D0EFC1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D0F0DB", "Ericsson" },
    { "D0F121", "Xi'an LINKSCI Technology Co., Ltd" },
    { "D0F27F", "BrewLogix, LLC" },
    { "D0F3F5", "Huawei Device Co., Ltd." },
    { "D0F405", "Hon Hai Precision Industry Co., Ltd." },
    { "D0F4F7", "Huawei Device Co., Ltd." },
    { "D0F520", "KYOCERA Corporation " },
    { "D0F73B", "Helmut Mauell GmbH Werk Weida" },
    { "D0F76E", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "D0F865", "ITEL MOBILE LIMITED" },
    { "D0F88C", "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd." },
    { "D0F8E7", "Shenzhen Shutong Space Technology Co., Ltd" },
    { "D0F928", "zte corporation" },
    { "D0F99B", "zte corporation" },
    { "D0FA1D", "Qihoo 360 Technology Co.,Ltd" },
    { "D0FCCC", "Samsung Electronics Co.,Ltd" },
    { "D0FCD0", "HUMAX Co., Ltd." },
    { "D0FF50", "Texas Instruments" },
    { "D0FF98", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D4000D", "Phoenix Broadband Technologies, LLC." },
    { "D40057", "MC Technologies GmbH" },
    { "D40068", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "D400CA", "Continental Automotive Systems S.R.L" },
    { "D40129", "Broadcom" },
    { "D40145", "ATW TECHNOLOGY, INC." },
    { "D4016D", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "D401C3", "Routerboard.com" },
    { "D4024A", "Delphian Systems LLC" },
    { "D404CD", "Commscope" },
    { "D404E6", "Broadcom Limited" },
    { "D404FF", "Juniper Networks" },
    { "D40598", "Commscope" },
    { "D405DE", "eero inc." },
    { "D4060F", "Texas Instruments" },
    { "D40868", "Beijing Lanxum Computer Technology CO.,LTD." },
    { "D40A9E", "GO development GmbH" },
    { "D40AA9", "Commscope" },
    { "D40ADC", "Qingdao Intelligent&Precise Electronics Co.,Ltd." },
    { "D40B1A", "HTC Corporation" },
    { "D40BB9", "Solid Semecs bv." },
    { "D40E60", "Nanjing phx-gctech Information Technology Co., Ltd" },
    { "D40F9E", "Apple, Inc." },
    { "D40FB2", "Applied Micro Electronics AME bv" },
    { "D41090", "iNFORM Systems AG" },
    { "D410CF", "Huanshun Network Science and Technology Co., Ltd." },
    { "D411A3", "Samsung Electronics Co.,Ltd" },
    { "D411D6", "ShotSpotter, Inc." },
    { "D41243", "AMPAK Technology, Inc." },
    { "D41296", "Anobit Technologies Ltd." },
    { "D412BB", "Quadrant Components Inc. Ltd" },
    { "D4136F", "Asia Pacific Brands" },
    { "D413B3", "Wu Qi Technologies,Inc." },
    { "D413F8", "Peplink International Ltd." },
    { "D41761", "Xiaomi Communications Co Ltd" },
    { "D41972", "Hewlett Packard Enterprise" },
    { "D419F6", "NXP Semiconductor (Tianjin) LTD." },
    { "D41A3F", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "D41AC8", "Nippon Printer Engineering" },
    { "D41AD1", "Zyxel Communications Corporation" },
    { "D41B81", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "D41C1C", "RCF S.P.A." },
    { "D41D71", "Palo Alto Networks" },
    { "D41E35", "TOHO Electronics INC." },
    { "D41F0C", "JAI Manufacturing" },
    { "D42000", "IEEE Registration Authority" },
    { "D4206D", "HTC Corporation" },
    { "D420B0", "Mist Systems, Inc." },
    { "D42122", "Sercomm Corporation." },
    { "D4223F", "Lenovo Mobile Communication Technology Ltd." },
    { "D4224E", "Alcatel Lucent" },
    { "D422CD", "Movella Technologies B.V." },
    { "D42493", "GW Technologies Co.,Ltd" },
    { "D424DD", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "D4258B", "Intel Corporate" },
    { "D425CC", "IEEE Registration Authority" },
    { "D42751", "Infopia Co., Ltd" },
    { "D42787", "Shanghai High-Flying Electronics Technology Co., Ltd" },
    { "D427FF", "Sagemcom Broadband SAS" },
    { "D428B2", "ioBridge, Inc." },
    { "D428D5", "TCT mobile ltd" },
    { "D429A7", "Infinix mobility limited" },
    { "D429EA", "Zimory GmbH" },
    { "D42C0F", "Commscope" },
    { "D42C3D", "Sky Light Digital Limited" },
    { "D42C44", "Cisco Systems, Inc" },
    { "D42C46", "BUFFALO.INC" },
    { "D42DC5", "i-PRO Co., Ltd." },
    { "D42F23", "Akenori PTE Ltd" },
    { "D42F4B", "Hon Hai Precision Industry Co.,LTD" },
    { "D42FCA", "Apple, Inc." },
    { "D43127", "Ruijie Networks Co.,LTD" },
    { "D4319D", "Sinwatec" },
    { "D43260", "GoPro" },
    { "D43266", "Fike Corporation" },
    { "D4351D", "Technicolor Delivery Technologies Belgium NV" },
    { "D43538", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "D4354A", "ALAXALA Networks Corporation" },
    { "D43639", "Texas Instruments" },
    { "D436DB", "Jiangsu Toppower Automotive Electronics Co., Ltd" },
    { "D437D7", "zte corporation" },
    { "D43844", "UNION MAN TECHNOLOGY CO.,LTD" },
    { "D4389C", "Sony Corporation" },
    { "D439B8", "Ciena Corporation" },
    { "D43A2C", "Google, Inc." },
    { "D43A2E", "SHENZHEN MTC CO LTD" },
    { "D43A2F", "SHENZHEN MTC CO LTD" },
    { "D43A65", "IGRS Engineering Lab Ltd." },
    { "D43AE9", "DONGGUAN ipt INDUSTRIAL CO., LTD" },
    { "D43B04", "Intel Corporate" },
    { "D43B8A", "Shenzhen Zhide technology Co., LTD" },
    { "D43D39", "Dialog Semiconductor" },
    { "D43D67", "Carma Industries Inc." },
    { "D43D7E", "Micro-Star Int'l Co, Ltd" },
    { "D43DF3", "Zyxel Communications Corporation" },
    { "D43F32", "eero inc." },
    { "D43FCB", "Commscope" },
    { "D440D0", "OCOSMOS Co., LTD" },
    { "D440F0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D4413F", "Gen IV Technology LLC" },
    { "D44165", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "D4430E", "Zhejiang Dahua Technology Co., Ltd." },
    { "D4438A", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "D443A8", "Changzhou Haojie Electric Co., Ltd." },
    { "D445E8", "Jiangxi Hongpai Technology Co., Ltd." },
    { "D44649", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D446E1", "Apple, Inc." },
    { "D4475A", "ScreenBeam, Inc." },
    { "D4482D", "Shenzhen Deejoy Lighting Technology Co.,Ltd." },
    { "D44867", "Silicon Laboratories" },
    { "D44B5E", "TAIYO YUDEN CO., LTD." },
    { "D44BB6", "Zhejiang Tmall Technology Co., Ltd." },
    { "D44C24", "Vuppalamritha Magnetic Components LTD" },
    { "D44C9C", "Shenzhen YOOBAO Technology Co.Ltd" },
    { "D44CA7", "Informtekhnika & Communication, LLC" },
    { "D44D77", "Nokia" },
    { "D44D9F", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "D44DA4", "Murata Manufacturing Co., Ltd." },
    { "D44F67", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D44F68", "Eidetic Communications Inc" },
    { "D44F80", "Kemper Digital GmbH" },
    { "D4503F", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "D4507A", "CEIVA Logic, Inc" },
    { "D450EE", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "D4522A", "TangoWiFi.com" },
    { "D45251", "IBT Ingenieurbureau Broennimann Thun" },
    { "D45297", "nSTREAMS Technologies, Inc." },
    { "D452C7", "Beijing L&S Lancom Platform Tech. Co., Ltd." },
    { "D452EE", "SKY UK LIMITED" },
    { "D4532A", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "D45347", "Merytronic 2012, S.L." },
    { "D45383", "Murata Manufacturing Co., Ltd." },
    { "D453AF", "VIGO System S.A." },
    { "D4548B", "Intel Corporate" },
    { "D45556", "Fiber Mountain Inc." },
    { "D455BE", "SHENZHEN FAST TECHNOLOGIES CO.,LTD" },
    { "D45763", "Apple, Inc." },
    { "D45800", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "D45944", "tonies GmbH" },
    { "D45A3F", "Juniper Networks" },
    { "D45AB2", "Galleon Systems" },
    { "D45B51", "Motorola Mobility LLC, a Lenovo Company" },
    { "D45C70", "Wi-Fi Alliance" },
    { "D45D42", "Nokia Corporation" },
    { "D45D64", "ASUSTek COMPUTER INC." },
    { "D45DDF", "PEGATRON CORPORATION" },
    { "D45E89", "Motorola Mobility LLC, a Lenovo Company" },
    { "D45EEC", "Beijing Xiaomi Electronics Co., Ltd." },
    { "D45F25", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "D45F2C", "zte corporation" },
    { "D45F7A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D46075", "Baidu Online Network Technology (Beijing) Co., Ltd" },
    { "D460E3", "Sercomm Corporation." },
    { "D4612E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D46132", "Pro Concept Manufacturer Co.,Ltd." },
    { "D46137", "IEEE Registration Authority" },
    { "D46195", "zte corporation" },
    { "D4619D", "Apple, Inc." },
    { "D461DA", "Apple, Inc." },
    { "D461FE", "Hangzhou H3C Technologies Co., Limited" },
    { "D462EA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D46352", "Vutility Inc." },
    { "D463C0", "Apple, Inc." },
    { "D463C6", "Motorola Mobility LLC, a Lenovo Company" },
    { "D463DE", "vivo Mobile Communication Co., Ltd." },
    { "D463FE", "Arcadyan Corporation" },
    { "D464F7", "CHENGDU USEE DIGITAL TECHNOLOGY CO., LTD" },
    { "D46624", "Cisco Systems, Inc" },
    { "D466A8", "Riedo Networks Ltd" },
    { "D46761", "XonTel Technology Co." },
    { "D467D3", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "D467E7", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "D4684D", "Ruckus Wireless" },
    { "D46867", "Neoventus Design Group" },
    { "D468AA", "Apple, Inc." },
    { "D468BA", "Shenzhen Sundray Technologies company Limited" },
    { "D469A5", "Miura Systems Ltd." },
    { "D46A35", "Cisco Systems, Inc" },
    { "D46A6A", "Hon Hai Precision Ind. Co.,Ltd." },
    { "D46A91", "SnapAV" },
    { "D46AA8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D46BA6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D46C62", "MultiTracks.com, LLC" },
    { "D46C6D", "Commscope" },
    { "D46CBF", "Goodrich ISR" },
    { "D46CDA", "CSM GmbH" },
    { "D46D50", "Cisco Systems, Inc" },
    { "D46D6D", "Intel Corporate" },
    { "D46E0E", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "D46E5C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D46F42", "WAXESS USA Inc" },
    { "D47208", "Bragi GmbH" },
    { "D47226", "zte corporation" },
    { "D47350", "DBG Commnunications Technology Co., Ltd." },
    { "D47415", "Huawei Device Co., Ltd." },
    { "D4741B", "Beijing HuaDa ZhiBao Electronic System Co.,Ltd." },
    { "D476A0", "Fortinet, Inc." },
    { "D476EA", "zte corporation" },
    { "D4772B", "Nanjing Ztlink Network Technology Co.,Ltd" },
    { "D47798", "Cisco Systems, Inc" },
    { "D477B2", "Netix Global B.V." },
    { "D47856", "Avaya Inc" },
    { "D4789B", "Cisco Systems, Inc" },
    { "D47954", "Huawei Device Co., Ltd." },
    { "D479C3", "Cameronet GmbH & Co. KG" },
    { "D47A97", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "D47AE2", "Samsung Electronics Co.,Ltd" },
    { "D47AEC", "Funshion Online Technologies Co.,Ltd" },
    { "D47B35", "NEO Monitors AS" },
    { "D47B6B", "Shanghai Cygnus Semiconductor Co., Ltd." },
    { "D47B75", "HARTING Electronics GmbH" },
    { "D47BB0", "ASKEY COMPUTER CORP" },
    { "D47C44", "IEEE Registration Authority" },
    { "D47DFC", "TECNO MOBILE LIMITED" },
    { "D47EE4", "China Mobile IOT Company Limited" },
    { "D47F35", "Cisco Systems, Inc" },
    { "D47F78", "Dopple B.V." },
    { "D4808B", "Seiko Epson Corporation" },
    { "D481CA", "iDevices, LLC" },
    { "D481D7", "Dell Inc." },
    { "D4823E", "Argosy Technologies, Ltd." },
    { "D48304", "SHENZHEN FAST TECHNOLOGIES CO.,LTD" },
    { "D48409", "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "D48457", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "D484D0", "Shanghai Xiaodu Technology Limited" },
    { "D48564", "Hewlett Packard" },
    { "D48660", "Arcadyan Corporation" },
    { "D487CC", "Sivantos GmbH" },
    { "D487D8", "Samsung Electronics Co.,Ltd" },
    { "D4883F", "HDPRO CO., LTD." },
    { "D48866", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D48890", "Samsung Electronics Co.,Ltd" },
    { "D48A39", "Samsung Electronics Co.,Ltd" },
    { "D48A3B", "HUNAN FN-LINK TECHNOLOGY LIMITED" },
    { "D48AFC", "Espressif Inc." },
    { "D48C49", "Espressif Inc." },
    { "D48CB5", "Cisco Systems, Inc" },
    { "D48D26", "LG Innotek" },
    { "D48DD9", "Meld Technology, Inc" },
    { "D48F33", "Microsoft Corporation" },
    { "D48FA2", "Huawei Device Co., Ltd." },
    { "D48FAA", "Sogecam Industrial, S.A." },
    { "D4909C", "Apple, Inc." },
    { "D490E0", "Wachendorff Automation GmbH & CO.KG" },
    { "D4910F", "Amazon Technologies Inc." },
    { "D491AF", "Electroacustica General Iberica, S.A." },
    { "D49234", "NEC Corporation" },
    { "D4925E", "Technicolor Delivery Technologies Belgium NV" },
    { "D492B9", "ORION NOVA, S.L." },
    { "D49390", "CLEVO CO." },
    { "D49398", "Nokia Corporation" },
    { "D493A0", "Fidelix Oy" },
    { "D49400", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D4945A", "COSMO CO., LTD" },
    { "D494A1", "Texas Instruments" },
    { "D494E8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D494FB", "Continental Automotive Systems Inc." },
    { "D49524", "Clover Network, Inc." },
    { "D4955D", "zte corporation" },
    { "D4958E", "Universal Electronics, Inc." },
    { "D496DF", "SUNGJIN C&T CO.,LTD" },
    { "D4970B", "Xiaomi Communications Co Ltd" },
    { "D498B9", "vivo Mobile Communication Co., Ltd." },
    { "D4996C", "Juniper Networks" },
    { "D49A20", "Apple, Inc." },
    { "D49AA0", "VNPT TECHNOLOGY" },
    { "D49AF6", "AzureWave Technology Inc." },
    { "D49B5C", "Chongqing Miedu Technology Co., Ltd." },
    { "D49B74", "Kinetic Technologies" },
    { "D49C28", "JayBird LLC" },
    { "D49C53", "NETCRAZE LLC" },
    { "D49C8E", "University of FUKUI" },
    { "D49CDD", "AMPAK Technology,Inc." },
    { "D49CF4", "Palo Alto Networks" },
    { "D49D9D", "Shenzhen Goodocom lnformation Technology Co.,Ltd." },
    { "D49DC0", "Samsung Electronics Co.,Ltd" },
    { "D49E05", "zte corporation" },
    { "D49E3B", "Guangzhou Shiyuan Electronic Technology Company Limited" },
    { "D49E6D", "Wuhan Zhongyuan Huadian Science & Technology Co.," },
    { "D49FDD", "Huawei Device Co., Ltd." },
    { "D4A02A", "Cisco Systems, Inc" },
    { "D4A0FB", "IEEE Registration Authority" },
    { "D4A148", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D4A23D", "New H3C Technologies Co., Ltd" },
    { "D4A2CD", "Dell Inc." },
    { "D4A33D", "Apple, Inc." },
    { "D4A365", "Xiaomi Communications Co Ltd" },
    { "D4A38B", "ELE(GROUP)CO.,LTD" },
    { "D4A3EB", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "D4A425", "SMAX Technology Co., Ltd." },
    { "D4A499", "InView Technology Corporation" },
    { "D4A5B4", "Hengji Jiaye (Hangzhou) Technology Co., Ltd" },
    { "D4A651", "Tuya Smart Inc." },
    { "D4A923", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D4A928", "GreenWave Reality Inc" },
    { "D4AAFF", "MICRO WORLD " },
    { "D4AB61", "Intel Corporate" },
    { "D4AB82", "Commscope" },
    { "D4ABCD", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "D4AC4E", "BODi rS, LLC" },
    { "D4AD20", "Jinan USR IOT Technology Limited" },
    { "D4AD2D", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "D4AD71", "Cisco Systems, Inc" },
    { "D4ADBD", "Cisco Systems, Inc" },
    { "D4ADFC", "Shenzhen Intellirocks Tech. Co. Ltd." },
    { "D4AE05", "Samsung Electronics Co.,Ltd" },
    { "D4AE52", "Dell Inc." },
    { "D4AFF7", "Arista Networks" },
    { "D4B110", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D4B169", "Le Shi Zhi Xin Electronic Technology (Tianjin) Limited" },
    { "D4B27A", "Commscope" },
    { "D4B43E", "Messcomp Datentechnik GmbH" },
    { "D4B4C0", "Fortinet, Inc." },
    { "D4B5CD", "Sagemcom Broadband SAS" },
    { "D4B680", "Shanghai Linkyum Microeletronics Co.,Ltd" },
    { "D4B709", "zte corporation" },
    { "D4B761", "Sichuan AI-Link Technology Co., Ltd." },
    { "D4B7D0", "Ciena Corporation" },
    { "D4B8FF", "Home Control Singapore Pte Ltd" },
    { "D4B92F", "Vantiva USA LLC" },
    { "D4BABA", "IEEE Registration Authority" },
    { "D4BAFA", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "D4BBC8", "vivo Mobile Communication Co., Ltd." },
    { "D4BBE6", "Huawei Device Co., Ltd." },
    { "D4BD1E", "5VT Technologies,Taiwan LTd." },
    { "D4BD4F", "Ruckus Wireless" },
    { "D4BED7", "Dell Inc." },
    { "D4BED9", "Dell Inc." },
    { "D4BEDC", "Roku, Inc" },
    { "D4BF2D", "SE Controls Asia Pacific Ltd" },
    { "D4BF7F", "UPVEL" },
    { "D4C19E", "Ruckus Wireless" },
    { "D4C1C8", "zte corporation" },
    { "D4C1FC", "Nokia Corporation" },
    { "D4C3B0", "Gearlinx Pty Ltd" },
    { "D4C766", "Acentic GmbH" },
    { "D4C8B0", "Prime Electronics & Satellitics Inc." },
    { "D4C93C", "Cisco Systems, Inc" },
    { "D4C94B", "Motorola Mobility LLC, a Lenovo Company" },
    { "D4C9B2", "Quanergy Solutions Inc" },
    { "D4C9EF", "Hewlett Packard" },
    { "D4CA6D", "Routerboard.com" },
    { "D4CA6E", "u-blox AG" },
    { "D4CBAF", "Nokia Corporation" },
    { "D4CBCC", "vivo Mobile Communication Co., Ltd." },
    { "D4CEB8", "Enatel LTD" },
    { "D4CF37", "Symbolic IO" },
    { "D4CFF9", "Shenzhen SEI Robotics Co.,Ltd" },
    { "D4D184", "ADB Broadband Italia" },
    { "D4D249", "Power Ethernet" },
    { "D4D252", "Intel Corporate" },
    { "D4D2D6", "FN-LINK TECHNOLOGY LIMITED" },
    { "D4D2E5", "BKAV Corporation" },
    { "D4D4DA", "Espressif Inc." },
    { "D4D50D", "Southwest Microwave, Inc" },
    { "D4D51B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D4D659", "Meta Platforms, Inc." },
    { "D4D748", "Cisco Systems, Inc" },
    { "D4D7A9", "Shanghai Kaixiang Info Tech LTD" },
    { "D4D7CF", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "D4D853", "Intel Corporate" },
    { "D4D892", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D4D898", "Korea CNO Tech Co., Ltd" },
    { "D4D919", "GoPro" },
    { "D4DA21", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "D4DACD", "SKY UK LIMITED" },
    { "D4DC09", "Mist Systems, Inc." },
    { "D4DC85", "Edgecore Americas Networking Corporation" },
    { "D4DCCD", "Apple, Inc." },
    { "D4DD0B", "WAVETEL TECHNOLOGY LIMITED" },
    { "D4DF57", "Alpinion Medical Systems" },
    { "D4E053", "Hewlett Packard Enterprise" },
    { "D4E08E", "ValueHD Corporation" },
    { "D4E22F", "Roku, Inc" },
    { "D4E2CB", "Vantiva USA LLC" },
    { "D4E32C", "S. Siedle & Sohne" },
    { "D4E33F", "Nokia" },
    { "D4E3C5", "zte corporation" },
    { "D4E4C3", "Ciena Corporation" },
    { "D4E5C9", "Senao Networks Inc." },
    { "D4E6B7", "Samsung Electronics Co.,Ltd" },
    { "D4E853", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "D4E880", "Cisco Systems, Inc" },
    { "D4E8B2", "Samsung Electronics Co.,Ltd" },
    { "D4E90B", "CVT CO.,LTD" },
    { "D4E95E", "Texas Instruments" },
    { "D4E98A", "Intel Corporate" },
    { "D4E9F4", "Espressif Inc." },
    { "D4EA0E", "Avaya Inc" },
    { "D4EB68", "Cisco Systems, Inc" },
    { "D4EC0C", "Harley-Davidson Motor Company" },
    { "D4EC86", "LinkedHope Intelligent Technologies Co., Ltd" },
    { "D4ECAB", "vivo Mobile Communication Co., Ltd." },
    { "D4EE07", "HIWIFI Co., Ltd." },
    { "D4EEDE", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "D4F027", "Trust Power Ltd." },
    { "D4F04A", "SKY UK LIMITED" },
    { "D4F057", "Nintendo Co.,Ltd" },
    { "D4F0B4", "Napco Security Technologies" },
    { "D4F0C9", "KYOCERA Document Solutions Inc." },
    { "D4F0EA", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "D4F143", "IPROAD.,Inc" },
    { "D4F207", "DIAODIAO(Beijing)Technology CO.,Ltd" },
    { "D4F242", "Huawei Device Co., Ltd." },
    { "D4F32D", "Intel Corporate" },
    { "D4F337", "Xunison Ltd." },
    { "D4F46F", "Apple, Inc." },
    { "D4F4BE", "Palo Alto Networks" },
    { "D4F513", "Texas Instruments" },
    { "D4F527", "SIEMENS AG" },
    { "D4F547", "Google, Inc." },
    { "D4F5EF", "Hewlett Packard Enterprise " },
    { "D4F63F", "IEA S.R.L." },
    { "D4F756", "zte corporation" },
    { "D4F786", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "D4F7D5", "Sony Interactive Entertainment Inc." },
    { "D4F829", "Sagemcom Broadband SAS" },
    { "D4F921", "Qingdao Intelligent&Precise Electronics Co.,Ltd." },
    { "D4F98D", "Espressif Inc." },
    { "D4F9A1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D4FB8E", "Apple, Inc." },
    { "D4FC13", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "D4FE28", "Silicon Laboratories" },
    { "D4FF26", "OHSUNG" },
    { "D8004D", "Apple, Inc." },
    { "D80093", "Aurender Inc." },
    { "D801D0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D801EB", "Infinity Electronics Ltd" },
    { "D8028A", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "D802C0", "Extreme Networks Headquarters" },
    { "D8031A", "Laird Connectivity" },
    { "D8032A", "COMMTACT LTD" },
    { "D8052E", "Skyviia Corporation" },
    { "D806D1", "Honeywell Fire System (Shanghai) Co,. Ltd." },
    { "D806F3", "Arista Networks" },
    { "D807B6", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "D80831", "Samsung Electronics Co.,Ltd" },
    { "D808F5", "Arcadia Networks Co. Ltd. " },
    { "D8094E", "Active Brains" },
    { "D8097F", "zte corporation" },
    { "D809C3", "Cercacor Labs" },
    { "D809D6", "ZEXELON CO., LTD." },
    { "D80A42", "Shanghai Lixun Information Technology Co., Ltd." },
    { "D80A60", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D80AE6", "zte corporation" },
    { "D80B9A", "Samsung Electronics Co.,Ltd" },
    { "D80BCB", "Telink Semiconductor (Shanghai) Co., Ltd." },
    { "D80CCF", "C.G.V. S.A.S." },
    { "D80D17", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "D80DE3", "FXI TECHNOLOGIES AS" },
    { "D80E29", "vivo Mobile Communication Co., Ltd." },
    { "D80F99", "Hon Hai Precision Ind. Co.,Ltd." },
    { "D80FB5", "SHENZHEN ULTRAEASY TECHNOLOGY CO LTD" },
    { "D81068", "Murata Manufacturing Co., Ltd." },
    { "D8109F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D810CB", "Andrea Informatique" },
    { "D81265", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "D8132A", "Espressif Inc." },
    { "D81399", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "D814D6", "SURE SYSTEM Co Ltd" },
    { "D814DF", "TCL King Electrical Appliances (Huizhou) Co., Ltd" },
    { "D8150D", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "D8160A", "Nippon Electro-Sensory Devices" },
    { "D816C1", "DEWAV (HK) ELECTRONICS LIMITED" },
    { "D8182B", "Conti Temic Microelectronic GmbH" },
    { "D818D3", "Juniper Networks" },
    { "D81909", "Wiwynn Technology Service Malaysia " },
    { "D8197A", "Nuheara Ltd" },
    { "D819CE", "Telesquare" },
    { "D81BB5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D81BFE", "TWINLINX CORPORATION" },
    { "D81C14", "Compacta International, Ltd." },
    { "D81C79", "Apple, Inc." },
    { "D81D13", "Texas Instruments" },
    { "D81D72", "Apple, Inc." },
    { "D81EDD", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "D81EDE", "B&W Group Ltd" },
    { "D81F12", "Tuya Smart Inc." },
    { "D81F65", "Private" },
    { "D81FCC", "Brocade Communications Systems LLC" },
    { "D8209F", "Cubro Acronet GesmbH" },
    { "D820A2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D821DA", "SERNET (SUZHOU) TECHNOLOGIES CORPORATION" },
    { "D822F4", "Avnet Silica" },
    { "D823E0", "SPEEDTECH CORP." },
    { "D82477", "Universal Electric Corporation" },
    { "D824BD", "Cisco Systems, Inc" },
    { "D824EC", "Plenom A/S" },
    { "D82522", "Commscope" },
    { "D825B0", "Rockeetech Systems Co.,Ltd." },
    { "D825DF", "CAME UK" },
    { "D826B9", "Guangdong Coagent Electronics S&amp;T Co.,Ltd." },
    { "D826FA", "Jiangxi Zhentian Technology CO.,LTD" },
    { "D8270C", "MaxTronic International Co., Ltd." },
    { "D828C9", "General Electric Consumer and Industrial" },
    { "D82916", "Ascent Communication Technology" },
    { "D82918", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D82986", "Best Wish Technology LTD" },
    { "D829F8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D82A15", "Leitner SpA" },
    { "D82A7E", "Nokia Corporation" },
    { "D82D40", "Janz - Contagem e Gestão de Fluídos S.A." },
    { "D82D9B", "Shenzhen G.Credit Communication Technology Co., Ltd" },
    { "D82DE1", "Tricascade Inc." },
    { "D82FE6", "Zhejiang Tmall Technology Co., Ltd." },
    { "D83062", "Apple, Inc." },
    { "D8312C", "zte corporation" },
    { "D83134", "Roku, Inc" },
    { "D831CF", "Samsung Electronics Co.,Ltd" },
    { "D83214", "Tenda Technology Co.,Ltd.Dongguan branch" },
    { "D8325A", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "D832E3", "Xiaomi Communications Co Ltd" },
    { "D8337F", "Office FA.com Co.,Ltd." },
    { "D833B7", "Sagemcom Broadband SAS" },
    { "D8341C", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "D834D1", "Shenzhen Orange Digital Technology Co.,Ltd" },
    { "D834EE", "SHURE INCORPORATED" },
    { "D8365F", "Intelbras" },
    { "D8373B", "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd" },
    { "D837BE", "SHENZHEN GONGJIN ELECTRONICS CO.,LT" },
    { "D8380D", "SHENZHEN IP-COM Network Co.,Ltd" },
    { "D838FC", "Ruckus Wireless" },
    { "D83ADD", "Raspberry Pi Trading Ltd" },
    { "D83AF5", "Wideband Labs LLC" },
    { "D83BBF", "Intel Corporate" },
    { "D83BDA", "Espressif Inc." },
    { "D83C69", "Shenzhen TINNO Mobile Technology Corp." },
    { "D83D3F", "JOYNED GmbH" },
    { "D83DCC", "shenzhen UDD Technologies,co.,Ltd" },
    { "D83EEF", "COOSEA GROUP (HK) COMPANY LIMITED" },
    { "D84008", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D842AC", "Shanghai Feixun Communication Co.,Ltd." },
    { "D842E2", "Canary Connect, Inc." },
    { "D842F7", "Tozed Kangwei Tech Co.,Ltd" },
    { "D843AE", "Micro-Star INTL CO., LTD." },
    { "D843EA", "SY Electronics Ltd" },
    { "D843ED", "Suzuken" },
    { "D8445C", "DEV Tecnologia Ind Com Man Eq LTDA" },
    { "D84489", "TP-Link Systems Inc" },
    { "D8452B", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "D84567", "TECNO MOBILE LIMITED" },
    { "D84606", "Silicon Valley Global Marketing" },
    { "D84710", "Sichuan Changhong Electric Ltd." },
    { "D84732", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "D8478F", "Microchip Technology Inc." },
    { "D847BB", "Huawei Device Co., Ltd." },
    { "D848EE", "Hangzhou Xueji Technology Co., Ltd." },
    { "D8490B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D8492F", "CANON INC." },
    { "D849BF", "CELESTICA INC." },
    { "D84A2B", "zte corporation" },
    { "D84A83", "vivo Mobile Communication Co., Ltd." },
    { "D84A87", "OI ELECTRIC CO.,LTD" },
    { "D84B2A", "Cognitas Technologies, Inc." },
    { "D84C90", "Apple, Inc." },
    { "D84DB9", "Wu Qi Technologies,Inc." },
    { "D84F37", "Proxis, spol. s r.o." },
    { "D84FB8", "LG ELECTRONICS" },
    { "D850A1", "Hunan Danuo Technology Co.,LTD" },
    { "D850E6", "ASUSTek COMPUTER INC." },
    { "D852FA", "Texas Instruments" },
    { "D8539A", "Juniper Networks" },
    { "D853AD", "Cisco Meraki" },
    { "D853BC", "Lenovo Information Products (Shenzhen)Co.,Ltd" },
    { "D8543A", "Texas Instruments" },
    { "D85482", "Oxit, LLC" },
    { "D854A2", "Extreme Networks Headquarters" },
    { "D854F2", "Huawei Device Co., Ltd." },
    { "D85575", "Samsung Electronics Co.,Ltd" },
    { "D855A3", "zte corporation" },
    { "D857EF", "Samsung Electronics Co.,Ltd" },
    { "D858C6", "Katch Asset Tracking Pty Limited " },
    { "D858D7", "CZ.NIC, z.s.p.o." },
    { "D85982", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D85A49", "INGCHIPS Technology Co., Ltd" },
    { "D85B22", "Shenzhen Hohunet Technology Co., Ltd" },
    { "D85B2A", "Samsung Electronics Co.,Ltd" },
    { "D85D4C", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "D85D84", "CAx soft GmbH" },
    { "D85DE2", "Hon Hai Precision Ind. Co.,Ltd." },
    { "D85DEF", "Busch-Jaeger Elektro GmbH" },
    { "D85DFB", "Private" },
    { "D85ED3", "GIGA-BYTE TECHNOLOGY CO.,LTD." },
    { "D85F77", "Telink Semiconductor (Shanghai) Co., Ltd." },
    { "D860B0", "bioMérieux Italia S.p.A." },
    { "D860B3", "Guangdong Global Electronic Technology CO.，LTD" },
    { "D86162", "Wistron Neweb Corporation" },
    { "D86194", "Objetivos y Sevicios de Valor Añadido" },
    { "D862DB", "Eno Inc." },
    { "D8630D", "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd." },
    { "D86375", "Xiaomi Communications Co Ltd" },
    { "D8638C", "Shenzhen Dttek Technology Co., Ltd." },
    { "D86595", "Toy's Myth Inc." },
    { "D866C6", "Shenzhen Daystar Technology Co.,ltd" },
    { "D866EE", "BOXIN COMMUNICATION CO.,LTD." },
    { "D867D3", "Huawei Device Co., Ltd." },
    { "D867D9", "Cisco Systems, Inc" },
    { "D86852", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D868A0", "Samsung Electronics Co.,Ltd" },
    { "D868C3", "Samsung Electronics Co.,Ltd" },
    { "D86960", "Steinsvik" },
    { "D86B83", "Nintendo Co.,Ltd" },
    { "D86BF7", "Nintendo Co., Ltd." },
    { "D86BFC", "zte corporation" },
    { "D86C02", "Huaqin Telecom Technology Co.,Ltd" },
    { "D86C5A", "HUMAX Co., Ltd." },
    { "D86C63", "Google, Inc." },
    { "D86CE9", "Sagemcom Broadband SAS" },
    { "D86D17", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D8714D", "Texas Instruments" },
    { "D87154", "Samsung Electronics Co.,Ltd" },
    { "D87157", "Lenovo Mobile Communication Technology Ltd." },
    { "D87475", "Apple, Inc." },
    { "D87495", "zte corporation" },
    { "D874DF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D874EF", "Guangzhou Shiyuan Electronic Technology Company Limited" },
    { "D87533", "Nokia Corporation" },
    { "D8760A", "Escort, Inc." },
    { "D876AE", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D87766", "Nurivoice Co., Ltd" },
    { "D8778B", "Intelbras" },
    { "D8787F", "Ubee Interactive Co., Limited" },
    { "D878C9", "SERVERCOM (INDIA) PRIVATE LIMITED" },
    { "D878E5", "KUHN SA" },
    { "D87988", "Hon Hai Precision Ind. Co.,Ltd." },
    { "D87A3B", "Silicon Laboratories" },
    { "D87CDD", "SANIX INCORPORATED" },
    { "D87D45", "Nicent Technology Co., Ltd." },
    { "D87D7F", "Sagemcom Broadband SAS" },
    { "D87E6F", "CASCINATION AG" },
    { "D87E76", "ITEL MOBILE LIMITED" },
    { "D87EB1", "x.o.ware, inc." },
    { "D88039", "Microchip Technology Inc." },
    { "D8803C", "Anhui Huami Information Technology Company Limited" },
    { "D88083", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "D880DC", "Huawei Device Co., Ltd." },
    { "D881CE", "AHN INC." },
    { "D88332", "TaiXin Semiconductor Co., Ltd" },
    { "D88466", "Extreme Networks Headquarters" },
    { "D8860B", "IEEE Registration Authority" },
    { "D887D5", "Leadcore Technology CO.,LTD" },
    { "D88863", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D888CE", "RF Technology Pty Ltd" },
    { "D88A3B", "UNIT-EM" },
    { "D88ADC", "Huawei Device Co., Ltd." },
    { "D88B4C", "KingTing Tech." },
    { "D88C73", "zte corporation" },
    { "D88C79", "Google, Inc." },
    { "D88D5C", "Elentec" },
    { "D88DC8", "Atil Technology Co., LTD" },
    { "D88ED4", "eero inc." },
    { "D88F76", "Apple, Inc." },
    { "D890E8", "Samsung Electronics Co.,Ltd" },
    { "D8912A", "Zyxel Communications Corporation" },
    { "D89136", "Dover Fueling Solutions" },
    { "D89341", "General Electric Global Research" },
    { "D893D4", "Xiaomi Communications Co Ltd" },
    { "D89403", "Hewlett Packard Enterprise " },
    { "D89424", "Mellanox Technologies, Inc." },
    { "D8952F", "Texas Instruments" },
    { "D89563", "Taiwan Digital Streaming Co." },
    { "D89685", "GoPro" },
    { "D89695", "Apple, Inc." },
    { "D896E0", "Alibaba Cloud Computing Ltd." },
    { "D8973B", "Artesyn Embedded Technologies" },
    { "D89760", "C2 Development, Inc." },
    { "D8977C", "Grey Innovation" },
    { "D89790", "Commonwealth Scientific and Industrial Research Organisation" },
    { "D897BA", "PEGATRON CORPORATION" },
    { "D89A0D", "zte corporation" },
    { "D89A34", "Beijing SHENQI Technology Co., Ltd." },
    { "D89AC1", "Nokia" },
    { "D89B3B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D89C67", "Hon Hai Precision Ind. Co.,Ltd." },
    { "D89C8E", "Comcast Cable Corporation" },
    { "D89D67", "Hewlett Packard" },
    { "D89DB9", "eMegatech International Corp." },
    { "D89E3F", "Apple, Inc." },
    { "D89E61", "Huawei Device Co., Ltd." },
    { "D89ED4", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "D89EF3", "Dell Inc." },
    { "D8A011", "WiZ" },
    { "D8A01D", "Espressif Inc." },
    { "D8A0E6", "Chongqing Yipingfang Technology Co., Ltd." },
    { "D8A0E8", "zte corporation" },
    { "D8A105", "Syslane, Co., Ltd." },
    { "D8A25E", "Apple, Inc." },
    { "D8A315", "vivo Mobile Communication Co., Ltd." },
    { "D8A35C", "Samsung Electronics Co.,Ltd" },
    { "D8A491", "Huawei Device Co., Ltd." },
    { "D8A534", "Spectronix Corporation" },
    { "D8A6F0", "Wu Qi Technologies,Inc." },
    { "D8A6FD", "Ghost Autonomy Inc." },
    { "D8A756", "Sagemcom Broadband SAS" },
    { "D8A8C8", "zte corporation" },
    { "D8A98B", "Texas Instruments" },
    { "D8AA59", "Tonly Technology Co. Ltd " },
    { "D8AD49", "Honor Device Co., Ltd." },
    { "D8ADDD", "Sonavation, Inc." },
    { "D8AE90", "Itibia Technologies" },
    { "D8AED0", "Shanghai Engineering Science & Technology Co.,LTD CGNPC" },
    { "D8AF3B", "Hangzhou Bigbright Integrated communications system Co.,Ltd" },
    { "D8AF81", "AO " },
    { "D8AFF1", "Panasonic Appliances Company" },
    { "D8B020", "Taicang T&W Electronics" },
    { "D8B02E", "Guangzhou Zonerich Business Machine Co., LTD." },
    { "D8B04C", "Jinan USR IOT Technology Co., Ltd." },
    { "D8B053", "Xiaomi Communications Co Ltd" },
    { "D8B061", "SHENZHEN WENXUN TECHNOLOGY CO.,LTD" },
    { "D8B122", "Juniper Networks" },
    { "D8B12A", "Panasonic Mobile Communications Co.,Ltd." },
    { "D8B190", "Cisco Systems, Inc" },
    { "D8B249", "Huawei Device Co., Ltd." },
    { "D8B293", "TOPSSD" },
    { "D8B2AA", "zte corporation" },
    { "D8B32F", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "D8B370", "Ubiquiti Inc" },
    { "D8B377", "HTC Corporation" },
    { "D8B673", "Texas Instruments" },
    { "D8B6B7", "Comtrend Corporation" },
    { "D8B6C1", "NetworkAccountant, Inc." },
    { "D8B6D6", "Blu Tether Limited" },
    { "D8B8F6", "Nantworks" },
    { "D8B90E", "Triple Domain Vision Co.,Ltd." },
    { "D8BB2C", "Apple, Inc." },
    { "D8BBC1", "Micro-Star INTL CO., LTD." },
    { "D8BC38", "Espressif Inc." },
    { "D8BC59", "Shenzhen DAPU Microelectronics Co., Ltd" },
    { "D8BE1F", "Apple, Inc." },
    { "D8BE65", "Amazon Technologies Inc." },
    { "D8BF4C", "Victory Concept Electronics Limited" },
    { "D8BFC0", "Espressif Inc." },
    { "D8C068", "Netgenetech.co.,ltd." },
    { "D8C06A", "Hunantv.com Interactive Entertainment Media Co.,Ltd." },
    { "D8C0A6", "AzureWave Technology Inc." },
    { "D8C3FB", "DETRACOM" },
    { "D8C46A", "Murata Manufacturing Co., Ltd." },
    { "D8C497", "Quanta Computer Inc." },
    { "D8C4E9", "Samsung Electronics Co.,Ltd" },
    { "D8C561", "CommFront Communications Pte Ltd" },
    { "D8C678", "MitraStar Technology Corp." },
    { "D8C691", "Hichan Technology Corp." },
    { "D8C6F9", "Tracklab Inc" },
    { "D8C771", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D8C7C8", "Hewlett Packard Enterprise" },
    { "D8C80C", "Tuya Smart Inc." },
    { "D8C8E9", "Phicomm (Shanghai) Co., Ltd." },
    { "D8C99D", "EA DISPLAY LIMITED" },
    { "D8CA06", "Titan DataCenters France" },
    { "D8CB8A", "Micro-Star INTL CO., LTD." },
    { "D8CC98", "Huawei Device Co., Ltd." },
    { "D8CD2C", "WUXI NEIHUA NETWORK TECHNOLOGY CO., LTD" },
    { "D8CE3A", "Xiaomi Communications Co Ltd" },
    { "D8CF61", "Sagemcom Broadband SAS" },
    { "D8CF89", "Beijing DoSee Science and Technology Co., Ltd." },
    { "D8CF9C", "Apple, Inc." },
    { "D8CFBF", "Motorola Mobility LLC, a Lenovo Company" },
    { "D8D090", "Dell Inc." },
    { "D8D1CB", "Apple, Inc." },
    { "D8D261", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "D8D27C", "JEMA ENERGY, SA" },
    { "D8D385", "Hewlett Packard" },
    { "D8D43C", "Sony Corporation" },
    { "D8D45D", "Orbic North America" },
    { "D8D4E6", "Hytec Inter Co., Ltd." },
    { "D8D5B9", "Rainforest Automation, Inc." },
    { "D8D668", "Tuya Smart Inc." },
    { "D8D67E", "GSK CNC EQUIPMENT CO.,LTD" },
    { "D8D6F3", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "D8D723", "IDS, Inc" },
    { "D8D775", "Sagemcom Broadband SAS" },
    { "D8D866", "SHENZHEN TOZED TECHNOLOGIES CO.,LTD." },
    { "D8D8E5", "Vantiva Connected Home - Technologies Telco" },
    { "D8DA52", "APATOR S.A." },
    { "D8DAF1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "D8DC40", "Apple, Inc." },
    { "D8DCE9", "Kunshan Erlab ductless filtration system Co.,Ltd" },
    { "D8DD5F", "BALMUDA Inc." },
    { "D8DDFD", "Texas Instruments" },
    { "D8DE3A", "Apple, Inc." },
    { "D8DECE", "ISUNG CO.,LTD" },
    { "D8DF0D", "beroNet GmbH" },
    { "D8DF7A", "Quest Software, Inc." },
    { "D8E004", "Vodia Networks Inc" },
    { "D8E016", "Extreme Networks Headquarters" },
    { "D8E0B8", "BULAT LLC" },
    { "D8E0E1", "Samsung Electronics Co.,Ltd" },
    { "D8E23F", "Qingdao Haier Technology Co.,Ltd" },
    { "D8E2DF", "Microsoft Corporation" },
    { "D8E35E", "LG Innotek" },
    { "D8E3AE", "CIRTEC MEDICAL SYSTEMS" },
    { "D8E56D", "TCT mobile ltd" },
    { "D8E593", "Apple, Inc." },
    { "D8E72B", "NETSCOUT SYSTEMS INC" },
    { "D8E72F", "Chipsea Technologies (Shenzhen) Corp." },
    { "D8E743", "Wush, Inc" },
    { "D8E844", "zte corporation" },
    { "D8E952", "KEOPSYS" },
    { "D8EB46", "Google, Inc." },
    { "D8EB97", "TRENDnet, Inc." },
    { "D8EC5E", "Belkin International Inc." },
    { "D8ECE5", "Zyxel Communications Corporation" },
    { "D8ED1C", "Magna Technology SL" },
    { "D8EE42", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "D8EE78", "Moog Protokraft" },
    { "D8EF42", "Huawei Device Co., Ltd." },
    { "D8EFCD", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "D8F0F2", "Zeebo Inc" },
    { "D8F12E", "TP-Link Systems Inc." },
    { "D8F15B", "Espressif Inc." },
    { "D8F1D8", "OI ELECTRIC CO.,LTD" },
    { "D8F1F0", "Pepxim International Limited" },
    { "D8F2CA", "Intel Corporate" },
    { "D8F3BC", "Liteon Technology Corporation" },
    { "D8F3DB", "Post CH AG" },
    { "D8F507", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "D8F710", "Libre Wireless Technologies Inc." },
    { "D8F760", "EM Microelectronic" },
    { "D8F883", "Intel Corporate" },
    { "D8F8AF", "DAONTEC" },
    { "D8FB11", "AXACORE" },
    { "D8FB5E", "ASKEY COMPUTER CORP" },
    { "D8FB68", "Cloud Corner Ltd." },
    { "D8FBD6", "Amazon Technologies Inc." },
    { "D8FC38", "Giantec Semiconductor Inc" },
    { "D8FC93", "Intel Corporate" },
    { "D8FE8F", "IDFone Co., Ltd." },
    { "D8FEE3", "D-Link International" },
    { "D8FFC3", "Shenzhen 3SNIC information technology company Limited" },
    { "DC0077", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "DC00B0", "FREEBOX SAS" },
    { "DC0265", "Meditech Kft" },
    { "DC028E", "zte corporation" },
    { "DC0398", "LG Innotek" },
    { "DC052F", "National Products Inc." },
    { "DC0539", "Cisco Systems, Inc" },
    { "DC0575", "SIEMENS ENERGY AUTOMATION" },
    { "DC05ED", "Nabtesco Corporation" },
    { "DC0675", "Espressif Inc." },
    { "DC0682", "Accessia Technology Ltd." },
    { "DC07C1", "HangZhou QiYang Technology Co.,Ltd." },
    { "DC07F8", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "DC080F", "Apple, Inc." },
    { "DC0856", "Alcatel-Lucent Enterprise" },
    { "DC08DA", "ASKEY COMPUTER CORP" },
    { "DC0914", "Talk-A-Phone Co." },
    { "DC094C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "DC0B09", "Cisco Systems, Inc" },
    { "DC0B1A", "ADB Broadband Italia" },
    { "DC0B34", "LG Electronics (Mobile Communications)" },
    { "DC0C2D", "WEIFANG GOERTEK ELECTRONICS CO.,LTD" },
    { "DC0C5C", "Apple, Inc." },
    { "DC0D30", "Shenzhen Feasycom Technology Co., Ltd." },
    { "DC0E96", "Palo Alto Networks" },
    { "DC0EA1", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "DC1057", "Apple, Inc." },
    { "DC121D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "DC152D", "China Mobile Group Device Co.,Ltd." },
    { "DC15C8", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "DC15DB", "Ge Ruili Intelligent Technology ( Beijing ) Co., Ltd." },
    { "DC16A2", "Medtronic Diabetes" },
    { "DC16B2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "DC175A", "Hitachi High-Technologies Corporation" },
    { "DC1792", "Captivate Network" },
    { "DC1A01", "Ecoliv Technology ( Shenzhen ) Ltd." },
    { "DC1AC5", "vivo Mobile Communication Co., Ltd." },
    { "DC1BA1", "Intel Corporate" },
    { "DC1D9F", "U & B tech" },
    { "DC1DD4", "Microstep-MIS spol. s r.o." },
    { "DC1EA3", "Accensus LLC" },
    { "DC1ED5", "Espressif Inc." },
    { "DC2008", "ASD Electronics Ltd " },
    { "DC2148", "Intel Corporate" },
    { "DC215C", "Intel Corporate" },
    { "DC21B9", "Sentec Co.Ltd" },
    { "DC21E2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "DC233B", "Extreme Networks Headquarters" },
    { "DC2727", "Huawei Device Co., Ltd." },
    { "DC2834", "HAKKO Corporation" },
    { "DC2919", "AltoBeam (Xiamen) Technology Ltd, Co." },
    { "DC293A", "Shenzhen Nuoshi Technology Co., LTD." },
    { "DC2A14", "Shanghai Longjing Technology Co." },
    { "DC2AA1", "MedHab LLC" },
    { "DC2B2A", "Apple, Inc." },
    { "DC2B61", "Apple, Inc." },
    { "DC2B66", "InfoBLOCK S.A. de C.V." },
    { "DC2BCA", "Zera GmbH" },
    { "DC2C26", "Iton Technology Limited" },
    { "DC2C6E", "Routerboard.com" },
    { "DC2D04", "vivo Mobile Communication Co., Ltd." },
    { "DC2D3C", "Huawei Device Co., Ltd." },
    { "DC2DCB", "Beijing Unis HengYue Technology Co., Ltd." },
    { "DC2DDE", "Ledworks SRL" },
    { "DC2E6A", "HCT. Co., Ltd." },
    { "DC2E97", "Quectel Wireless Solutions Co.,Ltd." },
    { "DC2F03", "Step forward Group Co., Ltd." },
    { "DC2FFA", "GE Lighting" },
    { "DC309C", "Heyrex Limited" },
    { "DC3130", "3onedata Technology Co. Ltd." },
    { "DC31D1", "vivo Mobile Communication Co., Ltd." },
    { "DC3262", "Nanjing Qinheng Microelectronics Co., Ltd." },
    { "DC330D", "QING DAO HAIER TELECOM CO.,LTD." },
    { "DC333D", "Huawei Device Co., Ltd." },
    { "DC3350", "TechSAT GmbH" },
    { "DC35F1", "Positivo Tecnologia S.A." },
    { "DC360C", "Hitron Technologies. Inc" },
    { "DC3642", "zte corporation" },
    { "DC3643", "IEEE Registration Authority" },
    { "DC3714", "Apple, Inc." },
    { "DC3752", "GE" },
    { "DC3757", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "DC37D2", "Hunan HKT Electronic Technology Co., Ltd" },
    { "DC38E1", "Juniper Networks" },
    { "DC396F", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "DC3979", "Cisco Systems, Inc" },
    { "DC3A5E", "Roku, Inc." },
    { "DC3C2E", "Manufacturing System Insights, Inc." },
    { "DC3C84", "Ticom Geomatics, Inc." },
    { "DC3CF6", "Atomic Rules LLC" },
    { "DC3E51", "Solberg & Andersen AS" },
    { "DC3EF8", "Nokia Corporation" },
    { "DC415F", "Apple, Inc." },
    { "DC41A9", "Intel Corporate" },
    { "DC41E5", "Shenzhen Zhixin Data Service Co., Ltd." },
    { "DC42C8", "Huawei Device Co., Ltd." },
    { "DC4427", "IEEE Registration Authority" },
    { "DC446D", "Allwinner Technology Co., Ltd" },
    { "DC44B6", "Samsung Electronics Co.,Ltd" },
    { "DC4517", "Commscope" },
    { "DC4546", "Intel Corporate" },
    { "DC45B8", "Apple, Inc." },
    { "DC4628", "Intel Corporate" },
    { "DC48B2", "Baraja Pty. Ltd." },
    { "DC4965", "DASAN Newtork Solutions" },
    { "DC49C9", "CASCO SIGNAL LTD" },
    { "DC4A3E", "Hewlett Packard" },
    { "DC4A9E", "IEEE Registration Authority" },
    { "DC4BA1", "Wistron Neweb Corporation" },
    { "DC4BDD", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "DC4BFE", "Shenzhen Belon Technology CO.,LTD" },
    { "DC4D23", "MRV Comunications" },
    { "DC4EDE", "SHINYEI TECHNOLOGY CO., LTD." },
    { "DC4EF4", "Shenzhen MTN Electronics CO., Ltd" },
    { "DC4F22", "Espressif Inc." },
    { "DC503A", "Nanjing Ticom Tech Co., Ltd." },
    { "DC5193", "zte corporation" },
    { "DC5285", "Apple, Inc." },
    { "DC5360", "Intel Corporate" },
    { "DC537C", "Compal Broadband Networks, Inc." },
    { "DC5392", "Apple, Inc." },
    { "DC543D", "ITEL MOBILE LIMITED" },
    { "DC5475", "Espressif Inc." },
    { "DC54AD", "Hangzhou RunZhou Fiber Technologies Co.,Ltd" },
    { "DC54D7", "Amazon Technologies Inc." },
    { "DC5583", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "DC567B", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "DC56E6", "Shenzhen Bococom Technology Co.,LTD" },
    { "DC56E7", "Apple, Inc." },
    { "DC5726", "Power-One" },
    { "DC58BC", "Thomas-Krenn.AG" },
    { "DC5E36", "Paterson Technology" },
    { "DC60A1", "Teledyne DALSA Professional Imaging" },
    { "DC6180", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "DC621F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "DC6279", "TP-Link Systems Inc" },
    { "DC6294", "Guangzhou Lango Electronics Technology Co.,Ltd." },
    { "DC6373", "OBARA KOREA" },
    { "DC647C", "C.R.S. iiMotion GmbH" },
    { "DC64B8", "Shenzhen JingHanDa Electronics Co.Ltd" },
    { "DC6555", "New H3C Intelligence Terminal Co., Ltd." },
    { "DC663A", "Apacer Technology Inc." },
    { "DC6672", "Samsung Electronics Co.,Ltd" },
    { "DC6723", "barox Kommunikation GmbH" },
    { "DC680C", "Hewlett Packard Enterprise " },
    { "DC6880", "zte corporation" },
    { "DC68EB", "Nintendo Co.,Ltd" },
    { "DC69B5", "eero inc." },
    { "DC69E2", "Samsung Electronics Co.,Ltd" },
    { "DC6AE7", "Xiaomi Communications Co Ltd" },
    { "DC6AEA", "Infinix mobility limited" },
    { "DC6B12", "worldcns inc." },
    { "DC6B1B", "Huawei Device Co., Ltd." },
    { "DC6DBC", "Apple, Inc." },
    { "DC6DCD", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "DC6F00", "Livescribe, Inc." },
    { "DC6F08", "Bay Storage Technology" },
    { "DC7014", "Private" },
    { "DC7035", "Shengzhen Gongjin Electronics" },
    { "DC7137", "zte corporation" },
    { "DC7144", "SAMSUNG ELECTRO MECHANICS CO., LTD." },
    { "DC7196", "Intel Corporate" },
    { "DC71D0", "Apple, Inc." },
    { "DC71DD", "AX Technologies" },
    { "DC7223", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "DC729B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "DC7306", "Vantiva Connected Home - Home Networks" },
    { "DC7385", "Huawei Device Co., Ltd." },
    { "DC74A8", "Samsung Electronics Co.,Ltd" },
    { "DC76C3", "IEEE Registration Authority" },
    { "DC774C", "Cisco Systems, Inc" },
    { "DC7794", "Huawei Device Co., Ltd." },
    { "DC7834", "LOGICOM SA" },
    { "DC7B94", "Cisco Systems, Inc" },
    { "DC7CF7", "China Mobile Group Device Co.,Ltd." },
    { "DC7E1D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "DC7FA4", "2Wire Inc" },
    { "DC8084", "Apple, Inc." },
    { "DC813D", "SHANGHAI XIANGCHENG COMMUNICATION TECHNOLOGY CO., LTD" },
    { "DC825B", "JANUS, spol. s r.o." },
    { "DC82F6", "iPort" },
    { "DC8403", "shenzhen trolink Technology Co.,Ltd" },
    { "DC84E9", "Shenzhen Qihoo Intelligent Technology Co.,Ltd" },
    { "DC85DE", "AzureWave Technology Inc." },
    { "DC868D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "DC86D8", "Apple, Inc." },
    { "DC87CB", "Beijing Perfectek Technologies Co., Ltd." },
    { "DC88A1", "ITEL MOBILE LIMITED" },
    { "DC8983", "Samsung Electronics Co.,Ltd" },
    { "DC8B28", "Intel Corporate" },
    { "DC8C1B", "vivo Mobile Communication Co., Ltd." },
    { "DC8C37", "Cisco Systems, Inc" },
    { "DC8D8A", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "DC8D91", "Infinix mobility limited" },
    { "DC8DB7", "ATW TECHNOLOGY, INC." },
    { "DC8E8D", "Netis Technology Co., Ltd." },
    { "DC8E95", "Silicon Laboratories" },
    { "DC9009", "Intel Corporate" },
    { "DC9020", "RURU TEK PRIVATE LIMITED " },
    { "DC9088", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "DC9166", "Huawei Device Co., Ltd." },
    { "DC91BF", "Amazon Technologies Inc." },
    { "DC9272", "Sagemcom Broadband SAS" },
    { "DC9396", "Apple, Inc." },
    { "DC9566", "Apple, Inc." },
    { "DC962C", "NST Audio Ltd" },
    { "DC973A", "Verana Networks" },
    { "DC9758", "Sichuan AI-Link Technology Co., Ltd." },
    { "DC97BA", "Intel Corporate" },
    { "DC97E6", "Sagemcom Broadband SAS" },
    { "DC9840", "Microsoft Corporation" },
    { "DC9914", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "DC99FE", "Armatura LLC" },
    { "DC9A7D", "HISENSE VISUAL TECHNOLOGY CO.,LTD" },
    { "DC9A8E", "Nanjing Cocomm electronics co., LTD" },
    { "DC9B1E", "Intercom, Inc." },
    { "DC9B95", "Phyplus Technology (Shanghai) Co., Ltd" },
    { "DC9B9C", "Apple, Inc." },
    { "DC9BD6", "TCT mobile ltd" },
    { "DC9C52", "Sapphire Technology Limited." },
    { "DC9C99", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "DC9C9F", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "DC9E8F", "Apple, Inc." },
    { "DC9EAB", "Chongqing Yipingfang Technology Co., Ltd." },
    { "DC9FA4", "Nokia Corporation" },
    { "DC9FDB", "Ubiquiti Inc" },
    { "DCA0D0", "Amazon Technologies Inc." },
    { "DCA120", "Nokia" },
    { "DCA266", "Hon Hai Precision Ind. Co.,Ltd." },
    { "DCA281", "Honor Device Co., Ltd." },
    { "DCA313", "Shenzhen Changjin Communication Technology Co.,Ltd" },
    { "DCA333", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "DCA3A2", "Feng mi(Beijing)technology co., LTD" },
    { "DCA3AC", "RBcloudtech" },
    { "DCA4CA", "Apple, Inc." },
    { "DCA5F4", "Cisco Systems, Inc" },
    { "DCA632", "Raspberry Pi Trading Ltd" },
    { "DCA633", "Commscope" },
    { "DCA6BD", "Beijing Lanbo Technology Co., Ltd." },
    { "DCA706", "CHENGDU KT ELECTRONIC HI-TECH CO.,LTD" },
    { "DCA782", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "DCA7D9", "Compressor Controls Corp" },
    { "DCA8CF", "New Spin Golf, LLC." },
    { "DCA904", "Apple, Inc." },
    { "DCA956", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "DCA971", "Intel Corporate" },
    { "DCA989", "MACANDC" },
    { "DCAA43", "Shenzhen Terca Information Technology Co., Ltd." },
    { "DCAC6F", "Everytale Inc" },
    { "DCAD9E", "GreenPriz" },
    { "DCAE04", "CELOXICA Ltd" },
    { "DCAEEB", "Ruckus Wireless" },
    { "DCAF68", "WEIFANG GOERTEK ELECTRONICS CO.,LTD" },
    { "DCB058", "Bürkert Werke GmbH" },
    { "DCB082", "Nokia" },
    { "DCB131", "SHENZHEN HUARUIAN TECHNOLOGY CO.,LTD" },
    { "DCB347", "SHENZHEN FAST TECHNOLOGIES CO.,LTD" },
    { "DCB3B4", "Honeywell Environmental & Combustion Controls (Tianjin) Co., Ltd." },
    { "DCB4AC", "FLEXTRONICS MANUFACTURING(ZHUHAI)CO.,LTD." },
    { "DCB4C4", "Microsoft XCG" },
    { "DCB4CA", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "DCB4D9", "Espressif Inc." },
    { "DCB54F", "Apple, Inc." },
    { "DCB72E", "Xiaomi Communications Co Ltd" },
    { "DCB7AC", "Hewlett Packard Enterprise" },
    { "DCB7FC", "Alps Electric (Ireland) Ltd" },
    { "DCB808", "Extreme Networks Headquarters" },
    { "DCBB3D", "Extreme Networks Headquarters" },
    { "DCBB96", "Full Solution Telecom" },
    { "DCBD7A", "Guangzhou Shiyuan Electronic Technology Company Limited" },
    { "DCBDCC", "Quectel Wireless Solutions Co.,Ltd." },
    { "DCBE04", "Texas Instruments" },
    { "DCBE49", "ITEL MOBILE LIMITED" },
    { "DCBE7A", "Zhejiang Nurotron Biotechnology Co." },
    { "DCBF90", "HUIZHOU QIAOXING TELECOMMUNICATION INDUSTRY CO.,LTD." },
    { "DCBFE9", "Motorola Mobility LLC, a Lenovo Company" },
    { "DCC0DB", "Shenzhen Kaiboer Technology Co., Ltd." },
    { "DCC0EB", "ASSA ABLOY CÔTE PICARDE" },
    { "DCC101", "SOLiD Technologies, Inc." },
    { "DCC2C9", "CANON INC." },
    { "DCC422", "Systembase Limited" },
    { "DCC49C", "Samsung Electronics Co.,Ltd" },
    { "DCC622", "BUHEUNG SYSTEM" },
    { "DCC64B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "DCC793", "Nokia Corporation" },
    { "DCC8F5", "Shanghai UMEinfo CO.,LTD." },
    { "DCCBA8", "Explora Technologies Inc" },
    { "DCCC8D", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "DCCCE6", "Samsung Electronics Co.,Ltd" },
    { "DCCD18", "Nintendo Co.,Ltd" },
    { "DCCD2F", "Seiko Epson Corporation" },
    { "DCCD66", "NXP Semiconductor (Tianjin) LTD." },
    { "DCCD74", "Japan E.M.Solutions Co., Ltd." },
    { "DCCE41", "FE GLOBAL HONG KONG LIMITED" },
    { "DCCEBC", "Shenzhen JSR Technology Co.,Ltd." },
    { "DCCEC1", "Cisco Systems, Inc" },
    { "DCCF94", "Beijing Rongcheng Hutong Technology Co., Ltd." },
    { "DCCF96", "Samsung Electronics Co.,Ltd" },
    { "DCD0F7", "Bentek Systems Ltd." },
    { "DCD160", "Tianjin Changdatong Technology Co.,LTD" },
    { "DCD255", "Kinpo Electronics, Inc." },
    { "DCD26A", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "DCD2FC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "DCD2FD", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "DCD321", "HUMAX Co., Ltd." },
    { "DCD3A2", "Apple, Inc." },
    { "DCD444", "Huawei Device Co., Ltd." },
    { "DCD52A", "Sunny Heart Limited" },
    { "DCD7A0", "Huawei Device Co., Ltd." },
    { "DCD87C", "Beijing Jingdong Century Trading Co., LTD." },
    { "DCD87F", "Shenzhen JoinCyber Telecom Equipment Ltd" },
    { "DCD916", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "DCD9AE", "Nokia Shanghai Bell Co., Ltd." },
    { "DCDA0C", "Espressif Inc." },
    { "DCDA4D", "CELESTICA INC." },
    { "DCDA4F", "GETCK TECHNOLOGY, INC" },
    { "DCDA80", "New H3C Technologies Co., Ltd" },
    { "DCDB27", "Huawei Device Co., Ltd." },
    { "DCDB70", "Tonfunk Systementwicklung und Service GmbH" },
    { "DCDC07", "TRP Systems BV" },
    { "DCDCC3", "Extreme Networks Headquarters" },
    { "DCDCE2", "Samsung Electronics Co.,Ltd" },
    { "DCDD24", "Energica Motor Company SpA" },
    { "DCDE4F", "Gionee Communication Equipment Co Ltd " },
    { "DCDECA", "Akyllor" },
    { "DCDFD6", "zte corporation" },
    { "DCE026", "Patrol Tag, Inc" },
    { "DCE0EB", "Nanjing Aozheng Information Technology Co.Ltd" },
    { "DCE1AD", "Shenzhen Wintop Photoelectric Technology Co., Ltd" },
    { "DCE2AC", "Lumens Digital Optics Inc." },
    { "DCE305", "AO " },
    { "DCE533", "IEEE Registration Authority" },
    { "DCE55B", "Google, Inc." },
    { "DCE578", "Experimental Factory of Scientific Engineering and Special Design Department" },
    { "DCE5D8", "zte corporation" },
    { "DCE650", "Extreme Networks Headquarters" },
    { "DCE71C", "AUG Elektronik GmbH" },
    { "DCE838", "CK Telecom (Shenzhen) Limited" },
    { "DCE994", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "DCEAE7", "D-Link Corporation" },
    { "DCEB4D", "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd." },
    { "DCEB53", "Wuhan QianXiao Elecronic Technology CO.,LTD" },
    { "DCEB69", "Vantiva USA LLC" },
    { "DCEB94", "Cisco Systems, Inc" },
    { "DCEC06", "Heimi Network Technology Co., Ltd." },
    { "DCEC4F", "Guangzhou Shiyuan Electronic Technology Company Limited" },
    { "DCECE3", "HORYS TECHNOLOGIES LLC" },
    { "DCED83", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "DCED84", "Haverford Systems Inc" },
    { "DCEE06", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "DCEE14", "ADT Technology" },
    { "DCEF09", "NETGEAR" },
    { "DCEF80", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "DCEFCA", "Murata Manufacturing Co., Ltd." },
    { "DCF05D", "Letta Teknoloji" },
    { "DCF090", "Nubia Technology Co.,Ltd." },
    { "DCF110", "Nokia Corporation" },
    { "DCF31C", "Texas Instruments" },
    { "DCF401", "Dell Inc." },
    { "DCF4CA", "Apple, Inc." },
    { "DCF505", "AzureWave Technology Inc." },
    { "DCF51B", "Arcadyan Corporation" },
    { "DCF56E", "Wellysis Corp." },
    { "DCF719", "Cisco Systems, Inc" },
    { "DCF755", "SITRONIK" },
    { "DCF756", "Samsung Electronics Co.,Ltd" },
    { "DCF858", "Lorent Networks, Inc." },
    { "DCF8B9", "zte corporation" },
    { "DCFAD5", "STRONG Ges.m.b.H." },
    { "DCFB02", "BUFFALO.INC" },
    { "DCFB48", "Intel Corporate" },
    { "DCFBB8", "Meizhou Guo Wei Electronics Co., Ltd" },
    { "DCFE07", "PEGATRON CORPORATION" },
    { "DCFE18", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "DCFE23", "Murata Manufacturing Co., Ltd." },
    { "E00084", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E001A6", "Edgecore Networks Corporation" },
    { "E001C7", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "E002A5", "ABB Robotics" },
    { "E0036B", "Samsung Electronics Co.,Ltd" },
    { "E00370", "ShenZhen Continental Wireless Technology Co., Ltd." },
    { "E0051C", "GigaDevice Semiconductor Inc." },
    { "E005C5", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "E00630", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E006E6", "Hon Hai Precision Ind. Co.,Ltd." },
    { "E0071B", "Hewlett Packard Enterprise " },
    { "E007C2", "FUJIAN STAR-NET COMMUNICATION CO.,LTD" },
    { "E00855", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "E00871", "Dongguan Liesheng Electronic Co., Ltd." },
    { "E009BF", "SHENZHEN TONG BO WEI TECHNOLOGY Co.,LTD" },
    { "E00AF6", "Liteon Technology Corporation" },
    { "E00B28", "Inovonics" },
    { "E00C7F", "Nintendo Co., Ltd." },
    { "E00CE5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E00DB9", "Cree, Inc." },
    { "E00DEE", "Huawei Device Co., Ltd." },
    { "E00ECE", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "E00EDA", "Cisco Systems, Inc" },
    { "E00EE1", "We Corporation Inc." },
    { "E00EE4", "DWnet Technologies(Suzhou) Corporation" },
    { "E0107F", "Ruckus Wireless" },
    { "E01283", "Shenzhen Fanzhuo Communication Technology Co., Lt" },
    { "E01333", "General Motors" },
    { "E013B5", "vivo Mobile Communication Co., Ltd." },
    { "E0143E", "Modoosis Inc." },
    { "E016B1", "Advanced Design Technology co.,ltd." },
    { "E01877", "FUJITSU LIMITED" },
    { "E0189F", "EM Microelectronic" },
    { "E0191D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E01954", "zte corporation" },
    { "E01995", "Nutanix" },
    { "E019D8", "BH TECHNOLOGIES" },
    { "E01AEA", "Allied Telesis, Inc." },
    { "E01C41", "Extreme Networks Headquarters" },
    { "E01CA7", "Arista Networks" },
    { "E01CEE", "Bravo Tech, Inc." },
    { "E01CFC", "D-Link International" },
    { "E01D38", "Beijing HuaqinWorld Technology Co.,Ltd" },
    { "E01D3B", "Cambridge Industries(Group) Co.,Ltd." },
    { "E01E07", "Anite Telecoms US. Inc" },
    { "E01F0A", "Xslent Energy Technologies. LLC" },
    { "E01F2B", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "E01F34", "HMD Global Oy" },
    { "E01F6A", "Huawei Device Co., Ltd." },
    { "E01F88", "Xiaomi Communications Co Ltd" },
    { "E01FED", "Nokia Shanghai Bell Co., Ltd." },
    { "E01FFC", "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd." },
    { "E021FE", "Richer Link Technologies CO.,LTD" },
    { "E02202", "Commscope" },
    { "E022A1", "AltoBeam Inc." },
    { "E0233B", "IEEE Registration Authority" },
    { "E023D7", "Sleep Number" },
    { "E023FF", "Fortinet, Inc." },
    { "E0247F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E02481", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E02538", "Titan Pet Products" },
    { "E02611", "Apple, Inc." },
    { "E02630", "Intrigue Technologies, Inc." },
    { "E02636", "Nortel Networks" },
    { "E0271A", "TTC Next-generation Home Network System WG" },
    { "E0276C", "Guangzhou Shiyuan Electronic Technology Company Limited" },
    { "E0280A", "TP-Link Systems Inc." },
    { "E02861", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E0286D", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "E028B1", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "E02967", "HMD Global Oy" },
    { "E02A66", "Cisco Systems, Inc" },
    { "E02A82", "Universal Global Scientific Industrial Co., Ltd." },
    { "E02AE6", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "E02B96", "Apple, Inc." },
    { "E02BE9", "Intel Corporate" },
    { "E02CB2", "Lenovo Mobile Communication (Wuhan) Company Limited" },
    { "E02CF3", "MRS Electronic GmbH" },
    { "E02DF0", "ALPSALPINE CO,.LTD" },
    { "E02E0B", "Intel Corporate" },
    { "E02E3F", "Huawei Device Co., Ltd." },
    { "E02EFE", "Zhejiang Dahua Technology Co., Ltd." },
    { "E02F6D", "Cisco Systems, Inc" },
    { "E03005", "Alcatel-Lucent Shanghai Bell Co., Ltd" },
    { "E030F9", "Juniper Networks" },
    { "E0319E", "Valve Corporation" },
    { "E031D0", "SZ Telstar CO., LTD" },
    { "E0338E", "Apple, Inc." },
    { "E034E4", "Feit Electric Company, Inc." },
    { "E03560", "Challenger Supply Holdings, LLC" },
    { "E03676", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E036E3", "Stage One International Co., Ltd." },
    { "E03717", "Vantiva USA LLC" },
    { "E037BF", "Wistron Neweb Corporation" },
    { "E0382D", "IEEE Registration Authority" },
    { "E0383F", "zte corporation" },
    { "E039D7", "Plexxi, Inc." },
    { "E03AAA", "Intel Corporate" },
    { "E03C1C", "IEEE Registration Authority" },
    { "E03C5B", "SHENZHEN JIAXINJIE ELECTRON CO.,LTD" },
    { "E03DA6", "Taicang T&W Electronics" },
    { "E03E44", "Broadcom" },
    { "E03E4A", "Cavanagh Group International" },
    { "E03E7D", "data-complex GmbH" },
    { "E03ECB", "Qingdao Intelligent&Precise Electronics Co.,Ltd." },
    { "E03F49", "ASUSTek COMPUTER INC." },
    { "E04007", "Huawei Device Co., Ltd." },
    { "E04027", "Huawei Device Co., Ltd." },
    { "E04102", "zte corporation" },
    { "E04136", "MitraStar Technology Corp." },
    { "E0426D", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "E043DB", "Shenzhen ViewAt Technology Co.,Ltd. " },
    { "E0456D", "China Mobile Group Device Co.,Ltd." },
    { "E0469A", "NETGEAR" },
    { "E046E5", "Gosuncn Technology Group Co., Ltd." },
    { "E046EE", "NETGEAR" },
    { "E04735", "Ericsson AB" },
    { "E04824", "Garmin International" },
    { "E048AF", "Premietech Limited" },
    { "E048D3", "MOBIWIRE MOBILES (NINGBO) CO.,LTD" },
    { "E048D8", "Guangzhi Wulian Technology(Guangzhou) Co., Ltd" },
    { "E04934", "Calix Inc." },
    { "E049ED", "Audeze LLC" },
    { "E04B41", "Hangzhou Beilian Low Carbon Technology Co., Ltd." },
    { "E04B45", "Hi-P Electronics Pte Ltd" },
    { "E04BA6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E04C05", "EverCharge" },
    { "E04C12", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "E04E5D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E04E7A", "Nanjing Qinheng Microelectronics Co., Ltd." },
    { "E04F43", "Universal Global Scientific Industrial Co., Ltd." },
    { "E04FBD", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "E0508B", "Zhejiang Dahua Technology Co., Ltd." },
    { "E05124", "NXP Semiconductors" },
    { "E05163", "Arcadyan Corporation" },
    { "E051D8", "China Dragon Technology Limited" },
    { "E0553D", "Cisco Meraki" },
    { "E05597", "Emergent Vision Technologies Inc." },
    { "E05689", "Lootom Telcovideo Network (Wuxi) Co Ltd" },
    { "E05694", "Yunhight Microelectronics" },
    { "E056F4", "AxesNetwork Solutions inc." },
    { "E0589E", "Laerdal Medical" },
    { "E05A1B", "Espressif Inc." },
    { "E05A9F", "IEEE Registration Authority" },
    { "E05B70", "Innovid, Co., Ltd." },
    { "E05D54", "Ruijie Networks Co.,LTD" },
    { "E05D5C", "Oy Everon Ab" },
    { "E05DA6", "Detlef Fink Elektronik & Softwareentwicklung" },
    { "E05F45", "Apple, Inc." },
    { "E05FB9", "Cisco Systems, Inc" },
    { "E0604A", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "E06066", "Sercomm Corporation." },
    { "E06089", "Cloudleaf, Inc." },
    { "E061B2", "HANGZHOU ZENOINTEL TECHNOLOGY CO., LTD" },
    { "E06234", "Texas Instruments" },
    { "E06267", "Xiaomi Communications Co Ltd" },
    { "E06290", "Jinan Jovision Science & Technology Co., Ltd." },
    { "E063DA", "Ubiquiti Inc" },
    { "E063E5", "Sony Corporation" },
    { "E064BB", "DigiView S.r.l." },
    { "E06678", "Apple, Inc." },
    { "E06781", "Dongguan Liesheng Electronic Co., Ltd." },
    { "E067B3", "Shenzhen C-Data Technology Co., Ltd." },
    { "E0686D", "Raybased AB" },
    { "E068EE", "Phyplus Microelectronics Limited" },
    { "E0693A", "Innophase Inc." },
    { "E06995", "PEGATRON CORPORATION" },
    { "E069BA", "Cisco Systems, Inc" },
    { "E06A05", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "E06C4E", "Shenzhen TINNO Mobile Technology Corp." },
    { "E06CA6", "Creotech Instruments S.A." },
    { "E06CC5", "Huawei Device Co., Ltd." },
    { "E06CF6", "ESSENCORE limited" },
    { "E06D17", "Apple, Inc." },
    { "E06D18", "PIONEERCORPORATION" },
    { "E070EA", "HP Inc." },
    { "E0720A", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "E0735F", "NUCOM" },
    { "E073E7", "HP Inc." },
    { "E0750A", "ALPSALPINE CO,.LTD" },
    { "E07526", "China Dragon Technology Limited" },
    { "E0757D", "Motorola Mobility LLC, a Lenovo Company" },
    { "E075AA", "Beijing Jingling Information System Technology Co., Ltd." },
    { "E076D0", "AMPAK Technology, Inc." },
    { "E07726", "Huawei Device Co., Ltd." },
    { "E078A3", "Shanghai Winner Information Technology Co.,Inc" },
    { "E0795E", "Wuxi Xiaohu Technology Co.,Ltd." },
    { "E0798D", "Silicon Laboratories" },
    { "E079C4", "iRay Technology Company Limited" },
    { "E07C13", "zte corporation" },
    { "E07C62", "Whistle Labs, Inc." },
    { "E07DEA", "Texas Instruments" },
    { "E07E5F", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "E07F53", "TECHBOARD SRL" },
    { "E07F88", "EVIDENCE Network SIA" },
    { "E0806B", "Xiaomi Communications Co Ltd" },
    { "E08177", "GreenBytes, Inc." },
    { "E084F3", "High Grade Controls Corporation" },
    { "E0854D", "LG Innotek" },
    { "E0859A", "SHENZHEN RF-LINK TECHNOLOGY CO.,LTD." },
    { "E08614", "Inseego Wireless, Inc" },
    { "E087B1", "Nata-Info Ltd." },
    { "E0885D", "Vantiva USA LLC" },
    { "E0897E", "Apple, Inc." },
    { "E0899D", "Cisco Systems, Inc" },
    { "E08A7E", "Exponent" },
    { "E08C3C", "Cisco Systems, Inc" },
    { "E08CFE", "Espressif Inc." },
    { "E08E3C", "Aztech Electronics Pte Ltd" },
    { "E08F4C", "Intel Corporate" },
    { "E08FEC", "REPOTEC CO., LTD." },
    { "E0913C", "Kyeungin CNS Co., Ltd." },
    { "E09153", "XAVi Technologies Corp." },
    { "E091F5", "NETGEAR" },
    { "E0925C", "Apple, Inc." },
    { "E0928F", "Texas Instruments" },
    { "E092A7", "Feitian Technologies Co., Ltd" },
    { "E09467", "Intel Corporate" },
    { "E09579", "ORTHOsoft inc, d/b/a Zimmer CAS" },
    { "E095FF", "Infinix mobility limited" },
    { "E09796", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E097F2", "Atomax Inc." },
    { "E09806", "Espressif Inc." },
    { "E09861", "Motorola Mobility LLC, a Lenovo Company" },
    { "E09971", "Samsung Electronics Co.,Ltd" },
    { "E09B27", "Ciena Corporation" },
    { "E09C8D", "Seakeeper, Inc." },
    { "E09CE5", "Shanghai Tricheer Technology Co.,Ltd." },
    { "E09D13", "Samsung Electronics Co.,Ltd" },
    { "E09D31", "Intel Corporate" },
    { "E09D73", "Mellanox Technologies, Inc." },
    { "E09DB8", "PLANEX COMMUNICATIONS INC." },
    { "E09DFA", "Wanan Hongsheng Electronic Co.Ltd" },
    { "E09F2A", "Iton Technology Corp. " },
    { "E0A129", "Extreme Networks Headquarters" },
    { "E0A198", "NOJA Power Switchgear Pty Ltd" },
    { "E0A1CE", "zte corporation" },
    { "E0A1D7", "SFR" },
    { "E0A258", "Wanbang Digital Energy Co.,Ltd" },
    { "E0A25A", "Shanghai Mo xiang Network Technology CO.,ltd" },
    { "E0A30F", "Pevco" },
    { "E0A3AC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E0A498", "SHENZHEN ORFA TECH CO.,LTD " },
    { "E0A509", "Bitmain Technologies Inc" },
    { "E0A670", "Nokia Corporation" },
    { "E0A700", "Verkada Inc" },
    { "E0A8B8", "Le Shi Zhi Xin Electronic Technology (Tianjin) Limited" },
    { "E0AA96", "Samsung Electronics Co.,Ltd" },
    { "E0AAB0", "SUNTAILI ENTERPRISE CO. LTD," },
    { "E0AADB", "Nanjing PANENG Technology Development Co.,Ltd" },
    { "E0ABFE", "Orb Networks, Inc." },
    { "E0ACCB", "Apple, Inc." },
    { "E0ACF1", "Cisco Systems, Inc" },
    { "E0AD9B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E0AE5E", "ALPSALPINE CO,.LTD" },
    { "E0AEA2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E0AEB2", "Bender GmbH &amp; Co.KG" },
    { "E0AEED", "LOENK" },
    { "E0AF4B", "Pluribus Networks, Inc." },
    { "E0AF4F", "Deutsche Telekom AG" },
    { "E0B260", "TENO NETWORK TECHNOLOGIES COMPANY LIMITED" },
    { "E0B2F1", "FN-LINK TECHNOLOGY LIMITED" },
    { "E0B52D", "Apple, Inc." },
    { "E0B55F", "Apple, Inc." },
    { "E0B655", "Beijing Xiaomi Electronics Co., Ltd." },
    { "E0B668", "zte corporation" },
    { "E0B6F5", "IEEE Registration Authority" },
    { "E0B70A", "Commscope" },
    { "E0B72E", "ShenZhen Qualmesh Technology Co.,Ltd." },
    { "E0B763", "Bosch Automotive Products (Suzhou) Co., Ltd. Changzhou Branch" },
    { "E0B7B1", "Commscope" },
    { "E0B94D", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "E0B98A", "Shenzhen Taike industrial automation company,Ltd" },
    { "E0B9A5", "AzureWave Technology Inc." },
    { "E0B9BA", "Apple, Inc." },
    { "E0B9E5", "Technicolor Delivery Technologies Belgium NV" },
    { "E0BAAD", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "E0BAB4", "Arrcus, Inc" },
    { "E0BB0C", "Synertau LLC" },
    { "E0BB9E", "Seiko Epson Corporation" },
    { "E0BC43", "C2 Microsystems, Inc." },
    { "E0BDA0", "Apple, Inc." },
    { "E0BE03", "Lite-On Network Communication (Dongguan) Limited" },
    { "E0BFB2", "Apple, Inc." },
    { "E0C0D1", "CK Telecom (Shenzhen) Limited" },
    { "E0C250", "NETGEAR" },
    { "E0C264", "Intel Corporate" },
    { "E0C286", "Aisai Communication Technology Co., Ltd." },
    { "E0C29E", "zte corporation" },
    { "E0C2B7", "Masimo Corporation" },
    { "E0C377", "Samsung Electronics Co.,Ltd" },
    { "E0C3EA", "Apple, Inc." },
    { "E0C3F3", "zte corporation" },
    { "E0C58F", "China Mobile IOT Company Limited" },
    { "E0C63C", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "E0C6B3", "MilDef AB" },
    { "E0C767", "Apple, Inc." },
    { "E0C79D", "Texas Instruments" },
    { "E0C86A", "SHENZHEN TW-SCIE Co., Ltd" },
    { "E0C922", "Jireh Energy Tech., Ltd." },
    { "E0C97A", "Apple, Inc." },
    { "E0CA3C", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "E0CA4D", "Shenzhen Unistar Communication Co.,LTD" },
    { "E0CA94", "ASKEY COMPUTER CORP" },
    { "E0CB19", "Nokia" },
    { "E0CB1D", "Amazon Technologies Inc." },
    { "E0CB4E", "ASUSTek COMPUTER INC." },
    { "E0CB56", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "E0CBBC", "Cisco Meraki" },
    { "E0CBEE", "Samsung Electronics Co.,Ltd" },
    { "E0CC7A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E0CCF8", "Xiaomi Communications Co Ltd" },
    { "E0CDFD", "Beijing E3Control Technology Co, LTD" },
    { "E0CEC3", "ASKEY COMPUTER CORP" },
    { "E0CF2D", "Gemintek Corporation" },
    { "E0D045", "Intel Corporate" },
    { "E0D083", "Samsung Electronics Co.,Ltd" },
    { "E0D10A", "Katoudenkikougyousyo co ltd" },
    { "E0D173", "Cisco Systems, Inc" },
    { "E0D1E6", "Aliph dba Jawbone" },
    { "E0D31A", "EQUES Technology Co., Limited" },
    { "E0D362", "TP-Link Systems Inc." },
    { "E0D3B4", "Cisco Meraki" },
    { "E0D462", "Huawei Device Co., Ltd." },
    { "E0D464", "Intel Corporate" },
    { "E0D491", "Cisco Systems, Inc" },
    { "E0D4E8", "Intel Corporate" },
    { "E0D55D", "Intel Corporate" },
    { "E0D55E", "GIGA-BYTE TECHNOLOGY CO.,LTD." },
    { "E0D738", "WireStar Networks" },
    { "E0D7BA", "Texas Instruments" },
    { "E0D848", "Dell Inc." },
    { "E0D8C4", "Qingdao Intelligent&Precise Electronics Co.,Ltd." },
    { "E0D9A2", "Hippih aps" },
    { "E0D9E3", "Eltex Enterprise Ltd." },
    { "E0DA90", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E0DAD7", "zte corporation" },
    { "E0DADC", "JVC KENWOOD Corporation" },
    { "E0DB10", "Samsung Electronics Co.,Ltd" },
    { "E0DB55", "Dell Inc." },
    { "E0DB88", "Open Standard Digital-IF Interface for SATCOM Systems" },
    { "E0DBD1", "Vantiva USA LLC" },
    { "E0DCA0", "Siemens Industrial Automation Products Ltd., Chengdu" },
    { "E0DCFF", "Xiaomi Communications Co Ltd" },
    { "E0DDC0", "vivo Mobile Communication Co., Ltd." },
    { "E0DEF2", "Texas Instruments" },
    { "E0DF13", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "E0E0C2", "China Mobile Group Device Co.,Ltd." },
    { "E0E0FC", "Huawei Device Co., Ltd." },
    { "E0E1A9", "Shenzhen Four Seas Global Link Network Technology Co., Ltd." },
    { "E0E258", "Intel Corporate" },
    { "E0E2D1", "Beijing Netswift Technology Co.,Ltd." },
    { "E0E2E6", "Espressif Inc." },
    { "E0E37C", "Huawei Device Co., Ltd." },
    { "E0E5CF", "Texas Instruments" },
    { "E0E62E", "TCT mobile ltd" },
    { "E0E631", "SNB TECHNOLOGIES LIMITED" },
    { "E0E656", "Nethesis srl" },
    { "E0E6E3", "TeamF1 Networks Pvt Limited" },
    { "E0E751", "Nintendo Co., Ltd." },
    { "E0E7BB", "Nureva, Inc." },
    { "E0E8BB", "Unicom Vsens Telecommunications Co., Ltd." },
    { "E0E8E6", "Shenzhen C-Data Technology Co., Ltd." },
    { "E0E8E8", "Olive Telecommunication Pvt. Ltd" },
    { "E0EB40", "Apple, Inc." },
    { "E0EB62", "Shanghai Hulu Devices Co., Ltd" },
    { "E0ED1A", "vastriver Technology Co., Ltd" },
    { "E0EDC7", "Shenzhen Friendcom Technology Development Co., Ltd" },
    { "E0EE1B", "Panasonic Automotive Systems Company of America" },
    { "E0EF02", "Chengdu Quanjing Intelligent Technology Co.,Ltd" },
    { "E0EF25", "Lintes Technology Co., Ltd." },
    { "E0EFBF", "Nintendo Co.,Ltd" },
    { "E0F211", "Digitalwatt" },
    { "E0F318", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "E0F325", "Elkor Technologies Inc." },
    { "E0F330", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E0F379", "Vaddio" },
    { "E0F442", "Huawei Device Co., Ltd." },
    { "E0F5C6", "Apple, Inc." },
    { "E0F5CA", "CHENG UEI PRECISION INDUSTRY CO.,LTD." },
    { "E0F62D", "Juniper Networks" },
    { "E0F678", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "E0F6B5", "Nintendo Co.,Ltd" },
    { "E0F728", "Amazon Technologies Inc." },
    { "E0F847", "Apple, Inc." },
    { "E0F9BE", "Cloudena Corp." },
    { "E0FAEC", "Platan sp. z o.o. sp. k." },
    { "E0FFF1", "Texas Instruments" },
    { "E0FFF7", "Softiron Inc." },
    { "E4029B", "Intel Corporate" },
    { "E40439", "TomTom Software Ltd" },
    { "E405F8", "Bytedance" },
    { "E406BF", "Silicon Laboratories" },
    { "E4072B", "Huawei Device Co., Ltd." },
    { "E408E7", "Quectel Wireless Solutions Co.,Ltd." },
    { "E40A16", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E40CFD", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "E40D36", "Intel Corporate" },
    { "E40D3B", "Ericsson AB" },
    { "E40EEE", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E41088", "Samsung Electronics Co.,Ltd" },
    { "E4115B", "Hewlett Packard" },
    { "E41218", "ShenZhen Rapoo Technology Co., Ltd." },
    { "E4121D", "Samsung Electronics Co.,Ltd" },
    { "E41226", "Continental Automotive Romania SLR" },
    { "E41289", "topsystem GmbH" },
    { "E415F6", "Texas Instruments" },
    { "E417D8", "8BITDO TECHNOLOGY HK LIMITED" },
    { "E4186B", "Zyxel Communications Corporation" },
    { "E4197F", "eero inc." },
    { "E419C1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E41A1D", "NOVEA ENERGIES" },
    { "E41A2C", "ZPE Systems, Inc." },
    { "E41C4B", "V2 TECHNOLOGY, INC." },
    { "E41D2D", "Mellanox Technologies, Inc." },
    { "E41E0A", "IEEE Registration Authority" },
    { "E41E33", "Continental Automotive Technologies GmbH" },
    { "E41F13", "IBM Corp" },
    { "E41F7B", "Cisco Systems, Inc" },
    { "E41FD5", "Intel Corporate" },
    { "E41FE9", "Dunkermotoren GmbH" },
    { "E42150", "Shanghai Chint low voltage electrical technology Co.,Ltd." },
    { "E422A5", "PLANTRONICS, INC." },
    { "E4233C", "Juniper Networks" },
    { "E42354", "SHENZHEN FUZHI SOFTWARE TECHNOLOGY CO.,LTD" },
    { "E4246C", "Zhejiang Dahua Technology Co., Ltd." },
    { "E425E7", "Apple, Inc." },
    { "E425E9", "Color-Chip" },
    { "E42686", "DWnet Technologies(Suzhou) Corporation" },
    { "E4268B", "Huawei Device Co., Ltd." },
    { "E426D5", "Motorola Mobility LLC, a Lenovo Company" },
    { "E42761", "Honor Device Co., Ltd." },
    { "E42771", "Smartlabs" },
    { "E42805", "Pivotal Optics" },
    { "E428A4", "Prama India Private Limited" },
    { "E4293D", "Shenzhen Sy-Fiber Optical Communication Technology.Co.,Ltd" },
    { "E42AAC", "Microsoft Corporation" },
    { "E42AD3", "Magneti Marelli S.p.A. Powertrain" },
    { "E42B34", "Apple, Inc." },
    { "E42B79", "Nokia" },
    { "E42C56", "Lilee Systems, Ltd." },
    { "E42D02", "TCT mobile ltd" },
    { "E42D7B", "China Mobile IOT Company Limited" },
    { "E42F26", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "E42F56", "OptoMET GmbH" },
    { "E42FF6", "Unicore communication Inc." },
    { "E43022", "Hanwha Vision VietNam" },
    { "E432CB", "Samsung Electronics Co.,Ltd" },
    { "E433AE", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "E43493", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E43593", "Hangzhou GoTo technology Co.Ltd" },
    { "E435C8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E435FB", "Sabre Technology (Hull) Ltd" },
    { "E4379F", "Cisco Systems, Inc" },
    { "E437D7", "HENRI DEPAEPE S.A.S." },
    { "E43819", "Shenzhen Hi-Link Electronic CO.,Ltd." },
    { "E4387E", "Cisco Systems, Inc" },
    { "E43883", "Ubiquiti Inc" },
    { "E4388C", "Digital Products Limited" },
    { "E438F2", "Advantage Controls" },
    { "E43A65", "MofiNetwork Inc" },
    { "E43A6E", "Shenzhen Zeroone Technology CO.,LTD" },
    { "E43BC9", "HISENSE VISUAL TECHNOLOGY CO.,LTD" },
    { "E43C80", "University of Oklahoma" },
    { "E43D1A", "Broadcom Limited" },
    { "E43EC6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E43ED7", "Arcadyan Corporation" },
    { "E43FA2", "Wuxi DSP Technologies Inc." },
    { "E44097", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "E440E2", "Samsung Electronics Co.,Ltd" },
    { "E44122", "OnePlus Technology (Shenzhen) Co., Ltd" },
    { "E44164", "Nokia" },
    { "E441D4", "vivo Mobile Communication Co., Ltd." },
    { "E441E6", "Ottec Technology GmbH" },
    { "E442A6", "Intel Corporate" },
    { "E4434B", "Dell Inc." },
    { "E44389", "Apple, Inc." },
    { "E444E5", "Extreme Networks Headquarters" },
    { "E44519", "Beijing Xiaomi Electronics Co.,Ltd" },
    { "E446B0", "Fujitsu Client Computing Limited" },
    { "E446BD", "C&C TECHNIC TAIWAN CO., LTD." },
    { "E446DA", "Xiaomi Communications Co Ltd" },
    { "E44790", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "E44791", "Iris ID Systems, Inc." },
    { "E447B3", "zte corporation" },
    { "E448C7", "Cisco SPVTG" },
    { "E44AE0", "Intel Corporate" },
    { "E44C6C", "Shenzhen Guo Wei Electronic Co,. Ltd." },
    { "E44CC7", "IEEE Registration Authority" },
    { "E44E12", "zte corporation" },
    { "E44E18", "Gardasoft VisionLimited" },
    { "E44E2D", "Cisco Systems, Inc" },
    { "E44E76", "CHAMPIONTECH ENTERPRISE (SHENZHEN) INC" },
    { "E44F29", "MA Lighting Technology GmbH" },
    { "E44F5F", "EDS Elektronik Destek San.Tic.Ltd.Sti" },
    { "E4509A", "HW Communications Ltd" },
    { "E450EB", "Apple, Inc." },
    { "E451A9", "Nanjing Xinlian Electronics Co., Ltd" },
    { "E4521E", "Texas Instruments" },
    { "E454E5", "Huawei Device Co., Ltd." },
    { "E454E8", "Dell Inc." },
    { "E455A8", "Cisco Meraki" },
    { "E455EA", "Dedicated Computing" },
    { "E45614", "Suttle Apparatus" },
    { "E456AC", "Silicon Laboratories" },
    { "E456CA", "Fractal BMS" },
    { "E45740", "Commscope" },
    { "E45768", "vivo Mobile Communication Co., Ltd." },
    { "E457A8", "Stuart Manufacturing, Inc." },
    { "E458B8", "Samsung Electronics Co.,Ltd" },
    { "E458BC", "Bose Corporation" },
    { "E458E7", "Samsung Electronics Co.,Ltd" },
    { "E45AA2", "vivo Mobile Communication Co., Ltd." },
    { "E45AD4", "Eltex Enterprise Ltd." },
    { "E45BB3", "zte corporation" },
    { "E45D37", "Juniper Networks" },
    { "E45D39", "Texas Instruments" },
    { "E45D51", "SFR" },
    { "E45D52", "Avaya Inc" },
    { "E45D75", "Samsung Electronics Co.,Ltd" },
    { "E45E1B", "Google, Inc." },
    { "E45E37", "Intel Corporate" },
    { "E45ECC", "Juniper Networks" },
    { "E45F01", "Raspberry Pi Trading Ltd" },
    { "E46017", "Intel Corporate" },
    { "E4604D", "zte corporation" },
    { "E46059", "Pingtek Co., Ltd." },
    { "E461F4", "shenzhen worldelite electronics co., LTD" },
    { "E46251", "HAO CHENG GROUP LIMITED" },
    { "E462C4", "Cisco Systems, Inc" },
    { "E46447", "EM Microelectronic" },
    { "E46449", "Commscope" },
    { "E46564", "SHENZHEN KTC TECHNOLOGY CO.,LTD" },
    { "E465B8", "Espressif Inc." },
    { "E466AB", "zte corporation" },
    { "E466E5", "Nanjing Qinheng Microelectronics Co., Ltd." },
    { "E4671E", "SHEN ZHEN NUO XIN CHENG TECHNOLOGY co., Ltd." },
    { "E467A6", "BSH Hausgeräte GmbH" },
    { "E467BA", "Danish Interpretation Systems A/S" },
    { "E467DD", "ELA INNOVATION" },
    { "E468A3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E4695A", "Dictum Health, Inc." },
    { "E46A35", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "E46C21", "messMa GmbH" },
    { "E46CD1", "Calix Inc." },
    { "E46D7F", "Ciena Corporation" },
    { "E46E8A", "BYD Lithium Battery Co., Ltd." },
    { "E46F13", "D-Link International" },
    { "E470B8", "Intel Corporate" },
    { "E47185", "Securifi Ltd" },
    { "E472E2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E47305", "Shenzhen INVT Electric CO.,Ltd" },
    { "E47450", "Shenzhen Grandsun Electronic Co.,Ltd." },
    { "E4751E", "Getinge Sterilization AB" },
    { "E475DC", "Arcadyan Corporation" },
    { "E47684", "Apple, Inc." },
    { "E47723", "zte corporation" },
    { "E47727", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E4776B", "AARTESYS AG" },
    { "E477D4", "Minrray Industry Co.,Ltd " },
    { "E47876", "Arista Networks" },
    { "E47A2C", "SZ DJI TECHNOLOGY CO.,LTD" },
    { "E47B3F", "BEIJING CO-CLOUD TECHNOLOGY LTD." },
    { "E47C65", "Sunstar Communication Technology Co., Ltd" },
    { "E47CF9", "Samsung Electronics Co.,Ltd" },
    { "E47D5A", "Beijing Hanbang Technology Corp." },
    { "E47DBD", "Samsung Electronics Co.,Ltd" },
    { "E47DEB", "Shanghai Notion Information Technology CO.,LTD." },
    { "E47E66", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E47E9A", "zte corporation" },
    { "E47F3C", "zte corporation" },
    { "E47FB2", "FUJITSU LIMITED" },
    { "E48184", "Nokia" },
    { "E481B3", "Shenzhen ACT Industrial Co.,Ltd." },
    { "E48210", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E482CC", "Jumptronic GmbH" },
    { "E482FF", "Ecliptic Enterprises Corp" },
    { "E48326", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E48399", "Commscope" },
    { "E48429", "New H3C Technologies Co., Ltd" },
    { "E4842B", "HANGZHOU SOFTEL OPTIC CO., LTD" },
    { "E484D3", "Xiaomi Communications Co Ltd" },
    { "E48501", "Geberit International AG" },
    { "E489CA", "Cisco Systems, Inc" },
    { "E48A93", "HISENSE VISUAL TECHNOLOGY CO.,LTD" },
    { "E48AD5", "RF WINDOW CO., LTD." },
    { "E48B7F", "Apple, Inc." },
    { "E48C0F", "Discovery Insure" },
    { "E48C73", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "E48D8C", "Routerboard.com" },
    { "E48E10", "CIG SHANGHAI CO LTD" },
    { "E48EBB", "Rockwell Automation" },
    { "E48F1D", "Huawei Device Co., Ltd." },
    { "E48F34", "Vodafone Italia S.p.A." },
    { "E48F65", "Yelatma Instrument Making Enterprise, JSC" },
    { "E4902A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E49069", "Rockwell Automation" },
    { "E4907E", "Motorola Mobility LLC, a Lenovo Company" },
    { "E490FD", "Apple, Inc." },
    { "E4922A", "DBG HOLDINGS LIMITED" },
    { "E49282", "Samsung Electronics Co.,Ltd" },
    { "E492E7", "Gridlink Tech. Co.,Ltd." },
    { "E492FB", "Samsung Electronics Co.,Ltd" },
    { "E4936A", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "E4956E", "IEEE Registration Authority" },
    { "E49652", "vivo Mobile Communication Co., Ltd." },
    { "E496AE", "ALTOGRAPHICS Inc." },
    { "E497F0", "Shanghai VLC Technologies Ltd. Co." },
    { "E498BB", "Phyplus Microelectronics Limited" },
    { "E498D1", "Microsoft Mobile Oy" },
    { "E498D6", "Apple, Inc." },
    { "E4995F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E49A79", "Apple, Inc." },
    { "E49ADC", "Apple, Inc." },
    { "E49C67", "Apple, Inc." },
    { "E49D73", "Edgecore Networks Corporation" },
    { "E49E12", "FREEBOX SAS" },
    { "E49F1E", "Commscope" },
    { "E49F7D", "Samsung Electronics Co.,Ltd" },
    { "E4A1E6", "Alcatel-Lucent Shanghai Bell Co., Ltd" },
    { "E4A32F", "Shanghai Artimen Technology Co., Ltd." },
    { "E4A387", "Control Solutions LLC" },
    { "E4A41C", "Cisco Systems, Inc" },
    { "E4A430", "Samsung Electronics Co.,Ltd" },
    { "E4A471", "Intel Corporate" },
    { "E4A5EF", "TRON LINK ELECTRONICS CO., LTD." },
    { "E4A634", "Universal Electronics, Inc." },
    { "E4A749", "Palo Alto Networks" },
    { "E4A7A0", "Intel Corporate" },
    { "E4A7C5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E4A7D0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E4A7FD", "Cellco Partnership" },
    { "E4A8B6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E4A8DF", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "E4AA5D", "Cisco Systems, Inc" },
    { "E4AAE4", "Xiaomi Communications Co Ltd" },
    { "E4AAEA", "Liteon Technology Corporation" },
    { "E4AAEC", "Tianjin Hualai Technology Co., Ltd" },
    { "E4AB46", "UAB Selteka" },
    { "E4AB89", "MitraStar Technology Corp." },
    { "E4AD7D", "SCL Elements" },
    { "E4AFA1", "HES-SO" },
    { "E4B005", "Beijing IQIYI Science & Technology Co., Ltd." },
    { "E4B021", "Samsung Electronics Co.,Ltd" },
    { "E4B063", "Espressif Inc." },
    { "E4B107", "Huawei Device Co., Ltd." },
    { "E4B16C", "Apple, Inc." },
    { "E4B224", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E4B2FB", "Apple, Inc." },
    { "E4B318", "Intel Corporate" },
    { "E4B323", "Espressif Inc." },
    { "E4B503", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "E4B555", "Huawei Device Co., Ltd." },
    { "E4B633", "Wuxi Stars Microsystem Technology Co., Ltd" },
    { "E4B731", "Hangzhou Advance IOT Connectivity System Co., Ltd. " },
    { "E4B97A", "Dell Inc." },
    { "E4BAD9", "360 Fly Inc." },
    { "E4BC96", "Versuni" },
    { "E4BCAA", "Xiaomi Communications Co Ltd" },
    { "E4BD4B", "zte corporation" },
    { "E4BEED", "Netis Technology Co., Ltd." },
    { "E4BEFB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E4BFFA", "Vantiva USA LLC" },
    { "E4C0CC", "China Mobile Group Device Co.,Ltd." },
    { "E4C0E2", "Sagemcom Broadband SAS" },
    { "E4C146", "Objetivos y Servicios de Valor A" },
    { "E4C1F1", "SHENZHEN SPOTMAU INFORMATION TECHNOLIGY CO., Ltd " },
    { "E4C2D1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E4C32A", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "E4C483", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "E4C62B", "Airware" },
    { "E4C63D", "Apple, Inc." },
    { "E4C6E6", "Mophie, LLC" },
    { "E4C722", "Cisco Systems, Inc" },
    { "E4C767", "Intel Corporate" },
    { "E4C770", "Raisecom Technology CO., LTD" },
    { "E4C801", "BLU Products Inc" },
    { "E4C806", "Ceiec Electric Technology Inc." },
    { "E4C90B", "Radwin" },
    { "E4CA12", "zte corporation" },
    { "E4CB59", "Beijing Loveair Science and Technology Co. Ltd." },
    { "E4CC9D", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "E4CDA7", "zte corporation" },
    { "E4CE02", "WyreStorm Technologies Ltd" },
    { "E4CE70", "Health & Life co., Ltd." },
    { "E4CE8F", "Apple, Inc." },
    { "E4D124", "Mojo Networks, Inc." },
    { "E4D332", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "E4D373", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E4D3AA", "FCNT LLC" },
    { "E4D3F1", "Cisco Systems, Inc" },
    { "E4D53D", "Hon Hai Precision Ind. Co.,Ltd." },
    { "E4D58B", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "E4D71D", "Oraya Therapeutics" },
    { "E4DADF", "Taicang T&W Electronics" },
    { "E4DB6D", "Beijing Xiaomi Electronics Co., Ltd." },
    { "E4DBAE", "Extreme Networks Headquarters" },
    { "E4DC43", "Huawei Device Co., Ltd." },
    { "E4DC5F", "Cofractal, Inc." },
    { "E4DCCC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E4DD79", "En-Vision America, Inc." },
    { "E4DE40", "Hewlett Packard Enterprise" },
    { "E4E0A6", "Apple, Inc." },
    { "E4E0C5", "Samsung Electronics Co.,Ltd" },
    { "E4E112", "Texas Instruments" },
    { "E4E130", "TCT mobile ltd" },
    { "E4E26C", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "E4E33D", "Qingdao Intelligent&Precise Electronics Co.,Ltd." },
    { "E4E409", "LEIFHEIT AG" },
    { "E4E4AB", "Apple, Inc." },
    { "E4E608", "Kiwibit Inc." },
    { "E4E66C", "Tiandy Technologies Co.,LTD" },
    { "E4E749", "Hewlett Packard" },
    { "E4EA83", "SHENZHEN GONGJIN ELECTRONICS CO.,LT" },
    { "E4EC10", "Nokia Corporation" },
    { "E4ECE8", "Samsung Electronics Co.,Ltd" },
    { "E4EEFD", "MR&D Manufacturing" },
    { "E4F004", "Dell Inc." },
    { "E4F042", "Google, Inc." },
    { "E4F14C", "Private" },
    { "E4F1D4", "vivo Mobile Communication Co., Ltd." },
    { "E4F27C", "Juniper Networks" },
    { "E4F327", "ATOL LLC" },
    { "E4F365", "Time-O-Matic, Inc." },
    { "E4F3C4", "Samsung Electronics Co.,Ltd" },
    { "E4F3E3", "Shanghai iComhome Co.,Ltd." },
    { "E4F3E8", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "E4F3F5", "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "E4F4C6", "NETGEAR" },
    { "E4F58E", "Schneider Electric USA" },
    { "E4F75B", "Commscope" },
    { "E4F7A1", "Datafox GmbH" },
    { "E4F89C", "Intel Corporate" },
    { "E4F8BE", "TECNO MOBILE LIMITED" },
    { "E4F8EF", "Samsung Electronics Co.,Ltd" },
    { "E4F939", "Minxon Hotel Technology INC." },
    { "E4FA1D", "PAD Peripheral Advanced Design Inc." },
    { "E4FA5B", "Texas Instruments" },
    { "E4FAC4", "TP-Link Systems Inc" },
    { "E4FAED", "Samsung Electronics Co.,Ltd" },
    { "E4FAFD", "Intel Corporate" },
    { "E4FB5D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E4FB8F", "MOBIWIRE MOBILES (NINGBO) CO.,LTD" },
    { "E4FC82", "Juniper Networks" },
    { "E4FD45", "Intel Corporate" },
    { "E4FD8C", "Extreme Networks Headquarters" },
    { "E4FDA1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E4FE43", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "E4FED9", "EDMI Europe Ltd" },
    { "E4FFDD", "ELECTRON INDIA" },
    { "E80036", "Befs co,. ltd" },
    { "E80115", "COOCAA Network Technology CO.,TD." },
    { "E8018D", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "E8039A", "Samsung Electronics Co.,Ltd" },
    { "E8040B", "Apple, Inc." },
    { "E80410", "Private" },
    { "E80462", "Cisco Systems, Inc" },
    { "E804F3", "Throughtek Co., Ltd." },
    { "E8056D", "Nortel Networks" },
    { "E805DC", "Verifone, Inc." },
    { "E80688", "Apple, Inc." },
    { "E80690", "Espressif Inc." },
    { "E806EB", "ShieldSOS LLC" },
    { "E80734", "Champion Optical Network Engineering, LLC" },
    { "E807BF", "SHENZHEN BOOMTECH INDUSTRY CO.,LTD" },
    { "E8088B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E808AF", "zte corporation" },
    { "E80945", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "E80959", "Guoguang Electric Co.,Ltd" },
    { "E80AB9", "Cisco Systems, Inc" },
    { "E80AEC", "Jiangsu Hengtong Optic-Electric Co., LTD" },
    { "E80B13", "Akib Systems Taiwan, INC" },
    { "E80C38", "DAEYOUNG INFORMATION SYSTEM CO., LTD" },
    { "E80C75", "Syncbak, Inc." },
    { "E80FC8", "Universal Electronics, Inc." },
    { "E8102E", "Really Simple Software, Inc" },
    { "E81098", "Hewlett Packard Enterprise" },
    { "E81132", "Samsung Electronics Co.,Ltd" },
    { "E811CA", "SHANDONG KAER ELECTRIC.CO.,LTD" },
    { "E8122D", "UNIONMAN TECHNOLOGY CO.,LTD" },
    { "E81324", "GuangZhou Bonsoninfo System CO.,LTD" },
    { "E81363", "Comstock RD, Inc." },
    { "E81367", "AIRSOUND Inc." },
    { "E8136E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E81499", "Yoqu Technology(Shenzhen)Co.,Ltd." },
    { "E8150E", "Nokia Corporation" },
    { "E8162B", "IDEO Security Co., Ltd." },
    { "E81656", "Hangzhou BroadLink Technology Co.,Ltd" },
    { "E81711", "Shenzhen Vipstech Co., Ltd" },
    { "E817FC", "Fujitsu Limited" },
    { "E81863", "IEEE Registration Authority" },
    { "E81A58", "TECHNOLOGIC SYSTEMS" },
    { "E81AAC", "ORFEO SOUNDWORKS Inc." },
    { "E81B4B", "amnimo Inc." },
    { "E81B69", "Sercomm Corporation." },
    { "E81CA5", "Hewlett Packard Enterprise" },
    { "E81CBA", "Fortinet, Inc." },
    { "E81CD8", "Apple, Inc." },
    { "E81DA8", "Ruckus Wireless" },
    { "E81DEE", "i-TEK RFID" },
    { "E81E92", "Huawei Device Co., Ltd." },
    { "E820E2", "HUMAX Co., Ltd." },
    { "E82281", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "E822B8", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "E82404", "Quectel Wireless Solutions Co.,Ltd." },
    { "E824A6", "Juniper Networks" },
    { "E82587", "Shenzhen Chilink IoT Technology Co., Ltd." },
    { "E82689", "Hewlett Packard Enterprise" },
    { "E8268D", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "E826B6", "Companies House to GlucoRx Technologies Ltd." },
    { "E826CF", "Shenzhen Jingxun Technology Co., Ltd." },
    { "E82725", "Axis Communications AB" },
    { "E82877", "TMY Co., Ltd." },
    { "E8288D", "Huawei Device Co., Ltd." },
    { "E828C1", "Eltex Enterprise Ltd." },
    { "E828D5", "Cots Technology" },
    { "E82A44", "Liteon Technology Corporation" },
    { "E82AEA", "Intel Corporate" },
    { "E82BC5", "Huawei Device Co., Ltd." },
    { "E82C6D", "SmartRG, Inc." },
    { "E82E0C", "NETINT Technologies Inc." },
    { "E82E24", "Out of the Fog Research LLC" },
    { "E831CD", "Espressif Inc." },
    { "E8330D", "Xaptec GmbH" },
    { "E83381", "Commscope" },
    { "E8343E", "Beijing Infosec Technologies Co., LTD." },
    { "E83617", "Apple, Inc." },
    { "E8361D", "Sense Labs, Inc." },
    { "E8377A", "Zyxel Communications Corporation" },
    { "E838A0", "Vizio, Inc" },
    { "E83935", "Hewlett Packard" },
    { "E839DF", "ASKEY COMPUTER CORP" },
    { "E83A12", "Samsung Electronics Co.,Ltd" },
    { "E83A4B", "China Mobile Group Device Co.,Ltd." },
    { "E83A97", "Toshiba Corporation" },
    { "E83EB6", "RIM" },
    { "E83EFB", "GEODESIC LTD." },
    { "E83EFC", "Commscope" },
    { "E83F67", "Huawei Device Co., Ltd." },
    { "E84040", "Cisco Systems, Inc" },
    { "E84074", "Barrot Technology Co.,Ltd." },
    { "E840F2", "PEGATRON CORPORATION" },
    { "E84368", "zte corporation" },
    { "E843B6", "QNAP Systems, Inc." },
    { "E8447E", "Bitdefender SRL" },
    { "E8458B", "MitraStar Technology Corp." },
    { "E845EB", "Kohler Ventures, Inc." },
    { "E84727", "Quectel Wireless Solutions Co.,Ltd." },
    { "E8473A", "Hon Hai Precision Industry Co.,LTD" },
    { "E8481F", "Advanced Automotive Antennas" },
    { "E848B8", "TP-Link Systems Inc" },
    { "E84943", "YUGE Information technology Co. Ltd" },
    { "E84A54", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "E84A78", "Apple, Inc." },
    { "E84C4A", "Amazon Technologies Inc." },
    { "E84C56", "INTERCEPT SERVICES LIMITED" },
    { "E84D74", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E84DD0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E84DEC", "Xerox Corporation" },
    { "E84E06", "EDUP INTERNATIONAL (HK) CO., LTD" },
    { "E84E84", "Samsung Electronics Co.,Ltd" },
    { "E84ECE", "Nintendo Co., Ltd." },
    { "E84F25", "Murata Manufacturing Co., Ltd." },
    { "E84F4B", "Shenzhen Delos Electronic Co., Ltd" },
    { "E84FA7", "Huawei Device Co., Ltd." },
    { "E8508B", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "E8516E", "TSMART Inc." },
    { "E85177", "Qingdao Intelligent&Precise Electronics Co.,Ltd." },
    { "E8519D", "Yeonhab Precision Co.,LTD" },
    { "E8519E", "Guangzhou Shiyuan Electronic Technology Company Limited" },
    { "E85484", "NEO Information Systems Co., Ltd." },
    { "E85497", "Samsung Electronics Co.,Ltd" },
    { "E85540", "WUXI FUNIDE DIGITAL CO.,LTD" },
    { "E855B4", "SAI Technology Inc." },
    { "E85659", "Advanced-Connectek Inc." },
    { "E856D6", "NCTech Ltd" },
    { "E85A8B", "Xiaomi Communications Co Ltd" },
    { "E85AA7", "LLC Emzior" },
    { "E85AD1", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "E85B5B", "LG ELECTRONICS INC" },
    { "E85BB7", "Ample Systems Inc." },
    { "E85BF0", "Imaging Diagnostics" },
    { "E85C0A", "Cisco Systems, Inc" },
    { "E85C5F", "FN-LINK TECHNOLOGY Ltd." },
    { "E85D6B", "Luminate Wireless" },
    { "E85D86", "CHANG YOW TECHNOLOGIES INTERNATIONAL CO.,LTD." },
    { "E85E53", "Infratec Datentechnik GmbH" },
    { "E85F02", "Apple, Inc." },
    { "E85FB4", "Xiaomi Communications Co Ltd" },
    { "E8611F", "Dawning Information Industry Co.,Ltd" },
    { "E8617E", "Liteon Technology Corporation" },
    { "E86183", "Black Diamond Advanced Technology, LLC" },
    { "E861BE", "Melec Inc." },
    { "E862BE", "Intel Corporate" },
    { "E86538", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "E86549", "Cisco Systems, Inc" },
    { "E8655F", "Dell Inc." },
    { "E865D4", "Tenda Technology Co.,Ltd.Dongguan branch" },
    { "E866C4", "Diamanti" },
    { "E86819", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E868E7", "Espressif Inc." },
    { "E86A64", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "E86BEA", "Espressif Inc." },
    { "E86CC7", "IEEE Registration Authority" },
    { "E86CDA", "Supercomputers and Neurocomputers Research Center" },
    { "E86D16", "Elmec Elettronica SRL" },
    { "E86D52", "Commscope" },
    { "E86D54", "Digit Mobile Inc" },
    { "E86D65", "AUDIO MOBIL Elektronik GmbH" },
    { "E86D6E", "voestalpine Signaling UK Ltd." },
    { "E86DCB", "Samsung Electronics Co.,Ltd" },
    { "E86DE9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E86E3A", "Sony Interactive Entertainment Inc." },
    { "E86E44", "zte corporation" },
    { "E86EAD", "Guangzhou Gizwits loT Technology Co.,Ltd" },
    { "E86F38", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "E86FF2", "Actiontec Electronics, Inc" },
    { "E87072", "Hangzhou BroadLink Technology Co.,Ltd" },
    { "E8718D", "Elsys Equipamentos Eletronicos Ltda" },
    { "E874C7", "Sentinhealth" },
    { "E874E6", "ADB Broadband Italia" },
    { "E8757F", "FIRS Technologies(Shenzhen) Co., Ltd" },
    { "E87640", "SKY UK LIMITED" },
    { "E87829", "IEEE Registration Authority" },
    { "E87865", "Apple, Inc." },
    { "E878A1", "BEOVIEW INTERCOM DOO" },
    { "E878EE", "New H3C Technologies Co., Ltd" },
    { "E879A3", "Cisco Systems, Inc" },
    { "E87AF3", "S5 Tech S.r.l." },
    { "E87E1C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E87F6B", "Samsung Electronics Co.,Ltd" },
    { "E87F95", "Apple, Inc." },
    { "E8802E", "Apple, Inc." },
    { "E88088", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "E880D8", "GNTEK Electronics Co.,Ltd." },
    { "E88152", "Apple, Inc." },
    { "E88175", "zte corporation" },
    { "E881AB", "Beijing Sankuai Online Technology Co.,Ltd" },
    { "E8825B", "Commscope" },
    { "E884A5", "Intel Corporate" },
    { "E884C6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E8854B", "Apple, Inc." },
    { "E886CF", "Nokia" },
    { "E887A3", "Loxley Public Company Limited" },
    { "E88843", "Xiaomi Communications Co Ltd" },
    { "E8886C", "Shenzhen SC Technologies Co.,LTD" },
    { "E8892C", "Commscope" },
    { "E88D28", "Apple, Inc." },
    { "E88DA6", "Quectel Wireless Solutions Co.,Ltd." },
    { "E88DF5", "ZNYX Networks, Inc." },
    { "E88E60", "NSD Corporation" },
    { "E88F6F", "TCT mobile ltd" },
    { "E88FC4", "MOBIWIRE MOBILES(NINGBO) CO.,LTD" },
    { "E8910F", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "E89120", "Motorola Mobility LLC, a Lenovo Company" },
    { "E89218", "Arcontia International AB" },
    { "E892A4", "LG Electronics (Mobile Communications)" },
    { "E89309", "Samsung Electronics Co.,Ltd" },
    { "E89363", "Nokia" },
    { "E893F3", "Graphiant Inc" },
    { "E8944C", "Cogent Healthcare Systems Ltd" },
    { "E894F6", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "E89505", "Shenzhen MiaoMing Intelligent Technology Co.,Ltd" },
    { "E89526", "Luxshare Precision Industry CO., LTD." },
    { "E89606", "testo Instruments (Shenzhen) Co., Ltd." },
    { "E8979A", "Quectel Wireless Solutions Co.,Ltd." },
    { "E897B8", "Chiun Mai Communication System, Inc" },
    { "E89847", "Xiaomi Communications Co Ltd" },
    { "E8986D", "Palo Alto Networks" },
    { "E898C2", "ZETLAB Company" },
    { "E898EE", "Apple, Inc." },
    { "E8995A", "PiiGAB, Processinformation i Goteborg AB" },
    { "E899C4", "HTC Corporation" },
    { "E89A8F", "Quanta Computer Inc." },
    { "E89AFF", "Fujian LANDI Commercial Equipment Co.,Ltd" },
    { "E89C25", "ASUSTek COMPUTER INC." },
    { "E89D87", "Toshiba" },
    { "E89E0C", "MAX8USA DISTRIBUTORS INC." },
    { "E89E13", "CRESYN" },
    { "E89E49", "Mellanox Technologies, Inc." },
    { "E89EB4", "Hon Hai Precision Ind. Co.,Ltd." },
    { "E89F39", "Nokia" },
    { "E89F6D", "Espressif Inc." },
    { "E89F80", "Belkin International Inc." },
    { "E89FEC", "CHENGDU KT ELECTRONIC HI-TECH CO.,LTD" },
    { "E8A0CD", "Nintendo Co.,Ltd" },
    { "E8A0ED", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "E8A1F8", "zte corporation" },
    { "E8A245", "Juniper Networks" },
    { "E8A34E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E8A364", "Signal Path International / Peachtree Audio" },
    { "E8A4C1", "Deep Sea Electronics Ltd" },
    { "E8A55A", "Juniper Networks" },
    { "E8A660", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E8A6CA", "Huawei Device Co., Ltd." },
    { "E8A72F", "Microsoft Corporation" },
    { "E8A730", "Apple, Inc." },
    { "E8A788", "XIAMEN LEELEN TECHNOLOGY CO., LTD" },
    { "E8A7F2", "sTraffic" },
    { "E8A848", "Wacom Co.,Ltd." },
    { "E8AACB", "Samsung Electronics Co.,Ltd" },
    { "E8ABF3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E8ABFA", "Shenzhen Reecam Tech.Ltd." },
    { "E8AC23", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E8AC7E", "TERAHOP PTE.LTD." },
    { "E8ACAD", "zte corporation" },
    { "E8ADA6", "Sagemcom Broadband SAS" },
    { "E8AEC5", "Arista Networks" },
    { "E8B0C5", "Intel Corporate" },
    { "E8B1FC", "Intel Corporate" },
    { "E8B265", "Dell Inc." },
    { "E8B2AC", "Apple, Inc." },
    { "E8B2FE", "HUMAX Co., Ltd." },
    { "E8B3EF", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "E8B470", "IEEE Registration Authority" },
    { "E8B4AE", "Shenzhen C&D Electronics Co.,Ltd" },
    { "E8B4C8", "Samsung Electronics Co.,Ltd" },
    { "E8B527", "Phyplus Technology (Shanghai) Co., Ltd" },
    { "E8B541", "zte corporation" },
    { "E8B5D0", "Dell Inc." },
    { "E8B6C2", "Juniper Networks" },
    { "E8B722", "GreenTrol Automation " },
    { "E8B723", "Shenzhen Vatilon Electronics Co.,Ltd" },
    { "E8B748", "Cisco Systems, Inc" },
    { "E8BA70", "Cisco Systems, Inc" },
    { "E8BAE2", "Xplora Technologies AS" },
    { "E8BB3D", "Sino Prime-Tech Limited" },
    { "E8BBA8", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "E8BCE4", "Cisco Systems, Inc" },
    { "E8BDD1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E8BE81", "Sagemcom Broadband SAS" },
    { "E8BFB8", "Intel Corporate" },
    { "E8BFDB", "Inodesign Group" },
    { "E8BFE1", "Intel Corporate" },
    { "E8C1B8", "Nanjing Bangzhong Electronic Commerce Limited" },
    { "E8C1D7", "Philips" },
    { "E8C1E8", "Shenzhen Xiao Bi En Culture Education Technology Co.,Ltd." },
    { "E8C229", "H-Displays (MSC) Bhd" },
    { "E8C2DD", "Infinix mobility limited" },
    { "E8C320", "Austco Marketing & Service (USA) ltd." },
    { "E8C417", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "E8C57A", "Ufispace Co., LTD." },
    { "E8C6E6", "CHANGHONG (HONGKONG) TRADING LIMITED" },
    { "E8C74F", "Liteon Technology Corporation" },
    { "E8C7CF", "Wistron Neweb Corporation" },
    { "E8C829", "Intel Corporate" },
    { "E8C913", "Samsung Electronics Co.,Ltd" },
    { "E8CA50", "Bouffalo Lab (Nanjing) Co., Ltd." },
    { "E8CAC8", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "E8CBA1", "Nokia Corporation" },
    { "E8CBED", "Chipsea Technologies(Shenzhen) Corp." },
    { "E8CBF5", "Laird Connectivity" },
    { "E8CC18", "D-Link International" },
    { "E8CC32", "Micronet LTD" },
    { "E8CC8C", "Chengdu Jiarui Hualian Communication Technology Co" },
    { "E8CD15", "Vantiva USA LLC" },
    { "E8CD2D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E8CE06", "SkyHawke Technologies, LLC." },
    { "E8CF83", "Dell Inc." },
    { "E8D03C", "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd" },
    { "E8D099", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "E8D0B9", "Taicang T&W Electronics" },
    { "E8D0FA", "MKS Instruments Deutschland GmbH" },
    { "E8D0FC", "Liteon Technology Corporation" },
    { "E8D11B", "ASKEY COMPUTER CORP" },
    { "E8D2FF", "Sagemcom Broadband SAS" },
    { "E8D322", "Cisco Systems, Inc" },
    { "E8D3EB", "eero inc." },
    { "E8D483", "ULTIMATE Europe Transportation Equipment GmbH" },
    { "E8D4E0", "Beijing BenyWave Technology Co., Ltd." },
    { "E8D52B", "Google, Inc." },
    { "E8D765", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E8D775", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E8D819", "AzureWave Technology Inc." },
    { "E8D87E", "Amazon Technologies Inc." },
    { "E8D8D1", "HP Inc." },
    { "E8DA00", "Kivo Technology, Inc." },
    { "E8DA20", "Nintendo Co.,Ltd" },
    { "E8DA3E", "Huawei Device Co., Ltd." },
    { "E8DA96", "Zhuhai Tianrui Electrical Power Tech. Co., Ltd." },
    { "E8DAAA", "VideoHome Technology Corp." },
    { "E8DB84", "Espressif Inc." },
    { "E8DC6C", "Cisco Systems, Inc" },
    { "E8DE00", "ChongQing GuanFang Technology Co.,LTD" },
    { "E8DE27", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "E8DE8E", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "E8DED6", "Intrising Networks, Inc." },
    { "E8DEFB", "MESOTIC SAS" },
    { "E8DF24", "OHSUNG" },
    { "E8DF70", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "E8DFF2", "PRF Co., Ltd." },
    { "E8E07E", "Silicon Laboratories" },
    { "E8E08F", "GRAVOTECH MARKING SAS" },
    { "E8E0B7", "Toshiba" },
    { "E8E1E1", "Gemtek Technology Co., Ltd." },
    { "E8E1E2", "Energotest" },
    { "E8E49D", "Nexthop Systems Inc." },
    { "E8E5D6", "Samsung Electronics Co.,Ltd" },
    { "E8E609", "Chongqing Zhouhai intelligent technology CO., Ltd" },
    { "E8E732", "Alcatel-Lucent Enterprise" },
    { "E8E770", "Warp9 Tech Design, Inc." },
    { "E8E776", "Shenzhen Kootion Technology Co., Ltd" },
    { "E8E7C3", "zte corporation" },
    { "E8E875", "iS5 Communications Inc." },
    { "E8E8B7", "Murata Manufacturing Co., Ltd." },
    { "E8E98E", "SOLAR controls s.r.o." },
    { "E8EA4D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E8EA6A", "StarTech.com" },
    { "E8EADA", "Denkovi Assembly Electronics LTD" },
    { "E8EAFA", "Qingdao Haier Technology Co.,Ltd" },
    { "E8EB11", "Texas Instruments" },
    { "E8EB1B", "Microchip Technology Inc." },
    { "E8EB34", "Cisco Systems, Inc" },
    { "E8EBD3", "Mellanox Technologies, Inc." },
    { "E8EBDD", "Guangzhou Qingying Acoustics Technology Co., Ltd" },
    { "E8ECA3", "Dongguan Liesheng Electronic Co.Ltd" },
    { "E8ED05", "Commscope" },
    { "E8EDD6", "Fortinet, Inc." },
    { "E8EDF3", "Cisco Systems, Inc" },
    { "E8EECC", "Fantasia Trading LLC" },
    { "E8EF05", "MIND TECH INTERNATIONAL LIMITED" },
    { "E8EF22", "Siemens Numerical Control Ltd., Nanjing" },
    { "E8EF89", "OPMEX Tech." },
    { "E8F085", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E8F0A4", "Antonios A. Chariton" },
    { "E8F1B0", "Sagemcom Broadband SAS" },
    { "E8F226", "MILLSON CUSTOM SOLUTIONS INC." },
    { "E8F2E2", "LG Innotek" },
    { "E8F2E3", "Starcor Beijing Co.,Limited" },
    { "E8F375", "Nokia" },
    { "E8F408", "Intel Corporate" },
    { "E8F494", "AltoBeam Inc." },
    { "E8F654", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E8F673", "Microsoft Corporation" },
    { "E8F724", "Hewlett Packard Enterprise " },
    { "E8F72F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E8F791", "Xiaomi Communications Co Ltd" },
    { "E8F8D0", "Nokia Shanghai Bell Co., Ltd." },
    { "E8F928", "ENGINKO SRL" },
    { "E8F9D4", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "E8FA23", "Huawei Device Co., Ltd." },
    { "E8FAF7", "Guangdong Uniteddata Holding Group Co., Ltd." },
    { "E8FB1C", "AzureWave Technology Inc." },
    { "E8FBE9", "Apple, Inc." },
    { "E8FC60", "ELCOM Innovations Private Limited" },
    { "E8FCAF", "NETGEAR" },
    { "E8FD35", "Huawei Device Co., Ltd." },
    { "E8FD72", "SHANGHAI LINGUO TECHNOLOGY CO., LTD." },
    { "E8FD90", "Turbostor" },
    { "E8FDE8", "CeLa Link Corporation" },
    { "E8FDF8", "Shanghai High-Flying Electronics Technology Co., Ltd" },
    { "E8FF1E", "IEEE Registration Authority" },
    { "E8FF98", "Huawei Device Co., Ltd." },
    { "E8FFF4", "Apple, Inc." },
    { "EC0133", "TRINUS SYSTEMS INC." },
    { "EC01D5", "Cisco Systems, Inc" },
    { "EC01E2", "FOXCONN INTERCONNECT TECHNOLOGY" },
    { "EC01EE", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "EC0273", "Hewlett Packard Enterprise" },
    { "EC0441", "ShenZhen TIGO Semiconductor Co., Ltd." },
    { "EC0482", "STL Systems AG" },
    { "EC086B", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "EC08E5", "Motorola Mobility LLC, a Lenovo Company" },
    { "EC09C9", "Texas Instruments" },
    { "EC0BAE", "Hangzhou BroadLink Technology Co.,Ltd" },
    { "EC0C96", "Nokia" },
    { "EC0D51", "Apple, Inc." },
    { "EC0D9A", "Mellanox Technologies, Inc." },
    { "EC0DE4", "Amazon Technologies Inc." },
    { "EC0EC4", "Hon Hai Precision Ind. Co.,Ltd." },
    { "EC0ED6", "ITECH INSTRUMENTS SAS" },
    { "EC1055", "Beijing Xiaomi Electronics Co.,Ltd" },
    { "EC107B", "Samsung Electronics Co.,Ltd" },
    { "EC1120", "FloDesign Wind Turbine Corporation" },
    { "EC1127", "Texas Instruments" },
    { "EC13B2", "Netonix" },
    { "EC13DB", "Juniper Networks" },
    { "EC14F6", "BioControl AS" },
    { "EC153D", "Beijing Yaxunhongda Technology Co., Ltd." },
    { "EC172F", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "EC1766", "Research Centre Module" },
    { "EC192E", "Cisco Systems, Inc" },
    { "EC1A02", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "EC1A59", "Belkin International Inc." },
    { "EC1B5F", "Hewlett Packard Enterprise" },
    { "EC1BBD", "Silicon Laboratories" },
    { "EC1BCF", "Brain Technologies, Inc." },
    { "EC1BFA", "EM Microelectronic" },
    { "EC1C5D", "Siemens AG" },
    { "EC1D53", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "EC1D7F", "zte corporation" },
    { "EC1D8B", "Cisco Systems, Inc" },
    { "EC1D9E", "Quectel Wireless Solutions Co.,Ltd." },
    { "EC1DA9", "YEALINK(XIAMEN) NETWORK TECHNOLOGY CO.,LTD." },
    { "EC1F72", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "EC2125", "Toshiba Corp." },
    { "EC2150", "vivo Mobile Communication Co., Ltd." },
    { "EC219F", "VidaBox LLC" },
    { "EC21E5", "Toshiba" },
    { "EC2257", "JiangSu NanJing University Electronic Information Technology Co.,Ltd" },
    { "EC2280", "D-Link International" },
    { "EC233D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "EC2368", "IntelliVoice Co.,Ltd." },
    { "EC237B", "zte corporation" },
    { "EC24B8", "Texas Instruments" },
    { "EC2651", "Apple, Inc." },
    { "EC26CA", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "EC26FB", "TECC CO.,LTD." },
    { "EC28D3", "Apple, Inc." },
    { "EC2A72", "Dell Inc." },
    { "EC2AF0", "Ypsomed AG" },
    { "EC2BEB", "Amazon Technologies Inc." },
    { "EC2C11", "CWD INNOVATION LIMITED" },
    { "EC2C49", "NakaoLab, The University of Tokyo" },
    { "EC2C73", "Apple, Inc." },
    { "EC2CE2", "Apple, Inc." },
    { "EC2E4E", "HITACHI-LG DATA STORAGE INC" },
    { "EC2E98", "AzureWave Technology Inc." },
    { "EC2F20", "HYUNDAI WIA" },
    { "EC2F90", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "EC308E", "Lierda Science & Technology Group Co., Ltd" },
    { "EC3091", "Cisco Systems, Inc" },
    { "EC30B3", "Xiaomi Communications Co Ltd" },
    { "EC3111", "Sichuan AI-Link Technology Co., Ltd." },
    { "EC314A", "Chengdu Quanjing Intelligent Technology Co.,Ltd" },
    { "EC316D", "Hansgrohe" },
    { "EC34E2", "Private" },
    { "EC3532", "Tactrix Inc." },
    { "EC354D", "Wingtech Mobile Communications Co.,Ltd" },
    { "EC3586", "Apple, Inc." },
    { "EC363F", "Markov Corporation" },
    { "EC3873", "Juniper Networks" },
    { "EC388F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "EC3A52", "Huawei Device Co., Ltd." },
    { "EC3BAF", "EM Microelectronic" },
    { "EC3BF0", "NovelSat" },
    { "EC3C5A", "SHEN ZHEN HENG SHENG HUI DIGITAL TECHNOLOGY CO.,LTD" },
    { "EC3C88", "MCNEX Co.,Ltd." },
    { "EC3CBB", "Huawei Device Co., Ltd." },
    { "EC3DFD", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "EC3E09", "PERFORMANCE DESIGNED PRODUCTS, LLC" },
    { "EC3EB3", "Zyxel Communications Corporation" },
    { "EC3EF7", "Juniper Networks" },
    { "EC3F05", "Institute 706, The Second Academy China Aerospace Science & Industry Corp" },
    { "EC4118", "XIAOMI Electronics,CO.,LTD" },
    { "EC41CA", "Shenzhen TecAnswer Technology co.,ltd" },
    { "EC41F9", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "EC4269", "HMD Global Oy" },
    { "EC42B4", "ADC Corporation" },
    { "EC42CC", "Apple, Inc." },
    { "EC42F0", "ADL Embedded Solutions, Inc." },
    { "EC438B", "YAPTV" },
    { "EC43E6", "AWCER Ltd." },
    { "EC43F6", "Zyxel Communications Corporation" },
    { "EC4476", "Cisco Systems, Inc" },
    { "EC462C", "Infinix mobility limited" },
    { "EC4644", "TTK SAS" },
    { "EC4654", "Apple, Inc." },
    { "EC4670", "Meinberg Funkuhren GmbH & Co. KG" },
    { "EC470C", "Universal Electronics, Inc." },
    { "EC473C", "Redwire, LLC" },
    { "EC4993", "Qihan Technology Co., Ltd " },
    { "EC4C4D", "ZAO NPK RoTeK" },
    { "EC4C8C", "Intel Corporate" },
    { "EC4D3E", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "EC4D47", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "EC4F82", "Calix Inc." },
    { "EC50AA", "Hewlett Packard Enterprise" },
    { "EC51BC", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "EC52DC", "WORLD MEDIA AND TECHNOLOGY Corp." },
    { "EC536F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "EC5382", "Honor Device Co., Ltd." },
    { "EC542E", "Shanghai XiMei Electronic Technology Co. Ltd" },
    { "EC551C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "EC554D", "Private" },
    { "EC55F9", "Hon Hai Precision Ind. Co.,Ltd." },
    { "EC5623", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "EC570D", "AFE Inc." },
    { "EC58EA", "Ruckus Wireless" },
    { "EC59E7", "Microsoft Corporation" },
    { "EC5A31", "BUFFALO.INC" },
    { "EC5A86", "Yulong Computer Telecommunication Scientific (Shenzhen) Co.,Ltd" },
    { "EC5AA3", "Huawei Device Co., Ltd." },
    { "EC5B73", "Advanced & Wise Technology Corp." },
    { "EC5BCD", "IEEE Registration Authority" },
    { "EC5C68", "CHONGQING FUGUI ELECTRONICS CO.,LTD." },
    { "EC5C69", "MITSUBISHI HEAVY INDUSTRIES MECHATRONICS SYSTEMS,LTD." },
    { "EC5C84", "Murata Manufacturing Co., Ltd." },
    { "EC5F23", "Qinghai Kimascend Electronics Technology Co. Ltd." },
    { "EC6073", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "EC60E0", "AVI-ON LABS" },
    { "EC6260", "Espressif Inc." },
    { "EC6264", "Global411 Internet Services, LLC" },
    { "EC63D7", "Intel Corporate" },
    { "EC63E5", "ePBoard Design LLC" },
    { "EC63ED", "Hyundai Autoever Corp." },
    { "EC6488", "Honor Device Co., Ltd." },
    { "EC64C9", "Espressif Inc." },
    { "EC64E7", "MOCACARE Corporation" },
    { "EC656E", "The Things Industries B.V." },
    { "EC65CC", "Panasonic Automotive Systems Company of America" },
    { "EC6652", "Info Fiber Solutions Pvt Ltd" },
    { "EC66D1", "B&W Group LTD" },
    { "EC6794", "Hewlett Packard Enterprise" },
    { "EC6881", "Palo Alto Networks" },
    { "EC6C9A", "Arcadyan Corporation" },
    { "EC6C9F", "Chengdu Volans Technology CO.,LTD" },
    { "EC6CB5", "zte corporation" },
    { "EC6E79", "InHand Networks, INC." },
    { "EC6F0B", "FADU, Inc." },
    { "EC7097", "Commscope" },
    { "EC71DB", "Reolink Innovation Limited" },
    { "EC725B", "zte corporation" },
    { "EC7359", "Shenzhen Cloudsky Technologies Co., Ltd." },
    { "EC7379", "Apple, Inc." },
    { "EC7427", "eero inc." },
    { "EC748C", "Sony Interactive Entertainment Inc." },
    { "EC74BA", "Hirschmann Automation and Control GmbH" },
    { "EC74CD", "IEEE Registration Authority" },
    { "EC74D7", "Grandstream Networks Inc" },
    { "EC750C", "TP-Link Systems Inc." },
    { "EC753E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "EC75ED", "Citrix Systems, Inc." },
    { "EC7949", "FUJITSU LIMITED" },
    { "EC79F2", "Startel" },
    { "EC7C2C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "EC7C5C", "Juniper Networks" },
    { "EC7C74", "Justone Technologies Co., Ltd." },
    { "EC7CB6", "Samsung Electronics Co.,Ltd" },
    { "EC7CBA", "Hewlett Packard Enterprise " },
    { "EC7D11", "vivo Mobile Communication Co., Ltd." },
    { "EC7D9D", "CPI" },
    { "EC7E91", "ITEL MOBILE LIMITED" },
    { "EC7FC6", "ECCEL CORPORATION SAS" },
    { "EC8009", "NovaSparks" },
    { "EC8150", "Apple, Inc." },
    { "EC8193", "Logitech, Inc" },
    { "EC819C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "EC8263", "zte corporation" },
    { "EC8350", "Microsoft Corporation" },
    { "EC836C", "RM Tech Co., Ltd." },
    { "EC83B7", "PUWELL CLOUD TECH LIMITED" },
    { "EC83D5", "GIRD Systems Inc" },
    { "EC8439", "Private" },
    { "EC84B4", "CIG SHANGHAI CO LTD" },
    { "EC852F", "Apple, Inc." },
    { "EC888F", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "EC8892", "Motorola Mobility LLC, a Lenovo Company" },
    { "EC8914", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "EC89F5", "Lenovo Mobile Communication Technology Ltd." },
    { "EC8A48", "Arista Networks" },
    { "EC8A4C", "zte corporation" },
    { "EC8AC4", "Amazon Technologies Inc." },
    { "EC8AC7", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "EC8C9A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "EC8CA2", "Ruckus Wireless" },
    { "EC8E12", "Nokia" },
    { "EC8E77", "Intel Corporate" },
    { "EC8EAD", "DLX" },
    { "EC8EAE", "Nagravision SA" },
    { "EC8EB5", "Hewlett Packard" },
    { "EC90C1", "Samsung Electronics Co.,Ltd" },
    { "EC9161", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "EC9233", "Eddyfi NDT Inc" },
    { "EC9327", "MEMMERT GmbH + Co. KG" },
    { "EC9365", "Mapper.ai, Inc." },
    { "EC937D", "Vantiva USA LLC" },
    { "EC93ED", "DDoS-Guard LTD" },
    { "EC9468", "META SYSTEM SPA " },
    { "EC94CB", "Espressif Inc." },
    { "EC94D5", "Juniper Networks" },
    { "EC9681", "2276427 Ontario Inc" },
    { "EC96BF", "eSystems MTG GmbH" },
    { "EC97A2", "Apple, Inc." },
    { "EC97B2", "SUMEC Machinery & Electric Co.,Ltd." },
    { "EC97E0", "Hangzhou Ezviz Software Co.,Ltd." },
    { "EC986C", "Lufft Mess- und Regeltechnik GmbH" },
    { "EC98C1", "Beijing Risbo Network Technology Co.,Ltd" },
    { "EC9A0C", "IEEE Registration Authority" },
    { "EC9A34", "Texas Instruments" },
    { "EC9A74", "Hewlett Packard" },
    { "EC9B2D", "China Mobile Group Device Co.,Ltd." },
    { "EC9B5B", "Nokia Corporation" },
    { "EC9B8B", "Hewlett Packard Enterprise " },
    { "EC9BF3", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "EC9C32", "Sichuan AI-Link Technology Co., Ltd." },
    { "EC9ECD", "Artesyn Embedded Technologies" },
    { "EC9EEA", "Xtra Technology LLC" },
    { "EC9F0D", "IEEE Registration Authority" },
    { "ECA138", "Amazon Technologies Inc." },
    { "ECA1D1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "ECA29B", "Kemppi Oy" },
    { "ECA2A0", "Taicang T&W Electronics" },
    { "ECA5DE", "ONYX WIFI Inc" },
    { "ECA62F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "ECA7AD", "Barrot Technology Co.,Ltd." },
    { "ECA7B1", "IEEE Registration Authority" },
    { "ECA81F", "Vantiva USA LLC" },
    { "ECA86B", "Elitegroup Computer Systems Co.,Ltd." },
    { "ECA907", "Apple, Inc." },
    { "ECA940", "Commscope" },
    { "ECA971", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "ECA9FA", "GUANGDONG GENIUS TECHNOLOGY CO., LTD." },
    { "ECAA25", "Samsung Electronics Co.,Ltd" },
    { "ECAA8F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "ECAAA0", "PEGATRON CORPORATION" },
    { "ECAB3E", "ESSYS" },
    { "ECADB8", "Apple, Inc." },
    { "ECADE0", "D-Link International" },
    { "ECAF97", "GIT" },
    { "ECAFF9", "Hailo Technologies Ltd." },
    { "ECB0D2", "EM Microelectronic" },
    { "ECB0E1", "Ciena Corporation" },
    { "ECB106", "Acuro Networks, Inc" },
    { "ECB1D7", "Hewlett Packard" },
    { "ECB1E0", "Eltex Enterprise LTD" },
    { "ECB313", "SHENZHEN GONGJIN ELECTRONICS CO.,LT" },
    { "ECB4E8", "Wistron Mexico SA de CV" },
    { "ECB541", "SHINANO E and E Co.Ltd." },
    { "ECB550", "Samsung Electronics Co.,Ltd" },
    { "ECB5FA", "Philips Lighting BV" },
    { "ECB870", "Beijing Heweinet Technology Co.,Ltd." },
    { "ECB878", "Huawei Device Co., Ltd." },
    { "ECB907", "CloudGenix Inc" },
    { "ECB970", "Ruijie Networks Co.,LTD" },
    { "ECBAFE", "GIROPTIC" },
    { "ECBB78", "Cisco Systems, Inc" },
    { "ECBBAE", "Digivoice Tecnologia em Eletronica Ltda" },
    { "ECBD09", "FUSION Electronics Ltd" },
    { "ECBD1D", "Cisco Systems, Inc" },
    { "ECBE5F", "Vestel Elektronik San ve Tic. A.S." },
    { "ECBEDD", "Sagemcom Broadband SAS" },
    { "ECBFD0", "Texas Instruments" },
    { "ECC018", "Cisco Systems, Inc" },
    { "ECC01B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "ECC06A", "PowerChord Group Limited" },
    { "ECC07A", "Laird Connectivity" },
    { "ECC1AB", "Guangzhou Shiyuan Electronic Technology Company Limited" },
    { "ECC302", "HUMAX Co., Ltd." },
    { "ECC342", "zte corporation" },
    { "ECC38A", "Accuenergy (CANADA) Inc" },
    { "ECC3B0", "zte corporation" },
    { "ECC40D", "Nintendo Co.,Ltd" },
    { "ECC57F", "Suzhou Pairlink Network Technology" },
    { "ECC5D2", "Huawei Device Co., Ltd." },
    { "ECC882", "Cisco Systems, Inc" },
    { "ECC89C", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "ECC9FF", "Espressif Inc." },
    { "ECCB30", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "ECCD4C", "New H3C Technologies Co., Ltd" },
    { "ECCD6D", "Allied Telesis, Inc." },
    { "ECCE13", "Cisco Systems, Inc" },
    { "ECCED7", "Apple, Inc." },
    { "ECCF70", "FUJIAN STAR-NET COMMUNICATION CO.,LTD" },
    { "ECD00E", "MiraeRecognition Co., Ltd." },
    { "ECD040", "GEA Farm Technologies GmbH" },
    { "ECD09F", "Xiaomi Communications Co Ltd" },
    { "ECD19A", "Zhuhai Liming Industries Co., Ltd" },
    { "ECD68A", "Shenzhen JMicron Intelligent Technology Developmen" },
    { "ECD909", "ALPSALPINE CO,.LTD" },
    { "ECD925", "RAMI" },
    { "ECD950", "IRT SA" },
    { "ECD9D1", "Shenzhen TG-NET Botone Technology Co.,Ltd." },
    { "ECDA3B", "Espressif Inc." },
    { "ECDA59", "New H3C Technologies Co., Ltd" },
    { "ECDB86", "API-K" },
    { "ECDD24", "Cisco Systems, Inc" },
    { "ECDE3D", "Lamprey Networks, Inc." },
    { "ECDF3A", "vivo Mobile Communication Co., Ltd." },
    { "ECDFC9", "Hangzhou Microimage Software Co., Ltd" },
    { "ECE09B", "Samsung Electronics Co.,Ltd" },
    { "ECE154", "Beijing Unisound Information Technology Co.,Ltd." },
    { "ECE1A9", "Cisco Systems, Inc" },
    { "ECE2FD", "SKG Electric Group(Thailand) Co., Ltd." },
    { "ECE334", "Espressif Inc." },
    { "ECE512", "tado GmbH" },
    { "ECE555", "Hirschmann Automation" },
    { "ECE61D", "Huawei Device Co., Ltd." },
    { "ECE660", "Qingdao Hisense Communications Co.,Ltd." },
    { "ECE6A2", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "ECE744", "Omntec mfg. inc" },
    { "ECE78E", "AsiaTelco Technologies Co." },
    { "ECE7A7", "Intel Corporate" },
    { "ECE7C2", "China Mobile Group Device Co.,Ltd." },
    { "ECE90B", "SISTEMA SOLUCOES ELETRONICAS LTDA - EASYTECH" },
    { "ECE915", "STI Ltd" },
    { "ECE9F5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "ECE9F8", "Guang Zhou TRI-SUN Electronics Technology Co., Ltd" },
    { "ECEA03", "DARFON LIGHTING CORP" },
    { "ECEBB8", "Hewlett Packard Enterprise " },
    { "ECED04", "Intel Corporate" },
    { "ECED73", "Motorola Mobility LLC, a Lenovo Company" },
    { "ECEED8", "ZTLX Network Technology Co.,Ltd" },
    { "ECEF17", "Sunplus Technology Co., Ltd." },
    { "ECF00E", "AboCom" },
    { "ECF0FE", "zte corporation" },
    { "ECF22B", "TECNO MOBILE LIMITED" },
    { "ECF236", "NEOMONTANA ELECTRONICS" },
    { "ECF342", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "ECF35B", "Nokia Corporation" },
    { "ECF37A", "Forecr OU" },
    { "ECF40C", "Cisco Systems, Inc" },
    { "ECF451", "Arcadyan Corporation" },
    { "ECF4BB", "Dell Inc." },
    { "ECF64C", "Silicon Laboratories" },
    { "ECF6BD", "SNCF MOBILITÉS" },
    { "ECF72B", "HD DIGITAL TECH CO., LTD." },
    { "ECF8D0", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "ECF8EB", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "ECFA03", "FCA" },
    { "ECFA5C", "Beijing Xiaomi Electronics Co., Ltd." },
    { "ECFAAA", "The IMS Company" },
    { "ECFABC", "Espressif Inc." },
    { "ECFAF4", "SenRa Tech Pvt. Ltd" },
    { "ECFC2F", "Sagemcom Broadband SAS" },
    { "ECFC55", "A. Eberle GmbH & Co. KG" },
    { "ECFCC6", "Hewlett Packard Enterprise" },
    { "ECFE7E", "BlueRadios, Inc." },
    { "ECFF3A", "Apple, Inc." },
    { "F0007F", "Janz - Contadores de Energia, SA" },
    { "F0016E", "Tianyi Telecom Terminals Company Limited" },
    { "F0022B", "Chrontel" },
    { "F00248", "SmarteBuilding" },
    { "F0038C", "AzureWave Technology Inc." },
    { "F003BC", "Cisco Systems, Inc" },
    { "F004E1", "Apple, Inc." },
    { "F0051B", "Samsung Electronics Co.,Ltd" },
    { "F00727", "INTEREL BUILDING AUTOMATION" },
    { "F00786", "Shandong Bittel Electronics Co., Ltd" },
    { "F008D1", "Espressif Inc." },
    { "F008F1", "Samsung Electronics Co.,Ltd" },
    { "F0090D", "TP-Link Systems Inc" },
    { "F00D5C", "JinQianMao Technology Co.,Ltd." },
    { "F00DF5", "ACOMA Medical Industry Co,. Ltd." },
    { "F00E1D", "Megafone Limited" },
    { "F00EBF", "ZettaHash Inc." },
    { "F00FEC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F01090", "New H3C Technologies Co., Ltd" },
    { "F010A5", "Texas Instruments" },
    { "F010AB", "China Mobile (Hangzhou) Information Technology Co., Ltd." },
    { "F01204", "IEEE Registration Authority" },
    { "F013C1", "Hannto Technology Co., Ltd" },
    { "F013C3", "SHENZHEN FENDA TECHNOLOGY CO., LTD" },
    { "F015A0", "KyungDong One Co., Ltd." },
    { "F015B9", "PlayFusion Limited" },
    { "F01628", "Technicolor (China) Technology Co., Ltd." },
    { "F0182B", "LG Chem" },
    { "F01898", "Apple, Inc." },
    { "F01AA0", "Hewlett Packard Enterprise" },
    { "F01B24", "zte corporation" },
    { "F01B6C", "vivo Mobile Communication Co., Ltd." },
    { "F01C13", "LG Electronics (Mobile Communications)" },
    { "F01C2D", "Juniper Networks" },
    { "F01D2D", "Cisco Systems, Inc" },
    { "F01DBC", "Microsoft Corporation" },
    { "F01E34", "ORICO Technologies Co., Ltd" },
    { "F01EAC", "Rentokil Initial" },
    { "F01FAF", "Dell Inc." },
    { "F01FC7", "Apple, Inc." },
    { "F020FF", "Intel Corporate" },
    { "F02178", "UNIONMAN TECHNOLOGY CO.,LTD" },
    { "F0219D", "Cal-Comp Electronics & Communications Company Ltd." },
    { "F021E0", "eero inc." },
    { "F0221D", "IEEE Registration Authority" },
    { "F0224E", "Esan electronic co." },
    { "F02329", "SHOWA DENKI CO.,LTD." },
    { "F023AE", "AMPAK Technology,Inc." },
    { "F023B9", "IEEE Registration Authority" },
    { "F02405", "OPUS High Technology Corporation" },
    { "F02408", "Talaris (Sweden) AB" },
    { "F02475", "Apple, Inc." },
    { "F024F9", "Espressif Inc." },
    { "F02572", "Cisco Systems, Inc" },
    { "F0258E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F025B7", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "F02624", "WAFA TECHNOLOGIES CO., LTD." },
    { "F0264C", "Sigrist-Photometer AG" },
    { "F026F8", "Worldcns Co.,Ltd." },
    { "F0272D", "Amazon Technologies Inc." },
    { "F02745", "F-Secure Corporation" },
    { "F02765", "Murata Manufacturing Co., Ltd." },
    { "F027A0", "Apple, Inc." },
    { "F02929", "Cisco Systems, Inc" },
    { "F02A23", "Creative Next Design" },
    { "F02A2B", "IEEE Registration Authority" },
    { "F02A61", "Waldo Networks, Inc." },
    { "F02B18", "Nanjing Jiahao Technology Co., Ltd." },
    { "F02B7C", "Extreme Networks Headquarters" },
    { "F02C59", "Chipsea Technologies (Shenzhen) Crop." },
    { "F02E51", "Casa Systems" },
    { "F02F4B", "Apple, Inc." },
    { "F02F74", "ASUSTek COMPUTER INC." },
    { "F02F9E", "Amazon Technologies Inc." },
    { "F02FA7", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F02FD8", "Bi2-Vision" },
    { "F0321A", "Mita-Teknik A/S" },
    { "F033E5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F03404", "TCT mobile ltd" },
    { "F03575", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "F037A1", "Huike Electronics (SHENZHEN) CO., LTD." },
    { "F037CF", "Huawei Device Co., Ltd." },
    { "F03965", "Samsung Electronics Co.,Ltd" },
    { "F03A4B", "Bloombase, Inc." },
    { "F03A55", "Omega Elektronik AS" },
    { "F03D03", "TECNO MOBILE LIMITED" },
    { "F03D29", "Actility" },
    { "F03E90", "Ruckus Wireless" },
    { "F03EBF", "GOGORO TAIWAN LIMITED" },
    { "F03F03", "Private" },
    { "F03F95", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F03FF8", "R L Drake" },
    { "F0407B", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "F040EC", "RainX PTE. LTD." },
    { "F041C6", "Heat Tech Company, Ltd." },
    { "F041C8", "IEEE Registration Authority" },
    { "F0421C", "Intel Corporate" },
    { "F042F5", "Huawei Device Co., Ltd." },
    { "F04335", "DVN(Shanghai)Ltd." },
    { "F04347", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F045DA", "Texas Instruments" },
    { "F0463B", "Comcast Cable Corporation" },
    { "F04A02", "Cisco Systems, Inc" },
    { "F04A2B", "PYRAMID Computer GmbH" },
    { "F04A3D", "Bosch Thermotechnik GmbH" },
    { "F04B3A", "Juniper Networks" },
    { "F04B6A", "Scientific Production Association Siberian Arsenal, Ltd." },
    { "F04B8A", "Vantiva USA LLC" },
    { "F04BF2", "JTECH Communications, Inc." },
    { "F04CD5", "Maxlinear, Inc" },
    { "F04DA2", "Dell Inc." },
    { "F04DD4", "Sagemcom Broadband SAS" },
    { "F04F7C", "Amazon Technologies Inc." },
    { "F05136", "TCT mobile ltd" },
    { "F051EA", "Fitbit, Inc." },
    { "F05494", "Honeywell Connected Building" },
    { "F05501", "Huawei Device Co., Ltd." },
    { "F0578D", "JetHome LLC" },
    { "F057A6", "Intel Corporate" },
    { "F05849", "CareView Communications" },
    { "F05A09", "Samsung Electronics Co.,Ltd" },
    { "F05B7B", "Samsung Electronics Co.,Ltd" },
    { "F05C0E", "Huawei Device Co., Ltd." },
    { "F05C19", "Hewlett Packard Enterprise" },
    { "F05C77", "Google, Inc." },
    { "F05CD5", "Apple, Inc." },
    { "F05D89", "Dycon Limited" },
    { "F05DC8", "Duracell Powermat" },
    { "F05ECD", "Texas Instruments" },
    { "F05F5A", "Getriebebau NORD GmbH and Co. KG" },
    { "F06130", "Advantage Pharmacy Services, LLC" },
    { "F061C0", "Hewlett Packard Enterprise" },
    { "F061F3", "Comcast Cable Corporation" },
    { "F0620D", "Shenzhen Egreat Tech Corp.,Ltd" },
    { "F0625A", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "F06281", "ProCurve Networking by HP" },
    { "F063F9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F06426", "Extreme Networks Headquarters" },
    { "F065AE", "Samsung Electronics Co.,Ltd" },
    { "F065C2", "Yanfeng Visteon Electronics Technology (Shanghai) Co.,Ltd." },
    { "F065DD", "Primax Electronics Ltd." },
    { "F06728", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "F06853", "Integrated Corporation" },
    { "F06865", "Taicang T&W Electronics" },
    { "F06BCA", "Samsung Electronics Co.,Ltd" },
    { "F06C5D", "Xiaomi Communications Co Ltd" },
    { "F06C73", "Nokia" },
    { "F06D78", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "F06E0B", "Microsoft Corporation" },
    { "F06E32", "MICROTEL INNOVATION S.R.L." },
    { "F06F46", "Ubiik" },
    { "F06FCE", "Ruckus Wireless" },
    { "F0704F", "Samsung Electronics Co.,Ltd" },
    { "F0728C", "Samsung Electronics Co.,Ltd" },
    { "F072EA", "Google, Inc." },
    { "F073AE", "PEAK-System Technik" },
    { "F07485", "NGD Systems, Inc." },
    { "F0748D", "Ruijie Networks Co.,LTD" },
    { "F074BF", "Silicon Laboratories" },
    { "F074E4", "Thundercomm Technology Co., Ltd" },
    { "F0761C", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "F0766F", "Apple, Inc." },
    { "F07765", "Sourcefire, Inc" },
    { "F077C3", "Intel Corporate" },
    { "F077D0", "Xcellen" },
    { "F07807", "Apple, Inc." },
    { "F07816", "Cisco Systems, Inc" },
    { "F07959", "ASUSTek COMPUTER INC." },
    { "F07960", "Apple, Inc." },
    { "F079E8", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "F07A55", "zte corporation" },
    { "F07B65", "Sagemcom Broadband SAS" },
    { "F07BCB", "Hon Hai Precision Ind. Co.,Ltd." },
    { "F07CC7", "Juniper Networks" },
    { "F07D68", "D-Link Corporation" },
    { "F07F06", "Cisco Systems, Inc" },
    { "F07F0C", "Leopold Kostal GmbH &Co. KG" },
    { "F08173", "Amazon Technologies Inc." },
    { "F08175", "Sagemcom Broadband SAS" },
    { "F081AF", "IRZ AUTOMATION TECHNOLOGIES LTD" },
    { "F08261", "Sagemcom Broadband SAS" },
    { "F082C0", "Silicon Laboratories" },
    { "F0842F", "ADB Broadband Italia" },
    { "F084C9", "zte corporation" },
    { "F085C1", "SHENZHEN RF-LINK TECHNOLOGY CO.,LTD." },
    { "F08620", "Arcadyan Corporation" },
    { "F0866F", "EM Microelectronic" },
    { "F08756", "Zyxel Communications Corporation" },
    { "F0877F", "Magnetar Technology Shenzhen Co., LTD." },
    { "F08A28", "JIANGSU HENGSION ELECTRONIC S and T CO.,LTD" },
    { "F08A76", "Samsung Electronics Co.,Ltd" },
    { "F08BFE", "COSTEL.,CO.LTD" },
    { "F08CFB", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "F08EDB", "VeloCloud Networks" },
    { "F09008", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "F0921C", "Hewlett Packard" },
    { "F092B4", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "F0933A", "NxtConect" },
    { "F093C5", "Garland Technology" },
    { "F095F1", "Carl Zeiss AG" },
    { "F09602", "Earda Technologies co Ltd" },
    { "F097E5", "TAMIO, INC" },
    { "F09838", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F0989D", "Apple, Inc." },
    { "F09919", "Garmin International" },
    { "F099B6", "Apple, Inc." },
    { "F099BF", "Apple, Inc." },
    { "F09A51", "Shanghai Viroyal Electronic Technology Company Limited" },
    { "F09BB8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F09CBB", "RaonThink Inc." },
    { "F09CD7", "Guangzhou Blue Cheetah Intelligent Technology Co., Ltd." },
    { "F09CE9", "Extreme Networks Headquarters" },
    { "F09E4A", "Intel Corporate" },
    { "F09E63", "Cisco Systems, Inc" },
    { "F09E9E", "Espressif Inc." },
    { "F09FC2", "Ubiquiti Inc" },
    { "F09FFC", "SHARP Corporation" },
    { "F0A0B1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F0A225", "Amazon Technologies Inc." },
    { "F0A35A", "Apple, Inc." },
    { "F0A3B2", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "F0A654", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "F0A731", "TP-Link Systems Inc" },
    { "F0A764", "GST Co., Ltd." },
    { "F0A7B2", "FUTABA CORPORATION" },
    { "F0A882", "shenzhen trolink Technology Co.,Ltd" },
    { "F0A951", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F0A968", "Antailiye Technology Co.,Ltd" },
    { "F0AA0B", "Arra Networks/ Spectramesh" },
    { "F0AB1F", "zte corporation" },
    { "F0AB54", "MITSUMI ELECTRIC CO.,LTD." },
    { "F0ABF3", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "F0ACA4", "HBC-radiomatic" },
    { "F0ACD7", "IEEE Registration Authority" },
    { "F0AD4E", "Globalscale Technologies, Inc." },
    { "F0AE51", "Xi3 Corp" },
    { "F0AE66", "Cosonic Intelligent Technologies Co., Ltd." },
    { "F0AF50", "Phantom Intelligence" },
    { "F0AF85", "Commscope" },
    { "F0B014", "AVM Audiovisuelles Marketing und Computersysteme GmbH" },
    { "F0B022", "TOHO Electronics INC." },
    { "F0B040", "HUNAN FN-LINK TECHNOLOGY LIMITED" },
    { "F0B052", "Ruckus Wireless" },
    { "F0B0E7", "Apple, Inc." },
    { "F0B107", "Ericsson AB" },
    { "F0B11D", "Nokia" },
    { "F0B13F", "Huawei Device Co., Ltd." },
    { "F0B2B9", "Intel Corporate" },
    { "F0B2E5", "Cisco Systems, Inc" },
    { "F0B31E", "Universal Electronics, Inc." },
    { "F0B3EC", "Apple, Inc." },
    { "F0B429", "Xiaomi Communications Co Ltd" },
    { "F0B479", "Apple, Inc." },
    { "F0B4D2", "D-Link International" },
    { "F0B5B7", "Disruptive Technologies Research AS" },
    { "F0B5D1", "Texas Instruments" },
    { "F0B61E", "Intel Corporate" },
    { "F0B661", "eero inc." },
    { "F0B6EB", "Poslab Technology Co., Ltd." },
    { "F0B968", "ITEL MOBILE LIMITED" },
    { "F0BCC8", "MaxID (Pty) Ltd" },
    { "F0BCC9", "PFU LIMITED" },
    { "F0BD2E", "H+S Polatis Ltd" },
    { "F0BDEE", "Huawei Device Co., Ltd." },
    { "F0BDF1", "Sipod Inc." },
    { "F0BE25", "Dongguan Cannice Precision Manufacturing Co., Ltd." },
    { "F0BF97", "Sony Corporation" },
    { "F0C1CE", "GoodWe Technologies CO., Ltd" },
    { "F0C1F1", "Apple, Inc." },
    { "F0C24C", "Zhejiang FeiYue Digital Technology Co., Ltd" },
    { "F0C27C", "Mianyang Netop Telecom Equipment Co.,Ltd." },
    { "F0C371", "Apple, Inc." },
    { "F0C42F", "Huawei Device Co., Ltd." },
    { "F0C478", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F0C558", "U.D.Electronic Corp." },
    { "F0C725", "Apple, Inc." },
    { "F0C745", "TECNO MOBILE LIMITED" },
    { "F0C77F", "Texas Instruments" },
    { "F0C814", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "F0C850", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F0C88C", "LeddarTech Inc." },
    { "F0C8B5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F0C9D1", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "F0CBA1", "Apple, Inc." },
    { "F0CCE0", "Shenzhen All-Smartlink Technology Co.,Ltd." },
    { "F0CD31", "Samsung Electronics Co.,Ltd" },
    { "F0CF4D", "BitRecords GmbH" },
    { "F0D08C", "TCT mobile ltd" },
    { "F0D14F", "LINEAR LLC" },
    { "F0D1A9", "Apple, Inc." },
    { "F0D1B8", "LEDVANCE" },
    { "F0D2F1", "Amazon Technologies Inc." },
    { "F0D31F", "Apple, Inc." },
    { "F0D3A7", "CobaltRay Co., Ltd" },
    { "F0D3E7", "Sensometrix SA" },
    { "F0D415", "Intel Corporate" },
    { "F0D4E2", "Dell Inc." },
    { "F0D4F6", "Lars Thrane A/S" },
    { "F0D4F7", "varram system" },
    { "F0D506", "Ubee Interactive Co., Limited" },
    { "F0D5BF", "Intel Corporate" },
    { "F0D635", "Apple, Inc." },
    { "F0D657", "ECHOSENS" },
    { "F0D767", "Axema Passagekontroll AB" },
    { "F0D793", "Apple, Inc." },
    { "F0D7AA", "Motorola Mobility LLC, a Lenovo Company" },
    { "F0D7AF", "IEEE Registration Authority" },
    { "F0D7DC", "Wesine (Wuhan) Technology Co., Ltd." },
    { "F0D7EE", "Huawei Device Co., Ltd." },
    { "F0D805", "Cisco Systems, Inc" },
    { "F0D9B2", "EXO S.A." },
    { "F0DA7C", "RLH INDUSTRIES,INC." },
    { "F0DB2A", "LANNER ELECTRONICS, INC." },
    { "F0DB30", "Yottabyte" },
    { "F0DBE2", "Apple, Inc." },
    { "F0DBF8", "Apple, Inc." },
    { "F0DCE2", "Apple, Inc." },
    { "F0DE71", "Shanghai EDO Technologies Co.,Ltd." },
    { "F0DEB9", "ShangHai Y&Y Electronics Co., Ltd" },
    { "F0DEF1", "Wistron Infocomm (Zhongshan) Corporation" },
    { "F0E3DC", "Tecon MT, LLC" },
    { "F0E4A2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F0E5C3", "Drägerwerk AG & Co. KG aA" },
    { "F0E752", "Shenzhen Huajuxin Semiconductor Co.,ltd" },
    { "F0E77E", "Samsung Electronics Co.,Ltd" },
    { "F0EBD0", "Shanghai Feixun Communication Co.,Ltd." },
    { "F0EC39", "Essec" },
    { "F0ED19", "zte corporation" },
    { "F0ED1E", "Bilkon Bilgisayar Kontrollu Cih. Im.Ltd." },
    { "F0EDB8", "SERVERCOM (INDIA) PRIVATE LIMITED" },
    { "F0EE10", "Samsung Electronics Co.,Ltd" },
    { "F0EE58", "PACE Telematics GmbH" },
    { "F0EE7A", "Apple, Inc." },
    { "F0EEBB", "VIPAR GmbH" },
    { "F0EF86", "Google, Inc." },
    { "F0EFD2", "TF PAYMENT SERVICE CO., LTD" },
    { "F0F002", "Hon Hai Precision Ind. Co.,Ltd." },
    { "F0F069", "Private" },
    { "F0F08F", "Nextek Solutions Pte Ltd" },
    { "F0F0A4", "Amazon Technologies Inc." },
    { "F0F249", "Hitron Technologies. Inc" },
    { "F0F260", "Mobitec AB" },
    { "F0F336", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "F0F564", "Samsung Electronics Co.,Ltd" },
    { "F0F56D", "Kubota Corporation" },
    { "F0F5AE", "Adaptrum Inc." },
    { "F0F5BD", "Espressif Inc." },
    { "F0F61C", "Apple, Inc." },
    { "F0F644", "Whitesky Science & Technology Co.,Ltd." },
    { "F0F669", "Motion Analysis Corporation" },
    { "F0F69C", "NIO Co., Ltd." },
    { "F0F6C1", "Sonos, Inc." },
    { "F0F755", "Cisco Systems, Inc" },
    { "F0F7B3", "Phorm" },
    { "F0F7E7", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F0F7FC", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F0F842", "KEEBOX, Inc." },
    { "F0F84A", "BUFFALO.INC" },
    { "F0F8F2", "Texas Instruments" },
    { "F0F9F7", "IES GmbH & Co. KG" },
    { "F0FAC7", "Huawei Device Co., Ltd." },
    { "F0FC65", "SynaXG Technologies Pte. Ltd." },
    { "F0FCC8", "Commscope" },
    { "F0FD45", "Silicon Laboratories" },
    { "F0FDA0", "Acurix Networks Pty Ltd" },
    { "F0FDDD", "Foxtron Vehicle Technologies Co., Ltd." },
    { "F0FE6B", "Shanghai High-Flying Electronics Technology Co., Ltd" },
    { "F0FEE7", "Huawei Device Co., Ltd." },
    { "F40046", "ON Semiconductor" },
    { "F40223", "PAX Computer Technology(Shenzhen) Ltd." },
    { "F40228", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "F40270", "Dell Inc." },
    { "F40304", "Google, Inc." },
    { "F40321", "BeNeXt B.V." },
    { "F4032A", "Amazon Technologies Inc." },
    { "F4032F", "Reduxio Systems" },
    { "F40343", "Hewlett Packard Enterprise " },
    { "F4044C", "ValenceTech Limited" },
    { "F40595", "Sagemcom Broadband SAS" },
    { "F40616", "Apple, Inc." },
    { "F4063C", "Texas Instruments" },
    { "F40669", "Intel Corporate" },
    { "F4068D", "devolo AG" },
    { "F406A5", "Hangzhou Bianfeng Networking Technology Co., Ltd." },
    { "F409D8", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "F40A2E", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "F40A4A", "INDUSNET Communication Technology Co.,LTD" },
    { "F40B93", "BlackBerry RTS" },
    { "F40B9F", "CIG SHANGHAI CO LTD" },
    { "F40E01", "Apple, Inc." },
    { "F40E11", "IEEE Registration Authority" },
    { "F40E22", "Samsung Electronics Co.,Ltd" },
    { "F40E83", "Commscope" },
    { "F40F1B", "Cisco Systems, Inc" },
    { "F40F24", "Apple, Inc." },
    { "F40F9B", "WAVELINK" },
    { "F412DA", "zte corporation" },
    { "F412FA", "Espressif Inc." },
    { "F41399", "Aerospace new generation communications Co.,Ltd" },
    { "F414BF", "LG Innotek" },
    { "F41532", "PETAiO (NanJing), Inc." },
    { "F41535", "SPON Communication Technology Co.,Ltd" },
    { "F41563", "F5 Inc." },
    { "F415FD", "Shanghai Pateo Electronic Equipment Manufacturing Co., Ltd." },
    { "F417B8", "AirTies Wireless Networks" },
    { "F419E2", "Volterra" },
    { "F41A79", "IEEE Registration Authority" },
    { "F41A9C", "Xiaomi Communications Co Ltd" },
    { "F41AB0", "Shenzhen Xingguodu Technology Co., Ltd." },
    { "F41BA1", "Apple, Inc." },
    { "F41C71", "SHENZHEN SANMU COMMUNICATION TECHNOLOGY CO., LTD" },
    { "F41C95", "BEIJING YUNYI TIMES TECHNOLOGY CO,.LTD" },
    { "F41D6B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F41E26", "Simon-Kaloi Engineering" },
    { "F41E57", "Routerboard.com" },
    { "F41E5E", "RtBrick Inc." },
    { "F41F0B", "YAMABISHI Corporation" },
    { "F41F88", "zte corporation" },
    { "F41FC2", "Cisco Systems, Inc" },
    { "F42012", "Cuciniale GmbH" },
    { "F42015", "Guangzhou Shiyuan Electronic Technology Company Limited" },
    { "F42055", "IEEE Registration Authority" },
    { "F421AE", "Shanghai Xiaodu Technology Limited" },
    { "F421CA", "Apple, Inc." },
    { "F4227A", "Guangdong Seneasy Intelligent Technology Co., Ltd." },
    { "F4239C", "SERNET (SUZHOU) TECHNOLOGIES CORPORATION" },
    { "F42462", "Selcom Electronics (Shanghai) Co., Ltd" },
    { "F4248B", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F42679", "Intel Corporate" },
    { "F42756", "DASAN Newtork Solutions" },
    { "F42833", "MMPC Inc." },
    { "F42853", "Zioncom Electronics (Shenzhen) Ltd." },
    { "F42896", "SPECTO PAINEIS ELETRONICOS LTDA" },
    { "F4289D", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "F42981", "vivo Mobile Communication Co., Ltd." },
    { "F42A7D", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "F42B48", "Ubiqam" },
    { "F42B7D", "Chipsguide technology CO.,LTD." },
    { "F42B8C", "Samsung Electronics Co.,Ltd" },
    { "F42C56", "SENOR TECH CO LTD" },
    { "F42D06", "zte corporation" },
    { "F42E48", "zte corporation" },
    { "F42E7F", "Hewlett Packard Enterprise" },
    { "F4308B", "Xiaomi Communications Co Ltd" },
    { "F430B9", "Hewlett Packard" },
    { "F43149", "Pixel FX" },
    { "F431C3", "Apple, Inc." },
    { "F4323D", "Sichuan tianyi kanghe communications co., LTD" },
    { "F4331C", "Toast, Inc." },
    { "F43328", "CIMCON Lighting Inc." },
    { "F43392", "Cisco Systems, Inc" },
    { "F433B7", "Apple, Inc." },
    { "F434F0", "Apple, Inc." },
    { "F436E1", "Abilis Systems SARL" },
    { "F437B7", "Apple, Inc." },
    { "F43814", "Shanghai Howell Electronic Co.,Ltd" },
    { "F438C1", "Huawei Device Co., Ltd." },
    { "F43909", "Hewlett Packard" },
    { "F439A6", "Apple, Inc." },
    { "F43A7B", "zte corporation" },
    { "F43AFA", "IEEE Registration Authority" },
    { "F43BD8", "Intel Corporate" },
    { "F43C3B", "HUNAN FN-LINK TECHNOLOGY LIMITED" },
    { "F43C96", "Ericsson AB" },
    { "F43D80", "FAG Industrial Services GmbH" },
    { "F43E61", "SHENZHEN GONGJIN ELECTRONICS CO.,LT" },
    { "F43E66", "Bee Computing (HK) Limited" },
    { "F43E9D", "Benu Networks, Inc." },
    { "F44156", "Arrikto Inc." },
    { "F4419E", "Huawei Device Co., Ltd." },
    { "F44227", "S & S Research Inc." },
    { "F44250", "Bouffalo Lab (Nanjing) Co., Ltd." },
    { "F4428F", "Samsung Electronics Co.,Ltd" },
    { "F4442C", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "F44450", "BND Co., Ltd." },
    { "F44588", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F445ED", "Portable Innovation Technology Ltd." },
    { "F4462A", "maxon zub" },
    { "F44637", "Intel Corporate" },
    { "F44713", "Leading Public Performance Co., Ltd." },
    { "F4472A", "Nanjing Rousing Sci. and Tech. Industrial Co., Ltd" },
    { "F44848", "Amscreen Group Ltd" },
    { "F44955", "MIMO TECH Co., Ltd." },
    { "F449EF", "EMSTONE" },
    { "F44B2A", "Cisco SPVTG" },
    { "F44C70", "Skyworth Digital Technology(Shenzhen) Co.,Ltd" },
    { "F44C7F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F44D17", "GOLDCARD HIGH-TECH CO.,LTD." },
    { "F44D30", "Elitegroup Computer Systems Co.,Ltd." },
    { "F44D5C", "Zyxel Communications Corporation" },
    { "F44DAD", "Cable Matters Inc." },
    { "F44E05", "Cisco Systems, Inc" },
    { "F44E38", "Olibra LLC" },
    { "F44EB4", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "F44EE3", "Intel Corporate" },
    { "F44EFD", "Actions Semiconductor Co.,Ltd.(Cayman Islands)" },
    { "F44FD3", "shenzhen hemuwei technology co.,ltd" },
    { "F450EB", "Telechips Inc" },
    { "F45214", "Mellanox Technologies, Inc." },
    { "F45246", "ASKEY COMPUTER CORP" },
    { "F45293", "Apple, Inc." },
    { "F45420", "TELLESCOM INDUSTRIA E COMERCIO EM TELECOMUNICACAO " },
    { "F45424", "Extreme Networks Headquarters" },
    { "F45433", "Rockwell Automation" },
    { "F45595", "HENGBAO Corporation LTD." },
    { "F4559C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F455E0", "Niceway CNC Technology Co.,Ltd.Hunan Province" },
    { "F4573E", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "F45842", "Boxx TV Ltd" },
    { "F45B29", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F45B73", "Wanjiaan Interconnected Technology Co., Ltd" },
    { "F45C89", "Apple, Inc." },
    { "F45EAB", "Texas Instruments" },
    { "F45F69", "Matsufu Electronics distribution Company" },
    { "F45FD4", "Cisco SPVTG" },
    { "F45FF7", "DQ Technology Inc." },
    { "F4600D", "Panoptic Technology, Inc" },
    { "F46077", "Texas Instruments" },
    { "F460E2", "Xiaomi Communications Co Ltd" },
    { "F462D0", "Not for Radio, LLC" },
    { "F462DC", "Huawei Device Co., Ltd." },
    { "F4631F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F46349", "Diffon Corporation" },
    { "F4635A", "NXP Semiconductor (Tianjin) LTD." },
    { "F463E7", "Nanjing Maxon O.E. Tech. Co., LTD" },
    { "F463FC", "vivo Mobile Communication Co., Ltd." },
    { "F46412", "Sony Interactive Entertainment Inc." },
    { "F4645D", "Toshiba" },
    { "F4650B", "Espressif Inc." },
    { "F465A6", "Apple, Inc." },
    { "F4672D", "ShenZhen Topstar Technology Company" },
    { "F46802", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F46942", "ASKEY COMPUTER CORP" },
    { "F469D5", "IEEE Registration Authority" },
    { "F46A92", "SHENZHEN FAST TECHNOLOGIES CO.,LTD" },
    { "F46ABC", "Adonit Corp. Ltd." },
    { "F46AD7", "Microsoft Corporation" },
    { "F46ADD", "Liteon Technology Corporation" },
    { "F46B8C", "Hon Hai Precision Industry Co., Ltd." },
    { "F46BEF", "Sagemcom Broadband SAS" },
    { "F46C68", "Wistron Neweb Corporation" },
    { "F46D04", "ASUSTek COMPUTER INC." },
    { "F46D2F", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "F46D3F", "Intel Corporate" },
    { "F46DE2", "zte corporation" },
    { "F46E24", "NEC Personal Computers, Ltd." },
    { "F46E95", "Extreme Networks Headquarters" },
    { "F46ED6", "EM Microelectronic" },
    { "F46F4E", "Echowell" },
    { "F46FA4", "Physik Instrumente GmbH & Co. KG" },
    { "F46FED", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "F4700C", "IEEE Registration Authority" },
    { "F470AB", "vivo Mobile Communication Co., Ltd." },
    { "F47190", "Samsung Electronics Co.,Ltd" },
    { "F47257", "Jiangxi Remote lntelligence Technology Co.,Ltd" },
    { "F47335", "Logitech Far East" },
    { "F473CA", "Conversion Sound Inc." },
    { "F47470", "Cisco Systems, Inc" },
    { "F47488", "New H3C Technologies Co., Ltd" },
    { "F47626", "Viltechmeda UAB " },
    { "F47946", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F47960", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F47A4E", "Woojeon&Handan" },
    { "F47ACC", "SolidFire, Inc." },
    { "F47B09", "Intel Corporate" },
    { "F47B5E", "Samsung Electronics Co.,Ltd" },
    { "F47DEF", "Samsung Electronics Co.,Ltd" },
    { "F47F35", "Cisco Systems, Inc" },
    { "F48139", "CANON INC." },
    { "F481C4", "Apple, Inc." },
    { "F4832C", "OHSUNG" },
    { "F483CD", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "F483E1", "Shanghai Clouder Semiconductor Co.,Ltd" },
    { "F4844C", "Texas Instruments" },
    { "F4848D", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "F485AE", "Senbiosys SA" },
    { "F485C6", "FDT Technologies" },
    { "F48771", "Infoblox" },
    { "F487C5", "Huawei Device Co., Ltd." },
    { "F48918", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F48B32", "Xiaomi Communications Co Ltd" },
    { "F48C50", "Intel Corporate" },
    { "F48CEB", "D-Link International" },
    { "F48E09", "Nokia Corporation" },
    { "F48E38", "Dell Inc." },
    { "F48E92", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F490CA", "Tensorcom" },
    { "F490CB", "IEEE Registration Authority" },
    { "F490EA", "Deciso B.V." },
    { "F4911E", "ZHUHAI EWPE INFORMATION TECHNOLOGY INC" },
    { "F492BF", "Ubiquiti Inc" },
    { "F4931C", "Universal Electronics, Inc." },
    { "F4939F", "Hon Hai Precision Industry Co., Ltd." },
    { "F49461", "NexGen Storage" },
    { "F49466", "CountMax, ltd" },
    { "F4951B", "Hefei Radio Communication Technology Co., Ltd " },
    { "F49634", "Intel Corporate" },
    { "F49651", "NAKAYO Inc" },
    { "F4979D", "IEEE Registration Authority" },
    { "F497C2", "Nebulon Inc" },
    { "F499AC", "WEBER Schraubautomaten GmbH" },
    { "F49C12", "Structab AB" },
    { "F49D8A", "Fantasia Trading LLC" },
    { "F49DA7", "Private" },
    { "F49EA4", "Epiq Solutions" },
    { "F49ECE", "Sena Technologies Co., Ltd." },
    { "F49EEF", "Taicang T&W Electronics" },
    { "F49F54", "Samsung Electronics Co.,Ltd" },
    { "F49FF3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F4A17F", "Marquardt Electronics Technology (Shanghai) Co.Ltd" },
    { "F4A294", "EAGLE WORLD DEVELOPMENT CO., LIMITED" },
    { "F4A310", "Apple, Inc." },
    { "F4A454", "IEEE Registration Authority" },
    { "F4A475", "Intel Corporate" },
    { "F4A4D6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F4A52A", "Hawa Technologies Inc" },
    { "F4A59D", "Huawei Device Co., Ltd." },
    { "F4A739", "Juniper Networks" },
    { "F4A80D", "Wistron InfoComm(Kunshan)Co.,Ltd." },
    { "F4A997", "CANON INC." },
    { "F4AAD0", "OHSUNG" },
    { "F4AB5C", "Quectel Wireless Solutions Co.,Ltd." },
    { "F4ACC1", "Cisco Systems, Inc" },
    { "F4AFE7", "Apple, Inc." },
    { "F4B164", "Lightning Telecommunications Technology Co. Ltd" },
    { "F4B19C", "AltoBeam (China) Inc." },
    { "F4B1C2", "Zhejiang Dahua Technology Co., Ltd." },
    { "F4B301", "Intel Corporate" },
    { "F4B381", "WindowMaster A/S" },
    { "F4B3B1", "Silicon Laboratories" },
    { "F4B520", "Biostar Microtech international corp." },
    { "F4B52F", "Juniper Networks" },
    { "F4B549", "Xiamen Yeastar Digital Technology Co., Ltd" },
    { "F4B5AA", "zte corporation" },
    { "F4B5BB", "CERAGON NETWORKS" },
    { "F4B62D", "Dongguan Huayin Electronic Technology Co., Ltd." },
    { "F4B688", "PLANTRONICS, INC." },
    { "F4B6C6", "Indra Heera Technology LLP" },
    { "F4B6E5", "TerraSem Co.,Ltd" },
    { "F4B72A", "TIME INTERCONNECT LTD" },
    { "F4B78D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F4B7B3", "vivo Mobile Communication Co., Ltd." },
    { "F4B7E2", "Hon Hai Precision Ind. Co.,Ltd." },
    { "F4B85E", "Texas Instruments" },
    { "F4B898", "Texas Instruments" },
    { "F4B8A7", "zte corporation" },
    { "F4BBC7", "vivo Mobile Communication Co., Ltd." },
    { "F4BC97", "Shenzhen Crave Communication Co., LTD" },
    { "F4BCDA", "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd" },
    { "F4BD7C", "Chengdu jinshi communication Co., LTD" },
    { "F4BD9E", "Cisco Systems, Inc" },
    { "F4BDB9", "Ubiqcom India Pvt Ltd" },
    { "F4BEEC", "Apple, Inc." },
    { "F4BF80", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F4BFA8", "Juniper Networks" },
    { "F4BFBB", "China Mobile Group Device Co.,Ltd." },
    { "F4C02F", "BlueBite" },
    { "F4C114", "Vantiva USA LLC" },
    { "F4C248", "Samsung Electronics Co.,Ltd" },
    { "F4C447", "Coagent International Enterprise Limited" },
    { "F4C4D6", "Shenzhen Xinfa Electronic Co.,ltd" },
    { "F4C613", "Alcatel-Lucent Shanghai Bell Co., Ltd" },
    { "F4C6D7", "blackned GmbH" },
    { "F4C714", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F4C795", "WEY Technology AG" },
    { "F4C7AA", "Marvell Semiconductors" },
    { "F4C7C8", "Kelvin Inc." },
    { "F4C88A", "Intel Corporate" },
    { "F4CA24", "FreeBit Co., Ltd." },
    { "F4CAE5", "FREEBOX SAS" },
    { "F4CAE7", "Arcadyan Corporation" },
    { "F4CB52", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F4CBE7", "Apple, Inc." },
    { "F4CC55", "Juniper Networks" },
    { "F4CD90", "Vispiron Rotec GmbH" },
    { "F4CE23", "Intel Corporate" },
    { "F4CE36", "Nordic Semiconductor ASA" },
    { "F4CE46", "Hewlett Packard" },
    { "F4CE48", "Extreme Networks Headquarters" },
    { "F4CFA2", "Espressif Inc." },
    { "F4CFE2", "Cisco Systems, Inc" },
    { "F4D032", "Yunnan Ideal Information&Technology.,Ltd" },
    { "F4D0A7", "Zhejiang Weilai Jingling Artificial Intelligence Technology Co., Ltd." },
    { "F4D108", "Intel Corporate" },
    { "F4D261", "SEMOCON Co., Ltd" },
    { "F4D454", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "F4D488", "Apple, Inc." },
    { "F4D580", "YAMAHA CORPORATION" },
    { "F4D58A", "Palo Alto Networks" },
    { "F4D620", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "F4D7B2", "LGS Innovations, LLC" },
    { "F4D7E4", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "F4D9C6", "UNION MAN TECHNOLOGY CO.,LTD" },
    { "F4D9FB", "Samsung Electronics Co.,Ltd" },
    { "F4DBE3", "Apple, Inc." },
    { "F4DBE6", "Cisco Systems, Inc" },
    { "F4DC41", "YOUNGZONE CULTURE (SHANGHAI) CORP" },
    { "F4DC4D", "Beijing CCD Digital Technology Co., Ltd" },
    { "F4DCA5", "DAWON DNS" },
    { "F4DCDA", "Zhuhai Jiahe Communication Technology Co., limited" },
    { "F4DCF9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F4DD06", "Samsung Electronics Co.,Ltd" },
    { "F4DD9E", "GoPro" },
    { "F4DE0C", "ESPOD Ltd." },
    { "F4DEAF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F4E11E", "Texas Instruments" },
    { "F4E142", "Delta Elektronika BV" },
    { "F4E204", "COYOTE SYSTEM" },
    { "F4E2C6", "Ubiquiti Inc" },
    { "F4E3FB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F4E451", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F4E4AD", "zte corporation" },
    { "F4E4D7", "FUJIAN STAR-NET COMMUNICATION CO.,LTD" },
    { "F4E578", "LLC Proizvodstvennaya Kompania TransService" },
    { "F4E5F2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F4E6D7", "Solar Power Technologies, Inc." },
    { "F4E84F", "zte corporation" },
    { "F4E8C7", "Apple, Inc." },
    { "F4E926", "Tianjin Zanpu Technology Inc." },
    { "F4E975", "New H3C Technologies Co., Ltd" },
    { "F4E9D4", "QLogic Corporation" },
    { "F4EA67", "Cisco Systems, Inc" },
    { "F4EAB5", "Extreme Networks Headquarters" },
    { "F4EB38", "Sagemcom Broadband SAS" },
    { "F4EB9F", "Ellu Company 2019 SL" },
    { "F4EC38", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "F4ED37", "Qingdao Yuze lntelligent Technology Co.,Ltd" },
    { "F4ED5F", "SHENZHEN KTC TECHNOLOGY GROUP " },
    { "F4EE08", "Dell Inc." },
    { "F4EE14", "MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD." },
    { "F4EE31", "Cisco Systems, Inc" },
    { "F4EF9E", "SGSG SCIENCE & TECHNOLOGY CO. LTD" },
    { "F4F15A", "Apple, Inc." },
    { "F4F197", "EMTAKE Inc" },
    { "F4F19E", "Wistron InforComm (Zhongshan) Corporation" },
    { "F4F1E1", "Motorola Mobility LLC, a Lenovo Company" },
    { "F4F26D", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "F4F28A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F4F309", "Samsung Electronics Co.,Ltd" },
    { "F4F3AA", "JBL GmbH & Co. KG" },
    { "F4F50B", "TP-Link Systems Inc." },
    { "F4F524", "Motorola Mobility LLC, a Lenovo Company" },
    { "F4F5A5", "Nokia Corporation" },
    { "F4F5D8", "Google, Inc." },
    { "F4F5DB", "Xiaomi Communications Co Ltd" },
    { "F4F5E8", "Google, Inc." },
    { "F4F646", "Dediprog Technology Co. Ltd." },
    { "F4F647", "zte corporation" },
    { "F4F70C", "Avang - neterbit" },
    { "F4F951", "Apple, Inc." },
    { "F4FBB8", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F4FBF5", "EM Microelectronic" },
    { "F4FC32", "Texas Instruments" },
    { "F4FC49", "zte corporation" },
    { "F4FCB1", "JJ Corp" },
    { "F4FD2B", "ZOYI Company" },
    { "F4FE3E", "Apple, Inc." },
    { "F4FEFB", "Samsung Electronics Co.,Ltd" },
    { "F8009D", "INTRACOM DEFENSE S.A." },
    { "F800A1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F80113", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F801B4", "LG Electronics" },
    { "F80278", "IEEE Registration Authority" },
    { "F80332", "Khomp" },
    { "F80377", "Apple, Inc." },
    { "F8042E", "SAMSUNG ELECTRO-MECHANICS(THAILAND)" },
    { "F8051C", "DRS Imaging and Targeting Solutions" },
    { "F80584", "EM Microelectronic" },
    { "F8075D", "Huawei Device Co., Ltd." },
    { "F8084F", "Sagemcom Broadband SAS" },
    { "F809A4", "Henan Thinker Rail Transportation Research Inc." },
    { "F80BBE", "Commscope" },
    { "F80BCB", "Cisco Systems, Inc" },
    { "F80BD0", "Datang Telecom communication terminal (Tianjin) Co., Ltd." },
    { "F80C58", "Taicang T&W Electronics" },
    { "F80CF3", "LG Electronics (Mobile Communications)" },
    { "F80D43", "Hon Hai Precision Ind. Co.,Ltd." },
    { "F80D4B", "Nextracker, Inc." },
    { "F80D60", "CANON INC." },
    { "F80DA9", "Zyxel Communications Corporation" },
    { "F80DAC", "HP Inc." },
    { "F80DEA", "ZyCast Technology Inc." },
    { "F80DF0", "zte corporation" },
    { "F80DF1", "Sontex SA" },
    { "F80F41", "Wistron Infocomm (Zhongshan) Corporation" },
    { "F80F6F", "Cisco Systems, Inc" },
    { "F80F84", "Natural Security SAS" },
    { "F80FF9", "Google, Inc." },
    { "F81037", "Atopia Systems, LP" },
    { "F81093", "Apple, Inc." },
    { "F810A0", "Xtreme Testek Inc." },
    { "F81308", "Nokia" },
    { "F814DD", "Cisco Systems, Inc" },
    { "F814FE", "UNION MAN TECHNOLOGY CO.,LTD" },
    { "F81547", "Avaya Inc" },
    { "F815E0", "Siemens Canada" },
    { "F8160C", "Shenzhen iComm Semiconductor CO.,LTD" },
    { "F81654", "Intel Corporate" },
    { "F8172D", "Tuya Smart Inc." },
    { "F81897", "2Wire Inc" },
    { "F81A2B", "Google, Inc." },
    { "F81A67", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "F81B04", "Zhong Shan City Richsound Electronic Industrial Ltd" },
    { "F81CE5", "Telefonbau Behnke GmbH" },
    { "F81D0F", "Hitron Technologies. Inc" },
    { "F81D78", "IEEE Registration Authority" },
    { "F81D90", "Solidwintech" },
    { "F81D93", "Longdhua(Beijing) Controls Technology Co.,Ltd" },
    { "F81E6F", "EBG compleo GmbH" },
    { "F81EDF", "Apple, Inc." },
    { "F81F32", "Motorola Mobility LLC, a Lenovo Company" },
    { "F82055", "Green Information System" },
    { "F82097", "Aditya Infotech Ltd." },
    { "F820A9", "Huawei Device Co., Ltd." },
    { "F820D2", "Vantiva USA LLC" },
    { "F82111", "HMD Global Oy" },
    { "F82229", "Nokia Shanghai Bell Co., Ltd." },
    { "F82285", "Cypress Technology CO., LTD." },
    { "F82387", "Shenzhen Horn Audio Co.,Ltd." },
    { "F823B2", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F82441", "Yeelink" },
    { "F824DB", "EntryPoint Networks, Inc" },
    { "F824E4", "Beyonics Technology Electronic (Changshu) Co., Ltd" },
    { "F82551", "Seiko Epson Corporation" },
    { "F8272E", "Mercku" },
    { "F82793", "Apple, Inc." },
    { "F82819", "Liteon Technology Corporation" },
    { "F828C9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F829C0", "Availink, Inc." },
    { "F82A53", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "F82B7F", "Huawei Device Co., Ltd." },
    { "F82BC8", "Jiangsu Switter Co., Ltd" },
    { "F82BE6", "IEEE Registration Authority" },
    { "F82C18", "2Wire Inc" },
    { "F82D7C", "Apple, Inc." },
    { "F82DC0", "Commscope" },
    { "F82E0C", "Texas Instruments" },
    { "F82E3F", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F82E8E", "Nanjing Kechen Electric Co., Ltd." },
    { "F82EDB", "RTW GmbH & Co. KG" },
    { "F82F08", "Molex CMS" },
    { "F82F5B", "eGauge Systems LLC" },
    { "F82F65", "Huawei Device Co., Ltd." },
    { "F82F6A", "ITEL MOBILE LIMITED" },
    { "F82FA8", "Hon Hai Precision Ind. Co.,Ltd." },
    { "F83002", "Texas Instruments" },
    { "F83094", "Alcatel-Lucent Telecom Limited" },
    { "F8313E", "endeavour GmbH" },
    { "F832E4", "ASUSTek COMPUTER INC." },
    { "F83331", "Texas Instruments" },
    { "F83376", "Good Mind Innovation Co., Ltd." },
    { "F83441", "Intel Corporate" },
    { "F83451", "Comcast-SRL" },
    { "F8345A", "Hitron Technologies. Inc" },
    { "F83553", "Magenta Research Ltd." },
    { "F835DD", "Gemtek Technology Co., Ltd." },
    { "F8369B", "Texas Instruments" },
    { "F83869", "LG Electronics" },
    { "F83880", "Apple, Inc." },
    { "F8388D", "New H3C Technologies Co., Ltd" },
    { "F83918", "Cisco Systems, Inc" },
    { "F83B1D", "Vantiva USA LLC" },
    { "F83B7E", "Huawei Device Co., Ltd." },
    { "F83C44", "SHENZHEN TRANSCHAN TECHNOLOGY LIMITED" },
    { "F83C80", "MITSUMI ELECTRIC CO.,LTD." },
    { "F83CBF", "BOTATO ELECTRONICS SDN BHD" },
    { "F83D4E", "Softlink Automation System Co., Ltd" },
    { "F83DC6", "AzureWave Technology Inc." },
    { "F83DFF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F83E95", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F83EB0", "Arcadyan Corporation" },
    { "F83F51", "Samsung Electronics Co.,Ltd" },
    { "F84288", "Apple, Inc." },
    { "F842FB", "Yasuda Joho Co.,ltd." },
    { "F843EF", "Xiaomi Communications Co Ltd" },
    { "F84477", "Silicon Laboratories" },
    { "F844E3", "Taicang T&W Electronics" },
    { "F845AD", "Konka Group Co., Ltd." },
    { "F845C4", "Shenzhen Netforward Micro-Electronic Co., Ltd." },
    { "F8461C", "Sony Interactive Entertainment Inc." },
    { "F8462D", "SYNTEC Incorporation" },
    { "F8472D", "X2gen Digital Corp. Ltd" },
    { "F847E3", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "F84897", "Hitachi, Ltd." },
    { "F848FD", "China Mobile Group Device Co.,Ltd." },
    { "F84A73", "EUMTECH CO., LTD" },
    { "F84A7F", "Innometriks Inc" },
    { "F84ABF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F84CDA", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F84D33", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "F84D89", "Apple, Inc." },
    { "F84D8B", "ecamtek" },
    { "F84DFC", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "F84E17", "Sony Corporation" },
    { "F84E58", "Samsung Electronics Co.,Ltd" },
    { "F84E73", "Apple, Inc." },
    { "F84F57", "Cisco Systems, Inc" },
    { "F84FAD", "Hui Zhou Gaoshengda Technology Co.,LTD" },
    { "F8501C", "Tianjin Geneuo Technology Co.,Ltd" },
    { "F85063", "Verathon" },
    { "F85128", "SimpliSafe" },
    { "F8516D", "Denwa Technology Corp." },
    { "F852DF", "VNL Europe AB" },
    { "F85329", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F854AF", "ECI Telecom Ltd." },
    { "F854B8", "Amazon Technologies Inc." },
    { "F854F6", "AzureWave Technology Inc." },
    { "F85548", "Texas Instruments" },
    { "F8554B", "WirelessMobility Engineering Centre SDN. BHD" },
    { "F855CD", "Visteon Corporation" },
    { "F856C3", "zte corporation" },
    { "F8572E", "Core Brands, LLC" },
    { "F85971", "Intel Corporate" },
    { "F85A00", "Sanford LP" },
    { "F85B3B", "ASKEY COMPUTER CORP" },
    { "F85B6E", "Samsung Electronics Co.,Ltd" },
    { "F85B9B", "iMercury" },
    { "F85B9C", "SB SYSTEMS Co.,Ltd" },
    { "F85BC9", "M-Cube Spa" },
    { "F85C24", "Sonos Inc." },
    { "F85C45", "IC Nexus Co. Ltd." },
    { "F85C4D", "Nokia" },
    { "F85C7D", "Shenzhen Honesty Electronics Co.,Ltd." },
    { "F85C7E", "Shenzhen Honesty Electronics Co.,Ltd." },
    { "F85E0B", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "F85E3C", "SHENZHEN ZHIBOTONG ELECTRONICS CO.,LTD" },
    { "F85E42", "Vantiva USA LLC" },
    { "F85EA0", "Intel Corporate" },
    { "F85F2A", "Nokia Corporation" },
    { "F860F0", "Hewlett Packard Enterprise" },
    { "F86214", "Apple, Inc." },
    { "F862AA", "xn systems" },
    { "F8633F", "Intel Corporate" },
    { "F863D9", "Commscope" },
    { "F86465", "Anova Applied Electronics, Inc." },
    { "F864B8", "zte corporation" },
    { "F86601", "Suzhou Chi-tek information technology Co., Ltd" },
    { "F8665A", "Apple, Inc." },
    { "F86691", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "F866D1", "Hon Hai Precision Ind. Co.,Ltd." },
    { "F866F2", "Cisco Systems, Inc" },
    { "F868FF", "Cisco Systems, Inc" },
    { "F86971", "Seibu Electric Co.," },
    { "F86B14", "Barrot Technology Co.,LTD" },
    { "F86BD9", "Cisco Systems, Inc" },
    { "F86BFA", "Infinix mobility limited" },
    { "F86C03", "Shenzhen Teleone Technology Co., Ltd" },
    { "F86CE1", "Taicang T&W Electronics" },
    { "F86D73", "Zengge Co., Limited" },
    { "F86DCC", "Wistron Neweb Corporation" },
    { "F86ECF", "Arcx Inc" },
    { "F86EEE", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F86FB0", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "F86FC1", "Apple, Inc." },
    { "F86FDE", "Shenzhen Goodix Technology Co.,Ltd." },
    { "F8710C", "Xiaomi Communications Co Ltd" },
    { "F871A6", "Apple, Inc." },
    { "F871FE", "The Goldman Sachs Group, Inc." },
    { "F872EA", "Cisco Systems, Inc" },
    { "F8731A", "zte corporation" },
    { "F87394", "NETGEAR" },
    { "F873A2", "Avaya Inc" },
    { "F873DF", "Apple, Inc." },
    { "F87588", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F875A4", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "F8769B", "Neopis Co., Ltd." },
    { "F877B8", "Samsung Electronics Co.,Ltd" },
    { "F87907", "Huawei Device Co., Ltd." },
    { "F8790A", "Commscope" },
    { "F87928", "zte corporation" },
    { "F87999", "Guangdong Jiuzhi Technology Co.,Ltd" },
    { "F87A39", "IEEE Registration Authority" },
    { "F87A41", "Cisco Systems, Inc" },
    { "F87AEF", "Rosonix Technology, Inc." },
    { "F87B20", "Cisco Systems, Inc" },
    { "F87B62", "FASTWEL INTERNATIONAL CO., LTD. Taiwan Branch" },
    { "F87B7A", "Commscope" },
    { "F87B8C", "Amped Wireless" },
    { "F87BE0", "Funtime Pickleball Inc." },
    { "F87D3F", "Huawei Device Co., Ltd." },
    { "F87D76", "Apple, Inc." },
    { "F87FA5", "GREATEK" },
    { "F88096", "Elsys Equipamentos Eletrônicos Ltda" },
    { "F8811A", "OVERKIZ" },
    { "F88200", "CaptionCall" },
    { "F88306", "Beijing Xiaomi Mobile Software Co., Ltd" },
    { "F88479", "Yaojin Technology(Shenzhen)Co.,Ltd" },
    { "F884F2", "Samsung Electronics Co.,Ltd" },
    { "F885F9", "Calix Inc." },
    { "F887F1", "Apple, Inc." },
    { "F8893C", "Inventec Appliances Corp." },
    { "F889D2", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "F88A3C", "IEEE Registration Authority" },
    { "F88A5E", "Texas Instruments" },
    { "F88B37", "Commscope" },
    { "F88C1C", "KAISHUN ELECTRONIC TECHNOLOGY CO., LTD. BEIJING" },
    { "F88C21", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "F88DEF", "Tenebraex" },
    { "F88E85", "Comtrend Corporation" },
    { "F88EA1", "Edgecore Networks Corporation" },
    { "F88F07", "Samsung Electronics Co.,Ltd" },
    { "F88FC8", "Chipsea Technologies (Shenzhen) Corp." },
    { "F88FCA", "Google, Inc." },
    { "F89066", "Nain Inc." },
    { "F8912A", "GLP German Light Products GmbH" },
    { "F8916F", "Texas Instruments" },
    { "F89173", "AEDLE SAS" },
    { "F893F3", "VOLANS" },
    { "F89497", "BUFFALO.INC" },
    { "F894C2", "Intel Corporate" },
    { "F89522", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F89550", "Proton Products Chengdu Ltd" },
    { "F895C7", "LG Electronics (Mobile Communications)" },
    { "F895EA", "Apple, Inc." },
    { "F896FE", "LG Innotek" },
    { "F89725", "OPPLE LIGHTING CO., LTD" },
    { "F89753", "Huawei Device Co., Ltd." },
    { "F897A9", "Ericsson AB" },
    { "F897B0", "Goki Pty Ltd" },
    { "F897CF", "DAESHIN-INFORMATION TECHNOLOGY CO., LTD." },
    { "F8983A", "Leeman International (HongKong) Limited" },
    { "F898B9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F898EF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F89910", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "F89955", "Fortress Technology Inc" },
    { "F89A25", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F89A78", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F89B6E", "Nokia Solutions and Networks GmbH & Co. KG" },
    { "F89D0D", "Control Technology Inc." },
    { "F89D9D", "Shenzhen MinewSemi Co.,LTD." },
    { "F89DBB", "Tintri" },
    { "F89E28", "Cisco Meraki" },
    { "F89E94", "Intel Corporate" },
    { "F89FB8", "YAZAKI Energy System Corporation" },
    { "F8A03D", "Dinstar Technologies Co., Ltd." },
    { "F8A097", "Commscope" },
    { "F8A188", "LED Roadway Lighting" },
    { "F8A26D", "CANON INC." },
    { "F8A2B4", "RHEWA-WAAGENFABRIK August Freudewald GmbH &amp;Co. KG" },
    { "F8A2D6", "Liteon Technology Corporation" },
    { "F8A34F", "zte corporation" },
    { "F8A45F", "Xiaomi Communications Co Ltd" },
    { "F8A475", "PT Indoreka Jaya Wutama" },
    { "F8A4FB", "Nanjing Decowell Automation Co.,LTD" },
    { "F8A5C5", "Cisco Systems, Inc" },
    { "F8A73A", "Cisco Systems, Inc" },
    { "F8A763", "Zhejiang Tmall Technology Co., Ltd." },
    { "F8A91F", "ZVISION Technologies Co., Ltd" },
    { "F8A963", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "F8A9D0", "LG Electronics (Mobile Communications)" },
    { "F8A9DE", "PUISSANCE PLUS" },
    { "F8AA3F", "DWnet Technologies(Suzhou) Corporation" },
    { "F8AA8A", "Axview Technology (Shenzhen) Co.,Ltd" },
    { "F8AAB3", "DESSMANN (China) Machinery & Electronic Co., Ltd" },
    { "F8AB05", "Sagemcom Broadband SAS" },
    { "F8AB82", "Xiaomi Communications Co Ltd" },
    { "F8ABE5", "shenzhen worldelite electronics co., LTD" },
    { "F8AC65", "Intel Corporate" },
    { "F8AC6D", "Deltenna Ltd" },
    { "F8ACC1", "InnoXings Co., LTD." },
    { "F8AD24", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "F8ADCB", "HMD Global Oy" },
    { "F8AE27", "John Deere Electronic Solutions" },
    { "F8AF05", "Huawei Device Co., Ltd." },
    { "F8AFDB", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "F8B132", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F8B156", "Dell Inc." },
    { "F8B1DD", "Apple, Inc." },
    { "F8B2F3", "GUANGZHOU BOSMA TECHNOLOGY CO.,LTD" },
    { "F8B3B7", "Espressif Inc." },
    { "F8B46A", "Hewlett Packard" },
    { "F8B54D", "Intel Corporate" },
    { "F8B568", "IEEE Registration Authority" },
    { "F8B599", "Guangzhou CHNAVS Digital Technology Co.,Ltd" },
    { "F8B797", "NEC Platforms, Ltd." },
    { "F8B7E2", "Cisco Systems, Inc" },
    { "F8B8B4", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "F8B95A", "LG Innotek" },
    { "F8BAE6", "Nokia" },
    { "F8BBBF", "eero inc." },
    { "F8BC0E", "eero inc." },
    { "F8BC12", "Dell Inc." },
    { "F8BC41", "Rosslare Enterprises Limited" },
    { "F8BE0D", "A2UICT Co.,Ltd." },
    { "F8BF09", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F8C001", "Juniper Networks" },
    { "F8C091", "Highgates Technology" },
    { "F8C116", "Juniper Networks" },
    { "F8C120", "Xi'an Link-Science Technology Co.,Ltd" },
    { "F8C249", "AMPERE COMPUTING LLC" },
    { "F8C288", "Cisco Systems, Inc" },
    { "F8C372", "TSUZUKI DENKI" },
    { "F8C397", "NZXT Corp. Ltd." },
    { "F8C39E", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F8C3CC", "Apple, Inc." },
    { "F8C3F1", "Raytron Photonics Co.,Ltd." },
    { "F8C4AE", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "F8C4F3", "Shanghai Infinity Wireless Technologies Co.,Ltd." },
    { "F8C4FA", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "F8C650", "Cisco Systems, Inc" },
    { "F8C678", "Carefusion" },
    { "F8C903", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "F8C96C", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "F8CA59", "NetComm Wireless" },
    { "F8CA85", "NEC Corporation" },
    { "F8CAB8", "Dell Inc." },
    { "F8CC6E", "DEPO Electronics Ltd" },
    { "F8CDC8", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "F8CE07", "ZHEJIANG DAHUA TECHNOLOGYCO.,LTD" },
    { "F8CE21", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "F8CE72", "Wistron Corporation" },
    { "F8CF52", "Intel Corporate" },
    { "F8CFC5", "Motorola Mobility LLC, a Lenovo Company" },
    { "F8D027", "Seiko Epson Corporation" },
    { "F8D0AC", "Sony Interactive Entertainment Inc." },
    { "F8D0BD", "Samsung Electronics Co.,Ltd" },
    { "F8D0C5", "Sector Alarm Tech S.L." },
    { "F8D111", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "F8D2AC", "Vantiva USA LLC" },
    { "F8D3A9", "AXAN Networks" },
    { "F8D3F0", "Apple, Inc." },
    { "F8D462", "Pumatronix Equipamentos Eletronicos Ltda." },
    { "F8D478", "Flextronics Tech.(Ind) Pvt Ltd" },
    { "F8D554", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "F8D756", "Simm Tronic Limited " },
    { "F8D758", "Veratron AG" },
    { "F8D7BF", "REV Ritter GmbH" },
    { "F8D9B8", "Open Mesh, Inc." },
    { "F8DA0C", "Hon Hai Precision Ind. Co.,Ltd." },
    { "F8DADF", "EcoTech, Inc." },
    { "F8DAE2", "NDC Technologies" },
    { "F8DAF4", "Taishan Online Technology Co., Ltd." },
    { "F8DB4C", "PNY Technologies, INC." },
    { "F8DB7F", "HTC Corporation" },
    { "F8DB88", "Dell Inc." },
    { "F8DC7A", "Variscite LTD" },
    { "F8DE73", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F8DF15", "Sunitec Enterprise Co.,Ltd" },
    { "F8DFA8", "zte corporation" },
    { "F8DFE1", "MyLight Systems" },
    { "F8E079", "Motorola Mobility LLC, a Lenovo Company" },
    { "F8E252", "Apple, Inc." },
    { "F8E35F", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "F8E43B", "ASIX Electronics Corporation" },
    { "F8E44E", "MCOT INC." },
    { "F8E4A4", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "F8E4E3", "Intel Corporate" },
    { "F8E4FB", "Actiontec Electronics, Inc" },
    { "F8E57E", "Cisco Systems, Inc" },
    { "F8E5CE", "Apple, Inc." },
    { "F8E5CF", "CGI IT UK LIMITED" },
    { "F8E61A", "Samsung Electronics Co.,Ltd" },
    { "F8E71E", "Ruckus Wireless" },
    { "F8E7A0", "vivo Mobile Communication Co., Ltd." },
    { "F8E7B5", "µTech Tecnologia LTDA" },
    { "F8E811", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F8E877", "Harman/Becker Automotive Systems GmbH" },
    { "F8E903", "D-Link International" },
    { "F8E94E", "Apple, Inc." },
    { "F8E94F", "Cisco Systems, Inc" },
    { "F8E968", "Egker Kft." },
    { "F8EA0A", "Dipl.-Math. Michael Rauch" },
    { "F8EDA5", "Commscope" },
    { "F8EDAE", "MOBIWIRE MOBILES(NINGBO) CO.,LTD" },
    { "F8EDFC", "HP Inc." },
    { "F8EF5D", "Motorola Mobility LLC, a Lenovo Company" },
    { "F8EFB1", "Hangzhou Zhongxinhui lntelligent Technology Co.,Ltd." },
    { "F8F005", "Newport Media Inc." },
    { "F8F014", "RackWare Inc." },
    { "F8F082", "NAGTECH LLC" },
    { "F8F09D", "Hangzhou Prevail Communication Technology Co., Ltd" },
    { "F8F0C5", "Suzhou Kuhan Information Technologies Co.,Ltd." },
    { "F8F1B6", "Motorola Mobility LLC, a Lenovo Company" },
    { "F8F1E6", "Samsung Electronics Co.,Ltd" },
    { "F8F21E", "Intel Corporate" },
    { "F8F25A", "G-Lab GmbH" },
    { "F8F295", "Annapurna labs" },
    { "F8F3D3", "Shenzhen Gotron electronic CO.,LTD" },
    { "F8F464", "Rawe Electonic GmbH" },
    { "F8F519", "Rulogic Inc." },
    { "F8F532", "Commscope" },
    { "F8F58C", "Apple, Inc." },
    { "F8F7B9", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "F8F7D2", "Equal Optics, LLC" },
    { "F8F7D3", "International Communications Corporation" },
    { "F8F7FF", "SYN-TECH SYSTEMS INC" },
    { "F8FB2F", "Santur Corporation" },
    { "F8FB90", "Texas Instruments" },
    { "F8FCE1", "Amazon Technologies Inc." },
    { "F8FE5C", "Reciprocal Labs Corp" },
    { "F8FE5E", "Intel Corporate" },
    { "F8FEA8", "Technico Japan Corporation" },
    { "F8FF0B", "Electronic Technology Inc." },
    { "F8FF5F", "Shenzhen Communication Technology Co.,Ltd" },
    { "F8FFC2", "Apple, Inc." },
    { "FC0012", "Toshiba Samsung Storage Technolgoy Korea Corporation " },
    { "FC012C", "Espressif Inc." },
    { "FC017C", "Hon Hai Precision Ind. Co.,Ltd." },
    { "FC019E", "VIEVU" },
    { "FC01CD", "FUNDACION TEKNIKER" },
    { "FC0296", "Xiaomi Communications Co Ltd" },
    { "FC039F", "Samsung Electronics Co.,Ltd" },
    { "FC041C", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "FC0647", "Cortland Research, LLC" },
    { "FC068C", "SHENZHEN MICIPC TECHNOLOGY CO.,LTD" },
    { "FC06ED", "M2Motive Technology Inc." },
    { "FC0736", "Huawei Device Co., Ltd." },
    { "FC07A0", "LRE Medical GmbH" },
    { "FC084A", "FUJITSU LIMITED" },
    { "FC0877", "Prentke Romich Company" },
    { "FC09D8", "ACTEON Group" },
    { "FC09F6", "GUANGDONG TONZE ELECTRIC CO.,LTD" },
    { "FC0A81", "Extreme Networks Headquarters" },
    { "FC0C45", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "FC0F4B", "Texas Instruments" },
    { "FC0FE6", "Sony Interactive Entertainment Inc." },
    { "FC0FE7", "Microchip Technology Inc." },
    { "FC101A", "Palo Alto Networks" },
    { "FC10BD", "Control Sistematizado S.A." },
    { "FC10C6", "Taicang T&W Electronics" },
    { "FC1165", "Cambium Networks Limited" },
    { "FC1186", "Logic3 plc" },
    { "FC1193", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "FC122C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "FC1263", "ASKEY COMPUTER CORP" },
    { "FC1349", "Global Apps Corp." },
    { "FC13F0", "Bouffalo Lab (Nanjing) Co., Ltd." },
    { "FC1499", "Aimore Acoustics Incorporation" },
    { "FC15B4", "Hewlett Packard" },
    { "FC1607", "Taian Technology(Wuxi) Co.,Ltd." },
    { "FC1794", "InterCreative Co., Ltd" },
    { "FC1803", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "FC183C", "Apple, Inc." },
    { "FC1910", "Samsung Electronics Co.,Ltd" },
    { "FC1928", "Actions Microelectronics" },
    { "FC1999", "Xiaomi Communications Co Ltd" },
    { "FC19D0", "Cloud Vision Networks Technology Co.,Ltd." },
    { "FC1A11", "vivo Mobile Communication Co., Ltd." },
    { "FC1A46", "Samsung Electronics Co.,Ltd" },
    { "FC1BD1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "FC1BFF", "V-ZUG AG" },
    { "FC1CA1", "Nokia" },
    { "FC1D2A", "vivo Mobile Communication Co., Ltd." },
    { "FC1D3A", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "FC1D43", "Apple, Inc." },
    { "FC1D59", "I Smart Cities HK Ltd" },
    { "FC1D84", "Autobase" },
    { "FC1E16", "IPEVO corp" },
    { "FC1F19", "SAMSUNG ELECTRO MECHANICS CO., LTD." },
    { "FC1FC0", "EURECAM" },
    { "FC221C", "Shenzhen Xunman Technology Co., Ltd" },
    { "FC229C", "Han Kyung I Net Co.,Ltd." },
    { "FC22D3", "FDSYS" },
    { "FC22F4", "Zyxel Communications Corporation" },
    { "FC2325", "EosTek (Shenzhen) Co., Ltd." },
    { "FC23CD", "SHENZHEN BILIAN ELECTRONIC CO.，LTD" },
    { "FC253F", "Apple, Inc." },
    { "FC27A2", "TRANS ELECTRIC CO., LTD." },
    { "FC29E3", "Infinix mobility limited" },
    { "FC29F3", "McPay Co.,LTD." },
    { "FC2A46", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "FC2A54", "Connected Data, Inc." },
    { "FC2A9C", "Apple, Inc." },
    { "FC2BB2", "Actiontec Electronics, Inc" },
    { "FC2CFD", "dormakaba Canada Inc. - Keyscan" },
    { "FC2D5E", "zte corporation" },
    { "FC2E19", "China Mobile Group Device Co.,Ltd." },
    { "FC2E2D", "Lorom Industrial Co.LTD." },
    { "FC2F40", "Calxeda, Inc." },
    { "FC2F6B", "Everspin Technologies, Inc." },
    { "FC2FAA", "Nokia" },
    { "FC2FEF", "UTT Technologies Co., Ltd." },
    { "FC315D", "Apple, Inc." },
    { "FC3288", "CELOT Wireless Co., Ltd" },
    { "FC3342", "Juniper Networks" },
    { "FC3357", "KAGA FEI Co., Ltd." },
    { "FC335F", "Polyera" },
    { "FC3497", "ASUSTek COMPUTER INC." },
    { "FC34E2", "Shenzhen YOUHUA Technology Co., Ltd" },
    { "FC3598", "Favite Inc." },
    { "FC35E6", "Visteon Corporation" },
    { "FC372B", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "FC386A", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "FC3876", "Forum Communication Systems, Inc" },
    { "FC3882", "Infinix mobility limited" },
    { "FC38C4", "China Grand Communications Co.,Ltd." },
    { "FC395A", "SonicWall " },
    { "FC3964", "ITEL MOBILE LIMITED" },
    { "FC3CD7", "Tuya Smart Inc." },
    { "FC3CE9", "Tsingtong Technologies Co, Ltd." },
    { "FC3D73", "eero inc." },
    { "FC3D93", "LONGCHEER TELECOMMUNICATION LIMITED" },
    { "FC3DA5", "Arcadyan Corporation" },
    { "FC3F7C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "FC3FA6", "eero inc." },
    { "FC3FAB", "Henan Lanxin Technology Co., Ltd" },
    { "FC3FDB", "Hewlett Packard" },
    { "FC3FFC", "Tozed Kangwei Tech Co.,Ltd" },
    { "FC4009", "zte corporation" },
    { "FC4116", "Google, Inc." },
    { "FC4203", "Samsung Electronics Co.,Ltd" },
    { "FC4265", "Zhejiang Tmall Technology Co., Ltd." },
    { "FC4345", "Xiaomi Communications Co Ltd" },
    { "FC4463", "Universal Audio, Inc" },
    { "FC4482", "Intel Corporate" },
    { "FC4499", "Swarco LEA d.o.o." },
    { "FC449F", "zte corporation" },
    { "FC455F", "JIANGXI SHANSHUI OPTOELECTRONIC TECHNOLOGY CO.,LTD" },
    { "FC4596", "COMPAL INFORMATION (KUNSHAN) CO., LTD. " },
    { "FC45C3", "Texas Instruments" },
    { "FC478D", "SHENZHEN TOPWELL TECHNOLOGY CO., LTD." },
    { "FC47D8", "Apple, Inc." },
    { "FC48C9", "Yobiiq Intelligence B.V." },
    { "FC48EF", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "FC492D", "Amazon Technologies Inc." },
    { "FC4AE9", "Castlenet Technology Inc." },
    { "FC4B1C", "INTERSENSOR S.R.L." },
    { "FC4B57", "Peerless Instrument Division of Curtiss-Wright " },
    { "FC4BBC", "Sunplus Technology Co., Ltd." },
    { "FC4CEA", "Dell Inc." },
    { "FC4CEF", "Huawei Device Co., Ltd." },
    { "FC4D6A", "Silicon Laboratories" },
    { "FC4D8C", "SHENZHEN PANTE ELECTRONICS TECHNOLOGY CO., LTD" },
    { "FC4DA6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "FC4DD4", "Universal Global Scientific Industrial Co., Ltd." },
    { "FC4E6D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "FC4EA4", "Apple, Inc." },
    { "FC500C", "Sitehop Ltd" },
    { "FC5090", "SIMEX Sp. z o.o." },
    { "FC50D6", "Huawei Device Co., Ltd." },
    { "FC51A4", "Commscope" },
    { "FC51B5", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "FC528D", "Vantiva USA LLC" },
    { "FC52CE", "Control iD" },
    { "FC539E", "Shanghai Wind Technologies Co.,Ltd" },
    { "FC5557", "Apple, Inc." },
    { "FC55DC", "Baltic Latvian Universal Electronics LLC" },
    { "FC5703", "Hisense broadband multimedia technology Co.,Ltd" },
    { "FC584A", "xiamenshi c-chip technology co., ltd" },
    { "FC589A", "Cisco Systems, Inc" },
    { "FC58DF", "Interphone Service" },
    { "FC58FA", "Shen Zhen Shi Xin Zhong Xin Technology Co.,Ltd." },
    { "FC599F", "Ruijie Networks Co.,LTD" },
    { "FC59C0", "Arista Networks" },
    { "FC5A1D", "Hitron Technologies. Inc" },
    { "FC5ADC", "Keenetic Limited" },
    { "FC5B24", "Weibel Scientific A/S" },
    { "FC5B26", "MikroBits" },
    { "FC5B39", "Cisco Systems, Inc" },
    { "FC5B8C", "Xiaomi Communications Co Ltd" },
    { "FC5C45", "Ruckus Wireless" },
    { "FC5CEE", "LCFC(Hefei) Electronics Technology co., ltd" },
    { "FC5F49", "Zhejiang Dahua Technology Co., Ltd." },
    { "FC6018", "Zhejiang Kangtai Electric Co., Ltd." },
    { "FC609B", "New H3C Technologies Co., Ltd" },
    { "FC6179", "IEEE Registration Authority" },
    { "FC6198", "NEC Personal Products, Ltd" },
    { "FC61E9", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "FC626E", "Beijing MDC Telecom" },
    { "FC626F", "Fortx" },
    { "FC62B9", "ALPSALPINE CO,.LTD" },
    { "FC643A", "Samsung Electronics Co.,Ltd" },
    { "FC64BA", "Xiaomi Communications Co Ltd" },
    { "FC65B3", "Huawei Device Co., Ltd." },
    { "FC65DE", "Amazon Technologies Inc." },
    { "FC66CF", "Apple, Inc." },
    { "FC671F", "Tuya Smart Inc." },
    { "FC683E", "Directed Perception, Inc" },
    { "FC6947", "Texas Instruments" },
    { "FC698C", "ANDREAS STIHL AG & Co. KG" },
    { "FC6A1C", "Mellanox Technologies, Inc." },
    { "FC6BF0", "TOPWELL INTERNATIONAL HOLDINDS LIMITED" },
    { "FC6C31", "LXinstruments GmbH" },
    { "FC6D77", "Intel Corporate" },
    { "FC6DC0", "BME CORPORATION" },
    { "FC6DD1", "APRESIA Systems, Ltd." },
    { "FC6FB7", "Commscope" },
    { "FC702E", "Sichuan AI-Link Technology Co., Ltd." },
    { "FC71FA", "Trane Technologies" },
    { "FC73FB", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "FC7516", "D-Link International" },
    { "FC75E6", "Handreamnet" },
    { "FC7692", "Semptian Co.,Ltd." },
    { "FC7774", "Intel Corporate" },
    { "FC777B", "Hitron Technologies. Inc" },
    { "FC790B", "Hitachi High Technologies America, Inc." },
    { "FC79DD", "Huawei Device Co., Ltd." },
    { "FC7A58", "Shenzhen Skyworth Digital Technology CO., Ltd" },
    { "FC7C02", "Phicomm (Shanghai) Co., Ltd." },
    { "FC7CE7", "FCI USA LLC" },
    { "FC7D6C", "HYESUNG TECHWIN Co., Ltd" },
    { "FC7F56", "CoSyst Control Systems GmbH" },
    { "FC7FF1", "Hewlett Packard Enterprise" },
    { "FC8329", "Trei technics" },
    { "FC8399", "Avaya Inc" },
    { "FC83C6", "N-Radio Technologies Co., Ltd." },
    { "FC8417", "Honor Device Co., Ltd." },
    { "FC84A7", "Murata Manufacturing Co., Ltd." },
    { "FC8596", "Axonne Inc." },
    { "FC862A", "Huawei Device Co., Ltd." },
    { "FC8743", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "FC8A3D", "zte corporation" },
    { "FC8AF7", "zte corporation" },
    { "FC8B97", "SHENZHEN GONGJIN ELECTRONICS CO.,LT" },
    { "FC8C11", "Microsoft Corporation" },
    { "FC8D13", "FUJIAN STAR-NET COMMUNICATION CO.,LTD" },
    { "FC8D3D", "Leapfive Tech. Ltd." },
    { "FC8E5B", "China Mobile Iot Limited company" },
    { "FC8E6E", "StreamCCTV, LLC" },
    { "FC8E7E", "Commscope" },
    { "FC8F7D", "SHENZHEN GONGJIN ELECTRONICS CO.,LT" },
    { "FC8F90", "Samsung Electronics Co.,Ltd" },
    { "FC8FC4", "Intelligent Technology Inc." },
    { "FC90FA", "Independent Technologies" },
    { "FC9114", "Vantiva USA LLC" },
    { "FC915D", "Google, Inc." },
    { "FC9189", "Sichuan Tianyi Comheart Telecom Co.,LTD" },
    { "FC923B", "Nokia Corporation" },
    { "FC9257", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "FC931D", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "FC936B", "Samsung Electronics Co.,Ltd" },
    { "FC942E", "Cisco Meraki" },
    { "FC9435", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "FC946C", "UBIVELOX" },
    { "FC94CE", "zte corporation" },
    { "FC94E3", "Vantiva USA LLC" },
    { "FC956A", "OCTAGON SYSTEMS CORP." },
    { "FC9643", "Juniper Networks" },
    { "FC97A8", "Cricut Inc." },
    { "FC9816", "ALPSALPINE CO,.LTD" },
    { "FC9947", "Cisco Systems, Inc" },
    { "FC9AFA", "Motus Global Inc." },
    { "FC9BC6", "Sumavision Technologies Co.,Ltd" },
    { "FC9BD4", "EdgeQ" },
    { "FC9C98", "Arlo Technology" },
    { "FC9CA7", "Apple, Inc." },
    { "FC9D05", "Micro-Star INTL CO., LTD." },
    { "FC9DD8", "Beijing TongTongYiLian Science and Technology Ltd." },
    { "FC9F2A", "Zyxel Communications Corporation" },
    { "FC9FAE", "Fidus Systems Inc" },
    { "FC9FE1", "CONWIN.Tech. Ltd" },
    { "FC9FFD", "Hangzhou Hikvision Digital Technology Co.,Ltd." },
    { "FCA05A", "Oray.com co., LTD." },
    { "FCA0F3", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "FCA13E", "Samsung Electronics Co.,Ltd" },
    { "FCA183", "Amazon Technologies Inc." },
    { "FCA22A", "PT. Callysta Multi Engineering" },
    { "FCA2DF", "IEEE Registration Authority" },
    { "FCA386", "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD" },
    { "FCA47A", "IEEE Registration Authority" },
    { "FCA5C8", "Apple, Inc." },
    { "FCA5D0", "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD" },
    { "FCA621", "Samsung Electronics Co.,Ltd" },
    { "FCA64C", "Alibaba cloud computing Co., Ltd" },
    { "FCA667", "Amazon Technologies Inc." },
    { "FCA6CD", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "FCA841", "Avaya Inc" },
    { "FCA84A", "Sentinum GmbH" },
    { "FCA89A", "Sunitec Enterprise Co.,Ltd" },
    { "FCA89B", "Texas Instruments" },
    { "FCA8E0", "THE FURUKAWA ELECTRIC CO., LTD" },
    { "FCA9B0", "MIARTECH (SHANGHAI),INC." },
    { "FCA9DC", "Renesas Electronics (Penang) Sdn. Bhd." },
    { "FCA9F5", "Xiaomi Communications Co Ltd" },
    { "FCAA14", "GIGA-BYTE TECHNOLOGY CO.,LTD." },
    { "FCAA81", "Apple, Inc." },
    { "FCAAB6", "Samsung Electronics Co.,Ltd" },
    { "FCAB90", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "FCABD0", "vivo Mobile Communication Co., Ltd." },
    { "FCABF5", "zte corporation" },
    { "FCAD0F", "QTS NETWORKS" },
    { "FCAE2B", "Titan Products Ltd." },
    { "FCAE34", "Commscope" },
    { "FCAF6A", "Qulsar Inc" },
    { "FCAFAC", "Socionext Inc." },
    { "FCAFBE", "TireCheck GmbH" },
    { "FCB0C4", "Shanghai DareGlobal Technologies Co.,Ltd" },
    { "FCB0DE", "CLOUD NETWORK TECHNOLOGY SINGAPORE PTE. LTD." },
    { "FCB10D", "Shenzhen Tian Kun Technology Co.,LTD." },
    { "FCB214", "Apple, Inc." },
    { "FCB2D6", "CIG SHANGHAI CO LTD" },
    { "FCB387", "Leapmotor (Jinhua) New Energy Vehicle Parts Technology Co., Ltd." },
    { "FCB3AA", "Intel Corporate" },
    { "FCB3BC", "Intel Corporate" },
    { "FCB467", "Espressif Inc." },
    { "FCB4E6", "ASKEY COMPUTER CORP" },
    { "FCB577", "Cortex Security Inc" },
    { "FCB585", "Shenzhen Water World Information Co.,Ltd." },
    { "FCB58A", "Wapice Ltd." },
    { "FCB662", "IC Holdings LLC" },
    { "FCB698", "Cambridge Industries(Group) Co.,Ltd." },
    { "FCB69D", "Zhejiang Dahua Technology Co., Ltd." },
    { "FCB6D8", "Apple, Inc." },
    { "FCB7F0", "Idaho National Laboratory" },
    { "FCB97E", "GE Appliances" },
    { "FCB9DF", "Motorola Mobility LLC, a Lenovo Company" },
    { "FCBBA1", "Shenzhen Minicreate Technology Co.,Ltd" },
    { "FCBC0E", "Zhejiang Cainiao Supply Chain Management Co., Ltd" },
    { "FCBC9C", "Vimar Spa" },
    { "FCBCD1", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "FCBD67", "Arista Networks" },
    { "FCBE7B", "vivo Mobile Communication Co., Ltd." },
    { "FCC0CC", "Yunke China Information Technology Limited" },
    { "FCC233", "ASUSTek COMPUTER INC." },
    { "FCC23D", "Atmel Corporation" },
    { "FCC2DE", "Murata Manufacturing Co., Ltd." },
    { "FCC2E5", "HOLOWITS TECHNOLOGIES CO.,LTD" },
    { "FCC734", "Samsung Electronics Co.,Ltd" },
    { "FCC737", "Shaanxi Gangsion Electronic Technology Co., Ltd" },
    { "FCC897", "zte corporation" },
    { "FCCAC4", "LifeHealth, LLC" },
    { "FCCCE4", "Ascon Ltd." },
    { "FCCD2F", "IEEE Registration Authority" },
    { "FCCF43", "HUIZHOU CITY HUIYANG DISTRICT MEISIQI INDUSTRY DEVELOPMENT CO,.LTD" },
    { "FCCF62", "IBM Corp" },
    { "FCD202", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "FCD290", "SKY UK LIMITED" },
    { "FCD2B6", "IEEE Registration Authority" },
    { "FCD436", "Motorola Mobility LLC, a Lenovo Company" },
    { "FCD4F2", "The Coca Cola Company" },
    { "FCD4F6", "Messana Air.Ray Conditioning s.r.l." },
    { "FCD586", "Shenzhen SuperElectron Technology Co.,Ltd." },
    { "FCD5D9", "Shenzhen SDMC Technology CO.,Ltd." },
    { "FCD6BD", "Robert Bosch GmbH" },
    { "FCD733", "TP-LINK TECHNOLOGIES CO.,LTD." },
    { "FCD749", "Amazon Technologies Inc." },
    { "FCD817", "Beijing Hesun Technologies Co.Ltd." },
    { "FCD848", "Apple, Inc." },
    { "FCD908", "Xiaomi Communications Co Ltd" },
    { "FCD96B", "Realme Chongqing Mobile Telecommunications Corp.,Ltd." },
    { "FCDB21", "SAMSARA NETWORKS INC" },
    { "FCDB96", "ENERVALLEY CO., LTD" },
    { "FCDBB3", "Murata Manufacturing Co., Ltd." },
    { "FCDC4A", "G-Wearables Corp." },
    { "FCDD55", "Shenzhen WeWins wireless Co.,Ltd" },
    { "FCDE90", "Samsung Electronics Co.,Ltd" },
    { "FCDEC5", "Texas Instruments" },
    { "FCDF00", "GD Midea Air-Conditioning Equipment Co.,Ltd." },
    { "FCE14F", "BRK Brands, Inc." },
    { "FCE186", "A3M Co., LTD" },
    { "FCE192", "Sichuan Jinwangtong Electronic Science&Technology Co,.Ltd" },
    { "FCE1A6", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "FCE1D9", "Stable Imaging Solutions LLC" },
    { "FCE1FB", "Array Networks" },
    { "FCE23F", "CLAY PAKY SPA" },
    { "FCE26C", "Apple, Inc." },
    { "FCE33C", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "FCE498", "IEEE Registration Authority" },
    { "FCE557", "Nokia Corporation" },
    { "FCE66A", "Industrial Software Co" },
    { "FCE806", "Edifier International" },
    { "FCE892", "Hangzhou Lancable Technology Co.,Ltd" },
    { "FCE8C0", "Espressif Inc." },
    { "FCE998", "Apple, Inc." },
    { "FCE9D8", "Amazon Technologies Inc." },
    { "FCEA50", "Integrated Device Technology (Malaysia) Sdn. Bhd." },
    { "FCECDA", "Ubiquiti Inc" },
    { "FCEDB9", "Arrayent" },
    { "FCEEE6", "FORMIKE ELECTRONIC CO., LTD" },
    { "FCF136", "Samsung Electronics Co.,Ltd" },
    { "FCF152", "Sony Corporation" },
    { "FCF1CD", "OPTEX-FA CO.,LTD." },
    { "FCF29F", "China Mobile Iot Limited company" },
    { "FCF528", "Zyxel Communications Corporation" },
    { "FCF5C4", "Espressif Inc." },
    { "FCF647", "Fiberhome Telecommunication Technologies Co.,LTD" },
    { "FCF738", "HUAWEI TECHNOLOGIES CO.,LTD" },
    { "FCF763", "KunGao Micro (JiangSu) Co., LTd" },
    { "FCF77B", "Huawei Device Co., Ltd." },
    { "FCF8AE", "Intel Corporate" },
    { "FCF8B7", "TRONTEQ Electronic" },
    { "FCFA21", "zte corporation" },
    { "FCFAF7", "Shanghai Baud Data Communication Co.,Ltd." },
    { "FCFBFB", "Cisco Systems, Inc" },
    { "FCFC48", "Apple, Inc." },
    { "FCFE77", "Hitachi Reftechno, Inc." },
    { "FCFEC2", "Invensys Controls UK Limited" },
    { "FCFFAA", "IEEE Registration Authority" },
    { NULL, NULL }
};

// Total 37875 items.
